# Copyright 2008-2018 pydicom authors. See LICENSE file for details.
"""Many point of entry for pydicom read and write functions"""
from pydicom.filereader import dcmread  # noqa: F401
from pydicom.filewriter import dcmwrite  # noqa: F401
