# Auto-generated by generate_concept_dicts.py.

# Dict with cid number as keys; value format is:
#   {scheme designator: <list of keywords for current cid>
#    scheme_designator: ...}

name_for_cid = {}
cid_concepts = {}

name_for_cid[2] = "AnatomicModifier"
cid_concepts[2] = {
    "DCM": ["Median"],
    "SCT": [
        "Peripheral",
        "Basal",
        "Bilateral",
        "Right",
        "Surface",
        "Caudal",
        "Posterior",
        "Vertical",
        "Sagittal",
        "Left",
        "External",
        "Unilateral",
        "Superior",
        "IntraArticular",
        "Edge",
        "Distal",
        "Proximal",
        "Axial",
        "Marginal",
        "Preaxial",
        "Deep",
        "Subcapsular",
        "Lateral",
        "Inferior",
        "Capsular",
        "Anterior",
        "Intermediate",
        "Transverse",
        "Cephalic",
        "Anterolateral",
        "Central",
        "Hilar",
        "Apical",
        "ExtraArticular",
        "Postaxial",
        "Horizontal",
        "Posterolateral",
        "Afferent",
        "Superficial",
        "Gutter",
        "Coronal",
        "Efferent",
        "Internal",
        "Medial",
        "Longitudinal",
    ],
}

name_for_cid[4] = "AnatomicRegion"
cid_concepts[4] = {
    "SCT": [
        "LeftHypochondriacRegion",
        "SubclavianVein",
        "ExternalIliacArtery",
        "BoneOfLowerLimb",
        "TruncusCoeliacus",
        "BasilarArtery",
        "ParanasalSinus",
        "ApexOfRightVentricle",
        "WristJoint",
        "Phantom",
        "FontanelOfSkull",
        "Mouth",
        "GenicularArtery",
        "LeftVentricleInflow",
        "SigmoidColon",
        "PharynxAndLarynx",
        "CarotidArtery",
        "Cheek",
        "UmbilicalArtery",
        "PrimitiveAorta",
        "LeftVentricleOutflowTract",
        "RadialArtery",
        "Intracranial",
        "RightFemoralArtery",
        "JawRegion",
        "LeftFemoralArtery",
        "LumenOfBloodVessel",
        "Calcaneus",
        "EndoEsophageal",
        "AntecubitalVein",
        "Mandible",
        "Parasternal",
        "BroadLigament",
        "PatentDuctusArteriosus",
        "MesentericVein",
        "IntraThoracic",
        "Pelvis",
        "PulmonaryVein",
        "CoronaryArtery",
        "UpperLimb",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "SubclavianArtery",
        "Flank",
        "Maxilla",
        "Lung",
        "SacroCoccygealSpine",
        "OphthalmicArtery",
        "Nose",
        "PancreaticDuct",
        "Hand",
        "BrachialVein",
        "RightUpperQuadrantOfAbdomen",
        "Eye",
        "Prostate",
        "ElbowJoint",
        "HepaticArtery",
        "Back",
        "FacialBones",
        "Sternum",
        "PeronealArtery",
        "NeckAndChest",
        "RightVentricleOutflowTract",
        "Duodenum",
        "RightHypochondriacRegion",
        "SuperiorLeftPulmonaryVein",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "UpperOuterQuadrantOfBreast",
        "AbdomenAndPelvis",
        "LumbarArtery",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "Cornea",
        "Larynx",
        "PulmonaryArtery",
        "FemoralArtery",
        "Ovary",
        "IliacVein",
        "Skull",
        "EndoUrethral",
        "Thyroid",
        "Head",
        "AdrenalGland",
        "AcromioclavicularJoint",
        "UmbilicalRegion",
        "Calyx",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "PosteriorMedialTributary",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "EndoNasopharyngeal",
        "EsophagusStomachAndDuodenum",
        "Breast",
        "Neck",
        "Finger",
        "Submental",
        "CranialVenousSystem",
        "PoplitealFossa",
        "PulmonaryArteryConduit",
        "RightAtrium",
        "RightVentricle",
        "CommonVentricle",
        "Coccyx",
        "HeadAndNeck",
        "Hypopharynx",
        "Ventricle",
        "Uterus",
        "TemporomandibularJoint",
        "TracheaAndBronchus",
        "HepaticVein",
        "Testis",
        "InternalMammaryArtery",
        "Bladder",
        "Vulva",
        "LargeIntestine",
        "Forearm",
        "Face",
        "Spine",
        "Foot",
        "NeckChestAndAbdomen",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "MuscleOfUpperLimb",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "IntraAbdominopelvic",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "SuperficialFemoralArtery",
        "UterusAndFallopianTubes",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "JuxtaposedAtrialAppendage",
        "AorticArch",
        "VertebralColumnAndCranium",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "AbdominalAorta",
        "EndoCardiac",
        "TarsalJoint",
        "Scalp",
        "CephalicVein",
        "Tongue",
        "LingualArtery",
        "GastricVein",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "Fibula",
        "OrbitalStructure",
        "Penis",
        "InferiorCardiacVein",
        "Baffle",
        "LacrimalArtery",
        "OccipitalArtery",
        "EndoVascular",
        "Sclera",
        "Brain",
        "SystemicVenousAtrium",
        "Parathyroid",
        "LiverAndBiliaryStructure",
        "AnomalousPulmonaryVein",
        "SuprasternalNotch",
        "Perineum",
        "EndoRectal",
        "Femur",
        "AnteriorCardiacVein",
        "Pancreas",
        "LeftAtrium",
        "CerebralArtery",
        "Scapula",
        "InferiorMesentericArtery",
        "EndoRenal",
        "GreatSaphenousVein",
        "OmentalBursa",
        "BladderAndUrethra",
        "CervicalSpine",
        "IliacAndOrFemoralArtery",
        "InternalCarotidArtery",
        "EndoUreteric",
        "Sacrum",
        "SmallIntestine",
        "InternalAuditoryCanal",
        "SuperiorRightPulmonaryVein",
        "IntraArticular",
        "RenalPelvis",
        "InferiorVenaCava",
        "RightPulmonaryArtery",
        "Axilla",
        "Retroperitoneum",
        "Thymus",
        "VenousNetwork",
        "CommonAtrium",
        "Humerus",
        "Ilium",
        "AorticFistula",
        "LeftLumbarRegion",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "NasalBone",
        "AnteriorTibialArtery",
        "Cervix",
        "TruncusArteriosusCommunis",
        "EndoArterial",
        "CommonCarotidArtery",
        "Joint",
        "Buttock",
        "SubmandibularArea",
        "OpticCanal",
        "Aorta",
        "SuperficialTemporalArtery",
        "EndoVenous",
        "MastoidBone",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "Thigh",
        "Spleen",
        "ChestAndAbdomen",
        "InferiorRightPulmonaryVein",
        "Trachea",
        "BodyConduit",
        "LowerOuterQuadrantOfBreast",
        "AnkleJoint",
        "EndoVesical",
        "Mediastinum",
        "Cerebellum",
        "RightVentricleInflow",
        "MesentericArtery",
        "PoplitealArtery",
        "LeftLowerQuadrantOfAbdomen",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "Bronchus",
        "SesamoidBonesOfFoot",
        "InternalIliacArtery",
        "PulmonaryChamberOfCorTriatriatum",
        "CommonBileDuct",
        "SuperiorVenaCava",
        "UlnarArtery",
        "LeftVentricle",
        "AnteriorSpinalArtery",
        "LumbarRegion",
        "Omentum",
        "AnusRectumAndSigmoidColon",
        "SplenicVein",
        "Colon",
        "SubmandibularGland",
        "ThoracicSpine",
        "SaphenousVein",
        "Gallbladder",
        "InnominateArtery",
        "SplenicArtery",
        "RenalArtery",
        "HipJoint",
        "UpperArm",
        "Extremity",
        "Thorax",
        "FacialArtery",
        "EpigastricRegion",
        "AzygosVein",
        "BrachialArtery",
        "CervicoThoracicSpine",
        "Liver",
        "RightLumbarRegion",
        "Hip",
        "Rib",
        "BoneOfUpperLimb",
        "EndoVaginal",
        "PelvisAndLowerExtremities",
        "LowerInnerQuadrantOfBreast",
        "Clavicle",
        "ParotidGland",
        "CommonFemoralArtery",
        "IntraPelvic",
        "Stomach",
        "Subxiphoid",
        "UpperInnerQuadrantOfBreast",
        "AnteriorCommunicatingArtery",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "RightInguinalRegion",
        "ThoracicAorta",
        "VenaCava",
        "CoronarySinus",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "BiliaryTract",
        "ApexOfLung",
        "Ileum",
        "Atrium",
        "Knee",
        "VertebralArtery",
        "RightLowerQuadrantOfAbdomen",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "InternalJugularVein",
        "EyeRegion",
        "LacrimalArteryOfRightEye",
        "SacroiliacJoint",
        "Anus",
        "UmbilicalVein",
        "EntireBody",
        "CommonIliacVein",
        "InguinalRegion",
        "EndoNasal",
        "HypogastricRegion",
        "Thumb",
        "LumbarSpine",
        "GreatCardiacVein",
        "PrimitivePulmonaryArtery",
        "SellaTurcica",
        "PortalVein",
        "Scrotum",
        "InferiorLeftPulmonaryVein",
        "LowerLeg",
        "RenalVein",
        "BoydPerforatingVein",
        "Ear",
        "SupraclavicularRegionOfNeck",
        "CommonIliacArtery",
        "Toe",
        "Shoulder",
        "SpinalCord",
        "CardiacSeptumProsthesis",
        "LeftAuricularAppendage",
        "CircleOfWillis",
        "LeftInguinalRegion",
        "Eyelid",
        "MuscleOfLowerLimb",
        "LeftUpperQuadrantOfAbdomen",
        "AscendingAorta",
        "HunterianPerforatingVein",
        "HypochondriacRegion",
        "ProfundaFemorisArtery",
        "Pharynx",
        "ExternalAuditoryCanal",
        "AxillaryVein",
        "Patella",
        "Nasopharynx",
        "OccipitalVein",
        "LumboSacralSpine",
        "DoddPerforatingVein",
    ]
}

name_for_cid[5] = "TransducerApproach"
cid_concepts[5] = {
    "SCT": [
        "Peripheral",
        "Ascending",
        "Superior",
        "FourthIntercostalSpace",
        "Distal",
        "PiaMater",
        "Proximal",
        "Lateral",
        "Inferior",
        "Capsular",
        "Subcostal",
        "WaterBagProsthesisForImagingProcedure",
        "EleventhIntercostalSpace",
        "FifthIntercostalSpace",
        "SalineBagProsthesisForImagingProcedure",
        "MidLongitudinal",
        "FirstIntercostalSpace",
        "Intercostal",
        "EighthIntercostalSpace",
        "DuraMater",
        "Internal",
        "Descending",
        "Midline",
        "Right",
        "Surface",
        "Caudal",
        "Posterior",
        "Left",
        "External",
        "SixthIntercostalSpace",
        "SecondIntercostalSpace",
        "Parasagittal",
        "Intraluminal",
        "Anterior",
        "Transverse",
        "GelProsthesisForImagingProcedure",
        "Cranial",
        "NinthIntercostalSpace",
        "Central",
        "SeventhIntercostalSpace",
        "Apical",
        "Parenchyma",
        "ThirdIntercostalSpace",
        "SubcutaneousTissue",
        "Lumen",
        "ExternalProsthesisForSonographicProcedureStandOff",
        "ContactWith",
        "Medial",
        "TenthIntercostalSpace",
    ]
}

name_for_cid[6] = "TransducerOrientation"
cid_concepts[6] = {
    "SCT": [
        "FiveChamber",
        "LongAxis",
        "OffAxis",
        "Parasagittal",
        "FourChamber",
        "ShortAxis",
        "TwoChamber",
        "Oblique",
        "Sagittal",
        "Coronal",
        "Transverse",
        "Longitudinal",
    ]
}

name_for_cid[7] = "UltrasoundBeamPath"
cid_concepts[7] = {
    "SCT": [
        "TransGastric",
        "TransUrethral",
        "TransMural",
        "TransHepatic",
        "TransAbdominal",
        "TransVaginal",
        "TransThecal",
        "TransVesical",
        "TransSplenic",
        "TransPleural",
        "TransRectal",
        "TransEsophageal",
        "TransPancreatic",
        "TransRenal",
        "TransOrbital",
        "TransTemporal",
    ]
}

name_for_cid[8] = "AngiographicInterventionalDevice"
cid_concepts[8] = {
    "DCM": ["Sphere"],
    "SCT": [
        "RotationalAtherectomyDevice",
        "GuidingCatheter",
        "Glue",
        "EmbolizationBall",
        "PercutaneousTransluminalAngioplastyBalloon",
        "AtherectomyDevice",
        "EmbolizationCoil",
        "EmbolizationParticulate",
        "Laser",
        "Catheter",
        "MeasuringRuler",
        "Stent",
        "DetachableBalloon",
    ],
}

name_for_cid[9] = "ImageGuidedTherapeuticProcedure"
cid_concepts[9] = {
    "SCT": [
        "Thoracentesis",
        "StentPlacement",
        "CatheterManipulation",
        "OcclusionOfCatheter",
        "UltrasonicGuidanceForAmniocentesis",
        "Amniocentesis",
        "AtherectomyByRotaryCutter",
        "PercutaneousRemovalOfIntravascularForeignBody",
        "AmnioinfusionInjectionOfAmnion",
        "Irrigation",
        "Vasoconstriction",
        "Fetocide",
        "PercutaneousInsertionOfIntravascularFilter",
        "IntraArterialInfusionOfThrombolyticAgent",
        "RemovalOfCatheter",
        "AtherectomyByLaser",
        "CatheterReplacement",
        "Catheterization",
        "IntrauterineCordocentesis",
        "TranscatheterTherapyForEmbolization",
        "Biopsy",
        "Atherectomy",
        "SelectiveEmbolizationOfArtery",
        "RemovalOfForeignBody",
        "ProstaglandinInjection",
        "PercutaneousTransluminalBalloonAngioplasty",
        "FailedAttemptedProcedure",
        "IntrauterineTransfusion",
        "TranscatheterDeploymentOfDetachableBalloon",
        "BreechVersionObstetricalVersion",
        "Vasodilatation",
    ]
}

name_for_cid[10] = "InterventionalDrug"
cid_concepts[10] = {
    "NCIt": ["CardiotonicAgent"],
    "SCT": [
        "Nitroglycerin",
        "Streptokinase",
        "HemostaticAgent",
        "AntihemophilicFactorPreparation",
        "UrokinasePreparation",
        "Verapamil",
        "Propranolol",
        "Thromboplastin",
        "LidocaineHydrochloride",
        "AntiHeparinAgent",
        "Vasodilator",
        "DiureticAgent",
        "GlucagonPreparation",
        "ThrombolyticAgent",
        "Fibrinolysin",
        "Ethanol",
        "Nifedipine",
        "Coagulant",
        "AnticoagulantAgent",
        "Warfarin",
        "AntiarrhythmicDrug",
        "InotropicAgent",
        "Heparin",
        "BetaBlockerContainingProduct",
        "ThrombinPreparation",
        "CentrallyActingHypotensiveAgent",
        "ProtamineSulfate",
        "ProstaglandinE1",
        "AlphaAdrenergicBlockingAgent",
        "Epinephrine",
        "Phentolamine",
        "TolazolineHydrocholoride",
        "Atropine",
        "Antihistamine",
        "Quinidine",
        "Dextran",
        "Papaverine",
        "MethyleneBlue",
        "LidocaineContainingProduct",
        "Digoxin",
        "CardiacAdrenergicBlockingAgent",
        "HypotensiveAgent",
        "AstringentDrug",
        "HumanFibrinogen",
    ],
}

name_for_cid[11] = "AdministrationRoute"
cid_concepts[11] = {
    "DCM": ["RetroOrbitalRoute"],
    "NCIt": ["ExtraluminalRoute", "IntraepithelialRoute", "TransluminalRoute"],
    "SCT": [
        "IntraluminalRoute",
        "TopicalRoute",
        "IntrathecalRoute",
        "OralRoute",
        "IntramuscularRoute",
        "IntravenousRoute",
        "SubcutaneousRoute",
        "IntracorpusCavernosumRoute",
        "ByInhalation",
        "IntraArterialRoute",
        "IntraArticularRoute",
        "IntracardiacRoute",
        "IntraventricularRouteCardiac",
        "IntracutaneousRoute",
        "IntramedullaryRoute",
        "NasalRoute",
        "PerRectum",
        "IntratumorRoute",
        "IntraperitonealRoute",
        "VaginalRoute",
        "IntracoronaryRoute",
    ],
}

name_for_cid[12] = "ImagingContrastAgent"
cid_concepts[12] = {
    "RADLEX": ["NonIonicIodinatedContrastAgent", "IonicIodinatedContrastAgent"],
    "RXNORM": ["MangafodipirTrisodium"],
    "SCT": [
        "Iophendylate",
        "Iodipamide",
        "Bunamiodyl",
        "GadobenateDimeglumine",
        "Gadodiamide",
        "Gadofosveset",
        "Iohexol",
        "SodiumAcetrizoate",
        "Propyliodone",
        "ContrastAgent",
        "GadoterateMeglumine",
        "Iothalamate",
        "MeglumineDiatrizoate",
        "Gadobutrol",
        "Diatrizoate",
        "NonRadiopaqueMedium",
        "ChloriodizedOil",
        "Air",
        "Iopamidol",
        "Iopromide",
        "IopanoicAcid",
        "Metrizoate",
        "SodiumIodomethamate",
        "MeglumineIodipamide",
        "GadopentetateDimeglumine",
        "Ioxilan",
        "Ioversol",
        "Iodixanol",
        "Metrizamide",
        "Ipodate",
        "BariumSulfate",
        "Oxygen",
        "SodiumTyropanoate",
        "Water",
        "IodoalphionicAcid",
        "Iodopyracet",
        "SodiumDiprotrizoate",
        "Iodophthalein",
        "GadoxetateDisodium",
        "Gadoversetamide",
        "IodamideMeglumine",
        "SodiumIodipamide",
        "Gadoteridol",
        "IophenoxicAcid",
        "Ioxaglate",
        "IodizedOil",
        "CarbonDioxide",
        "GadoliniumContainingProduct",
    ],
}

name_for_cid[13] = "ImagingContrastAgentIngredient"
cid_concepts[13] = {
    "SCT": [
        "Gadolinium",
        "Barium",
        "Iodine",
        "Iron",
        "CarbonDioxide",
        "Oxygen",
        "Air",
        "Water",
        "Xenon",
    ]
}

name_for_cid[18] = "RadiopharmaceuticalIsotope"
cid_concepts[18] = {
    "DCM": ["_191mIridium"],
    "NCIt": ["_166Holmium"],
    "SCT": [
        "_22Sodium",
        "_113mIndium",
        "_42Potassium",
        "_97Ruthenium",
        "_201Thallium",
        "_188Rhenium",
        "_198Gold",
        "_199Gold",
        "_75Selenium",
        "_153Gadolinium",
        "_24Sodium",
        "_153Samarium",
        "_57Cobalt",
        "_59Iron",
        "_67Gallium",
        "_18Fluorine",
        "_64Copper",
        "_32Phosphorus",
        "_111Indium",
        "_186Rhenium",
        "_81mKrypton",
        "_178Tantalum",
        "_43Potassium",
        "_67Copper",
        "_125Iodine",
        "_89Strontium",
        "_90Yttrium",
        "_127Xenon",
        "_87mStrontium",
        "_203Lead",
        "_169Ytterbium",
        "_223Radium",
        "_14Carbon",
        "_133Xenon",
        "_177Lutetium",
        "_51Chromium",
        "_85Strontium",
        "_58Cobalt",
        "_99mTechnetium",
        "_133Barium",
        "_123Iodine",
        "_60Cobalt",
        "_85Krypton",
        "_131Iodine",
    ],
}

name_for_cid[19] = "PatientOrientation"
cid_concepts[19] = {"NCIt": ["Erect"], "SCT": ["Recumbent", "SemiErect"]}

name_for_cid[20] = "PatientOrientationModifier"
cid_concepts[20] = {
    "SCT": [
        "CurledUp",
        "SemiProne",
        "Anatomical",
        "Supine",
        "LateralDecubitus",
        "KneeChest",
        "Kneeling",
        "Lithotomy",
        "Prone",
        "Standing",
        "LeftLateralDecubitus",
        "StoopedOver",
        "RightLateralDecubitus",
        "InverseTrendelenburg",
        "Sitting",
        "Frog",
        "Trendelenburg",
        "Lordotic",
    ]
}

name_for_cid[21] = "PatientEquipmentRelationship"
cid_concepts[21] = {
    "DCM": ["LeftFirst", "RightFirst", "PosteriorFirst", "AnteriorFirst"],
    "SCT": ["Headfirst", "FeetFirst", "TransverseBodyPosition", "ObliqueBodyPosition"],
}

name_for_cid[23] = "CranioCaudadAngulation"
cid_concepts[23] = {"SCT": ["Caudal", "Cephalic"]}

name_for_cid[25] = "Radiopharmaceutical"
cid_concepts[25] = {
    "DCM": ["Lutetium177NAcetylaspartylglutamate", "IodinatedI125DPA713"],
    "MSH": ["Technetium99mTRODAT1"],
    "NCIt": ["Technetium99mTrofolastat"],
    "SCT": [
        "TechnetiumTc99mTetrofosmin",
        "IodinatedI131Albumin",
        "FerrousChlorideFe59",
        "Indium113mChloride",
        "SodiumIodideI131",
        "Chromium51Chloride",
        "Indium111RedCellLabel",
        "DiagnosticRadioisotope",
        "SodiumIodideI123",
        "Carbon14DXylose",
        "Iodine131Polyvinylpyrrolidone",
        "TechnetiumTc99mSulfurColloid",
        "TechnetiumTc99mTeboroxime",
        "Technetium99mPhytate",
        "TechnetiumTc99mIronAscorbate",
        "StrontiumNitrateSr87",
        "Iodine1233IodobenzylguanidineMIBG",
        "TechnetiumTc99mSestamibi",
        "RoseBengalSodiumI131",
        "Copper64Acetate",
        "Indium113mPentetate",
        "TechnetiumTc99mMedronate",
        "TechnetiumTc99mSuccimer",
        "TechnetiumTc99mSerumAlbumin",
        "IodinatedI131AggregatedAlbumin",
        "SodiumFluorideF18",
        "SodiumPertechnetateTc99m",
        "StrontiumNitrateSr85",
        "TechnetiumTc99mDisofenin",
        "Indium111Pentetreotide",
        "TechnetiumTc99mMebrofenin",
        "IodinatedI131GammaGlobulin",
        "Radium223Dichloride",
        "Indium111Chloride",
        "SodiumChromateCr51",
        "TechnetiumTc99mStannousEtidronate",
        "FluorodeoxyglucoseF18",
        "Indium111FeOHGreaterThan3LesserThan",
        "TechnetiumTc99mLidofenin",
        "SelenomethionioneSe75",
        "FibrinogenI123",
        "TechnetiumTc99mTaggedRedCells",
        "TechnetiumTc99mOxidronate",
        "IodinatedI125Povidone",
        "IodohippurateI123Sodium",
        "Iodine1313IodobenzylguanidineMIBG",
        "TechnetiumTc99cAlbuminMicrospheres",
        "StrontiumChlorideSr87",
        "Technetium99mLabeledCarbon",
        "Selenium75HCAT",
        "TechnetiumTc99mDepreotide",
        "StrontiumChlorideSr85",
        "ChromicPhosphateP32",
        "TechnetiumTc99mPyrophosphate",
        "IothalamateSodiumI125",
        "CyanocobalaminCo57",
        "Indium111Transferrin",
        "CyanocobalaminCo58",
        "Technetium99mStannousColloid",
        "Yttrium90Microspheres",
        "Indium111Oxyquinoline",
        "Indium111Pentetate",
        "Copper67Ceruloplasmin",
        "Xenon133Gas",
        "Technetium99mMercaptoacetylTriglycineMAG3",
        "SodiumChlorideNa24",
        "Carbon14Triolein",
        "Krypton81m",
        "IodohippurateI131Sodium",
        "StrontiumChlorideSr89",
        "Copper64Versenate",
        "IodohippurateI125Sodium",
        "PotassiumCarbonateK42",
        "IofetamineI123Hydrochloride",
        "TechnetiumTc99mPentetate",
        "IodocholesterolI131",
        "Chromium51Albumin",
        "TechnetiumTc99mBicisate",
        "Iodine123154Iodophenyl3RSMethylpentadecanoicAcid",
        "TechnetiumTc99mMicroaggregatedAlbumin",
        "TechnetiumTc99mExametazime",
        "Indium111CapromabPendetide",
        "TherapeuticRadioisotope",
        "Technetium99mNPyridoxyl5Methyltryptophan",
        "TechnetiumTc99mSodiumGlucoheptonate",
        "PotassiumChlorideK42",
        "PentetateCalciumTrisodiumYb169",
        "FerrousCitrateFe59",
        "Technetium99mGalactosylHumanSerumAlbuminDiethylenetriamine",
        "CyanocobalaminCo60",
        "OleicAcidI125",
        "TechnetiumTc99mAggregatedAlbumin",
        "IodinatedI125SodiumIodine",
        "SodiumPhosphateP32",
        "PotassiumChlorideK43",
        "ThallousChlorideTl201",
        "Lutetium177DOTATATE",
        "IodinatedI125HumanSerumAlbumin",
        "IronFe59LabeledDextran",
        "IodinatedI125Albumin",
        "IoflupaneI123",
        "Indium113mBleomycin",
        "FerrousSulfateFe59",
        "Gallium67Citrate",
        "TechnetiumTc99mNSubstitutedIminodiacetate",
        "SodiumChlorideNa22",
    ],
}

name_for_cid[26] = "NuclearMedicineProjection"
cid_concepts[26] = {
    "SCT": [
        "MedialLateralEmissiveProjection",
        "SagittalObliqueAxisEmissiveProjection",
        "LeftLateralEmissiveProjection",
        "VerticalLongAxis",
        "HorizontalLongAxis",
        "Sagittal",
        "RightPosteriorObliqueEmissiveProjection",
        "LeftAnteriorObliqueEmissiveProjection",
        "PosteriorProjection",
        "AnteriorProjection",
        "Axial",
        "LateralMedialEmissiveProjection",
        "RightAnteriorObliqueEmissiveProjection",
        "LeftPosteriorObliqueEmissiveProjection",
        "ShortAxis",
        "MedioLateralObliqueEmissiveProjection",
        "Lateral",
        "RightLateralEmissiveProjection",
        "LateroMedialObliqueEmissiveProjection",
        "Coronal",
        "ObliqueAxialEmissiveProjection",
        "Transverse",
    ]
}

name_for_cid[27] = "BasicCardiacView"
cid_concepts[27] = {"SCT": ["VerticalLongAxis", "ShortAxis", "HorizontalLongAxis"]}

name_for_cid[29] = "AcquisitionModality"
cid_concepts[29] = {
    "DCM": [
        "BoneMineralDensitometry",
        "Electrooculography",
        "SlideMicroscopy",
        "OphthalmicPhotography",
        "Electrocardiography",
        "GeneralMicroscopy",
        "XRayAngiography",
        "ExternalCameraPhotography",
        "Dermoscopy",
        "IntravascularUltrasound",
        "ComputedRadiography",
        "ComputedTomography",
        "Photoacoustic",
        "OpticalSurfaceScanner",
        "Thermography",
        "Lensometry",
        "OphthalmicTomographyEnFace",
        "HemodynamicWaveform",
        "OpticalCoherenceTomography",
        "UltrasoundBoneDensitometry",
        "Diaphanography",
        "PositronEmissionTomography",
        "CardiacElectrophysiology",
        "LaserScan",
        "PanoramicXRay",
        "OphthalmicMapping",
        "OphthalmicTomographyBScanVolumeAnalysis",
        "OphthalmicVisualField",
        "DigitalRadiography",
        "OphthalmicTomography",
        "Mammography",
        "Ultrasound",
        "Electromyography",
        "IntravascularOpticalCoherenceTomography",
        "MagneticResonance",
        "IntraOralRadiography",
        "Electroencephalography",
        "RTImage",
        "VisualAcuity",
        "RespiratoryWaveform",
        "Endoscopy",
        "Autorefraction",
        "PositionSensor",
        "RadiographicImaging",
        "Radiofluoroscopy",
        "Keratometry",
        "OphthalmicAxialMeasurements",
        "NuclearMedicine",
        "BiomagneticImaging",
        "SubjectiveRefraction",
        "ConfocalMicroscopy",
    ]
}

name_for_cid[30] = "DICOMDevice"
cid_concepts[30] = {
    "DCM": [
        "BoneMineralDensitometry",
        "Electrooculography",
        "SlideMicroscopy",
        "OphthalmicPhotography",
        "FilmDigitizer",
        "Workstation",
        "Electrocardiography",
        "GeneralMicroscopy",
        "XRayAngiography",
        "ExternalCameraPhotography",
        "Dermoscopy",
        "IntravascularUltrasound",
        "ComputedRadiography",
        "ComputationServer",
        "MediaCreationDevice",
        "ComputedTomography",
        "Photoacoustic",
        "OpticalSurfaceScanner",
        "Thermography",
        "Lensometry",
        "OphthalmicTomographyEnFace",
        "HemodynamicWaveform",
        "OpticalCoherenceTomography",
        "_3DManufacturingModelingSystem",
        "UltrasoundBoneDensitometry",
        "Diaphanography",
        "PositronEmissionTomography",
        "CardiacElectrophysiology",
        "LaserScan",
        "HardCopyPrintServer",
        "PanoramicXRay",
        "OphthalmicMapping",
        "OphthalmicTomographyBScanVolumeAnalysis",
        "OphthalmicVisualField",
        "DigitalRadiography",
        "OphthalmicTomography",
        "Mammography",
        "Ultrasound",
        "Electromyography",
        "IntravascularOpticalCoherenceTomography",
        "DepartmentSystemScheduler",
        "ProcedureLogging",
        "MagneticResonance",
        "IntraOralRadiography",
        "ComputerAssistedDetectionDiagnosis",
        "Electroencephalography",
        "RTImage",
        "VisualAcuity",
        "RespiratoryWaveform",
        "Endoscopy",
        "Autorefraction",
        "PositionSensor",
        "RadiographicImaging",
        "Radiofluoroscopy",
        "Keratometry",
        "Archive",
        "ImageCapture",
        "AutomatedSlideStainer",
        "OphthalmicAxialMeasurements",
        "NuclearMedicine",
        "RadiationTherapyDevice",
        "BiomagneticImaging",
        "SubjectiveRefraction",
        "ConfocalMicroscopy",
    ]
}

name_for_cid[31] = "AbstractPrior"
cid_concepts[31] = {
    "DCM": [
        "AtLastAppointment",
        "OnDischargeFromUnit",
        "OnDischarge",
        "PostIntervention",
        "PreIntervention",
        "OnAdmissionToUnit",
    ],
    "SCT": [
        "Intraoperative",
        "PreOperative",
        "OnAdmission",
        "PreAdmission",
        "PreDose",
        "PostOperative",
        "PostDose",
    ],
}

name_for_cid[32] = "NonAcquisitionModality"
cid_concepts[32] = {
    "DCM": [
        "Registration",
        "Segmentation",
        "TextureMap",
        "BasicVoiceAudio",
        "RTDose",
        "DocumentProducer",
        "StereometricRelationship",
        "RTTreatmentRecord",
        "HardCopy",
        "CTProtocol",
        "RTPlan",
        "AutomatedSlideStainer",
        "StructuredReportDocument",
        "SpatialFiducialsProducer",
        "ModelFor3DManufacturing",
        "RTStructureSet",
        "Plan",
        "PresentationState",
        "Other",
        "KeyObjectSelection",
        "IntraocularLensCalculation",
        "ContentAssessmentResult",
        "RealWorldValueMap",
    ]
}

name_for_cid[33] = "Modality"
cid_concepts[33] = {
    "DCM": [
        "BoneMineralDensitometry",
        "Electrooculography",
        "SlideMicroscopy",
        "StructuredReportDocument",
        "SpatialFiducialsProducer",
        "OphthalmicPhotography",
        "Other",
        "KeyObjectSelection",
        "Electrocardiography",
        "GeneralMicroscopy",
        "XRayAngiography",
        "IntraocularLensCalculation",
        "ExternalCameraPhotography",
        "ContentAssessmentResult",
        "Dermoscopy",
        "IntravascularUltrasound",
        "ComputedRadiography",
        "ComputedTomography",
        "Photoacoustic",
        "OpticalSurfaceScanner",
        "Thermography",
        "Registration",
        "Lensometry",
        "TextureMap",
        "RTDose",
        "OphthalmicTomographyEnFace",
        "HardCopy",
        "RTPlan",
        "HemodynamicWaveform",
        "OpticalCoherenceTomography",
        "UltrasoundBoneDensitometry",
        "ModelFor3DManufacturing",
        "Diaphanography",
        "PresentationState",
        "PositronEmissionTomography",
        "CardiacElectrophysiology",
        "LaserScan",
        "PanoramicXRay",
        "OphthalmicMapping",
        "OphthalmicTomographyBScanVolumeAnalysis",
        "OphthalmicVisualField",
        "DigitalRadiography",
        "OphthalmicTomography",
        "DocumentProducer",
        "RTTreatmentRecord",
        "Mammography",
        "Ultrasound",
        "Electromyography",
        "IntravascularOpticalCoherenceTomography",
        "RTStructureSet",
        "MagneticResonance",
        "IntraOralRadiography",
        "Electroencephalography",
        "RTImage",
        "VisualAcuity",
        "RespiratoryWaveform",
        "Endoscopy",
        "RealWorldValueMap",
        "Autorefraction",
        "PositionSensor",
        "Segmentation",
        "RadiographicImaging",
        "Radiofluoroscopy",
        "BasicVoiceAudio",
        "Keratometry",
        "StereometricRelationship",
        "CTProtocol",
        "AutomatedSlideStainer",
        "OphthalmicAxialMeasurements",
        "NuclearMedicine",
        "Plan",
        "BiomagneticImaging",
        "SubjectiveRefraction",
        "ConfocalMicroscopy",
    ]
}

name_for_cid[42] = "NumericValueQualifier"
cid_concepts[42] = {
    "DCM": [
        "ValueUnknown",
        "NegativeInfinity",
        "ValueIndeterminate",
        "NotANumber",
        "DivideByZero",
        "PositiveInfinity",
        "Overflow",
        "Underflow",
        "MeasurementNotAttempted",
        "MeasurementFailure",
        "ValueOutOfRange",
        "CalculationFailure",
    ]
}

name_for_cid[43] = "NumericValueFailureQualifier"
cid_concepts[43] = {
    "DCM": [
        "NegativeInfinity",
        "NotANumber",
        "DivideByZero",
        "PositiveInfinity",
        "Overflow",
        "Underflow",
        "MeasurementFailure",
        "ValueOutOfRange",
        "CalculationFailure",
    ]
}

name_for_cid[44] = "NumericValueUnknownQualifier"
cid_concepts[44] = {
    "DCM": ["ValueUnknown", "ValueIndeterminate", "MeasurementNotAttempted"]
}

name_for_cid[50] = "InstanceAvailabilityStatus"
cid_concepts[50] = {"DCM": ["Unavailable", "Nearline", "Offline", "Online"]}

name_for_cid[60] = "ImagingAgentAdministrationAdverseEvent"
cid_concepts[60] = {
    "DCM": ["ExtravasationVisibleInImage", "PatientConditionPreventedContinuing"],
    "SCT": [
        "PallorPaleComplexion",
        "InjectionSiteNerveDamage",
        "InjectionSiteFibrosis",
        "InjectionSiteExtravasation",
        "Cough",
        "ChillsAndFever",
        "InjectionSiteHypersensitivity",
        "InjectionSitePigmentationChange",
        "InjectionSiteScar",
        "InjectionSiteIrritation",
        "NauseaAndVomiting",
        "Sweating",
        "InjectionSiteAnesthesia",
        "FacialSwelling",
        "InjectionSiteInduration",
        "InjectionSiteUrticaria",
        "Bronchospasm",
        "InjectionSiteMalabsorption",
        "InjectionSiteSterileAbscess",
        "Headache",
        "SensationOfBeingWarm",
        "TachycardiaBradycardia",
        "InjectionSiteNecrosis",
        "Anxiety",
        "InjectionSiteDermatitis",
        "InjectionSiteDisorder",
        "Shaking",
        "InjectionSiteBruising",
        "DrugInducedFlushing",
        "CardiacArrhythmia",
        "LaryngealEdema",
        "InjectionSiteThrombosis",
        "InjectionSiteAbscess",
        "DrugInducedDizziness",
        "CardiorespiratoryArrest",
        "InjectionSiteCyst",
        "Hypertension",
        "InjectionSiteEdema",
        "Dyspnea",
        "InjectionSiteAtrophy",
        "DrugInducedHypotension",
        "InjectionSiteBurning",
        "InjectionSiteHemorrhage",
        "InjectionSiteUlcer",
        "EpilepticConvulsions",
        "Itching",
        "DrugRash",
        "InjectionSiteInflammation",
        "InjectionSiteInfection",
        "Hypotension",
        "DiffuseInflammatoryErythema",
        "TasteSenseAltered",
        "InjectionSiteParesthesia",
        "NasalCongestion",
        "NoMotorResponseToCommand",
        "InjectionSitePain",
        "InjectionSiteMass",
        "InjectionSiteGranuloma",
    ],
}

name_for_cid[62] = "ImagingAgentAdministrationPhaseType"
cid_concepts[62] = {
    "DCM": [
        "AutomaticProgrammedAdministrationPhase",
        "AutomaticProgrammedDelayPhase",
        "AutomatedManualInjectPhase",
        "AutomaticProgrammedWaitPhase",
    ]
}

name_for_cid[63] = "ImagingAgentAdministrationMode"
cid_concepts[63] = {"DCM": ["ManualAdministration", "AutomatedAdministration"]}

name_for_cid[64] = "ImagingAgentAdministrationPatientState"
cid_concepts[64] = {
    "DCM": [
        "AcuteUnilateralRenalBlockage",
        "LowThyroidUptake",
        "HighThyroidUptake",
        "SeverelyJaundiced",
        "PatientAllergicToMediaContrast",
    ],
    "SCT": [
        "Cardiomyopathy",
        "MalignantEpithelialNeoplasmOfThyroid",
        "HistoryOfHypertension",
        "HistoryOfDiabetesMellitus",
        "AnginaPectoris",
        "Paraproteinemia",
        "HistoryOfCongestiveHeartFailure",
        "PulmonaryHypertension",
        "MultipleMyeloma",
        "Anxiety",
        "Asthma",
        "HistoryOfBetaBlockingAgentsTherapy",
        "HistoryOfRenalFailure",
        "AorticStenosis",
        "AbnormalRenalFunction",
    ],
}

name_for_cid[65] = "ImagingAgentAdministrationPremedication"
cid_concepts[65] = {
    "SCT": [
        "Alfentanil",
        "Remifentanil",
        "Propofol",
        "Midazolam",
        "Sufentanil",
        "Methylprednisolone",
        "HydrocortisoneSodiumSuccinate",
        "DexamethasoneSodiumPhosphate_Inactive",
        "Papaverine",
        "Diphenhydramine",
        "MethylprednisoloneSodiumPhosphate",
        "H1Antihistamine",
        "Prednisone",
        "Ephedrine",
    ]
}

name_for_cid[66] = "ImagingAgentAdministrationMedication"
cid_concepts[66] = {
    "SCT": ["Alfentanil", "Remifentanil", "Propofol", "Midazolam", "Sufentanil"]
}

name_for_cid[67] = "ImagingAgentAdministrationCompletionStatus"
cid_concepts[67] = {
    "DCM": [
        "TerminatedDueToFlowRateAboveTerminationLimit",
        "TerminatedDueToConsumableRemoval",
        "TerminatedDueToPressureAboveTerminationLimit",
        "TerminatedDueToCriticalBatteryLevel",
        "TerminatedByScanner",
        "TerminatedDueToAirDetected",
        "TerminatedDueToRequestFromOperator",
        "TerminatedDueToInjectorCommunicationLoss",
        "TerminatedDueToExcessiveDurationPause",
        "TerminatedDueToUnspecifiedInjectorFailure",
    ],
    "SCT": ["Complete"],
}

name_for_cid[68] = "ImagingAgentAdministrationPharmaceuticalPresentationUnit"
cid_concepts[68] = {
    "SCT": ["ParenteralEnteralSolutionBag", "SyringeUnit", "Cartridge", "Bottle"]
}

name_for_cid[69] = "ImagingAgentAdministrationConsumable"
cid_concepts[69] = {"SCT": ["Syringe", "Catheter", "Needle", "Tube", "Bottle"]}

name_for_cid[70] = "Flush"
cid_concepts[70] = {"MSH": ["LactatedRingerS"], "SCT": ["Dextran", "Saline"]}

name_for_cid[71] = "ImagingAgentAdministrationInjectorEventType"
cid_concepts[71] = {
    "DCM": [
        "TerminatedDueToUnspecifiedInjectorFailure",
        "SyringeDetached",
        "SyringeAttached",
        "KeepVeinOpenEnded",
        "KeepVeinOpenStarted",
        "ManualHoldStarted",
        "ProgrammedHoldStarted",
        "TerminatedHoldDueToTimeout",
        "ManualResumeFromHold",
        "FlowRateAboveAdjustmentLimit",
        "AirDetected",
        "FlowRateAboveWarningLimit",
        "PressureAboveAdjustmentLimit",
        "PressureAboveWarningLimit",
        "TerminatedDueToFlowRateAboveTerminationLimit",
        "TerminatedDueToConsumableRemoval",
        "TerminatedDueToPressureAboveTerminationLimit",
        "TerminatedDueToCriticalBatteryLevel",
        "FixedDurationPauseEnded",
        "TerminatedByScanner",
        "EquipmentFailure",
        "TerminatedDueToAirDetected",
        "TerminatedDueToRequestFromOperator",
        "TerminatedDueToInjectorCommunicationLoss",
        "TerminatedDueToExcessiveDurationPause",
        "ResourceInadequate",
    ]
}

name_for_cid[72] = "ImagingAgentAdministrationStepType"
cid_concepts[72] = {
    "DCM": [
        "DiagnosticAdministration",
        "TransitTimeTestInjection",
        "PatencyTestInjection",
        "FlushAdministration",
    ]
}

name_for_cid[73] = "BolusShapingCurve"
cid_concepts[73] = {"DCM": ["LinearCurve", "NegativeExponential"]}

name_for_cid[74] = "ImagingAgentAdministrationConsumableCatheterType"
cid_concepts[74] = {
    "SCT": [
        "PeripheralIntravenousCatheter",
        "CentralVenousCatheter",
        "ImplantableVenousAccessPort",
        "RectalCatheter",
    ]
}

name_for_cid[75] = "LowHighorEqual"
cid_concepts[75] = {"SCT": ["Low", "High", "Equal"]}

name_for_cid[76] = "PremedicationType"
cid_concepts[76] = {
    "DCM": ["ContrastReactionProphylacticAgent", "PreMedication"],
    "SCT": ["Sedative", "Antiemetic"],
}

name_for_cid[83] = "RealWorldValueMappingUnit"
cid_concepts[83] = {
    "UCUM": [
        "MicromolePerMinutePerMilliliter",
        "SquareCentimeterPerMilliliter",
        "SquareCentimeter",
        "MilliliterPerGram",
        "StandardizedUptakeValueBodySurfaceArea",
        "StandardizedUptakeValueLeanBodyMassJanma",
        "ProportionalToCounts",
        "Counts",
        "Percent",
        "MilliliterPerMinutePerGram",
        "StandardizedUptakeValueBodyWeight",
        "MicromolePerMilliliter",
        "HounsfieldUnit",
        "ProportionalToCountsPerSecond",
        "MilligramsPerMinutePerMilliliter",
        "StandardizedUptakeValueIdealBodyWeight",
        "PerCentimeter",
        "BecquerelsPerMilliliter",
        "CountsPerSecond",
        "StandardizedUptakeValueLeanBodyMassJames",
        "StandardizedUptakeValueLeanBodyMassJames128Multiplier",
    ]
}

name_for_cid[84] = "PETUnit"
cid_concepts[84] = {
    "UCUM": [
        "MicromolePerMinutePerMilliliter",
        "SquareCentimeterPerMilliliter",
        "SquareCentimeter",
        "MilliliterPerGram",
        "StandardizedUptakeValueBodySurfaceArea",
        "StandardizedUptakeValueLeanBodyMassJanma",
        "ProportionalToCounts",
        "Counts",
        "Percent",
        "MilliliterPerMinutePerGram",
        "StandardizedUptakeValueBodyWeight",
        "MicromolePerMilliliter",
        "ProportionalToCountsPerSecond",
        "MilligramsPerMinutePerMilliliter",
        "StandardizedUptakeValueIdealBodyWeight",
        "PerCentimeter",
        "BecquerelsPerMilliliter",
        "CountsPerSecond",
        "StandardizedUptakeValueLeanBodyMassJames",
        "StandardizedUptakeValueLeanBodyMassJames128Multiplier",
    ]
}

name_for_cid[85] = "SUVUnit"
cid_concepts[85] = {
    "UCUM": [
        "StandardizedUptakeValueIdealBodyWeight",
        "StandardizedUptakeValueBodySurfaceArea",
        "StandardizedUptakeValueLeanBodyMassJanma",
        "StandardizedUptakeValueBodyWeight",
        "StandardizedUptakeValueLeanBodyMassJames",
        "StandardizedUptakeValueLeanBodyMassJames128Multiplier",
    ]
}

name_for_cid[91] = "FunctionalConditionPresentDuringAcquisition"
cid_concepts[91] = {
    "DCM": ["DuringVoiding", "PostVoiding", "PriorToVoiding"],
    "SCT": [
        "WeightBearing",
        "DrugInfusion",
        "LegUp",
        "ValsalvaManeuver",
        "Phonation",
        "PostVolumeChallenge",
        "HeldVentilation",
        "HeadUp",
        "VagalStimulation",
        "Pacing",
        "NegativeLowerBodyPressure",
        "Handgrip",
        "HeldInspiration",
        "ExerciseChallenge",
    ],
}

name_for_cid[92] = "JointPositionDuringAcquisition"
cid_concepts[92] = {
    "DCM": ["NeutralMusculoskeletalPosition"],
    "SCT": [
        "Torsion",
        "ExternalRotation",
        "Pronation",
        "Flexion",
        "InternalRotation",
        "Abduction",
        "Extension",
        "Adduction",
        "Supination",
    ],
}

name_for_cid[93] = "JointPositioningMethod"
cid_concepts[93] = {"SCT": ["PassiveMovement", "ManipulationOfJoint"]}

name_for_cid[94] = "PhysicalForceAppliedDuringAcquisition"
cid_concepts[94] = {"SCT": ["TractionAction", "RotationAction", "CompressionAction"]}

name_for_cid[100] = "QuantitativeDiagnosticImagingProcedure"
cid_concepts[100] = {
    "DCM": ["MultiparametricMRIOfWholeBody", "MultiparametricMRI"],
    "LN": [
        "XRUnspecifiedBodyRegion",
        "NMUnspecifiedBodyRegion",
        "PETWholeBody",
        "SPECTBrain",
        "MRIUnspecifiedBodyRegion",
        "PETUnspecifiedBodyRegion",
        "CTUnspecifiedBodyRegion",
        "CTPerfusionHeadWithContrastIV",
    ],
    "RADLEX": ["NMHeadPerfusionBrainPETCTAV45"],
    "SCT": [
        "DynamicMagneticResonanceImagingOfKnee",
        "DynamicMagneticResonanceImagingOfPelvis",
        "PETCTFDGImagingOfWholeBody",
        "MultiparametricMRIOfProstate",
        "MagneticResonanceImagingGuidance",
        "ImagingProcedure",
        "PETCTMETImagingOfWholeBody",
        "Angiography",
    ],
}

name_for_cid[101] = "ImagingProcedure"
cid_concepts[101] = {}

name_for_cid[210] = "QualitativeEvaluationModifierType"
cid_concepts[210] = {"SCT": ["TopographicalModifier", "Laterality"]}

name_for_cid[211] = "QualitativeEvaluationModifierValue"
cid_concepts[211] = {
    "DCM": ["Median"],
    "NCIt": ["Middle"],
    "SCT": [
        "Bilateral",
        "Central",
        "Right",
        "Posterior",
        "Left",
        "Unilateral",
        "Superior",
        "Distal",
        "Proximal",
        "MidLongitudinal",
        "Lateral",
        "Inferior",
        "Anterior",
    ],
}

name_for_cid[212] = "GenericAnatomicLocationModifier"
cid_concepts[212] = {
    "DCM": ["Median"],
    "NCIt": ["Middle"],
    "SCT": [
        "Distal",
        "Proximal",
        "MidLongitudinal",
        "Central",
        "Lateral",
        "Posterior",
        "Inferior",
        "Anterior",
        "Superior",
    ],
}

name_for_cid[217] = "VisualExplanation"
cid_concepts[217] = {
    "DCM": ["GradientWeightedClassActivation", "ClassActivation", "Saliency"]
}

name_for_cid[218] = "QuantitativeImageFeature"
cid_concepts[218] = {
    "DCM": [
        "WaterFatOutOfPhase",
        "FractionalOccupancySegmentation",
        "WaterFatInPhase",
        "FractionalProbabilisticSegmentation",
        "BinarySegmentation",
        "WaterFraction",
        "MRSignalIntensity",
        "XRayAttenuation",
        "FatFraction",
        "YBRICTCRComponent",
        "YBRICTCBComponent",
        "YBRICTYComponent",
        "YBRPARTIALCRComponent",
        "YBRPARTIALCBComponent",
        "YBRPARTIALYComponent",
        "RootAngularSecondMomentOfGLCM",
        "Echogenicity",
        "YBRRCTCRComponent",
        "YBRRCTCBComponent",
        "YBRRCTYComponent",
        "RGBRComponent",
        "UltrasoundAttenuation",
        "SpeedOfSound",
        "Perfusion",
        "Elasticity",
        "YBRFULLCRComponent",
        "FractalDimension",
        "YBRFULLCBComponent",
        "YBRFULLYComponent",
        "RGBBComponent",
        "RGBGComponent",
        "CreatineCholineCitrateRatio",
        "MagnetizationTransferRatio",
        "CreatineAndCholine",
        "LipidAndLactate",
        "SpatialDisplacementYComponent",
        "SpatialDisplacementXComponent",
        "NuclearMedicineTomographicActivity",
        "NuclearMedicineProjectionActivity",
        "FlowVariance",
        "FlowVelocity",
        "TissueVelocity",
        "IndexedHemodynamicResistance",
        "HemodynamicResistance",
        "SpatialDisplacementZComponent",
        "AttenuationCoefficient",
        "Ktrans",
        "Kep",
        "Ve",
        "Suvbsa",
        "Suvibw",
        "SuvlbmJanma",
        "SuvlbmJames128",
        "StandardizedUptakeValue",
        "Suvbw",
        "Suvlbm",
        "ApparentDiffusionCoefficient",
        "DiffusionWeighted",
        "LineSegmentLength",
        "AcetabularAngle",
        "VolumeRatio",
        "VolumeOfSphere",
        "DiffusionCoefficient",
        "VolumeOfCircumscribedSphere",
        "VolumeOfEllipsoid",
        "VolumeEstimatedFromThreeOrMoreNonCoplanar2DRegions",
        "VolumeEstimatedFromTwoNonCoplanar2DRegions",
        "VolumeEstimatedFromSingle2DRegion",
        "PathLength",
        "VolumeOfBoundingThreeDimensionalRegion",
        "R2Coefficient",
        "ChiSquare",
        "DW",
        "AIC",
        "BIC",
        "Distance",
        "Height",
        "RelativeLinearStoppingPower",
        "TauM",
        "Vp",
        "EquivalentDose",
        "GradientWeightedClassActivation",
        "ClassActivation",
        "AbsorbedDose",
        "Saliency",
        "IAUC90BN",
        "IAUC180BN",
        "IAUC",
        "IAUC60",
        "IAUC90",
        "IAUC180",
        "IAUCBN",
        "IAUC60BN",
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "NAcetylaspartateCholineRatio",
        "Tmax",
        "R2Star",
        "VolumetricDiffusionDxzComponent",
        "MagneticSusceptibility",
        "VolumetricDiffusionDxyComponent",
        "VolumetricDiffusionDxxComponent",
        "RelativeRegionalBloodFlow",
        "RelativeRegionalBloodVolume",
        "BloodOxygenationLevel",
        "T2StarWeightedDynamicContrastEnhancedMRSignalIntensity",
        "T2WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodFlow",
        "T1WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodVolume",
        "VolumetricDiffusionDzzComponent",
        "OxygenExtractionFraction",
        "R1",
        "VolumetricDiffusionDyzComponent",
        "R2",
        "VolumetricDiffusionDyyComponent",
        "VelocityEncoded",
        "ZScore",
        "ContrastAgentAngioMRSignalIntensity",
        "SpinTaggingPerfusionMRSignalIntensity",
        "RelativeAnisotropy",
        "FractionalAnisotropy",
        "FieldMapMRSignalIntensity",
        "T2StarWeightedMRSignalIntensity",
        "T2WeightedMRSignalIntensity",
        "T1WeightedMRSignalIntensity",
        "ProtonDensityWeightedMRSignalIntensity",
        "TimeOfFlightAngioMRSignalIntensity",
        "T1",
        "T2Star",
        "T2",
        "TimeCourseOfSignal",
        "TemperatureEncoded",
        "StudentTTest",
        "TimeToPeak",
        "SignalToNoise",
        "TemporalDerivativeThreshold",
        "MaximumSlope",
        "MaximumDifference",
        "TracerConcentration",
        "TimeOfPeakConcentration",
        "BolusArrivalTime",
        "TimeOfLeadingHalfPeakConcentration",
        "MeanTransitTime",
        "KurtosisDiffusionCoefficient",
        "GammaDistributionScaleParameter",
        "NegativeEnhancementIntegral",
        "GammaDistributionShapeParameter",
        "GammaDistributionMode",
        "DistributedDiffusionCoefficient",
        "RCoefficient",
        "AnomalousExponentParameter",
        "ProtonDensity",
        "SignalChange",
        "MonoExponentialApparentDiffusionCoefficient",
        "SlowDiffusionCoefficient",
        "FastDiffusionCoefficient",
        "FastDiffusionCoefficientFraction",
    ],
    "IBSI": [
        "LongRunHighGrayLevelEmphasis",
        "IntensityHistogramMedian",
        "GreyLevelNonUniformityOfDistanceZoneCounts",
        "SurfaceToVolumeRatio",
        "InverseVarianceOfGLCM",
        "DependenceCountPercentage",
        "FirstMeasureOfInformationCorrelationOfGLCM",
        "ZoneSizeNonuniformity",
        "DependenceCountEnergy",
        "MoranIIndex",
        "InverseDifferenceOfGLCM",
        "JointAverageOfGLCM",
        "InverseDifferenceMomentOfGLCM",
        "IntensityHistogram90thPercentile",
        "NormalizedGrayLevelNonuniformityOfSizeZoneCounts",
        "IntensityHistogramMedianAbsoluteDeviation",
        "ShortRunLowGrayLevelEmphasis",
        "SizeZonePercentage",
        "GrayLevelVarianceInRuns",
        "IntensityHistogramRobustMeanAbsoluteDeviation",
        "Compactness1",
        "HighDependenceHighGreyLevelEmphasis",
        "IntensityHistogramMinimumGradientGrayLevel",
        "GreyLevelVarianceOfNGLDM",
        "DistanceZonePercentage",
        "AutocorrelationOfGLCM",
        "LongRunsEmphasis",
        "NormalizedRunLengthNonuniformity",
        "DependenceCountVariance",
        "ClusterTendencyOfGLCM",
        "LowGreyLevelZoneEmphasis",
        "DifferenceAverageOfGLCM",
        "SumAverageOfGLCM",
        "IntensityHistogramQuartileCoefficientOfDispersion",
        "LargeDistanceEmphasis",
        "NormalizedGreyLevelNonUniformityOfNGLDM",
        "SumVarianceOfGLCM",
        "IntensityHistogramEntropy",
        "IntensityHistogramUniformity",
        "LargeDistanceHighGreyLevelEmphasis",
        "SmallZoneHighGrayLevelEmphasis",
        "DissimilarityOfGLCM",
        "Maximum3DDiameterOfAMesh",
        "GearyCMeasure",
        "LowGreyLevelCountEmphasis",
        "LargeZoneEmphasis",
        "RunEntropy",
        "VolumeOfMesh",
        "ZoneDistanceVariance",
        "LowGrayLevelRunEmphasis",
        "Flatness",
        "DifferenceVarianceOfGLCM",
        "CorrelationOfGLCM",
        "BusynessOfNGTDM",
        "IntensityHistogramKurtosis",
        "StrengthOfNGTDM",
        "AreaDensityInMinimumVolumeEnclosingEllipsoid",
        "GreyLevelNonUniformityOfNGLDM",
        "Compactness2",
        "JointEntropyOfGLCM",
        "LargeDistanceLowGreyLevelEmphasis",
        "VolumeFromVoxelSummation",
        "SecondMeasureOfInformationCorrelationOfGLCM",
        "SmallZoneLowGrayLevelEmphasis",
        "MajorAxisIn3DLength",
        "NormalizedZoneDistanceNonUniformity",
        "LowGrayLevelZoneEmphasis",
        "ZoneSizeVariance",
        "NormalizedZoneSizeNonuniformity",
        "IntensityHistogramRange",
        "AreaDensityInApproximateEnclosingEllipsoid",
        "CoarsenessOfNGTDM",
        "LowDependenceLowGreyLevelEmphasis",
        "DependenceCountNonUniformityNormalized",
        "ContrastOfNGTDM",
        "GrayLevelVarianceInSizeZones",
        "IntensityHistogramMinimumGradient",
        "ClusterShadeOfGLCM",
        "ClusterProminenceOfGLCM",
        "IntensityHistogramVariance",
        "IntensityHistogramMeanAbsoluteDeviation",
        "IntensityHistogramInterquartileRange",
        "ContrastOfGLCM",
        "SmallZoneEmphasis",
        "SmallDistanceLowGreyLevelEmphasis",
        "LowDependenceHighGreyLevelEmphasis",
        "AreaDensityInOrientedMinimumBoundingBox",
        "HighGrayLevelZoneEmphasis",
        "SurfaceAreaOfMesh",
        "IntensityHistogram10thPercentile",
        "VolumeDensityInFrameOfReferenceAxisAlignedBoundingBox",
        "Asphericity",
        "VolumeDensityInOrientedMinimumBoundingBox",
        "DependenceCountEntropy",
        "VolumeDensityInApproximateEnclosingEllipsoid",
        "RunPercentage",
        "IntensityHistogramSkewness",
        "IntensityHistogramMaximumGradient",
        "LargeZoneLowGrayLevelEmphasis",
        "HighGreyLevelCountEmphasis",
        "HighDependenceLowGreyLevelEmphasis",
        "IntensityHistogramMaximumGrayLevel",
        "LowDependenceEmphasis",
        "Elongation",
        "LongRunLowGrayLevelEmphasis",
        "HighGrayLevelRunEmphasis",
        "ShortRunsEmphasis",
        "HighDependenceEmphasis",
        "ShortRunHighGrayLevelEmphasis",
        "SphericalDisproportion",
        "GrayLevelNonuniformityOfSizeZoneCounts",
        "RunLengthNonuniformity",
        "IntensityHistogramMean",
        "RunLengthVariance",
        "ZoneDistanceEntropy",
        "CentreOfMassShift",
        "ComplexityOfNGTDM",
        "GreyLevelVarianceInDistanceZones",
        "ZoneDistanceNonUniformity",
        "IntensityHistogramMode",
        "JointMaximumOfGLCM",
        "LeastAxisIn3DLength",
        "HighGreyLevelZoneEmphasis",
        "MinorAxisIn3DLength",
        "IntegratedIntensity",
        "LargeZoneHighGrayLevelEmphasis",
        "AngularSecondMomentOfGLCM",
        "Sphericity",
        "NormalizedInverseDifferenceMomentOfGLCM",
        "VolumeDensityInMinimumVolumeEnclosingEllipsoid",
        "JointVarianceOfGLCM",
        "AreaDensityInConvexHull",
        "NormalizedGrayLevelNonuniformityInRuns",
        "IntensityHistogramMaximumGradientGrayLevel",
        "IntensityHistogramMinimumGrayLevel",
        "NormalizedInverseDifferenceOfGLCM",
        "GrayLevelNonuniformityInRuns",
        "ZoneSizeEntropy",
        "SmallDistanceHighGreyLevelEmphasis",
        "DifferenceEntropyOfGLCM",
        "AreaDensityInFrameOfReferenceAxisAlignedBoundingBox",
        "SmallDistanceEmphasis",
        "VolumeDensityInConvexHull",
        "IntensityHistogramCoefficientOfVariation",
        "DependenceCountNonUniformity",
        "SumEntropyOfGLCM",
        "NormalizedGreyLevelNonUniformityOfDistanceZoneCounts",
    ],
    "SCT": [
        "Lipid",
        "DiameterOfCircumscribedCircle",
        "Depth",
        "PowerDoppler",
        "Perimeter",
        "CobbAngle",
        "Lactate",
        "Length",
        "Inositol",
        "PerpendicularAxis",
        "Radius",
        "AreaOfDefinedRegion",
        "Width",
        "Volume",
        "Fat",
        "Area",
        "NAcetylaspartate",
        "Creatine",
        "Glutamine",
        "MinorAxis",
        "Diameter",
        "Tuarine",
        "Citrate",
        "MajorAxis",
        "LongAxis",
        "Circumference",
        "Water",
        "Choline",
        "ShortAxis",
    ],
}

name_for_cid[219] = "GeometryGraphicalRepresentation"
cid_concepts[219] = {
    "DCM": [
        "CenterOfMass",
        "GeometricCenterpoint",
        "SeedPoint",
        "Centerline",
        "Center",
        "BoundaryPoint",
        "Outline",
        "RepresentativePoint",
    ],
    "SCT": ["BoundedBy"],
}

name_for_cid[220] = "SignificanceLevel"
cid_concepts[220] = {
    "SCT": [
        "Significant",
        "SignificanceUndetermined",
        "MostSignificant",
        "HighlySignificant",
        "NotSignificant",
    ]
}

name_for_cid[221] = "MeasurementRangeConcept"
cid_concepts[221] = {
    "DCM": [
        "PercentileRankingOfMeasurement",
        "ZScoreOfMeasurement",
        "StandardDeviationOfPopulation",
        "_2SigmaDeviationOfPopulation",
        "InterquartileRangeToMedianRatioOfPopulation",
        "InterquartileRangeOfPopulation",
    ],
    "SCT": [
        "_1SigmaLowerValueOfPopulation",
        "_2SigmaLowerValueOfPopulation",
        "_5thPercentileValueOfPopulation",
        "_10thPercentileValueOfPopulation",
        "_1SigmaUpperValueOfPopulation",
        "_2SigmaUpperValueOfPopulation",
        "MeanValueOfPopulation",
        "_95thPercentileValueOfPopulation",
        "_90thPercentileValueOfPopulation",
        "MedianValueOfPopulation",
    ],
}

name_for_cid[222] = "Normality"
cid_concepts[222] = {
    "SCT": [
        "AbnormallyHigh",
        "Abnormal",
        "AbnormallyLow",
        "Normal",
        "NormalityUndetermined",
    ]
}

name_for_cid[223] = "NormalRangeValue"
cid_concepts[223] = {"SCT": ["NormalRangeLowerLimit", "NormalRangeUpperLimit"]}

name_for_cid[224] = "SelectionMethod"
cid_concepts[224] = {
    "DCM": ["MostRecentValueChosen", "MeanValueChosen", "UserChosenValue"]
}

name_for_cid[225] = "MeasurementUncertaintyConcept"
cid_concepts[225] = {
    "SCT": [
        "RangeOfUpperMeasurementUncertainty",
        "RangeOfMeasurementUncertainty",
        "RangeOfLowerMeasurementUncertainty",
    ]
}

name_for_cid[226] = "PopulationStatisticalDescriptor"
cid_concepts[226] = {
    "DCM": [
        "StandardDeviationOfPopulation",
        "_2SigmaDeviationOfPopulation",
        "InterquartileRangeToMedianRatioOfPopulation",
        "InterquartileRangeOfPopulation",
    ],
    "SCT": [
        "_1SigmaLowerValueOfPopulation",
        "_2SigmaLowerValueOfPopulation",
        "_5thPercentileValueOfPopulation",
        "_10thPercentileValueOfPopulation",
        "_1SigmaUpperValueOfPopulation",
        "_2SigmaUpperValueOfPopulation",
        "MeanValueOfPopulation",
        "_95thPercentileValueOfPopulation",
        "_90thPercentileValueOfPopulation",
        "MedianValueOfPopulation",
    ],
}

name_for_cid[227] = "SampleStatisticalDescriptor"
cid_concepts[227] = {"DCM": ["PercentileRankingOfMeasurement", "ZScoreOfMeasurement"]}

name_for_cid[228] = "EquationorTable"
cid_concepts[228] = {
    "DCM": [
        "TableOfValues",
        "TableOfValuesCitation",
        "MethodCitation",
        "Equation",
        "EquationCitation",
    ]
}

name_for_cid[230] = "YesNo"
cid_concepts[230] = {"SCT": ["Yes", "No", "Undetermined"]}

name_for_cid[231] = "YesNoOnly"
cid_concepts[231] = {"SCT": ["Yes", "No"]}

name_for_cid[240] = "PresentAbsent"
cid_concepts[240] = {"SCT": ["Absent", "Present", "Undetermined"]}

name_for_cid[241] = "PresentAbsentOnly"
cid_concepts[241] = {"SCT": ["Absent", "Present"]}

name_for_cid[242] = "NormalAbnormal"
cid_concepts[242] = {"SCT": ["Abnormal", "Normal", "NormalityUndetermined"]}

name_for_cid[244] = "Laterality"
cid_concepts[244] = {"SCT": ["Bilateral", "Right", "Left", "Unilateral"]}

name_for_cid[245] = "LateralitywithMedian"
cid_concepts[245] = {
    "DCM": ["Median"],
    "SCT": ["Bilateral", "Right", "Left", "Unilateral"],
}

name_for_cid[246] = "RelativeLaterality"
cid_concepts[246] = {"SCT": ["Contralateral", "Ipsilateral"]}

name_for_cid[247] = "LateralityLeftRightOnly"
cid_concepts[247] = {"SCT": ["Right", "Left"]}

name_for_cid[250] = "PositiveNegative"
cid_concepts[250] = {"SCT": ["Negative", "Positive"]}

name_for_cid[251] = "ComplicationSeverity"
cid_concepts[251] = {"SCT": ["Minor", "Major"]}

name_for_cid[252] = "SMLSizeDescriptor"
cid_concepts[252] = {"SCT": ["Medium", "Small", "Large"]}

name_for_cid[270] = "ObserverType"
cid_concepts[270] = {"DCM": ["Person", "Device"]}

name_for_cid[271] = "ObservationSubjectClass"
cid_concepts[271] = {"DCM": ["Fetus", "Specimen", "Patient", "DeviceSubject"]}

name_for_cid[272] = "SpecificObservationSubjectClass"
cid_concepts[272] = {
    "DCM": ["ImageFrame", "Study", "Patient", "Series"],
    "NCIt": ["ImagingRegionOfInterest", "Image"],
    "SCT": ["Waveform"],
}

name_for_cid[280] = "LongitudinalTemporalEventType"
cid_concepts[280] = {"DCM": ["Baseline"], "NCIt": ["Enrollment"]}

name_for_cid[300] = "MultienergyRelevantMaterial"
cid_concepts[300] = {
    "SCT": [
        "Silicon",
        "Yttrium",
        "Iodine",
        "UricAcid",
        "Iron",
        "CobaltChromiumAlloy",
        "Silicone",
        "Hafnium",
        "Tantalum",
        "Water",
        "Fat",
        "Gold",
        "Calcium",
        "Barium",
        "Gadolinium",
        "RenalStone",
        "NickelCobaltChromium",
        "Bismuth",
        "CalciumHydroxyapatite",
        "Titanium",
        "Platinum",
        "Silver",
        "AluminumOxide",
    ]
}

name_for_cid[301] = "MultienergyMaterialUnit"
cid_concepts[301] = {
    "DCM": ["ModifiedHounsfieldUnit", "EffectiveAtomicNumber"],
    "UCUM": [
        "MilligramsPerCubicCentimeter",
        "HounsfieldUnit",
        "ElectronDensity",
        "Percent",
        "MilligramsPerMilliliter",
    ],
}

name_for_cid[400] = "AuditEventID"
cid_concepts[400] = {
    "DCM": [
        "ApplicationActivity",
        "ProcedureRecord",
        "PatientRecord",
        "NetworkEntry",
        "Import",
        "Export",
        "DICOMStudyDeleted",
        "DICOMInstancesTransferred",
        "DICOMInstancesAccessed",
        "UserAuthentication",
        "BeginTransferringDICOMInstances",
        "SecurityAlert",
        "AuditLogUsed",
        "Query",
        "OrderRecord",
    ]
}

name_for_cid[401] = "AuditEventTypeCode"
cid_concepts[401] = {
    "DCM": [
        "AuditRecordingStopped",
        "UseOfRestrictedFunction",
        "SoftwareConfiguration",
        "HardwareConfiguration",
        "RemoteServiceOperationStarted",
        "EmergencyOverrideStopped",
        "UserSecurityAttributesChanged",
        "SecurityRolesChanged",
        "ObjectSecurityAttributesChanged",
        "AuditRecordingStarted",
        "Login",
        "AuthorizationDecision",
        "ApplicationStop",
        "AuthenticationDecision",
        "ApplicationStart",
        "LocalServiceOperationStopped",
        "LocalServiceOperationStarted",
        "RemoteServiceOperationStopped",
        "SecurityConfiguration",
        "NetworkConfiguration",
        "EmergencyOverrideStarted",
        "NodeAuthentication",
        "Detach",
        "AccessControlDecision",
        "Attach",
        "SessionStop",
        "Logout",
        "SessionStart",
    ]
}

name_for_cid[402] = "AuditActiveParticipantRoleIDCode"
cid_concepts[402] = {
    "DCM": [
        "SourceMedia",
        "DestinationMedia",
        "SourceRoleID",
        "DestinationRoleID",
        "ApplicationLauncher",
        "Application",
    ]
}

name_for_cid[403] = "SecurityAlertTypeCode"
cid_concepts[403] = {
    "DCM": [
        "AuditRecordingStopped",
        "UseOfRestrictedFunction",
        "SoftwareConfiguration",
        "HardwareConfiguration",
        "RemoteServiceOperationStarted",
        "EmergencyOverrideStopped",
        "UserSecurityAttributesChanged",
        "SecurityRolesChanged",
        "ObjectSecurityAttributesChanged",
        "AuditRecordingStarted",
        "Login",
        "AuthorizationDecision",
        "ApplicationStop",
        "AuthenticationDecision",
        "ApplicationStart",
        "LocalServiceOperationStopped",
        "LocalServiceOperationStarted",
        "RemoteServiceOperationStopped",
        "SecurityConfiguration",
        "NetworkConfiguration",
        "EmergencyOverrideStarted",
        "NodeAuthentication",
        "Detach",
        "AccessControlDecision",
        "Attach",
        "SessionStop",
        "Logout",
        "SessionStart",
    ]
}

name_for_cid[404] = "AuditParticipantObjectIDTypeCode"
cid_concepts[404] = {"DCM": ["StudyInstanceUID", "NodeID", "SOPClassUID"]}

name_for_cid[405] = "MediaTypeCode"
cid_concepts[405] = {
    "DCM": [
        "PaperDocument",
        "URI",
        "SecureDigitalCard",
        "MultiMediaCard",
        "CompactFlash",
        "DVD",
        "CD",
        "Film",
        "Email",
        "USBDiskEmulation",
    ]
}

name_for_cid[501] = "VolumetricViewDescription"
cid_concepts[501] = {
    "DCM": ["ApicalFourChamberRVFocused", "ApicalFourChamberRVModified"],
    "SCT": [
        "RightPosteriorOblique",
        "LeftLateralEmissiveProjection",
        "ParasternalLongAxisViewOfTheRVOutflowTract",
        "ParasternalShortAxisAtThePapillaryMuscleLevel",
        "Tangential",
        "CranioCaudal",
        "LateroMedialOblique",
        "Acanthioparietal",
        "SuprasternalSagittalView",
        "Orbitoparietal",
        "PosteriorProjection",
        "FiveChamber",
        "ObliqueCranioCaudal",
        "OffAxis",
        "LateralOblique",
        "RightOblique",
        "Lateral",
        "MedioLateralObliqueProjection",
        "Verticosubmental",
        "MedialLateralEmissiveProjection",
        "Frontal",
        "HorizontalLongAxis",
        "ParietoOrbital",
        "ParasternalShortAxisAtTheAorticValveLevel",
        "SubcostalShortAxisViewAtPapillaryMuscleLevel",
        "AnteroPosterior",
        "SubcostalShortAxisViewAtVenousInflowLevel",
        "LeftPosteriorObliqueEmissiveProjection",
        "ParasternalShortAxisAtTheLevelOfTheMitralChords",
        "LeftPosteriorOblique",
        "ParasternalLongAxis",
        "ObliqueCaudoCranial",
        "SuprasternalShortAxis",
        "SagittalObliqueAxisEmissiveProjection",
        "VerticalLongAxis",
        "Sagittal",
        "LeftAnteriorObliqueEmissiveProjection",
        "SuprasternalLongAxis",
        "TransesophagealShortAxisView",
        "Axial",
        "Plantodorsal",
        "RightLateralEmissiveProjection",
        "SuprasternalLongAxisViewOfAorticArch",
        "CaudoCranial",
        "TissueSpecimen",
        "SubcostalLongAxis",
        "LeftLateral",
        "LateralMedial",
        "Dorsoplantar",
        "ObliqueProjection",
        "AnteriorProjection",
        "LateroMedialObliqueEmissiveProjection",
        "ObliqueAxial",
        "Coronal",
        "SubcostalShortAxis",
        "LongAxis",
        "RightAnteriorObliqueEmissiveProjection",
        "RightVentricularInflowTractView",
        "TwoChamber",
        "RightLateral",
        "ObliqueAxialEmissiveProjection",
        "SubcostalObliqueCoronalView",
        "Parietoacanthial",
        "PosteroAnterior",
        "Submentovertical",
        "AnteroPosteriorOblique",
        "RightPosteriorObliqueEmissiveProjection",
        "LeftOblique",
        "SubcostalShortAxisViewAtMitralValveLevel",
        "MedialOblique",
        "FrontalObliqueAxial",
        "FrontalOblique",
        "FourChamber",
        "RightVentricularOutflowTractView",
        "ParasternalShortAxis",
        "ParasternalShortAxisAtTheMitralValveLevel",
        "SubcostalViewOfCardiacOutletsDirectedAnteriorly",
        "PosteroAnteriorOblique",
        "AxialProjection",
        "OcclusalProjection",
        "SuprasternalCoronalView",
        "Parasagittal",
        "LeftAnteriorOblique",
        "ParasternalLongAxisViewOfTheRVInflowTract",
        "Oblique",
        "Transverse",
        "RightAnteriorOblique",
        "ApicalLongAxis",
        "SubcostalShortAxisViewAtAorticValveLevel",
        "SagittalObliqueAxial",
        "LateralMedialEmissiveProjection",
        "MedioLateralProjection",
        "ShortAxis",
        "MedioLateralObliqueEmissiveProjection",
        "LateralProjection",
        "Longitudinal",
    ],
}

name_for_cid[502] = "VolumetricViewModifier"
cid_concepts[502] = {
    "DCM": ["Crosstable"],
    "SCT": [
        "Transoral",
        "Caudal",
        "Sagittal",
        "Caudad",
        "FiveChamber",
        "LongAxis",
        "OffAxis",
        "Parasagittal",
        "Transforamenal",
        "FourChamber",
        "ShortAxis",
        "TwoChamber",
        "Oblique",
        "Cephalad",
        "Coronal",
        "Transthoracic",
        "Transverse",
        "Transorbital",
        "MouthClosed",
        "Cephalic",
        "Longitudinal",
    ],
}

name_for_cid[601] = "BiosafetyLevel"
cid_concepts[601] = {
    "SCT": ["BiosafetyLevel2", "BiosafetyLevel1", "BiosafetyLevel4", "BiosafetyLevel3"]
}

name_for_cid[602] = "BiosafetyControlReason"
cid_concepts[602] = {
    "SCT": ["BiohazardousMaterial", "Carcinogen", "PatientImmunocompromised"],
    "UMLS": ["TransgenicAnimal"],
}

name_for_cid[603] = "AnimalRoomType"
cid_concepts[603] = {
    "DCM": ["AnimalHousingRoom", "PreparationRoom", "ImagingProcedureRoom"],
    "SCT": ["InductionRoom", "IsolationRoom", "RecoveryRoom"],
}

name_for_cid[604] = "DeviceReuse"
cid_concepts[604] = {"DCM": ["Unused", "Reused"]}

name_for_cid[605] = "AnimalBeddingMaterial"
cid_concepts[605] = {
    "DCM": [
        "AspenShavingBedding",
        "CornCobBedding",
        "PaperBasedBedding",
        "PineChipBedding",
        "PineShavingBedding",
        "AspenChipBedding",
    ]
}

name_for_cid[606] = "AnimalShelterType"
cid_concepts[606] = {"DCM": ["Igloo", "RedTranslucentIgloo"], "SCT": ["None_"]}

name_for_cid[607] = "AnimalFeedType"
cid_concepts[607] = {"DCM": ["NIH31", "NIH07", "AIN76", "AIN93G", "AIN93M"]}

name_for_cid[608] = "AnimalFeedSource"
cid_concepts[608] = {
    "DCM": ["LocallyManufacturedProduct"],
    "UMLS": ["CommercialProduct"],
}

name_for_cid[609] = "AnimalFeedingMethod"
cid_concepts[609] = {
    "DCM": ["FoodTreat"],
    "NCIt": ["AdLibitum"],
    "SCT": ["Gavage"],
    "UMLS": ["RestrictedDiet"],
}

name_for_cid[610] = "WaterType"
cid_concepts[610] = {
    "DCM": ["ReverseOsmosisPurifiedWater", "ReverseOsmosisPurifiedHclAcidifiedWater"],
    "SCT": ["TapWater", "DistilledWater"],
}

name_for_cid[611] = "AnesthesiaCategoryCodeTypeforSmallAnimalAnesthesia"
cid_concepts[611] = {
    "SCT": [
        "MonitoredAnesthesiaCareMAC",
        "GeneralAnesthesia",
        "LocalAnesthesia",
        "Sedation",
        "SpinalAnesthesia",
        "EpiduralAnesthesia",
        "RegionalAnesthesia",
        "TopicalLocalAnesthesia",
    ]
}

name_for_cid[612] = "AnesthesiaCategoryCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[612] = {
    "SCT": [
        "MonitoredAnesthesiaCareMAC",
        "GeneralAnesthesia",
        "LocalAnesthesia",
        "Sedation",
        "SpinalAnesthesia",
        "EpiduralAnesthesia",
        "RegionalAnesthesia",
        "TopicalLocalAnesthesia",
    ]
}

name_for_cid[613] = "AnesthesiaInductionCodeTypeforSmallAnimalAnesthesia"
cid_concepts[613] = {
    "SCT": [
        "IntramuscularRoute",
        "PerRectum",
        "IntravenousRoute",
        "IntraperitonealRoute",
        "ByInhalation",
    ]
}

name_for_cid[614] = "AnesthesiaInductionCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[614] = {
    "SCT": ["IntramuscularRoute", "PerRectum", "IntravenousRoute", "ByInhalation"]
}

name_for_cid[615] = "AnesthesiaMaintenanceCodeTypeforSmallAnimalAnesthesia"
cid_concepts[615] = {
    "SCT": [
        "InhalationAnesthesiaSystemClosedRebreathingPrimaryAgent",
        "InhalationAnesthesiaSystemClosedNoRebreathingPrimaryAgent",
    ]
}

name_for_cid[616] = "AnesthesiaMaintenanceCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[616] = {
    "SCT": [
        "InhalationAnesthesiaSystemClosedRebreathingPrimaryAgent",
        "InhalationAnesthesiaSystemClosedNoRebreathingPrimaryAgent",
    ]
}

name_for_cid[617] = "AirwayManagementMethodCodeTypeforSmallAnimalAnesthesia"
cid_concepts[617] = {
    "DCM": ["NoseCone", "NasalCannula"],
    "SCT": [
        "ViaTracheostomy",
        "AnestheticFaceMask",
        "ArtificialRespiration",
        "ControlledVentilation",
        "OxygenTherapy",
        "IntubationOfRespiratoryTract",
        "LaryngealMaskAirwayLMA",
    ],
}

name_for_cid[618] = "AirwayManagementMethodCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[618] = {
    "DCM": ["NasalCannula"],
    "SCT": [
        "ViaTracheostomy",
        "AnestheticFaceMask",
        "ArtificialRespiration",
        "ControlledVentilation",
        "OxygenTherapy",
        "IntubationOfRespiratoryTract",
        "LaryngealMaskAirwayLMA",
    ],
}

name_for_cid[619] = "AirwayManagementSubMethodCodeTypeforSmallAnimalAnesthesia"
cid_concepts[619] = {
    "SCT": [
        "ContinuousFlowApneicVentilation",
        "HighFrequencyVentilation",
        "TranstrachealJetVentilation",
    ],
    "UMLS": ["InverseRatioVentilation"],
}

name_for_cid[620] = "AirwayManagementSubMethodCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[620] = {
    "SCT": [
        "ContinuousFlowApneicVentilation",
        "HighFrequencyVentilation",
        "TranstrachealJetVentilation",
    ],
    "UMLS": ["InverseRatioVentilation"],
}

name_for_cid[621] = "MedicationTypeforSmallAnimalAnesthesia"
cid_concepts[621] = {
    "DCM": ["CarrierGas"],
    "SCT": [
        "CaloricAgent",
        "CarbapenemAntibiotic",
        "BetaLactamAntibiotic",
        "AminoglycosideAntibiotic",
        "Potassium",
        "LocalAnesthetic",
        "Anticoagulant",
        "PenicillinAntibiotic",
        "ThyroidHormone",
        "MetronidazoleAntibiotic",
        "Calcium",
        "Magnesium",
        "AntidiureticHormone",
        "Oxytocic",
        "InotropicAgent",
        "Antiarrhythmic",
        "Barbiturate",
        "Benzodiazepine",
        "EstrogenReceptorAgonist",
        "Decongestant",
        "SulfonamideAntibiotic",
        "OcularLubricant",
        "Diuretic",
        "VancomycinAntibiotic",
        "Antifungal",
        "Hypoglycemic",
        "Steroid",
        "CephalosporinAntibiotic",
        "LowMolecularWeightHeparin",
        "RespiratoryStimulant",
        "Anticholinergic",
        "NeuromuscularBlockingNMBNonDepolarizing",
        "DrugDiluent",
        "GeneralAnesthetic",
        "HaemostaticAgent",
        "MacrolideAntibiotic",
        "Antihypertensive",
        "Antibiotic",
        "HistomineReceptorAntagonist",
        "AdrenergicAgent",
        "QuinoloneAntibiotic",
        "NSAID",
        "Vitamin",
        "Antiemetic",
        "BloodProduct",
        "AntiHeparinAgent",
        "Vasodilator",
        "BenzodiazepineAntagonist",
        "CalciumChannelBlocker",
        "CholinergicAgent",
        "PlasmaExpander",
        "Anticonvulsant",
        "BetaBlocker",
        "Bronchodilator",
        "TetracyclineAntibiotic",
        "SkeletalMuscleRelaxant",
        "NarcoticOpiateAntagonist",
        "LincomycinAntibiotic",
        "NeuromuscularBlockingNMBDepolarizing",
        "Vasopressor",
        "LinezolidAntibiotic",
        "GastrointestinalProkinetic",
        "Analgesic",
    ],
    "UMLS": ["H2Antagonist", "NarcoticAnalgesic", "Antihypoglycemic"],
}

name_for_cid[622] = "MedicationTypeCodeTypefromAnesthesiaQualityInitiative"
cid_concepts[622] = {
    "SCT": [
        "CaloricAgent",
        "CarbapenemAntibiotic",
        "BetaLactamAntibiotic",
        "AminoglycosideAntibiotic",
        "Potassium",
        "LocalAnesthetic",
        "Anticoagulant",
        "PenicillinAntibiotic",
        "ThyroidHormone",
        "MetronidazoleAntibiotic",
        "Calcium",
        "Magnesium",
        "AntidiureticHormone",
        "Oxytocic",
        "InotropicAgent",
        "Antiarrhythmic",
        "Barbiturate",
        "Benzodiazepine",
        "EstrogenReceptorAgonist",
        "Decongestant",
        "SulfonamideAntibiotic",
        "OcularLubricant",
        "Diuretic",
        "VancomycinAntibiotic",
        "Antifungal",
        "Hypoglycemic",
        "Steroid",
        "CephalosporinAntibiotic",
        "LowMolecularWeightHeparin",
        "RespiratoryStimulant",
        "Anticholinergic",
        "NeuromuscularBlockingNMBNonDepolarizing",
        "DrugDiluent",
        "GeneralAnesthetic",
        "HaemostaticAgent",
        "MacrolideAntibiotic",
        "Antihypertensive",
        "Antibiotic",
        "HistomineReceptorAntagonist",
        "AdrenergicAgent",
        "QuinoloneAntibiotic",
        "NSAID",
        "Vitamin",
        "Antiemetic",
        "BloodProduct",
        "AntiHeparinAgent",
        "Vasodilator",
        "BenzodiazepineAntagonist",
        "CalciumChannelBlocker",
        "CholinergicAgent",
        "PlasmaExpander",
        "Anticonvulsant",
        "BetaBlocker",
        "Bronchodilator",
        "TetracyclineAntibiotic",
        "SkeletalMuscleRelaxant",
        "NarcoticOpiateAntagonist",
        "LincomycinAntibiotic",
        "NeuromuscularBlockingNMBDepolarizing",
        "Vasopressor",
        "LinezolidAntibiotic",
        "GastrointestinalProkinetic",
        "Analgesic",
    ],
    "UMLS": ["H2Antagonist", "NarcoticAnalgesic", "Antihypoglycemic"],
}

name_for_cid[623] = "MedicationforSmallAnimalAnesthesia"
cid_concepts[623] = {
    "SCT": [
        "Chloroform",
        "Succinylcholine",
        "Pancuronium",
        "Lidocaine",
        "Pentobarbital",
        "Xylazine",
        "Chlorpromazine",
        "Diazepam",
        "Enflurane",
        "Urethane",
        "Acepromazine",
        "Thiamylal",
        "Sevoflurane",
        "Etomidate",
        "Tribromoethanol",
        "Methohexital",
        "Bupivacaine",
        "LidocainePrilocaine",
        "Azaperone",
        "Halothane",
        "Tiletamine",
        "Zolazepam",
        "Isoflurane",
        "Propofol",
        "Midazolam",
        "Desflurane",
        "Alphachloralose",
        "ChloralHydrate",
        "Methoxyflurane",
        "Alphadolone",
        "Butabarbital",
        "Ketamine",
        "DiethylEther",
        "MedicalAir",
        "Thiopental",
        "CarbonDioxide",
        "Droperidol",
        "NitrousOxide",
        "OxygenGas",
    ],
    "UMLS": ["RoomAir", "Alphaxalone", "Fluanisone", "Metomidate"],
}

name_for_cid[624] = "InhalationalAnesthesiaAgentforSmallAnimalAnesthesia"
cid_concepts[624] = {
    "SCT": [
        "Chloroform",
        "Isoflurane",
        "Methoxyflurane",
        "Enflurane",
        "DiethylEther",
        "CarbonDioxide",
        "Desflurane",
        "Halothane",
        "Sevoflurane",
        "NitrousOxide",
    ]
}

name_for_cid[625] = "InjectableAnesthesiaAgentforSmallAnimalAnesthesia"
cid_concepts[625] = {
    "SCT": [
        "Pentobarbital",
        "Xylazine",
        "Diazepam",
        "Propofol",
        "Urethane",
        "Midazolam",
        "Thiamylal",
        "Etomidate",
        "Alphachloralose",
        "ChloralHydrate",
        "Tribromoethanol",
        "Methohexital",
        "Alphadolone",
        "Butabarbital",
        "Ketamine",
        "Thiopental",
        "Azaperone",
        "Droperidol",
        "Tiletamine",
        "Zolazepam",
    ],
    "UMLS": ["Alphaxalone", "Fluanisone", "Metomidate"],
}

name_for_cid[626] = "PremedicationAgentforSmallAnimalAnesthesia"
cid_concepts[626] = {"SCT": ["Chlorpromazine", "Acepromazine"]}

name_for_cid[627] = "NeuromuscularBlockingAgentforSmallAnimalAnesthesia"
cid_concepts[627] = {"SCT": ["Succinylcholine", "Pancuronium"]}

name_for_cid[628] = "AncillaryMedicationsforSmallAnimalAnesthesia"
cid_concepts[628] = {}

name_for_cid[629] = "CarrierGasesforSmallAnimalAnesthesia"
cid_concepts[629] = {"SCT": ["MedicalAir", "OxygenGas"], "UMLS": ["RoomAir"]}

name_for_cid[630] = "LocalAnestheticsforSmallAnimalAnesthesia"
cid_concepts[630] = {"SCT": ["Lidocaine", "Bupivacaine", "LidocainePrilocaine"]}

name_for_cid[631] = "ProcedurePhaseRequiringAnesthesia"
cid_concepts[631] = {
    "SCT": [
        "AfterProcedure",
        "BeforeProcedure",
        "Intraoperative",
        "Preoperative",
        "Postoperative",
        "DuringProcedure",
    ]
}

name_for_cid[632] = "SurgicalProcedurePhaseRequiringAnesthesia"
cid_concepts[632] = {"SCT": ["Intraoperative", "Preoperative", "Postoperative"]}

name_for_cid[634] = "AnimalHandlingPhase"
cid_concepts[634] = {
    "DCM": [
        "InHomeCage",
        "DuringTransport",
        "StagingPriorToImaging",
        "PreparationForImaging",
    ],
    "SCT": ["AnesthesiaInduction", "ImagingProcedure"],
    "UMLS": ["AnesthesiaRecoveryPeriod"],
}

name_for_cid[635] = "HeatingMethod"
cid_concepts[635] = {
    "DCM": [
        "HeatedWaterBlanket",
        "PreHeatedPad",
        "Unheated",
        "ForcedAirHeater",
        "HeatedImagingDevice",
        "HeatedPatientSupport",
    ],
    "SCT": [
        "NonElectricHeatingPad",
        "ForcedAirWarmingBlanket",
        "ElectricHeatingPad",
        "WarmerDevice",
        "AirHeatingPad",
        "ElectricBlanket",
        "WarmingBlanket",
    ],
    "UMLS": ["HeatLamp"],
}

name_for_cid[636] = "TemperatureSensorDeviceComponentTypeforSmallAnimalProcedure"
cid_concepts[636] = {
    "DCM": ["CarrierTemperatureSensor"],
    "SCT": ["RectalTemperature"],
    "UMLS": ["Thermography"],
}

name_for_cid[637] = "ExogenousSubstanceType"
cid_concepts[637] = {
    "DCM": ["TumorGraft"],
    "SCT": ["Fibril", "Virus", "Cytokine", "Toxin", "GraftMaterial"],
}

name_for_cid[638] = "ExogenousSubstance"
cid_concepts[638] = {
    "DCM": [
        "NonUterineLeiomyosarcoma",
        "HumanAlphaSynucleinPreformedFibrils",
        "MouseAlphaSynucleinPreformedFibrils",
        "HumanTauPreformedFibrils",
        "MouseTauPreformedFibrils",
    ],
    "NCIt": [
        "BronchioloalveolarAdenocarcinoma",
        "SoftTissueSarcomaExcludingRhabdomyosarcoma",
    ],
    "PUBCHEM_CID": ["_6Hydroxydopamine"],
    "SCT": [
        "CarcinosarcomaOfUterus",
        "NonSmallCellCarcinoma",
        "Osteosarcoma",
        "SquamousCellCarcinomaOfLung",
        "AmelanoticMelanoma",
        "SpindleCellCarcinoma",
        "TransitionalCellCarcinoma",
        "Lipopolysaccharide",
        "Carcinosarcoma",
        "TumorNecrosisFactorAlpha",
        "EwingSarcomaPeripheralNeuroectodermalTumor",
        "BladderCancer",
        "CancerOfColon",
        "VascularEndothelialGrowthFactor",
        "CancerOfCervix",
        "NeuroendocrineTumor",
        "LungCancer",
        "InterferonGamma",
        "RenalCellCarcinoma",
        "Sarcoma",
        "AdenocarcinomaOfRectum",
        "SquamousCellCarcinomaOfMouth",
        "SquamousCellCarcinomaOfAnus",
        "CancerOfSalivaryGland",
        "Carcinoma",
        "SquamousCellCarcinomaOfHeadAndNeck",
        "Lysophosphatidylcholine",
        "InfiltratingDuctalCarcinomaOfBreast",
        "MalignantMesothelioma",
        "Cystadenocarcinoma",
        "SarcomaOfSoftTissue",
        "NonSmallCellLungCancer",
        "AdenoAssociatedVirusGroup",
        "SmallCellCarcinoma",
        "AdenocarcinomaOfLargeIntestine",
        "Fibrosarcoma",
        "MerkelCellCarcinoma",
        "Astrocytoma",
        "Adenocarcinoma",
        "MixedSmallCellCarcinoma",
        "Glioblastoma",
        "AdenocarcinomaOfLung",
        "SquamousCellCarcinomaOfSkin",
        "MalignantMelanoma",
        "LargeCellCarcinoma",
        "SquamousCellCarcinoma",
        "PapillaryThyroidCarcinoma",
        "EwingSarcoma",
        "MalignantPeripheralNerveSheathTumor",
        "OvarianCancer",
        "SquamousCellCarcinomaOfLaryngealCartilage",
        "AlveolarSoftPartSarcoma",
        "Leukemia",
        "AdenocarcinomaOfPancreas",
        "TheilerMurineEncephalomyelitisVirus",
    ],
    "UMLS": ["EthidiumBromide"],
}

name_for_cid[639] = "TumorGraftHistologicType"
cid_concepts[639] = {
    "DCM": ["NonUterineLeiomyosarcoma"],
    "NCIt": [
        "BronchioloalveolarAdenocarcinoma",
        "SoftTissueSarcomaExcludingRhabdomyosarcoma",
    ],
    "SCT": [
        "CarcinosarcomaOfUterus",
        "NonSmallCellCarcinoma",
        "Osteosarcoma",
        "SquamousCellCarcinomaOfLung",
        "AmelanoticMelanoma",
        "SpindleCellCarcinoma",
        "TransitionalCellCarcinoma",
        "Carcinosarcoma",
        "EwingSarcomaPeripheralNeuroectodermalTumor",
        "BladderCancer",
        "CancerOfColon",
        "CancerOfCervix",
        "NeuroendocrineTumor",
        "LungCancer",
        "RenalCellCarcinoma",
        "Sarcoma",
        "AdenocarcinomaOfRectum",
        "SquamousCellCarcinomaOfMouth",
        "SquamousCellCarcinomaOfAnus",
        "CancerOfSalivaryGland",
        "Carcinoma",
        "SquamousCellCarcinomaOfHeadAndNeck",
        "InfiltratingDuctalCarcinomaOfBreast",
        "MalignantMesothelioma",
        "Cystadenocarcinoma",
        "SarcomaOfSoftTissue",
        "NonSmallCellLungCancer",
        "SmallCellCarcinoma",
        "AdenocarcinomaOfLargeIntestine",
        "Fibrosarcoma",
        "MerkelCellCarcinoma",
        "Astrocytoma",
        "Adenocarcinoma",
        "MixedSmallCellCarcinoma",
        "Glioblastoma",
        "AdenocarcinomaOfLung",
        "SquamousCellCarcinomaOfSkin",
        "MalignantMelanoma",
        "LargeCellCarcinoma",
        "SquamousCellCarcinoma",
        "PapillaryThyroidCarcinoma",
        "EwingSarcoma",
        "MalignantPeripheralNerveSheathTumor",
        "OvarianCancer",
        "SquamousCellCarcinomaOfLaryngealCartilage",
        "AlveolarSoftPartSarcoma",
        "Leukemia",
        "AdenocarcinomaOfPancreas",
    ],
}

name_for_cid[640] = "Fibril"
cid_concepts[640] = {
    "DCM": [
        "HumanAlphaSynucleinPreformedFibrils",
        "MouseAlphaSynucleinPreformedFibrils",
        "HumanTauPreformedFibrils",
        "MouseTauPreformedFibrils",
    ]
}

name_for_cid[641] = "Virus"
cid_concepts[641] = {
    "SCT": ["TheilerMurineEncephalomyelitisVirus", "AdenoAssociatedVirusGroup"]
}

name_for_cid[642] = "Cytokine"
cid_concepts[642] = {
    "SCT": [
        "InterferonGamma",
        "VascularEndothelialGrowthFactor",
        "TumorNecrosisFactorAlpha",
    ]
}

name_for_cid[643] = "Toxin"
cid_concepts[643] = {
    "PUBCHEM_CID": ["_6Hydroxydopamine"],
    "SCT": ["Lysophosphatidylcholine", "Lipopolysaccharide"],
    "UMLS": ["EthidiumBromide"],
}

name_for_cid[644] = "ExogenousSubstanceAdministrationSite"
cid_concepts[644] = {
    "NCIt": ["MouseMammaryFatPad"],
    "SCT": ["Liver", "BoneMarrow", "Tumor", "Brain", "Flank"],
}

name_for_cid[645] = "ExogenousSubstanceOriginTissue"
cid_concepts[645] = {
    "DCM": ["EmbryonicKidney", "BoneAndSoftTissue"],
    "SCT": [
        "Lung",
        "PulmonaryMetastasis",
        "Breast",
        "AsciticFluid",
        "Mesothelium",
        "PleuralEffusion",
        "Mouth",
        "Skin",
        "Prostate",
        "PeripheralNerveMyelinSheath",
        "LymphoidTissue",
        "LymphNode",
        "HeadAndNeck",
        "Colon",
        "Uterus",
        "CentralNervousSystem",
        "SoftTissue",
        "SalivaryGland",
        "Cervix",
        "Bladder",
        "ConnectiveTissue",
        "Brain",
        "Larynx",
        "Lymph",
        "Rectum",
        "BoneTissue",
        "Ovary",
        "Pancreas",
        "Anus",
        "Thyroid",
        "Kidney",
        "Metastasis",
    ],
}

name_for_cid[646] = "PreclinicalSmallAnimalImagingProcedure"
cid_concepts[646] = {
    "DCM": ["SPECTOfWholeBody", "SPECTCTOfWholeBody"],
    "LN": [
        "MRIWholeBody",
        "HeadCT",
        "RadionuclideScanOfWholeBody",
        "BrainMRI",
        "RadionuclideBrainScan",
        "PETWholeBody",
        "WholeBodyCT",
        "BrainPET",
        "BrainSPECT",
    ],
    "SCT": ["UltrasonographyOfTotalBody"],
}

name_for_cid[647] = "PositionReferenceIndicatorforFrameofReference"
cid_concepts[647] = {"FMA": ["Bregma", "Lambda"]}

name_for_cid[701] = "ContentAssessmentType"
cid_concepts[701] = {
    "DCM": ["RTPreTreatmentConsistencyCheck", "RTPreTreatmentDoseCheck"]
}

name_for_cid[702] = "RTContentAssessmentType"
cid_concepts[702] = {
    "DCM": ["RTPreTreatmentConsistencyCheck", "RTPreTreatmentDoseCheck"]
}

name_for_cid[703] = "AssessmentBasis"
cid_concepts[703] = {"DCM": ["AssessmentByRules", "AssessmentByComparison"]}

name_for_cid[800] = "ProtocolAssertion"
cid_concepts[800] = {
    "DCM": [
        "ApprovedForUseInTheClinicalTrial",
        "IneligibleForReimbursement",
        "ApprovedForUseAtTheInstitution",
        "EligibleForReimbursementOnPerPatientBasis",
        "DisapprovedForUseOnPregnantPatients",
        "AppropriateForTheDevice",
        "ApprovedForUseOnPregnantPatients",
        "OutsideOperationalLimitsOfTheDevice",
        "OptimizedForTheDeviceInstance",
        "InappropriateForTheDevice",
        "InsideOperationalLimitsOfTheDevice",
        "DisapprovedForAnyUse",
        "NotOptimizedForTheDeviceInstance",
        "InconsistentWithLabelingOfTheDevice",
        "ApprovedForExperimentalUse",
        "InappropriateForTheIndications",
        "DeprecatedProtocol",
        "ConsistentWithLabelingOfTheDevice",
        "DisapprovedForUseInTheClinicalTrial",
        "EligibleForReimbursement",
        "AppropriateForTheIndications",
        "DisapprovedForUseAtTheInstitution",
        "DisapprovedForExperimentalUse",
    ]
}

name_for_cid[1000] = "CTTransversePlaneReferenceBasis"
cid_concepts[1000] = {
    "DCM": ["AcquiredVolume"],
    "FMA": ["Nasion"],
    "SCT": [
        "KneeJoint",
        "TemporalBone",
        "Sacrum",
        "LevelOfL1L2IntervertebralDisc",
        "ApexOfHeart",
        "SternoclavicularJoint",
        "LevelOfT7T8IntervertebralDisc",
        "OrthopedicDevice",
        "C1Vertebra",
        "LevelOfC5C6IntervertebralDisc",
        "WristJoint",
        "LevelOfT11T12IntervertebralDisc",
        "Ischium",
        "Diaphragm",
        "LevelOfT3T4IntervertebralDisc",
        "CarotidArtery",
        "LevelOfT2T3IntervertebralDisc",
        "Talus",
        "FemoralHead",
        "MaxillarySinus",
        "LevelOfT10T11IntervertebralDisc",
        "MalleolarStructureOfTibia",
        "LevelOfT6T7IntervertebralDisc",
        "LevelOfC2C3IntervertebralDisc",
        "Mandible",
        "Metacarpal",
        "Foot",
        "CommonIliacArteryBifurcation",
        "LevelOfC6C7IntervertebralDisc",
        "MastoidBone",
        "XiphoidProcess",
        "Acetabulum",
        "LevelOfL4L5IntervertebralDisc",
        "ForamenMagnum",
        "AnkleJoint",
        "LevelOfL5S1IntervertebralDisc",
        "LevelOfT9T10IntervertebralDisc",
        "Lung",
        "LevelOfT5T6IntervertebralDisc",
        "PituitaryFossa",
        "LevelOfL3L4IntervertebralDisc",
        "SymphysisPubis",
        "LevelOfT12L1IntervertebralDisc",
        "TibialPlateau",
        "LesserTrochanter",
        "AorticArch",
        "Carina",
        "LevelOfC3C4IntervertebralDisc",
        "Heart",
        "Toe",
        "ShoulderRegionStructure",
        "ElbowJoint",
        "MorphologicallyAbnormalStructure",
        "LateralCanthus",
        "Sternum",
        "ThoracicInlet",
        "IliacCrest",
        "OrbitalStructure",
        "LevelOfT8T9IntervertebralDisc",
        "LevelOfL2L3IntervertebralDisc",
        "HipJoint",
        "LevelOfC7T1IntervertebralDisc",
        "LevelOfC4C5IntervertebralDisc",
        "MastoidCellsAndAntra",
        "SuprasternalNotch",
        "FrontalSinus",
        "CardiacPacemaker",
        "Skull",
        "Pancreas",
        "LevelOfT4T5IntervertebralDisc",
        "InternalAuditoryMeatus",
        "Liver",
        "LevelOfT1T2IntervertebralDisc",
        "CoronaryArteryGraft",
        "ExternalAuditoryMeatus",
        "VertexOfHead",
        "Scapula",
        "Patella",
        "Kidney",
        "AdrenalGland",
        "StentDevice",
        "AcromioclavicularJoint",
        "Scaphoid",
        "ImplantDevice",
    ],
}

name_for_cid[1001] = "AnatomicalReferenceBasis"
cid_concepts[1001] = {
    "FMA": ["Nasion"],
    "SCT": [
        "KneeJoint",
        "TemporalBone",
        "Sacrum",
        "LevelOfL1L2IntervertebralDisc",
        "ApexOfHeart",
        "SternoclavicularJoint",
        "LevelOfT7T8IntervertebralDisc",
        "C1Vertebra",
        "LevelOfC5C6IntervertebralDisc",
        "WristJoint",
        "LevelOfT11T12IntervertebralDisc",
        "Ischium",
        "Diaphragm",
        "LevelOfT3T4IntervertebralDisc",
        "CarotidArtery",
        "LevelOfT2T3IntervertebralDisc",
        "Talus",
        "FemoralHead",
        "MaxillarySinus",
        "LevelOfT10T11IntervertebralDisc",
        "MalleolarStructureOfTibia",
        "LevelOfT6T7IntervertebralDisc",
        "LevelOfC2C3IntervertebralDisc",
        "Mandible",
        "Metacarpal",
        "Foot",
        "CommonIliacArteryBifurcation",
        "LevelOfC6C7IntervertebralDisc",
        "MastoidBone",
        "XiphoidProcess",
        "Acetabulum",
        "LevelOfL4L5IntervertebralDisc",
        "ForamenMagnum",
        "AnkleJoint",
        "LevelOfL5S1IntervertebralDisc",
        "LevelOfT9T10IntervertebralDisc",
        "Lung",
        "LevelOfT5T6IntervertebralDisc",
        "PituitaryFossa",
        "LevelOfL3L4IntervertebralDisc",
        "SymphysisPubis",
        "LevelOfT12L1IntervertebralDisc",
        "TibialPlateau",
        "LesserTrochanter",
        "AorticArch",
        "Carina",
        "LevelOfC3C4IntervertebralDisc",
        "Heart",
        "Toe",
        "ShoulderRegionStructure",
        "ElbowJoint",
        "LateralCanthus",
        "Sternum",
        "ThoracicInlet",
        "IliacCrest",
        "OrbitalStructure",
        "LevelOfT8T9IntervertebralDisc",
        "LevelOfL2L3IntervertebralDisc",
        "HipJoint",
        "LevelOfC7T1IntervertebralDisc",
        "LevelOfC4C5IntervertebralDisc",
        "MastoidCellsAndAntra",
        "SuprasternalNotch",
        "FrontalSinus",
        "Skull",
        "Pancreas",
        "LevelOfT4T5IntervertebralDisc",
        "InternalAuditoryMeatus",
        "Liver",
        "LevelOfT1T2IntervertebralDisc",
        "CoronaryArteryGraft",
        "ExternalAuditoryMeatus",
        "VertexOfHead",
        "Scapula",
        "Patella",
        "Kidney",
        "AdrenalGland",
        "AcromioclavicularJoint",
        "Scaphoid",
    ],
}

name_for_cid[1002] = "AnatomicalReferenceBasisHead"
cid_concepts[1002] = {
    "FMA": ["Nasion"],
    "SCT": [
        "TemporalBone",
        "MaxillarySinus",
        "PituitaryFossa",
        "Mandible",
        "MastoidCellsAndAntra",
        "MastoidBone",
        "FrontalSinus",
        "Skull",
        "InternalAuditoryMeatus",
        "CarotidArtery",
        "ExternalAuditoryMeatus",
        "VertexOfHead",
        "ForamenMagnum",
        "LateralCanthus",
        "OrbitalStructure",
    ],
}

name_for_cid[1003] = "AnatomicalReferenceBasisSpine"
cid_concepts[1003] = {
    "SCT": [
        "LevelOfT9T10IntervertebralDisc",
        "LevelOfT8T9IntervertebralDisc",
        "LevelOfT5T6IntervertebralDisc",
        "LevelOfL1L2IntervertebralDisc",
        "LevelOfL2L3IntervertebralDisc",
        "LevelOfL3L4IntervertebralDisc",
        "LevelOfT7T8IntervertebralDisc",
        "LevelOfC7T1IntervertebralDisc",
        "LevelOfT10T11IntervertebralDisc",
        "LevelOfT12L1IntervertebralDisc",
        "LevelOfT6T7IntervertebralDisc",
        "LevelOfC2C3IntervertebralDisc",
        "LevelOfC4C5IntervertebralDisc",
        "C1Vertebra",
        "LevelOfC5C6IntervertebralDisc",
        "LevelOfT11T12IntervertebralDisc",
        "LevelOfC6C7IntervertebralDisc",
        "LevelOfC3C4IntervertebralDisc",
        "LevelOfT3T4IntervertebralDisc",
        "LevelOfT4T5IntervertebralDisc",
        "LevelOfT1T2IntervertebralDisc",
        "LevelOfL4L5IntervertebralDisc",
        "LevelOfT2T3IntervertebralDisc",
        "LevelOfL5S1IntervertebralDisc",
    ]
}

name_for_cid[1004] = "AnatomicalReferenceBasisChest"
cid_concepts[1004] = {
    "SCT": [
        "Lung",
        "ApexOfHeart",
        "SternoclavicularJoint",
        "AorticArch",
        "Carina",
        "SuprasternalNotch",
        "Diaphragm",
        "Heart",
        "XiphoidProcess",
        "CoronaryArteryGraft",
        "ShoulderRegionStructure",
        "Scapula",
        "AcromioclavicularJoint",
        "Sternum",
        "ThoracicInlet",
    ]
}

name_for_cid[1005] = "AnatomicalReferenceBasisAbdomen/Pelvis"
cid_concepts[1005] = {
    "SCT": [
        "FemoralHead",
        "Sacrum",
        "SymphysisPubis",
        "HipJoint",
        "LesserTrochanter",
        "Ischium",
        "CommonIliacArteryBifurcation",
        "Pancreas",
        "Acetabulum",
        "Liver",
        "Kidney",
        "AdrenalGland",
        "IliacCrest",
    ]
}

name_for_cid[1006] = "AnatomicalReferenceBasisExtremity"
cid_concepts[1006] = {
    "SCT": [
        "Foot",
        "KneeJoint",
        "Toe",
        "MalleolarStructureOfTibia",
        "TibialPlateau",
        "ElbowJoint",
        "Patella",
        "WristJoint",
        "AnkleJoint",
        "Metacarpal",
        "Scaphoid",
        "Talus",
    ]
}

name_for_cid[1010] = "ReferenceGeometryPlane"
cid_concepts[1010] = {
    "DCM": [
        "PlaneThroughDistalExtent",
        "PlaneThroughProximalExtent",
        "PlaneThroughLateralExtent",
        "PlaneThroughMedialExtent",
        "PlaneThroughRightmostExtent",
        "PlaneThroughLeftmostExtent",
        "PlaneThroughPosteriorExtent",
        "PlaneThroughAnteriorExtent",
        "PlaneThroughCenter",
        "PlaneThroughInferiorExtent",
        "PlaneThroughSuperiorExtent",
    ]
}

name_for_cid[1011] = "ReferenceGeometryPoint"
cid_concepts[1011] = {"DCM": ["CenterOfMass", "GeometricCenterpoint"]}

name_for_cid[1015] = "PatientAlignmentMethod"
cid_concepts[1015] = {"DCM": ["LaserCrossHairs"]}

name_for_cid[1200] = "ContraindicationsForCTImaging"
cid_concepts[1200] = {
    "SCT": ["ContrastMediaAllergy", "PatientCurrentlyPregnant", "ImpairedRenalFunction"]
}

name_for_cid[1201] = "ContraindicationsForXAImaging"
cid_concepts[1201] = {
    "SCT": [
        "ContrastMediaAllergy",
        "BloodCoagulationDisorders",
        "CardiacArrhythmia",
        "PatientCurrentlyPregnant",
        "Sepsis",
        "ImpairedRenalFunction",
    ]
}

name_for_cid[3000] = "AudioChannelSource"
cid_concepts[3000] = {
    "DCM": [
        "PhysiologicalAudioSignal",
        "Phonocardiogram",
        "OperatorNarrative",
        "Voice",
        "DopplerAudio",
        "AmbientRoomEnvironment",
    ]
}

name_for_cid[3001] = "ECGLead"
cid_concepts[3001] = {
    "MDC": [
        "FrankLeadC",
        "FrankLeadA",
        "FrankLeadM",
        "FrankLeadF",
        "ChestManubriumLeadPerV5Placement",
        "LeftArmLead",
        "CanineSixthLeftIntercostalSpaceAtCostochondralJunction",
        "RightArmLead",
        "LeftLegLead",
        "CanineOverDorsalSpinousProcessOf7thThoracicVertebra",
        "FrankLeadI",
        "FrankLeadE",
        "DerivedLeadV5",
        "DerivedLeadV6",
        "FrankLeadH",
        "DerivedLeadI",
        "DerivedLeadII",
        "DerivedLeadV1",
        "DerivedLeadV2",
        "DerivedLeadV3",
        "DerivedLeadV4",
        "EASILeadAS",
        "EASILeadAI",
        "EASIUpperSternumLead",
        "AuxiliaryBipolarLead4",
        "EASILeadES",
        "RightLegLead",
        "CanineFifthRightIntercostalSpaceNearEdgeOfSternum",
        "CanineSixthLeftIntercostalSpaceNearEdgeOfSternum",
        "LeadI",
        "UnspecifiedLead",
        "LeadV1",
        "LeadII",
        "LeadV3",
        "LeadV2",
        "LeadV5",
        "LeadV4",
        "LeadV7",
        "LeadV6",
        "DerivedLeadAvf",
        "AvrAugmentedVoltageRight",
        "ChestManubriumLeadPerV6Placement",
        "AvlAugmentedVoltageLeft",
        "DerivedLeadIII",
        "AvfAugmentedVoltageFoot",
        "DerivedLeadAvr",
        "Avr",
        "DerivedLeadAvl",
        "LeadV8",
        "LeadV9",
        "LeadV8R",
        "LeadV9R",
        "LeadIII",
        "AuxiliaryBipolarLead3",
        "LeadCR5",
        "ModifiedLimbLead",
        "AuxiliaryBipolarLead1",
        "AuxiliaryBipolarLead2",
        "ChestManubriumLeadPerV7Placement",
        "DefibrillatorLeadAnteriorLateral",
        "LeadCH5",
        "ExternalPacingLeadAnteriorPosterior",
        "AuxiliaryUnipolarLead1",
        "NegativeRightInfraclavicularFossa",
        "AuxiliaryUnipolarLead2",
        "NegativeLowRightScapulaLead",
        "AuxiliaryUnipolarLead3",
        "AuxiliaryUnipolarLead4",
        "LeadDNehbDorsal",
        "LeadANehbAnterior",
        "LeadJNehbInferior",
        "ChestLead",
        "PrecordialLead",
        "LeadVRNonaugmentedVoltageVectorOfRA",
        "LeadVLNonaugmentedVoltageVectorOfLA",
        "ChestManubriumLeadPerV2Placement",
        "LeadV7R",
        "ChestManubriumLeadPerV3Placement",
        "LeadX",
        "ChestManubriumLeadPerV4Placement",
        "LeadY",
        "LeadZ",
        "ChestLeadPerV5AndV5RPlacement",
        "ChestLeadPerV6AndV6RPlacement",
        "ChestLeadPerV7AndV8RPlacement",
        "ChestManubriumLead",
        "ChestManubriumLeadPerV1Placement",
        "ModifiedChestLeadPerV4Placement",
        "ChestLeadPerV2AndV2RPlacement",
        "ModifiedChestLeadPerV5Placement",
        "ChestLeadPerV3AndV3RPlacement",
        "ModifiedChestLeadPerV6Placement",
        "ChestLeadSymmetricPlacement",
        "ChestLeadPerV4AndV4RPlacement",
        "LeadV2R",
        "ChestLeadPerV1AndV1RPlacement",
        "LeadV3R",
        "LeadV4R",
        "LeadV5R",
        "LeadV6R",
        "LeadVFNonaugmentedVoltageVectorOfLL",
        "ModifiedChestLead",
        "ModifiedChestLeadPerV1Placement",
        "ModifiedChestLeadPerV2Placement",
        "ModifiedChestLeadPerV3Placement",
    ]
}

name_for_cid[3003] = "HemodynamicWaveformSource"
cid_concepts[3003] = {
    "SCT": [
        "HemodynamicOxygenSaturationWaveform",
        "LeftAtriumPressureWaveform",
        "RightVentriclePressureWaveform",
        "ThermalCardiacOutputWaveform",
        "HemodynamicPressureWaveform",
        "PulmonaryValvePullbackPressureWaveform",
        "TemperatureWaveform",
        "PulmonaryCapillaryWedgePressureWaveform",
        "ArterialPressureWaveform",
        "TricuspidValvePullbackPressureWaveform",
        "AorticPressureWaveform",
        "FemoralArteryPressureWaveform",
        "HemodynamicFlowWaveform",
        "LeftVentriclePressureWaveform",
        "RightAtriumPressureWaveform",
        "DyeDilutionCardiacOutputWaveform",
        "MitralValvePullbackPressureWaveform",
        "AorticValvePullbackPressureWaveform",
        "HemodynamicWaveform",
        "RespirationImpedanceWaveform",
        "HemodynamicImpedanceWaveform",
        "CentralVenousPressureWaveform",
        "PulmonaryArteryOxygenSaturationWaveform",
        "PulmonaryArteryWedgePressureWaveform",
        "PulmonaryArteryPressureWaveform",
    ]
}

name_for_cid[3004] = "ArterialPulseWaveform"
cid_concepts[3004] = {"DCM": ["ArterialPulseWaveform"]}

name_for_cid[3005] = "RespirationWaveform"
cid_concepts[3005] = {
    "DCM": [
        "Airflow",
        "CO2Respiration",
        "AirflowCPAP",
        "CO2TrendSideStream",
        "CO2TrendMainStream",
        "CO2WaveformSideStream",
        "EsophagealPressure",
        "PAPTidalVolume",
        "RespirationWaveform",
        "PAPLeakPressure",
        "PAPPressure",
        "ThoracicRespiratoryPVDF",
        "AbdominalRespiratoryInductance",
        "ThoracicRespiratoryInductance",
        "RespiratoryPressure",
        "AbdominalRespiratoryEffort",
        "ThoracicRespiratoryEffort",
        "AbdominalRespiratoryPVDF",
        "CO2TrendEndTidalMainStream",
        "CO2WaveformEndTidalMainStream",
        "CO2Transcutaneous",
        "RespiratoryEffort",
        "AirflowNasalProng",
        "AirflowThermocouple",
        "CO2WaveformMainStream",
        "AirflowThermistor",
        "CO2TrendEndTidalSideStream",
        "CO2WaveformEndTidalSideStream",
        "AirflowPVDF",
    ]
}

name_for_cid[3010] = "CardiovascularAnatomicStructure"
cid_concepts[3010] = {
    "SCT": [
        "InternalCarotidArtery",
        "SubclavianVein",
        "ExternalIliacArtery",
        "TruncusCoeliacus",
        "BasilarArtery",
        "ApexOfRightVentricle",
        "SuperiorRightPulmonaryVein",
        "InferiorVenaCava",
        "RightPulmonaryArtery",
        "GenicularArtery",
        "LeftVentricleInflow",
        "VenousNetwork",
        "CarotidArtery",
        "CommonAtrium",
        "UmbilicalArtery",
        "PrimitiveAorta",
        "AorticFistula",
        "LeftVentricleOutflowTract",
        "RadialArtery",
        "RightFemoralArtery",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "LeftFemoralArtery",
        "AnteriorTibialArtery",
        "TruncusArteriosusCommunis",
        "Artery",
        "AntecubitalVein",
        "CommonCarotidArtery",
        "Aorta",
        "PatentDuctusArteriosus",
        "SuperficialTemporalArtery",
        "MesentericVein",
        "Vein",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "PulmonaryVein",
        "InferiorRightPulmonaryVein",
        "CoronaryArtery",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "SubclavianArtery",
        "BodyConduit",
        "OphthalmicArtery",
        "RightVentricleInflow",
        "MesentericArtery",
        "PoplitealArtery",
        "PulmonaryVenousAtrium",
        "BrachialVein",
        "InternalIliacArtery",
        "PulmonaryChamberOfCorTriatriatum",
        "SuperiorVenaCava",
        "UlnarArtery",
        "LeftVentricle",
        "AnteriorSpinalArtery",
        "HepaticArtery",
        "SplenicVein",
        "PeronealArtery",
        "RightVentricleOutflowTract",
        "SaphenousVein",
        "InnominateArtery",
        "SplenicArtery",
        "RenalArtery",
        "SuperiorLeftPulmonaryVein",
        "SystemicCollateralArteryToLung",
        "FacialArtery",
        "LumbarArtery",
        "ApexOfLeftVentricle",
        "AzygosVein",
        "FemoralArtery",
        "PulmonaryArtery",
        "BrachialArtery",
        "IliacVein",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "PosteriorMedialTributary",
        "CommonFemoralArtery",
        "PulmonaryArteryConduit",
        "RightAtrium",
        "AnteriorCommunicatingArtery",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "RightVentricle",
        "ThoracicAorta",
        "CommonVentricle",
        "VenaCava",
        "CoronarySinus",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "HepaticVein",
        "InternalMammaryArtery",
        "VertebralArtery",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "InternalJugularVein",
        "LacrimalArteryOfRightEye",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "UmbilicalVein",
        "CommonIliacVein",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "GreatCardiacVein",
        "InnominateVein",
        "PrimitivePulmonaryArtery",
        "SuperficialFemoralArtery",
        "PortalVein",
        "InferiorLeftPulmonaryVein",
        "PosteriorCommunicationArtery",
        "JuxtaposedAtrialAppendage",
        "AorticArch",
        "RenalVein",
        "BoydPerforatingVein",
        "AbdominalAorta",
        "CommonIliacArtery",
        "CephalicVein",
        "LingualArtery",
        "GastricVein",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "InferiorCardiacVein",
        "Baffle",
        "LeftAuricularAppendage",
        "LacrimalArtery",
        "OccipitalArtery",
        "SystemicVenousAtrium",
        "AscendingAorta",
        "HunterianPerforatingVein",
        "AnomalousPulmonaryVein",
        "AnteriorCardiacVein",
        "ProfundaFemorisArtery",
        "LeftAtrium",
        "CerebralArtery",
        "AxillaryVein",
        "InferiorMesentericArtery",
        "GreatSaphenousVein",
        "OccipitalVein",
        "DoddPerforatingVein",
    ]
}

name_for_cid[3011] = "ElectrophysiologyAnatomicLocation"
cid_concepts[3011] = {
    "SCT": [
        "GreatCardiacVein",
        "LateralHighRightAtrium",
        "LowRightAtrium",
        "RightVentricleInflow",
        "InferiorLeftPulmonaryVein",
        "ApexOfRightVentricle",
        "SuperiorRightPulmonaryVein",
        "PurkinjeFibers",
        "LeftPosteriorDivisionOfLeftBranchAtrioventricularBundle",
        "MitralRing",
        "RightAtrium",
        "LeftVentricleInflow",
        "RightVentricle",
        "LeftVentricle",
        "CommonVentricle",
        "MiddleCardiacVein",
        "PulmonicRing",
        "CoronarySinus",
        "HighRightAtrium",
        "AtrioventricularNode",
        "LeftVentricleOutflowTract",
        "MidRightAtrium",
        "TricuspidRing",
        "RightVentricleOutflowTract",
        "TendonOfTodaro",
        "SuperiorLeftPulmonaryVein",
        "LeftAuricularAppendage",
        "Epicardium",
        "RightBranchOfAtrioventricularBundle",
        "AccessoryAtrioventricularBundle",
        "ApexOfLeftVentricle",
        "OstiumOfCoronarySinus",
        "RightAtrioventricularOstium",
        "LeftAnteriorDivisionOfLeftBranchAtrioventricularBundle",
        "LeftAtrium",
        "PulmonaryVein",
        "InferiorRightPulmonaryVein",
        "LeftBranchOfAtrioventricularBundle",
        "RightAuricularAppendage",
        "AtrioventricularBundle",
        "SinoAtrialNode",
    ]
}

name_for_cid[3014] = "CoronaryArterySegment"
cid_concepts[3014] = {
    "BARI": [
        "_2ndMarginalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_1stMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "SCT": [
        "ProximalRightCoronaryArtery",
        "_3rdLeftPosterolateralCoronaryArtery",
        "RightPosteriorAVCoronaryArtery",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "MidCircumflexCoronaryArtery",
        "_1stLeftPosterolateralCoronaryArtery",
        "IntermediateArteryRamus",
        "LeftMainCoronaryArteryOstium",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "RightCoronaryArteryOstium",
        "LeftMainCoronaryArtery",
        "_2ndLeftPosterolateralCoronaryArtery",
        "DistalRightCoronaryArtery",
        "MarginalCoronaryArtery",
        "_2ndRightPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "_1stMarginalCoronaryArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "_3rdDiagonalCoronaryArtery",
        "_1stSeptalCoronaryArtery",
        "_3rdMarginalCoronaryArtery",
        "_1stDiagonalCoronaryArtery",
        "MidRightCoronaryArtery",
        "_2ndDiagonalCoronaryArtery",
        "AVGrooveContinuationOfCircumflexArtery",
        "DistalCircumflexCoronaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "_1stRightPosterolateralCoronaryArtery",
        "_3rdRightPosterolateralCoronaryArtery",
    ],
}

name_for_cid[3015] = "CoronaryArtery"
cid_concepts[3015] = {
    "BARI": [
        "_2ndMarginalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_1stMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "SCT": [
        "ProximalRightCoronaryArtery",
        "_3rdLeftPosterolateralCoronaryArtery",
        "RightPosteriorAVCoronaryArtery",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "MidCircumflexCoronaryArtery",
        "_1stLeftPosterolateralCoronaryArtery",
        "IntermediateArteryRamus",
        "CircumflexCoronaryArtery",
        "LeftMainCoronaryArteryOstium",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "RightCoronaryArteryOstium",
        "LeftCoronaryArtery",
        "LeftMainCoronaryArtery",
        "_2ndLeftPosterolateralCoronaryArtery",
        "DistalRightCoronaryArtery",
        "MarginalCoronaryArtery",
        "_2ndRightPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "LeftAnteriorDescendingCoronaryArtery",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "_1stMarginalCoronaryArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "_3rdDiagonalCoronaryArtery",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "_3rdMarginalCoronaryArtery",
        "_1stDiagonalCoronaryArtery",
        "MidRightCoronaryArtery",
        "_2ndDiagonalCoronaryArtery",
        "AVGrooveContinuationOfCircumflexArtery",
        "DistalCircumflexCoronaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "CoronaryArteryGraft",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "_1stRightPosterolateralCoronaryArtery",
        "_3rdRightPosterolateralCoronaryArtery",
        "PosterolateralBranchOfRightCoronaryArtery",
    ],
}

name_for_cid[3016] = "MajorCoronaryArtery"
cid_concepts[3016] = {
    "SCT": [
        "CircumflexCoronaryArtery",
        "LeftAnteriorDescendingCoronaryArtery",
        "RightCoronaryArtery",
        "LeftCoronaryArtery",
        "LeftMainCoronaryArtery",
    ]
}

name_for_cid[3019] = "CardiovascularAnatomicStructureModifier"
cid_concepts[3019] = {
    "DCM": [
        "GraftToCitedSegmentDistalSection",
        "GraftToCitedSegmentProximalSection",
        "GraftToCitedSegmentMidSection",
        "Median",
        "AneurysmOnCitedVessel",
    ],
    "NCIt": ["Middle"],
    "SCT": [
        "EntireVessel",
        "Central",
        "Right",
        "Posterior",
        "Left",
        "Superior",
        "Distal",
        "Proximal",
        "MidLongitudinal",
        "Lateral",
        "Inferior",
        "GraftToDistalAnastomosis",
        "Ostium",
        "Anterior",
        "GraftToProximalAnastomosis",
    ],
}

name_for_cid[3020] = "AssessmentfromCADRADS™"
cid_concepts[3020] = {
    "DCM": [
        "NNonDiagnosticStudy",
        "_5TotalCoronaryOcclusion",
        "_3ModerateStenosis",
        "_2MildNonObstructiveCAD",
        "_1MinimalNonObstructiveCAD",
        "_0DocumentedAbsenceOfCAD",
        "_4SevereStenosis",
    ]
}

name_for_cid[3021] = "CADRADS™StenosisAssessmentModifier"
cid_concepts[3021] = {"DCM": ["_4BSevereStenosis", "_4ASevereStenosis"]}

name_for_cid[3022] = "CADRADS™AssessmentModifier"
cid_concepts[3022] = {
    "DCM": ["SStent", "NNonDiagnosticSegments", "VVulnerablePlaque", "GGraft"]
}

name_for_cid[3030] = "EEGLead"
cid_concepts[3030] = {
    "DCM": ["Fp10", "Fp9", "PO10", "PO9", "AF10", "AF9", "O10", "O9"],
    "MDC": [
        "P4",
        "F6",
        "TP7",
        "Oz",
        "CP1",
        "AF8",
        "FC6",
        "P6",
        "F8",
        "P5",
        "T10",
        "F7",
        "C6",
        "Sp2",
        "PO4",
        "FT8",
        "PO3",
        "Iz",
        "FT7",
        "TP8",
        "CP2",
        "Sp1",
        "Cpz",
        "P3",
        "T6",
        "F5",
        "C4",
        "T9",
        "P2",
        "F4",
        "Poz",
        "FC5",
        "Pg1",
        "Pg2",
        "Pz",
        "C5",
        "AF4",
        "FC4",
        "AF7",
        "A2",
        "FC3",
        "CP6",
        "F2",
        "T1",
        "Fcz",
        "P1",
        "T4",
        "T2",
        "FC2",
        "AF3",
        "F3",
        "C2",
        "T5",
        "Cz",
        "C3",
        "O2",
        "F9",
        "Fp2",
        "F10",
        "PO7",
        "Fpz",
        "FT9",
        "TP10",
        "FC1",
        "CP4",
        "TP9",
        "CP3",
        "Fp1",
        "T3",
        "Fz",
        "C1",
        "P10",
        "P9",
        "A1",
        "Afz",
        "CP5",
        "PO8",
        "F1",
        "FT10",
        "Nz",
        "O1",
    ],
}

name_for_cid[3031] = "LeadLocationNearorinMuscle"
cid_concepts[3031] = {
    "MDC": [
        "MusculusPeroneusBrevis",
        "MusculusPeroneusLongusRight",
        "MusculustricepsSurae",
        "MusculusPeroneusBrevisRight",
        "MusculusPeroneusBrevisLeft",
        "MusculustricepsSuraeRight",
        "MusculustricepsSuraeLeft",
        "MusculusExtensorHallucisLongusLeft",
        "MusculusExtensorHallucisLongus",
        "MusculusExtensorHallucisLongusRight",
        "MusculusPeroneusLongusLeft",
        "MusculusPeroneusLongus",
        "MusculusRectusSuperiorRight",
        "MusculusRectusSuperiorLeft",
        "MusculusCricothyroideusRight",
        "MusculusRectusSuperior",
        "MusculusCricothyroideusLeft",
        "MusculusCricothyroideus",
        "MusculusRectusInferiorRight",
        "MusculusRectusInferiorLeft",
        "MusculusTyroartenoideusRight",
        "MusculusRectusInferior",
        "MusculusTyroartenoideusLeft",
        "MusculusTyroartenoideus",
        "MusculusRectusMedialisLeft",
        "MusculusRectusMedialis",
        "MusculiColliLeft",
        "MusculiColli",
        "MusculusGenioglossus",
        "MusculusSemispinalisLeft",
        "MusculusSemispinalis",
        "MusculiLinguaeRight",
        "MusculusSpinalisCapitisRight",
        "MusculiLaringis",
        "MusculusSemispinalisThoracisLeft",
        "MusculiCapitisRight",
        "MusculusSemispinalisThoracis",
        "MusculiCapitisLeft",
        "MusculusGenioglossusRight",
        "MusculiCapitis",
        "MusculusGenioglossusLeft",
        "MusculusSemispinalisRight",
        "MusculiBulbiRight",
        "MusculiBulbiLeft",
        "MusculiLaringisRight",
        "MusculiBulbi",
        "MusculiLaringisLeft",
        "MusculusSemispinalisThoracisRight",
        "MusculusObliquusInferior",
        "MusculiFacialesEtMasticatores",
        "MusculusObliquusInferiorRight",
        "MusculusObliquusInferiorLeft",
        "MusculusOccipitofrontalisVenterFrontalis",
        "MusculiFacialesEtMasticatoresRight",
        "MusculiFacialesEtMasticatoresLeft",
        "MusculusOccipitofrontalisVenterFrontalisLeft",
        "MusculiColliRight",
        "MusculusRectusLateralisLeft",
        "PlatysmaRight",
        "MusculusRectusLateralis",
        "PlatysmaLeft",
        "Platysma",
        "MusculusRectusMedialisRight",
        "MusculusObliquusSuperiorLeft",
        "MusculusCapitisLongusRight",
        "MusculusObliquusSuperior",
        "MusculusCapitisLongusLeft",
        "MusculusCapitisLongus",
        "MusculusRectusLateralisRight",
        "MusculusObliquusSuperiorRight",
        "MusculusOccipitofrontalisVenterFrontalisRight",
        "MusculusOrbicularisOculiRight",
        "MusculusOrbicularisOculiLeft",
        "MusculusOrbicularisOculi",
        "MusculusOrbicularisOculiParsOrbitalisRight",
        "MusculusOrbicularisOculiParsOrbitalisLeft",
        "MusculusOrbicularisOculiParsOrbitalis",
        "MusculusMylohyoideusLeft",
        "MusculusPectoralisMinorRight",
        "MusculusMylohyoideus",
        "MusculusPectoralisMinorLeft",
        "MusculusAnconeusRight",
        "MusculusPectoralisMinor",
        "MusculusAnconeusLeft",
        "MusculiInterosseiPalmaresRight",
        "MusculusDigastricusVenterPosteriorRight",
        "MusculusAnconeus",
        "MusculiInterosseiPalmaresLeft",
        "MusculusExtensorDigitorumLongusRight",
        "MusculusMylohyoideusRight",
        "MusculusSubclaviusLeft",
        "MusculusPronatorTeresRight",
        "MusculusSubclavius",
        "MusculusPronatorTeresLeft",
        "MusculusPronatorTeres",
        "MusculiDorsi",
        "MusculusSerratusAnteriorLeft",
        "MusculusFlexorCarpiRadialisRight",
        "MusculusAuricularisPosteriorRight",
        "MusculusSerratusAnterior",
        "MusculusFlexorCarpiRadialisLeft",
        "MusculusAuricularisPosteriorLeft",
        "MusculusFlexorCarpiRadialis",
        "MusculusAuricularisPosterior",
        "MusculusSubclaviusRight",
        "MusculusDigastricusRight",
        "MusculusTricepsBrachiiCaputLaterale",
        "MusculiLumbricalesLeft",
        "MusculusDigastricusLeft",
        "MusculiInterspinalesLumborumRight",
        "MusculiLumbricales",
        "MusculusSemimembranosusLeft",
        "MusculusDigastricus",
        "MusculiInterspinalesLumborumLeft",
        "MusculusTricepsBrachiiCaputLongumRight",
        "MusculusSemimembranosus",
        "MusculiInterspinalesLumborum",
        "MusculusTricepsBrachiiCaputLongumLeft",
        "MusculusOpponensDigitiMinimiRight",
        "MusculusDigastricusVenterAnteriorLeft",
        "MusculusDigastricusVenterAnterior",
        "MusculiThoracisLeft",
        "MusculiThoracis",
        "MusculusTricepsBrachiiCaputLateraleLeft",
        "MusculiThoracisRight",
        "MusculiInterosseiDorsales",
        "MusculusTibialisAnteriorLeft",
        "MusculusTricepsBrachiiCaputLateraleRight",
        "MusculusTibialisAnterior",
        "MusculiLumbricalesRight",
        "MusculusSemimembranosusRight",
        "MusculusDigastricusVenterPosteriorLeft",
        "MusculusPectoralisMajorRight",
        "MusculiInterosseiPalmares",
        "MusculusExtensorDigitorumLongusLeft",
        "MusculusDigastricusVenterPosterior",
        "MusculusPectoralisMajorLeft",
        "MusculusTricepsBrachiiCaputMedialeRight",
        "MusculusExtensorDigitorumLongus",
        "MusculusPectoralisMajor",
        "MusculusTricepsBrachiiCaputMedialeLeft",
        "MusculiInterosseiDorsalesRight",
        "MusculusDigastricusVenterAnteriorRight",
        "MusculusTricepsBrachiiCaputMediale",
        "MusculiInterosseiDorsalesLeft",
        "MusculusTibialisAnteriorRight",
        "MusculusZygomaticusMajorRight",
        "MusculusZygomaticusMajorLeft",
        "MusculusTrapeziusRight",
        "MusculusZygomaticusMajor",
        "MusculusTrapeziusLeft",
        "MusculiAbdominisRight",
        "MusculusTrapezius",
        "MusculiAbdominisLeft",
        "MusculusFlexorDigitorumSuperficialisRight",
        "MusculusFlexorDigitorumProfundus",
        "MusculusZygomaticusMinorLeft",
        "MusculusLatissimusDorsiRight",
        "MusculusFlexorPollicisLongus",
        "MusculusZygomaticusMinor",
        "MusculusLatissimusDorsiLeft",
        "MusculusRectusAbdominisRight",
        "MusculusLatissimusDorsi",
        "MusculusRectusAbdominisLeft",
        "MusculusFlexorDigitorumProfundusRight",
        "MusculusRectusAbdominis",
        "MusculusFlexorDigitorumProfundusLeft",
        "MusculusLevatorLabiiSuperiorisLeft",
        "MusculusRhomboideusMajorRight",
        "MusculusPronatorQuadratus",
        "MusculusLevatorLabiiSuperioris",
        "MusculusRhomboideusMajorLeft",
        "MusculusObliquusExternusAbdominisRight",
        "MusculusRhomboideusMajor",
        "MusculusObliquusExternusAbdominisLeft",
        "MusculusFlexorPollicisLongusRight",
        "MusculusZygomaticusMinorRight",
        "MusculusObliquusExternusAbdominis",
        "MusculusFlexorPollicisLongusLeft",
        "MusculiIntercostalesLeft",
        "MusculusOrbicularisOrisRight",
        "MusculiIntercostales",
        "MusculusPalmarisLongusLeft",
        "MusculusOrbicularisOrisLeft",
        "MusculiDorsiRight",
        "MusculusPalmarisLongus",
        "MusculusOrbicularisOris",
        "MusculiDorsiLeft",
        "MusculusSerratusAnteriorRight",
        "MusculusDepressorAnguliOrisLeft",
        "MusculusDepressorAnguliOris",
        "MusculusDepressorAnguliOrisRight",
        "Diaphragma",
        "MusculusFlexorCarpiUlnarisLeft",
        "MusculusIliopsoasRight",
        "MusculusFlexorCarpiUlnaris",
        "MusculusIliopsoasLeft",
        "MusculiIntercostalesRight",
        "MusculusIliopsoas",
        "MusculusPalmarisLongusRight",
        "MusculusRisoriusRight",
        "MusculiAbdominis",
        "MusculusFlexorDigitorumSuperficialisLeft",
        "MusculusGluteusMaximusRight",
        "MusculusRisoriusLeft",
        "MusculusFlexorDigitorumSuperficialis",
        "MusculusGluteusMaximusLeft",
        "MusculusRisorius",
        "DiaphragmaRight",
        "MusculusGluteusMaximus",
        "DiaphragmaLeft",
        "MusculusFlexorCarpiUlnarisRight",
        "MusculusLevatorAnguliOrisRight",
        "MusculusLevatorAnguliOrisLeft",
        "MusculusSerratusPosteriorRight",
        "MusculusSpleniusCapitis",
        "MusculiDiaphragmatisPelvisLeft",
        "MusculiDiaphragmatisPelvis",
        "MusculusSpleniusCervicis",
        "MusculusPuborectalisLeft",
        "MusculusBuccinatorRight",
        "MusculusPuborectalis",
        "MusculusBuccinatorLeft",
        "MusculusSpleniusCapitisRight",
        "MusculusBuccinator",
        "MusculusSpleniusCapitisLeft",
        "MusculiDiaphragmatisPelvisRight",
        "MusculusSplenius",
        "MusculusCoccygeusLeft",
        "MusculusMentalisRight",
        "MusculusCoccygeus",
        "MusculusMentalisLeft",
        "MusculusSpleniusCervicisRight",
        "MusculusMentalis",
        "MusculusSpleniusCervicisLeft",
        "MusculusPuborectalisRight",
        "MusculusLevatorLabiiSuperiorisAlaequeNasiLeft",
        "MusculusLevatorLabiiSuperiorisAlaequeNasi",
        "MusculusRhomboideusMinorLeft",
        "MusculusRhomboideusMinor",
        "MusculusObliquusInternusAbdominisLeft",
        "MusculusLevatorLabiiSuperiorisRight",
        "MusculusObliquusInternusAbdominis",
        "MusculusPronatorQuadratusLeft",
        "MusculusObliquusInternusAbdominisRight",
        "MusculusPronatorQuadratusRight",
        "MusculusDepressorLabiiInferioris",
        "MusculusLevatorScapulaeLeft",
        "MusculusTransversusAbdominisRight",
        "MusculusLevatorScapulae",
        "MusculusTransversusAbdominisLeft",
        "MusculusBrachioradialisRight",
        "MusculusLevatorLabiiSuperiorisAlaequeNasiRight",
        "MusculusTransversusAbdominis",
        "MusculusBrachioradialisLeft",
        "MusculusRhomboideusMinorRight",
        "MusculusBrachioradialis",
        "MusculusLevatorAnguliOris",
        "MusculusSerratusPosteriorLeft",
        "MusculusQuadratusLumborumRight",
        "MusculusSerratusPosterior",
        "MusculusQuadratusLumborumLeft",
        "MusculusExtensorCarpiRadialisLongusRight",
        "MusculusDepressorLabiiInferiorisRight",
        "MusculusQuadratusLumborum",
        "MusculusExtensorCarpiRadialisLongusLeft",
        "MusculusDepressorLabiiInferiorisLeft",
        "MusculusLevatorScapulaeRight",
        "MusculusExtensorCarpiRadialisLongus",
        "MusculusPterygoideusLateralisLeft",
        "MusculusSpinalisThoracisRight",
        "MusculusPterygoideusLateralis",
        "MusculusSpinalisThoracisLeft",
        "MusculusSpinalisThoracis",
        "MusculusPterygoideusMedialisLeft",
        "MusculusSpinalisCervicisRight",
        "MusculusPterygoideusMedialis",
        "MusculusSpinalisCervicisLeft",
        "MusculusSpinalisCervicis",
        "MusculusPterygoideusLateralisRight",
        "MusculiLinguaeLeft",
        "MusculiLinguae",
        "MusculusSpinalisCapitisLeft",
        "MusculusSpinalisCapitis",
        "MusculusPterygoideusMedialisRight",
        "MusculusMasseterLeft",
        "MusculusMasseter",
        "MusculusSpleniusLeft",
        "MusculusMasseterRight",
        "MusculusSphincterAni",
        "MusculusSpleniusRight",
        "MusculusCoccygeusRight",
        "MusculusTemporalisRight",
        "MusculusSphincterAniExternus",
        "MusculusTemporalisLeft",
        "MusculusErectorSpinaeRight",
        "MusculusTemporalis",
        "MusculusErectorSpinaeLeft",
        "MusculusErectorSpinae",
        "MusculusPterygoideusRight",
        "MusculusPterygoideusLeft",
        "MusculusSpinalisRight",
        "MusculusPterygoideus",
        "MusculusSpinalisLeft",
        "MusculusSpinalis",
        "MusculusPiriformisLeft",
        "MusculusSoleusRight",
        "MusculusPiriformis",
        "MusculusSoleusLeft",
        "MusculusSoleus",
        "MusculusTensorFasciaeLataeRight",
        "MusculusObturatorLeft",
        "MusculusPlantarisRight",
        "MusculusObturator",
        "MusculusPlantarisLeft",
        "MusculusPlantaris",
        "MusculusPiriformisRight",
        "MusculusObturatorRight",
        "MusculusGmellus",
        "MusculusPopliteusLeft",
        "MusculusPopliteus",
        "MusculusGluteusMediusRight",
        "MusculusGluteusMediusLeft",
        "MusculusGastrocnemiusRight",
        "MusculusGluteusMedius",
        "MusculusGastrocnemiusLeft",
        "MusculusGastrocnemius",
        "MusculusGluteusMinimusRight",
        "MusculusGluteusMinimusLeft",
        "MusculusGastrocnemiusCaputLateraleRight",
        "MusculusGluteusMinimus",
        "MusculusGastrocnemiusCaputLateraleLeft",
        "MusculusGastrocnemiusCaputLaterale",
        "MusculusTensorFasciaeLataeLeft",
        "MusculusTensorFasciaeLatae",
        "MusculusGastrocnemiusCaputMedialeLeft",
        "MusculusGastrocnemiusCaputMediale",
        "MusculusGastrocnemiusCaputMedialeRight",
        "MusculusExtensorCarpiUlnarisRight",
        "MusculusExtensorDigitorumBrevis",
        "MusculusExtensorCarpiUlnarisLeft",
        "MusculusQuadricepsFemorisRight",
        "MusculusExtensorCarpiUlnaris",
        "MusculusQuadricepsFemorisLeft",
        "MusculusExtensorHallucisBrevisRight",
        "MusculusQuadricepsFemoris",
        "MusculusExtensorHallucisBrevisLeft",
        "MusculusSupinatorRight",
        "MusculusSupinatorLeft",
        "MusculusRectusFemorisRight",
        "MusculusSupinator",
        "MusculusRectusFemorisLeft",
        "MusculusExtensorDigitorumBrevisRight",
        "MusculusRectusFemoris",
        "MusculusExtensorDigitorumBrevisLeft",
        "MusculiMembriSuperioris",
        "MusculusAbductorPollicisLongussLeft",
        "MusculusVastusLateralisRight",
        "MusculusAbductorPollicisLonguss",
        "MusculusVastusLateralisLeft",
        "MusculusAbductorHallucisRight",
        "MusculusVastusLateralis",
        "MusculusAbductorHallucisLeft",
        "MusculusAbductorHallucis",
        "MusculusQuadratusFemoris",
        "MusculusTibialisPosteriorLeft",
        "MusculusExtensorCarpiRadialisBrevisRight",
        "MusculusTibialisPosterior",
        "MusculusExtensorCarpiRadialisBrevisLeft",
        "MusculusGmellusRight",
        "MusculusExtensorCarpiRadialisBrevis",
        "MusculusGmellusLeft",
        "MusculusPopliteusRight",
        "MusculusSartorius",
        "MusculusFlexorDigitorumLongusLeft",
        "MusculusExtensorDigitorumRight",
        "MusculusFlexorDigitorumLongus",
        "MusculusExtensorDigitorumLeft",
        "MusculusQuadratusFemorisRight",
        "MusculusExtensorDigitorum",
        "MusculusQuadratusFemorisLeft",
        "MusculusTibialisPosteriorRight",
        "MusculusExtensorDigitiMinimiLeft",
        "MusculusExtensorDigitiMinimi",
        "MusculusSartoriusLeft",
        "MusculusExtensorDigitiMinimiRight",
        "MusculusExtensorHallucisBrevis",
        "MusculusSartoriusRight",
        "MusculusFlexorDigitorumLongusRight",
        "MusculusPalmarisBrevis",
        "MusculusAdductorLongusLeft",
        "MusculusFlexorDigitiMinimiBrevisFootRight",
        "MusculusInfraspinatusRight",
        "MusculusAdductorLongus",
        "MusculusFlexorDigitiMinimiBrevisFootLeft",
        "MusculusInfraspinatusLeft",
        "MusculusExtensorIndicisRight",
        "MusculusFlexorDigitiMinimiBrevisFoot",
        "MusculusInfraspinatus",
        "MusculusExtensorIndicisLeft",
        "MusculusPectineusRight",
        "MusculusTeresMinorRight",
        "MusculusTeresMinorLeft",
        "MusculusPalmarisBrevisRight",
        "MusculusTeresMinor",
        "MusculusPalmarisBrevisLeft",
        "MusculusAdductorLongusRight",
        "MusculusAdductorBrevis",
        "MusculusQuadratusPlantaeLeft",
        "MusculusQuadratusPlantae",
        "MusculusSemispinalisCervicisLeft",
        "MusculusTeresMajorRight",
        "MusculusAdductorMagnus",
        "MusculiLunbricalesLeft",
        "MusculusSemispinalisCervicis",
        "MusculusTeresMajorLeft",
        "MusculusAbductorPollicisBrevisRight",
        "MusculiLunbricales",
        "MusculusTeresMajor",
        "MusculusAbductorPollicisBrevisLeft",
        "MusculusAdductorBrevisRight",
        "MusculusAbductorPollicisBrevis",
        "MusculusAdductorBrevisLeft",
        "MusculusQuadratusPlantaeRight",
        "MusculusDeltoideus",
        "MusculusExtensorPollicisBrevisLeft",
        "MusculusVastusIntermediusRight",
        "MusculusExtensorPollicisBrevis",
        "MusculusVastusIntermediusLeft",
        "MusculusFlexorHallucisBrevisRight",
        "MusculiMembriSuperioriRight",
        "MusculusVastusIntermedius",
        "MusculusFlexorHallucisBrevisLeft",
        "MusculiMembriSuperiorisLeft",
        "MusculusAbductorPollicisLongussRight",
        "MusculusFlexorHallucisBrevis",
        "MusculusSupraspinatus",
        "MusculusExtensorPollicisLongusLeft",
        "MusculusExtensorPollicisLongus",
        "MusculusVastusMedialisLeft",
        "MusculusDeltoideusRight",
        "MusculusVastusMedialis",
        "MusculusAdductorHallucisLeft",
        "MusculusDeltoideusLeft",
        "MusculusExtensorPollicisBrevisRight",
        "MusculusAdductorHallucis",
        "MusculusSupraspinatusLeft",
        "MusculusAdductorHallucisRight",
        "MusculusExtensorIndicis",
        "MusculusPectineusLeft",
        "MusculusAbductorDigitiMinimiFootRight",
        "MusculusSupraspinatusRight",
        "MusculusPectineus",
        "MusculusAbductorDigitiMinimiFootLeft",
        "MusculusExtensorPollicisLongusRight",
        "MusculusAbductorDigitiMinimiFoot",
        "MusculusVastusMedialisRight",
        "MusculiInterspinalesCervicis",
        "MusculusCoracobrachialisLeft",
        "MusculusAbductorDigitiMinimiHandRight",
        "MusculusCoracobrachialis",
        "MusculusAbductorDigitiMinimiHandLeft",
        "MusculusBicepsFemorisCaputLongumRight",
        "MusculiInterpsinalesRight",
        "MusculusAbductorDigitiMinimiHand",
        "MusculusBicepsFemorisCaputLongumLeft",
        "MusculiInterspinalesLeft",
        "MusculusBrachialisRight",
        "MusculusBicepsFemorisCaputLongum",
        "MusculiInterspinalesCervicisRight",
        "MusculiInterspinalesCervicisLeft",
        "MusculusCoracobrachialisRight",
        "MusculusTricepsBrachii",
        "MusculusFlexorDigitiMinimiBrevisHandLeft",
        "MusculusBicepsFemorisCaputBreveRight",
        "MusculusFlexorDigitiMinimiBrevisHand",
        "MusculusBicepsFemorisCaputBreveLeft",
        "MusculusBicepsFemorisCaputBreve",
        "MusculusSternocleidomastoideusRight",
        "MusculusTricepsBrachiiCaputLongum",
        "MusculusOpponensDigitiMinimiLeft",
        "MusculusSemitendinosusRight",
        "MusculusSternocleidomastoideusLeft",
        "MusculiInterspinalesThoracisRight",
        "MusculusOpponensDigitiMinimi",
        "MusculusSemitendinosusLeft",
        "MusculusSternocleidomastoideus",
        "MusculiInterspinalesThoracisLeft",
        "MusculusTricepsBrachiiRight",
        "MusculusSemitendinosus",
        "MusculiInterspinalesThoracis",
        "MusculusTricepsBrachiiLeft",
        "MusculusFlexorDigitiMinimiBrevisHandRight",
        "MusculusSemispinalisCapitisLeft",
        "MusculusSubscapularisRight",
        "MusculusGracilis",
        "MusculusInterosseiDorsalesLeft",
        "MusculusSemispinalisCapitis",
        "MusculusSubscapularisLeft",
        "MusculusFlexorPollicisBrevisRight",
        "MusculusInterosseiDorsales",
        "MusculusSubscapularis",
        "MusculusFlexorPollicisBrevisLeft",
        "MusculusAdductorMagnusRight",
        "MusculusSemispinalisCervicisRight",
        "MusculusFlexorPollicisBrevis",
        "MusculusAdductorMagnusLeft",
        "MusculiLunbricalesRight",
        "MusculiMultifidiiLeft",
        "MusculiMultifidii",
        "MusculusBicepsBrachiiLeft",
        "MusculusBicepsBrachii",
        "MusculusOpponensPollicisLeft",
        "MusculusSemispinalisCapitisRight",
        "MusculusOpponensPollicis",
        "MusculusGracilisLeft",
        "MusculusOpponensPollicisRight",
        "MusculusInterosseiPlantares",
        "MusculusGracilisRight",
        "MusculusInterosseiDorsalesRight",
        "MusculiInterspinales",
        "MusculusBrachialisLeft",
        "MusculusAdductorPollicisRight",
        "MusculusBrachialis",
        "MusculusAdductorPollicisLeft",
        "MusculusBicepsFemorisRight",
        "MusculiMultifidiiRight",
        "MusculusAdductorPollicis",
        "MusculusBicepsFemorisLeft",
        "MusculusInterosseiPlantaresRight",
        "MusculusBicepsBrachiiRight",
        "MusculusBicepsFemoris",
        "MusculusInterosseiPlantaresLeft",
    ]
}

name_for_cid[3032] = "LeadLocationNearPeripheralNerve"
cid_concepts[3032] = {
    "MDC": [
        "NervusMaxillarisLeft",
        "NervusMaxillaris",
        "NervusSupraorbitalisRight",
        "NervusMaxillarisRight",
        "NervusSupraorbitalisLeft",
        "RamusPalmarisNerviUlnarisLeft",
        "NervusSupraorbitalis",
        "RamusPalmarisNerviUlnaris",
        "NervusOphtalmicusRight",
        "NervusUlnarisNerviDigitalesPalmaresPropriiLeft",
        "NervusUlnarisNerviDigitalesPalmaresProprii",
        "RamusPalmarisNerviUlnarisRight",
        "NervusRadialisLeft",
        "NervusRadialis",
        "NervusUlnarisNerviDigitalesPalmaresPropriiRight",
        "NervusMedianusNerviDigitalesPalmaresPropriiLeft",
        "NervusMedianusNerviDigitalesPalmaresProprii",
        "NervusAbducensVI",
        "NervusMandibularisRight",
        "NervusMandibularisLeft",
        "NervusAbducensVILeft",
        "NervusInfraorbitalis",
        "NervusMandibularis",
        "NervusMedianusNerviDigitalesPalmaresPropriiRight",
        "NervusInfraorbitalisRight",
        "NervusInfraorbitalisLeft",
        "NervusUlnarisRight",
        "NervusUlnarisLeft",
        "NervusUlnaris",
        "RamusDorsalisNerviUlnarisRight",
        "RamusDorsalisNerviUlnarisLeft",
        "RamusDorsalisNerviUlnaris",
        "NerviCraniales",
        "NerviCranialesLeft",
        "NervusVestibulocochlearisVIIIRight",
        "NervusVestibulocochlearisVIIILeft",
        "NervusVestibulocochlearisVIII",
        "NervusAbducensVIRight",
        "NervusFacialisVIIRight",
        "NervusAxillarisRight",
        "NervusFacialisVIILeft",
        "NervusAxillarisLeft",
        "NervusFacialisVII",
        "NervusAxillaris",
        "NervusSaphenus",
        "NerviThoraciciRight",
        "NerviThoraciciLeft",
        "NerviThoracici",
        "NerviSacralesAtNervusCoccygeus",
        "NerviLumbalesLeft",
        "NervusSaphenusRight",
        "NerviLumbales",
        "NervusSaphenusLeft",
        "NervusGlossopharingeusIXLeft",
        "NervusGlossopharingeusIX",
        "NervusVestibularisRight",
        "NervusVestibularisLeft",
        "NervusVestibularis",
        "NervusCochlearisRight",
        "NervusRadialisRamusSuperficialis",
        "NervusCochlearisLeft",
        "NervusCochlearis",
        "NervusRadialisRight",
        "NerviSubscapulares",
        "NervusRadialisRamusSuperficialisRight",
        "NervusRadialisRamusSuperficialisLeft",
        "NerviSubscapularesRight",
        "NerviSubscapularesLeft",
        "NervusFibularisCommunisRight",
        "NervusFibularisCommunisLeft",
        "NervusFibularisCommunis",
        "NervusAcessoriusXIRadicesCranialesRight",
        "NervusVagusXLeft",
        "NervusIliohypogastricusLeft",
        "NervusVagusX",
        "NervusIliohypogastricus",
        "NervusGlossopharingeusIXRight",
        "NervusAcessoriusXIRadicesCranialesLeft",
        "NervusIlioInguinalisLeft",
        "NervusAcessoriusXIRadicesCraniales",
        "NervusIlioInguinalis",
        "NervusVagusXRight",
        "NervusIliohypogastricusRight",
        "PlexusBrachialisLeft",
        "NervusCutaneusFemorisLateralisLeft",
        "NervusFibularisProfundusRight",
        "PlexusBrachialis",
        "NervusCutaneusFemorisLateralis",
        "NervusFibularisProfundusLeft",
        "NervusFibularisProfundus",
        "NervusIlioInguinalisRight",
        "NervusThoracicusLongusLeft",
        "NervusFibularisSuperficialisRight",
        "NervusThoracicusLongus",
        "NervusFibularisSuperficialisLeft",
        "NervusFibularisSuperficialis",
        "PlexusBrachialisRight",
        "PlexusSacralis",
        "NerviSacralesAtNervusCoccygeusRight",
        "NerviSacralesAtNervusCoccygeusLeft",
        "PlexusSacralisLeft",
        "NerviSpinalesLeft",
        "NervusHypoglossusXII",
        "NerviLumbalesRight",
        "NervusAcessoriusXIRadicesSpinalesRight",
        "NervusAcessoriusXIRadicesSpinalesLeft",
        "NerviSpinales",
        "PlexusLumbosacralisRight",
        "PlexusLumbosacralisLeft",
        "NervusHypoglossusXIIRight",
        "PlexusLumbosacralis",
        "NervusHypoglossusXIILeft",
        "PlexusLumbalisRight",
        "PlexusLumbalisLeft",
        "PlexusSacralisRight",
        "PlexusLumbalis",
        "NervusAcessoriusXIRadicesSpinales",
        "NervusIschiadicusRight",
        "NervusIschiadicusLeft",
        "NervusIschiadicus",
        "NervusCutaneusAntebrachiiMedialisRight",
        "NervusCutaneusAntebrachiiMedialisLeft",
        "NervusPlantarisMedialisRight",
        "NervusOpticusIIRight",
        "NervusOpticusIILeft",
        "NervusOpticusII",
        "NervusOculomotoriusIIIRight",
        "NervusOculomotoriusIIILeft",
        "NervusOculomotoriusIII",
        "NerviCervicalesRight",
        "NerviCervicalesLeft",
        "NerviCervicales",
        "NervusPlantarisLateralis",
        "NerviCranialesRight",
        "NervusPhrenicusRight",
        "NervusPhrenicusLeft",
        "NervusPhrenicus",
        "NervusPudendus",
        "NervusMedianusRight",
        "NervusMedianusLeft",
        "NervusPlantarisLateralisRight",
        "NervusMedianus",
        "NervusPlantarisLateralisLeft",
        "NerviSpinalesRight",
        "RamusPalmarisNerviMedianiRight",
        "RamusPalmarisNerviMedianiLeft",
        "NervusPudendusRight",
        "RamusPalmarisNerviMediani",
        "NervusPudendusLeft",
        "NervusMusculocutaneusLeft",
        "NervusMusculocutaneus",
        "NervusTibialisLeft",
        "NervusTibialis",
        "NervusThoracicusLongusRight",
        "NervusTrigeminusVRight",
        "NervusTrigeminusVLeft",
        "NervusTrigeminusV",
        "NervusOphtalmicusLeft",
        "NervusOphtalmicus",
        "NervusObturatorius",
        "NervusCutaneusFemorisLateralisRight",
        "NervusTrochlearisIVRight",
        "NervusFemoralis",
        "NervusTrochlearisIVLeft",
        "NervusTrochlearisIV",
        "NervusObturatoriusRight",
        "NervusObturatoriusLeft",
        "NervusCutaneusAntebrachiiLateralis",
        "NervusSurealisLeft",
        "NervusSurealis",
        "NervusMusculocutaneusRight",
        "NervusFemoralisRight",
        "NervusFemoralisLeft",
        "NervusTibialisRight",
        "NervusCutaneusAntebrachiiMedialis",
        "NervusPlantarisMedialisLeft",
        "NervusPlantarisMedialis",
        "NervusCutaneusAntebrachiiLateralisRight",
        "NervusCutaneusAntebrachiiLateralisLeft",
        "NervusSurealisRight",
    ]
}

name_for_cid[3033] = "EOGLead"
cid_concepts[3033] = {
    "MDC": [
        "Er3",
        "El5",
        "Er6",
        "E0",
        "Erl",
        "Er5",
        "Ell",
        "Elb",
        "El1",
        "Er2",
        "Erb",
        "El4",
        "El7",
        "Er1",
        "Era",
        "El3",
        "Er4",
        "El6",
        "Er7",
        "Ela",
        "El2",
    ]
}

name_for_cid[3034] = "BodyPositionChannel"
cid_concepts[3034] = {
    "DCM": ["PatientPosition", "PatientElevation", "PatientRotationLongitudinal"]
}

name_for_cid[3035] = "EEGAnnotation–NeurophysiologicEnumeration"
cid_concepts[3035] = {
    "DCM": ["SleepStageN3", "SleepStageN2", "SleepStageN1", "LineNoiseArtifact"],
    "MDC": [
        "ExternalInterferenceArtifactualActivity",
        "PeriodicMovementsOfSleepWithArousals",
        "FastIrregularEyeMovements",
        "SharpAppearingOrEpileptiformActivity",
        "SleepSpindle",
        "BackgroundActivityDelta",
        "PeriodicTriphasicWaves",
        "PhantomSpikeAndWaveActivity",
        "RhythmicSharpWaves",
        "SharpTransient",
        "PulseArtifactualActivity",
        "MyogenicHemifacialSpasms",
        "PeriodicBurstsWithSuppressions",
        "SleepSawtoothWave",
        "SleepStageUnstageable",
        "BackgroundActivityGamma",
        "EyeRelatedActivityElectroretinogram",
        "RapidEyeMovements",
        "EyeRelatedActivity",
        "ExtraocularMuscleActivity",
        "EpilepticOrPotentiallyEpileptogenicSpike",
        "Wicket",
        "QuasiperiodicCerebralSharpWaves",
        "BackgroundActivityBisynchronousDelta",
        "RespiratoryArtifactualActivity",
        "BurstSuppression",
        "MyogenicPalatalMyoclonus",
        "ElectrodeInstrumentalArtifactualActivity",
        "MultipleSpike",
        "PeriodicCerebralComplexes",
        "EyeRelatedPhotodrivingActivity",
        "SleepStageShift",
        "ECGArtifactualActivity",
        "BackgroundActivityAlpha",
        "MyogenicNoncerebralActivity",
        "_14And6HzPositiveBursts",
        "EyeBlinks",
        "SleepVWave",
        "ArtifactualActivity",
        "SleepStageWake",
        "PeriodicEpileptiformDischarges",
        "MyogenicTremorActivity",
        "EyeRelatedPhotomyogenicActivity",
        "BackgroundActivityBeta",
        "SpikeAndWaveComplex",
        "EpilepticOrPotentiallyEpileptogenicActivity",
        "SleepArousal",
        "BackgroundMuActivity",
        "MovementArtifactualActivity",
        "SleepStageREMWithSleepSpindle",
        "PeriodicAndQuasiperiodicCerebralActivity",
        "ZetaWave",
        "SleepKComplex",
        "BackgroundActivitySlowFusedTransients",
        "BackgroundActivity",
        "MyoclonicActivity",
        "LambdaWave",
        "NystagmoidEyeMovements",
        "SleepStageREM",
        "MultipleIndependentSpikesAndAsynchronousSlowWaves",
        "MyogenicNoncerebralMyokymia",
        "SmallSharpSpike",
        "PeriodicSharpWaves",
        "SleepFWave",
        "BackgroundActivityArrhythmicDelta",
        "GlossokineticArtifactualActivity",
        "PeriodicMovementsOfSleep",
        "AtypicalSpikeAndWaveComplex",
        "SleepAwakening",
        "AlphadeltaSleep",
        "BackgroundActivityTheta",
        "SleepStageUnspecified",
        "QuasiperiodicTriphasicWaves",
        "EpilepticOrPotentiallyEpileptogenicSharpWave",
        "PositiveOccipitalSharpTransientOfSleep",
        "BackgroundActivityBisynchronousTheta",
        "SharpAndSlowWaveComplex",
        "TriphasicWave",
        "SleepActivityAndEvent",
        "SwallowingAndChewingArtifactualActivity",
        "MyogenicNoncerebralFacialSynkinesis",
        "SlowEyeMovements",
        "SweatOfGalvanicArtifactualActivity",
        "BackgroundActivitySigma",
        "EyeRelatedPhotoparadoxysmalActivity",
        "PeriodicSuppressions",
    ],
    "SCT": ["Drowsy"],
}

name_for_cid[3036] = "EMGAnnotation–NeurophysiologicalEnumeration"
cid_concepts[3036] = {
    "MDC": [
        "EMGEndplateNoise",
        "EMGDoubletWaveform",
        "EMGComplexRepetitiveDischarge",
        "EMGFibrillationPotential",
        "EMGUnspecifiedWaveform",
        "EMGPositiveSharpWave",
        "EMGUnspecifiedIterativeDischarge",
        "EMGMotorUnitPotential",
        "EMGMyotonicDischarge",
        "EMGInsertionalActivity",
        "EMGWaveformAfterDischarge",
        "EMGEndplateSpike",
        "EMGTripletWaveform",
        "EMGMyokymicDischarge",
        "EMGMultipletWaveform",
        "EMGCrampDischarge",
        "EMGWaveformUnderVoluntaryControl",
        "EMGFasciculationPotential",
    ]
}

name_for_cid[3037] = "EOGAnnotation–NeurophysiologicalEnumeration"
cid_concepts[3037] = {
    "MDC": [
        "EyesOpen",
        "SlowEyeMovement",
        "EyeBlink",
        "EyesClosed",
        "REM",
        "OtherEyeMovement",
        "Saccade",
    ]
}

name_for_cid[3038] = "PatternEvent"
cid_concepts[3038] = {
    "DCM": ["HyperventilationEnd", "HyperventilationBegin", "PostHyperventilation"],
    "MDC": [
        "AtrialFlutter",
        "SpikesAndWaves",
        "ClinicalSeizureDischarge",
        "Grade1AVBlock",
        "ArrhythmiaEvent",
        "Grade2AVBlock",
        "ThirdDegreeAVBlock",
        "IrregularHeartRate",
        "Desaturation",
        "AtrialFibrillation",
        "IrregularRhythm",
        "SupraventricularExtrasystole",
        "Apnea",
        "Apnea30Sec",
        "SharpSpikes",
        "PacerArtifact",
        "Bradycardia",
        "Asystole",
        "Apnea15Sec",
        "SinusTachycardia",
    ],
    "SCT": ["Hyperventilation"],
}

name_for_cid[3039] = "DevicerelatedandEnvironmentrelatedEvent"
cid_concepts[3039] = {
    "DCM": [
        "PreamplifierDisconnected",
        "EventButtonTest",
        "EventButtonPressed",
        "BreakoutBoxDisconnected",
        "BreakoutBoxConnected",
        "VideoRecordingOn",
        "TapTestEnd",
        "TapTestBegin",
        "PreamplifierConnected",
        "VideoRecordingOff",
    ],
    "MDC": ["PowerLineProblem", "Artifact", "LeadDisconnected", "PowerSupplyProblem"],
}

name_for_cid[3040] = "EEGAnnotationNeurologicalMonitoringMeasurement"
cid_concepts[3040] = {"MDC": ["Arousal", "CircumHead"]}

name_for_cid[3041] = "NeurophysiologicStimulationMode"
cid_concepts[3041] = {"MDC": ["FlashStimulus"]}

name_for_cid[3042] = "AnalogWaveformFilter"
cid_concepts[3042] = {
    "DCM": [
        "EllipticFilter",
        "ChebyshevFilter",
        "ButterworthFilter",
        "MDerivedFilter",
        "KTypeFilter",
        "GaussianFilter",
        "BesselFilter",
        "ZobelFilter",
        "BodeFilter",
        "DoubleMDerivedFilter",
    ]
}

name_for_cid[3043] = "DigitalWaveformFilter"
cid_concepts[3043] = {"DCM": ["BiquadFilter", "IIRFilter", "CICFilter", "FIRFilter"]}

name_for_cid[3044] = "WaveformFilterLookupTableInputFrequencyUnit"
cid_concepts[3044] = {
    "DCM": ["HzAngular", "HzNormalized"],
    "UCUM": ["RadiansPerSecond", "Hz"],
}

name_for_cid[3045] = "WaveformFilterLookupTableOutputMagnitudeUnit"
cid_concepts[3045] = {
    "DCM": ["MagnitudeNormalized"],
    "UCUM": ["Microvolt", "DecibelsMillivolt", "Millivolt", "DecibelsMicrovolt"],
}

name_for_cid[3046] = "SleepStage"
cid_concepts[3046] = {
    "DCM": ["SleepStageN3", "SleepStageN2", "SleepStageN1"],
    "MDC": [
        "SleepAwakening",
        "AlphadeltaSleep",
        "SleepStageUnspecified",
        "SleepSpindle",
        "SleepArousal",
        "SleepStageREMWithSleepSpindle",
        "PositiveOccipitalSharpTransientOfSleep",
        "SleepKComplex",
        "SleepActivityAndEvent",
        "SleepStageShift",
        "SleepStageREM",
        "SleepSawtoothWave",
        "SleepStageUnstageable",
        "SleepVWave",
        "SleepStageWake",
        "SleepFWave",
    ],
}

name_for_cid[3047] = "WaveformAnnotationClassification"
cid_concepts[3047] = {
    "DCM": [
        "EEGAnnotation",
        "PatternEvent",
        "PatientConsciousness",
        "DeviceRelatedAndEnvironmentRelatedEvent",
        "EOGAnnotation",
        "EMGAnnotation",
    ]
}

name_for_cid[3048] = "WaveformAnnotationsDocumentTitle"
cid_concepts[3048] = {
    "DCM": [
        "NeurophysiologyAutomatedAnalysisAnnotations",
        "NeurophysiologyPostHocReviewAnnotations",
        "NeurophysiologyRecordingAnnotations",
    ]
}

name_for_cid[3049] = "EEGProcedure"
cid_concepts[3049] = {
    "SCT": ["ScalpEEG", "EEG", "AmbulatoryEEG", "VideoEEG", "SleepEEG"]
}

name_for_cid[3050] = "PatientConsciousness"
cid_concepts[3050] = {"SCT": ["Asleep", "Awake", "Drowsy"]}

name_for_cid[3083] = "UnitRadioactivity"
cid_concepts[3083] = {"UCUM": ["Millicurie", "Becquerel", "Megabecquerel"]}

name_for_cid[3090] = "TimeSynchronizationChannelType"
cid_concepts[3090] = {
    "DCM": [
        "XRayOnTrigger",
        "XRayFluoroscopyOnSignal",
        "DigitalTimecodeNOS",
        "IRIGBTimecode",
        "ECGBasedGatingSignalProcessed",
    ]
}

name_for_cid[3101] = "CardiacProceduralStateValue"
cid_concepts[3101] = {
    "DCM": ["ReinjectionState", "DelayedRedistributionState", "RedistributionState"],
    "SCT": ["CardiacStressState", "RestingState"],
}

name_for_cid[3102] = "Rest/StressState"
cid_concepts[3102] = {"DCM": ["CardiacStressState"], "SCT": ["RestingState"]}

name_for_cid[3104] = "CardiacSynchronizationTechnique"
cid_concepts[3104] = {
    "DCM": ["ProspectiveGating", "RealTimeAcquisition", "Paced", "RetrospectiveGating"]
}

name_for_cid[3106] = "PETCardiologyProtocol"
cid_concepts[3106] = {
    "DCM": [
        "PETMyocardialViabilityRestAndStress",
        "PETMyocardialViabilityRestOnly",
        "PETMyocardialViabilityStressOnly",
        "PETMyocardialPerfusionRestAndStress",
        "PETMyocardialPerfusionRestOnly",
        "PETMyocardialPerfusionStressOnly",
    ]
}

name_for_cid[3107] = "PETCardiologyRadiopharmaceutical"
cid_concepts[3107] = {"SCT": ["_82Rubidium", "FluorodeoxyglucoseF18", "_13Nitrogen"]}

name_for_cid[3108] = "NM/PETProcedure"
cid_concepts[3108] = {
    "SCT": [
        "PETHeartStudy",
        "PETBrainStudy",
        "RadioisotopeStudyOfRespiratorySystem",
        "PETStudyForLocalizationOfTumor",
        "RadioisotopeStudyOfGenitourinarySystem",
        "PETBreastStudy",
        "RadioisotopeStudyOfMusculoskeletalSystem",
        "RadioisotopeStudyOfHematopoieticSystem",
        "RadioisotopeStudyOfGastrointestinalSystem",
        "NuclearMedicineCardiovascularStudy",
        "RadioisotopeStudyOfEndocrineSystem",
        "RadionuclideStudyForLocalizationOfInflammatoryDisease",
        "RadionuclideLocalizationOfTumor",
    ]
}

name_for_cid[3110] = "NuclearCardiologyProtocol"
cid_concepts[3110] = {
    "DCM": [
        "StressTechnetiumRestTechnetium1DayProcedure",
        "NMMyocardialViabilityProcedure",
        "RestTechnetiumStressTechnetium1DayProcedure",
        "RestTechnetiumStressTechnetium2DayProcedure",
        "RestThalliumStressTechnetiumProcedure",
    ],
    "SCT": ["StressThalliumProcedure"],
}

name_for_cid[3111] = "NuclearCardiologyRadiopharmaceutical"
cid_concepts[3111] = {"SCT": ["Tc99mTetrofosmin", "Thallium201", "Tc99mSestamibi"]}

name_for_cid[3112] = "AttenuationCorrection"
cid_concepts[3112] = {
    "DCM": [
        "CTBasedAttenuationCorrection",
        "NoAttenuationCorrection",
        "AlgorithmicAttenuationCorrection",
        "NMTransmissionAttenuationCorrection",
    ]
}

name_for_cid[3113] = "PerfusionDefectType"
cid_concepts[3113] = {
    "DCM": ["FalsePositiveDefectFinding"],
    "SCT": [
        "PartiallyReversibleMyocardialPerfusionDefect",
        "FixedMyocardialPerfusionDefect",
        "ReversibleMyocardialPerfusionDefect",
    ],
}

name_for_cid[3114] = "StudyQuality"
cid_concepts[3114] = {
    "DCM": [
        "PoorImageQuality",
        "ExcellentImageQuality",
        "GoodImageQuality",
        "UnusableQualityRendersImageUnusable",
    ]
}

name_for_cid[3115] = "StressImagingQualityIssue"
cid_concepts[3115] = {
    "DCM": [
        "BreastAttenuation",
        "DiaphragmaticAttenuation",
        "BodyHabitusAttenuation",
        "MotionBlur",
    ],
    "SCT": ["SubdiaphragmaticUptake"],
}

name_for_cid[3116] = "NMExtracardiacFinding"
cid_concepts[3116] = {
    "SCT": [
        "IncreasedLungUptake",
        "AbnormalExtracardiacUptake",
        "NormalExtracardiacUptake",
    ]
}

name_for_cid[3117] = "AttenuationCorrectionMethod"
cid_concepts[3117] = {"DCM": ["OSEMAlgorithm", "ChangMethod"]}

name_for_cid[3118] = "LevelofRisk"
cid_concepts[3118] = {
    "SCT": [
        "ModerateToHighRisk",
        "HighRisk",
        "LowToModerateRisk",
        "ModerateRisk",
        "NormalRisk",
        "LowRisk",
        "UncertainRisk",
    ]
}

name_for_cid[3119] = "LVFunction"
cid_concepts[3119] = {"SCT": ["Normal", "ImpairedLeftVentricularFunction"]}

name_for_cid[3120] = "PerfusionFinding"
cid_concepts[3120] = {"SCT": ["Abnormal", "Equivocal", "MyocardialPerfusionNormal"]}

name_for_cid[3121] = "PerfusionMorphology"
cid_concepts[3121] = {
    "SCT": ["MyocardialIschemia", "MyocardialInfarction", "MixedIschemiaAndInfarction"]
}

name_for_cid[3122] = "VentricularEnlargement"
cid_concepts[3122] = {
    "SCT": [
        "MarkedlyEnlargedCardiacChamber",
        "MildlyEnlargedCardiacChamber",
        "ModeratelyEnlargedCardiacChamber",
        "NormalSizeCardiacChamber",
    ]
}

name_for_cid[3200] = "StressTestProcedure"
cid_concepts[3200] = {
    "SCT": [
        "ExerciseStressTest",
        "PharmacologicStressTest",
        "PacedStressTest",
        "PharmacologicAndExerciseStressTest",
    ]
}

name_for_cid[3201] = "IndicationsforStressTest"
cid_concepts[3201] = {
    "SCT": [
        "AbnormalExerciseToleranceTest",
        "ConductionDisorderOfTheHeart",
        "Palpitations",
        "PreOperative",
        "AnginaPectoris",
        "Syncope",
        "HeartFailure",
        "PostPTCA",
        "AbnormalECG",
        "CoronaryArteryDisease",
        "SupraventricularTachycardia",
        "HistoryOfCABG",
        "LeftBundleBranchBlock",
        "ValvularHeartDisease",
        "ChestPain",
        "Hypertension",
        "OccupationalRequirement",
        "HistoryOfMyocardialInfarction",
        "HeartDiseaseRiskFactors",
        "Dyspnea",
    ]
}

name_for_cid[3202] = "ChestPain"
cid_concepts[3202] = {
    "DCM": ["AnginalEquivalent"],
    "SCT": [
        "AtypicalAngina",
        "TypicalAngina",
        "NoncardiacChestPain",
        "ChestPainNotPresent",
        "StableAngina",
        "UnstableAngina",
    ],
}

name_for_cid[3203] = "ExerciserDevice"
cid_concepts[3203] = {"SCT": ["BicycleErgometer", "Treadmill", "ArmErgometer"]}

name_for_cid[3204] = "StressAgent"
cid_concepts[3204] = {
    "SCT": [
        "Dobutamine",
        "Adenosine",
        "Dipyridamole",
        "Atropine",
        "AdenosineA2ReceptorAgonist",
    ]
}

name_for_cid[3205] = "IndicationsforPharmacologicalStressTest"
cid_concepts[3205] = {
    "DCM": ["PatientWeightExceedsEquipmentLimit"],
    "SCT": [
        "RecentMyocardialInfarction",
        "Asthenia",
        "AtaxiaOrIncoordination",
        "OpenWoundOfLowerLimb",
        "Cachexia",
        "TransientLimbParalysis",
        "PatientHasPacemaker",
        "FractureOfLowerLimb",
        "DependenceOnEnablingMachineOrDevice",
        "PeripheralVascularDisease",
        "LeftBundleBranchBlock",
        "ParalyticSyndrome",
        "GaitProblem",
        "RequestByPhysician",
        "PulmonaryDisease",
        "LowerLimbAmputation",
        "CannotReachTargetHeartRate",
    ],
}

name_for_cid[3206] = "NoninvasiveCardiacImagingProcedure"
cid_concepts[3206] = {
    "SCT": [
        "SPECT",
        "PETHeartStudy",
        "CardiacBloodPoolImaging",
        "NuclearMedicineCardiovascularStudy",
        "CardiacMRI",
        "Echocardiography",
    ]
}

name_for_cid[3207] = "StressTestProcedurePhase"
cid_concepts[3207] = {
    "SCT": [
        "CardiacStressState",
        "Hyperventilation",
        "PeakCardiacStressState",
        "CardiacStressRecoveryState",
        "RestingState",
    ]
}

name_for_cid[3208] = "ExerciseECGSummaryCode"
cid_concepts[3208] = {
    "SCT": [
        "ExerciseECGAbnormal",
        "ExerciseECGEquivocal",
        "ExerciseECGNormal",
        "NotPerformed",
    ]
}

name_for_cid[3209] = "StressImagingSummaryCode"
cid_concepts[3209] = {
    "SCT": [
        "ImagingResultEquivocal",
        "ImagingResultNormal",
        "ImagingResultAbnormal",
        "NotPerformed",
    ]
}

name_for_cid[3210] = "SpeedofResponse"
cid_concepts[3210] = {"SCT": ["Normal", "Blunted", "Accentuated"]}

name_for_cid[3211] = "BPResponse"
cid_concepts[3211] = {"SCT": ["Normal", "Blunted", "Hypertensive", "Hypotensive"]}

name_for_cid[3212] = "TreadmillSpeed"
cid_concepts[3212] = {"UCUM": ["KilometersPerHour", "MilesPerHour"]}

name_for_cid[3213] = "StressHemodynamicFinding"
cid_concepts[3213] = {
    "SCT": [
        "ExertionalHypotension",
        "ChronotropicIncompetence",
        "ExertionalHypertension",
    ]
}

name_for_cid[3215] = "PerfusionFindingMethod"
cid_concepts[3215] = {"SCT": ["ECGAnalysis", "ImageAnalysis"]}

name_for_cid[3217] = "ComparisonFinding"
cid_concepts[3217] = {
    "DCM": ["AgreementWithPriorFindings", "DisagreementWithPriorFindings"]
}

name_for_cid[3220] = "StressSymptom"
cid_concepts[3220] = {
    "DCM": ["AnginalEquivalent"],
    "SCT": [
        "Dizziness",
        "Nausea",
        "Claudication",
        "AtypicalAngina",
        "TypicalAngina",
        "Syncope",
        "Flushing",
        "Fatigue",
        "NoncardiacChestPain",
        "UnstableAngina",
        "ChestPain",
        "ChestPainNotPresent",
        "ChestDiscomfort",
        "Dyspnea",
        "StableAngina",
    ],
}

name_for_cid[3221] = "StressTestTerminationReason"
cid_concepts[3221] = {
    "SCT": [
        "HypotensiveEpisode",
        "EndOfProtocol",
        "HypertensiveEpisode",
        "ConductionDisorderOfTheHeart",
        "TargetHeartRateAchieved",
        "Claudication",
        "ChestPain",
        "Syncope",
        "AbnormalECG",
        "Fatigue",
        "PatientRefusedExerciseTest",
        "Dyspnea",
    ]
}

name_for_cid[3227] = "QTcMeasurement"
cid_concepts[3227] = {
    "MDC": [
        "QtcPerLeadUsingFredericiaFormula",
        "QtcIntervalPerLead",
        "QtcIntervalGlobal",
        "QtcGlobalUsingFredericiaFormula",
        "QtcGlobalUsingBazettFormula",
        "QtcPerLeadUsingBazettFormula",
    ]
}

name_for_cid[3228] = "ECGTimingMeasurement"
cid_concepts[3228] = {
    "MDC": [
        "PDurationGlobal",
        "PPIntervalGlobal",
        "PPIntervalPerLead",
        "QTIntervalGlobal",
        "QRSDurationPerLead",
        "QTIntervalPerLead",
        "PDurationPerLead",
        "QRSDurationGlobal",
        "PRIntervalGlobal",
        "PRIntervalPerLead",
        "RRIntervalPerLead",
        "RRIntervalGlobal",
    ]
}

name_for_cid[3229] = "ECGAxisMeasurement"
cid_concepts[3229] = {"MDC": ["QRSAxis", "PAxis", "TAxis"]}

name_for_cid[3230] = "ECGFinding"
cid_concepts[3230] = {
    "SCT": [
        "VentricularFibrillation",
        "VentricularPreExcitation",
        "AtrialPrematureContraction",
        "STSegmentDepression",
        "RightBundleBranchBlock",
        "SecondDegreeAtrioventricularBlock",
        "VentricularPrematureContraction",
        "ThirdDegreeAtrioventricularBlock",
        "SecondarySTTAbnormality",
        "ECGNormal",
        "EarlyRepolarization",
        "NonSustainedVentricularTachycardia",
        "IntraventricularConductionDisturbance",
        "AtrialFibrillation",
        "BifascicularBlock",
        "VentricularTachycardia",
        "STSegmentElevation",
        "SupraventricularTachycardia",
        "LeftAnteriorFascicularBlock",
        "IncompleteRightBundleBranchBlock",
        "LeftBundleBranchBlock",
        "IncompleteLeftBundleBranchBlock",
        "FirstDegreeAtrioventricularBlock",
        "NonspecificSTTAbnormality",
        "LeftPosteriorFascicularBlock",
    ]
}

name_for_cid[3231] = "STSegmentFinding"
cid_concepts[3231] = {
    "DCM": [
        "StronglyPositive",
        "StronglyPositiveSTElevation",
        "NonDiagnosticRestingSTAbnormalities",
        "NonDiagnosticVentricularPacingOrLBBB",
        "NonDiagnosticLowHeartRate",
    ],
    "SCT": ["WeaklyPositive", "STIntervalNormal", "Positive"],
}

name_for_cid[3232] = "STSegmentLocation"
cid_concepts[3232] = {
    "SCT": [
        "LeftVentricleLateralSegment",
        "LeftVentricleInferiorSegment",
        "LeftVentricleApicalSegment",
        "LeftVentricleSeptalSegment",
        "LeftVentricleAnteriorSegment",
    ]
}

name_for_cid[3233] = "STSegmentMorphology"
cid_concepts[3233] = {
    "DCM": [
        "STDepressionHorizontal",
        "STDepressionUpsloping",
        "STDepressionDownsloping",
    ],
    "SCT": ["STSegmentElevation", "STSegmentDepression"],
}

name_for_cid[3234] = "EctopicBeatMorphology"
cid_concepts[3234] = {
    "SCT": [
        "VentricularTachycardiaPolymorphic",
        "VentricularBigeminy",
        "VentricularTachycardia",
        "UnifocalPvcs",
        "MultifocalPvcs",
    ]
}

name_for_cid[3235] = "PerfusionComparisonFinding"
cid_concepts[3235] = {
    "SCT": [
        "ResolutionOfIschemia",
        "MoreIschemia",
        "NewIschemia",
        "LessIschemia",
        "NoChange",
        "NewInfarction",
    ]
}

name_for_cid[3236] = "ToleranceComparisonFinding"
cid_concepts[3236] = {"SCT": ["DecreasedTolerance", "NoChange", "IncreasedTolerance"]}

name_for_cid[3237] = "WallMotionComparisonFinding"
cid_concepts[3237] = {
    "SCT": ["NewWallMotionAbnormality", "NoChange", "ImprovementOfWallMotion"]
}

name_for_cid[3238] = "StressScoringScale"
cid_concepts[3238] = {
    "DCM": [
        "STHRIndex",
        "RatioOfAchievedToPredictedFunctionalCapacity",
        "AerobicIndex",
        "RatioOfAchievedToPredictedMaximalOxygenConsumption",
    ],
    "SCT": ["DukeTreadmillScore"],
}

name_for_cid[3239] = "PerceivedExertionScale"
cid_concepts[3239] = {"DCM": ["BorgCR10Scale", "BorgRPEScale"]}

name_for_cid[3240] = "ElectrophysiologyMeasurementFunction/Technique"
cid_concepts[3240] = {
    "DCM": [
        "PacingElectricalStimulusVoltage",
        "MonopoleSignal",
        "HisBundleElectrogram",
        "DifferentialSignal",
        "VoltageMeasurementByMappingCatheter",
        "VoltageMeasurementByBasketCatheter",
        "VoltageMeasurement",
        "RadioFrequencyAblationPower",
    ]
}

name_for_cid[3241] = "HemodynamicMeasurementTechnique"
cid_concepts[3241] = {
    "SCT": [
        "ThermistorCatheterMethod",
        "TipManometerMethod",
        "FiberopticCatheterMethod",
        "DopplerCatheterMethod",
        "FluidFilledCatheterMethod",
        "AveragedHemodynamicMeasurementMethod",
        "PulmonaryCapillaryWedgeMethod",
        "ConductanceCatheterMethod",
        "StaticCatheterMethod",
        "CompositeHemodynamicMeasurementMethod",
        "WedgeMethod",
        "ComputedHemodynamicMeasurementMethod",
        "DualCatheterMethod",
        "PullbackMethod",
    ]
}

name_for_cid[3250] = "CatheterizationProcedurePhase"
cid_concepts[3250] = {
    "SCT": [
        "CardiacCatheterizationBaselinePhase",
        "CardiacCatheterizationImageAcquisitionPhase",
        "CatheterizationOfBothLeftAndRightHeartWithGraft",
        "TransseptalCatheterization",
        "DrugInfusionChallenge",
        "CardiacCatheterizationInterventionPhase",
        "CardiacCatheterizationTherapyPhase",
        "CatheterizationOfBothLeftAndRightHeartWithoutGraft",
        "CardiacCatheterizationPostContrastPhase",
        "RestingState",
        "CardiacCatheterizationPreInterventionPhase",
        "CardiacCatheterizationTestChallengePhase",
        "CatheterizationOfLeftHeart",
        "CardiacCatheterizationPostInterventionPhase",
        "CardiacCatheterizationBailoutPhase",
        "CatheterizationOfRightHeart",
        "ExerciseChallenge",
    ]
}

name_for_cid[3254] = "ElectrophysiologyProcedurePhase"
cid_concepts[3254] = {
    "SCT": [
        "AtrialEffectiveRefractoryPeriodEvaluationOf",
        "PostDefibrillationProcedurePhase",
        "VentricularEffectiveRefractoryPeriodEvaluationOf",
        "ElectrophysiologyProcedureBaselinePhase",
        "CarotidSinusMassageProcedurePhase",
        "PostAblationPhase",
        "ElectrophysiologyMappingPhase",
        "SinusNodeRecoveryTimeEvaluationOf",
        "RadiofrequencyAblationProcedurePhase",
    ]
}

name_for_cid[3261] = "StressProtocol"
cid_concepts[3261] = {
    "SCT": [
        "StressTestUsingBicycleErgometer",
        "EllestadProtocol",
        "PharmacologicStressProtocol",
        "DobutamineStressProtocol",
        "NaughtonProtocol",
        "PepperProtocol",
        "AdenosineStressProtocol",
        "StressTestUsingCardiacPacing",
        "PharmacologicAndExerciseStressTest",
        "ModifiedNaughtonProtocol",
        "DipyridamoleStressProtocol",
        "RampProtocol",
        "ExerciseStressECGTest",
        "BalkeProtocol",
        "ModifiedBruceProtocol",
        "BruceProtocol",
    ]
}

name_for_cid[3262] = "ECGPatientStateValue"
cid_concepts[3262] = {
    "SCT": [
        "BaselineState",
        "ExerciseState",
        "SupineBodyPosition",
        "PostExerciseState",
        "RestingState",
    ]
}

name_for_cid[3263] = "ElectrodePlacementValue"
cid_concepts[3263] = {
    "MDC": [
        "_12LeadFromLimbLeadsIIIAndOneOrMoreVLeads",
        "Unspecified12LeadSystem",
        "Standard12LeadAndXYZ",
        "Standard12LeadPositionsElectrodesPlacedIndividually",
        "Standard12LeadAndNEHB",
        "MasonLikarLeadPositionsElectrodesPlacedIndividually",
        "Standard12LeadAndCC5CM5ML",
        "MasonLikarLeadPositionsV1V6InElectrodePad",
        "Standard12LeadAndCM5CC5CH5",
        "_12LeadElectrodePad",
        "_12LeadDerivedFromFrankXYZLeads",
        "Standard12LeadWithExtraLeadsToTheRightAndOrLeftSides",
        "Standard12LeadExtendedToTheRightByV5RV4RV3R",
        "_12LeadDerivedFromNonStandardLeads",
        "Standard12LeadExtendedToTheLeftByV7V8V9",
        "_12LeadForBicycleExerciseTestingLimbLeadsOnBackOfPatient",
        "Standard12LeadPositionsOneIntercostalSpaceHigher",
        "UnspecifiedXYZLeadSystem",
        "FrankXYZLeadSystem",
        "McfeeParungaoXYZLeadSystem",
        "CubeXYZLeadSystem",
        "BipolarUncorrectedXYZLeadSystem",
        "PseudoOrthogonalXYZLeadSystem",
        "XYZLeadsDerivedFromStandard12Lead",
        "NEHBLeadSystem",
        "_3LeadSystemCC5CM5ML",
        "_3LeadSystemCC5CM5CH5",
        "_12LeadFromFrankLeadsXYZLeadsByDowerTransformation",
        "_12LeadFromEASILeadsESASAIByDowerEASITransformation",
    ]
}

name_for_cid[3271] = "HemodynamicPhysiologicalChallenge"
cid_concepts[3271] = {
    "SCT": [
        "LegUp",
        "ValsalvaManeuver",
        "PostVolumeChallenge",
        "HeldVentilation",
        "DrugInfusion",
        "HeadUp",
        "VagalStimulation",
        "Pacing",
        "NegativeLowerBodyPressure",
        "Handgrip",
        "HeldInspiration",
        "ExerciseChallenge",
    ]
}

name_for_cid[3335] = "ECGAnnotation"
cid_concepts[3335] = {
    "MDC": [
        "CalibrationPulse",
        "AtriumAntitachycardiaPaceSpike",
        "AberratedAtrialPrematureBeatAshmanBeat",
        "PreExcitationBeat",
        "LeftVentricleAntibradycardiaPaceSpike",
        "LeftPosteriorFascicularBlockBeat",
        "AtrialFlutterWave",
        "AbnormalBeat",
        "EntireBeatPonToToffExcludingU",
        "UWave",
        "LeftBundleBranchBlockBeat",
        "TWaveChange",
        "JPoint",
        "TransthoracicCardioversionSpike",
        "NoNoise",
        "VentricleDefibrillationSpike",
        "LeftAtriumAntibradycardiaPaceSpike",
        "NoECGSignalIsAvailable",
        "SWave",
        "QSWave",
        "FusionOfVentricularAndNormalBeat",
        "AtrialPrematureContraction",
        "BilateralBundleBranchBlockBeat",
        "DefibrillationSpike",
        "AtriumCardioversionSpike",
        "ModerateNoiseBeatsCanBeDetectedButCannotBeClassified",
        "IsoelectricRegionFromGlobalQRSOnsetToSpecificLeadOnset",
        "IncompleteRightBundleBranchBlockBeat",
        "VentricularAntibradycardiaPaceSpike",
        "VentricularEscapeBeat",
        "TWaveSecondDeflectionInTWave",
        "PacemakerLearningBeat",
        "IsolatedQRSLikeArtifact",
        "WolfParkinsonTypeBQRSNegativeInV1V2",
        "AtriumAntibradycardiaPaceSpike",
        "VentricleAntitachycardiaPaceSpike",
        "PRSegment",
        "EntireQRSExcludingPTAndU",
        "TransthoracicAntibradycardiaPaceSpike",
        "VentricularFlutterWave",
        "OsborneWave",
        "Notch",
        "QWave",
        "BifascicularBlockBeat",
        "SWaveThirdDeflectionInSWave",
        "DominantBeat",
        "PWaveSecondDeflectionInPWave",
        "RWaveSecondDeflectionInRWave",
        "IncompleteLeftBundleBranchBlockBeat",
        "RightVentricleAntibradycardiaPaceSpike",
        "AtriumDefibrillationSpike",
        "ROnTPrematureVentricularBeat",
        "PacemakerFusionBeat",
        "LownGanongLevineSyndromeNormalQRS",
        "TransthoracicDefibrillationSpike",
        "AtrialEscapeBeat",
        "NonConductedPWave",
        "AntitachycardiaPaceSpike",
        "JunctionalNodalPrematureContraction",
        "WolfParkinsonWhiteSyndromeLessSpecific",
        "IntraventricularConductionDisturbance",
        "IsoelectricPointOrSegment",
        "LeftAnteriorFascicularBlockBeat",
        "EntireBeatQonToToffExcludingPAndU",
        "DeltaWave",
        "STChange",
        "NormalBeatSinusBeatNormalConduction",
        "RightAtriumAntibradycardiaPaceSpike",
        "SWaveSecondDeflectionInSWave",
        "VentricleCardioversionSpike",
        "PWave",
        "RWave",
        "WolfParkinsonTypeAQRSPositiveInV1V2",
        "VentricularPrematureContractionBeat",
        "STMeasurementPoint",
        "AntibradycardiaPaceSpike",
        "SupraventricularPrematureContraction",
        "CardioversionSpike",
        "TrifascicularBlockBeat",
        "RightBundleBranchBlockBeat",
        "STSegment",
        "TUFusedWave",
        "IsoelectricRegionFromSpecificLeadQRSOffsetToGlobalOffset",
        "SevereNoiseBeatsCannotBeDetectedOrClassified",
        "TWave",
        "VentricularActivationTime",
        "TransthoracicAntitachycardiaPaceSpike",
        "JunctionalNodalEscapeBeat",
        "UnclassifiableBeat",
        "RWaveThirdDeflectionInRWave",
        "SupraventricularEscapeBeat",
        "PWaveThirdDeflectionInPWave",
    ]
}

name_for_cid[3337] = "HemodynamicAnnotation"
cid_concepts[3337] = {
    "DCM": [
        "BeatsDetectedRejected",
        "AWavePeakPressure",
        "_70PercentOfThermalDyeDilutionCO",
        "ValveOpen",
        "BeatsDetectedAccepted",
        "AWavePressureAverage",
        "VWavePeakPressure",
        "_35PercentOfThermalDyeDilutionCO",
        "ValveClose",
        "VWavePressureAverage",
        "StartOfInspiration",
        "VMaxMyocardial",
        "StartOfThermalCO",
        "EndOfSystole",
        "Tau",
        "IndicatorMeanTransitTime",
        "MaxNegDpDt",
        "StartOfExpiration",
        "PeakOfThermalCardiacOutputBolus",
        "EndOfExpiration",
        "MaxDpDt",
        "EndOfInspiration",
    ],
    "SCT": [
        "AverageDiastolicBloodPressure",
        "MinimumDiastolicBloodPressure",
        "MaximumSystolicBloodPressure",
        "MeanBloodPressure",
        "EndDiastole",
        "AverageSystolicBloodPressure",
    ],
}

name_for_cid[3339] = "ElectrophysiologyAnnotation"
cid_concepts[3339] = {
    "DCM": [
        "StimulationAtRate4Interval",
        "AblationOff",
        "StimulationAtRate3Interval",
        "StimulationAtRate2Interval",
        "AblationOn",
        "StartOfAtrialContraction",
        "SWave",
        "StimulationAtRate1Interval",
        "StartOfAtrialContractionSubsequent",
        "PWave",
        "VWave",
        "HISBundleWave",
        "TWave",
        "RWave",
        "QWave",
        "VWaveOfNextBeat",
    ]
}

name_for_cid[3400] = "ProcedureLogTitle"
cid_concepts[3400] = {"DCM": ["CathLabProcedureLog"]}

name_for_cid[3401] = "LogNoteType"
cid_concepts[3401] = {
    "DCM": [
        "PatientStatusOrEvent",
        "ProcedureNote",
        "NursingNote",
        "PhysicianNote",
        "TechNote",
    ]
}

name_for_cid[3402] = "PatientStatusandEvent"
cid_concepts[3402] = {
    "DCM": [
        "PatientPronouncedDead",
        "PatientTransferredToMorgue",
        "PatientGivenPostProcedureInstruction",
        "PatientDischargedFromDepartment",
        "PatientTransferredToCCU",
        "HemostasisNotAchievedOozing",
        "HemostasisNotAchievedActivelyBleeding",
        "PatientTransferredToHoldingArea",
        "PatientDisconnectedFromContinuousMonitoring",
        "PatientTransferredToSurgery",
        "HemostasisAchieved",
        "PatientHasRespiratoryDifficulty",
        "PatientCoughed",
        "PatientConnectedToContinuousMonitoring",
        "PatientAssistedToTable",
        "PatientUnresponsive",
        "PatientPreppedAndDraped",
        "PatientAdvanceDirectiveGiven",
        "PatientSedated",
        "NilPerOsNPOStatusConfirmed",
        "PatientAsleep",
        "PatientGivenPreProcedureInstruction",
        "PatientAlert",
        "PatientInformedConsentGiven",
        "PatientRestless",
        "PatientCalledToProcedureRoom",
        "PatientReportsChestPain",
        "PatientAdmittedToProcedureRoom",
        "PatientReportsNoPain",
        "PatientReportsNausea",
        "PatientReportsDiscomfort",
        "PatientDisoriented",
    ]
}

name_for_cid[3403] = "PercutaneousEntry"
cid_concepts[3403] = {
    "SCT": [
        "ViaFemoralVein",
        "ViaFemoralArtery",
        "ViaArtery",
        "ViaBrachialArtery",
        "ViaArmVein",
        "ViaRadialArtery",
        "ViaVein",
    ]
}

name_for_cid[3404] = "StaffAction"
cid_concepts[3404] = {
    "DCM": [
        "OfficeCalled",
        "PageSentTo",
        "ConsultationWith",
        "PersonnelArrived",
        "PersonnelDeparted",
    ]
}

name_for_cid[3405] = "ProcedureActionValue"
cid_concepts[3405] = {
    "DCM": [
        "ArterialConduitAngiography",
        "VascularIntervention",
        "MyocardialBiopsy",
        "AorticIntervention",
        "SeptalDefectIntervention",
        "ValvularIntervention",
    ],
    "SCT": [
        "CardiacCatheterizationBaselinePhase",
        "TransthoracicEchocardiography",
        "StentPlacement",
        "CardiacVentriculography",
        "TransseptalCatheterization",
        "DrugInfusionChallenge",
        "AtherectomyByRotaryCutter",
        "PercutaneousRemovalOfIntravascularForeignBody",
        "CardiacCatheterizationInterventionPhase",
        "CardiacCatheterizationTherapyPhase",
        "LeftVentriculography",
        "CatheterizationOfBothLeftAndRightHeartWithoutGraft",
        "IntravascularUltrasound",
        "CardiacCatheterizationPostContrastPhase",
        "Aortography",
        "CatheterizationOfLeftHeart",
        "CardiacCatheterizationPostInterventionPhase",
        "PercutaneousInsertionOfIntravascularFilter",
        "CardiacCatheterizationImageAcquisitionPhase",
        "CatheterizationOfBothLeftAndRightHeartWithGraft",
        "AtherectomyByLaser",
        "RightVentriculography",
        "TranscatheterTherapyForEmbolization",
        "RestingState",
        "CardiacCatheterizationPreInterventionPhase",
        "Atherectomy",
        "SelectiveEmbolizationOfArtery",
        "IntracardiacEchocardiography",
        "CoronaryArteriography",
        "CardiacCatheterizationTestChallengePhase",
        "PercutaneousTransluminalBalloonAngioplasty",
        "EpicardialEchocardiography",
        "BypassGraftAngiography",
        "CardiacCatheterizationBailoutPhase",
        "Angiography",
        "TransesophagealEchocardiography",
        "TranscatheterDeploymentOfDetachableBalloon",
        "CatheterizationOfRightHeart",
        "ExerciseChallenge",
    ],
}

name_for_cid[3406] = "NoncoronaryTranscatheterIntervention"
cid_concepts[3406] = {
    "DCM": [
        "VascularIntervention",
        "MyocardialBiopsy",
        "AorticIntervention",
        "SeptalDefectIntervention",
        "ValvularIntervention",
    ]
}

name_for_cid[3407] = "ObjectReferencePurpose"
cid_concepts[3407] = {
    "DCM": [
        "PriorReportForCurrentPatient",
        "PreProcedureLog",
        "AnalysisOrMeasurementsForCurrentProcedure",
    ]
}

name_for_cid[3408] = "ConsumableAction"
cid_concepts[3408] = {
    "DCM": [
        "RemainingConsumableDisposed",
        "ConsumableUnusable",
        "ConsumableTakenFromInventory",
        "ConsumableReturnedToInventory",
    ]
}

name_for_cid[3409] = "Drug/ContrastAdministration"
cid_concepts[3409] = {
    "DCM": [
        "DrugStart",
        "DrugEnd",
        "InfusateStart",
        "InfusateEnd",
        "ContrastEnd",
        "ContrastAdministered",
        "DrugAdministered",
        "ContrastStart",
    ]
}

name_for_cid[3410] = "Drug/ContrastNumericParameter"
cid_concepts[3410] = {
    "DCM": [
        "UndilutedDoseAdministered",
        "Concentration",
        "VolumeAdministered",
        "MassAdministered",
        "VolumeUnadministeredOrDiscarded",
        "QuantityAdministered",
        "RateOfAdministration",
        "DurationOfAdministration",
    ]
}

name_for_cid[3411] = "IntracoronaryDevice"
cid_concepts[3411] = {
    "SCT": [
        "SalineThrombectomy",
        "RotationalAtherectomyDevice",
        "GuidingCatheter",
        "PercutaneousTransluminalAngioplastyBalloon",
        "GuideWire",
        "IntracoronaryPressureGuideWire",
        "IntravascularOpticalCoherenceTomographyDevice",
        "CuttingBalloonAngioplastyCBADevice",
        "Laser",
        "IntracoronaryDopplerGuideWire",
        "IntravascularUltrasoundIVUSDevice",
        "RadiofrequencyAblationDevice",
        "Stent",
        "BrachytherapyDevice",
        "TransluminalExtractionCatheterTEC",
        "DirectionalCoronaryAtherectomyDCADevice",
    ]
}

name_for_cid[3412] = "InterventionAction/Status"
cid_concepts[3412] = {
    "DCM": [
        "StentReExpanded",
        "ObjectRemoved",
        "AngioplastyBalloonDeflated",
        "DeviceDeployed",
        "GuidewireCrossingLesionSuccessful",
        "InterventionalDeviceWithdrawn",
        "AngioplastyBalloonInflated",
        "InterventionalDevicePlaced",
        "GuidewireCrossingLesionUnsuccessful",
        "InterventionPerformed",
        "RadiationApplied",
        "RadiationRemoved",
        "InterventionalDevicePlacementUnsuccessful",
    ]
}

name_for_cid[3413] = "AdverseOutcome"
cid_concepts[3413] = {
    "DCM": ["DeathDuringCatheterization"],
    "SCT": [
        "LossOfDistalPulse",
        "Pseudoaneurysm",
        "EmergencyPercutaneousCoronaryIntervention",
        "CardiogenicShock",
        "ConductionDisorderOfTheHeart",
        "ContrastMediaAdverseReaction",
        "ArterialDissection",
        "CardiacTamponade",
        "CardiacArrest",
        "AVFistula",
        "CongestiveHeartFailure",
        "CerebrovascularAccidentOrStroke",
        "OcclusionOfArtery",
        "EmergencyCoronaryArteryBypass",
        "RenalFailure",
        "Bleeding",
    ],
}

name_for_cid[3414] = "ProcedureUrgency"
cid_concepts[3414] = {
    "SCT": [
        "EmergentProcedure",
        "UrgentProcedure",
        "ElectiveProcedure",
        "SalvageProcedure",
    ]
}

name_for_cid[3415] = "CardiacRhythm"
cid_concepts[3415] = {
    "MDC": [
        "AVDissociation",
        "VentricularCouplet",
        "FirstDegreeAVBlock",
        "NonRespiratorySinusArrhythmia",
        "VentricularParasystole",
        "NonsustainedVentricularTachycardia",
        "AVJunctionalNodalEscapeRhythm",
        "IrregularRhythmLowHeartRate",
        "WolfParkinsonWhiteSyndrome",
        "BrachycardiaAny",
        "LownGanongLevineSyndrome",
        "CompleteAVDissociation",
        "AutomaticAtrialTachycardia",
        "SecondDegreeAVBlockTypeIIMobitzTypeII",
        "AtrialEctopicRhythm",
        "TorsadesDePointesVentricularTachycardia",
        "SinusArrhythmia",
        "WanderingSinusPacemakerWithinTheSinusNode",
        "AcceleratedIdioventricularRhythm",
        "SustainedMonomorphicVentricularTachycardia",
        "WolfParkinsonTypeA",
        "AcceleratedAVJunctionalNodalRhythm",
        "VentricularBigeminy",
        "CalibrationSignal",
        "SinusBradycardia",
        "AVReciprocatingTachycardia",
        "SupraventricularAtrialOrJunctionalEctopicRhythm",
        "VentricularRhythm",
        "VentricularFlutter",
        "SupraventricularParoxsysmalTachycardia",
        "VentricularDemandModePacing",
        "SecondDegreeSABlockTypeII",
        "AVDissociationWithInterference",
        "ParoxsysmalAtrialTachycardia",
        "VentricularRun",
        "SecondDegreeAVBlock",
        "LowHeartRateVariability",
        "PreExcitation",
        "RespiratorySinusArrhythmia",
        "IdioventricularVentricularEscapeRhythm",
        "VentricularFibrillation",
        "AsystoleRhythm",
        "AVJunctionalNodalRhythm",
        "SinusRhythm",
        "FirstDegreeSABlock",
        "TWaveAlternans",
        "AtrialFlutterRhythm",
        "ThirdDegreeAVBlockCompleteAVBlock",
        "AtrialBigeminy",
        "AtrialDemandModePacing",
        "VentricularTrigeminy",
        "ReentrantAVNodalTachycardia",
        "JunctionalTachycardia",
        "SinusTachycardiaRhythm",
        "SupraventricularTachycardia",
        "AntiTachycardiaPacing",
        "IsorhythmicAVDissociation",
        "MultifocalAtrialTachycardia",
        "VentricularTachycardia",
        "SecondDegreeAVBlockTypeIWenckebachOrMobitzTypeI",
        "WanderingPacemakerBetweenTheSinusNodeAndTheAVNode",
        "PolymorphicVentricularTachycardia",
        "WolfParkinsonTypeB",
        "SlowVentricularTachycardiaIdioventricularTachycardia",
        "ThirdDegreeSABlockCompleteSABlock",
        "NormalSinusRhythm",
        "SecondDegreeSABlockTypeIWenckebach",
        "AtrialFibrillationRhythm",
        "AtrialTachycardia",
    ]
}

name_for_cid[3416] = "RespirationRhythm"
cid_concepts[3416] = {
    "SCT": [
        "GaspingRespiration",
        "RespirationIntermittent",
        "AbnormalRespiratoryRhythm",
        "NormalRespiratoryRhythm",
        "IrregularBreathing",
    ]
}

name_for_cid[3418] = "LesionRisk"
cid_concepts[3418] = {}

name_for_cid[3419] = "FindingTitle"
cid_concepts[3419] = {"DCM": ["Recommendation", "Impression", "Finding"]}

name_for_cid[3421] = "ProcedureAction"
cid_concepts[3421] = {
    "DCM": [
        "SuspendProcedureAction",
        "ResumeProcedureAction",
        "StartProcedureAction",
        "EndProcedureAction",
    ]
}

name_for_cid[3422] = "DeviceUseAction"
cid_concepts[3422] = {
    "DCM": ["DeviceCrossedLesion"],
    "SCT": [
        "DeviceUsed",
        "DeviceAtSiteOfInterest",
        "DeviceAppliedToPatient",
        "DeviceWithdrawnRemoved",
        "DeviceInsertedIntoSheath",
        "DeviceCrossedSeptum",
    ],
}

name_for_cid[3423] = "NumericDeviceCharacteristic"
cid_concepts[3423] = {
    "DCM": ["TransmitFrequency", "CatheterCurve", "InterMarkerDistance"],
    "SCT": ["Length", "Diameter", "Volume"],
}

name_for_cid[3425] = "InterventionParameter"
cid_concepts[3425] = {
    "SCT": [
        "AblationPower",
        "AblationFrequency",
        "AngioplastyInflationPressure",
        "AngioplastyInflationDuration",
        "RotationalAtherectomySpeed",
        "DeliveredRadiationDose",
    ]
}

name_for_cid[3426] = "ConsumablesParameter"
cid_concepts[3426] = {
    "DCM": ["LotIdentifier", "UnitSerialIdentifier", "DescriptionOfMaterial"]
}

name_for_cid[3427] = "EquipmentEvent"
cid_concepts[3427] = {
    "DCM": [
        "EquipmentFailure",
        "EquipmentRemoved",
        "EquipmentBroughtToProcedureRoom",
        "EquipmentReady",
    ]
}

name_for_cid[3428] = "CardiovascularImagingProcedure"
cid_concepts[3428] = {
    "DCM": ["ArterialConduitAngiography"],
    "SCT": [
        "TransthoracicEchocardiography",
        "CardiacVentriculography",
        "RightVentriculography",
        "LeftVentriculography",
        "IntravascularUltrasound",
        "IntracardiacEchocardiography",
        "CoronaryArteriography",
        "Aortography",
        "EpicardialEchocardiography",
        "BypassGraftAngiography",
        "Angiography",
        "TransesophagealEchocardiography",
    ],
}

name_for_cid[3429] = "CatheterizationDevice"
cid_concepts[3429] = {
    "DCM": ["Bioptome"],
    "SCT": [
        "SwannGanzCatheter",
        "FiberopticCatheter",
        "IntracoronaryPressureGuideWire",
        "IntravascularOpticalCoherenceTomographyDevice",
        "CuttingBalloonAngioplastyCBADevice",
        "Laser",
        "ThermistorCatheter",
        "BrachytherapyDevice",
        "FluidFilledCatheter",
        "DirectionalCoronaryAtherectomyDCADevice",
        "SalineThrombectomy",
        "RotationalAtherectomyDevice",
        "GuidingCatheter",
        "PercutaneousTransluminalAngioplastyBalloon",
        "GuideWire",
        "TipManometer",
        "IntracoronaryDopplerGuideWire",
        "Sheath",
        "IntravascularUltrasoundIVUSDevice",
        "TransseptalCatheter",
        "RadiofrequencyAblationDevice",
        "Stent",
        "TransluminalExtractionCatheterTEC",
        "IntraAorticBalloonPumpIABP",
    ],
}

name_for_cid[3430] = "DateTimeQualifier"
cid_concepts[3430] = {"DCM": ["DatetimeUnsynchronized", "DatetimeEstimated"]}

name_for_cid[3440] = "PeripheralPulseLocation"
cid_concepts[3440] = {
    "SCT": [
        "FemoralArtery",
        "BrachialArtery",
        "UlnarArtery",
        "CarotidArtery",
        "DorsalisPedisArtery",
        "PoplitealArtery",
        "PosteriorTibialArtery",
        "RadialArtery",
    ]
}

name_for_cid[3441] = "PatientAssessment"
cid_concepts[3441] = {
    "LN": ["RespirationRhythm", "CardiacRhythm"],
    "SCT": [
        "RespirationAssessment",
        "PatientMentalStateAssessment",
        "SkinConditionAssessment",
    ],
}

name_for_cid[3442] = "PeripheralPulseMethod"
cid_concepts[3442] = {"SCT": ["Doppler", "Palpation"]}

name_for_cid[3446] = "SkinCondition"
cid_concepts[3446] = {
    "DCM": [
        "SkinConditionMottled",
        "SkinConditionPale",
        "SkinConditionDiaphoretic",
        "SkinConditionFlush",
        "SkinConditionDry",
        "SkinConditionClammy",
        "SkinConditionCool",
        "SkinConditionCold",
        "SkinConditionWarm",
    ]
}

name_for_cid[3448] = "AirwayAssessment"
cid_concepts[3448] = {
    "DCM": [
        "AirwaySeverelyObstructed",
        "AirwayUnobstructed",
        "AirwayPartiallyObstructed",
    ]
}

name_for_cid[3451] = "CalibrationObject"
cid_concepts[3451] = {"DCM": ["Sphere"], "SCT": ["Catheter", "MeasuringRuler"]}

name_for_cid[3452] = "CalibrationMethod"
cid_concepts[3452] = {
    "DCM": ["GeometricNonIsocenter", "CalibrationObjectUsed", "GeometricIsocenter"]
}

name_for_cid[3453] = "CardiacVolumeMethod"
cid_concepts[3453] = {
    "DCM": [
        "AreaLengthDodge",
        "AreaLengthKennedy",
        "Parallelepiped",
        "TSPyramid",
        "TwoChamber",
        "MultipleSlices",
        "Boak",
        "AreaLengthWynne",
    ]
}

name_for_cid[3455] = "IndexMethod"
cid_concepts[3455] = {"DCM": ["BSA1Point219"], "LN": ["BSA"]}

name_for_cid[3456] = "SubsegmentMethod"
cid_concepts[3456] = {"DCM": ["UserSelectedMethod", "EquidistantMethod"]}

name_for_cid[3458] = "ContourRealignment"
cid_concepts[3458] = {"DCM": ["LongAxisBased", "NoRealignment", "CenterOfGravity"]}

name_for_cid[3460] = "CircumferentialExtent"
cid_concepts[3460] = {"DCM": ["RCARegionInRAOProjection", "LADRegionInRAOProjection"]}

name_for_cid[3461] = "RegionalExtent"
cid_concepts[3461] = {
    "DCM": [
        "MultipleRCARegionInRAOProjection",
        "SingleRCARegionInRAOProjection",
        "MultipleLADRegionInRAOProjection",
        "SingleLADRegionInRAOProjection",
        "CFXRegionInLAOProjection",
        "LADRegionInLAOProjection",
        "RCARegionInLAOProjection",
    ]
}

name_for_cid[3462] = "ChamberIdentification"
cid_concepts[3462] = {
    "SCT": ["RightAtrium", "RightVentricle", "LeftVentricle", "LeftAtrium"]
}

name_for_cid[3463] = "VentricleIdentification"
cid_concepts[3463] = {"SCT": ["RightVentricle", "LeftVentricle"]}

name_for_cid[3465] = "QAReferenceMethod"
cid_concepts[3465] = {
    "DCM": ["CurveFittedReference", "InterpolatedLocalReference", "MeanLocalReference"]
}

name_for_cid[3466] = "PlaneIdentification"
cid_concepts[3466] = {
    "SCT": [
        "AnteroPosterior",
        "LeftAnteriorOblique",
        "LeftLateral",
        "CaudalRAO",
        "RightAnteriorOblique",
        "CranialRAO",
        "CaudalLAO",
        "CranialLAO",
    ]
}

name_for_cid[3467] = "EjectionFraction"
cid_concepts[3467] = {
    "DCM": ["LeftAtrialEjectionFractionByAngiography"],
    "LN": [
        "RightVentricularEjectionFractionByAngiography",
        "LeftVentricularEjectionFractionByAngiography",
    ],
}

name_for_cid[3468] = "EDVolume"
cid_concepts[3468] = {
    "DCM": ["LeftAtrialEDVolume"],
    "LN": ["RightVentricularEDVolume", "LeftVentricularEDVolume"],
}

name_for_cid[3469] = "ESVolume"
cid_concepts[3469] = {
    "DCM": ["LeftAtrialESVolume"],
    "LN": ["RightVentricularESVolume", "LeftVentricularESVolume"],
}

name_for_cid[3470] = "VesselLumenCrosssectionalAreaCalculationMethod"
cid_concepts[3470] = {"DCM": ["DensitometricMethod", "CircularMethod"]}

name_for_cid[3471] = "EstimatedVolume"
cid_concepts[3471] = {
    "DCM": [
        "VolumeEstimatedFromTwoNonCoplanar2DRegions",
        "VolumeEstimatedFromSingle2DRegion",
    ]
}

name_for_cid[3472] = "CardiacContractionPhase"
cid_concepts[3472] = {"SCT": ["Diastolic", "Systolic"]}

name_for_cid[3480] = "IVUSProcedurePhase"
cid_concepts[3480] = {
    "SCT": [
        "CardiacCatheterizationPostInterventionPhase",
        "CardiacCatheterizationPreInterventionPhase",
    ]
}

name_for_cid[3481] = "IVUSDistanceMeasurement"
cid_concepts[3481] = {
    "DCM": ["PlaquePlusMediaThickness", "LumenPerimeter", "EEMDiameter"],
    "SCT": ["StentDiameter", "VesselLumenDiameter"],
}

name_for_cid[3482] = "IVUSAreaMeasurement"
cid_concepts[3482] = {
    "DCM": [
        "InStentNeointimalCrossSectionalArea",
        "EEMCrossSectionalArea",
        "PlaquePlusMediaCrossSectionalArea",
    ],
    "SCT": ["VesselLumenCrossSectionalArea", "StentCrossSectionalArea"],
}

name_for_cid[3483] = "IVUSLongitudinalMeasurement"
cid_concepts[3483] = {
    "DCM": ["StentGap", "CalciumLength"],
    "SCT": ["StentLength", "StenoticLesionLength"],
}

name_for_cid[3484] = "IVUSIndex/Ratio"
cid_concepts[3484] = {
    "DCM": [
        "LumenShapeIndex",
        "StentSymmetryIndex",
        "StentExpansionIndex",
        "PlaquePlusMediaEccentricityIndex",
        "RemodelingIndex",
        "LumenEccentricityIndex",
        "StentDiameterRatio",
        "EEMDiameterRatio",
        "LumenDiameterRatio",
    ]
}

name_for_cid[3485] = "IVUSVolumeMeasurement"
cid_concepts[3485] = {
    "DCM": [
        "NativePlaqueVolume",
        "TotalPlaqueVolume",
        "InStentNeointimalVolume",
        "EEMVolume",
        "LumenVolume",
    ],
    "SCT": ["StentVolume"],
}

name_for_cid[3486] = "VascularMeasurementSite"
cid_concepts[3486] = {
    "DCM": [
        "SiteOfLumenMaximum",
        "SiteOfLumenMinimum",
        "ProximalReference",
        "DistalReference",
    ]
}

name_for_cid[3487] = "IntravascularVolumetricRegion"
cid_concepts[3487] = {
    "DCM": [
        "DistalStentMargin",
        "StentedRegion",
        "ProximalStentMargin",
        "EntirePullback",
    ],
    "SCT": ["Lesion", "MorphologicallyAbnormalStructure", "CulpritLesion"],
}

name_for_cid[3488] = "Min/Max/Mean"
cid_concepts[3488] = {"SCT": ["Minimum", "Maximum", "Mean"]}

name_for_cid[3489] = "CalciumDistribution"
cid_concepts[3489] = {"SCT": ["Deep", "Superficial"]}

name_for_cid[3491] = "IVUSLesionMorphology"
cid_concepts[3491] = {
    "DCM": [
        "InStentNeointima",
        "SoftPlaque",
        "VulnerablePlaque",
        "ErodedPlaque",
        "NecroticLipidicPlaque",
        "PlaqueRupture",
        "FalseLumen",
        "FibroLipidicPlaque",
    ],
    "SCT": [
        "VascularCalcification",
        "PseudoAneurysm",
        "Eccentric",
        "PlaqueUlceration",
        "Concentric",
        "ArterialTrueAneurysm",
        "Thrombus",
        "FibrousPlaque",
    ],
}

name_for_cid[3492] = "VascularDissectionClassification"
cid_concepts[3492] = {
    "DCM": [
        "MedialDissection",
        "IntraStentDissection",
        "AdventitialDissection",
        "IntimalDissection",
    ],
    "SCT": ["IntramuralHematoma"],
}

name_for_cid[3493] = "IVUSRelativeStenosisSeverity"
cid_concepts[3493] = {"DCM": ["T2Secondary", "T3Secondary", "T1Worst", "T4Secondary"]}

name_for_cid[3494] = "IVUSNonMorphologicalFinding"
cid_concepts[3494] = {
    "DCM": ["TrueLumen"],
    "SCT": [
        "ArterialBloodStasis",
        "AcquiredIncompleteStentApposition",
        "IncompleteStentApposition",
    ],
}

name_for_cid[3495] = "IVUSPlaqueComposition"
cid_concepts[3495] = {
    "DCM": ["NecroticLipidicPlaque", "FibroLipidicPlaque"],
    "SCT": ["VascularCalcification", "Thrombus", "FibrousPlaque"],
}

name_for_cid[3496] = "IVUSFiducialPoint"
cid_concepts[3496] = {
    "SCT": [
        "VascularCalcification",
        "Vein",
        "VesselOrigin",
        "Stent",
        "FibrousPlaque",
        "CollateralBranchOfVessel",
    ]
}

name_for_cid[3497] = "IVUSArterialMorphology"
cid_concepts[3497] = {
    "DCM": ["NecroticLipidicPlaque", "FibroLipidicPlaque"],
    "SCT": [
        "LumenOfArtery",
        "VascularCalcification",
        "Thrombus",
        "ExternalElasticMembrane",
        "FibrousPlaque",
    ],
}

name_for_cid[3500] = "PressureUnit"
cid_concepts[3500] = {"UCUM": ["MillimetersHg", "Kilopascal"]}

name_for_cid[3502] = "HemodynamicResistanceUnit"
cid_concepts[3502] = {"UCUM": ["WoodU", "PRU", "DynScm5"]}

name_for_cid[3503] = "IndexedHemodynamicResistanceUnit"
cid_concepts[3503] = {"UCUM": ["WoodUM2", "DynScm5M2", "PRUPerSquareMeter"]}

name_for_cid[3510] = "CatheterSizeUnit"
cid_concepts[3510] = {"UCUM": ["French", "Millimeter"]}

name_for_cid[3515] = "SpecimenCollection"
cid_concepts[3515] = {
    "SCT": [
        "BloodSamplingFromExtracorporealBloodCircuit",
        "SpecimenCollection",
        "CollectionOfBloodSpecimenForLaboratory",
    ]
}

name_for_cid[3520] = "BloodSourceType"
cid_concepts[3520] = {
    "SCT": [
        "SystemicArteryBlood",
        "PulmonaryVeinBlood",
        "MixedVenousBlood",
        "PulmonaryArteryBlood",
    ]
}

name_for_cid[3524] = "BloodGasPressure"
cid_concepts[3524] = {
    "LN": [
        "BloodCarbonDioxidePartialPressure",
        "BloodOxygenPartialPressure",
        "VenousOxygenPartialPressure",
        "OxygenPartialPressureAt50PercentSaturationP50",
        "ArterialOxygenPartialPressureAt50PercentSaturation",
        "VenousBloodCarbonDioxidePartialPressure",
        "VenousOxygenPartialPressureAt50PercentSaturation",
        "ArterialOxygenPartialPressure",
        "ArterialBloodCarbonDioxidePartialPressure",
    ]
}

name_for_cid[3525] = "BloodGasContent"
cid_concepts[3525] = {
    "DCM": ["BloodOxygenContent"],
    "LN": [
        "ArterialOxygenContent",
        "PulmonaryArteryMainOxygenContent",
        "PulmonaryWedgeOxygenContent",
        "ArterialBloodCarbonDioxideContent",
        "BloodCarbonDioxideContent",
        "VenousBloodCarbonDioxideContent",
        "AorticRootOxygenContent",
        "VenousOxygenContent",
    ],
}

name_for_cid[3526] = "BloodGasSaturation"
cid_concepts[3526] = {
    "DCM": ["BloodCarbonDioxideSaturation"],
    "LN": [
        "CapillaryBloodOxygenSaturation",
        "ArterialOxygenSaturation",
        "VenousOxygenSaturation",
        "CapillaryBloodOxygenSaturationByOximetry",
        "BloodOxygenSaturation",
    ],
}

name_for_cid[3527] = "BloodBaseExcess"
cid_concepts[3527] = {
    "LN": ["BloodBaseExcess", "VenousBloodBaseExcess", "ArterialBloodBaseExcess"]
}

name_for_cid[3528] = "BloodpH"
cid_concepts[3528] = {"LN": ["BloodPh", "VenousBloodPh", "ArterialBloodPh"]}

name_for_cid[3529] = "Arterial/VenousContent"
cid_concepts[3529] = {
    "DCM": ["PulmonaryArterialContentFcpa", "PulmonaryVenousContentFcpv"],
    "LN": ["ArterialContentFca", "VenousContentFcv"],
}

name_for_cid[3530] = "OxygenAdministrationAction"
cid_concepts[3530] = {"DCM": ["BeginOxygenAdministration", "EndOxygenAdministration"]}

name_for_cid[3531] = "OxygenAdministration"
cid_concepts[3531] = {
    "DCM": ["OxygenAdministrationByVentilator"],
    "SCT": ["OxygenAdministrationByMask", "OxygenAdministrationByNasalCannula"],
}

name_for_cid[3550] = "CirculatorySupportAction"
cid_concepts[3550] = {"DCM": ["EndCirculatorySupport", "BeginCirculatorySupport"]}

name_for_cid[3551] = "VentilationAction"
cid_concepts[3551] = {"DCM": ["EndVentilation", "BeginVentilation"]}

name_for_cid[3552] = "PacingAction"
cid_concepts[3552] = {"DCM": ["EndPacing", "BeginPacing"]}

name_for_cid[3553] = "CirculatorySupport"
cid_concepts[3553] = {
    "SCT": [
        "ExternalCounterPulsation",
        "LeftVentricularAssistDevice",
        "CardiopulmonaryBypass",
        "IntraAorticBalloonPump",
        "ExtraCorporealCirculation",
    ]
}

name_for_cid[3554] = "Ventilation"
cid_concepts[3554] = {
    "SCT": ["PressureSupportVentilator", "AmbuBag", "VolumeSupportVentilator"]
}

name_for_cid[3555] = "Pacing"
cid_concepts[3555] = {
    "SCT": [
        "AtrialPacing",
        "TemporaryTranscutaneousPacing",
        "PacingWithMagnet",
        "Pacing",
        "AVSequentialPacing",
        "VentricularPacing",
    ]
}

name_for_cid[3560] = "BloodPressureMethod"
cid_concepts[3560] = {"SCT": ["BloodPressureCuffMethod"]}

name_for_cid[3600] = "RelativeTime"
cid_concepts[3600] = {"SCT": ["During", "After", "Before"]}

name_for_cid[3602] = "HemodynamicPatientState"
cid_concepts[3602] = {
    "SCT": [
        "BaselineState",
        "ExerciseState",
        "SupineBodyPosition",
        "RestingState",
        "PostExerciseState",
    ]
}

name_for_cid[3604] = "ArterialLesionLocation"
cid_concepts[3604] = {
    "BARI": [
        "_1stDiagonalCoronaryArteryLaterals",
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "SCT": [
        "InternalCarotidArtery",
        "ProximalRightCoronaryArtery",
        "RightPosteriorAVCoronaryArtery",
        "BasilarArtery",
        "LateralPlantarArtery",
        "MidCircumflexCoronaryArtery",
        "RightPulmonaryArtery",
        "IntermediateArteryRamus",
        "GeniculateArtery",
        "CarotidArtery",
        "UmbilicalArtery",
        "NeoAorta",
        "AorticFistula",
        "RadialArtery",
        "RightFemoralArtery",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "LeftFemoralArtery",
        "DistalRightCoronaryArtery",
        "LeftAnteriorDescendingCoronaryArtery",
        "TruncusArteriosusCommunis",
        "Artery",
        "CommonCarotidArtery",
        "_3rdMarginalCoronaryArtery",
        "Aorta",
        "PatentDuctusArteriosus",
        "SuperficialTemporalArtery",
        "MidRightCoronaryArtery",
        "FistulaCoronaryToLeftVentricle",
        "CoronaryArtery",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "StructureOfDescendingThoracicAorta",
        "PulmonaryArteriovenousFistula",
        "SubclavianArtery",
        "TibialArtery",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OphthalmicArtery",
        "MesentericArtery",
        "MedialPlantarArtery",
        "PoplitealArtery",
        "LeftMainCoronaryArteryOstium",
        "AnteriorSpinalArtery",
        "HepaticArtery",
        "PeronealArtery",
        "_2ndRightPosterolateralCoronaryArtery",
        "BrachiocephalicTrunk",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "RenalArtery",
        "_1stMarginalCoronaryArtery",
        "SystemicCollateralArteryToLung",
        "FacialArtery",
        "LumbarArtery",
        "FemoralArtery",
        "PulmonaryArtery",
        "BrachialArtery",
        "_2ndDiagonalCoronaryArtery",
        "DistalCircumflexCoronaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "CoronaryArteryGraft",
        "FistulaCoronaryToRightAtrium",
        "FistulaCoronaryToRightVentricle",
        "CommonFemoralArtery",
        "_1stLeftPosterolateralCoronaryArtery",
        "PulmonaryArteryConduit",
        "CircumflexCoronaryArtery",
        "AnteriorCommunicatingArtery",
        "FistulaCoronaryToLeftAtrium",
        "ThoracicAorta",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "PulmonaryVeinWedge",
        "RightCoronaryArteryOstium",
        "LeftCoronaryArtery",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "MarginalCoronaryArtery",
        "InternalMammaryArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "VertebralArtery",
        "_3rdDiagonalCoronaryArtery",
        "AVGrooveContinuationOfCircumflexArtery",
        "_3rdRightPosterolateralCoronaryArtery",
        "PosterolateralBranchOfRightCoronaryArtery",
        "NeonatalPulmonaryArteryPrimitivePA",
        "SuperficialFemoralArtery",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "PosteriorCommunicatingArtery",
        "AorticArch",
        "AbdominalAorta",
        "LingualArtery",
        "LeftMainCoronaryArtery",
        "_2ndLeftPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "Baffle",
        "LacrimalArtery",
        "OccipitalArtery",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "AscendingAorta",
        "_1stDiagonalCoronaryArtery",
        "ProfundaFemorisArtery",
        "CerebralArtery",
        "_1stRightPosterolateralCoronaryArtery",
    ],
}

name_for_cid[3606] = "ArterialSourceLocation"
cid_concepts[3606] = {
    "SCT": [
        "InternalCarotidArtery",
        "BasilarArtery",
        "CommonFemoralArtery",
        "LateralPlantarArtery",
        "RightPulmonaryArtery",
        "PulmonaryArteryConduit",
        "AnteriorCommunicatingArtery",
        "FistulaCoronaryToLeftAtrium",
        "GeniculateArtery",
        "ThoracicAorta",
        "CarotidArtery",
        "UmbilicalArtery",
        "NeoAorta",
        "PulmonaryVeinWedge",
        "AorticFistula",
        "RadialArtery",
        "RightFemoralArtery",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "LeftFemoralArtery",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "InternalMammaryArtery",
        "TruncusArteriosusCommunis",
        "Artery",
        "VertebralArtery",
        "CommonCarotidArtery",
        "Aorta",
        "PatentDuctusArteriosus",
        "SuperficialTemporalArtery",
        "FistulaCoronaryToLeftVentricle",
        "CoronaryArtery",
        "StructureOfDescendingThoracicAorta",
        "PulmonaryArteriovenousFistula",
        "SubclavianArtery",
        "NeonatalPulmonaryArteryPrimitivePA",
        "TibialArtery",
        "SuperficialFemoralArtery",
        "OphthalmicArtery",
        "MesentericArtery",
        "MedialPlantarArtery",
        "PosteriorCommunicatingArtery",
        "PoplitealArtery",
        "AorticArch",
        "AbdominalAorta",
        "LingualArtery",
        "AnteriorSpinalArtery",
        "HepaticArtery",
        "PeronealArtery",
        "Baffle",
        "BrachiocephalicTrunk",
        "RenalArtery",
        "SystemicCollateralArteryToLung",
        "FacialArtery",
        "LacrimalArtery",
        "LumbarArtery",
        "OccipitalArtery",
        "AscendingAorta",
        "FemoralArtery",
        "PulmonaryArtery",
        "BrachialArtery",
        "ProfundaFemorisArtery",
        "CerebralArtery",
        "FistulaCoronaryToRightAtrium",
        "FistulaCoronaryToRightVentricle",
    ]
}

name_for_cid[3607] = "VenousSourceLocation"
cid_concepts[3607] = {
    "SCT": [
        "InnominateVein",
        "SubclavianVein",
        "PosteriorMedialTributary",
        "PortalVein",
        "BrachialVein",
        "InferiorVenaCava",
        "RenalVein",
        "BoydPerforatingVein",
        "SuperiorVenaCava",
        "CephalicVein",
        "CentralVenousSystem",
        "GastricVein",
        "SplenicVein",
        "SaphenousVein",
        "HepaticVein",
        "AntecubitalVein",
        "BasilicVein",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "HunterianPerforatingVein",
        "InternalJugularVein",
        "AnomalousPulmonaryVein",
        "AzygosVein",
        "MesentericVein",
        "Vein",
        "UmbilicalVein",
        "AxillaryVein",
        "PulmonaryVein",
        "FemoralVein",
        "GreatSaphenousVein",
        "DoddPerforatingVein",
    ]
}

name_for_cid[3608] = "AtrialSourceLocation"
cid_concepts[3608] = {
    "SCT": [
        "RightAtrium",
        "PulmonaryCapillaryWedge",
        "Baffle",
        "LeftAtrium",
        "CommonAtrium",
        "JuxtaposedAppendage",
        "CoronarySinus",
        "SystemicVenousAtrium",
        "PulmonaryVenousAtrium",
        "PulmonaryArteryWedge",
        "PulmonaryChamberInCorTriatriatum",
    ]
}

name_for_cid[3609] = "VentricularSourceLocation"
cid_concepts[3609] = {
    "SCT": [
        "RightVentricleOutflowTract",
        "LeftVentricleInflow",
        "RightVentricle",
        "LeftVentricle",
        "RightVentricleInflow",
        "CommonVentricle",
        "LeftVentricleApex",
        "RightVentricleApex",
        "LeftVentricleOutflowTract",
    ]
}

name_for_cid[3610] = "GradientSourceLocation"
cid_concepts[3610] = {
    "SCT": [
        "TricuspidValve",
        "RightVentricleOutflowTract",
        "PulmonaryValve",
        "PulmonaryArtery",
        "CoarctationOfAorta",
        "AorticValve",
        "AtrialSeptalDefect",
        "MitralValve",
        "VentricularSeptalDefect",
        "LeftVentricleOutflowTract",
    ]
}

name_for_cid[3611] = "PressureMeasurement"
cid_concepts[3611] = {
    "DCM": [
        "XDescentPressure",
        "YDescentPressure",
        "VentricularEndDiastolicPressure",
        "AWavePeakPressure",
        "PressureAtDpDtMax",
        "VWavePeakPressure",
        "ZPointPressure",
        "GradientPressureAverage",
        "GradientPressurePeak",
        "CWavePressure",
    ],
    "LN": ["IntravascularDiastolicBloodPressure", "IntravascularSystolicBloodPressure"],
    "SCT": [
        "AverageDiastolicBloodPressure",
        "MinimumDiastolicBloodPressure",
        "AverageSystolicBloodPressure",
        "MaximumSystolicBloodPressure",
        "MeanBloodPressure",
    ],
}

name_for_cid[3612] = "BloodVelocityMeasurement"
cid_concepts[3612] = {
    "DCM": [
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ]
}

name_for_cid[3613] = "HemodynamicTimeMeasurement"
cid_concepts[3613] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RightVentricularFillingTime",
        "LeftVentricularFillingTime",
        "RightVentricularEjectionTime",
        "LeftVentricularEjectionTime",
        "RRInterval",
    ]
}

name_for_cid[3614] = "NonmitralValveArea"
cid_concepts[3614] = {
    "DCM": ["DerivedNonValveArea"],
    "SCT": ["TricuspidValveArea", "PulmonicValveArea", "AorticValveArea"],
}

name_for_cid[3615] = "ValveArea"
cid_concepts[3615] = {
    "DCM": ["DerivedNonValveArea"],
    "SCT": [
        "MitralValveArea",
        "TricuspidValveArea",
        "PulmonicValveArea",
        "AorticValveArea",
    ],
}

name_for_cid[3616] = "HemodynamicPeriodMeasurement"
cid_concepts[3616] = {
    "SCT": [
        "PulmonarySystolicEjectionPeriodSepp",
        "AorticSystolicEjectionPeriodSepa",
        "DerivedPeriodNonValve",
        "TricuspidDiastolicFillingPeriodDfpt",
        "MitralDiastolicFillingPeriodDfpm",
    ]
}

name_for_cid[3617] = "ValveFlow"
cid_concepts[3617] = {
    "SCT": [
        "DerivedFlowNonValve",
        "AorticValveFlow",
        "MitralValveFlow",
        "TricuspidValveFlow",
        "PulmonaryValveFlow",
    ]
}

name_for_cid[3618] = "HemodynamicFlow"
cid_concepts[3618] = {"DCM": ["SystemicFlow", "PulmonaryFlow"]}

name_for_cid[3619] = "HemodynamicResistanceMeasurement"
cid_concepts[3619] = {
    "DCM": ["TotalVascularResistance", "TotalPulmonaryResistance"],
    "SCT": ["PulmonaryVascularResistance", "SystemicVascularResistance"],
}

name_for_cid[3620] = "HemodynamicRatio"
cid_concepts[3620] = {
    "DCM": [
        "DiastolicSystolicVelocityRatio",
        "HyperemicRatio",
        "CoronaryFlowReserve",
        "HemodynamicResistanceIndex",
    ],
    "LN": ["TibialBrachialIndex"],
    "SCT": [
        "FractionalFlowReserve",
        "FractionalFlowReserveUsingIntracoronaryBolus",
        "PulmonarySystemicFlowRatio",
        "PulsatilityIndex",
        "FractionalFlowReserveUsingIntravenousInfusion",
    ],
}

name_for_cid[3621] = "FractionalFlowReserve"
cid_concepts[3621] = {
    "SCT": [
        "FractionalFlowReserve",
        "FractionalFlowReserveUsingIntracoronaryBolus",
        "FractionalFlowReserveUsingIntravenousInfusion",
    ]
}

name_for_cid[3627] = "MeasurementType"
cid_concepts[3627] = {
    "SCT": [
        "Estimated",
        "PeakToPeak",
        "Measured",
        "Mode",
        "VisualEstimation",
        "BestValue",
        "Calculated",
        "Mean",
        "PointSourceMeasurement",
        "Median",
    ]
}

name_for_cid[3628] = "CardiacOutputMethod"
cid_concepts[3628] = {"SCT": ["ThermalBath", "ThermalInline", "DyeDilution"]}

name_for_cid[3629] = "ProcedureIntent"
cid_concepts[3629] = {
    "DCM": ["QualityControlIntent"],
    "SCT": [
        "DiagnosticIntent",
        "StagingIntent",
        "TherapeuticIntent",
        "CurativeIntent",
        "ProphylacticIntent",
        "AdjunctIntent",
        "ForensicIntent",
        "NeoAdjuvantIntent",
        "SupportiveIntent",
        "GuidanceIntent",
        "PalliativeIntent",
        "AdjuvantIntent",
        "CombinedDiagnosticAndTherapeuticIntent",
        "ScreeningIntent",
        "PreventiveIntent",
    ],
}

name_for_cid[3630] = "CardiovascularAnatomicLocation"
cid_concepts[3630] = {
    "SCT": [
        "SubclavianVein",
        "InternalCarotidArtery",
        "PulmonaryValve",
        "BasilarArtery",
        "AorticValve",
        "LateralPlantarArtery",
        "RightVentricleApex",
        "InferiorVenaCava",
        "RightPulmonaryArtery",
        "LeftVentricleInflow",
        "GeniculateArtery",
        "CentralVenousSystem",
        "CarotidArtery",
        "CommonAtrium",
        "UmbilicalArtery",
        "NeoAorta",
        "AorticFistula",
        "LeftVentricleOutflowTract",
        "RadialArtery",
        "RightFemoralArtery",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "LeftFemoralArtery",
        "TruncusArteriosusCommunis",
        "Artery",
        "AntecubitalVein",
        "CommonCarotidArtery",
        "Aorta",
        "PatentDuctusArteriosus",
        "SuperficialTemporalArtery",
        "MesentericVein",
        "Vein",
        "FistulaCoronaryToLeftVentricle",
        "PulmonaryVein",
        "CoronaryArtery",
        "StructureOfDescendingThoracicAorta",
        "PulmonaryArteriovenousFistula",
        "SubclavianArtery",
        "TibialArtery",
        "RightVentricleInflow",
        "OphthalmicArtery",
        "MesentericArtery",
        "MedialPlantarArtery",
        "PoplitealArtery",
        "PulmonaryVenousAtrium",
        "BrachialVein",
        "PulmonaryChamberInCorTriatriatum",
        "PulmonaryCapillaryWedge",
        "SuperiorVenaCava",
        "LeftVentricle",
        "AnteriorSpinalArtery",
        "HepaticArtery",
        "SplenicVein",
        "PeronealArtery",
        "RightVentricleOutflowTract",
        "SaphenousVein",
        "BrachiocephalicTrunk",
        "RenalArtery",
        "SystemicCollateralArteryToLung",
        "FacialArtery",
        "LumbarArtery",
        "LeftVentricleApex",
        "VentricularSeptalDefect",
        "AzygosVein",
        "PulmonaryArtery",
        "FemoralArtery",
        "BrachialArtery",
        "FistulaCoronaryToRightAtrium",
        "MitralValve",
        "FistulaCoronaryToRightVentricle",
        "PosteriorMedialTributary",
        "CoarctationOfAorta",
        "CommonFemoralArtery",
        "PulmonaryArteryConduit",
        "TricuspidValve",
        "RightAtrium",
        "AnteriorCommunicatingArtery",
        "FistulaCoronaryToLeftAtrium",
        "RightVentricle",
        "ThoracicAorta",
        "CommonVentricle",
        "AtrialSeptalDefect",
        "PulmonaryVeinWedge",
        "CoronarySinus",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "HepaticVein",
        "InternalMammaryArtery",
        "VertebralArtery",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "InternalJugularVein",
        "UmbilicalVein",
        "FemoralVein",
        "InnominateVein",
        "NeonatalPulmonaryArteryPrimitivePA",
        "SuperficialFemoralArtery",
        "PortalVein",
        "PosteriorCommunicatingArtery",
        "JuxtaposedAppendage",
        "AorticArch",
        "RenalVein",
        "BoydPerforatingVein",
        "AbdominalAorta",
        "CephalicVein",
        "LingualArtery",
        "GastricVein",
        "Baffle",
        "LacrimalArtery",
        "BasilicVein",
        "OccipitalArtery",
        "SystemicVenousAtrium",
        "HunterianPerforatingVein",
        "AscendingAorta",
        "AnomalousPulmonaryVein",
        "ProfundaFemorisArtery",
        "LeftAtrium",
        "CerebralArtery",
        "AxillaryVein",
        "GreatSaphenousVein",
        "PulmonaryArteryWedge",
        "DoddPerforatingVein",
    ]
}

name_for_cid[3640] = "Hypertension"
cid_concepts[3640] = {"SCT": ["PulmonaryHypertension", "SystemicArterialHypertension"]}

name_for_cid[3641] = "HemodynamicAssessment"
cid_concepts[3641] = {
    "SCT": [
        "PulmonaryVascularResistance",
        "LeftVentricularSystolicPressure",
        "RightAtrialPressure",
        "LeftVentricularEndDiastolicPressure",
        "RightVentricularEndDiastolicPressure",
        "PulmonaryCapillaryWedgePressure",
        "RightVentricularSystolicPressure",
        "DiastolicPressure",
        "PulmonaryArteryPressure",
        "VascularResistanceFunction",
    ]
}

name_for_cid[3642] = "DegreeFinding"
cid_concepts[3642] = {
    "SCT": ["SeverelyElevated", "Decreased", "Elevated", "NormalRange"]
}

name_for_cid[3651] = "HemodynamicMeasurementPhase"
cid_concepts[3651] = {
    "SCT": [
        "CardiacCatheterizationBaselinePhase",
        "CardiacCatheterizationTestChallengePhase",
        "CardiacCatheterizationGradientAssessmentPhase",
        "DrugInfusionChallenge",
        "CardiacCatheterizationPostInterventionPhase",
        "CardiacCatheterizationPostContrastPhase",
        "RestingState",
        "ExerciseChallenge",
    ]
}

name_for_cid[3663] = "BodySurfaceAreaEquation"
cid_concepts[3663] = {
    "DCM": [
        "BSAEquals0Point038189WT0Point423HT0Point362",
        "BSAEquals13210Point3433WT",
        "BSAEquals0Point010265WT0Point423HT0Point651",
        "BSAEquals0Point0004688WT0Point8168To0Point0154LogWT",
        "BSAEquals0Point008883WT0Point444HT0Point663",
        "BSAEquals0Point024265WT0Point5378Htcm0Point3964",
        "BSAEqualsHTWT360Point5",
        "BSAEquals0Point007358WT0Point425HT0Point725",
        "BSAEquals0Point007184WT0Point425HT0Point725",
        "BSAEquals0Point0235WT0Point51456Htcm0Point42246",
        "BSAEquals0Point003207WT0Point7285To0Point0188LogWTHT0Point3",
        "BSAEquals0Point009568WT0Point473HT0Point655",
    ]
}

name_for_cid[3664] = "OxygenConsumptionEquation/Table"
cid_concepts[3664] = {
    "DCM": [
        "VO2EqualsVestpd10FIO2FE02",
        "Vo2maleEqualsBSA138Point111Point49LogeAge0Point378Hrf",
        "Vo2femaleEqualsBSA138Point117Point04LogeAge0Point378Hrf",
        "FleischTable",
        "BoothbyTable",
        "VO2Equals176BSA",
        "RobertsonReidTable",
        "VO2Equals152BSA",
        "VO2Equals175BSA",
    ]
}

name_for_cid[3666] = "P50Equation"
cid_concepts[3666] = {
    "DCM": [
        "IfAgeLesserThan30DayP50Equals24Point6",
        "IfAgeLesserThan18YearsP50Equals27Point2",
        "IfPremAgeLesserThan3DaysP50Equals19Point9",
        "IfAgeLesserThan1DayP50Equals21Point6",
        "IfAgeGreaterThan60YearsP50Equals29Point3",
        "IfAgeLesserThan40YearsP50Equals27Point4",
    ]
}

name_for_cid[3667] = "FraminghamScore"
cid_concepts[3667] = {
    "DCM": [
        "ComparativeLow10YearCHDRisk",
        "_10YearCHDRisk",
        "ComparativeAverage10YearCHDRisk",
    ]
}

name_for_cid[3668] = "FraminghamTable"
cid_concepts[3668] = {
    "DCM": [
        "TotalCholesterolScoreSheetForWomen",
        "LDLCholesterolScoreSheetForWomen",
        "TotalCholesterolScoreSheetForMen",
        "LDLCholesterolScoreSheetForMen",
    ]
}

name_for_cid[3670] = "ECGProcedureType"
cid_concepts[3670] = {"SCT": ["_15LeadECG", "_12LeadECG", "_18LeadECG"]}

name_for_cid[3671] = "ReasonforECGStudy"
cid_concepts[3671] = {
    "SCT": [
        "PreSurgeryTesting",
        "RoutineProcedure",
        "EmergencyDepartmentPatientVisit",
        "EmergencyProcedure",
        "ProcedureInCardiacCareUnit",
        "EvaluationOfMurmur",
        "OutpatientProcedure",
    ]
}

name_for_cid[3676] = "LeadMeasurementTechnique"
cid_concepts[3676] = {
    "SCT": ["Representative", "SingleBeats", "Averaged", "Routine", "Median"]
}

name_for_cid[3677] = "SummaryCodesECG"
cid_concepts[3677] = {
    "DCM": ["NonDiagnosticECG"],
    "SCT": ["BorderlineNormalECG", "ECGEquivocal", "ECGNormal", "AbnormalECG"],
}

name_for_cid[3678] = "QTCorrectionAlgorithm"
cid_concepts[3678] = {
    "DCM": [
        "FraminghamQtcAlgorithm",
        "HodgesQTCorrectionAlgorithm",
        "FridericiaQtcAlgorithm",
        "BazettQTCorrectionAlgorithm",
    ]
}

name_for_cid[3680] = "ECGLeadNoiseDescription"
cid_concepts[3680] = {"MDC": ["ModerateNoise", "SevereNoise", "NoNoise", "NoSignal"]}

name_for_cid[3687] = "ElectrophysiologyWaveformDuration"
cid_concepts[3687] = {
    "MDC": [
        "R3DurationPerLead",
        "POnsetToP1DurationPerLead",
        "S3DurationPerLead",
        "PQSegmentTimePeriodPerLead",
        "PPTimePeriodPerLead",
        "QRSDurationPerLead",
        "VentricularActivationTimePerLead",
        "S1DurationPerLead",
        "QTUTimePeriodPerLead",
        "PDurationPerLead",
        "POnsetToP2DurationPerLead",
        "R2DurationPerLead",
        "POffsetToQRSOnsetDurationPerLead",
        "S2DurationPerLead",
        "RRTimePeriodPerLead",
        "R1DurationPerLead",
        "POnsetToP3DurationPerLead",
        "QDurationPerLead",
        "PQTimePeriodPerLead",
        "QTDurationPerLead",
    ]
}

name_for_cid[3688] = "ElectrophysiologyWaveformVoltage"
cid_concepts[3688] = {
    "MDC": [
        "PMinimumAmplitudePerLead",
        "R1AmplitudePerLead",
        "S2AmplitudePerLead",
        "AmplitudeAt20msIntoSTSegmentPerLead",
        "STAmplitudePerLead",
        "TMinimumAmplitudePerLead",
        "SAmplitudePerLead",
        "S1AmplitudePerLead",
        "AmplitudeAt40msIntoSTSegmentPerLead",
        "QAmplitudePerLead",
        "RAmplitudePerLead",
        "R3AmplitudePerLead",
        "AmplitudeAt60msIntoSTSegmentPerLead",
        "TMaximumAmplitudePerLead",
        "JPointAmplitudePerLead",
        "R2AmplitudePerLead",
        "P3AmplitudePerLead",
        "AmplitudeAt80msIntoSTSegmentPerLead",
        "S3AmplitudePerLead",
        "PMaximumAmplitudePerLead",
    ]
}

name_for_cid[3689] = "ECGGlobalWaveformDuration"
cid_concepts[3689] = {
    "MDC": [
        "PDurationGlobal",
        "PPTimePeriodGlobal",
        "RRTimePeriodGlobal",
        "QTDurationGlobal",
        "PQSegmentTimePeriodGlobal",
        "QRSDurationGlobal",
        "PQTimePeriodGlobal",
        "PRTimePeriodGlobal",
        "QTUTimePeriodGlobal",
    ]
}

name_for_cid[3690] = "ECGControlNumericVariable"
cid_concepts[3690] = {
    "MDC": [
        "LowPassFilterCutoffFreq",
        "InterpolatorSNR",
        "NotchFilterFrequency",
        "SampleRate",
        "NotchFilterBandwidth",
        "Sensitivity",
        "ZeroOffset",
        "PadValue",
        "TimeSkew",
        "SampleSkew",
        "TimeOffset",
    ]
}

name_for_cid[3691] = "ECGControlTextVariable"
cid_concepts[3691] = {
    "MDC": [
        "Interpolator",
        "HighPassFilter",
        "InterpolatorDescription",
        "HighPassFilterDescription",
        "NotchFilter",
        "NotchFilterDescription",
        "BaselineDescription",
        "LowPassFilter",
    ]
}

name_for_cid[3700] = "CathDiagnosis"
cid_concepts[3700] = {
    "SCT": [
        "AorticDissection",
        "Arrhythmia",
        "VentricularFibrillation",
        "RecurrentAnginaPostStent",
        "PericardialTamponade",
        "AorticValveDisease",
        "UnstableAnginaProgressiveAngina",
        "MitralRegurgitation",
        "PulmonaryHypertension",
        "PulmonaryEdema",
        "MitralStenosis",
        "AtrialSeptalDefect",
        "PericardialDisease",
        "AcuteVentricularSeptalRupture",
        "PostInfarctionAngina",
        "MitralValveProlapse",
        "StableAngina",
        "HeartDiseaseCongenital",
        "AtypicalChestPain",
        "RecurrentAnginaPostRotationalAtherectomy",
        "PulmonicValveDisease",
        "AorticAneurysm",
        "AorticInsufficiency",
        "AtrialFibrillation",
        "CardiogenicShock",
        "TricuspidValveDisease",
        "RecurrentAnginaPostDCA",
        "Myocarditis",
        "VentricularTachycardia",
        "HypertensiveHeartDisease",
        "RecurrentAnginaPostPTCA",
        "CongestiveCardiomyopathy",
        "VentricularSeptalDefect",
        "CoronaryArteryDisease",
        "AcuteMyocardialInfarction",
        "SilentIschemia",
        "RestrictiveCardiomyopathy",
        "HypertrophicObstructiveCardiomyopathy",
        "SPMIPositiveStressForIschemia",
        "AtypicalAnginaVariantAngina",
        "CongestiveHeartFailure",
        "SubacuteBacterialEndocarditis",
        "AorticStenosis",
        "MitralValveDisease",
        "TricuspidRegurgitation",
        "AcuteMitralRegurgitation",
        "RecurrentAnginaPostCABG",
        "ConstrictivePericarditis",
    ]
}

name_for_cid[3701] = "CardiacValve/Tract"
cid_concepts[3701] = {
    "SCT": [
        "TricuspidValve",
        "PulmonaryValve",
        "AorticValve",
        "MitralValve",
        "LeftVentricleOutflowTract",
    ]
}

name_for_cid[3703] = "WallMotion"
cid_concepts[3703] = {
    "DCM": ["NotVisualized"],
    "SCT": [
        "ModerateHypokinesis",
        "HyperkineticRegion",
        "Hypokinesis",
        "SevereHypokinesis",
        "Unknown",
        "NormalWallMotion",
        "MildHypokinesis",
        "Akinesis",
        "NotEvaluated",
        "Dyskinesis",
    ],
}

name_for_cid[3704] = "MyocardiumWallMorphologyFinding"
cid_concepts[3704] = {
    "DCM": ["ScarredMyocardium", "ThinningMyocardium", "NormalMyocardium"],
    "SCT": ["VentricularAneurysm"],
}

name_for_cid[3705] = "ChamberSize"
cid_concepts[3705] = {
    "SCT": [
        "MarkedlyEnlargedCardiacChamber",
        "MildlyEnlargedCardiacChamber",
        "AbnormallySmallCardiacChamber",
        "ModeratelyEnlargedCardiacChamber",
        "NormalSizeCardiacChamber",
    ]
}

name_for_cid[3706] = "OverallContractility"
cid_concepts[3706] = {
    "SCT": ["Hypokinesis", "Hyperkinesis", "Akinesis", "NormalWallContractility"]
}

name_for_cid[3707] = "VSDDescription"
cid_concepts[3707] = {"SCT": ["NonRestrictive", "Membranous", "Restrictive", "None_"]}

name_for_cid[3709] = "AorticRootDescription"
cid_concepts[3709] = {
    "SCT": [
        "EnlargedAorticRoot",
        "RupturedSinusOfValsalva",
        "PostStenoticDilation",
        "NormalAorticRoot",
        "AneurysmOfAorticRoot",
        "AnnularAbscessOfAorticRoot",
    ]
}

name_for_cid[3710] = "CoronaryDominance"
cid_concepts[3710] = {
    "SCT": [
        "BalancedCoronaryDominance",
        "LeftCoronaryDominance",
        "RightCoronaryDominance",
    ]
}

name_for_cid[3711] = "ValvularAbnormality"
cid_concepts[3711] = {
    "DCM": ["HeartValveFlail"],
    "SCT": [
        "Regurgitation",
        "CalcifiedHeartValve",
        "HeartValveStenosis",
        "ValvularEndocarditis",
        "ImmobileHeartValve",
    ],
}

name_for_cid[3712] = "VesselDescriptor"
cid_concepts[3712] = {
    "SCT": [
        "DiffuseDisease",
        "Calcified",
        "Stented",
        "Restenotic",
        "Tortuous",
        "Stenotic",
        "Culprit",
        "Bifurcation",
        "LuminalIrregularities",
        "Aneurysmal",
        "Thrombus",
        "MuscleBridge",
        "Ulcerated",
        "Ectatic",
    ]
}

name_for_cid[3713] = "TIMIFlowCharacteristic"
cid_concepts[3713] = {
    "SCT": [
        "_1PenetrationWithoutPerfusion",
        "_2PartialPerfusion",
        "_3CompletePerfusion",
        "_0NoPerfusion",
    ]
}

name_for_cid[3714] = "Thrombus"
cid_concepts[3714] = {
    "SCT": ["PossibleThrombus", "SevereThrombus", "NoThrombus", "DefiniteThrombus"]
}

name_for_cid[3715] = "LesionMargin"
cid_concepts[3715] = {
    "SCT": ["Smooth", "MultipleIrregularities", "Ulcerative", "Irregular"]
}

name_for_cid[3716] = "Severity"
cid_concepts[3716] = {
    "SCT": [
        "Moderate",
        "Mild",
        "MildToModerate",
        "ModerateToSevere",
        "None_",
        "Fatal",
        "Severe",
    ]
}

name_for_cid[3717] = "LeftVentricleMyocardialWall17SegmentModel"
cid_concepts[3717] = {
    "SCT": [
        "LeftVentricleMidAnteriorSegment",
        "LeftVentricleBasalAnteroseptalSegment",
        "LeftVentricleBasalInferolateralSegment",
        "LeftVentricleBasalInferoseptalSegment",
        "LeftVentricleMidInferiorSegment",
        "LeftVentricleApicalInferiorSegment",
        "LeftVentricleApicalLateralSegment",
        "LeftVentricleApicalAnteriorSegment",
        "ApexOfLeftVentricle",
        "LeftVentricleBasalAnteriorSegment",
        "LeftVentricleApicalSeptalSegment",
        "LeftVentricleBasalAnterolateralSegment",
        "LeftVentricleBasalInferiorSegment",
        "LeftVentricleMidInferolateralSegment",
        "LeftVentricleMidAnterolateralSegment",
        "LeftVentricleMidInferoseptalSegment",
        "LeftVentricleMidAnteroseptalSegment",
    ]
}

name_for_cid[3718] = "MyocardialWallSegmentsinProjection"
cid_concepts[3718] = {
    "SCT": [
        "MyocardiumOfPosterolateralRegion",
        "MyocardiumOfApexOfHeart",
        "MyocardiumOfInferolateralRegion",
        "LeftVentricleBasalAnteriorSegment",
        "LeftVentricleApicalSeptalSegment",
        "MyocardiumOfDiaphragmaticRegion",
        "LeftVentricleBasalInferiorSegment",
        "MyocardiumOfAnterolateralRegion",
    ]
}

name_for_cid[3719] = "CanadianClinicalClassification"
cid_concepts[3719] = {
    "SCT": [
        "AnginaClassII",
        "AnginaClassI",
        "AnginaClassIV",
        "AnginaClassIII",
        "ChestPainNotPresent",
    ]
}

name_for_cid[3721] = "CardiovascularSurgery"
cid_concepts[3721] = {
    "SCT": [
        "CorrectionOfCongenitalCardiovascularDeformity",
        "CoronaryArteryBypassGraft",
        "OperationOnHeartValve",
        "HeartTransplant",
        "ImplantationOfCardiacPacemaker",
        "AbdominalAorticAneurysmStenting",
        "PercutaneousCoronaryIntervention",
        "ImplantationOfAutomaticCardiacDefibrillator",
        "AblationOperationForArrhythmia",
    ]
}

name_for_cid[3722] = "DiabeticTherapy"
cid_concepts[3722] = {
    "SCT": [
        "DiabeticOnOralTreatment",
        "DiabeticOnInsulin",
        "DiabeticOnDietaryTreatment",
    ]
}

name_for_cid[3723] = "MIType"
cid_concepts[3723] = {
    "SCT": ["NonSTElevationMyocardialInfarction", "STElevationMyocardialInfarction"]
}

name_for_cid[3724] = "SmokingHistory"
cid_concepts[3724] = {"SCT": ["CurrentSmoker", "NoHistoryOfSmoking", "FormerSmoker"]}

name_for_cid[3726] = "CoronaryInterventionIndication"
cid_concepts[3726] = {
    "DCM": ["CoronaryLesionGreaterThanEquals50PercentStenosis"],
    "SCT": ["CardiogenicShock"],
}

name_for_cid[3727] = "CatheterizationIndication"
cid_concepts[3727] = {
    "SCT": [
        "Cardiomyopathy",
        "HeartDisease",
        "ValvularHeartDisease",
        "CardiogenicShock",
        "ConductionDisorderOfTheHeart",
        "IschemicHeartDisease",
        "HeartTransplant",
        "CardiacFunctionTestAbnormal",
        "HeartDiseaseCongenital",
    ]
}

name_for_cid[3728] = "CathFinding"
cid_concepts[3728] = {
    "SCT": [
        "AcuteMitralRegurgitationFromChordalRupture",
        "NormalLeftVentricularSystolicFunctionAndWallMotion",
        "MildIntimalCoronaryIrregularitiesNoSignificantStenoses",
        "NormalRightHeartHemodynamics",
        "MultiVesselCoronaryArteryDisease",
        "SignificantCoronaryBypassGraftDisease",
        "PericardialTamponade",
        "MitralRegurgitation",
        "PulmonaryHypertension",
        "MitralStenosis",
        "SingleVesselCoronaryArteryDisease",
        "AtrialSeptalDefect",
        "AcuteMitralRegurgitationFromPapillaryMuscleDysfunction",
        "AcuteVentricularSeptalRupture",
        "TripleVesselCoronaryArteryDisease",
        "MitralValveProlapse",
        "HeartDiseaseCongenital",
        "NormalLeftHeartHemodynamics",
        "AorticInsufficiency",
        "NormalCoronaryArteries",
        "DepressionOfLeftVentricularSystolicFunction",
        "DoubleVesselCoronaryArteryDisease",
        "AcuteMitralRegurgitationFromChordalDysfunction",
        "HypertensiveHeartDisease",
        "CongestiveCardiomyopathy",
        "VentricularSeptalDefect",
        "NormalLeftAndRightHeartHemodynamics",
        "LeftMainCoronaryArteryDisease",
        "RestrictiveCardiomyopathy",
        "HypertrophicCardiomyopathyWithObstruction",
        "HypertrophicCardiomyopathyWithoutObstruction",
        "AcuteMitralRegurgitationFromPapillaryMuscleRupture",
        "AorticStenosis",
        "ConstrictivePericarditis",
    ]
}

name_for_cid[3729] = "AdmissionStatus"
cid_concepts[3729] = {
    "SCT": ["HospitalAdmissionElective", "EmergencyDepartment", "Transfer"]
}

name_for_cid[3730] = "InsurancePayor"
cid_concepts[3730] = {}

name_for_cid[3733] = "PrimaryCauseofDeath"
cid_concepts[3733] = {}

name_for_cid[3735] = "AcuteCoronarySyndromeTimePeriod"
cid_concepts[3735] = {}

name_for_cid[3736] = "NYHAClassification"
cid_concepts[3736] = {
    "SCT": ["NYHAClassII", "NYHAClassIII", "NYHAClassI", "NYHAClassIV"]
}

name_for_cid[3737] = "IschemiaNoninvasiveTest"
cid_concepts[3737] = {}

name_for_cid[3738] = "PreCathAnginaType"
cid_concepts[3738] = {}

name_for_cid[3739] = "CathProcedureType"
cid_concepts[3739] = {
    "DCM": ["PercutaneousCoronaryIntervention"],
    "SCT": [
        "CatheterizationOfBothLeftAndRightHeartWithGraft",
        "CatheterizationOfLeftHeart",
        "CatheterizationOfBothLeftAndRightHeartWithoutGraft",
        "CatheterizationOfRightHeart",
    ],
}

name_for_cid[3740] = "ThrombolyticAdministration"
cid_concepts[3740] = {
    "SCT": [
        "ThrombolyticAgentContraindicated",
        "AdministeredLessThan3HoursBeforePCI",
        "AdministeredBetween3And6HoursBeforePCI",
        "AdministeredBetween6HoursAnd7DaysBeforePCI",
    ]
}

name_for_cid[3741] = "LabVisitMedicationAdministration"
cid_concepts[3741] = {
    "SCT": [
        "Contraindicated",
        "AdministeredAfterLabVisit",
        "AdministeredBeforeLabVisit",
        "AdministeredDuringLabVisit",
    ]
}

name_for_cid[3742] = "PCIMedicationAdministration"
cid_concepts[3742] = {
    "SCT": [
        "Contraindicated",
        "AdministeredAfterPercutaneousCoronaryIntervention",
        "AdministeredDuringPercutaneousCoronaryIntervention",
        "NotAdministered",
        "AdministeredPriorToPercutaneousCoronaryIntervention",
    ]
}

name_for_cid[3743] = "Clopidogrel/TiclopidineAdministration"
cid_concepts[3743] = {
    "SCT": [
        "Contraindicated",
        "AdministeredAfterPercutaneousCoronaryIntervention",
        "NotAdministered",
        "AdministeredLessThan72HoursBeforePCI",
    ]
}

name_for_cid[3744] = "EFTestingMethod"
cid_concepts[3744] = {
    "SCT": [
        "CardiacVentriculography",
        "RadionuclideVentriculography",
        "Echocardiography",
    ]
}

name_for_cid[3745] = "CalculationMethod"
cid_concepts[3745] = {"SCT": ["Estimated", "Calculated"]}

name_for_cid[3746] = "PercutaneousEntrySite"
cid_concepts[3746] = {
    "SCT": [
        "ViaFemoralVein",
        "ViaFemoralArtery",
        "ViaArtery",
        "ViaBrachialArtery",
        "ViaArmVein",
        "ViaRadialArtery",
        "ViaVein",
    ]
}

name_for_cid[3747] = "PercutaneousClosure"
cid_concepts[3747] = {}

name_for_cid[3748] = "AngiographicEFTestingMethod"
cid_concepts[3748] = {"DCM": ["BiPlaneAngiography", "SinglePlaneAngiography"]}

name_for_cid[3749] = "PCIProcedureResult"
cid_concepts[3749] = {}

name_for_cid[3750] = "PreviouslyDilatedLesion"
cid_concepts[3750] = {}

name_for_cid[3752] = "GuidewireCrossing"
cid_concepts[3752] = {
    "DCM": ["GuidewireCrossingLesionSuccessful", "GuidewireCrossingLesionUnsuccessful"]
}

name_for_cid[3754] = "VascularComplication"
cid_concepts[3754] = {
    "SCT": [
        "LossOfDistalPulse",
        "Pseudoaneurysm",
        "AVFistula",
        "ArterialDissection",
        "OcclusionOfArtery",
        "Bleeding",
    ]
}

name_for_cid[3755] = "CathComplication"
cid_concepts[3755] = {
    "SCT": [
        "EmergencyPercutaneousCoronaryIntervention",
        "CardiogenicShock",
        "ConductionDisorderOfTheHeart",
        "ContrastMediaAdverseReaction",
        "CongestiveHeartFailure",
        "CerebrovascularAccidentOrStroke",
        "CardiacTamponade",
        "EmergencyCoronaryArteryBypass",
        "RenalFailure",
        "CardiacArrest",
    ]
}

name_for_cid[3756] = "CardiacPatientRiskFactor"
cid_concepts[3756] = {
    "SCT": [
        "ConductionDisorderOfTheHeart",
        "HistoryOfHypertension",
        "HistoryOfDiabetes",
        "HistoryOfCongestiveHeartFailure",
        "PeripheralVascularDisease",
        "InsulinDependentMotherIDM",
        "HIVPositive",
        "HistoryOfChronicLungDisease",
        "HistoryOfHypercholesterolemia",
        "HistoryOfRenalFailure",
        "HistoryOfCerebrovascularDisease",
        "HistoryOfMyocardialInfarction",
    ],
    "UMLS": ["InfantOfMotherWithGestationalDiabetes"],
}

name_for_cid[3757] = "CardiacDiagnosticProcedure"
cid_concepts[3757] = {
    "SCT": [
        "CardiacCT",
        "ExerciseStressEchocardiography",
        "SPECT",
        "TransthoracicEchocardiography",
        "PETHeartStudy",
        "CardiacCTForCalciumScoring",
        "CardiacCathCoronaryAngiogramAndLeftVentriculogram",
        "PharmacologicalStressTest",
        "CardiacBloodPoolImaging",
        "StressTestUsingCardiacPacing",
        "CardiacCatheterization",
        "RadionuclideMyocardialPerfusionStudy",
        "PharmacologicAndExerciseStressTest",
        "CardiacCatheterizationCoronaryAngiogram",
        "PerfusionImaging",
        "CTAngiographyOfCoronaryArteries",
        "CardiacMRIStress",
        "ExerciseToleranceTest",
        "RadionuclideAngiocardiography",
        "NuclearMedicineCardiovascularStudy",
        "MagneticResonanceAngiography",
        "TransesophagealEchocardiography",
        "CardiacMRI",
        "Echocardiography",
    ]
}

name_for_cid[3758] = "CardiovascularFamilyHistory"
cid_concepts[3758] = {
    "SCT": [
        "NoFamilyHistoryOfCardiovascularDisease",
        "FamilyHistoryOfDiabetesMellitus",
        "FamilyHistoryOfCoronaryArteriosclerosis",
        "NoFamilyHistoryOfDiabetes",
        "FamilyHistoryUnknown",
        "FamilyHistoryOfCardiovascularDisease",
        "FamilyHistoryOfMyocardialInfarction",
    ]
}

name_for_cid[3760] = "HypertensionTherapy"
cid_concepts[3760] = {
    "SCT": [
        "BetaBlockerContainingProduct",
        "DiureticAgent",
        "NitrateVasodilator",
        "AngiotensinIIReceptorAntagonist",
        "ACEInhibitor",
        "CalciumChannelBlockerContainingProduct",
    ]
}

name_for_cid[3761] = "AntilipemicAgent"
cid_concepts[3761] = {
    "NCIt": ["Fibrate"],
    "SCT": ["FishOils", "Statins", "AnionExchangeResin", "BileAcidSequestrant"],
}

name_for_cid[3762] = "AntiarrhythmicAgent"
cid_concepts[3762] = {
    "SCT": [
        "ClassIIIAntiarrhythmicAgent",
        "ClassIIAntiarrhythmicAgent",
        "ClassIVAntiarrhythmicAgent",
        "ClassIAntiarrhythmicAgent",
    ]
}

name_for_cid[3764] = "MyocardialInfarctionTherapy"
cid_concepts[3764] = {
    "SCT": [
        "InsertionOfCoronaryArteryStent",
        "ThrombolyticTherapy",
        "CoronaryArteryBypassGraft",
        "PercutaneousCoronaryIntervention",
    ]
}

name_for_cid[3769] = "ConcernType"
cid_concepts[3769] = {
    "SCT": [
        "Problem",
        "Complaint",
        "FindingReportedByPatientInformant",
        "Disease",
        "FunctionalPerformanceAndActivity",
        "Finding",
    ]
}

name_for_cid[3770] = "ProblemStatus"
cid_concepts[3770] = {
    "SCT": [
        "Suspected",
        "ActiveProblem",
        "InactiveProblem",
        "KnownAbsent",
        "Recurrent",
        "WellControlled",
        "Chronic",
        "Intermittent",
        "ProblemResolved",
    ]
}

name_for_cid[3772] = "HealthStatus"
cid_concepts[3772] = {
    "SCT": [
        "LostToFollowUp",
        "Disabled",
        "Deceased",
        "SymptomFree",
        "Alive",
        "ChronicallyIll",
        "AliveAndWell",
        "InRemission",
        "SeverelyDisabled",
        "SeverelyIll",
    ]
}

name_for_cid[3773] = "UseStatus"
cid_concepts[3773] = {"SCT": ["Suspended", "Ended", "InProgress"]}

name_for_cid[3774] = "SocialHistory"
cid_concepts[3774] = {
    "SCT": [
        "Nutrition",
        "TobaccoSmokingBehavior",
        "Exercise",
        "AlcoholConsumption",
        "DrugMisuseBehavior",
    ]
}

name_for_cid[3777] = "ImplantedDevice"
cid_concepts[3777] = {
    "SCT": [
        "ImplantableDefibrillator",
        "CardiacPacemaker",
        "LeftVentricularAssistDevice",
        "InsulinPump",
    ]
}

name_for_cid[3778] = "Stage"
cid_concepts[3778] = {
    "SCT": ["Stage1", "Stage5", "Stage0", "Stage4", "Stage2", "Stage3"]
}

name_for_cid[3781] = "LeftVentricleMyocardialWall18SegmentModel"
cid_concepts[3781] = {
    "DCM": [
        "LeftVentricleApicalAnterolateralSegment",
        "LeftVentricleApicalInferolateralSegment",
        "LeftVentricleApicalInferoseptalSegment",
        "LeftVentricleApicalAnteroseptalSegment",
    ],
    "SCT": [
        "LeftVentricleMidAnteriorSegment",
        "LeftVentricleBasalAnteroseptalSegment",
        "LeftVentricleBasalInferolateralSegment",
        "LeftVentricleBasalInferoseptalSegment",
        "LeftVentricleMidInferiorSegment",
        "LeftVentricleApicalInferiorSegment",
        "LeftVentricleApicalAnteriorSegment",
        "LeftVentricleBasalAnteriorSegment",
        "LeftVentricleBasalAnterolateralSegment",
        "LeftVentricleBasalInferiorSegment",
        "LeftVentricleMidInferolateralSegment",
        "LeftVentricleMidAnterolateralSegment",
        "LeftVentricleMidInferoseptalSegment",
        "LeftVentricleMidAnteroseptalSegment",
    ],
}

name_for_cid[3782] = "LeftVentricleBasalWall6Segments"
cid_concepts[3782] = {
    "SCT": [
        "LeftVentricleBasalAnteroseptalSegment",
        "LeftVentricleBasalInferolateralSegment",
        "LeftVentricleBasalInferoseptalSegment",
        "LeftVentricleBasalAnteriorSegment",
        "LeftVentricleBasalAnterolateralSegment",
        "LeftVentricleBasalInferiorSegment",
    ]
}

name_for_cid[3783] = "LeftVentricleMidlevelWall6Segments"
cid_concepts[3783] = {
    "SCT": [
        "LeftVentricleMidAnteriorSegment",
        "LeftVentricleMidInferolateralSegment",
        "LeftVentricleMidAnterolateralSegment",
        "LeftVentricleMidInferiorSegment",
        "LeftVentricleMidInferoseptalSegment",
        "LeftVentricleMidAnteroseptalSegment",
    ]
}

name_for_cid[3784] = "LeftVentricleApicalWall4Segments"
cid_concepts[3784] = {
    "SCT": [
        "LeftVentricleApicalInferiorSegment",
        "LeftVentricleApicalLateralSegment",
        "LeftVentricleApicalAnteriorSegment",
        "LeftVentricleApicalSeptalSegment",
    ]
}

name_for_cid[3785] = "LeftVentricleApicalWall6Segments"
cid_concepts[3785] = {
    "DCM": [
        "LeftVentricleApicalAnterolateralSegment",
        "LeftVentricleApicalInferolateralSegment",
        "LeftVentricleApicalInferoseptalSegment",
        "LeftVentricleApicalAnteroseptalSegment",
    ],
    "SCT": ["LeftVentricleApicalInferiorSegment", "LeftVentricleApicalAnteriorSegment"],
}

name_for_cid[3802] = "PlaqueStructure"
cid_concepts[3802] = {
    "SCT": [
        "Mixed",
        "NonCalcified",
        "Hyperplasia",
        "Fibrous",
        "PathologicCalcification",
        "FattyDegeneration",
    ]
}

name_for_cid[3804] = "StenosisMeasurementMethod"
cid_concepts[3804] = {
    "DCM": ["ECST", "NASCET", "AreaBasedMethod", "DiameterBasedMethod"]
}

name_for_cid[3805] = "StenosisType"
cid_concepts[3805] = {
    "SCT": [
        "Embolism",
        "Thrombosis",
        "Entrapment",
        "Fibrous",
        "Compression",
        "ArterioscleroticVascularDisease",
        "Vasculitis",
        "CysticAdventitialDisease",
        "RaynaudDisease",
    ]
}

name_for_cid[3806] = "StenosisShape"
cid_concepts[3806] = {"SCT": ["Eccentric", "Concentric"]}

name_for_cid[3807] = "VolumeMeasurementMethod"
cid_concepts[3807] = {
    "DCM": ["VolumeBasedMethod", "AreaBasedMethod", "DiameterBasedMethod"]
}

name_for_cid[3808] = "AneurysmType"
cid_concepts[3808] = {
    "SCT": [
        "SaccularAneurysm",
        "RupturedAneurysm",
        "TraumaticAneurysm",
        "DissectingAneurysm",
        "SerpentineAneurysm",
        "RacemoseAneurysm",
        "CalcifiedAneurysm",
        "MiliaryAneurysm",
        "MultipleAneurysm",
        "InflammatoryAbdominalAorticAneurysm",
        "CirsoidAneurysm",
        "FusiformAneurysm",
        "ThrombosedAneurysm",
        "ExpandingAneurysm",
        "CylindroidAneurysm",
        "VaricoseAneurysm",
        "MycoticAneurysm",
        "MixedAneurysm",
    ]
}

name_for_cid[3809] = "AssociatedCondition"
cid_concepts[3809] = {"SCT": ["TraumaticAbnormality", "MarfanSyndrome"]}

name_for_cid[3810] = "VascularMorphology"
cid_concepts[3810] = {
    "SCT": [
        "Embolism",
        "Pseudoaneurysm",
        "ArterialDissection",
        "Stenosis",
        "Plaque",
        "Tortuosity",
        "Dilatation",
        "VascularCoiling",
        "Angioma",
        "VascularSclerosis",
        "ArteriovenousFistula",
        "Diverticulum",
        "Thrombus",
        "FibromuscularDysplasia",
        "Aneurysm",
        "Occlusion",
        "Stent",
    ]
}

name_for_cid[3813] = "StentFinding"
cid_concepts[3813] = {
    "DCM": ["StentFracture", "StentDisintegration", "Endoleak"],
    "SCT": ["Hypoplasia", "MigrationOfImplantOrInternalDevice", "Stenosis"],
}

name_for_cid[3814] = "StentComposition"
cid_concepts[3814] = {"SCT": ["MetalStent", "PlasticStent"]}

name_for_cid[3815] = "SourceofVascularFinding"
cid_concepts[3815] = {
    "SCT": [
        "Embolism",
        "AfterProcedure",
        "Trauma",
        "Hyperplasia",
        "Tumor",
        "ThromboticDisorder",
        "Surgical",
        "Vasculitis",
    ]
}

name_for_cid[3817] = "VascularSclerosisType"
cid_concepts[3817] = {
    "SCT": [
        "VascularWallDegeneration",
        "Arteriosclerosis",
        "Phlebosclerosis",
        "UlceratedAtheromatousPlaque",
        "ComplicatedAtheromatousPlaque",
        "AdventitialDegeneration",
        "ElasticVascularSclerosis",
        "Fibroelastosis",
        "Arteriolosclerosis",
        "AtheroscleroticFibrousPlaque",
        "Atheroma",
        "FocalFibroelastosis",
        "CysticMedicalNecrosis",
        "FattyStreaks",
        "CalcifiedAtheromatousPlaque",
        "ArteriosclerosisWithFibrinoidNecrosis",
        "DiffuseFibroelastosis",
    ]
}

name_for_cid[3820] = "NoninvasiveVascularProcedure"
cid_concepts[3820] = {
    "SCT": [
        "CTArteriographyOfAbdominalAortaWithContrast",
        "CardiovascularCT",
        "PeripheralAngiography",
        "CTOfAbdominalAorta",
        "CTOfAbdominalAortaWithoutContrast",
        "VascularMRI",
        "TrunkAngiography",
        "CardiacMRI",
    ]
}

name_for_cid[3821] = "PapillaryMuscleIncluded/Excluded"
cid_concepts[3821] = {"DCM": ["PapillaryMuscleIncluded", "PapillaryMuscleExcluded"]}

name_for_cid[3823] = "RespiratoryStatus"
cid_concepts[3823] = {
    "DCM": ["CentralBreathingPosition"],
    "SCT": [
        "ValsalvaManeuver",
        "Expiration",
        "ShallowBreathing",
        "Inspiration",
        "AutonomousBreathing",
    ],
}

name_for_cid[3826] = "HeartRhythm"
cid_concepts[3826] = {
    "SCT": ["VentricularArrhythmia", "NormalSinusRhythm", "AtrialArrhythmia"]
}

name_for_cid[3827] = "VesselSegment"
cid_concepts[3827] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
    ],
    "SCT": [
        "ProfundaFemorisVein",
        "SubclavianVein",
        "ExternalIliacArtery",
        "ProximalRightCoronaryArtery",
        "CeliacAxis",
        "OvarianVein",
        "PosteriorCerebralArtery",
        "BasilarArtery",
        "DeepPalmarVenousArch",
        "LateralCalfPerforator",
        "MidCircumflexCoronaryArtery",
        "CentralRetinalVein",
        "IntermediateArteryRamus",
        "GastricArtery",
        "FourthLumbarArtery",
        "SmallestCardiacVein",
        "ThirdLumbarArtery",
        "UmbilicalArtery",
        "RadialArtery",
        "SolealVein",
        "LobarArtery",
        "ThighPerforator",
        "MidRightCoronaryArtery",
        "RightBranchOfHepaticArtery",
        "ExternalIliacVein",
        "SubclavianArtery",
        "CentralRetinalArtery",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OphthalmicArtery",
        "SegmentalArtery",
        "BrachialVein",
        "LeftMainCoronaryArteryOstium",
        "SuperficialPalmarArch",
        "OvarianArtery",
        "PeronealArtery",
        "SuperficialFemoralVein",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "SuperiorLeftPulmonaryVein",
        "_1stMarginalCoronaryArtery",
        "SuperiorMesentericVein",
        "LumbarArtery",
        "PeronealVein",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "AtrioventricularVein",
        "CoronaryArteryGraft",
        "InfraRenalAorta",
        "PulmonaryVeinGreatVessel",
        "CarotidBulb",
        "PoplitealVein",
        "GiacominiVein",
        "InternalCarotidArteryC5Segment",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "RightPulmonaryVein",
        "RightCoronaryArteryOstium",
        "MarginalCoronaryArtery",
        "HepaticVein",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "PosteriorTibialArtery",
        "GreatSaphenousVeinOfThigh",
        "PosterolateralBranchOfRightCoronaryArtery",
        "LeftHepaticVein",
        "InnominateVein",
        "SuperficialFemoralArtery",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "PosteriorCommunicatingArtery",
        "LeftBranchOfHepaticArtery",
        "ArcuateArteryOfTheKidney",
        "AbdominalAorta",
        "RightHepaticVein",
        "GastrocnemiusVein",
        "CephalicVein",
        "GastricVein",
        "LeftMainCoronaryArtery",
        "LeftMainBranchOfPortalVein",
        "InferiorRightHepaticVein",
        "VentricularVein",
        "CarotidSiphon",
        "PlantarArterialArch",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "AtrialVein",
        "_1stDiagonalCoronaryArtery",
        "AnteriorCardiacVein",
        "InterlobularArteryOfKidney",
        "LesserSaphenousVein",
        "InferiorMesentericArtery",
        "DigitalArteryOfHand",
        "GreatSaphenousVein",
        "SecondLumbarArtery",
        "_1stRightPosterolateralCoronaryArtery",
        "InternalCarotidArtery",
        "RightPosteriorAVCoronaryArtery",
        "UlnarVein",
        "SuperiorRightPulmonaryVein",
        "InferiorVenaCava",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "AnteriorMiddleCerebralArteryBifurcation",
        "PerforatingArteryOfKidney",
        "InternalCarotidArteryC6Segment",
        "DistalRightCoronaryArtery",
        "AnteriorTibialArtery",
        "LeftAnteriorDescendingCoronaryArtery",
        "CommonCarotidArtery",
        "GreatSaphenousVeinOfCalf",
        "LeftPulmonaryVein",
        "_3rdMarginalCoronaryArtery",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "RightMainBranchOfPortalVein",
        "RadialVein",
        "InferiorRightPulmonaryVein",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "CommonHepaticArtery",
        "UterineArtery",
        "FirstLumbarArtery",
        "PoplitealArtery",
        "MiddleCerebralArteryM1Segment",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "UlnarArtery",
        "SixthLumbarArtery",
        "PosteriorArchVein",
        "SplenicVein",
        "CarotidBifurcation",
        "SaphenousVein",
        "_2ndRightPosterolateralCoronaryArtery",
        "SplenicArtery",
        "InnominateArtery",
        "RenalArtery",
        "GastroduodenalArtery",
        "AccessoryRenalArtery",
        "AzygosVein",
        "BrachialArtery",
        "_2ndDiagonalCoronaryArtery",
        "DistalCircumflexCoronaryArtery",
        "DorsalisPedisArtery",
        "CommonFemoralArtery",
        "SmallCardiacVein",
        "SuperficialPalmarVenousArch",
        "CommonFemoralVein",
        "_1stLeftPosterolateralCoronaryArtery",
        "CircumflexCoronaryArtery",
        "InferiorMesentericVein",
        "AnteriorCommunicatingArtery",
        "MedianCubitalVein",
        "CoronarySinus",
        "HilarArtery",
        "TerminalInternalCarotidArtery",
        "LeftCoronaryArtery",
        "AxillaryArtery",
        "VertebralArtery",
        "_3rdDiagonalCoronaryArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "SaphenofemoralJunction",
        "SupraRenalAorta",
        "InternalJugularVein",
        "CockettPerforatingVein",
        "AVGrooveContinuationOfCircumflexArtery",
        "UmbilicalVein",
        "CommonIliacVein",
        "PosteriorCerebralArteryP1Segment",
        "TesticularArtery",
        "_3rdRightPosterolateralCoronaryArtery",
        "GreatCardiacVein",
        "PortalVein",
        "InferiorLeftPulmonaryVein",
        "AnteriorCerebralArtery",
        "RenalVein",
        "InternalIliacVein",
        "BoydPerforatingVein",
        "IlealVein",
        "CommonIliacArtery",
        "MiddleCardiacVein",
        "_2ndLeftPosterolateralCoronaryArtery",
        "MiddleCerebralArteryM2Segment",
        "MiddleCerebralArtery",
        "PosteriorCerebralArteryP2Segment",
        "PosteriorDescendingRightCoronaryArtery",
        "TesticularVein",
        "AnteriorTibialVein",
        "PosteriorTibialVein",
        "SaphenopoplitealJunction",
        "BasilicVein",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "HunterianPerforatingVein",
        "DeepPalmarArchOfRadialArtery",
        "ProfundaFemorisArtery",
        "FifthLumbarArtery",
        "AxillaryVein",
    ],
}

name_for_cid[3829] = "PulmonaryArtery"
cid_concepts[3829] = {
    "SCT": [
        "TrunkOfPulmonaryArtery",
        "PulmonaryValveSinuses",
        "RightPulmonaryArtery",
        "SuprapulmonicValveArea",
        "LeftPulmonaryArtery",
    ]
}

name_for_cid[3831] = "StenosisLength"
cid_concepts[3831] = {"SCT": ["Long", "Short"]}

name_for_cid[3832] = "StenosisGrade"
cid_concepts[3832] = {"SCT": ["Moderate", "Mild", "Severe"]}

name_for_cid[3833] = "CardiacEjectionFraction"
cid_concepts[3833] = {
    "LN": [
        "LeftVentricularEjectionFractionByCT",
        "RightVentricularEjectionFractionByMR",
        "LeftVentricularEjectionFractionByMR",
        "RightVentricularEjectionFractionByCT",
    ]
}

name_for_cid[3835] = "CardiacVolumeMeasurement"
cid_concepts[3835] = {
    "DCM": ["LeftAtrialEDVolume", "LeftAtrialESVolume"],
    "LN": [
        "RightVentricularESVolume",
        "LeftVentricularESVolume",
        "RightVentricularEDVolume",
        "LeftVentricularEDVolume",
    ],
    "SCT": ["StrokeVolume"],
}

name_for_cid[3836] = "TimebasedPerfusionMeasurement"
cid_concepts[3836] = {
    "DCM": [
        "SignalTimeToPeak",
        "SignalIncreaseStartTime",
        "SignalEarliestPeakTime",
        "SignalBaselineStart",
        "SignalBaselineEnd",
    ]
}

name_for_cid[3837] = "FiducialFeature"
cid_concepts[3837] = {
    "SCT": ["CommonIliacBifurcation", "RenalArtery", "Ostium", "AorticBifurcation"]
}

name_for_cid[3838] = "DiameterDerivation"
cid_concepts[3838] = {
    "DCM": ["AnteriorPosterior"],
    "SCT": ["Minimum", "Maximum", "Mean", "Transverse"],
}

name_for_cid[3839] = "CoronaryVein"
cid_concepts[3839] = {
    "SCT": [
        "GreatCardiacVein",
        "AnteriorCardiacVein",
        "VentricularVein",
        "AtrioventricularVein",
        "SmallCardiacVein",
        "SmallestCardiacVein",
        "MiddleCardiacVein",
        "CoronarySinus",
        "AtrialVein",
        "AzygosVein",
    ]
}

name_for_cid[3840] = "PulmonaryVein"
cid_concepts[3840] = {
    "SCT": [
        "PulmonaryVeinGreatVessel",
        "SuperiorLeftPulmonaryVein",
        "InferiorLeftPulmonaryVein",
        "RightPulmonaryVein",
        "InferiorRightPulmonaryVein",
        "LeftPulmonaryVein",
        "SuperiorRightPulmonaryVein",
    ]
}

name_for_cid[3843] = "MyocardialSubsegment"
cid_concepts[3843] = {"SCT": ["Endocardial", "Epicardial"]}

name_for_cid[3850] = "ContrastBolusSubstance"
cid_concepts[3850] = {
    "MSH": ["LactatedRingerS"],
    "RADLEX": ["NonIonicIodinatedContrastAgent", "IonicIodinatedContrastAgent"],
    "RXNORM": ["MangafodipirTrisodium"],
    "SCT": [
        "Iophendylate",
        "Iodipamide",
        "Bunamiodyl",
        "GadobenateDimeglumine",
        "Gadodiamide",
        "Gadofosveset",
        "Iohexol",
        "SodiumAcetrizoate",
        "Propyliodone",
        "ContrastAgent",
        "GadoterateMeglumine",
        "Iothalamate",
        "MeglumineDiatrizoate",
        "Gadobutrol",
        "Diatrizoate",
        "NonRadiopaqueMedium",
        "ChloriodizedOil",
        "Air",
        "Iopamidol",
        "Iopromide",
        "IopanoicAcid",
        "Metrizoate",
        "SodiumIodomethamate",
        "MeglumineIodipamide",
        "GadopentetateDimeglumine",
        "Ioxilan",
        "Ioversol",
        "Iodixanol",
        "Metrizamide",
        "Ipodate",
        "BariumSulfate",
        "Oxygen",
        "SodiumTyropanoate",
        "Water",
        "Saline",
        "IodoalphionicAcid",
        "Iodopyracet",
        "SodiumDiprotrizoate",
        "Iodophthalein",
        "GadoxetateDisodium",
        "Gadoversetamide",
        "IodamideMeglumine",
        "SodiumIodipamide",
        "Gadoteridol",
        "IophenoxicAcid",
        "Dextran",
        "Ioxaglate",
        "IodizedOil",
        "CarbonDioxide",
        "GadoliniumContainingProduct",
    ],
}

name_for_cid[4005] = "PartialViewSectionforMammography"
cid_concepts[4005] = {
    "SCT": [
        "Central",
        "Posterior",
        "Lateral",
        "Inferior",
        "Anterior",
        "Superior",
        "Medial",
    ]
}

name_for_cid[4009] = "DXAnatomyImaged"
cid_concepts[4009] = {
    "SCT": [
        "BoneOfLowerLimb",
        "Ureter",
        "Sacrum",
        "SmallIntestine",
        "ParanasalSinus",
        "InternalAuditoryCanal",
        "WristJoint",
        "Phantom",
        "Humerus",
        "Ilium",
        "JawRegion",
        "Calcaneus",
        "NasalBone",
        "Esophagus",
        "Mandible",
        "OpticCanal",
        "MastoidBone",
        "NeckChestAbdomenAndPelvis",
        "Thigh",
        "Pelvis",
        "ChestAndAbdomen",
        "Trachea",
        "UpperLimb",
        "AnkleJoint",
        "Maxilla",
        "Mediastinum",
        "SacroCoccygealSpine",
        "PancreaticDuct",
        "Hand",
        "UpperUrinaryTract",
        "Bronchus",
        "SesamoidBonesOfFoot",
        "CommonBileDuct",
        "Eye",
        "Prostate",
        "ElbowJoint",
        "FacialBones",
        "Colon",
        "Sternum",
        "SubmandibularGland",
        "ThoracicSpine",
        "NeckAndChest",
        "Duodenum",
        "Gallbladder",
        "BileDuct",
        "HipJoint",
        "UpperArm",
        "Extremity",
        "AbdomenAndPelvis",
        "Chest",
        "PancreaticDuctAndBileDuctSystems",
        "Larynx",
        "CervicoThoracicSpine",
        "Skull",
        "Urethra",
        "Hip",
        "Rib",
        "BoneOfUpperLimb",
        "Head",
        "AcromioclavicularJoint",
        "PelvisAndLowerExtremities",
        "Clavicle",
        "ParotidGland",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "Breast",
        "EsophagusStomachAndDuodenum",
        "Neck",
        "Finger",
        "Stomach",
        "Coccyx",
        "HeadAndNeck",
        "TemporomandibularJoint",
        "BiliaryTract",
        "ApexOfLung",
        "Ileum",
        "Bladder",
        "Knee",
        "LargeIntestine",
        "Forearm",
        "EyeRegion",
        "Spine",
        "Foot",
        "NeckChestAndAbdomen",
        "SacroiliacJoint",
        "Anus",
        "MuscleOfUpperLimb",
        "EntireBody",
        "Thumb",
        "LumbarSpine",
        "Zygoma",
        "Abdomen",
        "SellaTurcica",
        "UterusAndFallopianTubes",
        "LowerLimb",
        "LowerLeg",
        "VertebralColumnAndCranium",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "Heart",
        "TarsalJoint",
        "Toe",
        "Shoulder",
        "Fibula",
        "OrbitalStructure",
        "MuscleOfLowerLimb",
        "LiverAndBiliaryStructure",
        "Femur",
        "Rectum",
        "Pancreas",
        "Scapula",
        "Patella",
        "LumboSacralSpine",
        "CervicalSpine",
    ]
}

name_for_cid[4010] = "DXView"
cid_concepts[4010] = {
    "SCT": [
        "RightPosteriorOblique",
        "ObliqueCaudoCranial",
        "PosteroAnteriorOblique",
        "AxialProjection",
        "Sagittal",
        "Tangential",
        "CranioCaudal",
        "LateroMedialOblique",
        "Acanthioparietal",
        "Orbitoparietal",
        "OcclusalProjection",
        "ObliqueCranioCaudal",
        "Plantodorsal",
        "LeftAnteriorOblique",
        "LateralOblique",
        "RightOblique",
        "MedioLateralObliqueProjection",
        "RightLateral",
        "CaudoCranial",
        "Verticosubmental",
        "TissueSpecimen",
        "Frontal",
        "LeftLateral",
        "Parietoacanthial",
        "PosteroAnterior",
        "RightAnteriorOblique",
        "Submentovertical",
        "AnteroPosteriorOblique",
        "LateralMedial",
        "LeftOblique",
        "Dorsoplantar",
        "ParietoOrbital",
        "ObliqueProjection",
        "MedialOblique",
        "FrontalObliqueAxial",
        "SagittalObliqueAxial",
        "AnteroPosterior",
        "MedioLateralProjection",
        "FrontalOblique",
        "ObliqueAxial",
        "LateralProjection",
        "LeftPosteriorOblique",
    ]
}

name_for_cid[4011] = "DXViewModifier"
cid_concepts[4011] = {
    "DCM": ["Crosstable"],
    "SCT": [
        "Transoral",
        "Transforamenal",
        "Caudad",
        "Cephalad",
        "Transthoracic",
        "Transorbital",
        "MouthClosed",
    ],
}

name_for_cid[4012] = "ProjectionEponymousName"
cid_concepts[4012] = {
    "SCT": [
        "KempHarper",
        "Holmblad",
        "Swanson",
        "Lawrence",
        "Fuchs",
        "LaquerrierePierquin",
        "MorticeProjection",
        "Teufel",
        "May",
        "Ottonello",
        "AlbersSchonberg",
        "LowBeer",
        "Cleaves",
        "DaneliusMiller",
        "Kasabach",
        "Stryker",
        "BrewertonProjection",
        "Arcelin",
        "Causton",
        "GaynorHart",
        "Henschen",
        "Pearson",
        "ColcherSussman",
        "Stecher",
        "ChassardLapine",
        "Valdini",
        "Hough",
        "Twining",
        "Zanelli",
        "Cahoon",
        "Chamberlain",
        "Tarrant",
        "Rafert",
        "Towne",
        "Rosenberg",
        "Kovacs",
        "Waters",
        "Kuchendorf",
        "Friedman",
        "Chausse",
        "Garth",
        "BlackettHealy",
        "Fleischner",
        "WolfProjection",
        "Kurzbauer",
        "Lysholm",
        "Kandel",
        "HarrisBeathAxialProjection",
        "Kite",
        "Miller",
        "Hsieh",
        "Stenvers",
        "Dunlap",
        "Clements",
        "Caldwell",
        "Apple",
        "Burman",
        "Fisk",
        "Grashey",
        "Haas",
        "Merchant",
        "RafertLong",
        "JudetProjection",
        "Ferguson",
        "Lewis",
        "Broden",
        "Schuller",
        "Titterington",
        "Pawlow",
        "Nolke",
        "Norgaard",
        "Taylor",
        "Grandy",
        "Hickey",
        "Moore",
        "Neer",
        "Law",
        "LeonardGeorge",
        "Pirie",
        "Folio",
        "Holly",
        "Beclere",
        "Rhese",
        "WigbyTaylor",
        "Hughston",
        "ErasoModification",
        "Bertel",
        "Lorenz",
        "ClementsNakayama",
        "Judd",
        "Lauenstein",
        "Settegast",
        "Mayer",
        "Staunig",
        "CampCoventry",
        "Isherwood",
        "Penner",
        "VanRosenProjection",
        "WestPoint",
        "Alexander",
        "HirtzModification",
        "Lilienfeld",
        "Robert",
        "Lindblom",
    ]
}

name_for_cid[4013] = "AnatomicRegionforMammography"
cid_concepts[4013] = {"SCT": ["Breast"]}

name_for_cid[4014] = "ViewforMammography"
cid_concepts[4014] = {
    "SCT": [
        "SuperolateralToInferomedialOblique",
        "TissueSpecimenFromBreast",
        "MedioLateralProjection",
        "CranioCaudalExaggeratedLaterally",
        "MedioLateralObliqueProjection",
        "LateroMedial",
        "LateroMedialOblique",
        "CranioCaudal",
        "InferomedialToSuperolateralOblique",
        "CaudoCranial",
        "CranioCaudalExaggeratedMedially",
    ]
}

name_for_cid[4015] = "ViewModifierforMammography"
cid_concepts[4015] = {
    "SCT": [
        "RolledMedial",
        "NippleInProfile",
        "AxillaryTissue",
        "AnteriorCompression",
        "RolledSuperior",
        "SpotCompression",
        "Tangential",
        "AxillaryTail",
        "Magnification",
        "ImplantDisplaced",
        "Cleavage",
        "InfraMammaryFold",
        "RolledLateral",
        "RolledInferior",
    ]
}

name_for_cid[4016] = "AnatomicRegionforIntraoralRadiography"
cid_concepts[4016] = {
    "SCT": ["TeethGumsAndSupportingStructures", "Mandible", "JawRegion", "Maxilla"]
}

name_for_cid[4017] = "AnatomicRegionModifierforIntraoralRadiography"
cid_concepts[4017] = {
    "SCT": [
        "FirstMolarRegion",
        "FirstPremolarRegion",
        "ThirdMolarRegion",
        "LateralIncisorRegion",
        "SecondMolarRegion",
        "SecondPremolarRegion",
        "CanineRegion",
        "CentralIncisorRegion",
    ]
}

name_for_cid[
    4018
] = "PrimaryAnatomicStructureforIntraoralRadiography(PermanentDentitionDesignationofTeeth)"
cid_concepts[4018] = {
    "SCT": [
        "PermanentMaxillaryLeftFirstPremolarTooth",
        "PermanentMaxillaryRightSecondMolarTooth",
        "PermanentMaxillaryRightLateralIncisorTooth",
        "MandibularLeftSecondPremolarTooth",
        "PermanentMaxillaryLeftCanineTooth",
        "PermanentMaxillaryLeftThirdMolarTooth",
        "MandibularRightCentralIncisorTooth",
        "PermanentMaxillaryLeftSecondPremolarTooth",
        "PermanentMaxillaryLeftFirstMolarTooth",
        "PermanentMaxillaryRightSecondPremolarTooth",
        "MandibularLeftLateralTooth",
        "MandibularLeftSecondMolarTooth",
        "MandibularRightSecondPremolarTooth",
        "PermanentMaxillaryRightFirstMolarTooth",
        "MandibularLeftFirstMolarTooth",
        "MandibularLeftFirstPremolarTooth",
        "PermanentMaxillaryRightCentralIncisorTooth",
        "MandibularRightFirstPremolarTooth",
        "MandibularRightCanineTooth",
        "PermanentMaxillaryLeftCentralIncisorTooth",
        "MandibularRightLateralIncisorTooth",
        "PermanentMaxillaryRightThirdMolarTooth",
        "MandibularLeftThirdMolarTooth",
        "PermanentMaxillaryRightFirstPremolarTooth",
        "PermanentMaxillaryRightCanineTooth",
        "PermanentMaxillaryLeftSecondMolarTooth",
        "MandibularLeftCentralIncisorTooth",
        "MandibularRightFirstMolarTooth",
        "PermanentMaxillaryLeftLateralIncisorTooth",
        "MandibularLeftCanineTooth",
        "MandibularRightSecondMolarTooth",
        "MandibularRightThirdMolarTooth",
    ]
}

name_for_cid[
    4019
] = "PrimaryAnatomicStructureforIntraoralRadiography(DeciduousDentitionDesignationofTeeth)"
cid_concepts[4019] = {
    "SCT": [
        "DeciduousMandibularRightLateralIncisorTooth",
        "DeciduousMandibularLeftSecondMolarTooth",
        "DeciduousMandibularRightSecondMolarTooth",
        "DeciduousMaxillaryRightCentralIncisorTooth",
        "DeciduousMaxillaryLeftCentralIncisorTooth",
        "DeciduousMaxillaryLeftSecondMolarTooth",
        "DeciduousMaxillaryLeftFirstMolarTooth",
        "DeciduousMandibularLeftCanineTooth",
        "DeciduousMaxillaryRightSecondMolarTooth",
        "DeciduousMandibularRightCanineTooth",
        "DeciduousMandibularLeftFirstMolarTooth",
        "DeciduousMaxillaryRightCanineTooth",
        "DeciduousMandibularRightCentralIncisorTooth",
        "DeciduousMaxillaryRightLateralIncisorTooth",
        "DeciduousMaxillaryRightFirstMolarTooth",
        "DeciduousMaxillaryLeftLateralIncisorTooth",
        "DeciduousMandibularLeftLateralIncisorTooth",
        "DeciduousMandibularLeftCentralIncisorTooth",
        "DeciduousMandibularRightFirstMolarTooth",
        "DeciduousMaxillaryLeftCanineTooth",
    ]
}

name_for_cid[4020] = "PETRadionuclide"
cid_concepts[4020] = {
    "DCM": [
        "_51Manganese",
        "_70Arsenic",
        "_90Niobium",
        "_43Scandium",
        "_152Terbium",
        "_52mManganese",
        "_44Scandium",
    ],
    "SCT": [
        "_22Sodium",
        "_73Selenium",
        "_60Copper",
        "_14Oxygen",
        "_66Gallium",
        "_76Bromine",
        "_45Titanium",
        "_68Germanium",
        "_89Zirconium",
        "_15Oxygen",
        "_82Rubidium",
        "_11Carbon",
        "_62Copper",
        "_68Gallium",
        "_75Bromine",
        "_18Fluorine",
        "_62Zinc",
        "_64Copper",
        "_77Bromine",
        "_94mTechnetium",
        "_90Yttrium",
        "_124Iodine",
        "_38Potassium",
        "_86Yttrium",
        "_61Copper",
        "_52Iron",
        "_52Manganese",
        "_72Arsenic",
        "_13Nitrogen",
    ],
}

name_for_cid[4021] = "PETRadiopharmaceutical"
cid_concepts[4021] = {
    "DCM": [
        "Cetuximab89Zr",
        "RO532344189Zr",
        "J59189Zr",
        "RO54290889Zr",
        "Cu3689Zr",
        "RO695894818F",
        "Bevacizumab89Zr",
        "PSMA1007F18",
        "Cg250FAb289Zr",
        "PSMA617Ga68",
        "R150789Zr",
        "E4G1089Zr",
        "_7E1189Zr",
        "_7D1289Zr",
        "_28H189Zr",
        "MonoclonalAntibody64Cu",
        "MonoclonalAntibody89Zr",
        "NanocolloidalAlbumin89Zr",
        "AntiB22089Zr",
        "Trastuzumab89Zr",
        "T807F18",
        "FlubatineF18",
        "CmabU3689Zr",
        "DN3089Zr",
        "Fresolimumab89Zr",
        "TRC10589Zr",
        "Margetuximab89Zr",
        "SAR341989Zr",
        "PittsburghCompoundBC11",
        "Ranibizumab89Zr",
        "Roledumab89Zr",
        "Xmab557489Zr",
        "Brentuximab89Zr",
        "Panitumumab89Zr",
        "Rituximab89Zr",
        "Mogamulizumab89Zr",
        "Ublituximab89Zr",
        "MEDI55189Zr",
        "GA20189Zr",
        "Ecromeximab89Zr",
        "Benralizumab89Zr",
        "Ocaratuzumab89Zr",
        "GlembatumumabVedotin89Zr",
        "PinatuzumabVedotin89Zr",
        "PolatuzumabVedotin89Zr",
        "Blinatumomab89Zr",
        "Pegdinetanib89Zr",
        "AGN15099889Zr",
        "RO693164311C",
        "Obinituzimab89Zr",
        "RO692496311C",
        "GlutamineF18",
        "_2FAF18",
        "NifeneF18",
        "CLR1404I124",
        "CLR1404I131",
        "FlurpiridazF18",
        "GlutamineC14",
        "GlutamineC11",
        "ATSMCu60",
        "ATSMCu61",
        "ATSMCu62",
        "CholineC11",
        "FallyprideC11",
        "FallyprideF18",
        "FLB457C11",
        "FluorotrioprideF18",
        "DPA71418F",
        "DfFK89Zr",
        "DfFKPEG389Zr",
        "DfFK289Zr",
        "DfCD4589Zr",
        "DfFK23PEG489Zr",
        "DPA71311C",
    ],
    "NCIt": ["PiflufolastatF18", "SarcosineC11", "DCFBCF18", "PSMA11Ga68"],
    "SCT": [
        "MespiperoneC11",
        "RacloprideC11",
        "FluciclatideF18",
        "EDTAGa68",
        "GermaniumGe68",
        "CarbonMonoxideO15",
        "CarbonMonoxideC11",
        "FluorobenzothiazoleF18",
        "FluoroLDopaF18",
        "FluoromethaneF18",
        "SodiumFluorideF18",
        "FlorbetabenF18",
        "GlutamateN13",
        "FluoroestradiolF18",
        "FlumazenilF18",
        "MethionineC11",
        "FluorodeoxyglucoseF18",
        "FlutemetamolF18",
        "OxygenO15",
        "MonoclonalAntibodyI124",
        "SodiumIodideI124",
        "FlortaucipirF18",
        "AcetateC11",
        "FluorocholineF18",
        "FluorethyltyrosinF18",
        "CarbonDioxideO15",
        "FlorbetapirF18",
        "ThymidineF18",
        "FluoromisonidazoleF18",
        "ButanolO15",
        "CarfentanilC11",
        "OxygenWaterO15",
        "ATSMCu64",
        "SodiumNa22",
        "FluciclovineF18",
        "SpiperoneF18",
        "PalmitateC11",
        "AmmoniaN13",
        "FlumazenilC11",
        "PTSMCu62",
        "DotatateGa68",
        "FluorouracilF18",
        "RubidiumChlorideRb82",
    ],
    "UMLS": [
        "MK6240F18",
        "ISO1F18",
        "PK1119511C",
        "FluoropropylDihydrotetrabenazineF18",
        "FluoroetanidazoleF18",
        "THK5317F18",
        "UCBJC11",
        "EdotreotideGa68",
        "THK5351F18",
    ],
}

name_for_cid[
    4025
] = "PrimaryAnatomicStructureforIntraoralRadiography(SupernumeraryDentitionDesignationofTeeth)"
cid_concepts[4025] = {
    "SCT": [
        "SupernumeraryPermanentMandibularLeftFirstMolarTooth",
        "SupernumeraryDeciduousMandibularLeftSecondMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftCentralIncisorTooth",
        "SupernumeraryPermanentMaxillaryLeftSecondMolarTooth",
        "SupernumeraryDeciduousMaxillaryLeftCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightFirstPremolarTooth",
        "SupernumeraryPermanentMaxillaryLeftLateralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryLeftLateralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryLeftSecondMolarTooth",
        "SupernumeraryPermanentMandibularLeftLateralIncisorTooth",
        "SupernumeraryDeciduousMandibularRightLateralIncisorTooth",
        "SupernumeraryDeciduousMandibularRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightSecondPremolarTooth",
        "SupernumeraryDeciduousMandibularLeftLateralIncisorTooth",
        "SupernumeraryPermanentMaxillaryRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightFirstMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftSecondPremolarTooth",
        "SupernumeraryDeciduousMandibularRightFirstMolarTooth",
        "SupernumeraryDeciduousMaxillaryLeftCanineTooth",
        "SupernumeraryPermanentMandibularLeftFirstPremolarTooth",
        "SupernumeraryPermanentMaxillaryLeftFirstPremolarTooth",
        "SupernumeraryDeciduousMandibularLeftCentralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryRightFirstMolarTooth",
        "SupernumeraryPermanentMandibularLeftCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightCentralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryRightCanineTooth",
        "SupernumeraryPermanentMaxillaryLeftThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftCanineTooth",
        "SupernumeraryPermanentMandibularLeftThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryRightCanineTooth",
        "SupernumeraryDeciduousMandibularLeftFirstMolarTooth",
        "SupernumeraryDeciduousMaxillaryRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightThirdMolarTooth",
        "SupernumeraryPermanentMandibularLeftSecondMolarTooth",
        "SupernumeraryDeciduousMandibularRightCanineTooth",
        "SupernumeraryPermanentMaxillaryRightFirstPremolarTooth",
        "SupernumeraryPermanentMaxillaryRightSecondPremolarTooth",
        "SupernumeraryPermanentMandibularRightLateralIncisorTooth",
        "SupernumeraryPermanentMaxillaryRightFirstMolarTooth",
        "SupernumeraryDeciduousMandibularRightSecondMolarTooth",
        "SupernumeraryDeciduousMaxillaryLeftFirstMolarTooth",
        "SupernumeraryPermanentMandibularRightSecondMolarTooth",
        "SupernumeraryPermanentMaxillaryRightLateralIncisorTooth",
        "SupernumeraryPermanentMaxillaryRightSecondMolarTooth",
        "SupernumeraryPermanentMandibularLeftCanineTooth",
        "SupernumeraryPermanentMandibularRightCanineTooth",
        "SupernumeraryDeciduousMaxillaryRightLateralIncisorTooth",
        "SupernumeraryPermanentMandibularLeftSecondPremolarTooth",
        "SupernumeraryDeciduousMaxillaryRightSecondMolarTooth",
        "SupernumeraryPermanentMaxillaryRightThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftFirstMolarTooth",
        "SupernumeraryDeciduousMandibularLeftCanineTooth",
    ]
}

name_for_cid[
    4026
] = "PrimaryAnatomicStructureforIntraoralandCraniofacialRadiographyTeeth"
cid_concepts[4026] = {
    "SCT": [
        "DeciduousMandibularRightLateralIncisorTooth",
        "SupernumeraryPermanentMandibularLeftFirstMolarTooth",
        "PermanentMaxillaryLeftSecondPremolarTooth",
        "DeciduousMandibularRightCanineTooth",
        "SupernumeraryDeciduousMaxillaryLeftCentralIncisorTooth",
        "SupernumeraryPermanentMaxillaryLeftLateralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryLeftLateralIncisorTooth",
        "PermanentMaxillaryRightFirstMolarTooth",
        "DeciduousMaxillaryRightFirstMolarTooth",
        "MandibularRightCanineTooth",
        "SupernumeraryDeciduousMandibularRightLateralIncisorTooth",
        "DeciduousMandibularLeftLateralIncisorTooth",
        "SupernumeraryDeciduousMandibularRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightSecondPremolarTooth",
        "PermanentMaxillaryRightFirstPremolarTooth",
        "SupernumeraryPermanentMandibularRightFirstMolarTooth",
        "DeciduousMaxillaryLeftCentralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryLeftCanineTooth",
        "DeciduousMaxillaryRightCanineTooth",
        "SupernumeraryPermanentMaxillaryLeftFirstPremolarTooth",
        "SupernumeraryDeciduousMandibularLeftCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightCentralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryRightCanineTooth",
        "SupernumeraryPermanentMaxillaryLeftThirdMolarTooth",
        "MandibularLeftCanineTooth",
        "MandibularRightSecondMolarTooth",
        "SupernumeraryPermanentMandibularRightThirdMolarTooth",
        "PermanentMaxillaryRightLateralIncisorTooth",
        "SupernumeraryPermanentMandibularLeftSecondMolarTooth",
        "DeciduousMandibularRightSecondMolarTooth",
        "PermanentMaxillaryLeftCanineTooth",
        "SupernumeraryPermanentMaxillaryRightFirstPremolarTooth",
        "MandibularRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularRightLateralIncisorTooth",
        "MandibularLeftFirstPremolarTooth",
        "MandibularRightFirstPremolarTooth",
        "DeciduousMaxillaryLeftLateralIncisorTooth",
        "PermanentMaxillaryLeftCentralIncisorTooth",
        "DeciduousMandibularLeftCentralIncisorTooth",
        "SupernumeraryPermanentMaxillaryRightLateralIncisorTooth",
        "PermanentMaxillaryLeftSecondMolarTooth",
        "SupernumeraryPermanentMandibularLeftCanineTooth",
        "DeciduousMandibularLeftFirstMolarTooth",
        "DeciduousMandibularRightCentralIncisorTooth",
        "SupernumeraryPermanentMandibularLeftSecondPremolarTooth",
        "SupernumeraryDeciduousMaxillaryRightSecondMolarTooth",
        "MandibularRightFirstMolarTooth",
        "SupernumeraryDeciduousMandibularLeftCanineTooth",
        "DeciduousMandibularRightFirstMolarTooth",
        "SupernumeraryDeciduousMandibularLeftSecondMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftCentralIncisorTooth",
        "DeciduousMaxillaryRightCentralIncisorTooth",
        "DeciduousMaxillaryLeftSecondMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftSecondMolarTooth",
        "PermanentMaxillaryRightSecondPremolarTooth",
        "SupernumeraryPermanentMandibularRightFirstPremolarTooth",
        "MandibularLeftSecondMolarTooth",
        "MandibularLeftFirstMolarTooth",
        "DeciduousMaxillaryRightLateralIncisorTooth",
        "SupernumeraryDeciduousMaxillaryLeftSecondMolarTooth",
        "SupernumeraryPermanentMandibularLeftLateralIncisorTooth",
        "MandibularRightLateralIncisorTooth",
        "SupernumeraryDeciduousMandibularLeftLateralIncisorTooth",
        "MandibularLeftThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryRightCentralIncisorTooth",
        "SupernumeraryPermanentMaxillaryLeftSecondPremolarTooth",
        "SupernumeraryDeciduousMandibularRightFirstMolarTooth",
        "DeciduousMaxillaryLeftFirstMolarTooth",
        "SupernumeraryPermanentMandibularLeftFirstPremolarTooth",
        "SupernumeraryDeciduousMaxillaryRightFirstMolarTooth",
        "SupernumeraryPermanentMandibularLeftCentralIncisorTooth",
        "PermanentMaxillaryLeftLateralIncisorTooth",
        "SupernumeraryPermanentMaxillaryLeftCanineTooth",
        "SupernumeraryPermanentMandibularLeftThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryRightCanineTooth",
        "PermanentMaxillaryLeftFirstPremolarTooth",
        "SupernumeraryDeciduousMandibularLeftFirstMolarTooth",
        "SupernumeraryDeciduousMaxillaryRightCentralIncisorTooth",
        "PermanentMaxillaryRightSecondMolarTooth",
        "DeciduousMandibularLeftSecondMolarTooth",
        "MandibularLeftSecondPremolarTooth",
        "SupernumeraryDeciduousMandibularRightCanineTooth",
        "PermanentMaxillaryLeftThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryRightSecondPremolarTooth",
        "PermanentMaxillaryLeftFirstMolarTooth",
        "MandibularLeftLateralTooth",
        "MandibularRightSecondPremolarTooth",
        "SupernumeraryPermanentMaxillaryRightFirstMolarTooth",
        "PermanentMaxillaryRightCentralIncisorTooth",
        "SupernumeraryDeciduousMandibularRightSecondMolarTooth",
        "SupernumeraryDeciduousMaxillaryLeftFirstMolarTooth",
        "PermanentMaxillaryRightThirdMolarTooth",
        "SupernumeraryPermanentMandibularRightSecondMolarTooth",
        "PermanentMaxillaryRightCanineTooth",
        "SupernumeraryPermanentMaxillaryRightSecondMolarTooth",
        "DeciduousMandibularLeftCanineTooth",
        "SupernumeraryPermanentMandibularRightCanineTooth",
        "DeciduousMaxillaryRightSecondMolarTooth",
        "SupernumeraryDeciduousMaxillaryRightLateralIncisorTooth",
        "MandibularLeftCentralIncisorTooth",
        "SupernumeraryPermanentMaxillaryRightThirdMolarTooth",
        "SupernumeraryPermanentMaxillaryLeftFirstMolarTooth",
        "MandibularRightThirdMolarTooth",
        "DeciduousMaxillaryLeftCanineTooth",
    ]
}

name_for_cid[4028] = "CraniofacialAnatomicRegion"
cid_concepts[4028] = {
    "SCT": [
        "TemporalBone",
        "VomerBone",
        "HyoidBone",
        "ParanasalSinus",
        "SoftPalate",
        "MiddleEar",
        "InternalAuditoryCanal",
        "Neck",
        "MuscleOfHead",
        "VertebralColumnAndCranium",
        "Lip",
        "Tongue",
        "InnerEar",
        "ExternalEar",
        "FacialBones",
        "HeadAndNeck",
        "SubmandibularGland",
        "JawRegion",
        "TemporomandibularJoint",
        "NasalBone",
        "OrbitalStructure",
        "SalivaryGland",
        "OccipitalBone",
        "Tooth",
        "LacrimalBone",
        "BoneStructureOfHeadAndOrNeck",
        "Mandible",
        "Eyeball",
        "EyeRegion",
        "Larynx",
        "OpticCanal",
        "MastoidBone",
        "Skull",
        "FrontalBone",
        "SphenoidBone",
        "EthmoidBone",
        "Head",
        "PalatineBone",
        "Trachea",
        "MuscleOfNeck",
        "ParietalBone",
        "Maxilla",
        "CervicalSpine",
        "Zygoma",
    ]
}

name_for_cid[4029] = "DermatologyAnatomicSite"
cid_concepts[4029] = {
    "DCM": [
        "SkinOfParaspinalAreaOfTheNeck",
        "SkinOfLowerParaspinalRegion",
        "SkinOfMidParaspinalRegion",
        "SkinOfUpperParaspinalRegion",
        "SkinOfParaspinalAreaOfTheSuperiorBack",
        "SkinOfInferiorPosteriorSurfaceOfThePinna",
        "SkinOfInferiorHelixOfEar",
        "SkinOfCaruncleOfEye",
        "SkinOfAnteriorHelixOfEar",
        "SkinOfLateralPartOfDorsumOfFoot",
        "SkinOfLowerEyelidMargin",
        "SkinOfLowerAntihelixOfEar",
        "SkinOfPosteriorHelixOfEar",
        "SkinOfParanasalCheek",
        "SkinOfMedialPartOfDorsumOfFoot",
        "SkinOfInfraalarGroove",
        "SkinOfSuperiorPosteriorHelixOfEar",
        "SkinOfSuperiorAntihelixOfEar",
        "SkinOfSoleOfForefoot",
        "SkinOfPosteriorLobuleOfTheEar",
        "SkinOfUpperEyelidMargin",
        "SkinOfUpperAntihelixOfEar",
        "SkinOfSuperiorPosteriorSurfaceOfThePinna",
        "SkinOfMidBack",
    ],
    "FMA": [
        "SkinOfLowerOuterQuadrantOfBreast",
        "SkinOfUpperInnerQuadrantOfBreast",
        "SkinOfLowerInnerQuadrantOfBreast",
        "SkinOfDorsalPartOfFourthToe",
        "SkinOfDorsalPartOfFifthToe",
        "SkinOfUpperOuterQuadrantOfLeftBreast",
        "SkinOfVermilionProperOfLowerLip",
        "SkinOfDorsalPartOfGreatToe",
        "SkinOfDorsalPartOfSecondToe",
        "SkinOfDorsalPartOfThirdToe",
        "SkinOfVermilionProperOfUpperLip",
        "SkinOfBackOfTrunk",
        "MucosaOfOralSegmentOfHardPalate",
        "MucosaOfUvula",
        "MucosaOfPalatoglossalArch",
        "SkinOfPlantarPartOfThirdToe",
        "SkinOfPlantarPartOfFourthToe",
        "SkinOfPalmarPartOfLittleFinger",
        "SkinOfPlantarPartOfFifthToe",
        "SkinOfPlantarPartOfSecondToe",
        "SkinOfPalmarPartOfRingFinger",
        "SkinOfPalmarPartOfIndexFinger",
        "SkinOfPalmarPartOfMiddleFinger",
        "SkinOfPlantarPartOfGreatToe",
        "SkinOfPalmarPartOfThumb",
        "SkinOfDorsalPartOfLittleFinger",
        "MucosaOfPharynx",
        "SkinOfDorsalPartOfRingFinger",
        "MucosaOfDorsumOfPharyngealPartOfTongue",
        "MucosaOfDorsumOfOralPartOfTongue",
        "SkinOfDorsalPartOfIndexFinger",
        "SkinOfDorsalPartOfMiddleFinger",
        "SkinOfDorsalPartOfThumb",
        "SkinOfDorsumOfNose",
        "MucosaOfPosteriorWallOfOropharynx",
    ],
    "SCT": [
        "MucosaOfFloorOfMouth",
        "SkinOfOccipitalRegion",
        "SkinOfSupraclavicularRegionOfNeck",
        "NailUnitOfFourthToe",
        "SkinOfAnteriorSurfaceOfLowerLeg",
        "SkinOfMedialCanthus",
        "FemaleExternalUrethralOrifice",
        "SkinOfAxilla",
        "SkinOfAnteriorSurfaceOfUpperArm",
        "SkinOfAnteriorSurfaceOfForearm",
        "NailUnitOfFinger",
        "SkinOfPosteriorSurfaceOfForearm",
        "SkinOfChin",
        "SkinOfTemporalRegion",
        "NailUnitOfIndexFinger",
        "SkinOfLowerEyelid",
        "MucosaOfMandibularGingiva",
        "SkinOfInguinalRegion",
        "SkinOfLabium",
        "SkinOfPerineum",
        "MaleExternalUrethralOrifice",
        "NailUnitOfThumb",
        "SkinOfUpperAbdomen",
        "SkinOfLowerLip",
        "Cornea",
        "Eyelash",
        "SkinOfEyeRegion",
        "SkinOfAntitragus",
        "SkinOfScalp",
        "SkinOfMedialSurfaceOfThigh",
        "SkinOfAreola",
        "SkinOfEarLobule",
        "SkinOfNuchalRegion",
        "SkinOfPrepuceOfClitoris",
        "SkinOfPosteriorSurfaceOfElbow",
        "SkinOfBack",
        "SkinOfPoplitealFossa",
        "VulvalVestibule",
        "SkinOfPosteriorSurfaceOfUpperArm",
        "SkinOfLateralPartOfHeel",
        "SkinOfUmbilicus",
        "FrenulumOfLabiaMinora",
        "NailUnitOfThirdToe",
        "SkinOfLateralCanthus",
        "SkinOfBackOfUpperThoracicRegion",
        "SkinOfSideOfNose",
        "SkinOfLowerChestWall",
        "SkinOfUpperExtremity",
        "SkinOfNasolabialFold",
        "SkinOfThenarRegionOfPalm",
        "SkinOfHeel",
        "SkinOfAntecubitalFossa",
        "SkinOfExternalGenitalia",
        "MucosaOfUpperLip",
        "SkinOfSubmentalArea",
        "SkinOfClitoris",
        "SkinOfPostauricularRegion",
        "SkinOfForehead",
        "MucosaOfMaxillaryGingiva",
        "SkinOfAnteriorSurfaceOfKnee",
        "SkinOfLip",
        "MucosaOfTipOfTongue",
        "SkinOfLowerBack",
        "SkinOfPhiltrum",
        "Sclera",
        "SkinOfTragus",
        "SkinOfFace",
        "SkinOfCavityOfConcha",
        "SkinOfEpigastricArea",
        "SkinOfPalmOfHand",
        "SkinOfTipOfNose",
        "NailUnitOfFifthToe",
        "SkinOfLateralAspectOfAnkle",
        "SkinOfCrusOfHelix",
        "VaginalIntroitus",
        "SkinOfAnteriorPortionOfNeck",
        "SkinOfLowerAbdomen",
        "SkinOfExternalAuditoryCanal",
        "SkinOfAnteriorSurfaceOfThigh",
        "SkinOfNeck",
        "SkinOfLateralPortionOfNeck",
        "SkinOfAbdomen",
        "SkinOfLabiumMinus",
        "Tooth",
        "SkinOfDigitOfHand",
        "SubmandibularTriangle",
        "SkinOfHead",
        "SkinOfLowerExtremity",
        "SkinOfMedialPartOfHeel",
        "SkinOfRootOfPenis",
        "SkinOfPartOfDorsalSurfaceOfHand",
        "SkinOfMedialAspectOfAnkle",
        "SkinOfGlutealFold",
        "SkinOfLabiumMajus",
        "SkinOfButtock",
        "SkinOfLateralBorderOfSoleOfFoot",
        "NailUnitOfLittleFinger",
        "SkinOfParietalRegion",
        "SkinOfEyebrow",
        "SkinOfHand",
        "SkinOfPosteriorSurfaceOfLowerLeg",
        "SkinOfAlaNasi",
        "PosteriorCommissureOfLabiumMajorum",
        "SkinOfUpperLip",
        "SkinOfPosteriorSurfaceOfThorax",
        "SkinOfToe",
        "SkinOfPosteriorSurfaceOfThigh",
        "SkinOfGlansPenis",
        "SkinOfNose",
        "SkinOfVertexOfScalp",
        "SkinOfPalmarAreaOfWrist",
        "SkinOfHelixOfEar",
        "NailUnitOfMiddleFinger",
        "SkinOfInfraclavicularRegion",
        "Iris",
        "SkinOfShaftOfPenis",
        "MucosaOfTongue",
        "SkinOfHypothenarRegionOfPalm",
        "SkinOfAnteriorSurfaceOfThorax",
        "OralMucosa",
        "GroinSkinCrease",
        "SkinOfAnteriorTrunk",
        "NailUnitOfGreatToe",
        "SkinOfUpperEyelid",
        "NailUnitOfToe",
        "SkinOfEar",
        "SkinOfPreauricularRegion",
        "NailUnitOfSecondToe",
        "SkinOfAnus",
        "SternalSkin",
        "Hair",
        "AnteriorTriangleOfNeck",
        "MucosaOfLowerLip",
        "Skin",
        "SkinOfNipple",
        "SkinOfPerioralRegionOfFace",
        "SkinOfCheek",
        "SkinOfJawline",
        "SkinOfFoot",
        "SkinOfIntertragalIncisure",
        "SkinOfDorsalAreaOfWrist",
        "NailUnitOfRingFinger",
        "SkinOfHypogastricRegion",
        "SkinOfScrotum",
        "SkinOfUpperTrunk",
        "Retina",
        "SkinOfPenis",
        "SkinOfMedialBorderOfSoleOfFoot",
    ],
}

name_for_cid[4030] = "CT,MRandPETAnatomyImaged"
cid_concepts[4030] = {
    "SCT": [
        "IliacAndOrFemoralArtery",
        "BoneOfLowerLimb",
        "Ureter",
        "Sacrum",
        "SmallIntestine",
        "ParanasalSinus",
        "InternalAuditoryCanal",
        "WristJoint",
        "Phantom",
        "Thymus",
        "CarotidArtery",
        "Humerus",
        "Ilium",
        "JawRegion",
        "Calcaneus",
        "NasalBone",
        "Esophagus",
        "Mandible",
        "OpticCanal",
        "MastoidBone",
        "NeckChestAbdomenAndPelvis",
        "Thigh",
        "Pelvis",
        "Spleen",
        "ChestAndAbdomen",
        "CoronaryArtery",
        "Trachea",
        "UpperLimb",
        "AnkleJoint",
        "Maxilla",
        "Cerebellum",
        "Mediastinum",
        "SacroCoccygealSpine",
        "PancreaticDuct",
        "Hand",
        "UpperUrinaryTract",
        "Bronchus",
        "SesamoidBonesOfFoot",
        "CommonBileDuct",
        "Eye",
        "Prostate",
        "ElbowJoint",
        "FacialBones",
        "Colon",
        "Sternum",
        "SubmandibularGland",
        "ThoracicSpine",
        "NeckAndChest",
        "Duodenum",
        "Gallbladder",
        "RenalArtery",
        "BileDuct",
        "HipJoint",
        "UpperArm",
        "Extremity",
        "AbdomenAndPelvis",
        "Chest",
        "PancreaticDuctAndBileDuctSystems",
        "Larynx",
        "PulmonaryArtery",
        "CervicoThoracicSpine",
        "Skull",
        "Liver",
        "Urethra",
        "Thyroid",
        "Hip",
        "Rib",
        "BoneOfUpperLimb",
        "Head",
        "AdrenalGland",
        "AcromioclavicularJoint",
        "PelvisAndLowerExtremities",
        "Clavicle",
        "ParotidGland",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "Breast",
        "EsophagusStomachAndDuodenum",
        "Neck",
        "Finger",
        "CranialVenousSystem",
        "Stomach",
        "ThoracicAorta",
        "Coccyx",
        "HeadAndNeck",
        "Uterus",
        "TemporomandibularJoint",
        "BiliaryTract",
        "ApexOfLung",
        "Ileum",
        "Testis",
        "Bladder",
        "Knee",
        "LargeIntestine",
        "Forearm",
        "EyeRegion",
        "Spine",
        "Foot",
        "NeckChestAndAbdomen",
        "SacroiliacJoint",
        "Anus",
        "MuscleOfUpperLimb",
        "EntireBody",
        "Thumb",
        "LumbarSpine",
        "Zygoma",
        "Abdomen",
        "SellaTurcica",
        "UterusAndFallopianTubes",
        "LowerLimb",
        "LowerLeg",
        "AorticArch",
        "VertebralColumnAndCranium",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "AbdominalAorta",
        "Heart",
        "TarsalJoint",
        "Toe",
        "Shoulder",
        "Fibula",
        "OrbitalStructure",
        "CircleOfWillis",
        "Brain",
        "MuscleOfLowerLimb",
        "Parathyroid",
        "LiverAndBiliaryStructure",
        "Femur",
        "Rectum",
        "Pancreas",
        "Scapula",
        "Kidney",
        "Patella",
        "LumboSacralSpine",
        "CervicalSpine",
    ]
}

name_for_cid[4031] = "CommonAnatomicRegion"
cid_concepts[4031] = {
    "SCT": [
        "BoneOfLowerLimb",
        "Ureter",
        "Sacrum",
        "SmallIntestine",
        "ParanasalSinus",
        "InternalAuditoryCanal",
        "WristJoint",
        "Phantom",
        "Humerus",
        "Ilium",
        "JawRegion",
        "Calcaneus",
        "NasalBone",
        "Esophagus",
        "Mandible",
        "OpticCanal",
        "MastoidBone",
        "NeckChestAbdomenAndPelvis",
        "Thigh",
        "Pelvis",
        "ChestAndAbdomen",
        "Trachea",
        "UpperLimb",
        "AnkleJoint",
        "Maxilla",
        "Mediastinum",
        "SacroCoccygealSpine",
        "PancreaticDuct",
        "Hand",
        "UpperUrinaryTract",
        "Bronchus",
        "SesamoidBonesOfFoot",
        "CommonBileDuct",
        "Eye",
        "Prostate",
        "ElbowJoint",
        "FacialBones",
        "Colon",
        "Sternum",
        "SubmandibularGland",
        "ThoracicSpine",
        "NeckAndChest",
        "Duodenum",
        "Gallbladder",
        "BileDuct",
        "HipJoint",
        "UpperArm",
        "Extremity",
        "AbdomenAndPelvis",
        "Chest",
        "PancreaticDuctAndBileDuctSystems",
        "Larynx",
        "CervicoThoracicSpine",
        "Skull",
        "Urethra",
        "Hip",
        "Rib",
        "BoneOfUpperLimb",
        "Head",
        "AcromioclavicularJoint",
        "PelvisAndLowerExtremities",
        "Clavicle",
        "ParotidGland",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "Breast",
        "EsophagusStomachAndDuodenum",
        "Neck",
        "Finger",
        "Stomach",
        "Coccyx",
        "HeadAndNeck",
        "TemporomandibularJoint",
        "BiliaryTract",
        "ApexOfLung",
        "Ileum",
        "Bladder",
        "Knee",
        "LargeIntestine",
        "Forearm",
        "EyeRegion",
        "Spine",
        "Foot",
        "NeckChestAndAbdomen",
        "SacroiliacJoint",
        "Anus",
        "MuscleOfUpperLimb",
        "EntireBody",
        "Thumb",
        "LumbarSpine",
        "Zygoma",
        "Abdomen",
        "SellaTurcica",
        "UterusAndFallopianTubes",
        "LowerLimb",
        "LowerLeg",
        "VertebralColumnAndCranium",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "Heart",
        "TarsalJoint",
        "Toe",
        "Shoulder",
        "Fibula",
        "OrbitalStructure",
        "MuscleOfLowerLimb",
        "LiverAndBiliaryStructure",
        "Femur",
        "Rectum",
        "Pancreas",
        "Scapula",
        "Patella",
        "LumboSacralSpine",
        "CervicalSpine",
    ]
}

name_for_cid[4032] = "MRSpectroscopyMetabolite"
cid_concepts[4032] = {
    "DCM": [
        "CreatineCholineCitrateRatio",
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "CreatineAndCholine",
        "NAcetylaspartateCholineRatio",
        "LipidAndLactate",
    ],
    "SCT": [
        "Inositol",
        "Citrate",
        "Choline",
        "Creatine",
        "Lactate",
        "Glutamine",
        "NAcetylaspartate",
        "Lipid",
        "Tuarine",
    ],
}

name_for_cid[4033] = "MRProtonSpectroscopyMetabolite"
cid_concepts[4033] = {
    "DCM": [
        "CreatineCholineCitrateRatio",
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "CreatineAndCholine",
        "NAcetylaspartateCholineRatio",
        "LipidAndLactate",
    ],
    "SCT": [
        "Inositol",
        "Citrate",
        "Choline",
        "Creatine",
        "Lactate",
        "Glutamine",
        "NAcetylaspartate",
        "Lipid",
        "Tuarine",
    ],
}

name_for_cid[4040] = "EndoscopyAnatomicRegion"
cid_concepts[4040] = {
    "SCT": [
        "Mediastinum",
        "Ureter",
        "UterusAndFallopianTubes",
        "ParanasalSinus",
        "PancreaticDuct",
        "EsophagusStomachAndDuodenum",
        "IntraPelvic",
        "UpperUrinaryTract",
        "Bronchus",
        "CommonBileDuct",
        "SigmoidColon",
        "PharynxAndLarynx",
        "Shoulder",
        "AnusRectumAndSigmoidColon",
        "LumenOfBloodVessel",
        "BiliaryTract",
        "TracheaAndBronchus",
        "Gallbladder",
        "Cervix",
        "Bladder",
        "Knee",
        "BileDuct",
        "Joint",
        "LargeIntestine",
        "PancreaticDuctAndBileDuctSystems",
        "Larynx",
        "Spine",
        "Rectum",
        "IntraThoracic",
        "Pharynx",
        "ExternalAuditoryCanal",
        "InguinalRegion",
        "Kidney",
        "Nasopharynx",
        "BladderAndUrethra",
        "IntraAbdominopelvic",
    ]
}

name_for_cid[4042] = "XA/XRFAnatomyImaged"
cid_concepts[4042] = {
    "SCT": [
        "SubclavianVein",
        "ExternalIliacArtery",
        "BoneOfLowerLimb",
        "TruncusCoeliacus",
        "BasilarArtery",
        "ParanasalSinus",
        "ApexOfRightVentricle",
        "WristJoint",
        "Phantom",
        "GenicularArtery",
        "LeftVentricleInflow",
        "CarotidArtery",
        "UmbilicalArtery",
        "PrimitiveAorta",
        "LeftVentricleOutflowTract",
        "RadialArtery",
        "RightFemoralArtery",
        "JawRegion",
        "LeftFemoralArtery",
        "Calcaneus",
        "Esophagus",
        "AntecubitalVein",
        "Mandible",
        "PatentDuctusArteriosus",
        "MesentericVein",
        "Pelvis",
        "PulmonaryVein",
        "CoronaryArtery",
        "UpperLimb",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "SubclavianArtery",
        "Maxilla",
        "SacroCoccygealSpine",
        "OphthalmicArtery",
        "PancreaticDuct",
        "Hand",
        "BrachialVein",
        "Eye",
        "Prostate",
        "ElbowJoint",
        "HepaticArtery",
        "FacialBones",
        "Sternum",
        "PeronealArtery",
        "NeckAndChest",
        "RightVentricleOutflowTract",
        "Duodenum",
        "SuperiorLeftPulmonaryVein",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "AbdomenAndPelvis",
        "LumbarArtery",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "Larynx",
        "FemoralArtery",
        "PulmonaryArtery",
        "IliacVein",
        "Skull",
        "Urethra",
        "Head",
        "AcromioclavicularJoint",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "PosteriorMedialTributary",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "Breast",
        "EsophagusStomachAndDuodenum",
        "Neck",
        "Finger",
        "PulmonaryArteryConduit",
        "RightAtrium",
        "RightVentricle",
        "CommonVentricle",
        "Coccyx",
        "HeadAndNeck",
        "TemporomandibularJoint",
        "HepaticVein",
        "InternalMammaryArtery",
        "Bladder",
        "LargeIntestine",
        "Forearm",
        "Spine",
        "Foot",
        "NeckChestAndAbdomen",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "MuscleOfUpperLimb",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "SuperficialFemoralArtery",
        "UterusAndFallopianTubes",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "JuxtaposedAtrialAppendage",
        "AorticArch",
        "VertebralColumnAndCranium",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "AbdominalAorta",
        "Heart",
        "TarsalJoint",
        "CephalicVein",
        "LingualArtery",
        "GastricVein",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "Fibula",
        "OrbitalStructure",
        "InferiorCardiacVein",
        "Baffle",
        "LacrimalArtery",
        "OccipitalArtery",
        "SystemicVenousAtrium",
        "LiverAndBiliaryStructure",
        "AnomalousPulmonaryVein",
        "Femur",
        "Rectum",
        "AnteriorCardiacVein",
        "Pancreas",
        "LeftAtrium",
        "CerebralArtery",
        "Scapula",
        "InferiorMesentericArtery",
        "GreatSaphenousVein",
        "CervicalSpine",
        "InternalCarotidArtery",
        "Ureter",
        "Sacrum",
        "SmallIntestine",
        "InternalAuditoryCanal",
        "SuperiorRightPulmonaryVein",
        "InferiorVenaCava",
        "RightPulmonaryArtery",
        "VenousNetwork",
        "CommonAtrium",
        "Humerus",
        "Ilium",
        "AorticFistula",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "NasalBone",
        "AnteriorTibialArtery",
        "TruncusArteriosusCommunis",
        "Artery",
        "CommonCarotidArtery",
        "OpticCanal",
        "Aorta",
        "SuperficialTemporalArtery",
        "MastoidBone",
        "Vein",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "Thigh",
        "ChestAndAbdomen",
        "InferiorRightPulmonaryVein",
        "Trachea",
        "BodyConduit",
        "AnkleJoint",
        "Mediastinum",
        "RightVentricleInflow",
        "MesentericArtery",
        "PoplitealArtery",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "Bronchus",
        "SesamoidBonesOfFoot",
        "InternalIliacArtery",
        "PulmonaryChamberOfCorTriatriatum",
        "CommonBileDuct",
        "SuperiorVenaCava",
        "UlnarArtery",
        "LeftVentricle",
        "AnteriorSpinalArtery",
        "SplenicVein",
        "Colon",
        "SubmandibularGland",
        "ThoracicSpine",
        "SaphenousVein",
        "Gallbladder",
        "InnominateArtery",
        "SplenicArtery",
        "RenalArtery",
        "HipJoint",
        "UpperArm",
        "Extremity",
        "Chest",
        "FacialArtery",
        "AzygosVein",
        "BrachialArtery",
        "CervicoThoracicSpine",
        "Hip",
        "Rib",
        "BoneOfUpperLimb",
        "PelvisAndLowerExtremities",
        "Clavicle",
        "ParotidGland",
        "CommonFemoralArtery",
        "Stomach",
        "AnteriorCommunicatingArtery",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "ThoracicAorta",
        "VenaCava",
        "CoronarySinus",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "BiliaryTract",
        "ApexOfLung",
        "Ileum",
        "Knee",
        "VertebralArtery",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "InternalJugularVein",
        "EyeRegion",
        "LacrimalArteryOfRightEye",
        "SacroiliacJoint",
        "Anus",
        "UmbilicalVein",
        "EntireBody",
        "CommonIliacVein",
        "Thumb",
        "LumbarSpine",
        "GreatCardiacVein",
        "PrimitivePulmonaryArtery",
        "SellaTurcica",
        "PortalVein",
        "InferiorLeftPulmonaryVein",
        "LowerLeg",
        "RenalVein",
        "BoydPerforatingVein",
        "CommonIliacArtery",
        "Toe",
        "Shoulder",
        "LeftAuricularAppendage",
        "MuscleOfLowerLimb",
        "AscendingAorta",
        "HunterianPerforatingVein",
        "ProfundaFemorisArtery",
        "AxillaryVein",
        "Patella",
        "OccipitalVein",
        "LumboSacralSpine",
        "DoddPerforatingVein",
    ]
}

name_for_cid[4050] = "DrugorContrastAgentCharacteristic"
cid_concepts[4050] = {
    "DCM": ["ContrastBolusIngredientOpaque", "ActiveIngredientUndilutedConcentration"],
    "SCT": ["ActiveIngredient", "Volume"],
}

name_for_cid[4051] = "GeneralDevice"
cid_concepts[4051] = {
    "DCM": [
        "ACRAccreditationPhantomCT",
        "ACRAccreditationPhantomMR",
        "ACRAccreditationPhantomMammography",
        "ACRAccreditationPhantomStereotacticBreastBiopsy",
        "ACRAccreditationPhantomECT",
        "_10cmDosimetryPhantom",
        "ACRAccreditationPhantomPET",
        "ACRAccreditationPhantomECTPET",
        "ACRAccreditationPhantomPETFaceplate",
        "IECHeadDosimetryPhantom",
        "IECBodyDosimetryPhantom",
        "NEMAXR212000Phantom",
        "Sphere",
    ],
    "SCT": [
        "Glue",
        "AtherectomyDevice",
        "EmbolizationCoil",
        "EmbolizationParticulate",
        "Phantom",
        "Laser",
        "Catheter",
        "MeasuringRuler",
        "RotationalAtherectomyDevice",
        "GuidingCatheter",
        "EmbolizationBall",
        "PercutaneousTransluminalAngioplastyBalloon",
        "Syringe",
        "Stent",
        "DetachableBalloon",
    ],
}

name_for_cid[4052] = "PhantomDevice"
cid_concepts[4052] = {
    "DCM": [
        "IECHeadDosimetryPhantom",
        "IECBodyDosimetryPhantom",
        "NEMAXR212000Phantom",
        "ACRAccreditationPhantomCT",
        "ACRAccreditationPhantomMR",
        "ACRAccreditationPhantomMammography",
        "ACRAccreditationPhantomStereotacticBreastBiopsy",
        "ACRAccreditationPhantomECT",
        "_10cmDosimetryPhantom",
        "ACRAccreditationPhantomPET",
        "ACRAccreditationPhantomECTPET",
        "ACRAccreditationPhantomPETFaceplate",
    ],
    "SCT": ["Phantom"],
}

name_for_cid[4100] = "T1MeasurementMethod"
cid_concepts[4100] = {
    "DCM": ["T1ByInversionRecovery", "T1ByFixedValue", "T1ByMultipleFlipAngles"]
}

name_for_cid[4101] = "TracerKineticModel"
cid_concepts[4101] = {
    "DCM": [
        "StandardToftsModel",
        "ExtendedToftsModel",
        "FirstPassLeakageProfileFPLPModel",
        "ShutterSpeedModelSSM",
        "GammaCapillaryTransitTimeGCTTModel",
        "AdiabaticTissueHomogeneityATHModel",
        "TwoCompartmentExchange2CXModel",
    ]
}

name_for_cid[4102] = "PerfusionMeasurementMethod"
cid_concepts[4102] = {
    "DCM": [
        "PerfusionAnalysisByStableXenonCTTechnique",
        "PerfusionAnalysisByIVIodinatedContrastCTTechnique",
        "PerfusionAnalysisByArterialSpinLabelingMRTechnique",
        "PerfusionAnalysisBySusceptibilityMRTechnique",
    ]
}

name_for_cid[4103] = "ArterialInputFunctionMeasurementMethod"
cid_concepts[4103] = {
    "DCM": [
        "UserDefinedAIFROI",
        "AutomaticallyDetectedAIFROI",
        "BlindEstimationOfAIF",
        "AIFIgnored",
        "PopulationAveragedAIF",
    ]
}

name_for_cid[4104] = "BolusArrivalTimeDerivationMethod"
cid_concepts[4104] = {
    "DCM": [
        "TemporalDerivativeExceedsThreshold",
        "TimeOfPeakConcentration",
        "TimeOfLeadingHalfPeakConcentration",
    ]
}

name_for_cid[4105] = "PerfusionAnalysisMethod"
cid_concepts[4105] = {
    "DCM": [
        "LeastMeanSquareLMSDeconvolution",
        "SingularValueDecompositionSVDDeconvolution",
    ]
}

name_for_cid[4106] = "QuantitativeMethodUsedforPerfusionandTracerKineticModel"
cid_concepts[4106] = {
    "DCM": [
        "T1ByInversionRecovery",
        "TemporalDerivativeExceedsThreshold",
        "T1ByFixedValue",
        "LeastMeanSquareLMSDeconvolution",
        "SingularValueDecompositionSVDDeconvolution",
        "TimeOfPeakConcentration",
        "T1ByMultipleFlipAngles",
        "TimeOfLeadingHalfPeakConcentration",
        "StandardToftsModel",
        "UserDefinedAIFROI",
        "ExtendedToftsModel",
        "AutomaticallyDetectedAIFROI",
        "ModelFreeConcentrationTimeQuantitification",
        "BlindEstimationOfAIF",
        "FirstPassLeakageProfileFPLPModel",
        "ShutterSpeedModelSSM",
        "PerfusionAnalysisByStableXenonCTTechnique",
        "GammaCapillaryTransitTimeGCTTModel",
        "PerfusionAnalysisByIVIodinatedContrastCTTechnique",
        "AdiabaticTissueHomogeneityATHModel",
        "PerfusionAnalysisByArterialSpinLabelingMRTechnique",
        "TwoCompartmentExchange2CXModel",
        "PerfusionAnalysisBySusceptibilityMRTechnique",
        "AIFIgnored",
        "PopulationAveragedAIF",
    ]
}

name_for_cid[4107] = "TracerKineticModelParameter"
cid_concepts[4107] = {"DCM": ["TauM", "Vp", "Ktrans", "Kep", "Ve"]}

name_for_cid[4108] = "PerfusionModelParameter"
cid_concepts[4108] = {
    "DCM": [
        "MeanTransitTime",
        "RelativeRegionalBloodFlow",
        "RelativeRegionalBloodVolume",
        "TimeToPeak",
        "AbsoluteRegionalBloodFlow",
        "AbsoluteRegionalBloodVolume",
        "OxygenExtractionFraction",
        "Tmax",
    ]
}

name_for_cid[4109] = "ModelIndependentDynamicContrastAnalysisParameter"
cid_concepts[4109] = {
    "DCM": [
        "TimeToPeak",
        "IAUC90BN",
        "IAUC180BN",
        "TemporalDerivativeThreshold",
        "IAUC",
        "MaximumSlope",
        "IAUC60",
        "MaximumDifference",
        "IAUC90",
        "TracerConcentration",
        "IAUC180",
        "IAUCBN",
        "IAUC60BN",
        "TimeOfPeakConcentration",
        "BolusArrivalTime",
        "TimeOfLeadingHalfPeakConcentration",
    ]
}

name_for_cid[4110] = "TracerKineticModelingCovariate"
cid_concepts[4110] = {"LN": ["Hematocrit"]}

name_for_cid[4111] = "ContrastCharacteristic"
cid_concepts[4111] = {"DCM": ["ContrastLongitudinalRelaxivity"]}

name_for_cid[4200] = "OphthalmicImagingAgent"
cid_concepts[4200] = {
    "SCT": [
        "IndocyanineGreen",
        "RoseBengalContainingProduct",
        "Fluorescein",
        "TrypanBlue",
        "MethylthioniniumChlorideContainingProduct",
    ]
}

name_for_cid[4201] = "PatientEyeMovementCommand"
cid_concepts[4201] = {
    "SCT": [
        "PrimaryGaze",
        "RightGaze",
        "ConvergentGaze",
        "LeftGaze",
        "LeftDowngaze",
        "LeftUpgaze",
        "RightDowngaze",
        "UpwardGaze",
        "RightUpgaze",
        "Downgaze",
    ]
}

name_for_cid[4202] = "OphthalmicPhotographyAcquisitionDevice"
cid_concepts[4202] = {
    "SCT": [
        "FundusCamera",
        "SlitLampBiomicroscope",
        "DirectOphthalmoscope",
        "OphthalmicEndoscope",
        "ScanningLaserOphthalmoscope",
        "Keratoscope",
        "ExternalCamera",
        "Pupillograph",
        "IndirectOphthalmoscope",
        "SpecularMicroscope",
        "OperatingMicroscope",
    ]
}

name_for_cid[4203] = "OphthalmicPhotographyIllumination"
cid_concepts[4203] = {
    "DCM": [
        "ScotopicLight",
        "MesopicLight",
        "PhotopicLight",
        "DynamicLight",
        "DiffuseDirectIllumination",
    ],
    "SCT": [
        "BroadTangentialDirectIllumination",
        "DualDiffuseDirectIllumination",
        "IndirectRetroilluminationFromTheIris",
        "IndirectRetroilluminationFromTheRetina",
        "FineSlitBeamDirectIllumination",
        "IndirectScleroticScatterIllumination",
        "IndirectIrisTransillumination",
    ],
}

name_for_cid[4204] = "OphthalmicFilter"
cid_concepts[4204] = {
    "DCM": ["NoFilter"],
    "SCT": [
        "BlueOpticalFilter",
        "PolarizingOpticalFilter",
        "YellowGreenOpticalFilter",
        "GreenOpticalFilter",
        "BlueGreenOpticalFilter",
        "InfraredOpticalFilter",
        "RedOpticalFilter",
    ],
}

name_for_cid[4205] = "OphthalmicLens"
cid_concepts[4205] = {
    "SCT": [
        "IndirectOphthalmoscopyLens",
        "ContactFundusLens",
        "ConcaveNoncontactFundusLens",
        "ConcaveContactFundusLens",
        "Goniolens",
        "NoncontactFundusLens",
        "ConvexContactFundusLens",
        "ConvexNoncontactFundusLens",
    ]
}

name_for_cid[4206] = "OphthalmicChannelDescription"
cid_concepts[4206] = {
    "SCT": [
        "Infrared",
        "Ultraviolet",
        "Red",
        "FullSpectrum",
        "Blue",
        "RedFree",
        "Green",
    ]
}

name_for_cid[4207] = "OphthalmicImagePosition"
cid_concepts[4207] = {
    "DCM": [
        "LesionCentered",
        "DiscMaculaCentered",
        "MidPeripheralSuperior",
        "MidPeripheralSuperiorTemporal",
        "MidPeripheralTemporal",
        "MidPeripheralInferiorTemporal",
        "MidPeripheralInferior",
        "MidPeripheralInferiorNasal",
        "Field1ForJoslin3Field",
        "Field2ForJoslin3Field",
        "Field3ForJoslin3Field",
        "MaculaCentered",
        "DiscCentered",
        "PeripheralSuperiorTemporal",
        "PeripheralTemporal",
        "PeripheralInferiorTemporal",
        "PeripheralInferior",
        "PeripheralInferiorNasal",
        "PeripheralNasal",
        "PeripheralSuperiorNasal",
        "MidPeripheralNasal",
        "MidPeripheralSuperiorNasal",
        "PeripheralSuperior",
    ],
    "SCT": [
        "DiabeticRetinopathyStudyField6",
        "DiabeticRetinopathyStudyField4",
        "DiabeticRetinopathyStudyField2",
        "DiabeticRetinopathyStudyField1",
        "DiabeticRetinopathyStudyField7",
        "DiabeticRetinopathyStudyField5",
        "DiabeticRetinopathyStudyField3",
    ],
}

name_for_cid[4208] = "MydriaticAgent"
cid_concepts[4208] = {
    "SCT": [
        "Tropicamide",
        "Homatropine",
        "Cyclopentolate",
        "AtropineInOcularDoseForm",
        "Phenylephrine",
    ]
}

name_for_cid[4209] = "OphthalmicAnatomicStructureImaged"
cid_concepts[4209] = {
    "SCT": [
        "BothEyes",
        "AnteriorChamberOfEye",
        "OphthalmicArtery",
        "LowerEyelid",
        "OpticNerveHead",
        "ChoroidOfEye",
        "Sclera",
        "Cornea",
        "Eyelid",
        "Lens",
        "Eye",
        "LacrimalSac",
        "Retina",
        "LacrimalCaruncle",
        "Iris",
        "UpperEyelid",
        "CiliaryBody",
        "Conjunctiva",
        "LacrimalGland",
        "FoveaCentralis",
    ]
}

name_for_cid[4210] = "OphthalmicTomographyAcquisitionDevice"
cid_concepts[4210] = {
    "DCM": [
        "ReflectionBasedCornealTopographer",
        "InterferometryBasedCornealTomographer",
        "ScheimpflugCamera",
        "ElevationBasedCornealTomographer",
    ],
    "SCT": [
        "ScanningLaserPolarimeter",
        "OpticalCoherenceTomographyScanner",
        "RetinalThicknessAnalyzer",
        "ConfocalScanningLaserOphthalmoscope",
    ],
}

name_for_cid[4211] = "OphthalmicOCTAnatomicStructureImaged"
cid_concepts[4211] = {
    "SCT": [
        "AnteriorChamberOfEye",
        "CornealEndothelium",
        "Vitreous",
        "OpticNerveHead",
        "ChoroidOfEye",
        "Sclera",
        "Cornea",
        "Lens",
        "CornealEpithelium",
        "Retina",
        "Iris",
        "CiliaryBody",
        "Conjunctiva",
    ]
}

name_for_cid[4214] = "OphthalmicHorizontalDirection"
cid_concepts[4214] = {"SCT": ["Inward", "Outward"]}

name_for_cid[4215] = "OphthalmicVerticalDirection"
cid_concepts[4215] = {"SCT": ["Up", "Down"]}

name_for_cid[4216] = "OphthalmicVisualAcuityType"
cid_concepts[4216] = {
    "DCM": [
        "AutorefractionVisualAcuity",
        "HabitualVisualAcuity",
        "PrescriptionVisualAcuity",
    ],
    "SCT": [
        "BestCorrectedVisualAcuity",
        "PinholeVisualAcuity",
        "BrightnessAcuityTestingVisualAcuity",
        "PotentialAcuityMeterVisualAcuity",
        "UncorrectedVisualAcuity",
    ],
}

name_for_cid[4220] = "VisualFixationQualityDuringAcquisition"
cid_concepts[4220] = {"SCT": ["Steady", "Indeterminate", "NotSteady"]}

name_for_cid[4221] = "VisualFixationQualityProblem"
cid_concepts[4221] = {
    "DCM": ["EquipmentFailure", "OperatorError", "PatientMovement"],
    "SCT": ["EccentricFixation"],
}

name_for_cid[4222] = "OphthalmicMacularGridProblem"
cid_concepts[4222] = {
    "DCM": [
        "InterferingTearsOrDrops",
        "PatientPositioningProblem",
        "EquipmentFailure",
        "OperatorError",
        "PatientMovement",
    ],
    "SCT": [
        "DryEyesProblem",
        "PoorVisualFixation",
        "EccentricFixation",
        "CornealOpacity",
        "EyelidDisease",
        "VitreousOpacity",
        "LensOpacity",
        "RefractiveError",
        "ConstrictedPupil",
    ],
}

name_for_cid[4230] = "OphthalmicUltrasoundAxialMeasurementsType"
cid_concepts[4230] = {"DCM": ["UltrasoundContact", "UltrasoundImmersion"]}

name_for_cid[4231] = "LensStatus"
cid_concepts[4231] = {
    "SCT": [
        "ArtificialLensPresent",
        "CrystallineLens",
        "PiggybackIOL",
        "Aphakic",
        "PhakicIOL",
    ]
}

name_for_cid[4232] = "VitreousStatus"
cid_concepts[4232] = {
    "SCT": ["PostVitrectomy", "GasInVitreousCavity", "VitreousOnly", "SiliconeOil"]
}

name_for_cid[4233] = "OphthalmicAxialLengthMeasurementsSegmentName"
cid_concepts[4233] = {
    "DCM": ["SingleOrAnteriorLens", "PosteriorLens"],
    "SCT": ["AnteriorChamber", "VitreousCavity", "Cornea"],
}

name_for_cid[4234] = "RefractiveSurgeryType"
cid_concepts[4234] = {"DCM": ["SMILE"], "SCT": ["RK", "LASIK", "LASEK", "PRK"]}

name_for_cid[4235] = "KeratometryDescriptor"
cid_concepts[4235] = {
    "DCM": [
        "ManualKeratometry",
        "AutoKeratometry",
        "SimulatedKeratometry",
        "EquivalentKReading",
    ]
}

name_for_cid[4236] = "IOLCalculationFormula"
cid_concepts[4236] = {
    "DCM": [
        "Haigis",
        "HaigisL",
        "HaigisToric",
        "Holladay1",
        "HaigisLToric",
        "Holladay2",
        "BarrettToric",
        "HofferQ",
        "BarrettTrueK",
        "Olsen",
        "BarrettTrueKToric",
        "SRKII",
        "BarrettUniversalII",
        "SRKT",
    ]
}

name_for_cid[4237] = "LensConstantType"
cid_concepts[4237] = {
    "DCM": [
        "HaigisA1",
        "HaigisA2",
        "HofferPacdConstant",
        "SurgeonFactor",
        "BarrettLensFactor",
        "ACDConstant",
        "BarrettDesignFactor",
        "HaigisA0",
    ],
    "SCT": ["AConstant"],
}

name_for_cid[4238] = "RefractiveErrorType"
cid_concepts[4238] = {"SCT": ["Myopia", "Hyperopia"]}

name_for_cid[4239] = "AnteriorChamberDepthDefinition"
cid_concepts[4239] = {
    "DCM": ["FrontOfCorneaToFrontOfLens", "BackOfCorneaToFrontOfLens"]
}

name_for_cid[4240] = "OphthalmicMeasurementorCalculationDataSource"
cid_concepts[4240] = {
    "DCM": [
        "MeasurementFromThisDevice",
        "ExternalDataSource",
        "AxialMeasurementsSOPInstance",
        "RefractiveMeasurementsSOPInstance",
        "AutorefractionMeasurementsSOPInstance",
        "ManualEntry",
        "KeratometryMeasurementsSOPInstance",
    ]
}

name_for_cid[4241] = "OphthalmicAxialLengthSelectionMethod"
cid_concepts[4241] = {"DCM": ["MeanValueChosen", "UserChosenValue"]}

name_for_cid[4242] = "CorneaMeasurementMethodDescriptor"
cid_concepts[4242] = {
    "DCM": [
        "PosteriorCorneaSurfaceMeasurementMethod",
        "ManualKeratometry",
        "AutoKeratometry",
        "SimulatedKeratometry",
        "EquivalentKReading",
        "TotalCorneaPowerMeasurementMethod",
    ]
}

name_for_cid[4243] = "OphthalmicQualityMetricType"
cid_concepts[4243] = {
    "DCM": ["StandardDeviationOfMeasurementsUsed", "SignalToNoiseRatio"]
}

name_for_cid[4244] = "OphthalmicAgentConcentrationUnit"
cid_concepts[4244] = {"UCUM": ["Percent", "MilligramsPerMilliliter"]}

name_for_cid[4245] = "WideFieldOphthalmicPhotographyTransformationMethod"
cid_concepts[4245] = {"DCM": ["SphericalProjection", "SurfaceContourMapping"]}

name_for_cid[4250] = "VisualFieldStaticPerimetryTestPattern"
cid_concepts[4250] = {
    "DCM": [
        "VisualField60To4TestPattern",
        "VisualFieldCentralTestPattern",
        "VisualFieldMaculaTestPattern",
        "VisualFieldCentral40PointTestPattern",
        "VisualFieldCentral76PointTestPattern",
        "VisualFieldPeripheral60PointTestPattern",
        "VisualFieldFullField81PointTestPattern",
        "VisualFieldFullField120PointTestPattern",
        "VisualFieldGTestPattern",
        "VisualField24To2TestPattern",
        "VisualFieldMTestPattern",
        "VisualField10To2TestPattern",
        "VisualField07TestPattern",
        "VisualField30To2TestPattern",
        "VisualFieldLVCTestPattern",
    ]
}

name_for_cid[4251] = "VisualFieldStaticPerimetryTestStrategy"
cid_concepts[4251] = {
    "DCM": [
        "VisualFieldTOPTestStrategy",
        "VisualFieldCLIPTestStrategy",
        "VisualFieldSITAStandardTestStrategy",
        "VisualFieldDynamicTestStrategy",
        "VisualFieldCLASSStrategy",
        "VisualFieldSITASWAPTestStrategy",
        "VisualFieldNormalTestStrategy",
        "VisualFieldSITAFastTestStrategy",
        "VisualField1LTTestStrategy",
        "VisualFieldFullThresholdTestStrategy",
        "VisualField2LTTestStrategy",
        "VisualFieldFastpacTestStrategy",
        "VisualFieldLVSTestStrategy",
        "VisualFieldFullFromPriorTestStrategy",
        "VisualFieldGATETestStrategy",
        "VisualFieldOptimaTestStrategy",
        "VisualFieldGateiTestStrategy",
        "VisualFieldTwoZoneTestStrategy",
        "VisualField2LTDynamicTestStrategy",
        "VisualFieldThreeZoneTestStrategy",
        "VisualField2LTNormalTestStrategy",
        "VisualFieldQuantifyDefectsTestStrategy",
        "VisualFieldFastThresholdTestStrategy",
    ]
}

name_for_cid[4252] = "VisualFieldStaticPerimetryScreeningTestMode"
cid_concepts[4252] = {
    "DCM": [
        "AgeCorrected",
        "ThresholdRelated",
        "UserChosenValue",
        "SingleLuminance",
        "FovealSensitivityRelated",
        "RelatedToNonMacularSensitivity",
    ]
}

name_for_cid[4253] = "VisualFieldStaticPerimetryFixationStrategy"
cid_concepts[4253] = {
    "DCM": [
        "AutomatedOptical",
        "BlindSpotMonitoring",
        "MacularFixationTesting",
        "ObservationByExaminer",
    ],
    "SCT": ["None_"],
}

name_for_cid[4254] = "VisualFieldStaticPerimetryTestAnalysisResult"
cid_concepts[4254] = {
    "DCM": [
        "OutsideNormalLimits",
        "Borderline",
        "AbnormallyHighSensitivity",
        "GeneralReductionInSensitivity",
        "BorderlineAndGeneralReductionInSensitivity",
    ],
    "SCT": ["WithinNormalLimits"],
}

name_for_cid[4255] = "VisualFieldIlluminationColor"
cid_concepts[4255] = {"SCT": ["Yellow", "Red", "White", "Blue", "Green"]}

name_for_cid[4256] = "VisualFieldProcedureModifier"
cid_concepts[4256] = {"SCT": ["Diagnostic", "Screening"]}

name_for_cid[4257] = "VisualFieldGlobalIndexName"
cid_concepts[4257] = {
    "DCM": [
        "VisualFieldIndex",
        "VisualFieldLossDueToDiffuseDefect",
        "VisualFieldLossDueToLocalDefect",
        "GlaucomaHemifieldTestAnalysis",
        "OpticalFixationMeasurements",
    ]
}

name_for_cid[4260] = "OphthalmicMappingUnitforRealWorldValueMapping"
cid_concepts[4260] = {"UCUM": ["Micrometer"]}

name_for_cid[4261] = "OphthalmicMappingAcquisitionMethod"
cid_concepts[4261] = {
    "DCM": [
        "RetinalTopography",
        "TimeDomain",
        "SpectralDomain",
        "NoCornealCompensation",
        "CornealBirefringenceCompensation",
    ]
}

name_for_cid[4262] = "RetinalThicknessDefinition"
cid_concepts[4262] = {
    "DCM": [
        "RetinalNerveFiberLayerThickness",
        "GanglionCellComplexThickness",
        "TotalRetinalThicknessILMToISOS",
        "TotalRetinalThicknessILMToRPE",
        "TotalRetinalThicknessILMToBM",
    ]
}

name_for_cid[4263] = "OphthalmicThicknessMapValueType"
cid_concepts[4263] = {
    "DCM": [
        "AbsoluteOphthalmicThickness",
        "ThicknessDeviationCategoryFromNormativeData",
        "ThicknessDeviationFromNormativeData",
    ]
}

name_for_cid[4264] = "OphthalmicMapPurposeofReference"
cid_concepts[4264] = {
    "DCM": [
        "Localizer",
        "SourceImageForImageProcessingOperation",
        "RelatedOphthalmicThicknessMap",
    ]
}

name_for_cid[4265] = "OphthalmicThicknessDeviationCategory"
cid_concepts[4265] = {
    "DCM": [
        "PGreaterThan5Percent",
        "PLesserThan5Percent",
        "PLesserThan2Percent",
        "PLesserThan1Percent",
        "PLesserThan0Point5Percent",
    ]
}

name_for_cid[4266] = "OphthalmicAnatomicStructureReferencePoint"
cid_concepts[4266] = {
    "DCM": ["DiscFovea"],
    "SCT": [
        "Lesion",
        "OpticNerveHead",
        "MorphologicallyAbnormalStructure",
        "Cornea",
        "FoveaCentralis",
    ],
}

name_for_cid[4267] = "CornealTopographyMappingUnitforRealWorldValueMapping"
cid_concepts[4267] = {"UCUM": ["Millimeter", "Micrometer", "Diopters"]}

name_for_cid[4268] = "CornealTopographyMapValueType"
cid_concepts[4268] = {
    "DCM": [
        "CornealAxialPowerMap",
        "CornealInstantaneousPowerMap",
        "CornealRefractivePowerMap",
        "CornealElevationMap",
        "CornealWavefrontMap",
    ]
}

name_for_cid[4270] = "OCTAProcessingAlgorithmFamily"
cid_concepts[4270] = {
    "DCM": [
        "OCTACorrelationMapping",
        "OCTASpeckleVariance",
        "DopplerOCTA",
        "OCTAOneSidedRatioGreater",
        "OCTAOneSidedRatioLesser",
        "OCTAComplexVariance",
        "OCTAAmplitudeDecorrelation",
    ]
}

name_for_cid[4271] = "EnFaceImageType"
cid_concepts[4271] = {
    "DCM": [
        "SuperficialRetinaStructuralReflectanceMap",
        "WholeEyeVasculatureFlow",
        "SuperficialRetinaVasculatureFlow",
        "ChoroidStructuralReflectanceMap",
        "RetinaDepthEncodedVasculatureFlow",
        "MiddleInnerStructuralReflectanceMap",
        "MiddleInnerRetinaVasculatureFlow",
        "WholeEyeStructuralReflectanceMap",
        "RetinaVasculatureFlow",
        "RetinaDepthEncodedStructuralReflectanceMap",
        "DeepRetinaVasculatureFlow",
        "RetinaStructuralReflectanceMap",
        "OuterRetinaVasculatureFlow",
        "DeepRetinaStructuralReflectanceMap",
        "VitreousStructuralReflectanceMap",
        "ChoriocapillarisVasculatureFlow",
        "VitreousVasculatureFlow",
        "OuterRetinaStructuralReflectanceMap",
        "RadialPeripapillaryStructuralReflectanceMap",
        "ChoroidVasculatureFlow",
        "RadialPeripapillaryVasculatureFlow",
        "ChoriocapillarisStructuralReflectanceMap",
    ]
}

name_for_cid[4272] = "OPTScanPatternType"
cid_concepts[4272] = {
    "DCM": [
        "GridBScanPattern",
        "CircleRadialBScanPattern",
        "CubeBScanPattern",
        "RasterBScanPattern",
        "RadialBScanPattern",
        "LineBScanPattern",
        "CircleBScanPattern",
        "CrossBScanPattern",
        "CircleRasterBScanPattern",
        "ConcentricCircleBScanPattern",
    ]
}

name_for_cid[4273] = "RetinalSegmentationSurface"
cid_concepts[4273] = {
    "DCM": [
        "PosteriorSurfaceOfTheRPE",
        "SurfaceOfTheCenterOfTheRPE",
        "SurfaceOfTheChoroidScleraInterface",
        "OuterSurfaceOfRNFL",
        "OuterSurfaceOfTheBM",
        "OuterSurfaceOfTheCC",
        "OuterSurfaceOfIPL",
        "OuterSurfaceOfGCL",
        "OuterSurfaceOfOPL",
        "OuterSurfaceOfINL",
        "SurfaceBetweenInnerAndOuterSegmentsOfThePhotoreceptors",
        "OuterSurfaceOfHFL",
        "AnteriorSurfaceOfTheRPE",
        "SurfaceOfTheInterdigitatingZoneBetweenRetinaAndRPE",
    ],
    "SCT": ["ELMExternalLimitingMembrane", "ILMInternalLimitingMembrane"],
}

name_for_cid[4401] = "FitzpatrickSkinType"
cid_concepts[4401] = {
    "LN": [
        "FitzpatrickSkinTypeVI",
        "FitzpatrickSkinTypeII",
        "FitzpatrickSkinTypeIII",
        "FitzpatrickSkinTypeIV",
        "FitzpatrickSkinTypeV",
        "FitzpatrickSkinTypeI",
    ]
}

name_for_cid[4402] = "HistoryofMalignantMelanoma"
cid_concepts[4402] = {
    "SCT": ["HistoryOfMalignantMelanomaOfTheSkin", "HistoryOfMalignantMelanoma"]
}

name_for_cid[4403] = "HistoryofMelanomainSitu"
cid_concepts[4403] = {"SCT": ["HistoryOfMelanomaInSituOfTheSkin"]}

name_for_cid[4404] = "HistoryofNonMelanomaSkinCancer"
cid_concepts[4404] = {
    "SCT": [
        "HistoryOfSquamousCellCarcinomaOfSkin",
        "HistoryOfMalignantBasalCellNeoplasmOfSkin",
        "HistoryOfMalignantNeoplasmOfSkinExcludingMelanoma",
    ]
}

name_for_cid[4405] = "SkinDisorder"
cid_concepts[4405] = {
    "SCT": [
        "AtypicalMoleSyndrome",
        "GorlinSyndrome",
        "LupusErythematosus",
        "Eczema",
        "SolarDegeneration",
        "TelangiectasiaCutaneousCancerSyndromeFamilial",
        "Psoriasis",
        "PTENHamartomaTumorSyndrome",
        "RomboSyndrome",
        "ActinicKeratosis",
        "Rosacea",
        "AtopicDermatitis",
    ]
}

name_for_cid[4406] = "PatientReportedLesionCharacteristic"
cid_concepts[4406] = {
    "SCT": [
        "Peeling",
        "BleedingSkin",
        "PainfulSkin",
        "SymptomHasChanged",
        "Itching",
        "Erythema",
    ]
}

name_for_cid[4407] = "LesionPalpationFinding"
cid_concepts[4407] = {
    "DCM": ["RaisedSkinLesion", "FirmSkinLesion"],
    "UMLS": ["MobileSkinLesion"],
}

name_for_cid[4408] = "LesionVisualFinding"
cid_concepts[4408] = {"SCT": ["BleedingSkin", "Erythema"]}

name_for_cid[4409] = "SkinProcedure"
cid_concepts[4409] = {
    "SCT": [
        "ExcisionOfSkin",
        "PhotodynamicTherapyOfSkin",
        "RadiofrequencyAblation",
        "BiopsyOfSkin",
        "CryotherapyToSkinLesion",
        "TopicalChemotherapyForMalignantNeoplasm",
        "LaserProcedureOnSkin",
    ]
}

name_for_cid[4410] = "TopicalTreatment"
cid_concepts[4410] = {
    "SCT": [
        "Antifungal",
        "Tretinoin",
        "Steroid",
        "Immunomodulator",
        "Antibiotic",
        "AluminumHydroxide",
        "KeratolyticAgent",
        "HaemostaticAgent",
        "CytotoxicAgent",
        "AscorbicAcid",
    ]
}

name_for_cid[4411] = "LesionColor"
cid_concepts[4411] = {
    "SCT": [
        "Black",
        "Purple",
        "Orange",
        "Brown",
        "Yellow",
        "Red",
        "Gray",
        "White",
        "Blue",
        "Pink",
    ]
}

name_for_cid[4412] = "SpecimenStainforConfocalMicroscopy"
cid_concepts[4412] = {
    "SCT": [
        "NileBlueStain",
        "AcridineOrangeStain",
        "MethylBlueStain",
        "ToluidineBlueStain",
        "PatentBlueVSodiumSaltStain",
        "AceticAcid",
        "ImmunofluorescentStain",
        "AluminumChloride",
        "CitricAcid",
        "FluoresceinStain",
    ]
}

name_for_cid[6000] = "OverallBreastComposition"
cid_concepts[6000] = {
    "SCT": [
        "ExtremelyDense",
        "AlmostEntirelyFat",
        "HeterogeneouslyDense",
        "ScatteredFibroglandularDensities",
    ]
}

name_for_cid[6001] = "OverallBreastCompositionfromBIRADS®"
cid_concepts[6001] = {
    "SCT": [
        "ExtremelyDense",
        "AlmostEntirelyFat",
        "HeterogeneouslyDense",
        "ScatteredFibroglandularDensities",
    ]
}

name_for_cid[6002] = "ChangeSinceLastMammogramorPriorSurgery"
cid_concepts[6002] = {
    "SCT": [
        "RemovalOfImplantSincePreviousMammogram",
        "ImplantRevisedSincePreviousMammogram",
        "NoSignificantChangesInTheFinding",
        "IncreaseInSize",
        "NewFinding",
        "IncreaseInNumberOfCalcifications",
        "DecreaseInNumberOfCalcifications",
        "LessDefined",
        "FindingPartiallyRemoved",
        "MoreDefined",
        "DecreaseInSize",
    ]
}

name_for_cid[6003] = "ChangeSinceLastMammogramorPriorSurgeryfromBIRADS®"
cid_concepts[6003] = {
    "SCT": [
        "RemovalOfImplantSincePreviousMammogram",
        "ImplantRevisedSincePreviousMammogram",
        "NoSignificantChangesInTheFinding",
        "IncreaseInSize",
        "NewFinding",
        "IncreaseInNumberOfCalcifications",
        "DecreaseInNumberOfCalcifications",
        "LessDefined",
        "FindingPartiallyRemoved",
        "MoreDefined",
        "DecreaseInSize",
    ]
}

name_for_cid[6004] = "MammographyShapeCharacteristic"
cid_concepts[6004] = {"SCT": ["Lobular", "RoundShape", "Irregular", "OvoidShapeOval"]}

name_for_cid[6005] = "ShapeCharacteristicfromBIRADS®"
cid_concepts[6005] = {"SCT": ["Lobular", "RoundShape", "Irregular", "OvoidShapeOval"]}

name_for_cid[6006] = "MammographyMarginCharacteristic"
cid_concepts[6006] = {
    "DCM": ["AngularMargins"],
    "SCT": [
        "SpiculatedLesion",
        "CircumscribedLesion",
        "ObscuredLesion",
        "IndistinctLesion",
        "MicrolobulatedLesion",
    ],
}

name_for_cid[6007] = "MarginCharacteristicfromBIRADS®"
cid_concepts[6007] = {
    "DCM": ["AngularMargins"],
    "SCT": [
        "SpiculatedLesion",
        "CircumscribedLesion",
        "ObscuredLesion",
        "IndistinctLesion",
        "MicrolobulatedLesion",
    ],
}

name_for_cid[6008] = "DensityModifier"
cid_concepts[6008] = {
    "SCT": [
        "LowDensityNotContainingFatLesion",
        "HighDensityLesion",
        "EqualDensityIsodenseLesion",
        "FatContainingRadiolucentLesion",
    ]
}

name_for_cid[6009] = "DensityModifierfromBIRADS®"
cid_concepts[6009] = {
    "SCT": [
        "LowDensityNotContainingFatLesion",
        "HighDensityLesion",
        "EqualDensityIsodenseLesion",
        "FatContainingRadiolucentLesion",
    ]
}

name_for_cid[6010] = "MammographyCalcificationType"
cid_concepts[6010] = {
    "DCM": ["FinePleomorphicCalcification", "Macrocalcifications"],
    "SCT": [
        "CalcifiedSkinOfBreast",
        "FineLinearCastingCalcification",
        "CalcifiedSutureMaterial",
        "AmorphousCalcification",
        "VascularCalcificationRadiographicFinding",
        "RoundShapedCalcification",
        "LucentCenteredCalcification",
        "PunctateCalcification",
        "MilkOfCalciumCalcification",
        "DystrophicCalcification",
        "EggshellCalcification",
        "LargeRodLikeCalcification",
        "FineLinearBranchingCastingCalcification",
        "MicrocalcificationsOfTheBreast",
        "CoarsePopcornLikeCalcification",
        "HeterogeneousCalcification",
    ],
}

name_for_cid[6011] = "CalcificationTypefromBIRADS®"
cid_concepts[6011] = {
    "DCM": ["FinePleomorphicCalcification", "Macrocalcifications"],
    "SCT": [
        "CalcifiedSkinOfBreast",
        "FineLinearCastingCalcification",
        "CalcifiedSutureMaterial",
        "AmorphousCalcification",
        "VascularCalcificationRadiographicFinding",
        "RoundShapedCalcification",
        "LucentCenteredCalcification",
        "PunctateCalcification",
        "MilkOfCalciumCalcification",
        "DystrophicCalcification",
        "EggshellCalcification",
        "LargeRodLikeCalcification",
        "FineLinearBranchingCastingCalcification",
        "MicrocalcificationsOfTheBreast",
        "CoarsePopcornLikeCalcification",
        "HeterogeneousCalcification",
    ],
}

name_for_cid[6012] = "CalcificationDistributionModifier"
cid_concepts[6012] = {
    "DCM": ["CalcificationsWithinAMass", "CalcificationsOutsideOfAMass"],
    "SCT": [
        "SegmentalCalcificationDistribution",
        "LinearCalcificationDistribution",
        "RegionalCalcificationDistribution",
        "DiffuseCalcificationDistribution",
        "GroupedCalcificationDistribution",
    ],
}

name_for_cid[6013] = "CalcificationDistributionModifierfromBIRADS®"
cid_concepts[6013] = {
    "DCM": ["CalcificationsWithinAMass", "CalcificationsOutsideOfAMass"],
    "SCT": [
        "SegmentalCalcificationDistribution",
        "LinearCalcificationDistribution",
        "RegionalCalcificationDistribution",
        "DiffuseCalcificationDistribution",
        "GroupedCalcificationDistribution",
    ],
}

name_for_cid[6014] = "MammographySingleImageFinding"
cid_concepts[6014] = {
    "DCM": [
        "BreastGeometry",
        "CooperLigamentChanges",
        "ImageQuality",
        "MassInTheSkin",
        "SelectedRegion",
        "NonLesion",
        "MassOnTheSkin",
    ],
    "SCT": [
        "BreastComposition",
        "AxillaryAdenopathy",
        "Edema",
        "AxillaryLymphNode",
        "TrabecularThickeningOfBreast",
        "SkinRetractionOfBreast",
        "ArchitecturalDistortionOfBreast",
        "TubularDensity",
        "MammographyBreastDensity",
        "SkinLesion",
        "IndividualCalcification",
        "Nipple",
        "SkinThickeningOfBreast",
        "CalcificationCluster",
        "IntramammaryLymphNode",
    ],
}

name_for_cid[6015] = "SingleImageFindingfromBIRADS®"
cid_concepts[6015] = {
    "DCM": ["CooperLigamentChanges", "MassInTheSkin", "MassOnTheSkin"],
    "SCT": [
        "BreastComposition",
        "AxillaryAdenopathy",
        "Edema",
        "AxillaryLymphNode",
        "TrabecularThickeningOfBreast",
        "SkinRetractionOfBreast",
        "ArchitecturalDistortionOfBreast",
        "TubularDensity",
        "MammographyBreastDensity",
        "SkinLesion",
        "IndividualCalcification",
        "SkinThickeningOfBreast",
        "CalcificationCluster",
        "IntramammaryLymphNode",
    ],
}

name_for_cid[6016] = "MammographyCompositeFeature"
cid_concepts[6016] = {
    "DCM": [
        "BreastGeometry",
        "ImageQuality",
        "NonLesion",
        "CooperLigamentChanges",
        "MassInTheSkin",
        "SelectedRegion",
        "MassOnTheSkin",
        "MassWithCalcifications",
    ],
    "SCT": [
        "BreastComposition",
        "AxillaryLymphNode",
        "AsymmetricBreastTissue",
        "SkinRetractionOfBreast",
        "ArchitecturalDistortionOfBreast",
        "TubularDensity",
        "MammographyBreastDensity",
        "SkinLesion",
        "SkinThickeningOfBreast",
        "FocalAsymmetricBreastTissue",
        "AxillaryAdenopathy",
        "Edema",
        "TrabecularThickeningOfBreast",
        "IndividualCalcification",
        "Nipple",
        "MammographicBreastMass",
        "CalcificationCluster",
        "IntramammaryLymphNode",
    ],
}

name_for_cid[6017] = "CompositeFeaturefromBIRADS®"
cid_concepts[6017] = {
    "SCT": [
        "FocalAsymmetricBreastTissue",
        "MammographicBreastMass",
        "AsymmetricBreastTissue",
    ]
}

name_for_cid[6018] = "ClockfaceLocationorRegion"
cid_concepts[6018] = {
    "SCT": [
        "_7OClockPosition",
        "_11OClockPosition",
        "_12OClockPosition",
        "ChestWall",
        "_5OClockPosition",
        "_3OClockPosition",
        "_6OClockPosition",
        "_4OClockPosition",
        "AxillaRegion",
        "_1OClockPosition",
        "_8OClockPosition",
        "_2OClockPosition",
        "CentralRegionOfBreast",
        "AxillaryTailRegion",
        "SubareolarRegion",
        "_9OClockPosition",
        "_10OClockPosition",
    ]
}

name_for_cid[6019] = "ClockfaceLocationorRegionfromBIRADS®"
cid_concepts[6019] = {
    "SCT": [
        "_7OClockPosition",
        "_11OClockPosition",
        "_12OClockPosition",
        "_5OClockPosition",
        "_3OClockPosition",
        "_6OClockPosition",
        "_4OClockPosition",
        "AxillaRegion",
        "_1OClockPosition",
        "_8OClockPosition",
        "_2OClockPosition",
        "CentralRegionOfBreast",
        "AxillaryTailRegion",
        "SubareolarRegion",
        "_9OClockPosition",
        "_10OClockPosition",
    ]
}

name_for_cid[6020] = "QuadrantLocation"
cid_concepts[6020] = {
    "SCT": [
        "UpperOuterQuadrantOfBreast",
        "LowerInnerQuadrantOfBreast",
        "LowerOuterQuadrantOfBreast",
        "UpperInnerQuadrantOfBreast",
    ]
}

name_for_cid[6021] = "QuadrantLocationfromBIRADS®"
cid_concepts[6021] = {
    "SCT": [
        "UpperOuterQuadrantOfBreast",
        "LowerInnerQuadrantOfBreast",
        "LowerOuterQuadrantOfBreast",
        "UpperInnerQuadrantOfBreast",
    ]
}

name_for_cid[6022] = "Side"
cid_concepts[6022] = {"SCT": ["LeftBreast", "RightBreast", "BothBreasts"]}

name_for_cid[6023] = "SidefromBIRADS®"
cid_concepts[6023] = {"SCT": ["LeftBreast", "RightBreast", "BothBreasts"]}

name_for_cid[6024] = "Depth"
cid_concepts[6024] = {"NCIt": ["Middle"], "SCT": ["Posterior", "Anterior"]}

name_for_cid[6025] = "DepthfromBIRADS®"
cid_concepts[6025] = {"NCIt": ["Middle"], "SCT": ["Posterior", "Anterior"]}

name_for_cid[6026] = "MammographyAssessment"
cid_concepts[6026] = {"DCM": ["PostProcedureMammogramsForMarkerPlacement"]}

name_for_cid[6027] = "AssessmentfromBIRADS®"
cid_concepts[6027] = {}

name_for_cid[6028] = "MammographyRecommendedFollowup"
cid_concepts[6028] = {
    "DCM": [
        "NormalIntervalFollowUp",
        "AnyDecisionToBiopsyShouldBeBasedOnClinicalAssessment",
        "FollowUpAtShortInterval111Months",
        "FollowUpPostBiopsyAsDirectedByClinician",
        "BiopsyShouldBeConsidered",
        "NeedleLocalizationAndBiopsy",
        "KnownBiopsyProvenMalignancyTakeAppropriateAction",
        "HistologyUsingCoreBiopsy",
        "SurgicalConsult",
        "AdditionalProjections",
        "SuggestiveOfMalignancyTakeAppropriateAction",
        "SpotMagnificationViews",
        "CytologicAnalysis",
        "BiopsyShouldBeStronglyConsidered",
        "OldFilmsForComparison",
        "HighlySuggestiveOfMalignancyTakeAppropriateAction",
    ],
    "SCT": [
        "SpotCompression",
        "NuclearMedicineProcedure",
        "MRIOfBreast",
        "DiagnosticUltrasonography",
        "MagnificationViews",
        "EvaluationProcedure",
        "MammaryDuctogram",
    ],
}

name_for_cid[6029] = "RecommendedFollowupfromBIRADS®"
cid_concepts[6029] = {
    "DCM": [
        "NormalIntervalFollowUp",
        "AnyDecisionToBiopsyShouldBeBasedOnClinicalAssessment",
        "FollowUpAtShortInterval111Months",
        "BiopsyShouldBeConsidered",
        "NeedleLocalizationAndBiopsy",
        "KnownBiopsyProvenMalignancyTakeAppropriateAction",
        "HistologyUsingCoreBiopsy",
        "AdditionalProjections",
        "SuggestiveOfMalignancyTakeAppropriateAction",
        "SpotMagnificationViews",
        "CytologicAnalysis",
        "BiopsyShouldBeStronglyConsidered",
        "OldFilmsForComparison",
        "HighlySuggestiveOfMalignancyTakeAppropriateAction",
    ],
    "SCT": [
        "SpotCompression",
        "MRIOfBreast",
        "DiagnosticUltrasonography",
        "MagnificationViews",
        "MammaryDuctogram",
    ],
}

name_for_cid[6030] = "MammographyPathologyCode"
cid_concepts[6030] = {
    "DCM": [
        "FibroadenomatoidHyperplasia",
        "Fibroadenolipoma",
        "CartilaginousAndOsseousChange",
        "AxillaryNodeWithLymphoma",
        "AxillaryNodalMetastases",
        "BloodVesselVascularInvasion",
        "CarcinomaInChildren",
        "CarcinomaInEctopicBreast",
        "CarcinomaWithEndocrineDifferentiation",
        "BasalCellCarcinomaOfNipple",
        "NormalAxillaryNode",
        "AxillaryNodeWithCalcifications",
        "AxillaryNodeHyperplasia",
        "BenignCystWithBlood",
        "BenignCalcifications",
        "DuctalAdenoma",
        "DiabeticFibrousMastopathy",
        "SquamousCellCarcinomaOfTheNipple",
        "IntraductalCarcinomaHighGrade",
        "PseudoangiomatousStromalHyperplasia",
        "SiliconeGranuloma",
        "MetastaticDiseaseToAxillaryNode",
        "VirginalHyperplasia",
        "PeripheralDuctPapillomas",
        "MultifocalInvasiveDuctalCarcinoma",
        "OilCyst",
        "PostReductionMammoplasty",
        "MetastasisToAnIntramammaryLymphNode",
        "MalignantMelanomaOfNipple",
        "RecurrentMalignancy",
        "LargeDuctPapilloma",
        "MicroglandularAdenosis",
        "MultipleIntraductalPapillomas",
        "NoAbnormality",
        "NormalBreastTissue",
        "LympathicVesselInvasion",
        "OccultCarcinomaPresentingWithAxillaryLymphNodeMetastases",
        "MetastaticCancerToTheBreast",
        "MetastaticCancerToTheBreastFromTheColon",
        "MetastaticCancerToTheBreastFromTheLung",
        "MetastaticMelanomaToTheBreast",
        "MetastaticCancerToTheBreastFromTheOvary",
        "MetastaticSarcomaToTheBreast",
        "MultifocalIntraductalCarcinoma",
        "JuvenilePapillomatosis",
        "CarcinomaInPregnancyAndLactation",
        "LactationalChange",
        "IntraductalComedocarcinomaWithNecrosis",
        "IntraductalCarcinomaLowGrade",
        "IntracysticPapillaryCarcinoma",
        "InvasiveAndInSituCarcinoma",
        "LeukemicInfiltration",
    ],
    "SCT": [
        "LactatingAdenoma",
        "Leiomyoma",
        "Carcinosarcoma",
        "TubularAdenocarcinoma",
        "CystOfBreast",
        "FatNecrosisOfBreast",
        "NonHodgkinLymphoma",
        "HemangiomaOfSubcutaneousTissue",
        "LymphNode",
        "Plasmacytoma",
        "BreastLobularHyperplasia",
        "CarcinomaInSituOfMaleBreast",
        "Hemangiopericytoma",
        "Adenoma",
        "EpidermalInclusionCyst",
        "Galactocele",
        "Lipoma",
        "ThrombophlebitisOfBreastMondorDisease",
        "ScarTissue",
        "HodgkinLymphoma",
        "Lymphoma",
        "LobularCarcinomaInSituOfBreast",
        "DuctalHyperplasiaUsual",
        "Angiosarcoma",
        "Neurofibroma",
        "Fibroadenoma",
        "Gynecomastia",
        "GiantFibroadenoma",
        "MalignantFibrousHistiocytoma",
        "ApocrineMetaplasia",
        "Fibrosarcoma",
        "SignetRingCellCarcinoma",
        "Hematoma",
        "Amyloid",
        "PhyllodesTumor",
        "Papilloma",
        "CarcinomaWithMetaplasia",
        "PleomorphicAdenoma",
        "IntraductalCarcinomaMicroPapillary",
        "SquamousCellCarcinoma",
        "PagetDiseaseMammary",
        "Adenomyoepithelioma",
        "FocalFibrosis",
        "InfiltratingDuctCarcinoma",
        "SpindleCellNodule",
        "Abscess",
        "SclerosingAdenosis",
        "Adenolipoma",
        "Comedocarcinoma",
        "MammaryDuctEctasia",
        "Chondroma",
        "Osteosarcoma",
        "InfarctionOfBreast",
        "ApocrineAdenocarcinoma",
        "Hemangioma",
        "PapillaryCarcinoma",
        "RadialScar",
        "InflammatoryCarcinoma",
        "HemangiomaVenous",
        "LipidRichLipidSecretingCarcinoma",
        "Leiomyosarcoma",
        "GranularCellTumor",
        "Angiolipoma",
        "ForeignBodyGiantCellGranuloma",
        "HyperplasiaUsual",
        "InvasiveCribriformCarcinoma",
        "AdenoidCysticCarcinoma",
        "Angiomatosis",
        "Adenosis",
        "Seroma",
        "MedullaryCarcinoma",
        "AtypicalIntraductalHyperplasia",
        "ExtraAbdominalDesmoid",
        "Fibromatosis",
        "Inflammation",
        "IntramammaryLymphNode",
        "AtypicalLobularHyperplasia",
        "Liposarcoma",
        "IntraductalPapilloma",
        "NeoplasmOfTheMammarySkin",
        "PhyllodesTumorMalignant",
        "GlycogenRichCarcinoma",
        "TubularAdenoma",
        "SecretoryJuvenileCarcinomaOfTheBreast",
        "Neurofibromatosis",
        "MucinousAdenocarcinomaColloidCarcinoma",
        "IntracysticPapilloma",
        "Myofibroblastoma",
        "Chondrosarcoma",
        "PapillaryCarcinomaInSitu",
        "JuvenileFibroadenoma",
        "EctopicAccessoryBreastTissue",
        "Edema",
        "Hamartoma",
        "AsynchronousInvolutionOfBreast",
        "FibrocysticDiseaseOfBreast",
        "BenignNeoplasmOfNippleOfFemaleBreastNippleAdenoma",
        "InvasiveLobularCarcinoma",
    ],
}

name_for_cid[6031] = "BenignPathologyCodefromBIRADS®"
cid_concepts[6031] = {
    "DCM": [
        "FibroadenomatoidHyperplasia",
        "Fibroadenolipoma",
        "PseudoangiomatousStromalHyperplasia",
        "NormalAxillaryNode",
        "AxillaryNodeWithCalcifications",
        "SiliconeGranuloma",
        "AxillaryNodeHyperplasia",
        "VirginalHyperplasia",
        "BenignCystWithBlood",
        "OilCyst",
        "PostReductionMammoplasty",
        "BenignCalcifications",
        "DuctalAdenoma",
        "DiabeticFibrousMastopathy",
        "LargeDuctPapilloma",
        "MicroglandularAdenosis",
        "MultipleIntraductalPapillomas",
        "NoAbnormality",
        "NormalBreastTissue",
        "JuvenilePapillomatosis",
        "LactationalChange",
    ],
    "SCT": [
        "LactatingAdenoma",
        "Leiomyoma",
        "CystOfBreast",
        "FatNecrosisOfBreast",
        "InfarctionOfBreast",
        "Hemangioma",
        "RadialScar",
        "HemangiomaOfSubcutaneousTissue",
        "LymphNode",
        "HemangiomaVenous",
        "BreastLobularHyperplasia",
        "Adenoma",
        "GranularCellTumor",
        "Angiolipoma",
        "EpidermalInclusionCyst",
        "Galactocele",
        "ForeignBodyGiantCellGranuloma",
        "HyperplasiaUsual",
        "Lipoma",
        "ThrombophlebitisOfBreastMondorDisease",
        "ScarTissue",
        "Angiomatosis",
        "Adenosis",
        "DuctalHyperplasiaUsual",
        "Neurofibroma",
        "Fibroadenoma",
        "Seroma",
        "Gynecomastia",
        "ExtraAbdominalDesmoid",
        "GiantFibroadenoma",
        "Fibromatosis",
        "Inflammation",
        "IntramammaryLymphNode",
        "ApocrineMetaplasia",
        "IntraductalPapilloma",
        "Hematoma",
        "Amyloid",
        "TubularAdenoma",
        "Papilloma",
        "Neurofibromatosis",
        "PleomorphicAdenoma",
        "Adenomyoepithelioma",
        "FocalFibrosis",
        "IntracysticPapilloma",
        "Myofibroblastoma",
        "JuvenileFibroadenoma",
        "EctopicAccessoryBreastTissue",
        "Abscess",
        "SclerosingAdenosis",
        "Edema",
        "Adenolipoma",
        "Hamartoma",
        "AsynchronousInvolutionOfBreast",
        "FibrocysticDiseaseOfBreast",
        "MammaryDuctEctasia",
        "Chondroma",
        "BenignNeoplasmOfNippleOfFemaleBreastNippleAdenoma",
    ],
}

name_for_cid[6032] = "HighRiskLesionPathologyCodefromBIRADS®"
cid_concepts[6032] = {
    "DCM": ["PeripheralDuctPapillomas"],
    "SCT": [
        "AtypicalLobularHyperplasia",
        "AtypicalIntraductalHyperplasia",
        "PhyllodesTumor",
        "LobularCarcinomaInSituOfBreast",
    ],
}

name_for_cid[6033] = "MalignantPathologyCodefromBIRADS®"
cid_concepts[6033] = {
    "DCM": [
        "SquamousCellCarcinomaOfTheNipple",
        "IntraductalCarcinomaHighGrade",
        "CartilaginousAndOsseousChange",
        "AxillaryNodeWithLymphoma",
        "AxillaryNodalMetastases",
        "BloodVesselVascularInvasion",
        "CarcinomaInChildren",
        "CarcinomaInEctopicBreast",
        "CarcinomaWithEndocrineDifferentiation",
        "BasalCellCarcinomaOfNipple",
        "MetastaticDiseaseToAxillaryNode",
        "MultifocalInvasiveDuctalCarcinoma",
        "MetastasisToAnIntramammaryLymphNode",
        "MalignantMelanomaOfNipple",
        "RecurrentMalignancy",
        "LympathicVesselInvasion",
        "OccultCarcinomaPresentingWithAxillaryLymphNodeMetastases",
        "MetastaticCancerToTheBreast",
        "MetastaticCancerToTheBreastFromTheColon",
        "MetastaticCancerToTheBreastFromTheLung",
        "MetastaticMelanomaToTheBreast",
        "MetastaticCancerToTheBreastFromTheOvary",
        "MetastaticSarcomaToTheBreast",
        "MultifocalIntraductalCarcinoma",
        "CarcinomaInPregnancyAndLactation",
        "IntraductalComedocarcinomaWithNecrosis",
        "IntraductalCarcinomaLowGrade",
        "IntracysticPapillaryCarcinoma",
        "InvasiveAndInSituCarcinoma",
        "LeukemicInfiltration",
    ],
    "SCT": [
        "Osteosarcoma",
        "Carcinosarcoma",
        "TubularAdenocarcinoma",
        "NonHodgkinLymphoma",
        "ApocrineAdenocarcinoma",
        "PapillaryCarcinoma",
        "InflammatoryCarcinoma",
        "Plasmacytoma",
        "LipidRichLipidSecretingCarcinoma",
        "CarcinomaInSituOfMaleBreast",
        "Hemangiopericytoma",
        "Leiomyosarcoma",
        "HodgkinLymphoma",
        "InvasiveCribriformCarcinoma",
        "Lymphoma",
        "AdenoidCysticCarcinoma",
        "Angiosarcoma",
        "MedullaryCarcinoma",
        "MalignantFibrousHistiocytoma",
        "Fibrosarcoma",
        "SignetRingCellCarcinoma",
        "Liposarcoma",
        "NeoplasmOfTheMammarySkin",
        "PhyllodesTumorMalignant",
        "GlycogenRichCarcinoma",
        "CarcinomaWithMetaplasia",
        "SecretoryJuvenileCarcinomaOfTheBreast",
        "IntraductalCarcinomaMicroPapillary",
        "SquamousCellCarcinoma",
        "MucinousAdenocarcinomaColloidCarcinoma",
        "PagetDiseaseMammary",
        "InfiltratingDuctCarcinoma",
        "Chondrosarcoma",
        "PapillaryCarcinomaInSitu",
        "SpindleCellNodule",
        "Comedocarcinoma",
        "InvasiveLobularCarcinoma",
    ],
}

name_for_cid[6034] = "CADOutputIntendedUse"
cid_concepts[6034] = {
    "DCM": [
        "PresentationRequiredRenderingDeviceIsExpectedToPresent",
        "PresentationOptionalRenderingDeviceMayPresent",
        "NotForPresentationRenderingDeviceExpectedNotToPresent",
    ]
}

name_for_cid[6035] = "CompositeFeatureRelation"
cid_concepts[6035] = {
    "DCM": [
        "TargetContentItemsAreRelatedTemporally",
        "TargetContentItemsAreRelatedSpatially",
        "TargetContentItemsAreRelatedContraLaterally",
    ]
}

name_for_cid[6036] = "FeatureScope"
cid_concepts[6036] = {
    "DCM": [
        "FeatureDetectedOnTheOnlyImage",
        "FeatureDetectedOnOnlyOneOfTheImages",
        "FeatureDetectedOnMultipleImages",
        "FeatureDetectedOnImagesFromMultipleModalities",
    ]
}

name_for_cid[6037] = "MammographyQuantitativeTemporalDifferenceType"
cid_concepts[6037] = {
    "SCT": [
        "DifferenceInSize",
        "DifferenceInLocation",
        "DifferenceInOpacity",
        "DifferenceInSpatialProximity",
        "DifferenceInNumberOfCalcifications",
    ]
}

name_for_cid[6038] = "MammographyQualitativeTemporalDifferenceType"
cid_concepts[6038] = {
    "SCT": ["DifferenceInMargin", "DifferenceInSymmetry", "DifferenceInShape"]
}

name_for_cid[6039] = "NippleCharacteristic"
cid_concepts[6039] = {"SCT": ["NippleRetraction", "NormalShape"]}

name_for_cid[6040] = "NonlesionObjectType"
cid_concepts[6040] = {
    "DCM": ["OtherMarker", "Unspecified"],
    "SCT": [
        "Suture",
        "PacemakerPulseGenerator",
        "CompressionPaddle",
        "JWire",
        "ScarTissue",
        "Bullet",
        "BBShotLeadPellet",
        "OpaqueMarker",
        "Clip",
        "ContrastAgent",
        "Catheter",
        "IDPlate",
        "Collimator",
        "Implant",
        "Staple",
    ],
}

name_for_cid[6041] = "MammographyImageQualityFinding"
cid_concepts[6041] = {
    "DCM": [
        "ViewAndLateralityMarkerIsPartiallyObscured",
        "ViewAndLateralityMarkerIsIncorrect",
        "ViewAndLateralityMarkerIsOffImage",
        "FlashIsNotNearEdgeOfFilm",
        "FlashIsIllegibleDoesNotFitOrIsLopsided",
        "FlashDoesnTIncludePatientNameAndAdditionalPatientId",
        "OtherFailure",
        "FlashDoesnTIncludeDateOfExamination",
        "UnknownFailure",
        "FlashDoesnTIncludeFacilityNameAndLocation",
        "MLOEvidenceOfMotionBlur",
        "ViewAndLateralityMarkerIsNotNearTheAxilla",
        "ViewAndLateralityMarkerOverlapsBreastTissue",
        "Positioning",
        "MLOInframammaryFoldIsNotOpen",
        "CCNotAllMedialTissueVisualized",
        "CCNippleNotCenteredOnImage",
        "CCPosteriorNippleLineDoesNotMeasureWithin1cmOfMLO",
        "NippleNotInProfile",
        "InsufficientImplantDisplacementIncorrect",
        "GridArtifacts",
        "DateStickerIsMissing",
        "TechnicalFactorsMissing",
        "CollimationTooCloseToBreast",
        "InadequateCompression",
        "MLOInsufficientPectoralMuscle",
        "MLONoFatIsVisualizedPosteriorToFibroglandularTissues",
        "ViewAndLateralityMarkerIsMissing",
        "MLOPoorSeparationOfDeepAndSuperficialBreastTissues",
        "MotionBlur",
        "ViewAndLateralityMarkerDoesNotHaveBothViewAndLaterality",
        "UnderExposed",
        "FlashDoesnTIncludeTechnologistIdentification",
        "FlashDoesnTIncludeCassetteScreenDetectorIdentification",
        "FlashDoesnTIncludeMammographyUnitIdentification",
        "ViewAndLateralityMarkerDoesNotHaveApprovedCodes",
        "OverExposed",
        "NoImage",
        "DetectorArtifacts",
        "ArtifactsOtherThanGridOrDetectorArtifact",
        "MechanicalFailure",
        "ElectricalFailure",
        "SoftwareFailure",
        "InappropriateImageProcessing",
    ]
}

name_for_cid[6042] = "ResultStatus"
cid_concepts[6042] = {
    "DCM": ["Succeeded", "PartiallySucceeded", "Failed", "NotAttempted"]
}

name_for_cid[6043] = "MammographyCADAnalysisType"
cid_concepts[6043] = {
    "DCM": [
        "IndividualImpressionRecommendationAnalysis",
        "OverallImpressionRecommendationAnalysis",
    ],
    "SCT": [
        "FocalAsymmetricDensityAnalysis",
        "BreastCompositionAnalysis",
        "SpatialCollocationAnalysis",
        "SpatialProximityAnalysis",
        "TemporalCorrelation",
        "ImageQualityAnalysis",
        "AsymmetricBreastTissueAnalysis",
    ],
}

name_for_cid[6044] = "ImageQualityAssessmentType"
cid_concepts[6044] = {
    "DCM": [
        "UnusableQualityRendersImageUnusable",
        "UsableDoesNotMeetTheQualityControlStandard",
        "UsableMeetsTheQualityControlStandard",
    ]
}

name_for_cid[6045] = "MammographyQualityControlStandardType"
cid_concepts[6045] = {
    "DCM": [
        "InstitutionallyDefinedQualityControlStandard",
        "MammographyQualityControlManual1999ACR",
        "Title21CFRSection900SubpartB",
    ]
}

name_for_cid[6046] = "FollowupIntervalUnit"
cid_concepts[6046] = {"UCUM": ["Year", "Day", "Week", "Month"]}

name_for_cid[6047] = "CADProcessingandFindingSummary"
cid_concepts[6047] = {
    "DCM": [
        "AllAlgorithmsSucceededWithoutFindings",
        "AllAlgorithmsSucceededWithFindings",
        "NotAllAlgorithmsSucceededWithoutFindings",
        "NotAllAlgorithmsSucceededWithFindings",
        "NoAlgorithmsSucceededWithoutFindings",
    ]
}

name_for_cid[6048] = "CADOperatingPointAxisLabel"
cid_concepts[6048] = {
    "DCM": [
        "ImageSpecificity",
        "CaseSpecificity",
        "ProbabilityOfCancer",
        "CertaintyOfFinding",
        "LesionSensitivity",
        "CaseSensitivity",
        "FalseMarkersPerCase",
        "FalseMarkersPerImage",
    ]
}

name_for_cid[6050] = "BreastProcedureReported"
cid_concepts[6050] = {
    "DCM": [
        "MarkerPlacement",
        "FilmScreenMammography",
        "SurgicalConsult",
        "MammographyCAD",
    ],
    "LN": [
        "BreastRightMammogram",
        "BreastFfdMammogram",
        "BreastUnilateralMrWoAndWContrastIv",
        "BreastBilateralMammogram",
        "WholeBodyPtWRncIv",
        "BreastUnilateralMammogram",
        "BreastMrWContrastIv",
        "BreastUnilateralMrWContrastIv",
        "BreastBilateralMrWContrastIv",
        "BreastRightMrWoAndWContrastIv",
        "BreastUnilateralMrWoContrast",
        "BreastLeftMr",
        "BreastLeftMrWoAndWContrastIv",
        "BreastMrWoAndWContrastIv",
        "BreastRightMr",
        "BreastBilateralMrWoAndWContrastIv",
        "WholeBodyCt",
        "BreastBilateralMr",
        "BreastLeftMammogram",
        "BreastLeftMrWContrastIv",
        "BreastRightMrWContrastIv",
        "BreastUnilateralMr",
    ],
    "SCT": [
        "ExaminationOfBreast",
        "FineNeedleAspirationOfBreast",
        "MRIOfBreast",
        "SpecimenRadiographyOfBreast",
        "MammaryDuctogram",
        "UltrasonographyOfBreast",
        "PreBiopsyLocalizationOfBreastLesion",
        "BreastSurgicalBiopsy",
        "InsertionOfCatheter",
        "CoreNeedleBiopsyOfBreast",
        "SentinelLymphNodeBiopsy",
        "CTOfBreast",
        "RadioisotopeScanOfLymphaticSystem",
        "DiagnosticAspirationOfBreastCyst",
        "RadionuclideLocalizationOfTumorLimitedArea",
    ],
}

name_for_cid[6051] = "BreastProcedureReason"
cid_concepts[6051] = {
    "DCM": [
        "NeedleLocalizationAndBiopsy",
        "HistoryOfBenignBreastBiopsy",
        "MarkerPlacement",
        "PersonalHistoryOfBreastCancerWithBreastConservationTherapy",
        "PersonalHistoryOfBreastCancerWithMastectomy",
        "KnownBiopsyProvenMalignancy",
        "ClinicalFinding",
        "Targeted",
        "Survey",
        "RecallForTechnicalReasons",
        "Calibration",
        "RecallForImagingFindings",
        "RecallForPatientSymptomsClinicalFindings",
        "ReviewOfAnOutsideStudy",
        "AdditionalEvaluationRequestedFromAbnormalScreeningStudy",
        "QualityControl",
        "AdditionalEvaluationRequestedFromPriorStudy",
        "FollowUpAtShortIntervalFromPriorStudy",
        "HistoryOfBreastAugmentationAsymptomatic",
    ],
    "SCT": [
        "ReductionMammoplasty",
        "Diagnostic",
        "HistoryOfRadiationTherapy",
        "PersonalHistoryOfBreastCancer",
        "AugmentationMammoplasty",
        "Brachytherapy",
        "Screening",
    ],
}

name_for_cid[6052] = "BreastImagingReportSectionTitle"
cid_concepts[6052] = {
    "DCM": [
        "ProcedureReported",
        "PhysicalExaminationResults",
        "ComparisonToPreviousStudies",
        "OverallAssessment",
    ],
    "LN": [
        "Addendum",
        "Conclusions",
        "Findings",
        "Recommendations",
        "Impressions",
        "IndicationsForProcedure",
    ],
    "SCT": ["BreastComposition"],
}

name_for_cid[6053] = "BreastImagingReportElement"
cid_concepts[6053] = {
    "DCM": [
        "ProcedureReported",
        "Conclusion",
        "PhysicalExaminationResults",
        "Recommendation",
        "ComparisonToPreviousStudies",
        "OverallAssessment",
        "Impression",
        "Finding",
    ],
    "LN": ["IndicationsForProcedure"],
    "SCT": ["BreastComposition"],
}

name_for_cid[6054] = "BreastImagingFinding"
cid_concepts[6054] = {
    "DCM": [
        "ComplexCyst",
        "IntracysticLesion",
        "SolidMass",
        "NormalBreastTissue",
        "BreastGeometry",
        "ImageQuality",
        "DuctNarrowing",
        "NonLesion",
        "IntraluminalFillingDefect",
        "ClusteredMicrocysts",
        "MultipleFillingDefect",
        "AbruptDuctTermination",
        "Extravasation",
        "ComplicatedCyst",
        "CystFill",
        "CooperLigamentChanges",
        "MassInTheSkin",
        "SelectedRegion",
        "MassOnTheSkin",
        "MassWithCalcifications",
    ],
    "SCT": [
        "CalcificationOfBreast",
        "BreastComposition",
        "ForeignBody",
        "CystOfBreast",
        "UltrasoundScanNormal",
        "AxillaryLymphNode",
        "AsymmetricBreastTissue",
        "SkinRetractionOfBreast",
        "ArchitecturalDistortionOfBreast",
        "TubularDensity",
        "MammographyBreastDensity",
        "SkinLesion",
        "BreastNormal",
        "SkinThickeningOfBreast",
        "FocalAsymmetricBreastTissue",
        "LymphNode",
        "AxillaryAdenopathy",
        "Edema",
        "TrabecularThickeningOfBreast",
        "SebaceousCystOfSkinOfBreast",
        "IndividualCalcification",
        "Nipple",
        "MammaryDuctEctasia",
        "MammographicBreastMass",
        "CalcificationCluster",
        "Implant",
        "IntramammaryLymphNode",
    ],
}

name_for_cid[6055] = "BreastClinicalFindingorIndicatedProblem"
cid_concepts[6055] = {
    "DCM": [
        "CancerElsewhere",
        "NonBloodyDischarge",
        "ImageDetectedMass",
        "DifficultPhysicalClinicalExamination",
    ],
    "SCT": [
        "OEAxillaryLymphadenopathy",
        "DiscolorationOfSkinOfBreast",
        "BloodyNippleDischarge",
        "PeauDOrangeSurfaceOfBreast",
        "DisorderOfBreastImplant",
        "MalignantTumourOfUnknownOrigin",
        "SkinRetractionOfBreast",
        "RadiographicCalcificationFinding",
        "OELymphadenopathy",
        "OEBreastLumpPalpated",
        "BreastPain",
        "SkinThickeningOfBreast",
        "NippleDischargeSymptom",
        "Erythema",
        "NippleProblem",
        "BreastLump",
    ],
}

name_for_cid[6056] = "AssociatedFindingforBreast"
cid_concepts[6056] = {
    "DCM": ["CooperLigamentChanges", "MassInTheSkin", "MassOnTheSkin"],
    "SCT": [
        "BreastHematoma",
        "BreastComposition",
        "AxillaryAdenopathy",
        "SurgicalScar",
        "Edema",
        "AxillaryLymphNode",
        "TrabecularThickeningOfBreast",
        "SkinRetractionOfBreast",
        "ArchitecturalDistortionOfBreast",
        "TubularDensity",
        "MammographyBreastDensity",
        "SkinLesion",
        "NippleRetraction",
        "IndividualCalcification",
        "SkinThickeningOfBreast",
        "CalcificationCluster",
        "IntramammaryLymphNode",
    ],
}

name_for_cid[6057] = "DuctographyFindingforBreast"
cid_concepts[6057] = {
    "DCM": [
        "DuctNarrowing",
        "NormalBreastTissue",
        "CystFill",
        "IntraluminalFillingDefect",
        "MultipleFillingDefect",
        "AbruptDuctTermination",
        "Extravasation",
    ],
    "SCT": ["MammaryDuctEctasia"],
}

name_for_cid[6058] = "ProcedureModifiersforBreast"
cid_concepts[6058] = {
    "DCM": [
        "SalineImplant",
        "PolyurethaneImplant",
        "PercutaneousSiliconeInjection",
        "CombinationImplant",
        "PrePectoralImplant",
        "NeedleLocalizationAndBiopsy",
        "RetroPectoralImplant",
        "MarkerPlacement",
        "_3DSpeckleTracking",
        "MammographicCrosshair",
        "MammographicGrid",
        "PalpationGuided",
        "Targeted",
        "_2DShearWaveElastography",
        "Survey",
        "VacuumAssisted",
        "Calibration",
        "_3DShearWaveElastography",
        "QualityControl",
        "PointShearWaveElastography",
        "_2DSpeckleTracking",
    ],
    "SCT": [
        "Lumpectomy",
        "BMode",
        "TissueDopplerImaging",
        "BreastImplantTypeNotSpecified",
        "_3DMode",
        "_2DMode",
        "Mammography",
        "DopplerContinuousWave",
        "MagneticResonanceImagingGuidedBiopsy",
        "Diagnostic",
        "QuadrantectomyOfBreast",
        "SiliconeGelImplant",
        "DopplerPulsed",
        "MMode",
        "DiagnosticRadiographyStereotacticLocalization",
        "Screening",
        "PowerDoppler",
        "UltrasonicGuidanceProcedure",
        "DopplerColorFlow",
        "ComputedTomographyGuidedBiopsy",
    ],
    "UMLS": ["Mastectomy"],
}

name_for_cid[6059] = "BreastImplantType"
cid_concepts[6059] = {
    "DCM": [
        "SalineImplant",
        "PolyurethaneImplant",
        "PercutaneousSiliconeInjection",
        "CombinationImplant",
        "PrePectoralImplant",
        "RetroPectoralImplant",
    ],
    "SCT": ["SiliconeGelImplant", "BreastImplantTypeNotSpecified"],
}

name_for_cid[6060] = "BreastBiopsyTechnique"
cid_concepts[6060] = {
    "DCM": [
        "VacuumAssisted",
        "MammographicCrosshair",
        "MammographicGrid",
        "PalpationGuided",
    ],
    "SCT": [
        "QuadrantectomyOfBreast",
        "Lumpectomy",
        "DiagnosticRadiographyStereotacticLocalization",
        "Mammography",
        "UltrasonicGuidanceProcedure",
        "MagneticResonanceImagingGuidedBiopsy",
        "ComputedTomographyGuidedBiopsy",
    ],
    "UMLS": ["Mastectomy"],
}

name_for_cid[6061] = "BreastImagingProcedureModifier"
cid_concepts[6061] = {
    "DCM": [
        "Calibration",
        "NeedleLocalizationAndBiopsy",
        "QualityControl",
        "MarkerPlacement",
        "Targeted",
        "Survey",
    ],
    "SCT": ["Diagnostic", "Screening"],
}

name_for_cid[6062] = "InterventionalProcedureComplication"
cid_concepts[6062] = {
    "DCM": ["AbnormalDischarge", "NoComplications"],
    "SCT": [
        "Pneumothorax",
        "Weal",
        "HealthcareAssociatedInfection",
        "Swelling",
        "HematomaPostoperative",
        "HemorrhagePostprocedure",
        "Rash",
        "VasovagalSyncope",
        "PersistentPainFollowingProcedure",
    ],
}

name_for_cid[6063] = "InterventionalProcedureResult"
cid_concepts[6063] = {
    "SCT": [
        "Benign",
        "Malignant",
        "InsufficientSample",
        "HighRiskTumor",
        "IndeterminateResult",
    ]
}

name_for_cid[6064] = "UltrasoundFindingforBreast"
cid_concepts[6064] = {
    "DCM": [
        "ComplexCyst",
        "ComplicatedCyst",
        "IntracysticLesion",
        "SolidMass",
        "ClusteredMicrocysts",
    ],
    "SCT": [
        "SebaceousCystOfSkinOfBreast",
        "ForeignBody",
        "LymphNode",
        "CystOfBreast",
        "MammaryDuctEctasia",
        "UltrasoundScanNormal",
    ],
}

name_for_cid[6065] = "InstrumentApproach"
cid_concepts[6065] = {
    "DCM": [
        "InferolateralToSuperomedial",
        "InferomedialToSuperolateral",
        "SuperolateralToInferomedial",
        "SuperomedialToInferolateral",
    ],
    "SCT": ["Lateral", "Inferior", "Superior", "Medial"],
}

name_for_cid[6066] = "TargetConfirmation"
cid_concepts[6066] = {
    "DCM": [
        "TargetContainedInTheSpecimen",
        "TargetPartiallyObtainedInTheSpecimen",
        "TargetNotInTheSpecimen",
        "CalcificationsSeenInTheCore",
        "LesionCompletelyRemoved",
        "LesionPartiallyRemoved",
        "FluidObtained",
    ]
}

name_for_cid[6067] = "FluidColor"
cid_concepts[6067] = {
    "DCM": [
        "LightBrownColor",
        "DarkRedColor",
        "DarkBrownColor",
        "BrightRedColor",
        "BloodTingedColor",
    ],
    "SCT": [
        "BlackColor",
        "Clear",
        "BrownColor",
        "YellowColor",
        "GrayColor",
        "WhiteColor",
        "GreenColor",
    ],
}

name_for_cid[6068] = "TumorStagesFromAJCC"
cid_concepts[6068] = {
    "DCM": [
        "Stage0",
        "StageI",
        "StageIIA",
        "StageIIB",
        "StageIIIA",
        "StageIIIB",
        "StageIIIC",
        "StageIV",
    ]
}

name_for_cid[6069] = "NottinghamCombinedHistologicGrade"
cid_concepts[6069] = {
    "SCT": [
        "NottinghamCombinedGradeI3To5Points",
        "NottinghamCombinedGradeII6To7Points",
        "NottinghamCombinedGradeIII8To9Points",
        "NottinghamCombinedGradeCannotBeDetermined",
    ]
}

name_for_cid[6070] = "BloomRichardsonHistologicGrade"
cid_concepts[6070] = {
    "SCT": [
        "Grade3PoorlyDifferentiated",
        "Grade1WellDifferentiated",
        "Grade4Undifferentiated",
        "Grade2ModeratelyDifferentiated",
    ]
}

name_for_cid[6071] = "HistologicGradingMethod"
cid_concepts[6071] = {
    "DCM": ["BloomRichardsonGrade"],
    "SCT": ["NottinghamCombinedGrade"],
}

name_for_cid[6072] = "BreastImplantFinding"
cid_concepts[6072] = {
    "DCM": [
        "DistortedImplant",
        "SiliconeLadenLymphNodes",
        "FreeSilicone",
        "HerniatedImplant",
        "Explantation",
        "NormalImplants",
        "AsymmetricImplants",
        "CalcifiedImplant",
    ],
    "SCT": ["RuptureOfBreastImplant"],
}

name_for_cid[6080] = "GynecologicalHormone"
cid_concepts[6080] = {
    "DCM": ["UnspecifiedGynecologicalHormone"],
    "SCT": [
        "Anastrozole",
        "Tamoxifen",
        "Raloxifene",
        "Contraceptives",
        "ProgesteroneProduct",
        "EstrogenProduct",
    ],
}

name_for_cid[6081] = "BreastCancerRiskFactor"
cid_concepts[6081] = {
    "DCM": [
        "IntermediateFamilyHistoryOfBreastCancer",
        "PersonalBreastCancerHistory",
        "VeryStrongFamilyHistoryOfBreastCancer",
        "HistoryOfEndometrialCancer",
        "FamilyHistoryOfProstateCancer",
        "HistoryOfOvarianCancer",
        "FamilyHistoryUnknown",
        "HistoryOfHighRiskLesionOnPreviousBiopsy",
        "PostMenopausalPatient",
        "LateChildBearingAfter30",
        "BRCA1BreastCancerGene",
        "BRCA2BreastCancerGene",
        "BRCA3BreastCancerGene",
        "WeakFamilyHistoryOfBreastCancer",
    ],
    "SCT": [
        "Nulliparous",
        "FamilyHistoryOfBreastCancer",
        "NoFamilyHistoryOfBreastCarcinoma",
    ],
}

name_for_cid[6082] = "GynecologicalProcedure"
cid_concepts[6082] = {
    "SCT": ["DilationAndCurettage", "EndometrialBiopsy", "Hysterectomy"]
}

name_for_cid[6083] = "ProceduresforBreast"
cid_concepts[6083] = {
    "DCM": [
        "SurgicalConsult",
        "MammographyCAD",
        "MarkerPlacement",
        "FilmScreenMammography",
        "NippleDischargeCytology",
    ],
    "LN": [
        "BreastBilateralMammogram",
        "WholeBodyPtWRncIv",
        "BreastUnilateralMrWoContrast",
        "BreastLeftMr",
        "BreastLeftMrWoAndWContrastIv",
        "BreastMrWoAndWContrastIv",
        "WholeBodyCt",
        "BreastBilateralMr",
        "BreastLeftMrWContrastIv",
        "BreastRightMammogram",
        "BreastFfdMammogram",
        "BreastUnilateralMrWoAndWContrastIv",
        "BreastUnilateralMammogram",
        "BreastMrWContrastIv",
        "BreastUnilateralMrWContrastIv",
        "BreastBilateralMrWContrastIv",
        "BreastRightMrWoAndWContrastIv",
        "BreastRightMr",
        "BreastBilateralMrWoAndWContrastIv",
        "BreastLeftMammogram",
        "BreastRightMrWContrastIv",
        "BreastUnilateralMr",
    ],
    "SCT": [
        "ExaminationOfBreast",
        "RemovalOfBreastImplant",
        "FineNeedleAspirationOfBreast",
        "MRIOfBreast",
        "ReductionMammoplasty",
        "MammaryDuctogram",
        "BreastReconstruction",
        "UltrasonographyOfBreast",
        "CoreNeedleBiopsyOfBreast",
        "HormoneTherapy",
        "BoneMarrowTransplant",
        "RadiationTherapy",
        "SpecimenRadiographyOfBreast",
        "Chemotherapy",
        "PreBiopsyLocalizationOfBreastLesion",
        "BreastSurgicalBiopsy",
        "InsertionOfCatheter",
        "BreastProsthesisInsertion",
        "SentinelLymphNodeBiopsy",
        "CTOfBreast",
        "RadioisotopeScanOfLymphaticSystem",
        "DiagnosticAspirationOfBreastCyst",
        "RadionuclideLocalizationOfTumorLimitedArea",
    ],
}

name_for_cid[6084] = "MammoplastyProcedure"
cid_concepts[6084] = {
    "SCT": [
        "BreastProsthesisInsertion",
        "ReductionMammoplasty",
        "RemovalOfBreastImplant",
        "BreastReconstruction",
    ]
}

name_for_cid[6085] = "TherapiesforBreast"
cid_concepts[6085] = {
    "SCT": [
        "HormoneTherapy",
        "BoneMarrowTransplant",
        "Chemotherapy",
        "RadiationTherapy",
    ]
}

name_for_cid[6086] = "MenopausalPhase"
cid_concepts[6086] = {
    "SCT": [
        "AfterMenopause",
        "BeforeMenopause",
        "PostsurgicalMenopause",
        "ArtificialMenopauseState",
        "DuringMenopause",
    ]
}

name_for_cid[6087] = "GeneralRiskFactor"
cid_concepts[6087] = {
    "DCM": [
        "IntermediateFamilyHistoryOfBreastCancer",
        "VeryStrongFamilyHistoryOfBreastCancer",
        "FamilyHistoryOfProstateCancer",
        "FamilyHistoryUnknown",
        "UterineMalformations",
        "SpontaneousAbortion",
        "GynecologicCondition",
        "GynecologicSurgery",
        "PreviousLBWOrIUGRBirth",
        "PreviousFetalMalformationSyndrome",
        "PreviousRHNegativeOrBloodDyscrasiaAtBirth",
        "HistoryOfMultipleFetuses",
        "PersonalBreastCancerHistory",
        "CurrentPregnancyKnownOrSuspectedMalformationsSyndromes",
        "HistoryOfEndometrialCancer",
        "FamilyHistoryFetalMalformationSyndrome",
        "HistoryOfOvarianCancer",
        "HistoryOfHighRiskLesionOnPreviousBiopsy",
        "PostMenopausalPatient",
        "LateChildBearingAfter30",
        "BRCA1BreastCancerGene",
        "BRCA2BreastCancerGene",
        "BRCA3BreastCancerGene",
        "WeakFamilyHistoryOfBreastCancer",
    ],
    "SCT": [
        "HistoryOfDiabetesMellitus",
        "HistoryOfEclampsia",
        "HistoryOfInfertility",
        "FamilyHistoryOfBreastCancer",
        "HistoryOfCardiovascularDisease",
        "HistoryOfObesity",
        "HistoryOfHypertension",
        "HistoryOfSubstanceAbuse",
        "HistoryOfEctopicPregnancy",
        "Nulliparous",
        "HistoryOfPrematureDelivery",
        "HistoryOfRegularMedication",
        "HistoryOfSeverePreEclampsia",
        "MultiplePregnancy",
        "NoFamilyHistoryOfBreastCarcinoma",
    ],
}

name_for_cid[6088] = "OBGYNMaternalRiskFactor"
cid_concepts[6088] = {
    "DCM": [
        "PreviousFetalMalformationSyndrome",
        "PreviousRHNegativeOrBloodDyscrasiaAtBirth",
        "HistoryOfMultipleFetuses",
        "CurrentPregnancyKnownOrSuspectedMalformationsSyndromes",
        "FamilyHistoryFetalMalformationSyndrome",
        "UterineMalformations",
        "SpontaneousAbortion",
        "GynecologicCondition",
        "GynecologicSurgery",
        "PreviousLBWOrIUGRBirth",
    ],
    "SCT": [
        "HistoryOfObesity",
        "HistoryOfHypertension",
        "HistoryOfSubstanceAbuse",
        "HistoryOfEctopicPregnancy",
        "HistoryOfDiabetesMellitus",
        "HistoryOfEclampsia",
        "HistoryOfInfertility",
        "HistoryOfPrematureDelivery",
        "HistoryOfRegularMedication",
        "HistoryOfSeverePreEclampsia",
        "MultiplePregnancy",
        "HistoryOfCardiovascularDisease",
    ],
}

name_for_cid[6089] = "Substance"
cid_concepts[6089] = {
    "SCT": [
        "LysergicAcidDiethlyamide",
        "Methlyphenidate",
        "Methadone",
        "Phencyclidine",
        "Morphine",
        "Cocaine",
        "Heroin",
        "Amphetamine",
        "Caffeine",
        "Marijuana",
        "EthylAlcohol",
        "ChewingTobacco",
        "CigaretteSmokingTobacco",
        "Mescaline",
    ]
}

name_for_cid[6090] = "RelativeUsage/ExposureAmount"
cid_concepts[6090] = {"DCM": ["High", "Medium", "NoKnownExposure", "Low"]}

name_for_cid[6091] = "RelativeFrequencyofEventValue"
cid_concepts[6091] = {
    "SCT": [
        "Continuous",
        "SingleEvent",
        "AsRequired",
        "Frequent",
        "MidFrequency",
        "Infrequent",
    ]
}

name_for_cid[6092] = "Usage/ExposureQualitativeConcept"
cid_concepts[6092] = {
    "DCM": ["VolumeOfUse", "DoseFrequency", "RateOfExposure"],
    "SCT": ["Dosage"],
}

name_for_cid[6093] = "Usage/Exposure/AmountQualitativeConcept"
cid_concepts[6093] = {
    "DCM": ["RelativeDoseAmount", "RelativeAmountOfExposure", "RelativeAmountOfUse"]
}

name_for_cid[6094] = "Usage/Exposure/FrequencyQualitativeConcept"
cid_concepts[6094] = {
    "DCM": [
        "RelativeDoseFrequency",
        "RelativeFrequencyOfExposure",
        "RelativeFrequencyOfUse",
    ]
}

name_for_cid[6095] = "ProcedureNumericProperty"
cid_concepts[6095] = {"DCM": ["NeedleGauge", "NeedleLength"]}

name_for_cid[6096] = "PregnancyStatus"
cid_concepts[6096] = {
    "SCT": ["PossiblePregnancy", "Unknown", "PatientCurrentlyPregnant", "NotPregnant"]
}

name_for_cid[6097] = "SideofFamily"
cid_concepts[6097] = {"DCM": ["Maternal"], "SCT": ["Paternal"]}

name_for_cid[6098] = "ClinicalCourseofDisease"
cid_concepts[6098] = {"NCIt": ["NoEvidenceOfDisease", "RecurrentDisease"]}

name_for_cid[6099] = "RacialGroup"
cid_concepts[6099] = {
    "NCIt": ["NativeHawaiianOrOtherPacificIslander"],
    "SCT": [
        "AmericanIndianOrAlaskaNative",
        "AsianOrPacificIslanderRace",
        "CaucasianRace",
        "Hispanic",
        "AustralianAborigineRace",
        "IndianRace",
        "AfricanRace",
        "MixedRacialGroup",
        "AsianRace",
    ],
    "UMLS": ["MiddleEasternOrNorthAfrican"],
}

name_for_cid[6100] = "ChestComponentCategory"
cid_concepts[6100] = {
    "DCM": ["Bronchovascular", "Osseous"],
    "SCT": [
        "Lung",
        "Mediastinum",
        "Heart",
        "PleuralStructure",
        "Muscular",
        "SystemicVascularStructure",
    ],
}

name_for_cid[6101] = "ChestFindingorFeature"
cid_concepts[6101] = {
    "DCM": [
        "AbnormalLucency",
        "AbnormalCalcifications",
        "AbnormalTexture",
        "AbnormalOpacity",
        "ImageQuality",
        "NonLesion",
        "RadiographicAnatomy",
        "SelectedRegion",
        "AbnormalLines1D",
    ]
}

name_for_cid[6102] = "ChestFindingorFeatureModifier"
cid_concepts[6102] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
    ],
    "DCM": [
        "KerleyBLine",
        "KerleyCLines",
        "ParenchymalBand",
        "ReticularPattern",
        "SeptalLines",
        "SubpleuralLine",
        "TramlineShadow",
        "TubularShadow",
        "Density",
        "DependentOpacity",
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "AirFluidLevel",
        "CoronaRadiata",
        "HoneycombPattern",
        "FleischnerLines",
        "IntralobularLines",
        "KerleyALine",
        "MosaicPattern",
        "GroundGlassOpacity",
        "Infiltrate",
        "Micronodule",
        "Opacity",
        "PhantomTumor",
        "Shadow",
        "AbnormalInterstitialPattern",
        "SmallIrregularOpacities",
        "SmallRoundedOpacities",
        "TreeInBudSign",
        "GranularPattern",
        "MiliaryPattern",
        "HaloSign",
        "AirBronchiologram",
        "AirBronchogram",
        "AirCrescent",
        "ReticulonodularPattern",
        "BeadedSeptumSign",
        "NodularPattern",
        "Pseudoplaque",
        "SignetRingSign",
        "Portacath",
        "ChestTube",
        "CentralLine",
        "KidneyStent",
        "SecondaryPulmonaryLobule",
        "PancreaticStent",
        "NippleRing",
        "Coin",
        "FiducialMark",
        "PrimaryComplex",
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
        "Lucency",
        "MidlungWindow",
        "CarinaAngle",
        "CentrilobularStructures",
        "AnteriorJunctionLine",
        "PosteriorJunctionLine",
        "Interface",
        "Line",
    ],
    "SCT": [
        "SubclavianVein",
        "SternocleidomastoidMuscle",
        "PulmonaryValve",
        "ProximalRightCoronaryArtery",
        "ErectorSpinaeMuscle",
        "MidCircumflexCoronaryArtery",
        "Prosthesis",
        "IntermediateArteryRamus",
        "SubscapularisMuscle",
        "EndotrachealTube",
        "LymphNode",
        "TrapeziusMuscle",
        "IntercostalArtery",
        "Esophagus",
        "PlateLikeAtelectasis",
        "ScarTissue",
        "TeresMajorMuscle",
        "LeftMainBronchus",
        "MidRightCoronaryArtery",
        "PulmonaryVein",
        "SubclavianArtery",
        "EsophagealHiatus",
        "_3rdLeftPosterolateralCoronaryArtery",
        "HeartValveProsthesis",
        "PectoralisMajorMuscle",
        "LeftMainCoronaryArteryOstium",
        "Tracheotomy",
        "Sternum",
        "Staple",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "_1stMarginalCoronaryArtery",
        "Pin",
        "PulmonaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "Thyroid",
        "CoronaryArteryGraft",
        "MitralValve",
        "SupraspinatusMuscle",
        "AxillaryFascia",
        "ExternalIntercostalMuscle",
        "InfraspinatusMuscle",
        "TricuspidValve",
        "Nodule",
        "FissureOfLung",
        "LevatoresCostarumMuscles",
        "SuperiorPhrenicArtery",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "AtrialSeptalDefect",
        "Catheter",
        "Ventricle",
        "RightCoronaryArteryOstium",
        "DescendingAorta",
        "MarginalCoronaryArtery",
        "InternalThoracicArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "Bullet",
        "Spine",
        "BronchialArtery",
        "PosterolateralBranchOfRightCoronaryArtery",
        "BrachiocephalicVein",
        "ProximalCircumflexCoronaryArtery",
        "VenaCavaFilter",
        "_2ndMarginalCoronaryArtery",
        "Needle",
        "Carina",
        "AorticArch",
        "PulmonaryEmbolism",
        "CarotidBody",
        "Heart",
        "AirwayStructure",
        "DeltoidMuscle",
        "LeftMainCoronaryArtery",
        "InternalIntercostalMuscle",
        "UretericStent",
        "_1stDiagonalCoronaryArtery",
        "InteratrialSeptum",
        "Scapula",
        "IliocostalisMuscle",
        "_1stRightPosterolateralCoronaryArtery",
        "Suture",
        "RightPosteriorAVCoronaryArtery",
        "FascialLayer",
        "InferiorVenaCava",
        "Jewelry",
        "ThymusGland",
        "SubcostalMuscle",
        "Humerus",
        "DistalRightCoronaryArtery",
        "Mass",
        "LeftAnteriorDescendingCoronaryArtery",
        "CommonCarotidArtery",
        "InnermostIntercostalMuscles",
        "_3rdMarginalCoronaryArtery",
        "PectoralisMinorMuscle",
        "EsophagealArtery",
        "LatissimusDorsiMuscle",
        "CervicalCollar",
        "Trachea",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "Bronchus",
        "ExternalJugularVein",
        "DorsalScapularArtery",
        "FeedingTube",
        "SuperiorVenaCava",
        "CostalCartilage",
        "BrachiocephalicTrunk",
        "_2ndRightPosterolateralCoronaryArtery",
        "PulmonaryTrunk",
        "AzygosVein",
        "BrachialArtery",
        "_2ndDiagonalCoronaryArtery",
        "DistalCircumflexCoronaryArtery",
        "Rib",
        "TeresMinorMuscle",
        "LongissimusMuscle",
        "SerratusAnteriorMuscle",
        "Clavicle",
        "AirTrapping",
        "AorticIsthmus",
        "Vertebra",
        "_1stLeftPosterolateralCoronaryArtery",
        "Pneumothorax",
        "CircumflexCoronaryArtery",
        "Diaphragm",
        "AxillaryArtery",
        "LeftCoronaryArtery",
        "VertebralArtery",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "InternalJugularVein",
        "ChordaeTendineaeCordis",
        "RightMainBronchus",
        "ScalenousAnteriorMuscle",
        "CardiacPacemakerLead",
        "AVGrooveContinuationOfCircumflexArtery",
        "TrabeculaeCarnae",
        "_3rdRightPosterolateralCoronaryArtery",
        "ThyrocervicalTrunk",
        "InferiorPhrenicArtery",
        "JejunostomyTube",
        "HilumOfLung",
        "ThoracicDuct",
        "CoinLesion",
        "TransversusThoracis",
        "InterventricularSeptum",
        "_2ndLeftPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "AscendingAorta",
        "PericardiophrenicArtery",
        "CardiacPacemaker",
        "CostocervicalTrunk",
        "AxillaryVein",
        "LigamentumArteriosum",
        "SegmentOfLung",
        "Pneumomediastinum",
        "LobeOfLung",
    ],
}

name_for_cid[6103] = "AbnormalLinesFindingorFeature"
cid_concepts[6103] = {
    "DCM": [
        "ReticulonodularPattern",
        "KerleyBLine",
        "KerleyCLines",
        "ParenchymalBand",
        "ReticularPattern",
        "SeptalLines",
        "AirFluidLevel",
        "SubpleuralLine",
        "CoronaRadiata",
        "TramlineShadow",
        "HoneycombPattern",
        "TubularShadow",
        "FleischnerLines",
        "IntralobularLines",
        "KerleyALine",
    ],
    "SCT": ["PlateLikeAtelectasis"],
}

name_for_cid[6104] = "AbnormalOpacityFindingorFeature"
cid_concepts[6104] = {
    "DCM": [
        "ReticulonodularPattern",
        "GroundGlassOpacity",
        "BeadedSeptumSign",
        "Infiltrate",
        "NodularPattern",
        "Micronodule",
        "Opacity",
        "Pseudoplaque",
        "PhantomTumor",
        "SignetRingSign",
        "Shadow",
        "PrimaryComplex",
        "AbnormalInterstitialPattern",
        "SmallIrregularOpacities",
        "SmallRoundedOpacities",
        "TreeInBudSign",
        "Density",
        "DependentOpacity",
    ],
    "SCT": ["Mass", "PulmonaryEmbolism", "Nodule", "CoinLesion"],
}

name_for_cid[6105] = "AbnormalLucencyFindingorFeature"
cid_concepts[6105] = {
    "DCM": ["HaloSign", "AirBronchiologram", "AirBronchogram", "AirCrescent"],
    "SCT": ["Pneumothorax", "AirTrapping", "Pneumomediastinum"],
}

name_for_cid[6106] = "AbnormalTextureFindingorFeature"
cid_concepts[6106] = {
    "DCM": [
        "MosaicPattern",
        "ReticulonodularPattern",
        "NodularPattern",
        "ReticularPattern",
        "AbnormalInterstitialPattern",
        "SmallIrregularOpacities",
        "GranularPattern",
        "HoneycombPattern",
        "MiliaryPattern",
    ]
}

name_for_cid[6107] = "WidthDescriptor"
cid_concepts[6107] = {
    "DCM": ["Vasoconstriction", "Vasodilation"],
    "SCT": ["Narrow", "Enlarged"],
}

name_for_cid[6108] = "ChestAnatomicStructureAbnormalDistribution"
cid_concepts[6108] = {
    "DCM": ["ArchitecturalDistortion", "MosaicPerfusion", "Pleonemia", "Oligemia"],
    "SCT": ["AirTrapping"],
}

name_for_cid[6109] = "RadiographicAnatomyFindingorFeature"
cid_concepts[6109] = {
    "BARI": [
        "_1stDiagonalCoronaryArteryLaterals",
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": [
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "SecondaryPulmonaryLobule",
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
        "Lucency",
        "MidlungWindow",
        "CarinaAngle",
        "CentrilobularStructures",
        "AnteriorJunctionLine",
        "PosteriorJunctionLine",
        "Interface",
        "Line",
    ],
    "SCT": [
        "SubclavianVein",
        "SternocleidomastoidMuscle",
        "PulmonaryValve",
        "ProximalRightCoronaryArtery",
        "RightPosteriorAVCoronaryArtery",
        "FascialLayer",
        "ErectorSpinaeMuscle",
        "MidCircumflexCoronaryArtery",
        "InferiorVenaCava",
        "IntermediateArteryRamus",
        "ThymusGland",
        "SubcostalMuscle",
        "Humerus",
        "SubscapularisMuscle",
        "LymphNode",
        "TrapeziusMuscle",
        "DistalRightCoronaryArtery",
        "IntercostalArtery",
        "Esophagus",
        "LeftAnteriorDescendingCoronaryArtery",
        "CommonCarotidArtery",
        "InnermostIntercostalMuscles",
        "TeresMajorMuscle",
        "LeftMainBronchus",
        "_3rdMarginalCoronaryArtery",
        "PectoralisMinorMuscle",
        "MidRightCoronaryArtery",
        "EsophagealArtery",
        "PulmonaryVein",
        "LatissimusDorsiMuscle",
        "Trachea",
        "SubclavianArtery",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "EsophagealHiatus",
        "_3rdLeftPosterolateralCoronaryArtery",
        "Bronchus",
        "ExternalJugularVein",
        "PectoralisMajorMuscle",
        "DorsalScapularArtery",
        "LeftMainCoronaryArteryOstium",
        "SuperiorVenaCava",
        "Sternum",
        "CostalCartilage",
        "BrachiocephalicTrunk",
        "_2ndRightPosterolateralCoronaryArtery",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "_1stMarginalCoronaryArtery",
        "PulmonaryTrunk",
        "AzygosVein",
        "PulmonaryArtery",
        "BrachialArtery",
        "_2ndDiagonalCoronaryArtery",
        "DistalCircumflexCoronaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "Thyroid",
        "Rib",
        "CoronaryArteryGraft",
        "TeresMinorMuscle",
        "MitralValve",
        "LongissimusMuscle",
        "SerratusAnteriorMuscle",
        "SupraspinatusMuscle",
        "Clavicle",
        "AorticIsthmus",
        "AxillaryFascia",
        "Vertebra",
        "ExternalIntercostalMuscle",
        "_1stLeftPosterolateralCoronaryArtery",
        "InfraspinatusMuscle",
        "TricuspidValve",
        "CircumflexCoronaryArtery",
        "Diaphragm",
        "FissureOfLung",
        "LevatoresCostarumMuscles",
        "SuperiorPhrenicArtery",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "AtrialSeptalDefect",
        "Ventricle",
        "RightCoronaryArteryOstium",
        "AxillaryArtery",
        "LeftCoronaryArtery",
        "DescendingAorta",
        "MarginalCoronaryArtery",
        "InternalThoracicArtery",
        "VertebralArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "InternalJugularVein",
        "Spine",
        "BronchialArtery",
        "ChordaeTendineaeCordis",
        "RightMainBronchus",
        "ScalenousAnteriorMuscle",
        "AVGrooveContinuationOfCircumflexArtery",
        "TrabeculaeCarnae",
        "_3rdRightPosterolateralCoronaryArtery",
        "ThyrocervicalTrunk",
        "PosterolateralBranchOfRightCoronaryArtery",
        "InferiorPhrenicArtery",
        "BrachiocephalicVein",
        "HilumOfLung",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "ThoracicDuct",
        "Carina",
        "AorticArch",
        "CarotidBody",
        "Heart",
        "AirwayStructure",
        "TransversusThoracis",
        "DeltoidMuscle",
        "InterventricularSeptum",
        "LeftMainCoronaryArtery",
        "_2ndLeftPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "InternalIntercostalMuscle",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "AscendingAorta",
        "_1stDiagonalCoronaryArtery",
        "PericardiophrenicArtery",
        "InteratrialSeptum",
        "CostocervicalTrunk",
        "AxillaryVein",
        "LigamentumArteriosum",
        "Scapula",
        "SegmentOfLung",
        "IliocostalisMuscle",
        "_1stRightPosterolateralCoronaryArtery",
        "LobeOfLung",
    ],
}

name_for_cid[6110] = "LungAnatomyFindingorFeature"
cid_concepts[6110] = {
    "DCM": ["MidlungWindow", "SecondaryPulmonaryLobule"],
    "SCT": ["SegmentOfLung", "LobeOfLung"],
}

name_for_cid[6111] = "BronchovascularAnatomyFindingorFeature"
cid_concepts[6111] = {
    "DCM": ["CarinaAngle", "CentrilobularStructures"],
    "SCT": ["HilumOfLung", "AirwayStructure", "Carina", "Bronchus"],
}

name_for_cid[6112] = "PleuraAnatomyFindingorFeature"
cid_concepts[6112] = {
    "DCM": ["AnteriorJunctionLine", "PosteriorJunctionLine"],
    "SCT": ["FissureOfLung"],
}

name_for_cid[6113] = "MediastinumAnatomyFindingorFeature"
cid_concepts[6113] = {
    "DCM": [
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
    ],
    "SCT": [
        "PulmonaryValve",
        "FascialLayer",
        "ThoracicDuct",
        "AxillaryFascia",
        "Carina",
        "CarotidBody",
        "TricuspidValve",
        "Heart",
        "ThymusGland",
        "AtrialSeptalDefect",
        "LymphNode",
        "CostalCartilage",
        "Ventricle",
        "TrapeziusMuscle",
        "Esophagus",
        "LeftMainBronchus",
        "RightMainBronchus",
        "Thyroid",
        "LigamentumArteriosum",
        "Trachea",
        "MitralValve",
        "EsophagealHiatus",
    ],
}

name_for_cid[6114] = "OsseousAnatomyFindingorFeature"
cid_concepts[6114] = {
    "SCT": ["Clavicle", "Rib", "Humerus", "Scapula", "Vertebra", "Spine", "Sternum"]
}

name_for_cid[6115] = "OsseousAnatomyModifier"
cid_concepts[6115] = {
    "DCM": [
        "RibScaleneTubercle",
        "VertebralIntervertebralNotch",
        "SubscapularFossa",
        "ScapularSpine",
        "ScapularSupraspinatusFossa",
        "ScapularInfraspinatusFossa",
    ],
    "SCT": [
        "VertebralForamen",
        "InferiorArticularFacetOfAxis",
        "DorsalAspectOfScapula",
        "GlenoidCavityOfScapula",
        "SpinousProcessOfVertebra",
        "TubercleOfRib",
        "SuperiorArticularProcessOfVertebra",
        "BodyOfSternum",
        "AngleOfRib",
        "InferiorArticularProcessOfVertebra",
        "ManubriumOfSternum",
        "SuperiorArticularFacetOfAxis",
        "PedicleOfVertebra",
        "AcromionProcessOfScapula",
        "HeadOfRib",
        "TransverseProcessOrVertebra",
        "LaminaOfVertebra",
        "ArchOfVertebra",
        "NeckOfRib",
        "ShaftOfRib",
        "SuprasternalNotch",
        "CoracoidProcessOfScapula",
        "XiphoidProcessOfSternum",
        "SternalAngle",
        "PectoralGirdle",
        "CostalGroove",
        "VertebralCanal",
        "ClavicularNotchOfSternum",
    ],
}

name_for_cid[6116] = "MuscularAnatomy"
cid_concepts[6116] = {
    "SCT": [
        "SternocleidomastoidMuscle",
        "ErectorSpinaeMuscle",
        "ExternalIntercostalMuscle",
        "PectoralisMajorMuscle",
        "InfraspinatusMuscle",
        "Diaphragm",
        "LevatoresCostarumMuscles",
        "SubcostalMuscle",
        "SubscapularisMuscle",
        "TransversusThoracis",
        "DeltoidMuscle",
        "InterventricularSeptum",
        "TrapeziusMuscle",
        "InternalIntercostalMuscle",
        "InnermostIntercostalMuscles",
        "TeresMajorMuscle",
        "SpinalisMuscle",
        "PectoralisMinorMuscle",
        "ChordaeTendineaeCordis",
        "InteratrialSeptum",
        "ScalenousAnteriorMuscle",
        "TeresMinorMuscle",
        "LatissimusDorsiMuscle",
        "TrabeculaeCarnae",
        "IliocostalisMuscle",
        "LongissimusMuscle",
        "SerratusAnteriorMuscle",
        "SupraspinatusMuscle",
    ]
}

name_for_cid[6117] = "VascularAnatomy"
cid_concepts[6117] = {
    "BARI": [
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": ["AorticKnob", "ArchOfTheAzygosVein"],
    "SCT": [
        "SubclavianVein",
        "ProximalRightCoronaryArtery",
        "RightPosteriorAVCoronaryArtery",
        "AorticIsthmus",
        "MidCircumflexCoronaryArtery",
        "InferiorVenaCava",
        "_1stLeftPosterolateralCoronaryArtery",
        "IntermediateArteryRamus",
        "CircumflexCoronaryArtery",
        "SuperiorPhrenicArtery",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "RightCoronaryArteryOstium",
        "AxillaryArtery",
        "LeftCoronaryArtery",
        "DescendingAorta",
        "DistalRightCoronaryArtery",
        "MarginalCoronaryArtery",
        "IntercostalArtery",
        "InternalThoracicArtery",
        "LeftAnteriorDescendingCoronaryArtery",
        "VertebralArtery",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "CommonCarotidArtery",
        "_3rdDiagonalCoronaryArtery",
        "InternalJugularVein",
        "_3rdMarginalCoronaryArtery",
        "BronchialArtery",
        "MidRightCoronaryArtery",
        "EsophagealArtery",
        "AVGrooveContinuationOfCircumflexArtery",
        "PulmonaryVein",
        "SubclavianArtery",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "_3rdRightPosterolateralCoronaryArtery",
        "ThyrocervicalTrunk",
        "PosterolateralBranchOfRightCoronaryArtery",
        "InferiorPhrenicArtery",
        "BrachiocephalicVein",
        "_3rdLeftPosterolateralCoronaryArtery",
        "ProximalCircumflexCoronaryArtery",
        "_2ndMarginalCoronaryArtery",
        "AorticArch",
        "ExternalJugularVein",
        "DorsalScapularArtery",
        "LeftMainCoronaryArteryOstium",
        "SuperiorVenaCava",
        "LeftMainCoronaryArtery",
        "_2ndLeftPosterolateralCoronaryArtery",
        "BrachiocephalicTrunk",
        "_2ndRightPosterolateralCoronaryArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "_1stMarginalCoronaryArtery",
        "PulmonaryTrunk",
        "_1stSeptalCoronaryArtery",
        "RightCoronaryArtery",
        "AscendingAorta",
        "AzygosVein",
        "_1stDiagonalCoronaryArtery",
        "PericardiophrenicArtery",
        "PulmonaryArtery",
        "BrachialArtery",
        "_2ndDiagonalCoronaryArtery",
        "CostocervicalTrunk",
        "DistalCircumflexCoronaryArtery",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "AxillaryVein",
        "CoronaryArteryGraft",
        "_1stRightPosterolateralCoronaryArtery",
    ],
}

name_for_cid[6118] = "SizeDescriptor"
cid_concepts[6118] = {
    "DCM": ["ExtremelySmall", "VerySmall", "TooSmall"],
    "SCT": ["Medium", "Enlarged", "Small", "Large"],
}

name_for_cid[6119] = "ChestBorderShape"
cid_concepts[6119] = {
    "DCM": ["Elliptic", "Lobulated", "Spiculated"],
    "SCT": ["RoundShape", "Irregular"],
}

name_for_cid[6120] = "ChestBorderDefinition"
cid_concepts[6120] = {
    "DCM": [
        "SharplyDemarcated",
        "PoorlyDemarcated",
        "SharplyDefined",
        "DistinctlyDefined",
        "WellDemarcated",
    ],
    "SCT": ["PoorlyDefined", "WellDefined", "Circumscribed"],
}

name_for_cid[6121] = "ChestOrientationDescriptor"
cid_concepts[6121] = {"SCT": ["Horizontal", "Vertical", "Oblique"]}

name_for_cid[6122] = "ChestContentDescriptor"
cid_concepts[6122] = {
    "DCM": ["Air", "Calcium"],
    "SCT": ["SoftTissue", "ForeignMaterialIodizedOilMercuryTalc", "Fat"],
}

name_for_cid[6123] = "ChestOpacityDescriptor"
cid_concepts[6123] = {
    "DCM": [
        "Linear",
        "Profusion",
        "SilhouetteSign",
        "Acinar",
        "AirSpace",
        "Fibronodular",
        "Fluffy",
    ]
}

name_for_cid[6124] = "LocationinChest"
cid_concepts[6124] = {
    "SCT": [
        "Peripheral",
        "Basal",
        "LowerLobeOfLung",
        "Central",
        "Subpleural",
        "AnteriorSegmentOfRightUpperLobe",
        "UpperZoneOfLung",
        "LowerZoneOfLung",
        "MiddleLobeOfRightLung",
        "Apical",
        "PosteriorSegmentOfRightUpperLobe",
        "MiddleZoneOfLung",
        "AnteriorSegmentOfLeftUpperLobe",
        "UpperLobeOfLung",
    ]
}

name_for_cid[6125] = "GeneralChestLocation"
cid_concepts[6125] = {"SCT": ["Peripheral", "Basal", "Central", "Apical"]}

name_for_cid[6126] = "LocationinLung"
cid_concepts[6126] = {
    "SCT": [
        "LowerLobeOfLung",
        "Subpleural",
        "UpperZoneOfLung",
        "LowerZoneOfLung",
        "MiddleLobeOfRightLung",
        "UpperLobeOfLung",
        "MiddleZoneOfLung",
    ]
}

name_for_cid[6127] = "SegmentLocationinLung"
cid_concepts[6127] = {
    "SCT": [
        "AnteriorSegmentOfLeftUpperLobe",
        "AnteriorSegmentOfRightUpperLobe",
        "PosteriorSegmentOfRightUpperLobe",
    ]
}

name_for_cid[6128] = "ChestDistributionDescriptor"
cid_concepts[6128] = {
    "DCM": [
        "BatWingDistribution",
        "ButterflyDistribution",
        "Centrilobular",
        "Coalescent",
        "Lobar",
    ],
    "SCT": [
        "Discoid",
        "Focal",
        "Generalized",
        "Multifocal",
        "Disseminated",
        "Systemic",
        "Diffuse",
        "Segmental",
    ],
}

name_for_cid[6129] = "ChestSiteInvolvement"
cid_concepts[6129] = {
    "DCM": ["Lobar"],
    "SCT": [
        "Lung",
        "HilumOfLung",
        "Mediastinum",
        "PleuralStructure",
        "ChestWall",
        "UpperAbdomen",
        "InterstitialTissue",
        "Bronchial",
        "Aorta",
    ],
}

name_for_cid[6130] = "SeverityDescriptor"
cid_concepts[6130] = {
    "DCM": ["HyperAcute"],
    "SCT": ["Moderate", "Mild", "SuddenOnset", "Chronic", "Subacute", "Severe"],
}

name_for_cid[6131] = "ChestTextureDescriptor"
cid_concepts[6131] = {"DCM": ["Inhomogeneous", "Homogeneous"]}

name_for_cid[6132] = "ChestCalcificationDescriptor"
cid_concepts[6132] = {
    "DCM": ["Target", "Fibrocalcific", "Flocculent"],
    "SCT": [
        "Nodular",
        "Ossification",
        "Laminated",
        "CoarsePopcornLikeCalcification",
        "EggshellCalcification",
    ],
}

name_for_cid[6133] = "ChestQuantitativeTemporalDifferenceType"
cid_concepts[6133] = {
    "SCT": ["FindingOfDifferenceInLocation", "FindingOfDifferenceInSize"]
}

name_for_cid[6134] = "ChestQualitativeTemporalDifferenceType"
cid_concepts[6134] = {
    "SCT": [
        "DifferenceInBorderDefinition",
        "DifferenceInDistribution",
        "NoSignificantChangesInTheFinding",
        "IncreaseInSize",
        "DifferenceInTexture",
        "DifferenceInBorderShape",
        "DifferenceInSiteInvolvement",
        "DifferenceInSubstance",
        "LessDefined",
        "FindingPartiallyRemoved",
        "MoreDefined",
        "DecreaseInSize",
    ]
}

name_for_cid[6135] = "ImageQualityFinding"
cid_concepts[6135] = {
    "DCM": [
        "OtherFailure",
        "MotionBlur",
        "UnknownFailure",
        "UnderExposed",
        "Positioning",
        "OverExposed",
        "NoImage",
        "DetectorArtifacts",
        "ArtifactsOtherThanGridOrDetectorArtifact",
        "MechanicalFailure",
        "ElectricalFailure",
        "SoftwareFailure",
        "GridArtifacts",
        "InappropriateImageProcessing",
    ],
    "RADLEX": ["BeamHardeningArtifact"],
}

name_for_cid[6136] = "ChestTypesofQualityControlStandard"
cid_concepts[6136] = {
    "DCM": [
        "PerformanceOfPediatricAndAdultThoracicCT",
        "PerformanceOfCTForDetectionOfPulmonaryEmbolismInAdults",
        "InstitutionallyDefinedQualityControlStandard",
        "PerformanceOfHighResolutionCTOfTheLungsInAdults",
        "PerformanceOfPediatricAndAdultChestRadiographyACR",
        "ACRPositionStatement",
    ]
}

name_for_cid[6137] = "CADAnalysisType"
cid_concepts[6137] = {
    "SCT": [
        "SpatialCollocationAnalysis",
        "SpatialProximityAnalysis",
        "TemporalCorrelation",
        "ImageQualityAnalysis",
    ]
}

name_for_cid[6138] = "ChestNonlesionObjectType"
cid_concepts[6138] = {
    "DCM": [
        "Portacath",
        "ChestTube",
        "CentralLine",
        "KidneyStent",
        "PancreaticStent",
        "NippleRing",
        "Coin",
        "FiducialMark",
    ],
    "SCT": [
        "JejunostomyTube",
        "Suture",
        "VenaCavaFilter",
        "HeartValveProsthesis",
        "Needle",
        "Jewelry",
        "Prosthesis",
        "FeedingTube",
        "Catheter",
        "EndotrachealTube",
        "Tracheotomy",
        "Staple",
        "ScarTissue",
        "UretericStent",
        "Bullet",
        "Pin",
        "CardiacPacemaker",
        "CardiacPacemakerLead",
        "CervicalCollar",
    ],
}

name_for_cid[6139] = "NonlesionModifier"
cid_concepts[6139] = {"SCT": ["External", "Internal"]}

name_for_cid[6140] = "CalculationMethod"
cid_concepts[6140] = {
    "DCM": [
        "UnspecifiedMethodOfCalculation",
        "AgatstonScoringMethod",
        "VolumeScoringMethod",
        "TwoDimensionalMethod",
        "MassScoringMethod",
        "ThreeDimensionalMethod",
    ],
    "SCT": ["Estimated"],
}

name_for_cid[6141] = "AttenuationCoefficientMeasurement"
cid_concepts[6141] = {
    "DCM": [
        "StandardDeviationOfAttenuationCoefficient",
        "AttenuationCoefficient",
        "MinimumAttenuationCoefficient",
        "MaximumAttenuationCoefficient",
        "MeanAttenuationCoefficient",
        "MedianAttenuationCoefficient",
    ]
}

name_for_cid[6142] = "CalculatedValue"
cid_concepts[6142] = {
    "DCM": [
        "MassOfParenchymalTissue",
        "MassOfBreast",
        "AreaOfVascularCalcification",
        "VolumeOfVascularCalcification",
        "PercentageOfVascularCalcification",
        "MassOfVascularCalcification",
        "AverageCalcificationDistanceInACalcificationCluster",
        "StandardDeviationDistanceOfCalcificationsInACluster",
        "BreastTissueDensity",
        "VolumeOfParenchymalTissue",
        "VolumeOfBreast",
        "CavityExtentAsPercentOfVolume",
        "CalcificationExtentAsPercentOfSurface",
        "CalcificationExtentAsPercentOfVolume",
    ],
    "SCT": ["CoronaryArteryCalciumScore"],
}

name_for_cid[6143] = "LesionResponse"
cid_concepts[6143] = {
    "DCM": [
        "TargetLesionCompleteResponse",
        "TargetLesionPartialResponse",
        "TargetLesionProgressiveDisease",
        "TargetLesionStableDisease",
        "NonTargetLesionCompleteResponse",
        "NonTargetLesionIncompleteResponseOrStableDisease",
        "NonTargetLesionProgressiveDisease",
    ]
}

name_for_cid[6144] = "RECISTDefinedLesionResponse"
cid_concepts[6144] = {
    "DCM": [
        "TargetLesionCompleteResponse",
        "TargetLesionPartialResponse",
        "TargetLesionProgressiveDisease",
        "TargetLesionStableDisease",
        "NonTargetLesionCompleteResponse",
        "NonTargetLesionIncompleteResponseOrStableDisease",
        "NonTargetLesionProgressiveDisease",
    ]
}

name_for_cid[6145] = "BaselineCategory"
cid_concepts[6145] = {
    "DCM": [
        "TargetLesionAtBaseline",
        "NonTargetLesionAtBaseline",
        "NonLesionAtBaseline",
    ]
}

name_for_cid[6146] = "TimePointType"
cid_concepts[6146] = {
    "DCM": ["Posttreatment", "Eligibility"],
    "SCT": [
        "RecurrenceOfTumor",
        "MaintenanceTherapy",
        "PreOperative",
        "PostOperative",
        "PostRadiation",
        "PostChemotherapy",
    ],
    "UMLS": ["Unscheduled", "Baseline", "Pretreatment", "Nadir"],
}

name_for_cid[6147] = "ResponseCriteria"
cid_concepts[6147] = {
    "DCM": ["RECIST1Point0", "RECIST1Point1", "WHO"],
    "NCIt": ["RANO"],
}

name_for_cid[6148] = "MediastinumFindingorFeature"
cid_concepts[6148] = {
    "DCM": [
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
    ],
    "SCT": ["FascialLayer", "AtrialSeptalDefect"],
}

name_for_cid[6149] = "MediastinumAnatomy"
cid_concepts[6149] = {
    "SCT": [
        "PulmonaryValve",
        "Esophagus",
        "ThoracicDuct",
        "AxillaryFascia",
        "Carina",
        "LeftMainBronchus",
        "CarotidBody",
        "TricuspidValve",
        "RightMainBronchus",
        "Heart",
        "ThymusGland",
        "Thyroid",
        "LigamentumArteriosum",
        "LymphNode",
        "Trachea",
        "MitralValve",
        "CostalCartilage",
        "EsophagealHiatus",
        "Ventricle",
        "TrapeziusMuscle",
    ]
}

name_for_cid[6151] = "BackgroundEchotexture"
cid_concepts[6151] = {
    "DCM": [
        "HomogeneousFatEchotexture",
        "HomogeneousFibroglandularEchotexture",
        "HeterogeneousEchotexture",
    ]
}

name_for_cid[6152] = "Orientation"
cid_concepts[6152] = {"DCM": ["Parallel", "NotParallel"]}

name_for_cid[6153] = "LesionBoundary"
cid_concepts[6153] = {"DCM": ["AbruptInterface", "EchogenicHalo"]}

name_for_cid[6154] = "EchoPattern"
cid_concepts[6154] = {
    "DCM": ["Anechoic", "Hyperechoic", "Complex", "Hypoechoic", "Isoechoic"]
}

name_for_cid[6155] = "PosteriorAcousticFeature"
cid_concepts[6155] = {
    "DCM": [
        "CombinedPosteriorEnhancementAndShadowing",
        "NoPosteriorAcousticFeatures",
        "PosteriorEnhancement",
        "PosteriorShadowing",
    ]
}

name_for_cid[6157] = "Vascularity"
cid_concepts[6157] = {
    "DCM": [
        "VascularityNotPresent",
        "VascularityNotAssessed",
        "VascularityPresentInLesion",
        "VascularityPresentImmediatelyAdjacentToLesion",
        "DiffuselyIncreasedVascularityInSurroundingTissue",
    ]
}

name_for_cid[6158] = "CorrelationtoOtherFinding"
cid_concepts[6158] = {
    "DCM": [
        "CorrelatesToPhysicalExaminationFindings",
        "CorrelatesToMammographyFindings",
        "CorrelatesToMRIFindings",
        "CorrelatesToUltrasoundFindings",
        "CorrelatesToOtherImagingFindings",
        "NoCorrelationToOtherImagingFindings",
        "NoCorrelationToClinicalFindings",
    ]
}

name_for_cid[6159] = "MalignancyType"
cid_concepts[6159] = {
    "DCM": ["InvasiveBreastCarcinoma", "OtherMalignancyType"],
    "SCT": ["NonInfiltratingIntraductalCarcinoma"],
}

name_for_cid[6160] = "BreastPrimaryTumorAssessmentFromAJCC"
cid_concepts[6160] = {
    "SCT": [
        "Pt3",
        "Ptx",
        "Ptis",
        "Pt4b",
        "Pt0",
        "Pt4c",
        "Pt1",
        "Pt1a",
        "Pt4a",
        "PtisDCIS",
        "Pt4",
        "Pt4d",
        "Pt2",
        "PtisPaget",
        "Pt1mi",
        "Pt1b",
        "Pt1c",
    ]
}

name_for_cid[6161] = "PathologicalRegionalLymphNodeAssessmentforBreast"
cid_concepts[6161] = {
    "SCT": ["Pn3b", "Pn0", "Pn2a", "Pn2", "Pn3", "Pn1", "Pn3c", "Pn3a", "Pnx", "Pn2b"]
}

name_for_cid[6162] = "AssessmentofMetastasisforBreast"
cid_concepts[6162] = {"SCT": ["Cm0", "MX", "Pm1"]}

name_for_cid[6163] = "MenstrualCyclePhase"
cid_concepts[6163] = {
    "DCM": ["_1stWeek", "_2ndWeek", "_3rdWeek"],
    "SCT": ["MenstruationPresent"],
}

name_for_cid[6164] = "TimeInterval"
cid_concepts[6164] = {
    "DCM": [
        "LesserThan3MonthsAgo",
        "_4MonthsTo1YearAgo",
        "GreaterThan1YearAgo",
        "NotSure",
    ]
}

name_for_cid[6165] = "BreastLinearMeasurement"
cid_concepts[6165] = {
    "DCM": [
        "Distance",
        "Height",
        "LineSegmentLength",
        "DistanceFromChestWall",
        "PathLength",
        "DistanceFromNipple",
        "DistanceFromSkin",
    ],
    "IBSI": [
        "LeastAxisIn3DLength",
        "MinorAxisIn3DLength",
        "MajorAxisIn3DLength",
        "Maximum3DDiameterOfAMesh",
    ],
    "SCT": [
        "DiameterOfCircumscribedCircle",
        "Radius",
        "Perimeter",
        "MinorAxis",
        "Width",
        "Diameter",
        "MajorAxis",
        "PerpendicularAxis",
        "LongAxis",
        "Depth",
        "ShortAxis",
        "Length",
        "Circumference",
    ],
}

name_for_cid[6166] = "CADGeometrySecondaryGraphicalRepresentation"
cid_concepts[6166] = {
    "DCM": [
        "OrthogonalLocationArcInnerMargin",
        "OrthogonalLocationArcOuterMargin",
        "OutlineOfLobulations",
        "InnerLimitsOfFuzzyMargin",
        "OuterLimitsOfFuzzyMargin",
        "OutlineOfSpiculations",
        "LinearSpiculation",
        "PixelatedSpiculations",
        "OrthogonalLocationArc",
    ],
    "SCT": ["LongAxis", "ShortAxis"],
}

name_for_cid[6170] = "LobeofLung"
cid_concepts[6170] = {
    "SCT": ["LowerLobeOfLung", "MiddleLobeOfRightLung", "UpperLobeOfLung"]
}

name_for_cid[6171] = "ZoneofLung"
cid_concepts[6171] = {"SCT": ["UpperZoneOfLung", "LowerZoneOfLung", "MiddleZoneOfLung"]}

name_for_cid[6200] = "ColonOverallAssessment"
cid_concepts[6200] = {
    "DCM": [
        "C0InadequateStudyAwaitingPriorComparisons",
        "C1NormalColonOrBenignLesion",
        "C2IntermediatePolypOrIndeterminateFinding",
        "C3PolypPossiblyAdvancedAdenoma",
        "C4ColonicMassLikelyMalignant",
    ]
}

name_for_cid[6201] = "ColonFindingorFeature"
cid_concepts[6201] = {
    "DCM": ["ImageQuality", "SelectedRegion", "NonLesion", "AnatomicNonColon"],
    "SCT": [
        "InvertedDiverticulum",
        "Lipoma",
        "IleocecalValve",
        "TumorOfColon",
        "Diverticulum",
        "Feces",
        "ContrastMedia",
        "IntraluminalFluid",
        "RectalMass",
        "OperativeSite",
        "PolypOfColon",
        "ColonicHaustra",
    ],
}

name_for_cid[6202] = "ColonFindingorFeatureModifier"
cid_concepts[6202] = {
    "DCM": ["ChestTube", "KidneyStent", "PancreaticStent", "Coin", "FiducialMark"],
    "SCT": [
        "JejunostomyTube",
        "Suture",
        "Lung",
        "IleostomyBag",
        "ColostomyBag",
        "VenaCavaFilter",
        "UrostomyBag",
        "StomaAppliance",
        "Appendix",
        "Needle",
        "InferiorVenaCava",
        "Prosthesis",
        "Jewelry",
        "RectalTube",
        "Bone",
        "Prostate",
        "Catheter",
        "Uterus",
        "Staple",
        "Testis",
        "UrethralCatheter",
        "Cervix",
        "Bladder",
        "ScarTissue",
        "UreteralStent",
        "Bullet",
        "BloodVessel",
        "Pin",
        "Aorta",
        "SeminalVesicle",
        "Rectum",
        "Ovary",
        "AppendicealStump",
        "Muscle",
        "Liver",
        "Spleen",
        "Kidney",
        "AdrenalGland",
    ],
}

name_for_cid[6203] = "ColonNonlesionObjectType"
cid_concepts[6203] = {
    "DCM": ["ChestTube", "KidneyStent", "PancreaticStent", "Coin", "FiducialMark"],
    "SCT": [
        "JejunostomyTube",
        "Suture",
        "IleostomyBag",
        "UrethralCatheter",
        "ColostomyBag",
        "VenaCavaFilter",
        "UrostomyBag",
        "StomaAppliance",
        "ScarTissue",
        "UreteralStent",
        "Needle",
        "Bullet",
        "Pin",
        "Prosthesis",
        "Jewelry",
        "RectalTube",
        "Catheter",
        "Staple",
    ],
}

name_for_cid[6204] = "AnatomicNoncolonFinding"
cid_concepts[6204] = {
    "SCT": [
        "Lung",
        "Testis",
        "Cervix",
        "Bladder",
        "Appendix",
        "BloodVessel",
        "Aorta",
        "InferiorVenaCava",
        "SeminalVesicle",
        "Rectum",
        "Ovary",
        "AppendicealStump",
        "Muscle",
        "Bone",
        "Liver",
        "Prostate",
        "Spleen",
        "Kidney",
        "AdrenalGland",
        "Uterus",
    ]
}

name_for_cid[6205] = "ClockfaceLocationforColon"
cid_concepts[6205] = {
    "SCT": [
        "_7OClockPosition",
        "_11OClockPosition",
        "_12OClockPosition",
        "_2OClockPosition",
        "_5OClockPosition",
        "_3OClockPosition",
        "_6OClockPosition",
        "_4OClockPosition",
        "_9OClockPosition",
        "_10OClockPosition",
        "_1OClockPosition",
        "_8OClockPosition",
    ]
}

name_for_cid[6206] = "RecumbentPatientOrientationforColon"
cid_concepts[6206] = {
    "SCT": ["LeftLateralDecubitus", "Supine", "RightLateralDecubitus", "Prone"]
}

name_for_cid[6207] = "ColonQuantitativeTemporalDifferenceType"
cid_concepts[6207] = {
    "SCT": [
        "FindingOfDifferenceInLocation",
        "DifferenceInAttenuation",
        "FindingOfDifferenceInSize",
    ]
}

name_for_cid[6208] = "ColonTypesofQualityControlStandard"
cid_concepts[6208] = {
    "DCM": [
        "InstitutionallyDefinedQualityControlStandard",
        "ACRPositionStatement",
        "ACRGuidelinePerformanceOfAdultCTColonography",
        "ACRStandardCTMedicalPhysicsPerformanceMonitoring",
    ]
}

name_for_cid[6209] = "ColonMorphologyDescriptor"
cid_concepts[6209] = {
    "SCT": ["Circumferential", "Sessile", "Flat", "Ulcer", "Pedunculated"]
}

name_for_cid[6210] = "LocationinIntestinalTract"
cid_concepts[6210] = {
    "SCT": [
        "Rectum",
        "SigmoidColon",
        "DescendingColon",
        "TransverseColon",
        "SplenicFlexureOfColon",
        "Cecum",
        "AscendingColon",
        "HepaticFlexureOfColon",
    ]
}

name_for_cid[6211] = "ColonCADMaterialDescription"
cid_concepts[6211] = {"SCT": ["SoftTissue", "BoneMatrix", "Air", "Fat"]}

name_for_cid[6212] = "CalculatedValueforColonFinding"
cid_concepts[6212] = {
    "DCM": ["PolypStalkWidth", "DistanceFromAnus"],
    "SCT": ["PolypSizeLargestDimension", "PolypStalkLength"],
}

name_for_cid[6300] = "ProstateAnatomy"
cid_concepts[6300] = {
    "SCT": [
        "LeftBasalPeripheralZoneOfProstate",
        "TransitionZoneOfTheProstate",
        "LeftPosteromedialBasalPeripheralZoneOfProstate",
        "MaleExternalUrethralSphincter",
        "RightApicalAnteriorFibromuscularStromaOfProstate",
        "LeftPosteromedialApicalPeripheralZoneOfProstate",
        "LeftPosterolateralMiddlePeripheralZoneOfProstate",
        "RightBasalTransitionZoneOfProstate",
        "LeftAnteriorApicalTransitionZoneOfProstate",
        "LeftMiddleTransitionZoneOfProstate",
        "LeftAnteriorBasalTransitionZoneOfProstate",
        "RightPosterolateralBasalPeripheralZoneOfProstate",
        "LeftApicalPeripheralZoneOfProstate",
        "LeftPosteriorMiddleTransitionZoneOfProstate",
        "LeftBasalAnteriorFibromuscularStromaOfProstate",
        "LeftPosterolateralApicalPeripheralZoneOfProstate",
        "RightSeminalVesicle",
        "RightPosteromedialMiddlePeripheralZoneOfProstate",
        "RightPosteromedialBasalPeripheralZoneOfProstate",
        "LeftApicalAnteriorFibromuscularStromaOfProstate",
        "RightBasalPeripheralZoneOfProstate",
        "RightPosteromedialApicalPeripheralZoneOfProstate",
        "BasalAnteriorFibromuscularStromaOfProstate",
        "LeftBasalPartTransitionZoneOfProstate",
        "PeripheralZoneOfTheProstate",
        "RightApicalPeripheralZoneOfProstate",
        "LeftPosterolateralBasalPeripheralZoneOfProstate",
        "RightPosterolateralApicalPeripheralZoneOfProstate",
        "RightPosteriorBasalTransitionZoneOfProstate",
        "RightAnteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddleTransitionZoneOfProstate",
        "RightLateralMiddlePeripheralZoneOfProstate",
        "AnteriorFibromuscularStromaOfProstate",
        "CentralZoneOfLeftHalfProstate",
        "LeftLateralMiddlePeripheralZoneOfProstate",
        "LeftPosteromedialMiddlePeripheralZoneOfProstate",
        "RightApicalTransitionZoneOfProstate",
        "RightBasalAnteriorFibromuscularStromaOfProstate",
        "MiddleAnteriorFibromuscularStromaOfProstate",
        "RightAnteriorBasalTransitionZoneOfProstate",
        "LeftPosteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorMiddleTransitionZoneOfProstate",
        "Prostate",
        "LeftPosteriorBasalTransitionZoneOfProstate",
        "RightAnteriorBasalPeripheralZoneOfProstate",
        "LeftMiddleAnteriorFibromuscularStromaOfProstate",
        "LeftAnteriorApicalPeripheralZoneOfProstate",
        "CentralZoneOfRightHalfProstate",
        "RightAnteriorMiddlePeripheralZoneOfProstate",
        "RightPosterolateralMiddlePeripheralZoneOfProstate",
        "LeftApicalTransitionZoneOfProstate",
        "ApicalAnteriorFibromuscularStromaOfProstate",
        "LeftSeminalVesicle",
        "SeminalVesicle",
        "RightAnteriorApicalPeripheralZoneOfProstate",
        "LeftAnteriorBasalPeripheralZoneOfProstate",
        "RightMiddleAnteriorFibromuscularStromaOfProstate",
        "RightPosteriorMiddleTransitionZoneOfProstate",
        "RightPosteriorApicalTransitionZoneOfProstate",
        "RightMiddleTransitionZoneOfProstate",
        "ProstaticUrethra",
    ]
}

name_for_cid[6301] = "ProstateSectorAnatomyfromPIRADSv2"
cid_concepts[6301] = {
    "SCT": [
        "CentralZoneOfLeftHalfProstate",
        "LeftPosteromedialMiddlePeripheralZoneOfProstate",
        "MaleExternalUrethralSphincter",
        "RightApicalAnteriorFibromuscularStromaOfProstate",
        "LeftPosteromedialApicalPeripheralZoneOfProstate",
        "LeftPosterolateralMiddlePeripheralZoneOfProstate",
        "RightBasalAnteriorFibromuscularStromaOfProstate",
        "RightAnteriorBasalTransitionZoneOfProstate",
        "LeftAnteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorBasalTransitionZoneOfProstate",
        "LeftPosteriorApicalTransitionZoneOfProstate",
        "RightPosterolateralBasalPeripheralZoneOfProstate",
        "LeftPosteriorMiddleTransitionZoneOfProstate",
        "LeftAnteriorMiddleTransitionZoneOfProstate",
        "LeftBasalAnteriorFibromuscularStromaOfProstate",
        "LeftPosteriorBasalTransitionZoneOfProstate",
        "RightAnteriorBasalPeripheralZoneOfProstate",
        "LeftMiddleAnteriorFibromuscularStromaOfProstate",
        "LeftPosterolateralApicalPeripheralZoneOfProstate",
        "RightSeminalVesicle",
        "LeftAnteriorApicalPeripheralZoneOfProstate",
        "CentralZoneOfRightHalfProstate",
        "RightPosteromedialMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddlePeripheralZoneOfProstate",
        "RightPosterolateralMiddlePeripheralZoneOfProstate",
        "LeftApicalAnteriorFibromuscularStromaOfProstate",
        "RightPosteromedialApicalPeripheralZoneOfProstate",
        "LeftSeminalVesicle",
        "RightAnteriorApicalPeripheralZoneOfProstate",
        "LeftAnteriorBasalPeripheralZoneOfProstate",
        "LeftPosterolateralBasalPeripheralZoneOfProstate",
        "RightMiddleAnteriorFibromuscularStromaOfProstate",
        "RightPosterolateralApicalPeripheralZoneOfProstate",
        "RightPosteriorBasalTransitionZoneOfProstate",
        "RightPosteriorMiddleTransitionZoneOfProstate",
        "RightAnteriorApicalTransitionZoneOfProstate",
        "RightPosteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddleTransitionZoneOfProstate",
    ]
}

name_for_cid[6302] = "ProstateSectorAnatomyfromEuropeanConcensus16Sector(Minimal)Model"
cid_concepts[6302] = {
    "SCT": [
        "CentralZoneOfRightHalfProstate",
        "RightPosteromedialMiddlePeripheralZoneOfProstate",
        "CentralZoneOfLeftHalfProstate",
        "LeftBasalPeripheralZoneOfProstate",
        "LeftLateralMiddlePeripheralZoneOfProstate",
        "LeftPosteromedialMiddlePeripheralZoneOfProstate",
        "RightBasalPeripheralZoneOfProstate",
        "LeftApicalTransitionZoneOfProstate",
        "MaleExternalUrethralSphincter",
        "RightApicalTransitionZoneOfProstate",
        "LeftBasalPartTransitionZoneOfProstate",
        "RightBasalTransitionZoneOfProstate",
        "LeftSeminalVesicle",
        "LeftMiddleTransitionZoneOfProstate",
        "LeftApicalPeripheralZoneOfProstate",
        "RightApicalPeripheralZoneOfProstate",
        "RightMiddleTransitionZoneOfProstate",
        "RightSeminalVesicle",
        "RightLateralMiddlePeripheralZoneOfProstate",
    ]
}

name_for_cid[6303] = "ProstateSectorAnatomyfromEuropeanConcensus27Sector(Optimal)Model"
cid_concepts[6303] = {
    "SCT": [
        "CentralZoneOfLeftHalfProstate",
        "LeftPosteromedialMiddlePeripheralZoneOfProstate",
        "MaleExternalUrethralSphincter",
        "LeftPosteromedialApicalPeripheralZoneOfProstate",
        "LeftPosterolateralMiddlePeripheralZoneOfProstate",
        "RightApicalTransitionZoneOfProstate",
        "MiddleAnteriorFibromuscularStromaOfProstate",
        "RightBasalTransitionZoneOfProstate",
        "LeftMiddleTransitionZoneOfProstate",
        "RightPosterolateralBasalPeripheralZoneOfProstate",
        "RightAnteriorBasalPeripheralZoneOfProstate",
        "LeftPosterolateralApicalPeripheralZoneOfProstate",
        "RightSeminalVesicle",
        "LeftAnteriorApicalPeripheralZoneOfProstate",
        "CentralZoneOfRightHalfProstate",
        "RightPosteromedialMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddlePeripheralZoneOfProstate",
        "RightPosterolateralMiddlePeripheralZoneOfProstate",
        "LeftApicalTransitionZoneOfProstate",
        "RightPosteromedialApicalPeripheralZoneOfProstate",
        "ApicalAnteriorFibromuscularStromaOfProstate",
        "BasalAnteriorFibromuscularStromaOfProstate",
        "LeftBasalPartTransitionZoneOfProstate",
        "LeftSeminalVesicle",
        "RightAnteriorApicalPeripheralZoneOfProstate",
        "LeftAnteriorBasalPeripheralZoneOfProstate",
        "LeftPosterolateralBasalPeripheralZoneOfProstate",
        "RightPosterolateralApicalPeripheralZoneOfProstate",
        "RightMiddleTransitionZoneOfProstate",
        "LeftAnteriorMiddlePeripheralZoneOfProstate",
    ]
}

name_for_cid[6304] = "ProstateSectorAnatomyfromPIRADSv2.1"
cid_concepts[6304] = {
    "SCT": [
        "CentralZoneOfLeftHalfProstate",
        "LeftPosteromedialBasalPeripheralZoneOfProstate",
        "LeftPosteromedialMiddlePeripheralZoneOfProstate",
        "MaleExternalUrethralSphincter",
        "RightApicalAnteriorFibromuscularStromaOfProstate",
        "LeftPosteromedialApicalPeripheralZoneOfProstate",
        "LeftPosterolateralMiddlePeripheralZoneOfProstate",
        "RightBasalAnteriorFibromuscularStromaOfProstate",
        "RightAnteriorBasalTransitionZoneOfProstate",
        "LeftAnteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorBasalTransitionZoneOfProstate",
        "LeftPosteriorApicalTransitionZoneOfProstate",
        "RightPosterolateralBasalPeripheralZoneOfProstate",
        "LeftPosteriorMiddleTransitionZoneOfProstate",
        "LeftAnteriorMiddleTransitionZoneOfProstate",
        "LeftBasalAnteriorFibromuscularStromaOfProstate",
        "LeftPosteriorBasalTransitionZoneOfProstate",
        "RightAnteriorBasalPeripheralZoneOfProstate",
        "LeftMiddleAnteriorFibromuscularStromaOfProstate",
        "LeftPosterolateralApicalPeripheralZoneOfProstate",
        "RightSeminalVesicle",
        "LeftAnteriorApicalPeripheralZoneOfProstate",
        "CentralZoneOfRightHalfProstate",
        "RightPosteromedialMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddlePeripheralZoneOfProstate",
        "RightPosterolateralMiddlePeripheralZoneOfProstate",
        "RightPosteromedialBasalPeripheralZoneOfProstate",
        "LeftApicalAnteriorFibromuscularStromaOfProstate",
        "RightPosteromedialApicalPeripheralZoneOfProstate",
        "LeftSeminalVesicle",
        "RightAnteriorApicalPeripheralZoneOfProstate",
        "LeftAnteriorBasalPeripheralZoneOfProstate",
        "LeftPosterolateralBasalPeripheralZoneOfProstate",
        "RightMiddleAnteriorFibromuscularStromaOfProstate",
        "RightPosterolateralApicalPeripheralZoneOfProstate",
        "RightPosteriorBasalTransitionZoneOfProstate",
        "RightPosteriorMiddleTransitionZoneOfProstate",
        "RightAnteriorApicalTransitionZoneOfProstate",
        "RightPosteriorApicalTransitionZoneOfProstate",
        "LeftAnteriorMiddlePeripheralZoneOfProstate",
        "RightAnteriorMiddleTransitionZoneOfProstate",
    ]
}

name_for_cid[6310] = "ProstateReportingSystem"
cid_concepts[6310] = {"DCM": ["PIRADSV2Point0", "PIRADSV2Point1"]}

name_for_cid[6311] = "MRSignalIntensity"
cid_concepts[6311] = {
    "DCM": [
        "ApparentDiffusionCoefficient",
        "DiffusionWeighted",
        "DynamicContrastEnhancedAcquisitionSubtractionImage",
        "DynamicContrastEnhancedAcquisition",
        "DiffusionWeightedAcquisitionHighestBValueImage",
        "T2WeightedMRSignalIntensity",
        "T1WeightedMRSignalIntensity",
    ]
}

name_for_cid[6312] = "CrosssectionalScanPlaneOrientation"
cid_concepts[6312] = {"SCT": ["Axial", "Oblique", "Sagittal", "Coronal"]}

name_for_cid[6313] = "HistoryofProstateDisease"
cid_concepts[6313] = {
    "DCM": ["UtricleCyst"],
    "SCT": ["ProstateCancer", "BenignProstateHyperplasia", "Prostatitis"],
}

name_for_cid[6314] = "ProstateMRIStudyQualityFinding"
cid_concepts[6314] = {
    "DCM": [
        "SuboptimalPatientPreparation",
        "IncompleteStudy",
        "StudyPerformedDidNotMatchRequest",
        "CoilSelectionConcern",
        "CoilPlacementConcern",
        "ProtocolNotFollowed",
    ],
    "RADLEX": ["SuboptimalPatientPositioning"],
}

name_for_cid[6315] = "ProstateMRISeriesQualityFinding"
cid_concepts[6315] = {
    "DCM": [
        "UnwantedSubtractionImages",
        "SubtractionImageMissing",
        "InadequateContrastEnhancement",
        "DistortionArtifactInTheAreaOfInterest",
        "ExpectedADCMapIsMissing",
        "SevereDistortionInTheAreaOfInterest",
        "ExpectedBValueIsMissing",
        "PoorContrastTiming",
        "PoorSNR",
        "PoorFOVSelection",
        "AnatomyCoverageIncomplete",
    ],
    "RADLEX": [
        "NonUniformSignalIntensityArtifact",
        "Smearing",
        "MotionRelatedArtifact",
        "Ghost",
        "TissueMagneticSusceptibilityEffect",
        "FerromagneticImplantMagneticSusceptibilityEffect",
        "ParallelImagingArtifact",
        "ChemicalShiftArtifact",
        "PhaseWraparoundArtifact",
    ],
}

name_for_cid[6316] = "MRImagingArtifact"
cid_concepts[6316] = {
    "RADLEX": [
        "TissueMagneticSusceptibilityEffect",
        "FerromagneticImplantMagneticSusceptibilityEffect",
        "NonUniformSignalIntensityArtifact",
        "ParallelImagingArtifact",
        "ChemicalShiftArtifact",
        "PhaseWraparoundArtifact",
        "Smearing",
        "MotionRelatedArtifact",
        "Ghost",
    ]
}

name_for_cid[6317] = "ProstateDCEMRIQualityFinding"
cid_concepts[6317] = {
    "DCM": [
        "PoorContrastTiming",
        "UnwantedSubtractionImages",
        "SubtractionImageMissing",
        "InadequateContrastEnhancement",
    ]
}

name_for_cid[6318] = "ProstateDWIMRIQualityFinding"
cid_concepts[6318] = {
    "DCM": [
        "DistortionArtifactInTheAreaOfInterest",
        "ExpectedADCMapIsMissing",
        "SevereDistortionInTheAreaOfInterest",
        "ExpectedBValueIsMissing",
    ]
}

name_for_cid[6319] = "AbdominalInterventionType"
cid_concepts[6319] = {
    "SCT": ["SurgicalProcedure", "DiagnosticIntent", "TherapeuticProcedure"]
}

name_for_cid[6320] = "AbdominopelvicIntervention"
cid_concepts[6320] = {
    "NCIt": [
        "FocusedUltrasoundAblation",
        "IrreversibleElectroporation",
        "Cryoablation",
    ],
    "SCT": ["OperationOnPelvicRegionOfTrunk", "Prostatectomy"],
}

name_for_cid[6321] = "ProstateCancerDiagnosticProcedure"
cid_concepts[6321] = {
    "SCT": ["DigitalExaminationOfRectum", "ProstateCancerAntigenMeasurement"]
}

name_for_cid[6322] = "ProstateCancerFamilyHistory"
cid_concepts[6322] = {
    "DCM": ["FamilyHistoryOfProstateCancer", "NoFamilyHistoryOfProstateCancer"],
    "SCT": ["FamilyHistoryUnknown"],
}

name_for_cid[6323] = "ProstateCancerTherapy"
cid_concepts[6323] = {
    "SCT": [
        "Prostatectomy",
        "ImplantationOfRadioactiveSeedIntoProstate",
        "AndrogenDeprivationTherapy",
    ]
}

name_for_cid[6324] = "ProstateMRIAssessment"
cid_concepts[6324] = {
    "RADLEX": [
        "PIRADS1VeryLow",
        "PIRADSXInadequateOrAbsent",
        "PIRADS2Low",
        "PIRADS5VeryHigh",
        "PIRADS4High",
        "PIRADS3Intermediate",
    ]
}

name_for_cid[6325] = "OverallAssessmentfromPIRADS®"
cid_concepts[6325] = {
    "RADLEX": [
        "PIRADS1VeryLow",
        "PIRADSXInadequateOrAbsent",
        "PIRADS2Low",
        "PIRADS5VeryHigh",
        "PIRADS4High",
        "PIRADS3Intermediate",
    ]
}

name_for_cid[6326] = "ImageQualityControlStandard"
cid_concepts[6326] = {
    "DCM": [
        "InstitutionallyDefinedQualityControlStandard",
        "MammographyQualityControlManual1999ACR",
        "PIRADS2Point1ProstateMRIAcquisitionRequirements",
        "Title21CFRSection900SubpartB",
        "PIRADS2Point0ProstateMRIAcquisitionRequirements",
        "ESUR2012ProstateMRIAcquisitionRequirements",
    ]
}

name_for_cid[6327] = "ProstateImagingIndication"
cid_concepts[6327] = {
    "DCM": [
        "ReviewOfAnOutsideStudy",
        "ActiveSurveillanceOfProstateCancer",
        "FamilyHistoryOfProstateCancer",
        "PreBiopsyLocalizationOfProstateLesion",
        "AssessChange",
    ],
    "I10": ["ElevatedProstateSpecificAntigen"],
    "SCT": ["Staging", "Brachytherapy"],
}

name_for_cid[6328] = "PIRADS®v2LesionAssessmentCategory"
cid_concepts[6328] = {
    "RADLEX": [
        "PIRADS3IntermediateLesion",
        "PIRADS2LowLesion",
        "PIRADS1VeryLowLesion",
        "PIRADSXInadequateOrAbsentLesion",
        "PIRADS5VeryHighLesion",
        "PIRADS4HighLesion",
    ]
}

name_for_cid[6329] = "PIRADS®v2T2WIPZLesionAssessmentCategory"
cid_concepts[6329] = {
    "RADLEX": [
        "PIRADS1T2WIPZVeryLow",
        "PIRADSXT2WIPZInadequateOrAbsent",
        "PIRADS5T2WIPZVeryHigh",
        "PIRADS4T2WIPZHigh",
        "PIRADS3T2WIPZIntermediate",
        "PIRADS2T2WIPZLow",
    ]
}

name_for_cid[6330] = "PIRADS®v2T2WITZLesionAssessmentCategory"
cid_concepts[6330] = {
    "RADLEX": [
        "PIRADS5T2WITZVeryHigh",
        "PIRADS4T2WITZHigh",
        "PIRADS3T2WITZIntermediate",
        "PIRADSXT2WITZInadequateOrAbsent",
        "PIRADS2T2WITZLow",
        "PIRADS1T2WITZVeryLow",
    ]
}

name_for_cid[6331] = "PIRADS®v2DWILesionAssessmentCategory"
cid_concepts[6331] = {
    "RADLEX": [
        "PIRADS4DWIHigh",
        "PIRADS3DWIIntermediate",
        "PIRADS2DWILow",
        "PIRADSXDWIInadequateOrAbsent",
        "PIRADS1DWIVeryLow",
        "PIRADS5DWIVeryHigh",
    ]
}

name_for_cid[6332] = "PIRADS®v2DCELesionAssessmentCategory"
cid_concepts[6332] = {
    "RADLEX": [
        "PIRADS_DCENegative",
        "PIRADS_DCEPositive",
        "PIRADSXDCEInadequateOrAbsent",
    ]
}

name_for_cid[6333] = "mpMRIAssessmentType"
cid_concepts[6333] = {
    "DCM": ["Margin"],
    "NCIt": ["Abnormality"],
    "RADLEX": ["EnhancementPattern", "SignalCharacteristic"],
    "SCT": ["Shape"],
}

name_for_cid[6334] = "mpMRIAssessmentTypefromPIRADS®"
cid_concepts[6334] = {
    "DCM": ["Margin"],
    "NCIt": ["Abnormality"],
    "RADLEX": ["EnhancementPattern", "SignalCharacteristic"],
    "SCT": ["Shape"],
}

name_for_cid[6335] = "mpMRIAssessmentValue"
cid_concepts[6335] = {
    "DCM": [
        "Linear",
        "LateContrastEnhancement",
        "PartiallyEncapsulated",
        "PersistentDelayedPhaseType1Curve",
        "ADCHypointense",
        "ADCHyperintense",
        "CompletelyEncapsulated",
        "WashoutDelayedPhaseType3Curve",
        "PlateauDelayedPhaseType2Curve",
        "Lobulated",
        "FocalAbnormality",
        "NegativeDCE",
        "PositiveDCE",
        "FocalContrastEnhancement",
        "DiffuseContrastEnhancement",
        "EarlyContrastEnhancement",
        "Homogeneous",
        "NonFocalAbnormality",
    ],
    "NCIt": ["IndexLesion"],
    "RADLEX": [
        "Heterogeneous",
        "MarkedlyHypointense",
        "T2Hypointensity",
        "DiffusionWeightedHypersensitivity",
        "DelayedPhase",
        "Lenticular",
        "ErasedCharcoalSign",
        "EarlyPhaseWashIn",
        "OrganizedChaos",
        "PersistentDelayedPhase",
        "WashoutDelayedPhase",
        "T2Hyperintensity",
        "PlateauDelayedPhase",
        "RestrictedDiffusion",
        "Hyperintense",
        "Isointense",
        "Hypointense",
        "NonCircumscribedMargin",
    ],
    "SCT": [
        "Atrophy",
        "Focal",
        "PathologicCalcification",
        "CircumscribedLesion",
        "Nodule",
        "Round",
        "Diffuse",
        "Irregular",
        "BenignProstateHyperplasia",
        "Hemorrhage",
        "Prostatitis",
        "Mass",
        "Regional",
        "Cyst",
        "WedgeShaped",
        "Fibrosis",
        "ObscuredLesion",
        "Encapsulated",
        "IndistinctLesion",
        "Multifocal",
        "Lesion",
        "OvoidShape",
        "SpiculatedLesion",
        "DropShaped",
    ],
}

name_for_cid[6336] = "MRIAbnormality"
cid_concepts[6336] = {
    "DCM": ["FocalAbnormality", "NonFocalAbnormality"],
    "NCIt": ["IndexLesion"],
    "SCT": [
        "Mass",
        "Regional",
        "Cyst",
        "Atrophy",
        "Focal",
        "PathologicCalcification",
        "Fibrosis",
        "Multifocal",
        "Nodule",
        "Lesion",
        "Diffuse",
        "BenignProstateHyperplasia",
        "Hemorrhage",
        "Prostatitis",
    ],
}

name_for_cid[6337] = "mpMRIProstateAbnormalityfromPIRADS®"
cid_concepts[6337] = {
    "DCM": ["FocalAbnormality", "NonFocalAbnormality"],
    "NCIt": ["IndexLesion"],
    "SCT": [
        "Mass",
        "Regional",
        "Cyst",
        "Atrophy",
        "Focal",
        "PathologicCalcification",
        "Fibrosis",
        "Multifocal",
        "Nodule",
        "Lesion",
        "Diffuse",
        "BenignProstateHyperplasia",
        "Hemorrhage",
        "Prostatitis",
    ],
}

name_for_cid[6338] = "mpMRIBenignProstateAbnormalityfromPIRADS®"
cid_concepts[6338] = {
    "SCT": [
        "Cyst",
        "Atrophy",
        "PathologicCalcification",
        "BenignProstateHyperplasia",
        "Fibrosis",
        "Hemorrhage",
        "Prostatitis",
    ]
}

name_for_cid[6339] = "MRIShapeCharacteristic"
cid_concepts[6339] = {
    "DCM": ["Linear", "Lobulated"],
    "RADLEX": ["Lenticular"],
    "SCT": ["Round", "WedgeShaped", "Irregular", "OvoidShape", "DropShaped"],
}

name_for_cid[6340] = "ProstateMRIShapeCharacteristicfromPIRADS®"
cid_concepts[6340] = {
    "DCM": ["Linear", "Lobulated"],
    "RADLEX": ["Lenticular"],
    "SCT": ["Round", "WedgeShaped", "Irregular", "OvoidShape", "DropShaped"],
}

name_for_cid[6341] = "MRIMarginCharacteristic"
cid_concepts[6341] = {
    "DCM": ["CompletelyEncapsulated", "PartiallyEncapsulated"],
    "RADLEX": ["ErasedCharcoalSign", "NonCircumscribedMargin"],
    "SCT": [
        "Encapsulated",
        "Irregular",
        "SpiculatedLesion",
        "CircumscribedLesion",
        "IndistinctLesion",
        "ObscuredLesion",
    ],
}

name_for_cid[6342] = "ProstateMRIMarginCharacteristicfromPIRADS®"
cid_concepts[6342] = {
    "DCM": ["CompletelyEncapsulated", "PartiallyEncapsulated"],
    "RADLEX": ["ErasedCharcoalSign", "NonCircumscribedMargin"],
    "SCT": [
        "Encapsulated",
        "Irregular",
        "SpiculatedLesion",
        "CircumscribedLesion",
        "IndistinctLesion",
        "ObscuredLesion",
    ],
}

name_for_cid[6343] = "MRISignalCharacteristic"
cid_concepts[6343] = {
    "DCM": ["Homogeneous", "ADCHypointense", "ADCHyperintense"],
    "RADLEX": [
        "Hyperintense",
        "Heterogeneous",
        "MarkedlyHypointense",
        "OrganizedChaos",
        "Isointense",
        "T2Hypointensity",
        "Hypointense",
        "RestrictedDiffusion",
        "DiffusionWeightedHypersensitivity",
        "T2Hyperintensity",
    ],
}

name_for_cid[6344] = "ProstateMRISignalCharacteristicfromPIRADS®"
cid_concepts[6344] = {
    "DCM": ["Homogeneous", "ADCHypointense", "ADCHyperintense"],
    "RADLEX": [
        "Hyperintense",
        "Heterogeneous",
        "MarkedlyHypointense",
        "OrganizedChaos",
        "Isointense",
        "T2Hypointensity",
        "Hypointense",
        "RestrictedDiffusion",
        "DiffusionWeightedHypersensitivity",
        "T2Hyperintensity",
    ],
}

name_for_cid[6345] = "MRIEnhancementPattern"
cid_concepts[6345] = {
    "DCM": [
        "LateContrastEnhancement",
        "PersistentDelayedPhaseType1Curve",
        "NegativeDCE",
        "PositiveDCE",
        "WashoutDelayedPhaseType3Curve",
        "PlateauDelayedPhaseType2Curve",
        "FocalContrastEnhancement",
        "DiffuseContrastEnhancement",
        "EarlyContrastEnhancement",
    ],
    "RADLEX": [
        "EarlyPhaseWashIn",
        "PersistentDelayedPhase",
        "WashoutDelayedPhase",
        "PlateauDelayedPhase",
        "DelayedPhase",
    ],
}

name_for_cid[6346] = "ProstateMRIEnhancementPatternfromPIRADS®"
cid_concepts[6346] = {
    "DCM": [
        "LateContrastEnhancement",
        "PersistentDelayedPhaseType1Curve",
        "NegativeDCE",
        "PositiveDCE",
        "WashoutDelayedPhaseType3Curve",
        "PlateauDelayedPhaseType2Curve",
        "FocalContrastEnhancement",
        "DiffuseContrastEnhancement",
        "EarlyContrastEnhancement",
    ],
    "RADLEX": [
        "EarlyPhaseWashIn",
        "PersistentDelayedPhase",
        "WashoutDelayedPhase",
        "PlateauDelayedPhase",
        "DelayedPhase",
    ],
}

name_for_cid[6347] = "ProstateMRIExtraprostaticFinding"
cid_concepts[6347] = {"SCT": ["Lymphadenopathy", "BonyMetastasis", "Entire"]}

name_for_cid[6348] = "ProstateMRIAssessmentofExtraprostaticAnatomicSite"
cid_concepts[6348] = {
    "RADLEX": ["NeurovascularBundleOfProstate"],
    "SCT": ["SeminalVesicle", "Rectum", "MaleExternalUrethralSphincter", "LymphNode"],
}

name_for_cid[6349] = "MRCoilType"
cid_concepts[6349] = {"RADLEX": ["EndorectalCoil"]}

name_for_cid[6350] = "EndorectalCoilFillSubstance"
cid_concepts[6350] = {
    "DCM": ["Perfluorocarbon"],
    "RADLEX": ["BariumSulfateSuspension"],
    "SCT": ["Air", "Water"],
}

name_for_cid[6351] = "ProstateRelationalMeasurement"
cid_concepts[6351] = {
    "DCM": ["DistanceFromNeurovascularBundle", "LesionCapsularContactLength"]
}

name_for_cid[6352] = "ProstateCancerDiagnosticBloodLabMeasurement"
cid_concepts[6352] = {
    "LN": ["ProstateSpecificAntigenDensity", "ProstateCancerAntigen"],
    "NCIt": ["_4kscore"],
}

name_for_cid[6353] = "ProstateImagingTypesofQualityControlStandard"
cid_concepts[6353] = {
    "DCM": [
        "InstitutionallyDefinedQualityControlStandard",
        "PIRADS2Point1ProstateMRIAcquisitionRequirements",
        "PIRADS2Point0ProstateMRIAcquisitionRequirements",
        "ESUR2012ProstateMRIAcquisitionRequirements",
    ]
}

name_for_cid[6401] = "NonlesionObjectTypePhysicalObject"
cid_concepts[6401] = {
    "DCM": ["OtherMarker"],
    "SCT": [
        "Suture",
        "PacemakerPulseGenerator",
        "CompressionPaddle",
        "JWire",
        "Bullet",
        "BBShotLeadPellet",
        "OpaqueMarker",
        "Clip",
        "Catheter",
        "IDPlate",
        "Collimator",
        "Implant",
        "Staple",
    ],
}

name_for_cid[6402] = "NonlesionObjectTypeSubstance"
cid_concepts[6402] = {"SCT": ["ContrastAgent"]}

name_for_cid[6403] = "NonlesionObjectTypeTissue"
cid_concepts[6403] = {"SCT": ["ScarTissue"]}

name_for_cid[6404] = "ChestNonlesionObjectTypePhysicalObject"
cid_concepts[6404] = {
    "DCM": [
        "Portacath",
        "ChestTube",
        "CentralLine",
        "KidneyStent",
        "PancreaticStent",
        "NippleRing",
        "Coin",
        "FiducialMark",
    ],
    "SCT": [
        "JejunostomyTube",
        "Suture",
        "VenaCavaFilter",
        "HeartValveProsthesis",
        "Needle",
        "Jewelry",
        "Prosthesis",
        "FeedingTube",
        "Catheter",
        "EndotrachealTube",
        "Tracheotomy",
        "Staple",
        "UretericStent",
        "Bullet",
        "Pin",
        "CardiacPacemaker",
        "CardiacPacemakerLead",
        "CervicalCollar",
    ],
}

name_for_cid[6405] = "ChestNonlesionObjectTypeTissue"
cid_concepts[6405] = {"SCT": ["ScarTissue"]}

name_for_cid[7000] = "DiagnosticImagingReportDocumentTitle"
cid_concepts[7000] = {
    "DCM": [
        "BreastImagingReport",
        "AdultEchocardiographyProcedureReport",
        "PediatricCardiacUltrasoundReport",
        "AdultCongenitalCardiacUltrasoundReport",
        "FetalCardiacUltrasoundReport",
    ],
    "LN": [
        "USCarotidArteriesReport",
        "USProstateTransrectalReport",
        "USVeinReport",
        "MRIHeadReport",
        "CTAbdomenReport",
        "NuclearMedicineReport",
        "USDopplerHeadVesselsReport",
        "CardiacCatheterizationReport",
        "USKidneyReport",
        "CTReport",
        "CTChestReport",
        "USArteryReport",
        "MammographyScreeningReport",
        "USLiverReport",
        "USDopplerUpperExtremityVesselsReport",
        "CardiacElectrophysiologyReport",
        "CTHeadReport",
        "PETScanReport",
        "RadiologyReport",
        "HolterStudyReport",
        "ExerciseStressTestReport",
        "USDopplerLowerExtremityVesselsReport",
        "MRISpineReport",
        "USTendonReport",
        "EchocardiographyReport",
        "XRayReport",
        "USSpleenReport",
        "USShoulderReport",
        "ECGReport",
        "USDopplerAbdominalVesselsReport",
        "UltrasoundReport",
        "FluoroscopyStudyReport",
        "USScrotumAndTesticleReport",
        "DEXASkeletalSystemStudyReport",
        "DiagnosticImagingReport",
        "NuclearMedicineThyroidScanReport",
        "DiagnosticInterventionalRadiologyReport",
        "USDopplerVesselsReport",
        "USPancreasReport",
        "USBreastReport",
        "USDopplerExtremityVesselsReport",
        "UltrasoundObstetricAndGynReport",
        "USThyroidReport",
        "MRIReport",
    ],
    "SCT": ["PeripheralVascularUltrasoundReport"],
}

name_for_cid[7001] = "DiagnosticImagingReportHeading"
cid_concepts[7001] = {
    "LN": [
        "Addendum",
        "ClinicalInformation",
        "MedicationsAdministered",
        "RadiationExposureAndProtectionInformation",
        "Complications",
        "PatientPresentation",
        "History",
        "Impressions",
        "CommunicationOfCriticalResults",
        "Conclusions",
        "Findings",
        "Recommendations",
        "PreviousFindings",
        "PriorProcedureDescriptions",
        "KeyImages",
        "IndicationsForProcedure",
        "Summary",
        "CurrentProcedureDescriptions",
        "Request",
    ]
}

name_for_cid[7002] = "DiagnosticImagingReportElement"
cid_concepts[7002] = {
    "DCM": [
        "PreviousFinding",
        "Conclusion",
        "Recommendation",
        "ProcedureDescription",
        "Impression",
        "Finding",
    ],
    "LN": ["PatientPresentation", "History", "Summary", "Request"],
    "SCT": ["ComplicationOfProcedure"],
}

name_for_cid[7003] = "DiagnosticImagingReportPurposeofReference"
cid_concepts[7003] = {
    "DCM": [
        "BestIllustrationOfFinding",
        "Baseline",
        "SourceOfMeasurement",
        "IllustrationOfROI",
    ]
}

name_for_cid[7004] = "WaveformPurposeofReference"
cid_concepts[7004] = {
    "DCM": [
        "SimultaneousRespiratoryWaveform",
        "SimultaneousArterialPulseWaveform",
        "SimultaneousECG",
        "SimultaneousVoiceNarrative",
        "SimultaneousDoppler",
        "SimultaneousHemodynamic",
        "SimultaneousPhonocardiographicWaveform",
    ]
}

name_for_cid[7005] = "ContributingEquipmentPurposeofReference"
cid_concepts[7005] = {
    "DCM": [
        "PortableMediaImporterEquipment",
        "FilmDigitizer",
        "DeIdentifyingEquipment",
        "ModifyingEquipment",
        "EnhancedMultiFrameConversionEquipment",
        "FrameExtractingEquipment",
        "VideoTapeDigitizerEquipment",
        "SynthesizingEquipment",
        "ProcessingEquipment",
        "AcquisitionEquipment",
        "DocumentDigitizerEquipment",
    ]
}

name_for_cid[7006] = "SRDocumentPurposeofReference"
cid_concepts[7006] = {
    "DCM": [
        "ExtendedRenderingForPresentation",
        "SourceDocument",
        "CompleteRenderingForPresentation",
        "PartialRenderingForPresentation",
        "EquivalentCDADocument",
    ]
}

name_for_cid[7008] = "MediaImport"
cid_concepts[7008] = {
    "DCM": [
        "InstancesImported",
        "StudiesImported",
        "MODImported",
        "DVDImported",
        "CDImported",
        "PaperDigitized",
        "VideoTapeDigitized",
        "CineFilmDigitized",
        "SheetFilmDigitized",
    ]
}

name_for_cid[7009] = "PurposeofReferencetoPredecessorReport"
cid_concepts[7009] = {
    "DCM": [
        "PartialReport",
        "AddendedReport",
        "PreliminaryReport",
        "ReplacedReport",
        "CurrentProcedureEvidence",
    ]
}

name_for_cid[7010] = "KeyObjectSelectionDocumentTitle"
cid_concepts[7010] = {
    "DCM": [
        "OfInterest",
        "RejectedForQualityReasons",
        "ForReferringProvider",
        "ForSurgery",
        "ForTeaching",
        "ContouringInputUsed",
        "DiagnosisInputUsed",
        "RTTreatmentPlanningInputUsed",
        "RTPrescriptionInputUsed",
        "PhysicianReviewInputUsed",
        "RTPlanSummationInputUsed",
        "PlanQualityAssuranceInputUsed",
        "PhysicistReviewInputUsed",
        "PatientSetupVerificationInputUsed",
        "MachineQualityAssuranceInputUsed",
        "ForConference",
        "ForTherapy",
        "ForPatient",
        "ForPeerReview",
        "ForResearch",
        "Manifest",
        "SignedManifest",
        "CompleteStudyContent",
        "SignedCompleteStudyContent",
        "CompleteAcquisitionContent",
        "SignedCompleteAcquisitionContent",
        "GroupOfFramesForDisplay",
        "RejectedForPatientSafetyReasons",
        "ForTumorRegistry",
        "ForClinicalTrialSubmission",
        "RTWorkflowInputUsed",
        "ForRTPlanSummation",
        "ForPhysicistReview",
        "ForPhysicianReview",
        "ForPlanQualityAssurance",
        "ForTumorBoard",
        "ForPatientSetupVerification",
        "ForMachineQualityAssurance",
        "IncorrectModalityWorklistEntry",
        "DataRetentionPolicyExpired",
        "ForReportAttachment",
        "ForLitigation",
        "CollectionOfPresentationStates",
        "ForPublicationExport",
        "ForResearchCollectionExport",
        "ForPlanComparison",
        "ForTeachingFileExport",
        "ForClinicalTrialExport",
        "RTTreatmentSessionResult",
        "PatientSetupVerificationResult",
        "RTTreatmentQAResult",
        "RTTreatmentCourseSummary",
        "ForSegmentation",
        "ForDiagnosis",
        "ForRTTreatmentPlanning",
        "ForRTPrescription",
        "QualityIssue",
        "BestInSet",
        "DosimetricResult",
        "RTVAudioAndStereoVideoRendition",
        "RTVStereoVideoRendition",
        "RTVAudioAndVideoRendition",
        "RTVRendition",
        "TumorRegistryInputUsed",
        "RTPlanningResult",
        "TumorBoardInputUsed",
        "RTTreatmentSimulationResult",
        "ClinicalTrialSubmissionInputUsed",
        "DiagnosticSourceImages",
        "ForRTWorkflow",
        "ForPrinting",
        "RegistrationResult",
        "SegmentationResult",
        "RTPrescriptionResult",
        "PrePlanningResult",
        "PlanComparisonInputUsed",
        "CoordinateAlignmentImageSeries",
        "RTPrescriptionResultForRTTreatmentPlanning",
    ]
}

name_for_cid[7011] = "RejectedforQualityReason"
cid_concepts[7011] = {
    "DCM": [
        "OtherFailure",
        "MotionBlur",
        "UnknownFailure",
        "UnderExposed",
        "DoubleExposure",
        "Positioning",
        "OverExposed",
        "NoImage",
        "DetectorArtifacts",
        "ArtifactsOtherThanGridOrDetectorArtifact",
        "MechanicalFailure",
        "ElectricalFailure",
        "ImageArtifacts",
        "SoftwareFailure",
        "GridArtifacts",
        "InappropriateImageProcessing",
    ]
}

name_for_cid[7012] = "BestinSet"
cid_concepts[7012] = {"DCM": ["Study", "Series", "PerformedProcedureStep", "StageView"]}

name_for_cid[7013] = "NonImageSourceInstancePurposeofReference"
cid_concepts[7013] = {
    "DCM": [
        "SourceMeasurement",
        "SourceRawData",
        "SourceReport",
        "SourceDeformableSpatialRegistration",
        "SourceRealWorldValueMap",
    ]
}

name_for_cid[7014] = "ExportAdditionalInformationDocumentTitle"
cid_concepts[7014] = {
    "DCM": [
        "ForTeachingFileExport",
        "ForClinicalTrialExport",
        "ForPublicationExport",
        "ForResearchCollectionExport",
    ]
}

name_for_cid[7015] = "ExportDelayReason"
cid_concepts[7015] = {
    "DCM": [
        "DelayExportUntilPatientDies",
        "DelayExportUntilPatientIsDischarged",
        "DelayExportUntilExpertReviewIsAvailable",
        "DelayExportUntilClinicalInformationIsAvailable",
        "DelayExportUntilFinalReportIsAvailable",
        "DelayExportUntilHistopathologyIsAvailable",
        "DelayExportUntilConfirmationOfDiagnosisIsAvailable",
        "DelayExportUntilOtherLaboratoryResultsAreAvailable",
    ]
}

name_for_cid[7016] = "LevelofDifficulty"
cid_concepts[7016] = {"DCM": ["PrimaryLevel", "AdvancedLevel", "IntermediateLevel"]}

name_for_cid[7017] = "CategoryofTeachingMaterialImaging"
cid_concepts[7017] = {
    "DCM": [
        "CardiovascularImagingSubjectMatter",
        "GenitourinaryImagingSubjectMatter",
        "GastrointestinalImagingSubjectMatter",
        "VascularAndInterventionalImagingSubjectMatter",
        "NeuroimagingSubjectMatter",
        "UltrasoundImagingSubjectMatter",
        "NuclearMedicineImagingSubjectMatter",
        "BreastImagingSubjectMatter",
        "PediatricImagingSubjectMatter",
        "PulmonaryImagingSubjectMatter",
        "MusculoskeletalImagingSubjectMatter",
    ]
}

name_for_cid[7018] = "MiscellaneousDocumentTitle"
cid_concepts[7018] = {"DCM": ["AdditionalTeachingFileInformation"]}

name_for_cid[7019] = "SegmentationNonImageSourcePurposeofReference"
cid_concepts[7019] = {"DCM": ["SourceRealWorldValueMap"]}

name_for_cid[7021] = "MeasurementReportDocumentTitle"
cid_concepts[7021] = {
    "DCM": [
        "ImagingMeasurementReport",
        "OncologyMeasurementReport",
        "DynamicContrastMRMeasurementReport",
        "PETMeasurementReport",
    ]
}

name_for_cid[7022] = "RadiotherapyPurposeofReference"
cid_concepts[7022] = {
    "DCM": [
        "DoseFromPriorTreatment",
        "RegistrationCreatedDuringTreatment",
        "RTTreatmentPlanForThePositionBeingVerified",
        "RegistrationUsedInPlanning",
    ]
}

name_for_cid[7023] = "RTProcessOutput"
cid_concepts[7023] = {
    "DCM": [
        "RTPlanningResult",
        "RTTreatmentSimulationResult",
        "DosimetricResult",
        "RTTreatmentSessionResult",
        "PatientSetupVerificationResult",
        "RTTreatmentQAResult",
        "RTTreatmentCourseSummary",
        "RTPrescriptionResult",
        "PrePlanningResult",
        "CoordinateAlignmentImageSeries",
        "RTPrescriptionResultForRTTreatmentPlanning",
    ]
}

name_for_cid[7024] = "RTProcessInput"
cid_concepts[7024] = {
    "DCM": [
        "ForRTPlanSummation",
        "ForPhysicistReview",
        "ForPhysicianReview",
        "ForPlanQualityAssurance",
        "ForPatientSetupVerification",
        "ForMachineQualityAssurance",
        "ForRTWorkflow",
        "ForRTTreatmentPlanning",
        "ForRTPrescription",
    ]
}

name_for_cid[7025] = "RTProcessInputUsed"
cid_concepts[7025] = {
    "DCM": [
        "RTTreatmentPlanningInputUsed",
        "RTPrescriptionInputUsed",
        "PhysicianReviewInputUsed",
        "RTPlanSummationInputUsed",
        "PlanQualityAssuranceInputUsed",
        "PhysicistReviewInputUsed",
        "PatientSetupVerificationInputUsed",
        "MachineQualityAssuranceInputUsed",
        "RTWorkflowInputUsed",
    ]
}

name_for_cid[7026] = "RadiotherapeuticDoseMeasurementDevice"
cid_concepts[7026] = {
    "DCM": [
        "IonChamberArray",
        "DiodeArray",
        "LiquidIonChamber",
        "Diode",
        "IonChamber",
        "OSLD",
        "RPLGD",
        "DiamondDetector",
        "_3DGel",
    ],
    "SCT": [
        "ThermoluminescentRadiationDosimeter",
        "MOSFETRadiationTherapyDosimetrySystemDosimeter",
        "DigitalImager",
        "MedicalXRayFilm",
    ],
}

name_for_cid[7027] = "SegmentedRadiotherapeuticDoseMeasurementDevice"
cid_concepts[7027] = {
    "DCM": [
        "IonChamberArray",
        "DiodeArray",
        "LiquidIonChamber",
        "Diode",
        "IonChamber",
        "OSLD",
        "RPLGD",
        "DiamondDetector",
    ],
    "SCT": [
        "MOSFETRadiationTherapyDosimetrySystemDosimeter",
        "DigitalImager",
        "ThermoluminescentRadiationDosimeter",
    ],
}

name_for_cid[7030] = "InstitutionalDepartment/Unit/Service"
cid_concepts[7030] = {
    "DCM": [
        "PediatricRadiology",
        "ThoracicRadiology",
        "AbdominalRadiology",
        "CardiovascularRadiology",
        "BiomedicalEngineering",
        "MedicalPhysics",
        "InformationTechnology",
        "MusculoskeletalRadiology",
    ],
    "SCT": [
        "Psychology",
        "Ultrasonography",
        "CardiacSurgery",
        "Dermatology",
        "GeneralMedicine",
        "MagneticResonanceImagingService",
        "CardiothoracicSurgery",
        "IntensiveCare",
        "GeneralSurgery",
        "TropicalMedicine",
        "AllergyAndImmunology",
        "Nephrology",
        "DiagnosticImaging",
        "Neurosurgery",
        "OralSurgery",
        "CardiacIntensiveCare",
        "Otorhinolaryngology",
        "ChildAndAdolescentPsychiatry",
        "PediatricSurgery",
        "Urology",
        "Cytology",
        "Radiotherapy",
        "ThoracicSurgery",
        "ClinicalBiochemistry",
        "GastrointestinalSurgery",
        "AccidentAndEmergency",
        "Endoscopy",
        "Physiotherapy",
        "Rheumatology",
        "NeonatalIntensiveCare",
        "Stroke",
        "MedicalIntensiveCare",
        "PainManagement",
        "PediatricIntensiveCare",
        "Ophthalmology",
        "Audiology",
        "ObstetricsAndGynecology",
        "Pulmonology",
        "MedicalMicrobiology",
        "TraumaSurgery",
        "HepatobiliarySurgery",
        "PalliativeCare",
        "PediatricOncology",
        "SurgicalIntensiveCare",
        "Obstetrics",
        "ClinicalOncology",
        "Endocrinology",
        "Rehabilitation",
        "BurnsIntensiveCare",
        "Surgery",
        "ColorectalSurgery",
        "Optometry",
        "ComputerizedTomographyService",
        "InfectiousDisease",
        "Cardiology",
        "NuclearMedicine",
        "DentalSurgery",
        "SpeechAndLanguageTherapy",
        "TransplantSurgery",
        "Anesthesiology",
        "PrimaryCareDepartment",
        "BreastSurgery",
        "HistopathologyDepartment",
        "Radiology",
        "HandSurgery",
        "Gynecology",
        "OrthopedicSurgery",
        "InterventionalRadiologyService",
        "PediatricMedicine",
        "Psychiatry",
        "Hematology",
        "Pathology",
        "PlasticSurgery",
        "Neurology",
        "EndocrineSurgery",
        "GeriatricMedicine",
        "VascularSurgery",
    ],
    "UMLS": ["Neuroradiology"],
}

name_for_cid[7031] = "ReasonforRemovalfromOperationalUse"
cid_concepts[7031] = {
    "DCM": [
        "QualityControlIntent",
        "RejectedForQualityReasons",
        "ReplacedByAlternateRepresentation",
        "ProcessedKeyObjectSelection",
        "RejectedForPatientSafetyReasons",
        "DataNotAccessibleFromStorage",
        "IncorrectModalityWorklistEntry",
        "DataRetentionPolicyExpired",
    ]
}

name_for_cid[7035] = "ActionableFindingClassification"
cid_concepts[7035] = {
    "RADLEX": [
        "ACRCategory2ActionableFinding",
        "ACRCategory1ActionableFinding",
        "ACRCategory3ActionableFinding",
    ]
}

name_for_cid[7036] = "ImageQualityAssessment"
cid_concepts[7036] = {
    "RADLEX": ["NonDiagnosticQuality", "LimitedQuality", "DiagnosticQuality"]
}

name_for_cid[7039] = "PediatricSizeCategory"
cid_concepts[7039] = {
    "SCT": [
        "BroselowLutenPurpleZone1011kg",
        "BroselowLutenYellowZone1214kg",
        "BroselowLutenWhiteZone1518kg",
        "BroselowLutenGreenZone3036kg",
        "BroselowLutenBlueZone1923kg",
        "BroselowLutenPinkZone67kg",
        "BroselowLutenOrangeZone2429kg",
        "BroselowLutenRedZone89kg",
    ]
}

name_for_cid[7040] = "BroselowLutenPediatricSizeCategory"
cid_concepts[7040] = {
    "SCT": [
        "BroselowLutenPurpleZone1011kg",
        "BroselowLutenYellowZone1214kg",
        "BroselowLutenWhiteZone1518kg",
        "BroselowLutenGreenZone3036kg",
        "BroselowLutenBlueZone1923kg",
        "BroselowLutenPinkZone67kg",
        "BroselowLutenOrangeZone2429kg",
        "BroselowLutenRedZone89kg",
    ]
}

name_for_cid[7041] = "CalciumScoringPatientSizeCategory"
cid_concepts[7041] = {
    "DCM": [
        "SmallLesserThan32Point0cmLateralThickness",
        "Medium32Point0To38Point0cmLateralThickness",
        "LargeGreaterThan38Point0cmLateralThickness",
    ]
}

name_for_cid[7042] = "CMDCTECCCalciumScoringPatientSizeCategory"
cid_concepts[7042] = {
    "DCM": [
        "SmallLesserThan32Point0cmLateralThickness",
        "Medium32Point0To38Point0cmLateralThickness",
        "LargeGreaterThan38Point0cmLateralThickness",
    ]
}

name_for_cid[7050] = "DeidentificationMethod"
cid_concepts[7050] = {
    "DCM": [
        "RetainUidsOption",
        "BasicApplicationConfidentialityProfile",
        "RetainSafePrivateOption",
        "CleanPixelDataOption",
        "RetainInstitutionIdentityOption",
        "CleanRecognizableVisualFeaturesOption",
        "CleanGraphicsOption",
        "CleanStructuredContentOption",
        "CleanDescriptorsOption",
        "RetainLongitudinalTemporalInformationFullDatesOption",
        "RetainLongitudinalTemporalInformationModifiedDatesOption",
        "RetainPatientCharacteristicsOption",
        "RetainDeviceIdentityOption",
    ]
}

name_for_cid[7060] = "EncapsulatedDocumentSourcePurposeofReference"
cid_concepts[7060] = {
    "DCM": [
        "SourceMeasurement",
        "SourceRawData",
        "SourceImage",
        "SourceReport",
        "SourceDeformableSpatialRegistration",
        "SourceRealWorldValueMap",
    ]
}

name_for_cid[7061] = "ModelDocumentTitle"
cid_concepts[7061] = {
    "DCM": [
        "US3DCAMModel",
        "SourceImage",
        "MixedModality3DCAMModel",
        "LaserScanning3DCAMModel",
        "PhotogrammetricImaging3DCAMModel",
    ],
    "LN": ["CT3DCAMModel", "MR3DCAMModel"],
}

name_for_cid[7062] = "PurposeofReferencetoPredecessor3DModel"
cid_concepts[7062] = {"DCM": ["EditedModel", "ComponentModel"]}

name_for_cid[7063] = "ModelScaleUnit"
cid_concepts[7063] = {"UCUM": ["Millimeter", "Meter", "Centimeter", "Micrometer"]}

name_for_cid[7064] = "ModelUsage"
cid_concepts[7064] = {
    "DCM": [
        "ToolFabrication",
        "PlanningIntent",
        "QualityControlIntent",
        "ImplantFabrication",
        "ProstheticFabrication",
        "SimulationIntent",
        "EducationalIntent",
    ],
    "SCT": ["DiagnosticIntent"],
}

name_for_cid[7070] = "RealTimeVideoRenditionTitle"
cid_concepts[7070] = {
    "DCM": [
        "RTVAudioAndStereoVideoRendition",
        "RTVStereoVideoRendition",
        "RTVAudioAndVideoRendition",
        "RTVRendition",
    ]
}

name_for_cid[7100] = "RCSRegistrationMethodType"
cid_concepts[7100] = {
    "DCM": [
        "ImageContentAndFiducialBasedAlignment",
        "VisualAlignment",
        "ImageContentBasedAlignment",
        "AcquisitionEquipmentAlignment",
        "FiducialAlignment",
        "FrameOfReferenceIdentity",
    ]
}

name_for_cid[7101] = "BrainAtlasFiducial"
cid_concepts[7101] = {
    "DCM": [
        "InterHemisphericPlane",
        "LeftHemisphereMostInferior",
        "LeftHemisphereMostSuperior",
        "LeftHemisphereMostPosterior",
        "LeftHemisphereMostAnterior",
        "RightHemisphereMostInferior",
        "RightHemisphereMostSuperior",
        "RightHemisphereMostPosterior",
        "RightHemisphereMostAnterior",
    ],
    "SCT": ["AnteriorCommissure", "PosteriorCommissure"],
}

name_for_cid[7110] = "FiducialCategory"
cid_concepts[7110] = {
    "DCM": ["FiducialMark"],
    "SCT": ["BodyReferencePointMarker", "BodySurfacePoint", "AnatomicalPoint"],
}

name_for_cid[7111] = "Fiducial"
cid_concepts[7111] = {
    "DCM": [
        "MRMarker",
        "InfraredReflectorMarker",
        "CoilMarker",
        "CylinderMarker",
        "TransponderMarker",
        "WireMarker",
        "InterHemisphericPlane",
        "VisibleReflectorMarker",
        "LeftHemisphereMostInferior",
        "Sphere",
        "LeftHemisphereMostSuperior",
        "LeftHemisphereMostPosterior",
        "LeftHemisphereMostAnterior",
        "RightHemisphereMostInferior",
        "RightHemisphereMostSuperior",
        "RightHemisphereMostPosterior",
        "RightHemisphereMostAnterior",
    ],
    "SCT": [
        "VascularCalcification",
        "RadioactiveImplant",
        "PosteriorCommissure",
        "ContrastMedia",
        "Ostium",
        "Catheter",
        "RenalArtery",
        "AnteriorCommissure",
        "VesselOrigin",
        "BBShotLeadPellet",
        "AorticBifurcation",
        "CommonIliacBifurcation",
        "Vein",
        "Clip",
        "Stent",
        "FibrousPlaque",
        "CollateralBranchOfVessel",
    ],
}

name_for_cid[7112] = "RadiotherapyFiducial"
cid_concepts[7112] = {
    "DCM": [
        "MRMarker",
        "InfraredReflectorMarker",
        "VisibleReflectorMarker",
        "CoilMarker",
        "CylinderMarker",
        "TransponderMarker",
        "WireMarker",
        "Sphere",
    ],
    "SCT": [
        "RadioactiveImplant",
        "BBShotLeadPellet",
        "Clip",
        "ContrastMedia",
        "Catheter",
    ],
}

name_for_cid[7140] = "BrainStructureforVolumetricMeasurement"
cid_concepts[7140] = {
    "DCM": [
        "WhiteMatterT2Hyperintensity",
        "WhiteMatterT1Hypointensity",
        "VentralDiencephalon",
    ],
    "SCT": [
        "Amygdala",
        "Cerebellum",
        "CavumOfSeptumPellucidum",
        "NucleusAccumbens",
        "CerebellarSubarachnoidSpace",
        "Hippocampus",
        "CerebellarWhiteMatter",
        "CranialCavity",
        "CranialSubarachnoidSpace",
        "FourthVentricle",
        "ThirdVentricle",
        "CerebralGrayMatter",
        "GlobusPallidus",
        "ThalamusPart",
        "CerebralWhiteMatter",
        "InferiorHornOfLateralVentricle",
        "LateralVentricle",
        "CaudateNucleus",
        "CerebellarCortex",
        "BrainStem",
        "Putamen",
        "IntracranialStructure",
    ],
}

name_for_cid[7150] = "SegmentationPropertyCategory"
cid_concepts[7150] = {
    "SCT": [
        "SpatialAndRelationalConcept",
        "Function",
        "Substance",
        "MorphologicallyAbnormalStructure",
        "Tissue",
        "AnatomicalStructure",
        "PhysicalObject",
        "BodySubstance",
    ]
}

name_for_cid[7151] = "SegmentationPropertyType"
cid_concepts[7151] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": [
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "IonChamberArray",
        "DiodeArray",
        "LiquidIonChamber",
        "Diode",
        "IonChamber",
        "OSLD",
        "RPLGD",
        "DiamondDetector",
        "Portacath",
        "ChestTube",
        "CentralLine",
        "KidneyStent",
        "PancreaticStent",
        "NippleRing",
        "Coin",
        "FiducialMark",
        "HeadFixationBoard",
        "AbdominalCompressionArch",
        "AbdominalCompressionBelt",
        "BreastBridge",
        "RegistrationInput",
        "Background",
        "SurfaceOfTheChoroidScleraInterface",
        "OuterSurfaceOfTheBM",
        "HandGrips",
        "OuterSurfaceOfTheCC",
        "HeelStop",
        "ShinRest",
        "SeatPan",
        "Mold",
        "HeadAndNeckMask",
        "HeadMask",
        "Headframe",
        "BodyFrame",
        "BreastBoard",
        "Cast",
        "WholeBodyPod",
        "VacuumMold",
        "RectalBalloon",
        "VaginalCylinder",
        "PosteriorSurfaceOfTheRPE",
        "SurfaceOfTheCenterOfTheRPE",
        "OuterSurfaceOfIPL",
        "OuterSurfaceOfGCL",
        "OuterSurfaceOfOPL",
        "OuterSurfaceOfINL",
        "SurfaceBetweenInnerAndOuterSegmentsOfThePhotoreceptors",
        "OuterSurfaceOfHFL",
        "AnteriorSurfaceOfTheRPE",
        "SurfaceOfTheInterdigitatingZoneBetweenRetinaAndRPE",
        "OuterSurfaceOfRNFL",
        "Sphere",
        "OtherMarker",
        "TissueNotOfInterest",
        "SecondaryPulmonaryLobule",
        "SingleSubjectExtractedFromGroup",
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
        "Lucency",
        "MidlungWindow",
        "CarinaAngle",
        "CentrilobularStructures",
        "AnteriorJunctionLine",
        "PosteriorJunctionLine",
        "Interface",
        "Line",
        "BrachytherapyChannelShield",
        "BrachytherapySourceApplicator",
        "BrachytherapyChannel",
    ],
    "FMA": [
        "ArcuateFasciculus",
        "LumenOfGallbladder",
        "SubcorticalGrayMatter",
        "WallOfGallbladder",
    ],
    "NCIt": ["NonEnhancingLesion", "ReferenceRegion", "EnhancingLesion"],
    "SCT": [
        "ProfundaFemorisVein",
        "LeftHypochondriacRegion",
        "SternocleidomastoidMuscle",
        "Ligament",
        "TruncusCoeliacus",
        "C3C4Disc",
        "InferiorLongitudinalFasciculus",
        "PelvicCavityTrue",
        "WristJoint",
        "IntermediateArteryRamus",
        "GenicularArtery",
        "LeftVentricleInflow",
        "FontanelOfSkull",
        "SigmoidColon",
        "RightFemoralArtery",
        "Intracranial",
        "JawRegion",
        "LobarArtery",
        "Wall",
        "DentalProsthesis",
        "ThighPerforator",
        "Pituitary",
        "MidRightCoronaryArtery",
        "IntraThoracic",
        "UpperLimb",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "ParietalBone",
        "Cartilage",
        "RightUpperQuadrantOfAbdomen",
        "Eye",
        "SuperficialPalmarArch",
        "Prostate",
        "FacialBones",
        "Back",
        "SoftTissue",
        "Duodenum",
        "_1stMarginalCoronaryArtery",
        "Larynx",
        "BonePin",
        "PeronealVein",
        "FemoralArtery",
        "Ovary",
        "Lesion",
        "Urethra",
        "Head",
        "AdrenalGland",
        "MitralValve",
        "Calyx",
        "PosteriorMedialTributary",
        "PulmonaryVeinGreatVessel",
        "ForeignBody",
        "Submental",
        "NucleusAccumbens",
        "PoplitealFossa",
        "PulmonaryArteryConduit",
        "InternalCarotidArteryC5Segment",
        "Nodule",
        "LevatoresCostarumMuscles",
        "FrontalLobe",
        "Adventitia",
        "Hypopharynx",
        "RightCoronaryArteryOstium",
        "HepaticVein",
        "MarginalCoronaryArtery",
        "InternalMammaryArtery",
        "BronchialArtery",
        "CouinaudHepaticSegmentIII",
        "ExternalCarotidArtery",
        "BiteBlock",
        "Neoplasm",
        "SystemicArtery",
        "IntraAbdominopelvic",
        "LesionOfBrain",
        "VomerBone",
        "UterusAndFallopianTubes",
        "PacemakerPulseGenerator",
        "Arachnoid",
        "VenaCavaFilter",
        "TarsalJoint",
        "LingualArtery",
        "NeoplasmSecondary",
        "AirwayStructure",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "DeltoidMuscle",
        "CerebralFornix",
        "Penis",
        "Baffle",
        "VentricularVein",
        "UretericStent",
        "Parathyroid",
        "Scapula",
        "Kidney",
        "OmentalBursa",
        "MuscleTissue",
        "InternalCarotidArtery",
        "L1Vertebra",
        "RightPosteriorAVCoronaryArtery",
        "FascialLayer",
        "CouinaudHepaticSegmentV",
        "RenalPelvis",
        "Thrombus",
        "VenousNetwork",
        "SubcostalMuscle",
        "CommonAtrium",
        "Ilium",
        "InternalCarotidArteryC6Segment",
        "AnteriorTibialArtery",
        "CranialNerve",
        "SkinOfAbdomen",
        "ConnectiveTissue",
        "CommonCarotidArtery",
        "InnermostIntercostalMuscles",
        "_3rdMarginalCoronaryArtery",
        "MastoidBone",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "CouinaudHepaticSegmentVI",
        "Thigh",
        "Trachea",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "BodyConduit",
        "Thalamus",
        "RightVentricleInflow",
        "LeftLowerQuadrantOfAbdomen",
        "SpinalCerebrospinalFluidPathway",
        "SesamoidBonesOfFoot",
        "Armrest",
        "C2C3Disc",
        "AbdominalWallMuscle",
        "SixthLumbarArtery",
        "BodyFluid",
        "Omentum",
        "SubmandibularGland",
        "CarotidBifurcation",
        "T3Vertebra",
        "InnominateArtery",
        "Extremity",
        "FacialArtery",
        "BoneStructureOfHeadAndOrNeck",
        "EpigastricRegion",
        "CervicoThoracicSpine",
        "DigitalImager",
        "RightLumbarRegion",
        "TeresMinorMuscle",
        "CommonFemoralVein",
        "SubstantiaNigra",
        "UpperInnerQuadrantOfBreast",
        "C5Vertebra",
        "ThoracicAorta",
        "RightInguinalRegion",
        "RightLowerQuadrantOfAbdomen",
        "CockettPerforatingVein",
        "RightMainBronchus",
        "AVGrooveContinuationOfCircumflexArtery",
        "Thumb",
        "GreatCardiacVein",
        "MiddleLobeOfRightLung",
        "T12Vertebra",
        "C3Vertebra",
        "L2L3Disc",
        "C5C6Disc",
        "_2ndLeftPosterolateralCoronaryArtery",
        "MiddleCerebralArteryM2Segment",
        "LacrimalBone",
        "BasilicVein",
        "HunterianPerforatingVein",
        "ProfundaFemorisArtery",
        "Pharynx",
        "LigamentumArteriosum",
        "Clothing",
        "Nasopharynx",
        "C6C7Disc",
        "SuprapulmonicValveArea",
        "SympatheticTrunk",
        "LobeOfLung",
        "OvarianVein",
        "ApexOfRightVentricle",
        "C1Vertebra",
        "LateralCalfPerforator",
        "Phantom",
        "FourthLumbarArtery",
        "ThirdLumbarArtery",
        "LimbicLobe",
        "SkinOfChest",
        "PrimitiveAorta",
        "LumenOfBloodVessel",
        "Hemorrhage",
        "AntecubitalVein",
        "SpinalCordGrayMatter",
        "Mandible",
        "Parasternal",
        "LeftMainBronchus",
        "SystemicVein",
        "OccipitalLobe",
        "C7T1Disc",
        "Pelvis",
        "SubclavianArtery",
        "Amygdala",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OpticChiasm",
        "OphthalmicArtery",
        "PancreaticDuct",
        "Nose",
        "Hand",
        "PectoralisMajorMuscle",
        "OvarianArtery",
        "Hippocampus",
        "SuperficialFemoralVein",
        "SuperiorLeftPulmonaryVein",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "BoneTissue",
        "Skull",
        "CerebralHemisphere",
        "AtrioventricularVein",
        "IDPlate",
        "OpticRadiation",
        "T7Vertebra",
        "EndoNasopharyngeal",
        "EsophagusStomachAndDuodenum",
        "Breast",
        "Meniscus",
        "AxillaryFascia",
        "PoplitealVein",
        "EmbolizationCoil",
        "PiaMater",
        "TricuspidValve",
        "RightAtrium",
        "VascularSclerosis",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "InnerEar",
        "Coccyx",
        "AtrialSeptalDefect",
        "DescendingAorta",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "OccipitalBone",
        "L2Vertebra",
        "Bullet",
        "Spine",
        "WhiteMatter",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "RadioactiveImplant",
        "Needle",
        "JuxtaposedAtrialAppendage",
        "Jejunum",
        "CarotidBody",
        "ChestAbdomenAndPelvis",
        "RightHepaticVein",
        "GastrocnemiusVein",
        "Scalp",
        "OrbitalStructure",
        "LeftMainBranchOfPortalVein",
        "BloodVessel",
        "Sclera",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "LiverAndBiliaryStructure",
        "_1stDiagonalCoronaryArtery",
        "Organ",
        "Perineum",
        "InteratrialSeptum",
        "Rectum",
        "Femur",
        "T10T11Disc",
        "LeftAtrium",
        "InterlobularArteryOfKidney",
        "CerebralArtery",
        "CouinaudHepaticSegmentVIII",
        "InferiorMesentericArtery",
        "IliocostalisMuscle",
        "LeftVentricularAssistDevice",
        "CervicalSpine",
        "IliacAndOrFemoralArtery",
        "SpinalCordWhiteMatter",
        "SmallIntestine",
        "SuperiorRightPulmonaryVein",
        "InferiorVenaCava",
        "IntraArticular",
        "Jewelry",
        "Axilla",
        "Thymus",
        "ELMExternalLimitingMembrane",
        "MeasuringRuler",
        "ParietalLobe",
        "IliacArtery",
        "T12L1Disc",
        "DistalRightCoronaryArtery",
        "SubmandibularArea",
        "PectoralisMinorMuscle",
        "OpticCanal",
        "SuperficialTemporalArtery",
        "MedialLemniscus",
        "InferiorCerebellarPeduncle",
        "RightMainBranchOfPortalVein",
        "SphenoidBone",
        "Spleen",
        "ChestAndAbdomen",
        "C4Vertebra",
        "LowerOuterQuadrantOfBreast",
        "Mediastinum",
        "Cerebellum",
        "FirstLumbarArtery",
        "PoplitealArtery",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "T9T10Disc",
        "EmbolizationParticulate",
        "DorsalScapularArtery",
        "L3L4Disc",
        "LeftVentricle",
        "CostalCartilage",
        "SaphenousVein",
        "Gallbladder",
        "_2ndRightPosterolateralCoronaryArtery",
        "RenalArtery",
        "PeritonealCavity",
        "UpperArm",
        "Chest",
        "GastroduodenalArtery",
        "Bile",
        "AzygosVein",
        "Liver",
        "Rib",
        "L5S1Disc",
        "Stent",
        "Implant",
        "CommonFemoralArtery",
        "SmallCardiacVein",
        "SuperficialPalmarVenousArch",
        "Vertebra",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "T7T8Disc",
        "VenaCava",
        "HilarArtery",
        "LeftCoronaryArtery",
        "Embolus",
        "ApexOfLung",
        "ThirdVentricle",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "SaphenofemoralJunction",
        "Anus",
        "FrontalBone",
        "Clip",
        "CommonIliacVein",
        "TrabeculaeCarnae",
        "BoneScrew",
        "PosteriorCerebralArteryP1Segment",
        "LumbarSpine",
        "ThyrocervicalTrunk",
        "Inflammation",
        "PrimitivePulmonaryArtery",
        "HilumOfLung",
        "SellaTurcica",
        "Scrotum",
        "InferiorLeftPulmonaryVein",
        "Intima",
        "LowerLeg",
        "RenalVein",
        "InternalIliacVein",
        "IlealVein",
        "Toe",
        "Skin",
        "Laser",
        "MiddleCardiacVein",
        "MiddleCerebralArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "CompressionPaddle",
        "CorpusStriatum",
        "Adenohypophysis",
        "PosteriorTibialVein",
        "MuscleOfLowerLimb",
        "AscendingAorta",
        "LeftUpperQuadrantOfAbdomen",
        "PericardiophrenicArtery",
        "CardiacPacemaker",
        "ExternalAuditoryCanal",
        "Patella",
        "OccipitalVein",
        "LumboSacralSpine",
        "BasilarArtery",
        "ParanasalSinus",
        "BrainCerebrospinalFluidPathway",
        "MidCircumflexCoronaryArtery",
        "CouinaudHepaticSegmentII",
        "CentralRetinalVein",
        "Prosthesis",
        "GastricArtery",
        "Blood",
        "SubscapularisMuscle",
        "Perfusion",
        "ExternalEar",
        "LeftVentricleOutflowTract",
        "Calcaneus",
        "SolealVein",
        "IntercostalArtery",
        "PeripheralNerve",
        "TeresMajorMuscle",
        "ScarTissue",
        "PatentDuctusArteriosus",
        "MesentericVein",
        "MaleInternalGenitalia",
        "PalatineBone",
        "CoronaryArtery",
        "RightBranchOfHepaticArtery",
        "DuraMater",
        "Flank",
        "Maxilla",
        "EsophagealHiatus",
        "CentralRetinalArtery",
        "Normal",
        "Neurohypophysis",
        "SegmentalArtery",
        "BrachialVein",
        "T10Vertebra",
        "HepaticArtery",
        "Tracheotomy",
        "PeronealArtery",
        "Staple",
        "RightHypochondriacRegion",
        "SuperiorMesentericVein",
        "LumbarArtery",
        "Cornea",
        "PercutaneousTransluminalAngioplastyBalloon",
        "L4Vertebra",
        "IliacVein",
        "CoronaryArteryGraft",
        "AcromioclavicularJoint",
        "UmbilicalRegion",
        "SupraspinatusMuscle",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "T3T4Disc",
        "SoftPalate",
        "T4Vertebra",
        "ExternalIntercostalMuscle",
        "CranialVenousSystem",
        "GiacominiVein",
        "Glue",
        "MiddleCerebellarPeduncle",
        "BloodClot",
        "MOSFETRadiationTherapyDosimetrySystemDosimeter",
        "Nerve",
        "FissureOfLung",
        "RightVentricle",
        "VagusNerve",
        "HeadAndNeck",
        "Bladder",
        "Vulva",
        "LargeIntestine",
        "EntorhinalCortex",
        "Forearm",
        "NeckChestAndAbdomen",
        "SuperiorMesentericArtery",
        "TemporalLobe",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "GreatSaphenousVeinOfThigh",
        "LeftHepaticVein",
        "L5Vertebra",
        "DetachableBalloon",
        "SuperficialFemoralArtery",
        "JWire",
        "_2ndMarginalCoronaryArtery",
        "T11T12Disc",
        "SuperiorCerebellarPeduncle",
        "AorticArch",
        "Carina",
        "Telencephalon",
        "LeftBranchOfHepaticArtery",
        "ArcuateArteryOfTheKidney",
        "Heart",
        "CephalicVein",
        "Lip",
        "GastricVein",
        "LeftMainCoronaryArtery",
        "InferiorRightHepaticVein",
        "CarotidSiphon",
        "SystemicVenousAtrium",
        "PlantarArterialArch",
        "L1L2Disc",
        "AnomalousPulmonaryVein",
        "LesserSaphenousVein",
        "AutonomicNerve",
        "GreatSaphenousVein",
        "T1T2Disc",
        "Necrosis",
        "Ureter",
        "UlnarVein",
        "MiddleEar",
        "InternalAuditoryCanal",
        "RightPulmonaryArtery",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "CorpusCallosum",
        "PerforatingArteryOfKidney",
        "Humerus",
        "T5T6Disc",
        "AorticFistula",
        "LeftLumbarRegion",
        "ILMInternalLimitingMembrane",
        "NasalBone",
        "Collimator",
        "Mass",
        "Cervix",
        "LeftAnteriorDescendingCoronaryArtery",
        "Artery",
        "Buttock",
        "FallopianTube",
        "Vein",
        "CerebrospinalFluid",
        "EsophagealArtery",
        "T8T9Disc",
        "RadialVein",
        "InferiorRightPulmonaryVein",
        "PulmonaryValveSinuses",
        "UterineArtery",
        "Gas",
        "EndoVesical",
        "MesentericArtery",
        "MuscleOfHead",
        "MiddleCerebralArteryM1Segment",
        "CouinaudHepaticSegmentIV",
        "ExternalJugularVein",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "FeedingTube",
        "Urine",
        "PosteriorArchVein",
        "AnteriorSpinalArtery",
        "Stretcher",
        "AnusRectumAndSigmoidColon",
        "SplenicVein",
        "Colon",
        "CentralNervousSystem",
        "NeoplasmPrimary",
        "SplenicArtery",
        "AccessoryRenalArtery",
        "Media",
        "BBShotLeadPellet",
        "SeminalVesicle",
        "Headrest",
        "DistalCircumflexCoronaryArtery",
        "Feces",
        "BoneOfUpperLimb",
        "DorsalisPedisArtery",
        "Vagina",
        "LongissimusMuscle",
        "LowerInnerQuadrantOfBreast",
        "Clavicle",
        "ParotidGland",
        "AorticIsthmus",
        "Capillary",
        "_1stLeftPosterolateralCoronaryArtery",
        "Subxiphoid",
        "Diaphragm",
        "VasDeferens",
        "T6T7Disc",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "Ileum",
        "SalivaryGland",
        "VertebralArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "SupraRenalAorta",
        "AdiposeTissue",
        "InternalJugularVein",
        "LacrimalArteryOfRightEye",
        "ChordaeTendineaeCordis",
        "OpaqueMarker",
        "SacroiliacJoint",
        "ScalenousAnteriorMuscle",
        "CardiacPacemakerLead",
        "EntireBody",
        "TesticularArtery",
        "HypogastricRegion",
        "HyoidBone",
        "C2Vertebra",
        "ThoracicDuct",
        "AtherectomyDevice",
        "BoydPerforatingVein",
        "BodyCavity",
        "Peritioneum",
        "CommonIliacArtery",
        "SupraclavicularRegionOfNeck",
        "Meninges",
        "Shoulder",
        "TransversusThoracis",
        "CardiacSeptumProsthesis",
        "CerebellarWhiteMatter",
        "InterventricularSeptum",
        "Cingulum",
        "LowerLobeOfLung",
        "TesticularVein",
        "AnteriorTibialVein",
        "LeftInguinalRegion",
        "RightCoronaryArtery",
        "Edema",
        "Putamen",
        "CostocervicalTrunk",
        "FifthLumbarArtery",
        "AxillaryVein",
        "SubclavianVein",
        "PulmonaryValve",
        "ExternalIliacArtery",
        "ProximalRightCoronaryArtery",
        "TemporalBone",
        "BoneOfLowerLimb",
        "Midbrain",
        "PosteriorCerebralArtery",
        "ErectorSpinaeMuscle",
        "DeepPalmarVenousArch",
        "T2T3Disc",
        "Mouth",
        "PharynxAndLarynx",
        "CarotidArtery",
        "SmallestCardiacVein",
        "UmbilicalArtery",
        "Cheek",
        "EndotrachealTube",
        "LymphNode",
        "RadialArtery",
        "T2Vertebra",
        "LeftFemoralArtery",
        "TrapeziusMuscle",
        "Backrest",
        "Esophagus",
        "RotationalAtherectomyDevice",
        "LateralCorticospinalTract",
        "BroadLigament",
        "CerebralAqueduct",
        "Insula",
        "PulmonaryVein",
        "MuscleOfNeck",
        "Lung",
        "SacroCoccygealSpine",
        "HeartValveProsthesis",
        "LeftMainCoronaryArteryOstium",
        "T9Vertebra",
        "CardiovascularSystem",
        "CouinaudHepaticSegmentI",
        "ElbowJoint",
        "Sternum",
        "NeckAndChest",
        "RightVentricleOutflowTract",
        "GlobusPallidus",
        "AbdomenAndPelvis",
        "UpperOuterQuadrantOfBreast",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "PericardialCavity",
        "PulmonaryArtery",
        "Defibrillator",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "C6Vertebra",
        "Thyroid",
        "EthmoidBone",
        "InfraRenalAorta",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "Neck",
        "CarotidBulb",
        "Finger",
        "InlayDentalRestoration",
        "InfraspinatusMuscle",
        "CouinaudHepaticSegmentVII",
        "SuperiorPhrenicArtery",
        "CommonVentricle",
        "RightPulmonaryVein",
        "Catheter",
        "Uterus",
        "TemporomandibularJoint",
        "Ventricle",
        "TracheaAndBronchus",
        "Testis",
        "Eyeball",
        "Face",
        "Tissue",
        "UrinarySystem",
        "BrainStem",
        "Foot",
        "MuscleOfUpperLimb",
        "PeripheralNervousSystem",
        "PinealGland",
        "UpperLobeOfLung",
        "PosterolateralBranchOfRightCoronaryArtery",
        "ProximalCircumflexCoronaryArtery",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "VertebralColumnAndCranium",
        "AbdominalAorta",
        "Tongue",
        "Table",
        "ChestWallMuscle",
        "GrayMatter",
        "Fibula",
        "SuperiorLongitudinalFasciculus",
        "InferiorCardiacVein",
        "InternalIntercostalMuscle",
        "LateralVentricle",
        "CaudateNucleus",
        "LacrimalArtery",
        "OccipitalArtery",
        "Brain",
        "WasteMaterial",
        "EmbolizationBall",
        "AtrialVein",
        "SuprasternalNotch",
        "AnteriorCardiacVein",
        "Pancreas",
        "BoneOfThorax",
        "DigitalArteryOfHand",
        "SecondLumbarArtery",
        "_1stRightPosterolateralCoronaryArtery",
        "BladderAndUrethra",
        "Suture",
        "Sacrum",
        "FemaleInternalGenitalia",
        "T8Vertebra",
        "MaleExternalGenitalia",
        "RetroperitonealSpace",
        "AnteriorMiddleCerebralArteryBifurcation",
        "SuperiorThyroidArtery",
        "CerebralCortex",
        "TruncusArteriosusCommunis",
        "T1Vertebra",
        "Tooth",
        "Joint",
        "GreatSaphenousVeinOfCalf",
        "LeftPulmonaryVein",
        "Aorta",
        "Diencephalon",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "T6Vertebra",
        "LatissimusDorsiMuscle",
        "CervicalCollar",
        "CommonHepaticArtery",
        "AnkleJoint",
        "C4C5Disc",
        "Bronchus",
        "PulmonaryChamberOfCorTriatriatum",
        "CommonBileDuct",
        "UlnarArtery",
        "LymphoidTissue",
        "LumbarRegion",
        "ThoracicSpine",
        "FourthVentricle",
        "T5Vertebra",
        "HipJoint",
        "OpticTract",
        "TrunkOfPulmonaryArtery",
        "L3Vertebra",
        "T4T5Disc",
        "Abscess",
        "BrachialArtery",
        "C7Vertebra",
        "_2ndDiagonalCoronaryArtery",
        "Hip",
        "PelvisAndLowerExtremities",
        "SerratusAnteriorMuscle",
        "Floor",
        "Chair",
        "LymphaticSystem",
        "IntraPelvic",
        "Stomach",
        "CircumflexCoronaryArtery",
        "AnteriorCommunicatingArtery",
        "InferiorMesentericVein",
        "MedianCubitalVein",
        "CoronarySinus",
        "Tendon",
        "TerminalInternalCarotidArtery",
        "BiliaryTract",
        "Atrium",
        "CerebralWhiteMatter",
        "Knee",
        "Cyst",
        "SubarachnoidSpace",
        "PulmonaryVeinConfluence",
        "GuidingCatheter",
        "EyeRegion",
        "ThermoluminescentRadiationDosimeter",
        "BrainVentricle",
        "UmbilicalVein",
        "InguinalRegion",
        "_3rdRightPosterolateralCoronaryArtery",
        "EndoNasal",
        "InferiorPhrenicArtery",
        "JejunostomyTube",
        "PortalVein",
        "PosteriorCerebralCommissure",
        "UncinateFasciculus",
        "AnteriorCerebralArtery",
        "L4L5Disc",
        "Ear",
        "LymphaticVessel",
        "SpinalCord",
        "T11Vertebra",
        "Pericardium",
        "PosteriorCerebralArteryP2Segment",
        "LeftAuricularAppendage",
        "PelvicCavityFalse",
        "CircleOfWillis",
        "SaphenopoplitealJunction",
        "_1stSeptalCoronaryArtery",
        "Eyelid",
        "DeepPalmarArchOfRadialArtery",
        "Pleura",
        "HypochondriacRegion",
        "SubcutaneousTissue",
        "SegmentOfLung",
        "DoddPerforatingVein",
    ],
}

name_for_cid[7152] = "CardiacStructureSegmentationType"
cid_concepts[7152] = {
    "SCT": [
        "Pericardium",
        "PericardialCavity",
        "Heart",
        "RightVentricle",
        "LeftVentricle",
        "Aorta",
    ]
}

name_for_cid[7153] = "CNSSegmentationType"
cid_concepts[7153] = {
    "FMA": ["ArcuateFasciculus", "SubcorticalGrayMatter"],
    "SCT": [
        "SpinalCordWhiteMatter",
        "InferiorLongitudinalFasciculus",
        "Midbrain",
        "BrainCerebrospinalFluidPathway",
        "NucleusAccumbens",
        "SubstantiaNigra",
        "MiddleCerebellarPeduncle",
        "PiaMater",
        "CorpusCallosum",
        "FrontalLobe",
        "LimbicLobe",
        "ParietalLobe",
        "ThirdVentricle",
        "CerebralCortex",
        "CerebralWhiteMatter",
        "EntorhinalCortex",
        "SpinalCordGrayMatter",
        "LateralCorticospinalTract",
        "SubarachnoidSpace",
        "BrainStem",
        "OccipitalLobe",
        "BrainVentricle",
        "MedialLemniscus",
        "Pituitary",
        "Diencephalon",
        "InferiorCerebellarPeduncle",
        "CerebrospinalFluid",
        "WhiteMatter",
        "CerebralAqueduct",
        "Insula",
        "TemporalLobe",
        "PinealGland",
        "DuraMater",
        "Thalamus",
        "Amygdala",
        "Cerebellum",
        "OpticChiasm",
        "Arachnoid",
        "Neurohypophysis",
        "PosteriorCerebralCommissure",
        "SuperiorCerebellarPeduncle",
        "UncinateFasciculus",
        "SpinalCerebrospinalFluidPathway",
        "Telencephalon",
        "Meninges",
        "SpinalCord",
        "Hippocampus",
        "GrayMatter",
        "CerebellarWhiteMatter",
        "SuperiorLongitudinalFasciculus",
        "CentralNervousSystem",
        "FourthVentricle",
        "CerebralFornix",
        "Cingulum",
        "GlobusPallidus",
        "LateralVentricle",
        "CorpusStriatum",
        "Adenohypophysis",
        "OpticTract",
        "CaudateNucleus",
        "Brain",
        "Putamen",
        "CerebralHemisphere",
        "OpticRadiation",
    ],
}

name_for_cid[7154] = "AbdominalSegmentationType"
cid_concepts[7154] = {
    "FMA": ["LumenOfGallbladder", "WallOfGallbladder"],
    "SCT": [
        "Abdomen",
        "SmallIntestine",
        "PancreaticDuct",
        "CouinaudHepaticSegmentV",
        "InferiorVenaCava",
        "CouinaudHepaticSegmentII",
        "CouinaudHepaticSegmentIV",
        "AbdominalAorta",
        "CouinaudHepaticSegmentVII",
        "Peritioneum",
        "RetroperitonealSpace",
        "AbdominalWallMuscle",
        "CouinaudHepaticSegmentI",
        "Omentum",
        "BiliaryTract",
        "Gallbladder",
        "SkinOfAbdomen",
        "PeritonealCavity",
        "BileDuct",
        "PancreaticDuctAndBileDuctSystems",
        "UrinarySystem",
        "CouinaudHepaticSegmentIII",
        "CouinaudHepaticSegmentVI",
        "Pancreas",
        "Liver",
        "Spleen",
        "CouinaudHepaticSegmentVIII",
        "Kidney",
        "AdrenalGland",
        "AbdominopelvicCavity",
    ],
}

name_for_cid[7155] = "ThoracicSegmentationType"
cid_concepts[7155] = {
    "SCT": [
        "Lung",
        "Mediastinum",
        "LowerLobeOfLung",
        "Clavicle",
        "Esophagus",
        "Thorax",
        "MiddleLobeOfRightLung",
        "BloodVessel",
        "Bronchus",
        "Diaphragm",
        "Pleura",
        "Thymus",
        "ChestCavity",
        "BoneOfThorax",
        "Rib",
        "ChestWallMuscle",
        "SkinOfChest",
        "Trachea",
        "UpperLobeOfLung",
        "Sternum",
        "ThoracicSpine",
    ]
}

name_for_cid[7156] = "VascularSegmentationType"
cid_concepts[7156] = {
    "SCT": [
        "CardiovascularSystem",
        "VascularSclerosis",
        "Thrombus",
        "Intima",
        "SystemicArtery",
        "Adventitia",
        "Media",
        "SystemicVein",
        "LumenOfBloodVessel",
    ]
}

name_for_cid[7157] = "DeviceSegmentationType"
cid_concepts[7157] = {
    "DCM": [
        "BrachytherapyChannel",
        "RectalBalloon",
        "VaginalCylinder",
        "Mold",
        "BrachytherapyChannelShield",
        "HeadAndNeckMask",
        "BrachytherapySourceApplicator",
        "HeadMask",
        "Headframe",
        "BodyFrame",
        "HeadFixationBoard",
        "HandGrips",
        "BreastBoard",
        "AbdominalCompressionArch",
        "HeelStop",
        "AbdominalCompressionBelt",
        "ShinRest",
        "Cast",
        "BreastBridge",
        "WholeBodyPod",
        "VacuumMold",
        "SeatPan",
    ],
    "SCT": [
        "Chair",
        "RadioactiveImplant",
        "Needle",
        "InlayDentalRestoration",
        "Armrest",
        "Table",
        "Stretcher",
        "Backrest",
        "Wall",
        "DentalProsthesis",
        "BonePin",
        "Defibrillator",
        "Headrest",
        "CardiacPacemaker",
        "BiteBlock",
        "BoneScrew",
        "LeftVentricularAssistDevice",
        "Stent",
        "Floor",
    ],
}

name_for_cid[7158] = "ArtifactSegmentationType"
cid_concepts[7158] = {"SCT": ["Table", "ForeignBody", "Clothing"]}

name_for_cid[7159] = "LesionSegmentationType"
cid_concepts[7159] = {
    "DCM": ["Background"],
    "NCIt": ["EnhancingLesion", "NonEnhancingLesion"],
    "SCT": [
        "Embolus",
        "NeoplasmPrimary",
        "Mass",
        "Normal",
        "Cyst",
        "Abscess",
        "Edema",
        "BloodClot",
        "Nodule",
        "Lesion",
        "Neoplasm",
        "NeoplasmSecondary",
        "Necrosis",
        "Hemorrhage",
        "Inflammation",
    ],
}

name_for_cid[7160] = "PelvicOrganSegmentationType"
cid_concepts[7160] = {
    "SCT": [
        "Testis",
        "Cervix",
        "Bladder",
        "Vulva",
        "PelvicCavityFalse",
        "PelvicCavityTrue",
        "FemaleInternalGenitalia",
        "PelvicCavityFalseAndOrTrue",
        "MaleExternalGenitalia",
        "FallopianTube",
        "MaleInternalGenitalia",
        "Rectum",
        "SeminalVesicle",
        "Ovary",
        "Pelvis",
        "Prostate",
        "VasDeferens",
        "Vagina",
        "Uterus",
    ]
}

name_for_cid[7161] = "PhysiologySegmentationType"
cid_concepts[7161] = {"SCT": ["Perfusion"]}

name_for_cid[7162] = "SurfaceProcessingAlgorithmFamily"
cid_concepts[7162] = {
    "DCM": [
        "ManualProcessing",
        "MultispectralProcessing",
        "ClusterAnalysis",
        "MultiScaleResolutionFiltering",
        "HistogramAnalysis",
        "MorphologicalOperations",
        "EdgeDetection",
        "AdaptiveFiltering",
        "NeighborhoodAnalysis",
        "DeformableModels",
        "ArtificialIntelligence",
    ]
}

name_for_cid[7165] = "AbstractSegmentationType"
cid_concepts[7165] = {
    "DCM": [
        "TissueNotOfInterest",
        "RegistrationInput",
        "Background",
        "SingleSubjectExtractedFromGroup",
    ],
    "NCIt": ["ReferenceRegion"],
    "SCT": ["Normal", "WasteMaterial", "Tissue"],
}

name_for_cid[7166] = "CommonTissueSegmentationType"
cid_concepts[7166] = {
    "SCT": [
        "Ligament",
        "SoftTissue",
        "Artery",
        "ConnectiveTissue",
        "LymphaticSystem",
        "Capillary",
        "Meniscus",
        "BloodVessel",
        "AdiposeTissue",
        "Tissue",
        "Organ",
        "Cartilage",
        "Vein",
        "BoneTissue",
        "Nerve",
        "Skin",
        "Blood",
        "SubcutaneousTissue",
        "LymphaticVessel",
        "LymphoidTissue",
        "LymphNode",
        "Tendon",
        "MuscleTissue",
    ]
}

name_for_cid[7167] = "PeripheralNervousSystemSegmentationType"
cid_concepts[7167] = {
    "SCT": [
        "CranialNerve",
        "PeripheralNervousSystem",
        "PeripheralNerve",
        "VagusNerve",
        "AutonomicNerve",
        "SympatheticTrunk",
    ]
}

name_for_cid[7168] = "BrainLesionSegmentationTypeWithNecrosis"
cid_concepts[7168] = {
    "DCM": ["Background"],
    "NCIt": ["NonEnhancingLesion", "EnhancingLesion"],
    "SCT": ["Normal", "Necrosis", "Edema"],
}

name_for_cid[7169] = "BrainLesionSegmentationTypeWithoutNecrosis"
cid_concepts[7169] = {
    "DCM": ["Background"],
    "NCIt": ["NonEnhancingLesion", "EnhancingLesion"],
    "SCT": ["Normal", "Edema"],
}

name_for_cid[7170] = "CouinaudLiverSegment"
cid_concepts[7170] = {
    "SCT": [
        "CouinaudHepaticSegmentIII",
        "CouinaudHepaticSegmentVII",
        "CouinaudHepaticSegmentVI",
        "CouinaudHepaticSegmentI",
        "CouinaudHepaticSegmentV",
        "CouinaudHepaticSegmentVIII",
        "CouinaudHepaticSegmentII",
        "CouinaudHepaticSegmentIV",
    ]
}

name_for_cid[7171] = "LiverSegmentationType"
cid_concepts[7171] = {
    "SCT": [
        "CouinaudHepaticSegmentIII",
        "CouinaudHepaticSegmentVII",
        "CouinaudHepaticSegmentVI",
        "CouinaudHepaticSegmentI",
        "CouinaudHepaticSegmentV",
        "CouinaudHepaticSegmentVIII",
        "CouinaudHepaticSegmentII",
        "CouinaudHepaticSegmentIV",
    ]
}

name_for_cid[7180] = "AbstractMultidimensionalImageModelComponentSemantic"
cid_concepts[7180] = {
    "DCM": [
        "ApparentDiffusionCoefficient",
        "DiffusionWeighted",
        "VolumeRatio",
        "DiffusionCoefficient",
        "WaterFatOutOfPhase",
        "FractionalOccupancySegmentation",
        "WaterFatInPhase",
        "FractionalProbabilisticSegmentation",
        "BinarySegmentation",
        "WaterFraction",
        "MRSignalIntensity",
        "XRayAttenuation",
        "FatFraction",
        "YBRICTCRComponent",
        "R2Coefficient",
        "YBRICTCBComponent",
        "ChiSquare",
        "YBRICTYComponent",
        "DW",
        "YBRPARTIALCRComponent",
        "AIC",
        "YBRPARTIALCBComponent",
        "BIC",
        "YBRPARTIALYComponent",
        "Echogenicity",
        "YBRRCTCRComponent",
        "YBRRCTCBComponent",
        "YBRRCTYComponent",
        "RelativeLinearStoppingPower",
        "RGBRComponent",
        "UltrasoundAttenuation",
        "TauM",
        "SpeedOfSound",
        "Vp",
        "Perfusion",
        "Elasticity",
        "EquivalentDose",
        "GradientWeightedClassActivation",
        "ClassActivation",
        "AbsorbedDose",
        "YBRFULLCRComponent",
        "Saliency",
        "YBRFULLCBComponent",
        "YBRFULLYComponent",
        "RGBBComponent",
        "RGBGComponent",
        "CreatineCholineCitrateRatio",
        "MagnetizationTransferRatio",
        "IAUC90BN",
        "IAUC180BN",
        "CreatineAndCholine",
        "LipidAndLactate",
        "SpatialDisplacementYComponent",
        "SpatialDisplacementXComponent",
        "NuclearMedicineTomographicActivity",
        "IAUC",
        "NuclearMedicineProjectionActivity",
        "IAUC60",
        "IAUC90",
        "IAUC180",
        "IAUCBN",
        "IAUC60BN",
        "FlowVariance",
        "FlowVelocity",
        "TissueVelocity",
        "IndexedHemodynamicResistance",
        "HemodynamicResistance",
        "SpatialDisplacementZComponent",
        "AttenuationCoefficient",
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "NAcetylaspartateCholineRatio",
        "Tmax",
        "R2Star",
        "VolumetricDiffusionDxzComponent",
        "MagneticSusceptibility",
        "VolumetricDiffusionDxyComponent",
        "VolumetricDiffusionDxxComponent",
        "RelativeRegionalBloodFlow",
        "RelativeRegionalBloodVolume",
        "Ktrans",
        "Kep",
        "Ve",
        "BloodOxygenationLevel",
        "T2StarWeightedDynamicContrastEnhancedMRSignalIntensity",
        "T2WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodFlow",
        "T1WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodVolume",
        "VolumetricDiffusionDzzComponent",
        "OxygenExtractionFraction",
        "R1",
        "VolumetricDiffusionDyzComponent",
        "R2",
        "VolumetricDiffusionDyyComponent",
        "VelocityEncoded",
        "ZScore",
        "ContrastAgentAngioMRSignalIntensity",
        "SpinTaggingPerfusionMRSignalIntensity",
        "RelativeAnisotropy",
        "FractionalAnisotropy",
        "FieldMapMRSignalIntensity",
        "T2StarWeightedMRSignalIntensity",
        "T2WeightedMRSignalIntensity",
        "T1WeightedMRSignalIntensity",
        "ProtonDensityWeightedMRSignalIntensity",
        "TimeOfFlightAngioMRSignalIntensity",
        "T1",
        "T2Star",
        "T2",
        "TimeCourseOfSignal",
        "TemperatureEncoded",
        "StudentTTest",
        "TimeToPeak",
        "SignalToNoise",
        "TemporalDerivativeThreshold",
        "MaximumSlope",
        "MaximumDifference",
        "TracerConcentration",
        "TimeOfPeakConcentration",
        "BolusArrivalTime",
        "TimeOfLeadingHalfPeakConcentration",
        "MeanTransitTime",
        "KurtosisDiffusionCoefficient",
        "GammaDistributionScaleParameter",
        "NegativeEnhancementIntegral",
        "GammaDistributionShapeParameter",
        "GammaDistributionMode",
        "DistributedDiffusionCoefficient",
        "RCoefficient",
        "AnomalousExponentParameter",
        "ProtonDensity",
        "SignalChange",
        "Suvbsa",
        "Suvibw",
        "SuvlbmJanma",
        "SuvlbmJames128",
        "MonoExponentialApparentDiffusionCoefficient",
        "SlowDiffusionCoefficient",
        "FastDiffusionCoefficient",
        "FastDiffusionCoefficientFraction",
        "StandardizedUptakeValue",
        "Suvbw",
        "Suvlbm",
    ],
    "SCT": [
        "Fat",
        "Lipid",
        "NAcetylaspartate",
        "PowerDoppler",
        "Creatine",
        "Glutamine",
        "Tuarine",
        "Citrate",
        "Lactate",
        "Inositol",
        "Water",
        "Choline",
    ],
}

name_for_cid[7181] = "AbstractMultidimensionalImageModelComponentUnit"
cid_concepts[7181] = {
    "UCUM": [
        "SquareCentimeterPerMilliliter",
        "SquareMillimeterPerSecond",
        "CubicCentimeter",
        "SquareCentimeter",
        "SquareMicrometerPerMillisecond",
        "StandardizedUptakeValueBodySurfaceArea",
        "StandardizedUptakeValueLeanBodyMassJanma",
        "Millitesla",
        "Sv",
        "SecondPerSquareMillimeter",
        "CubicMillimeter",
        "SquareMillimeter",
        "MilligramsPerMinutePerMilliliter",
        "StandardizedUptakeValueIdealBodyWeight",
        "_10EMinus6SquareMillimetersPerSecond",
        "BecquerelsPerMilliliter",
        "Micrometer",
        "MillimolesPerKilogramWetWeight",
        "Gy",
        "MicromolePerMinutePerMilliliter",
        "Decibel",
        "ArbitraryUnit",
        "WoodUM2",
        "DynScm5",
        "MilliliterPerMinutePerGram",
        "SquareMicrometerPerSecond",
        "StandardizedUptakeValueBodyWeight",
        "MilliliterPer100Milliliter",
        "MillimeterPerSecond",
        "MicromolePerMilliliter",
        "ProportionalToCountsPerSecond",
        "Centimeter",
        "MilliliterPerMinute",
        "CountsPerSecond",
        "Ppm",
        "Second",
        "PRUPerSquareMeter",
        "StandardizedUptakeValueLeanBodyMassJames128Multiplier",
        "WoodU",
        "MilliliterPerGram",
        "ProportionalToCounts",
        "Kilopascal",
        "NumberParticlesPer100GramOfTissue",
        "MilliliterPer100GramPerMinute",
        "PerMinute",
        "PerCentimeter",
        "CubicDecimeter",
        "Millimeter",
        "CentimeterPerSecond",
        "PRU",
        "Ratio",
        "CubicMicrometer",
        "SquareMicrometer",
        "Counts",
        "Millisecond",
        "DynScm5M2",
        "Percent",
        "PerSecond",
        "NoUnits",
        "HounsfieldUnit",
        "MilliliterPerSecond",
        "DegreesCelsius",
        "MillimetersHg",
        "StandardizedUptakeValueLeanBodyMassJames",
        "Hertz",
    ]
}

name_for_cid[7182] = "AbstractMultidimensionalImageModelDimensionSemantic"
cid_concepts[7182] = {"DCM": ["LinearDisplacement", "Angle", "Time", "PhotonEnergy"]}

name_for_cid[7183] = "AbstractMultidimensionalImageModelDimensionUnit"
cid_concepts[7183] = {
    "UCUM": [
        "Millimeter",
        "Radian",
        "Degree",
        "Centimeter",
        "Micrometer",
        "Millisecond",
        "Second",
    ]
}

name_for_cid[7184] = "AbstractMultidimensionalImageModelAxisDirection"
cid_concepts[7184] = {
    "DCM": [
        "SeptumWallAxis",
        "AnteriorInferiorAxis",
        "ApexBaseAxis",
        "AnteriorPosteriorAxis",
        "HeadFootAxis",
        "LeftRightAxis",
    ]
}

name_for_cid[7185] = "AbstractMultidimensionalImageModelAxisOrientation"
cid_concepts[7185] = {
    "DCM": [
        "LeftToRight",
        "RightToLeft",
        "WallToSeptum",
        "SeptumToWall",
        "InferiorToAnterior",
        "AnteriorToInferior",
        "BaseToApex",
        "ApexToBase",
        "PosteriorToAnterior",
        "AnteriorToPosterior",
        "FootToHead",
        "HeadToFoot",
    ]
}

name_for_cid[
    7186
] = "AbstractMultidimensionalImageModelQualitativeDimensionSampleSemantic"
cid_concepts[7186] = {
    "DCM": [
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "NAcetylaspartateCholineRatio",
        "VolumetricDiffusionDxzComponent",
        "RGBRComponent",
        "VolumetricDiffusionDxyComponent",
        "VolumetricDiffusionDxxComponent",
        "YBRFULLCRComponent",
        "YBRFULLCBComponent",
        "VolumetricDiffusionDzzComponent",
        "YBRFULLYComponent",
        "VolumetricDiffusionDyzComponent",
        "RGBBComponent",
        "VolumetricDiffusionDyyComponent",
        "RGBGComponent",
        "CreatineCholineCitrateRatio",
        "CreatineAndCholine",
        "LipidAndLactate",
        "YBRICTCRComponent",
        "YBRICTCBComponent",
        "YBRICTYComponent",
        "YBRPARTIALCRComponent",
        "YBRPARTIALCBComponent",
        "YBRPARTIALYComponent",
        "YBRRCTCRComponent",
        "YBRRCTCBComponent",
        "YBRRCTYComponent",
    ],
    "SCT": [
        "Inositol",
        "Citrate",
        "Creatine",
        "Lactate",
        "Glutamine",
        "Lipid",
        "Choline",
        "NAcetylaspartate",
        "Tuarine",
    ],
}

name_for_cid[7191] = "TissueSegmentationPropertyType"
cid_concepts[7191] = {
    "SCT": [
        "Ligament",
        "SoftTissue",
        "Artery",
        "ConnectiveTissue",
        "LymphaticSystem",
        "Capillary",
        "Meniscus",
        "ScarTissue",
        "BloodVessel",
        "AdiposeTissue",
        "Tissue",
        "Organ",
        "Cartilage",
        "Vein",
        "BoneTissue",
        "Nerve",
        "Skin",
        "Blood",
        "SubcutaneousTissue",
        "LymphaticVessel",
        "LymphoidTissue",
        "LymphNode",
        "Tendon",
        "MuscleTissue",
    ]
}

name_for_cid[7192] = "AnatomicalStructureSegmentationPropertyType"
cid_concepts[7192] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": [
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "SecondaryPulmonaryLobule",
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
        "MidlungWindow",
        "CarinaAngle",
        "CentrilobularStructures",
        "AnteriorJunctionLine",
        "PosteriorJunctionLine",
        "SurfaceOfTheChoroidScleraInterface",
        "OuterSurfaceOfTheBM",
        "OuterSurfaceOfTheCC",
        "PosteriorSurfaceOfTheRPE",
        "SurfaceOfTheCenterOfTheRPE",
        "OuterSurfaceOfIPL",
        "OuterSurfaceOfGCL",
        "OuterSurfaceOfOPL",
        "OuterSurfaceOfINL",
        "SurfaceBetweenInnerAndOuterSegmentsOfThePhotoreceptors",
        "OuterSurfaceOfHFL",
        "AnteriorSurfaceOfTheRPE",
        "SurfaceOfTheInterdigitatingZoneBetweenRetinaAndRPE",
        "OuterSurfaceOfRNFL",
    ],
    "FMA": [
        "SubcorticalGrayMatter",
        "ArcuateFasciculus",
        "WallOfGallbladder",
        "LumenOfGallbladder",
    ],
    "SCT": [
        "ProfundaFemorisVein",
        "LeftHypochondriacRegion",
        "SternocleidomastoidMuscle",
        "TruncusCoeliacus",
        "C3C4Disc",
        "InferiorLongitudinalFasciculus",
        "BasilarArtery",
        "ParanasalSinus",
        "PelvicCavityTrue",
        "BrainCerebrospinalFluidPathway",
        "WristJoint",
        "MidCircumflexCoronaryArtery",
        "CouinaudHepaticSegmentII",
        "CentralRetinalVein",
        "IntermediateArteryRamus",
        "GenicularArtery",
        "LeftVentricleInflow",
        "FontanelOfSkull",
        "SigmoidColon",
        "GastricArtery",
        "SubscapularisMuscle",
        "ExternalEar",
        "LeftVentricleOutflowTract",
        "RightFemoralArtery",
        "Intracranial",
        "JawRegion",
        "Calcaneus",
        "SolealVein",
        "IntercostalArtery",
        "PeripheralNerve",
        "LobarArtery",
        "TeresMajorMuscle",
        "PatentDuctusArteriosus",
        "ThighPerforator",
        "MesentericVein",
        "Pituitary",
        "MaleInternalGenitalia",
        "MidRightCoronaryArtery",
        "IntraThoracic",
        "PalatineBone",
        "CoronaryArtery",
        "RightBranchOfHepaticArtery",
        "UpperLimb",
        "DuraMater",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "ParietalBone",
        "Flank",
        "Maxilla",
        "EsophagealHiatus",
        "CentralRetinalArtery",
        "Neurohypophysis",
        "SegmentalArtery",
        "BrachialVein",
        "RightUpperQuadrantOfAbdomen",
        "T10Vertebra",
        "Eye",
        "SuperficialPalmarArch",
        "Prostate",
        "HepaticArtery",
        "FacialBones",
        "Back",
        "PeronealArtery",
        "Duodenum",
        "RightHypochondriacRegion",
        "_1stMarginalCoronaryArtery",
        "SuperiorMesentericVein",
        "LumbarArtery",
        "Cornea",
        "Larynx",
        "PeronealVein",
        "L4Vertebra",
        "FemoralArtery",
        "IliacVein",
        "Ovary",
        "Urethra",
        "CoronaryArteryGraft",
        "Head",
        "AdrenalGland",
        "MitralValve",
        "AcromioclavicularJoint",
        "UmbilicalRegion",
        "SupraspinatusMuscle",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "Calyx",
        "PosteriorMedialTributary",
        "PulmonaryVeinGreatVessel",
        "T3T4Disc",
        "SoftPalate",
        "T4Vertebra",
        "Submental",
        "ExternalIntercostalMuscle",
        "CranialVenousSystem",
        "NucleusAccumbens",
        "GiacominiVein",
        "PoplitealFossa",
        "MiddleCerebellarPeduncle",
        "PulmonaryArteryConduit",
        "InternalCarotidArteryC5Segment",
        "FissureOfLung",
        "RightVentricle",
        "LevatoresCostarumMuscles",
        "FrontalLobe",
        "VagusNerve",
        "Adventitia",
        "HeadAndNeck",
        "Hypopharynx",
        "RightCoronaryArteryOstium",
        "HepaticVein",
        "MarginalCoronaryArtery",
        "InternalMammaryArtery",
        "Bladder",
        "Vulva",
        "LargeIntestine",
        "EntorhinalCortex",
        "Forearm",
        "BronchialArtery",
        "CouinaudHepaticSegmentIII",
        "NeckChestAndAbdomen",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "TemporalLobe",
        "SystemicArtery",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "GreatSaphenousVeinOfThigh",
        "IntraAbdominopelvic",
        "LeftHepaticVein",
        "L5Vertebra",
        "SuperficialFemoralArtery",
        "VomerBone",
        "UterusAndFallopianTubes",
        "Arachnoid",
        "_2ndMarginalCoronaryArtery",
        "T11T12Disc",
        "SuperiorCerebellarPeduncle",
        "AorticArch",
        "Carina",
        "Telencephalon",
        "LeftBranchOfHepaticArtery",
        "ArcuateArteryOfTheKidney",
        "Heart",
        "TarsalJoint",
        "CephalicVein",
        "LingualArtery",
        "Lip",
        "GastricVein",
        "AirwayStructure",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "DeltoidMuscle",
        "LeftMainCoronaryArtery",
        "CerebralFornix",
        "Penis",
        "Baffle",
        "InferiorRightHepaticVein",
        "VentricularVein",
        "CarotidSiphon",
        "SystemicVenousAtrium",
        "PlantarArterialArch",
        "L1L2Disc",
        "Parathyroid",
        "AnomalousPulmonaryVein",
        "Scapula",
        "LesserSaphenousVein",
        "Kidney",
        "AutonomicNerve",
        "GreatSaphenousVein",
        "T1T2Disc",
        "OmentalBursa",
        "InternalCarotidArtery",
        "L1Vertebra",
        "Ureter",
        "RightPosteriorAVCoronaryArtery",
        "UlnarVein",
        "FascialLayer",
        "MiddleEar",
        "CouinaudHepaticSegmentV",
        "InternalAuditoryCanal",
        "RightPulmonaryArtery",
        "RenalPelvis",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "Thrombus",
        "VenousNetwork",
        "SubcostalMuscle",
        "CorpusCallosum",
        "CommonAtrium",
        "PerforatingArteryOfKidney",
        "Humerus",
        "T5T6Disc",
        "Ilium",
        "AorticFistula",
        "InternalCarotidArteryC6Segment",
        "LeftLumbarRegion",
        "ILMInternalLimitingMembrane",
        "NasalBone",
        "AnteriorTibialArtery",
        "CranialNerve",
        "Cervix",
        "SkinOfAbdomen",
        "LeftAnteriorDescendingCoronaryArtery",
        "Artery",
        "CommonCarotidArtery",
        "Buttock",
        "InnermostIntercostalMuscles",
        "_3rdMarginalCoronaryArtery",
        "FallopianTube",
        "MastoidBone",
        "Vein",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "CouinaudHepaticSegmentVI",
        "CerebrospinalFluid",
        "EsophagealArtery",
        "T8T9Disc",
        "Thigh",
        "RadialVein",
        "InferiorRightPulmonaryVein",
        "Trachea",
        "PulmonaryValveSinuses",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "UterineArtery",
        "BodyConduit",
        "Thalamus",
        "EndoVesical",
        "RightVentricleInflow",
        "MesentericArtery",
        "LeftLowerQuadrantOfAbdomen",
        "MuscleOfHead",
        "SpinalCerebrospinalFluidPathway",
        "MiddleCerebralArteryM1Segment",
        "CouinaudHepaticSegmentIV",
        "ExternalJugularVein",
        "SesamoidBonesOfFoot",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "C2C3Disc",
        "AbdominalWallMuscle",
        "SixthLumbarArtery",
        "PosteriorArchVein",
        "AnteriorSpinalArtery",
        "AnusRectumAndSigmoidColon",
        "Omentum",
        "SplenicVein",
        "Colon",
        "SubmandibularGland",
        "CentralNervousSystem",
        "CarotidBifurcation",
        "T3Vertebra",
        "InnominateArtery",
        "SplenicArtery",
        "Extremity",
        "FacialArtery",
        "AccessoryRenalArtery",
        "BoneStructureOfHeadAndOrNeck",
        "EpigastricRegion",
        "Media",
        "SeminalVesicle",
        "CervicoThoracicSpine",
        "DistalCircumflexCoronaryArtery",
        "RightLumbarRegion",
        "TeresMinorMuscle",
        "BoneOfUpperLimb",
        "DorsalisPedisArtery",
        "Vagina",
        "LongissimusMuscle",
        "LowerInnerQuadrantOfBreast",
        "Clavicle",
        "ParotidGland",
        "AorticIsthmus",
        "CommonFemoralVein",
        "SubstantiaNigra",
        "_1stLeftPosterolateralCoronaryArtery",
        "Subxiphoid",
        "UpperInnerQuadrantOfBreast",
        "Diaphragm",
        "C5Vertebra",
        "ThoracicAorta",
        "RightInguinalRegion",
        "VasDeferens",
        "T6T7Disc",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "Ileum",
        "SalivaryGland",
        "VertebralArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "RightLowerQuadrantOfAbdomen",
        "SupraRenalAorta",
        "InternalJugularVein",
        "LacrimalArteryOfRightEye",
        "CockettPerforatingVein",
        "ChordaeTendineaeCordis",
        "SacroiliacJoint",
        "RightMainBronchus",
        "ScalenousAnteriorMuscle",
        "AVGrooveContinuationOfCircumflexArtery",
        "EntireBody",
        "TesticularArtery",
        "Thumb",
        "HypogastricRegion",
        "GreatCardiacVein",
        "HyoidBone",
        "C2Vertebra",
        "ThoracicDuct",
        "MiddleLobeOfRightLung",
        "BoydPerforatingVein",
        "BodyCavity",
        "Peritioneum",
        "CommonIliacArtery",
        "SupraclavicularRegionOfNeck",
        "Meninges",
        "T12Vertebra",
        "C3Vertebra",
        "Shoulder",
        "TransversusThoracis",
        "CardiacSeptumProsthesis",
        "CerebellarWhiteMatter",
        "L2L3Disc",
        "InterventricularSeptum",
        "C5C6Disc",
        "_2ndLeftPosterolateralCoronaryArtery",
        "MiddleCerebralArteryM2Segment",
        "Cingulum",
        "LowerLobeOfLung",
        "TesticularVein",
        "AnteriorTibialVein",
        "LacrimalBone",
        "BasilicVein",
        "LeftInguinalRegion",
        "RightCoronaryArtery",
        "HunterianPerforatingVein",
        "Putamen",
        "ProfundaFemorisArtery",
        "CostocervicalTrunk",
        "Pharynx",
        "FifthLumbarArtery",
        "AxillaryVein",
        "LigamentumArteriosum",
        "Nasopharynx",
        "C6C7Disc",
        "SuprapulmonicValveArea",
        "SympatheticTrunk",
        "LobeOfLung",
        "SubclavianVein",
        "PulmonaryValve",
        "ExternalIliacArtery",
        "ProximalRightCoronaryArtery",
        "TemporalBone",
        "BoneOfLowerLimb",
        "Midbrain",
        "OvarianVein",
        "PosteriorCerebralArtery",
        "ErectorSpinaeMuscle",
        "DeepPalmarVenousArch",
        "ApexOfRightVentricle",
        "T2T3Disc",
        "C1Vertebra",
        "LateralCalfPerforator",
        "Phantom",
        "Mouth",
        "PharynxAndLarynx",
        "CarotidArtery",
        "FourthLumbarArtery",
        "SmallestCardiacVein",
        "ThirdLumbarArtery",
        "LimbicLobe",
        "SkinOfChest",
        "UmbilicalArtery",
        "Cheek",
        "LymphNode",
        "PrimitiveAorta",
        "RadialArtery",
        "T2Vertebra",
        "LeftFemoralArtery",
        "LumenOfBloodVessel",
        "TrapeziusMuscle",
        "Esophagus",
        "AntecubitalVein",
        "SpinalCordGrayMatter",
        "Mandible",
        "LateralCorticospinalTract",
        "Parasternal",
        "LeftMainBronchus",
        "BroadLigament",
        "SystemicVein",
        "OccipitalLobe",
        "C7T1Disc",
        "CerebralAqueduct",
        "Insula",
        "Pelvis",
        "PulmonaryVein",
        "MuscleOfNeck",
        "SubclavianArtery",
        "Amygdala",
        "Lung",
        "SacroCoccygealSpine",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OpticChiasm",
        "OphthalmicArtery",
        "PancreaticDuct",
        "Nose",
        "Hand",
        "PectoralisMajorMuscle",
        "LeftMainCoronaryArteryOstium",
        "T9Vertebra",
        "CardiovascularSystem",
        "CouinaudHepaticSegmentI",
        "OvarianArtery",
        "ElbowJoint",
        "Hippocampus",
        "Sternum",
        "NeckAndChest",
        "RightVentricleOutflowTract",
        "GlobusPallidus",
        "SuperficialFemoralVein",
        "SuperiorLeftPulmonaryVein",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "AbdomenAndPelvis",
        "UpperOuterQuadrantOfBreast",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "PericardialCavity",
        "PulmonaryArtery",
        "Skull",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "C6Vertebra",
        "CerebralHemisphere",
        "AtrioventricularVein",
        "Thyroid",
        "EthmoidBone",
        "OpticRadiation",
        "InfraRenalAorta",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "T7Vertebra",
        "EndoNasopharyngeal",
        "EsophagusStomachAndDuodenum",
        "Breast",
        "Neck",
        "AxillaryFascia",
        "CarotidBulb",
        "Finger",
        "PoplitealVein",
        "InfraspinatusMuscle",
        "PiaMater",
        "TricuspidValve",
        "RightAtrium",
        "CouinaudHepaticSegmentVII",
        "VascularSclerosis",
        "SuperiorPhrenicArtery",
        "CommonVentricle",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "InnerEar",
        "Coccyx",
        "AtrialSeptalDefect",
        "RightPulmonaryVein",
        "Uterus",
        "TemporomandibularJoint",
        "Ventricle",
        "DescendingAorta",
        "TracheaAndBronchus",
        "Testis",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "OccipitalBone",
        "L2Vertebra",
        "Eyeball",
        "Face",
        "Spine",
        "UrinarySystem",
        "BrainStem",
        "Foot",
        "WhiteMatter",
        "MuscleOfUpperLimb",
        "PeripheralNervousSystem",
        "PinealGland",
        "UpperLobeOfLung",
        "PosterolateralBranchOfRightCoronaryArtery",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "ProximalCircumflexCoronaryArtery",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "JuxtaposedAtrialAppendage",
        "VertebralColumnAndCranium",
        "Jejunum",
        "CarotidBody",
        "ChestAbdomenAndPelvis",
        "AbdominalAorta",
        "RightHepaticVein",
        "GastrocnemiusVein",
        "Scalp",
        "Tongue",
        "ChestWallMuscle",
        "GrayMatter",
        "Fibula",
        "SuperiorLongitudinalFasciculus",
        "OrbitalStructure",
        "InferiorCardiacVein",
        "LeftMainBranchOfPortalVein",
        "InternalIntercostalMuscle",
        "LateralVentricle",
        "CaudateNucleus",
        "LacrimalArtery",
        "OccipitalArtery",
        "Brain",
        "BloodVessel",
        "Sclera",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "LiverAndBiliaryStructure",
        "AtrialVein",
        "_1stDiagonalCoronaryArtery",
        "SuprasternalNotch",
        "Perineum",
        "InteratrialSeptum",
        "Rectum",
        "Femur",
        "AnteriorCardiacVein",
        "T10T11Disc",
        "Pancreas",
        "BoneOfThorax",
        "LeftAtrium",
        "InterlobularArteryOfKidney",
        "CerebralArtery",
        "CouinaudHepaticSegmentVIII",
        "InferiorMesentericArtery",
        "DigitalArteryOfHand",
        "SecondLumbarArtery",
        "IliocostalisMuscle",
        "_1stRightPosterolateralCoronaryArtery",
        "BladderAndUrethra",
        "CervicalSpine",
        "IliacAndOrFemoralArtery",
        "SpinalCordWhiteMatter",
        "Sacrum",
        "SmallIntestine",
        "FemaleInternalGenitalia",
        "SuperiorRightPulmonaryVein",
        "T8Vertebra",
        "InferiorVenaCava",
        "MaleExternalGenitalia",
        "IntraArticular",
        "Axilla",
        "RetroperitonealSpace",
        "Thymus",
        "ELMExternalLimitingMembrane",
        "AnteriorMiddleCerebralArteryBifurcation",
        "ParietalLobe",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "T12L1Disc",
        "DistalRightCoronaryArtery",
        "CerebralCortex",
        "TruncusArteriosusCommunis",
        "T1Vertebra",
        "Tooth",
        "Joint",
        "SubmandibularArea",
        "GreatSaphenousVeinOfCalf",
        "LeftPulmonaryVein",
        "PectoralisMinorMuscle",
        "OpticCanal",
        "Aorta",
        "SuperficialTemporalArtery",
        "MedialLemniscus",
        "Diencephalon",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "InferiorCerebellarPeduncle",
        "RightMainBranchOfPortalVein",
        "SphenoidBone",
        "T6Vertebra",
        "LatissimusDorsiMuscle",
        "Spleen",
        "ChestAndAbdomen",
        "CommonHepaticArtery",
        "C4Vertebra",
        "AnkleJoint",
        "LowerOuterQuadrantOfBreast",
        "C4C5Disc",
        "Mediastinum",
        "Cerebellum",
        "FirstLumbarArtery",
        "PoplitealArtery",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "Bronchus",
        "PulmonaryChamberOfCorTriatriatum",
        "T9T10Disc",
        "DorsalScapularArtery",
        "CommonBileDuct",
        "UlnarArtery",
        "L3L4Disc",
        "LeftVentricle",
        "LumbarRegion",
        "CostalCartilage",
        "ThoracicSpine",
        "FourthVentricle",
        "SaphenousVein",
        "T5Vertebra",
        "Gallbladder",
        "_2ndRightPosterolateralCoronaryArtery",
        "RenalArtery",
        "PeritonealCavity",
        "HipJoint",
        "UpperArm",
        "OpticTract",
        "TrunkOfPulmonaryArtery",
        "Chest",
        "GastroduodenalArtery",
        "L3Vertebra",
        "T4T5Disc",
        "AzygosVein",
        "BrachialArtery",
        "C7Vertebra",
        "_2ndDiagonalCoronaryArtery",
        "Liver",
        "Hip",
        "Rib",
        "L5S1Disc",
        "PelvisAndLowerExtremities",
        "SerratusAnteriorMuscle",
        "CommonFemoralArtery",
        "SmallCardiacVein",
        "IntraPelvic",
        "SuperficialPalmarVenousArch",
        "Vertebra",
        "Stomach",
        "CircumflexCoronaryArtery",
        "AnteriorCommunicatingArtery",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "InferiorMesentericVein",
        "T7T8Disc",
        "VenaCava",
        "MedianCubitalVein",
        "CoronarySinus",
        "HilarArtery",
        "TerminalInternalCarotidArtery",
        "LeftCoronaryArtery",
        "BiliaryTract",
        "ApexOfLung",
        "ThirdVentricle",
        "Atrium",
        "CerebralWhiteMatter",
        "Knee",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "SubarachnoidSpace",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "EyeRegion",
        "BrainVentricle",
        "Anus",
        "UmbilicalVein",
        "FrontalBone",
        "CommonIliacVein",
        "TrabeculaeCarnae",
        "InguinalRegion",
        "PosteriorCerebralArteryP1Segment",
        "_3rdRightPosterolateralCoronaryArtery",
        "EndoNasal",
        "LumbarSpine",
        "ThyrocervicalTrunk",
        "InferiorPhrenicArtery",
        "PrimitivePulmonaryArtery",
        "HilumOfLung",
        "SellaTurcica",
        "PortalVein",
        "Scrotum",
        "PosteriorCerebralCommissure",
        "InferiorLeftPulmonaryVein",
        "UncinateFasciculus",
        "Intima",
        "AnteriorCerebralArtery",
        "LowerLeg",
        "RenalVein",
        "L4L5Disc",
        "InternalIliacVein",
        "IlealVein",
        "Ear",
        "Toe",
        "SpinalCord",
        "MiddleCardiacVein",
        "T11Vertebra",
        "Pericardium",
        "MiddleCerebralArtery",
        "PosteriorCerebralArteryP2Segment",
        "PosteriorDescendingRightCoronaryArtery",
        "CorpusStriatum",
        "LeftAuricularAppendage",
        "PelvicCavityFalse",
        "CircleOfWillis",
        "Adenohypophysis",
        "PosteriorTibialVein",
        "SaphenopoplitealJunction",
        "_1stSeptalCoronaryArtery",
        "MuscleOfLowerLimb",
        "Eyelid",
        "AscendingAorta",
        "LeftUpperQuadrantOfAbdomen",
        "DeepPalmarArchOfRadialArtery",
        "PericardiophrenicArtery",
        "Pleura",
        "HypochondriacRegion",
        "ExternalAuditoryCanal",
        "Patella",
        "SegmentOfLung",
        "OccipitalVein",
        "LumboSacralSpine",
        "DoddPerforatingVein",
    ],
}

name_for_cid[7193] = "PhysicalObjectSegmentationPropertyType"
cid_concepts[7193] = {
    "DCM": [
        "Mold",
        "BrachytherapyChannelShield",
        "HeadAndNeckMask",
        "BrachytherapySourceApplicator",
        "HeadMask",
        "Headframe",
        "BodyFrame",
        "BreastBoard",
        "Cast",
        "WholeBodyPod",
        "VacuumMold",
        "Portacath",
        "ChestTube",
        "CentralLine",
        "KidneyStent",
        "PancreaticStent",
        "NippleRing",
        "Coin",
        "BrachytherapyChannel",
        "RectalBalloon",
        "FiducialMark",
        "VaginalCylinder",
        "HeadFixationBoard",
        "AbdominalCompressionArch",
        "AbdominalCompressionBelt",
        "BreastBridge",
        "Sphere",
        "OtherMarker",
        "IonChamberArray",
        "DiodeArray",
        "LiquidIonChamber",
        "Diode",
        "IonChamber",
        "OSLD",
        "RPLGD",
        "DiamondDetector",
        "HandGrips",
        "HeelStop",
        "ShinRest",
        "SeatPan",
    ],
    "SCT": [
        "Suture",
        "Jewelry",
        "Prosthesis",
        "EndotrachealTube",
        "MeasuringRuler",
        "Collimator",
        "Backrest",
        "RotationalAtherectomyDevice",
        "Wall",
        "DentalProsthesis",
        "CervicalCollar",
        "HeartValveProsthesis",
        "EmbolizationParticulate",
        "Armrest",
        "FeedingTube",
        "Stretcher",
        "Tracheotomy",
        "Staple",
        "PercutaneousTransluminalAngioplastyBalloon",
        "BonePin",
        "BBShotLeadPellet",
        "Defibrillator",
        "Headrest",
        "DigitalImager",
        "IDPlate",
        "Stent",
        "Floor",
        "Implant",
        "Chair",
        "ForeignBody",
        "Glue",
        "InlayDentalRestoration",
        "EmbolizationCoil",
        "MOSFETRadiationTherapyDosimetrySystemDosimeter",
        "Catheter",
        "Bullet",
        "GuidingCatheter",
        "ThermoluminescentRadiationDosimeter",
        "OpaqueMarker",
        "BiteBlock",
        "CardiacPacemakerLead",
        "Clip",
        "BoneScrew",
        "DetachableBalloon",
        "JejunostomyTube",
        "RadioactiveImplant",
        "PacemakerPulseGenerator",
        "JWire",
        "VenaCavaFilter",
        "Needle",
        "AtherectomyDevice",
        "Laser",
        "Table",
        "CompressionPaddle",
        "UretericStent",
        "EmbolizationBall",
        "CardiacPacemaker",
        "Clothing",
        "LeftVentricularAssistDevice",
    ],
}

name_for_cid[7194] = "MorphologicallyAbnormalStructureSegmentationPropertyType"
cid_concepts[7194] = {
    "DCM": ["Background"],
    "NCIt": ["EnhancingLesion", "NonEnhancingLesion"],
    "SCT": [
        "LesionOfBrain",
        "Embolus",
        "NeoplasmPrimary",
        "Mass",
        "Normal",
        "Cyst",
        "Abscess",
        "Edema",
        "BloodClot",
        "Nodule",
        "Lesion",
        "Neoplasm",
        "NeoplasmSecondary",
        "Necrosis",
        "Hemorrhage",
        "Inflammation",
    ],
}

name_for_cid[7195] = "FunctionSegmentationPropertyType"
cid_concepts[7195] = {"SCT": ["Perfusion"]}

name_for_cid[7196] = "SpatialandRelationalConceptSegmentationPropertyType"
cid_concepts[7196] = {
    "DCM": [
        "Lucency",
        "TissueNotOfInterest",
        "RegistrationInput",
        "Background",
        "SingleSubjectExtractedFromGroup",
        "Interface",
        "Line",
    ],
    "NCIt": ["ReferenceRegion"],
    "SCT": ["Normal", "WasteMaterial", "Tissue"],
}

name_for_cid[7197] = "BodySubstanceSegmentationPropertyType"
cid_concepts[7197] = {"SCT": ["Feces", "Urine", "BodyFluid", "Bile", "Gas"]}

name_for_cid[7198] = "SubstanceSegmentationPropertyType"
cid_concepts[7198] = {"SCT": ["ContrastAgent"]}

name_for_cid[7199] = "AnatomicallyLocalizedLesionSegmentationType"
cid_concepts[7199] = {"SCT": ["LesionOfBrain"]}

name_for_cid[7201] = "ReferencedImagePurposeofReference"
cid_concepts[7201] = {
    "DCM": [
        "ImagesRelatedToStandaloneObject",
        "AnatomicImage",
        "TemporalSuccessor",
        "Spectroscopy",
        "FunctionalImage",
        "OtherImageOfBiplanePair",
        "VolumeCorrespondingToSpatiallyRelatedAcquisitionFrames",
        "OtherImageOfStereoscopicPair",
        "TemporalPredecessor",
        "BiopsyLocalizer",
        "OtherPartialViews",
        "AcquisitionFramesCorrespondingToVolume",
        "Localizer",
        "ImageUsedForTreatmentPlanning",
        "QualityControlMaterialImage",
        "ImageAcquiredDuringTreatment",
        "ImageUsedForDoseCalculation",
        "ImageUsedAsReferenceImageForTreatment",
        "DeviceLocalizer",
        "SpectralFilteredImage",
    ]
}

name_for_cid[7202] = "SourceImagePurposeofReference"
cid_concepts[7202] = {
    "DCM": [
        "PredecessorContainingGroupOfImagingSubjects",
        "ForProcessingPredecessor",
        "MaskImageForImageProcessingOperation",
        "SourceImageForImageProcessingOperation",
        "LossyCompressedPredecessor",
        "UncompressedPredecessor",
        "FlowImageForImageProcessing",
        "StructuralImageForImageProcessing",
        "SourceImageForMontage",
    ]
}

name_for_cid[7203] = "ImageDerivation"
cid_concepts[7203] = {
    "DCM": [
        "SpatialResampling",
        "PixelByPixelAddition",
        "EdgeEnhancement",
        "Smoothing",
        "GaussianBlur",
        "UnsharpMask",
        "PixelByPixelDivision",
        "PixelByPixelMask",
        "PixelByPixelMaximum",
        "LossyCompression",
        "PixelByPixelSubtraction",
        "PerfusionImageAnalysis",
        "DiffusionTractography",
        "DiffusionImageAnalysis",
        "PixelByPixelMean",
        "DeformedForRegistration",
        "VolumeRendering",
        "PixelByPixelMultiplication",
        "SurfaceRendering",
        "MultiEnergyProportionalWeighting",
        "SegmentationImageDerivation",
        "ExtractionOfIndividualSubjectFromGroup",
        "VolumeEditing",
        "MaximumIntensityProjection",
        "MinimumIntensityProjection",
        "ImageStitching",
        "SpatiallyRelatedFramesExtractedFromTheVolume",
        "TemporallyRelatedFramesExtractedFromTheSetOfVolumes",
        "PolarToRectangularScanConversion",
        "MetaboliteMapsFromSpectroscopyData",
        "MultiplanarReformatting",
        "PixelByPixelMinimum",
        "CurvedMultiplanarReformatting",
        "OCTBScanAnalysis",
    ]
}

name_for_cid[7205] = "PurposeofReferencetoAlternateRepresentation"
cid_concepts[7205] = {
    "DCM": [
        "FullFidelityImage",
        "AlternatePhotometricInterpretationImage",
        "LossyCompressedImage",
        "AlternateSOPClassInstance",
        "SourceImage",
    ]
}

name_for_cid[7210] = "RelatedSeriesPurposeofReference"
cid_concepts[7210] = {
    "DCM": [
        "ForAttenuationCorrection",
        "SameAnatomy",
        "SourceSeriesForImageProcessingOperation",
        "SameIndication",
        "SimultaneouslyAcquired",
    ]
}

name_for_cid[7215] = "SpectroscopyPurposeofReference"
cid_concepts[7215] = {"DCM": ["SpectroscopyDataForWaterPhaseCorrection"]}

name_for_cid[7220] = "RTDoseDerivation"
cid_concepts[7220] = {
    "DCM": [
        "ComposedWithWeightingForFractionsDelivered",
        "ComposedWithRadiobiologicalEffects",
        "ComposedFromPriorDoses",
        "ComposedFromPriorDosesAndCurrentPlan",
    ]
}

name_for_cid[7221] = "RTDosePurposeofReference"
cid_concepts[7221] = {"DCM": ["SourceDoseForComposingCurrentDose"]}

name_for_cid[7222] = "ParametricMapDerivationImagePurposeofReference"
cid_concepts[7222] = {"DCM": ["SourceImageForImageProcessingOperation"]}

name_for_cid[7230] = "AutomationofMeasurement"
cid_concepts[7230] = {"SCT": ["Automated", "Manual"]}

name_for_cid[7250] = "MultiFrameSubsetType"
cid_concepts[7250] = {"DCM": ["ReferencedFrames", "ReferencedSegment"]}

name_for_cid[7260] = "DiffusionAcquisitionValueType"
cid_concepts[7260] = {
    "DCM": [
        "HARDI",
        "DKI",
        "DTI",
        "DSI",
        "LSDI",
        "SingleShotEPI",
        "MultipleShotEPI",
        "ParallelImaging",
    ]
}

name_for_cid[7261] = "DiffusionModelValueType"
cid_concepts[7261] = {
    "DCM": [
        "SingleTensor",
        "MultiTensor",
        "ModelFree",
        "CHARMED",
        "DSI",
        "DOT",
        "PAS",
        "SphericalDeconvolution",
    ]
}

name_for_cid[7262] = "DiffusionTractographyAlgorithmFamily"
cid_concepts[7262] = {
    "DCM": [
        "Deterministic",
        "Probabilistic",
        "Global",
        "FACT",
        "Streamline",
        "TEND",
        "Bootstrap",
        "Euler",
        "RungeKutta",
    ]
}

name_for_cid[7263] = "DiffusionTractographyMeasurementType"
cid_concepts[7263] = {
    "DCM": [
        "ApparentDiffusionCoefficient",
        "Trace",
        "MeanDiffusivity",
        "VolumetricDiffusionDxzComponent",
        "VolumetricDiffusionDxyComponent",
        "VolumetricDiffusionDxxComponent",
        "RelativeAnisotropy",
        "RadialDiffusivity",
        "FractionalAnisotropy",
        "AxialDiffusivity",
        "MeanKurtosis",
        "ApparentKurtosisCoefficient",
        "RadialKurtosis",
        "AxialKurtosis",
        "VolumetricDiffusionDzzComponent",
        "FractionalKurtosisAnisotropy",
        "VolumetricDiffusionDyzComponent",
        "VolumetricDiffusionDyyComponent",
    ]
}

name_for_cid[7270] = "MRDiffusionComponentSemantic"
cid_concepts[7270] = {
    "DCM": [
        "VolumetricDiffusionDxzComponent",
        "VolumetricDiffusionDxyComponent",
        "DiffusionWeighted",
        "VolumetricDiffusionDxxComponent",
        "VolumetricDiffusionDzzComponent",
        "VolumetricDiffusionDyzComponent",
        "VolumetricDiffusionDyyComponent",
    ]
}

name_for_cid[7271] = "MRDiffusionAnisotropyIndex"
cid_concepts[7271] = {
    "DCM": ["VolumeRatio", "RelativeAnisotropy", "FractionalAnisotropy"]
}

name_for_cid[7272] = "MRDiffusionModelParameter"
cid_concepts[7272] = {
    "DCM": [
        "ApparentDiffusionCoefficient",
        "KurtosisDiffusionCoefficient",
        "GammaDistributionScaleParameter",
        "GammaDistributionShapeParameter",
        "GammaDistributionMode",
        "DistributedDiffusionCoefficient",
        "AnomalousExponentParameter",
        "DiffusionCoefficient",
        "MonoExponentialApparentDiffusionCoefficient",
        "SlowDiffusionCoefficient",
        "FastDiffusionCoefficient",
        "FastDiffusionCoefficientFraction",
    ]
}

name_for_cid[7273] = "MRDiffusionModel"
cid_concepts[7273] = {
    "DCM": [
        "MonoExponentialDiffusionModel",
        "BiExponentialIVIMDiffusionModel",
        "KurtosisDiffusionModel",
        "GammaDistributionModel",
        "StretchedExponentialDiffusionModel",
        "TruncatedGaussianDiffusionModel",
    ]
}

name_for_cid[7274] = "MRDiffusionModelFittingMethod"
cid_concepts[7274] = {
    "DCM": [
        "LeastSquaresFitOfMultipleSamples",
        "LevenbergMarquardt",
        "TrustRegion",
        "FixedDp",
        "SegmentedUnconstrained",
        "SegmentedConstrained",
        "BayesianProbability",
        "LogOfRatioOfTwoSamples",
    ]
}

name_for_cid[7275] = "MRDiffusionModelSpecificMethod"
cid_concepts[7275] = {"DCM": ["VoxelwiseSelectionOfBValue"]}

name_for_cid[7276] = "MRDiffusionModelInput"
cid_concepts[7276] = {"DCM": ["SourceImageDiffusionBValue"]}

name_for_cid[7277] = "DiffusionRateAreaOverTimeUnit"
cid_concepts[7277] = {
    "UCUM": [
        "SquareMillimeterPerSecond",
        "SquareMicrometerPerMillisecond",
        "_10EMinus6SquareMillimetersPerSecond",
        "SquareMicrometerPerSecond",
    ]
}

name_for_cid[7300] = "ImplantMaterial"
cid_concepts[7300] = {
    "DCM": ["TitaniumAlloyTitaniumMolybdenum15Zirconium5Aluminium3"],
    "SCT": [
        "Polymer",
        "CarbonFiber",
        "StainlessSteelMaterial",
        "GoldAlloy",
        "NickelTitanium",
    ],
}

name_for_cid[7301] = "InterventionType"
cid_concepts[7301] = {
    "SCT": [
        "HipJointReconstruction",
        "ResurfacingOfTheFemoralHead",
        "InsertionOfHipProsthesis",
        "ResurfacingOfThePatella",
    ]
}

name_for_cid[7302] = "ImplantTemplateViewOrientation"
cid_concepts[7302] = {
    "SCT": ["AnteroPosterior", "MedioLateralObliqueProjection", "LateralMedial"]
}

name_for_cid[7303] = "ImplantTemplateModifiedViewOrientation"
cid_concepts[7303] = {"DCM": ["APPlus45", "APMinus45"]}

name_for_cid[7304] = "ImplantTargetAnatomy"
cid_concepts[7304] = {
    "SCT": [
        "SacroCoccygealSpine",
        "Clavicle",
        "ThoracoLumbarSpine",
        "Tibia",
        "DistalFemur",
        "ProximalRadius",
        "ProximalUlna",
        "WristJoint",
        "HandBone",
        "ShaftOfTibia",
        "Shoulder",
        "BoneStructureOfPelvis",
        "Humerus",
        "ElbowJoint",
        "FacialBones",
        "DistalRadius",
        "ShaftOfUlna",
        "Fibula",
        "JawRegion",
        "Talus",
        "ThoracicSpine",
        "Calcaneus",
        "Ulna",
        "HeadOfFemur",
        "RadiusBone",
        "Knee",
        "HipJoint",
        "DistalUlna",
        "AnatomicalReferencePlane",
        "Mandible",
        "FingerJoint",
        "ShaftOfRadius",
        "Femur",
        "ShaftOfFemur",
        "CervicoThoracicSpine",
        "Skull",
        "ShaftOfHumerus",
        "DistalHumerus",
        "ProximalHumerus",
        "ProximalFemur",
        "Patella",
        "DistalTibia",
        "AnkleJoint",
        "LumbarSpine",
        "Maxilla",
        "CervicalSpine",
    ]
}

name_for_cid[7305] = "ImplantPlanningLandmark"
cid_concepts[7305] = {
    "DCM": [
        "AnatomicalAxisOfFemur",
        "AcetabularCenterOfRotation",
        "FemurHeadCenterOfRotation",
    ]
}

name_for_cid[7306] = "HumanHipImplantPlanningLandmark"
cid_concepts[7306] = {
    "DCM": [
        "AnatomicalAxisOfFemur",
        "AcetabularCenterOfRotation",
        "FemurHeadCenterOfRotation",
    ]
}

name_for_cid[7307] = "ImplantComponentType"
cid_concepts[7307] = {
    "DCM": [
        "FemoralStem",
        "FemoralStemDistalComponent",
        "FemoralHeadConeTaperComponent",
        "FemoralStemProximalComponent",
        "FemoralStemComponent",
        "NeckComponent",
        "DistalCentralizer",
        "MonoblockStem",
        "AcetabularCupShell",
        "ProstheticShaftAugment",
        "AcetabularCupInsert",
        "FemoralHeadResurfacingComponent",
        "AcetabularCupMonoblock",
        "FemoralHeadBallComponent",
    ],
    "SCT": ["DHSPlate", "FemoralHeadProsthesis", "Screw", "BonePlate", "BoneNail"],
}

name_for_cid[7308] = "HumanHipImplantComponentType"
cid_concepts[7308] = {
    "DCM": [
        "FemoralStem",
        "FemoralStemDistalComponent",
        "FemoralHeadConeTaperComponent",
        "FemoralStemProximalComponent",
        "FemoralStemComponent",
        "NeckComponent",
        "DistalCentralizer",
        "MonoblockStem",
        "AcetabularCupShell",
        "ProstheticShaftAugment",
        "AcetabularCupInsert",
        "FemoralHeadResurfacingComponent",
        "AcetabularCupMonoblock",
        "FemoralHeadBallComponent",
    ],
    "SCT": ["FemoralHeadProsthesis"],
}

name_for_cid[7309] = "HumanTraumaImplantComponentType"
cid_concepts[7309] = {"SCT": ["DHSPlate", "Screw", "BonePlate", "BoneNail"]}

name_for_cid[7310] = "ImplantFixationMethod"
cid_concepts[7310] = {
    "DCM": ["Bolting", "Wedging", "Pinning", "Sewing"],
    "SCT": [
        "RepairByNailing",
        "Anchoring",
        "Gluing",
        "InternalSkeletalFixation",
        "UncementedComponentFixation",
        "CementedComponentFixation",
        "InternalFixationUsingStaple",
        "Fusion",
        "InternalFixationUsingPlate",
        "InternalFixationUsingScrew",
    ],
}

name_for_cid[7320] = "PlanningMethod"
cid_concepts[7320] = {
    "DCM": [
        "Generic2DPlanning",
        "Generic3DPlanning",
        "GenericPlanningForHipReplacement",
        "GenericPlanningForKneeReplacement",
        "MüllerMethodPlanningForHipReplacement",
    ]
}

name_for_cid[7445] = "DeviceParticipatingRole"
cid_concepts[7445] = {"DCM": ["Recording", "XRayReadingDevice", "IrradiatingDevice"]}

name_for_cid[7449] = "ReaderSpecialty"
cid_concepts[7449] = {
    "DCM": [
        "OncologicImagingSpecialty",
        "OBGynImagingSpecialty",
        "ThoracicImagingSpecialty",
        "OncologySpecialty",
        "AbdominalImagingSpecialty",
        "VascularImagingSpecialty",
        "PediatricImagingSpecialty",
        "HeadAndNeckImagingSpecialty",
        "CardiacImagingSpecialty",
        "MusculoskeletalImagingSpecialty",
        "NeuroradiologyImagingSpecialty",
        "NeurologySpecialty",
    ]
}

name_for_cid[7450] = "PersonRole"
cid_concepts[7450] = {
    "DCM": [
        "Patient",
        "AdministratorOfRadiologyDepartment",
        "HeadOfCardiology",
        "LeadRadiologicTechnologist",
        "RepresentativeOfEthicsCommittee",
        "RepresentativeOfProtocolCommittee",
        "Fellow",
        "ChairOfProtocolCommittee",
        "HeadOfRadiology",
        "Sonologist",
    ],
    "SCT": [
        "NaturalGrandFather",
        "MaleFirstCousin",
        "NaturalFather",
        "HealthcareProfessional",
        "HalfSister",
        "NaturalMother",
        "Resident",
        "NaturalSister",
        "Friend",
        "Physician",
        "Attending",
        "NaturalBrother",
        "ScrubNurse",
        "Registrar",
        "RadiologicTechnologist",
        "Aunt",
        "Surgeon",
        "NaturalGrandMother",
        "Uncle",
        "Consultant",
        "Nurse",
        "NaturalDaughter",
        "NaturalSon",
        "FemaleFirstCousin",
        "MedicalPractitioner",
        "HalfBrother",
        "RadiationTherapist",
    ],
    "UMLS": [
        "Intern",
        "MedicalPhysicist",
        "ConsultingPhysician",
        "RadiationPhysicist",
        "Sonographer",
    ],
}

name_for_cid[7451] = "FamilyMember"
cid_concepts[7451] = {
    "SCT": [
        "NaturalGrandFather",
        "NaturalBrother",
        "MaleFirstCousin",
        "NaturalFather",
        "Aunt",
        "NaturalGrandMother",
        "Uncle",
        "HalfSister",
        "NaturalMother",
        "NaturalDaughter",
        "NaturalSon",
        "FemaleFirstCousin",
        "NaturalSister",
        "HalfBrother",
    ]
}

name_for_cid[7452] = "OrganizationalRole"
cid_concepts[7452] = {
    "DCM": [
        "AdministratorOfRadiologyDepartment",
        "HeadOfCardiology",
        "LeadRadiologicTechnologist",
        "RepresentativeOfEthicsCommittee",
        "RepresentativeOfProtocolCommittee",
        "Fellow",
        "ChairOfProtocolCommittee",
        "HeadOfRadiology",
        "Sonologist",
    ],
    "SCT": [
        "Resident",
        "Physician",
        "Attending",
        "ScrubNurse",
        "Registrar",
        "RadiologicTechnologist",
        "Surgeon",
        "Consultant",
        "Nurse",
        "MedicalPractitioner",
        "RadiationTherapist",
    ],
    "UMLS": [
        "Intern",
        "MedicalPhysicist",
        "ConsultingPhysician",
        "RadiationPhysicist",
        "Sonographer",
    ],
}

name_for_cid[7453] = "PerformingRole"
cid_concepts[7453] = {
    "DCM": [
        "Standby",
        "IrradiationAdministering",
        "IrradiationAuthorizing",
        "ImageQualityController",
        "Designator",
        "ResultsQualityController",
        "Assisting",
        "Recording",
        "Verifying",
        "Requesting",
        "EligibilityReader",
        "Performing",
    ],
    "NCIt": ["Reader", "Adjudicator", "Reviewer"],
    "SCT": ["CirculatingNurse"],
    "UMLS": ["Referring"],
}

name_for_cid[7454] = "AnimalTaxonomicRankValue"
cid_concepts[7454] = {
    "ITIS_TSN": ["PeromyscusLeucopus", "PeromyscusManiculatus", "SigmodonGenus"],
    "SCT": [
        "CanisLupus",
        "RattusNorvegicus",
        "FelisCatus",
        "HomoSapiens",
        "Canis",
        "BosTaurus",
        "OryctolagusCuniculus",
        "Capra",
        "MustelaPutoriusFuro",
        "EquusCaballus",
        "SusScrofa",
        "Equus",
        "Ovis",
        "Bovinae",
        "Bos",
        "CallithrixJacchus",
        "CaviaPorcellus",
        "Felis",
        "Sus",
        "CanisLupusFamiliaris",
        "Rattus",
        "MusGenus",
        "OvisAries",
        "MusMusculus",
        "CapraHircus",
    ],
}

name_for_cid[7455] = "Sex"
cid_concepts[7455] = {
    "DCM": [
        "MalePseudohermaphrodite",
        "UnknownSex",
        "UndeterminedSex",
        "Female",
        "MaleChangedToFemale",
        "AmbiguousSex",
        "Hermaphrodite",
        "OtherSex",
        "FemalePseudohermaphrodite",
        "Male",
        "FemaleChangedToMale",
    ]
}

name_for_cid[7456] = "AgeUnit"
cid_concepts[7456] = {"UCUM": ["Year", "Minute", "Hour", "Week", "Day", "Month"]}

name_for_cid[7457] = "SexMaleFemaleorBoth"
cid_concepts[7457] = {"DCM": ["MixedSex", "Female", "Male"]}

name_for_cid[7460] = "LinearMeasurementUnit"
cid_concepts[7460] = {"UCUM": ["Millimeter", "Centimeter", "Micrometer"]}

name_for_cid[7461] = "AreaMeasurementUnit"
cid_concepts[7461] = {
    "UCUM": ["SquareMillimeter", "SquareCentimeter", "SquareMicrometer"]
}

name_for_cid[7462] = "VolumeMeasurementUnit"
cid_concepts[7462] = {
    "UCUM": ["CubicMillimeter", "CubicCentimeter", "CubicMicrometer", "CubicDecimeter"]
}

name_for_cid[7464] = "GeneralRegionofInterestMeasurementModifier"
cid_concepts[7464] = {
    "DCM": ["Skewness", "Kurtosis", "PeakValueWithinROI"],
    "SCT": [
        "StandardDeviation",
        "Mode",
        "Minimum",
        "Maximum",
        "Total",
        "Mean",
        "Median",
    ],
    "UMLS": ["CoefficientOfVariance", "Variance", "RootMeanSquare"],
}

name_for_cid[7465] = "MeasurementDerivedFromMultipleROIMeasurements"
cid_concepts[7465] = {
    "DCM": [
        "PercentileRankingOfMeasurement",
        "ZScoreOfMeasurement",
        "StandardDeviationOfPopulation",
        "_2SigmaDeviationOfPopulation",
        "InterquartileRangeToMedianRatioOfPopulation",
        "InterquartileRangeOfPopulation",
    ],
    "SCT": [
        "_1SigmaLowerValueOfPopulation",
        "_2SigmaLowerValueOfPopulation",
        "_5thPercentileValueOfPopulation",
        "_10thPercentileValueOfPopulation",
        "_1SigmaUpperValueOfPopulation",
        "_2SigmaUpperValueOfPopulation",
        "MeanValueOfPopulation",
        "_95thPercentileValueOfPopulation",
        "_90thPercentileValueOfPopulation",
        "MedianValueOfPopulation",
    ],
}

name_for_cid[7466] = "PETRegionofInterestMeasurement"
cid_concepts[7466] = {
    "DCM": [
        "MetabolicVolume",
        "TotalLesionGlycolysis",
        "Glycolysis",
        "TotalLesionProliferation",
        "ProliferativeActivity",
        "StandardizedAddedMetabolicActivity",
        "StandardizedAddedMetabolicActivityBackground",
        "LesionToBackgroundSUVRatio",
        "BackgroundForLesionToBackgroundSUVRatio",
    ]
}

name_for_cid[7467] = "GrayLevelCooccurrenceMatrixMeasurement"
cid_concepts[7467] = {
    "DCM": ["RootAngularSecondMomentOfGLCM"],
    "IBSI": [
        "CorrelationOfGLCM",
        "DissimilarityOfGLCM",
        "InverseVarianceOfGLCM",
        "AutocorrelationOfGLCM",
        "JointEntropyOfGLCM",
        "FirstMeasureOfInformationCorrelationOfGLCM",
        "AngularSecondMomentOfGLCM",
        "ClusterShadeOfGLCM",
        "ClusterProminenceOfGLCM",
        "ClusterTendencyOfGLCM",
        "DifferenceAverageOfGLCM",
        "InverseDifferenceOfGLCM",
        "JointAverageOfGLCM",
        "SumAverageOfGLCM",
        "InverseDifferenceMomentOfGLCM",
        "NormalizedInverseDifferenceMomentOfGLCM",
        "ContrastOfGLCM",
        "SumVarianceOfGLCM",
        "JointVarianceOfGLCM",
        "SecondMeasureOfInformationCorrelationOfGLCM",
        "NormalizedInverseDifferenceOfGLCM",
        "DifferenceEntropyOfGLCM",
        "SumEntropyOfGLCM",
        "JointMaximumOfGLCM",
        "DifferenceVarianceOfGLCM",
    ],
}

name_for_cid[7468] = "TextureMeasurement"
cid_concepts[7468] = {
    "DCM": ["RootAngularSecondMomentOfGLCM", "FractalDimension"],
    "IBSI": [
        "LowGrayLevelZoneEmphasis",
        "ZoneSizeVariance",
        "NormalizedZoneSizeNonuniformity",
        "IntensityHistogramRange",
        "LongRunHighGrayLevelEmphasis",
        "IntensityHistogramMedian",
        "GreyLevelNonUniformityOfDistanceZoneCounts",
        "InverseVarianceOfGLCM",
        "CoarsenessOfNGTDM",
        "LowDependenceLowGreyLevelEmphasis",
        "DependenceCountNonUniformityNormalized",
        "DependenceCountPercentage",
        "FirstMeasureOfInformationCorrelationOfGLCM",
        "ContrastOfNGTDM",
        "GrayLevelVarianceInSizeZones",
        "IntensityHistogramMinimumGradient",
        "ClusterShadeOfGLCM",
        "ClusterProminenceOfGLCM",
        "ZoneSizeNonuniformity",
        "DependenceCountEnergy",
        "IntensityHistogramVariance",
        "InverseDifferenceOfGLCM",
        "IntensityHistogramMeanAbsoluteDeviation",
        "JointAverageOfGLCM",
        "InverseDifferenceMomentOfGLCM",
        "IntensityHistogramInterquartileRange",
        "ContrastOfGLCM",
        "SmallZoneEmphasis",
        "IntensityHistogram90thPercentile",
        "SmallDistanceLowGreyLevelEmphasis",
        "NormalizedGrayLevelNonuniformityOfSizeZoneCounts",
        "LowDependenceHighGreyLevelEmphasis",
        "IntensityHistogramMedianAbsoluteDeviation",
        "ShortRunLowGrayLevelEmphasis",
        "SizeZonePercentage",
        "GrayLevelVarianceInRuns",
        "HighGrayLevelZoneEmphasis",
        "IntensityHistogramRobustMeanAbsoluteDeviation",
        "HighDependenceHighGreyLevelEmphasis",
        "IntensityHistogram10thPercentile",
        "DependenceCountEntropy",
        "IntensityHistogramMinimumGradientGrayLevel",
        "RunPercentage",
        "GreyLevelVarianceOfNGLDM",
        "DistanceZonePercentage",
        "IntensityHistogramSkewness",
        "IntensityHistogramMaximumGradient",
        "AutocorrelationOfGLCM",
        "LongRunsEmphasis",
        "NormalizedRunLengthNonuniformity",
        "LargeZoneLowGrayLevelEmphasis",
        "DependenceCountVariance",
        "ClusterTendencyOfGLCM",
        "LowGreyLevelZoneEmphasis",
        "HighGreyLevelCountEmphasis",
        "HighDependenceLowGreyLevelEmphasis",
        "DifferenceAverageOfGLCM",
        "IntensityHistogramMaximumGrayLevel",
        "SumAverageOfGLCM",
        "LowDependenceEmphasis",
        "IntensityHistogramQuartileCoefficientOfDispersion",
        "LongRunLowGrayLevelEmphasis",
        "LargeDistanceEmphasis",
        "NormalizedGreyLevelNonUniformityOfNGLDM",
        "SumVarianceOfGLCM",
        "HighGrayLevelRunEmphasis",
        "IntensityHistogramEntropy",
        "ShortRunsEmphasis",
        "HighDependenceEmphasis",
        "IntensityHistogramUniformity",
        "ShortRunHighGrayLevelEmphasis",
        "LargeDistanceHighGreyLevelEmphasis",
        "GrayLevelNonuniformityOfSizeZoneCounts",
        "SmallZoneHighGrayLevelEmphasis",
        "DissimilarityOfGLCM",
        "RunLengthNonuniformity",
        "IntensityHistogramMean",
        "RunLengthVariance",
        "ZoneDistanceEntropy",
        "LowGreyLevelCountEmphasis",
        "LargeZoneEmphasis",
        "RunEntropy",
        "ComplexityOfNGTDM",
        "GreyLevelVarianceInDistanceZones",
        "ZoneDistanceNonUniformity",
        "ZoneDistanceVariance",
        "IntensityHistogramMode",
        "LowGrayLevelRunEmphasis",
        "JointMaximumOfGLCM",
        "DifferenceVarianceOfGLCM",
        "CorrelationOfGLCM",
        "HighGreyLevelZoneEmphasis",
        "BusynessOfNGTDM",
        "IntensityHistogramKurtosis",
        "StrengthOfNGTDM",
        "GreyLevelNonUniformityOfNGLDM",
        "JointEntropyOfGLCM",
        "LargeZoneHighGrayLevelEmphasis",
        "AngularSecondMomentOfGLCM",
        "LargeDistanceLowGreyLevelEmphasis",
        "NormalizedInverseDifferenceMomentOfGLCM",
        "JointVarianceOfGLCM",
        "NormalizedGrayLevelNonuniformityInRuns",
        "SecondMeasureOfInformationCorrelationOfGLCM",
        "IntensityHistogramMaximumGradientGrayLevel",
        "IntensityHistogramMinimumGrayLevel",
        "NormalizedInverseDifferenceOfGLCM",
        "GrayLevelNonuniformityInRuns",
        "SmallZoneLowGrayLevelEmphasis",
        "ZoneSizeEntropy",
        "SmallDistanceHighGreyLevelEmphasis",
        "DifferenceEntropyOfGLCM",
        "SmallDistanceEmphasis",
        "IntensityHistogramCoefficientOfVariation",
        "DependenceCountNonUniformity",
        "SumEntropyOfGLCM",
        "NormalizedGreyLevelNonUniformityOfDistanceZoneCounts",
        "NormalizedZoneDistanceNonUniformity",
    ],
}

name_for_cid[7469] = "GenericIntensityandSizeMeasurement"
cid_concepts[7469] = {
    "DCM": [
        "ApparentDiffusionCoefficient",
        "DiffusionWeighted",
        "LineSegmentLength",
        "VolumeRatio",
        "VolumeOfSphere",
        "DiffusionCoefficient",
        "WaterFatOutOfPhase",
        "FractionalOccupancySegmentation",
        "WaterFatInPhase",
        "FractionalProbabilisticSegmentation",
        "BinarySegmentation",
        "WaterFraction",
        "MRSignalIntensity",
        "XRayAttenuation",
        "VolumeOfCircumscribedSphere",
        "VolumeOfEllipsoid",
        "FatFraction",
        "VolumeEstimatedFromThreeOrMoreNonCoplanar2DRegions",
        "VolumeEstimatedFromTwoNonCoplanar2DRegions",
        "VolumeEstimatedFromSingle2DRegion",
        "PathLength",
        "VolumeOfBoundingThreeDimensionalRegion",
        "YBRICTCRComponent",
        "R2Coefficient",
        "YBRICTCBComponent",
        "ChiSquare",
        "YBRICTYComponent",
        "DW",
        "YBRPARTIALCRComponent",
        "AIC",
        "YBRPARTIALCBComponent",
        "BIC",
        "YBRPARTIALYComponent",
        "Echogenicity",
        "YBRRCTCRComponent",
        "YBRRCTCBComponent",
        "YBRRCTYComponent",
        "Distance",
        "Height",
        "RelativeLinearStoppingPower",
        "RGBRComponent",
        "UltrasoundAttenuation",
        "TauM",
        "SpeedOfSound",
        "Vp",
        "Perfusion",
        "Elasticity",
        "EquivalentDose",
        "GradientWeightedClassActivation",
        "ClassActivation",
        "AbsorbedDose",
        "YBRFULLCRComponent",
        "Saliency",
        "YBRFULLCBComponent",
        "YBRFULLYComponent",
        "RGBBComponent",
        "RGBGComponent",
        "CreatineCholineCitrateRatio",
        "MagnetizationTransferRatio",
        "IAUC90BN",
        "IAUC180BN",
        "CreatineAndCholine",
        "LipidAndLactate",
        "SpatialDisplacementYComponent",
        "SpatialDisplacementXComponent",
        "NuclearMedicineTomographicActivity",
        "IAUC",
        "NuclearMedicineProjectionActivity",
        "IAUC60",
        "IAUC90",
        "IAUC180",
        "IAUCBN",
        "IAUC60BN",
        "FlowVariance",
        "FlowVelocity",
        "TissueVelocity",
        "IndexedHemodynamicResistance",
        "HemodynamicResistance",
        "SpatialDisplacementZComponent",
        "AttenuationCoefficient",
        "GlutamateAndGlutamine",
        "CholineCreatineRatio",
        "NAcetylaspartateCreatineRatio",
        "NAcetylaspartateCholineRatio",
        "Tmax",
        "R2Star",
        "VolumetricDiffusionDxzComponent",
        "MagneticSusceptibility",
        "VolumetricDiffusionDxyComponent",
        "VolumetricDiffusionDxxComponent",
        "RelativeRegionalBloodFlow",
        "RelativeRegionalBloodVolume",
        "Ktrans",
        "Kep",
        "Ve",
        "BloodOxygenationLevel",
        "T2StarWeightedDynamicContrastEnhancedMRSignalIntensity",
        "T2WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodFlow",
        "T1WeightedDynamicContrastEnhancedMRSignalIntensity",
        "AbsoluteRegionalBloodVolume",
        "VolumetricDiffusionDzzComponent",
        "OxygenExtractionFraction",
        "R1",
        "VolumetricDiffusionDyzComponent",
        "R2",
        "VolumetricDiffusionDyyComponent",
        "VelocityEncoded",
        "ZScore",
        "ContrastAgentAngioMRSignalIntensity",
        "SpinTaggingPerfusionMRSignalIntensity",
        "RelativeAnisotropy",
        "FractionalAnisotropy",
        "FieldMapMRSignalIntensity",
        "T2StarWeightedMRSignalIntensity",
        "T2WeightedMRSignalIntensity",
        "T1WeightedMRSignalIntensity",
        "ProtonDensityWeightedMRSignalIntensity",
        "TimeOfFlightAngioMRSignalIntensity",
        "T1",
        "T2Star",
        "T2",
        "TimeCourseOfSignal",
        "TemperatureEncoded",
        "StudentTTest",
        "TimeToPeak",
        "SignalToNoise",
        "TemporalDerivativeThreshold",
        "MaximumSlope",
        "MaximumDifference",
        "TracerConcentration",
        "TimeOfPeakConcentration",
        "BolusArrivalTime",
        "TimeOfLeadingHalfPeakConcentration",
        "MeanTransitTime",
        "KurtosisDiffusionCoefficient",
        "GammaDistributionScaleParameter",
        "NegativeEnhancementIntegral",
        "GammaDistributionShapeParameter",
        "GammaDistributionMode",
        "DistributedDiffusionCoefficient",
        "RCoefficient",
        "AnomalousExponentParameter",
        "ProtonDensity",
        "SignalChange",
        "Suvbsa",
        "Suvibw",
        "SuvlbmJanma",
        "SuvlbmJames128",
        "MonoExponentialApparentDiffusionCoefficient",
        "SlowDiffusionCoefficient",
        "FastDiffusionCoefficient",
        "FastDiffusionCoefficientFraction",
        "StandardizedUptakeValue",
        "Suvbw",
        "Suvlbm",
    ],
    "IBSI": [
        "SurfaceAreaOfMesh",
        "Maximum3DDiameterOfAMesh",
        "VolumeOfMesh",
        "LeastAxisIn3DLength",
        "MinorAxisIn3DLength",
        "VolumeFromVoxelSummation",
        "MajorAxisIn3DLength",
    ],
    "SCT": [
        "Width",
        "Volume",
        "Fat",
        "Area",
        "Lipid",
        "DiameterOfCircumscribedCircle",
        "Depth",
        "NAcetylaspartate",
        "PowerDoppler",
        "Perimeter",
        "Creatine",
        "Glutamine",
        "MinorAxis",
        "Diameter",
        "Tuarine",
        "Citrate",
        "MajorAxis",
        "Lactate",
        "LongAxis",
        "Length",
        "Circumference",
        "Inositol",
        "PerpendicularAxis",
        "Water",
        "Radius",
        "Choline",
        "AreaOfDefinedRegion",
        "ShortAxis",
    ],
}

name_for_cid[7470] = "LinearMeasurement"
cid_concepts[7470] = {
    "DCM": ["Distance", "Height", "LineSegmentLength", "PathLength"],
    "IBSI": [
        "LeastAxisIn3DLength",
        "MinorAxisIn3DLength",
        "MajorAxisIn3DLength",
        "Maximum3DDiameterOfAMesh",
    ],
    "SCT": [
        "DiameterOfCircumscribedCircle",
        "Radius",
        "Perimeter",
        "MinorAxis",
        "Width",
        "Diameter",
        "MajorAxis",
        "PerpendicularAxis",
        "LongAxis",
        "Depth",
        "ShortAxis",
        "Length",
        "Circumference",
    ],
}

name_for_cid[7471] = "AreaMeasurement"
cid_concepts[7471] = {
    "IBSI": ["SurfaceAreaOfMesh"],
    "SCT": ["Area", "AreaOfDefinedRegion"],
}

name_for_cid[7472] = "VolumeMeasurement"
cid_concepts[7472] = {
    "DCM": [
        "VolumeEstimatedFromThreeOrMoreNonCoplanar2DRegions",
        "VolumeEstimatedFromTwoNonCoplanar2DRegions",
        "VolumeEstimatedFromSingle2DRegion",
        "VolumeOfSphere",
        "VolumeOfCircumscribedSphere",
        "VolumeOfEllipsoid",
        "VolumeOfBoundingThreeDimensionalRegion",
    ],
    "IBSI": ["VolumeFromVoxelSummation", "VolumeOfMesh"],
    "SCT": ["Volume"],
}

name_for_cid[7473] = "GeneralAreaCalculationMethod"
cid_concepts[7473] = {"DCM": ["AreaOfAClosedNURBS", "AreaOfClosedIrregularPolygon"]}

name_for_cid[7474] = "GeneralVolumeCalculationMethod"
cid_concepts[7474] = {
    "DCM": [
        "IntegrationOfSumOfClosedAreasOnContiguousSlicesMethodForVolume",
        "LWHMethodForVolumeOfEllipsoid",
        "SumOfSegmentedVoxelMethodForVolume",
    ]
}

name_for_cid[7475] = "GrayLevelRunLengthBasedFeature"
cid_concepts[7475] = {
    "IBSI": [
        "RunEntropy",
        "LongRunHighGrayLevelEmphasis",
        "HighGrayLevelRunEmphasis",
        "NormalizedGrayLevelNonuniformityInRuns",
        "RunPercentage",
        "LongRunsEmphasis",
        "NormalizedRunLengthNonuniformity",
        "ShortRunsEmphasis",
        "GrayLevelNonuniformityInRuns",
        "ShortRunLowGrayLevelEmphasis",
        "RunLengthNonuniformity",
        "GrayLevelVarianceInRuns",
        "RunLengthVariance",
        "ShortRunHighGrayLevelEmphasis",
        "LowGrayLevelRunEmphasis",
        "LongRunLowGrayLevelEmphasis",
    ]
}

name_for_cid[7476] = "GrayLevelSizeZoneBasedFeature"
cid_concepts[7476] = {
    "IBSI": [
        "LargeZoneEmphasis",
        "LowGrayLevelZoneEmphasis",
        "ZoneSizeVariance",
        "NormalizedZoneSizeNonuniformity",
        "GrayLevelNonuniformityOfSizeZoneCounts",
        "SmallZoneEmphasis",
        "SmallZoneHighGrayLevelEmphasis",
        "SmallZoneLowGrayLevelEmphasis",
        "NormalizedGrayLevelNonuniformityOfSizeZoneCounts",
        "ZoneSizeEntropy",
        "LargeZoneLowGrayLevelEmphasis",
        "LargeZoneHighGrayLevelEmphasis",
        "GrayLevelVarianceInSizeZones",
        "ZoneSizeNonuniformity",
        "SizeZonePercentage",
        "HighGrayLevelZoneEmphasis",
    ]
}

name_for_cid[7477] = "GlobalShapeDescriptor"
cid_concepts[7477] = {
    "IBSI": [
        "VolumeDensityInOrientedMinimumBoundingBox",
        "LeastAxisIn3DLength",
        "VolumeDensityInApproximateEnclosingEllipsoid",
        "AreaDensityInApproximateEnclosingEllipsoid",
        "SurfaceToVolumeRatio",
        "MinorAxisIn3DLength",
        "IntegratedIntensity",
        "AreaDensityInMinimumVolumeEnclosingEllipsoid",
        "Compactness2",
        "Maximum3DDiameterOfAMesh",
        "GearyCMeasure",
        "MoranIIndex",
        "Sphericity",
        "Elongation",
        "VolumeFromVoxelSummation",
        "CentreOfMassShift",
        "VolumeDensityInMinimumVolumeEnclosingEllipsoid",
        "AreaDensityInConvexHull",
        "VolumeOfMesh",
        "MajorAxisIn3DLength",
        "AreaDensityInFrameOfReferenceAxisAlignedBoundingBox",
        "AreaDensityInOrientedMinimumBoundingBox",
        "VolumeDensityInConvexHull",
        "Compactness1",
        "SphericalDisproportion",
        "Flatness",
        "SurfaceAreaOfMesh",
        "VolumeDensityInFrameOfReferenceAxisAlignedBoundingBox",
        "Asphericity",
    ]
}

name_for_cid[7478] = "IntensityHistogramFeature"
cid_concepts[7478] = {
    "IBSI": [
        "IntensityHistogramInterquartileRange",
        "IntensityHistogramRange",
        "IntensityHistogramMedian",
        "IntensityHistogramMinimumGradientGrayLevel",
        "IntensityHistogramSkewness",
        "IntensityHistogramMaximumGradientGrayLevel",
        "IntensityHistogram90thPercentile",
        "IntensityHistogramMaximumGradient",
        "IntensityHistogramKurtosis",
        "IntensityHistogramMinimumGrayLevel",
        "IntensityHistogramEntropy",
        "IntensityHistogramMedianAbsoluteDeviation",
        "IntensityHistogramMode",
        "IntensityHistogramMinimumGradient",
        "IntensityHistogramMean",
        "IntensityHistogramVariance",
        "IntensityHistogramCoefficientOfVariation",
        "IntensityHistogramRobustMeanAbsoluteDeviation",
        "IntensityHistogramUniformity",
        "IntensityHistogramMaximumGrayLevel",
        "IntensityHistogramMeanAbsoluteDeviation",
        "IntensityHistogram10thPercentile",
        "IntensityHistogramQuartileCoefficientOfDispersion",
    ]
}

name_for_cid[7479] = "GreyLevelDistanceZoneBasedFeature"
cid_concepts[7479] = {
    "IBSI": [
        "LargeDistanceHighGreyLevelEmphasis",
        "HighGreyLevelZoneEmphasis",
        "GreyLevelNonUniformityOfDistanceZoneCounts",
        "DistanceZonePercentage",
        "SmallDistanceLowGreyLevelEmphasis",
        "GreyLevelVarianceInDistanceZones",
        "ZoneDistanceNonUniformity",
        "SmallDistanceHighGreyLevelEmphasis",
        "ZoneDistanceVariance",
        "SmallDistanceEmphasis",
        "LowGreyLevelZoneEmphasis",
        "ZoneDistanceEntropy",
        "LargeDistanceLowGreyLevelEmphasis",
        "LargeDistanceEmphasis",
        "NormalizedGreyLevelNonUniformityOfDistanceZoneCounts",
        "NormalizedZoneDistanceNonUniformity",
    ]
}

name_for_cid[7480] = "Breed"
cid_concepts[7480] = {
    "SCT": [
        "CanastrãoPigBreed",
        "FrenchArdennaisHorseBreed",
        "CanadianDurocPigBreed",
        "CriouloHorseBreed",
        "AvetonouCattleBreed",
        "Spaniel",
        "KigeziCattleBreed",
        "HuazhongTwoEndBlackSatzelingPigBreed",
        "CoopworthSheepBreed",
        "RatTerrierDogBreed",
        "MixedBreedDog",
        "AustralianBrafordXZebuCattleBreed",
        "EskimoDog",
        "SouthMalawiZebuCattleBreed",
        "TropicalDairyCattleCattleBreed",
        "CorsicanGoatBreed",
        "ShropshireSheepBreed",
        "KirgizHorseBreed",
        "LaoshanPigBreed",
        "SanMartineroCattleBreed",
        "ClunForestSheepBreed",
        "PapillonDog",
        "HrbineckyCattleBreed",
        "SeshagaCattleBreed",
        "WirehairStyrianMountainDogBreed",
        "PerianganHorseBreed",
        "ScottishDeerhound",
        "MurgeseHorseBreed",
        "UmblacheryCattleBreed",
        "ToggenburgGoatBreed",
        "SarabiCattleBreed",
        "CalabrianHorseBreed",
        "Dalland020PigBreed",
        "GeorgianMountainCattleBreed",
        "GasaraCattleBreed",
        "StandardDachshundDogBreed",
        "AfghanHound",
        "BelgianHalfbloodHorseBreed",
        "MonarchPigBreed",
        "IstrianPramenkaSheepBreed",
        "ShilohShepherdDogBreed",
        "BaluchiSheepBreed",
        "RostovPigBreed",
        "NormanzuXZebuCattleBreed",
        "TaishuhHorseBreed",
        "CubanPasoHorseBreed",
        "SchwyzZeboidXZebuCattleBreed",
        "UralBlackPiedCattleBreed",
        "LonghornCowBreed",
        "ConnemaraPonyHorseBreed",
        "GermanSpanielDogBreed",
        "OuessantSheepBreed",
        "AustralianBrumbyHorseBreed",
        "NorthSwedishHorseHorseBreed",
        "DanishRedPiedCattleBreed",
        "EnglishHunterHorseBreed",
        "AmericanBuckskinHorseBreed",
        "ChinPigBreed",
        "SouthAfricanBrownSwissCattleBreed",
        "SmallEastAfricanZebuCattleBreed",
        "BraccoItalianoDogBreed",
        "NurasCattleBreed",
        "IlocosCattleBreed",
        "CriolloSheepBreed",
        "AmericanBerkshirePigBreed",
        "NorthSwedishTrotterHorseBreed",
        "AmericanLandracePigBreed",
        "LantangPigBreed",
        "MangalargaPaulistaHorseBreed",
        "JiaoxiPigBreed",
        "ItalianRedPiedCattleBreed",
        "WakwaCattleBreed",
        "YugoslavianHoundDogBreed",
        "MiniatureZebuCattleBreed",
        "IrishWolfhound",
        "SmoothFoxTerrierDogBreed",
        "GermanBerkshirePigBreed",
        "VyatkaHorseBreed",
        "BielleseSheepBreed",
        "OldFormatWelshCorgiDogBreed",
        "HinisHorseBreed",
        "PolledLincolnRedCattleBreed",
        "BeijingBlackPiedCattleBreed",
        "WirehairedPointingGriffonDog",
        "ChaparGoatBreed",
        "LöwchenDogBreed",
        "DairyZebuOfUberabaCattleBreed",
        "AmericanCockerSpanielDogBreed",
        "PhilippineNativeIlocosPigBreed",
        "TouabireSheepBreed",
        "KhurasaniZebuCattleBreed",
        "YujiangPigBreed",
        "HampshireSheepBreed",
        "RhoenschafSheepBreed",
        "LeopardCurDogBreed",
        "HeilongjiangSpottedPigBreed",
        "MeishanPigBreed",
        "IcelandicHorseBreed",
        "SiberianHuskie",
        "CachenaCattleBreed",
        "ThraceCattleBreed",
        "BritishWhiteCattleBreed",
        "SlovakWarmbloodHorseBreed",
        "DachshundMiniatureDogBreed",
        "BoarPowerPig59PigBreed",
        "OrobicaGoatBreed",
        "GlenOfImaalTerrierDogBreed",
        "AlpineChamoiseeGoatBreed",
        "SwaledaleSheepBreed",
        "KirhizDogBreed",
        "GentileDiPugliaSheepBreed",
        "BelgianLaekenDog",
        "TunisSheepBreed",
        "HunterHorseBreed",
        "SanPierrePigBreed",
        "KarakulSheepBreed",
        "AmericanPaintHorseBreed",
        "DamaraCattleBreed",
        "COOPELSO93CattleBreed",
        "KaghaniGoatBreed",
        "LipizzanerHorseBreed",
        "AzerbaijanZebuCattleBreed",
        "CasertanaPigBreed",
        "DobrogeaBlackPigBreed",
        "GermanWhiteheadedMuttonSheepBreed",
        "Line32PigBreed",
        "LucernaCattleBreed",
        "BassetArtésianNormandDogBreed",
        "LaguneCattleBreed",
        "YonaguniHorseBreed",
        "AostaRedPiedCattleBreed",
        "WenshanCattleBreed",
        "JapanesePointerDogBreed",
        "BritishBlueCatBreed",
        "NiloPigBreed",
        "PonwarCattleBreed",
        "SL96PigBreed",
        "DutchBeltedCattleBreed",
        "PomeranianDog",
        "VestlandRedPolledCattleBreed",
        "BrafordXZebuCattleBreed",
        "FineMerinoSheepBreed",
        "CavallinoPigBreed",
        "FinnhorseDraftHorseBreed",
        "PyreneanMastiffDogBreed",
        "LaManchaGoatBreed",
        "GasconCattleBreed",
        "AustralianTerrier",
        "BedlingtonTerrier",
        "NamibDesertHorseHorseBreed",
        "DutchDraftHorseBreed",
        "PartiColorCockerSpanielDogBreed",
        "BelgianGriffonRoughDogBreed",
        "CasanarenoCattleBreed",
        "JapaneseChinDog",
        "KemerovoPigBreed",
        "RussianHeavyDraftHorseBreed",
        "WestHighlandWhiteTerrier",
        "PolledSussexCattleBreed",
        "ShakhansurriCattleBreed",
        "GordonSetterDogBreed",
        "AuxoisHorseBreed",
        "CanadianLandracePigBreed",
        "PoitouMuleProducerHorseBreed",
        "IrishTerrier",
        "DingoDogBreed",
        "EastAndSoutheastAnadoluHorseBreed",
        "WhitebredShorthornCattleBreed",
        "LesserCaucasusCattleBreed",
        "ToroCattleBreed",
        "BasutoCattleBreed",
        "BrownSwissCowBreed",
        "BeagleElizabethanDogBreed",
        "BalkhiSheepBreed",
        "SuksunCattleBreed",
        "EnglishShepherdDogBreed",
        "IsraeliHolsteinCattleBreed",
        "SaintBernardLongHairedDogBreed",
        "YorkshireTerrier",
        "EpagneulPontAudemerDogBreed",
        "BalinesePigBreed",
        "MirgorodPigBreed",
        "BergerShetlandDogBreed",
        "IndoBrazilianCattleBreed",
        "CholistaniSheepBreed",
        "YananPigBreed",
        "CroatianRedCattleBreed",
        "KangayamCattleBreed",
        "CharolaisCattleBreed",
        "Terrier",
        "SarPlaninaSheepBreed",
        "TurkomanHorseBreed",
        "GoleCattleBreed",
        "KaranSwissXZebuCattleBreed",
        "SardinianCattleBreed",
        "EurolinePigBreed",
        "JutlandHorseBreed",
        "VerataGoatBreed",
        "KoreanImprovedPigBreed",
        "EdelschweinPigBreed",
        "PenArLan77PigBreed",
        "LimpurgerCattleBreed",
        "MurcianCattleBreed",
        "LokaiHorseBreed",
        "ColoredBullTerrierDogBreed",
        "EasternNuerCattleBreed",
        "BrahmaCowBreed",
        "GriffonFauveDeBretegneDogBreed",
        "MurcianaGranadinaGoatBreed",
        "AlistanaSanabresaCattleBreed",
        "MalteseCatBreed",
        "HerefordPigBreed",
        "EvolèneCattleBreed",
        "TropicalXZebuCattleBreed",
        "VerzascaGoatBreed",
        "CerbatHorseBreed",
        "CharnequeriaGoatBreed",
        "MandiPigBreed",
        "MagalCattleBreed",
        "PolesianPigBreed",
        "WelshSpringerSpanielDogBreed",
        "TerskHorseBreed",
        "AustralianMilkingZebuXZebuCattleBreed",
        "AriégeoisPonyHorseBreed",
        "MixedBreedChicken",
        "FleuveHorseBreed",
        "GermanLonghairedPointerDogBreed",
        "OropaCattleBreed",
        "PigghamPigBreed",
        "BrazilianSomaliSheepBreed",
        "AnadoluHorseBreed",
        "KoreanBlackCattleBreed",
        "CatalinaIslandPigBreed",
        "BergerDePyreneesDogBreed",
        "LonghairedManx",
        "EnderbyIslandCattleBreed",
        "AnatolianBlackCattleBreed",
        "ZnamenskCattleBreed",
        "SumbaHorseBreed",
        "XinjiangWhitePigBreed",
        "ToronkeCattleBreed",
        "EnglishToyTerrierDogBreed",
        "EastFriesianSheepBreed",
        "JemJemZebuCattleBreed",
        "BoarPowerPig747PigBreed",
        "BakosiPigBreed",
        "NamaCattleBreed",
        "HungarianGrayCattleBreed",
        "HissarCattleBreed",
        "EstonianHoundDogBreed",
        "CalabrianCattleBreed",
        "RedPollCowBreed",
        "Camborough15PigBreed",
        "PresticePigBreed",
        "PriobHorseBreed",
        "LohiSheepBreed",
        "ChartPolskiDogBreed",
        "BrazilianDairyHybridXZebuCattleBreed",
        "KleenLeenWhitePigBreed",
        "CabanninaCattleBreed",
        "TurkishBrownCattleBreed",
        "BritishAppaloosaHorseBreed",
        "KabyleCattleBreed",
        "DøleHorseBreed",
        "AmericanBobtailCatBreed",
        "VizslaSuperbreedDogBreed",
        "AztecaHorseBreed",
        "TransylvanianHoundDogBreed",
        "BlackForestHoundDogBreed",
        "SouthSuffolkSheepBreed",
        "HorroCattleBreed",
        "SprecaCattleBreed",
        "BragadoDoSorraiaCattleBreed",
        "LithuanianRedCattleBreed",
        "BelgianTervurenDog",
        "BreitovPigBreed",
        "RahvanHorseBreed",
        "CantonesePigBreed",
        "VietnamesePigBreed",
        "PhilippineNativePigBreed",
        "SwedishLapphundDogBreed",
        "AbyssinianShorthornedZebuCattleBreed",
        "JidduCattleBreed",
        "SouthRussianOvcharkaDogBreed",
        "CharaHorseHorseBreed",
        "SovietMeatPigBreed",
        "ChantillyTiffanyCatBreed",
        "MerinolandschafSheepBreed",
        "CaldeanoCattleBreed",
        "AmericanSaddlebredHorseBreed",
        "BardigianoHorseBreed",
        "ThuringianGoatBreed",
        "SpitiHorseBreed",
        "GrayAlpineCattleBreed",
        "AndalusianBlackCattleBreed",
        "BritishLopPigBreed",
        "AmericanEskimoToyDogBreed",
        "MarchigianaCowBreed",
        "YorkshireCoachHorseBreed",
        "AradoCattleBreed",
        "LleynSheepBreed",
        "GolpayeganiCattleBreed",
        "PinzhouCattleBreed",
        "ArmantDogBreed",
        "SmallWhitePigBreed",
        "PonyHorseBreed",
        "AngloFrancaisDePetitVenerieDogBreed",
        "RottalHorseBreed",
        "PixieBobCatBreed",
        "GuineaHogPigBreed",
        "CalasparrenaCattleBreed",
        "WestAfricanDwarfSheepBreed",
        "RätienGrayCattleBreed",
        "SinoVietnamesePigBreed",
        "RomanianLargeWhitePigBreed",
        "KhorsanCattleBreed",
        "CornishRexCatBreed",
        "KabardaHorseBreed",
        "KisantuCattleBreed",
        "JabeHorseBreed",
        "SelkirkRexCatBreed",
        "EnglishCoonhoundDogBreed",
        "BritishAlpineGoatBreed",
        "MazandaraniCattleBreed",
        "PagliarolaSheepBreed",
        "OldDanishBirdDogBreed",
        "BelgianBlueCattleBreed",
        "HattonXZebuCattleBreed",
        "NewfoundlandPonyHorseBreed",
        "RomanianBrownCattleBreed",
        "FinnishAyrshireCattleBreed",
        "LiangGuangSmallSpottedLuchuanPigBreed",
        "YemeniHorsesHorseBreed",
        "WallisCountrySheepBreed",
        "CotswoldGoldPigBreed",
        "MudiDogBreed",
        "BorneoZebuCattleBreed",
        "DBIPigBreed",
        "SchwerfurtMeatPigBreed",
        "ErlunchunHorseBreed",
        "BelgianWolfhoundDogBreed",
        "MukotaPigBreed",
        "GuzeratCattleBreed",
        "DerbyshireGritstoneSheepBreed",
        "CampanianBarbarySheepBreed",
        "SykiaCattleBreed",
        "ChifordCattleBreed",
        "KhevsurianCattleBreed",
        "ZanzibarZebuCattleBreed",
        "FloridaCrackerHorseBreed",
        "ModicanaLowlandCattleBreed",
        "BorderTerrier",
        "MixedBreedPig",
        "SwissWarmbloodHorseBreed",
        "GiantSchnauzerDogBreed",
        "NavajoChurroSheepBreed",
        "NorwichTerrier",
        "JamaicaHopeXZebuCattleBreed",
        "NewForestPonyHorseBreed",
        "ItalianGreyhound",
        "BlondeDAquitaineCowBreed",
        "SlovakianWhitePigBreed",
        "BelgiumLandracePigBreed",
        "CalascibettaPigBreed",
        "AdaevHorseBreed",
        "MiniatureSchnauzerDogBreed",
        "ChinoSantandereanoCattleBreed",
        "AmericanBulldogBreed",
        "RussianWhiteGoatBreed",
        "MixedBreedCattle",
        "BapediCattleBreed",
        "BeltsvillePigBreed",
        "CaspianHorseBreed",
        "PolishMarshPigBreed",
        "PoodleSuperbreedDogBreed",
        "ZlotnikiSpottedPigBreed",
        "RomanianNativeStocliPigBreed",
        "NelloreCowBreed",
        "GermanBlackheadedMuttonSheepBreed",
        "JindoDogBreed",
        "HainanTunchangPigBreed",
        "BelgianStreetDogBreed",
        "AustralianMistCatBreed",
        "AsturianValleyCattleBreed",
        "CãoDeCastroLaboreiroDogBreed",
        "ArouquesaCattleBreed",
        "DesFossesCommunesDeLOuestGoatBreed",
        "BoarPowerPig616PigBreed",
        "CiritHorseBreed",
        "CamargueCattleBreed",
        "LinebackCattleBreed",
        "DKPig61PigBreed",
        "BameiPigBreed",
        "CoPigBreed",
        "GambianNDamaXZebuCattleBreed",
        "YanbianCattleBreed",
        "TajmaCattleBreed",
        "DnepropetrovskPigBreed",
        "AbigarCattleBreed",
        "HeilongkaingHorseBreed",
        "LargePolishLongEaredPigBreed",
        "MurrayGrayCattleBreed",
        "HanoverianHoundDogBreed",
        "ResavaPigBreed",
        "AsturianTarpanHorseXDomesticHorseBreed",
        "SomaliCat",
        "PuertoRicanCriolloCattleBreed",
        "PerdigueiroPortugueseDogBreed",
        "NorwegianForestCatBreed",
        "AidiDogBreed",
        "DalmatianDog",
        "BassetFauveDeBretagneDogBreed",
        "RackaSheepBreed",
        "BlacksidedTrondheimAndNorlandCattleBreed",
        "BolivianPigBreed",
        "AmericanTrotterHorseBreed",
        "NorwegianLandracePigBreed",
        "UjumqinCattleBreed",
        "DamaniGoatBreed",
        "EnglishHackHorseBreed",
        "RyelandSheepBreed",
        "NorthSomaliZebuCattleBreed",
        "Cotswold90PigBreed",
        "BelgianMalinoisDog",
        "KushumHorseBreed",
        "PeacockGoatBreed",
        "ThaiBinhPigBreed",
        "TharparkarCattleBreed",
        "ChamoiseeGoatBreed",
        "PomeranianPigBreed",
        "KholmogoryCattleBreed",
        "DhanniCattleBreed",
        "EastMacedonianCattleBreed",
        "SwedishVallhundDogBreed",
        "BombayCat",
        "BlueTickCoonhoundDogBreed",
        "TanzanianZebuCattleBreed",
        "PindosPonyHorseBreed",
        "TawleedHorseBreed",
        "GuangxiWhitePigBreed",
        "ChienFrançaisBlancEtOrangeDogBreed",
        "PankotaRedCattleBreed",
        "SwedishFurSheepBreed",
        "TaraiCattleBreed",
        "BarbariGoatBreed",
        "KarachaiHorseBreed",
        "RaadPigBreed",
        "BretonBlackPiedCattleBreed",
        "CanastrãoVermelhoPigBreed",
        "MustangHorseBreed",
        "ItalianLandracePigBreed",
        "CzechoslovakianSmallRidingHorseHorseBreed",
        "PesterCattleBreed",
        "DanubianHorseBreed",
        "IbanPigBreed",
        "PalmeraCattleBreed",
        "LaviniaXZebuCattleBreed",
        "BurmeseCat",
        "CanadianSportHorseHorseBreed",
        "MalviCattleBreed",
        "FrenchGuardDogBreed",
        "HampshirePigBreed",
        "DevonRexCatBreed",
        "MaryutiCattleBreed",
        "NarymHorseBreed",
        "AlaTauCattleBreed",
        "GalicenoHorseBreed",
        "GüssingForestPigPigBreed",
        "KangarooIslandPigBreed",
        "KyogaCattleBreed",
        "OwczarekPodhalandskiDogBreed",
        "LundyPonyHorseBreed",
        "BosTaurusIndicusCowBreed",
        "NoniusHorseBreed",
        "WessexSaddlebackPigBreed",
        "MezenHorseBreed",
        "QinchuanCattleBreed",
        "MampatiCattleBreed",
        "BullTerrier",
        "SpanishAmericanHorseHorseBreed",
        "FrijolilloCattleBreed",
        "ZlotnikiWhitePigBreed",
        "WeisseHornloseHeidschnuckeSheepBreed",
        "XinjinJilinBlackPigBreed",
        "ChiTerrierDogBreed",
        "SonValleyCattleBreed",
        "ScottishHighlandCowBreed",
        "BenadirGoatBreed",
        "BoarPowerPig474PigBreed",
        "ChesterWhitePigBreed",
        "RazorBackPigBreed",
        "PhilippineNativeCattleBreed",
        "CanadianRoyalBluePigBreed",
        "HornedHerefordCowBreed",
        "CarpathianPonyHorseBreed",
        "DKPig77PigBreed",
        "NiloticSangaCattleBreed",
        "BoulonnaisHorseBreed",
        "ReniteloXZebuCattleBreed",
        "NorwegianRedCattleBreed",
        "CaucasianMountainDogBreed",
        "ChenghuaPigBreed",
        "BlancDeBouscatRabbitBreed",
        "BarkaZebuCattleBreed",
        "SicilianBarbarySheepBreed",
        "BeagleStandardDogBreed",
        "AdamawaCattleBreed",
        "BorderLeicesterSheepBreed",
        "TibetanSpaniel",
        "LincolnRedCowBreed",
        "AnkoleWatusiCattleBreed",
        "PekingeseDog",
        "CréolePigBreed",
        "StCroixSheepBreed",
        "KikoGoatBreed",
        "SouthRussianSteppeHoundDogBreed",
        "SpanishNormanHorseBreed",
        "FortCrossCattleBreed",
        "BorgouXZebuCattleBreed",
        "SpanishMastiffDogBreed",
        "BurlinaCattleBreed",
        "HillRadnorSheepBreed",
        "MastergiltPigBreed",
        "DRUTerminalsPigBreed",
        "ChineseCrestedDogBreed",
        "SchwyzCattleBreed",
        "SouthwestSpanishMustangHorseBreed",
        "CzechWarmbloodHorseBreed",
        "BulgarianNativePigBreed",
        "AmericanCashmereGoatBreed",
        "GriffonDog",
        "HangPigBreed",
        "SphynxCatBreed",
        "DongolaCattleBreed",
        "PolledNeloreCattleBreed",
        "DebouilletSheepBreed",
        "MerinoSheepSuperbreed",
        "HawaiianWildCattleBreed",
        "SaanenGoatBreed",
        "LonkSheepBreed",
        "BantuPigBreed",
        "HuazhongTwoEndBlackPigBreed",
        "KalakheriCattleBreed",
        "BlackBaldyCattleBreed",
        "ThanhHoaCattleBreed",
        "PindosHorseBreed",
        "HavanaBrownCat",
        "ChianinaCowBreed",
        "MixedDairyCattleBreed",
        "BadenWurttemburgHorseBreed",
        "BostonTerrier",
        "BlackFacedHighlandSheepBreed",
        "JersindXZebuCattleBreed",
        "SchnauzerSuperbreedDogBreed",
        "GroningenWhiteheadedCattleBreed",
        "BraqueDAriégeDogBreed",
        "DidingaCattleBreed",
        "KatahdinSheepBreed",
        "XinjiangFinewoolSheepBreed",
        "PembrokeWelshCorgiDogBreed",
        "HungarianDunHorseBreed",
        "PolishBlackAndWhiteLowlandCattleBreed",
        "NewZealandHeadingDogBreed",
        "SaidiCattleBreed",
        "BalikunHorseBreed",
        "PechoraCattleBreed",
        "MessaoriaCattleBreed",
        "AliabDinkaCattleBreed",
        "SkyrosPonyHorseBreed",
        "RoughAndSmoothDogBreed",
        "OliventinaPigBreed",
        "KrskopoljePigBreed",
        "AmericanLamaleseDogBreed",
        "EnglishSetterDogBreed",
        "PirapetingaPigBreed",
        "MinisibPigBreed",
        "FalabellaHorseBreed",
        "PatibarcinaHorseBreed",
        "SarvarHorseBreed",
        "VorderwaldCattleBreed",
        "AlpineDachsbrackeDogBreed",
        "MissouriFoxTrottingHorseBreed",
        "EnglishCobHorseBreed",
        "QingpingPigBreed",
        "LewitzerHorseBreed",
        "RendenaCattleBreed",
        "TeeswaterSheepBreed",
        "DomesticMediumHairedCat",
        "BashkirCurlyHorseBreed",
        "OldNorwegianSheepBreed",
        "PitangueirasXZebuCattleBreed",
        "DonGoatBreed",
        "DachshundSuperbreedOfDog",
        "FoxTerrierSuperbreedDogBreed",
        "PutianPigBreed",
        "GuanlingPigBreed",
        "PorcelaineDogBreed",
        "MiniaturePinscherDog",
        "JapaneseSpaniel",
        "SardoModicanaCattleBreed",
        "KamdhinoCattleBreed",
        "MongolianCattleBreed",
        "SinoGasconyPigBreed",
        "JerseyCowBreed",
        "ShkodraRedCattleBreed",
        "ToyPoodleDogBreed",
        "LatukaCattleBreed",
        "BulgarianWhitePigBreed",
        "SomaliSheepBreed",
        "KalmykCattleBreed",
        "AlentejanaCattlebreed",
        "MegezhHorseBreed",
        "KaiKenDogBreed",
        "DamascusCattleBreed",
        "BamiCattleBreed",
        "WalachenschafSheepBreed",
        "AngelnSaddlebackPigBreed",
        "ShetlandPonyHorseBreed",
        "FauveDeBretagneDogBreed",
        "CambridgeSheepBreed",
        "ModeneseCattleBreed",
        "RussianBlueCat",
        "GulfCoastNativeSheepBreed",
        "SouthDevonCattleBreed",
        "HolsteinerHorseBreed",
        "KenyaBoranCattleBreed",
        "EastFinnishCattleBreed",
        "TreeingCurDogBreed",
        "RyaSheepBreed",
        "MonchinaCattleBreed",
        "NigerianDwarfGoatBreed",
        "KunekunePigBreed",
        "GermanRedPiedPigBreed",
        "MarylandPigBreed",
        "ReggianaCattleBreed",
        "LurcherHoundDogBreed",
        "RomneletSheepBreed",
        "PantaneiroHorseBreed",
        "MarwariHorseBreed",
        "GuadarramaGoatBreed",
        "FonthillMerinoSheepBreed",
        "BordeauxDogBreed",
        "SlovakianPinzgauCattleBreed",
        "BoarPowerPig141PigBreed",
        "TielingHorseBreed",
        "BlackHampshirePigBreed",
        "SouthernCriouloCattleBreed",
        "QuarterPonyHorseBreed",
        "UkrainianGrayCattleBreed",
        "BraqueSaintGermainDogBreed",
        "SunandiniXZebuCattleBreed",
        "TarentaiseCowBreed",
        "NewHuaiPigBreed",
        "BritishShorthairedCat",
        "BlancaCeltibericaGoatBreed",
        "GratiCattleBreed",
        "GermanPinscherDogBreed",
        "NantaisCattleBreed",
        "CanadianCowBreed",
        "CuinoPigBreed",
        "AmericanTunisSheepBreed",
        "HokkaidoHorseBreed",
        "FellPonyHorseBreed",
        "MiKiDogBreed",
        "AnatolianShepherdDogBreed",
        "GrandGasconSaintongeoisDogBreed",
        "DutchKooikerDogBreed",
        "KoratCat",
        "PetitBleuDeGascogneDogBreed",
        "RedSindhiCattleBreed",
        "HungarofriesCattleBreed",
        "ToriHorseBreed",
        "FuriosoHorseBreed",
        "NepaleseHillZebuCattleBreed",
        "CyprusCattleBreed",
        "AmericanCurlCatBreed",
        "BarrosoCattleBreed",
        "TonkineseCat",
        "CharfordXZebuCattleBreed",
        "MuongKhuongPigBreed",
        "BelgianLandraceBNPigBreed",
        "KirdiPonyHorseBreed",
        "SomaliGoatBreed",
        "HogIslandSheepBreed",
        "SiberianBlackPiedPigBreed",
        "OldFormatMinToyPoodleDogBreed",
        "TyrolMountainSheepBreed",
        "MezzalinaCattleBreed",
        "GermanMountainSheepBreed",
        "GermanShorthornCattleBreed",
        "AinuDogBreed",
        "DashtiariCattleBreed",
        "MaduraWildJavanXZebuCattleBreed",
        "DutchTuigpaardHorseBreed",
        "WanzheSpottedChunanSpottedPigBreed",
        "MontafonCattleBreed",
        "FlatCoatedRetrieverDogBreed",
        "NorthernBlueCattleBreed",
        "LiberianDwarfCattleBreed",
        "MhaswadCattleBreed",
        "KyiApsoDogBreed",
        "SardaGoatBreed",
        "KuiMlkDogBreed",
        "VladimirHeavyDraftHorseBreed",
        "VestlandFjordCattleBreed",
        "DKPig30PigBreed",
        "LipetskPigBreed",
        "AostaChestnutCattleBreed",
        "ZhemaichuHorseBreed",
        "IberianBlackIberianPigBreed",
        "MagyarAgårDogBreed",
        "ShireHorseBreed",
        "RoyaVesubieGoatBreed",
        "AmericanCreamDraftHorseBreed",
        "NorfolkTerrier",
        "AustralianFriesianSahiwalXZebuCattleBreed",
        "AustralianStockHorseHorseBreed",
        "FHCElitePig7PigBreed",
        "ByelorussianSyntheticCattleBreed",
        "CanaryIslandCattleBreed",
        "NyoroCattleBreed",
        "SokolkaHorseBreed",
        "ShanghaiWhitePigBreed",
        "WanzheSpottedPigBreed",
        "KaninganCattleBreed",
        "TibetanMastiffDogBreed",
        "PuliDog",
        "StreiPigBreed",
        "SouthAnatolianRedCattleBreed",
        "LithuanianWhitePigBreed",
        "DatongHorseBreed",
        "EgyptianHorseBreed",
        "VolynskCattleBreed",
        "PolesianHorseBreed",
        "BriardDog",
        "MontanaraCattleBreed",
        "BuryatHorseBreed",
        "CamarronGoatBreed",
        "AmericanHairlessTerrierDogBreed",
        "StandardbredHorseBreed",
        "DanishLargeWhitePigBreed",
        "BaggaraCattleBreed",
        "NakaliCattleBreed",
        "PlevenHorseBreed",
        "SahfordXZebuCattleBreed",
        "GarganicaGoatBreed",
        "PoltavaPigBreed",
        "MurleCattleBreed",
        "CostenoHorseBreed",
        "CentralAsianZebuXZebuCattleBreed",
        "ManorMeishanPigBreed",
        "SpottedPigBreed",
        "SpanishGoatBreed",
        "DKPigBreed",
        "GallowayCowBreed",
        "DonPigBreed",
        "TurkishVanCatBreed",
        "KhamalaCattleBreed",
        "VosgesCattleBreed",
        "MeusseRhineIjsselCowBreed",
        "DanishOldenborgHorseBreed",
        "GermanRedPiedCattleBreed",
        "KedahKelantanCattleBreed",
        "FinnhorseHorseBreed",
        "DutchFriesianCattleBreed",
        "SouthAfricanMerinoSheepBreed",
        "BerkshirePigBreed",
        "BosnianHoundDogBreed",
        "RastreadorBrasileiroDogBreed",
        "MiniatureBullTerrierDogBreed",
        "AltaiHorseBreed",
        "DomesticLonghairedCat",
        "SkuddeSheepBreed",
        "MurcianPigBreed",
        "DamiettaCattleBreed",
        "NejdiCattleBreed",
        "AmericanBisonXCattleBreed",
        "SiberianPigBreed",
        "CanadianHampshirePigBreed",
        "BassetGriffonVendéenDogBreed",
        "BaXuyenPigBreed",
        "RockyMountainHorseHorseBreed",
        "PolledWelshBlackCattleBreed",
        "BeulahSpeckledFaceSheepBreed",
        "BelgianArdennaisHorseBreed",
        "GermanRidingPonyHorseBreed",
        "KoreanNativePigBreed",
        "YorkshireBlueAndWhitePigBreed",
        "AustralianKelpieDogBreed",
        "TuniCattleBreed",
        "KigerMustangHorseBreed",
        "LapermShorthairCatBreed",
        "EastFriesianWarmbloodModernTypeHorseBreed",
        "ValdresCattleBreed",
        "DrentsePatrijshondDogBreed",
        "WindsnyerPigBreed",
        "BoarPowerPig282PigBreed",
        "PiauSorocabaPigBreed",
        "IrishMoiledCattleBreed",
        "RioLimonDairyCriolloCattleBreed",
        "OxfordDownSheepBreed",
        "OldeEnglishBulldoggeDogBreed",
        "CanindéGoatBreed",
        "KarabakhHorseBreed",
        "AustrianYellowCattleBreed",
        "DairySyntheticCattleBreed",
        "TropicanaCattleBreed",
        "KomondorDog",
        "NiloCanastraPigBreed",
        "Keeshond",
        "GuernseyCowBreed",
        "CaiuaXZebuCattleBreed",
        "BuohaiHorseBreed",
        "DKPig63PigBreed",
        "HetaoLopEarPigBreed",
        "HolmongerCattleBreed",
        "RathCattleBreed",
        "BarzonaCattleBreed",
        "AmericanShetlandPonyHorseBreed",
        "SwissAlpineGoatBreed",
        "SokokeCatBreed",
        "ChineseKazakhHorseBreed",
        "JerdiXZebuCattleBreed",
        "LakenvelderCattleBreed",
        "IrishGoatBreed",
        "MirandesaCattleBreed",
        "DabieshanCattleBreed",
        "SwabianHallPigBreed",
        "PolledShorthornUSCattleBreed",
        "GalicianPigBreed",
        "GraueGehoernteHeidschnuckeSheepBreed",
        "SwedishWarmbloodHorseBreed",
        "OrkneySheepBreed",
        "UzunyaylaHorseBreed",
        "SlovakCuvakDogBreed",
        "GermanSpitzDogBreed",
        "PittIslandSheepBreed",
        "BeefSyntheticCattleBreed",
        "SiberianBlackPiedCattleBreed",
        "SalersCowBreed",
        "VoronezhCoachHorseHorseBreed",
        "EurasianDogBreed",
        "SwallowBeliedMangalitzaPigBreed",
        "KankrejCattleBreed",
        "HarbinWhitePigBreed",
        "AmericanYorkshirePigBreed",
        "LiangGuangSmallSpottedGuangdongSmallEarPigBreed",
        "SistaniCattleBreed",
        "LuciePigBreed",
        "Dalland030PigBreed",
        "GermanAngusCattleBreed",
        "BichonTeneriffeDogBreed",
        "YanqiHorseBreed",
        "ShagyaHorseBreed",
        "MadonieSicilianPigBreed",
        "BavarianForestSheepBreed",
        "NomaHorseBreed",
        "LeiqiongCattleBreed",
        "ColourpointShorthairedCat",
        "BlackMangalitsaPigBreed",
        "IstrianCattleBreed",
        "LuxiCattleBreed",
        "HinterwaldCattleBreed",
        "RussianBrownCattleBreed",
        "FHCElitePig2PigBreed",
        "AndalusianGrayCattleBreed",
        "CaliforniaSpangledCatBreed",
        "MinusinHorseBreed",
        "BSIPigBreed",
        "TriggHoundDogBreed",
        "JinhuaPigBreed",
        "BalearicHorseBreed",
        "IleDeFranceSheepBreed",
        "MandalongXZebuCattleBreed",
        "CarpatheGoatBreed",
        "DoranCattleBreed",
        "SpanishPointerDogBreed",
        "BakosiCattleBreed",
        "RedEngadineSheepBreed",
        "MorganHorseBreed",
        "BoXuPigBreed",
        "TurkishAngoraCat",
        "SouthdownSheepBreed",
        "AmritMahalZebuCattleBreed",
        "PolishDraftHorseBreed",
        "CeskyTerrierDogBreed",
        "KalmykHorseBreed",
        "JenubiCattleBreed",
        "RedKandhariCattleBreed",
        "YakutCattleBreed",
        "BelgianRedCattleBreed",
        "KarabairHorseBreed",
        "LincolnRedShorthornCattleBreed",
        "DoayoCattleBreed",
        "SardinianBrownCattleBreed",
        "AustralianPonyHorseBreed",
        "RagamuffinCatBreed",
        "WhiteGallowayCattleBreed",
        "AcehCattleBreed",
        "DexterCowBreed",
        "KachhiSheepBreed",
        "BaluchiHorseBreed",
        "GuzerandoXZebuCattleBreed",
        "LucanianPigBreed",
        "RideauArcottSheepBreed",
        "LonghairedMiniatureDachshundDogBreed",
        "AkitaDog",
        "BelgianRedPiedCattleBreed",
        "WhiteWelshCattleBreed",
        "TargheeSheepBreed",
        "XinjinXinjinPigBreed",
        "TroteEnGallopeHorseBreed",
        "SwissBlackPiedCattleBreed",
        "SulawesiHorseBreed",
        "MeixinPigBreed",
        "DianiPigBreed",
        "AngloFrancaisDeMoyenVenerieDogBreed",
        "LincolnSheepBreed",
        "AustralianGreyhoundDogBreed",
        "RomanianSaddleHorseHorseBreed",
        "EquusCaballusGmeliniHorseBreed",
        "MangalargaMarchadorHorseBreed",
        "BibrikSheepBreed",
        "CretanMountainCattleBreed",
        "BelgianGriffonSmoothDogBreed",
        "WensleydaleSheepBreed",
        "RedDesertCattleBreed",
        "PersianCat",
        "MassifCentralGoatBreed",
        "MurakozHorseBreed",
        "WestrainPigBreed",
        "NdaguXZebuCattleBreed",
        "HamplinePigBreed",
        "NanyangCattleBreed",
        "CharSwissCattleBreed",
        "KareloFinnishLaikaDogBreed",
        "NorwegianDunkerhoundDogBreed",
        "SilesianHorseBreed",
        "MurnauWerdenfelsCattleBreed",
        "MaureCattleBreed",
        "ErhulianPigBreed",
        "SemirechenskPigBreed",
        "FujianSmallPigBreed",
        "BejaCattleBreed",
        "ButanaCattleBreed",
        "ItalianHeavyDraftHorseBreed",
        "MinPigBreed",
        "LeeSungPigBreed",
        "WallisBlacknosedSheepBreed",
        "SaintBernardShortHairedDogBreed",
        "MalteseGoatBreed",
        "CastlemilkMooritSheepBreed",
        "FerrandaisCattleBreed",
        "ApulianPodolianCattleBreed",
        "SmallPolishPrickEaredPigBreed",
        "GorbatovRedCattleBreed",
        "DandieDinmontTerrier",
        "PortoAmboimCattleBreed",
        "DahePigBreed",
        "AmericanBlueGasconHoundDogBreed",
        "JapaneseBlackCattleBreed",
        "IranianZebuCattleBreed",
        "PolishRedCattleBreed",
        "OICPigBreed",
        "DamaraSheepBreed",
        "BorderCollieDogBreed",
        "KoninkTarpanHorseXDomesticHorseBreed",
        "BooroolaMerinoSheepBreed",
        "NorwegianGoatBreed",
        "MiyakoHorseBreed",
        "ArapawaIslandGoatBreed",
        "KaranFriesXZebuCattleBreed",
        "NewYorkRedPigBreed",
        "BündnerOberlandSheepBreed",
        "AlbanianHorseBreed",
        "BoarPowerPig161PigBreed",
        "GadoDaTerraCattleBreed",
        "PineywoodsPigBreed",
        "HessenHorseBreed",
        "PodengoCanarioDogBreed",
        "UzbekBlackGoatBreed",
        "Bulldog",
        "SabreXZebuCattleBreed",
        "SmoothCollieDogBreed",
        "ToposaCattleBreed",
        "GermanFleckViehCowBreed",
        "PrisheksninskPigBreed",
        "FHCElitePig6PigBreed",
        "SmoothHairedVizslaDogBreed",
        "PR1050PigBreed",
        "RomneyMarshSheepBreed",
        "CoburgerFuchsschafSheepBreed",
        "GoldenAmericanSaddlebredHorseBreed",
        "OulmesBlondCattleBreed",
        "DutchLandracePigBreed",
        "CarunchinhoPigBreed",
        "KonariCattleBreed",
        "SuboticaWhitePigBreed",
        "ArsiCattleBreed",
        "MurbodenCattleBreed",
        "KapsikiCattleBreed",
        "JagdterrierDogBreed",
        "DjermaHorseBreed",
        "Pudelpointer",
        "EriskayPonyHorseBreed",
        "BoarPowerPig414PigBreed",
        "LargeWhitePigBreed",
        "KeErQinHorseBreed",
        "MergellandSheepBreed",
        "SouthWalesMountainSheepBreed",
        "CapeHorseHorseBreed",
        "FreibergHorseBreed",
        "BoreraySheepBreed",
        "PunganurCattleBreed",
        "FrenchSpanielDogBreed",
        "PeruvianIncaOrchidDogBreed",
        "JielinHorseBreed",
        "PyreneanCattleBreed",
        "TribredPigBreed",
        "AmericanIndianDogBreed",
        "Greyhound",
        "CatahoulaLeopardDogBreed",
        "QuarterHorseBreed",
        "VenezuelanBlackPigBreed",
        "PalominoHorseBreed",
        "DorsetSheepSuperbreed",
        "SinclairMiniaturePigBreed",
        "EastAnatolianRedCattleBreed",
        "VichPigBreed",
        "MewatiCattleBreed",
        "AkhaPigBreed",
        "BarbadosSheepBreed",
        "ASCOBCockerSpanielDogBreed",
        "PicCambourghPigBreed",
        "KetekuXZebuCattleBreed",
        "BritishSaanenGoatBreed",
        "MixedBreedHorse",
        "Ocicat",
        "KerryCattleBreed",
        "CestrumParqui",
        "PharaohHound",
        "RamoGrandeCattleBreed",
        "IrishCobHorseBreed",
        "SmoothStandardDachshundDogBreed",
        "BraMaineXZebuCattleBreed",
        "BloodhoundSouthernHoundDogBreed",
        "LimousinCowBreed",
        "FinnishLandraceGoatBreed",
        "MarinhoaCattleBreed",
        "NorwegianElkhound",
        "BritishSaddlebackPigBreed",
        "DKPig31PigBreed",
        "BoarPowerPig292PigBreed",
        "AmericanWaterSpanielDogBreed",
        "YaroslavlCattleBreed",
        "GaolaoCattleBreed",
        "JinhuaDongyangPigBreed",
        "GarfagninaCattleBreed",
        "MBayarHorseBreed",
        "AustralianCattleDog",
        "JianchangHorseBreed",
        "TamworthPigBreed",
        "KookaSheepBreed",
        "ChernigovCattleBreed",
        "VillardDeLansCattleBreed",
        "FrenchBrownCattleBreed",
        "BarbetDogBreed",
        "GuadianaSpottedCattleBreed",
        "BengaliCattleBreed",
        "NDamaCattleBreed",
        "MarianasCattleBreed",
        "RangerXZebuCattleBreed",
        "NorfolkHornSheepBreed",
        "CatalanSheepdogBreed",
        "BaouleCattleBreed",
        "RomanianNativeBaltaretPigBreed",
        "BarrosaCattleBreed",
        "BerneseHoundDogBreed",
        "FHCElitePig8PigBreed",
        "EstrelaMountainDogBreed",
        "NovaScotiaDuckTollingRetrieverDogBreed",
        "CaptainCookerPigBreed",
        "CurraleiroCattleBreed",
        "MinhotaCattleBreed",
        "NooitgedachtHorseBreed",
        "QuarabHorseBreed",
        "SunkumaCattleBreed",
        "AmericanEssexPigBreed",
        "IrishWaterSpanielDogBreed",
        "GuoxiaHorseBreed",
        "MoravianLargeYorkshirePigBreed",
        "SpanishAngloArabHorseBreed",
        "HanzhongWhitePigBreed",
        "AraappaloosaHorseBreed",
        "SilkyTerrier",
        "ChianinaPigBreed",
        "KilisCattleBreed",
        "HezuoPigBreed",
        "HighlandPonyHorseBreed",
        "BelgianGroenendaelDog",
        "KleenLeenBlackPigBreed",
        "AwassiSheepBreed",
        "AmericanMerinoSheepBreed",
        "RussianBlackPiedCattleBreed",
        "UrzhumPigBreed",
        "SchapendoesDogBreed",
        "PanjeHorseBreed",
        "JapaneseRetrieverDogBreed",
        "MoxotóGoatBreed",
        "GermanShepherdDog",
        "StandardPoodleDogBreed",
        "BahimaCattleBreed",
        "MessaraHorseBreed",
        "NavajoSheepBreed",
        "BeijingBlackPigBreed",
        "AfricanaSheepBreed",
        "NewKirgizHorseBreed",
        "SavinjaGrayCattleBreed",
        "JellicutCattleBreed",
        "TadzhikZebuCattleBreed",
        "CarthusianHorseBreed",
        "ShetlandSheepdog",
        "JamaicaBlackXZebuCattleBreed",
        "FjordHorseBreed",
        "OtterHound",
        "CheurfaCattleBreed",
        "NormandieCowBreed",
        "SiskaPigBreed",
        "LobiCattleBreed",
        "TaiwanYellowCattleBreed",
        "KharkovPigBreed",
        "GuangxiHorseBreed",
        "ChineseBlackAndWhiteCattleBreed",
        "ShorthornCowBreed",
        "BlackSlavonianPigBreed",
        "AltaiMountainGoatBreed",
        "ComtoisHorseBreed",
        "TambovRedCattleBreed",
        "BlackCockerSpanielDogBreed",
        "RambouilletSheepBreed",
        "DomesticShorthairedCat",
        "MinzhuPigBreed",
        "HerdwickSheepBreed",
        "StyrianMountainDogBreed",
        "SingleFootingHorseHorseBreed",
        "SwedishRedPolledCattleBreed",
        "ChamDocCattleBreed",
        "XinjinNingAnPigBreed",
        "BraqueDAuvergneDogBreed",
        "LombokHorseBreed",
        "PhilippineGoatBreed",
        "SchleswigHorseBreed",
        "FellataCattleBreed",
        "CãoDeFilaMiguelDogBreed",
        "DutchWarmbloodHorseBreed",
        "LangheSheepBreed",
        "IcelandicSheepdogBreed",
        "CanadianRusticPonyHorseBreed",
        "SantaInêsSheepBreed",
        "PeruvianPasoHorseBreed",
        "KachchaSiriCattleBreed",
        "Cotswold29PigBreed",
        "HainanLingaoPigBreed",
        "NorthBangladeshGrayCattleBreed",
        "EnglishGoatBreed",
        "BirmanCat",
        "FightingBullCattleBreed",
        "HartónCattleBreed",
        "GermanKoolieDogBreed",
        "ManxameseCatBreed",
        "MertolengaCattleBreed",
        "TelomainDogBreed",
        "XiangxiBlackPigBreed",
        "ChinookDogBreed",
        "BritishMilkSheepBreed",
        "AfricangnusCattleBreed",
        "CalvanaCattleBreed",
        "BritishLandracePigBreed",
        "WhitefaceWoodlandSheepBreed",
        "CanastrãoCabanoPigBreed",
        "ToubouXZebuCattleBreed",
        "MinnesotaPigBreed",
        "KenanaCattleBreed",
        "PercheronHorseBreed",
        "DevonCattleBreed",
        "FuzhouBlackPigBreed",
        "OrlovTrotterHorseBreed",
        "AlbanianIllyrianCattleBreed",
        "BosnianHorseBreed",
        "TaiwanSmallRedPigBreed",
        "BashiCattleBreed",
        "AmericanTunisHorseBreed",
        "SlovenianBrownCattleBreed",
        "SouthAfricanMuttonMerinoSheepBreed",
        "CildirCattleBreed",
        "MaremmanaHorseBreed",
        "BhirumPonyHorseBreed",
        "NkoneCattleBreed",
        "ShetlandSheepBreed",
        "MiniatureAustralianShepherdDogBreed",
        "FriesianHorseBreed",
        "DnieperPigBreed",
        "LebaneseCattleBreed",
        "HaitiCreolePigBreed",
        "PolishSimmentalCattleBreed",
        "BanyoCattleBreed",
        "BraqueFrancaisDePetiteTailleDogBreed",
        "HuaiPigBreed",
        "TuliCattleBreed",
        "SopravissanaSheepBreed",
        "StrongWoolMerinoSheepBreed",
        "MacedonianPigBreed",
        "PaphosCattleBreed",
        "TamankaduwaCattleBreed",
        "BaliHorseBreed",
        "PoitouGoatBreed",
        "WestAfricanBarbHorseBreed",
        "DanishRedCattleBreed",
        "LeineschafSheepBreed",
        "SimbrahCattleBreed",
        "SealyhamTerrier",
        "GirCowBreed",
        "PerendaleSheepBreed",
        "AltaySheepBreed",
        "NoTailSheepBreed",
        "Line500DurocPigBreed",
        "GermanPasturePigBreed",
        "MoghaniSheepBreed",
        "MinPeiDogBreed",
        "LepingPigBreed",
        "RojhanCattleBreed",
        "SandanHorseBreed",
        "ManorHybridPigBreed",
        "GermanCornwallPigBreed",
        "ToyHavaneseTerrierDogBreed",
        "AbyssinianCat",
        "KisoHorseBreed",
        "BraqueDupuyDogBreed",
        "AmericanCoonhoundDogBreed",
        "SharPeiDogBreed",
        "BritishWarmbloodHorseBreed",
        "MaineAnjouCowBreed",
        "CreoleAntillesGoatBreed",
        "GalwaySheepBreed",
        "DorperSheepBreed",
        "SloughiDogBreed",
        "PolishLandracePigBreed",
        "CarpathianBrownCattleBreed",
        "HamrocPigBreed",
        "SpinoneItalianoDogBreed",
        "DutchShepherdDogBreed",
        "GrandBleuDeGascogneDogBreed",
        "DroughtmasterXZebuCattleBreed",
        "FHCPigBreed",
        "RedSteppeCattleBreed",
        "ChienFrançaisTricoloreDogBreed",
        "SteigarSheepBreed",
        "CorsicanPigBreed",
        "ToyFoxTerrierDogBreed",
        "PieRougeDeLEstCattleBreed",
        "SuisbuXZebuCattleBreed",
        "HondurasSwitchTailPigBreed",
        "SonkheriCattleBreed",
        "EastBalkanPigBreed",
        "KochiCattleBreed",
        "CanastraPigBreed",
        "SanfratelloHorseBreed",
        "MeoPigBreed",
        "OldSwedishSpottedPigBreed",
        "KoronadalPigBreed",
        "BetizuakCattleBreed",
        "AmericanBobtailLonghairCatBreed",
        "TibetanCattleBreed",
        "WelshMountainPonyHorseBreed",
        "BambaraXZebuCattleBreed",
        "TexelSheepBreed",
        "TokaraHorseBreed",
        "FinnishHoundDogBreed",
        "TibetanPigBreed",
        "AlentejanaPigBreed",
        "RedWattlePigBreed",
        "CardiganWelshCorgiDogBreed",
        "KhillariCattleBreed",
        "ImprovedRodopiCattleBreed",
        "ArmoricanCattleBreed",
        "MurgheseHorseBreed",
        "AubracCattleBreed",
        "OhminiPigBreed",
        "DajalCattleBreed",
        "BraviaGoatBreed",
        "BlackheadPersianSheepBreed",
        "HavaneseDogBreed",
        "GoldenRetrieverDogBreed",
        "RexCatBreed",
        "CheviotSheepBreed",
        "MetohijaRedCattleBreed",
        "BalwenWelshMountainSheepBreed",
        "PortugueseGuardDogBreed",
        "AlpineGoatBreed",
        "SpanishBarbHorseBreed",
        "BusaCattleBreed",
        "AnkoleCattleBreed",
        "CosentinaPigBreed",
        "BulgarianRedCattleBreed",
        "BatanesBlackXZebuCattleBreed",
        "BoarPowerPig84PigBreed",
        "CharbrayXZebuCattleBreed",
        "ThalliSheepBreed",
        "SengologaCattleBreed",
        "TswanaCattleBreed",
        "HuazhongTwoEndBlackJianliPigBreed",
        "GreatDaneDog",
        "MouflonSheepBreed",
        "WestAfricanDwarfShorthornCattleBreed",
        "MontdaleSheepBreed",
        "LargeBlackWhitePigBreed",
        "BashkirHorseBreed",
        "ElliottdaleSheepBreed",
        "NebelungCatBreed",
        "LundehundDogBreed",
        "SienaBeltedPigBreed",
        "KrolevetsPigBreed",
        "BouvierDesFlandres",
        "AustrianSmoothHairedBrackeDogBreed",
        "BlancoOrejinegroCattleBreed",
        "HanfordMiniaturePigBreed",
        "BulgarianLandracePigBreed",
        "AzoresHorseBreed",
        "TibetanHorseBreed",
        "HerelandCattleBreed",
        "XinjiangBrownCattleBreed",
        "ColoradoRangerHorseBreed",
        "AmericanIndianHorseHorseBreed",
        "ManchesterTerrierSuperbreedDogBreed",
        "ExmoorHornSheepBreed",
        "AustralianShepherdDogBreed",
        "NanjingBlackPigBreed",
        "JinnanCattleBreed",
        "NortheasternHorseBreed",
        "PortuguesePointerDogBreed",
        "OutaouaisArcottSheepBreed",
        "TavdaHorseBreed",
        "BoarPowerPig48PigBreed",
        "ChumyshHorseBreed",
        "HellenicHoundDogBreed",
        "DjakoreXZebuCattleBreed",
        "KinderGoatBreed",
        "AustralianShorthornCattleBreed",
        "LadakhiCattleBreed",
        "RagdollCatBreed",
        "AnkinaCattleBreed",
        "GidranHorseBreed",
        "HackneyHorseBreed",
        "AriégeoisDogBreed",
        "RathiCattleBreed",
        "WireFoxTerrierDogBreed",
        "BazadaisCattleBreed",
        "YimengBlackPigBreed",
        "BatardDogBreed",
        "CzechMiniaturePigBreed",
        "BlackGrisonneGoatBreed",
        "MingrelianRedCattleBreed",
        "GreatMünsterländerDogBreed",
        "PrianganSheepBreed",
        "GlanCattleBreed",
        "ShortCoatChihuahuaDogBreed",
        "BeagleSmoothDogBreed",
        "BeltedGallowayCowBreed",
        "ChaouiaCattleBreed",
        "SardinianAngloArabHorseBreed",
        "SuffolkSheepBreed",
        "DartmoorSheepBreed",
        "CharollandaisCattleBreed",
        "AlanoEspañolDogBreed",
        "AmericanWhiteParkCattleBreed",
        "Weimaraner",
        "DanishDurocPigBreed",
        "TrakehnerHorseBreed",
        "SheepBreedUndeterminedSheepBreed",
        "BeefmakerCattleBreed",
        "GromarkSheepBreed",
        "DunkerDogBreed",
        "LhasaApso",
        "NormanCoachHorseBreed",
        "PelibüeySheepBreed",
        "IllawarraCattleBreed",
        "BavendaCattleBreed",
        "BelgianGriffonDogBreed",
        "KolbroekPigBreed",
        "RodopiCattleBreed",
        "ManorRangerPigBreed",
        "SouthernUkranianCattleBreed",
        "YorkshirePigBreed",
        "FHCElitePig4PigBreed",
        "MastiffDog",
        "SlovenianWhitePigBreed",
        "JavaneseCat",
        "FellTerrierDogBreed",
        "ChiangusCattleBreed",
        "MiddleWhitePigBreed",
        "BeagleHarrierDogBreed",
        "KenkathaCattleBreed",
        "CorriedaleSheepBreed",
        "ByelorussianRedCattleBreed",
        "LimousinPigBreed",
        "ArgentineDogoDogBreed",
        "Bullmastiff",
        "PolledHerefordCowBreed",
        "AmericanWirehairedCat",
        "CarinthianBlondCattleBreed",
        "EnglishSpringerSpanielDogBreed",
        "WujinPigBreed",
        "SpælsauSheepBreed",
        "PygoraGoatBreed",
        "TuvaHorseBreed",
        "UdaSheepBreed",
        "SiboneyXZebuCattleBreed",
        "MiddleAsianOvtcharkaDogBreed",
        "LatvianBlueRoanCattleBreed",
        "KakhetianPigBreed",
        "SerpentinaGoatBreed",
        "GermanRexCatBreed",
        "HaflingerHorseBreed",
        "BentheimBlackPiedPigBreed",
        "NorthAmericanShepherdDogBreed",
        "MontbeliardCattleBreed",
        "ChiloteHorseBreed",
        "SouthAfricanLandracePigBreed",
        "MilkingDevonCattleBreed",
        "PonyOfTheAmericasHorseBreed",
        "HanSheepBreed",
        "LandracePigBreed",
        "GreekLocalPigBreed",
        "MeatlinePigBreed",
        "NationalShowHorseHorseBreed",
        "AostaBlackPiedCattleBreed",
        "VietnameseYellowCattleBreed",
        "NorwegianLundehundDogBreed",
        "LonglinPigBreed",
        "BasutoPonyHorseBreed",
        "BraqueDuBourbonnaisDogBreed",
        "MilkingShorthornCowBreed",
        "AustralianTerrierSilkyDogBreed",
        "WelshPonyHorseBreed",
        "DobermanPinscher",
        "AustralianTerrierRoughCoatedDogBreed",
        "RhinelandHeavyDraftHorseBreed",
        "NGabouCattleBreed",
        "OvamboCattleBreed",
        "ThuocNhieuPigBreed",
        "RomagnolaPigBreed",
        "INRA95CattleBreed",
        "LongCoatChihuahuaDogBreed",
        "RedAngusCattleBreed",
        "KleenLeenPigBreed",
        "HungairanColdbloodHorseBreed",
        "BoarPowerPig454PigBreed",
        "ModicanaCattleBreed",
        "AmericanQuarterHorseHorseBreed",
        "Manx",
        "VendéenSheepBreed",
        "FumatiPigBreed",
        "OldFormatManchesterTerrierDogBreed",
        "LonghairedStandardDachshundDogBreed",
        "AmericanDraftPonyHorseBreed",
        "VlaamperdHorseBreed",
        "IraqiXZebuCattleBreed",
        "CarazebuXZebuCattleBreed",
        "MongCaiPigBreed",
        "PetitBassetGriffonVendeenDog",
        "NooitgedachtPonyHorseBreed",
        "PolledSimmentalCattleBreed",
        "BeefShorthornCattleBreed",
        "SnowshoeCatBreed",
        "GermanShorthairedPointerDogBreed",
        "RedFlemishCattleBreed",
        "RoveGoatBreed",
        "GujamavuCattleBreed",
        "SovereignPigBreed",
        "NorthEastChinaSpottedPigBreed",
        "PiauPigBreed",
        "KazakhHorseBreed",
        "KazakhWhiteheadCattleBreed",
        "BushuevXZebuCattleBreed",
        "HashtNagriSheepBreed",
        "KamanPigBreed",
        "JapaneseShorthornCattleBreed",
        "BritishSpottedPonyHorseBreed",
        "TudancaCattleBreed",
        "FHCElitePig5PigBreed",
        "LandaisHorseBreed",
        "BonsmaraCattleBreed",
        "AkhalTekeHorseBreed",
        "ColumbiaSheepBreed",
        "BKB1PigBreed",
        "CanaanDogBreed",
        "RedPolledØstlandCattleBreed",
        "MixedPerijaneroXZebuCattleBreed",
        "HarrierDog",
        "GermanBlackPiedDairyCattleBreed",
        "FinnishLandraceSheepBreed",
        "ArapawaIslandPigBreed",
        "AlmanzorenaCattleBreed",
        "ChanniCattleBreed",
        "WaiChowPigBreed",
        "EnglishFoxhound",
        "PantaneiroCattleBreed",
        "JezerskosolcavskaSheepBreed",
        "AchhamCattleBreed",
        "GirgentanaGoatBreed",
        "SRDGoatBreed",
        "McnabDogBreed",
        "DairyShorthornCattleBreed",
        "MpwapwaCattleBreed",
        "BlackIberianCattleBreed",
        "GhanaShorthornCattleBreed",
        "YunnanHorseBreed",
        "StaffordshireBullTerrierDogBreed",
        "CriolloHorseBreed",
        "SteppeMeatPigBreed",
        "IrishHunterHorseBreed",
        "KameroenSheepBreed",
        "SinhalaCattleBreed",
        "ParthenaisCattleBreed",
        "FrenchSaddlePonyHorseBreed",
        "HackneyPonyHorseBreed",
        "MongolianHorseBreed",
        "DamaniSheepBreed",
        "BologneseDogBreed",
        "TungchangPigBreed",
        "GarranoTarpanHorseXDomesticHorseBreed",
        "DalesbredSheepBreed",
        "GurktalPigBreed",
        "JapaneseBrownCattleBreed",
        "DKPig33PigBreed",
        "FengjingPigBreed",
        "YemeniZebuCattleBreed",
        "JianliPigBreed",
        "PontremoleseCattleBreed",
        "WesternSudanPonyHorseBreed",
        "ScottishFoldCat",
        "KurganCattleBreed",
        "SussexSpanielDogBreed",
        "BelgianBlackPiedHolsteinCattleBreed",
        "AksaîBlackPiedPigBreed",
        "GirolandoXZebuCattleBreed",
        "PodengoPequenoDogBreed",
        "BraqueFrançaisDeGrandTailleDogBreed",
        "TurkishGraySteppeCattleBreed",
        "BrazilianPolledCattleBreed",
        "CanaryIslandGoatBreed",
        "ArgentinePoloPonyHorseBreed",
        "LoenbergerDogBreed",
        "EstonianBlackPiedCattleBreed",
        "CubanTrotterHorseBreed",
        "MunichMiniaturePigBreed",
        "ChittagongRedCattleBreed",
        "SocotraCattleBreed",
        "DelibozHorseBreed",
        "SalornCattleBreed",
        "CharollaisSheepBreed",
        "NingxiangPigBreed",
        "DanishLandraceSheepBreed",
        "CeskyFousekDogBreed",
        "FloresHorseBreed",
        "PetisoArgentinoHorseBreed",
        "ProvençaleGoatBreed",
        "KathiawariHorseBreed",
        "AmericanPonyHorseBreed",
        "PoitevinDogBreed",
        "PinziritaSheepBreed",
        "KateriniCattleBreed",
        "KinniyaCattleBreed",
        "SiniHorseBreed",
        "PalousePigBreed",
        "CanastrãoZabumbaPigBreed",
        "YuedongBlackPigBreed",
        "TyroleanHoundDogBreed",
        "NguniCattleBreed",
        "MantiqueiraXZebuCattleBreed",
        "Retriever",
        "RussianShortEaredWhitePigBreed",
        "BrandroodLjsselveeCattleBreed",
        "GackoCattleBreed",
        "ChihuahuaSuperbreedDogBreed",
        "BizanianHoundDogBreed",
        "JiaxingBlackPigBreed",
        "MacedonianBusaCattleBreed",
        "GrisonsStripedGoatBreed",
        "AngelnCattleBreed",
        "AtpadiMahalCattleBreed",
        "MasaiCowBreed",
        "IskarCattleBreed",
        "SmallMünsterländerDogBreed",
        "MenufiCattleBreed",
        "BavarianWarmbloodHorseBreed",
        "CampolinaHorseBreed",
        "GreekShorthornCattleBreed",
        "VaynolCattleBreed",
        "HamiltonstövareDogBreed",
        "EnglishLonghornCattleBreed",
        "WestAfricanPigBreed",
        "DexterKerryCattleBreed",
        "NeloreCattleBreed",
        "DanishWarmbloodHorseBreed",
        "PernaCurtaPigBreed",
        "MalnadGiddaCattleBreed",
        "InkukuCattleBreed",
        "IrishRedAndWhiteSetterDogBreed",
        "ColNoirDuValaisGoatBreed",
        "Coonhound",
        "SuperfineMerinoSheepBreed",
        "JackRusselTerrier",
        "PolledGuzeratCattleBreed",
        "Camborough12PigBreed",
        "AmericanPitBullTerrierDogBreed",
        "AlaskanKleeKaiDogBreed",
        "BraqueBelgeDogBreed",
        "SmoothMiniatureDachshundDogBreed",
        "Setter",
        "MalopolskiHorseBreed",
        "AberdeenAngusCowBreed",
        "FinnishYorkshirePigBreed",
        "WokalupXZebuCattleBreed",
        "MexicanHairlessDog",
        "DreverDog",
        "AlterRealHorseBreed",
        "HuangHuaiHaiBlackPigBreed",
        "HanoverianHorseBreed",
        "SelleFrancaisHorseBreed",
        "LibyanCattleBreed",
        "IsraeliRedCattleBreed",
        "RustaqiXZebuCattleBreed",
        "TexonCattleBreed",
        "SwedishArdennesHorseBreed",
        "BarbadosBlackbellySheepBreed",
        "CaucasianShepherdDogBreed",
        "AmericanShorthairCat",
        "PolishHoundDogBreed",
        "SychevkaCattleBreed",
        "BovskaSheepBreed",
        "DelaineMerinoSheepBreed",
        "PugDog",
        "ZambiaAngoniCattleBreed",
        "RedBororoCattleBreed",
        "IberianJabugoSpottedPigBreed",
        "LanYuPigBreed",
        "MalagueñaGoatBreed",
        "DraftPonySuperbreedHorseBreed",
        "RussoLaikaDogBreed",
        "GayalCattleBreed",
        "SchoonebekerSheepBreed",
        "WestSiberianLaikaDogBreed",
        "AvilenaBlackIberianCattleBreed",
        "RasaAragonesaSheepBreed",
        "TennesseeWalkingHorseBreed",
        "SwissMountainDogBreed",
        "CretanLowlandCattleBreed",
        "KustanaiHorseBreed",
        "BagotGoatBreed",
        "FHCElitePig9PigBreed",
        "CukurovaHorseBreed",
        "SuffolkHorseBreed",
        "GrandGriffonVendeenDogBreed",
        "SenepolCattleBreed",
        "StBernardDog",
        "FriesianMilkSheepBreed",
        "KingShepherdDogBreed",
        "SwissImprovedLandracePigBreed",
        "LaconiePigBreed",
        "TexasLonghornCattleBreed",
        "PortugueseWaterDog",
        "TagilCattleBreed",
        "BanatWhitePigBreed",
        "SardinianPigBreed",
        "CotonDeTuléarDogBreed",
        "NagoriCattleBreed",
        "DnieperCattleBreed",
        "CostaRicanSaddleHorseHorseBreed",
        "ChileanCorraleroHorseBreed",
        "YunnanZebuCattleBreed",
        "BlackHairlessPigBreed",
        "WelshCorgiSuperbreedDogBreed",
        "WitrikCattleBreed",
        "KromfohrländerDogBreed",
        "ÍPigBreed",
        "BlackMouthCurDogBreed",
        "BraSwissXZebuCattleBreed",
        "BankerHorseHorseBreed",
        "SmithfieldDogBreed",
        "GorynCattleBreed",
        "BhujGoatBreed",
        "FjallCattleBreed",
        "ThibarXZebuCattleBreed",
        "RussianSaddleHorseHorseBreed",
        "PosavinaCattleBreed",
        "MaremmanaCattleBreed",
        "HebrideanSheepBreed",
        "BisonBisonXSimmentalHybrid",
        "BoarPowerPig27PigBreed",
        "TahltanBearDog",
        "MontmorillonPigBreed",
        "TuyHoaHairlessPigBreed",
        "LivnyPigBreed",
        "LlaneroHorseBreed",
        "TsivilskPigBreed",
        "DongolaHorseBreed",
        "VogelsbergCattleBreed",
        "NDamaSangaCattleBreed",
        "BravonXZebuCattleBreed",
        "RanaXZebuCattleBreed",
        "SpanishColonialHorseHorseBreed",
        "NorthCountyCheviotSheepBreed",
        "LijiangHorseBreed",
        "MouraPigBreed",
        "TuxZillertalCattleBreed",
        "MerensHorseBreed",
        "PR1075PigBreed",
        "WirehairedMiniatureDachshundDogBreed",
        "NorthFujianBlackAndWhitePigBreed",
        "ErzgebirgGoatBreed",
        "BrazilianSportHorseHorseBreed",
        "KurdiCattleBreed",
        "SableIslandHorseHorseBreed",
        "EpagneulPicardDogBreed",
        "JamaicaBrahmanCattleBreed",
        "AngloKabardaHorseBreed",
        "WelshTerrier",
        "WelshMountainBadgerFacedSheepBreed",
        "DutchYorkshirePigBreed",
        "JavaneseZebuCattleBreed",
        "UkrainianSteppeWhitePigBreed",
        "SouthChinaPigBreed",
        "AfricanderCattleBreed",
        "SouthernBlackmouthCurDogBreed",
        "BrillenschafSheepBreed",
        "BamaxiangPigBreed",
        "HallikarCattleBreed",
        "ChineseImperialChInDogBreed",
        "LusitanoHorseBreed",
        "BralersXZebuCattleBreed",
        "HungarianWhitePigBreed",
        "HeidschnuckeSheepBreed",
        "SovietHeavyDraftHorseBreed",
        "PeeWeeCattleBreed",
        "PyreneanGoatBreed",
        "RomosinuanoCattleBreed",
        "SahelTypeSheepBreed",
        "DortyolCattleBreed",
        "AmericanCreamHorseBreed",
        "GreaterSwissMountainHoundDogBreed",
        "ManxLoaghtanSheepBreed",
        "HovawartDogBreed",
        "TurkmenZebuCattleBreed",
        "KabotaCattleBreed",
        "SaddlebackPigBreed",
        "LargeSpanishHoundDogBreed",
        "BestuzhevCattleBreed",
        "SterlingPigBreed",
        "TyrolGrayCattleBreed",
        "VitaVetLabMinipigPigBreed",
        "LourdaisCattleBreed",
        "Foxhound",
        "SenneHorseBreed",
        "CanadianEskimoDogBreed",
        "Wolf",
        "HormelMiniaturePigBreed",
        "MixedBreedGoat",
        "RedFriesianCattleBreed",
        "HarlequinCatBreed",
        "ZavotCattleBreed",
        "BündnerGoatBreed",
        "KappiliyanCattleBreed",
        "CPFPigBreed",
        "DingPigBreed",
        "MaronesaCattleBreed",
        "DKPig51PigBreed",
        "BlackWelshCowBreed",
        "ArapawaIslandSheepBreed",
        "WhippetDog",
        "WhiteParkCowBreed",
        "ArabiSheepBreed",
        "GreenlandDogBreed",
        "AngloNubianGoatBreed",
        "FogeraCattleBreed",
        "GroningenHorseBreed",
        "LincolnLongwoolSheepBreed",
        "RussianTrotterHorseBreed",
        "AgeroleseCattleBreed",
        "WhitefaceDartmoorSheepBreed",
        "OssabawIslandPigBreed",
        "CavalierKingCharlesSpaniel",
        "BelmontAdaptaurCattleBreed",
        "MountainCurDogBreed",
        "BlackAndTanCoonhoundDogBreed",
        "LacombePigBreed",
        "AustrianPinscherDogBreed",
        "OrehlaDeColherPigBreed",
        "MadaripurCattleBreed",
        "KostromaCattleBreed",
        "SchwarzwalderFuchseHorseBreed",
        "HuertanaCattleBreed",
        "RoughFellSheepBreed",
        "NkediCattleBreed",
        "XilingolHorseBreed",
        "MorochucoHorseBreed",
        "TukidaleSheepBreed",
        "BariCattleBreed",
        "Cotswold16PigBreed",
        "CatalanaChickenBreed",
        "PerdigueroNavarroDogBreed",
        "CukurovaCattleBreed",
        "HyperLargeWhitePigBreed",
        "MixedBreedSheep",
        "NeijiangPigBreed",
        "BergerDePicardDogBreed",
        "StandardManchesterTerrier",
        "SayaguesaCattleBreed",
        "BeardedCollieDogBreed",
        "BlondeDuCapBonCattleBreed",
        "CzechImprovedWhitePigBreed",
        "ChesiCattleBreed",
        "SiameseCat",
        "DonHorseBreed",
        "ArdennesHorseBreed",
        "CaliforniaRedSheepBreed",
        "HornDorsetSheepBreed",
        "CorrienteCattleBreed",
        "HuculHorseBreed",
        "UjumqinSheepBreed",
        "SussexCattleBreed",
        "YurinoCattleBreed",
        "BargurCattleBreed",
        "KaramajongCattleBreed",
        "BasqueBlackPiedPigBreed",
        "BergerDuLanguedocDogBreed",
        "DevonClosewoolSheepBreed",
        "BorghigianaPigBreed",
        "MinorcanCattleBreed",
        "TaiwanZebuCattleBreed",
        "ColombianCriolloHorseBreed",
        "PremierPigBreed",
        "LongyouBlackPigBreed",
        "JinhuaYongkangPigBreed",
        "PulMBorCattleBreed",
        "OrientalShorthairedCat",
        "ThaiCattleBreed",
        "WhiteBullTerrierDogBreed",
        "SandalwoodHorseBreed",
        "GujaratiCowBreed",
        "VikingHorseBreed",
        "GöttingenMiniaturePigBreed",
        "TiaMeslanPigBreed",
        "EurasierDogBreed",
        "BrazilianShorthairCatBreed",
        "MyotonicGoatBreed",
        "BalticBlackPiedCattleBreed",
        "RoughCollieDogBreed",
        "GuelmaCattleBreed",
        "OldenburgHorseBreed",
        "NorthernArdennaisHorseBreed",
        "GermanMuttonMerinoSheepBreed",
        "BritishToggenburgGoatBreed",
        "TrakyaHorseBreed",
        "ClawnPigBreed",
        "CanchimXZebuCattleBreed",
        "WhiteCáceresCattleBreed",
        "SumadijaPigBreed",
        "KisberFelverHorseBreed",
        "HighConformationWhitePigBreed",
        "KumamotoCattleBreed",
        "AndalusianHorseBreed",
        "GrandBassettGriffonVendeenDogBreed",
        "ThessalyCattleBreed",
        "HalhinGolCattleBreed",
        "GotlandHorseBreed",
        "BrownWelshCowBreed",
        "EastSiberianLaikaDogBreed",
        "SyntheticSL98PigBreed",
        "LoashanGoatBreed",
        "CubanZebuXZebuCattleBreed",
        "KaiDogBreed",
        "MytileneHorseBreed",
        "BraunviehCattleBreed",
        "LeicesterSheepBreed",
        "WelshCobHorseBreed",
        "TongSheepBreed",
        "TarinaCattleBreed",
        "HolsteinFriesianCowBreed",
        "PatterdaleTerrierDogBreed",
        "AmericanBobtailShorthairCatBreed",
        "ToyManchesterTerrier",
        "KladrubyHorseBreed",
        "DagestanMountainCattleBreed",
        "PetitGriffonBleuDeGascogneDogBreed",
        "TongConPigBreed",
        "MangalistaPigBreed",
        "SahiwalCattleBreed",
        "GelbrayXZebuCattleBreed",
        "WaziriSheepBreed",
        "PietrainPigBreed",
        "ThaiPonyHorseBreed",
        "AzawakhDogBreed",
        "KumauniCattleBreed",
        "ManagraPigBreed",
        "PresaCanarioDogBreed",
        "TreeingWalkerCoonhoundDogBreed",
        "LebedinCattleBreed",
        "MengguCattleBreed",
        "Dalland080PigBreed",
        "HollandShepherdDogBreed",
        "ChinampoCattleBreed",
        "TabapuaCattleBreed",
        "FrederiksborgHorseBreed",
        "ShanxiBlackPigBreed",
        "EastBulgarianHorseBreed",
        "TinosCattleBreed",
        "RepartidaGoatBreed",
        "AustrianSimmentalCattleBreed",
        "OssimiSheepBreed",
        "MunchkinShorthairCatBreed",
        "NitraHybridPigBreed",
        "ShetlandCattleBreed",
        "CupremHybridCattleBreed",
        "MulefootPigBreed",
        "ApulianPigBreed",
        "BoarPowerPig656PigBreed",
        "CanadianCuttingHorseHorseBreed",
        "BeauceronDogBreed",
        "KrishnaValleyCattleBreed",
        "FinnishLapphundDogBreed",
        "GreatPyreneeDog",
        "EnglishGuernseyGoatBreed",
        "DanubeWhitePigBreed",
        "DrysdaleSheepBreed",
        "SarnyPigBreed",
        "FrenchBulldog",
        "ThessalonianHorseBreed",
        "ChumbivilcasHorseBreed",
        "SchipperkeDog",
        "OldFarmCollieDogBreed",
        "DalesPonyHorseBreed",
        "VictorianBulldoggeDogBreed",
        "CaracuCattleBreed",
        "ForestMountainPigBreed",
        "PolledJerseyCattleBreed",
        "DishtyXZebuCattleBreed",
        "PortlandSheepBreed",
        "RandallLinebackCattleBreed",
        "DzumaliaPigBreed",
        "InternationalStripedHorseHorseBreed",
        "EstonianNativeHorseBreed",
        "SomaliCattleBreed",
        "HubeiWhitePigBreed",
        "BeetalGoatBreed",
        "SanheCattleBreed",
        "NorthSiberianPigBreed",
        "AfghanCattleBreed",
        "ComisanaSheepBreed",
        "CarolinaDogBreed",
        "PeneiaHorseBreed",
        "SerereCattleBreed",
        "AzerbaijanHorseBreed",
        "WestFinnishCattleBreed",
        "WielkopolskiHorseBreed",
        "WanzheSpottedWannanSpottedPigBreed",
        "BarbHorseBreed",
        "BambeyXZebuCattleBreed",
        "NorwegianFurSheepBreed",
        "MunchkinLonghairCatBreed",
        "NorthMalawiAngoniCattleBreed",
        "WirehairedStandardDachshundDogBreed",
        "PicPigBreed",
        "BhagnariCattleBreed",
        "TaiwanSmallBlackPigBreed",
        "SkyeTerrier",
        "MinbeiSpottedPigBreed",
        "Bodmin950PigBreed",
        "BlackSeaHorseHorseBreed",
        "CriolloCowBreed",
        "SiamesePigBreed",
        "YucaPigBreed",
        "BarraDoCuanzoCattleBreed",
        "BalkanHoundDogBreed",
        "JaulanCattleBreed",
        "KazakhCattleBreed",
        "AlambadiCattleBreed",
        "AmericanMiniatureHorseBreed",
        "EstonianBaconPigBreed",
        "PisanaCattleBreed",
        "RetintaExtremenaGoatBreed",
        "BleuDuMaineSheepBreed",
        "GobraCattleBreed",
        "IrishSetterDogBreed",
        "SteinschafSheepBreed",
        "WiltshirePigBreed",
        "SouthAfricanMiniatureHorseBreed",
        "StabyhounDogBreed",
        "EstonianNativeCattleBreed",
        "HarnaiSheepBreed",
        "KulaPigBreed",
        "BulgarianBrownCattleBreed",
        "DangiCattleBreed",
        "GuizhouHorseBreed",
        "WirehairedVizslaDogBreed",
        "MorabHorseBreed",
        "KoreanNativeCattleBreed",
        "JinzhouHorseBreed",
        "CanastrinhoPigBreed",
        "GoldenGuernseyGoatBreed",
        "FaeroesSheepBreed",
        "KanemXZebuCattleBreed",
        "LiangGuangSmallSpottedPigBreed",
        "FaeroesCattleBreed",
        "TuropoljePigBreed",
        "CormoSheepBreed",
        "NigerianHorseBreed",
        "ChejuHorseBreed",
        "NorthCaucasusPigBreed",
        "KarelianBearDog",
        "AlapahaBluebloodBulldogBreed",
        "SpanishMustangHorseBreed",
        "NuerCattleBreed",
        "BrunoDeJuraDogBreed",
        "WelshSheepdogBreed",
        "ShihTzuDog",
        "ArgentineFriesianCattleBreed",
        "AlbanianDwarfCattleBreed",
        "AulieAtaCattleBreed",
        "WestMacedonianCattleBreed",
        "WatusiUSACattleBreed",
        "SiberianCatBreed",
        "DanakilCattleBreed",
        "AmericanMustangHorseBreed",
        "MashonaCattleBreed",
        "AveligneseHorseBreed",
        "CanadianYorkshirePigBreed",
        "DanishLandraceGoatBreed",
        "BaldingerSpottedPigBreed",
        "IbageXZebuCattleBreed",
        "HirzaiHorseBreed",
        "ShekoCattleBreed",
        "BassetHound",
        "BelgianHorseBreed",
        "BoarPowerPigBreed",
        "PygmyGoatBreed",
        "ChickasawHorseBreed",
        "CholistaniCattleBreed",
        "BelgianSheepdog",
        "DanishSportPonyHorseBreed",
        "BerneseMountainDog",
        "JarmelistaCattleBreed",
        "NewfoundlandDog",
        "AshantiDwarfPigBreed",
        "BahiaPigBreed",
        "FilaBrasileiroDogBreed",
        "WelshPonyOfCobTypeHorseBreed",
        "OrientalHorseHorseBreed",
        "NoricHorseBreed",
        "MecklenburgHorseBreed",
        "BlackWelshMountainSheepBreed",
        "CanadianArcottSheepBreed",
        "WoodenLegGoatBreed",
        "KherigarhCattleBreed",
        "AmericanEskimoDogBreed",
        "CorsicanCattleBreed",
        "ChienDArtoisDogBreed",
        "CamboroughBluePigBreed",
        "NghiaBinhPigBreed",
        "SokotoGudaliCattleBreed",
        "VietnamesePotBelliedPigPigBreed",
        "SanheHorseBreed",
        "FabrianeseSheepBreed",
        "KentuckyRedBerkshirePigBreed",
        "GypsyVannerHorseBreed",
        "DanishLandracePigBreed",
        "YiwuHorseBreed",
        "YiliHorseBreed",
        "BatangasCattleBreed",
        "KerryBlueTerrier",
        "BelgianWarmbloodHorseBreed",
        "RomanianRedCattleBreed",
        "MoradaNovaSheepBreed",
        "LangHongPigBreed",
        "NubaMountainXZebuCattleBreed",
        "JavaHorseBreed",
        "BisaroPigBreed",
        "FlemishCattleBreed",
        "UkrainianWhiteheadedCattleBreed",
        "GoomsurCattleBreed",
        "LithuanianNativePigBreed",
        "HazaragieSheepBreed",
        "MangalargaHorseBreed",
        "HolgusCattleBreed",
        "PanamaSheepBreed",
        "FHCElitePig3PigBreed",
        "CzechMeatPigBreed",
        "HuazhongTwoEndBlackTongchengPigBreed",
        "TongaCattleBreed",
        "BichonsFriseDog",
        "AvilenaCattleBreed",
        "MashenPigBreed",
        "RedAndWhiteCattleBreed",
        "DurocPigBreed",
        "AustrianHoundDogBreed",
        "LorquinaCattleBreed",
        "VenezuelanZebuCattleBreed",
        "Basenji",
        "GalicianPonyHorseBreed",
        "SuiaXZebuCattleBreed",
        "RussianLongEaredWhitePigBreed",
        "GarreCattleBreed",
        "DanishHampshirePigBreed",
        "BoerboelDogBreed",
        "JacobSheepBreed",
        "GermanBlackPiedCattleBreed",
        "HamlinePigBreed",
        "JonicaGoatBreed",
        "MozambiqueAngoniCattleBreed",
        "SpanishBrownAlpineCattleBreed",
        "CentralRussianBlackPiedCattleBreed",
        "KuznetHorseBreed",
        "FrenchLandracePigBreed",
        "ChesapeakeBayRetrieverDogBreed",
        "BassetBleuDeGascogneDogBreed",
        "HequHorseBreed",
        "PumiDogBreed",
        "DartmoorPonyHorseBreed",
        "KrishnagariCattleBreed",
        "FannongSpottedPigBreed",
        "BentheimerLandschafSheepBreed",
        "GrandAngloFrançaisDogBreed",
        "WagyuCattleBreed",
        "DølafeCattleBreed",
        "AbyssinianHorseBreed",
        "PollMerinoSheepBreed",
        "BelmontRedCattleBreed",
        "DalaSheepBreed",
        "PressaMallorquinDogBreed",
        "SpiegelSheepBreed",
        "PabnaXZebuCattleBreed",
        "ShkodraPigBreed",
        "BeagleRoughDogBreed",
        "GelderlandHorseBreed",
        "KleenLeenRedPigBreed",
        "BavarianMountainHoundDogBreed",
        "SerranaGoatBreed",
        "RomanovSheepBreed",
        "CutchiCattleBreed",
        "IberianPigBreed",
        "EgyptianMauCat",
        "MoiCattleBreed",
        "SantaCruzXZebuCattleBreed",
        "MorenasDelNoroesteCattleBreed",
        "MiPigBreed",
        "DanishJerseyCattleBreed",
        "HuchuanMountainPigBreed",
        "IcelandicPigBreed",
        "AmerifaxcattleBreed",
        "Line21PigBreed",
        "AlbanianNativePigBreed",
        "WelshWalkingHorseBreed",
        "NoordhollanderSheepBreed",
        "BarbadoSheepBreed",
        "ChiosSheepBreed",
        "NordlandHorseBreed",
        "PhilippineNativeJalajalaPigBreed",
        "MiniaturePoodleDogBreed",
        "NegraSerranaGoatBreed",
        "SantaCruzSheepBreed",
        "RougeDeLQuestSheepBreed",
        "RedBrangusXZebuCattleBreed",
        "ChamoisOfTheAlpsGoatBreed",
        "AlurCattleBreed",
        "AlentejoHerderDogBreed",
        "SalukiDog",
        "RongchangPigBreed",
        "KangalDogBreed",
        "ItalianBrownCattleBreed",
        "HallingdalCattleBreed",
        "SkyrosHorseBreed",
        "BurwashCattleBreed",
        "BurmesePigBreed",
        "SombaCattleBreed",
        "HerefordCattleSuperbreed",
        "RaboLargoSheepBreed",
        "IsraeliHorseBreed",
        "SharabiCattleBreed",
        "ShahutouPigBreed",
        "CaneCorsoDogBreed",
        "DulmenPonyHorseBreed",
        "BoarPowerPig72PigBreed",
        "WhiteShepherdDogBreed",
        "PersianArabHorseBreed",
        "TosaInuDogBreed",
        "WhiteFulaniCattleBreed",
        "CotswoldSheepBreed",
        "HuangHuaiHaiBlackShenxianPigBreed",
        "FRSCattleBreed",
        "AiredaleTerrier",
        "Camborough22PigBreed",
        "GreekSteppeCattleBreed",
        "GansuAlpineFineWoolSheepBreed",
        "TelemarkCattleBreed",
        "ToupouriCattleBreed",
        "Bloodhound",
        "SwedishLandraceGoatBreed",
        "CotswoldPlatinumPigBreed",
        "CirnecoDellEtnaDogBreed",
        "FinnishSpitzDog",
        "MauritiusCreoleCattleBreed",
        "CaldelanaCattleBreed",
        "RiopardenzeXZebuCattleBreed",
        "OregonRexCatBreed",
        "AzaouakCattleBreed",
        "BaznaPigBreed",
        "BoarPowerPig565PigBreed",
        "BosTaurusXBisonBisonHybridCowBreed",
        "CorsicanHorseBreed",
        "SmallSpanishHoundDogBreed",
        "SouthernTswanaCattleBreed",
        "TSSH1XZebuCattleBreed",
        "EstonianRedCattleBreed",
        "GuteSheepBreed",
        "KilaraCattleBreed",
        "KuriCattleBreed",
        "DanishBroholmerDogBreed",
        "SarplaninacDogBreed",
        "Pointer",
        "FrenchAlpineGoatBreed",
        "SlovakianBlackPiedPigBreed",
        "AmericanEskimoMiniatureDogBreed",
        "IncaHairlessDogBreed",
        "RX3CattleBreed",
        "LiaoningBlackPigBreed",
        "KarakacanHorseBreed",
        "AmericanBashkirCurlyHorseBreed",
        "DeogirCattleBreed",
        "CubanPintoHorseBreed",
        "MontanaPigBreed",
        "ChakouyiHorseBreed",
        "GalicianBlondCattleBreed",
        "SoaySheepBreed",
        "NorthFinncattleCattleBreed",
        "GlanDonnersbergCattleBreed",
        "DaeraDinPanahGoatBreed",
        "WuzhishanPigBreed",
        "AmericanEskimoStandardDogBreed",
        "KishuDogBreed",
        "BorzoiDog",
        "GriffonNivernaisDogBreed",
        "LuingCowBreed",
        "MeraukeCattleBreed",
        "KM1PigBreed",
        "NorlinePigBreed",
        "OldFormatDachsundDogBreed",
        "LimianaCattleBreed",
        "LithuanianHeavyDraftHorseBreed",
        "VitoriaPigBreed",
        "TreeingTennesseeBrindleDogBreed",
        "IberianExtremaduraRedPigBreed",
        "RijnlamASheepBreed",
        "CzechColdbloodHorseBreed",
        "NimariCattleBreed",
        "MezohegyesSportHorseHorseBreed",
        "TurinoCattleBreed",
        "PasoFinoHorseBreed",
        "CentralRussianPigBreed",
        "PantelleriaCattleBreed",
        "AmericanWalkingPonyHorseBreed",
        "BurmeseXZebuCattleBreed",
        "SouthYunnanShortEaredPigBreed",
        "BritishRidingPonyHorseBreed",
        "WelshPigBreed",
        "ThaiRidgebackDogBreed",
        "KuvaszDog",
        "AweilDinkaCattleBreed",
        "BelorusBlackPiedPigBreed",
        "RomanianMeatPigPigBreed",
        "MalakanHorseBreed",
        "JapanesePollCattleBreed",
        "LabradorRetrieverDogBreed",
        "WalerHorseBreed",
        "VietnameseYorkshirePigBreed",
        "MaineCoonCat",
        "GhoriPigBreed",
        "KolubaraCattleBreed",
        "BelarusRedCattleBreed",
        "HainanPigBreed",
        "CanastrãoCapitãoChicoPigBreed",
        "AndeanHorseBreed",
        "IomudHorseBreed",
        "BillyDogBreed",
        "LaVelasquezCattleBreed",
        "PomeranianCoarsewoolSheepBreed",
        "JiningGrayGoatBreed",
        "ShillukCattleBreed",
        "SingapuraCat",
        "GermanLandraceBPigBreed",
        "BretonHorseBreed",
        "NewfoundlandSheepBreed",
        "BeiroaCattleBreed",
        "ShendiCattleBreed",
        "YacumentoCattleBreed",
        "TushinHorseBreed",
        "EpagneulBleuDePicardieDogBreed",
        "ArgentineCriolloHorseBreed",
        "LeicomaPigBreed",
        "MunsterLanderPointerDogBreed",
        "SpanishGreyhoundDogBreed",
        "MixedBreedCat",
        "AppenzellerDogBreed",
        "BrecknockHillCheviotSheepBreed",
        "RoyalCanadianMountedPoliceHorseHorseBreed",
        "SouthChinaBlackPigBreed",
        "FrenchCobHorseBreed",
        "PenshirePigBreed",
        "BaltaretPigBreed",
        "FinnishLandracePigBreed",
        "ShuwaCattleBreed",
        "DomesticLeopardCat",
        "GloucesterOldSpotPigBreed",
        "AlgarviaGoatBreed",
        "LimCattleBreed",
        "AngoraGoatBreed",
        "LatvianHorseBreed",
        "NewZealandRabbitBreed",
        "RamgarhiCattleBreed",
        "LakelandTerrier",
        "SaddlebredHorseSuperbreedHorseBreed",
        "PenbukPigBreed",
        "ByelorussianHarnessHorseBreed",
        "BraunesBergschafSheepBreed",
        "BatakHorseBreed",
        "VerinesaCattleBreed",
        "PretaCattleBreed",
        "LutaiWhitePigBreed",
        "TuyHoaCattleBreed",
        "WeleraPonyHorseBreed",
        "BloodhoundStHubertDogBreed",
        "VictoriaXZebuCattleBreed",
        "SumbawaHorseBreed",
        "RomneySheepBreed",
        "YiliWhitePigBreed",
        "MakaweliCattleBreed",
        "ShorthairCatBreed",
        "AmericanAlbinoHorseBreed",
        "WestFrenchWhitePigBreed",
        "IcelandicCattleBreed",
        "PicLinePig26PigBreed",
        "AmericanFoxhound",
        "BragancaCattleBreed",
        "MalteseDog",
        "PolskiOwczarekNizinnyDogBreed",
        "NigerianPigBreed",
        "IngessanaCattleBreed",
        "ScottishTerrier",
        "MiniatureHerefordCattleBreed",
        "AndalusianBlondCattleBreed",
        "DengchuanCattleBreed",
        "Beagle",
        "ClevelandBayHorseBreed",
        "MoraRomagnolaPigBreed",
        "SardinianSheepBreed",
        "BudyonnyHorseBreed",
        "CanastrãoJunqueiraPigBreed",
        "VianesaCattleBreed",
        "IloiloCattleBreed",
        "CaucasianCattleBreed",
        "GermanLandracePigBreed",
        "HarianaCattleBreed",
        "MalawiZebuCattleBreed",
        "FieldSpanielDogBreed",
        "XiangPigBreed",
        "BerrendasCattleBreed",
        "TimorHorseBreed",
        "RygjaSheepBreed",
        "BanEiHorseBreed",
        "SalernoHorseBreed",
        "AbyssinianHighlandZebuCattleBreed",
        "MadagascarZebuCattleBreed",
        "OberhasliGoatBreed",
        "CanikHorseBreed",
        "WuyiBlackPigBreed",
        "MoscowLonghairedToyTerrierDogBreed",
        "KuchinoshimaCattleBreed",
        "JijjigaZebuCattleBreed",
        "MasaiSheepBreed",
        "SwedishLandracePigBreed",
        "PolwarthSheepBreed",
        "ExoticShorthairedCat",
        "ShanHorseBreed",
        "TreeingFeistDogBreed",
        "RomagnolaCattleBreed",
        "MubanPigBreed",
        "PajunaCattleBreed",
        "NorbottenspetsDogBreed",
        "LugwareCattleBreed",
        "PulawyPigBreed",
        "StumpyTailCattleDogBreed",
        "ChartreuxCat",
        "ZaobeiCattleBreed",
        "CambodianCattleBreed",
        "AustralianGoatBreed",
        "WelshMountainSheepBreed",
        "RottweilerDog",
        "MegrelHorseBreed",
        "TaihuPigBreed",
        "ThillariCattleBreed",
        "PeloponnesusCattleBreed",
        "DLSSheepBreed",
        "ManapariCattleBreed",
        "WhiteSuffolkSheepBreed",
        "SanjiangWhitePigBreed",
        "PiauCarunchoPiauPigBreed",
        "MacauPigBreed",
        "NgandaCattleBreed",
        "BariaVietnamMadagascarCattleBreed",
        "PintoHorseBreed",
        "FrenchLargeWhitePigBreed",
        "MisakiHorseBreed",
        "CayuseHorseBreed",
        "SicilianPigBreed",
        "TaoyuanPigBreed",
        "BimalCattleBreed",
        "BrandenburgHorseBreed",
        "AbtenauerHorseBreed",
        "MediumWoolMerinoSheepBreed",
        "BengalCatBreed",
        "FrenchTrotterHorseBreed",
        "DrakensbergerCattleBreed",
        "JamaicaRedXZebuCattleBreed",
        "KraskyOvcarDogBreed",
        "PlottHound",
        "LucanianCattleBreed",
        "Affenpinscher",
        "BoarPowerPig828PigBreed",
        "SeferihisarCattleBreed",
        "CaliforniaVariegatedMutantSheepBreed",
        "ArabianHorseBreed",
        "GreaterSwissMountainDogBreed",
        "SuffolkCattleBreed",
        "ZhongweiGoatBreed",
        "LlanwenogSheepBreed",
        "CosteñoConCuernosCattleBreed",
        "SolognoteSheepBreed",
        "BrownAtlasCattleBreed",
        "TeddyRooseveltTerrierDogBreed",
        "BriquetBassetGriffonVendeenDogBreed",
        "PolledGirCattleBreed",
        "SoftCoatedWheatenTerrier",
        "DaweiziPigBreed",
        "HarzRedCattleBreed",
        "PicLinePig24PigBreed",
        "DorsetDownSheepBreed",
        "OrientalLonghairCatBreed",
        "KyiLeoDogBreed",
        "BakwiriCattleBreed",
        "OxfordSandyBlockPigBreed",
        "KamoriGoatBreed",
        "PieRougeDesPlainesCattleBreed",
        "DermantsiPiedPigBreed",
        "RasNLansaPigBreed",
        "PitmanMooreMiniaturePigBreed",
        "KnabstrupHorseBreed",
        "BlackRussianTerrierDogBreed",
        "ChryakPICPigBreed",
        "SiriZebuCattleBreed",
        "InobutaInterSpeciesHybridPigBreed",
        "BachaurCattleBreed",
        "BosTaurusTaurusSubspeciesDomesticEuropeanCowBreed",
        "SouthChinaZebuCattleBreed",
        "ManipuriHorseBreed",
        "BoerGoatBreed",
        "DeutschesBlaukoepfigesFleischschafSheepBreed",
        "BergamascoDogBreed",
        "AngloArabHorseBreed",
        "LargeBlackPigBreed",
        "BrabantHorseBreed",
        "JavaneseOngoleCattleBreed",
        "IrishDraftHorseBreed",
        "JapaneseSpitzDogBreed",
        "GhanaSangaXZebuCattleBreed",
        "EstonianDraftHorseBreed",
        "DeutscheBrackeDogBreed",
        "DrentsHeideschaapSheepBreed",
        "MiamiPigBreed",
        "HerensCattleBreed",
        "KenranCattleBreed",
        "HalfSaddlebredHorseBreed",
        "HungahybPigBreed",
        "EntlebucherDogBreed",
        "LocalIndianDairyXZebuCattleBreed",
        "AlaskanMalamute",
        "LohaniCattleBreed",
        "BeltedWelshCattleBreed",
        "SomaliPonyHorseBreed",
        "ShropshireTerrierDogBreed",
        "DornaCattleBreed",
        "YucatanMinaturePigBreed",
        "AureEtSaintGironsCattleBreed",
        "AbondanceCattleBreed",
        "WurttembergHorseBreed",
        "FuanSpottedPigBreed",
        "JämthundDogBreed",
        "KravarskyCattleBreed",
        "EpirusCattleBreed",
        "AsturianMountainCattleBreed",
        "HaBacPigBreed",
        "PerdigueroDeBurgosDogBreed",
        "Hound",
        "RomanianSteppeCattleBreed",
        "YakutHorseBreed",
        "LatvianBrownCattleBreed",
        "WaziriHorseBreed",
        "HanjiangBlackPigBreed",
        "BelgianWhiteAndRedCattleBreed",
        "GreaterCaucasusCattleBreed",
        "MaremmanaPigBreed",
        "BrandlbrackeDogBreed",
        "MazuryCattleBreed",
        "BoykinSpanielDogBreed",
        "BrahornXZebuCattleBreed",
        "EgyptianCattleBreed",
        "PottokTarpanHorseXDomesticHorseBreed",
        "BoarPowerPig929PigBreed",
        "SorraiaHorseBreed",
        "HeiheHorseBreed",
        "FriuliBlackPigBreed",
        "FaeroesIslandHorseHorseBreed",
        "RegusCattleBreed",
        "EnglishToySpaniel",
        "ClydesdaleHorseBreed",
        "OldFormatSheepBreed",
        "CalabrianPigBreed",
        "MaremmaSheepdogsDogBreed",
        "RedHampracePigBreed",
        "BoranCattleBreed",
        "SwissEdelschweinPigBreed",
        "NadbuzanskaPigBreed",
        "AppaloosaHorseBreed",
        "HailumPigBreed",
        "BanjaraGreyhoundDogBreed",
        "NationalSpottedSaddleHorseHorseBreed",
        "NGaoundereCattleBreed",
        "GermanColoredGoatBreed",
        "BearnaisCattleBreed",
        "AkbashDogBreed",
        "EnglishBulldogBreed",
        "GuadeloupeCreolePigBreed",
        "OldGermanShepherdDogBreed",
        "KaokoveldCattleBreed",
        "SenegusCattleBreed",
        "LapermLonghairCatBreed",
        "RackingHorseHorseBreed",
        "SimmentalCowBreed",
        "GanzhouWhitePigBreed",
        "ChineseMongolianHorseBreed",
        "EastFriesianOldTypeHorseBreed",
        "JapaneseBobtailCat",
        "KwaiPigBreed",
        "WarmbloodHorseBreed",
        "AlbeseCattleBreed",
        "GuanzhongHorseBreed",
        "KelePigBreed",
        "ParmensePigBreed",
        "ChowChow",
        "NormandPigBreed",
        "MishimaCattleBreed",
        "ComebackSheepBreed",
        "BergamascaSheepBreed",
        "BlackForrestCattleBreed",
        "PrewakwaCattleBreed",
        "HungarianPiedCattleBreed",
        "NachiGoatBreed",
        "UkrainianSaddleHorseHorseBreed",
        "ThailandFightingZebuCattleBreed",
        "ElegantWarmbloodHorseBreed",
        "BalineseCat",
        "TibetanTerrier",
        "AustralianLowlineCattleBreed",
        "PetitGasconSaintongeoisDogBreed",
        "IstobenCattleBreed",
        "AdenZebuCattleBreed",
        "JotkoXZebuCattleBreed",
        "AracenaCattleBreed",
        "BondSheepBreed",
        "PolypaySheepBreed",
        "OldEnglishSheepdog",
        "BoarPowerPig545PigBreed",
        "BlancaAndaluzaGoatBreed",
        "DeoniCattleBreed",
        "BluefacedLeicesterSheepBreed",
        "BrittanySpanielDogBreed",
        "AppenzellGoatBreed",
        "CairnTerrier",
        "ApennineSheepBreed",
        "IbizanHound",
        "CurlyCoatedRetrieverDogBreed",
        "SudanCountryBredHorseBreed",
        "AleppoCattleBreed",
        "LongAndShortCoatChihuahuaDogBreed",
        "SouthGermanColdbloodHorseBreed",
        "BaladicattleCattleBreed",
        "MoyleHorseBreed",
        "IcelandicSheepBreed",
        "BrahmousinXZebuCattleBreed",
        "FratiCattleBreed",
        "XinjinPigBreed",
        "RhaetianGrayCattleBreed",
        "GaunlingCattleBreed",
        "HaysConverterCowBreed",
        "JinhongHorseBreed",
        "SardinianPonyHorseBreed",
        "LichuanHorseBreed",
        "GanzhongnanSpottedPigBreed",
        "StandardSchnauzerDogBreed",
        "HainanWenchangPigBreed",
        "WestphalianHorseBreed",
        "CãoDaSerraDeAiresDogBreed",
        "RhodesianRidgebackDog",
        "CamargueHorseBreed",
        "CinisaraCattleBreed",
        "SabuesosEspañolesDogBreed",
        "MoruchaCattleBreed",
        "HampenPigBreed",
        "QuasahXZebuCattleBreed",
        "BlackAngusCowBreed",
        "LatiSheepBreed",
        "SamoyedDog",
        "FHCElitePig1PigBreed",
        "ConnorPrairiePigBreed",
        "EnglishCockerSpanielDogBreed",
        "RockAlpineGoatBreed",
        "CzechPiedCattleBreed",
        "SanClementeGoatBreed",
        "CubanCriolloXZebuCattleBreed",
        "LatvianWhitePigBreed",
        "ShibaInuDogBreed",
        "KandahariCattleBreed",
        "ClumberSpanielDogBreed",
        "FloridaCrackerPineywoodsCattleBreed",
        "GermanBrownCattleBreed",
        "Slovhyb1PigBreed",
        "PolandChinaPigBreed",
        "TainoXZebuCattleBreed",
        "GloucesterCattleBreed",
        "KajliSheepBreed",
        "PiquiraPonyHorseBreed",
        "GelbveihCowBreed",
        "ChineseFooDogBreed",
        "GasconyPigBreed",
        "RomedaleSheepBreed",
        "TinimaCattleBreed",
        "ChincoteaguePonyHorseBreed",
        "CanadianHorseBreed",
        "DialiCattleBreed",
        "SakhalinWhitePigBreed",
        "BoxerDog",
        "MongallaCattleBreed",
        "PelónPigBreed",
        "PechoraHorseBreed",
        "ChienFrançaisBlancEtNoirDogBreed",
        "NeopolitanMastiffDogBreed",
        "RetintaCattleBreed",
        "EleiaHorseBreed",
        "ShengxianSpottedPigBreed",
        "KenyaZebuCattleBreed",
        "PintabianHorseBreed",
        "PetitBrabaçonDogBreed",
        "SlovakianPiedCattleBreed",
        "MexicanWattledPigBreed",
        "Collie",
        "KerryHillSheepBreed",
        "BarotseCattleBreed",
        "BhutiaHorseBreed",
        "GermanWirehairedPointerDogBreed",
        "MorangCattleBreed",
        "RussianLargeWhitePigBreed",
        "ChimaineCattleBreed",
        "LuzeinSheepBreed",
        "FoutaHorseBreed",
        "RedGallowayCattleBreed",
        "TransylvanianPinzguaCattleBreed",
        "FroxfieldPygmyPigBreed",
        "AyrshireCowBreed",
        "MoravaPigBreed",
        "HampshireDownSheepBreed",
        "BambawaCattleBreed",
        "AustralianWhiteCattleBreed",
        "SudaneseFulaniCattleBreed",
        "WelshHillSpeckledFaceSheepBreed",
        "SchillerHoundDogBreed",
        "AlbèresCattlebreed",
        "RedboneCoonhoundDogBreed",
        "BootedGoatBreed",
        "HungarianSportHorseHorseBreed",
        "SPMPigBreed",
        "ExmoorPonyHorseBreed",
        "PinzgauerCowBreed",
        "WiltshireHornSheepBreed",
    ]
}

name_for_cid[7481] = "BreedRegistry"
cid_concepts[7481] = {
    "DCM": [
        "CanadianBorderCollieAssociation",
        "UnitedKennelClub",
        "AnimalResearchFoundation",
        "UnitedAllBreedRegistry",
        "CanadianLivestockRecordsAssociation",
        "CanadianKennelClub",
        "UniversalKennelClubInternational",
        "AmericanPurebredRegistry",
        "InternationalProgressiveDogBreedersAlliance",
        "AmericanCanineAssociation",
        "FederationOfInternationalCanines",
        "AnimalRegistryUnlimited",
        "NorthAmericanPurebredDogRegistry",
        "AmericanRareBreedAssociation",
        "NationalKennelClub",
        "CanineFederationOfCanada",
        "WorldKennelClub",
        "WorkingCanineAssociationOfCanada",
        "AmericaPetRegistryInc",
        "DogRegistryOfAmerica",
        "AmericaKennelClub",
        "ContinentalKennelClub",
        "WorldWideKennelClub",
    ]
}

name_for_cid[7482] = "DXAnatomyImagedforAnimal"
cid_concepts[7482] = {
    "SCT": [
        "FetlockOfHindlimb",
        "Sacrum",
        "ThoracoLumbarSpine",
        "NasalSinus",
        "Metatarsus",
        "NavicularOfHindfoot",
        "CalcanealTubercle",
        "Humerus",
        "FetlockOfForelimb",
        "Esophagus",
        "Bladder",
        "AtlantalAxialJoint",
        "Mandible",
        "PasternOfHindfoot",
        "Wing",
        "Metacarpus",
        "MaxillaryIncisorTeeth",
        "UrinaryTract",
        "MandibularDentalArch",
        "Pelvis",
        "Carpus",
        "EntireBody",
        "ChestAndAbdomen",
        "RadiusAndUlna",
        "LumbarSpine",
        "Abdomen",
        "Digit",
        "Hindfoot",
        "MaxillaryDentalArch",
        "TibiaAndFibula",
        "PasternOfForefoot",
        "Forefoot",
        "Tarsus",
        "UpperGastroIntestinalTract",
        "Shoulder",
        "Stiffle",
        "ElbowJoint",
        "Colon",
        "AllLegs",
        "OrbitalStructure",
        "ThoracicSpine",
        "HipJoint",
        "Chest",
        "CoccygealVertrebrae",
        "MandibularIncisorTeeth",
        "Femur",
        "FrontalSinus",
        "CervicoThoracicSpine",
        "Skull",
        "Urethra",
        "Patella",
        "AtlantoOccipitalJoint",
        "LumboSacralSpine",
        "NavicularOfForefoot",
        "CervicalSpine",
    ],
    "UMLS": ["DistalPhalanx"],
}

name_for_cid[7483] = "CommonAnatomicRegionsforAnimal"
cid_concepts[7483] = {
    "SCT": [
        "FetlockOfHindlimb",
        "Sacrum",
        "ThoracoLumbarSpine",
        "NasalSinus",
        "Metatarsus",
        "NavicularOfHindfoot",
        "CalcanealTubercle",
        "Humerus",
        "FetlockOfForelimb",
        "Esophagus",
        "Bladder",
        "AtlantalAxialJoint",
        "Mandible",
        "PasternOfHindfoot",
        "Wing",
        "Metacarpus",
        "MaxillaryIncisorTeeth",
        "UrinaryTract",
        "MandibularDentalArch",
        "Pelvis",
        "Carpus",
        "EntireBody",
        "ChestAndAbdomen",
        "RadiusAndUlna",
        "LumbarSpine",
        "Abdomen",
        "Digit",
        "Hindfoot",
        "MaxillaryDentalArch",
        "TibiaAndFibula",
        "PasternOfForefoot",
        "Forefoot",
        "Tarsus",
        "UpperGastroIntestinalTract",
        "Shoulder",
        "Stiffle",
        "ElbowJoint",
        "Colon",
        "AllLegs",
        "OrbitalStructure",
        "ThoracicSpine",
        "HipJoint",
        "Chest",
        "CoccygealVertrebrae",
        "MandibularIncisorTeeth",
        "Femur",
        "FrontalSinus",
        "CervicoThoracicSpine",
        "Skull",
        "Urethra",
        "Patella",
        "AtlantoOccipitalJoint",
        "LumboSacralSpine",
        "NavicularOfForefoot",
        "CervicalSpine",
    ],
    "UMLS": ["DistalPhalanx"],
}

name_for_cid[7484] = "DXViewforAnimal"
cid_concepts[7484] = {
    "DCM": ["Caudal10DegreeDistalCranioproximalOblique"],
    "SCT": [
        "Rostrocaudal",
        "Dorsal40DegreeLateralPlantaromedialOblique",
        "Dorsal45DegreeMedialPlantarolateralOblique",
        "Right20DegreeDorsalLeftVentralOblique",
        "DorsorostralVentrocaudalOblique",
        "Lateral45DegDorsal50DegProximalMedioplantarodistalObliq",
        "DorsoVentral",
        "Dorsal45DegreeMedialPalmarolateral",
        "Ventral30DegreeLeftDorsalRightOblique",
        "LeftVentralRightDorsalOblique",
        "VentrorostralDorsocaudalOblique",
        "RightDorsalLeftVentralOblique",
        "PalmaroproximalDorsodistalOblique",
        "CaudodistalCranioproximalOblique",
        "Dorsopalmar",
        "LeftRightLateral",
        "Left45DegreeDorsalRightVentralOblique",
        "Dorsal20DegreeRostralVentrocaudalOblique",
        "Left20DegreeRostralRightCaudalOblique",
        "Left20DegreeVentralRightDorsalOblique",
        "Lateral45DegDorsal50DegProximalMediopalmarodistalOblique",
        "Dorsal45DegreeLateralPlantaromedialOblique",
        "Dorsal60DegreeLateralPlantaromedialOblique",
        "RostroventralCaudodorsal",
        "VentralLeftDorsalRightOblique",
        "Dorsal40DegreeMedialPlantarolateralOblique",
        "Right45DegreeDorsalLeftVentralOblique",
        "ProximoDistal",
        "Plantar60DegreeLateralDorsomedialOblique",
        "Right30DegreeCaudalLeftRostralOblique",
        "Right20DegreeVentralLeftDorsalOblique",
        "PlantarolateralDorsomedialOblique",
        "Dorsal45DegreeLateralPalmaromedialOblique",
        "PalmaromedialDorsolateral",
        "RostrodorsalCaudoventralOblique",
        "DorsomedialPalmarolateral",
        "Dorsal35DegreeLateralPalmaromedialOblique",
        "Ventral20DegreeRostralDorsocaudalOblique",
        "Rostral20DegreeDorsalCaudoventralOblique",
        "DorsolateralPalmaromedialOblique",
        "Dorsal65DegreeProximalPalmarodistalOblique",
        "Dorsal35DegreeMedialPalmarolateralOblique",
        "RightCaudalLeftRostralOblique",
        "Dorsal60DegreeMedialPalmarolateralOblique",
        "Ventral30DegreeRightDorsalLeftOblique",
        "Dorsal40DegreeMedialPalmarolateralOblique",
        "LeftDorsalRightVentralOblique",
        "Dorsal35DegreeLateralPlantaromedialOblique",
        "VentralRightDorsalLeftOblique",
        "LaterodorsoproximalMediopalmarodistalOblique",
        "VentroDorsal",
        "DorsoproximalPlantarodistalOblique",
        "PlantaroproximalDorsodistalOblique",
        "DorsomedialPlantarolateralOblique",
        "Caudocranial",
        "Left30DegreeCaudalRightRostralOblique",
        "Right20DegreeRostralLeftCaudalOblique",
        "RightRostralLeftCaudalOblique",
        "LeftCaudalRightRostralOblique",
        "Dorsal60DegreeLateralPalmaromedialOblique",
        "Left20DegreeDorsalRightVentralOblique",
        "DorsoproximalPalmarodistalOblique",
        "RightLeftLateral",
        "Dorsal60DegreeMedialPlantarolateralOblique",
        "Lateromedial",
        "Plantar75DegreeProximalDorsodistalOblique",
        "Rostral30DegreeVentralCaudodorsal",
        "Dorsal35DegreeMedialPlantarolateralOblique",
        "Dorsal65DegreeProximalPlantarodistalOblique",
        "Dorsoplantar",
        "LeftRostralRightCaudalOblique",
        "LaterodorsoproximalMedioplantarodistalOblique",
        "Right45DegreeVentralLeftDorsalOblique",
        "Left45DegreeVentralRightDorsalOblique",
        "MedioLateralProjection",
        "Palmar45DegreeMedialDorsolateral",
        "DorsolateralPlantaromedialOblique",
        "Palmar75DegreeProximalDorsodistalOblique",
        "RightVentralLeftDorsalOblique",
    ],
}

name_for_cid[7486] = "MixedBreed"
cid_concepts[7486] = {
    "SCT": [
        "MixedBreedGoat",
        "MixedBreedCattle",
        "MixedBreedChicken",
        "MixedBreedDog",
        "MixedBreedCat",
        "MixedBreedPig",
        "MixedBreedHorse",
        "MixedBreedSheep",
    ]
}

name_for_cid[7490] = "ResearchAnimalSourceRegistry"
cid_concepts[7490] = {"DCM": ["ILCR"]}

name_for_cid[7500] = "NeighbourhoodGreyToneDifferenceBasedFeature"
cid_concepts[7500] = {
    "IBSI": [
        "ContrastOfNGTDM",
        "ComplexityOfNGTDM",
        "BusynessOfNGTDM",
        "CoarsenessOfNGTDM",
        "StrengthOfNGTDM",
    ]
}

name_for_cid[7501] = "NeighbouringGreyLevelDependenceBasedFeature"
cid_concepts[7501] = {
    "IBSI": [
        "DependenceCountEntropy",
        "NormalizedGreyLevelNonUniformityOfNGLDM",
        "GreyLevelVarianceOfNGLDM",
        "LowDependenceLowGreyLevelEmphasis",
        "DependenceCountNonUniformityNormalized",
        "GreyLevelNonUniformityOfNGLDM",
        "DependenceCountPercentage",
        "LowDependenceHighGreyLevelEmphasis",
        "HighDependenceEmphasis",
        "DependenceCountVariance",
        "DependenceCountEnergy",
        "HighGreyLevelCountEmphasis",
        "HighDependenceLowGreyLevelEmphasis",
        "DependenceCountNonUniformity",
        "LowDependenceEmphasis",
        "HighDependenceHighGreyLevelEmphasis",
        "LowGreyLevelCountEmphasis",
    ]
}

name_for_cid[7550] = "AngleMeasurement"
cid_concepts[7550] = {"DCM": ["AcetabularAngle"], "SCT": ["CobbAngle"]}

name_for_cid[7551] = "GenericPurposeofReferencetoImagesandCoordinatesinMeasurement"
cid_concepts[7551] = {
    "DCM": ["ApproximateSpatialLocation", "Vector", "SourceOfMeasurement", "ArmOfAngle"]
}

name_for_cid[7552] = "GenericPurposeofReferencetoImagesinMeasurement"
cid_concepts[7552] = {"DCM": ["SourceOfMeasurement"]}

name_for_cid[7553] = "GenericPurposeofReferencetoCoordinatesinMeasurement"
cid_concepts[7553] = {"DCM": ["ApproximateSpatialLocation", "Vector", "ArmOfAngle"]}

name_for_cid[7600] = "LymphNodeAnatomicSite"
cid_concepts[7600] = {
    "FMA": ["SubiliacLymphNode", "SupramammaryLymphNode"],
    "SCT": [
        "DeepLateralCervicalLymphNode",
        "AnteriorAuricularLymphNode",
        "IntermediateExternalIliacLymphNode",
        "LymphNodeOfHead",
        "InferiorInguinalLymphNode",
        "AnteriorCervicalLymphNode",
        "IntrapulmonaryLymphNode",
        "RetropharyngealLymphNode",
        "PectoralAxillaryLymphNode",
        "SuperiorGlutealLymphNode",
        "ParasternalLymphNode",
        "LymphNode",
        "CommonDuctLymphNode",
        "MedialLacunarLymphNode",
        "SegmentalLymphNodeOfTheLung",
        "BuccinatorLymphNode",
        "LateralPericardialLymphNode",
        "HighestMediastinalLymphNode",
        "SubscapularAxillaryLymphNode",
        "PrefemoralLymphNode",
        "ExtrapulmonaryLymphNodeOfLung",
        "ParotidLymphNode",
        "LowerParatrachealIncludingAzygousLymphNode",
        "DelphianLymphNode",
        "PrececalLymphNode",
        "SuperiorRectalLymphNode",
        "SuperficialParotidLymphNode",
        "HypogastricLymphNode",
        "IliacLymphNode",
        "LymphNodeOfMesentery",
        "EpitrochlearLymphNode",
        "AxillaryLymphNode",
        "JuxtaintestinalLymphNode",
        "ParatrachealLymphNode",
        "DeepAnteriorCervicalLymphNode",
        "LumbarLymphNode",
        "UpperParatrachealLymphNode",
        "RegionalLymphNode",
        "PyloricLymphNode",
        "LacunarLymphNode",
        "PosteriorTibialLymphNode",
        "PancreaticosplenicLymphNode",
        "DeepParotidLymphNode",
        "PelvicLymphNode",
        "LymphNodeOfAortopulmonaryWindow",
        "CeliacLymphNode",
        "LateralJugularLymphNode",
        "PosteriorMediastinalLymphNode",
        "LymphNodeOfLesserCurvatureOfStomach",
        "MiddleDeepCervicalLymphNode",
        "InfraclavicularLymphNode",
        "SupraclavicularLymphNode",
        "SubcarinalLymphNode",
        "ParametrialLymphNode",
        "ParavesicularLymphNode",
        "SuperficialPoplitealLymphNode",
        "PrevesicularLymphNode",
        "GlutealLymphNode",
        "SacralLymphNode",
        "RetrotrachealLymphNode",
        "RetroperitonealLymphNode",
        "DeepCervicalLymphNode",
        "DeepPoplitealLymphNode",
        "SuperiorMediastinalLymphNode",
        "VesicularLymphNode",
        "IntermediateCommonIliacLymphNode",
        "SplenicLymphNode",
        "OccipitalLymphNode",
        "InferiorAuricularLymphNode",
        "NasolabialLymphNode",
        "RenalHilarLymphNode",
        "HilarLymphNode",
        "SuperficialInguinalLymphNode",
        "UterineParacervicalLymphNode",
        "InterlobarLymphNodeOfTheLung",
        "LymphNodeOfStomach",
        "LateralExternalIliacLymphNode",
        "EpigastricLymphNode",
        "LymphNodeOfGreaterCurvatureOfStomach",
        "PoplitealLymphNode",
        "IntramammaryLymphNode",
        "SubaorticCommonIliacLymphNode",
        "HepaticLymphNode",
        "SuperficialIntraparotidLymphNode",
        "PresymphysialLymphNode",
        "IntercostalLymphNode",
        "LymphNodeOfAorticArch",
        "PararectalLymphNode",
        "PeripancreaticLymphNode",
        "TracheobronchialLymphNodeLocatedNearCarina",
        "AorticLymphNode",
        "SuperficialLymphNode",
        "TibialLymphNode",
        "CentralAxillaryLymphNode",
        "SuperiorMesentericLymphNode",
        "IntraglandularParotidLymphNode",
        "SigmoidLymphNode",
        "PostvesicularLymphNode",
        "MidcolicLymphNode",
        "PosteriorTriangleCervicalLymphNode",
        "InferiorGlutealLymphNode",
        "EsophagealLymphNode",
        "TracheobronchialLymphNode",
        "CardiophrenicAngleLymphNode",
        "LobarLymphNodeOfTheLung",
        "SubclavianLymphNode",
        "SuperiorMedialInguinalLymphNode",
        "PancreaticLymphNode",
        "CervicalLymphNode",
        "IleocolicLymphNode",
        "LateralCommonIliacLymphNode",
        "SuperiorTracheobronchialLymphNode",
        "UpperDeepCervicalLymphNode",
        "RetropyloricLymphNode",
        "SuperficialCervicalLymphNode",
        "MiddleJugularLymphNode",
        "LymphNodeOfEpiploicForamen",
        "AppendicularLymphNode",
        "LateralVesicularLymphNode",
        "PretrachealLymphNode",
        "LymphNodeOfLimb",
        "RetrocecalLymphNode",
        "AnteriorMediastinalLymphNode",
        "AbdominalLymphNode",
        "PromontoryCommonIliacLymphNode",
        "CysticLymphNode",
        "SubmentalLymphNode",
        "MedialExternalIliacLymphNode",
        "LowerJugularLymphNode",
        "PancreaticoduodenalLymphNode",
        "MandibularLymphNode",
        "PrepericardialLymphNode",
        "MediastinalLymphNode",
        "LowerLimbLymphNode",
        "LungAndTracheobronchialLymphNodes",
        "PrevascularRetrotrachealLymphNode",
        "DiaphragmaticLymphNode",
        "InferiorMesentericLymphNode",
        "FemaleGenitalLymphNode",
        "LymphNodeOfThePulmonaryLigament",
        "PostauricularLymphNode",
        "LymphNodeRingOfCardiaOfStomach",
        "ParavaginalLymphNode",
        "AxillaryVeinLymphNode",
        "DeepInguinalLymphNode",
        "DeepIntraparotidLymphNode",
        "UpperLimbLymphNode",
        "IntestinalLymphNode",
        "SubpyloricLymphNode",
        "InferiorPancreaticoduodenalLymphNode",
        "LymphNodeOfTrunk",
        "ParaesophagealLymphNodeBelowCarina",
        "ParaAorticLymphNodeOfTheAnteriorMediastinum",
        "PerigastricLymphNode",
        "UpperJugularLymphNode",
        "MedialCommonIliacLymphNode",
        "AnteriorTibialLymphNode",
        "InguinalLymphNode",
        "LateralCervicalLymphNode",
        "InterpectoralLymphNode",
        "LowerDeepCervicalLymphNode",
        "LymphNodeOfThorax",
        "ExternalMammaryLymphNode",
        "DeepLymphNode",
        "HemolymphNode",
        "MedianRetropharyngealLymphNode",
        "ColicLymphNode",
        "FacialLymphNode",
        "InteriliacLymphNode",
        "PrelaryngealLymphNode",
        "PosteriorAuricularLymphNode",
        "SubsegmentalLymphNodeOfTheLung",
        "SuperiorPancreaticoduodenalLymphNode",
        "ApicalAxillaryLymphNode",
        "FibularLymphNode",
        "SuprapyloricLymphNode",
        "InnominateLymphNode",
        "SuperficialLateralCervicalLymphNode",
        "PrevertebralLymphNode",
        "MesentericArteryLymphNode",
        "SuperficialAnteriorCervicalLymphNode",
        "LateralRetropharyngealLymphNode",
        "SuperiorPancreaticLymphNode",
        "SubmandibularLymphNode",
        "ExternalIliacLymphNode",
        "ParamammaryLymphNode",
        "ObturatorLymphNode",
        "SuperiorLateralInguinalLymphNode",
        "MastoidLymphNode",
        "JugularLymphNode",
        "LymphNodeOfHeadAndNeck",
        "GastroOmentalLymphNode",
        "LateralAxillaryLymphNode",
        "CubitalLymphNode",
        "ScaleneLymphNode",
        "InferiorPancreaticLymphNode",
    ],
}

name_for_cid[7601] = "HeadandNeckCancerAnatomicSite"
cid_concepts[7601] = {
    "FMA": ["TorusOfPharyngotympanicTube"],
    "SCT": [
        "PharyngealTonsil",
        "ParanasalSinus",
        "Oropharynx",
        "FloorOfMouth",
        "BuccalMucosa",
        "Supraglottis",
        "BaseOfTongue",
        "Lip",
        "Tongue",
        "PharyngealRecess",
        "TonsilAndAdenoid",
        "Hypopharynx",
        "AlveolarProcessOfMandible",
        "OropharyngealTonsil",
        "SalivaryGland",
        "MaxillarySinus",
        "PyriformSinus",
        "Glottis",
        "LingualTonsil",
        "Larynx",
        "Uvula",
        "OralCavity",
        "PharyngotympanicTube",
        "PalatineTonsil",
        "NasalCavity",
        "Nasopharyngeal",
        "TubalTonsil",
        "RetromolarTrigone",
    ],
    "UMLS": ["UnknownPrimaryNeoplasiaSite"],
}

name_for_cid[7602] = "VertebralAnatomicStructure"
cid_concepts[7602] = {
    "SCT": [
        "L1Vertebra",
        "T3T4Disc",
        "C3C4Disc",
        "T7Vertebra",
        "T11T12Disc",
        "C2Vertebra",
        "T2T3Disc",
        "T4Vertebra",
        "C1Vertebra",
        "T8Vertebra",
        "T9T10Disc",
        "L4L5Disc",
        "T9Vertebra",
        "T10Vertebra",
        "C2C3Disc",
        "C5Vertebra",
        "L3L4Disc",
        "T7T8Disc",
        "T12Vertebra",
        "C3Vertebra",
        "T5T6Disc",
        "T11Vertebra",
        "T2Vertebra",
        "T6T7Disc",
        "L2L3Disc",
        "C5C6Disc",
        "T12L1Disc",
        "T3Vertebra",
        "T5Vertebra",
        "T1Vertebra",
        "L2Vertebra",
        "L3Vertebra",
        "T4T5Disc",
        "L1L2Disc",
        "L4Vertebra",
        "C7T1Disc",
        "C7Vertebra",
        "T10T11Disc",
        "T8T9Disc",
        "C6Vertebra",
        "T6Vertebra",
        "T1T2Disc",
        "L5S1Disc",
        "C6C7Disc",
        "C4Vertebra",
        "L5Vertebra",
        "C4C5Disc",
    ]
}

name_for_cid[7603] = "Vertebra"
cid_concepts[7603] = {
    "SCT": [
        "L1Vertebra",
        "T3Vertebra",
        "T5Vertebra",
        "T1Vertebra",
        "T7Vertebra",
        "C2Vertebra",
        "L2Vertebra",
        "L3Vertebra",
        "T4Vertebra",
        "C1Vertebra",
        "T8Vertebra",
        "L4Vertebra",
        "T9Vertebra",
        "T10Vertebra",
        "C7Vertebra",
        "C5Vertebra",
        "C6Vertebra",
        "T12Vertebra",
        "C3Vertebra",
        "T6Vertebra",
        "T11Vertebra",
        "C4Vertebra",
        "T2Vertebra",
        "L5Vertebra",
    ]
}

name_for_cid[7604] = "IntervertebralDisc"
cid_concepts[7604] = {
    "SCT": [
        "T3T4Disc",
        "C3C4Disc",
        "T11T12Disc",
        "T2T3Disc",
        "T4T5Disc",
        "L1L2Disc",
        "T9T10Disc",
        "L4L5Disc",
        "C2C3Disc",
        "C7T1Disc",
        "T10T11Disc",
        "T8T9Disc",
        "L3L4Disc",
        "T7T8Disc",
        "T5T6Disc",
        "T1T2Disc",
        "L5S1Disc",
        "C6C7Disc",
        "T6T7Disc",
        "L2L3Disc",
        "C4C5Disc",
        "C5C6Disc",
        "T12L1Disc",
    ]
}

name_for_cid[7701] = "FiberTractInBrainstem"
cid_concepts[7701] = {
    "SCT": [
        "CorticospinalTractInBrainstem",
        "MedialLemniscus",
        "MedialLongitudinalFasciculus",
        "InferiorCerebellarPeduncle",
        "SuperiorCerebellarPeduncle",
        "LateralLemniscus",
        "MiddleCerebellarPeduncle",
    ]
}

name_for_cid[7702] = "ProjectionandThalamicFiber"
cid_concepts[7702] = {
    "NEU": [
        "CorticobulbarTract",
        "AnteriorThalamicRadiation",
        "SuperiorOpticRadiationBaumLoop",
        "CorticospinalTract",
        "InferiorOpticRadiationMeyerLoop",
        "SuperiorThalamicRadiation",
        "InferiorThalamicRadiation",
        "PosteriorThalamicRadiation",
    ],
    "SCT": [
        "AuditoryRadiation",
        "InternalCapsule",
        "ExternalCapsule",
        "OpticRadiation",
    ],
}

name_for_cid[7703] = "AssociationFiber"
cid_concepts[7703] = {
    "DCM": [
        "SuperiorLongitudinalFasciculusIII",
        "SuperiorLongitudinalFasciculusII",
        "SuperiorLongitudinalFasciculusI",
    ],
    "NEU": ["ArcuateFasciculus", "SuperiorLongitudinalFasciculus"],
    "SCT": [
        "SuperiorFrontoOccipitalFasciculus",
        "InferiorLongitudinalFasciculus",
        "InferiorFrontoOccipitalFasciculus",
        "UncinateFasciculus",
        "VerticalOccipitalFasciculus",
    ],
}

name_for_cid[7704] = "LimbicSystemTract"
cid_concepts[7704] = {"NEU": ["StriaTerminalis"], "SCT": ["Fornix", "Cingulum"]}

name_for_cid[7705] = "CommissuralFiber"
cid_concepts[7705] = {
    "SCT": [
        "ForcepsMinor",
        "GenuOfCorpusCallosum",
        "SpleniumOfCorpusCallosum",
        "AnteriorCommissure",
        "ForcepsMajor",
        "PosteriorCommissure",
        "CorpusCallosum",
        "HabenularCommissure",
        "TapetumOfCorpusCallosum",
    ]
}

name_for_cid[7706] = "CranialNerve"
cid_concepts[7706] = {
    "SCT": [
        "AbducensNerve",
        "AccessoryNerve",
        "TrochlearNerve",
        "FacialNerve",
        "GlossopharyngealNerve",
        "TrigeminalNerve",
        "VestibulocochlearNerve",
        "VagusNerve",
        "HypoglossalNerve",
        "OculomotorNerve",
        "OlfactoryTract",
        "OpticNerve",
    ]
}

name_for_cid[7707] = "SpinalCordFiber"
cid_concepts[7707] = {
    "SCT": ["LateralFuniculus", "DorsalFuniculus", "VentralFuniculus"]
}

name_for_cid[7710] = "TractographyAnatomicSite"
cid_concepts[7710] = {
    "DCM": [
        "PerilesionalWhiteMatter",
        "SuperiorLongitudinalFasciculusIII",
        "SuperiorLongitudinalFasciculusII",
        "SuperiorLongitudinalFasciculusI",
    ],
    "NEU": [
        "SuperiorLongitudinalFasciculus",
        "SuperiorThalamicRadiation",
        "InferiorThalamicRadiation",
        "PosteriorThalamicRadiation",
        "StriaTerminalis",
        "InferiorOpticRadiationMeyerLoop",
        "CorticospinalTract",
        "CorticobulbarTract",
        "ArcuateFasciculus",
        "SuperiorOpticRadiationBaumLoop",
        "AnteriorThalamicRadiation",
    ],
    "SCT": [
        "SpinalCordWhiteMatter",
        "ForcepsMinor",
        "LateralFuniculus",
        "SuperiorFrontoOccipitalFasciculus",
        "InferiorLongitudinalFasciculus",
        "SkeletalMuscle",
        "MiddleCerebellarPeduncle",
        "Phantom",
        "AbducensNerve",
        "GenuOfCorpusCallosum",
        "ForcepsMajor",
        "InferiorFrontoOccipitalFasciculus",
        "CorpusCallosum",
        "VagusNerve",
        "VentralFuniculus",
        "OpticNerve",
        "ExternalCapsule",
        "MedialLongitudinalFasciculus",
        "AnteriorCommissure",
        "CerebralWhiteMatter",
        "PeripheralNerve",
        "OlfactoryTract",
        "MedialLemniscus",
        "AccessoryNerve",
        "InferiorCerebellarPeduncle",
        "FacialNerve",
        "GlossopharyngealNerve",
        "WhiteMatterOfBrainAndSpinalCord",
        "OculomotorNerve",
        "LateralLemniscus",
        "TapetumOfCorpusCallosum",
        "CorticospinalTractInBrainstem",
        "PosteriorCommissure",
        "SuperiorCerebellarPeduncle",
        "UncinateFasciculus",
        "HabenularCommissure",
        "CardiacMuscle",
        "AuditoryRadiation",
        "SpleniumOfCorpusCallosum",
        "VestibulocochlearNerve",
        "CerebellarWhiteMatter",
        "Fornix",
        "Cingulum",
        "InternalCapsule",
        "TrigeminalNerve",
        "HypoglossalNerve",
        "VerticalOccipitalFasciculus",
        "TrochlearNerve",
        "DorsalFuniculus",
        "OpticRadiation",
    ],
}

name_for_cid[8101] = "ContainerType"
cid_concepts[8101] = {
    "SCT": [
        "SpecimenContainer",
        "TissueMicroarrayCassette",
        "ElectronMicroscopyGrid",
        "TissueCassette",
        "MicroscopeSlide",
        "SpecimenVial",
        "SpecimenWell",
    ]
}

name_for_cid[8102] = "ContainerComponentType"
cid_concepts[8102] = {
    "SCT": [
        "MicroscopeSlideCoverSlip",
        "SpecimenContainer",
        "TissueMicroarrayCassette",
        "ElectronMicroscopyGrid",
        "TissueCassette",
        "MicroscopeSlideMountingMedia",
        "SpecimenContainerLid",
        "MicroscopeSlide",
        "SpecimenVial",
        "SpecimenWell",
    ]
}

name_for_cid[8103] = "AnatomicPathologySpecimenType"
cid_concepts[8103] = {
    "SCT": [
        "SpecimenFromBreastObtainedByExcision",
        "SmearSample",
        "Aspirate",
        "SpecimenFromBreastObtainedByTotalMastectomy",
        "SegmentectomyBreastSample",
        "BreastTruCutBiopsySample",
        "BodySubstanceSample",
        "TissueSection",
        "SpecimenFromBreastObtainedByImageGuidedCoreBiopsy",
        "TissueSpot",
        "LumpectomyBreastSample",
        "LiquidBasedCytologicMaterial",
        "BreastDuctSample",
        "SpecimenFromBreastStereotacticallyGuidedCoreNeedleBiopsy",
        "TissueSpecimen",
        "GeneticSample",
        "SpecimenFromBreastObtainedByCoreNeedleBiopsy",
        "CoreSampleOfTissueBlock",
        "Slide",
        "BodyFluidSample",
        "FrozenSectionBreastSample",
        "TouchPreparationCytologicMaterial",
        "EntireBody",
        "SpecimenFromBreastByIncisionalBiopsyOfBreastMass",
        "GrossSpecimen",
    ]
}

name_for_cid[8104] = "BreastTissueSpecimenType"
cid_concepts[8104] = {
    "SCT": [
        "SpecimenFromBreastObtainedByExcision",
        "SpecimenFromBreastObtainedByTotalMastectomy",
        "SpecimenFromBreastObtainedByCoreNeedleBiopsy",
        "SegmentectomyBreastSample",
        "LumpectomyBreastSample",
        "BreastTruCutBiopsySample",
        "SpecimenFromBreastByIncisionalBiopsyOfBreastMass",
        "BreastDuctSample",
        "FrozenSectionBreastSample",
        "SpecimenFromBreastStereotacticallyGuidedCoreNeedleBiopsy",
        "SpecimenFromBreastObtainedByImageGuidedCoreBiopsy",
    ]
}

name_for_cid[8109] = "SpecimenCollectionProcedure"
cid_concepts[8109] = {
    "SCT": [
        "Scraping",
        "Venipuncture",
        "TakingOfSwab",
        "SpecimenCollection",
        "Excision",
        "Aspiration",
        "RemovalOfDevice",
        "HarvestingOfTissue",
        "Biopsy",
    ]
}

name_for_cid[8110] = "SpecimenSamplingProcedure"
cid_concepts[8110] = {
    "DCM": [
        "DissectionWithEntireSpecimenSubmission",
        "DissectionWithRepresentativeSectionsSubmission",
    ],
    "SCT": [
        "CoreSampling",
        "SmearProcedure",
        "BlockSectioning",
        "BlockSurfaceRecut",
        "BlockStepSectioning",
        "Dissection",
        "LaserMicrodissection",
        "TouchPreparation",
    ],
}

name_for_cid[8111] = "SpecimenPreparationProcedure"
cid_concepts[8111] = {
    "DCM": ["SpecimenStorage"],
    "SCT": [
        "Staining",
        "SpecimenProcessing",
        "SpecimenCollection",
        "SamplingOfTissueSpecimen",
        "SpecimenReceiving",
    ],
}

name_for_cid[8112] = "SpecimenStain"
cid_concepts[8112] = {
    "SCT": [
        "MayGrunwaldGiemsaStain",
        "FastBlueBBSaltStain",
        "EriochromeBlueBlackSEStain",
        "CarmoisineAStain",
        "FieldStain",
        "BrazilinStain",
        "ThioflavineTStain",
        "AlkaliBlue5B4BStain",
        "GramStain",
        "AlizarinYellowRStain",
        "NaphtholASDChloracetateEsteraseStain",
        "SudanIIStain",
        "MethylGreenStain",
        "FastRedITRStain",
        "WaxolineBlueStain",
        "WaterSolubleEosinStain",
        "PonceauSStain",
        "SilverNitrateStain",
        "Chromotrope2RStain",
        "RoseBengalStain",
        "SudanStain",
        "VictoriaBlueBStain",
        "PeriodicAcidSchiffStain",
        "OilRedOStain",
        "ButyrateEsteraseStain",
        "FouchetStain",
        "MethylRedStain",
        "AzocarmineGGXStain",
        "AtebrinFSStain",
        "PeroxidaseStain",
        "FluorexonStain",
        "NeutralRedStain",
        "MeldolaBlueStain",
        "QuinacrineFluorescentStain",
        "FluoresceinSodiumStain",
        "ErythrosinBStain",
        "BismarkBrownRStain",
        "AlbertStain",
        "CarminicAcidStain",
        "AlizarinYellowGGStain",
        "FatRed7BStain",
        "AlizarinRedSStain",
        "ModifiedTrichromeStain",
        "ThionineStain",
        "VictoriaBlue4RStain",
        "NaphthaleneBlack12BStain",
        "FastGreenFCFStain",
        "MethyleneVioletStainBernthsen",
        "SaffronStain",
        "SpiritSolubleEosinStain",
        "TrypanBlueStain",
        "LeishmanStain",
        "BrilliantCresylBlueStain",
        "OrangeGStain",
        "PontamineSkyBlue6BXStain",
        "SpiritSolubleAnilineBlueStain",
        "KenacidBlueRStain",
        "VerhoeffHematoxylinStain",
        "PyronineBStain",
        "HemateinStain",
        "PontamineSkyBlue5BXStain",
        "MethylVioletStain",
        "AuramineGStain",
        "NonspecificEsteraseStain",
        "FluorescentStain",
        "PhloxinBStain",
        "SunsetYellowFCFStain",
        "BismarkBrownYStain",
        "TerminalDeoxynucleotidylTransferaseStain",
        "SporeStain",
        "ChromicAcidStain",
        "SolochromeAzurineBSStain",
        "FluoresceinStain",
        "IndophenolFromNaphtholStain",
        "MethylOrangeStain",
        "JennerGiemsaStain",
        "NitrazineYellowStain",
        "FastBlueBSaltStain",
        "AcidPhosphataseStain",
        "CresylVioletStain",
        "ProtargolSStain",
        "MethylViolet6BStain",
        "InsolubleBerlinBlueStain",
        "AlizarinCyanineGreenStain",
        "NaphtholGreenBStain",
        "VitalNewRedStain",
        "MetanilYellowStain",
        "MucicarmineStain",
        "ProcionBrilliantBlueMRSStain",
        "AldehydeFuchsinStain",
        "PageBlue83Stain",
        "BrilliantOrangeStain",
        "FeulgenReactionStain",
        "DiamondBlackStain",
        "JanusGreenBStain",
        "GiemsaStain",
        "PrussianBlueStain",
        "AlcoholSolubleNigrosineStain",
        "MethylBlueStain",
        "CarbolFuchsinStain",
        "HematoxylinStain",
        "WaterSolubleAnthraceneBrownStain",
        "FastGarnetGBCSaltStain",
        "BenzoFastScarletStain",
        "NeutrophilAlkalinePhosphataseStain",
        "AcriflavineStain",
        "AcridineOrangeStain",
        "EthylVioletStain",
        "CurcuminStain",
        "SolubleBerlinBlueStain",
        "MalloryBleachStain",
        "FastSulfonBlackFStain",
        "ErythrosinYStain",
        "FastVioletBSaltStain",
        "FuchsinAcidStain",
        "LuxolFastBlueStain",
        "LissamineRhodamineStain",
        "ImmunofluorescentStain",
        "SolochromeCyanineRStain",
        "BetaGlucuronidaseStain",
        "AlizarinBlueSStain",
        "CarmineStain",
        "GallocyanineStain",
        "NightBlueStain",
        "BrilliantCroceinStain",
        "WaysonStain",
        "CrystalPonceauStain",
        "XyleneCyanolFFStain",
        "AzureAStain",
        "SolochromeDarkBlueStain",
        "WaterSolubleAnilineBlueStain",
        "BauerChromicAcidLeucofuchsinStain",
        "TropaeolinOStain",
        "ChrysoidineRStain",
        "ChloroacetateEsteraseStain",
        "AlkalinePhosphataseStain",
        "SiriusRedF3BStain",
        "Permethrin",
        "SudanBlueStain",
        "ColloidalIronStain",
        "MartiusYellowStain",
        "MyeloperoxidaseStain",
        "ChromeAzurolSStain",
        "LissamineFastRedBStain",
        "AlcianBlue8GXStain",
        "LissamineFastYellowStain",
        "CongoRedStain",
        "ToluidineBlueStain",
        "CresylEchtVioletStain",
        "SudanBlackBStain",
        "ZiehlNeelsenStain",
        "AzoBlackStain",
        "RhodamineStain",
        "NileBlueStain",
        "CelestineBlueBStain",
        "AzureBStain",
        "LissamineGreenBStain",
        "TitanYellowStain",
        "CoriphosphineStain",
        "MethylViolet2BStain",
        "SafraninStain",
        "SudanIIIStain",
        "LacmoidStain",
        "RhodamineBStain",
        "AlkaliBlue6BStain",
        "AuramineStain",
        "SudanIVStain",
        "Ponceau3RStain",
        "FastBlueRRSaltStain",
        "IndiaInkStain",
        "RosolicAcidSodiumSaltStain",
        "VanGiesonStain",
        "TropaeolinOOStain",
        "AzophloxinStain",
        "BlueShadeEosinStain",
        "WaterSolubleNigrosineStain",
        "OrceinStain",
        "TrichromeStain",
        "WrightStain",
        "AzorubinSStain",
        "HanselStain",
        "AzureCStain",
        "NewFuchsinStain",
        "FlagellarStain",
        "PageBlueG90Stain",
        "PotassiumHydroxideStain",
        "RomanowskyStain",
        "LightGreenSFStain",
        "ErieGarnetStain",
        "MethyleneBlueStain",
        "MethylGreenPyroninStain",
        "DurazolRedStain",
        "MethyleneVioletStain",
        "FastRedBSaltStain",
        "NaphtholYellowSStain",
        "ChrysoidineYStain",
        "SilverStain",
        "AnilineBlueStain",
        "TartrazineStain",
        "PyronineGStain",
        "BrilliantYellowStain",
        "PatentBlueVSodiumSaltStain",
        "MalachiteGreenStain",
        "IndigoCarmine",
        "AuramineOStain",
        "OrangeIIStain",
        "DibromofluoresceinStain",
        "EvansBlueStain",
        "SudanBlackStain",
        "NuclearFastRedStain",
        "FuchsinBasicStain",
        "BiebrichScarletStain",
        "PonceauXylidineStain",
        "ThioflavineSStain",
        "PhosphotungsticAcidHematoxylinStain",
        "FastRedTRSaltStain",
        "SolochromeBlack6BStain",
    ]
}

name_for_cid[8113] = "SpecimenPreparationStep"
cid_concepts[8113] = {
    "SCT": [
        "SpecimenClearing",
        "SpecimenSteamHeating",
        "SpecimenFreezing",
        "SpecimenMicrowaveHeating",
        "ProteaseDigestionOfTissueSpecimen",
        "SpecimenDehydration",
    ]
}

name_for_cid[8114] = "SpecimenFixative"
cid_concepts[8114] = {
    "SCT": [
        "Chloroform",
        "OsmiumTetroxide",
        "CarnoyFluid",
        "HellyFluid",
        "MichelMedium",
        "BouinFluid",
        "ChromiumTrioxide",
        "PotassiumDichromate",
        "NeutralBufferedFormalin",
        "MercuricChloride",
        "FormolSublimate",
        "AceticAcid",
        "PicricAcid",
        "Formalin",
        "Methanol",
        "Formaldehyde",
        "Paraformaldehyde",
        "Ethanol",
        "ZenkerFluid",
    ]
}

name_for_cid[8115] = "SpecimenEmbeddingMedia"
cid_concepts[8115] = {
    "SCT": [
        "EpoxyResin",
        "AcrylicResin",
        "TissueFreezingMedium",
        "ParaffinWax",
        "Plastic",
        "Agar",
    ]
}

name_for_cid[8120] = "WholeSlideMicroscopyImageReferencedImagePurposeofReference"
cid_concepts[8120] = {
    "DCM": [
        "ImagedContainerLabel",
        "Localizer",
        "SameAcquisitionAtDifferentFocalDepth",
        "SameAcquisitionAtDifferentSpectralBand",
        "SameAcquisitionAtLowerResolution",
        "SameAcquisitionAtHigherResolution",
    ]
}

name_for_cid[8121] = "MicroscopyLensType"
cid_concepts[8121] = {
    "SCT": ["SlideOverviewLens", "OilImmersionLens", "HighPowerNonImmersionLens"]
}

name_for_cid[8122] = "MicroscopyIlluminatorandSensorColor"
cid_concepts[8122] = {
    "SCT": ["Infrared", "Ultraviolet", "Red", "FullSpectrum", "Blue", "Green"]
}

name_for_cid[8123] = "MicroscopyIlluminationMethod"
cid_concepts[8123] = {
    "DCM": [
        "DifferentialInterferenceContrast",
        "TotalInternalReflectionFluorescence",
        "TransmissionIllumination",
        "ReflectionIllumination",
        "EpifluorescenceIllumination",
        "BrightfieldIllumination",
        "DarkfieldIllumination",
        "ObliqueIllumination",
        "PhaseContrastIllumination",
    ]
}

name_for_cid[8124] = "MicroscopyFilter"
cid_concepts[8124] = {
    "DCM": ["NoFilter"],
    "SCT": [
        "BlueOpticalFilter",
        "NomarskiPrism",
        "GreenOpticalFilter",
        "HoffmanModulator",
        "CondenserAnnulus",
        "PolarizingOpticalFilter",
        "VioletOpticalFilter",
        "InfraredOpticalFilter",
        "DarkfieldStop",
        "RheinbergFilter",
        "UltravioletOpticalFilter",
        "DichroicBeamsplitter",
        "PhaseContrastPlate",
        "RedOpticalFilter",
        "DeSénarmontCompensator",
    ],
}

name_for_cid[8125] = "MicroscopyIlluminatorType"
cid_concepts[8125] = {
    "SCT": [
        "TungstenHalogenLamp",
        "LightEmittingDiode",
        "LaserDevice",
        "MercuryArcLamp",
        "XenonArcLamp",
    ]
}

name_for_cid[8130] = "StainingProtocol"
cid_concepts[8130] = {
    "SCT": ["HematoxylinAndEosinStainMethod", "Staining", "AcidFastStainMethod"]
}

name_for_cid[8131] = "PathologyImagingProtocol"
cid_concepts[8131] = {
    "DCM": [
        "WSI40XRGB",
        "PeriOperativePhotographicImaging",
        "SlideMicroscopyPathologyImaging",
        "WholeSlideImaging",
        "WSI20XRGB",
    ],
    "SCT": [
        "LightMicroscopy",
        "PhotographyOfPatient",
        "GrossPathologyRequest",
        "Histology",
        "Immunohistochemistry",
        "Photomicrography",
        "Histopathology",
        "PhotographyOfGrossOrgan",
        "Microscopy",
    ],
}

name_for_cid[8132] = "MagnificationSelection"
cid_concepts[8132] = {"DCM": ["_10X", "_20X", "_40X", "_5X"]}

name_for_cid[8133] = "TissueSelection"
cid_concepts[8133] = {
    "DCM": [
        "NominalEmptyTileSuppression",
        "HighThresholdEmptyTileSuppression",
        "NoEmptyTileSuppression",
    ]
}

name_for_cid[8134] = "AnatomicStructure"
cid_concepts[8134] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": [
        "ScapularSupraspinatusFossa",
        "ScapularInfraspinatusFossa",
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "Lobar",
        "BoneAndSoftTissue",
        "PerilesionalWhiteMatter",
        "SuperiorLongitudinalFasciculusIII",
        "SuperiorLongitudinalFasciculusII",
        "SuperiorLongitudinalFasciculusI",
        "SkinOfParaspinalAreaOfTheNeck",
        "SkinOfLowerParaspinalRegion",
        "SkinOfMidParaspinalRegion",
        "SkinOfUpperParaspinalRegion",
        "SkinOfParaspinalAreaOfTheSuperiorBack",
        "SkinOfInferiorPosteriorSurfaceOfThePinna",
        "SkinOfInferiorHelixOfEar",
        "SkinOfCaruncleOfEye",
        "SkinOfAnteriorHelixOfEar",
        "SkinOfLateralPartOfDorsumOfFoot",
        "SkinOfLowerEyelidMargin",
        "SkinOfLowerAntihelixOfEar",
        "SkinOfPosteriorHelixOfEar",
        "SkinOfParanasalCheek",
        "SkinOfMedialPartOfDorsumOfFoot",
        "SkinOfInfraalarGroove",
        "SkinOfSuperiorPosteriorHelixOfEar",
        "SkinOfSuperiorAntihelixOfEar",
        "SkinOfSoleOfForefoot",
        "SkinOfPosteriorLobuleOfTheEar",
        "SkinOfUpperEyelidMargin",
        "SkinOfUpperAntihelixOfEar",
        "SkinOfSuperiorPosteriorSurfaceOfThePinna",
        "SkinOfMidBack",
        "EmbryonicKidney",
        "RibScaleneTubercle",
        "VertebralIntervertebralNotch",
        "SubscapularFossa",
        "ScapularSpine",
    ],
    "FMA": [
        "ArcuateFasciculus",
        "SkinOfDorsalPartOfFourthToe",
        "SkinOfDorsalPartOfFifthToe",
        "SkinOfVermilionProperOfLowerLip",
        "SkinOfDorsalPartOfGreatToe",
        "SkinOfDorsalPartOfSecondToe",
        "SkinOfDorsalPartOfThirdToe",
        "SkinOfVermilionProperOfUpperLip",
        "SupramammaryLymphNode",
        "MucosaOfUvula",
        "MucosaOfPalatoglossalArch",
        "SkinOfPlantarPartOfThirdToe",
        "SkinOfPlantarPartOfFourthToe",
        "SkinOfPalmarPartOfLittleFinger",
        "SkinOfPlantarPartOfFifthToe",
        "LumenOfGallbladder",
        "SkinOfPlantarPartOfSecondToe",
        "SkinOfPalmarPartOfRingFinger",
        "SkinOfPalmarPartOfIndexFinger",
        "SkinOfPalmarPartOfMiddleFinger",
        "SkinOfPlantarPartOfGreatToe",
        "SkinOfPalmarPartOfThumb",
        "SkinOfDorsalPartOfLittleFinger",
        "MucosaOfPharynx",
        "SkinOfDorsalPartOfRingFinger",
        "SkinOfDorsalPartOfIndexFinger",
        "SkinOfDorsalPartOfMiddleFinger",
        "SkinOfDorsalPartOfThumb",
        "MucosaOfPosteriorWallOfOropharynx",
        "SubcorticalGrayMatter",
        "Nasion",
        "TorusOfPharyngotympanicTube",
        "SkinOfLowerOuterQuadrantOfBreast",
        "SkinOfUpperInnerQuadrantOfBreast",
        "SkinOfLowerInnerQuadrantOfBreast",
        "SkinOfUpperOuterQuadrantOfLeftBreast",
        "WallOfGallbladder",
        "SubiliacLymphNode",
        "SkinOfBackOfTrunk",
        "MucosaOfOralSegmentOfHardPalate",
        "MucosaOfDorsumOfPharyngealPartOfTongue",
        "MucosaOfDorsumOfOralPartOfTongue",
        "SkinOfDorsumOfNose",
    ],
    "NCIt": ["MouseMammaryFatPad"],
    "NEU": [
        "StriaTerminalis",
        "AnteriorThalamicRadiation",
        "CorticospinalTract",
        "CorticobulbarTract",
        "ArcuateFasciculus",
        "SuperiorLongitudinalFasciculus",
        "SuperiorThalamicRadiation",
        "InferiorThalamicRadiation",
        "PosteriorThalamicRadiation",
        "InferiorOpticRadiationMeyerLoop",
        "SuperiorOpticRadiationBaumLoop",
    ],
    "SCT": [
        "LeftHypochondriacRegion",
        "SternocleidomastoidMuscle",
        "ProfundaFemorisVein",
        "MucosaOfFloorOfMouth",
        "TruncusCoeliacus",
        "C3C4Disc",
        "InferiorLongitudinalFasciculus",
        "LowRightAtrium",
        "PelvicCavityTrue",
        "WristJoint",
        "IntermediateArteryRamus",
        "LeftVentricleInflow",
        "GenicularArtery",
        "FontanelOfSkull",
        "SigmoidColon",
        "SuperiorGlutealLymphNode",
        "VentralFuniculus",
        "SkinOfAnteriorSurfaceOfUpperArm",
        "RightFemoralArtery",
        "Intracranial",
        "MedialLacunarLymphNode",
        "JawRegion",
        "SkinOfAnteriorSurfaceOfForearm",
        "FetlockOfForelimb",
        "OropharyngealTonsil",
        "Glottis",
        "LobarArtery",
        "SkinOfPosteriorSurfaceOfForearm",
        "ParotidLymphNode",
        "SkinOfChin",
        "ThighPerforator",
        "LowerParatrachealIncludingAzygousLymphNode",
        "PrececalLymphNode",
        "Pituitary",
        "MidRightCoronaryArtery",
        "ChestCavity",
        "RadiusAndUlna",
        "ExternalIliacVein",
        "UpperLimb",
        "ParietalBone",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "ParatrachealLymphNode",
        "RightUpperQuadrantOfAbdomen",
        "Eye",
        "SuperficialPalmarArch",
        "UpperGastroIntestinalTract",
        "Prostate",
        "PeripheralNerveMyelinSheath",
        "FacialBones",
        "Back",
        "SoftTissue",
        "Duodenum",
        "_1stMarginalCoronaryArtery",
        "SkinOfLowerLip",
        "DeepParotidLymphNode",
        "LateralJugularLymphNode",
        "Larynx",
        "PeronealVein",
        "MandibularIncisorTeeth",
        "PosteriorMediastinalLymphNode",
        "Lymph",
        "SkinOfScalp",
        "FemoralArtery",
        "LymphNodeOfLesserCurvatureOfStomach",
        "MiddleDeepCervicalLymphNode",
        "Ovary",
        "SupraclavicularLymphNode",
        "DistalHumerus",
        "Urethra",
        "Head",
        "MitralValve",
        "AdrenalGland",
        "SkinOfEarLobule",
        "PrevesicularLymphNode",
        "Calyx",
        "LateralFuniculus",
        "PosteriorMedialTributary",
        "GlutealLymphNode",
        "PulmonaryMetastasis",
        "SkeletalMuscle",
        "Submental",
        "UpperAbdomen",
        "NucleusAccumbens",
        "SkinOfPoplitealFossa",
        "PoplitealFossa",
        "PulmonaryArteryConduit",
        "InternalCarotidArteryC5Segment",
        "DeepPoplitealLymphNode",
        "SkinOfUmbilicus",
        "LevatoresCostarumMuscles",
        "FrontalLobe",
        "Adventitia",
        "IntermediateCommonIliacLymphNode",
        "Hypopharynx",
        "RightCoronaryArteryOstium",
        "MarginalCoronaryArtery",
        "SkinOfBackOfUpperThoracicRegion",
        "Ulna",
        "HepaticVein",
        "InternalMammaryArtery",
        "DistalUlna",
        "InferiorAuricularLymphNode",
        "SkinOfThenarRegionOfPalm",
        "RenalHilarLymphNode",
        "HilarLymphNode",
        "BronchialArtery",
        "AccessoryNerve",
        "CouinaudHepaticSegmentIII",
        "ExternalCarotidArtery",
        "UterineParacervicalLymphNode",
        "InterlobarLymphNodeOfTheLung",
        "SkinOfHeel",
        "Tumor",
        "SystemicArtery",
        "AbdominopelvicCavity",
        "TapetumOfCorpusCallosum",
        "SubaorticCommonIliacLymphNode",
        "VomerBone",
        "MucosaOfUpperLip",
        "Digit",
        "SkinOfClitoris",
        "UterusAndFallopianTubes",
        "Arachnoid",
        "SkinOfAnteriorSurfaceOfKnee",
        "TarsalJoint",
        "ShaftOfTibia",
        "LingualArtery",
        "PararectalLymphNode",
        "PeripancreaticLymphNode",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "DeltoidMuscle",
        "CerebralFornix",
        "Penis",
        "Baffle",
        "SkinOfLowerBack",
        "TransverseProcessOrVertebra",
        "Parathyroid",
        "PostvesicularLymphNode",
        "PosteriorTriangleCervicalLymphNode",
        "SkinOfTipOfNose",
        "Scapula",
        "Nasopharyngeal",
        "InferiorGlutealLymphNode",
        "Kidney",
        "CardiophrenicAngleLymphNode",
        "LobarLymphNodeOfTheLung",
        "OmentalBursa",
        "InternalCarotidArtery",
        "L1Vertebra",
        "RightPosteriorAVCoronaryArtery",
        "CouinaudHepaticSegmentV",
        "ProximalRadius",
        "RenalPelvis",
        "AbducensNerve",
        "Thrombus",
        "VenousNetwork",
        "SubcostalMuscle",
        "CommonAtrium",
        "BoneStructureOfPelvis",
        "SkinOfExternalAuditoryCanal",
        "Ilium",
        "InternalCarotidArteryC6Segment",
        "SkinOfLateralPortionOfNeck",
        "AlveolarProcessOfMandible",
        "AnteriorTibialArtery",
        "CranialNerve",
        "SkinOfAbdomen",
        "ConnectiveTissue",
        "CommonCarotidArtery",
        "InnermostIntercostalMuscles",
        "_3rdMarginalCoronaryArtery",
        "UpperDeepCervicalLymphNode",
        "UrinaryTract",
        "SuperficialCervicalLymphNode",
        "MastoidBone",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "CouinaudHepaticSegmentVI",
        "SkinOfHead",
        "Thigh",
        "SkinOfLowerExtremity",
        "SkinOfRootOfPenis",
        "Trachea",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "BodyConduit",
        "Thalamus",
        "AnteriorMediastinalLymphNode",
        "SkinOfMedialAspectOfAnkle",
        "LateralHighRightAtrium",
        "RightVentricleInflow",
        "LeftLowerQuadrantOfAbdomen",
        "CardiacMuscle",
        "SpinalCerebrospinalFluidPathway",
        "SesamoidBonesOfFoot",
        "SkinOfButtock",
        "C2C3Disc",
        "AbdominalWallMuscle",
        "PharyngealRecess",
        "SixthLumbarArtery",
        "LacrimalCaruncle",
        "Omentum",
        "LateralCanthus",
        "SubmandibularGland",
        "SkinOfEyebrow",
        "SkinOfHand",
        "CarotidBifurcation",
        "SuperiorArticularFacetOfAxis",
        "T3Vertebra",
        "InnominateArtery",
        "Cecum",
        "AcromionProcessOfScapula",
        "Extremity",
        "BoneStructureOfHeadAndOrNeck",
        "FacialArtery",
        "EpigastricRegion",
        "SkinOfAlaNasi",
        "VerticalOccipitalFasciculus",
        "PosteriorCommissureOfLabiumMajorum",
        "SkinOfUpperLip",
        "CervicoThoracicSpine",
        "PectoralGirdle",
        "RightLumbarRegion",
        "TeresMinorMuscle",
        "PostauricularLymphNode",
        "LymphNodeRingOfCardiaOfStomach",
        "ParavaginalLymphNode",
        "KneeJoint",
        "SkinOfPosteriorSurfaceOfThigh",
        "SkinOfNose",
        "SkinOfVertexOfScalp",
        "CommonFemoralVein",
        "DeepIntraparotidLymphNode",
        "SubstantiaNigra",
        "SkinOfPalmarAreaOfWrist",
        "UpperInnerQuadrantOfBreast",
        "C5Vertebra",
        "InferiorPancreaticoduodenalLymphNode",
        "LymphNodeOfTrunk",
        "ThoracicAorta",
        "RightInguinalRegion",
        "Conjunctiva",
        "Vitreous",
        "PyriformSinus",
        "AtlantalAxialJoint",
        "RightLowerQuadrantOfAbdomen",
        "AnteriorTibialLymphNode",
        "InguinalLymphNode",
        "CockettPerforatingVein",
        "RightMainBronchus",
        "RightAtrioventricularOstium",
        "ExternalMammaryLymphNode",
        "DeepLymphNode",
        "MedianRetropharyngealLymphNode",
        "AVGrooveContinuationOfCircumflexArtery",
        "InteriliacLymphNode",
        "ProximalFemur",
        "SubsegmentalLymphNodeOfTheLung",
        "NailUnitOfGreatToe",
        "CostalGroove",
        "Thumb",
        "GreatCardiacVein",
        "SuperiorPancreaticoduodenalLymphNode",
        "Hindfoot",
        "SuprapyloricLymphNode",
        "PasternOfForefoot",
        "MiddleLobeOfRightLung",
        "SkinOfAnus",
        "SternalSkin",
        "BaseOfTongue",
        "T12Vertebra",
        "VestibulocochlearNerve",
        "C3Vertebra",
        "ShaftOfUlna",
        "L2L3Disc",
        "C5C6Disc",
        "_2ndLeftPosterolateralCoronaryArtery",
        "MiddleCerebralArteryM2Segment",
        "LacrimalBone",
        "BasilicVein",
        "HunterianPerforatingVein",
        "SkinOfScrotum",
        "SuperiorLateralInguinalLymphNode",
        "DorsalFuniculus",
        "LeftAnteriorDivisionOfLeftBranchAtrioventricularBundle",
        "SkinOfUpperTrunk",
        "ProfundaFemorisArtery",
        "Pharynx",
        "LigamentumArteriosum",
        "Nasopharynx",
        "C6C7Disc",
        "SympatheticTrunk",
        "SkinOfOccipitalRegion",
        "OvarianVein",
        "BrachialPlexus",
        "AnteriorAuricularLymphNode",
        "IntermediateExternalIliacLymphNode",
        "OpticNerveHead",
        "DistalFemur",
        "ApexOfRightVentricle",
        "C1Vertebra",
        "NavicularOfHindfoot",
        "LymphNodeOfHead",
        "LateralCalfPerforator",
        "InferiorInguinalLymphNode",
        "IntrapulmonaryLymphNode",
        "Phantom",
        "RetropharyngealLymphNode",
        "NailUnitOfFourthToe",
        "SkinOfMedialCanthus",
        "FourthLumbarArtery",
        "ParasternalLymphNode",
        "ThirdLumbarArtery",
        "LimbicLobe",
        "SkinOfChest",
        "FemaleExternalUrethralOrifice",
        "PrimitiveAorta",
        "SkinOfAxilla",
        "LumenOfBloodVessel",
        "CornealEndothelium",
        "HighestMediastinalLymphNode",
        "AntecubitalVein",
        "SpinalCordGrayMatter",
        "Mandible",
        "Parasternal",
        "LeftMainBronchus",
        "Lens",
        "SystemicVein",
        "OccipitalLobe",
        "Trunk",
        "MandibularDentalArch",
        "DelphianLymphNode",
        "C7T1Disc",
        "SuperiorRectalLymphNode",
        "Pelvis",
        "SubclavianArtery",
        "SkinOfInguinalRegion",
        "Amygdala",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OpticChiasm",
        "OphthalmicArtery",
        "SymphysisPubis",
        "PancreaticDuct",
        "Nose",
        "EpitrochlearLymphNode",
        "Hand",
        "AxillaryLymphNode",
        "PectoralisMajorMuscle",
        "DeepAnteriorCervicalLymphNode",
        "OvarianArtery",
        "Hippocampus",
        "TonsilAndAdenoid",
        "ThoracicInlet",
        "LacrimalGland",
        "IliacCrest",
        "LacunarLymphNode",
        "SuperficialFemoralVein",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "SuperiorLeftPulmonaryVein",
        "PosteriorTibialLymphNode",
        "PancreaticosplenicLymphNode",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "LymphNodeOfAortopulmonaryWindow",
        "CeliacLymphNode",
        "BoneTissue",
        "Skull",
        "InfraclavicularLymphNode",
        "SubcarinalLymphNode",
        "CerebralHemisphere",
        "ParavesicularLymphNode",
        "Metastasis",
        "OpticRadiation",
        "SinoAtrialNode",
        "SacralLymphNode",
        "SkinOfNuchalRegion",
        "T7Vertebra",
        "EndoNasopharyngeal",
        "EsophagusStomachAndDuodenum",
        "Breast",
        "AxillaryFascia",
        "PoplitealVein",
        "SkinOfPosteriorSurfaceOfElbow",
        "TricuspidValve",
        "VulvalVestibule",
        "PiaMater",
        "RightAtrium",
        "VascularSclerosis",
        "SkinOfLateralPartOfHeel",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "InnerEar",
        "Coccyx",
        "BodyOfSternum",
        "AngleOfRib",
        "DescendingAorta",
        "OccipitalLymphNode",
        "MaxillarySinus",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "OccipitalBone",
        "MalleolarStructureOfTibia",
        "L2Vertebra",
        "Wing",
        "Spine",
        "SuperficialInguinalLymphNode",
        "LacrimalSac",
        "Muscle",
        "WhiteMatter",
        "Acetabulum",
        "LymphNodeOfStomach",
        "SkinOfExternalGenitalia",
        "RetromolarTrigone",
        "IntramammaryLymphNode",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "HepaticLymphNode",
        "SuperficialIntraparotidLymphNode",
        "SkinOfForehead",
        "JuxtaposedAtrialAppendage",
        "MucosaOfMaxillaryGingiva",
        "CarotidBody",
        "Jejunum",
        "ChestAbdomenAndPelvis",
        "RightHepaticVein",
        "Scalp",
        "GastrocnemiusVein",
        "TracheobronchialLymphNodeLocatedNearCarina",
        "DistalRadius",
        "OrbitalStructure",
        "LeftMainBranchOfPortalVein",
        "AorticLymphNode",
        "SuperficialLymphNode",
        "TrigeminalNerve",
        "TibialLymphNode",
        "SkinOfPhiltrum",
        "Sclera",
        "BloodVessel",
        "ArchOfVertebra",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "LiverAndBiliaryStructure",
        "_1stDiagonalCoronaryArtery",
        "Uvula",
        "Perineum",
        "InteratrialSeptum",
        "Femur",
        "Rectum",
        "T10T11Disc",
        "SkinOfEpigastricArea",
        "LeftAtrium",
        "InterlobularArteryOfKidney",
        "CerebralArtery",
        "CouinaudHepaticSegmentVIII",
        "InferiorMesentericArtery",
        "IliocostalisMuscle",
        "TracheobronchialLymphNode",
        "AtlantoOccipitalJoint",
        "CervicalSpine",
        "SpinalCordWhiteMatter",
        "IliacAndOrFemoralArtery",
        "VertebralForamen",
        "SmallIntestine",
        "Oropharynx",
        "SkinOfCrusOfHelix",
        "SuperiorRightPulmonaryVein",
        "Mesothelium",
        "InferiorVenaCava",
        "IntraArticular",
        "Axilla",
        "PancreaticLymphNode",
        "ThymusGland",
        "SkinOfLowerAbdomen",
        "SkinOfAnteriorSurfaceOfThigh",
        "AtrioventricularNode",
        "ParietalLobe",
        "IliacArtery",
        "T12L1Disc",
        "DistalRightCoronaryArtery",
        "RadiusBone",
        "SkinOfLabiumMinus",
        "LateralCommonIliacLymphNode",
        "SubmandibularTriangle",
        "PectoralisMinorMuscle",
        "OpticCanal",
        "SuperficialTemporalArtery",
        "MedialLemniscus",
        "InferiorCerebellarPeduncle",
        "RightMainBranchOfPortalVein",
        "SphenoidBone",
        "AppendicularLymphNode",
        "LateralVesicularLymphNode",
        "ProximalHumerus",
        "ChestAndAbdomen",
        "Spleen",
        "C4Vertebra",
        "LowerOuterQuadrantOfBreast",
        "RetrocecalLymphNode",
        "BothEyes",
        "Mediastinum",
        "Cerebellum",
        "FirstLumbarArtery",
        "AbdominalLymphNode",
        "PoplitealArtery",
        "TibialPlateau",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "T9T10Disc",
        "DorsalScapularArtery",
        "Endometrium",
        "L3L4Disc",
        "LeftVentricle",
        "MedialExternalIliacLymphNode",
        "NailUnitOfLittleFinger",
        "LowerJugularLymphNode",
        "CostalCartilage",
        "SaphenousVein",
        "_2ndRightPosterolateralCoronaryArtery",
        "Gallbladder",
        "MandibularLymphNode",
        "RenalArtery",
        "PeritonealCavity",
        "UmbilicalCord",
        "PrepericardialLymphNode",
        "RightBranchOfAtrioventricularBundle",
        "UpperArm",
        "SkinOfPosteriorSurfaceOfLowerLeg",
        "Chest",
        "GastroduodenalArtery",
        "AzygosVein",
        "ShaftOfFemur",
        "LymphNodeOfThePulmonaryLigament",
        "Liver",
        "Rib",
        "L5S1Disc",
        "TubalTonsil",
        "AxillaryVeinLymphNode",
        "CommonFemoralArtery",
        "DeepInguinalLymphNode",
        "SuperficialPalmarVenousArch",
        "SkinOfGlansPenis",
        "UpperLimbLymphNode",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "SubpyloricLymphNode",
        "NailUnitOfMiddleFinger",
        "T7T8Disc",
        "VenaCava",
        "SkinOfInfraclavicularRegion",
        "UpperJugularLymphNode",
        "HilarArtery",
        "Talus",
        "MedialCommonIliacLymphNode",
        "LeftCoronaryArtery",
        "ThirdVentricle",
        "ApexOfLung",
        "MucosaOfTongue",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "SaphenofemoralJunction",
        "LowerDeepCervicalLymphNode",
        "FrontalBone",
        "Anus",
        "CommonIliacVein",
        "TrabeculaeCarnae",
        "PosteriorCerebralArteryP1Segment",
        "SkinOfUpperEyelid",
        "LateralLemniscus",
        "LumbarSpine",
        "ThyrocervicalTrunk",
        "PrimitivePulmonaryArtery",
        "SkinOfEar",
        "HilumOfLung",
        "SellaTurcica",
        "FibularLymphNode",
        "Scrotum",
        "InferiorLeftPulmonaryVein",
        "MaxillaryDentalArch",
        "InnominateLymphNode",
        "Intima",
        "Forefoot",
        "LowerLeg",
        "RenalVein",
        "Hair",
        "InternalIliacVein",
        "IlealVein",
        "Tarsus",
        "Skin",
        "SkinOfNipple",
        "Toe",
        "SkinOfJawline",
        "MiddleCardiacVein",
        "MiddleCerebralArtery",
        "PosteriorDescendingRightCoronaryArtery",
        "SkinOfDorsalAreaOfWrist",
        "CorpusStriatum",
        "InternalCapsule",
        "Adenohypophysis",
        "PosteriorTibialVein",
        "SuperiorPancreaticLymphNode",
        "SubmandibularLymphNode",
        "ExternalIliacLymphNode",
        "MuscleOfLowerLimb",
        "AscendingAorta",
        "LeftUpperQuadrantOfAbdomen",
        "NeckOfRib",
        "PericardiophrenicArtery",
        "SternalAngle",
        "JugularLymphNode",
        "ExternalAuditoryCanal",
        "GastroOmentalLymphNode",
        "Patella",
        "OccipitalVein",
        "CubitalLymphNode",
        "LumboSacralSpine",
        "ForcepsMinor",
        "SplenicFlexureOfColon",
        "DeepLateralCervicalLymphNode",
        "BasilarArtery",
        "NasalSinus",
        "BrainCerebrospinalFluidPathway",
        "MidCircumflexCoronaryArtery",
        "CouinaudHepaticSegmentII",
        "CentralRetinalVein",
        "PleuralEffusion",
        "CalcanealTubercle",
        "AnteriorCervicalLymphNode",
        "GenuOfCorpusCallosum",
        "SkinOfSupraclavicularRegionOfNeck",
        "GastricArtery",
        "SubscapularisMuscle",
        "ExternalEar",
        "LeftVentricleOutflowTract",
        "SegmentalLymphNodeOfTheLung",
        "Calcaneus",
        "IntercostalArtery",
        "SolealVein",
        "LateralPericardialLymphNode",
        "PeripheralNerve",
        "SubscapularAxillaryLymphNode",
        "PrefemoralLymphNode",
        "TeresMajorMuscle",
        "AnteriorHornLateralVentricle",
        "PatentDuctusArteriosus",
        "MaxillaryIncisorTeeth",
        "MesentericVein",
        "NailUnitOfIndexFinger",
        "MaleInternalGenitalia",
        "XiphoidProcessOfSternum",
        "SuperficialParotidLymphNode",
        "IliacLymphNode",
        "PalatineBone",
        "OculomotorNerve",
        "CoronaryArtery",
        "RightBranchOfHepaticArtery",
        "VertebralCanal",
        "DuraMater",
        "Flank",
        "EsophagealHiatus",
        "Maxilla",
        "CentralRetinalArtery",
        "Neurohypophysis",
        "TibiaAndFibula",
        "LymphNodeOfMesentery",
        "SkinOfLabium",
        "SegmentalArtery",
        "BuccalMucosa",
        "JuxtaintestinalLymphNode",
        "MitralRing",
        "BrachialVein",
        "T10Vertebra",
        "HepaticArtery",
        "InterstitialTissue",
        "PeronealArtery",
        "PyloricLymphNode",
        "TendonOfTodaro",
        "RightHypochondriacRegion",
        "OccipitalRegionOfScalp",
        "SuperiorMesentericVein",
        "HeadOfRib",
        "LumbarArtery",
        "LingualTonsil",
        "Cornea",
        "L4Vertebra",
        "SkinOfEyeRegion",
        "IliacVein",
        "CoronaryArteryGraft",
        "SkinOfAreola",
        "AcromioclavicularJoint",
        "UmbilicalRegion",
        "SupraspinatusMuscle",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "ChestWall",
        "SuperiorFrontoOccipitalFasciculus",
        "T3T4Disc",
        "SkinOfPrepuceOfClitoris",
        "RetrotrachealLymphNode",
        "SoftPalate",
        "T4Vertebra",
        "ExternalIntercostalMuscle",
        "CranialVenousSystem",
        "GiacominiVein",
        "MiddleCerebellarPeduncle",
        "RightVentricle",
        "VagusNerve",
        "NailUnitOfThirdToe",
        "HeadAndNeck",
        "SkinOfLateralCanthus",
        "OpticNerve",
        "ExternalCapsule",
        "Bladder",
        "Vulva",
        "Epicardium",
        "EntorhinalCortex",
        "LargeIntestine",
        "SkinOfLowerChestWall",
        "SkinOfUpperExtremity",
        "SkinOfNasolabialFold",
        "Forearm",
        "OlfactoryTract",
        "NeckChestAndAbdomen",
        "SuperiorMesentericArtery",
        "TemporalLobe",
        "EpigastricLymphNode",
        "PosteriorTibialArtery",
        "FemoralVein",
        "CiliaryBody",
        "RightAuricularAppendage",
        "GreatSaphenousVeinOfThigh",
        "L5Vertebra",
        "LeftHepaticVein",
        "SuperficialFemoralArtery",
        "_2ndMarginalCoronaryArtery",
        "T11T12Disc",
        "SuperiorCerebellarPeduncle",
        "DorsalAspectOfScapula",
        "ChoroidOfEye",
        "Carina",
        "AorticArch",
        "AsciticFluid",
        "PurkinjeFibers",
        "Supraglottis",
        "Telencephalon",
        "LeftBranchOfHepaticArtery",
        "PresymphysialLymphNode",
        "ArcuateArteryOfTheKidney",
        "Heart",
        "CephalicVein",
        "Lip",
        "GastricVein",
        "AllLegs",
        "LeftMainCoronaryArtery",
        "SkinOfLip",
        "InferiorRightHepaticVein",
        "MucosaOfTipOfTongue",
        "CarotidSiphon",
        "CentralAxillaryLymphNode",
        "AccessoryAtrioventricularBundle",
        "MastoidCellsAndAntra",
        "SuperiorMesentericLymphNode",
        "SystemicVenousAtrium",
        "L1L2Disc",
        "PlantarArterialArch",
        "AnomalousPulmonaryVein",
        "SkinOfFace",
        "SkinOfCavityOfConcha",
        "VertexOfHead",
        "LesserSaphenousVein",
        "AutonomicNerve",
        "GreatSaphenousVein",
        "T1T2Disc",
        "DistalTibia",
        "EsophagealLymphNode",
        "SkinOfLateralAspectOfAnkle",
        "EndoUreteric",
        "UlnarVein",
        "InferiorArticularFacetOfAxis",
        "SubclavianLymphNode",
        "Tibia",
        "MiddleEar",
        "SuperiorMedialInguinalLymphNode",
        "InternalAuditoryCanal",
        "RightPulmonaryArtery",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "ForcepsMajor",
        "HandBone",
        "CorpusCallosum",
        "PerforatingArteryOfKidney",
        "Humerus",
        "T5T6Disc",
        "HighRightAtrium",
        "AorticFistula",
        "SkinOfNeck",
        "LeftLumbarRegion",
        "NasalBone",
        "Cervix",
        "LeftAnteriorDescendingCoronaryArtery",
        "Artery",
        "Buttock",
        "AnatomicalReferencePlane",
        "Epididymis",
        "FallopianTube",
        "RetropyloricLymphNode",
        "Vein",
        "LymphNodeOfEpiploicForamen",
        "FacialNerve",
        "CerebrospinalFluid",
        "EsophagealArtery",
        "T8T9Disc",
        "ShaftOfHumerus",
        "Carpus",
        "PretrachealLymphNode",
        "SkinOfMedialPartOfHeel",
        "RadialVein",
        "LymphNodeOfLimb",
        "InferiorRightPulmonaryVein",
        "UterineArtery",
        "EndoVesical",
        "SkinOfGlutealFold",
        "MesentericArtery",
        "PromontoryCommonIliacLymphNode",
        "MuscleOfHead",
        "MiddleCerebralArteryM1Segment",
        "CouinaudHepaticSegmentIV",
        "ExternalJugularVein",
        "PosteriorHornLateralVentricle",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "AuditoryRadiation",
        "CornealEpithelium",
        "SubmentalLymphNode",
        "PosteriorArchVein",
        "AnteriorSpinalArtery",
        "SplenicVein",
        "AnusRectumAndSigmoidColon",
        "Colon",
        "CentralNervousSystem",
        "SplenicArtery",
        "AccessoryRenalArtery",
        "MediastinalLymphNode",
        "LowerLimbLymphNode",
        "Media",
        "SeminalVesicle",
        "DescendingColon",
        "DistalCircumflexCoronaryArtery",
        "BoneOfUpperLimb",
        "DorsalisPedisArtery",
        "NasalCavity",
        "Vagina",
        "LongissimusMuscle",
        "LowerInnerQuadrantOfBreast",
        "SkinOfToe",
        "TransverseColon",
        "Clavicle",
        "ParotidGland",
        "AorticIsthmus",
        "FloorOfMouth",
        "ProximalUlna",
        "_1stLeftPosterolateralCoronaryArtery",
        "Subxiphoid",
        "Diaphragm",
        "InferiorFrontoOccipitalFasciculus",
        "VasDeferens",
        "T6T7Disc",
        "SkinOfShaftOfPenis",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "SalivaryGland",
        "Ileum",
        "VertebralArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "AscendingColon",
        "SkinOfAnteriorSurfaceOfThorax",
        "SupraRenalAorta",
        "LateralCervicalLymphNode",
        "InternalJugularVein",
        "LacrimalArteryOfRightEye",
        "ChordaeTendineaeCordis",
        "SacroiliacJoint",
        "ScalenousAnteriorMuscle",
        "GroinSkinCrease",
        "HemolymphNode",
        "ColicLymphNode",
        "FacialLymphNode",
        "EntireBody",
        "PrelaryngealLymphNode",
        "SkinOfAnteriorTrunk",
        "PalatineTonsil",
        "PosteriorAuricularLymphNode",
        "TesticularArtery",
        "HypogastricRegion",
        "PharyngealTonsil",
        "ApicalAxillaryLymphNode",
        "NailUnitOfSecondToe",
        "HyoidBone",
        "C2Vertebra",
        "ThoracicDuct",
        "BoydPerforatingVein",
        "AnteriorTriangleOfNeck",
        "Peritioneum",
        "CommonIliacArtery",
        "SupraclavicularRegionOfNeck",
        "Meninges",
        "Shoulder",
        "SkinOfCheek",
        "TransversusThoracis",
        "SkinOfFoot",
        "SkinOfIntertragalIncisure",
        "CardiacSeptumProsthesis",
        "MidRightAtrium",
        "CerebellarWhiteMatter",
        "InterventricularSeptum",
        "Cingulum",
        "LowerLobeOfLung",
        "TesticularVein",
        "AnteriorTibialVein",
        "LeftInguinalRegion",
        "RightCoronaryArtery",
        "SkinOfHypogastricRegion",
        "Putamen",
        "InternalAuditoryMeatus",
        "CostocervicalTrunk",
        "MastoidLymphNode",
        "FifthLumbarArtery",
        "LymphNodeOfHeadAndNeck",
        "AxillaryVein",
        "LateralAxillaryLymphNode",
        "SkinOfPenis",
        "ScaleneLymphNode",
        "ClavicularNotchOfSternum",
        "SkinOfMedialBorderOfSoleOfFoot",
        "InferiorPancreaticLymphNode",
        "SubclavianVein",
        "ProximalRightCoronaryArtery",
        "PulmonaryValve",
        "TemporalBone",
        "ExternalIliacArtery",
        "BoneOfLowerLimb",
        "Midbrain",
        "PosteriorCerebralArtery",
        "ErectorSpinaeMuscle",
        "DeepPalmarVenousArch",
        "T2T3Disc",
        "Mouth",
        "PectoralAxillaryLymphNode",
        "PharynxAndLarynx",
        "SkinOfAnteriorSurfaceOfLowerLeg",
        "CarotidArtery",
        "UmbilicalArtery",
        "Cheek",
        "PulmonicRing",
        "LymphNode",
        "RadialArtery",
        "CommonDuctLymphNode",
        "T2Vertebra",
        "LeftFemoralArtery",
        "TrapeziusMuscle",
        "BuccinatorLymphNode",
        "Esophagus",
        "NailUnitOfFinger",
        "LateralCorticospinalTract",
        "ExtrapulmonaryLymphNodeOfLung",
        "BroadLigament",
        "Metacarpus",
        "SkinOfTemporalRegion",
        "CoracoidProcessOfScapula",
        "CerebralAqueduct",
        "Insula",
        "BoneMarrow",
        "HypogastricLymphNode",
        "PulmonaryVein",
        "SkinOfLowerEyelid",
        "MuscleOfNeck",
        "AtrioventricularBundle",
        "MucosaOfMandibularGingiva",
        "CorticospinalTractInBrainstem",
        "Lung",
        "SacroCoccygealSpine",
        "SkinOfPerineum",
        "GlenoidCavityOfScapula",
        "MaleExternalUrethralOrifice",
        "NailUnitOfThumb",
        "SkinOfUpperAbdomen",
        "LeftMainCoronaryArteryOstium",
        "T9Vertebra",
        "CardiovascularSystem",
        "CouinaudHepaticSegmentI",
        "LumbarLymphNode",
        "UpperParatrachealLymphNode",
        "ElbowJoint",
        "RegionalLymphNode",
        "UpperEyelid",
        "Sternum",
        "InferiorArticularProcessOfVertebra",
        "RightVentricleOutflowTract",
        "NeckAndChest",
        "GlobusPallidus",
        "UpperOuterQuadrantOfBreast",
        "AbdomenAndPelvis",
        "PelvicLymphNode",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "OstiumOfCoronarySinus",
        "Eyelash",
        "ShaftOfRib",
        "PericardialCavity",
        "SkinOfAntitragus",
        "PulmonaryArtery",
        "SkinOfMedialSurfaceOfThigh",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "C6Vertebra",
        "Thyroid",
        "EthmoidBone",
        "ParametrialLymphNode",
        "NavicularOfForefoot",
        "SuperficialPoplitealLymphNode",
        "InfraRenalAorta",
        "FetlockOfHindlimb",
        "ThoracoLumbarSpine",
        "SternoclavicularJoint",
        "RetroperitonealLymphNode",
        "Neck",
        "CarotidBulb",
        "Finger",
        "DeepCervicalLymphNode",
        "SkinOfBack",
        "Ischium",
        "InfraspinatusMuscle",
        "CouinaudHepaticSegmentVII",
        "SkinOfPosteriorSurfaceOfUpperArm",
        "SuperiorPhrenicArtery",
        "CommonVentricle",
        "FrenulumOfLabiaMinora",
        "SuperiorMediastinalLymphNode",
        "VesicularLymphNode",
        "SplenicLymphNode",
        "Uterus",
        "Ventricle",
        "TemporomandibularJoint",
        "TricuspidRing",
        "TracheaAndBronchus",
        "Testis",
        "FemoralHead",
        "AnteriorChamberOfEye",
        "SkinOfSideOfNose",
        "NasolabialLymphNode",
        "LaminaOfVertebra",
        "Eyeball",
        "Face",
        "UrinarySystem",
        "BrainStem",
        "OralCavity",
        "Foot",
        "PeripheralNervousSystem",
        "MuscleOfUpperLimb",
        "LateralExternalIliacLymphNode",
        "PinealGland",
        "SkinOfAntecubitalFossa",
        "UpperLobeOfLung",
        "LymphNodeOfGreaterCurvatureOfStomach",
        "PoplitealLymphNode",
        "PosterolateralBranchOfRightCoronaryArtery",
        "SkinOfSubmentalArea",
        "ProximalCircumflexCoronaryArtery",
        "SkinOfPostauricularRegion",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "Appendix",
        "HabenularCommissure",
        "VertebralColumnAndCranium",
        "AbdominalAorta",
        "Bone",
        "Tongue",
        "IntercostalLymphNode",
        "LymphNodeOfAorticArch",
        "ChestWallMuscle",
        "GrayMatter",
        "Fibula",
        "SuperiorLongitudinalFasciculus",
        "InferiorCardiacVein",
        "InternalIntercostalMuscle",
        "LateralVentricle",
        "CaudateNucleus",
        "LacrimalArtery",
        "OccipitalArtery",
        "Brain",
        "IntraglandularParotidLymphNode",
        "HepaticFlexureOfColon",
        "SigmoidLymphNode",
        "SkinOfTragus",
        "SuprasternalNotch",
        "AnteriorCardiacVein",
        "MidcolicLymphNode",
        "SkinOfPalmOfHand",
        "Pancreas",
        "BoneOfThorax",
        "NailUnitOfFifthToe",
        "DigitalArteryOfHand",
        "SecondLumbarArtery",
        "_1stRightPosterolateralCoronaryArtery",
        "LeftBranchOfAtrioventricularBundle",
        "BladderAndUrethra",
        "Sacrum",
        "Metatarsus",
        "FemaleInternalGenitalia",
        "VaginalIntroitus",
        "T8Vertebra",
        "MaleExternalGenitalia",
        "SkinOfAnteriorPortionOfNeck",
        "TubercleOfRib",
        "RetroperitonealSpace",
        "CervicalLymphNode",
        "IleocolicLymphNode",
        "AnteriorMiddleCerebralArteryBifurcation",
        "SuperiorArticularProcessOfVertebra",
        "SuperiorThyroidArtery",
        "CerebralCortex",
        "AnteriorCommissure",
        "TruncusArteriosusCommunis",
        "PedicleOfVertebra",
        "T1Vertebra",
        "Tooth",
        "Joint",
        "SkinOfDigitOfHand",
        "PasternOfHindfoot",
        "GreatSaphenousVeinOfCalf",
        "SuperiorTracheobronchialLymphNode",
        "Aorta",
        "MiddleJugularLymphNode",
        "AppendicealStump",
        "Diencephalon",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "T6Vertebra",
        "LatissimusDorsiMuscle",
        "ForamenMagnum",
        "CommonHepaticArtery",
        "SkinOfPartOfDorsalSurfaceOfHand",
        "AnkleJoint",
        "C4C5Disc",
        "SkinOfLabiumMajus",
        "LesserTrochanter",
        "SpinousProcessOfVertebra",
        "Bronchial",
        "LeftPosteriorDivisionOfLeftBranchAtrioventricularBundle",
        "PulmonaryChamberOfCorTriatriatum",
        "CommonBileDuct",
        "CysticLymphNode",
        "SkinOfLateralBorderOfSoleOfFoot",
        "UlnarArtery",
        "Stiffle",
        "LymphoidTissue",
        "SkinOfParietalRegion",
        "PancreaticoduodenalLymphNode",
        "LumbarRegion",
        "ThoracicSpine",
        "ManubriumOfSternum",
        "FourthVentricle",
        "T5Vertebra",
        "HipJoint",
        "OpticTract",
        "PulmonaryTrunk",
        "HypoglossalNerve",
        "L3Vertebra",
        "T4T5Disc",
        "LungAndTracheobronchialLymphNodes",
        "PrevascularRetrotrachealLymphNode",
        "ShaftOfRadius",
        "SkinOfPosteriorSurfaceOfThorax",
        "TrochlearNerve",
        "DiaphragmaticLymphNode",
        "InferiorMesentericLymphNode",
        "BrachialArtery",
        "C7Vertebra",
        "Placenta",
        "_2ndDiagonalCoronaryArtery",
        "FemaleGenitalLymphNode",
        "Hip",
        "PelvisAndLowerExtremities",
        "SerratusAnteriorMuscle",
        "ApexOfHeart",
        "PelvicCavityFalseAndOrTrue",
        "Stomach",
        "CircumflexCoronaryArtery",
        "IntestinalLymphNode",
        "AnteriorCommunicatingArtery",
        "InferiorMesentericVein",
        "SkinOfHelixOfEar",
        "ParaesophagealLymphNodeBelowCarina",
        "ParaAorticLymphNodeOfTheAnteriorMediastinum",
        "PerigastricLymphNode",
        "MedianCubitalVein",
        "Iris",
        "CoronarySinus",
        "TerminalInternalCarotidArtery",
        "FoveaCentralis",
        "BiliaryTract",
        "MedialLongitudinalFasciculus",
        "Atrium",
        "SkinOfHypothenarRegionOfPalm",
        "CerebralWhiteMatter",
        "Knee",
        "SubarachnoidSpace",
        "PulmonaryVeinConfluence",
        "InterpectoralLymphNode",
        "EyeRegion",
        "FingerJoint",
        "LymphNodeOfThorax",
        "BrainVentricle",
        "OralMucosa",
        "PharyngotympanicTube",
        "GlossopharyngealNerve",
        "UmbilicalVein",
        "InguinalRegion",
        "InternalNose",
        "_3rdRightPosterolateralCoronaryArtery",
        "NailUnitOfToe",
        "InferiorPhrenicArtery",
        "SkinOfPreauricularRegion",
        "PortalVein",
        "PosteriorCerebralCommissure",
        "SuperficialLateralCervicalLymphNode",
        "UncinateFasciculus",
        "AnteriorCerebralArtery",
        "L4L5Disc",
        "PrevertebralLymphNode",
        "Ear",
        "SpleniumOfCorpusCallosum",
        "MucosaOfLowerLip",
        "SkinOfPerioralRegionOfFace",
        "MesentericArteryLymphNode",
        "SuperficialAnteriorCervicalLymphNode",
        "SpinalCord",
        "T11Vertebra",
        "Pericardium",
        "PosteriorCerebralArteryP2Segment",
        "LateralRetropharyngealLymphNode",
        "PelvicCavityFalse",
        "LeftAuricularAppendage",
        "CircleOfWillis",
        "LowerEyelid",
        "SaphenopoplitealJunction",
        "_1stSeptalCoronaryArtery",
        "CoccygealVertrebrae",
        "Eyelid",
        "ParamammaryLymphNode",
        "ObturatorLymphNode",
        "DeepPalmarArchOfRadialArtery",
        "NailUnitOfRingFinger",
        "FrontalSinus",
        "PleuralStructure",
        "HypochondriacRegion",
        "Retina",
        "CisternaMagna",
        "DoddPerforatingVein",
    ],
    "UMLS": ["UnknownPrimaryNeoplasiaSite", "DistalPhalanx"],
}

name_for_cid[8135] = "MicroscopyAnnotationPropertyType"
cid_concepts[8135] = {"SCT": ["Cell", "Nucleus"]}

name_for_cid[8136] = "MicroscopyMeasurementType"
cid_concepts[8136] = {"SCT": ["Area"]}

name_for_cid[8201] = "SurfaceScanAcquisitionType"
cid_concepts[8201] = {
    "DCM": [
        "TimeOfFlight",
        "LaserScanning",
        "Interferometry",
        "ShapeFromShading",
        "PatternProjection",
        "ConfocalImaging",
        "ShapeFromMotion",
        "PointCloudAlgorithmic",
    ]
}

name_for_cid[8202] = "SurfaceScanModeType"
cid_concepts[8202] = {
    "DCM": ["HighResolution", "FastMode", "Checkerboard", "TurntableScanMethod"]
}

name_for_cid[8203] = "SurfaceScanRegistrationMethodType"
cid_concepts[8203] = {"DCM": ["IterativeClosestPoint", "Freehand", "FiducialAlignment"]}

name_for_cid[8300] = "VisualEvaluationMethod"
cid_concepts[8300] = {
    "DCM": [
        "GrayscaleResolutionEvaluation",
        "OverallImageQualityEvaluation",
        "LuminanceUniformityEvaluation",
        "LuminanceResponseEvaluation",
        "ClinicalEvaluation",
        "AngularViewingEvaluation",
        "PixelFaultsEvaluation",
        "ChromaticityEvaluation",
        "GeometricalImageEvaluation",
        "VeilingGlareEvaluation",
    ]
}

name_for_cid[8301] = "TestPatternCode"
cid_concepts[8301] = {
    "DCM": [
        "TG18LPV50Pattern",
        "TG18LPH50Pattern",
        "TG18LPH10Pattern",
        "TG18LPV10Pattern",
        "TG18LPH89Pattern",
        "TG18RV89Pattern",
        "TG18RV50Pattern",
        "TG18CXPattern",
        "TG18PXPattern",
        "TG18RV10Pattern",
        "BN05Pattern",
        "BN04Pattern",
        "BN06Pattern",
        "BN01Pattern",
        "GDPattern",
        "BN03Pattern",
        "BN02Pattern",
        "TG18GQPattern",
        "TG18GVNPattern",
        "TG18GQBPattern",
        "ANGPattern",
        "TG18GQNPattern",
        "OIQPattern",
        "TG18NS50Pattern",
        "TG18NS10Pattern",
        "TG18GVPattern",
        "TG18NS89Pattern",
        "TG18AFCPattern",
        "TG18LPV89Pattern",
        "TG18LN1216Pattern",
        "TG18LN1215Pattern",
        "TG18LN1217Pattern",
        "TG18LN1212Pattern",
        "TG18LN1211Pattern",
        "TG18LN1214Pattern",
        "TG18LN1213Pattern",
        "TG18LN1208Pattern",
        "TG18LN1210Pattern",
        "TG18LN1209Pattern",
        "TG18RH89Pattern",
        "TG18RH50Pattern",
        "TG18ADPattern",
        "TG18UNL80Pattern",
        "TG18RH10Pattern",
        "TG18MPPattern",
        "TG18UN10Pattern",
        "TG18LN1218Pattern",
        "TG18UNL10Pattern",
        "TG18UN80Pattern",
        "TG18LN815Pattern",
        "TG18LN812Pattern",
        "TG18LN811Pattern",
        "TG18LN814Pattern",
        "TG18LN813Pattern",
        "TG18LN808Pattern",
        "DINResolutionPattern",
        "TG18LN807Pattern",
        "DINGeometryPatternBild3",
        "TG18LN810Pattern",
        "TG18LN809Pattern",
        "TG18LN806Pattern",
        "DINGrayscalePatternBild2",
        "TG18LN1205Pattern",
        "TG18LN1204Pattern",
        "TG18LN1207Pattern",
        "TG18LN1206Pattern",
        "TG18LN1201Pattern",
        "TG18LN818Pattern",
        "SMPTEPattern",
        "TG18LN1203Pattern",
        "TG18LN1202Pattern",
        "TG18LN817Pattern",
        "TG18LN816Pattern",
        "WhitePattern",
        "BN16Pattern",
        "BN15Pattern",
        "BN12Pattern",
        "BN11Pattern",
        "BN14Pattern",
        "BN13Pattern",
        "TG18CHImage",
        "BN08Pattern",
        "TG18GA30Pattern",
        "BN07Pattern",
        "BN10Pattern",
        "TG18KNImage",
        "BN09Pattern",
        "TG18GA15Pattern",
        "TG18GA10Pattern",
        "TG18GA25Pattern",
        "TG18GA20Pattern",
        "TG18GA03Pattern",
        "TG18GA08Pattern",
        "TG18GA05Pattern",
        "TG18LN805Pattern",
        "TG18LN804Pattern",
        "TG18LN801Pattern",
        "TG18CTPattern",
        "TG18LN803Pattern",
        "TG18LN802Pattern",
        "TG18QCPattern",
        "BN18Pattern",
        "TG18PQCPattern",
        "TG18BRPattern",
        "BN17Pattern",
        "TG18MM2Image",
        "TG18MM1Image",
    ]
}

name_for_cid[8302] = "MeasurementPatternCode"
cid_concepts[8302] = {"DCM": ["TG18UNL80Pattern", "TG18UNL10Pattern"]}

name_for_cid[8303] = "DisplayDeviceType"
cid_concepts[8303] = {
    "DCM": [
        "CRTFrontProjectionSystem",
        "DLPRearProjectionSystem",
        "OLED",
        "CRTRearProjectionSystem",
        "DLPFrontProjectionSystem",
        "CRTDisplay",
        "PlasmaDisplay",
        "LiquidCrystalDisplay",
    ]
}

name_for_cid[9000] = "PhysicalQuantityDescriptor"
cid_concepts[9000] = {
    "DCM": [
        "SourceImageDiffusionBValue",
        "ModelFittingMethod",
        "Derivation",
        "Finding",
        "EquivalentMeaningOfConceptName",
    ],
    "NCIt": ["ReferenceRegion"],
    "SCT": ["Quantity", "FindingSite", "MeasurementMethod"],
}

name_for_cid[9231] = "WorkitemDefinition"
cid_concepts[9231] = {
    "DCM": [
        "NoSubsequentWorkitems",
        "AddAddendumToReport",
        "Print",
        "ReportVerification",
        "Transcription",
        "Interpretation",
        "ComputerAidedDetection",
        "ComputerAidedDiagnosis",
        "QualityControl",
        "MediaImport",
        "ImageProcessing",
    ]
}

name_for_cid[9233] = "RequestedReportType"
cid_concepts[9233] = {"DCM": ["FinalReport", "PreliminaryReport"]}

name_for_cid[9241] = "RadiotherapyGeneralWorkitemDefinition"
cid_concepts[9241] = {
    "DCM": [
        "RTPatientSetup",
        "RTPatientPositionInTreatmentSessionReview",
        "RTTreatmentQAByRTIonPlanDifferenceCheck",
        "RTTreatmentSimulationWithInternalVerification",
        "RTBrachyTreatment",
        "RTTreatmentQAByRTPlanDifferenceCheck",
        "RTPatientPositionAdjustment",
        "RTTreatmentQAByRTIonPlanDoseCheck",
        "RTMachineQA",
        "RTTreatmentQAByRTPlanDoseCheck",
        "RTTreatmentQAWithExternalVerification",
        "RTTreatmentWithExternalVerification",
        "RTTreatmentQAWithInternalVerification",
        "RTTreatmentSimulationWithExternalVerification",
        "RTImageGuidedPatientPositioningAndTreatmentDelivery",
        "RTTreatmentWithInternalVerification",
        "RTTreatmentWithAdHocPlanning",
    ]
}

name_for_cid[9242] = "RadiotherapyAcquisitionWorkitemDefinition"
cid_concepts[9242] = {
    "DCM": [
        "RTPatientPositionAcquisitionSinglePlaneMV",
        "RTPatientPositionAcquisitionUltrasound",
        "RTPatientPositionAcquisitionSpatialFiducials",
        "RTPatientPositionAcquisitionOptical",
        "RTPatientPositionAcquisitionCTKv",
        "RTPatientPositionAcquisitionCTMV",
        "RTPatientPositionAcquisitionSinglePlaneKv",
        "RTPatientPositionAcquisitionMR",
        "RTPatientPositionAcquisitionFilmCassetteKv",
        "RTPatientPositionAcquisitionFilmCassetteMV",
        "RTPatientPositionAcquisitionIntegratedDoseMV",
        "RTPatientPositionAcquisitionConventionalCTMV",
        "RTPatientPositionAcquisitionConeBeamCTMV",
        "RTPatientPositionAcquisitionConventionalCTKv",
        "RTPatientPositionAcquisitionConeBeamCTKv",
    ]
}

name_for_cid[9243] = "RadiotherapyRegistrationWorkitemDefinition"
cid_concepts[9243] = {
    "DCM": [
        "RTPatientPositionRegistrationSinglePlane",
        "RTPatientPositionRegistrationDualPlane",
        "RTPatientPositionRegistrationSpatialFiducials",
        "RTPatientPositionRegistrationUltrasound",
        "RTPatientPositionRegistration2DOn3DReference",
        "RTPatientPositionRegistrationOptical",
        "RTPatientPositionRegistration3DCTVolumeBased",
        "RTPatientPositionRegistrationMR",
        "RTPatientPositionRegistration3DOn2DReference",
        "RTPatientPositionRegistration3DCTGeneral",
        "RTPatientPositionRegistration3DCTMarkerBased",
    ]
}

name_for_cid[9250] = "ScheduledProcessingParameterConceptCodesforRTTreatment"
cid_concepts[9250] = {"DCM": ["TreatmentDeliveryType"]}

name_for_cid[9260] = "RadiotherapyAcquisitionWorkItemSubtasks"
cid_concepts[9260] = {
    "DCM": [
        "RTPatientPositionAcquisitionSinglePlaneMV",
        "RTPatientPositionAcquisitionCTKv",
        "RTPatientPositionAcquisitionCTMV",
        "RTPatientPositionAcquisitionDualPlaneKv",
        "RTPatientPositionAcquisitionDualPlaneKvMV",
        "RTPatientPositionAcquisitionDualPlaneMV",
        "RTPatientPositionAcquisitionSinglePlaneKv",
        "RTPatientPositionAcquisitionFilmCassetteKv",
        "RTPatientPositionAcquisitionFilmCassetteMV",
        "RTPatientPositionAcquisitionIntegratedDoseMV",
        "RTPatientPositionAcquisitionConventionalCTMV",
        "RTPatientPositionAcquisitionConeBeamCTMV",
        "RTPatientPositionAcquisitionConventionalCTKv",
        "RTPatientPositionAcquisitionConeBeamCTKv",
    ]
}

name_for_cid[9261] = "PatientPositionAcquisitionRadiationSourceLocations"
cid_concepts[9261] = {
    "DCM": ["NominalRadiationSourceLocation", "NominalImagingSourceLocation"]
}

name_for_cid[9262] = "EnergyDerivationTypes"
cid_concepts[9262] = {
    "DCM": ["ConfiguredDefaultImagingEnergy", "ConfiguredLowestImagingEnergy"]
}

name_for_cid[9263] = "KVImagingAcquisitionTechniques"
cid_concepts[9263] = {
    "DCM": [
        "RTPatientPositionAcquisitionFilmCassetteKv",
        "RTPatientPositionAcquisitionConventionalCTKv",
        "RTPatientPositionAcquisitionConeBeamCTKv",
        "RTPatientPositionAcquisitionCTKv",
        "RTPatientPositionAcquisitionSinglePlaneKv",
    ]
}

name_for_cid[9264] = "MVImagingAcquisitionTechniques"
cid_concepts[9264] = {
    "DCM": [
        "RTPatientPositionAcquisitionSinglePlaneMV",
        "RTPatientPositionAcquisitionFilmCassetteMV",
        "RTPatientPositionAcquisitionIntegratedDoseMV",
        "RTPatientPositionAcquisitionConventionalCTMV",
        "RTPatientPositionAcquisitionConeBeamCTMV",
        "RTPatientPositionAcquisitionCTMV",
    ]
}

name_for_cid[9265] = "PatientPositionAcquisitionProjectionTechniques"
cid_concepts[9265] = {
    "DCM": [
        "RTPatientPositionAcquisitionFilmCassetteKv",
        "RTPatientPositionAcquisitionSinglePlaneMV",
        "RTPatientPositionAcquisitionFilmCassetteMV",
        "RTPatientPositionAcquisitionIntegratedDoseMV",
        "RTPatientPositionAcquisitionSinglePlaneKv",
    ]
}

name_for_cid[9266] = "PatientPositionAcquisition–CTTechniques"
cid_concepts[9266] = {
    "DCM": [
        "RTPatientPositionAcquisitionConventionalCTMV",
        "RTPatientPositionAcquisitionConeBeamCTMV",
        "RTPatientPositionAcquisitionConventionalCTKv",
        "RTPatientPositionAcquisitionConeBeamCTKv",
        "RTPatientPositionAcquisitionCTKv",
        "RTPatientPositionAcquisitionCTMV",
    ]
}

name_for_cid[9267] = "PatientPositioningRelatedObjectPurposes"
cid_concepts[9267] = {"DCM": ["RTPatientPositioningReferenceImage"]}

name_for_cid[9268] = "PatientPositionAcquisitionDevices"
cid_concepts[9268] = {
    "SCT": [
        "DigitalImagerRadiationTherapy",
        "XRayFilmCassetteManual",
        "DigitalImagingScannerComputedRadiography",
        "MagneticResonanceImagingUnit",
    ]
}

name_for_cid[9269] = "RTRadiationMetersetUnits"
cid_concepts[9269] = {"UCUM": ["MonitorUnits", "Second"]}

name_for_cid[9270] = "AcquisitionInitiationTypes"
cid_concepts[9270] = {
    "DCM": [
        "AcquisitionInitiationByTriggeringParameter",
        "AcquisitionInitiationAfterEndOfRadiation",
        "AcquisitionInitiationBeforeStartOfRadiation",
        "AcquisitionInitiationNotDefined",
    ]
}

name_for_cid[9271] = "RTImagePatientPositionAcquisitionDevices"
cid_concepts[9271] = {
    "SCT": ["DigitalImagerRadiationTherapy", "XRayFilmCassetteManual"]
}

name_for_cid[9272] = "RTROIImageAcquisitionContext"
cid_concepts[9272] = {
    "DCM": [
        "CentralBreathingPosition",
        "PostVoiding",
        "PriorToVoiding",
        "PreSurgicalAnatomy",
    ],
    "SCT": [
        "ValsalvaManeuver",
        "EmptyRectum",
        "FullRectum",
        "Expiration",
        "ShallowBreathing",
        "Inspiration",
        "AutonomousBreathing",
        "PostSurgicalAnatomy",
    ],
}

name_for_cid[9273] = "PatientPositionAcquisition–MRTechniques"
cid_concepts[9273] = {"DCM": ["RTPatientPositionAcquisitionMR"]}

name_for_cid[9300] = "ProcedureDiscontinuationReason"
cid_concepts[9300] = {
    "DCM": [
        "WorkitemExpired",
        "WorkitemAssignmentRejectedByAssignedResource",
        "ExtravasationVisibleInImage",
        "ResourcePreEmpted",
        "MediaFailure",
        "ObjectSetIncomplete",
        "ObjectTypesNotSupported",
        "ObjectsIncorrectlyFormatted",
        "DiscontinuedProcedureStepReschedulingRecommended",
        "DiscontinuedProcedureStepRescheduled",
        "ResourceInadequate",
        "PatientConditionPreventedContinuing",
        "IncorrectWorklistEntrySelected",
        "DiscontinuedForUnspecifiedReason",
        "IncorrectSideOrdered",
        "NursingUnitCancel",
        "DuplicateOrder",
        "EquipmentChange",
        "PatientDied",
        "PatientAllergicToMediaContrast",
        "IncorrectProcedureOrdered",
        "EquipmentFailure",
        "DoctorCanceledProcedure",
        "ChangeOfProcedureForCorrectCharging",
        "PatientPregnant",
        "PatientDidNotArrive",
        "PatientTakenForTreatmentOrSurgery",
        "PatientRefusedToContinueProcedure",
    ],
    "SCT": [
        "RadiopharmaceuticalAdverseReaction",
        "PallorPaleComplexion",
        "InjectionSiteNerveDamage",
        "InjectionSiteFibrosis",
        "InjectionSiteExtravasation",
        "Cough",
        "ChillsAndFever",
        "InjectionSiteHypersensitivity",
        "InjectionSitePigmentationChange",
        "InjectionSiteScar",
        "InjectionSiteIrritation",
        "NauseaAndVomiting",
        "Sweating",
        "InjectionSiteAnesthesia",
        "FacialSwelling",
        "InjectionSiteInduration",
        "InjectionSiteUrticaria",
        "Bronchospasm",
        "InjectionSiteMalabsorption",
        "InjectionSiteSterileAbscess",
        "Headache",
        "SensationOfBeingWarm",
        "TachycardiaBradycardia",
        "InjectionSiteNecrosis",
        "Anxiety",
        "InjectionSiteDermatitis",
        "InjectionSiteDisorder",
        "Shaking",
        "InjectionSiteBruising",
        "DrugInducedFlushing",
        "CardiacArrhythmia",
        "LaryngealEdema",
        "InjectionSiteThrombosis",
        "InjectionSiteAbscess",
        "DrugInducedDizziness",
        "CardiorespiratoryArrest",
        "InjectionSiteCyst",
        "Hypertension",
        "InjectionSiteEdema",
        "Dyspnea",
        "InjectionSiteAtrophy",
        "DrugInducedHypotension",
        "InjectionSiteBurning",
        "InjectionSiteHemorrhage",
        "InjectionSiteUlcer",
        "EpilepticConvulsions",
        "Itching",
        "DrugRash",
        "InjectionSiteInflammation",
        "InjectionSiteInfection",
        "Hypotension",
        "DiffuseInflammatoryErythema",
        "TasteSenseAltered",
        "InjectionSiteParesthesia",
        "NasalCongestion",
        "NoMotorResponseToCommand",
        "InjectionSitePain",
        "InjectionSiteMass",
        "InjectionSiteGranuloma",
    ],
}

name_for_cid[9301] = "ModalityPPSDiscontinuationReason"
cid_concepts[9301] = {
    "DCM": [
        "PatientDied",
        "PatientConditionPreventedContinuing",
        "PatientAllergicToMediaContrast",
        "IncorrectWorklistEntrySelected",
        "IncorrectProcedureOrdered",
        "DiscontinuedForUnspecifiedReason",
        "EquipmentFailure",
        "IncorrectSideOrdered",
        "DoctorCanceledProcedure",
        "NursingUnitCancel",
        "DuplicateOrder",
        "ChangeOfProcedureForCorrectCharging",
        "PatientPregnant",
        "PatientDidNotArrive",
        "PatientTakenForTreatmentOrSurgery",
        "PatientRefusedToContinueProcedure",
        "EquipmentChange",
    ],
    "SCT": ["RadiopharmaceuticalAdverseReaction", "InjectionSiteExtravasation"],
}

name_for_cid[9302] = "MediaImportPPSDiscontinuationReason"
cid_concepts[9302] = {
    "DCM": [
        "IncorrectWorklistEntrySelected",
        "MediaFailure",
        "DiscontinuedForUnspecifiedReason",
        "ObjectSetIncomplete",
        "EquipmentFailure",
        "ObjectTypesNotSupported",
        "ObjectsIncorrectlyFormatted",
        "DuplicateOrder",
    ]
}

name_for_cid[9303] = "InterpretationRequestDiscontinuationReason"
cid_concepts[9303] = {
    "DCM": [
        "ResourcePreEmpted",
        "PatientDied",
        "IncorrectProcedureOrdered",
        "DiscontinuedForUnspecifiedReason",
        "ObjectSetIncomplete",
        "DoctorCanceledProcedure",
        "DuplicateOrder",
        "InterpretationRequiresSpecialistExpertise",
        "InsufficientQualityForInterpretation",
        "WorkitemAssignmentRejectedByAssignedResource",
        "ChangeOfProcedureForCorrectCharging",
    ]
}

name_for_cid[9401] = "IEC61217DevicePositionParameter"
cid_concepts[9401] = {
    "DCM": [
        "IEC61217TableTopLongitudinalPosition",
        "IEC61217TableTopVerticalPosition",
        "IEC61217GantryContinuousRollAngle",
        "IEC61217GantryContinuousPitchAngle",
        "IEC61217GantryContinuousYawAngle",
        "IEC61217PatientSupportContinuousYawAngle",
        "IEC61217TableTopContinuousPitchAngle",
        "IEC61217TableTopContinuousRollAngle",
        "IEC61217TableTopEccentricAxisDistance",
        "IEC61217TableTopContinuousEccentricAngle",
        "IEC61217TableTopLateralPosition",
    ]
}

name_for_cid[9402] = "IEC61217GantryPositionParameter"
cid_concepts[9402] = {
    "DCM": [
        "IEC61217GantryContinuousRollAngle",
        "IEC61217GantryContinuousPitchAngle",
        "IEC61217GantryContinuousYawAngle",
    ]
}

name_for_cid[9403] = "IEC61217PatientSupportPositionParameter"
cid_concepts[9403] = {
    "DCM": [
        "IEC61217TableTopLongitudinalPosition",
        "IEC61217TableTopVerticalPosition",
        "IEC61217PatientSupportContinuousYawAngle",
        "IEC61217TableTopContinuousPitchAngle",
        "IEC61217TableTopContinuousRollAngle",
        "IEC61217TableTopEccentricAxisDistance",
        "IEC61217TableTopContinuousEccentricAngle",
        "IEC61217TableTopLateralPosition",
    ]
}

name_for_cid[9500] = "DosimetricObjectiveType"
cid_concepts[9500] = {
    "DCM": [
        "SpecifiedConformationNumber",
        "SpecifiedHealthyTissueConformityIndex",
        "SpecifiedConformityIndex",
        "MinimumPercentVolumeAtRadiationDose",
        "MinimumRadiationDose",
        "MaximumSurfaceRadiationDose",
        "MinimumSurfaceRadiationDose",
        "SpecifiedHomogeneityIndex",
        "MinimizeMeterset",
        "MinimumEquivalentUniformDose",
        "MaximumAbsoluteVolumeAtRadiationDose",
        "MaximumMeanRadiationDose",
        "MinimumAbsoluteVolumeAtRadiationDose",
        "MinimumMeanRadiationDose",
        "MaximumPercentVolumeAtRadiationDose",
        "MaximumRadiationDose",
        "PrescriptionRadiationDose",
        "MaximumEquivalentUniformDose",
    ]
}

name_for_cid[9501] = "PrescriptionAnatomyCategory"
cid_concepts[9501] = {
    "DCM": ["ExternalBodyModel", "NonSpecificVolume"],
    "SCT": ["AnatomicalStructure", "PhysicalObject"],
}

name_for_cid[9502] = "RTSegmentAnnotationCategory"
cid_concepts[9502] = {
    "DCM": [
        "RTGeometricInformation",
        "RTDoseCalculationStructure",
        "RTTarget",
        "ExternalBodyModel",
        "NonSpecificVolume",
        "BrachytherapyDevice",
        "FixationOrPositioningDevice",
        "RadiotherapyFiducial",
        "RadiotherapyDoseRegion",
        "RTRegistrationMark",
        "PatientAttachedDoseControlObject",
    ],
    "SCT": [
        "Function",
        "Tissue",
        "SpatialAndRelationalConcept",
        "Substance",
        "MorphologicallyAbnormalStructure",
        "AnatomicalStructure",
        "PhysicalObject",
        "BodySubstance",
    ],
}

name_for_cid[9503] = "RadiotherapyTherapeuticRoleCategory"
cid_concepts[9503] = {
    "DCM": ["RTRegistrationMark", "RTDoseCalculationStructure", "RTTarget"]
}

name_for_cid[9504] = "RTGeometricInformation"
cid_concepts[9504] = {
    "DCM": [
        "PatientSetupPoint",
        "ReferenceAcquisitionPoint",
        "MoveableLaserPatientSetupPoint",
        "RoomLaserPatientSetupPoint",
        "IsocentricTreatmentLocationPoint",
    ]
}

name_for_cid[9505] = "FixationorPositioningDevice"
cid_concepts[9505] = {
    "DCM": [
        "RectalBalloon",
        "VaginalCylinder",
        "Mold",
        "HeadAndNeckMask",
        "HeadMask",
        "Headframe",
        "BodyFrame",
        "HeadFixationBoard",
        "HandGrips",
        "BreastBoard",
        "AbdominalCompressionArch",
        "HeelStop",
        "AbdominalCompressionBelt",
        "ShinRest",
        "Cast",
        "BreastBridge",
        "WholeBodyPod",
        "VacuumMold",
        "SeatPan",
    ],
    "SCT": [
        "Chair",
        "Armrest",
        "Table",
        "Stretcher",
        "Backrest",
        "Wall",
        "Headrest",
        "BiteBlock",
        "Floor",
    ],
}

name_for_cid[9506] = "BrachytherapyDevice"
cid_concepts[9506] = {
    "DCM": [
        "BrachytherapyChannelShield",
        "BrachytherapySourceApplicator",
        "BrachytherapyChannel",
    ]
}

name_for_cid[9507] = "ExternalBodyModel"
cid_concepts[9507] = {"DCM": ["ExtendedPatientAnatomyModel", "PatientAnatomyModel"]}

name_for_cid[9508] = "NonspecificVolume"
cid_concepts[9508] = {"DCM": ["UnclassifiedVolume", "UnclassifiedCombination"]}

name_for_cid[9509] = "PurposeofReferenceforRTPhysicianIntentInput"
cid_concepts[9509] = {
    "DCM": [
        "HistoricalRTPrescription",
        "RTPrescriptionInputImages",
        "DiagnosticSourceImages",
        "RegistrationResult",
        "SegmentationResult",
    ],
    "LN": ["RelevantDiagnosticTestsAndOrLaboratoryData"],
}

name_for_cid[9510] = "PurposeofReferenceforRTTreatmentPlanningInput"
cid_concepts[9510] = {
    "DCM": [
        "RTTreatmentPlanningInputImages",
        "DiagnosticSourceImages",
        "RegistrationResult",
        "SegmentationResult",
        "RTPrescriptionResultForRTTreatmentPlanning",
    ]
}

name_for_cid[9511] = "GeneralExternalRadiotherapyProcedureTechnique"
cid_concepts[9511] = {
    "DCM": [
        "StaticBeam",
        "SlidingWindowBeam",
        "StepAndShootBeam",
        "ConformalArcBeam",
        "ArcBeam",
        "VMAT",
    ]
}

name_for_cid[9512] = "TomotherapeuticRadiotherapyProcedureTechnique"
cid_concepts[9512] = {"DCM": ["TopographicBeam", "HelicalBeam"]}

name_for_cid[9513] = "FixationDevice"
cid_concepts[9513] = {
    "DCM": [
        "RectalBalloon",
        "Mold",
        "HeadAndNeckMask",
        "HeadMask",
        "Headframe",
        "VaginalCylinder",
        "BodyFrame",
        "HeadFixationBoard",
        "HandGrips",
        "BreastBoard",
        "AbdominalCompressionArch",
        "HeelStop",
        "AbdominalCompressionBelt",
        "ShinRest",
        "Cast",
        "BreastBridge",
        "WholeBodyPod",
        "VacuumMold",
        "SeatPan",
    ],
    "SCT": ["Backrest", "Armrest", "Headrest", "BiteBlock"],
}

name_for_cid[9514] = "AnatomicalStructureforRadiotherapy"
cid_concepts[9514] = {
    "BARI": [
        "_1stMarginalCoronaryArteryLaterals",
        "_2ndMarginalCoronaryArteryLaterals",
        "_3rdMarginalCoronaryArteryLaterals",
        "_1stDiagonalCoronaryArteryLaterals",
        "_2ndDiagonalCoronaryArteryLaterals",
        "RamusLaterals",
        "_3rdDiagonalCoronaryArteryLaterals",
        "PosteriorDescendingSeptalPerforators",
    ],
    "DCM": [
        "AorticKnob",
        "ArchOfTheAzygosVein",
        "SecondaryPulmonaryLobule",
        "Hiatus",
        "AzygoesophagealRecessInterface",
        "ParaspinalLine",
        "PosteriorTrachealStripe",
        "RightTrachealStripe",
        "Stripe",
        "MidlungWindow",
        "CarinaAngle",
        "CentrilobularStructures",
        "AnteriorJunctionLine",
        "PosteriorJunctionLine",
        "SurfaceOfTheChoroidScleraInterface",
        "OuterSurfaceOfTheBM",
        "OuterSurfaceOfTheCC",
        "PosteriorSurfaceOfTheRPE",
        "SurfaceOfTheCenterOfTheRPE",
        "OuterSurfaceOfIPL",
        "OuterSurfaceOfGCL",
        "OuterSurfaceOfOPL",
        "OuterSurfaceOfINL",
        "SurfaceBetweenInnerAndOuterSegmentsOfThePhotoreceptors",
        "OuterSurfaceOfHFL",
        "AnteriorSurfaceOfTheRPE",
        "SurfaceOfTheInterdigitatingZoneBetweenRetinaAndRPE",
        "OuterSurfaceOfRNFL",
    ],
    "FMA": [
        "SubcorticalGrayMatter",
        "ArcuateFasciculus",
        "WallOfGallbladder",
        "LumenOfGallbladder",
    ],
    "SCT": [
        "ProfundaFemorisVein",
        "LeftHypochondriacRegion",
        "SternocleidomastoidMuscle",
        "TruncusCoeliacus",
        "C3C4Disc",
        "InferiorLongitudinalFasciculus",
        "BasilarArtery",
        "ParanasalSinus",
        "PelvicCavityTrue",
        "BrainCerebrospinalFluidPathway",
        "WristJoint",
        "MidCircumflexCoronaryArtery",
        "CouinaudHepaticSegmentII",
        "CentralRetinalVein",
        "IntermediateArteryRamus",
        "GenicularArtery",
        "LeftVentricleInflow",
        "FontanelOfSkull",
        "SigmoidColon",
        "GastricArtery",
        "SubscapularisMuscle",
        "ExternalEar",
        "LeftVentricleOutflowTract",
        "RightFemoralArtery",
        "Intracranial",
        "JawRegion",
        "Calcaneus",
        "SolealVein",
        "IntercostalArtery",
        "PeripheralNerve",
        "LobarArtery",
        "TeresMajorMuscle",
        "PatentDuctusArteriosus",
        "ThighPerforator",
        "MesentericVein",
        "Pituitary",
        "MaleInternalGenitalia",
        "MidRightCoronaryArtery",
        "IntraThoracic",
        "PalatineBone",
        "CoronaryArtery",
        "RightBranchOfHepaticArtery",
        "UpperLimb",
        "DuraMater",
        "ExternalIliacVein",
        "PulmonaryArteriovenousFistula",
        "StructureOfDescendingThoracicAorta",
        "ParietalBone",
        "Flank",
        "Maxilla",
        "EsophagealHiatus",
        "CentralRetinalArtery",
        "Neurohypophysis",
        "SegmentalArtery",
        "BrachialVein",
        "RightUpperQuadrantOfAbdomen",
        "T10Vertebra",
        "Eye",
        "SuperficialPalmarArch",
        "Prostate",
        "HepaticArtery",
        "FacialBones",
        "Back",
        "PeronealArtery",
        "Duodenum",
        "RightHypochondriacRegion",
        "_1stMarginalCoronaryArtery",
        "SuperiorMesentericVein",
        "LumbarArtery",
        "Cornea",
        "Larynx",
        "PeronealVein",
        "L4Vertebra",
        "FemoralArtery",
        "IliacVein",
        "Ovary",
        "Urethra",
        "CoronaryArteryGraft",
        "Head",
        "AdrenalGland",
        "MitralValve",
        "AcromioclavicularJoint",
        "UmbilicalRegion",
        "SupraspinatusMuscle",
        "CongenitalCoronaryArteryFistulaToRightVentricle",
        "Calyx",
        "PosteriorMedialTributary",
        "PulmonaryVeinGreatVessel",
        "T3T4Disc",
        "SoftPalate",
        "T4Vertebra",
        "Submental",
        "ExternalIntercostalMuscle",
        "CranialVenousSystem",
        "NucleusAccumbens",
        "GiacominiVein",
        "PoplitealFossa",
        "MiddleCerebellarPeduncle",
        "PulmonaryArteryConduit",
        "InternalCarotidArteryC5Segment",
        "FissureOfLung",
        "RightVentricle",
        "LevatoresCostarumMuscles",
        "FrontalLobe",
        "VagusNerve",
        "Adventitia",
        "HeadAndNeck",
        "Hypopharynx",
        "RightCoronaryArteryOstium",
        "HepaticVein",
        "MarginalCoronaryArtery",
        "InternalMammaryArtery",
        "Bladder",
        "Vulva",
        "LargeIntestine",
        "EntorhinalCortex",
        "Forearm",
        "BronchialArtery",
        "NeckChestAndAbdomen",
        "CouinaudHepaticSegmentIII",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "TemporalLobe",
        "SystemicArtery",
        "PosteriorTibialArtery",
        "FemoralVein",
        "RightAuricularAppendage",
        "GreatSaphenousVeinOfThigh",
        "IntraAbdominopelvic",
        "LeftHepaticVein",
        "L5Vertebra",
        "SuperficialFemoralArtery",
        "VomerBone",
        "UterusAndFallopianTubes",
        "Arachnoid",
        "_2ndMarginalCoronaryArtery",
        "T11T12Disc",
        "SuperiorCerebellarPeduncle",
        "AorticArch",
        "Carina",
        "Telencephalon",
        "LeftBranchOfHepaticArtery",
        "ArcuateArteryOfTheKidney",
        "Heart",
        "TarsalJoint",
        "CephalicVein",
        "LingualArtery",
        "Lip",
        "GastricVein",
        "AirwayStructure",
        "CongenitalCoronaryArteryFistulaToRightAtrium",
        "DeltoidMuscle",
        "LeftMainCoronaryArtery",
        "CerebralFornix",
        "Penis",
        "Baffle",
        "InferiorRightHepaticVein",
        "VentricularVein",
        "CarotidSiphon",
        "SystemicVenousAtrium",
        "PlantarArterialArch",
        "L1L2Disc",
        "Parathyroid",
        "AnomalousPulmonaryVein",
        "Scapula",
        "LesserSaphenousVein",
        "Kidney",
        "AutonomicNerve",
        "GreatSaphenousVein",
        "T1T2Disc",
        "OmentalBursa",
        "InternalCarotidArtery",
        "L1Vertebra",
        "Ureter",
        "RightPosteriorAVCoronaryArtery",
        "UlnarVein",
        "FascialLayer",
        "MiddleEar",
        "InternalAuditoryCanal",
        "CouinaudHepaticSegmentV",
        "RightPulmonaryArtery",
        "RenalPelvis",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "Thrombus",
        "VenousNetwork",
        "SubcostalMuscle",
        "CorpusCallosum",
        "CommonAtrium",
        "PerforatingArteryOfKidney",
        "Humerus",
        "T5T6Disc",
        "Ilium",
        "AorticFistula",
        "InternalCarotidArteryC6Segment",
        "LeftLumbarRegion",
        "ILMInternalLimitingMembrane",
        "NasalBone",
        "AnteriorTibialArtery",
        "CranialNerve",
        "Cervix",
        "SkinOfAbdomen",
        "LeftAnteriorDescendingCoronaryArtery",
        "Artery",
        "CommonCarotidArtery",
        "Buttock",
        "InnermostIntercostalMuscles",
        "_3rdMarginalCoronaryArtery",
        "FallopianTube",
        "MastoidBone",
        "Vein",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "CouinaudHepaticSegmentVI",
        "CerebrospinalFluid",
        "EsophagealArtery",
        "T8T9Disc",
        "Thigh",
        "RadialVein",
        "InferiorRightPulmonaryVein",
        "Trachea",
        "PulmonaryValveSinuses",
        "LeftPosterolateralCircumflexCoronaryArtery",
        "UterineArtery",
        "BodyConduit",
        "Thalamus",
        "EndoVesical",
        "RightVentricleInflow",
        "MesentericArtery",
        "LeftLowerQuadrantOfAbdomen",
        "MuscleOfHead",
        "SpinalCerebrospinalFluidPathway",
        "MiddleCerebralArteryM1Segment",
        "CouinaudHepaticSegmentIV",
        "ExternalJugularVein",
        "SesamoidBonesOfFoot",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "C2C3Disc",
        "AbdominalWallMuscle",
        "SixthLumbarArtery",
        "PosteriorArchVein",
        "AnteriorSpinalArtery",
        "AnusRectumAndSigmoidColon",
        "Omentum",
        "SplenicVein",
        "Colon",
        "SubmandibularGland",
        "CentralNervousSystem",
        "CarotidBifurcation",
        "T3Vertebra",
        "InnominateArtery",
        "SplenicArtery",
        "Extremity",
        "FacialArtery",
        "AccessoryRenalArtery",
        "BoneStructureOfHeadAndOrNeck",
        "EpigastricRegion",
        "Media",
        "SeminalVesicle",
        "CervicoThoracicSpine",
        "DistalCircumflexCoronaryArtery",
        "RightLumbarRegion",
        "TeresMinorMuscle",
        "BoneOfUpperLimb",
        "DorsalisPedisArtery",
        "Vagina",
        "LongissimusMuscle",
        "LowerInnerQuadrantOfBreast",
        "Clavicle",
        "ParotidGland",
        "AorticIsthmus",
        "CommonFemoralVein",
        "SubstantiaNigra",
        "_1stLeftPosterolateralCoronaryArtery",
        "Subxiphoid",
        "UpperInnerQuadrantOfBreast",
        "Diaphragm",
        "C5Vertebra",
        "ThoracicAorta",
        "RightInguinalRegion",
        "VasDeferens",
        "T6T7Disc",
        "AxillaryArtery",
        "LeftPulmonaryArtery",
        "Ileum",
        "SalivaryGland",
        "VertebralArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "RightLowerQuadrantOfAbdomen",
        "SupraRenalAorta",
        "InternalJugularVein",
        "LacrimalArteryOfRightEye",
        "CockettPerforatingVein",
        "ChordaeTendineaeCordis",
        "SacroiliacJoint",
        "RightMainBronchus",
        "ScalenousAnteriorMuscle",
        "AVGrooveContinuationOfCircumflexArtery",
        "EntireBody",
        "TesticularArtery",
        "Thumb",
        "HypogastricRegion",
        "GreatCardiacVein",
        "HyoidBone",
        "C2Vertebra",
        "ThoracicDuct",
        "MiddleLobeOfRightLung",
        "BoydPerforatingVein",
        "BodyCavity",
        "Peritioneum",
        "CommonIliacArtery",
        "SupraclavicularRegionOfNeck",
        "Meninges",
        "T12Vertebra",
        "C3Vertebra",
        "Shoulder",
        "TransversusThoracis",
        "CardiacSeptumProsthesis",
        "CerebellarWhiteMatter",
        "L2L3Disc",
        "InterventricularSeptum",
        "C5C6Disc",
        "_2ndLeftPosterolateralCoronaryArtery",
        "MiddleCerebralArteryM2Segment",
        "Cingulum",
        "LowerLobeOfLung",
        "TesticularVein",
        "AnteriorTibialVein",
        "LacrimalBone",
        "BasilicVein",
        "LeftInguinalRegion",
        "RightCoronaryArtery",
        "HunterianPerforatingVein",
        "Putamen",
        "ProfundaFemorisArtery",
        "CostocervicalTrunk",
        "Pharynx",
        "FifthLumbarArtery",
        "AxillaryVein",
        "LigamentumArteriosum",
        "Nasopharynx",
        "C6C7Disc",
        "SuprapulmonicValveArea",
        "SympatheticTrunk",
        "LobeOfLung",
        "SubclavianVein",
        "PulmonaryValve",
        "ExternalIliacArtery",
        "ProximalRightCoronaryArtery",
        "TemporalBone",
        "BoneOfLowerLimb",
        "Midbrain",
        "OvarianVein",
        "PosteriorCerebralArtery",
        "ErectorSpinaeMuscle",
        "DeepPalmarVenousArch",
        "ApexOfRightVentricle",
        "T2T3Disc",
        "C1Vertebra",
        "LateralCalfPerforator",
        "Phantom",
        "Mouth",
        "PharynxAndLarynx",
        "CarotidArtery",
        "FourthLumbarArtery",
        "SmallestCardiacVein",
        "ThirdLumbarArtery",
        "LimbicLobe",
        "SkinOfChest",
        "UmbilicalArtery",
        "Cheek",
        "LymphNode",
        "PrimitiveAorta",
        "RadialArtery",
        "T2Vertebra",
        "LeftFemoralArtery",
        "LumenOfBloodVessel",
        "TrapeziusMuscle",
        "Esophagus",
        "AntecubitalVein",
        "SpinalCordGrayMatter",
        "Mandible",
        "LateralCorticospinalTract",
        "Parasternal",
        "LeftMainBronchus",
        "BroadLigament",
        "SystemicVein",
        "OccipitalLobe",
        "C7T1Disc",
        "CerebralAqueduct",
        "Insula",
        "Pelvis",
        "PulmonaryVein",
        "MuscleOfNeck",
        "SubclavianArtery",
        "Amygdala",
        "Lung",
        "SacroCoccygealSpine",
        "_3rdLeftPosterolateralCoronaryArtery",
        "OpticChiasm",
        "OphthalmicArtery",
        "PancreaticDuct",
        "Nose",
        "Hand",
        "PectoralisMajorMuscle",
        "LeftMainCoronaryArteryOstium",
        "T9Vertebra",
        "CardiovascularSystem",
        "CouinaudHepaticSegmentI",
        "OvarianArtery",
        "ElbowJoint",
        "Hippocampus",
        "Sternum",
        "NeckAndChest",
        "RightVentricleOutflowTract",
        "GlobusPallidus",
        "SuperficialFemoralVein",
        "SuperiorLeftPulmonaryVein",
        "LeftPosteriorDescendingCircumflexCoronaryArtery",
        "BileDuct",
        "SystemicCollateralArteryToLung",
        "AbdomenAndPelvis",
        "UpperOuterQuadrantOfBreast",
        "PancreaticDuctAndBileDuctSystems",
        "ApexOfLeftVentricle",
        "PericardialCavity",
        "PulmonaryArtery",
        "Skull",
        "MidLeftAnteriorDescendingCoronaryArtery",
        "C6Vertebra",
        "CerebralHemisphere",
        "AtrioventricularVein",
        "Thyroid",
        "EthmoidBone",
        "OpticRadiation",
        "InfraRenalAorta",
        "SternoclavicularJoint",
        "ThoracoLumbarSpine",
        "T7Vertebra",
        "EndoNasopharyngeal",
        "Breast",
        "EsophagusStomachAndDuodenum",
        "Neck",
        "AxillaryFascia",
        "CarotidBulb",
        "Finger",
        "PoplitealVein",
        "InfraspinatusMuscle",
        "PiaMater",
        "TricuspidValve",
        "RightAtrium",
        "CouinaudHepaticSegmentVII",
        "VascularSclerosis",
        "SuperiorPhrenicArtery",
        "CommonVentricle",
        "DistalLeftAnteriorDescendingCoronaryArtery",
        "InnerEar",
        "Coccyx",
        "AtrialSeptalDefect",
        "RightPulmonaryVein",
        "TemporomandibularJoint",
        "Uterus",
        "Ventricle",
        "DescendingAorta",
        "TracheaAndBronchus",
        "Testis",
        "ProximalLeftAnteriorDescendingCoronaryArtery",
        "OccipitalBone",
        "L2Vertebra",
        "Eyeball",
        "Face",
        "Spine",
        "UrinarySystem",
        "BrainStem",
        "Foot",
        "WhiteMatter",
        "MuscleOfUpperLimb",
        "PeripheralNervousSystem",
        "PinealGland",
        "UpperLobeOfLung",
        "PosterolateralBranchOfRightCoronaryArtery",
        "Zygoma",
        "InnominateVein",
        "Abdomen",
        "ProximalCircumflexCoronaryArtery",
        "LowerLimb",
        "PosteriorCommunicationArtery",
        "JuxtaposedAtrialAppendage",
        "VertebralColumnAndCranium",
        "Jejunum",
        "CarotidBody",
        "ChestAbdomenAndPelvis",
        "AbdominalAorta",
        "RightHepaticVein",
        "GastrocnemiusVein",
        "Scalp",
        "Tongue",
        "ChestWallMuscle",
        "GrayMatter",
        "Fibula",
        "SuperiorLongitudinalFasciculus",
        "OrbitalStructure",
        "InferiorCardiacVein",
        "LeftMainBranchOfPortalVein",
        "InternalIntercostalMuscle",
        "LateralVentricle",
        "CaudateNucleus",
        "LacrimalArtery",
        "OccipitalArtery",
        "Brain",
        "BloodVessel",
        "Sclera",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "LiverAndBiliaryStructure",
        "AtrialVein",
        "_1stDiagonalCoronaryArtery",
        "SuprasternalNotch",
        "Perineum",
        "InteratrialSeptum",
        "Femur",
        "Rectum",
        "AnteriorCardiacVein",
        "T10T11Disc",
        "Pancreas",
        "BoneOfThorax",
        "LeftAtrium",
        "InterlobularArteryOfKidney",
        "CerebralArtery",
        "CouinaudHepaticSegmentVIII",
        "InferiorMesentericArtery",
        "DigitalArteryOfHand",
        "SecondLumbarArtery",
        "IliocostalisMuscle",
        "_1stRightPosterolateralCoronaryArtery",
        "BladderAndUrethra",
        "CervicalSpine",
        "IliacAndOrFemoralArtery",
        "SpinalCordWhiteMatter",
        "Sacrum",
        "SmallIntestine",
        "FemaleInternalGenitalia",
        "SuperiorRightPulmonaryVein",
        "T8Vertebra",
        "InferiorVenaCava",
        "MaleExternalGenitalia",
        "IntraArticular",
        "Axilla",
        "RetroperitonealSpace",
        "Thymus",
        "ELMExternalLimitingMembrane",
        "AnteriorMiddleCerebralArteryBifurcation",
        "ParietalLobe",
        "SuperiorThyroidArtery",
        "IliacArtery",
        "T12L1Disc",
        "DistalRightCoronaryArtery",
        "CerebralCortex",
        "TruncusArteriosusCommunis",
        "T1Vertebra",
        "Tooth",
        "Joint",
        "SubmandibularArea",
        "GreatSaphenousVeinOfCalf",
        "LeftPulmonaryVein",
        "PectoralisMinorMuscle",
        "OpticCanal",
        "Aorta",
        "SuperficialTemporalArtery",
        "MedialLemniscus",
        "Diencephalon",
        "CongenitalCoronaryArteryFistulaToLeftVentricle",
        "NeckChestAbdomenAndPelvis",
        "InferiorCerebellarPeduncle",
        "RightMainBranchOfPortalVein",
        "SphenoidBone",
        "T6Vertebra",
        "LatissimusDorsiMuscle",
        "ChestAndAbdomen",
        "Spleen",
        "CommonHepaticArtery",
        "C4Vertebra",
        "AnkleJoint",
        "LowerOuterQuadrantOfBreast",
        "C4C5Disc",
        "Mediastinum",
        "Cerebellum",
        "FirstLumbarArtery",
        "PoplitealArtery",
        "UpperUrinaryTract",
        "PulmonaryVenousAtrium",
        "Bronchus",
        "PulmonaryChamberOfCorTriatriatum",
        "T9T10Disc",
        "DorsalScapularArtery",
        "CommonBileDuct",
        "UlnarArtery",
        "L3L4Disc",
        "LeftVentricle",
        "LumbarRegion",
        "CostalCartilage",
        "ThoracicSpine",
        "FourthVentricle",
        "SaphenousVein",
        "T5Vertebra",
        "Gallbladder",
        "_2ndRightPosterolateralCoronaryArtery",
        "RenalArtery",
        "PeritonealCavity",
        "HipJoint",
        "UpperArm",
        "OpticTract",
        "TrunkOfPulmonaryArtery",
        "Chest",
        "GastroduodenalArtery",
        "L3Vertebra",
        "T4T5Disc",
        "AzygosVein",
        "BrachialArtery",
        "C7Vertebra",
        "_2ndDiagonalCoronaryArtery",
        "Liver",
        "Hip",
        "Rib",
        "L5S1Disc",
        "PelvisAndLowerExtremities",
        "SerratusAnteriorMuscle",
        "CommonFemoralArtery",
        "SmallCardiacVein",
        "IntraPelvic",
        "SuperficialPalmarVenousArch",
        "Vertebra",
        "Stomach",
        "CircumflexCoronaryArtery",
        "AnteriorCommunicatingArtery",
        "CongenitalCoronaryArteryFistulaToLeftAtrium",
        "InferiorMesentericVein",
        "T7T8Disc",
        "VenaCava",
        "MedianCubitalVein",
        "CoronarySinus",
        "HilarArtery",
        "TerminalInternalCarotidArtery",
        "LeftCoronaryArtery",
        "BiliaryTract",
        "ApexOfLung",
        "ThirdVentricle",
        "Atrium",
        "CerebralWhiteMatter",
        "Knee",
        "_3rdDiagonalCoronaryArtery",
        "SpinalisMuscle",
        "SubarachnoidSpace",
        "PulmonaryVeinConfluence",
        "SaphenofemoralJunction",
        "EyeRegion",
        "BrainVentricle",
        "Anus",
        "UmbilicalVein",
        "FrontalBone",
        "CommonIliacVein",
        "TrabeculaeCarnae",
        "InguinalRegion",
        "PosteriorCerebralArteryP1Segment",
        "_3rdRightPosterolateralCoronaryArtery",
        "EndoNasal",
        "LumbarSpine",
        "ThyrocervicalTrunk",
        "InferiorPhrenicArtery",
        "PrimitivePulmonaryArtery",
        "HilumOfLung",
        "SellaTurcica",
        "PortalVein",
        "Scrotum",
        "PosteriorCerebralCommissure",
        "InferiorLeftPulmonaryVein",
        "UncinateFasciculus",
        "Intima",
        "AnteriorCerebralArtery",
        "LowerLeg",
        "RenalVein",
        "L4L5Disc",
        "InternalIliacVein",
        "IlealVein",
        "Ear",
        "Toe",
        "SpinalCord",
        "MiddleCardiacVein",
        "T11Vertebra",
        "Pericardium",
        "MiddleCerebralArtery",
        "PosteriorCerebralArteryP2Segment",
        "PosteriorDescendingRightCoronaryArtery",
        "CorpusStriatum",
        "LeftAuricularAppendage",
        "PelvicCavityFalse",
        "CircleOfWillis",
        "Adenohypophysis",
        "PosteriorTibialVein",
        "SaphenopoplitealJunction",
        "_1stSeptalCoronaryArtery",
        "MuscleOfLowerLimb",
        "Eyelid",
        "AscendingAorta",
        "LeftUpperQuadrantOfAbdomen",
        "DeepPalmarArchOfRadialArtery",
        "PericardiophrenicArtery",
        "Pleura",
        "HypochondriacRegion",
        "ExternalAuditoryCanal",
        "Patella",
        "SegmentOfLung",
        "OccipitalVein",
        "LumboSacralSpine",
        "DoddPerforatingVein",
    ],
}

name_for_cid[9515] = "RTPatientSupportDevice"
cid_concepts[9515] = {"SCT": ["Chair", "Table", "Wall", "Stretcher", "Floor"]}

name_for_cid[9516] = "RadiotherapyBolusDeviceType"
cid_concepts[9516] = {"SCT": ["SurfaceBolus"]}

name_for_cid[9517] = "RadiotherapyBlockDeviceType"
cid_concepts[9517] = {"DCM": ["ApertureBlock"], "SCT": ["ShieldingBlock"]}

name_for_cid[9518] = "RadiotherapyAccessoryNoSlotHolderDeviceType"
cid_concepts[9518] = {"DCM": ["AccessoryTray"]}

name_for_cid[9519] = "RadiotherapyAccessorySlotHolderDeviceType"
cid_concepts[9519] = {"DCM": ["RadiotherapyApplicator"]}

name_for_cid[9520] = "SegmentedRTAccessoryDevice"
cid_concepts[9520] = {
    "DCM": [
        "BrachytherapyChannel",
        "RectalBalloon",
        "AccessoryTray",
        "ApertureBlock",
        "VaginalCylinder",
        "RadiotherapyApplicator",
        "Mold",
        "HeadAndNeckMask",
        "BrachytherapyChannelShield",
        "HeadMask",
        "BrachytherapySourceApplicator",
        "Headframe",
        "BodyFrame",
        "HeadFixationBoard",
        "HandGrips",
        "BreastBoard",
        "AbdominalCompressionArch",
        "HeelStop",
        "AbdominalCompressionBelt",
        "ShinRest",
        "Cast",
        "BreastBridge",
        "WholeBodyPod",
        "VacuumMold",
        "SeatPan",
    ],
    "SCT": [
        "ShieldingBlock",
        "Chair",
        "Armrest",
        "Table",
        "Stretcher",
        "Backrest",
        "Wall",
        "SurfaceBolus",
        "Headrest",
        "BiteBlock",
        "Floor",
    ],
}

name_for_cid[9521] = "RadiotherapyTreatmentEnergyUnit"
cid_concepts[9521] = {"UCUM": ["Kilovolt", "Megavolt", "MegaElectronVolt"]}

name_for_cid[9522] = "MultiSourceRadiotherapyProcedureTechnique"
cid_concepts[9522] = {"DCM": ["MultipleFixedSources"]}

name_for_cid[9523] = "RoboticRadiotherapyProcedureTechnique"
cid_concepts[9523] = {
    "DCM": ["SynchronizedRoboticTreatment", "NonSynchronizedRoboticTreatment"]
}

name_for_cid[9524] = "RadiotherapyProcedureTechnique"
cid_concepts[9524] = {
    "DCM": [
        "StaticBeam",
        "SlidingWindowBeam",
        "SynchronizedRoboticTreatment",
        "StepAndShootBeam",
        "MultipleFixedSources",
        "ConformalArcBeam",
        "ArcBeam",
        "TopographicBeam",
        "HelicalBeam",
        "VMAT",
        "NonSynchronizedRoboticTreatment",
    ]
}

name_for_cid[9525] = "RadiationTherapyParticle"
cid_concepts[9525] = {"SCT": ["Electron", "Photon"]}

name_for_cid[9526] = "IonTherapyParticle"
cid_concepts[9526] = {
    "DCM": [
        "_16OxygenNucleus",
        "_12CarbonNucleus",
        "_4HeliumNucleus",
        "_3HeliumNucleus",
    ],
    "NCIt": ["Ion"],
    "SCT": ["Proton"],
}

name_for_cid[9527] = "TeletherapyIsotope"
cid_concepts[9527] = {"SCT": ["_60Cobalt"]}

name_for_cid[9528] = "BrachytherapyIsotope"
cid_concepts[9528] = {
    "SCT": [
        "_32Phosphorus",
        "_222Radon",
        "_252Californium",
        "_198Gold",
        "_90Strontium",
        "_137Cesium",
        "_125Iodine",
        "_226Radium",
        "_90Yttrium",
        "_103Palladium",
        "_169Ytterbium",
        "_131Cesium",
        "_60Cobalt",
        "_106Ruthenium",
        "_192Iridium",
    ]
}

name_for_cid[9529] = "SingleDoseDosimetricObjective"
cid_concepts[9529] = {
    "DCM": [
        "MinimumRadiationDose",
        "MaximumSurfaceRadiationDose",
        "MinimumSurfaceRadiationDose",
        "MinimumEquivalentUniformDose",
        "MaximumMeanRadiationDose",
        "MinimumMeanRadiationDose",
        "MaximumRadiationDose",
        "PrescriptionRadiationDose",
        "MaximumEquivalentUniformDose",
    ]
}

name_for_cid[9530] = "PercentageandDoseDosimetricObjective"
cid_concepts[9530] = {
    "DCM": [
        "MinimumPercentVolumeAtRadiationDose",
        "MaximumPercentVolumeAtRadiationDose",
    ]
}

name_for_cid[9531] = "VolumeandDoseDosimetricObjective"
cid_concepts[9531] = {
    "DCM": [
        "MaximumAbsoluteVolumeAtRadiationDose",
        "MinimumAbsoluteVolumeAtRadiationDose",
    ]
}

name_for_cid[9532] = "NoParameterDosimetricObjective"
cid_concepts[9532] = {"DCM": ["MinimizeMeterset"]}

name_for_cid[9533] = "DeliveryTimeStructure"
cid_concepts[9533] = {
    "DCM": [
        "ContinuousPermanent",
        "ContinuousTemporary",
        "HyperFractionation",
        "HypoFractionation",
        "StandardFractionation",
        "SingleFraction",
    ]
}

name_for_cid[9534] = "RadiotherapyTarget"
cid_concepts[9534] = {
    "DCM": [
        "CTVPrimary",
        "PTVPrimary",
        "PTVNodal",
        "RadiationDoseReferencePoint",
        "GTVPrimary",
        "RadiationDoseNormalizationPoint",
        "GTVNodal",
        "ITV",
        "EntireBodyTargetVolume",
        "CTVNodal",
        "TreatedVolume",
    ],
    "SCT": ["IrradiatedVolume", "GTV", "PTV", "CTV"],
}

name_for_cid[9535] = "RadiotherapyDoseCalculationRole"
cid_concepts[9535] = {
    "DCM": [
        "AvoidanceVolume",
        "PlanningOrganAtRiskVolume",
        "RadiationInteractionVolume",
        "RadiationDoseShapingVolume",
        "OrganAtRisk",
        "DoseCalculationBoundingVolume",
        "ConformalityShell",
    ]
}

name_for_cid[9536] = "RadiotherapyPrescribingandSegmentingPersonRole"
cid_concepts[9536] = {
    "DCM": ["LeadRadiologicTechnologist"],
    "SCT": [
        "RadiologicTechnologist",
        "Resident",
        "MedicalPractitioner",
        "Physician",
        "RadiationTherapist",
    ],
    "UMLS": ["ConsultingPhysician", "RadiationPhysicist", "MedicalPhysicist"],
}

name_for_cid[9537] = "EffectiveDoseCalculationMethodCategory"
cid_concepts[9537] = {
    "DCM": [
        "FractionationBasedOrTemporallyBasedMethods",
        "RadiationTransportBasedMethods",
    ]
}

name_for_cid[9538] = "RadiationTransportBasedEffectiveDoseMethodModifier"
cid_concepts[9538] = {"DCM": ["LocalEffectModel", "MicrodosimetricKineticModel"]}

name_for_cid[9539] = "FractionationBasedEffectiveDoseMethodModifer"
cid_concepts[9539] = {
    "DCM": [
        "LinearQuadraticModelForLowDoseRateBrachytherapy",
        "LinearQuadraticLinearModel",
        "LinearQuadraticModelWithTimeFactor",
        "LinearQuadraticModel",
        "Equivalent2GrayFractionsModel",
    ]
}

name_for_cid[9540] = "MovableBeamLimitingDeviceType"
cid_concepts[9540] = {
    "DCM": ["SingleLeaves", "VariableCircularCollimator", "LeafPairs", "JawPair"]
}

name_for_cid[9541] = "BeamLimitingDeviceType"
cid_concepts[9541] = {
    "DCM": [
        "SingleLeaves",
        "PhotonFixedAperture",
        "VariableCircularCollimator",
        "ElectronFixedAperture",
        "ApertureBlock",
        "LeafPairs",
        "JawPair",
        "IntraoperativeFixedAperture",
    ],
    "SCT": ["ShieldingBlock"],
}

name_for_cid[9542] = "CompensatorDeviceType"
cid_concepts[9542] = {"DCM": ["PhysicalCompensator"]}

name_for_cid[9543] = "RadiotherapyTreatmentMachineMode"
cid_concepts[9543] = {"DCM": ["TotalSkinIrradiation", "TotalBodyIrradiation"]}

name_for_cid[9544] = "RadiotherapyDistanceReferenceLocation"
cid_concepts[9544] = {
    "DCM": ["TreatmentMachineIsocenter", "NominalRadiationSourceLocation"]
}

name_for_cid[9545] = "FixedBeamLimitingDeviceType"
cid_concepts[9545] = {
    "DCM": [
        "PhotonFixedAperture",
        "ElectronFixedAperture",
        "ApertureBlock",
        "IntraoperativeFixedAperture",
    ],
    "SCT": ["ShieldingBlock"],
}

name_for_cid[9546] = "RadiotherapyWedgeType"
cid_concepts[9546] = {"DCM": ["DynamicWedge", "MotorizedWedge", "HardWedge"]}

name_for_cid[9547] = "RTBeamLimitingDeviceOrientationLabel"
cid_concepts[9547] = {"DCM": ["YOrientation", "XOrientation"]}

name_for_cid[9548] = "GeneralAccessoryDeviceType"
cid_concepts[9548] = {
    "DCM": [
        "BowtieFilter",
        "WinstonLutzPointer",
        "FilmHolder",
        "Graticule",
        "ImageDetector",
        "Reticle",
    ]
}

name_for_cid[9549] = "RadiationGenerationModeType"
cid_concepts[9549] = {
    "DCM": [
        "FlatteningFilterBeam",
        "PartialFlatteningFilterBeam",
        "NonFlatteningFilterBeam",
    ]
}

name_for_cid[9550] = "CArmPhotonElectronDeliveryRateUnit"
cid_concepts[9550] = {"UCUM": ["MonitorUnitsPerSecond"]}

name_for_cid[9551] = "TreatmentDeliveryDeviceType"
cid_concepts[9551] = {"DCM": ["RadiotherapyTreatmentDevice"]}

name_for_cid[9552] = "CArmPhotonElectronDosimeterUnit"
cid_concepts[9552] = {"UCUM": ["MonitorUnits"]}

name_for_cid[9553] = "TreatmentPoint"
cid_concepts[9553] = {"DCM": ["IsocentricTreatmentLocationPoint"]}

name_for_cid[9554] = "EquipmentReferencePoint"
cid_concepts[9554] = {
    "DCM": [
        "TreatmentMachineIsocenter",
        "NominalRadiationSourceLocation",
        "FixedLaserSetupPoint",
    ]
}

name_for_cid[9555] = "RadiotherapyTreatmentPlanningPersonRole"
cid_concepts[9555] = {
    "DCM": ["PhysicsAssistant"],
    "NCIt": ["Dosimetrist"],
    "SCT": ["Resident", "MedicalPractitioner", "Physician"],
    "UMLS": ["ConsultingPhysician", "RadiationPhysicist", "MedicalPhysicist"],
}

name_for_cid[9556] = "RadiotherapyRoboticNodeSet"
cid_concepts[9556] = {
    "DCM": ["QANode", "QANodePair", "TrigeminalNodeSet", "BodyNodeSet", "HeadNodeSet"]
}

name_for_cid[9557] = "TomotherapeuticDosimeterUnit"
cid_concepts[9557] = {"UCUM": ["MonitorUnits", "Second"]}

name_for_cid[9558] = "TomotherapeuticDoseRateUnit"
cid_concepts[9558] = {"UCUM": ["GrayPerSecond", "MonitorUnitsPerSecond"]}

name_for_cid[9559] = "RoboticDeliveryDeviceDosimeterUnit"
cid_concepts[9559] = {"UCUM": ["MonitorUnits"]}

name_for_cid[9560] = "RoboticDeliveryDeviceDoseRateUnit"
cid_concepts[9560] = {"UCUM": ["GrayPerSecond"]}

name_for_cid[9561] = "TreatmentTerminationReason"
cid_concepts[9561] = {
    "DCM": [
        "PatientConditionPreventedContinuing",
        "MachineClearanceInterlock",
        "DiscontinuedForUnspecifiedReason",
        "OutOfToleranceInterlock",
        "AccessoryClearanceInterlock",
        "PatientClearanceInterlock",
        "ScanSpotPositionInterlock",
        "ScanSpotInterlock",
        "OperatorError",
        "MLCPositionInterlock",
        "PatientMovement",
        "RadiationDetectedDuringBeamHoldInterlock",
        "ScanSpotMetersetInterlock",
        "ScanSpotSizeInterlock",
        "PatientDied",
        "SecondaryFluenceMonitoringSystemInterlock",
        "PrimaryFluenceMonitoringSystemInterlock",
        "EquipmentFailure",
        "DoctorCanceledProcedure",
        "PatientMotionInterlock",
        "BeamEnergyInterlock",
        "ChannelObstruction",
        "DoorInterlock",
        "MetersetRateInterlock",
        "TimerInterlock",
        "ClearanceInterlock",
        "PatientRefusedToContinueProcedure",
        "DoseLimitInterlock",
    ]
}

name_for_cid[9562] = "RadiotherapyTreatmentDeliveryPersonRole"
cid_concepts[9562] = {
    "DCM": ["PhysicsAssistant"],
    "NCIt": ["Dosimetrist"],
    "SCT": ["Resident", "MedicalPractitioner", "RadiationTherapist", "Physician"],
    "UMLS": ["ConsultingPhysician", "RadiationPhysicist", "MedicalPhysicist"],
}

name_for_cid[9563] = "RadiotherapyInterlockResolution"
cid_concepts[9563] = {
    "DCM": [
        "InterlockOverridden",
        "TreatmentTerminated",
        "InterlockResolved",
        "PatientRepositioned",
    ]
}

name_for_cid[9564] = "TreatmentSessionConfirmationAssertion"
cid_concepts[9564] = {
    "DCM": [
        "BlockPresent",
        "Conepresent",
        "BolusPresent",
        "ApplicatorPresent",
        "HeadframePresent",
    ]
}

name_for_cid[9565] = "TreatmentToleranceViolationCause"
cid_concepts[9565] = {
    "DCM": [
        "MachineCalibrationAdjustment",
        "ChangeInPatientAnatomy",
        "MachineNotAvailable",
        "InappropriatePatientPosition",
        "MachineCapabilityLicenseExpired",
        "UnavailabilityOfABeamModifier",
        "InappropriatePatientOrientation",
    ]
}

name_for_cid[9566] = "ClinicalToleranceViolationType"
cid_concepts[9566] = {
    "DCM": [
        "MetersetToleranceViolation",
        "BeamTargetingToleranceViolation",
        "DeliveryRateToleranceViolation",
    ]
}

name_for_cid[9567] = "MachineToleranceViolationType"
cid_concepts[9567] = {
    "DCM": [
        "DoseRateToleranceViolation",
        "SourcePositionToleranceViolation",
        "MLCPositionToleranceViolation",
        "JawPositionToleranceViolation",
    ]
}

name_for_cid[9568] = "RadiotherapyTreatmentInterlock"
cid_concepts[9568] = {
    "DCM": [
        "SecondaryFluenceMonitoringSystemInterlock",
        "MachineClearanceInterlock",
        "PrimaryFluenceMonitoringSystemInterlock",
        "PatientMotionInterlock",
        "OutOfToleranceInterlock",
        "BeamEnergyInterlock",
        "DoorInterlock",
        "AccessoryClearanceInterlock",
        "MetersetRateInterlock",
        "TimerInterlock",
        "PatientClearanceInterlock",
        "ScanSpotPositionInterlock",
        "ScanSpotInterlock",
        "MLCPositionInterlock",
        "RadiationDetectedDuringBeamHoldInterlock",
        "ClearanceInterlock",
        "ScanSpotMetersetInterlock",
        "DoseLimitInterlock",
        "ScanSpotSizeInterlock",
    ]
}

name_for_cid[9569] = "IsocentricPatientSupportPositionParameter"
cid_concepts[9569] = {
    "DCM": [
        "IsocentricPatientSupportContinuousPitchAngle",
        "IsocentricPatientSupportContinuousRollAngle",
        "IsocentricPatientSupportContinuousYawAngle",
        "IsocentricPatientSupportLateralPosition",
        "IsocentricPatientSupportLongitudinalPosition",
        "IsocentricPatientSupportVerticalPosition",
    ]
}

name_for_cid[9570] = "RTOverriddenTreatmentParameter"
cid_concepts[9570] = {
    "DCM": [
        "IEC61217TableTopLongitudinalPosition",
        "IEC61217TableTopVerticalPosition",
        "IEC61217GantryContinuousRollAngle",
        "IEC61217GantryContinuousPitchAngle",
        "IEC61217GantryContinuousYawAngle",
        "IsocentricPatientSupportContinuousPitchAngle",
        "IEC61217PatientSupportContinuousYawAngle",
        "IsocentricPatientSupportContinuousRollAngle",
        "IEC61217TableTopContinuousPitchAngle",
        "IsocentricPatientSupportContinuousYawAngle",
        "IEC61217TableTopContinuousRollAngle",
        "IsocentricPatientSupportLateralPosition",
        "IEC61217TableTopEccentricAxisDistance",
        "IsocentricPatientSupportLongitudinalPosition",
        "IEC61217TableTopContinuousEccentricAngle",
        "IsocentricPatientSupportVerticalPosition",
        "IEC61217TableTopLateralPosition",
    ]
}

name_for_cid[9571] = "PatientTreatmentPreparationMethod"
cid_concepts[9571] = {
    "DCM": [
        "IsocentricSetupMethod",
        "SkinAppositionSetupMethod",
        "StereotacticSetupMethod",
        "TBISetupMethod",
        "ControlledSSDSetupMethod",
        "OcularGazeSetupMethod",
    ]
}

name_for_cid[9572] = "PatientShieldingDevice"
cid_concepts[9572] = {
    "DCM": ["IndependentRadiationShield", "CavityRadiationShield"],
    "SCT": ["GonadRadiationShield", "EyeRadiationShield"],
}

name_for_cid[9573] = "PatientTreatmentPreparationDevice"
cid_concepts[9573] = {
    "DCM": [
        "RectalBalloon",
        "IndependentRadiationShield",
        "CavityRadiationShield",
        "VaginalCylinder",
        "InfraredMarker",
        "RadiofrequencyTransponder",
        "MechanicalPointer",
        "OpticalDistanceMeter",
        "OcularFixationLight",
        "CombinedStructuredLightThermalImager",
        "ThermalImager",
        "RadioactiveMarker",
        "LaserCrossHairs",
        "OpticalSurfaceScanner",
        "Mold",
        "HeadAndNeckMask",
        "PatientDistractionDevice",
        "HeadMask",
        "CoachingDevice",
        "Headframe",
        "LaserSurfaceScan",
        "BodyFrame",
        "HeadFixationBoard",
        "HandGrips",
        "BreastBoard",
        "AbdominalCompressionArch",
        "HeelStop",
        "AbdominalCompressionBelt",
        "ShinRest",
        "Cast",
        "BreastBridge",
        "WholeBodyPod",
        "VacuumMold",
        "SeatPan",
    ],
    "SCT": [
        "Chair",
        "Armrest",
        "GonadRadiationShield",
        "Table",
        "Stretcher",
        "Backrest",
        "Wall",
        "Headrest",
        "EyeRadiationShield",
        "BiteBlock",
        "Floor",
    ],
}

name_for_cid[9574] = "PatientPositionDisplacementReferencePoint"
cid_concepts[9574] = {
    "DCM": [
        "PatientSetupPoint",
        "MoveableLaserPatientSetupPoint",
        "RoomLaserPatientSetupPoint",
        "IsocentricTreatmentLocationPoint",
    ]
}

name_for_cid[9575] = "PatientAlignmentDevice"
cid_concepts[9575] = {
    "DCM": [
        "LaserSurfaceScan",
        "InfraredMarker",
        "RadiofrequencyTransponder",
        "MechanicalPointer",
        "OpticalDistanceMeter",
        "OcularFixationLight",
        "CombinedStructuredLightThermalImager",
        "ThermalImager",
        "RadioactiveMarker",
        "LaserCrossHairs",
        "OpticalSurfaceScanner",
    ]
}

name_for_cid[9576] = "ReasonsforRTRadiationTreatmentOmission"
cid_concepts[9576] = {
    "DCM": [
        "RTRadiationPreviouslyDelivered",
        "TreatmentExceedsPatientTolerance",
        "TreatmentUnnecessary",
    ]
}

name_for_cid[9577] = "PatientTreatmentPreparationProcedure"
cid_concepts[9577] = {
    "DCM": [
        "CollisionChecks",
        "PatientAlignmentProcedure",
        "PatientFixationProcedure",
        "PatientShieldingProcedure",
        "PatientMotionManagementSetupProcedure",
    ],
    "SCT": ["PlacingRestraint", "Sedation", "VitalSignsMonitoring"],
}

name_for_cid[9578] = "MotionManagementSetupDevice"
cid_concepts[9578] = {"DCM": ["PatientDistractionDevice", "CoachingDevice"]}

name_for_cid[9579] = "RTSegmentMaterial"
cid_concepts[9579] = {
    "DCM": ["TitaniumAlloyTitaniumMolybdenum15Zirconium5Aluminium3"],
    "SCT": [
        "Polymer",
        "CarbonFiber",
        "StainlessSteelMaterial",
        "GoldAlloy",
        "NickelTitanium",
    ],
}

name_for_cid[9580] = "RTSegmentationPropertyCategory"
cid_concepts[9580] = {
    "DCM": [
        "RTGeometricInformation",
        "RTTarget",
        "ExternalBodyModel",
        "NonSpecificVolume",
        "BrachytherapyDevice",
        "FixationOrPositioningDevice",
        "RadiotherapyFiducial",
        "RadiotherapyDoseRegion",
        "PatientAttachedDoseControlObject",
    ],
    "SCT": [
        "Function",
        "Tissue",
        "SpatialAndRelationalConcept",
        "Substance",
        "MorphologicallyAbnormalStructure",
        "AnatomicalStructure",
        "PhysicalObject",
        "BodySubstance",
    ],
}

name_for_cid[9581] = "RadiotherapyRegistrationMark"
cid_concepts[9581] = {"DCM": ["FiducialMark"]}

name_for_cid[9582] = "RadiotherapyDoseRegion"
cid_concepts[9582] = {"DCM": ["IsodoseVolume"]}

name_for_cid[9583] = "RTPlanRadiotherapyProcedureTechnique"
cid_concepts[9583] = {
    "DCM": [
        "StaticBeam",
        "SlidingWindowBeam",
        "StepAndShootBeam",
        "ConformalArcBeam",
        "ArcBeam",
        "TopographicBeam",
        "HelicalBeam",
        "VMAT",
    ]
}

name_for_cid[10000] = "ScopeofAccumulation"
cid_concepts[10000] = {
    "DCM": [
        "ProcedureStepToThisPoint",
        "Study",
        "Series",
        "PerformedProcedureStep",
        "IrradiationEvent",
    ]
}

name_for_cid[10001] = "UIDType"
cid_concepts[10001] = {
    "DCM": [
        "StudyInstanceUID",
        "PerformedProcedureStepSOPInstanceUID",
        "SeriesInstanceUID",
        "IrradiationEventUID",
    ]
}

name_for_cid[10002] = "IrradiationEventType"
cid_concepts[10002] = {
    "DCM": ["StationaryAcquisition", "SteppingAcquisition", "RotationalAcquisition"],
    "SCT": ["Fluoroscopy"],
}

name_for_cid[10003] = "EquipmentPlaneIdentification"
cid_concepts[10003] = {"DCM": ["AllPlanes", "PlaneA", "PlaneB", "SinglePlane"]}

name_for_cid[10004] = "FluoroMode"
cid_concepts[10004] = {"DCM": ["Continuous", "Pulsed"]}

name_for_cid[10005] = "XRayRadiationDoseProcedureTypeReported"
cid_concepts[10005] = {
    "SCT": [
        "ConeBeamComputedTomography",
        "DualEnergyXRayAbsorptiometry",
        "DentalRadiography",
        "PlainRadiography",
        "ComputedTomography",
        "Fluoroscopy",
        "FluoroscopyAndRadiography",
        "Mammography",
    ]
}

name_for_cid[10006] = "XRayFilterMaterial"
cid_concepts[10006] = {
    "SCT": [
        "Copper",
        "Niobium",
        "Tin",
        "Molybdenum",
        "Silver",
        "Aluminum",
        "Europium",
        "Tantalum",
        "Rhodium",
        "Lead",
    ]
}

name_for_cid[10007] = "XRayFilterType"
cid_concepts[10007] = {
    "DCM": ["StripFilter", "NoFilter", "WedgeFilter", "ButterflyFilter", "FlatFilter"]
}

name_for_cid[10008] = "DoseRelatedDistanceMeasurement"
cid_concepts[10008] = {
    "DCM": [
        "DistanceSourceToTablePlane",
        "DistanceSourceToDetector",
        "TableHeightEndPosition",
        "TableLateralEndPosition",
        "TableLateralPosition",
        "TableLongitudinalPosition",
        "TableXEndPositionToIsocenter",
        "TableZPositionToIsocenter",
        "TableZEndPositionToIsocenter",
        "TableYPositionToIsocenter",
        "TableXPositionToIsocenter",
        "TableHeightPosition",
        "TableYEndPositionToIsocenter",
        "DistanceSourceToIsocenter",
        "DistanceSourceToReferencePoint",
        "TableLongitudinalEndPosition",
    ]
}

name_for_cid[10009] = "Measured/Calculated"
cid_concepts[10009] = {"SCT": ["Estimated", "Measured", "Calculated"]}

name_for_cid[10010] = "DoseMeasurementDevice"
cid_concepts[10010] = {"SCT": ["Dosimeter"]}

name_for_cid[10011] = "EffectiveDoseEvaluationMethod"
cid_concepts[10011] = {
    "DCM": [
        "DLPToEConversionViaMCComputation",
        "DLPToEConversionViaMeasurement",
        "CtdifreeairToEConversionViaMCComputation",
        "CtdifreeairToEConversionViaMeasurement",
    ]
}

name_for_cid[10013] = "CTAcquisitionType"
cid_concepts[10013] = {
    "DCM": [
        "StationaryAcquisitionCT",
        "ConstantAngleAcquisition",
        "FreeAcquisition",
        "SequencedAcquisition",
    ],
    "SCT": ["SpiralAcquisition", "ConeBeamAcquisition"],
}

name_for_cid[10014] = "ContrastImagingTechnique"
cid_concepts[10014] = {
    "SCT": ["DiagnosticRadiographyWithContrastMedia", "CTWithoutContrast"]
}

name_for_cid[10015] = "CTDoseReferenceAuthority"
cid_concepts[10015] = {"DCM": ["ICRPPub60", "ICRPPub103"]}

name_for_cid[10016] = "AnodeTargetMaterial"
cid_concepts[10016] = {"SCT": ["Molybdenum", "Tungsten", "Rhodium"]}

name_for_cid[10017] = "XRayGrid"
cid_concepts[10017] = {
    "DCM": [
        "VirtualGrid",
        "FixedGrid",
        "FocusedGrid",
        "ReciprocatingGrid",
        "ParallelGrid",
        "CrossedGrid",
        "NoGrid",
    ]
}

name_for_cid[10020] = "SourceofProjectionXRayDoseInformation"
cid_concepts[10020] = {
    "DCM": [
        "SystemCalculated",
        "CopiedFromImageAttributes",
        "ManualEntry",
        "DerivedFromHumanReadableReports",
        "AutomatedDataCollection",
        "ComputedFromImageAttributes",
        "MPPSContent",
    ],
    "SCT": ["Dosimeter"],
}

name_for_cid[10021] = "SourceofCTDoseInformation"
cid_concepts[10021] = {
    "DCM": [
        "CopiedFromImageAttributes",
        "ManualEntry",
        "DerivedFromHumanReadableReports",
        "AutomatedDataCollection",
        "ComputedFromImageAttributes",
    ]
}

name_for_cid[10022] = "LabelType"
cid_concepts[10022] = {"DCM": ["SeriesNumber", "AcquisitionNumber", "InstanceNumber"]}

name_for_cid[10023] = "SizeSpecificDoseEstimationMethodforCT"
cid_concepts[10023] = {
    "DCM": [
        "AAPM204EffectiveDiameterEstimatedFromPatientAge",
        "AAPM204LateralDimension",
        "ArithmeticAverageOfSSDEZ",
        "EstimatedFromWaterEquivalentDiameter",
        "AAPM204SumOfLateralAndAPDimension",
        "AAPM204APDimension",
    ]
}

name_for_cid[10024] = "WaterEquivalentDiameterMethod"
cid_concepts[10024] = {
    "DCM": [
        "WaterEquivalentDiameterRepresentativeValue",
        "WaterEquivalentDiameterFromRawData",
        "WaterEquivalentDiameterIntegratedAcrossScanRange",
        "WaterEquivalentDiameterFromLocalizer",
    ]
}

name_for_cid[10025] = "RadiationDoseReferencePoint"
cid_concepts[10025] = {
    "DCM": [
        "_15cmFromIsocenterTowardSource",
        "_1cmAboveTabletop",
        "_30cmInFrontOfImageInputSurface",
        "_15cmFromTableCenterline",
        "InDetectorPlane",
        "_30cmAboveTabletop",
        "_100cmFromXRaySource",
        "_4Point2cmAboveBreastSupportSurface",
        "AtSurfaceOfPatient",
    ]
}

name_for_cid[10030] = "DetectorType"
cid_concepts[10030] = {
    "DCM": ["IndirectDetector", "DirectDetector", "StorageDetector", "FilmDetectorType"]
}

name_for_cid[10031] = "CR/DRMechanicalConfiguration"
cid_concepts[10031] = {
    "DCM": ["TableMount", "UprightStandMount", "UnmountedDetector", "CArmMount"]
}

name_for_cid[10032] = "ProjectionXRayAcquisitionDeviceType"
cid_concepts[10032] = {
    "DCM": [
        "CassetteBasedProjectionRadiographySystem",
        "IntegratedProjectionRadiographySystem",
        "FluoroscopyGuidedProjectionRadiographySystem",
    ]
}

name_for_cid[10033] = "CTReconstructionAlgorithm"
cid_concepts[10033] = {"DCM": ["IterativeReconstruction", "FilteredBackProjection"]}

name_for_cid[10034] = "ReasonforRepeatingAcquisition"
cid_concepts[10034] = {"DCM": ["SuboptimalContrastTiming", "PatientMotion"]}

name_for_cid[10040] = "RadiopharmaceuticalOrganDoseReferenceAuthority"
cid_concepts[10040] = {
    "DCM": [
        "InvestigationalNewDrug",
        "MIRDPamphlet1",
        "ICRPPublication53",
        "ICRPPublication80",
        "ICRPPublication106",
        "MIRDOSE",
        "OLINDAEXM",
        "PackageInsert",
        "InstitutionallyApprovedEstimates",
    ]
}

name_for_cid[10041] = "SourceofRadioisotopeActivityInformation"
cid_concepts[10041] = {
    "DCM": ["DoseCalibrator", "InfusionSystem", "RadioisotopeGenerator"]
}

name_for_cid[10043] = "IntravenousExtravasationSymptom"
cid_concepts[10043] = {
    "DCM": ["ExtravasationVisibleInImage"],
    "SCT": [
        "InjectionSiteBruising",
        "InjectionSiteThrombosis",
        "InjectionSiteAbscess",
        "InjectionSiteNerveDamage",
        "InjectionSiteFibrosis",
        "InjectionSiteHypersensitivity",
        "InjectionSitePigmentationChange",
        "InjectionSiteScar",
        "InjectionSiteIrritation",
        "InjectionSiteAnesthesia",
        "InjectionSiteCyst",
        "InjectionSiteInduration",
        "InjectionSiteEdema",
        "InjectionSiteUrticaria",
        "InjectionSiteAtrophy",
        "InjectionSiteBurning",
        "InjectionSiteHemorrhage",
        "InjectionSiteUlcer",
        "InjectionSiteMalabsorption",
        "InjectionSiteSterileAbscess",
        "InjectionSiteInflammation",
        "InjectionSiteInfection",
        "InjectionSiteNecrosis",
        "InjectionSiteParesthesia",
        "InjectionSiteDermatitis",
        "InjectionSiteDisorder",
        "InjectionSitePain",
        "InjectionSiteMass",
        "InjectionSiteGranuloma",
    ],
}

name_for_cid[10044] = "RadiosensitiveOrgan"
cid_concepts[10044] = {
    "SCT": [
        "Lung",
        "SmallIntestine",
        "Breast",
        "Stomach",
        "Heart",
        "Skin",
        "Thymus",
        "Prostate",
        "LymphNode",
        "Colon",
        "Uterus",
        "SalivaryGlands",
        "Gallbladder",
        "Testis",
        "Esophagus",
        "Bladder",
        "Brain",
        "EyeLenses",
        "OralMucosa",
        "Ovary",
        "Muscle",
        "Pancreas",
        "BoneMarrow",
        "Liver",
        "Thyroid",
        "Spleen",
        "Kidney",
        "AdrenalGland",
        "BoneSurface",
    ]
}

name_for_cid[10045] = "RadiopharmaceuticalPatientState"
cid_concepts[10045] = {
    "DCM": [
        "CardiacStressState",
        "AcuteUnilateralRenalBlockage",
        "LowThyroidUptake",
        "HighThyroidUptake",
        "SeverelyJaundiced",
    ],
    "SCT": ["AbnormalRenalFunction", "RestingState"],
}

name_for_cid[10046] = "GFRMeasurement"
cid_concepts[10046] = {
    "LN": [
        "GlomerularFiltrationRateBlackMDRD",
        "CockroftGaultFormulaEstimationOfGFR",
        "GlomerularFiltrationRateCystatinBasedFormula",
        "CKDEPIFormulaEstimationOfGFR",
        "GlomerularFiltrationRateMDRD",
        "GlomerularFiltrationRateCreatinineBasedFormulaSchwartz",
        "GlomerularFiltrationRateNonBlackMDRD",
        "GlomerularFiltrationRateFemaleMDRD",
    ]
}

name_for_cid[10047] = "GFRMeasurementMethod"
cid_concepts[10047] = {
    "DCM": [
        "CockroftGaultFormulaEstimationOfGFR",
        "CKDEPIFormulaEstimationOfGFR",
        "GlomerularFiltrationRateMDRD",
        "GlomerularFiltrationRateNonBlackMDRD",
        "GlomerularFiltrationRateBlackMDRD",
        "GlomerularFiltrationRateFemaleMDRD",
        "GlomerularFiltrationRateCystatinBasedFormula",
        "GlomerularFiltrationRateCreatinineBasedFormulaSchwartz",
    ]
}

name_for_cid[10050] = "SummaryRadiationExposureQuantity"
cid_concepts[10050] = {
    "DCM": [
        "EffectiveDose",
        "TotalFluoroTime",
        "TotalNumberOfRadiographicFrames",
        "MeanCtdivol",
        "DoseAreaProductTotal",
        "CTDoseLengthProductSubTotal",
        "CTDoseLengthProductTotal",
        "EntranceExposureAtRP",
        "AcquisitionDoseAreaProductTotal",
        "AccumulatedAverageGlandularDose",
        "FluoroDoseAreaProductTotal",
        "AdministeredActivity",
    ]
}

name_for_cid[10060] = "OrganforRadiationDoseEstimate"
cid_concepts[10060] = {
    "SCT": [
        "Lung",
        "SmallIntestine",
        "Breast",
        "Stomach",
        "Phantom",
        "Heart",
        "Skin",
        "Thymus",
        "Prostate",
        "LymphNode",
        "Colon",
        "Uterus",
        "SalivaryGlands",
        "Gallbladder",
        "Testis",
        "Esophagus",
        "Bladder",
        "Brain",
        "EyeLenses",
        "OralMucosa",
        "Ovary",
        "Muscle",
        "Pancreas",
        "BoneMarrow",
        "Liver",
        "EntireBody",
        "Thyroid",
        "Spleen",
        "Kidney",
        "AdrenalGland",
        "BoneSurface",
    ]
}

name_for_cid[10061] = "AbsorbedRadiationDoseType"
cid_concepts[10061] = {
    "DCM": [
        "MinimumAbsorbedRadiationDose",
        "MaximumAbsorbedRadiationDose",
        "ModeAbsorbedRadiationDose",
        "MeanAbsorbedRadiationDose",
        "MedianAbsorbedRadiationDose",
    ]
}

name_for_cid[10062] = "EquivalentRadiationDoseType"
cid_concepts[10062] = {
    "DCM": [
        "MedianEquivalentRadiationDose",
        "MinimumEquivalentRadiationDose",
        "MaximumEquivalentRadiationDose",
        "ModeEquivalentRadiationDose",
        "MeanEquivalentRadiationDose",
    ]
}

name_for_cid[10063] = "RadiationDoseEstimateDistributionRepresentation"
cid_concepts[10063] = {
    "DCM": [
        "SkinDoseMap",
        "DosePointCloud",
        "DoseGradient",
        "_3DDoseMap",
        "DoseImage",
        "Isodose",
    ]
}

name_for_cid[10064] = "PatientModelType"
cid_concepts[10064] = {
    "DCM": ["AnthropomorphicModel", "SimpleObjectModel", "PatientSegmentedModel"]
}

name_for_cid[10065] = "RadiationTransportModelType"
cid_concepts[10065] = {
    "DCM": [
        "MeasuredRadiationDose",
        "VoxelizedRadiationTransportModel",
        "GeometricRadiationTransportModel",
        "NURBSRadiationTransportModel",
        "MeshRadiationTransportModel",
        "BREPRadiationTransportModel",
    ]
}

name_for_cid[10066] = "AttenuatorCategory"
cid_concepts[10066] = {
    "DCM": [
        "BeamBlock",
        "XRayFilters",
        "Table",
        "TableCore",
        "PatientSupport",
        "TablePad",
        "TableOuterLiner",
    ],
    "SCT": ["XRayShield", "ShieldingBlock"],
}

name_for_cid[10067] = "RadiationAttenuatorMaterial"
cid_concepts[10067] = {
    "SCT": [
        "CarbonFiber",
        "Tin",
        "Tantalum",
        "Rhodium",
        "Lead",
        "Copper",
        "Niobium",
        "Molybdenum",
        "Beryllium",
        "Silver",
        "Aluminum",
        "Europium",
    ],
    "UMLS": ["KevlarAramidFiber"],
}

name_for_cid[10068] = "EstimateMethodType"
cid_concepts[10068] = {
    "DCM": ["TabularDataAlgorithm", "AnalyticalAlgorithm", "EmpiricalAlgorithm"],
    "MSH": ["MonteCarloMethod"],
}

name_for_cid[10069] = "RadiationDoseEstimateParameter"
cid_concepts[10069] = {
    "DCM": [
        "Distance",
        "AttenuationCoefficient",
        "WaterEquivalentDiameterRepresentativeValue",
        "WaterEquivalentDiameterFromRawData",
        "WaterEquivalentDiameterIntegratedAcrossScanRange",
        "PercentFibroglandularTissue",
        "DistanceCorrection",
        "BreastThickness",
        "TissueFraction",
        "PatientAPDimension",
        "Dgn",
        "PatientLateralDimension",
        "CurveFitParameter",
        "CorrectionFactor",
        "Backscatter",
        "TissueAirRatio",
        "HomogeneityFactor",
        "SSDEConversionFactor",
        "OffsetFactor",
        "NormalizationFactor",
        "WaterEquivalentDiameterFromLocalizer",
        "HalfValueLayer",
    ],
    "NCIt": ["UnitConversionFactor"],
}

name_for_cid[10070] = "RadiationDoseType"
cid_concepts[10070] = {"DCM": ["EquivalentDose", "AbsorbedDose"]}

name_for_cid[10071] = "RadiationDoseUnit"
cid_concepts[10071] = {"UCUM": ["Sv", "Gy"]}

name_for_cid[10072] = "ReportedValueType"
cid_concepts[10072] = {
    "SCT": ["Mode", "Nominal", "Minimum", "Yes", "Maximum", "Mean", "Median"]
}

name_for_cid[10073] = "ValueTiming"
cid_concepts[10073] = {
    "DCM": [
        "MiddleOfTimePeriod",
        "EndOfTimePeriod",
        "BeginningOfTimePeriod",
        "DurationOfTimePeriod",
    ]
}

name_for_cid[10074] = "RDSRFrameofReferenceOrigin"
cid_concepts[10074] = {
    "DCM": [
        "PatientCoordinateSystemOrigin",
        "IsocenterOrigin",
        "PatientSupportOrigin",
        "EquipmentOrigin",
        "RoomOrigin",
    ]
}

name_for_cid[11001] = "PhotoacousticIlluminationMethod"
cid_concepts[11001] = {
    "DCM": [
        "SingleSideIllumination",
        "InterstitialIllumination",
        "ThroughTransducerIllumination",
        "MultiSideIllumination",
        "DualSideIllumination",
    ]
}

name_for_cid[11002] = "AcousticCouplingMedium"
cid_concepts[11002] = {
    "SCT": ["UltrasoundCouplingGel", "DeuteriumOxide", "Water", "Air"]
}

name_for_cid[11003] = "UltrasoundTransducerTechnology"
cid_concepts[11003] = {
    "DCM": [
        "InterferometricTransducer",
        "MEMSBasedTransducer",
        "PiezocompositeTransducer",
    ]
}

name_for_cid[11004] = "SpeedofSoundCorrectionMechanisms"
cid_concepts[11004] = {
    "DCM": [
        "DualSpeedOfSoundCorrection",
        "SpeedOfSoundMapCorrection",
        "UniformSpeedOfSoundCorrection",
    ]
}

name_for_cid[11005] = "PhotoacousticReconstructionAlgorithmFamily"
cid_concepts[11005] = {"DCM": ["SphericalBackProjection", "NumericalAcousticModel"]}

name_for_cid[11006] = "PhotoacousticImagedProperty"
cid_concepts[11006] = {
    "DCM": [
        "Perfusion",
        "Elasticity",
        "BloodOxygenationLevel",
        "BloodDeoxygenationLevel",
    ],
    "SCT": ["Hemoglobin", "Melanin", "Water", "Lipid"],
}

name_for_cid[12001] = "UltrasoundProtocolType"
cid_concepts[12001] = {
    "SCT": [
        "TransthoracicEchocardiography",
        "EllestadProtocol",
        "DobutamineStressProtocol",
        "NaughtonProtocol",
        "StressTestUsingCardiacPacing",
        "PharmacologicAndExerciseStressTest",
        "ModifiedNaughtonProtocol",
        "RampProtocol",
        "IntravascularEchocardiography",
        "BalkeProtocol",
        "BruceProtocol",
        "ExerciseStressEchocardiography",
        "FetalEchocardiography",
        "StressTestUsingBicycleErgometer",
        "IntraoperativeEchocardiography",
        "ContrastEchocardiography",
        "PharmacologicStressProtocol",
        "PepperProtocol",
        "AdenosineStressProtocol",
        "IntracardiacEchocardiography",
        "DipyridamoleStressProtocol",
        "EpicardialEchocardiography",
        "TransesophagealEchocardiography",
        "ExerciseStressECGTest",
        "ModifiedBruceProtocol",
        "Echocardiography",
        "PediatricEchocardiography",
    ]
}

name_for_cid[12002] = "UltrasoundProtocolStageType"
cid_concepts[12002] = {
    "SCT": [
        "AfterProcedure",
        "ValsalvaManeuver",
        "CardiacStressState",
        "Hyperventilation",
        "BeforeProcedure",
        "PeakCardiacStressState",
        "CardiacPacing",
        "CardiacStressRecoveryState",
        "DuringProcedure",
        "HandGrip",
        "RestingState",
    ]
}

name_for_cid[12003] = "OBGYNDate"
cid_concepts[12003] = {
    "LN": [
        "ConceptionDate",
        "LMP",
        "EstimatedLMPByEDD",
        "EDDFromAverageUltrasoundAge",
        "EDDFromOvulationDate",
        "EDD",
        "EDDFromLMP",
        "OvulationDate",
    ]
}

name_for_cid[12004] = "FetalBiometryRatio"
cid_concepts[12004] = {"LN": ["HCAC", "FLAC", "CephalicIndex", "FLBPD", "FLHC"]}

name_for_cid[12005] = "FetalBiometryMeasurement"
cid_concepts[12005] = {
    "LN": [
        "OccipitalFrontalDiameter",
        "BPDAreaCorrected",
        "LeftFetalEarLength",
        "TransverseAbdominalDiameter",
        "APADTAD",
        "CisternaMagnaLength",
        "AbdominalCircumference",
        "FemurLength",
        "ThoracicCircumference",
        "HeadCircumference",
        "FootLength",
        "BiparietalDiameter",
        "LeftKidneyThickness",
        "RightFetalEarLength",
        "TransverseThoracicDiameter",
        "LeftKidneyLength",
        "LeftKidneyWidth",
        "AnteriorPosteriorAbdominalDiameter",
        "ThoracicArea",
        "RightKidneyThickness",
        "AnteriorPosteriorTrunkDiameter",
        "RightKidneyWidth",
        "TransverseCerebellarDiameter",
        "RightKidneyLength",
    ]
}

name_for_cid[12006] = "FetalLongBonesBiometryMeasurement"
cid_concepts[12006] = {
    "LN": [
        "ClavicleLength",
        "FemurLength",
        "FibulaLength",
        "HumerusLength",
        "RadiusLength",
        "TibiaLength",
        "UlnaLength",
    ]
}

name_for_cid[12007] = "FetalCraniumMeasurement"
cid_concepts[12007] = {
    "LN": [
        "NuchalFoldThickness",
        "InnerOrbitalDiameter",
        "CisternaMagnaLength",
        "PosteriorHornLateralVentricularWidth",
        "OuterOrbitalDiameter",
        "WidthOfHemisphere",
        "LateralVentricleWidth",
        "AnteriorHornLateralVentricularWidth",
        "NuchalTranslucency",
        "TransverseCerebellarDiameter",
    ]
}

name_for_cid[12008] = "OBGYNAmnioticSacMeasurement"
cid_concepts[12008] = {
    "LN": [
        "ThirdQuadrantDiameter",
        "SecondQuadrantDiameter",
        "AmnioticFluidIndex",
        "FourthQuadrantDiameter",
        "FirstQuadrantDiameter",
    ],
    "SCT": ["Diameter"],
}

name_for_cid[12009] = "EarlyGestationBiometryMeasurement"
cid_concepts[12009] = {
    "LN": [
        "GestationalSacDiameter",
        "CrownRumpLength",
        "SpineLength",
        "YolkSacLength",
        "NuchalTranslucency",
    ]
}

name_for_cid[12011] = "UltrasoundPelvisandUterusMeasurement"
cid_concepts[12011] = {"LN": ["EndometriumThickness", "CervixLength"]}

name_for_cid[12012] = "OBEquation/Table"
cid_concepts[12012] = {
    "DCM": ["FWPByGACampbell1991", "FWPByGAHadlock1991"],
    "LN": [
        "ACJeanty1984",
        "CRLByGARempen1991",
        "ACJeanty1982",
        "OFDASUM2000",
        "FibulaByGAJeanty1983",
        "CRLHansmann1985",
        "FemaleSingletonBWPByGAArbuckle1993",
        "FetalTrunkCrossSectionalAreaOsaka1989",
        "BPDMertz1988",
        "HCJeanty1984",
        "EFWByACFLHCHadlock1985",
        "FLByGAJeanty1982",
        "HCMerz1988",
        "RadiusJeanty1983",
        "CephalicIndexByGAChitty1994",
        "FLTokyo1986",
        "EFWByACBPDFLHadlock1984",
        "CRLHansmann1986",
        "ACByGAASUM2000",
        "BPDTokyo1986",
        "IODMayden1982",
        "TibiaJeanty1984",
        "CRLASUM1991",
        "EFW1ByShinozuka1996",
        "GestationalAgeByLMP",
        "MaleTwinsBWPByGAArbuckle1993",
        "BDJeanty1982",
        "HCByGAASUM2000",
        "BPDByGAJeanty1982",
        "BPDDoubilet1993",
        "GSTokyo1986",
        "EFWByACBPDHadlock1984",
        "HCByGAMerz1988",
        "TCDHill1990",
        "CRLOsaka1989",
        "HCDerivedChitty1997",
        "BPDOuterInnerByGAChitty1994",
        "TADTokyo1986",
        "ACLessoway1998",
        "AxtShinozuka1996",
        "FibulaJeanty1983",
        "RadiusMerz1987",
        "EFWByGAHansmann1986",
        "MaleSingletonBWPByGAArbuckle1993",
        "FLByGAChitty1994",
        "OODTrout1994",
        "EFWByACFLHadlock1984",
        "EFWByBPDAPADTADFLTokyo1987",
        "BPDLessoway1998",
        "FLHansmann1986",
        "FLShinozuka1996",
        "OFDLessoway1998",
        "BPDShinozuka1996",
        "FLJeanty1984",
        "ClavicleLengthYarkoni1985",
        "EFWByBPDFTAFLOsaka1990",
        "FLMerz1987",
        "GSByGARempen1991",
        "BPDJeanty1984",
        "BPDOuterOuterByGAChitty1994",
        "GSHellman1969",
        "HCByGAJeanty1982",
        "BPDByGAShinozuka1996",
        "BPDByGAHadlock1984",
        "OFDByGAChitty1994",
        "EFWByACHCHadlock1984",
        "FLHadlock1984",
        "EFWByACBPDFLHadlock1985",
        "BpdaHadlock1982",
        "ACByGAShinozuka1996",
        "GSNyberg1992",
        "EFWByACAndBPDShepard1982",
        "LengthOfVertebraTokyo1986",
        "CRLJeanty1982",
        "HCMeasuredChitty1997",
        "TransverseThoracicDiameterLessoway1998",
        "HCLessoway1998",
        "BPDHansmann1986",
        "TCDGoldstein1987",
        "ACHansmann1985",
        "APAbdominalDiameterLessoway1998",
        "HCACByGACampbell1977",
        "CRLTokyo1986",
        "FWPByGAHadlock1985",
        "FWPByGAAlexander1996",
        "FLByGAHadlock1984",
        "CRLNelson1981",
        "FWPByGABrenner1976",
        "HumerusJeanty1984",
        "TCDByGAGoldstein1987",
        "EFWByGAHadlock1991",
        "BPDJeanty1982",
        "CRLJeanty1984",
        "FLOsaka1989",
        "HCASUM2000",
        "EFW3ByShinozuka1996",
        "FLMerz1988",
        "TADEriksen1985",
        "EFWByACFLHadlock1985",
        "BPDRempen1991",
        "OFDHansmann1986",
        "ACMeasuredByGAChitty1994",
        "ACDerivedByGAChitty1994",
        "BPDOoChitty1997",
        "FLByGAShinozuka1996",
        "BPDKurtz1980",
        "BPDByGAASUM2000",
        "GSRempen1991",
        "HCDerivedByGAChitty1994",
        "FLHansmann1985",
        "CRLDaya1993",
        "BPDByGARempen1991",
        "TADHansmann1979",
        "BPDHadlock1984",
        "GSHansmann1982",
        "ACByGAMerz1988",
        "OFDByGAASUM2000",
        "EFWByACBPDFLHCHadlock1985",
        "UlnaJeanty1984",
        "CRLByGAASUM2000",
        "HumerusLengthOsaka1989",
        "ACASUM2000",
        "TransverseThoracicDiameterHansmann1985",
        "HCHadlock1984",
        "HCJeanty1982",
        "ACShinozuka1996",
        "CRLShinozuka1996",
        "TCDChitty1994",
        "FLJeanty1982",
        "CRLByGAShinozuka1996",
        "ACHadlock1984",
        "CRLRobinson1975",
        "FWPByGAWilliams1982",
        "FLByGAASUM2000",
        "HumerusMerz1987",
        "FemaleTwinsBWPByGAArbuckle1993",
        "FibulaMerz1987",
        "EFWByBPDTTDHansmann1986",
        "CRLHadlock1992",
        "FLByGAMerz1988",
        "HumerusLengthASUM2000",
        "BPDOiChitty1997",
        "CephalicIndexByGAHadlock1981",
        "FLLessoway1998",
        "HCHansmann1986",
        "SpineLengthTokyo1989",
        "GSDaya1991",
        "ACByGAHadlock1984",
        "IODTrout1994",
        "EFW2ByShinozuka1996",
        "BPDOsaka1989",
        "RadiusByGAJeanty1983",
        "ACCampbell1975",
        "BPDSabbagha1978",
        "AverageUltrasoundAge",
        "AxtByGAShinozuka1996",
        "UlnaMerz1987",
        "FLHohler1982",
        "HCByGAHadlock1984",
        "CRLASUM2000",
        "OFDHansmann1985",
        "BPDHansmann1985",
        "BPDByGAMerz1988",
        "FootLengthMercer1987",
        "HumerusLengthByGAASUM2000",
        "GSHansmann1979",
        "OODMayden1982",
        "EFWByACCampbell1975",
        "CRLRempen1991",
        "TCDNimrod1986",
        "ACMertz1988",
        "HCHansmann1985",
        "ThcChitkara1987",
        "FLChitty1997",
        "BPDASUM1989",
    ],
}

name_for_cid[12013] = "GestationalAgeEquation/Table"
cid_concepts[12013] = {
    "LN": [
        "ACJeanty1984",
        "ACJeanty1982",
        "OFDASUM2000",
        "CRLHansmann1985",
        "FetalTrunkCrossSectionalAreaOsaka1989",
        "BPDMertz1988",
        "HCJeanty1984",
        "HCMerz1988",
        "RadiusJeanty1983",
        "FLTokyo1986",
        "CRLHansmann1986",
        "BPDTokyo1986",
        "IODMayden1982",
        "TibiaJeanty1984",
        "CRLASUM1991",
        "GestationalAgeByLMP",
        "BDJeanty1982",
        "BPDDoubilet1993",
        "GSTokyo1986",
        "TCDHill1990",
        "CRLOsaka1989",
        "HCDerivedChitty1997",
        "TADTokyo1986",
        "ACLessoway1998",
        "AxtShinozuka1996",
        "FibulaJeanty1983",
        "RadiusMerz1987",
        "OODTrout1994",
        "BPDLessoway1998",
        "FLHansmann1986",
        "FLShinozuka1996",
        "OFDLessoway1998",
        "BPDShinozuka1996",
        "FLJeanty1984",
        "ClavicleLengthYarkoni1985",
        "FLMerz1987",
        "BPDJeanty1984",
        "GSHellman1969",
        "FLHadlock1984",
        "BpdaHadlock1982",
        "GSNyberg1992",
        "LengthOfVertebraTokyo1986",
        "CRLJeanty1982",
        "HCMeasuredChitty1997",
        "TransverseThoracicDiameterLessoway1998",
        "HCLessoway1998",
        "BPDHansmann1986",
        "TCDGoldstein1987",
        "ACHansmann1985",
        "APAbdominalDiameterLessoway1998",
        "CRLTokyo1986",
        "CRLNelson1981",
        "HumerusJeanty1984",
        "BPDJeanty1982",
        "CRLJeanty1984",
        "FLOsaka1989",
        "HCASUM2000",
        "FLMerz1988",
        "TADEriksen1985",
        "BPDRempen1991",
        "OFDHansmann1986",
        "BPDOoChitty1997",
        "BPDKurtz1980",
        "GSRempen1991",
        "FLHansmann1985",
        "CRLDaya1993",
        "TADHansmann1979",
        "BPDHadlock1984",
        "GSHansmann1982",
        "UlnaJeanty1984",
        "HumerusLengthOsaka1989",
        "ACASUM2000",
        "TransverseThoracicDiameterHansmann1985",
        "HCHadlock1984",
        "HCJeanty1982",
        "ACShinozuka1996",
        "CRLShinozuka1996",
        "TCDChitty1994",
        "FLJeanty1982",
        "ACHadlock1984",
        "CRLRobinson1975",
        "HumerusMerz1987",
        "FibulaMerz1987",
        "CRLHadlock1992",
        "HumerusLengthASUM2000",
        "BPDOiChitty1997",
        "FLLessoway1998",
        "HCHansmann1986",
        "SpineLengthTokyo1989",
        "GSDaya1991",
        "IODTrout1994",
        "BPDOsaka1989",
        "ACCampbell1975",
        "BPDSabbagha1978",
        "AverageUltrasoundAge",
        "UlnaMerz1987",
        "FLHohler1982",
        "CRLASUM2000",
        "OFDHansmann1985",
        "BPDHansmann1985",
        "FootLengthMercer1987",
        "GSHansmann1979",
        "OODMayden1982",
        "CRLRempen1991",
        "TCDNimrod1986",
        "ACMertz1988",
        "HCHansmann1985",
        "ThcChitkara1987",
        "FLChitty1997",
        "BPDASUM1989",
    ]
}

name_for_cid[12014] = "OBFetalBodyWeightEquation/Table"
cid_concepts[12014] = {
    "LN": [
        "EFWByACHCHadlock1984",
        "EFWByACBPDFLHadlock1985",
        "EFWByBPDTTDHansmann1986",
        "EFWByACBPDFLHCHadlock1985",
        "EFWByACFLHCHadlock1985",
        "EFWByACBPDHadlock1984",
        "EFWByACCampbell1975",
        "EFWByACAndBPDShepard1982",
        "EFWByACFLHadlock1984",
        "EFW3ByShinozuka1996",
        "EFWByBPDAPADTADFLTokyo1987",
        "EFWByACBPDFLHadlock1984",
        "EFWByACFLHadlock1985",
        "EFWByBPDFTAFLOsaka1990",
        "EFW1ByShinozuka1996",
        "EFW2ByShinozuka1996",
    ]
}

name_for_cid[12015] = "FetalGrowthEquation/Table"
cid_concepts[12015] = {
    "LN": [
        "CRLByGAShinozuka1996",
        "CRLByGARempen1991",
        "FLByGAHadlock1984",
        "FLByGAASUM2000",
        "FibulaByGAJeanty1983",
        "TCDByGAGoldstein1987",
        "EFWByGAHansmann1986",
        "EFWByGAHadlock1991",
        "FLByGAMerz1988",
        "FLByGAJeanty1982",
        "FLByGAChitty1994",
        "CephalicIndexByGAHadlock1981",
        "CephalicIndexByGAChitty1994",
        "ACByGAASUM2000",
        "ACByGAHadlock1984",
        "ACMeasuredByGAChitty1994",
        "ACDerivedByGAChitty1994",
        "RadiusByGAJeanty1983",
        "FLByGAShinozuka1996",
        "GSByGARempen1991",
        "BPDOuterOuterByGAChitty1994",
        "BPDByGAASUM2000",
        "HCByGAJeanty1982",
        "AxtByGAShinozuka1996",
        "HCDerivedByGAChitty1994",
        "BPDByGAShinozuka1996",
        "HCByGAASUM2000",
        "BPDByGARempen1991",
        "HCByGAHadlock1984",
        "BPDByGAHadlock1984",
        "OFDByGAChitty1994",
        "BPDByGAMerz1988",
        "BPDByGAJeanty1982",
        "HumerusLengthByGAASUM2000",
        "ACByGAMerz1988",
        "OFDByGAASUM2000",
        "ACByGAShinozuka1996",
        "CRLByGAASUM2000",
        "HCByGAMerz1988",
        "BPDOuterInnerByGAChitty1994",
        "HCACByGACampbell1977",
    ]
}

name_for_cid[12016] = "EstimatedFetalWeightPercentileEquation/Table"
cid_concepts[12016] = {
    "DCM": ["FWPByGACampbell1991", "FWPByGAHadlock1991"],
    "LN": [
        "FWPByGAAlexander1996",
        "FWPByGAWilliams1982",
        "MaleTwinsBWPByGAArbuckle1993",
        "FWPByGABrenner1976",
        "FemaleSingletonBWPByGAArbuckle1993",
        "FemaleTwinsBWPByGAArbuckle1993",
        "MaleSingletonBWPByGAArbuckle1993",
        "FWPByGAHadlock1985",
    ],
}

name_for_cid[12017] = "GrowthDistributionRank"
cid_concepts[12017] = {"DCM": ["GrowthZScore", "GrowthPercentileRank"]}

name_for_cid[12018] = "OBGYNSummary"
cid_concepts[12018] = {"LN": ["NumberOfFetusesByUS", "GestationalAgeByOvulationDate"]}

name_for_cid[12019] = "OBGYNFetusSummary"
cid_concepts[12019] = {
    "LN": [
        "EFWPercentileRank",
        "EFWDiscordance",
        "FetalHeartRate",
        "GestationalAgeByLMP",
        "EFWRatio",
        "GestationalAge",
        "EstimatedWeight",
        "CompositeUltrasoundAge",
    ]
}

name_for_cid[12020] = "FetalBiometryAnatomicSite"
cid_concepts[12020] = {
    "SCT": [
        "Foot",
        "Femur",
        "Abdomen",
        "Cerebellum",
        "Skull",
        "Thorax",
        "CisternaMagna",
        "Kidney",
        "Trunk",
    ]
}

name_for_cid[12021] = "FetalLongBoneAnatomicSite"
cid_concepts[12021] = {
    "SCT": ["Femur", "Ulna", "RadiusBone", "Clavicle", "Tibia", "Fibula"]
}

name_for_cid[12022] = "FetalCraniumAnatomicSite"
cid_concepts[12022] = {
    "SCT": [
        "Cerebellum",
        "LateralVentricle",
        "CerebralHemisphere",
        "OccipitalRegionOfScalp",
        "CisternaMagna",
        "AnteriorHornLateralVentricle",
        "PosteriorHornLateralVentricle",
        "Orbit",
    ]
}

name_for_cid[12023] = "PelvisandUterusAnatomicSite"
cid_concepts[12023] = {"SCT": ["Endometrium", "Cervix", "Uterus"]}

name_for_cid[12024] = "VascularUltrasoundReportDocumentTitle"
cid_concepts[12024] = {
    "DCM": ["OBGYNUltrasoundProcedureReport"],
    "LN": ["USPelvis"],
    "SCT": ["ObstetricUSScan"],
}

name_for_cid[12025] = "OBGYNUltrasoundBeamPathUltrasoundBeamPath"
cid_concepts[12025] = {"SCT": ["TransAbdominal", "TransVaginal"]}

name_for_cid[12030] = "UltrasoundContrast/BolusAgent"
cid_concepts[12030] = {
    "DCM": [
        "Sonazoid",
        "Sonovue",
        "TargestarB",
        "TargestarP",
        "Cardiosphere",
        "Echovist",
        "Imagify",
        "Levovist",
    ],
    "NDC": ["Definity", "Optison"],
}

name_for_cid[12031] = "ProtocolIntervalEvent"
cid_concepts[12031] = {
    "DCM": [
        "CessationOfStimulation",
        "OnsetOfStimulation",
        "CessationOfExercise",
        "OnsetOfExercise",
        "DestructionOfMicrobubbles",
        "StartOfContrastAgentAdministration",
        "StartOfDrugDoseAdministration",
    ]
}

name_for_cid[12032] = "TransducerScanPattern"
cid_concepts[12032] = {
    "DCM": ["VolumeScanPattern", "PlaneScanPattern", "LineScanPattern"]
}

name_for_cid[12033] = "UltrasoundTransducerGeometry"
cid_concepts[12033] = {
    "DCM": [
        "HemisphericalUltrasoundTransducerGeometry",
        "PlanarMatrixUltrasoundTransducerGeometry",
        "RingUltrasoundTransducerGeometry",
        "RadialUltrasoundTransducerGeometry",
        "SectorUltrasoundTransducerGeometry",
        "CurvedLinearUltrasoundTransducerGeometry",
        "LinearUltrasoundTransducerGeometry",
        "NonImagingDopplerUltrasoundTransducerGeometry",
    ]
}

name_for_cid[12034] = "UltrasoundTransducerBeamSteering"
cid_concepts[12034] = {
    "DCM": ["PhasedBeamSteering", "MechanicalBeamSteering", "FixedBeamDirection"]
}

name_for_cid[12035] = "UltrasoundTransducerApplication"
cid_concepts[12035] = {
    "DCM": [
        "ExternalTransducer",
        "IntraoperativeTransducer",
        "EndocavitaryTransducer",
        "IntravascularTransducer",
        "EndorectalTransducer",
        "EndovaginalTransducer",
        "TransesophagealTransducer",
    ]
}

name_for_cid[12100] = "VascularUltrasoundReportDocumentTitle"
cid_concepts[12100] = {
    "LN": [
        "USCarotidArteriesReport",
        "USDopplerExtremityVesselsReport",
        "USDopplerHeadVesselsReport",
        "USDopplerVesselsReport",
        "USArteryReport",
        "USVeinReport",
        "USDopplerLowerExtremityVesselsReport",
        "USDopplerUpperExtremityVesselsReport",
        "USDopplerAbdominalVesselsReport",
    ],
    "SCT": ["PeripheralVascularUltrasoundReport"],
}

name_for_cid[12101] = "VascularSummary"
cid_concepts[12101] = {"DCM": ["Comment"]}

name_for_cid[12102] = "TemporalPeriodRelatingtoProcedureorTherapy"
cid_concepts[12102] = {"SCT": ["AfterProcedure", "BeforeProcedure", "DuringProcedure"]}

name_for_cid[12103] = "VascularUltrasoundAnatomicLocation"
cid_concepts[12103] = {
    "SCT": [
        "SubclavianVein",
        "ProfundaFemorisVein",
        "InternalCarotidArtery",
        "ExternalIliacArtery",
        "UlnarVein",
        "CeliacAxis",
        "OvarianVein",
        "PosteriorCerebralArtery",
        "BasilarArtery",
        "DeepPalmarVenousArch",
        "LateralCalfPerforator",
        "InferiorVenaCava",
        "CentralRetinalVein",
        "ProperHepaticArtery",
        "InterlobarArteryOfKidney",
        "GastricArtery",
        "FourthLumbarArtery",
        "AnteriorMiddleCerebralArteryBifurcation",
        "ThirdLumbarArtery",
        "PerforatingArteryOfKidney",
        "UmbilicalArtery",
        "InternalCarotidArteryC6Segment",
        "RadialArtery",
        "AnteriorTibialArtery",
        "SolealVein",
        "CommonCarotidArtery",
        "LobarArtery",
        "GreatSaphenousVeinOfCalf",
        "ThighPerforator",
        "CommonIliacArteryBifurcation",
        "MiddleHepaticVein",
        "RightMainBranchOfPortalVein",
        "RadialVein",
        "RightBranchOfHepaticArtery",
        "ExternalIliacVein",
        "CommonHepaticArtery",
        "UterineArtery",
        "SubclavianArtery",
        "CentralRetinalArtery",
        "FirstLumbarArtery",
        "OphthalmicArtery",
        "PoplitealArtery",
        "SegmentalArtery",
        "MiddleCerebralArteryM1Segment",
        "BrachialVein",
        "InternalIliacArtery",
        "SuperiorVenaCava",
        "SuperficialPalmarArch",
        "UlnarArtery",
        "OvarianArtery",
        "SixthLumbarArtery",
        "PosteriorArchVein",
        "SplenicVein",
        "PeronealArtery",
        "CarotidBifurcation",
        "SaphenousVein",
        "SuperficialFemoralVein",
        "SplenicArtery",
        "InnominateArtery",
        "RenalArtery",
        "SuperiorMesentericVein",
        "GastroduodenalArtery",
        "AccessoryRenalArtery",
        "LumbarArtery",
        "PeronealVein",
        "BrachialArtery",
        "DorsalisPedisArtery",
        "InfraRenalAorta",
        "CommonFemoralArtery",
        "SuperficialPalmarVenousArch",
        "CarotidBulb",
        "CommonFemoralVein",
        "PoplitealVein",
        "GiacominiVein",
        "InternalCarotidArteryC5Segment",
        "InferiorMesentericVein",
        "AnteriorCommunicatingArtery",
        "MedianCubitalVein",
        "HilarArtery",
        "TerminalInternalCarotidArtery",
        "AxillaryArtery",
        "HepaticVein",
        "VertebralArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "SaphenofemoralJunction",
        "SupraRenalAorta",
        "InternalJugularVein",
        "CockettPerforatingVein",
        "ExternalCarotidArtery",
        "SuperiorMesentericArtery",
        "UmbilicalVein",
        "CommonIliacVein",
        "PosteriorTibialArtery",
        "PosteriorCerebralArteryP1Segment",
        "TesticularArtery",
        "GreatSaphenousVeinOfThigh",
        "LeftHepaticVein",
        "InnominateVein",
        "SuperficialFemoralArtery",
        "PortalVein",
        "PosteriorCommunicatingArtery",
        "AnteriorCerebralArtery",
        "LeftBranchOfHepaticArtery",
        "RenalVein",
        "InternalIliacVein",
        "BoydPerforatingVein",
        "IlealVein",
        "ArcuateArteryOfTheKidney",
        "AbdominalAorta",
        "CommonIliacArtery",
        "RightHepaticVein",
        "CephalicVein",
        "GastrocnemiusVein",
        "GastricVein",
        "MiddleCerebralArteryM2Segment",
        "LeftMainBranchOfPortalVein",
        "MiddleCerebralArtery",
        "PosteriorCerebralArteryP2Segment",
        "InferiorRightHepaticVein",
        "TesticularVein",
        "CarotidSiphon",
        "AnteriorTibialVein",
        "PosteriorTibialVein",
        "SaphenopoplitealJunction",
        "BasilicVein",
        "PlantarArterialArch",
        "HunterianPerforatingVein",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "DeepPalmarArchOfRadialArtery",
        "ProfundaFemorisArtery",
        "InterlobularArteryOfKidney",
        "FifthLumbarArtery",
        "AxillaryVein",
        "LesserSaphenousVein",
        "InferiorMesentericArtery",
        "DigitalArteryOfHand",
        "GreatSaphenousVein",
        "SecondLumbarArtery",
    ]
}

name_for_cid[12104] = "ExtracranialArtery"
cid_concepts[12104] = {
    "SCT": [
        "InternalCarotidArtery",
        "ExternalCarotidArtery",
        "VertebralArtery",
        "CommonCarotidArtery",
        "CarotidBulb",
        "SubclavianArtery",
        "CarotidBifurcation",
    ]
}

name_for_cid[12105] = "IntracranialCerebralVessel"
cid_concepts[12105] = {
    "SCT": [
        "InternalCarotidArtery",
        "MiddleCerebralArteryM2Segment",
        "MiddleCerebralArtery",
        "PosteriorCerebralArteryP2Segment",
        "CarotidSiphon",
        "OphthalmicArtery",
        "PosteriorCerebralArtery",
        "PosteriorCommunicatingArtery",
        "AnteriorCerebralArtery",
        "AnteriorPosteriorCerebralArteryBifurcation",
        "MiddleCerebralArteryM1Segment",
        "CentralRetinalVein",
        "InternalCarotidArteryC5Segment",
        "AnteriorCommunicatingArtery",
        "AnteriorMiddleCerebralArteryBifurcation",
        "PosteriorCerebralArteryP1Segment",
        "InternalCarotidArteryC6Segment",
        "CentralRetinalArtery",
        "TerminalInternalCarotidArtery",
    ]
}

name_for_cid[12106] = "IntracranialCerebralVessel(Unilateral)"
cid_concepts[12106] = {"SCT": ["BasilarArtery"]}

name_for_cid[12107] = "UpperExtremityArtery"
cid_concepts[12107] = {
    "SCT": [
        "BrachialArtery",
        "InnominateArtery",
        "SuperficialPalmarArch",
        "UlnarArtery",
        "DigitalArteryOfHand",
        "RadialArtery",
        "SubclavianArtery",
        "DeepPalmarArchOfRadialArtery",
        "AxillaryArtery",
    ]
}

name_for_cid[12108] = "UpperExtremityVein"
cid_concepts[12108] = {
    "SCT": [
        "InnominateVein",
        "SubclavianVein",
        "UlnarVein",
        "DeepPalmarVenousArch",
        "BasilicVein",
        "SuperficialPalmarVenousArch",
        "InternalJugularVein",
        "BrachialVein",
        "SuperiorVenaCava",
        "CephalicVein",
        "AxillaryVein",
        "MedianCubitalVein",
        "RadialVein",
    ]
}

name_for_cid[12109] = "LowerExtremityArtery"
cid_concepts[12109] = {
    "SCT": [
        "AnteriorTibialArtery",
        "ExternalIliacArtery",
        "SuperficialFemoralArtery",
        "CommonFemoralArtery",
        "PoplitealArtery",
        "PlantarArterialArch",
        "InternalIliacArtery",
        "CommonIliacArteryBifurcation",
        "CommonIliacArtery",
        "ProfundaFemorisArtery",
        "DorsalisPedisArtery",
        "PosteriorTibialArtery",
        "PeronealArtery",
    ]
}

name_for_cid[12110] = "LowerExtremityVein"
cid_concepts[12110] = {
    "SCT": [
        "ProfundaFemorisVein",
        "CommonFemoralVein",
        "PoplitealVein",
        "LateralCalfPerforator",
        "GiacominiVein",
        "InternalIliacVein",
        "BoydPerforatingVein",
        "GastrocnemiusVein",
        "PosteriorArchVein",
        "SaphenousVein",
        "SolealVein",
        "SuperficialFemoralVein",
        "AnteriorTibialVein",
        "PosteriorTibialVein",
        "SaphenopoplitealJunction",
        "GreatSaphenousVeinOfCalf",
        "SaphenofemoralJunction",
        "HunterianPerforatingVein",
        "CockettPerforatingVein",
        "PeronealVein",
        "ThighPerforator",
        "CommonIliacVein",
        "LesserSaphenousVein",
        "GreatSaphenousVein",
        "ExternalIliacVein",
        "GreatSaphenousVeinOfThigh",
    ]
}

name_for_cid[12111] = "AbdominopelvicArtery(Paired)"
cid_concepts[12111] = {
    "SCT": [
        "FirstLumbarArtery",
        "AccessoryRenalArtery",
        "LumbarArtery",
        "CommonIliacArtery",
        "GastricArtery",
        "OvarianArtery",
        "FourthLumbarArtery",
        "FifthLumbarArtery",
        "SixthLumbarArtery",
        "ThirdLumbarArtery",
        "UmbilicalArtery",
        "SecondLumbarArtery",
        "TesticularArtery",
        "UterineArtery",
    ]
}

name_for_cid[12112] = "AbdominopelvicArtery(Unpaired)"
cid_concepts[12112] = {
    "SCT": [
        "AbdominalAorta",
        "InfraRenalAorta",
        "SplenicArtery",
        "SuperiorMesentericArtery",
        "CeliacAxis",
        "GastroduodenalArtery",
        "InferiorMesentericArtery",
        "RightBranchOfHepaticArtery",
        "SupraRenalAorta",
        "CommonHepaticArtery",
        "LeftBranchOfHepaticArtery",
        "ProperHepaticArtery",
    ]
}

name_for_cid[12113] = "AbdominopelvicVein(Paired)"
cid_concepts[12113] = {
    "SCT": [
        "TesticularVein",
        "OvarianVein",
        "CommonIliacVein",
        "GastricVein",
        "IlealVein",
    ]
}

name_for_cid[12114] = "AbdominopelvicVein(Unpaired)"
cid_concepts[12114] = {
    "SCT": [
        "HepaticVein",
        "LeftMainBranchOfPortalVein",
        "InferiorRightHepaticVein",
        "PortalVein",
        "SuperiorMesentericVein",
        "TransjugularIntrahepaticPortosystemicShunt",
        "InferiorVenaCava",
        "MiddleHepaticVein",
        "InferiorMesentericVein",
        "RightHepaticVein",
        "RightMainBranchOfPortalVein",
        "UmbilicalVein",
        "SplenicVein",
        "LeftHepaticVein",
    ]
}

name_for_cid[12115] = "RenalVessel"
cid_concepts[12115] = {
    "SCT": [
        "RenalArtery",
        "InterlobularArteryOfKidney",
        "LobarArtery",
        "PerforatingArteryOfKidney",
        "AccessoryRenalArtery",
        "SegmentalArtery",
        "HilarArtery",
        "RenalVein",
        "ArcuateArteryOfTheKidney",
        "InterlobarArteryOfKidney",
    ]
}

name_for_cid[12116] = "VesselSegmentModifier"
cid_concepts[12116] = {
    "NCIt": ["Middle"],
    "SCT": ["Distal", "Proximal", "DilatedPortionOfSegment", "OriginOfVessel"],
}

name_for_cid[12117] = "VesselBranchModifier"
cid_concepts[12117] = {
    "NCIt": ["Middle"],
    "SCT": [
        "Main",
        "Right",
        "Lateral",
        "Inferior",
        "Left",
        "Superior",
        "CollateralBranchOfVessel",
        "Medial",
    ],
}

name_for_cid[12118] = "MeasurementOrientation"
cid_concepts[12118] = {
    "DCM": ["AnteriorPosterior"],
    "SCT": ["Transverse", "Longitudinal"],
}

name_for_cid[12119] = "VascularUltrasoundProperty"
cid_concepts[12119] = {
    "DCM": ["FlowVelocity"],
    "LN": [
        "MinimumDiastolicVelocity",
        "ResistivityIndex",
        "VelocityTimeIntegral",
        "MeanGradient",
        "PeakSystolicVelocity",
        "VolumeFlow",
        "VelocityRatio",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "PeakGradient",
        "AccelerationIndex",
        "DecelerationTime",
        "AccelerationTime",
        "SystolicToDiastolicVelocityRatio",
        "PeakBloodVelocity",
        "PulsatilityIndex",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "VesselDepthFromSurface",
        "VesselOutsideDiameter",
        "VesselIntimalCrossSectionalDiameter",
        "LumenDiameterStenosis",
        "VesselLumenDiameter",
        "LengthOfSegment",
        "VesselLumenCrossSectionalArea",
        "LumenAreaStenosis",
        "VesselIntimalDiameter",
    ],
}

name_for_cid[12120] = "UltrasoundBloodVelocityMeasurement"
cid_concepts[12120] = {
    "DCM": ["FlowVelocity"],
    "LN": [
        "MinimumDiastolicVelocity",
        "TimeAveragedPeakVelocity",
        "VelocityTimeIntegral",
        "PeakBloodVelocity",
        "PeakSystolicVelocity",
        "EndDiastolicVelocity",
        "TimeAveragedMeanVelocity",
    ],
}

name_for_cid[12121] = "VascularIndex/Ratio"
cid_concepts[12121] = {
    "LN": [
        "AccelerationIndex",
        "ResistivityIndex",
        "VelocityRatio",
        "SystolicToDiastolicVelocityRatio",
        "PulsatilityIndex",
    ],
    "SCT": ["LumenDiameterStenosis", "LumenAreaStenosis"],
}

name_for_cid[12122] = "OtherVascularProperty"
cid_concepts[12122] = {
    "LN": [
        "DecelerationTime",
        "PeakGradient",
        "AccelerationTime",
        "MeanGradient",
        "VolumeFlow",
    ],
    "SCT": [
        "VesselLumenCrossSectionalArea",
        "VesselDepthFromSurface",
        "VesselOutsideDiameter",
        "VesselLumenDiameter",
        "VesselIntimalCrossSectionalDiameter",
        "LengthOfSegment",
        "VesselIntimalDiameter",
    ],
}

name_for_cid[12123] = "CarotidRatio"
cid_concepts[12123] = {"LN": ["ICACCAVelocityRatio"]}

name_for_cid[12124] = "RenalRatio"
cid_concepts[12124] = {"LN": ["RenalArteryAortaVelocityRatio"]}

name_for_cid[12125] = "AbdominopelvicVessel"
cid_concepts[12125] = {
    "SCT": [
        "InfraRenalAorta",
        "CeliacAxis",
        "FirstLumbarArtery",
        "PortalVein",
        "OvarianVein",
        "LeftBranchOfHepaticArtery",
        "InferiorVenaCava",
        "ProperHepaticArtery",
        "IlealVein",
        "AbdominalAorta",
        "InferiorMesentericVein",
        "CommonIliacArtery",
        "RightHepaticVein",
        "GastricArtery",
        "OvarianArtery",
        "FourthLumbarArtery",
        "SixthLumbarArtery",
        "ThirdLumbarArtery",
        "GastricVein",
        "UmbilicalArtery",
        "SplenicVein",
        "HepaticVein",
        "LeftMainBranchOfPortalVein",
        "SplenicArtery",
        "InferiorRightHepaticVein",
        "TesticularVein",
        "SuperiorMesentericVein",
        "GastroduodenalArtery",
        "AccessoryRenalArtery",
        "LumbarArtery",
        "TransjugularIntrahepaticPortosystemicShunt",
        "SupraRenalAorta",
        "MiddleHepaticVein",
        "SuperiorMesentericArtery",
        "RightMainBranchOfPortalVein",
        "UmbilicalVein",
        "FifthLumbarArtery",
        "CommonIliacVein",
        "InferiorMesentericArtery",
        "RightBranchOfHepaticArtery",
        "SecondLumbarArtery",
        "TesticularArtery",
        "CommonHepaticArtery",
        "UterineArtery",
        "LeftHepaticVein",
    ]
}

name_for_cid[12130] = "OrganPart(NonLateralized)"
cid_concepts[12130] = {"SCT": ["ThyroidIsthmus", "ThyroidPart", "LobeOfThyroid"]}

name_for_cid[12131] = "OrganPart(Lateralized)"
cid_concepts[12131] = {"SCT": ["RightLobeOfThyroid", "LeftLobeOfThyroid"]}

name_for_cid[12140] = "PelvicVasculatureAnatomicalLocation"
cid_concepts[12140] = {
    "SCT": [
        "FetalUmbilicalVein",
        "CommonIliacArtery",
        "VitellineVeinOfPlacenta",
        "OvarianArtery",
        "OvarianVein",
        "UterineVein",
        "UmbilicalArtery",
        "UterineArtery",
        "VitellineArteryOfPlacenta",
    ]
}

name_for_cid[12141] = "FetalVasculatureAnatomicalLocation"
cid_concepts[12141] = {
    "SCT": [
        "DescendingAorta",
        "MiddleCerebralArtery",
        "PulmonaryArtery",
        "PulmonaryVein",
        "Aorta",
    ]
}

name_for_cid[12200] = "EchocardiographyLeftVentricleMeasurement"
cid_concepts[12200] = {
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "LeftVentricularEndDiastolicVolume",
        "LeftVentricleMass",
        "VolumeFlow",
        "FlowRadius",
        "LeftVentricularEjectionFractionByUS",
        "LeftVentricleInternalSystolicDimension",
        "InterventricularSeptumSystolicThickness",
        "HeartRate",
        "LeftVentriclePosteriorWallDiastolicThickness",
        "EndDiastolicVelocity",
        "LeftVentriclePosteriorWallSystolicThickness",
        "InterventricularSeptumDiastolicThickness",
        "PeakGradient",
        "LeftVentricleInternalEndDiastolicDimension",
        "LeftVentricleDiastolicMajorAxis",
        "InterventricularSeptumPercentThickening",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "LeftVentricularIsovolumicRelaxationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "ACInterval",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeAveragedPeakVelocity",
        "InterventricularSeptumToPosteriorWallThicknessRatio",
        "VelocityOfFlowPropagation",
        "LeftVentriclePosteriorWallPercentThickening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "LeftVentricularEndSystolicVolume",
        "LeftVentricularFractionalShortening",
        "LeftVentricleSystolicMajorAxis",
        "SystolicToDiastolicVelocityRatio",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "LeftVentricleSemiMajorAxisDiastolicDimension",
        "LeftVentricularSystolicArea",
        "CardiacOutput",
        "LeftVentricularPeakEarlyDiastolicTissueVelocity",
        "LeftVentricularPeakSystolicTissueVelocity",
        "LeftVentricularIndexOfMyocardialPerformance",
        "LeftVentricularDiastolicArea",
        "CardiovascularOrificeDiameter",
        "StrokeIndex",
        "LeftVentricularFractionalAreaChange",
        "LeftVentricleTruncatedSemiMajorAxisDiastolicDimension",
        "LVPeakDiastolicTissueVelocityDuringAtrialSystole",
        "LeftVentricularIsovolumicContractionTime",
        "CardiacIndex",
        "LeftVentricleEpicardialDiastolicAreaPsaxPapView",
        "RegurgitantFraction",
        "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12201] = "LeftVentricleLinearMeasurement"
cid_concepts[12201] = {
    "LN": [
        "LeftVentricleInternalEndDiastolicDimension",
        "LeftVentricleDiastolicMajorAxis",
        "InterventricularSeptumPercentThickening",
        "LeftVentriclePosteriorWallPercentThickening",
        "LeftVentricleInternalSystolicDimension",
        "LeftVentricularFractionalShortening",
        "LeftVentricleSystolicMajorAxis",
        "InterventricularSeptumSystolicThickness",
        "LeftVentriclePosteriorWallDiastolicThickness",
        "LeftVentriclePosteriorWallSystolicThickness",
        "InterventricularSeptumToPosteriorWallThicknessRatio",
        "InterventricularSeptumDiastolicThickness",
    ],
    "SCT": [
        "LeftVentricleTruncatedSemiMajorAxisDiastolicDimension",
        "LeftVentricleSemiMajorAxisDiastolicDimension",
    ],
}

name_for_cid[12202] = "LeftVentricleVolumeMeasurement"
cid_concepts[12202] = {
    "LN": [
        "LeftVentricularEjectionFractionByUS",
        "LeftVentricularEndSystolicVolume",
        "LeftVentricularEndDiastolicVolume",
    ]
}

name_for_cid[12203] = "LeftVentricleOtherMeasurement"
cid_concepts[12203] = {
    "LN": ["LeftVentricularIsovolumicRelaxationTime", "LeftVentricleMass"],
    "SCT": [
        "LeftVentricularPeakSystolicTissueVelocity",
        "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave",
        "LeftVentricularIndexOfMyocardialPerformance",
        "LVPeakDiastolicTissueVelocityDuringAtrialSystole",
        "LeftVentricularIsovolumicContractionTime",
        "LeftVentricularPeakEarlyDiastolicTissueVelocity",
    ],
}

name_for_cid[12204] = "EchocardiographyRightVentricleMeasurement"
cid_concepts[12204] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "RightVentricularAnteriorWallSystolicThickness",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "RightVentricularAnteriorWallDiastolicThickness",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "RightVentricularInternalDiastolicDimension",
        "PeakGradient",
        "AliasVelocity",
        "RightVentricularInternalSystolicDimension",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RVStrokeIndex",
        "RVCardiacOutput",
        "RightVentricularIndexOfMyocardialPerformance",
        "RegurgitantFraction",
        "RVStrokeVolume",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "RVCardiacIndex",
        "RightVentricularPeakSystolicPressure",
    ],
}

name_for_cid[12205] = "EchocardiographyLeftAtriumMeasurement"
cid_concepts[12205] = {
    "LN": [
        "LeftAtriumAreaA4CView",
        "LeftAtriumToAorticRootRatio",
        "HeartRate",
        "LeftAtrialAppendagePeakVelocity",
        "LeftAtriumAnteroPosteriorSystolicDimension",
    ],
    "SCT": ["LeftAtriumSystolicVolume"],
}

name_for_cid[12206] = "EchocardiographyRightAtriumMeasurement"
cid_concepts[12206] = {
    "LN": ["RightAtriumSystolicPressure", "RightAtriumAreaA4CView", "HeartRate"]
}

name_for_cid[12207] = "EchocardiographyMitralValveMeasurement"
cid_concepts[12207] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "MitralValveAWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "MitralValveEFSlopeByMMode",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "MitralValveEPSSEWave",
        "MitralValveEWavePeakVelocity",
        "MitralRegurgitationDpDtDerivedFromMitralRegVelocity",
        "AliasVelocity",
        "MitralValveEToARatio",
        "MitralValveDiastolicPeakInstantaneousGradient",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MitralValveAWaveDuration",
        "CardiacIndex",
        "MitralValveClosureToOpeningTime",
        "CardiacOutput",
        "MitralValveATDTRatio",
        "RegurgitantFraction",
        "MitralValveEWaveDecelerationTime",
        "StrokeVolume",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "StrokeIndex",
    ],
}

name_for_cid[12208] = "EchocardiographyTricuspidValveMeasurement"
cid_concepts[12208] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "TimeFromQWaveToTricuspidValveOpens",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "TricuspidValveEToARatio",
        "VelocityOfFlowPropagation",
        "TricuspidRegurgitationDpDt",
        "PeakGradient",
        "TricuspidValveEWavePeakVelocity",
        "AliasVelocity",
        "TricuspidValveAWavePeakVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RegurgitantFraction",
        "TricuspidValveClosureToOpeningTime",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
    ],
}

name_for_cid[12209] = "EchocardiographyPulmonicValveMeasurement"
cid_concepts[12209] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "TimeFromQWaveToPulmonicValveCloses",
        "VolumeFlow",
        "FlowRadius",
        "PulmonicValveEjectionTime",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "PulmonicValveAreaByContinuity",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RatioOfPulmonicValveAccelerationTimeToEjectionTime",
        "RegurgitantFraction",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
    ],
}

name_for_cid[12210] = "EchocardiographyPulmonaryArteryMeasurement"
cid_concepts[12210] = {
    "LN": [
        "HeartRate",
        "MainPulmonaryArteryDiameter",
        "LeftPulmonaryArteryDiameter",
        "RightPulmonaryArteryDiameter",
    ],
    "SCT": ["MainPulmonaryArteryPeakVelocity"],
}

name_for_cid[12211] = "EchocardiographyAorticValveMeasurement"
cid_concepts[12211] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "AorticValveEjectionTime",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "AorticValveCuspSeparation",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RegurgitantFraction",
        "RatioOfAorticValveAccelerationTimeToEjectionTime",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
    ],
}

name_for_cid[12212] = "EchocardiographyAortaMeasurement"
cid_concepts[12212] = {
    "LN": [
        "AorticRootAnnuloAorticJunctionDiameter",
        "HeartRate",
        "AscendingAorticDiameter",
        "AorticArchDiameter",
        "AorticIsthmusDiameter",
        "ThoracicAortaCoarctationSystolicPeakVelocity",
        "DescendingAorticDiameter",
        "ThoracicAortaCoarctationSystolicPeakInstantaneousGradient",
    ]
}

name_for_cid[12214] = "EchocardiographyPulmonaryVeinMeasurement"
cid_concepts[12214] = {
    "LN": [
        "PulmonaryVeinSystolicPeakVelocity",
        "PulmonaryVeinDiastolicPeakVelocity",
        "HeartRate",
        "PulmonaryVeinAtrialContractionReversalPeakVelocity",
        "PulmonaryVeinSystolicToDiastolicRatio",
    ],
    "SCT": [
        "PulmonaryVeinAWaveDuration",
        "PulmonaryVeinSWaveVelocityTimeIntegral",
        "PulmonaryVeinDWaveVelocityTimeIntegral",
    ],
}

name_for_cid[12215] = "EchocardiographyVenaCavaMeasurement"
cid_concepts[12215] = {
    "LN": ["InferiorVenaCavaDiameter", "InferiorVenaCavaPercentCollapse", "HeartRate"]
}

name_for_cid[12216] = "EchocardiographyHepaticVeinMeasurement"
cid_concepts[12216] = {
    "LN": [
        "HepaticVeinAtrialContractionReversalPeakVelocity",
        "HepaticVeinSystolicToDiastolicRatio",
        "HepaticVeinDiastolicPeakVelocity",
        "HeartRate",
        "HepaticVeinSystolicPeakVelocity",
    ]
}

name_for_cid[12217] = "EchocardiographyCardiacShuntMeasurement"
cid_concepts[12217] = {"LN": ["PulmonaryToSystemicShuntFlowRatio", "HeartRate"]}

name_for_cid[12218] = "EchocardiographyCongenitalAnomalyMeasurement"
cid_concepts[12218] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "EndDiastolicVelocity",
        "ROIInternalDimensionByUS",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
    ],
}

name_for_cid[12219] = "PulmonaryVeinModifier"
cid_concepts[12219] = {
    "SCT": [
        "RightLowerSegment",
        "RightUpperSegment",
        "LeftLowerSegment",
        "LeftUpperSegment",
    ]
}

name_for_cid[12220] = "EchocardiographyCommonMeasurement"
cid_concepts[12220] = {"LN": ["HeartRate"]}

name_for_cid[12221] = "FlowDirection"
cid_concepts[12221] = {
    "SCT": [
        "RetrogradeFlow",
        "AntegradeFlow",
        "RegurgitantFlow",
        "RightToLeftCardiovascularShunt",
        "LeftToRightCardiovascularShunt",
    ]
}

name_for_cid[12222] = "OrificeFlowProperty"
cid_concepts[12222] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RegurgitantFraction",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
    ],
}

name_for_cid[12223] = "EchocardiographyStrokeVolumeOrigin"
cid_concepts[12223] = {
    "SCT": [
        "RightVentricleOutflowTract",
        "LeftVentricle",
        "MitralValve",
        "LeftVentricleOutflowTract",
        "Aorta",
    ]
}

name_for_cid[12224] = "UltrasoundImageMode"
cid_concepts[12224] = {
    "DCM": [
        "_3DSpeckleTracking",
        "_3DShearWaveElastography",
        "PointShearWaveElastography",
        "_2DSpeckleTracking",
        "_2DShearWaveElastography",
    ],
    "SCT": [
        "BMode",
        "TissueDopplerImaging",
        "_3DMode",
        "DopplerPulsed",
        "MMode",
        "_2DMode",
        "DopplerContinuousWave",
        "PowerDoppler",
        "DopplerColorFlow",
    ],
}

name_for_cid[12226] = "EchocardiographyImageView"
cid_concepts[12226] = {
    "DCM": ["ApicalFourChamberRVFocused", "ApicalFourChamberRVModified"],
    "SCT": [
        "ParasternalShortAxisAtTheMitralValveLevel",
        "SubcostalShortAxis",
        "ParasternalLongAxis",
        "SuprasternalShortAxis",
        "SubcostalViewOfCardiacOutletsDirectedAnteriorly",
        "ParasternalLongAxisViewOfTheRVOutflowTract",
        "ParasternalShortAxisAtThePapillaryMuscleLevel",
        "SuprasternalSagittalView",
        "SuprasternalLongAxis",
        "TransesophagealShortAxisView",
        "SuprasternalCoronalView",
        "ParasternalLongAxisViewOfTheRVInflowTract",
        "RightVentricularInflowTractView",
        "ApicalTwoChamber",
        "SuprasternalLongAxisViewOfAorticArch",
        "SubcostalObliqueCoronalView",
        "SubcostalLongAxis",
        "SubcostalShortAxisViewAtMitralValveLevel",
        "ApicalLongAxis",
        "SubcostalShortAxisViewAtAorticValveLevel",
        "ParasternalShortAxisAtTheAorticValveLevel",
        "SubcostalShortAxisViewAtPapillaryMuscleLevel",
        "SubcostalShortAxisViewAtVenousInflowLevel",
        "ApicalFourChamber",
        "ParasternalShortAxisAtTheLevelOfTheMitralChords",
        "RightVentricularOutflowTractView",
        "ParasternalShortAxis",
    ],
}

name_for_cid[12227] = "EchocardiographyMeasurementMethod"
cid_concepts[12227] = {
    "DCM": [
        "Teichholz",
        "MethodOfDisksSinglePlane",
        "MethodOfDisksBiplane",
        "CubeMethod",
        "BulletMethod",
        "AreaLengthSinglePlane",
        "ModifiedSimpson",
        "AreaLengthBiplane",
        "SinglePlaneEllipse",
        "MyocardialMidlayerMethod",
        "EpicardialMethod",
        "EndocardialMethod",
        "LeftVentricleMassByTruncatedEllipse",
        "LeftVentricleMassByMMode",
        "Planimetry",
        "DopplerVolumeFlow",
        "SimplifiedBernoulli",
        "DirectlyMeasured",
        "FullBernoulli",
        "ProximalIsovelocitySurfaceArea",
        "ContinuityEquationByVelocityTimeIntegral",
        "LeftVentricleMassByTruncatedEllipseAdjustedByHeight",
        "LeftVentricleMassByMModeAdjustedByHeight",
        "LeftVentricleMassByAreaLength",
        "ContinuityEquationByPeakVelocity",
        "ContinuityEquationByMeanVelocity",
        "ContinuityEquation",
        "BiplaneEllipse",
        "AreaByPressureHalfTime",
        "LeftVentricleMassByAreaLengthAdjustedByHeight",
    ]
}

name_for_cid[12228] = "EchocardiographyVolumeMethod"
cid_concepts[12228] = {
    "DCM": [
        "Teichholz",
        "MethodOfDisksSinglePlane",
        "MethodOfDisksBiplane",
        "CubeMethod",
        "BulletMethod",
        "AreaLengthSinglePlane",
        "ModifiedSimpson",
        "AreaLengthBiplane",
        "SinglePlaneEllipse",
        "BiplaneEllipse",
    ]
}

name_for_cid[12229] = "EchocardiographyAreaMethod"
cid_concepts[12229] = {
    "DCM": [
        "ProximalIsovelocitySurfaceArea",
        "ContinuityEquationByVelocityTimeIntegral",
        "ContinuityEquationByPeakVelocity",
        "ContinuityEquationByMeanVelocity",
        "ContinuityEquation",
        "AreaByPressureHalfTime",
        "Planimetry",
    ]
}

name_for_cid[12230] = "GradientMethod"
cid_concepts[12230] = {"DCM": ["SimplifiedBernoulli", "FullBernoulli"]}

name_for_cid[12231] = "VolumeFlowMethod"
cid_concepts[12231] = {"DCM": ["DopplerVolumeFlow", "ProximalIsovelocitySurfaceArea"]}

name_for_cid[12232] = "MyocardiumMassMethod"
cid_concepts[12232] = {
    "DCM": [
        "LeftVentricleMassByTruncatedEllipseAdjustedByHeight",
        "LeftVentricleMassByMModeAdjustedByHeight",
        "LeftVentricleMassByAreaLength",
        "LeftVentricleMassByTruncatedEllipse",
        "LeftVentricleMassByMMode",
        "LeftVentricleMassByAreaLengthAdjustedByHeight",
    ]
}

name_for_cid[12233] = "CardiacPhase"
cid_concepts[12233] = {
    "SCT": [
        "Diastole",
        "EndDiastole",
        "VentricularIsovolumicContraction",
        "VentricularSystole",
        "EarlyDiastole",
        "Diastasis",
        "EndSystole",
        "AtrialSystole",
        "PeakSystolic",
        "VentricularEjection",
        "DiastolicRapidInflow",
        "VentricularIsovolumicRelaxation",
        "Systole",
    ]
}

name_for_cid[12234] = "RespirationState"
cid_concepts[12234] = {"SCT": ["Expiration", "Inspiration"]}

name_for_cid[12235] = "MitralValveAnatomicSite"
cid_concepts[12235] = {
    "SCT": ["MedialMitralAnnulus", "LateralMitralAnnulus", "MitralAnnulus"]
}

name_for_cid[12236] = "EchocardiographyAnatomicSite"
cid_concepts[12236] = {
    "SCT": [
        "RightVentricleOutflowTract",
        "PericardialEffusion",
        "VentricularSeptalDefect",
        "ThoracicAorticCoarctation",
        "MitralAnnulus",
        "Aorta",
        "MedialMitralAnnulus",
        "LeftVentricle",
        "AtrialSeptalDefect",
        "LateralMitralAnnulus",
        "MitralValve",
        "LeftVentricleOutflowTract",
        "AorticValveRing",
        "TricuspidAnnulus",
    ]
}

name_for_cid[12237] = "EchocardiographyAnatomicSiteModifier"
cid_concepts[12237] = {
    "SCT": [
        "RightLowerSegment",
        "RightUpperSegment",
        "LeftLowerSegment",
        "LeftUpperSegment",
    ]
}

name_for_cid[12238] = "WallMotionScoringScheme"
cid_concepts[12238] = {
    "DCM": [
        "_5PointSegmentFindingScaleWithGradedHypokinesis",
        "_5PointSegmentFindingScale",
        "_4PointSegmentFindingScale",
    ]
}

name_for_cid[12239] = "CardiacOutputProperty"
cid_concepts[12239] = {
    "SCT": ["CardiacOutput", "StrokeIndex", "CardiacIndex", "StrokeVolume"]
}

name_for_cid[12240] = "LeftVentricleAreaMeasurement"
cid_concepts[12240] = {
    "SCT": [
        "LeftVentricleEpicardialDiastolicAreaPsaxPapView",
        "LeftVentricularDiastolicArea",
        "LeftVentricularFractionalAreaChange",
        "LeftVentricularSystolicArea",
    ]
}

name_for_cid[12241] = "TricuspidValveFindingSite"
cid_concepts[12241] = {"SCT": ["TricuspidAnnulus"]}

name_for_cid[12242] = "AorticValveFindingSite"
cid_concepts[12242] = {"SCT": ["AorticValveRing"]}

name_for_cid[12243] = "LeftVentricleFindingSite"
cid_concepts[12243] = {"SCT": ["LeftVentricleOutflowTract"]}

name_for_cid[12244] = "CongenitalFindingSite"
cid_concepts[12244] = {"SCT": ["AtrialSeptalDefect", "VentricularSeptalDefect"]}

name_for_cid[12245] = "CardiacUltrasoundReportTitle"
cid_concepts[12245] = {
    "DCM": [
        "PediatricCardiacUltrasoundReport",
        "AdultCongenitalCardiacUltrasoundReport",
        "FetalCardiacUltrasoundReport",
    ]
}

name_for_cid[12246] = "CardiacUltrasoundIndicationforStudy"
cid_concepts[12246] = {
    "SCT": [
        "Hemoptysis",
        "ConductionDisorderOfTheHeart",
        "Palpitations",
        "AnginaPectoris",
        "Syncope",
        "AbnormalECG",
        "CoronaryArteryDisease",
        "SupraventricularTachycardia",
        "ImagingGuidance",
        "Fever",
        "LeftBundleBranchBlock",
        "ValvularHeartDisease",
        "ChestPain",
        "Hypertension",
        "HeartDiseaseRiskFactors",
        "Murmur",
        "CongenitalHeartDisease",
        "Dyspnea",
    ]
}

name_for_cid[12247] = "Pediatric,FetalandCongenitalCardiacSurgicalIntervention"
cid_concepts[12247] = {
    "SCT": [
        "RastelliOperation",
        "CreationOfConduitRightAtriumToPulmonaryTrunk",
        "RightSidedBidirectionalGlennShuntProcedure",
        "LeftSidedBidirectionalGlennShuntProcedure",
        "ImplantationOfHeartValveWithTissueGraft",
        "BlalockTaussigShuntPulmonarySubclavianArteryAnastomosis",
        "HemiFontanOperation",
        "NorwoodTypeOperation",
        "InsertionOfAtrialBaffle",
        "PatentDuctusArteriosusCoilOrDeviceClosure",
        "PartialAnomalousPulmonaryVenousConnectionOperation",
        "CentralAortopulmonaryShuntOperation",
        "ImplantationOfHeartValveProsthesisOrSyntheticDevice",
        "RepairOfCoarctationOfAorta",
        "ConstructionOfLVToAortaTunnelWRVToPAValvedConduit",
        "RepairOfTotalAnomalousPulmonaryVenousConnection",
        "InsertionOfInteratrialBaffle",
        "PercutaneousProstheticClosureOfAtrialSeptalDefect",
        "CoarctationOfTheAortaAngioplastyWithImplantOfStent",
        "VentricularSeptalDefectDeviceClosure",
        "RadicalAortopulmonaryReconstructWRVToPAValvelessConduit",
        "ArterialSwitchOperation",
        "CoarctationOfTheAortaBalloonAngioplasty",
        "DamusStanselKayeOperation",
        "ClosureOfDuctusArteriosusWithClip",
        "AtrialInversionOperationUsingAtrialWall",
        "RightGlennShuntProcedure",
        "LateralCavalFontanProcedure",
        "SanoProcedure",
        "RepairOfDefectOfTheAtrioventricularSeptum",
        "CorrectionOfVentricularSeptalDefect",
        "AtrialSeptalDefectRepair",
        "MustardOperation",
        "LeftGlennShuntProcedure",
    ]
}

name_for_cid[12248] = "CardiacUltrasoundSummaryCode"
cid_concepts[12248] = {
    "SCT": [
        "CorrectedTranspositionOfGreatVessels",
        "CoarctationOfTheAorta",
        "PulmonaryAtresiaWithIntactVentricularSeptum",
        "TetralogyOfFallot",
        "CompleteTranspositionOfGreatVessels",
        "Mesocardia",
        "FunctionalSingleVentricle",
        "MitralRegurgitation",
        "HypoplasticLeftHeartSyndrome",
        "CongenitalStenosisOfAorticValve",
        "MitralStenosis",
        "AtrialSeptalDefect",
        "RheumaticFever",
        "PulmonaryAtresiaWithVentricularSeptalDefect",
        "MitralValveProlapse",
        "AorticRegurgitation",
        "Dextrocardia",
        "CommonAtrioventricularCanal",
        "VentricularSeptalDefect",
        "PartialAnomalousPulmonaryVenousConnection",
        "AcuteFebrileMucocutaneousLymphNodeSyndrome",
        "Cardiomegaly",
        "Cyanosis",
        "AtrialSeptalDefectWithEndocardialCushionDefectPartial",
        "PulmonicValveStenosis",
        "AorticStenosis",
        "InterruptedAorticArch",
    ]
}

name_for_cid[12249] = "CardiacUltrasoundFetalSummaryCode"
cid_concepts[12249] = {
    "SCT": [
        "CorrectedTranspositionOfGreatVessels",
        "CoarctationOfTheAorta",
        "PulmonaryAtresiaWithIntactVentricularSeptum",
        "TetralogyOfFallot",
        "CompleteTranspositionOfGreatVessels",
        "Mesocardia",
        "FunctionalSingleVentricle",
        "MitralRegurgitation",
        "HypoplasticLeftHeartSyndrome",
        "CongenitalStenosisOfAorticValve",
        "MitralStenosis",
        "AtrialSeptalDefect",
        "RheumaticFever",
        "PulmonaryAtresiaWithVentricularSeptalDefect",
        "MitralValveProlapse",
        "AorticRegurgitation",
        "EdemaOfFetalChestWall",
        "Dextrocardia",
        "CommonAtrioventricularCanal",
        "VentricularSeptalDefect",
        "EdemaOfFetalScalp",
        "PartialAnomalousPulmonaryVenousConnection",
        "AcuteFebrileMucocutaneousLymphNodeSyndrome",
        "Cardiomegaly",
        "Cyanosis",
        "AtrialSeptalDefectWithEndocardialCushionDefectPartial",
        "PulmonicValveStenosis",
        "AorticStenosis",
        "InterruptedAorticArch",
    ]
}

name_for_cid[12250] = "CardiacUltrasoundCommonLinearMeasurement"
cid_concepts[12250] = {
    "LN": ["ROIInternalDimensionByUS", "ROIThicknessByUS"],
    "SCT": ["Radius", "MinorAxis", "Length", "Diameter", "MajorAxis", "Circumference"],
}

name_for_cid[12251] = "CardiacUltrasoundLinearValveMeasurement"
cid_concepts[12251] = {
    "LN": [
        "LeafletThickness",
        "LeafletSeparation",
        "ROIInternalDimensionByUS",
        "ROIThicknessByUS",
        "CEDistance",
        "DEExcursion",
    ],
    "SCT": ["Radius", "MinorAxis", "Length", "Diameter", "MajorAxis", "Circumference"],
}

name_for_cid[12252] = "CardiacUltrasoundCardiacFunction"
cid_concepts[12252] = {
    "LN": [
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "FractionalShortening",
        "PercentThickening",
    ],
    "SCT": ["CardiacEjectionFraction"],
}

name_for_cid[12253] = "CardiacUltrasoundAreaMeasurement"
cid_concepts[12253] = {"LN": ["JetArea", "FlowArea"], "SCT": ["Area"]}

name_for_cid[12254] = "CardiacUltrasoundHemodynamicMeasurement"
cid_concepts[12254] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "PreEjectionPeriod",
        "EjectionTime",
        "AWaveDuration",
        "EnvelopeDuration",
        "TimeToPeakByUS",
        "DpDtByUS",
        "HeartRateCorrectedEjectionTime",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "PreEjectionPeriodEjectionTimeRatio",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "IsovolumicRelaxationTime",
    ],
    "SCT": ["BloodPressure"],
}

name_for_cid[12255] = "CardiacUltrasoundMyocardiumMeasurement"
cid_concepts[12255] = {
    "DCM": ["WallMass"],
    "LN": ["EndocardialArea", "EpicardialArea", "MyocardialPerformanceIndexTei"],
}

name_for_cid[12257] = "CardiacUltrasoundLeftVentricleMeasurement"
cid_concepts[12257] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval", "WallMass"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleAWave",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "EndocardialArea",
        "PeakTissueVelocity",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "TimeToLeftVentricleETissueVelocity",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleEWave",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "InterventricularSeptumToPosteriorWallThicknessRatio",
        "IsovolumicRelaxationTime",
        "TimeToLeftVentricleSTissueVelocity",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "LeftVentricleMeridionalWallStress",
        "LeftVentricleEToATissueVelocityRatio",
        "SystolicToDiastolicVelocityRatio",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Volume",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12258] = "CardiacUltrasoundRightVentricleMeasurement"
cid_concepts[12258] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval", "WallMass"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "EndocardialArea",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "VelocityOfFlowPropagation",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Volume",
        "Area",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
        "BloodPressure",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12259] = "CardiacUltrasoundVentriclesMeasurement"
cid_concepts[12259] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval", "WallMass"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleAWave",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "EndocardialArea",
        "PeakTissueVelocity",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "TimeToLeftVentricleETissueVelocity",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleEWave",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "InterventricularSeptumToPosteriorWallThicknessRatio",
        "IsovolumicRelaxationTime",
        "TimeToLeftVentricleSTissueVelocity",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "LeftVentricleMeridionalWallStress",
        "LeftVentricleEToATissueVelocityRatio",
        "SystolicToDiastolicVelocityRatio",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Volume",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12260] = "CardiacUltrasoundPulmonaryArteryMeasurement"
cid_concepts[12260] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "VelocityOfFlowPropagation",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
        "BloodPressure",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12261] = "CardiacUltrasoundPulmonaryVein"
cid_concepts[12261] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "PulmVeinADurationToMVADurationDifference",
        "PeakGradient",
        "EjectionTime",
        "PulmonaryVeinADurationMitralValveADurationRatio",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "VelocityOfFlowPropagation",
        "AliasVelocity",
        "PulmonaryVeinAVTIToMitralValveVTIRatio",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "CardiovascularOrificeDiameter",
        "BloodPressure",
        "RegurgitantFraction",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12262] = "CardiacUltrasoundPulmonaryValveMeasurement"
cid_concepts[12262] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "AWaveAmplitude",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "BCSlope",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeFromQWaveToPulmonicValveCloses",
        "CEDistance",
        "PulmonaryArteryPressureUsingAccelTime",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "DEExcursion",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12263] = "CardiacUltrasoundVenousReturnPulmonaryMeasurement"
cid_concepts[12263] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "AWaveAmplitude",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "PulmVeinADurationToMVADurationDifference",
        "EjectionTime",
        "PulmonaryVeinADurationMitralValveADurationRatio",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "BCSlope",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeFromQWaveToPulmonicValveCloses",
        "CEDistance",
        "PulmonaryArteryPressureUsingAccelTime",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PulmonaryVeinAVTIToMitralValveVTIRatio",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "DEExcursion",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12264] = "CardiacUltrasoundVenousReturnSystemicMeasurement"
cid_concepts[12264] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12265] = "CardiacUltrasoundAtriaandAtrialSeptumMeasurement"
cid_concepts[12265] = {
    "DCM": ["WallMass"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "VolumeFlow",
        "FlowRadius",
        "LeftAtriumToAorticRootRatio",
        "HeartRate",
        "FlowArea",
        "EndDiastolicVelocity",
        "EndocardialArea",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "LeftAtriumVolumeToRightAtriumVolumeRatio",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "ACInterval",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "VelocityOfFlowPropagation",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Volume",
        "Area",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12266] = "CardiacUltrasoundMitralValveMeasurement"
cid_concepts[12266] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "MitralRegurgitationDpDtDerivedFromMitralRegVelocity",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "MitralValveESeptalSeparation",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "CEDistance",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "MitralValveEPSSEWave",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "DEExcursion",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12267] = "CardiacUltrasoundTricuspidValveMeasurement"
cid_concepts[12267] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "TimeFromQWaveToTricuspidValveOpens",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "CEDistance",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "DEExcursion",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "TricuspidDiastolicFillingPeriodDfpt",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12268] = "CardiacUltrasoundAtrioventricularValveMeasurement"
cid_concepts[12268] = {
    "DCM": [
        "Tau",
        "IndicatorMeanTransitTime",
        "RRInterval",
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "TimeFromQWaveToTricuspidValveOpens",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "MitralRegurgitationDpDtDerivedFromMitralRegVelocity",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "MitralValveESeptalSeparation",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "CEDistance",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "MitralValveEPSSEWave",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "DEExcursion",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "TricuspidDiastolicFillingPeriodDfpt",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12269] = "CardiacUltrasoundInterventricularSeptumMeasurement"
cid_concepts[12269] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "JetArea",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FlowArea",
        "EndDiastolicVelocity",
        "ROIInternalDimensionByUS",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
    ],
}

name_for_cid[12270] = "CardiacUltrasoundAorticValveMeasurement"
cid_concepts[12270] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "AorticValveCuspSeparation",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12271] = "CardiacUltrasoundOutflowTractMeasurement"
cid_concepts[12271] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval", "WallMass"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleAWave",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "AWaveAmplitude",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "EndocardialArea",
        "PeakTissueVelocity",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeToLeftVentricleETissueVelocity",
        "PercentThickening",
        "ACInterval",
        "BCSlope",
        "PreEjectionPeriod",
        "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleEWave",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeFromQWaveToPulmonicValveCloses",
        "CEDistance",
        "DpDtByUS",
        "PulmonaryArteryPressureUsingAccelTime",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "InterventricularSeptumToPosteriorWallThicknessRatio",
        "IsovolumicRelaxationTime",
        "TimeToLeftVentricleSTissueVelocity",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "LeftVentricleMeridionalWallStress",
        "LeftVentricleEToATissueVelocityRatio",
        "SystolicToDiastolicVelocityRatio",
        "AorticValveCuspSeparation",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "TimeAveragedMeanVelocity",
        "DEExcursion",
    ],
    "SCT": [
        "MajorAxis",
        "Volume",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12272] = "CardiacUltrasoundSemilunarValve,AnnulateandSinusMeasurement"
cid_concepts[12272] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "AWaveAmplitude",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "LeafletThickness",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "LeafletSeparation",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "BCSlope",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "TimeFromQWaveToPulmonicValveCloses",
        "CEDistance",
        "DpDtByUS",
        "PulmonaryArteryPressureUsingAccelTime",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "AorticValveCuspSeparation",
        "TimeAveragedMeanVelocity",
        "DEExcursion",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12273] = "CardiacUltrasoundAorticSinotubularJunctionMeasurement"
cid_concepts[12273] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "EndDiastolicVelocity",
        "AorticSinotubularJunctionToAorticRootRatio",
        "ROIInternalDimensionByUS",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
    ],
}

name_for_cid[12274] = "CardiacUltrasoundAortaMeasurement"
cid_concepts[12274] = {
    "DCM": ["Tau", "IndicatorMeanTransitTime", "RRInterval"],
    "LN": [
        "DESlope",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "StenosisPeakGradient",
        "JetArea",
        "EnvelopeDuration",
        "VolumeFlow",
        "TimeToPeakByUS",
        "FlowRadius",
        "HeartRateCorrectedEjectionTime",
        "HeartRate",
        "PreEjectionPeriodEjectionTimeRatio",
        "FlowArea",
        "EndDiastolicVelocity",
        "IsovolumicContractionTime",
        "ClosureToOpeningTime",
        "HRCorrectedMeanVelocityOfCircumferentialFiberShortening",
        "AorticSinotubularJunctionToAorticRootRatio",
        "ROIInternalDimensionByUS",
        "PeakGradient",
        "EjectionTime",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "PercentThickening",
        "ACInterval",
        "PreEjectionPeriod",
        "AWavePeakVelocity",
        "AWaveDuration",
        "PeakSystolicVelocity",
        "StenosisPeakVelocity",
        "DpDtByUS",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "FillingTime",
        "HeartRateCorrectedPreEjectionPeriod",
        "IsovolumicRelaxationTime",
        "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv",
        "VelocityOfFlowPropagation",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "AccelerationSlope",
        "DecelerationSlope",
        "SystolicToDiastolicVelocityRatio",
        "AorticValveCuspSeparation",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "Area",
        "CardiacOutput",
        "CardiacEjectionFraction",
        "CardiovascularOrificeDiameter",
        "Length",
        "StrokeIndex",
        "Circumference",
        "BloodPressure",
        "CardiacIndex",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "StrokeVolume",
        "CardiovascularOrificeArea",
    ],
}

name_for_cid[12275] = "CardiacUltrasoundCoronaryArteryMeasurement"
cid_concepts[12275] = {
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "ROIThicknessByUS",
        "EndDiastolicVelocity",
        "ROIInternalDimensionByUS",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "MajorAxis",
        "RegurgitantFraction",
        "Radius",
        "MinorAxis",
        "Diameter",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
        "Length",
        "Circumference",
    ],
}

name_for_cid[12276] = "CardiacUltrasoundAortoPulmonaryConnectionMeasurement"
cid_concepts[12276] = {
    "DCM": [
        "BloodVelocityPeak",
        "BloodVelocityMean",
        "BloodVelocityMinimum",
        "SystolicBloodVelocityMean",
        "SystolicBloodVelocityPeak",
        "DiastolicBloodVelocityMean",
        "DiastolicBloodVelocityPeak",
    ],
    "LN": [
        "DESlope",
        "ACInterval",
        "PeakEWavePeakAWaveByUS",
        "PulmonaryToSystemicShuntFlowRatio",
        "EWavePeakVelocity",
        "MeanGradient",
        "VelocityTimeIntegral",
        "AWavePeakVelocity",
        "PeakSystolicVelocity",
        "StenosisPeakGradient",
        "StenosisPeakVelocity",
        "VolumeFlow",
        "FlowRadius",
        "HeartRate",
        "TimeAveragedPeakVelocity",
        "EndDiastolicVelocity",
        "VelocityOfFlowPropagation",
        "PeakGradient",
        "AliasVelocity",
        "PressureHalfTime",
        "EVelocityToAnnulusEVelocityRatio",
        "PeakInstantaneousFlowRate",
        "AccelerationSlope",
        "DecelerationSlope",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "SystolicToDiastolicVelocityRatio",
        "EFSlope",
        "PeakReversalVelocityDuringAtrialContraction",
        "TimeAveragedMeanVelocity",
    ],
    "SCT": [
        "RegurgitantFraction",
        "CardiovascularOrificeArea",
        "CardiovascularOrificeDiameter",
    ],
}

name_for_cid[12277] = "CardiacUltrasoundPericardiumandPleuraMeasurement"
cid_concepts[12277] = {
    "LN": ["ROIInternalDimensionByUS", "ROIThicknessByUS"],
    "SCT": ["Radius", "MinorAxis", "Length", "Diameter", "MajorAxis", "Circumference"],
}

name_for_cid[12279] = "CardiacUltrasoundFetalGeneralMeasurement"
cid_concepts[12279] = {
    "LN": [
        "OccipitalFrontalDiameter",
        "FLAC",
        "AbdominalCircumference",
        "HeadCircumference",
        "BiparietalDiameter",
        "CephalicIndex",
        "NuchalTranslucency",
        "TransverseThoracicDiameter",
        "ThoracicArea",
        "TransverseCerebellarDiameter",
        "HCAC",
        "BPDAreaCorrected",
        "TransverseAbdominalDiameter",
        "CisternaMagnaLength",
        "FemurLength",
        "ThoracicCircumference",
        "LeftVentricleRightVentricleDiameterRatio",
        "ForamenOvaleDiameterAorticRootDiameter",
        "CardiacCircumferenceTransverseByUS",
        "CardiothoracicAreaRatio",
        "FLBPD",
        "CardiacCrossSectionalAreaTransverseByUS",
        "AnteriorPosteriorAbdominalDiameter",
        "FLHC",
        "AnteriorPosteriorTrunkDiameter",
        "CardiothoracicCircumferenceRatio",
    ],
    "SCT": ["NumberOfUmbilicalArteries"],
}

name_for_cid[12280] = "CardiacUltrasoundTargetSite"
cid_concepts[12280] = {
    "SCT": [
        "RightVentricleOutflowTract",
        "LeftSuperiorPulmonaryVein",
        "TransverseAorticArch",
        "LeftInferiorPulmonaryVein",
        "Effusion",
        "RightSuperiorPulmonaryVein",
        "RightPulmonaryArtery",
        "PosteriorWall",
        "PulmonaryVeinCommonRightSegment",
        "PreductalRegionOfAorticArch",
        "VenaContracta",
        "PulmonaryVeinCommonLeftSegment",
        "RightInferiorPulmonaryVein",
        "CardiacValveLeaflet",
        "LeftVentricleOutflowTract",
        "AnteriorWall",
        "CardiacValveAnnulus",
        "PulmonaryVeinConfluenceToAtriumConnection",
        "PostductalRegionOfAorticArch",
        "LeftPulmonaryArtery",
    ]
}

name_for_cid[12281] = "CardiacUltrasoundTargetSiteModifier"
cid_concepts[12281] = {
    "NCIt": ["Middle"],
    "SCT": [
        "Distal",
        "Proximal",
        "Lateral",
        "Posterior",
        "Inferior",
        "Anterior",
        "Medial",
    ],
}

name_for_cid[12282] = "CardiacUltrasoundVenousReturnSystemicFindingSite"
cid_concepts[12282] = {
    "SCT": [
        "HepaticVein",
        "LeftSuperiorVenaCava",
        "FontanFenestration",
        "MustardOrSenningSuperiorVenaCavaPathway",
        "RightSuperiorVenaCava",
        "InferiorVenaCava",
        "FontanPathway",
        "FontanBaffleLeak",
        "SuperiorVenaCava",
        "HemiFontanPathway",
        "FontanInferiorVenaCavaPathway",
        "FontanPulmonaryArteryConnection",
        "MustardOrSenningCommonSystemicVenousPathway",
        "GlennPathway",
        "MustardOrSenningInferiorVenaCavaPathway",
    ]
}

name_for_cid[12283] = "CardiacUltrasoundVenousReturnPulmonaryFindingSite"
cid_concepts[12283] = {
    "SCT": [
        "PulmonaryVeinGreatVessel",
        "PulmonaryVein",
        "PulmonaryVeinToAtriumConnection",
        "MustardOrSenningPulmonaryVenousPathway",
        "CorTriatriatumOrifice",
    ]
}

name_for_cid[12284] = "CardiacUltrasoundAtriaandAtrialSeptumFindingSite"
cid_concepts[12284] = {
    "SCT": [
        "RightAtrium",
        "InteratrialSeptumStructure",
        "FontanBaffleLeak",
        "LimbusOfFossaOvalis",
        "LeftAtrium",
        "LeftAuricularAppendage",
        "CommonAtrium",
        "PatentForamenOvale",
        "FontanFenestration",
        "AtrialSeptalDefect",
        "RightAuricularAppendage",
    ]
}

name_for_cid[12285] = "CardiacUltrasoundAtrioventricularValveFindingSite"
cid_concepts[12285] = {
    "SCT": [
        "TricuspidValve",
        "CommonNonMitralNonTricuspidAtrioventricularValveStructure",
        "MitralValve",
    ]
}

name_for_cid[12286] = "CardiacUltrasoundInterventricularSeptumFindingSite"
cid_concepts[12286] = {
    "SCT": [
        "BulboventricularForamen",
        "VentricularSeptalDefect",
        "InterventricularSeptum",
    ]
}

name_for_cid[12287] = "CardiacUltrasoundVentricleFindingSite"
cid_concepts[12287] = {"SCT": ["RightVentricle", "LeftVentricle", "CommonVentricle"]}

name_for_cid[12288] = "CardiacUltrasoundOutflowTractFindingSite"
cid_concepts[12288] = {
    "SCT": [
        "RastelliInterventricularTunnel",
        "PulmonicValve",
        "LeftVentricleToPulmonaryArteryConduitAnastomosis",
        "RightVentricle",
        "LeftVentricle",
        "RightVentricleToPulmonaryArteryConduitAnastomosis",
        "AorticValve",
    ]
}

name_for_cid[12289] = "CardiacUltrasoundSemilunarValve,AnnulusandSinusFindingSite"
cid_concepts[12289] = {
    "SCT": [
        "PulmonicValve",
        "NeoaorticValve",
        "AorticValve",
        "TruncalValveStructure",
        "NeoaorticRoot",
        "RootOfAorta",
    ]
}

name_for_cid[12290] = "CardiacUltrasoundPulmonaryArteryFindingSite"
cid_concepts[12290] = {
    "SCT": ["PulmonaryArtery", "PulmonaryTrunk", "AortaToPulmonaryArteryConnection"]
}

name_for_cid[12291] = "CardiacUltrasoundAortaFindingSite"
cid_concepts[12291] = {
    "SCT": [
        "InfraRenalAorta",
        "InnominateArtery",
        "CoarctationOfAorta",
        "LeftSubclavianArtery",
        "NonCoronarySinus",
        "AorticIsthmus",
        "RightSinusOfValsalva",
        "AorticSinotubularJunction",
        "AorticArch",
        "SupraRenalAorta",
        "AscendingAorta",
        "RootOfAorta",
        "LeftCommonCarotidArtery",
        "AbdominalAorta",
        "StructureSinusOfValsalva",
        "ThoracicAorta",
        "RightCommonCarotidArtery",
        "LeftSinusOfValsalva",
        "RightSubclavianArtery",
    ]
}

name_for_cid[12292] = "CardiacUltrasoundCoronaryArteryFindingSite"
cid_concepts[12292] = {
    "SCT": [
        "CircumflexCoronaryArtery",
        "AnteriorDescendingBranchOfLeftCoronaryArtery",
        "ProximalCircumflexCoronaryArtery",
        "DistalCircumflexCoronaryArtery",
        "PosteriorDescendingCoronaryArtery",
        "SeptalArtery",
        "RightCoronaryArtery",
        "MidCircumflexCoronaryArtery",
        "LeftMainCoronaryArtery",
    ]
}

name_for_cid[12293] = "CardiacUltrasoundAortopulmonaryConnectionFindingSite"
cid_concepts[12293] = {
    "SCT": [
        "ArteriovenousFistulaDisorder",
        "AortaToPulmonaryArteryConnection",
        "PatentDuctusArteriosus",
    ]
}

name_for_cid[12294] = "CardiacUltrasoundPericardiumandPleuraFindingSite"
cid_concepts[12294] = {"SCT": ["Pericardium", "Pleura", "LeftPleura", "RightPleura"]}

name_for_cid[12300] = "CoreEchoMeasurement"
cid_concepts[12300] = {
    "DCM": [
        "LVEndSystolicGLSALAXEndocardial",
        "LVEndSystolicGLS2CHEndocardial",
        "LVEndSystolicGLS4CHEndocardial",
        "LVEndSystolicGLSMultiviewMeanEndocardial",
        "LVPeakSystolicGLS2CHEndocardial",
        "LVPeakSystolicGLS4CHEndocardial",
        "LVPeakSystolicGLSMultiviewMeanEndocardial",
        "LVPeakGLS2CHEndocardial",
        "LVPeakGLS4CHEndocardial",
        "LVPeakGLSMultiviewMeanEndocardial",
        "LVPeakSystolicGLSALAXEndocardial",
        "LVEndSystolicMechanicalDispersion",
        "LVPeakGLSALAXEndocardial",
    ],
    "LN": [
        "TricuspidValveDiastolicVTI",
        "MitralValvePeakInstantaneousGradient",
        "LeftAtrialEndSystolicVolumeBiplaneAreaLength",
        "LeftVentricularInternalSystolicDimensionByUSMModeBSA",
        "LeftVentricularPreEjectionPeriod",
        "LVBasalAnteriorTimeToSVmaxTsBasalAnterior",
        "MitralRegurgitationPISARadius",
        "PulmonicValveVmax",
        "PulmonaryVeinSWaveVmax",
        "LeftAtrialEndSystolicDiameterAPMMBSA",
        "RightVentricularMidCavityDimension4C",
        "LeftVentricularEndSystolicVolumeBiplaneMOD",
        "MitralRegurgitationVmax",
        "LeftVentricularEjectionFractionSinglePlane4CMOD",
        "InterventricularSeptumSystolicDimensionMM",
        "LeftVentricularMassDimensionMethodMMHeight2Point7",
        "LVMidSeptalTimeToSVmaxTsMidSeptal",
        "AorticSinotubularJunctionDimension",
        "LeftVentricularInternalDiastolicDimensionByUSMModeBSA",
        "TricuspidRegurgitationPISARadius",
        "InferiorVenaCavaDiameterAtEndExpiration",
        "AorticRegurgitantJetWidthLVOTWidthPercent",
        "MitralValveMeanGradient",
        "RightVentricularEjectionTime",
        "MitralAnnulusDiastolicDiameterA2C",
        "LeftAtrialEndSystolicVolumeBiplaneAreaLengthBSA",
        "MitralRegurgitationPeakGradient",
        "LeftVentricularInternalSystolicDimensionMM",
        "LeftVentricularStrokeVolume3D",
        "TricuspidRegurgitationVmax",
        "AorticValveAreaContinuityVmax",
        "TricuspidRegurgitationVenaContractaWidth",
        "PulmonicValveVTI",
        "PulmonicRegurgitationEndDiastolicVelocity",
        "MitralRegurgitationVenaContractaWidth",
        "LeftVentricularEndSystolicVolume3D",
        "LeftVentricularMassDimensionMethodMMBSA",
        "MitralValveDecelerationTime",
        "RightVentricularFreeWallThicknessMM",
        "LeftAtrialEndSystolicLength2C",
        "LVMidPosteriorTimeToSVmaxTsMidPosterior",
        "InterventricularSeptumSystolicDimension2D",
        "LeftVentricularFractionalShorteningOfMinorAxisMM",
        "LeftVentricularInternalDiastolicDimensionMM",
        "MitralValveAreaPressureHalfTime",
        "TricuspidValvePressureHalfTime",
        "LeftVentricularEndDiastolicLength4C",
        "InterventricularSeptumDiastolicDimensionMM",
        "MitralValveFlowPropagationVelocityVp",
        "LeftVentricularInternalSystolicDimension2D",
        "AorticValveEffectiveRegurgitantOrificeArea",
        "MitralRegurgitantFlowPISA",
        "LeftVentricularPosteriorWallSystolicThicknessByUS2D",
        "LeftAtrialEndSystolicVolumeBiplaneMOD",
        "LeftVentricularEjectionFractionTeichholzMM",
        "TricuspidValveAPrimeVmax",
        "AorticRegurgitationPISARadius",
        "LeftVentricularMassDimensionMethod2D",
        "AorticValveAreaContinuityVmaxBSA",
        "MitralSeptalEPrimeVmax",
        "PulmonicRegurgitationEndDiastolicPeakGradient",
        "MitralValveAreaPlanimetry",
        "MitralRegurgitationPISAVelocity",
        "PulmonicRegurgitationVmax",
        "LVMidLateralTimeToSVmaxTsMidLateral",
        "LeftVentricularMassDimensionMethodMM",
        "RightVentricularFreeWallThickness2D",
        "LeftVentricularEndDiastolicVolume3D",
        "InterventricularSeptumTimeToPeakDisplacement",
        "LeftVentricularFractionalShorteningOfMinorAxis2D",
        "LeftVentricularInternalDiastolicDimension2D",
        "TricuspidValveAWaveVmax",
        "InterventricularSeptumDiastolicDimension2D",
        "AorticValveMeanBloodVelocity",
        "MitralValveEffectiveRegurgitantOrificeAreaPISA",
        "LeftVentricularInternalDiastolicDimensionByUS2DBSA",
        "LeftVentricularPosteriorWallSystolicThicknessByUSMMode",
        "LeftAtrialEndSystolicVolumeBiplaneMODBSA",
        "MitralRegurgitantFractionPISA",
        "RightPulmonaryArteryDiameterAtEndSystole",
        "AorticRegurgitationPISAVelocity",
        "LeftVentricularMassAreaLengthHeight2Point7",
        "AorticValveAreaContinuityVTI",
        "PulmonicValveEjectionTime",
        "PulmonicValvePeakGradient",
        "LVMidInferiorTimeToSVmaxTsMidInferior",
        "RightVentricularEndSystolicArea4C",
        "LeftVentricularEpicardialAreaSAXPMLevel",
        "MitralValveAreaPISA",
        "LeftAtrialEndSystolicArea2C",
        "LeftVentricularMassDimensionMethod2DHeight2Point7",
        "LeftVentricularEndDiastolicVolumeBiplaneMOD",
        "LeftVentricularOutflowTractVmax",
        "MitralValvePressureHalfTime",
        "AorticRegurgitationVolumeContinuityVTI",
        "MitralValveEToARatio",
        "TricuspidValveEPrimeVmax",
        "AorticRegurgitationPressureHalfTime",
        "LeftAtrialEndSystolicVolumeSinglePlane2CMOD",
        "PulmonicAnnulusDiameter",
        "MitralValveEWaveVmax",
        "LeftVentricularPosteriorWallDiastolicThicknessByUS2D",
        "RightAtrialMinorAxisDimension4CBSA",
        "MitralRegurgitantFractionContinuityVTI",
        "AorticValveMeanGradient",
        "LeftVentricularMassAreaLengthBSA",
        "AorticValveAreaContinuityVTIBSA",
        "LeftVentricularEjectionFractionTeichholz2D",
        "RightVentricularFractionalAreaChange",
        "LeftVentricularMassDimensionMethod2DBSA",
        "LVMidAnteroseptalTimeToSVmaxTsMidAnteroseptal",
        "RightVentricularEndDiastolicArea4C",
        "RightVentricularOutflowTractVTI",
        "MitralValveAWaveVmax",
        "LeftAtrialEndSystolicArea4C",
        "LeftVentricularEndocardialAreaSAXPMLevel",
        "LeftVentricularEndDiastolicVolumeBiplaneMODBSA",
        "LeftVentricularOutflowTractDimension2D",
        "MitralValveVmax",
        "AorticRegurgitationVolumePISA",
        "AorticRegurgitationVenaContractaWidth",
        "TricuspidValveEWaveVmax",
        "LeftAtrialEndSystolicVolumeSinglePlane4CMOD",
        "RightAtrialMinorAxisDimension4C",
        "MitralLateralEPrimeVmax",
        "LeftVentricularPosteriorWallDiastolicThicknessByUSMMode",
        "LVBasalLateralTimeToSVmaxTsBasalLateral",
        "PulmonaryVeinAWaveDuration",
        "AorticValvePeakInstantaneousGradient",
        "LeftVentricularMassAreaLength",
        "AorticAnnulusDiameter",
        "MitralValveVTI",
        "AorticRegurgitationVTI",
        "LeftAtrialEndSystolicDiameterAP2D",
        "RightVentricularBasalDimension4C",
        "TricuspidRegurgitationPeakGradient",
        "MitralValveAWaveDuration",
        "RightVentricularOutflowTractDiameterAtSubvalvularLevelRVOTProximal",
        "LVMidAnteriorTimeToSVmaxTsMidAnterior",
        "LeftVentricularEjectionFraction3D",
        "LeftVentricularEndDiastolicVolumeSinglePlane2CMOD",
        "LeftVentricularEndSystolicVolumeSinglePlane4CMOD",
        "LeftVentricularMassTruncatedEllipseHeight2Point7",
        "AorticRegurgitationVmax",
        "TricuspidAnnulusDiameter",
        "TricuspidValveSPrimeVmax",
        "LeftVentricularOutflowTractVTI",
        "RightAtrialMajorAxisDimension4C",
        "MitralAnnulusVTI",
        "MainPulmonaryArteryVmax",
        "LeftVentricularIsovolumicRelaxationTimeByTDI",
        "LeftPulmonaryArteryDiameterAtEndSystole",
        "LVBasalInferiorTimeToSVmaxTsBasalInferior",
        "AorticValveVmax",
        "AorticRegurgitantFlow",
        "PulmonaryVeinAWaveVmax",
        "AorticRootDiameter",
        "RightVentricularOutflowTractDiameterAtPulmonicValveRVOTDistal",
        "LeftAtrialEndSystolicDiameterAP2DBSA",
        "LeftVentricularEndDiastolicVolumeSinglePlane4CMOD",
        "TricuspidValveMeanGradient",
        "MainPulmonaryArteryDiameterAtEndSystole",
        "LeftVentricularEjectionFractionBiplaneMOD",
        "LVBasalSeptalTimeToSVmaxTsBasalSeptal",
        "LeftVentricularEndSystolicVolumeSinglePlane2CMOD",
        "MitralRegurgitationVolumePISA",
        "LeftVentricularMassTruncatedEllipseBSA",
        "TricuspidValveClosureToOpeningTime",
        "TricuspidValveVmax",
        "AorticRegurgitantFraction",
        "LeftVentricularIsovolumicRelaxationTimeByDoppler",
        "MitralAnnulusDiastolicDiameterPLAX",
        "RightVentricularPreEjectionPeriod",
        "RightAtrialEndSystolicArea4C",
        "LVBasalAnteroseptalTimeToSVmaxTSBasalAnteroseptal",
        "LeftAtrialEndSystolicLength4C",
        "LeftVentricularInternalSystolicDimensionByUS2DBSA",
        "LeftVenticularPosteriorWallTimeToPeakDisplacement",
        "AorticValveVTI",
        "TricuspidAnnularPlaneSystolicExcursionTAPSE",
        "PulmonaryVeinDWaveVmax",
        "TricuspidValvePeakGradient",
        "MitralRegurgitationVolumeContinuityVTI",
        "LeftAtrialEndSystolicDiameterAPMM",
        "RightVentricularMyocardialPerformanceIndex",
        "LVBasalPosteriorTimeToSVmaxTsBasalPosterior",
        "LeftVentricularEjectionFractionSinglePlane2CMOD",
        "LeftVentricularMass",
        "LVTsSDDyssynchronyIndex",
        "AorticRootDiameterBSA",
        "LeftVentricularEndSystolicVolumeBiplaneMODBSA",
        "TricuspidValveDecelerationTime",
        "AscendingAortaDimension",
        "AorticRegurgitantJetAreaLVOTAreaPercent",
        "MitralAnnulusDiastolicDiameterA4C",
        "PulmonicValveAccelerationTime",
    ],
}

name_for_cid[12301] = "MeasurementSelectionReason"
cid_concepts[12301] = {
    "DCM": ["MostRecentValueChosen", "MeanValueChosen", "UserChosenValue"],
    "SCT": ["Minimum", "Maximum"],
}

name_for_cid[12302] = "EchoFindingObservationType"
cid_concepts[12302] = {
    "DCM": ["BehaviorOfTheFindingSite", "StructureOfTheFindingSite"],
    "SCT": ["HemodynamicMeasurements"],
}

name_for_cid[12303] = "EchoMeasurementType"
cid_concepts[12303] = {
    "DCM": [
        "DirectlyMeasured",
        "Calculated",
        "FractionalChange",
        "Indexed",
        "ManualEntry",
    ],
    "SCT": ["Ratio"],
}

name_for_cid[12304] = "EchoMeasuredProperty"
cid_concepts[12304] = {
    "DCM": [
        "LongitudinalLagrangianStrain",
        "LongitudinalLagrangianStrainRate",
        "MechanicalDispersion",
        "TimeToPeakLongitudinalLagrangianStrain",
        "PeakLongitudinalLagrangianStrain",
        "CircumferentialLagrangianStrainRate",
        "CircumferentialLagrangianStrain",
        "RadialLagrangianStrainRate",
        "RadialLagrangianStrain",
        "VenaContractaWidth",
        "RegurgitationJetWidth",
        "RegurgitationJetArea",
        "PISARadius",
        "PeakTissueVelocity",
        "PeakBloodPressure",
        "MaximumOrificeArea",
        "ExcursionDistance",
        "EffectiveOrificeArea",
        "DyssynchronyIndex",
    ],
    "LN": [
        "MeanGradientPressureByDoppler",
        "VelocityTimeIntegral",
        "VolumeFlowRate",
        "InternalDimension",
        "PeakGradientPressure",
        "PeakInstantaneousFlowRate",
        "DecelerationTime",
        "AccelerationTime",
        "PeakBloodVelocity",
        "DpDtByUS",
        "Thickness",
        "FractionalShortening",
        "AliasVelocity",
        "PressureHalfTime",
        "EpicardialArea",
        "MyocardialPerformanceIndexTei",
        "MeanBloodVelocity",
    ],
    "SCT": [
        "Interval",
        "VascularResistance",
        "Volume",
        "Area",
        "MeanBloodPressure",
        "CardiacEjectionFraction",
        "Length",
        "BloodPressure",
        "MassQuantity",
        "RegurgitantFraction",
        "MinorAxis",
        "Angle",
        "Diameter",
        "StrokeVolume",
    ],
}

name_for_cid[12305] = "BasicEchoAnatomicSite"
cid_concepts[12305] = {
    "DCM": [
        "LeftVentricleApicalAnterolateralSegment",
        "LeftVentricleApicalInferolateralSegment",
        "LeftVentricleApicalInferoseptalSegment",
        "LeftVentricleApicalAnteroseptalSegment",
        "RightVentricleAnteriorWall",
        "RightVentricleOutflowTractProximal",
        "RightVentricleOutflowTractDistal",
    ],
    "SCT": [
        "PulmonicValve",
        "PulmonaryVeinGreatVessel",
        "AorticValve",
        "AorticSinotubularJunction",
        "ApexOfRightVentricle",
        "LeftVentricleBasalInferiorSegment",
        "InferiorVenaCava",
        "RightPulmonaryArtery",
        "AorticRoot",
        "RightAtrium",
        "TricuspidValve",
        "LeftVentricleMidAnterolateralSegment",
        "RightVentricle",
        "RightVentricleMidventricularSegment",
        "PulmonicRing",
        "LeftVentricleMidAnteroseptalSegment",
        "LeftVentricleOutflowTract",
        "LeftPulmonaryArtery",
        "TricuspidAnnulus",
        "LeftVentricleMidInferiorSegment",
        "LeftVentricleApicalLateralSegment",
        "LeftVentricleBasalAnterolateralSegment",
        "LeftVentricleMidInferolateralSegment",
        "LateralMitralAnnulus",
        "LeftVentricleMidAnteriorSegment",
        "LeftVentricleMyocardium",
        "LeftVentricleApicalAnteriorSegment",
        "LeftVentricleBasalAnteriorSegment",
        "MitralAnnulus",
        "MedialMitralAnnulus",
        "LeftVentricle",
        "LeftVentricleMidInferoseptalSegment",
        "AorticValveRing",
        "InterventricularSeptum",
        "RightVentricleOutflowTract",
        "LeftVentricleBasalAnteroseptalSegment",
        "LeftVentricleBasalInferolateralSegment",
        "LeftVentricleApicalInferiorSegment",
        "LeftVentricleBasalInferoseptalSegment",
        "TrunkOfPulmonaryArtery",
        "ApexOfLeftVentricle",
        "LeftVentricleApicalSeptalSegment",
        "AscendingAorta",
        "PulmonaryArtery",
        "LeftAtrium",
        "RightVentricleBasalSegment",
        "MitralValve",
    ],
}

name_for_cid[12306] = "EchoFlowDirection"
cid_concepts[12306] = {"SCT": ["RetrogradeDirection", "AntegradeDirection"]}

name_for_cid[12307] = "CardiacPhaseandTimePoint"
cid_concepts[12307] = {
    "DCM": [
        "MitralValveOpening",
        "MitralValveClosure",
        "FullCardiacCycle",
        "ARWave",
        "AtrialDiastolicFillingDWave",
        "AorticValveOpening",
        "PreEjectionPeriod",
        "AorticValveClosure",
        "ElectromechanicalDelay",
    ],
    "SCT": [
        "Diastole",
        "VentricularIsovolumicContraction",
        "EndDiastole",
        "Diastasis",
        "EndSystole",
        "AtrialSystoleAWave",
        "VentricularEjectionSWave",
        "DiastolicRapidInflowEWave",
        "VentricularIsovolumicRelaxation",
        "Systole",
    ],
}

name_for_cid[12308] = "UltrasoundShearWaveMeasurement"
cid_concepts[12308] = {
    "DCM": ["Elasticity", "ShearWaveDispersionSlope", "ShearWaveSpeed", "ROIDepth"],
    "SCT": ["AreaOfDefinedRegion"],
}

name_for_cid[12309] = "CoreEchoStrainMeasurement"
cid_concepts[12309] = {
    "DCM": [
        "LVPeakSystolicGLS2CHEndocardial",
        "LVPeakSystolicGLS4CHEndocardial",
        "LVPeakSystolicGLSMultiviewMeanEndocardial",
        "LVPeakGLS2CHEndocardial",
        "LVPeakGLS4CHEndocardial",
        "LVPeakGLSMultiviewMeanEndocardial",
        "LVPeakSystolicGLSALAXEndocardial",
        "LVEndSystolicGLSALAXEndocardial",
        "LVEndSystolicGLS2CHEndocardial",
        "LVEndSystolicGLS4CHEndocardial",
        "LVEndSystolicMechanicalDispersion",
        "LVEndSystolicGLSMultiviewMeanEndocardial",
        "LVPeakGLSALAXEndocardial",
    ]
}

name_for_cid[12310] = "MyocardialStrainMethod"
cid_concepts[12310] = {
    "DCM": ["MyocardialMidlayerMethod", "EpicardialMethod", "EndocardialMethod"]
}

name_for_cid[12311] = "EchoMeasuredStrainProperty"
cid_concepts[12311] = {
    "DCM": [
        "LongitudinalLagrangianStrainRate",
        "MechanicalDispersion",
        "TimeToPeakLongitudinalLagrangianStrain",
        "PeakLongitudinalLagrangianStrain",
        "CircumferentialLagrangianStrainRate",
        "CircumferentialLagrangianStrain",
        "RadialLagrangianStrainRate",
        "RadialLagrangianStrain",
        "LongitudinalLagrangianStrain",
    ]
}

name_for_cid[12320] = "GeneralUltrasoundReportDocumentTitle"
cid_concepts[12320] = {
    "LN": [
        "USTendonReport",
        "USPancreasReport",
        "USBreastReport",
        "USProstateTransrectalReport",
        "USThyroidReport",
        "UltrasoundReport",
        "USSpleenReport",
        "USShoulderReport",
        "USScrotumAndTesticleReport",
        "USLiverReport",
        "USKidneyReport",
    ]
}

name_for_cid[12321] = "ElastographySite"
cid_concepts[12321] = {
    "SCT": [
        "Testis",
        "Pancreas",
        "Liver",
        "Prostate",
        "TendonOfRotatorCuffOfShoulder",
        "Thyroid",
        "Breast",
        "Spleen",
        "Kidney",
        "PatellarTendon",
        "AchillesTendon",
    ]
}

name_for_cid[12322] = "ElastographyMeasurementSite"
cid_concepts[12322] = {
    "DCM": ["Background"],
    "SCT": [
        "CouinaudHepaticSegmentIII",
        "CouinaudHepaticSegmentVII",
        "CouinaudHepaticSegmentVI",
        "CouinaudHepaticSegmentI",
        "ThyroidNodule",
        "CouinaudHepaticSegmentV",
        "CouinaudHepaticSegmentVIII",
        "CouinaudHepaticSegmentII",
        "CouinaudHepaticSegmentIV",
    ],
}

name_for_cid[12323] = "UltrasoundRelevantPatientCondition"
cid_concepts[12323] = {
    "SCT": ["NotFasting", "AcuteHepatitis", "HepaticCongestion", "PatientHasPacemaker"]
}

name_for_cid[12324] = "ShearWaveDetectionMethod"
cid_concepts[12324] = {"DCM": ["ParticleVelocityMethod", "ParticleDisplacementMethod"]}

name_for_cid[12325] = "LiverUltrasoundStudyIndication"
cid_concepts[12325] = {
    "SCT": [
        "AlcoholicLiverDisease",
        "NonalcoholicFattyLiverDiseaseNAFLD",
        "NonalcoholicSteatohepatitisNASH",
        "SerumAlbuminBelowReferenceRange",
        "SerumBilirubinRaised",
        "Jaundice",
        "SerumAlanineAminotransferaseLevelRaised",
        "HepatitisC",
        "Hepatomegaly",
        "NonalcoholicFattyLiverNAFL",
        "Hepatitis",
        "HepatitisB",
        "CirrhosisOfLiver",
    ]
}
