# Auto-generated by generate_concept_dicts.py.

# Dict with scheme designator keys; value format is:
#   {keyword: {code1: (meaning, cid_list), code2: ...}
#
# Most keyword identifiers map to a single code, but not all

concepts = {}

concepts["DCM"] = {
    "AAPM204APDimension": {"113935": ("AAPM 204 AP Dimension", [10023])},
    "AAPM204EffectiveDiameterEstimatedFromPatientAge": {
        "113937": ("AAPM 204 Effective Diameter Estimated From Patient Age", [10023])
    },
    "AAPM204LateralDimension": {"113934": ("AAPM 204 Lateral Dimension", [10023])},
    "AAPM204SumOfLateralAndAPDimension": {
        "113936": ("AAPM 204 Sum of Lateral and AP Dimension", [10023])
    },
    "AAPM220": {"113987": ("AAPM 220", [])},
    "ACDConstant": {"111768": ("ACD Constant", [4237])},
    "ACRAccreditationPhantomCT": {
        "113682": ("ACR Accreditation Phantom - CT", [4051, 4052])
    },
    "ACRAccreditationPhantomECT": {
        "113686": ("ACR Accreditation Phantom - ECT", [4051, 4052])
    },
    "ACRAccreditationPhantomECTPET": {
        "113688": ("ACR Accreditation Phantom - ECT/PET", [4051, 4052])
    },
    "ACRAccreditationPhantomMR": {
        "113683": ("ACR Accreditation Phantom - MR", [4051, 4052])
    },
    "ACRAccreditationPhantomMammography": {
        "113684": ("ACR Accreditation Phantom - Mammography", [4051, 4052])
    },
    "ACRAccreditationPhantomPET": {
        "113687": ("ACR Accreditation Phantom - PET", [4051, 4052])
    },
    "ACRAccreditationPhantomPETFaceplate": {
        "113689": ("ACR Accreditation Phantom - PET Faceplate", [4051, 4052])
    },
    "ACRAccreditationPhantomStereotacticBreastBiopsy": {
        "113685": (
            "ACR Accreditation Phantom - Stereotactic Breast Biopsy",
            [4051, 4052],
        )
    },
    "ACRGuidelinePerformanceOfAdultCTColonography": {
        "112248": ("ACR Guideline, Performance of Adult CT Colonography", [6208])
    },
    "ACRPositionStatement": {"112036": ("ACR Position Statement", [6136, 6208])},
    "ACRStandardCTMedicalPhysicsPerformanceMonitoring": {
        "112249": ("ACR Standard, CT medical physics performance monitoring", [6208])
    },
    "ADCHyperintense": {"130595": ("ADC Hyperintense", [6335, 6343, 6344])},
    "ADCHypointense": {"130596": ("ADC Hypointense", [6335, 6343, 6344])},
    "AF10": {"130713": ("AF10", [3030])},
    "AF9": {"130712": ("AF9", [3030])},
    "AGN15099889Zr": {"126729": ("AGN-150998 ^89^Zr", [4021])},
    "AIC": {"126223": ("AIC", [218, 7180, 7469])},
    "AIFIgnored": {"126360": ("AIF Ignored", [4103, 4106])},
    "AIN76": {"127272": ("AIN76", [607])},
    "AIN93G": {"127273": ("AIN93G", [607])},
    "AIN93M": {"127274": ("AIN93M", [607])},
    "ANGPattern": {"109902": ("ANG Pattern", [8301])},
    "APMinus45": {"112301": ("AP-45", [7303])},
    "APPlus45": {"112300": ("AP+45", [7303])},
    "ARWave": {"125323": ("AR-wave", [12307])},
    "ATSMCu60": {"126700": ("ATSM Cu^60^", [4021])},
    "ATSMCu61": {"126701": ("ATSM Cu^61^", [4021])},
    "ATSMCu62": {"126702": ("ATSM Cu^62^", [4021])},
    "AUCBN180": {"126327": ("AUCBN180", [])},
    "AWavePeakPressure": {"109016": ("A wave peak pressure", [3337, 3611])},
    "AWavePressureAverage": {"109017": ("A wave pressure, average", [3337])},
    "AbdominalCompressionArch": {
        "130655": (
            "Abdominal Compression Arch",
            [7151, 7157, 7193, 9505, 9513, 9520, 9573],
        )
    },
    "AbdominalCompressionBelt": {
        "130654": (
            "Abdominal Compression Belt",
            [7151, 7157, 7193, 9505, 9513, 9520, 9573],
        )
    },
    "AbdominalCompressionBeltLength": {
        "130660": ("Abdominal Compression Belt Length", [])
    },
    "AbdominalCompressionBeltPressure": {
        "130661": ("Abdominal Compression Belt Pressure", [])
    },
    "AbdominalCompressionPlatePositionNumber": {
        "130659": ("Abdominal Compression Plate Position Number", [])
    },
    "AbdominalImagingSpecialty": {"128006": ("Abdominal Imaging Specialty", [7449])},
    "AbdominalRadiology": {"128170": ("Abdominal Radiology", [7030])},
    "AbdominalRespiratoryEffort": {"130432": ("Abdominal Respiratory Effort", [3005])},
    "AbdominalRespiratoryInductance": {
        "130428": ("Abdominal Respiratory Inductance", [3005])
    },
    "AbdominalRespiratoryPVDF": {"130430": ("Abdominal Respiratory PVDF", [3005])},
    "AblationOff": {"109038": ("Ablation off", [3339])},
    "AblationOn": {"109039": ("Ablation on", [3339])},
    "AbnormalCalcifications": {"112063": ("Abnormal calcifications", [6101])},
    "AbnormalDischarge": {"111491": ("Abnormal discharge", [6062])},
    "AbnormalDistributionOfAnatomicStructure": {
        "112028": ("Abnormal Distribution of Anatomic Structure", [])
    },
    "AbnormalInterstitialPattern": {
        "112004": ("Abnormal interstitial pattern", [6102, 6104, 6106])
    },
    "AbnormalLines1D": {"112061": ("Abnormal lines (1D)", [6101])},
    "AbnormalLucency": {"112062": ("Abnormal lucency", [6101])},
    "AbnormalOpacity": {"112033": ("Abnormal opacity", [6101])},
    "AbnormalRegion": {"122451": ("Abnormal Region", [])},
    "AbnormalTexture": {"112064": ("Abnormal texture", [6101])},
    "AbnormallyHighSensitivity": {"111849": ("Abnormally high sensitivity", [4254])},
    "AbruptDuctTermination": {"111427": ("Abrupt duct termination", [6054, 6057])},
    "AbruptInterface": {"111358": ("Abrupt interface", [6153])},
    "Absent": {"121054": ("Absent", [])},
    "AbsoluteOphthalmicThickness": {
        "111930": ("Absolute ophthalmic thickness", [4263])
    },
    "AbsoluteRegionalBloodFlow": {
        "126390": ("Absolute Regional Blood Flow", [218, 4108, 7180, 7469])
    },
    "AbsoluteRegionalBloodVolume": {
        "126391": ("Absolute Regional Blood Volume", [218, 4108, 7180, 7469])
    },
    "AbsoluteValuesOfVentricularMeasurements": {
        "122608": ("Absolute Values Of Ventricular Measurements", [])
    },
    "AbsorbedDose": {"128513": ("Absorbed Dose", [218, 7180, 7469, 10070])},
    "AccessControlDecision": {"110147": ("Access Control Decision", [401, 403])},
    "AccessionNumber": {"121022": ("Accession Number", [])},
    "AccessoryClearanceInterlock": {
        "130742": ("Accessory Clearance Interlock", [9561, 9568])
    },
    "AccessoryTray": {"130124": ("Accessory Tray", [9518, 9520])},
    "AccumulatedAverageGlandularDose": {
        "111637": ("Accumulated Average Glandular Dose (mammo)", [10050])
    },
    "AccumulatedCtdivolForwardEstimate": {
        "113906": ("Accumulated CTDIvol Forward Estimate", [])
    },
    "AccumulatedDLPForwardEstimate": {
        "113905": ("Accumulated DLP Forward Estimate", [])
    },
    "AccumulatedDoseData": {"130500": ("Accumulated Dose Data", [])},
    "AccumulatedXRayDoseData": {"113702": ("Accumulated X-Ray Dose Data", [])},
    "AcetabularAngle": {"121224": ("Acetabular angle", [218, 7550])},
    "AcetabularCenterOfRotation": {
        "112303": ("Acetabular Center of Rotation", [7305, 7306])
    },
    "AcetabularCupInsert": {"112306": ("Acetabular Cup Insert", [7307, 7308])},
    "AcetabularCupMonoblock": {"112307": ("Acetabular Cup Monoblock", [7307, 7308])},
    "AcetabularCupShell": {"112305": ("Acetabular Cup Shell", [7307, 7308])},
    "Acinar": {"112146": ("Acinar", [6123])},
    "AcquiredImage": {"113795": ("Acquired Image", [])},
    "AcquiredVolume": {"128160": ("Acquired Volume", [1000])},
    "AcquisitionDate": {"126201": ("Acquisition Date", [])},
    "AcquisitionDeviceID": {"122143": ("Acquisition Device ID", [])},
    "AcquisitionDeviceType": {"122142": ("Acquisition Device Type", [])},
    "AcquisitionDoseAreaProductTotal": {
        "113727": ("Acquisition Dose Area Product Total", [10050])
    },
    "AcquisitionDoseRPTotal": {"113729": ("Acquisition Dose (RP) Total", [])},
    "AcquisitionDuration": {"121142": ("Acquisition Duration", [])},
    "AcquisitionEquipment": {"109101": ("Acquisition Equipment", [7005])},
    "AcquisitionEquipmentAlignment": {
        "125023": ("Acquisition Equipment Alignment", [7100])
    },
    "AcquisitionFramesCorrespondingToVolume": {
        "121346": ("Acquisition frames corresponding to volume", [7201])
    },
    "AcquisitionInitiationAfterEndOfRadiation": {
        "130794": ("Acquisition Initiation after end of Radiation", [9270])
    },
    "AcquisitionInitiationBeforeStartOfRadiation": {
        "130793": ("Acquisition Initiation before start of Radiation", [9270])
    },
    "AcquisitionInitiationByTriggeringParameter": {
        "130795": ("Acquisition Initiation by triggering parameter", [9270])
    },
    "AcquisitionInitiationNotDefined": {
        "130792": ("Acquisition Initiation not defined", [9270])
    },
    "AcquisitionInitiationType": {"130791": ("Acquisition Initiation Type", [])},
    "AcquisitionNumber": {"113608": ("Acquisition Number", [10022])},
    "AcquisitionPlane": {"113764": ("Acquisition Plane", [])},
    "AcquisitionProtocol": {"125203": ("Acquisition Protocol", [])},
    "AcquisitionTime": {"126202": ("Acquisition Time", [])},
    "ActionOnFluid": {"111456": ("Action on fluid", [])},
    "ActiveIngredientUndilutedConcentration": {
        "121380": ("Active Ingredient Undiluted Concentration", [4050])
    },
    "ActiveSurveillanceOfProstateCancer": {
        "130589": ("Active surveillance of prostate cancer", [6327])
    },
    "ActivityMeasurementDevice": {"113540": ("Activity Measurement Device", [])},
    "ActivityWorkload": {"122709": ("Activity workload", [])},
    "AcuteMIPresent": {"122172": ("Acute MI Present", [])},
    "AcuteUnilateralRenalBlockage": {
        "113560": ("Acute unilateral renal blockage", [64, 10045])
    },
    "AdaptiveFiltering": {"123102": ("Adaptive Filtering", [7162])},
    "AdaptiveRadiationTherapyNote": {"130039": ("Adaptive Radiation Therapy Note", [])},
    "AddAddendumToReport": {"128001": ("Add Addendum to Report", [9231])},
    "AddIntermediate": {"111673": ("Add Intermediate", [])},
    "AddNear": {"111672": ("Add Near", [])},
    "AddOther": {"111674": ("Add Other", [])},
    "AddendedReport": {"121361": ("Addended report", [7009])},
    "Addendum": {"121078": ("Addendum", [])},
    "AdditionalEvaluationRequestedFromAbnormalScreeningStudy": {
        "111419": (
            "Additional evaluation requested from abnormal screening study",
            [6051],
        )
    },
    "AdditionalEvaluationRequestedFromPriorStudy": {
        "111415": ("Additional evaluation requested from prior study", [6051])
    },
    "AdditionalProjections": {"111135": ("Additional projections", [6028, 6029])},
    "AdditionalTeachingFileInformation": {
        "128712": ("Additional Teaching File Information", [7018])
    },
    "Adenolipoma": {"111248": ("Adenolipoma", [])},
    "Adenomyoepithelioma": {"111250": ("Adenomyoepithelioma", [])},
    "AdhocMeasurements": {"125303": ("Adhoc Measurements", [])},
    "AdiabaticTissueHomogeneityATHModel": {
        "126346": ("Adiabatic Tissue Homogeneity (ATH) Model", [4101, 4106])
    },
    "AdministeredActivity": {"113507": ("Administered activity", [10050])},
    "AdministrationDiscontinued": {"130220": ("Administration discontinued", [])},
    "AdministrationMode": {"130181": ("Administration Mode", [])},
    "AdministrationStepType": {"130250": ("Administration Step Type", [])},
    "AdministratorOfRadiologyDepartment": {
        "128673": ("Administrator of Radiology Department", [7450, 7452])
    },
    "AdultCongenitalCardiacUltrasoundReport": {
        "125197": ("Adult Congenital Cardiac Ultrasound Report", [7000, 12245])
    },
    "AdultEchocardiographyProcedureReport": {
        "125200": ("Adult Echocardiography Procedure Report", [7000])
    },
    "AdvancedLevel": {"128727": ("Advanced level", [7016])},
    "AdventitialDissection": {"122397": ("Adventitial Dissection", [3492])},
    "AdverseEventDetectionDatetime": {
        "130215": ("Adverse Event Detection DateTime", [])
    },
    "AerobicIndex": {"122772": ("Aerobic index", [3238])},
    "AgatstonScoreThreshold": {"122657": ("Agatston Score Threshold", [])},
    "AgatstonScoringMethod": {"112055": ("Agatston scoring method", [6140])},
    "AgeAtFirstFullTermPregnancy": {"111519": ("Age at First Full Term Pregnancy", [])},
    "AgeAtMenopause": {"111520": ("Age at Menopause", [])},
    "AgeAtOccurrence": {"111538": ("Age at Occurrence", [])},
    "AgeCorrected": {"111838": ("Age corrected", [4252])},
    "AgeEnded": {"111525": ("Age Ended", [])},
    "AgeStarted": {"111524": ("Age Started", [])},
    "AgeWhenFirstMenstrualPeriodOccurred": {
        "111518": ("Age when first menstrual period occurred", [])
    },
    "AgeWhenHysterectomyPerformed": {"111521": ("Age when hysterectomy performed", [])},
    "AgeWhenLeftOvaryRemoved": {"111522": ("Age when left ovary removed", [])},
    "AgeWhenRightOvaryRemoved": {"111523": ("Age when right ovary removed", [])},
    "AgreementWithPriorFindings": {"122775": ("Agreement with prior findings", [3217])},
    "Air": {"112143": ("Air", [6122])},
    "AirBronchiologram": {"112070": ("Air bronchiologram", [6102, 6105])},
    "AirBronchogram": {"112071": ("Air bronchogram", [6102, 6105])},
    "AirChanges": {"127172": ("Air changes", [])},
    "AirCrescent": {"112072": ("Air crescent", [6102, 6105])},
    "AirDetected": {"130175": ("Air detected", [71])},
    "AirFluidLevel": {"112104": ("Air-fluid level", [6102, 6103])},
    "AirKermaAtOutputMeasurementPoint": {
        "130515": ("Air Kerma at Output Measurement Point", [])
    },
    "AirSpace": {"112147": ("Air space", [6123])},
    "Airflow": {"130421": ("Airflow", [3005])},
    "AirflowCPAP": {"130420": ("Airflow CPAP", [3005])},
    "AirflowNasalProng": {"130418": ("Airflow Nasal Prong", [3005])},
    "AirflowPVDF": {"130419": ("Airflow PVDF", [3005])},
    "AirflowThermistor": {"130416": ("Airflow Thermistor", [3005])},
    "AirflowThermocouple": {"130417": ("Airflow Thermocouple", [3005])},
    "AirwayManagementMethod": {"127312": ("Airway Management Method", [])},
    "AirwayManagementSet": {"127310": ("Airway Management Set", [])},
    "AirwayPartiallyObstructed": {"122282": ("airway partially obstructed", [3448])},
    "AirwaySeverelyObstructed": {"122283": ("airway severely obstructed", [3448])},
    "AirwaySubManagementMethod": {"127313": ("Airway Sub-Management Method", [])},
    "AirwayUnobstructed": {"122281": ("airway unobstructed", [3448])},
    "AlgorithmFamily": {"111000": ("Algorithm Family", [])},
    "AlgorithmManufacturer": {"122405": ("Algorithm Manufacturer", [])},
    "AlgorithmName": {"111001": ("Algorithm Name", [])},
    "AlgorithmParameters": {"111002": ("Algorithm Parameters", [])},
    "AlgorithmVersion": {"111003": ("Algorithm Version", [])},
    "AlgorithmicAttenuationCorrection": {
        "122726": ("Algorithmic Attenuation Correction", [3112])
    },
    "AllAlgorithmsSucceededWithFindings": {
        "111242": ("All algorithms succeeded; with findings", [6047])
    },
    "AllAlgorithmsSucceededWithoutFindings": {
        "111241": ("All algorithms succeeded; without findings", [6047])
    },
    "AllPlanes": {"113890": ("All Planes", [10003])},
    "AlphaGeudValue": {"130095": ("alpha gEUD Value", [])},
    "AlternatePhotometricInterpretationImage": {
        "121328": ("Alternate Photometric Interpretation image", [7205])
    },
    "AlternateSOPClassInstance": {"121326": ("Alternate SOP Class instance", [7205])},
    "AlternativeDoseAlertBehaviorActive": {
        "113915": ("Alternative dose alert behavior active", [])
    },
    "AmbientRoomEnvironment": {"109112": ("Ambient room environment", [3000])},
    "AmbiguousSex": {"121104": ("Ambiguous sex", [7455])},
    "AmericaKennelClub": {"109200": ("America Kennel Club", [7481])},
    "AmericaPetRegistryInc": {"109201": ("America's Pet Registry Inc.", [7481])},
    "AmericanCanineAssociation": {"109202": ("American Canine Association", [7481])},
    "AmericanPurebredRegistry": {"109203": ("American Purebred Registry", [7481])},
    "AmericanRareBreedAssociation": {
        "109204": ("American Rare Breed Association", [7481])
    },
    "AnalysisOrMeasurementsForCurrentProcedure": {
        "122073": ("Analysis or measurements for current procedure", [3407])
    },
    "AnalysisPerformed": {"111004": ("Analysis Performed", [])},
    "AnalysisQualityRating": {"111693": ("Analysis Quality Rating", [])},
    "AnalyticalAlgorithm": {"128480": ("Analytical Algorithm", [10068])},
    "AnatomicIdentifier": {"112050": ("Anatomic Identifier", [])},
    "AnatomicImage": {"121338": ("Anatomic image", [7201])},
    "AnatomicNonColon": {"112238": ("Anatomic non-colon", [6201])},
    "AnatomicalAxisOfFemur": {"112302": ("Anatomical axis of femur", [7305, 7306])},
    "AnatomyCoverageIncomplete": {"130576": ("Anatomy coverage incomplete", [6315])},
    "Anechoic": {"111361": ("Anechoic", [6154])},
    "AnesthesiaCategory": {"127302": ("Anesthesia Category", [])},
    "AnesthesiaMethod": {"127301": ("Anesthesia Method", [])},
    "AnesthesiaMethodSet": {"127300": ("Anesthesia Method Set", [])},
    "AnesthesiaSubcategory": {"127303": ("Anesthesia SubCategory", [])},
    "AneurysmOnCitedVessel": {"122101": ("Aneurysm on cited vessel", [3019])},
    "AnginalEquivalent": {"122799": ("Anginal Equivalent", [3202, 3220])},
    "AngioplastyBalloonDeflated": {"122304": ("Angioplasty balloon deflated", [3412])},
    "AngioplastyBalloonInflated": {"122303": ("Angioplasty balloon inflated", [3412])},
    "Angiosarcoma": {"111302": ("Angiosarcoma", [])},
    "Angioscopy": {"AS": ("Angioscopy", [])},
    "Angle": {"110859": ("Angle", [7182])},
    "AngularMargins": {"111343": ("Angular margins", [6006, 6007])},
    "AngularSecondMomentOfGLCM": {"126065": ("Angular Second Moment of GLCM", [])},
    "AngularViewingEvaluation": {"109709": ("Angular viewing evaluation", [8300])},
    "AnimalHandlingDuringSpecifiedPhase": {
        "127005": ("Animal handling during specified phase", [])
    },
    "AnimalHousing": {"127120": ("Animal housing", [])},
    "AnimalHousingRoom": {"127370": ("Animal housing room", [603])},
    "AnimalRegistryUnlimited": {"109205": ("Animal Registry Unlimited", [7481])},
    "AnimalResearchFoundation": {"109206": ("Animal Research Foundation", [7481])},
    "AnimalRoomIdentifier": {"127122": ("Animal room identifier", [])},
    "AnimalRoomType": {"127121": ("Animal room type", [])},
    "Annotation": {"ANN": ("Annotation", [])},
    "AnodeTargetMaterial": {"111632": ("Anode Target Material", [])},
    "AnomalousExponentParameter": {
        "113299": ("Anomalous Exponent Parameter", [218, 7180, 7272, 7469])
    },
    "AnteriorFirst": {"126833": ("anterior first", [21])},
    "AnteriorInferiorAxis": {"110864": ("Anterior-Inferior Axis", [7184])},
    "AnteriorJunctionLine": {
        "112088": ("Anterior junction line", [6102, 6109, 6112, 7151, 7192, 9514])
    },
    "AnteriorPosterior": {"122675": ("Anterior-Posterior", [3838, 12118])},
    "AnteriorPosteriorAxis": {"110862": ("Anterior-Posterior Axis", [7184])},
    "AnteriorSurfaceOfTheRPE": {
        "128297": ("Anterior surface of the RPE", [4273, 7151, 7192, 9514])
    },
    "AnteriorToInferior": {"110874": ("Anterior To Inferior", [7185])},
    "AnteriorToPosterior": {"110870": ("Anterior To Posterior", [7185])},
    "AnthropomorphicModel": {"128404": ("Anthropomorphic Model", [10064])},
    "AntiB22089Zr": {"126754": ("Anti-B220 ^89^Zr", [4021])},
    "AnyDecisionToBiopsyShouldBeBasedOnClinicalAssessment": {
        "111141": (
            "Any decision to biopsy should be based on clinical assessment",
            [6028, 6029],
        )
    },
    "AorticIntervention": {"122054": ("Aortic Intervention", [3405, 3406])},
    "AorticKnob": {
        "112102": ("Aortic knob", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "AorticValveClosure": {"130701": ("Aortic Valve Closure", [12307])},
    "AorticValveOpening": {"130702": ("Aortic Valve Opening", [12307])},
    "ApertureBlock": {"130123": ("Aperture Block", [9517, 9520, 9541, 9545])},
    "ApexBaseAxis": {"110863": ("Apex-Base Axis", [7184])},
    "ApexToBase": {"110872": ("Apex To Base", [7185])},
    "ApicalFourChamberRVFocused": {
        "130681": ("Apical four chamber RV Focused", [501, 12226])
    },
    "ApicalFourChamberRVModified": {
        "130682": ("Apical four chamber RV Modified", [501, 12226])
    },
    "ApparentDiffusionCoefficient": {
        "113041": (
            "Apparent Diffusion Coefficient",
            [218, 6311, 7180, 7263, 7272, 7469],
        )
    },
    "ApparentKurtosisCoefficient": {
        "113206": ("Apparent Kurtosis Coefficient", [7263])
    },
    "Application": {"110150": ("Application", [402])},
    "ApplicationActivity": {"110100": ("Application Activity", [400])},
    "ApplicationLauncher": {"110151": ("Application Launcher", [402])},
    "ApplicationStart": {"110120": ("Application Start", [401, 403])},
    "ApplicationStop": {"110121": ("Application Stop", [401, 403])},
    "ApplicatorPresent": {"130459": ("Applicator Present", [9564])},
    "AppropriateForTheDevice": {"128606": ("Appropriate for the device", [800])},
    "AppropriateForTheIndications": {
        "128601": ("Appropriate for the indications", [800])
    },
    "ApprovedForExperimentalUse": {"128611": ("Approved for experimental use", [800])},
    "ApprovedForUseAtTheInstitution": {
        "128603": ("Approved for use at the institution", [800])
    },
    "ApprovedForUseInTheClinicalTrial": {
        "128604": ("Approved for use in the clinical trial", [800])
    },
    "ApprovedForUseOnPregnantPatients": {
        "128605": ("Approved for use on pregnant patients", [800])
    },
    "ApproximateSpatialLocation": {
        "121226": ("Approximate spatial location", [7551, 7553])
    },
    "ArcBeam": {"130103": ("Arc Beam", [9511, 9524, 9583])},
    "ArcOfCalcium": {"122355": ("Arc of Calcium", [])},
    "ArchOfTheAzygosVein": {
        "112103": (
            "Arch of the Azygos vein",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "ArchitecturalDistortion": {"112079": ("Architectural distortion", [6108])},
    "Archive": {"ARCHIVE": ("Archive", [30])},
    "AreaBasedMethod": {"122650": ("Area Based Method", [3804, 3807])},
    "AreaByPressureHalfTime": {
        "125210": ("Area by Pressure Half-Time", [12227, 12229])
    },
    "AreaEqualsFlow44Point5SqrtGradientMmhg": {
        "122262": ("Area = Flow / 44.5 * sqrt(Gradient[mmHg])", [])
    },
    "AreaLengthBiplane": {"125204": ("Area-Length Biplane", [12227, 12228])},
    "AreaLengthDodge": {"122559": ("Area Length Dodge", [3453])},
    "AreaLengthKennedy": {"122558": ("Area Length Kennedy", [3453])},
    "AreaLengthMethod": {"122428": ("Area Length Method", [])},
    "AreaLengthSinglePlane": {"125205": ("Area-Length Single Plane", [12227, 12228])},
    "AreaLengthWynne": {"122560": ("Area Length Wynne", [3453])},
    "AreaOfAClosedNURBS": {"122502": ("Area of a closed NURBS", [7473])},
    "AreaOfClosedIrregularPolygon": {
        "122501": ("Area of closed irregular polygon", [7473])
    },
    "AreaOfDefinedRegion": {"121202": ("Area of Defined Region", [])},
    "AreaOfVascularCalcification": {
        "112196": ("Area of Vascular Calcification", [6142])
    },
    "AreaOutline": {"121056": ("Area outline", [])},
    "AreaOutline_Retired": {"121201": ("Area Outline", [])},
    "AreaSymmetry": {"122545": ("Area Symmetry", [])},
    "ArithmeticAverageOfDwZ": {"113990": ("Arithmetic Average of Dw(z)", [])},
    "ArithmeticAverageOfSSDEZ": {"113989": ("Arithmetic Average of SSDE(z)", [10023])},
    "ArmOfAngle": {"121223": ("Arm of angle", [7551, 7553])},
    "ArterialConduitAngiography": {
        "122058": ("Arterial conduit angiography", [3405, 3428])
    },
    "ArterialPulseWaveform": {"109116": ("Arterial Pulse Waveform", [3004])},
    "ArteriovenousDifference": {"122229": ("Arteriovenous difference", [])},
    "ArtifactsOtherThanGridOrDetectorArtifact": {
        "111215": (
            "Artifact(s) other than grid or detector artifact",
            [6041, 6135, 7011],
        )
    },
    "ArtificialIntelligence": {"123110": ("Artificial Intelligence", [7162])},
    "AspenChipBedding": {"127230": ("Aspen chip bedding", [605])},
    "AspenShavingBedding": {"127231": ("Aspen shaving bedding", [605])},
    "Assembly": {"112355": ("Assembly", [])},
    "AssessChange": {"130590": ("Assess change", [6327])},
    "AssessmentByComparison": {"121375": ("Assessment By Comparison", [703])},
    "AssessmentByRules": {"121376": ("Assessment By Rules", [703])},
    "AssessmentCategory": {"111005": ("Assessment Category", [])},
    "Assisting": {"121099": ("Assisting", [7453])},
    "AssociatedChestComponent": {"112003": ("Associated Chest Component", [])},
    "AsymmetricImplants": {"111504": ("Asymmetric implants", [6072])},
    "AsynchronousInvolution": {"111254": ("Asynchronous involution", [])},
    "AtLastAppointment": {"109125": ("At last appointment", [31])},
    "AtSurfaceOfPatient": {"113964": ("At Surface of Patient", [10025])},
    "AtomicNumber": {"130093": ("Atomic Number", [])},
    "AtrialDiastolicFillingDWave": {
        "125322": ("Atrial Diastolic Filling (D-wave)", [12307])
    },
    "AtrialPressureMeasurements": {"122121": ("Atrial pressure measurements", [])},
    "Attach": {"110124": ("Attach", [401, 403])},
    "AttendingConsultant": {"121089": ("Attending [Consultant]", [])},
    "AttenuationCoefficient": {
        "112031": ("Attenuation Coefficient", [218, 6141, 7180, 7469, 10069])
    },
    "AttenuationCorrectionMethod": {"122713": ("Attenuation correction method", [])},
    "AttenuatorCategory": {"128458": ("Attenuator Category", [])},
    "AttenuatorCharacteristics": {"130531": ("Attenuator Characteristics", [])},
    "AttenuatorDescription": {"128468": ("Attenuator Description", [])},
    "AttenuatorPosition": {"130528": ("Attenuator Position", [])},
    "Attenuators": {"130513": ("Attenuators", [])},
    "Audio": {"AU": ("Audio", [])},
    "AuditLogUsed": {"110101": ("Audit Log Used", [400])},
    "AuditRecordingStarted": {"110134": ("Audit Recording Started", [401, 403])},
    "AuditRecordingStopped": {"110133": ("Audit Recording Stopped", [401, 403])},
    "AuthenticationDecision": {"110143": ("Authentication Decision", [401, 403])},
    "AuthorizationDecision": {"110144": ("Authorization Decision", [401, 403])},
    "AutoKeratometry": {"111754": ("Auto Keratometry", [4235, 4242])},
    "AutocorrelationOfGLCM": {"128794": ("Autocorrelation of GLCM", [])},
    "AutomatedAdministration": {"130173": ("Automated Administration", [63])},
    "AutomatedDataCollection": {
        "113856": ("Automated Data Collection", [10020, 10021])
    },
    "AutomatedManualInjectPhase": {"130171": ("Automated Manual Inject Phase", [62])},
    "AutomatedOptical": {"111843": ("Automated Optical", [4253])},
    "AutomatedSlideStainer": {"STAIN": ("Automated Slide Stainer", [30, 32, 33])},
    "AutomaticProgrammedAdministrationPhase": {
        "130168": ("Automatic Programmed Administration Phase", [62])
    },
    "AutomaticProgrammedDelayPhase": {
        "130169": ("Automatic Programmed Delay Phase", [62])
    },
    "AutomaticProgrammedWaitPhase": {
        "130263": ("Automatic Programmed Wait Phase", [62])
    },
    "AutomaticWithManualHoldPhase": {
        "130170": ("Automatic with Manual Hold Phase", [])
    },
    "AutomaticallyDetectedAIFROI": {
        "126363": ("Automatically Detected AIF ROI", [4103, 4106])
    },
    "Autorefraction": {"AR": ("Autorefraction", [29, 30, 33])},
    "AutorefractionMeasurementsSOPInstance": {
        "111784": ("Autorefraction Measurements SOP Instance", [4240])
    },
    "AutorefractionVisualAcuity": {"111685": ("Autorefraction Visual Acuity", [4216])},
    "AverageBreastFeedingPeriod": {"111544": ("Average breast feeding period", [])},
    "AverageCalcificationDistanceInACalcificationCluster": {
        "112200": ("Average calcification distance in a calcification cluster", [6142])
    },
    "AverageGlandularDose": {"111631": ("Average Glandular Dose", [])},
    "AverageXRayTubeCurrent": {"113767": ("Average X-Ray Tube Current", [])},
    "AvoidanceVolume": {"130058": ("Avoidance Volume", [9535])},
    "AxialDiffusivity": {"113204": ("Axial Diffusivity", [7263])},
    "AxialKurtosis": {"113208": ("Axial Kurtosis", [7263])},
    "AxialMeasurementsSOPInstance": {
        "111782": ("Axial Measurements SOP Instance", [4240])
    },
    "AxillaryNodalMetastases": {"111301": ("Axillary nodal metastases", [6030, 6033])},
    "AxillaryNodeHyperplasia": {"111253": ("Axillary node hyperplasia", [6030, 6031])},
    "AxillaryNodeWithCalcifications": {
        "111252": ("Axillary node with calcifications", [6030, 6031])
    },
    "AxillaryNodeWithLymphoma": {
        "111300": ("Axillary node with lymphoma", [6030, 6033])
    },
    "AzygoesophagealRecessInterface": {
        "112090": (
            "Azygoesophageal recess interface",
            [6102, 6109, 6113, 6148, 7151, 7192, 9514],
        )
    },
    "BIC": {"126224": ("BIC", [218, 7180, 7469])},
    "BMIEqualsWtHt2": {"122265": ("BMI = Wt / Ht ^ 2", [])},
    "BN01Pattern": {"109904": ("BN01 Pattern", [8301])},
    "BN02Pattern": {"109905": ("BN02 Pattern", [8301])},
    "BN03Pattern": {"109906": ("BN03 Pattern", [8301])},
    "BN04Pattern": {"109907": ("BN04 Pattern", [8301])},
    "BN05Pattern": {"109908": ("BN05 Pattern", [8301])},
    "BN06Pattern": {"109909": ("BN06 Pattern", [8301])},
    "BN07Pattern": {"109910": ("BN07 Pattern", [8301])},
    "BN08Pattern": {"109911": ("BN08 Pattern", [8301])},
    "BN09Pattern": {"109912": ("BN09 Pattern", [8301])},
    "BN10Pattern": {"109913": ("BN10 Pattern", [8301])},
    "BN11Pattern": {"109914": ("BN11 Pattern", [8301])},
    "BN12Pattern": {"109915": ("BN12 Pattern", [8301])},
    "BN13Pattern": {"109916": ("BN13 Pattern", [8301])},
    "BN14Pattern": {"109917": ("BN14 Pattern", [8301])},
    "BN15Pattern": {"109918": ("BN15 Pattern", [8301])},
    "BN16Pattern": {"109919": ("BN16 Pattern", [8301])},
    "BN17Pattern": {"109920": ("BN17 Pattern", [8301])},
    "BN18Pattern": {"109921": ("BN18 Pattern", [8301])},
    "BRCA1BreastCancerGene": {"111556": ("BRCA1 breast cancer gene", [6081, 6087])},
    "BRCA2BreastCancerGene": {"111557": ("BRCA2 breast cancer gene", [6081, 6087])},
    "BRCA3BreastCancerGene": {"111558": ("BRCA3 breast cancer gene", [6081, 6087])},
    "BREPRadiationTransportModel": {
        "128406": ("BREP Radiation Transport Model", [10065])
    },
    "BSA1Point219": {"122572": ("BSA^1.219", [3455])},
    "BSAEquals0Point0004688WT0Point8168To0Point0154LogWT": {
        "122246": ("BSA = 0.0004688*WT^(0.8168-0.0154*log(WT))", [3663])
    },
    "BSAEquals0Point003207WT0Point7285To0Point0188LogWTHT0Point3": {
        "122240": ("BSA = 0.003207*WT^(0.7285-0.0188*log(WT))*HT^0.3", [3663])
    },
    "BSAEquals0Point007184WT0Point425HT0Point725": {
        "122241": ("BSA = 0.007184*WT^0.425*HT^0.725", [3663])
    },
    "BSAEquals0Point007358WT0Point425HT0Point725": {
        "122266": ("BSA = 0.007358*WT^0.425*HT^0.725", [3663])
    },
    "BSAEquals0Point008883WT0Point444HT0Point663": {
        "122268": ("BSA = 0.008883*WT^0.444*HT^0.663", [3663])
    },
    "BSAEquals0Point009568WT0Point473HT0Point655": {
        "122270": ("BSA = 0.009568*WT^0.473*HT^0.655", [3663])
    },
    "BSAEquals0Point010265WT0Point423HT0Point651": {
        "122267": ("BSA = 0.010265*WT^0.423*HT^0.651", [3663])
    },
    "BSAEquals0Point0235WT0Point51456HT0Point42246": {
        "122242": ("BSA = 0.0235*WT^0.51456\u200b*HT^0.42246", [])
    },
    "BSAEquals0Point0235WT0Point51456Htcm0Point42246": {
        "122242": ("BSA = 0.0235*WT^0.51456*HTcm^0.42246", [3663])
    },
    "BSAEquals0Point024265WT0Point5378HT0Point3964": {
        "122243": ("BSA = 0.024265*WT^0.5378\u200b*HT^0.3964", [])
    },
    "BSAEquals0Point024265WT0Point5378Htcm0Point3964": {
        "122243": ("BSA = 0.024265*WT^0.5378*HTcm^0.3964", [3663])
    },
    "BSAEquals0Point038189WT0Point423HT0Point362": {
        "122269": ("BSA = 0.038189*WT^0.423*HT^0.362", [3663])
    },
    "BSAEquals13210Point3433WT": {"122245": ("BSA = 1321+0.3433*WT", [3663])},
    "BSAEqualsHTWT360Point5": {"122244": ("BSA = (HT*WT/36)^0.5", [3663])},
    "BackOfCorneaToFrontOfLens": {
        "111777": ("Back Of Cornea To Front Of Lens", [4239])
    },
    "Background": {
        "125040": ("Background", [7151, 7159, 7165, 7168, 7169, 7194, 7196, 12322])
    },
    "BackgroundForLesionToBackgroundSUVRatio": {
        "126040": ("Background for Lesion to Background SUV Ratio", [7466])
    },
    "Backscatter": {"128411": ("Backscatter", [10069])},
    "BarcodeValue": {"130231": ("Barcode Value", [])},
    "BarrettDesignFactor": {"111867": ("Barrett Design Factor", [4237])},
    "BarrettLensFactor": {"111866": ("Barrett Lens Factor", [4237])},
    "BarrettToric": {"111862": ("Barrett Toric", [4236])},
    "BarrettTrueK": {"111863": ("Barrett True-K", [4236])},
    "BarrettTrueKToric": {"111864": ("Barrett True-K Toric", [4236])},
    "BarrettUniversalII": {"111865": ("Barrett Universal II", [4236])},
    "BasalCellCarcinomaOfNipple": {
        "111307": ("Basal cell carcinoma of nipple", [6030, 6033])
    },
    "BaseToApex": {"110873": ("Base To Apex", [7185])},
    "Baseline": {"121079": ("Baseline", [280, 7003])},
    "BaselineCategory": {"112016": ("Baseline Category", [])},
    "BaselineResultCorrection": {"122628": ("Baseline Result Correction", [])},
    "BaselineScreeningMammogram": {"111403": ("Baseline screening mammogram", [])},
    "BaselineStenosisMeasurement": {"122107": ("Baseline Stenosis Measurement", [])},
    "BaselineTIMIFlow": {"122109": ("Baseline TIMI Flow", [])},
    "BasicApplicationConfidentialityProfile": {
        "113100": ("Basic Application Confidentiality Profile", [7050])
    },
    "BasicVoiceAudio": {"AU": ("Basic Voice Audio", [32, 33])},
    "BatWingDistribution": {"112154": ("Bat's wing distribution", [6128])},
    "BayesianProbability": {"113270": ("Bayesian-Probability", [7274])},
    "BazettQTCorrectionAlgorithm": {
        "122730": ("Bazett QT Correction Algorithm", [3678])
    },
    "BazettQtcAlgorithm": {"122730": ("Bazett QTc Algorithm", [])},
    "BeadedSeptumSign": {"112066": ("Beaded septum sign", [6102, 6104])},
    "BeamBlock": {"128431": ("Beam Block", [10066])},
    "BeamEnergyInterlock": {"130831": ("Beam Energy Interlock", [9561, 9568])},
    "BeamPosition": {"130524": ("Beam Position", [])},
    "BeamShapingNote": {"130023": ("Beam Shaping Note", [])},
    "BeamTargetingToleranceViolation": {
        "130468": ("Beam Targeting Tolerance Violation", [9566])
    },
    "BeatNumber": {"122149": ("Beat Number", [])},
    "BeatsDetectedAccepted": {"109018": ("Beat detected (accepted)", [3337])},
    "BeatsDetectedRejected": {"109019": ("Beat detected (rejected)", [3337])},
    "BeddingDepth": {"127185": ("Bedding depth", [])},
    "BeddingManufacturer": {"127180": ("Bedding manufacturer", [])},
    "BeddingMass": {"127184": ("Bedding mass", [])},
    "BeddingProductCode": {"127182": ("Bedding product code", [])},
    "BeddingProductName": {"127181": ("Bedding product name", [])},
    "BeddingVolume": {"127183": ("Bedding volume", [])},
    "BeginCirculatorySupport": {"121157": ("Begin Circulatory Support", [3550])},
    "BeginOxygenAdministration": {"121161": ("Begin oxygen administration", [3530])},
    "BeginPacing": {"121166": ("Begin Pacing", [3552])},
    "BeginTransferringDICOMInstances": {
        "110102": ("Begin Transferring DICOM Instances", [400])
    },
    "BeginVentilation": {"121168": ("Begin Ventilation", [3551])},
    "BeginningOfTimePeriod": {"130533": ("Beginning of Time Period", [10073])},
    "BehaviorOfTheFindingSite": {"125312": ("Behavior of the Finding Site", [12302])},
    "BenignCalcifications": {"111256": ("Benign Calcifications", [6030, 6031])},
    "BenignCystWithBlood": {"111255": ("Benign cyst with blood", [6030, 6031])},
    "Benralizumab89Zr": {"126722": ("Benralizumab ^89^Zr", [4021])},
    "BesselFilter": {"130763": ("Bessel filter", [3042])},
    "BestIllustrationOfFinding": {"121080": ("Best illustration of finding", [7003])},
    "BestInSet": {"113013": ("Best In Set", [7010])},
    "BestOverallResponse": {"112049": ("Best Overall Response", [])},
    "Bevacizumab89Zr": {"126516": ("Bevacizumab ^89^Zr", [4021])},
    "BiExponentialIVIMDiffusionModel": {
        "113251": ("Bi-exponential (IVIM) diffusion model", [7273])
    },
    "BiPlaneAngiography": {"122060": ("Bi-plane Angiography", [3748])},
    "BillingCode": {"121147": ("Billing Code", [])},
    "BinarySegmentation": {"110853": ("Binary Segmentation", [218, 7180, 7469])},
    "BiomagneticImaging": {"BI": ("Biomagnetic Imaging", [29, 30, 33])},
    "BiomedicalEngineering": {"128171": ("Biomedical Engineering", [7030])},
    "BiometryGroup": {"125005": ("Biometry Group", [])},
    "BiophysicalProfile": {"125006": ("Biophysical Profile", [])},
    "BiopsyLocalizer": {"121312": ("Biopsy localizer", [7201])},
    "BiopsyShouldBeConsidered": {
        "111143": ("Biopsy should be considered", [6028, 6029])
    },
    "BiopsyShouldBeStronglyConsidered": {
        "111148": ("Biopsy should be strongly considered", [6028, 6029])
    },
    "Bioptome": {"122052": ("Bioptome", [3429])},
    "BiosafetyConditions": {"127010": ("Biosafety conditions", [])},
    "BiplaneEllipse": {"125211": ("Biplane Ellipse", [12227, 12228])},
    "BiquadFilter": {"130773": ("Biquad filter", [3043])},
    "Blinatumomab89Zr": {"126727": ("Blinatumomab ^89^Zr", [4021])},
    "BlindEstimationOfAIF": {"126364": ("Blind Estimation of AIF", [4103, 4106])},
    "BlindSpotMonitoring": {"111844": ("Blind Spot Monitoring", [4253])},
    "BlockPresent": {"130458": ("Block Present", [9564])},
    "BloodCarbonDioxideSaturation": {
        "122187": ("Blood Carbon dioxide saturation", [3526])
    },
    "BloodDeoxygenationLevel": {"130823": ("Blood Deoxygenation Level", [11006])},
    "BloodLabMeasurements": {"122125": ("Blood lab measurements", [])},
    "BloodOxygenContent": {"122185": ("Blood Oxygen content", [3525])},
    "BloodOxygenationLevel": {
        "110819": ("Blood Oxygenation Level", [218, 7180, 7469, 11006])
    },
    "BloodTemperature": {"122183": ("Blood temperature", [])},
    "BloodTingedColor": {"111454": ("Blood tinged color", [6067])},
    "BloodVelocityMean": {
        "122205": (
            "Blood velocity, mean",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "BloodVelocityMeasurements": {"122124": ("Blood velocity measurements", [])},
    "BloodVelocityMinimum": {
        "122206": (
            "Blood velocity, minimum",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "BloodVelocityPeak": {
        "122207": (
            "Blood velocity, peak",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "BloodVelocityVsTimeOfCardiacCycle": {
        "122667": ("Blood velocity vs. time of cardiac cycle", [])
    },
    "BloodVesselVascularInvasion": {
        "111303": ("Blood vessel (vascular) invasion", [6030, 6033])
    },
    "BloomRichardsonGrade": {"111502": ("Bloom-Richardson Grade", [6071])},
    "BlueFilter": {"111603": ("Blue filter", [])},
    "BlueGreenFilter": {"111605": ("Blue-green filter", [])},
    "Boak": {"122563": ("Boak", [3453])},
    "BodeFilter": {"130768": ("Bode filter", [3042])},
    "BodyFrame": {"130117": ("Body Frame", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "BodyHabitusAttenuation": {"122743": ("Body habitus attenuation", [3115])},
    "BodyNodeSet": {"130363": ("Body Node Set", [9556])},
    "Bolting": {"112320": ("Bolting", [7310])},
    "BolusArrivalTime": {"126371": ("Bolus Arrival Time", [218, 4109, 7180, 7469])},
    "BolusPresent": {"130456": ("Bolus Present", [9564])},
    "BolusShapingCurve": {"130210": ("Bolus Shaping Curve", [])},
    "BoneAndSoftTissue": {"130407": ("Bone and soft tissue", [645, 8134])},
    "BoneMineralDensitometry": {"BMD": ("Bone Mineral Densitometry", [29, 30, 33])},
    "BoothbyTable": {"122255": ("Boothby table", [3664])},
    "Bootstrap": {"113217": ("Bootstrap", [7262])},
    "BootstrapTrackingAlgorithm": {"113217": ("Bootstrap Tracking Algorithm", [])},
    "BorderDefinition": {"112007": ("Border definition", [])},
    "BorderShape": {"112015": ("Border shape", [])},
    "Borderline": {"111848": ("Borderline", [4254])},
    "BorderlineAndGeneralReductionInSensitivity": {
        "111851": ("Borderline and general reduction in sensitivity", [4254])
    },
    "BorgCR10Scale": {"122735": ("Borg CR10 Scale", [3239])},
    "BorgRPEScale": {"122734": ("Borg RPE Scale", [3239])},
    "BottomZLocationOfReconstructableVolume": {
        "113896": ("Bottom Z Location of Reconstructable Volume", [])
    },
    "BottomZLocationOfScanningLength": {
        "113898": ("Bottom Z Location of Scanning Length", [])
    },
    "BoundaryPoint": {"128141": ("Boundary point", [219])},
    "BowtieFilter": {"130354": ("Bowtie Filter", [9548])},
    "BrachytherapyChannel": {
        "130080": ("Brachytherapy channel", [7151, 7157, 7193, 9506, 9520])
    },
    "BrachytherapyChannelShield": {
        "130079": ("Brachytherapy channel shield", [7151, 7157, 7193, 9506, 9520])
    },
    "BrachytherapyDevice": {"130045": ("Brachytherapy Device", [9502, 9580])},
    "BrachytherapyIsotope": {"130038": ("Brachytherapy Isotope", [])},
    "BrachytherapySourceApplicator": {
        "130078": ("Brachytherapy source applicator", [7151, 7157, 7193, 9506, 9520])
    },
    "BrandName": {"111529": ("Brand Name", [])},
    "BreakoutBoxConnected": {"130891": ("Breakout box connected", [3039])},
    "BreakoutBoxDisconnected": {"130892": ("Breakout box disconnected", [3039])},
    "BreastAttenuation": {"122744": ("Breast attenuation", [3115])},
    "BreastBackgroundEchotexture": {"111350": ("Breast background echotexture", [])},
    "BreastBoard": {
        "130116": ("Breast Board", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "BreastBridge": {
        "130653": ("Breast Bridge", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "BreastComposition": {"111006": ("Breast composition", [])},
    "BreastFeedingHistory": {"111543": ("Breast feeding history", [])},
    "BreastGeometry": {"111100": ("Breast geometry", [6014, 6016, 6054])},
    "BreastImagingReport": {"111400": ("Breast Imaging Report", [7000])},
    "BreastImagingSubjectMatter": {"128738": ("Breast imaging subject matter", [7017])},
    "BreastOutlineIncludingPectoralMuscleTissue": {
        "111007": ("Breast Outline including Pectoral Muscle Tissue", [])
    },
    "BreastThickness": {"128405": ("Breast Thickness", [10069])},
    "BreastTissueDensity": {"112191": ("Breast tissue density", [6142])},
    "Brentuximab89Zr": {"126735": ("Brentuximab ^89^Zr", [4021])},
    "BrightRedColor": {"111453": ("Bright red color", [6067])},
    "BrightfieldIllumination": {"111744": ("Brightfield illumination", [8123])},
    "Bronchovascular": {"112052": ("Bronchovascular", [6100])},
    "BulletMethod": {"125228": ("Bullet Method", [12227, 12228])},
    "ButterflyDistribution": {"112155": ("Butterfly distribution", [6128])},
    "ButterflyFilter": {"113652": ("Butterfly filter", [10007])},
    "ButterworthFilter": {"130760": ("Butterworth filter", [3042])},
    "ByVentilator": {"121163": ("By ventilator", [])},
    "C0InadequateStudyAwaitingPriorComparisons": {
        "112240": ("C0 - Inadequate Study/Awaiting Prior Comparisons", [6200])
    },
    "C1NormalColonOrBenignLesion": {
        "112241": ("C1 - Normal Colon or Benign Lesion", [6200])
    },
    "C2IntermediatePolypOrIndeterminateFinding": {
        "112242": ("C2 - Intermediate Polyp or Indeterminate Finding", [6200])
    },
    "C3PolypPossiblyAdvancedAdenoma": {
        "112243": ("C3 - Polyp, Possibly Advanced Adenoma", [6200])
    },
    "C4ColonicMassLikelyMalignant": {
        "112244": ("C4 - Colonic Mass, Likely Malignant", [6200])
    },
    "CADOperatingPoint": {"111071": ("CAD Operating Point", [])},
    "CADOperatingPointDescription": {"111081": ("CAD Operating Point Description", [])},
    "CADOperatingPointTable": {"111093": ("CAD Operating Point Table", [])},
    "CADProcessingAndFindingsSummary": {
        "111017": ("CAD Processing and Findings Summary", [])
    },
    "CArmMount": {"113955": ("C-Arm Mount", [10031])},
    "CCNippleNotCenteredOnImage": {
        "111203": ("CC Nipple not centered on image", [6041])
    },
    "CCNotAllMedialTissueVisualized": {
        "111202": ("CC Not all medial tissue visualized", [6041])
    },
    "CCPosteriorNippleLineDoesNotMeasureWithin1cmOfMLO": {
        "111204": (
            "CC Posterior nipple line does not measure within 1 cm of MLO",
            [6041],
        )
    },
    "CD": {"110032": ("CD", [405])},
    "CDImported": {"110024": ("CD Imported", [7008])},
    "CFXRegionInLAOProjection": {"122472": ("CFX Region in LAO Projection", [3461])},
    "CHARMED": {"113234": ("CHARMED", [7261])},
    "CICFilter": {"130771": ("CIC filter", [3043])},
    "CKDEPIFormulaEstimationOfGFR": {
        "113571": ("CKD-EPI Formula estimation of GFR", [10047])
    },
    "CKMBBaseline": {"122180": ("CK-MB baseline", [])},
    "CKMBPeak": {"122181": ("CK-MB peak", [])},
    "CLR1404I124": {"126715": ("CLR1404 I^124^", [4021])},
    "CLR1404I131": {"126716": ("CLR1404 I^131^", [4021])},
    "CO2Respiration": {"130443": ("CO2 Respiration", [3005])},
    "CO2Transcutaneous": {"130434": ("CO2 Transcutaneous", [3005])},
    "CO2TrendEndTidalMainStream": {
        "130436": ("CO2 Trend End-tidal Main-stream", [3005])
    },
    "CO2TrendEndTidalSideStream": {
        "130438": ("CO2 Trend End-tidal Side-stream", [3005])
    },
    "CO2TrendMainStream": {"130441": ("CO2 Trend Main-stream", [3005])},
    "CO2TrendSideStream": {"130442": ("CO2 Trend Side-stream", [3005])},
    "CO2WaveformEndTidalMainStream": {
        "130435": ("CO2 Waveform End-tidal Main-stream", [3005])
    },
    "CO2WaveformEndTidalSideStream": {
        "130437": ("CO2 Waveform End-tidal Side-stream", [3005])
    },
    "CO2WaveformMainStream": {"130439": ("CO2 Waveform Main-stream", [3005])},
    "CO2WaveformSideStream": {"130440": ("CO2 Waveform Side-stream", [3005])},
    "CRDRMechanicalConfiguration": {"113956": ("CR/DR Mechanical Configuration", [])},
    "CRTDisplay": {"109991": ("CRT Display", [8303])},
    "CRTFrontProjectionSystem": {"109998": ("CRT Front Projection System", [8303])},
    "CRTRearProjectionSystem": {"109997": ("CRT Rear Projection System", [8303])},
    "CTAccumulatedDoseData": {"113811": ("CT Accumulated Dose Data", [])},
    "CTAcquisition": {"113819": ("CT Acquisition", [])},
    "CTAcquisitionParameters": {"113822": ("CT Acquisition Parameters", [])},
    "CTAcquisitionType": {"113820": ("CT Acquisition Type", [])},
    "CTBasedAttenuationCorrection": {
        "122728": ("CT-based Attenuation Correction", [3112])
    },
    "CTDose": {"113829": ("CT Dose", [])},
    "CTDoseLengthProductSubTotal": {
        "130745": ("CT Dose Length Product Sub-Total", [10050])
    },
    "CTDoseLengthProductTotal": {"113813": ("CT Dose Length Product Total", [10050])},
    "CTEffectiveDoseTotal": {"113814": ("CT Effective Dose Total", [])},
    "CTProtocol": {"CTPROTOCOL": ("CT Protocol", [32, 33])},
    "CTVNodal": {"130049": ("CTV Nodal", [9534])},
    "CTVPrimary": {"130050": ("CTV Primary", [9534])},
    "CTXRaySourceParameters": {"113831": ("CT X-Ray Source Parameters", [])},
    "CWavePressure": {"122196": ("C wave pressure", [3611])},
    "CalcificationCluster": {"111105": ("Calcification Cluster", [])},
    "CalcificationDescriptor": {"112030": ("Calcification Descriptor", [])},
    "CalcificationDistribution": {"111008": ("Calcification Distribution", [])},
    "CalcificationExtentAsPercentOfSurface": {
        "112018": ("Calcification extent as percent of surface", [6142])
    },
    "CalcificationExtentAsPercentOfVolume": {
        "112019": ("Calcification extent as percent of volume", [6142])
    },
    "CalcificationType": {"111009": ("Calcification Type", [])},
    "CalcificationsOutsideOfAMass": {
        "111347": ("Calcifications outside of a mass", [6012, 6013])
    },
    "CalcificationsSeenInTheCore": {
        "111446": ("Calcifications seen in the core", [6066])
    },
    "CalcificationsWithinAMass": {
        "111346": ("Calcifications within a mass", [6012, 6013])
    },
    "CalcifiedImplant": {"111505": ("Calcified implant", [6072])},
    "Calcium": {"112145": ("Calcium", [6122])},
    "CalciumLength": {"122341": ("Calcium Length", [3483])},
    "CalciumMass": {"122661": ("Calcium Mass", [])},
    "CalciumMassThreshold": {"122658": ("Calcium Mass Threshold", [])},
    "CalciumScore": {"112058": ("Calcium score", [])},
    "CalciumScoringAnalysis": {"122603": ("Calcium Scoring Analysis", [])},
    "CalciumScoringCalibration": {"122659": ("Calcium Scoring Calibration", [])},
    "CalciumVolume": {"122660": ("Calcium Volume", [])},
    "Calculated": {"125315": ("Calculated", [12303])},
    "Calculated_Retired": {"121428": ("Calculated", [])},
    "CalculationDescription": {"112034": ("Calculation Description", [])},
    "CalculationFailure": {"114008": ("Calculation failure", [42, 43])},
    "Calibration": {"122505": ("Calibration", [6051, 6058, 6061])},
    "CalibrationDatetime": {"113723": ("Calibration DateTime", [])},
    "CalibrationFactor": {"122322": ("Calibration Factor", [])},
    "CalibrationMethod": {"122422": ("Calibration Method", [])},
    "CalibrationObject": {"122421": ("Calibration Object", [])},
    "CalibrationObjectSize": {"122423": ("Calibration Object Size", [])},
    "CalibrationObjectUsed": {"122488": ("Calibration Object Used", [3452])},
    "CalibrationProtocol": {"113720": ("Calibration Protocol", [])},
    "CalibrationResponsibleParty": {"113724": ("Calibration Responsible Party", [])},
    "CalibrationUncertainty": {"113763": ("Calibration Uncertainty", [])},
    "CanadianBorderCollieAssociation": {
        "109207": ("Canadian Border Collie Association", [7481])
    },
    "CanadianKennelClub": {"109208": ("Canadian Kennel Club", [7481])},
    "CanadianLivestockRecordsAssociation": {
        "109209": ("Canadian Livestock Records Association", [7481])
    },
    "CancerElsewhere": {"111480": ("Cancer elsewhere", [6055])},
    "CanineFederationOfCanada": {"109210": ("Canine Federation of Canada", [7481])},
    "CarcinomaInChildren": {"111304": ("Carcinoma in children", [6030, 6033])},
    "CarcinomaInEctopicBreast": {
        "111305": ("Carcinoma in ectopic breast", [6030, 6033])
    },
    "CarcinomaInPregnancyAndLactation": {
        "111310": ("Carcinoma in pregnancy and lactation", [6030, 6033])
    },
    "CarcinomaWithEndocrineDifferentiation": {
        "111306": ("Carcinoma with endocrine differentiation", [6030, 6033])
    },
    "CarcinomaWithMetaplasia": {"111308": ("Carcinoma with metaplasia", [])},
    "Carcinosarcoma": {"111311": ("Carcinosarcoma", [])},
    "CardiacElectrophysiology": {"EPS": ("Cardiac Electrophysiology", [29, 30, 33])},
    "CardiacImagingSpecialty": {"128007": ("Cardiac Imaging Specialty", [7449])},
    "CardiacStressState": {"109091": ("Cardiac Stress State", [3102, 10045])},
    "Cardiosphere": {"125901": ("CARDIOsphere", [12030])},
    "CardiovascularAnalysisReport": {"122600": ("Cardiovascular Analysis Report", [])},
    "CardiovascularImagingSubjectMatter": {
        "128730": ("Cardiovascular imaging subject matter", [7017])
    },
    "CardiovascularRadiology": {"128172": ("Cardiovascular Radiology", [7030])},
    "CarinaAngle": {"112086": ("Carina angle", [6102, 6109, 6111, 7151, 7192, 9514])},
    "CarrierGas": {"127330": ("Carrier gas", [621])},
    "CarrierTemperatureSensor": {"127240": ("Carrier temperature sensor", [636])},
    "CartilaginousAndOsseousChange": {
        "111309": ("Cartilaginous and osseous change", [6030, 6033])
    },
    "CaseSensitivity": {"111088": ("Case Sensitivity", [6048])},
    "CaseSpecificity": {"111090": ("Case Specificity", [6048])},
    "CassetteBasedProjectionRadiographySystem": {
        "113959": ("Cassette-based Projection Radiography System", [10032])
    },
    "Cast": {"130114": ("Cast", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "CathLabProcedureLog": {"121120": ("Cath Lab Procedure Log", [3400])},
    "CatheterCurve": {"122097": ("Catheter Curve", [3423])},
    "CatheterSize": {"122319": ("Catheter Size", [])},
    "CatheterizationProcedurePhase": {
        "109057": ("Catheterization Procedure Phase", [])
    },
    "Caudal10DegreeDistalCranioproximalOblique": {
        "123019": ("Caudal 10 degree distal-cranioproximal oblique", [7484])
    },
    "CavityExtentAsPercentOfVolume": {
        "112017": ("Cavity extent as percent of volume", [6142])
    },
    "CavityRadiationShield": {"130640": ("Cavity radiation shield", [9572, 9573])},
    "Center": {"111010": ("Center", [219])},
    "CenterOfGravity": {"122475": ("Center of Gravity", [3458])},
    "CenterOfMass": {"128138": ("Center of Mass", [219, 1011])},
    "CenterOfRotation": {"130521": ("Center of Rotation", [])},
    "CenterOfTableHead": {"128751": ("Center of Table Head", [])},
    "Centerline": {"130490": ("Centerline", [219])},
    "CenterlineWallMotionAnalysis": {"122449": ("Centerline Wall Motion Analysis", [])},
    "CentralBreathingPosition": {
        "122612": ("central breathing position", [3823, 9272])
    },
    "CentralLine": {"112174": ("Central line", [6102, 6138, 6404, 7151, 7193])},
    "Centrilobular": {"112156": ("Centrilobular", [6128])},
    "CentrilobularStructures": {
        "112087": ("Centrilobular structures", [6102, 6109, 6111, 7151, 7192, 9514])
    },
    "CertaintyOfFeature": {"111011": ("Certainty of Feature", [])},
    "CertaintyOfFinding": {"111012": ("Certainty of Finding", [6048])},
    "CertaintyOfImpression": {"111013": ("Certainty of Impression", [])},
    "CessationOfExercise": {"125237": ("Cessation of exercise", [12031])},
    "CessationOfStimulation": {"125239": ("Cessation of stimulation", [12031])},
    "Cetuximab89Zr": {"126513": ("Cetuximab ^89^Zr", [4021])},
    "Cg250FAb289Zr": {"126517": ("cG250-F(ab')(2) ^89^Zr", [4021])},
    "ChairOfProtocolCommittee": {
        "128671": ("Chair of Protocol Committee", [7450, 7452])
    },
    "ChangMethod": {"122721": ("Chang method", [3117])},
    "ChangeInPatientAnatomy": {"130464": ("Change in Patient Anatomy", [9565])},
    "ChangeOfProcedureForCorrectCharging": {
        "110509": ("Change of procedure for correct charging", [9300, 9301, 9303])
    },
    "ChannelObstruction": {"110520": ("Channel Obstruction", [9561])},
    "ChebyshevFilter": {"130761": ("Chebyshev filter", [3042])},
    "CheckedInStatus": {"121388": ("Checked-In Status", [])},
    "Checkerboard": {"114216": ("Checkerboard", [8202])},
    "ChestCADReport": {"112000": ("Chest CAD Report", [])},
    "ChestTube": {"112173": ("Chest tube", [6102, 6138, 6202, 6203, 6404, 7151, 7193])},
    "ChiSquare": {"126221": ("Chi-square", [218, 7180, 7469])},
    "CholineC11": {"126703": ("Choline C^11^", [4021])},
    "CholineCreatineRatio": {
        "113081": ("Choline/Creatine Ratio", [218, 4032, 4033, 7180, 7186, 7469])
    },
    "ChoriocapillarisStructuralReflectanceMap": {
        "128274": ("Choriocapillaris structural reflectance map", [4271])
    },
    "ChoriocapillarisVasculatureFlow": {
        "128273": ("Choriocapillaris vasculature flow", [4271])
    },
    "ChoroidStructuralReflectanceMap": {
        "128276": ("Choroid structural reflectance map", [4271])
    },
    "ChoroidVasculatureFlow": {"128275": ("Choroid vasculature flow", [4271])},
    "ChromaticityEvaluation": {"109705": ("Chromaticity evaluation", [8300])},
    "CineFilmDigitized": {"110021": ("Cine Film Digitized", [7008])},
    "Cinefluorography": {"CF": ("Cinefluorography", [])},
    "CircadianEffects": {"127050": ("Circadian effects", [])},
    "CircleBScanPattern": {"128284": ("Circle B-scan pattern", [4272])},
    "CircleRadialBScanPattern": {"128287": ("Circle-radial B-scan pattern", [4272])},
    "CircleRasterBScanPattern": {"128286": ("Circle-raster B-scan pattern", [4272])},
    "CircularMethod": {"122473": ("Circular method", [3470])},
    "Circulating": {"121100": ("Circulating", [])},
    "CirculatorySupport": {"122138": ("Circulatory Support", [])},
    "CircumferentialLagrangianStrain": {
        "130677": ("Circumferential Lagrangian Strain", [12304, 12311])
    },
    "CircumferentialLagrangianStrainRate": {
        "130678": ("Circumferential Lagrangian Strain Rate", [12304, 12311])
    },
    "Circumscribed": {"112142": ("Circumscribed", [])},
    "ClassActivation": {"130402": ("Class activation", [217, 218, 7180, 7469])},
    "CleanDescriptorsOption": {"113105": ("Clean Descriptors Option", [7050])},
    "CleanGraphicsOption": {"113103": ("Clean Graphics Option", [7050])},
    "CleanPixelDataOption": {"113101": ("Clean Pixel Data Option", [7050])},
    "CleanRecognizableVisualFeaturesOption": {
        "113102": ("Clean Recognizable Visual Features Option", [7050])
    },
    "CleanStructuredContentOption": {
        "113104": ("Clean Structured Content Option", [7050])
    },
    "ClearanceInterlock": {"130739": ("Clearance Interlock", [9561, 9568])},
    "ClinicalContext": {"122127": ("Clinical Context", [])},
    "ClinicalEvaluation": {"109710": ("Clinical evaluation", [8300])},
    "ClinicalFinding": {"111402": ("Clinical finding", [6051])},
    "ClinicalInterpretation": {"122147": ("Clinical Interpretation", [])},
    "ClinicalTrialSubmissionInputUsed": {
        "128223": ("Clinical Trial Submission Input Used", [7010])
    },
    "ClockfaceOrRegion": {"111014": ("Clockface or region", [])},
    "ClusterAnalysis": {"123107": ("Cluster Analysis", [7162])},
    "ClusterProminenceOfGLCM": {"128797": ("Cluster Prominence of GLCM", [])},
    "ClusterShadeOfGLCM": {"128796": ("Cluster Shade of GLCM", [])},
    "ClusterTendencyOfGLCM": {"128795": ("Cluster Tendency of GLCM", [])},
    "ClusteredMicrocysts": {"111129": ("Clustered microcysts", [6054, 6064])},
    "CmabU3689Zr": {"126746": ("cMAb U36 ^89^Zr", [4021])},
    "CoachingDevice": {"130650": ("Coaching Device", [9573, 9578])},
    "Coalescent": {"112157": ("Coalescent", [6128])},
    "CockroftGaultFormulaEstimationOfGFR": {
        "113570": ("Cockroft-Gault Formula estimation of GFR", [10047])
    },
    "CoilMarker": {"129301": ("Coil Marker", [7111, 7112])},
    "CoilPlacementConcern": {"130571": ("Coil placement concern", [6314])},
    "CoilSelectionConcern": {"130572": ("Coil selection concern", [6314])},
    "Coin": {"112178": ("Coin", [6102, 6138, 6202, 6203, 6404, 7151, 7193])},
    "CollectionOfPresentationStates": {
        "113022": ("Collection of Presentation States", [7010])
    },
    "CollimatedFieldArea": {"113790": ("Collimated Field Area", [])},
    "CollimatedFieldHeight": {"113788": ("Collimated Field Height", [])},
    "CollimatedFieldWidth": {"113789": ("Collimated Field Width", [])},
    "CollimationTooCloseToBreast": {
        "111195": ("Collimation too close to breast", [6041])
    },
    "Collimator": {"111173": ("Collimator", [])},
    "CollisionChecks": {"130652": ("Collision Checks", [9577])},
    "ColonCADReport": {"112220": ("Colon CAD Report", [])},
    "ColonOverallAssessment": {"112222": ("Colon Overall Assessment", [])},
    "ColorFlowDoppler": {"CD": ("Color flow Doppler", [])},
    "ColumnAngulation": {"113770": ("Column Angulation", [])},
    "CombinationImplant": {"111484": ("Combination implant", [6058, 6059])},
    "CombinedPosteriorEnhancementAndShadowing": {
        "111370": ("Combined posterior enhancement and shadowing", [6155])
    },
    "CombinedStructuredLightThermalImager": {
        "130648": ("Combined Structured Light/Thermal Imager", [9573, 9575])
    },
    "Comment": {"121106": ("Comment", [12101])},
    "CompactFlash": {"110034": ("Compact Flash", [405])},
    "ComparativeAverage10YearCHDRisk": {
        "122231": ("Comparative Average10 Year CHD Risk", [3667])
    },
    "ComparativeLow10YearCHDRisk": {
        "122232": ("Comparative Low10 Year CHD Risk", [3667])
    },
    "ComparisonToPreviousStudies": {
        "111424": ("Comparison to previous studies", [6052, 6053])
    },
    "ComparisonWithPriorStudyDone": {
        "122140": ("Comparison with Prior Study Done", [])
    },
    "CompleteAcquisitionContent": {"113034": ("Complete Acquisition Content", [7010])},
    "CompleteRenderingForPresentation": {
        "121332": ("Complete Rendering for Presentation", [7006])
    },
    "CompleteStudyContent": {"113032": ("Complete Study Content", [7010])},
    "CompletelyEncapsulated": {
        "130594": ("Completely encapsulated", [6335, 6341, 6342])
    },
    "Complex": {"111363": ("Complex", [6154])},
    "ComplexCyst": {"111460": ("Complex cyst", [6054, 6064])},
    "ComplicatedCyst": {"111130": ("Complicated cyst", [6054, 6064])},
    "Complications": {"121113": ("Complications", [])},
    "ComponentConnection": {"112350": ("Component Connection", [])},
    "ComponentID": {"112347": ("Component ID", [])},
    "ComponentModel": {"129011": ("Component Model", [7062])},
    "ComponentType": {"112370": ("Component Type", [])},
    "ComponentVolume": {"130239": ("Component Volume", [])},
    "ComposedFromPriorDoses": {"121370": ("Composed from prior doses", [7220])},
    "ComposedFromPriorDosesAndCurrentPlan": {
        "121371": ("Composed from prior doses and current plan", [7220])
    },
    "ComposedWithRadiobiologicalEffects": {
        "121377": ("Composed with radiobiological effects", [7220])
    },
    "ComposedWithWeightingForFractionsDelivered": {
        "121378": ("Composed with weighting for fractions delivered", [7220])
    },
    "CompositeFeature": {"111015": ("Composite Feature", [])},
    "CompositeFeatureModifier": {"112023": ("Composite Feature Modifier", [])},
    "CompositeType": {"111016": ("Composite type", [])},
    "CompoundStatement": {"122150": ("Compound Statement", [])},
    "CompressionContactArea": {"111649": ("Compression Contact Area", [])},
    "CompressionForce": {"111647": ("Compression Force", [])},
    "CompressionPressure": {"111648": ("Compression Pressure", [])},
    "CompressionThickness": {"111633": ("Compression Thickness", [])},
    "ComputationServer": {"COMP": ("Computation Server", [30])},
    "ComputedFromImageAttributes": {
        "113867": ("Computed From Image Attributes", [10020, 10021])
    },
    "ComputedRadiography": {"CR": ("Computed Radiography", [29, 30, 33])},
    "ComputedTomography": {"CT": ("Computed Tomography", [29, 30, 33])},
    "ComputerAidedDetection": {"110004": ("Computer Aided Detection", [9231])},
    "ComputerAidedDiagnosis": {"110003": ("Computer Aided Diagnosis", [9231])},
    "ComputerAssistedDetectionDiagnosis": {
        "CAD": ("Computer Assisted Detection/Diagnosis", [30])
    },
    "Concentration": {"122093": ("Concentration", [3410])},
    "ConcentricCircleBScanPattern": {
        "128285": ("Concentric circle B-scan pattern", [4272])
    },
    "Concern": {"121430": ("Concern", [])},
    "Conclusion": {"121077": ("Conclusion", [6053, 7002])},
    "Conclusions": {"121076": ("Conclusions", [])},
    "ConditionEffectiveDoseMeasured": {
        "113816": ("Condition Effective Dose measured", [])
    },
    "ConePresent": {"130457": ("Cone Present", [])},
    "Conepresent": {"130457": ("ConePresent", [9564])},
    "ConfiguredDefaultImagingEnergy": {
        "130807": ("Configured Default Imaging Energy", [9262])
    },
    "ConfiguredLowestImagingEnergy": {
        "130806": ("Configured Lowest Imaging Energy", [9262])
    },
    "ConfirmationOfTarget": {"111442": ("Confirmation of target", [])},
    "ConfocalImaging": {"114207": ("Confocal imaging", [8201])},
    "ConfocalMicroscopy": {"CFM": ("Confocal Microscopy", [29, 30, 33])},
    "ConformalArcBeam": {"130104": ("Conformal Arc Beam", [9511, 9524, 9583])},
    "ConformalityShell": {"130062": ("Conformality Shell", [9535])},
    "ConjunctiveTerm": {"122154": ("Conjunctive Term", [])},
    "ConnectedImplantationPlanComponent": {
        "112374": ("Connected Implantation Plan Component", [])
    },
    "ConsistentWithLabelingOfTheDevice": {
        "128602": ("Consistent with labeling of the device", [800])
    },
    "ConstantAngleAcquisition": {"113805": ("Constant Angle Acquisition", [10013])},
    "ConsultationWith": {"122044": ("Consultation With", [3404])},
    "ConsumableCatheterType": {"130257": ("Consumable Catheter Type", [])},
    "ConsumableIsNew": {"130224": ("Consumable is New", [])},
    "ConsumableReturnedToInventory": {
        "122077": ("Consumable returned to inventory", [3408])
    },
    "ConsumableTakenFromInventory": {
        "122076": ("Consumable taken from inventory", [3408])
    },
    "ConsumableUnusable": {"122079": ("Consumable unusable", [3408])},
    "ContentAssessmentResult": {"ASMT": ("Content Assessment Result", [32, 33])},
    "ContentDate": {"111018": ("Content Date", [])},
    "ContentTime": {"111019": ("Content Time", [])},
    "ContinentalKennelClub": {"109211": ("Continental Kennel Club", [7481])},
    "ContinuityEquation": {"125212": ("Continuity Equation", [12227, 12229])},
    "ContinuityEquationByMeanVelocity": {
        "125213": ("Continuity Equation by Mean Velocity", [12227, 12229])
    },
    "ContinuityEquationByPeakVelocity": {
        "125214": ("Continuity Equation by Peak Velocity", [12227, 12229])
    },
    "ContinuityEquationByVelocityTimeIntegral": {
        "125215": ("Continuity Equation by Velocity Time Integral", [12227, 12229])
    },
    "Continuous": {"113630": ("Continuous", [10004])},
    "ContinuousPermanent": {"130101": ("Continuous Permanent", [9533])},
    "ContinuousTemporary": {"130100": ("Continuous Temporary", [9533])},
    "ContourEnd": {"122482": ("Contour End", [])},
    "ContourRealignment": {"122410": ("Contour Realignment", [])},
    "ContourStart": {"122481": ("Contour Start", [])},
    "ContouringInputUsed": {"128219": ("Contouring Input Used", [7010])},
    "ContrastAdministered": {"122086": ("Contrast administered", [3409])},
    "ContrastAgentAngioMRSignalIntensity": {
        "110801": ("Contrast Agent Angio MR Signal Intensity", [218, 7180, 7469])
    },
    "ContrastBolusAgent": {"123011": ("Contrast/Bolus Agent", [])},
    "ContrastBolusIngredientOpaque": {
        "121381": ("Contrast/Bolus Ingredient Opaque", [4050])
    },
    "ContrastEnd": {"122085": ("Contrast end", [3409])},
    "ContrastLongitudinalRelaxivity": {
        "126380": ("Contrast Longitudinal Relaxivity", [4111])
    },
    "ContrastOfGLCM": {"126063": ("Contrast of GLCM", [])},
    "ContrastPhase": {"109058": ("Contrast Phase", [])},
    "ContrastReactionProphylacticAgent": {
        "130259": ("Contrast Reaction Prophylactic Agent", [76])
    },
    "ContrastStart": {"122084": ("Contrast start", [3409])},
    "ContrastTransverseRelaxivity": {"130188": ("Contrast Transverse Relaxivity", [])},
    "ContrastVolumeLimit": {"130228": ("Contrast Volume Limit", [])},
    "ControlledSSDSetupMethod": {"130631": ("Controlled SSD Setup Method", [9571])},
    "ConversionFactor": {"128528": ("Conversion Factor", [])},
    "CooperLigamentChanges": {
        "111111": ("Cooper's ligament changes", [6014, 6015, 6016, 6054, 6056])
    },
    "CoordinateAlignmentImageSeries": {
        "128187": ("Coordinate Alignment Image Series", [7010, 7023])
    },
    "CopiedFromImageAttributes": {
        "113866": ("Copied From Image Attributes", [10020, 10021])
    },
    "CornCobBedding": {"127232": ("Corn cob bedding", [605])},
    "CornealAxialPowerMap": {"111940": ("Corneal axial power map", [4268])},
    "CornealBirefringenceCompensation": {
        "111923": ("Corneal birefringence compensation", [4261])
    },
    "CornealElevationMap": {"111943": ("Corneal elevation map", [4268])},
    "CornealInstantaneousPowerMap": {
        "111941": ("Corneal instantaneous power map", [4268])
    },
    "CornealRefractivePowerMap": {"111942": ("Corneal refractive power map", [4268])},
    "CornealWavefrontMap": {"111944": ("Corneal wavefront map", [4268])},
    "CoronaRadiata": {"112105": ("Corona radiata", [6102, 6103])},
    "CoronaryArteryBypassDuringThisAdmission": {
        "122164": ("Coronary Artery Bypass During This Admission", [])
    },
    "CoronaryArteryDiseaseAssessment": {
        "130720": ("Coronary Artery Disease Assessment", [])
    },
    "CoronaryArteryDiseaseStenosisAssessmentModifier": {
        "130722": ("Coronary Artery Disease Stenosis Assessment Modifier", [])
    },
    "CoronaryFlowReserve": {"122217": ("Coronary Flow reserve", [3620])},
    "CoronaryLesionGreaterThanEquals50PercentStenosis": {
        "122171": ("Coronary lesion > = 50% stenosis", [3726])
    },
    "CorrectedSinusNodeRecoveryTime": {
        "122237": ("Corrected Sinus Node Recovery Time", [])
    },
    "CorrectionFactor": {"128452": ("Correction Factor", [10069])},
    "CorrectionOfCongenitalCardiovascularDeformity": {
        "121439": ("Correction of congenital cardiovascular deformity", [])
    },
    "CorrelatesToMRIFindings": {"111383": ("Correlates to MRI findings", [6158])},
    "CorrelatesToMammographyFindings": {
        "111382": ("Correlates to mammography findings", [6158])
    },
    "CorrelatesToOtherImagingFindings": {
        "111385": ("Correlates to other imaging findings", [6158])
    },
    "CorrelatesToPhysicalExaminationFindings": {
        "111381": ("Correlates to physical examination findings", [6158])
    },
    "CorrelatesToUltrasoundFindings": {
        "111384": ("Correlates to ultrasound findings", [6158])
    },
    "CorrelationOfGLCM": {"126066": ("Correlation of GLCM", [])},
    "CorrelationToOtherFindings": {"111380": ("Correlation to other Findings", [])},
    "CouchIndexLabel": {"130657": ("Couch Index Label", [])},
    "CountryOfLanguage": {"121046": ("Country of Language", [])},
    "CreatineAndCholine": {
        "113094": ("Creatine and Choline", [218, 4032, 4033, 7180, 7186, 7469])
    },
    "CreatineCholineCitrateRatio": {
        "113096": (
            "Creatine+Choline/Citrate Ratio",
            [218, 4032, 4033, 7180, 7186, 7469],
        )
    },
    "CrossBScanPattern": {"128283": ("Cross B-scan pattern", [4272])},
    "CrossSectionalScanPlaneOrientation": {
        "130546": ("Cross-sectional scan plane orientation", [])
    },
    "CrossedGrid": {"111645": ("Crossed grid", [10017])},
    "Crosstable": {"111069": ("Crosstable", [502, 4011])},
    "CtdifreeairCalculationFactor": {"113836": ("CTDIfreeair Calculation Factor", [])},
    "CtdifreeairToEConversionViaMCComputation": {
        "113801": ("CTDIfreeair to E conversion via MC computation", [10011])
    },
    "CtdifreeairToEConversionViaMeasurement": {
        "113803": ("CTDIfreeair to E conversion via measurement", [10011])
    },
    "CtdivolAlertValue": {"113904": ("CTDIvol Alert Value", [])},
    "CtdivolAlertValueConfigured": {"113902": ("CTDIvol Alert Value Configured", [])},
    "CtdivolForwardEstimate": {"113914": ("CTDIvol Forward Estimate", [])},
    "CtdivolNotificationValue": {"113912": ("CTDIvol Notification Value", [])},
    "CtdivolNotificationValueConfigured": {
        "113910": ("CTDIvol Notification Value Configured", [])
    },
    "CtdiwPhantomType": {"113835": ("CTDIw Phantom Type", [])},
    "Cu3689Zr": {"126515": ("cU36 ^89^Zr", [4021])},
    "Cube": {"125206": ("Cube", [])},
    "CubeBScanPattern": {"128279": ("Cube B-scan pattern", [4272])},
    "CubeMethod": {"125206": ("Cube Method", [12227, 12228])},
    "Culposcopy": {"CP": ("Culposcopy", [])},
    "CurrentFractionNumber": {"121385": ("Current Fraction Number", [])},
    "CurrentPregnancyKnownOrSuspectedMalformationsSyndromes": {
        "111573": (
            "Current pregnancy, known or suspected malformations/syndromes",
            [6087, 6088],
        )
    },
    "CurrentProcedureDescriptions": {"121064": ("Current Procedure Descriptions", [])},
    "CurrentProcedureEvidence": {"122073": ("Current procedure evidence", [7009])},
    "CurrentResponse": {"112048": ("Current Response", [])},
    "CurveFitMethod": {"122627": ("Curve Fit Method", [])},
    "CurveFitParameter": {"128453": ("Curve Fit Parameter", [10069])},
    "CurveFittedReference": {"122489": ("Curve Fitted Reference", [3465])},
    "CurvedLinearUltrasoundTransducerGeometry": {
        "125253": ("Curved linear ultrasound transducer geometry", [12033])
    },
    "CurvedMultiplanarReformatting": {
        "113073": ("Curved multiplanar reformatting", [7203])
    },
    "CylinderMarker": {"129303": ("Cylinder Marker", [7111, 7112])},
    "CystFill": {"111430": ("Cyst fill", [6054, 6057])},
    "Cystoscopy": {"CS": ("Cystoscopy", [])},
    "CytologicAnalysis": {"111147": ("Cytologic analysis", [6028, 6029])},
    "DICOMInstancesAccessed": {"110103": ("DICOM Instances Accessed", [400])},
    "DICOMInstancesTransferred": {"110104": ("DICOM Instances Transferred", [400])},
    "DICOMObjectCatalog": {"121181": ("DICOM Object Catalog", [])},
    "DICOMStudyDeleted": {"110105": ("DICOM Study Deleted", [400])},
    "DINGeometryPatternBild3": {"109932": ("DIN Grayscale Pattern", [8301])},
    "DINGrayscalePatternBild2": {"109931": ("DIN Geometry Pattern", [8301])},
    "DINResolutionPattern": {"109933": ("DIN Resolution Pattern", [8301])},
    "DKI": {"113222": ("DKI", [7260])},
    "DLP": {"113838": ("DLP", [])},
    "DLPAlertValue": {"113903": ("DLP Alert Value", [])},
    "DLPAlertValueConfigured": {"113901": ("DLP Alert Value Configured", [])},
    "DLPForwardEstimate": {"113913": ("DLP Forward Estimate", [])},
    "DLPFrontProjectionSystem": {"109996": ("DLP Front Projection System", [8303])},
    "DLPNotificationValue": {"113911": ("DLP Notification Value", [])},
    "DLPNotificationValueConfigured": {
        "113909": ("DLP Notification Value Configured", [])
    },
    "DLPRearProjectionSystem": {"109995": ("DLP Rear Projection System", [8303])},
    "DLPToEConversionViaMCComputation": {
        "113800": ("DLP to E conversion via MC computation", [10011])
    },
    "DLPToEConversionViaMeasurement": {
        "113802": ("DLP to E conversion via measurement", [10011])
    },
    "DN3089Zr": {"126747": ("DN30 ^89^Zr", [4021])},
    "DOT": {"113236": ("DOT", [7261])},
    "DPA71311C": {"126765": ("DPA-713 ^11^C", [4021])},
    "DPA71418F": {"126766": ("DPA-714 ^18^F", [4021])},
    "DSI": {"113224": ("DSI", [7260, 7261])},
    "DTI": {"113223": ("DTI", [7260])},
    "DVD": {"110033": ("DVD", [405])},
    "DVDImported": {"110025": ("DVD Imported", [7008])},
    "DW": {"126222": ("D-W", [218, 7180, 7469])},
    "DarkBrownColor": {"111452": ("Dark brown color", [6067])},
    "DarkRedColor": {"111451": ("Dark red color", [6067])},
    "DarkfieldIllumination": {"111745": ("Darkfield illumination", [8123])},
    "DataNotAccessibleFromStorage": {
        "130752": ("Data Not Accessible from Storage", [7031])
    },
    "DataRetentionPolicyExpired": {
        "113039": ("Data Retention Policy Expired", [7010, 7031])
    },
    "DateStickerIsMissing": {"111193": ("Date sticker is missing", [6041])},
    "DatetimeConcernNoted": {"121431": ("DateTime Concern Noted", [])},
    "DatetimeConcernResolved": {"121432": ("DateTime Concern Resolved", [])},
    "DatetimeEnded": {"111527": ("DateTime Ended", [])},
    "DatetimeEstimated": {"121137": ("DateTime Estimated", [3430])},
    "DatetimeOfDeath": {"122165": ("DateTime of Death", [])},
    "DatetimeOfIntervention": {"122105": ("DateTime of Intervention", [])},
    "DatetimeOfLastEvaluation": {"111536": ("DateTime of last evaluation", [])},
    "DatetimeOfProcessing": {"111702": ("DateTime of processing", [])},
    "DatetimeOfRecordingOfLogEntry": {
        "121125": ("DateTime of Recording of Log Entry", [])
    },
    "DatetimeProblemObserved": {"111535": ("DateTime problem observed", [])},
    "DatetimeProblemResolved": {"121433": ("DateTime Problem Resolved", [])},
    "DatetimeStarted": {"111526": ("DateTime Started", [])},
    "DatetimeUnsynchronized": {"121136": ("DateTime Unsynchronized", [3430])},
    "DeIdentifyingEquipment": {"109104": ("De-identifying Equipment", [7005])},
    "DeathDuringCatheterization": {"122167": ("Death During Catheterization", [3413])},
    "DeathDuringThisAdmission": {"122166": ("Death During This Admission", [])},
    "DeepRetinaStructuralReflectanceMap": {
        "128270": ("Deep retina structural reflectance map", [4271])
    },
    "DeepRetinaVasculatureFlow": {"128269": ("Deep retina vasculature flow", [4271])},
    "DeformableModels": {"123111": ("Deformable Models", [7162])},
    "DeformedForRegistration": {"125027": ("Deformed for Registration", [7203])},
    "DegreeOfFreedomExactRotationalTranslationValue": {
        "112379": ("Degree of Freedom Exact Rotational Translation Value", [])
    },
    "DegreeOfFreedomExactTranslationalValue": {
        "112376": ("Degree of Freedom Exact Translational Value", [])
    },
    "DegreeOfFreedomID": {"112363": ("Degree of Freedom ID", [])},
    "DegreeOfFreedomMaximumRotationalValue": {
        "112381": ("Degree of Freedom Maximum Rotational Value", [])
    },
    "DegreeOfFreedomMaximumTranslationalValue": {
        "112378": ("Degree of Freedom Maximum Translational Value", [])
    },
    "DegreeOfFreedomMinimumRotationalValue": {
        "112380": ("Degree of Freedom Minimum Rotational Value", [])
    },
    "DegreeOfFreedomMinimumTranslationalValue": {
        "112377": ("Degree of Freedom Minimum Translational Value", [])
    },
    "DegreeOfThrombus": {"122131": ("Degree of Thrombus", [])},
    "DegreesOfFreedomSpecification": {
        "112362": ("Degrees of Freedom Specification", [])
    },
    "DelayExportUntilClinicalInformationIsAvailable": {
        "128716": ("Delay export until clinical information is available", [7015])
    },
    "DelayExportUntilConfirmationOfDiagnosisIsAvailable": {
        "128717": ("Delay export until confirmation of diagnosis is available", [7015])
    },
    "DelayExportUntilExpertReviewIsAvailable": {
        "128722": ("Delay export until expert review is available", [7015])
    },
    "DelayExportUntilFinalReportIsAvailable": {
        "128715": ("Delay export until final report is available", [7015])
    },
    "DelayExportUntilHistopathologyIsAvailable": {
        "128718": ("Delay export until histopathology is available", [7015])
    },
    "DelayExportUntilOtherLaboratoryResultsAreAvailable": {
        "128719": ("Delay export until other laboratory results are available", [7015])
    },
    "DelayExportUntilPatientDies": {
        "128721": ("Delay export until patient dies", [7015])
    },
    "DelayExportUntilPatientIsDischarged": {
        "128720": ("Delay export until patient is discharged", [7015])
    },
    "DelayedRedistributionState": {"109094": ("Delayed Redistribution State", [3101])},
    "DeliveryRateToleranceViolation": {
        "130470": ("Delivery Rate Tolerance Violation", [9566])
    },
    "DeliveryVerificationNote": {"130031": ("Delivery Verification Note", [])},
    "DensitometricLuminalCrossSectionalAreaGraph": {
        "122517": ("Densitometric Luminal Cross-sectional Area Graph", [])
    },
    "DensitometricMethod": {"122474": ("Densitometric method", [3470])},
    "Density": {"112118": ("Density", [6102, 6104])},
    "Density_Retired": {"111103": ("Density", [])},
    "DepartmentSystemScheduler": {"DSS": ("Department System Scheduler", [30])},
    "DependentOpacity": {"112119": ("Dependent opacity", [6102, 6104])},
    "Deployment": {"121155": ("Deployment", [])},
    "DeprecatedProtocol": {"128610": ("Deprecated protocol", [800])},
    "Depth": {"111020": ("Depth", [])},
    "Derivation": {"121401": ("Derivation", [9000])},
    "DerivedAreaNonValve": {"122160": ("Derived Area, Non-Valve", [])},
    "DerivedEffectiveDiameter": {"113933": ("Derived Effective Diameter", [])},
    "DerivedFiducial": {"112357": ("Derived Fiducial", [])},
    "DerivedFromHumanReadableReports": {
        "113868": ("Derived From Human-Readable Reports", [10020, 10021])
    },
    "DerivedHemodynamicMeasurements": {
        "122126": ("Derived Hemodynamic Measurements", [])
    },
    "DerivedImagingMeasurements": {"126011": ("Derived Imaging Measurements", [])},
    "DerivedNonValveArea": {"122160": ("Derived Non-Valve Area", [3614, 3615])},
    "DerivedPlanningImages": {"112372": ("Derived Planning Images", [])},
    "Dermoscopy": {"DMS": ("Dermoscopy", [29, 30, 33])},
    "DescriptionOfChange": {"111021": ("Description of Change", [])},
    "DescriptionOfMaterial": {"121145": ("Description of Material", [3426])},
    "Designator": {"129002": ("Designator", [7453])},
    "DestinationMedia": {"110154": ("Destination Media", [402])},
    "DestinationRoleID": {"110152": ("Destination Role ID", [402])},
    "DestructionOfMicrobubbles": {"125235": ("Destruction of microbubbles", [12031])},
    "Detach": {"110125": ("Detach", [401, 403])},
    "DetectionPerformed": {"111022": ("Detection Performed", [])},
    "DetectorArtifacts": {"111214": ("Detector artifact(s)", [6041, 6135, 7011])},
    "DetectorType": {"113947": ("Detector Type", [])},
    "Deterministic": {"113211": ("Deterministic", [7262])},
    "DeterministicTrackingAlgorithm": {
        "113211": ("Deterministic Tracking Algorithm", [])
    },
    "DeviationIndex": {"113847": ("Deviation Index", [])},
    "Device": {"121007": ("Device", [270])},
    "DeviceCode": {"121150": ("Device Code", [])},
    "DeviceCrossedLesion": {"122089": ("Device crossed lesion", [3422])},
    "DeviceDeployed": {"122305": ("Device deployed", [3412])},
    "DeviceDescription": {"120999": ("Device Description", [])},
    "DeviceLocalizer": {"121341": ("Device localizer", [7201])},
    "DeviceManufacturer": {"113878": ("Device Manufacturer", [])},
    "DeviceModelName": {"113879": ("Device Model Name", [])},
    "DeviceName": {"113877": ("Device Name", [])},
    "DeviceObserverManufacturer": {"121014": ("Device Observer Manufacturer", [])},
    "DeviceObserverManufacturerClassUID": {
        "121061": ("Device Observer Manufacturer Class UID", [])
    },
    "DeviceObserverModelName": {"121015": ("Device Observer Model Name", [])},
    "DeviceObserverName": {"121013": ("Device Observer Name", [])},
    "DeviceObserverPhysicalLocationDuringObservation": {
        "121017": ("Device Observer Physical Location During Observation", [])
    },
    "DeviceObserverSerialNumber": {"121016": ("Device Observer Serial Number", [])},
    "DeviceObserverUID": {"121012": ("Device Observer UID", [])},
    "DeviceRelatedAndEnvironmentRelatedEvent": {
        "130864": ("Device-related and Environment-related Event", [3047])
    },
    "DeviceRoleInProcedure": {"113876": ("Device Role in Procedure", [])},
    "DeviceSerialNumber": {"113880": ("Device Serial Number", [])},
    "DeviceSubject": {"121192": ("Device Subject", [271])},
    "DeviceSubjectManufacturer": {"121194": ("Device Subject Manufacturer", [])},
    "DeviceSubjectModelName": {"121195": ("Device Subject Model Name", [])},
    "DeviceSubjectName": {"121193": ("Device Subject Name", [])},
    "DeviceSubjectPhysicalLocationDuringObservation": {
        "121197": ("Device Subject Physical Location during observation", [])
    },
    "DeviceSubjectSerialNumber": {"121196": ("Device Subject Serial Number", [])},
    "DeviceSubjectUID": {"121198": ("Device Subject UID", [])},
    "DfCD4589Zr": {"126520": ("Df-CD45 ^89^Zr", [4021])},
    "DfFK23PEG489Zr": {"126763": ("Df-[FK](2)-3PEG(4) ^89^Zr", [4021])},
    "DfFK289Zr": {"126762": ("Df-[FK](2) ^89^Zr", [4021])},
    "DfFK89Zr": {"126760": ("Df-FK ^89^Zr", [4021])},
    "DfFKPEG389Zr": {"126761": ("Df-FK-PEG(3) ^89^Zr", [4021])},
    "Dgn": {"128407": ("DgN", [10069])},
    "DiabeticFibrousMastopathy": {
        "111259": ("Diabetic fibrous mastopathy", [6030, 6031])
    },
    "DiagnosisInputUsed": {"128218": ("Diagnosis Input Used", [7010])},
    "DiagnosticAdministration": {"130249": ("Diagnostic Administration", [72])},
    "DiagnosticSourceImages": {
        "128181": ("Diagnostic Source Images", [7010, 9509, 9510])
    },
    "DiameterBasedMethod": {"122651": ("Diameter Based Method", [3804, 3807])},
    "DiameterGraph": {"122509": ("Diameter Graph", [])},
    "DiameterSymmetry": {"122544": ("Diameter Symmetry", [])},
    "DiamondDetector": {"128708": ("Diamond Detector", [7026, 7027, 7151, 7193])},
    "Diaphanography": {"DG": ("Diaphanography", [29, 30, 33])},
    "DiaphragmaticAttenuation": {"122745": ("Diaphragmatic attenuation", [3115])},
    "DiastolicBloodVelocityMean": {
        "122201": (
            "Diastolic blood velocity, mean",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "DiastolicBloodVelocityPeak": {
        "122202": (
            "Diastolic blood velocity, peak",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "DiastolicPressureAverage": {"109020": ("Diastolic pressure, average", [])},
    "DiastolicPressureNadir": {"109021": ("Diastolic pressure nadir", [])},
    "DiastolicSystolicVelocityRatio": {
        "122218": ("Diastolic/Systolic velocity ratio", [3620])
    },
    "Dictation": {"110011": ("Dictation", [])},
    "DifferenceAverageOfGLCM": {"128784": ("Difference Average of GLCM", [])},
    "DifferenceEntropyOfGLCM": {"128786": ("Difference Entropy of GLCM", [])},
    "DifferenceInBorderDefinition": {"112166": ("Difference in border definition", [])},
    "DifferenceInBorderShape": {"112165": ("Difference in border shape", [])},
    "DifferenceInDistribution": {"112167": ("Difference in distribution", [])},
    "DifferenceInEDLVVolume": {"122769": ("Difference in ED LV Volume", [])},
    "DifferenceInEjectionFraction": {"122768": ("Difference in Ejection Fraction", [])},
    "DifferenceInSiteInvolvement": {"112168": ("Difference in site involvement", [])},
    "DifferenceInTexture": {"112170": ("Difference in Texture", [])},
    "DifferenceInTypeOfContent": {"112169": ("Difference in Type of Content", [])},
    "DifferenceVarianceOfGLCM": {"128785": ("Difference Variance of GLCM", [])},
    "DifferentialDiagnosisImpression": {
        "111023": ("Differential Diagnosis/Impression", [])
    },
    "DifferentialInterferenceContrast": {
        "111748": ("Differential interference contrast", [8123])
    },
    "DifferentialSignal": {"109006": ("Differential signal", [3240])},
    "DifficultPhysicalClinicalExamination": {
        "111479": ("Difficult physical/clinical examination", [6055])
    },
    "DiffuseContrastEnhancement": {
        "130603": ("Diffuse Contrast Enhancement", [6335, 6345, 6346])
    },
    "DiffuseDirectIllumination": {"111625": ("Diffuse direct illumination", [4203])},
    "DiffuselyIncreasedVascularityInSurroundingTissue": {
        "111377": ("Diffusely increased vascularity in surrounding tissue", [6157])
    },
    "DiffusionAnisotropy": {"113044": ("Diffusion Anisotropy", [])},
    "DiffusionAttenuated": {"113045": ("Diffusion Attenuated", [])},
    "DiffusionCoefficient": {
        "113289": ("Diffusion Coefficient", [218, 7180, 7272, 7469])
    },
    "DiffusionImageAnalysis": {"129105": ("Diffusion image analysis", [7203])},
    "DiffusionTractography": {"129106": ("Diffusion tractography", [7203])},
    "DiffusionWeighted": {
        "113043": ("Diffusion weighted", [218, 6311, 7180, 7270, 7469])
    },
    "DiffusionWeightedAcquisitionHighestBValueImage": {
        "130566": ("Diffusion-weighted Acquisition Highest b-value image", [6311])
    },
    "DigitalFluoroscopy": {"DF": ("Digital fluoroscopy", [])},
    "DigitalMammography": {"111409": ("Digital Mammography", [])},
    "DigitalMicroscopy": {"DM": ("Digital microscopy", [])},
    "DigitalRadiography": {"DX": ("Digital Radiography", [29, 30, 33])},
    "DigitalSubtractionAngiography": {"DS": ("Digital Subtraction Angiography", [])},
    "DigitalTimecodeNOS": {"109001": ("Digital timecode (NOS)", [3090])},
    "Diode": {"128704": ("Diode", [7026, 7027, 7151, 7193])},
    "DiodeArray": {"128702": ("Diode Array", [7026, 7027, 7151, 7193])},
    "DirectDetector": {"113948": ("Direct Detector", [10030])},
    "DirectlyMeasured": {"125316": ("Directly measured", [12227, 12303])},
    "DisagreementWithPriorFindings": {
        "122776": ("Disagreement with prior findings", [3217])
    },
    "DisapprovedForAnyUse": {"128609": ("Disapproved for any use", [800])},
    "DisapprovedForExperimentalUse": {
        "128612": ("Disapproved for experimental use", [800])
    },
    "DisapprovedForUseAtTheInstitution": {
        "128623": ("Disapproved for use at the institution", [800])
    },
    "DisapprovedForUseInTheClinicalTrial": {
        "128624": ("Disapproved for use in the clinical trial", [800])
    },
    "DisapprovedForUseOnPregnantPatients": {
        "128617": ("Disapproved for use on pregnant patients", [800])
    },
    "DiscCentered": {"111901": ("Disc centered", [4207])},
    "DiscFovea": {"111934": ("Disc-Fovea", [4266])},
    "DiscMaculaCentered": {"111903": ("Disc-macula centered", [4207])},
    "Discarded": {"111458": ("Discarded", [])},
    "DischargeDatetime": {"122163": ("Discharge DateTime", [])},
    "DischargeSummary": {"121115": ("Discharge Summary", [])},
    "DiscontinuedForUnspecifiedReason": {
        "110513": (
            "Discontinued for unspecified reason",
            [9300, 9301, 9302, 9303, 9561],
        )
    },
    "DiscontinuedProcedureStepRescheduled": {
        "110528": ("Discontinued Procedure Step rescheduled", [9300])
    },
    "DiscontinuedProcedureStepReschedulingRecommended": {
        "110529": ("Discontinued Procedure Step rescheduling recommended", [9300])
    },
    "DissectionClassification": {"122387": ("Dissection Classification", [])},
    "DissectionWithEntireSpecimenSubmission": {
        "111726": ("Dissection with entire specimen submission", [8110])
    },
    "DissectionWithRepresentativeSectionsSubmission": {
        "111727": ("Dissection with representative sections submission", [8110])
    },
    "DissimilarityOfGLCM": {"126064": ("Dissimilarity of GLCM", [])},
    "DistalCentralizer": {"112325": ("Distal Centralizer", [7307, 7308])},
    "DistalFindingSite": {"121117": ("Distal Finding Site", [])},
    "DistalReference": {"122381": ("Distal Reference", [3486])},
    "DistalStentMargin": {"122386": ("Distal Stent Margin", [3487])},
    "Distance": {"121206": ("Distance", [218, 6165, 7469, 7470, 10069])},
    "DistanceCorrection": {"128527": ("Distance Correction", [10069])},
    "DistanceFromAnus": {"112233": ("Distance from anus", [6212])},
    "DistanceFromChestWall": {"121244": ("Distance from chest wall", [6165])},
    "DistanceFromNeurovascularBundle": {
        "130557": ("Distance from neurovascular bundle", [6351])
    },
    "DistanceFromNipple": {"121242": ("Distance from nipple", [6165])},
    "DistanceFromSkin": {"121243": ("Distance from skin", [6165])},
    "DistancePupillaryDistance": {"111679": ("Distance Pupillary Distance", [])},
    "DistanceSourceToDetector": {"113750": ("Distance Source to Detector", [10008])},
    "DistanceSourceToIsocenter": {"113748": ("Distance Source to Isocenter", [10008])},
    "DistanceSourceToReferencePoint": {
        "113737": ("Distance Source to Reference Point", [10008])
    },
    "DistanceSourceToTablePlane": {
        "113792": ("Distance Source to Table Plane", [10008])
    },
    "DistinctlyDefined": {"112138": ("Distinctly defined", [6120])},
    "DistortedImplant": {"111506": ("Distorted implant", [6072])},
    "DistortionArtifactInTheAreaOfInterest": {
        "130586": ("Distortion artifact in the area of interest", [6315, 6318])
    },
    "DistributedDiffusionCoefficient": {
        "113298": ("Distributed Diffusion Coefficient", [218, 7180, 7272, 7469])
    },
    "DistributionDescriptor": {"112006": ("Distribution Descriptor", [])},
    "DistributionRepresentation": {"128413": ("Distribution Representation", [])},
    "DivideByZero": {"114003": ("Divide by zero", [42, 43])},
    "DoctorCanceledProcedure": {
        "110500": ("Doctor canceled procedure", [9300, 9301, 9303, 9561])
    },
    "Document": {"121003": ("Document", [])},
    "DocumentDigitizerEquipment": {"DOCD": ("Document Digitizer Equipment", [7005])},
    "DocumentProducer": {"DOC": ("Document", [32, 33])},
    "DocumentTitle": {"121144": ("Document Title", [])},
    "DocumentTitleModifier": {"113011": ("Document Title Modifier", [])},
    "DogRegistryOfAmerica": {"109212": ("Dog Registry of America", [7481])},
    "DoorInterlock": {"130478": ("Door Interlock", [9561, 9568])},
    "DopplerAngle": {"125106": ("Doppler Angle", [])},
    "DopplerAudio": {"109113": ("Doppler audio", [3000])},
    "DopplerOCTA": {"128256": ("Doppler OCT-A", [4270])},
    "DopplerVolumeFlow": {"125219": ("Doppler Volume Flow", [12227, 12231])},
    "DoseAreaProduct": {"122130": ("Dose Area Product", [])},
    "DoseAreaProductTotal": {"113722": ("Dose Area Product Total", [10050])},
    "DoseCalculationBoundingVolume": {
        "130065": ("Dose Calculation Bounding Volume", [9535])
    },
    "DoseCalibrator": {"113541": ("Dose Calibrator", [10041])},
    "DoseCheckAlertDetails": {"113900": ("Dose Check Alert Details", [])},
    "DoseCheckNotificationDetails": {"113908": ("Dose Check Notification Details", [])},
    "DoseFrequency": {"111578": ("Dose frequency", [6092])},
    "DoseFromPriorTreatment": {"129212": ("Dose from prior Treatment", [7022])},
    "DoseGradient": {"128488": ("Dose Gradient", [10063])},
    "DoseImage": {"121342": ("Dose Image", [10063])},
    "DoseLimitInterlock": {"130738": ("Dose Limit Interlock", [9561, 9568])},
    "DoseMeasurementDevice": {"113794": ("Dose Measurement Device", [])},
    "DosePointCloud": {"128496": ("Dose Point Cloud", [10063])},
    "DoseRP": {"113738": ("Dose (RP)", [])},
    "DoseRPTotal": {"113725": ("Dose (RP) Total", [])},
    "DoseRateToleranceViolation": {"130474": ("Dose Rate Tolerance Violation", [9567])},
    "DosimeterType": {"113818": ("Dosimeter Type", [])},
    "DosimetricResult": {"128190": ("Dosimetric Result", [7010, 7023])},
    "DosingFactor": {"130190": ("Dosing Factor", [])},
    "DoubleExposure": {"113026": ("Double exposure", [7011])},
    "DoubleMDerivedFilter": {"130767": ("Double-m-derived filter", [3042])},
    "DoubleProduct": {"122708": ("Double Product", [])},
    "DrugAdministered": {"122083": ("Drug administered", [3409])},
    "DrugEnd": {"122082": ("Drug end", [3409])},
    "DrugProductIdentifier": {"113510": ("Drug Product Identifier", [])},
    "DrugStart": {"122081": ("Drug start", [3409])},
    "DualSideIllumination": {"130811": ("Dual-side illumination", [11001])},
    "DualSpeedOfSoundCorrection": {
        "130819": ("Dual Speed of Sound Correction", [11004])
    },
    "DuctNarrowing": {"111429": ("Duct narrowing", [6054, 6057])},
    "DuctalAdenoma": {"111258": ("Ductal adenoma", [6030, 6031])},
    "DuctalHyperplasia": {"111249": ("Ductal hyperplasia", [])},
    "Ductography": {"111139": ("Ductography", [])},
    "DuplexDoppler": {"DD": ("Duplex Doppler", [])},
    "DuplicateOrder": {"110510": ("Duplicate order", [9300, 9301, 9302, 9303])},
    "DurationOfAdministration": {"122095": ("Duration of administration", [3410])},
    "DurationOfIntervention": {"122106": ("Duration of Intervention", [])},
    "DurationOfTimePeriod": {"130532": ("Duration of Time Period", [10073])},
    "DuringTransport": {"127102": ("During transport", [634])},
    "DuringVoiding": {"109137": ("During voiding", [91])},
    "DwConversionFactorCoefficients": {
        "113991": ("Dw Conversion Factor Coefficients", [])
    },
    "DynamicContrastEnhancedAcquisition": {
        "130567": ("Dynamic Contrast-Enhanced Acquisition", [6311])
    },
    "DynamicContrastEnhancedAcquisitionSubtractionImage": {
        "130568": ("Dynamic Contrast-Enhanced Acquisition Subtraction image", [6311])
    },
    "DynamicContrastEnhancedTemporalResolution": {
        "130547": ("Dynamic contrast-enhanced temporal resolution", [])
    },
    "DynamicContrastMRMeasurementReport": {
        "126002": ("Dynamic Contrast MR Measurement Report", [7021])
    },
    "DynamicLight": {"111630": ("Dynamic light", [4203])},
    "DynamicWedge": {"130348": ("Dynamic Wedge", [9546])},
    "DyssynchronyIndex": {"125325": ("Dyssynchrony Index", [12304])},
    "E4G1089Zr": {"126519": ("E4G10 ^89^Zr", [4021])},
    "ECGBasedGatingSignalProcessed": {
        "109002": ("ECG-based gating signal, processed", [3090])
    },
    "ECGGlobalMeasurements": {"122158": ("ECG Global Measurements", [])},
    "ECGLeadMeasurements": {"122159": ("ECG Lead Measurements", [])},
    "ECST": {"122656": ("ECST", [3804])},
    "EEGAnnotation": {"130861": ("EEG Annotation", [3047])},
    "EEMCrossSectionalArea": {"122333": ("EEM Cross-Sectional Area", [3482])},
    "EEMDiameter": {"122330": ("EEM Diameter", [3481])},
    "EEMDiameterRatio": {"122352": ("EEM Diameter Ratio", [3484])},
    "EEMVolume": {"122371": ("EEM Volume", [3485])},
    "EMGAnnotation": {"130862": ("EMG Annotation", [3047])},
    "EOGAnnotation": {"130863": ("EOG Annotation", [3047])},
    "EPProcedurePhase": {"109061": ("EP Procedure Phase", [])},
    "ESUR2012ProstateMRIAcquisitionRequirements": {
        "130606": ("ESUR 2012 prostate MRI acquisition requirements", [6326, 6353])
    },
    "EarlyContrastEnhancement": {
        "130602": ("Early Contrast Enhancement", [6335, 6345, 6346])
    },
    "EarlyGestation": {"125009": ("Early Gestation", [])},
    "EchoPattern": {"111360": ("Echo pattern", [])},
    "Echocardiography": {"EC": ("Echocardiography", [])},
    "EchogenicHalo": {"111359": ("Echogenic halo", [6153])},
    "Echogenicity": {"110849": ("Echogenicity", [218, 7180, 7469])},
    "Echovist": {"125902": ("Echovist", [12030])},
    "Ecromeximab89Zr": {"126732": ("Ecromeximab ^89^Zr", [4021])},
    "EdgeDetection": {"123103": ("Edge Detection", [7162])},
    "EdgeEnhancement": {"113086": ("Edge enhancement", [7203])},
    "EditedModel": {"129010": ("Edited Model", [7062])},
    "EducationalIntent": {"129012": ("Educational Intent", [7064])},
    "EffectiveAtomicNumber": {"129320": ("Effective Atomic Number", [301])},
    "EffectiveDose": {"113839": ("Effective Dose", [10050])},
    "EffectiveDoseConversionFactor": {
        "113840": ("Effective Dose Conversion Factor", [])
    },
    "EffectiveDosePhantomType": {"113817": ("Effective Dose Phantom Type", [])},
    "EffectiveOrificeArea": {"125326": ("Effective Orifice Area", [12304])},
    "EffectiveZ": {"130084": ("Effective Z", [])},
    "EffectiveZPerA": {"130085": ("Effective Z per A", [])},
    "Elasticity": {"110830": ("Elasticity", [218, 7180, 7469, 11006, 12308])},
    "ElectricalFailure": {"111217": ("Electrical failure", [6041, 6135, 7011])},
    "Electrocardiography": {"ECG": ("Electrocardiography", [29, 30, 33])},
    "ElectrodePlacement": {"122141": ("Electrode Placement", [])},
    "Electroencephalography": {"EEG": ("Electroencephalography", [29, 30, 33])},
    "ElectromechanicalDelay": {"125320": ("Electromechanical Delay", [12307])},
    "Electromyography": {"EMG": ("Electromyography", [29, 30, 33])},
    "ElectronFixedAperture": {"130343": ("Electron Fixed Aperture", [9541, 9545])},
    "Electrooculography": {"EOG": ("Electrooculography", [29, 30, 33])},
    "ElementalCompositionAtomicMassFraction": {
        "130094": ("Elemental Composition Atomic Mass Fraction", [])
    },
    "ElevationBasedCornealTomographer": {
        "111945": ("Elevation-based corneal tomographer", [4210])
    },
    "Eligibility": {"126075": ("Eligibility", [6146])},
    "EligibilityReader": {"129001": ("Eligibility Reader", [7453])},
    "EligibleForReimbursement": {"128613": ("Eligible for reimbursement", [800])},
    "EligibleForReimbursementOnPerPatientBasis": {
        "128614": ("Eligible for reimbursement on per patient basis", [800])
    },
    "Elliptic": {"112134": ("Elliptic", [6119])},
    "EllipticFilter": {"130762": ("Elliptic filter", [3042])},
    "Email": {"110031": ("Email", [405])},
    "EmbryonicKidney": {"127801": ("Embryonic kidney", [645, 8134])},
    "EmergencyOverrideStarted": {"110127": ("Emergency Override Started", [401, 403])},
    "EmergencyOverrideStopped": {"110138": ("Emergency Override Stopped", [401, 403])},
    "EmpiricalAlgorithm": {"128481": ("Empirical Algorithm", [10068])},
    "EndCirculatorySupport": {"121158": ("End Circulatory Support", [3550])},
    "EndDiastole": {"109022": ("End diastole", [])},
    "EndOfExpiration": {"109023": ("End of expiration", [3337])},
    "EndOfInspiration": {"109024": ("End of inspiration", [3337])},
    "EndOfSystole": {"109070": ("End of systole", [3337])},
    "EndOfTimePeriod": {"130534": ("End of Time Period", [10073])},
    "EndOfXRayIrradiation": {"113810": ("End of X-Ray Irradiation", [])},
    "EndOxygenAdministration": {"121162": ("End oxygen administration", [3530])},
    "EndPacing": {"121167": ("End Pacing", [3552])},
    "EndProcedureAction": {"121131": ("End Procedure Action", [3421])},
    "EndProcedureActionItem": {"121131": ("End Procedure Action Item", [])},
    "EndVentilation": {"121169": ("End Ventilation", [3551])},
    "EndingFlowRateOfAdministration": {
        "130209": ("Ending Flow Rate of administration", [])
    },
    "EndocardialMethod": {"130683": ("Endocardial Method", [12227, 12310])},
    "EndocavitaryTransducer": {"125266": ("Endocavitary Transducer", [12035])},
    "Endoleak": {"122680": ("endoleak", [3813])},
    "EndorectalCoilType": {"130544": ("Endorectal coil type", [])},
    "EndorectalCoilUsed": {"130543": ("Endorectal coil used", [])},
    "EndorectalTransducer": {"125264": ("Endorectal Transducer", [12035])},
    "Endoscopy": {"ES": ("Endoscopy", [29, 30, 33])},
    "EndovaginalTransducer": {"125263": ("Endovaginal Transducer", [12035])},
    "EnhancedMultiFrameConversionEquipment": {
        "109106": ("Enhanced Multi-frame Conversion Equipment", [7005])
    },
    "EnrichmentManufacturer": {"127191": ("Enrichment manufacturer", [])},
    "EnrichmentMaterial": {"127190": ("Enrichment material", [])},
    "EnrichmentMaterialPresent": {"127192": ("Enrichment material present", [])},
    "EntireBodyTargetVolume": {"130055": ("Entire Body Target Volume", [9534])},
    "EntirePullback": {"122383": ("Stented Region", [3487])},
    "EntranceExposureAtRP": {"111636": ("Entrance Exposure at RP", [10050])},
    "EnvironmentalExposureHistory": {"111547": ("Environmental Exposure History", [])},
    "EnvironmentalFactor": {"111548": ("Environmental Factor", [])},
    "EpicardialMethod": {"130684": ("Epicardial Method", [12227, 12310])},
    "EpidermalInclusionCyst": {"111262": ("Epidermal inclusion cyst", [])},
    "EpifluorescenceIllumination": {"111743": ("Epifluorescence illumination", [8123])},
    "Equation": {"121420": ("Equation", [228])},
    "EquationCitation": {"121421": ("Equation Citation", [228])},
    "EquidistantMethod": {"122574": ("Equidistant method", [3456])},
    "EquipmentBroughtToProcedureRoom": {
        "122047": ("Equipment brought to procedure room", [3427])
    },
    "EquipmentChange": {"110516": ("Equipment change", [9300, 9301])},
    "EquipmentFailure": {
        "110501": ("Equipment failure", [71, 3427, 4221, 4222, 9300, 9301, 9302, 9561])
    },
    "EquipmentFailure_Retired": {"122046": ("Equipment failure", [])},
    "EquipmentIdentification": {"121122": ("Equipment Identification", [])},
    "EquipmentLandmark": {"128750": ("Equipment Landmark", [])},
    "EquipmentLandmarkToPatientFiducialZDistance": {
        "128756": ("Equipment Landmark to Patient Fiducial Z Distance", [])
    },
    "EquipmentLandmarkXPosition": {"128752": ("Equipment Landmark X Position", [])},
    "EquipmentLandmarkZPosition": {"128753": ("Equipment Landmark Z Position", [])},
    "EquipmentOrigin": {"130537": ("Equipment Origin", [10074])},
    "EquipmentReady": {"122048": ("Equipment ready", [3427])},
    "EquipmentRemoved": {"122049": ("Equipment removed", [3427])},
    "Equivalent2GrayFractionsModel": {
        "130130": ("Equivalent 2-Gray Fractions Model", [9539])
    },
    "EquivalentAttenuatorMaterial": {"128465": ("Equivalent Attenuator Material", [])},
    "EquivalentAttenuatorThickness": {
        "128469": ("Equivalent Attenuator Thickness", [])
    },
    "EquivalentCDADocument": {"121331": ("Equivalent CDA Document", [7006])},
    "EquivalentDose": {"128512": ("Equivalent Dose", [218, 7180, 7469, 10070])},
    "EquivalentKReading": {"111756": ("Equivalent K-reading", [4235, 4242])},
    "EquivalentMeaningOfConceptName": {
        "121050": ("Equivalent Meaning of Concept Name", [9000])
    },
    "EquivalentMeaningOfValue": {"121051": ("Equivalent Meaning of Value", [])},
    "ErgometerPower": {"122704": ("Ergometer power", [])},
    "ErodedPlaque": {"122390": ("Eroded Plaque", [3491])},
    "EsophagealPressure": {"130425": ("Esophageal Pressure", [3005])},
    "Estimated": {"121427": ("Estimated", [])},
    "EstimatedExtravasationActivity": {
        "113506": ("Estimated Extravasation Activity", [])
    },
    "EstimatedExtravasationVolume": {"130214": ("Estimated Extravasation Volume", [])},
    "EstimatedFromWaterEquivalentDiameter": {
        "113988": ("Estimated from Water Equivalent Diameter", [10023])
    },
    "EstimatedNormalFlow": {"122555": ("Estimated Normal Flow", [])},
    "EstimatedTimeframe": {"111395": ("Estimated Timeframe", [])},
    "EstrogenReceptor": {"111475": ("Estrogen receptor", [])},
    "Euler": {"113218": ("Euler", [7262])},
    "EuropiumOrEuropiumCompound": {"113711": ("Europium or Europium compound", [])},
    "EventButtonPressed": {"130893": ("Event button pressed", [3039])},
    "EventButtonTest": {"130894": ("Event button test", [3039])},
    "EventUIDUsed": {"128429": ("Event UID Used", [])},
    "ExcellentImageQuality": {"122740": ("Excellent image quality", [3114])},
    "ExcursionDistance": {"125327": ("Excursion Distance", [12304])},
    "ExerciseDurationAfterStressAgentInjection": {
        "122711": ("Exercise duration after stress agent injection", [])
    },
    "ExerciserDevicePresent": {"127193": ("Exerciser device present", [])},
    "ExogenousSubstance": {"127400": ("Exogenous substance", [])},
    "ExpectedADCMapIsMissing": {
        "130585": ("Expected ADC map is missing", [6315, 6318])
    },
    "ExpectedBValueIsMissing": {
        "130583": ("Expected b-value is missing", [6315, 6318])
    },
    "Explantation": {"111510": ("Explantation", [6072])},
    "Export": {"110106": ("Export", [400])},
    "ExposedRange": {"113899": ("Exposed Range", [])},
    "Exposure": {"113736": ("Exposure", [])},
    "ExposureIndex": {"113845": ("Exposure Index", [])},
    "ExposureTime": {"113824": ("Exposure Time", [])},
    "ExposureTimePerRotation": {"113834": ("Exposure Time per Rotation", [])},
    "ExposureTime_Retired": {"113735": ("Exposure Time", [])},
    "ExtendedPatientAnatomyModel": {
        "130068": ("Extended Patient Anatomy Model", [9507])
    },
    "ExtendedRenderingForPresentation": {
        "121334": ("Extended Rendering for Presentation", [7006])
    },
    "ExtendedToftsModel": {"126341": ("Extended Tofts Model", [4101, 4106])},
    "ExternalBodyModel": {"130047": ("External Body Model", [9501, 9502, 9580])},
    "ExternalBodyStructure": {"130047": ("External Body Structure", [])},
    "ExternalCameraPhotography": {"XC": ("External-camera Photography", [29, 30, 33])},
    "ExternalDataSource": {"111781": ("External Data Source", [4240])},
    "ExternalTransducer": {"125261": ("External Transducer", [12035])},
    "ExtraAbdominalDesmoid": {"111260": ("Extra abdominal desmoid", [])},
    "ExtraProstaticFinding": {"130559": ("Extra-prostatic Finding", [])},
    "ExtractionOfIndividualSubjectFromGroup": {
        "113131": ("Extraction of individual subject from group", [7203])
    },
    "Extravasation": {"111428": ("Extravasation", [6054, 6057])},
    "ExtravasationVisibleInImage": {
        "113568": ("Extravasation visible in image", [60, 9300, 10043])
    },
    "ExtremelySmall": {"112131": ("Extremely small", [6118])},
    "FACT": {"113214": ("FACT", [7262])},
    "FIRFilter": {"130770": ("FIR filter", [3043])},
    "FLB457C11": {"126706": ("FLB 457 C^11^", [4021])},
    "FWPByGACampbell1991": {"128040": ("FWP by GA, Campbell, 1991", [12012, 12016])},
    "FWPByGAHadlock1991": {"128041": ("FWP by GA, Hadlock, 1991", [12012, 12016])},
    "Failed": {"111224": ("Failed", [6042])},
    "FailedAnalyses": {"111024": ("Failed Analyses", [])},
    "FailedDetections": {"111025": ("Failed Detections", [])},
    "FallyprideC11": {"126704": ("Fallypride C^11^", [4021])},
    "FallyprideF18": {"126705": ("Fallypride F^18^", [4021])},
    "FalseLumen": {"122361": ("False Lumen", [3491])},
    "FalseMarkersPerCase": {"111087": ("False Markers per Case", [6048])},
    "FalseMarkersPerImage": {"111086": ("False Markers per Image", [6048])},
    "FalsePositiveDefectFinding": {"122748": ("False Positive defect finding", [3113])},
    "FamilyHistoryFetalMalformationSyndrome": {
        "111574": ("Family history, fetal malformation/syndrome", [6087, 6088])
    },
    "FamilyHistoryOfMelanomaInSitu": {
        "130481": ("Family history of melanoma in situ", [])
    },
    "FamilyHistoryOfNonMelanomaSkinCancer": {
        "130480": ("Family history of non-melanoma skin cancer", [])
    },
    "FamilyHistoryOfProstateCancer": {
        "111562": ("Family history of prostate cancer", [6081, 6087, 6322, 6327])
    },
    "FamilyHistoryUnknown": {"111563": ("Family history unknown", [6081, 6087])},
    "FamilyMemberWithRiskFactor": {"111537": ("Family Member with Risk Factor", [])},
    "FastDiffusionCoefficient": {
        "113292": ("Fast Diffusion Coefficient", [218, 7180, 7272, 7469])
    },
    "FastDiffusionCoefficientFraction": {
        "113293": ("Fast Diffusion Coefficient Fraction", [218, 7180, 7272, 7469])
    },
    "FastMode": {"114211": ("Fast mode", [8202])},
    "FastingDuration": {"113550": ("Fasting Duration", [])},
    "FatFraction": {"129100": ("Fat fraction", [218, 7180, 7469])},
    "FeatureDetectedOnImagesFromMultipleModalities": {
        "111159": ("Feature detected on images from multiple modalities", [6036])
    },
    "FeatureDetectedOnMultipleImages": {
        "111158": ("Feature detected on multiple images", [6036])
    },
    "FeatureDetectedOnOnlyOneOfTheImages": {
        "111157": ("Feature detected on only one of the images", [6036])
    },
    "FeatureDetectedOnTheOnlyImage": {
        "111156": ("Feature detected on the only image", [6036])
    },
    "FederationOfInternationalCanines": {
        "109213": ("Federation of International Canines", [7481])
    },
    "FeedManufacturer": {"127200": ("Feed manufacturer", [])},
    "FeedProductCode": {"127202": ("Feed product code", [])},
    "FeedProductName": {"127201": ("Feed product name", [])},
    "FeedSource": {"127205": ("Feed source", [])},
    "FeedbackTemperatureRegulation": {
        "127210": ("Feedback temperature regulation", [])
    },
    "Fellow": {"121088": ("Fellow", [7450, 7452])},
    "Female": {"F": ("Female", [7455, 7457])},
    "FemaleChangedToMale": {"FC": ("Female changed to Male", [7455])},
    "FemalePseudohermaphrodite": {"FP": ("Female Pseudohermaphrodite", [7455])},
    "FemoralHeadBallComponent": {
        "112308": ("Femoral Head Ball Component", [7307, 7308])
    },
    "FemoralHeadConeTaperComponent": {
        "112309": ("Femoral Head Cone Taper Component", [7307, 7308])
    },
    "FemoralHeadResurfacingComponent": {
        "112317": ("Femoral Head Resurfacing Component", [7307, 7308])
    },
    "FemoralStem": {"112310": ("Femoral Stem", [7307, 7308])},
    "FemoralStemComponent": {"112313": ("Femoral Stem Component", [7307, 7308])},
    "FemoralStemDistalComponent": {
        "112311": ("Femoral Stem Distal Component", [7307, 7308])
    },
    "FemoralStemProximalComponent": {
        "112312": ("Femoral Stem Proximal Component", [7307, 7308])
    },
    "FemurHeadCenterOfRotation": {
        "112304": ("Femur Head Center of Rotation", [7305, 7306])
    },
    "FetalBiometry": {"125002": ("Fetal Biometry", [])},
    "FetalBiometryRatios": {"125001": ("Fetal Biometry Ratios", [])},
    "FetalCardiacUltrasoundReport": {
        "125196": ("Fetal Cardiac Ultrasound Report", [7000, 12245])
    },
    "FetalCranium": {"125004": ("Fetal Cranium", [])},
    "FetalLongBones": {"125003": ("Fetal Long Bones", [])},
    "FetalMeasurements": {"125016": ("Fetal Measurements", [])},
    "Fetus": {"121026": ("Fetus", [271])},
    "FetusCharacteristics": {"125015": ("Fetus Characteristics", [])},
    "FetusNumber": {"121037": ("Fetus number", [])},
    "FetusSummary": {"125008": ("Fetus Summary", [])},
    "FibroLipidicPlaque": {"122394": ("Fibro-Lipidic Plaque", [3491, 3495, 3497])},
    "Fibroadenolipoma": {"111264": ("Fibroadenolipoma", [6030, 6031])},
    "FibroadenomatoidHyperplasia": {
        "111263": ("Fibroadenomatoid hyperplasia", [6030, 6031])
    },
    "Fibrocalcific": {"112163": ("Fibrocalcific", [6132])},
    "Fibronodular": {"112148": ("Fibronodular", [6123])},
    "FiducialAlignment": {"125022": ("Fiducial Alignment", [7100, 8203])},
    "FiducialFeature": {"122340": ("Fiducial feature", [])},
    "FiducialIntent": {"112369": ("Fiducial Intent", [])},
    "FiducialMark": {
        "112171": (
            "Fiducial mark",
            [6102, 6138, 6202, 6203, 6404, 7110, 7151, 7193, 9581],
        )
    },
    "Field1ForJoslin3Field": {"111621": ("Field 1 for Joslin3 field", [4207])},
    "Field2ForJoslin3Field": {"111622": ("Field 2 for Joslin 3 field", [4207])},
    "Field3ForJoslin3Field": {"111623": ("Field 3 for Joslin 3 field", [4207])},
    "FieldMapMRSignalIntensity": {
        "110807": ("Field Map MR Signal Intensity", [218, 7180, 7469])
    },
    "FillerNumber": {"121021": ("Filler Number", [])},
    "Film": {"110010": ("Film", [405])},
    "FilmDetectorType": {"113951": ("Film", [10030])},
    "FilmDigitizer": {"FILMD": ("Film Digitizer", [30, 7005])},
    "FilmHolder": {"130352": ("Film Holder", [9548])},
    "FilmScreenMammography": {"111408": ("Film Screen Mammography", [6050, 6083])},
    "FilteredBackProjection": {"113962": ("Filtered Back Projection", [10033])},
    "Filtration": {"130512": ("Filtration", [])},
    "FinalReport": {"128005": ("Final Report", [9233])},
    "Finding": {"121071": ("Finding", [3419, 6053, 7002, 9000])},
    "FindingObservationType": {"125305": ("Finding Observation Type", [])},
    "Findings": {"121070": ("Findings", [])},
    "FinePleomorphicCalcification": {
        "111344": ("Fine pleomorphic calcification", [6010, 6011])
    },
    "FirmSkinLesion": {"130485": ("Firm skin lesion", [4407])},
    "FirstChordOfAbnormalRegion": {"122452": ("First Chord of Abnormal Region", [])},
    "FirstMammogramEver": {"111404": ("First mammogram ever", [])},
    "FirstMeasureOfInformationCorrelationOfGLCM": {
        "128798": ("First Measure of Information Correlation of GLCM", [])
    },
    "FirstPassLeakageProfileFPLPModel": {
        "126343": ("First Pass Leakage Profile (FPLP) Model", [4101, 4106])
    },
    "FixationDeviceAngle": {"130658": ("Fixation Device Angle", [])},
    "FixationOrPositioningDevice": {
        "130044": ("Fixation or Positioning Device", [9502, 9580])
    },
    "FixedBeamDirection": {"125257": ("Fixed beam direction", [12034])},
    "FixedDp": {"113267": ("Fixed-Dp", [7274])},
    "FixedDurationPauseEnded": {"130155": ("Fixed duration pause ended", [71])},
    "FixedGrid": {"111641": ("Fixed grid", [10017])},
    "FixedLaserSetupPoint": {"130360": ("Fixed Laser Setup Point", [9554])},
    "FlashDoesnTIncludeCassetteScreenDetectorIdentification": {
        "111191": (
            "Flash doesn't include cassette/screen/detector identification",
            [6041],
        )
    },
    "FlashDoesnTIncludeDateOfExamination": {
        "111188": ("Flash doesn't include date of examination", [6041])
    },
    "FlashDoesnTIncludeFacilityNameAndLocation": {
        "111189": ("Flash doesn't include facility name and location", [6041])
    },
    "FlashDoesnTIncludeMammographyUnitIdentification": {
        "111192": ("Flash doesn't include mammography unit identification", [6041])
    },
    "FlashDoesnTIncludePatientNameAndAdditionalPatientId": {
        "111187": (
            "Flash doesn't include patient name and additional patient id",
            [6041],
        )
    },
    "FlashDoesnTIncludeTechnologistIdentification": {
        "111190": ("Flash doesn't include technologist identification", [6041])
    },
    "FlashIsIllegibleDoesNotFitOrIsLopsided": {
        "111186": ("Flash is illegible, does not fit, or is lopsided", [6041])
    },
    "FlashIsNotNearEdgeOfFilm": {"111185": ("Flash is not near edge of film", [6041])},
    "FlatFilter": {"113653": ("Flat filter", [10007])},
    "FlatteningFilterBeam": {"130355": ("Flattening Filter Beam", [9549])},
    "FleischTable": {"122254": ("Fleisch table", [3664])},
    "FleischnerLines": {"112107": ("Fleischner's line(s)", [6102, 6103])},
    "Flocculent": {"112164": ("Flocculent", [6132])},
    "FlorbetabenF18": {"126501": ("Florbetaben F^18^", [])},
    "FlowImageForImageProcessing": {
        "128251": ("Flow image for image processing", [7202])
    },
    "FlowQuantification": {"122604": ("Flow Quantification", [])},
    "FlowRateAboveAdjustmentLimit": {
        "130153": ("Flow rate above adjustment limit", [71])
    },
    "FlowRateAboveWarningLimit": {"130152": ("Flow rate above warning limit", [71])},
    "FlowRateVsTime": {"130229": ("Flow Rate vs Time", [])},
    "FlowVariance": {"110829": ("Flow Variance", [218, 7180, 7469])},
    "FlowVelocity": {"110828": ("Flow Velocity", [218, 7180, 7469, 12119, 12120])},
    "FlubatineF18": {"126503": ("Flubatine F^18^", [4021])},
    "FlubatineF18_Retired": {"126712": ("Flubatine F^18^", [])},
    "Fluffy": {"112149": ("Fluffy", [6123])},
    "FluidObtained": {"111449": ("Fluid obtained", [6066])},
    "FluoresceinAngiography": {"FA": ("Fluorescein angiography", [])},
    "FluoroDoseAreaProductTotal": {
        "113726": ("Fluoro Dose Area Product Total", [10050])
    },
    "FluoroDoseRPTotal": {"113728": ("Fluoro Dose (RP) Total", [])},
    "FluoroMode": {"113732": ("Fluoro Mode", [])},
    "FluoromisonidazoleFMISOF18": {"126708": ("Fluoromisonidazole (FMISO) F^18^", [])},
    "FluoroscopyGuidedProjectionRadiographySystem": {
        "113957": ("Fluoroscopy-Guided Projection Radiography System", [10032])
    },
    "FluorotrioprideF18": {"126707": ("Fluorotriopride F^18^", [4021])},
    "FlurpiridazF18": {"126718": ("Flurpiridaz F^18^", [4021])},
    "FlushAdministration": {"130251": ("Flush Administration", [72])},
    "FocalAbnormality": {"130591": ("Focal abnormality", [6335, 6336, 6337])},
    "FocalContrastEnhancement": {
        "130604": ("Focal Contrast Enhancement", [6335, 6345, 6346])
    },
    "FocalPlaneZOffset": {"112708": ("Focal plane Z offset", [])},
    "FocalSpotSize": {"113766": ("Focal Spot Size", [])},
    "FocusedGrid": {"111642": ("Focused grid", [10017])},
    "FollowUpAtShortInterval111Months": {
        "111142": ("Follow-up at short interval (1-11 months)", [6028, 6029])
    },
    "FollowUpAtShortIntervalFromPriorStudy": {
        "111416": ("Follow-up at short interval from prior study", [6051])
    },
    "FollowUpPostBiopsyAsDirectedByClinician": {
        "111121": ("Follow-up post biopsy as directed by clinician", [6028])
    },
    "FoodTreat": {"127391": ("Food treat", [609])},
    "FootToHead": {"110869": ("Foot To Head", [7185])},
    "ForAttenuationCorrection": {"122403": ("For Attenuation Correction", [7210])},
    "ForClinicalTrialExport": {"128711": ("For Clinical Trial Export", [7010, 7014])},
    "ForClinicalTrialSubmission": {"128207": ("For Clinical Trial Submission", [7010])},
    "ForConference": {"113005": ("For Conference", [7010])},
    "ForDiagnosis": {"128195": ("For Diagnosis", [7010])},
    "ForLitigation": {"113021": ("For Litigation", [7010])},
    "ForMachineQualityAssurance": {
        "128205": ("For Machine Quality Assurance", [7010, 7024])
    },
    "ForPatient": {"113007": ("For Patient", [7010])},
    "ForPatientSetupVerification": {
        "128206": ("For Patient Setup Verification", [7010, 7024])
    },
    "ForPeerReview": {"113008": ("For Peer Review", [7010])},
    "ForPhysicianReview": {"128201": ("For Physician Review", [7010, 7024])},
    "ForPhysicistReview": {"128202": ("For Physicist Review", [7010, 7024])},
    "ForPlanComparison": {"128199": ("For Plan Comparison", [7010])},
    "ForPlanQualityAssurance": {"128204": ("For Plan Quality Assurance", [7010, 7024])},
    "ForPrinting": {"113018": ("For Printing", [7010])},
    "ForProcessingPredecessor": {"121358": ("For Processing predecessor", [7202])},
    "ForPublicationExport": {"128714": ("For Publication Export", [7010, 7014])},
    "ForRTPlanSummation": {"128200": ("For RT Plan Summation", [7010, 7024])},
    "ForRTPrescription": {"128197": ("For RT Prescription", [7010, 7024])},
    "ForRTTreatmentPlanning": {"128198": ("For RT Treatment Planning", [7010, 7024])},
    "ForRTWorkflow": {"128180": ("For RT Workflow", [7010, 7024])},
    "ForReferringProvider": {"113002": ("For Referring Provider", [7010])},
    "ForReportAttachment": {"113020": ("For Report Attachment", [7010])},
    "ForResearch": {"113009": ("For Research", [7010])},
    "ForResearchCollectionExport": {
        "128713": ("For Research Collection Export", [7010, 7014])
    },
    "ForSegmentation": {"128196": ("For Segmentation", [7010])},
    "ForSurgery": {"113003": ("For Surgery", [7010])},
    "ForTeaching": {"113004": ("For Teaching", [7010])},
    "ForTeachingFileExport": {"128710": ("For Teaching File Export", [7010, 7014])},
    "ForTherapy": {"113006": ("For Therapy", [7010])},
    "ForTumorBoard": {"128203": ("For Tumor Board", [7010])},
    "ForTumorRegistry": {"128208": ("For Tumor Registry", [7010])},
    "ForcedAirHeater": {"127250": ("Forced air heater", [635])},
    "ForeignBody": {"111265": ("Foreign body (reaction)", [])},
    "FovealSensitivityRelated": {"111841": ("Foveal sensitivity related", [4252])},
    "Fp10": {"130711": ("Fp10", [3030])},
    "Fp9": {"130710": ("Fp9", [3030])},
    "FractalDimension": {"126050": ("Fractal Dimension", [218, 7468])},
    "FractionalAnisotropy": {
        "110808": ("Fractional Anisotropy", [218, 7180, 7263, 7271, 7469])
    },
    "FractionalChange": {"125314": ("Fractional Change", [12303])},
    "FractionalKurtosisAnisotropy": {
        "113209": ("Fractional Kurtosis Anisotropy", [7263])
    },
    "FractionalOccupancySegmentation": {
        "110855": ("Fractional Occupancy Segmentation", [218, 7180, 7469])
    },
    "FractionalProbabilisticSegmentation": {
        "110854": ("Fractional Probabilistic Segmentation", [218, 7180, 7469])
    },
    "FractionationBasedOrTemporallyBasedMethods": {
        "130127": ("Fractionation-based or temporally-based methods", [9537])
    },
    "FrameExtractingEquipment": {"109105": ("Frame Extracting Equipment", [7005])},
    "FrameOfReferenceIdentity": {"125021": ("Frame of Reference Identity", [7100])},
    "FrameOfReferenceUID": {"112227": ("Frame of Reference UID", [])},
    "FrameToFrameAnalysis": {"122499": ("Frame to Frame Analysis", [])},
    "FraminghamQtcAlgorithm": {"122733": ("Framingham QTc Algorithm", [3678])},
    "FreeAcquisition": {"113807": ("Free Acquisition", [10013])},
    "FreeSilicone": {"111508": ("Free silicone", [6072])},
    "Freehand": {"114215": ("Freehand", [8203])},
    "FrequencyOfStimulusEvents": {"130495": ("Frequency of Stimulus Events", [])},
    "Fresolimumab89Zr": {"126748": ("Fresolimumab ^89^Zr", [4021])},
    "FridericiaQtcAlgorithm": {"122732": ("Fridericia QTc Algorithm", [3678])},
    "FrontOfCorneaToFrontOfLens": {
        "111776": ("Front Of Cornea To Front Of Lens", [4239])
    },
    "FullBernoulli": {"125217": ("Full Bernoulli", [12227, 12230])},
    "FullCardiacCycle": {"125324": ("Full Cardiac Cycle", [12307])},
    "FullFidelityImage": {"121327": ("Full fidelity image", [7205])},
    "FunctionalConditionPresentDuringAcquisition": {
        "130324": ("Functional condition present during acquisition", [])
    },
    "FunctionalImage": {"121339": ("Functional image", [7201])},
    "Fundoscopy": {"FS": ("Fundoscopy", [])},
    "GA20189Zr": {"126731": ("GA201 ^89^Zr", [4021])},
    "GDPattern": {"109903": ("GD Pattern", [8301])},
    "GGraft": {"130734": ("G - Graft", [3022])},
    "GTVNodal": {"130051": ("GTV Nodal", [9534])},
    "GTVPrimary": {"130052": ("GTV Primary", [9534])},
    "Galactocele": {"111269": ("Galactocele", [])},
    "GammaCapillaryTransitTimeGCCTModel": {
        "126345": ("Gamma Capillary Transit Time (GCCT) Model", [])
    },
    "GammaCapillaryTransitTimeGCTTModel": {
        "126345": ("Gamma Capillary Transit Time (GCTT) Model", [4101, 4106])
    },
    "GammaDistributionMode": {
        "113297": ("Gamma Distribution Mode", [218, 7180, 7272, 7469])
    },
    "GammaDistributionModel": {"113253": ("Gamma distribution model", [7273])},
    "GammaDistributionScaleParameter": {
        "113295": ("Gamma Distribution Scale Parameter", [218, 7180, 7272, 7469])
    },
    "GammaDistributionShapeParameter": {
        "113296": ("Gamma Distribution Shape Parameter", [218, 7180, 7272, 7469])
    },
    "GanglionCellComplexThickness": {
        "111926": ("Ganglion cell complex thickness", [4262])
    },
    "GastrointestinalImagingSubjectMatter": {
        "128731": ("Gastrointestinal imaging subject matter", [7017])
    },
    "GaussianBlur": {"113088": ("Gaussian blur", [7203])},
    "GaussianFilter": {"130764": ("Gaussian filter", [3042])},
    "GeneralMicroscopy": {"GM": ("General Microscopy", [29, 30, 33])},
    "GeneralReductionInSensitivity": {
        "111850": ("General reduction in sensitivity", [4254])
    },
    "Generator": {"113543": ("Generator", [])},
    "Generic2DPlanning": {"112340": ("Generic 2D Planning", [7320])},
    "Generic3DPlanning": {"112341": ("Generic 3D Planning", [7320])},
    "GenericPlanningForHipReplacement": {
        "112342": ("Generic Planning for Hip Replacement", [7320])
    },
    "GenericPlanningForKneeReplacement": {
        "112343": ("Generic Planning for Knee Replacement", [7320])
    },
    "GeneticModifications": {"127414": ("Genetic modifications", [])},
    "GeneticModificationsDescription": {
        "127415": ("Genetic modifications description", [])
    },
    "GenitourinaryHistory": {"130548": ("Genitourinary History", [])},
    "GenitourinaryImagingSubjectMatter": {
        "128732": ("Genitourinary imaging subject matter", [7017])
    },
    "GeometricCenterpoint": {"128137": ("Geometric Centerpoint", [219, 1011])},
    "GeometricIsocenter": {"122486": ("Geometric Isocenter", [3452])},
    "GeometricNonIsocenter": {"122487": ("Geometric Non-Isocenter", [3452])},
    "GeometricPurposeOfRegion": {"130400": ("Geometric purpose of region", [])},
    "GeometricRadiationTransportModel": {
        "128421": ("Geometric Radiation Transport Model", [10065])
    },
    "GeometricalImageEvaluation": {"109708": ("Geometrical image evaluation", [8300])},
    "GlaucomaHemifieldTestAnalysis": {
        "111855": ("Glaucoma Hemifield Test Analysis", [4257])
    },
    "GlembatumumabVedotin89Zr": {"126724": ("Glembatumumab vedotin ^89^Zr", [4021])},
    "Global": {"113213": ("Global", [7262])},
    "GlobalTrackingAlgorithm": {"113213": ("Global Tracking Algorithm", [])},
    "GlomerularFiltrationRateBlackMDRD": {
        "113574": ("Glomerular Filtration Rate black (MDRD)", [10047])
    },
    "GlomerularFiltrationRateCreatinineBasedFormulaSchwartz": {
        "113577": (
            "Glomerular Filtration Rate Creatinine-based formula (Schwartz)",
            [10047],
        )
    },
    "GlomerularFiltrationRateCystatinBasedFormula": {
        "113576": ("Glomerular Filtration Rate Cystatin-based formula", [10047])
    },
    "GlomerularFiltrationRateFemaleMDRD": {
        "113575": ("Glomerular Filtration Rate female (MDRD)", [10047])
    },
    "GlomerularFiltrationRateMDRD": {
        "113572": ("Glomerular Filtration Rate (MDRD)", [10047])
    },
    "GlomerularFiltrationRateNonBlackMDRD": {
        "113573": ("Glomerular Filtration Rate non-black (MDRD)", [10047])
    },
    "GlucoseMeasurementDate": {"127857": ("Glucose Measurement Date", [])},
    "GlucoseMeasurementTime": {"127858": ("Glucose Measurement Time", [])},
    "GlutamateAndGlutamine": {
        "113080": ("Glutamate and glutamine", [218, 4032, 4033, 7180, 7186, 7469])
    },
    "GlutamineC11": {"126709": ("Glutamine C^11^", [4021])},
    "GlutamineC14": {"126710": ("Glutamine C^14^", [4021])},
    "GlutamineF18": {"126711": ("Glutamine F^18^", [4021])},
    "Glycolysis": {"126034": ("Glycolysis", [7466])},
    "GoodImageQuality": {"122741": ("Good image quality", [3114])},
    "GradientAssessment": {"122123": ("Gradient assessment", [])},
    "GradientPressureAverage": {"122197": ("Gradient pressure, average", [3611])},
    "GradientPressurePeak": {"122198": ("Gradient pressure, peak", [3611])},
    "GradientWeightedClassActivation": {
        "130403": ("Gradient-weighted class activation", [217, 218, 7180, 7469])
    },
    "GraftToCitedSegmentDistalSection": {
        "122104": ("Graft to cited segment, distal section", [3019])
    },
    "GraftToCitedSegmentMidSection": {
        "122103": ("Graft to cited segment, mid section", [3019])
    },
    "GraftToCitedSegmentProximalSection": {
        "122102": ("Graft to cited segment, proximal section", [3019])
    },
    "GraftType": {"125102": ("Graft Type", [])},
    "GranularPattern": {"112128": ("Granular pattern", [6102, 6106])},
    "GraphIncrement": {"122511": ("Graph Increment", [])},
    "Graticule": {"130349": ("Graticule", [9548])},
    "GrayLevelCoOccurrenceMatrix": {"126067": ("Gray Level Co-occurrence Matrix", [])},
    "GrayLevelDistanceZoneMatrix": {"128778": ("Gray Level Distance Zone Matrix", [])},
    "GrayLevelNonuniformityInRuns": {
        "128809": ("Gray Level Nonuniformity in Runs", [])
    },
    "GrayLevelNonuniformityInRunsNormalized": {
        "128810": ("Gray Level Nonuniformity in Runs Normalized", [])
    },
    "GrayLevelNonuniformityOfZoneCounts": {
        "128829": ("Gray Level Nonuniformity of Zone Counts", [])
    },
    "GrayLevelNonuniformityOfZoneCountsNormalized": {
        "128830": ("Gray Level Nonuniformity of Zone Counts Normalized", [])
    },
    "GrayLevelRunLengthMatrix": {"128776": ("Gray Level Run Length Matrix", [])},
    "GrayLevelSizeZoneMatrix": {"128777": ("Gray Level Size Zone Matrix", [])},
    "GrayLevelVarianceInRuns": {"128814": ("Gray Level Variance in Runs", [])},
    "GrayLevelVarianceInZones": {"128834": ("Gray Level Variance in Zones", [])},
    "GrayscaleResolutionEvaluation": {
        "109702": ("Grayscale resolution evaluation", [8300])
    },
    "GreaterThan1YearAgo": {"111398": ("> 1 year ago", [6164])},
    "GreenFilter": {"111601": ("Green filter", [])},
    "GridArtifacts": {"111208": ("Grid artifact(s)", [6041, 6135, 7011])},
    "GridBScanPattern": {"128288": ("Grid B-scan pattern", [4272])},
    "GrossSpecimenImaging": {"112701": ("Gross Specimen Imaging", [])},
    "GroundGlassOpacity": {"112120": ("Ground glass opacity", [6102, 6104])},
    "GroupOfFramesForDisplay": {"113036": ("Group of Frames for Display", [7010])},
    "GrowthPercentileRank": {"125012": ("Growth Percentile Rank", [12017])},
    "GrowthZScore": {"125013": ("Growth Z-score", [12017])},
    "GuidewireCrossingLesionSuccessful": {
        "122302": ("Guidewire crossing lesion successful", [3412, 3752])
    },
    "GuidewireCrossingLesionUnsuccessful": {
        "122301": ("Guidewire crossing lesion unsuccessful", [3412, 3752])
    },
    "GynecologicCondition": {"111567": ("Gynecologic condition", [6087, 6088])},
    "GynecologicSurgery": {"111568": ("Gynecologic surgery", [6087, 6088])},
    "HARDI": {"113221": ("HARDI", [7260])},
    "HISBundleWave": {"109040": ("HIS bundle wave", [3339])},
    "HabitualVisualAcuity": {"111686": ("Habitual Visual Acuity", [4216])},
    "Haigis": {"111760": ("Haigis", [4236])},
    "HaigisA0": {"111769": ("Haigis a0", [4237])},
    "HaigisA1": {"111770": ("Haigis a1", [4237])},
    "HaigisA2": {"111771": ("Haigis a2", [4237])},
    "HaigisL": {"111761": ("Haigis-L", [4236])},
    "HaigisLToric": {"111861": ("Haigis-L Toric", [4236])},
    "HaigisToric": {"111860": ("Haigis Toric", [4236])},
    "HalfTimeForTissueRepair": {"130091": ("Half-time for Tissue Repair", [])},
    "HalfValueLayer": {"111634": ("Half Value Layer", [10069])},
    "HaloSign": {"112073": ("Halo sign", [6102, 6105])},
    "HandGrips": {"130854": ("Hand Grips", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "HardCopy": {"HC": ("Hard Copy", [32, 33])},
    "HardCopyPrintServer": {"PRINT": ("Hard Copy Print Server", [30])},
    "HardWedge": {"130346": ("Hard Wedge", [9546])},
    "HardwareConfiguration": {"110130": ("Hardware Configuration", [401, 403])},
    "HeadAndNeckImagingSpecialty": {
        "128008": ("Head and Neck Imaging Specialty", [7449])
    },
    "HeadAndNeckMask": {
        "130112": ("Head and Neck Mask", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "HeadFixationBoard": {
        "130656": ("Head Fixation Board", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "HeadFootAxis": {"110861": ("Head-Foot Axis", [7184])},
    "HeadMask": {"130111": ("Head Mask", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "HeadNodeSet": {"130362": ("Head Node Set", [9556])},
    "HeadOfCardiology": {"128675": ("Head of Cardiology", [7450, 7452])},
    "HeadOfRadiology": {"128670": ("Head of Radiology", [7450, 7452])},
    "HeadToFoot": {"110868": ("Head To Foot", [7185])},
    "Headframe": {"130110": ("Headframe", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "HeadframePresent": {"130460": ("Headframe Present", [9564])},
    "HeartValveFlail": {"127856": ("Heart Valve Flail", [3711])},
    "HeatedImagingDevice": {"127251": ("Heated imaging device", [635])},
    "HeatedPatientSupport": {"127252": ("Heated patient support", [635])},
    "HeatedWaterBlanket": {"127253": ("Heated water blanket", [635])},
    "HeatingConditions": {"127040": ("Heating conditions", [])},
    "HeelStop": {"130853": ("Heel Stop", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "Height": {"121207": ("Height", [218, 6165, 7469, 7470])},
    "HelicalBeam": {"130108": ("Helical Beam", [9512, 9524, 9583])},
    "HemangiomaNonparenchymalSubcutaneous": {
        "111271": ("Hemangioma - nonparenchymal, subcutaneous", [])
    },
    "HemisphericalUltrasoundTransducerGeometry": {
        "130809": ("Hemispherical ultrasound transducer geometry", [12033])
    },
    "HemodynamicResistance": {"110825": ("Hemodynamic Resistance", [218, 7180, 7469])},
    "HemodynamicResistanceIndex": {"122220": ("Hemodynamic Resistance Index", [3620])},
    "HemodynamicWaveform": {"HD": ("Hemodynamic Waveform", [29, 30, 33])},
    "HemodynamicsReport": {"122120": ("Hemodynamics Report", [])},
    "HemostasisAchieved": {"122033": ("Hemostasis achieved", [3402])},
    "HemostasisNotAchievedActivelyBleeding": {
        "122035": ("Hemostasis not achieved - actively bleeding", [3402])
    },
    "HemostasisNotAchievedOozing": {
        "122034": ("Hemostasis not achieved - oozing", [3402])
    },
    "Hermaphrodite": {"H": ("Hermaphrodite", [7455])},
    "HerniatedImplant": {"111509": ("Herniated implant", [6072])},
    "HeterogeneousEchotexture": {"111353": ("Heterogeneous echotexture", [6151])},
    "Hiatus": {"112095": ("Hiatus", [6102, 6109, 6113, 6148, 7151, 7192, 9514])},
    "High": {"111575": ("High", [6090])},
    "HighDoseFractionLinearCellKillFactor": {
        "130090": ("High Dose Fraction Linear Cell Kill Factor", [])
    },
    "HighDoseFractionTransitionDose": {
        "130092": ("High Dose Fraction Transition Dose", [])
    },
    "HighGrayLevelRunEmphasis": {"128804": ("High Gray Level Run Emphasis", [])},
    "HighGrayLevelZoneEmphasis": {"128824": ("High Gray Level Zone Emphasis", [])},
    "HighResolution": {"114210": ("High resolution", [8202])},
    "HighThresholdEmptyTileSuppression": {
        "112720": ("High threshold empty tile suppression", [8133])
    },
    "HighThyroidUptake": {"113562": ("High Thyroid Uptake", [64, 10045])},
    "HighlySuggestiveOfMalignancyTakeAppropriateAction": {
        "111149": (
            "Highly suggestive of malignancy - take appropriate action",
            [6028, 6029],
        )
    },
    "HisBundleElectrogram": {"109007": ("His bundle electrogram", [3240])},
    "HistogramAnalysis": {"123105": ("Histogram Analysis", [7162])},
    "HistologyUsingCoreBiopsy": {
        "111145": ("Histology using core biopsy", [6028, 6029])
    },
    "HistoricalRTPrescription": {"130135": ("Historical RT Prescription", [9509])},
    "History": {"121060": ("History", [])},
    "HistoryOfBenignBreastBiopsy": {
        "111420": ("History of benign breast biopsy", [6051])
    },
    "HistoryOfBreastAugmentationAsymptomatic": {
        "111417": ("History of breast augmentation, asymptomatic", [6051])
    },
    "HistoryOfEndometrialCancer": {
        "111551": ("History of endometrial cancer", [6081, 6087])
    },
    "HistoryOfHighRiskLesionOnPreviousBiopsy": {
        "111553": ("History of high risk lesion on previous biopsy", [6081, 6087])
    },
    "HistoryOfMultipleFetuses": {
        "111572": ("History of multiple fetuses", [6087, 6088])
    },
    "HistoryOfNonMelanomaSkinCancer": {
        "130482": ("History of non-melanoma skin cancer", [])
    },
    "HistoryOfOvarianCancer": {"111552": ("History of ovarian cancer", [6081, 6087])},
    "HodgesQTCorrectionAlgorithm": {
        "122731": ("Hodges QT Correction Algorithm", [3678])
    },
    "HodgesQtcAlgorithm": {"122731": ("Hodges QTc Algorithm", [])},
    "HofferPacdConstant": {"111772": ("Hoffer pACD Constant", [4237])},
    "HofferQ": {"111764": ("Hoffer Q", [4236])},
    "Holladay1": {"111762": ("Holladay 1", [4236])},
    "Holladay2": {"111763": ("Holladay 2", [4236])},
    "HomogeneityFactor": {"128455": ("Homogeneity Factor", [10069])},
    "Homogeneous": {"112160": ("Homogeneous", [6131, 6335, 6343, 6344])},
    "HomogeneousFatEchotexture": {"111351": ("Homogeneous fat echotexture", [6151])},
    "HomogeneousFibroglandularEchotexture": {
        "111352": ("Homogeneous fibroglandular echotexture", [6151])
    },
    "HoneycombPattern": {"112106": ("Honeycomb pattern", [6102, 6103, 6106])},
    "HorizontalPixelSpacing": {"111026": ("Horizontal Pixel Spacing", [])},
    "HorizontalPrismBase": {"111676": ("Horizontal Prism Base", [])},
    "HorizontalPrismPower": {"111675": ("Horizontal Prism Power", [])},
    "HousingChangeInterval": {"127151": ("Housing change interval", [])},
    "HousingIndividuallyVentilated": {
        "127170": ("Housing individually ventilated", [])
    },
    "HousingManufacturer": {"127125": ("Housing manufacturer", [])},
    "HousingRackProductCode": {"127127": ("Housing rack product code", [])},
    "HousingRackProductName": {"127126": ("Housing rack product name", [])},
    "HousingRole": {"127110": ("Housing role", [])},
    "HousingUnitHeight": {"127161": ("Housing unit height", [])},
    "HousingUnitLength": {"127162": ("Housing unit length", [])},
    "HousingUnitLidProductCode": {"127131": ("Housing unit lid product code", [])},
    "HousingUnitLidProductName": {"127130": ("Housing unit lid product name", [])},
    "HousingUnitLocationInRack": {"127142": ("Housing unit location in rack", [])},
    "HousingUnitMovement": {"127153": ("Housing unit movement", [])},
    "HousingUnitProductCode": {"127129": ("Housing unit product code", [])},
    "HousingUnitProductName": {"127128": ("Housing unit product name", [])},
    "HousingUnitReuse": {"127175": ("Housing unit reuse", [])},
    "HousingUnitWidth": {"127160": ("Housing unit width", [])},
    "HumanAlphaSynucleinPreformedFibrils": {
        "127851": ("Human alpha synuclein preformed fibrils", [638, 640])
    },
    "HumanTauPreformedFibrils": {"127853": ("Human Tau preformed fibrils", [638, 640])},
    "HydrationVolume": {"113551": ("Hydration Volume", [])},
    "HyperAcute": {"112159": ("Hyper-acute", [6130])},
    "HyperFractionation": {"130099": ("Hyper-fractionation", [9533])},
    "Hyperechoic": {"111362": ("Hyperechoic", [6154])},
    "HyperemicRatio": {"122219": ("Hyperemic ratio", [3620])},
    "HyperplasiaUsual": {"111273": ("Hyperplasia, usual", [])},
    "HyperventilationBegin": {"130413": ("Hyperventilation begin", [3038])},
    "HyperventilationEnd": {"130414": ("Hyperventilation end", [3038])},
    "HypoFractionation": {"130098": ("Hypo-fractionation", [9533])},
    "Hypoechoic": {"111364": ("Hypoechoic", [6154])},
    "HzAngular": {"130775": ("Hz angular", [3044])},
    "HzNormalized": {"130774": ("Hz normalized", [3044])},
    "IAUC": {"126320": ("IAUC", [218, 4109, 7180, 7469])},
    "IAUC180": {"126323": ("IAUC180", [218, 4109, 7180, 7469])},
    "IAUC180BN": {"126327": ("IAUC180BN", [218, 4109, 7180, 7469])},
    "IAUC60": {"126321": ("IAUC60", [218, 4109, 7180, 7469])},
    "IAUC60BN": {"126325": ("IAUC60BN", [218, 4109, 7180, 7469])},
    "IAUC90": {"126322": ("IAUC90", [218, 4109, 7180, 7469])},
    "IAUC90BN": {"126326": ("IAUC90BN", [218, 4109, 7180, 7469])},
    "IAUCBN": {"126324": ("IAUCBN", [218, 4109, 7180, 7469])},
    "IAUCBN60": {"126325": ("IAUCBN60", [])},
    "IAUCBN90": {"126326": ("IAUCBN90", [])},
    "ICRPPub103": {"113841": ("ICRP Pub 103", [10015])},
    "ICRPPub60": {"113808": ("ICRP Pub 60", [10015])},
    "ICRPPublication106": {"113523": ("ICRP Publication 106", [10040])},
    "ICRPPublication53": {"113521": ("ICRP Publication 53", [10040])},
    "ICRPPublication80": {"113522": ("ICRP Publication 80", [10040])},
    "IDPlate": {"111174": ("ID Plate", [])},
    "IEC61217GantryContinuousPitchAngle": {
        "126810": ("IEC61217 Gantry Continuous Pitch Angle", [9401, 9402, 9570])
    },
    "IEC61217GantryContinuousRollAngle": {
        "126809": ("IEC61217 Gantry Continuous Roll Angle", [9401, 9402, 9570])
    },
    "IEC61217GantryContinuousYawAngle": {
        "126811": ("IEC61217 Gantry Continuous Yaw Angle", [9401, 9402, 9570])
    },
    "IEC61217ImagingSourceToAxisDistance": {
        "130801": ("IEC61217 Imaging Source to Axis Distance", [])
    },
    "IEC61217PatientSupportContinuousYawAngle": {
        "126801": ("IEC61217 Patient Support Continuous Yaw Angle", [9401, 9403, 9570])
    },
    "IEC61217TableTopContinuousEccentricAngle": {
        "126805": ("IEC61217 Table Top Continuous Eccentric Angle", [9401, 9403, 9570])
    },
    "IEC61217TableTopContinuousPitchAngle": {
        "126802": ("IEC61217 Table Top Continuous Pitch Angle", [9401, 9403, 9570])
    },
    "IEC61217TableTopContinuousRollAngle": {
        "126803": ("IEC61217 Table Top Continuous Roll Angle", [9401, 9403, 9570])
    },
    "IEC61217TableTopEccentricAxisDistance": {
        "126804": ("IEC61217 Table Top Eccentric Axis Distance", [9401, 9403, 9570])
    },
    "IEC61217TableTopLateralPosition": {
        "126806": ("IEC61217 Table Top Lateral Position", [9401, 9403, 9570])
    },
    "IEC61217TableTopLongitudinalPosition": {
        "126807": ("IEC61217 Table Top Longitudinal Position", [9401, 9403, 9570])
    },
    "IEC61217TableTopVerticalPosition": {
        "126808": ("IEC61217 Table Top Vertical Position", [9401, 9403, 9570])
    },
    "IEC61217XRayImageReceptorLateralDisplacement": {
        "130804": ("IEC61217 X-Ray Image Receptor Lateral Displacement", [])
    },
    "IEC61217XRayImageReceptorLongitudinalDisplacement": {
        "130803": ("IEC61217 X-Ray Image Receptor Longitudinal Displacement", [])
    },
    "IEC61217XRayImageReceptorRadialDisplacementFromIsocenter": {
        "130802": (
            "IEC61217 X-Ray Image Receptor Radial Displacement from Isocenter",
            [],
        )
    },
    "IEC61217XRayImageReceptorRotation": {
        "130805": ("IEC61217 X-Ray Image Receptor Rotation", [])
    },
    "IECBodyDosimetryPhantom": {"113691": ("IEC Body Dosimetry Phantom", [4051, 4052])},
    "IECHeadDosimetryPhantom": {"113690": ("IEC Head Dosimetry Phantom", [4051, 4052])},
    "IIRFilter": {"130772": ("IIR filter", [3043])},
    "ILCR": {"126850": ("ILCR", [7490])},
    "IRIGBTimecode": {"109003": ("IRIG-B timecode", [3090])},
    "ITV": {"130056": ("ITV", [9534])},
    "IVUSReport": {"122325": ("IVUS Report", [])},
    "IdentifiableEffectOnSurroundingTissues": {
        "111371": ("Identifiable effect on surrounding tissues", [])
    },
    "IdentificationOfTheAttenuator": {
        "130527": ("Identification of the Attenuator", [])
    },
    "IdentificationOfTheXRaySource": {
        "113832": ("Identification of the X-Ray Source", [])
    },
    "Identifier": {"125010": ("Identifier", [])},
    "IdentifierWithinPersonObserverRole": {
        "128775": ("Identifier within Person Observer's Role", [])
    },
    "IdentifyingSegment": {"112229": ("Identifying Segment", [])},
    "IfAgeGreaterThan60YearsP50Equals29Point3": {
        "122261": ("if (age > 60y) P50 = 29.3", [3666])
    },
    "IfAgeLesserThan18YearsP50Equals27Point2": {
        "122259": ("if (age < 18y) P50 = 27.2", [3666])
    },
    "IfAgeLesserThan1DayP50Equals21Point6": {
        "122257": ("if (age < 1day) P50 = 21.6", [3666])
    },
    "IfAgeLesserThan30DayP50Equals24Point6": {
        "122258": ("if (age < 30day) P50 = 24.6", [3666])
    },
    "IfAgeLesserThan40YearsP50Equals27Point4": {
        "122260": ("if (age < 40y) P50 = 27.4", [3666])
    },
    "IfPremAgeLesserThan3DaysP50Equals19Point9": {
        "122256": ("if (prem age< 3days) P50 = 19.9", [3666])
    },
    "Igloo": {"127220": ("Igloo", [606])},
    "IlluminationMethod": {"112706": ("Illumination Method", [])},
    "IlluminationSpectralBand": {"112711": ("Illumination spectral band", [])},
    "IlluminationWavelength": {"112710": ("Illumination wavelength", [])},
    "IllustrationOfFinding": {"125201": ("Illustration of Finding", [])},
    "IllustrationOfROI": {"121200": ("Illustration of ROI", [7003])},
    "ImageAcquired": {"121138": ("Image Acquired", [])},
    "ImageAcquiredDuringTreatment": {
        "129203": ("Image Acquired during Treatment", [7201])
    },
    "ImageArtifacts": {"111207": ("Image artifact(s)", [7011])},
    "ImageCapture": {"CAPTURE": ("Image Capture", [30])},
    "ImageContentAndFiducialBasedAlignment": {
        "125026": ("Image Content and Fiducial Based Alignment", [7100])
    },
    "ImageContentBasedAlignment": {"125024": ("Image Content-based Alignment", [7100])},
    "ImageDetectedMass": {"111126": ("Image detected mass", [6055])},
    "ImageDetector": {"130351": ("Image Detector", [9548])},
    "ImageFrame": {"130781": ("Image frame", [272])},
    "ImageInterval": {"122640": ("Image Interval", [])},
    "ImageLaterality": {"111027": ("Image Laterality", [])},
    "ImageLibrary": {"111028": ("Image Library", [])},
    "ImageLibraryGroup": {"126200": ("Image Library Group", [])},
    "ImageOrientationPatientColumnX": {
        "110907": ("Image Orientation (Patient) Column X", [])
    },
    "ImageOrientationPatientColumnY": {
        "110908": ("Image Orientation (Patient) Column Y", [])
    },
    "ImageOrientationPatientColumnZ": {
        "110909": ("Image Orientation (Patient) Column Z", [])
    },
    "ImageOrientationPatientRowX": {
        "110904": ("Image Orientation (Patient) Row X", [])
    },
    "ImageOrientationPatientRowY": {
        "110905": ("Image Orientation (Patient) Row Y", [])
    },
    "ImageOrientationPatientRowZ": {
        "110906": ("Image Orientation (Patient) Row Z", [])
    },
    "ImagePositionPatientX": {"110901": ("Image Position (Patient) X", [])},
    "ImagePositionPatientY": {"110902": ("Image Position (Patient) Y", [])},
    "ImagePositionPatientZ": {"110903": ("Image Position (Patient) Z", [])},
    "ImageProcessing": {"110001": ("Image Processing", [9231])},
    "ImageQuality": {"111101": ("Image Quality", [6014, 6016, 6054, 6101, 6201])},
    "ImageQualityController": {"129003": ("Image Quality Controller", [7453])},
    "ImageQualityRating": {"111029": ("Image Quality Rating", [])},
    "ImageRegion": {"111030": ("Image Region", [])},
    "ImageSetProperties": {"112224": ("Image Set Properties", [])},
    "ImageSetQualityRating": {"111694": ("Image Set Quality Rating", [])},
    "ImageSpecificity": {"111091": ("Image Specificity", [6048])},
    "ImageStitching": {"113090": ("Image stitching", [7203])},
    "ImageType": {"121141": ("Image Type", [])},
    "ImageUsedAsReferenceImageForTreatment": {
        "129204": ("Image used as Reference Image for Treatment", [7201])
    },
    "ImageUsedForDoseCalculation": {
        "129202": ("Image used for Dose Calculation", [7201])
    },
    "ImageUsedForTreatmentPlanning": {
        "129201": ("Image used for Treatment Planning", [7201])
    },
    "ImageView": {"111031": ("Image View", [])},
    "ImageViewModifier": {"111032": ("Image View Modifier", [])},
    "ImagedContainerLabel": {"121354": ("Imaged container label", [8120])},
    "ImagesRelatedToStandaloneObject": {
        "121316": ("Images related to standalone object", [7201])
    },
    "Imagify": {"125903": ("Imagify", [12030])},
    "ImagingAgentAdministrationActivity": {
        "130237": ("Imaging Agent Administration Activity", [])
    },
    "ImagingAgentAdministrationAdverseEvents": {
        "130212": ("Imaging Agent Administration Adverse Events", [])
    },
    "ImagingAgentAdministrationCompletionStatus": {
        "130211": ("Imaging Agent Administration Completion Status", [])
    },
    "ImagingAgentAdministrationConsumable": {
        "130222": ("Imaging Agent Administration Consumable", [])
    },
    "ImagingAgentAdministrationConsumableType": {
        "130223": ("Imaging Agent Administration Consumable Type", [])
    },
    "ImagingAgentAdministrationDelay": {
        "130197": ("Imaging Agent Administration Delay", [])
    },
    "ImagingAgentAdministrationGraph": {
        "130232": ("Imaging Agent Administration Graph", [])
    },
    "ImagingAgentAdministrationInjectorEventType": {
        "130234": ("Imaging Agent Administration Injector Event Type", [])
    },
    "ImagingAgentAdministrationInjectorEvents": {
        "130233": ("Imaging Agent Administration Injector Events", [])
    },
    "ImagingAgentAdministrationInjectorPhaseIdentifier": {
        "130264": ("Imaging Agent Administration Injector Phase Identifier", [])
    },
    "ImagingAgentAdministrationPerformedPhaseUID": {
        "130261": ("Imaging Agent Administration Performed Phase UID", [])
    },
    "ImagingAgentAdministrationPerformedStepUID": {
        "130246": ("Imaging Agent Administration Performed Step UID", [])
    },
    "ImagingAgentAdministrationPhase": {
        "130202": ("Imaging Agent Administration Phase", [])
    },
    "ImagingAgentAdministrationPhaseIdentifier": {
        "130203": ("Imaging Agent Administration Phase Identifier", [])
    },
    "ImagingAgentAdministrationPhaseType": {
        "130204": ("Imaging Agent Administration Phase Type", [])
    },
    "ImagingAgentAdministrationPhaseWithManualHold": {
        "130265": ("Imaging Agent Administration Phase with Manual Hold", [])
    },
    "ImagingAgentAdministrationProtocolName": {
        "130200": ("Imaging Agent Administration Protocol Name", [])
    },
    "ImagingAgentAdministrationStep": {
        "130195": ("Imaging Agent Administration Step", [])
    },
    "ImagingAgentAdministrationStepIdentifier": {
        "130196": ("Imaging Agent Administration Step Identifier", [])
    },
    "ImagingAgentAdministrationStepSequenceNumber": {
        "130445": ("Imaging Agent Administration Step Sequence Number", [])
    },
    "ImagingAgentAdministrationSteps": {
        "130192": ("Imaging Agent Administration Steps", [])
    },
    "ImagingAgentAdministrationStepsDescription": {
        "130199": ("Imaging Agent Administration Steps Description", [])
    },
    "ImagingAgentComponent": {"130238": ("Imaging Agent Component", [])},
    "ImagingAgentComponentUsage": {"130191": ("Imaging Agent Component Usage", [])},
    "ImagingAgentIdentifier": {"130254": ("Imaging Agent Identifier", [])},
    "ImagingAgentInformation": {"130183": ("Imaging Agent Information", [])},
    "ImagingAgentVolumePerUnitOfPresentation": {
        "130221": ("Imaging Agent Volume per Unit of Presentation", [])
    },
    "ImagingAgentWarmed": {"130187": ("Imaging Agent Warmed", [])},
    "ImagingConditions": {"123016": ("Imaging Conditions", [])},
    "ImagingDirection": {"123015": ("Imaging Direction", [])},
    "ImagingMeasurementReport": {"126000": ("Imaging Measurement Report", [7021])},
    "ImagingMeasurements": {"126010": ("Imaging Measurements", [])},
    "ImagingProcedureRoom": {"127372": ("Imaging procedure room", [603])},
    "ImagingSeriesQuality": {"130550": ("Imaging Series Quality", [])},
    "ImagingStartDatetime": {"122712": ("Imaging Start DateTime", [])},
    "ImagingStudyQuality": {"130549": ("Imaging Study Quality", [])},
    "ImpairedRenalFunction": {"128144": ("Impaired Renal Function", [])},
    "ImplantAssemblyTemplate": {"112366": ("Implant Assembly Template", [])},
    "ImplantComponentList": {"112360": ("Implant Component List", [])},
    "ImplantFabrication": {"129016": ("Implant Fabrication", [7064])},
    "ImplantFinding": {"111407": ("Implant finding", [])},
    "ImplantTemplate": {"112348": ("Implant Template", [])},
    "ImplantType": {"111405": ("Implant type", [])},
    "ImplantationPatientPositioning": {
        "112368": ("Implantation Patient Positioning", [])
    },
    "ImplantationPlan": {"112345": ("Implantation Plan", [])},
    "Import": {"110107": ("Import", [400])},
    "Impression": {"121073": ("Impression", [3419, 6053, 7002])},
    "ImpressionDescription": {"111033": ("Impression Description", [])},
    "Impressions": {"121072": ("Impressions", [])},
    "InDetectorPlane": {"113941": ("In Detector Plane", [10025])},
    "InHomeCage": {"127101": ("In home cage", [634])},
    "InStentNeointima": {"122357": ("In-Stent Neointima", [3491])},
    "InStentNeointimalCrossSectionalArea": {
        "122335": ("In-Stent Neointimal Cross-Sectional Area", [3482])
    },
    "InStentNeointimalVolume": {"122374": ("In-Stent Neointimal Volume", [3485])},
    "InadequateCompression": {"111196": ("Inadequate compression", [6041])},
    "InadequateContrastEnhancement": {
        "130580": ("Inadequate contrast enhancement", [6315, 6317])
    },
    "InappropriateForTheDevice": {"128618": ("Inappropriate for the device", [800])},
    "InappropriateForTheIndications": {
        "128621": ("Inappropriate for the indications", [800])
    },
    "InappropriateImageProcessing": {
        "111219": ("Inappropriate image processing", [6041, 6135, 7011])
    },
    "InappropriatePatientOrientation": {
        "130461": ("Inappropriate Patient Orientation", [9565])
    },
    "InappropriatePatientPosition": {
        "130462": ("Inappropriate Patient Position", [9565])
    },
    "IncisionMade": {"111440": ("Incision made", [])},
    "IncompleteStudy": {"130574": ("Incomplete study", [6314])},
    "InconsistentWithLabelingOfTheDevice": {
        "128622": ("Inconsistent with labeling of the device", [800])
    },
    "IncorrectModalityWorklistEntry": {
        "113038": ("Incorrect Modality Worklist Entry", [7010, 7031])
    },
    "IncorrectProcedureOrdered": {
        "110502": ("Incorrect procedure ordered", [9300, 9301, 9303])
    },
    "IncorrectSideOrdered": {"110512": ("Incorrect side ordered", [9300, 9301])},
    "IncorrectWorklistEntrySelected": {
        "110514": ("Incorrect worklist entry selected", [9300, 9301, 9302])
    },
    "IncrementalAcquisitionTriggering": {
        "130796": ("Incremental Acquisition Triggering", [])
    },
    "IndependentRadiationShield": {
        "130641": ("Independent radiation shield", [9572, 9573])
    },
    "Index": {"121425": ("Index", [])},
    "Indexed": {"125313": ("Indexed", [12303])},
    "IndexedHemodynamicResistance": {
        "110826": ("Indexed Hemodynamic Resistance", [218, 7180, 7469])
    },
    "IndicatedProblem": {"111533": ("Indicated Problem", [])},
    "IndicationsForPharmacologicalStress": {
        "122700": ("Indications for Pharmacological Stress", [])
    },
    "IndicationsForProcedure": {"121109": ("Indications for Procedure", [])},
    "IndicatorAppearanceTime": {"122192": ("Indicator appearance time", [])},
    "IndicatorMeanTransitTime": {
        "109071": (
            "Indicator mean transit time",
            [
                3337,
                3613,
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "IndirectDetector": {"113949": ("Indirect Detector", [10030])},
    "IndividualCalcification": {"111104": ("Individual Calcification", [])},
    "IndividualImpressionRecommendation": {
        "111034": ("Individual Impression/\u200bRecommendation", [])
    },
    "IndividualImpressionRecommendationAnalysis": {
        "111233": ("Individual Impression/\u200bRecommendation Analysis", [6043])
    },
    "IneligibleForReimbursement": {"128615": ("Ineligible for reimbursement", [800])},
    "InferiorToAnterior": {"110875": ("Inferior To Anterior", [7185])},
    "InferolateralToSuperomedial": {
        "111432": ("Inferolateral to superomedial", [6065])
    },
    "InferomedialToSuperolateral": {
        "111433": ("Inferomedial to superolateral", [6065])
    },
    "Infiltrate": {"112121": ("Infiltrate", [6102, 6104])},
    "InflatableEndorectalCoilFillSubstance": {
        "130545": ("Inflatable endorectal coil fill substance", [])
    },
    "InflowAngle": {"122546": ("Inflow Angle", [])},
    "InformationTechnology": {"128173": ("Information Technology", [7030])},
    "InformationUsedForPlanning": {"112358": ("Information used for planning", [])},
    "InfraredFilter": {"111606": ("Infrared filter", [])},
    "InfraredMarker": {"130645": ("Infrared Marker", [9573, 9575])},
    "InfraredReflectorMarker": {"129309": ("Infrared Reflector Marker", [7111, 7112])},
    "InfusateEnd": {"122088": ("Infusate end", [3409])},
    "InfusateStart": {"122087": ("Infusate start", [3409])},
    "InfusionSystem": {"113542": ("Infusion System", [10041])},
    "Inhomogeneous": {"112161": ("Inhomogeneous", [6131])},
    "InitialVolumeOfImagingAgentInContainer": {
        "130205": ("Initial Volume of Imaging Agent in Container", [])
    },
    "InjectateTemperature": {"122320": ("Injectate Temperature", [])},
    "InjectateVolume": {"122321": ("Injectate Volume", [])},
    "InjectorEventDetectionDatetime": {
        "130235": ("Injector Event Detection DateTime", [])
    },
    "InnerLimitsOfFuzzyMargin": {"113662": ("Inner limits of fuzzy margin", [6166])},
    "InsideOperationalLimitsOfTheDevice": {
        "128607": ("Inside operational limits of the device", [800])
    },
    "InstanceNumber": {"113609": ("Instance Number", [10022])},
    "InstancesImported": {"110028": ("Instances Imported", [7008])},
    "InstitutionallyApprovedEstimates": {
        "113529": ("Institutionally Approved Estimates", [10040])
    },
    "InstitutionallyDefinedQualityControlStandard": {
        "111240": (
            "Institutionally defined quality control standard",
            [6045, 6136, 6208, 6326, 6353],
        )
    },
    "InstrumentApproach": {"111431": ("Instrument Approach", [])},
    "InsufficientImplantDisplacementIncorrect": {
        "111206": ("Insufficient implant displacement incorrect", [6041])
    },
    "InsufficientQualityForInterpretation": {
        "110531": ("Insufficient quality for interpretation", [9303])
    },
    "IntegratedProjectionRadiographySystem": {
        "113958": ("Integrated Projection Radiography System", [10032])
    },
    "IntegrationOfSumOfClosedAreasOnContiguousSlicesMethodForVolume": {
        "122503": (
            "Integration of sum of closed areas on contiguous slices method for volume",
            [7474],
        )
    },
    "InterHemisphericPlane": {"125030": ("Inter-Hemispheric Plane", [7101, 7111])},
    "InterMarkerDistance": {"121208": ("Inter-Marker Distance", [3423])},
    "Interface": {"112082": ("Interface", [6102, 6109, 7151, 7196])},
    "InterferingTearsOrDrops": {"111695": ("Interfering Tears or Drops", [4222])},
    "InterferometricTransducer": {"130817": ("Interferometric Transducer", [11003])},
    "Interferometry": {"114202": ("Interferometry", [8201])},
    "InterferometryBasedCornealTomographer": {
        "111947": ("Interferometry-based corneal tomographer", [4210])
    },
    "InterlockOverridden": {"130454": ("Interlock Overridden", [9563])},
    "InterlockResolved": {"130744": ("Interlock Resolved", [9563])},
    "IntermediateFamilyHistoryOfBreastCancer": {
        "111560": ("Intermediate family history of breast cancer", [6081, 6087])
    },
    "IntermediateLevel": {"128726": ("Intermediate level", [7016])},
    "Intern": {"121085": ("Intern", [])},
    "InternationalProgressiveDogBreedersAlliance": {
        "109214": ("International Progressive Dog Breeders' Alliance", [7481])
    },
    "InterpolatedLocalReference": {"122490": ("Interpolated Local Reference", [3465])},
    "Interpretation": {"110005": ("Interpretation", [9231])},
    "InterpretationRequiresSpecialistExpertise": {
        "110532": ("Interpretation requires specialist expertise", [9303])
    },
    "InterquartileRangeOfPopulation": {
        "130614": ("Interquartile Range of population", [221, 226, 7465])
    },
    "InterquartileRangeToMedianRatioOfPopulation": {
        "130615": (
            "Interquartile Range to Median ratio of population",
            [221, 226, 7465],
        )
    },
    "InterstitialIllumination": {"130814": ("Interstitial illumination", [11001])},
    "InterventionAction": {"122090": ("Intervention Action", [])},
    "InterventionAttemptIdentifier": {
        "121154": ("Intervention attempt identifier", [])
    },
    "InterventionPerformed": {"122312": ("Intervention performed", [3412])},
    "InterventionalDevicePlaced": {"122311": ("Interventional device placed", [3412])},
    "InterventionalDevicePlacementUnsuccessful": {
        "122310": ("Interventional device placement unsuccessful", [3412])
    },
    "InterventionalDeviceWithdrawn": {
        "122313": ("Interventional device withdrawn", [3412])
    },
    "IntimalDissection": {"122398": ("Intimal Dissection", [3492])},
    "IntraOralRadiography": {"IO": ("Intra-oral Radiography", [29, 30, 33])},
    "IntraStentDissection": {"122388": ("Intra-stent Dissection", [3492])},
    "IntracysticLesion": {"111461": ("Intracystic lesion", [6054, 6064])},
    "IntracysticPapillaryCarcinoma": {
        "111315": ("Intracystic papillary carcinoma", [6030, 6033])
    },
    "IntracysticPapilloma": {"111257": ("Intracystic papilloma", [])},
    "IntraductalCarcinomaHighGrade": {
        "111341": ("Intraductal carcinoma, high grade", [6030, 6033])
    },
    "IntraductalCarcinomaLowGrade": {
        "111313": ("Intraductal carcinoma, low grade", [6030, 6033])
    },
    "IntraductalCarcinomaMicroPapillary": {
        "111314": ("Intraductal carcinoma micro-papillary", [])
    },
    "IntraductalComedocarcinomaWithNecrosis": {
        "111312": ("Intraductal comedocarcinoma with necrosis", [6030, 6033])
    },
    "IntralobularLines": {"112108": ("Intralobular lines", [6102, 6103])},
    "IntraluminalFillingDefect": {
        "111425": ("Intraluminal filling defect", [6054, 6057])
    },
    "IntraocularLensCalculation": {"IOL": ("Intraocular Lens Calculation", [32, 33])},
    "IntraoperativeFixedAperture": {
        "130345": ("Intraoperative Fixed Aperture", [9541, 9545])
    },
    "IntraoperativeTransducer": {"125267": ("Intraoperative Transducer", [12035])},
    "IntravascularOpticalCoherenceTomography": {
        "IVOCT": ("Intravascular Optical Coherence Tomography", [29, 30, 33])
    },
    "IntravascularTransducer": {"125265": ("Intravascular Transducer", [12035])},
    "IntravascularUltrasound": {"IVUS": ("Intravascular Ultrasound", [29, 30, 33])},
    "IntravenousExtravasationSymptoms": {
        "113505": ("Intravenous Extravasation Symptoms", [])
    },
    "InvasiveAndInSituCarcinoma": {
        "111316": ("Invasive and in-situ carcinoma", [6030, 6033])
    },
    "InvasiveBreastCarcinoma": {"111389": ("Invasive breast carcinoma", [6159])},
    "InvasiveCribriformCarcinoma": {"111342": ("Invasive cribriform carcinoma", [])},
    "InvasiveLobularCarcinoma": {"111317": ("Invasive lobular carcinoma", [])},
    "InverseDifferenceMomentNormalizedOfGLCM": {
        "128792": ("Inverse Difference Moment Normalized of GLCM", [])
    },
    "InverseDifferenceMomentOfGLCM": {
        "126062": ("Inverse Difference Moment of GLCM", [])
    },
    "InverseDifferenceNormalizedOfGLCM": {
        "128791": ("Inverse Difference Normalized of GLCM", [])
    },
    "InverseDifferenceOfGLCM": {"128790": ("Inverse Difference of GLCM", [])},
    "InverseVarianceOfGLCM": {"128793": ("Inverse Variance of GLCM", [])},
    "InvestigationalNewDrug": {"113530": ("Investigational New Drug", [10040])},
    "Involved": {"111471": ("Involved", [])},
    "IodinatedI125DPA713": {"126764": ("Iodinated I^125^ DPA-713", [25])},
    "IonChamber": {"128707": ("Ion Chamber", [7026, 7027, 7151, 7193])},
    "IonChamberArray": {"128703": ("Ion Chamber Array", [7026, 7027, 7151, 7193])},
    "IonTherapyParticle": {"130037": ("Ion Therapy Particle", [])},
    "IrradiatingDevice": {"113859": ("Irradiating Device", [7445])},
    "IrradiationAdministering": {"113851": ("Irradiation Administering", [7453])},
    "IrradiationAuthorizing": {"113850": ("Irradiation Authorizing", [7453])},
    "IrradiationDetails": {"130505": ("Irradiation Details", [])},
    "IrradiationDuration": {"113742": ("Irradiation Duration", [])},
    "IrradiationEvent": {"113852": ("Irradiation Event", [10000])},
    "IrradiationEventLabel": {"113605": ("Irradiation Event Label", [])},
    "IrradiationEventSummaryData": {"130501": ("Irradiation Event Summary Data", [])},
    "IrradiationEventType": {"113721": ("Irradiation Event Type", [])},
    "IrradiationEventUID": {"113769": ("Irradiation Event UID", [10001])},
    "IrradiationEventUID_Retired": {"113853": ("Irradiation Event UID", [])},
    "IrradiationEventXRayData": {"113706": ("Irradiation Event X-Ray Data", [])},
    "IsIonic": {"130189": ("Is Ionic", [])},
    "IsRejectedAcquisition": {"130503": ("Is Rejected Acquisition", [])},
    "IsRepeatedAcquisition": {"128551": ("Is Repeated Acquisition", [])},
    "IsocenterOrigin": {"130539": ("Isocenter Origin", [10074])},
    "IsocentricPatientSupportContinuousPitchAngle": {
        "126812": ("Isocentric Patient Support Continuous Pitch Angle", [9569, 9570])
    },
    "IsocentricPatientSupportContinuousRollAngle": {
        "126813": ("Isocentric Patient Support Continuous Roll Angle", [9569, 9570])
    },
    "IsocentricPatientSupportContinuousYawAngle": {
        "126814": ("Isocentric Patient Support Continuous Yaw Angle", [9569, 9570])
    },
    "IsocentricPatientSupportLateralPosition": {
        "126815": ("Isocentric Patient Support Lateral Position", [9569, 9570])
    },
    "IsocentricPatientSupportLongitudinalPosition": {
        "126816": ("Isocentric Patient Support Longitudinal Position", [9569, 9570])
    },
    "IsocentricPatientSupportVerticalPosition": {
        "126817": ("Isocentric Patient Support Vertical Position", [9569, 9570])
    },
    "IsocentricSetupMethod": {"130630": ("Isocentric Setup Method", [9571])},
    "IsocentricTreatmentLocationPoint": {
        "130073": ("Isocentric Treatment Location Point", [9504, 9553, 9574])
    },
    "Isodose": {"128484": ("Isodose", [10063])},
    "IsodoseVolume": {"130747": ("Isodose Volume", [9582])},
    "Isoechoic": {"111365": ("Isoechoic", [6154])},
    "IssuerOfIdentifier": {"110190": ("Issuer of Identifier", [])},
    "IssuerOfParentSpecimenIdentifier": {
        "111706": ("Issuer of Parent Specimen Identifier", [])
    },
    "IssuerOfSpecimenIdentifier": {"111724": ("Issuer of Specimen Identifier", [])},
    "IterativeClosestPoint": {"114213": ("Iterative Closest Point", [8203])},
    "IterativeReconstruction": {"113963": ("Iterative Reconstruction", [10033])},
    "J59189Zr": {"126514": ("J591 ^89^Zr", [4021])},
    "JWire": {"111170": ("J Wire", [])},
    "JawPair": {"130330": ("Jaw Pair", [9540, 9541])},
    "JawPositionToleranceViolation": {
        "130471": ("Jaw Position Tolerance Violation", [9567])
    },
    "JointAverageOfGLCM": {"128782": ("Joint Average of GLCM", [])},
    "JointEntropyOfGLCM": {"126060": ("Joint Entropy of GLCM", [])},
    "JointMaximumOfGLCM": {"128781": ("Joint Maximum of GLCM", [])},
    "JointPositionMethod": {"109132": ("Joint position method", [])},
    "JointVarianceOfGLCM": {"128783": ("Joint Variance of GLCM", [])},
    "JuvenilePapillomatosis": {"111277": ("Juvenile papillomatosis", [6030, 6031])},
    "KTypeFilter": {"130765": ("K-type filter", [3042])},
    "KVP": {"113733": ("KVP", [])},
    "KeepVeinOpenEnded": {"130162": ("Keep vein open ended", [71])},
    "KeepVeinOpenStarted": {"130161": ("Keep vein open started", [71])},
    "Kep": {"126313": ("kep", [218, 4107, 7180, 7469])},
    "Keratometry": {"KER": ("Keratometry", [29, 30, 33])},
    "KeratometryMeasurementsSOPInstance": {
        "111757": ("Keratometry Measurements SOP Instance", [4240])
    },
    "KerleyALine": {"112109": ("Kerley A line", [6102, 6103])},
    "KerleyBLine": {"112110": ("Kerley B line", [6102, 6103])},
    "KerleyCLines": {"112111": ("Kerley C lines", [6102, 6103])},
    "KeyImages": {"121180": ("Key Images", [])},
    "KeyObjectDescription": {"113012": ("Key Object Description", [])},
    "KeyObjectSelection": {"KO": ("Key Object Selection", [32, 33])},
    "KidneyStent": {
        "112175": ("Kidney stent", [6102, 6138, 6202, 6203, 6404, 7151, 7193])
    },
    "KnownBiopsyProvenMalignancy": {
        "111125": ("Known biopsy proven malignancy", [6051])
    },
    "KnownBiopsyProvenMalignancyTakeAppropriateAction": {
        "111122": (
            "Known biopsy proven malignancy - take appropriate action",
            [6028, 6029],
        )
    },
    "Ktrans": {"126312": ("Ktrans", [218, 4107, 7180, 7469])},
    "Kurtosis": {"126052": ("Kurtosis", [7464])},
    "KurtosisDiffusionCoefficient": {
        "113294": ("Kurtosis Diffusion Coefficient", [218, 7180, 7272, 7469])
    },
    "KurtosisDiffusionModel": {"113252": ("Kurtosis diffusion model", [7273])},
    "LADRegionInLAOProjection": {"122470": ("LAD Region in LAO Projection", [3461])},
    "LADRegionInRAOProjection": {"122464": ("LAD Region in RAO Projection", [3460])},
    "LBWOrIUGR": {"111593": ("LBW or IUGR", [])},
    "LDLCholesterolScoreSheetForMen": {
        "122233": ("LDL Cholesterol Score Sheet for Men", [3668])
    },
    "LDLCholesterolScoreSheetForWomen": {
        "122234": ("LDL Cholesterol Score Sheet for Women", [3668])
    },
    "LSDI": {"113225": ("LSDI", [7260])},
    "LVEndSystolicGLS2CHEndocardial": {
        "130688": ("LV End-systolic GLS 2CH (Endocardial)", [12300, 12309])
    },
    "LVEndSystolicGLS4CHEndocardial": {
        "130687": ("LV End-systolic GLS 4CH (Endocardial)", [12300, 12309])
    },
    "LVEndSystolicGLSALAXEndocardial": {
        "130689": ("LV End-systolic GLS ALAX (Endocardial)", [12300, 12309])
    },
    "LVEndSystolicGLSMultiviewMeanEndocardial": {
        "130686": ("LV End-systolic GLS Multiview Mean (Endocardial)", [12300, 12309])
    },
    "LVEndSystolicMechanicalDispersion": {
        "130698": ("LV End-systolic Mechanical Dispersion", [12300, 12309])
    },
    "LVPeakGLS2CHEndocardial": {
        "130696": ("LV Peak GLS 2CH (Endocardial)", [12300, 12309])
    },
    "LVPeakGLS4CHEndocardial": {
        "130695": ("LV Peak GLS 4CH (Endocardial)", [12300, 12309])
    },
    "LVPeakGLSALAXEndocardial": {
        "130697": ("LV Peak GLS ALAX (Endocardial)", [12300, 12309])
    },
    "LVPeakGLSMultiviewMeanEndocardial": {
        "130694": ("LV Peak GLS Multiview Mean (Endocardial)", [12300, 12309])
    },
    "LVPeakSystolicGLS2CHEndocardial": {
        "130692": ("LV Peak Systolic GLS 2CH (Endocardial)", [12300, 12309])
    },
    "LVPeakSystolicGLS4CHEndocardial": {
        "130691": ("LV Peak Systolic GLS 4CH (Endocardial)", [12300, 12309])
    },
    "LVPeakSystolicGLSALAXEndocardial": {
        "130693": ("LV Peak Systolic GLS ALAX (Endocardial)", [12300, 12309])
    },
    "LVPeakSystolicGLSMultiviewMeanEndocardial": {
        "130690": ("LV Peak Systolic GLS Multiview Mean (Endocardial)", [12300, 12309])
    },
    "LVWallMotionScoreIndex": {"125202": ("LV Wall Motion Score Index", [])},
    "LWHMethodForVolumeOfEllipsoid": {
        "126029": ("LWH method for volume of ellipsoid", [7474])
    },
    "LabelType": {"113606": ("Label Type", [])},
    "LactatingAdenoma": {"111278": ("Lactating adenoma", [])},
    "LactationalChange": {"111279": ("Lactational change", [6030, 6031])},
    "LandmarkBasedWallMotionAnalysis": {
        "122497": ("Landmark Based Wall Motion Analysis", [])
    },
    "Language": {"121045": ("Language", [])},
    "LanguageOfContentItemAndDescendants": {
        "121049": ("Language of Content Item and Descendants", [])
    },
    "LanguageOfNameAndValue": {"121048": ("Language of Name and Value", [])},
    "LanguageOfValue": {"121047": ("Language of Value", [])},
    "Laparoscopy": {"LP": ("Laparoscopy", [])},
    "LargeDuctPapilloma": {"111281": ("Large duct papilloma", [6030, 6031])},
    "LargeGreaterThan38Point0cmLateralThickness": {
        "113603": ("Large: > 38.0 cm lateral thickness", [7041, 7042])
    },
    "LargeZoneEmphasis": {"128822": ("Large Zone Emphasis", [])},
    "LargeZoneHighGrayLevelEmphasis": {
        "128828": ("Large Zone High Gray Level Emphasis", [])
    },
    "LargeZoneLowGrayLevelEmphasis": {
        "128827": ("Large Zone Low Gray Level Emphasis", [])
    },
    "LaserCrossHairs": {"128151": ("Laser Cross-hairs", [1015, 9573, 9575])},
    "LaserScan": {"LS": ("Laser Scan", [29, 30, 33])},
    "LaserScanning": {"114203": ("Laser scanning", [8201])},
    "LaserScanning3DCAMModel": {"129021": ("Laser Scanning 3D CAM model", [7061])},
    "LaserSurfaceScan": {"LS": ("Laser Surface Scan", [9573, 9575])},
    "LastChordOfAbnormalRegion": {"122453": ("Last Chord of Abnormal Region", [])},
    "LateChildBearingAfter30": {
        "111555": ("Late child bearing (after 30)", [6081, 6087])
    },
    "LateContrastEnhancement": {
        "122664": ("Late Contrast Enhancement", [6335, 6345, 6346])
    },
    "LeadID": {"122148": ("Lead ID", [])},
    "LeadRadiologicTechnologist": {
        "128674": ("Lead Radiologic Technologist", [7450, 7452, 9536])
    },
    "LeafPairs": {"130331": ("Leaf Pairs", [9540, 9541])},
    "LeastMeanSquareLMSDeconvolution": {
        "126310": ("Least Mean Square (LMS) deconvolution", [4105, 4106])
    },
    "LeastSquaresFitOfMultipleSamples": {
        "113261": ("Least squares fit of multiple samples", [7274])
    },
    "LeftAtrialEDVolume": {"122407": ("Left Atrial ED Volume", [3468, 3835])},
    "LeftAtrialESVolume": {"122408": ("Left Atrial ES Volume", [3469, 3835])},
    "LeftAtrialEjectionFractionByAngiography": {
        "122406": ("Left Atrial Ejection Fraction by Angiography", [3467])
    },
    "LeftContour": {"122507": ("Left Contour", [])},
    "LeftEyeRx": {"111689": ("Left Eye Rx", [])},
    "LeftFirst": {"126830": ("left first", [21])},
    "LeftHemisphereMostAnterior": {
        "125035": ("Left Hemisphere Most Anterior", [7101, 7111])
    },
    "LeftHemisphereMostInferior": {
        "125038": ("Left Hemisphere Most Inferior", [7101, 7111])
    },
    "LeftHemisphereMostPosterior": {
        "125036": ("Left Hemisphere Most Posterior", [7101, 7111])
    },
    "LeftHemisphereMostSuperior": {
        "125037": ("Left Hemisphere Most Superior", [7101, 7111])
    },
    "LeftRightAxis": {"110860": ("Left-Right Axis", [7184])},
    "LeftToRight": {"110867": ("Left To Right", [7185])},
    "LeftToRightFlow": {"122227": ("Left to Right Flow", [])},
    "LeftVentricleApicalAnterolateralSegment": {
        "130623": ("left ventricle apical anterolateral segment", [3781, 3785, 12305])
    },
    "LeftVentricleApicalAnteroseptalSegment": {
        "130620": ("left ventricle apical anteroseptal segment", [3781, 3785, 12305])
    },
    "LeftVentricleApicalInferolateralSegment": {
        "130622": ("left ventricle apical inferolateral segment", [3781, 3785, 12305])
    },
    "LeftVentricleApicalInferoseptalSegment": {
        "130621": ("left ventricle apical inferoseptal segment", [3781, 3785, 12305])
    },
    "LeftVentricleMassByAreaLength": {
        "125270": ("Left Ventricle Mass by Area Length", [12227, 12232])
    },
    "LeftVentricleMassByAreaLengthAdjustedByHeight": {
        "125273": (
            "Left Ventricle Mass by Area Length - adjusted by Height",
            [12227, 12232],
        )
    },
    "LeftVentricleMassByMMode": {
        "125221": ("Left Ventricle Mass by M-mode", [12227, 12232])
    },
    "LeftVentricleMassByMModeAdjustedByHeight": {
        "125271": ("Left Ventricle Mass by M-mode - adjusted by Height", [12227, 12232])
    },
    "LeftVentricleMassByTruncatedEllipse": {
        "125222": ("Left Ventricle Mass by Truncated Ellipse", [12227, 12232])
    },
    "LeftVentricleMassByTruncatedEllipseAdjustedByHeight": {
        "125272": (
            "Left Ventricle Mass by Truncated Ellipse - adjusted by Height",
            [12227, 12232],
        )
    },
    "LeftVentricularEjectionTime": {
        "122211": ("Left Ventricular ejection time", [3613])
    },
    "LeftVentricularFillingTime": {"122212": ("Left Ventricular filling time", [3613])},
    "LeftVentricularPosterobasalSegment": {
        "122582": ("Left ventricular posterobasal segment", [])
    },
    "LengthLuminalSegment": {"122510": ("Length Luminal Segment", [])},
    "LengthOfReconstructableVolume": {
        "113893": ("Length of Reconstructable Volume", [])
    },
    "Lensometry": {"LEN": ("Lensometry", [29, 30, 33])},
    "LesionBoundary": {"111357": ("Lesion boundary", [])},
    "LesionCapsularContactLength": {
        "130558": ("Lesion capsular contact length", [6351])
    },
    "LesionCentered": {"111902": ("Lesion centered", [4207])},
    "LesionCompletelyRemoved": {"111447": ("Lesion completely removed", [6066])},
    "LesionDensity": {"111035": ("Lesion Density", [])},
    "LesionIdentifier": {"121151": ("Lesion Identifier", [])},
    "LesionInterventionInformation": {
        "122178": ("Lesion Intervention Information", [])
    },
    "LesionMorphology": {"122133": ("Lesion Morphology", [])},
    "LesionPartiallyRemoved": {"111448": ("Lesion partially removed", [6066])},
    "LesionRisk": {"121153": ("Lesion Risk", [])},
    "LesionSensitivity": {"111089": ("Lesion Sensitivity", [6048])},
    "LesionToBackgroundSUVRatio": {
        "126039": ("Lesion to Background SUV Ratio", [7466])
    },
    "LesserThan3MonthsAgo": {"111396": ("< 3 months ago", [6164])},
    "LeukemicInfiltration": {"111318": ("Leukemic infiltration", [6030, 6033])},
    "LevelOfDifficulty": {"128724": ("Level of Difficulty", [])},
    "LevelOfSignificance": {"121403": ("Level of Significance", [])},
    "LevenbergMarquardt": {"113265": ("Levenberg-Marquardt", [7274])},
    "Levovist": {"125904": ("Levovist", [12030])},
    "LightBrownColor": {"111450": ("Light brown color", [6067])},
    "LightsOnTimeOfDay": {"127215": ("Lights on time of day", [])},
    "Line": {"112083": ("Line", [6102, 6109, 7151, 7196])},
    "LineBScanPattern": {"128281": ("Line B-scan pattern", [4272])},
    "LineNoiseArtifact": {"130886": ("Line noise artifact", [3035])},
    "LineScanPattern": {"125240": ("Line scan pattern", [12032])},
    "LineSegmentLength": {"121227": ("Line segment length", [218, 6165, 7469, 7470])},
    "Linear": {"112150": ("Linear", [6123, 6335, 6339, 6340])},
    "LinearCellKillFactor": {"130088": ("Linear Cell Kill Factor", [])},
    "LinearCurve": {"130253": ("Linear Curve", [73])},
    "LinearDisplacement": {"110856": ("Linear Displacement", [7182])},
    "LinearQuadraticLinearModel": {"130133": ("Linear-Quadratic-Linear Model", [9539])},
    "LinearQuadraticModel": {"130131": ("Linear-Quadratic Model", [9539])},
    "LinearQuadraticModelForLowDoseRateBrachytherapy": {
        "130134": ("Linear-Quadratic Model for Low-Dose Rate Brachytherapy", [9539])
    },
    "LinearQuadraticModelWithTimeFactor": {
        "130132": ("Linear-Quadratic Model with Time Factor", [9539])
    },
    "LinearSpiculation": {"113665": ("Linear spiculation", [6166])},
    "LinearUltrasoundTransducerGeometry": {
        "125252": ("Linear ultrasound transducer geometry", [12033])
    },
    "LipidAndLactate": {
        "113095": ("Lipid and Lactate", [218, 4032, 4033, 7180, 7186, 7469])
    },
    "LiquidCrystalDisplay": {"109992": ("Liquid Crystal Display", [8303])},
    "LiquidIonChamber": {"128705": ("Liquid Ion Chamber", [7026, 7027, 7151, 7193])},
    "Lobar": {"112158": ("Lobar", [6128, 6129, 8134])},
    "Lobulated": {"112135": ("Lobulated", [6119, 6335, 6339, 6340])},
    "LocalEffectModel": {"130128": ("Local Effect Model", [9538])},
    "LocalServiceOperationStarted": {
        "110141": ("Local Service Operation Started", [401, 403])
    },
    "LocalServiceOperationStopped": {
        "110142": ("Local Service Operation Stopped", [401, 403])
    },
    "LocalizedProstateFinding": {"130555": ("Localized Prostate Finding", [])},
    "Localizer": {"121311": ("Localizer", [4264, 7201, 8120])},
    "LocallyManufacturedProduct": {"127390": ("Locally manufactured product", [608])},
    "LocationInChest": {"112013": ("Location in Chest", [])},
    "LocationOfSamplingSite": {"111709": ("Location of sampling site", [])},
    "LocationOfSamplingSiteXOffset": {
        "111710": ("Location of sampling site X offset", [])
    },
    "LocationOfSamplingSiteYOffset": {
        "111711": ("Location of sampling site Y offset", [])
    },
    "LocationOfSamplingSiteZOffset": {
        "111712": ("Location of sampling site Z offset", [])
    },
    "LocationOfSpecimen": {"111718": ("Location of Specimen", [])},
    "LocationOfSpecimenXOffset": {"111719": ("Location of Specimen X offset", [])},
    "LocationOfSpecimenYOffset": {"111720": ("Location of Specimen Y offset", [])},
    "LocationOfSpecimenZOffset": {"111721": ("Location of Specimen Z offset", [])},
    "LogOfRatioOfTwoSamples": {"113260": ("Log of ratio of two samples", [7274])},
    "Login": {"110122": ("Login", [401, 403])},
    "Logout": {"110123": ("Logout", [401, 403])},
    "LongAxisBased": {"122476": ("Long Axis Based", [3458])},
    "LongRunHighGrayLevelEmphasis": {
        "128808": ("Long Run High Gray Level Emphasis", [])
    },
    "LongRunLowGrayLevelEmphasis": {"128807": ("Long Run Low Gray Level Emphasis", [])},
    "LongRunsEmphasis": {"128802": ("Long Runs Emphasis", [])},
    "LongitudinalLagrangianStrain": {
        "130670": ("Longitudinal Lagrangian Strain", [12304, 12311])
    },
    "LongitudinalLagrangianStrainRate": {
        "130674": ("Longitudinal Lagrangian Strain Rate", [12304, 12311])
    },
    "LongitudinalPositionZ": {"113994": ("Longitudinal Position Z", [])},
    "LongitudinalTemporalEventType": {
        "128741": ("Longitudinal Temporal Event Type", [])
    },
    "LongitudinalTemporalOffsetFromEvent": {
        "128740": ("Longitudinal Temporal Offset from Event", [])
    },
    "LossyCompressedImage": {"121325": ("Lossy compressed image", [7205])},
    "LossyCompressedPredecessor": {"121330": ("Lossy compressed predecessor", [7202])},
    "LossyCompression": {"113040": ("Lossy Compression", [7203])},
    "LotIdentifier": {"121149": ("Lot Identifier", [3426])},
    "Low": {"111577": ("Low", [6090])},
    "LowGrayLevelRunEmphasis": {"128803": ("Low Gray Level Run Emphasis", [])},
    "LowGrayLevelZoneEmphasis": {"128823": ("Low Gray Level Zone Emphasis", [])},
    "LowThyroidUptake": {"113561": ("Low Thyroid Uptake", [64, 10045])},
    "Lucency": {"112084": ("Lucency", [6102, 6109, 7151, 7196])},
    "LumenDiameterRatio": {"122350": ("Lumen Diameter Ratio", [3484])},
    "LumenEccentricityIndex": {"122343": ("Lumen Eccentricity Index", [3484])},
    "LumenPerimeter": {"122332": ("Lumen Perimeter", [3481])},
    "LumenShapeIndex": {"122348": ("Lumen Shape Index", [3484])},
    "LumenVolume": {"122372": ("Lumen Volume", [3485])},
    "LuminanceResponseEvaluation": {
        "109703": ("Luminance response evaluation", [8300])
    },
    "LuminanceUniformityEvaluation": {
        "109704": ("Luminance uniformity evaluation", [8300])
    },
    "Lutetium177NAcetylaspartylglutamate": {
        "126509": ("Lutetium^177^ n-acetylaspartylglutamate", [25])
    },
    "LympathicVesselInvasion": {"111320": ("Lympathic vessel invasion", [6030, 6033])},
    "Lymphoma": {"111321": ("Lymphoma", [])},
    "MDerivedFilter": {"130766": ("M-derived filter", [3042])},
    "MEDI55189Zr": {"126730": ("MEDI-551 ^89^Zr", [4021])},
    "MEMSBasedTransducer": {"130816": ("MEMS-based Transducer", [11003])},
    "MIRDOSE": {"113526": ("MIRDOSE", [10040])},
    "MIRDPamphlet1": {"113520": ("MIRD Pamphlet 1", [10040])},
    "MLCPositionInterlock": {"130829": ("MLC Position Interlock", [9561, 9568])},
    "MLCPositionToleranceViolation": {
        "130472": ("MLC Position Tolerance Violation", [9567])
    },
    "MLOEvidenceOfMotionBlur": {"111200": ("MLO Evidence of motion blur", [6041])},
    "MLOInframammaryFoldIsNotOpen": {
        "111201": ("MLO Inframammary fold is not open", [6041])
    },
    "MLOInsufficientPectoralMuscle": {
        "111197": ("MLO Insufficient pectoral muscle", [6041])
    },
    "MLONoFatIsVisualizedPosteriorToFibroglandularTissues": {
        "111198": (
            "MLO No fat is visualized posterior to fibroglandular tissues",
            [6041],
        )
    },
    "MLOPoorSeparationOfDeepAndSuperficialBreastTissues": {
        "111199": ("MLO Poor separation of deep and superficial breast tissues", [6041])
    },
    "MODImported": {"110026": ("MOD Imported", [7008])},
    "MPPSContent": {"113858": ("MPPS Content", [10020])},
    "MRMarker": {"129308": ("MR Marker", [7111, 7112])},
    "MRPerfusionPeak": {"122635": ("MR Perfusion Peak", [])},
    "MRPerfusionSlope": {"122636": ("MR Perfusion Slope", [])},
    "MRPerfusionTimeIntegral": {"122637": ("MR Perfusion Time Integral", [])},
    "MRSignalIntensity": {"110852": ("MR signal intensity", [218, 7180, 7469])},
    "MVAEqualsFlow38Point0SqrtGradientMmhg": {
        "122263": ("MVA = Flow / 38.0 * sqrt(Gradient[mmHg])", [])
    },
    "MachineCalibrationAdjustment": {
        "130465": ("Machine Calibration Adjustment", [9565])
    },
    "MachineCapabilityLicenseExpired": {
        "130467": ("Machine Capability License Expired", [9565])
    },
    "MachineClearanceInterlock": {
        "130740": ("Machine Clearance Interlock", [9561, 9568])
    },
    "MachineNotAvailable": {"130463": ("Machine Not Available", [9565])},
    "MachineQualityAssuranceInputUsed": {
        "128216": ("Machine Quality Assurance Input Used", [7010, 7025])
    },
    "Macrocalcifications": {"111345": ("Macrocalcifications", [6010, 6011])},
    "MaculaCentered": {"111900": ("Macula centered", [4207])},
    "MacularFixationTesting": {"111845": ("Macular Fixation Testing", [4253])},
    "MacularGridThicknessAndVolumeReport": {
        "111690": ("Macular Grid Thickness and Volume Report", [])
    },
    "MagneticFieldStrength": {"130542": ("Magnetic field strength", [])},
    "MagneticResonance": {"MR": ("Magnetic Resonance", [29, 30, 33])},
    "MagneticResonanceAngiography": {"MA": ("Magnetic resonance angiography", [])},
    "MagneticResonanceSpectroscopy": {"MS": ("Magnetic resonance spectroscopy", [])},
    "MagneticSusceptibility": {
        "126396": ("Magnetic Susceptibility", [218, 7180, 7469])
    },
    "MagnetizationTransferRatio": {
        "113098": ("Magnetization Transfer Ratio", [218, 7180, 7469])
    },
    "MagnificationSelection": {"112709": ("Magnification selection", [])},
    "MagnitudeNormalized": {"130776": ("Magnitude normalized", [3045])},
    "Male": {"M": ("Male", [7455, 7457])},
    "MaleChangedToFemale": {"MC": ("Male changed to Female", [7455])},
    "MalePseudohermaphrodite": {"MP": ("Male Pseudohermaphrodite", [7455])},
    "MalignancyType": {"111388": ("Malignancy Type", [])},
    "MalignantFibrousHistiocytoma": {"111331": ("Malignant fibrous histiocytoma", [])},
    "MalignantMelanomaOfNipple": {
        "111334": ("Malignant melanoma of nipple", [6030, 6033])
    },
    "MammographicCrosshair": {"111487": ("Mammographic (crosshair)", [6058, 6060])},
    "MammographicGrid": {"111488": ("Mammographic (grid)", [6058, 6060])},
    "Mammography": {"MG": ("Mammography", [29, 30, 33])},
    "MammographyCAD": {"111411": ("Mammography CAD", [6050, 6083])},
    "MammographyCADReport": {"111036": ("Mammography CAD Report", [])},
    "MammographyQualityControlManual1999ACR": {
        "111238": ("Mammography Quality Control Manual 1999, ACR", [6045, 6326])
    },
    "Manifest": {"113030": ("Manifest", [7010])},
    "ManualAdministration": {"130174": ("Manual Administration", [63])},
    "ManualEntry": {"113857": ("Manual Entry", [4240, 10020, 10021, 12303])},
    "ManualHandlingInterval": {"127152": ("Manual handling interval", [])},
    "ManualHoldStarted": {"130267": ("Manual hold started", [71])},
    "ManualKeratometry": {"111753": ("Manual Keratometry", [4235, 4242])},
    "ManualProcessing": {"123109": ("Manual Processing", [7162])},
    "ManualResumeFromHold": {"130268": ("Manual resume from hold", [71])},
    "ManuallyTriggeredInjectionInformation": {
        "130172": ("Manually Triggered Injection Information", [])
    },
    "ManufacturerImplantTemplate": {"112371": ("Manufacturer Implant Template", [])},
    "Margetuximab89Zr": {"126740": ("Margetuximab ^89^Zr", [4021])},
    "Margin": {"Margin": ("Margin", [6333, 6334])},
    "Margins": {"111037": ("Margins", [])},
    "MarkerPlacement": {"111123": ("Marker placement", [6050, 6051, 6058, 6061, 6083])},
    "MaskImageForImageProcessingOperation": {
        "121321": ("Mask image for image processing operation", [7202])
    },
    "MassAdministered": {"121383": ("Mass administered", [3410])},
    "MassInTheSkin": {"111112": ("Mass in the skin", [6014, 6015, 6016, 6054, 6056])},
    "MassOccupancyImage": {"129329": ("Mass Occupancy Image", [])},
    "MassOfBreast": {"112195": ("Mass of breast", [6142])},
    "MassOfParenchymalTissue": {"112194": ("Mass of parenchymal tissue", [6142])},
    "MassOfVascularCalcification": {
        "112199": ("Mass of Vascular Calcification", [6142])
    },
    "MassOnTheSkin": {"111113": ("Mass on the skin", [6014, 6015, 6016, 6054, 6056])},
    "MassScoringMethod": {"112057": ("Mass scoring method", [6140])},
    "MassWithCalcifications": {"111459": ("Mass with calcifications", [6016, 6054])},
    "MaterialHighlightedImage": {"129325": ("Material Highlighted Image", [])},
    "MaterialRecalculatedImage": {"129327": ("Material Recalculated Image", [])},
    "MaterialRemovedImage": {"129324": ("Material Removed Image", [])},
    "MaterialSpecificImage": {"129323": ("Material Specific Image", [])},
    "MaterialSuppressedImage": {"129326": ("Material Suppressed Image", [])},
    "Maternal": {"111541": ("Maternal", [6097])},
    "MatingFeatureID": {"112352": ("Mating Feature ID", [])},
    "MatingFeatureSetID": {"112351": ("Mating Feature Set ID", [])},
    "MaxDpDt": {"109025": ("Max dp/dt", [3337])},
    "MaxDpDtP": {"122190": ("Max dp/dt/P", [])},
    "MaxNegDpDt": {"109026": ("Max neg dp/dt", [3337])},
    "MaxVolumeNormalizedTo50mmhgPulsePressure": {
        "122238": ("Max volume normalized to 50mmHg pulse pressure", [])
    },
    "MaximumAbsoluteVolumeAtRadiationDose": {
        "130017": ("Maximum Absolute Volume at Radiation Dose", [9500, 9531])
    },
    "MaximumAbsorbedRadiationDose": {
        "128531": ("Maximum Absorbed Radiation Dose", [10061])
    },
    "MaximumAttenuationCoefficient": {
        "112180": ("Maximum Attenuation Coefficient", [6141])
    },
    "MaximumCADOperatingPoint": {"111072": ("Maximum CAD Operating Point", [])},
    "MaximumDifference": {"126376": ("Maximum Difference", [218, 4109, 7180, 7469])},
    "MaximumEquivalentRadiationDose": {
        "128535": ("Maximum Equivalent Radiation Dose", [10062])
    },
    "MaximumEquivalentUniformDose": {
        "130008": ("Maximum Equivalent Uniform Dose", [9500, 9529])
    },
    "MaximumHomogeneityIndex": {"130013": ("Maximum Homogeneity Index", [])},
    "MaximumIntensityProjection": {"113078": ("Maximum intensity projection", [7203])},
    "MaximumMeanRadiationDose": {
        "130006": ("Maximum Mean Radiation Dose", [9500, 9529])
    },
    "MaximumOrificeArea": {"125328": ("Maximum Orifice Area", [12304])},
    "MaximumPercentVolumeAtRadiationDose": {
        "130015": ("Maximum Percent Volume at Radiation Dose", [9500, 9530])
    },
    "MaximumPowerOutputAchieved": {"122716": ("Maximum Power Output Achieved", [])},
    "MaximumPressureAcceleration": {"122193": ("Maximum pressure acceleration", [])},
    "MaximumRadiationDose": {"130004": ("Maximum Radiation Dose", [9500, 9529])},
    "MaximumSlope": {"126375": ("Maximum Slope", [218, 4109, 7180, 7469])},
    "MaximumSurfaceRadiationDose": {
        "130002": ("Maximum Surface Radiation Dose", [9500, 9529])
    },
    "MaximumXRayTubeCurrent": {"113833": ("Maximum X-Ray Tube Current", [])},
    "MeanAbsorbedRadiationDose": {"128533": ("Mean Absorbed Radiation Dose", [10061])},
    "MeanAttenuationCoefficient": {"112181": ("Mean Attenuation Coefficient", [6141])},
    "MeanBloodPressure": {"109027": ("Mean blood pressure", [])},
    "MeanCtdifreeair": {"113837": ("Mean CTDIfreeair", [])},
    "MeanCtdivol": {"113830": ("Mean CTDIvol", [10050])},
    "MeanDiffusivity": {"113202": ("Mean Diffusivity", [7263])},
    "MeanEquivalentRadiationDose": {
        "128537": ("Mean Equivalent Radiation Dose", [10062])
    },
    "MeanKurtosis": {"113205": ("Mean Kurtosis", [7263])},
    "MeanLocalReference": {"122491": ("Mean Local Reference", [3465])},
    "MeanTransitTime": {"113052": ("Mean Transit Time", [218, 4108, 7180, 7469])},
    "MeanValueChosen": {"121412": ("Mean value chosen", [224, 4241, 12301])},
    "MeasuredAPDimension": {"113932": ("Measured AP Dimension", [])},
    "MeasuredLateralDimension": {"113931": ("Measured Lateral Dimension", [])},
    "MeasuredProperty": {"125307": ("Measured Property", [])},
    "MeasuredRadiationDose": {"128497": ("Measured Radiation Dose", [10065])},
    "MeasurementDivisor": {"125308": ("Measurement Divisor", [])},
    "MeasurementFailure": {"114006": ("Measurement failure", [42, 43])},
    "MeasurementFromThisDevice": {"111780": ("Measurement From This Device", [4240])},
    "MeasurementGroup": {"125007": ("Measurement Group", [])},
    "MeasurementNotAttempted": {"114007": ("Measurement not attempted", [42, 44])},
    "MeasurementOfResponse": {"112051": ("Measurement of Response", [])},
    "MeasurementOrientation": {"125105": ("Measurement Orientation", [])},
    "MeasurementType": {"125306": ("Measurement Type", [])},
    "MechanicalBeamSteering": {"125258": ("Mechanical beam steering", [12034])},
    "MechanicalDispersion": {"130673": ("Mechanical Dispersion", [12304, 12311])},
    "MechanicalFailure": {"111216": ("Mechanical failure", [6041, 6135, 7011])},
    "MechanicalPointer": {"130643": ("Mechanical Pointer", [9573, 9575])},
    "MediaCreationDevice": {"MCD": ("Media Creation Device", [30])},
    "MediaFailure": {"110524": ("Media Failure", [9300, 9302])},
    "MediaImport": {"110013": ("Media Import", [9231])},
    "MedialDissection": {"122399": ("Medial Dissection", [3492])},
    "Median": {"130290": ("Median", [2, 211, 212, 245, 3019])},
    "MedianAbsorbedRadiationDose": {
        "128539": ("Median Absorbed Radiation Dose", [10061])
    },
    "MedianAttenuationCoefficient": {
        "112182": ("Median Attenuation Coefficient", [6141])
    },
    "MedianEquivalentRadiationDose": {
        "128540": ("Median Equivalent Radiation Dose", [10062])
    },
    "MedicalDeviceUsed": {"121436": ("Medical Device Used", [])},
    "MedicalPhysics": {"128174": ("Medical Physics", [7030])},
    "MedicationHistory": {"111512": ("Medication History", [])},
    "MedicationType": {"111516": ("Medication Type", [])},
    "MedicationsSet": {"127320": ("Medications Set", [])},
    "Medium": {"111576": ("Medium", [6090])},
    "Medium32Point0To38Point0cmLateralThickness": {
        "113602": ("Medium: 32.0-38.0 cm lateral thickness", [7041, 7042])
    },
    "MenopausalPhase": {"111539": ("Menopausal phase", [])},
    "MenstrualCyclePhase": {"111391": ("Menstrual Cycle Phase", [])},
    "MeshRadiationTransportModel": {
        "128423": ("Mesh Radiation Transport Model", [10065])
    },
    "MesopicLight": {"111628": ("Mesopic light", [4203])},
    "MetabolicVolume": {"126032": ("Metabolic Volume", [7466])},
    "MetaboliteMapsFromSpectroscopyData": {
        "113050": ("Metabolite Maps from spectroscopy data", [7203])
    },
    "MetastasisToAnIntramammaryLymphNode": {
        "111333": ("Metastasis to an intramammary lymph node", [6030, 6033])
    },
    "MetastaticCancerToTheBreast": {
        "111323": ("Metastatic cancer to the breast", [6030, 6033])
    },
    "MetastaticCancerToTheBreastFromTheColon": {
        "111324": ("Metastatic cancer to the breast from the colon", [6030, 6033])
    },
    "MetastaticCancerToTheBreastFromTheLung": {
        "111325": ("Metastatic cancer to the breast from the lung", [6030, 6033])
    },
    "MetastaticCancerToTheBreastFromTheOvary": {
        "111327": ("Metastatic cancer to the breast from the ovary", [6030, 6033])
    },
    "MetastaticDiseaseToAxillaryNode": {
        "111330": ("Metastatic disease to axillary node", [6030, 6033])
    },
    "MetastaticMelanomaToTheBreast": {
        "111326": ("Metastatic melanoma to the breast", [6030, 6033])
    },
    "MetastaticSarcomaToTheBreast": {
        "111328": ("Metastatic sarcoma to the breast", [6030, 6033])
    },
    "Meterset": {"130797": ("Meterset", [])},
    "MetersetRateInterlock": {"130830": ("Meterset Rate Interlock", [9561, 9568])},
    "MetersetToleranceViolation": {"130469": ("Meterset Tolerance Violation", [9566])},
    "MethodCitation": {"121423": ("Method Citation", [228])},
    "MethodOfDisksBiplane": {"125207": ("Method of Disks, Biplane", [12227, 12228])},
    "MethodOfDisksSinglePlane": {
        "125208": ("Method of Disks, Single Plane", [12227, 12228])
    },
    "MicroclipPlaced": {"111441": ("Microclip placed", [])},
    "MicrodosimetricKineticModel": {
        "130129": ("Microdosimetric Kinetic Model", [9538])
    },
    "MicroglandularAdenosis": {"111284": ("Microglandular adenosis", [6030, 6031])},
    "Micronodule": {"112122": ("Micronodule", [6102, 6104])},
    "MidPeripheralInferior": {"111908": ("Mid-peripheral-inferior", [4207])},
    "MidPeripheralInferiorNasal": {"111909": ("Mid-peripheral-inferior nasal", [4207])},
    "MidPeripheralInferiorTemporal": {
        "111907": ("Mid-peripheral-inferior temporal", [4207])
    },
    "MidPeripheralNasal": {"111910": ("Mid-peripheral-nasal", [4207])},
    "MidPeripheralSuperior": {"111904": ("Mid-peripheral-superior", [4207])},
    "MidPeripheralSuperiorNasal": {"111911": ("Mid-peripheral-superior nasal", [4207])},
    "MidPeripheralSuperiorTemporal": {
        "111905": ("Mid-peripheral-superior temporal", [4207])
    },
    "MidPeripheralTemporal": {"111906": ("Mid-peripheral-temporal", [4207])},
    "MiddleInnerRetinaVasculatureFlow": {
        "128267": ("Middle inner retina vasculature flow", [4271])
    },
    "MiddleInnerStructuralReflectanceMap": {
        "128268": ("Middle inner structural reflectance map", [4271])
    },
    "MiddleOfTimePeriod": {"130535": ("Middle of Time Period", [10073])},
    "MidlungWindow": {
        "112085": ("Midlung window", [6102, 6109, 6110, 7151, 7192, 9514])
    },
    "MiliaryPattern": {"112129": ("Miliary pattern", [6102, 6106])},
    "MinimizeMeterset": {"130018": ("Minimize Meterset", [9500, 9532])},
    "MinimumAbsoluteVolumeAtRadiationDose": {
        "130016": ("Minimum Absolute Volume at Radiation Dose", [9500, 9531])
    },
    "MinimumAbsorbedRadiationDose": {
        "128532": ("Minimum Absorbed Radiation Dose", [10061])
    },
    "MinimumAttenuationCoefficient": {
        "112179": ("Minimum Attenuation Coefficient", [6141])
    },
    "MinimumConformationNumber": {"130012": ("Minimum Conformation Number", [])},
    "MinimumConformityIndex": {"130010": ("Minimum Conformity Index", [])},
    "MinimumEquivalentRadiationDose": {
        "128536": ("Minimum Equivalent Radiation Dose", [10062])
    },
    "MinimumEquivalentUniformDose": {
        "130007": ("Minimum Equivalent Uniform Dose", [9500, 9529])
    },
    "MinimumHealthyTissueConformityIndex": {
        "130011": ("Minimum Healthy Tissue Conformity Index", [])
    },
    "MinimumIntensityProjection": {"113079": ("Minimum intensity projection", [7203])},
    "MinimumMeanRadiationDose": {
        "130005": ("Minimum Mean Radiation Dose", [9500, 9529])
    },
    "MinimumPercentVolumeAtRadiationDose": {
        "130014": ("Minimum Percent Volume at Radiation Dose", [9500, 9530])
    },
    "MinimumRadiationDose": {"130003": ("Minimum Radiation Dose", [9500, 9529])},
    "MinimumSurfaceRadiationDose": {
        "130001": ("Minimum Surface Radiation Dose", [9500, 9529])
    },
    "MitralValveClosure": {"130699": ("Mitral Valve Closure", [12307])},
    "MitralValveOpening": {"130700": ("Mitral Valve Opening", [12307])},
    "MixedModality3DCAMModel": {"129019": ("Mixed Modality 3D CAM model", [7061])},
    "MixedSex": {"127146": ("Mixed sex", [7457])},
    "Modality": {"121139": ("Modality", [])},
    "ModalityToRead": {"128002": ("Modality to Read", [])},
    "ModeAbsorbedRadiationDose": {"128534": ("Mode Absorbed Radiation Dose", [10061])},
    "ModeEquivalentRadiationDose": {
        "128538": ("Mode Equivalent Radiation Dose", [10062])
    },
    "ModelFittingMethod": {"113241": ("Model fitting method", [9000])},
    "ModelFor3DManufacturing": {"M3D": ("Model for 3D Manufacturing", [32, 33])},
    "ModelFree": {"113233": ("Model Free", [7261])},
    "ModelFreeConcentrationTimeQuantitification": {
        "126342": ("Model-free concentration-time quantitification", [4106])
    },
    "ModelMaximumAge": {"128430": ("Model Maximum Age", [])},
    "ModelMaximumHeight": {"128442": ("Model Maximum Height", [])},
    "ModelMaximumWeight": {"128441": ("Model Maximum Weight", [])},
    "ModelMinimumAge": {"128428": ("Model Minimum Age", [])},
    "ModelMinimumHeight": {"128439": ("Model Minimum Height", [])},
    "ModelMinimumWeight": {"128438": ("Model Minimum Weight", [])},
    "ModelPatientSex": {"128437": ("Model Patient Sex", [])},
    "ModifiedHounsfieldUnit": {"129321": ("Modified Hounsfield Unit", [301])},
    "ModifiedSimpson": {"125227": ("Modified Simpson", [12227, 12228])},
    "ModifyingEquipment": {"109103": ("Modifying Equipment", [7005])},
    "Mogamulizumab89Zr": {"126738": ("Mogamulizumab ^89^Zr", [4021])},
    "Mold": {"130113": ("Mold", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "MonoExponentialApparentDiffusionCoefficient": {
        "113290": (
            "Mono-exponential Apparent Diffusion Coefficient",
            [218, 7180, 7272, 7469],
        )
    },
    "MonoExponentialDiffusionModel": {
        "113250": ("Mono-exponential diffusion model", [7273])
    },
    "MonoblockStem": {"112315": ("Monoblock Stem", [7307, 7308])},
    "MonoclonalAntibody64Cu": {"126510": ("Monoclonal Antibody ^64^Cu", [4021])},
    "MonoclonalAntibody89Zr": {"126511": ("Monoclonal Antibody ^89^Zr", [4021])},
    "MonoclonalAntibodyMab64Cu": {"126510": ("Monoclonal Antibody (mAb) ^64^Cu", [])},
    "MonoclonalAntibodyMab89Zr": {"126511": ("Monoclonal Antibody (mAb) ^89^Zr", [])},
    "MonopoleSignal": {"109008": ("Monopole signal", [3240])},
    "MorphologicalOperations": {"123104": ("Morphological Operations", [7162])},
    "MosaicPattern": {"112130": ("Mosaic pattern", [6102, 6106])},
    "MosaicPerfusion": {"112080": ("Mosaic perfusion", [6108])},
    "MostRecentValueChosen": {"121411": ("Most recent value chosen", [224, 12301])},
    "MotherOfFetus": {"121036": ("Mother of fetus", [])},
    "MotionBlur": {"111210": ("Motion blur", [3115, 6041, 6135, 7011])},
    "MotorizedWedge": {"130347": ("Motorized Wedge", [9546])},
    "MouseAlphaSynucleinPreformedFibrils": {
        "127852": ("Mouse alpha synuclein preformed fibrils", [638, 640])
    },
    "MouseTauPreformedFibrils": {"127854": ("Mouse Tau preformed fibrils", [638, 640])},
    "MoveableLaserPatientSetupPoint": {
        "130071": ("Moveable Laser Patient Setup Point", [9504, 9574])
    },
    "MultiEnergyProportionalWeighting": {
        "113097": ("Multi-energy proportional weighting", [7203])
    },
    "MultiMediaCard": {"110035": ("Multi-media Card", [405])},
    "MultiScaleResolutionFiltering": {
        "123106": ("Multi-Scale/Resolution Filtering", [7162])
    },
    "MultiShotEPI": {"113227": ("Multi Shot EPI", [])},
    "MultiSideIllumination": {"130812": ("Multi-side illumination", [11001])},
    "MultiTensor": {"113232": ("Multi Tensor", [7261])},
    "MultifocalIntraductalCarcinoma": {
        "111329": ("Multifocal intraductal carcinoma", [6030, 6033])
    },
    "MultifocalInvasiveDuctalCarcinoma": {
        "111332": ("Multifocal invasive ductal carcinoma", [6030, 6033])
    },
    "MultiparametricMRI": {"126020": ("Multiparametric MRI", [100])},
    "MultiparametricMRIOfProstate": {"126021": ("Multiparametric MRI of prostate", [])},
    "MultiparametricMRIOfWholeBody": {
        "126022": ("Multiparametric MRI of whole body", [100])
    },
    "MultiplanarReformatting": {"113072": ("Multiplanar reformatting", [7203])},
    "MultipleFillingDefect": {"111426": ("Multiple filling defect", [6054, 6057])},
    "MultipleFixedSources": {"130138": ("Multiple Fixed Sources", [9522, 9524])},
    "MultipleIntraductalPapillomas": {
        "111285": ("Multiple Intraductal Papillomas", [6030, 6031])
    },
    "MultipleLADRegionInRAOProjection": {
        "122468": ("Multiple LAD Region in RAO Projection", [3461])
    },
    "MultiplePlanes": {"112714": ("Multiple planes", [])},
    "MultipleRCARegionInRAOProjection": {
        "122469": ("Multiple RCA Region in RAO Projection", [3461])
    },
    "MultipleShotEPI": {"113227": ("Multiple Shot EPI", [7260])},
    "MultipleSlices": {"122562": ("Multiple Slices", [3453])},
    "MultispectralProcessing": {"123108": ("Multispectral Processing", [7162])},
    "MusculoskeletalImagingSpecialty": {
        "128009": ("Musculoskeletal Imaging Specialty", [7449])
    },
    "MusculoskeletalImagingSubjectMatter": {
        "128728": ("Musculoskeletal imaging subject matter", [7017])
    },
    "MusculoskeletalRadiology": {"128175": ("Musculoskeletal Radiology", [7030])},
    "MyocardialBiopsy": {"122057": ("Myocardial biopsy", [3405, 3406])},
    "MyocardialMidlayerMethod": {
        "130685": ("Myocardial Midlayer Method", [12227, 12310])
    },
    "MyocardialPerfusionAnalysis": {"122602": ("Myocardial Perfusion Analysis", [])},
    "Myofibroblastoma": {"111283": ("Myofibroblastoma", [])},
    "MüllerMethodPlanningForHipReplacement": {
        "112344": ("Müller Method Planning for Hip Replacement", [7320])
    },
    "NASCET": {"122655": ("NASCET", [3804])},
    "NAcetylaspartateCholineRatio": {
        "113083": (
            "N-acetylaspartate/Choline Ratio",
            [218, 4032, 4033, 7180, 7186, 7469],
        )
    },
    "NAcetylaspartateCreatineRatio": {
        "113082": (
            "N-acetylaspartate/Creatine Ratio",
            [218, 4032, 4033, 7180, 7186, 7469],
        )
    },
    "NEMAXR212000Phantom": {"113692": ("NEMA XR21-2000 Phantom", [4051, 4052])},
    "NIH07": {"127271": ("NIH07", [607])},
    "NIH31": {"127270": ("NIH31", [607])},
    "NMMyocardialViabilityProcedure": {
        "122785": ("NM Myocardial Viability procedure", [3110])
    },
    "NMTransmissionAttenuationCorrection": {
        "122727": ("NM Transmission Attenuation Correction", [3112])
    },
    "NNonDiagnosticSegments": {"130732": ("N - Non-diagnostic segments", [3022])},
    "NNonDiagnosticStudy": {"130731": ("N - Non-diagnostic study", [3020])},
    "NURBSRadiationTransportModel": {
        "128424": ("NURBS Radiation Transport Model", [10065])
    },
    "NanocolloidalAlbumin89Zr": {"126753": ("Nanocolloidal albumin ^89^Zr", [4021])},
    "NarrativeSummary": {"111412": ("Narrative Summary", [])},
    "NasalCannula": {"127061": ("Nasal cannula", [617, 618])},
    "NationalKennelClub": {"109215": ("National Kennel Club", [7481])},
    "NativePlaqueVolume": {"122375": ("Native Plaque Volume", [3485])},
    "NearPupillaryDistance": {"111680": ("Near Pupillary Distance", [])},
    "Nearline": {"NEARLINE": ("Nearline", [50])},
    "NeckComponent": {"112314": ("Neck Component", [7307, 7308])},
    "NecroticLipidicPlaque": {
        "122395": ("Necrotic-Lipidic Plaque", [3491, 3495, 3497])
    },
    "NeedleGauge": {"111465": ("Needle Gauge", [6095])},
    "NeedleInTarget": {"111438": ("Needle in target", [])},
    "NeedleLength": {"111467": ("Needle Length", [6095])},
    "NeedleLocalizationAndBiopsy": {
        "111144": ("Needle localization and biopsy", [6028, 6029, 6051, 6058, 6061])
    },
    "NegativeDCE": {"130601": ("Negative DCE", [6335, 6345, 6346])},
    "NegativeEnhancementIntegral": {
        "113054": ("Negative enhancement integral", [218, 7180, 7469])
    },
    "NegativeExponential": {"130252": ("Negative exponential", [73])},
    "NegativeInfinity": {"114001": ("Negative Infinity", [42, 43])},
    "NeighborhoodAnalysis": {"123101": ("Neighborhood Analysis", [7162])},
    "NeighbourhoodGreyToneDifferenceMatrix": {
        "128779": ("Neighbourhood Grey Tone Difference Matrix", [])
    },
    "NeighbouringGreyLevelDependenceMatrix": {
        "128780": ("Neighbouring Grey Level Dependence Matrix", [])
    },
    "NeoplasmOfTheMammarySkin": {"111335": ("Neoplasm of the mammary skin", [])},
    "NetForwardVolume": {"122645": ("Net Forward Volume", [])},
    "NetworkConfiguration": {"110128": ("Network Configuration", [401, 403])},
    "NetworkEntry": {"110108": ("Network Entry", [400])},
    "Neurofibromatosis": {"111288": ("Neurofibromatosis", [])},
    "NeuroimagingSubjectMatter": {"128733": ("Neuroimaging subject matter", [7017])},
    "NeurologySpecialty": {"128010": ("Neurology Specialty", [7449])},
    "NeurophysiologyAutomatedAnalysisAnnotations": {
        "130869": ("Neurophysiology Automated Analysis Annotations", [3048])
    },
    "NeurophysiologyPostHocReviewAnnotations": {
        "130868": ("Neurophysiology Post-hoc Review Annotations", [3048])
    },
    "NeurophysiologyRecordingAnnotations": {
        "130867": ("Neurophysiology Recording Annotations", [3048])
    },
    "NeuroradiologicImagingSpecialty": {
        "128011": ("Neuroradiologic Imaging Specialty", [])
    },
    "NeuroradiologyImagingSpecialty": {
        "128011": ("Neuroradiology Imaging Specialty", [7449])
    },
    "NeutralMusculoskeletalPosition": {
        "109136": ("Neutral musculoskeletal position", [92])
    },
    "NifeneF18": {"126714": ("Nifene F^18^", [4021])},
    "NilPerOsNPOStatusConfirmed": {
        "122006": ("Nil Per Os (NPO) status confirmed", [3402])
    },
    "NiobiumOrNiobiumCompound": {"113710": ("Niobium or Niobium compound", [])},
    "NippleCharacteristic": {"111297": ("Nipple Characteristic", [])},
    "NippleDischargeCytology": {"111564": ("Nipple discharge cytology", [6083])},
    "NippleInvolved": {"111472": ("Nipple involved", [])},
    "NippleNotInProfile": {"111205": ("Nipple not in profile", [6041])},
    "NippleRing": {"112177": ("Nipple ring", [6102, 6138, 6404, 7151, 7193])},
    "NoAbnormality": {"111286": ("No abnormality", [6030, 6031])},
    "NoAlgorithmsSucceededWithoutFindings": {
        "111245": ("No algorithms succeeded; without findings", [6047])
    },
    "NoAttenuationCorrection": {"122729": ("No Attenuation Correction", [3112])},
    "NoComplications": {"111492": ("No complications", [6062])},
    "NoCornealCompensation": {"111922": ("No corneal compensation", [4261])},
    "NoCorrelationToClinicalFindings": {
        "111387": ("No correlation to clinical findings", [6158])
    },
    "NoCorrelationToOtherImagingFindings": {
        "111386": ("No correlation to other imaging findings", [6158])
    },
    "NoEmptyTileSuppression": {"112721": ("No empty tile suppression", [8133])},
    "NoFamilyHistoryOfProstateCancer": {
        "130587": ("No family history of prostate cancer", [6322])
    },
    "NoFilter": {"111609": ("No filter", [4204, 8124, 10007])},
    "NoGrid": {"111646": ("No grid", [10017])},
    "NoImage": {"111213": ("No image", [6041, 6135, 7011])},
    "NoKnownExposure": {"111587": ("No known exposure", [6090])},
    "NoPosteriorAcousticFeatures": {
        "111367": ("No posterior acoustic features", [6155])
    },
    "NoRealignment": {"122477": ("No Realignment", [3458])},
    "NoSubsequentWorkitems": {"110009": ("No subsequent Workitems", [9231])},
    "NodeAuthentication": {"110126": ("Node Authentication", [401, 403])},
    "NodeID": {"110182": ("Node ID", [404])},
    "NodularPattern": {"112067": ("Nodular pattern", [6102, 6104, 6106])},
    "Nomenclature": {"127413": ("Nomenclature", [])},
    "NominalEmptyTileSuppression": {
        "112719": ("Nominal empty tile suppression", [8133])
    },
    "NominalImagingSourceLocation": {
        "130789": ("Nominal Imaging Source Location", [9261])
    },
    "NominalRadiationSourceLocation": {
        "130358": ("Nominal Radiation Source Location", [9261, 9544, 9554])
    },
    "NominalSingleCollimationWidth": {
        "113826": ("Nominal Single Collimation Width", [])
    },
    "NominalTotalCollimationWidth": {"113827": ("Nominal Total Collimation Width", [])},
    "NonBloodyDischarge": {"111478": ("Non-bloody discharge (from nipple)", [6055])},
    "NonDiagnosticECG": {"122753": ("Non-diagnostic ECG", [3677])},
    "NonDiagnosticLowHeartRate": {
        "122750": ("Non-diagnostic - low heart rate", [3231])
    },
    "NonDiagnosticRestingSTAbnormalities": {
        "122751": ("Non-diagnostic - resting ST abnormalities", [3231])
    },
    "NonDiagnosticVentricularPacingOrLBBB": {
        "122752": ("Non-diagnostic - ventricular pacing or LBBB", [3231])
    },
    "NonFlatteningFilterBeam": {"130356": ("Non-Flattening Filter Beam", [9549])},
    "NonFocalAbnormality": {"130592": ("Non-focal abnormality", [6335, 6336, 6337])},
    "NonImagingDopplerUltrasoundTransducerGeometry": {
        "125251": ("Non-imaging Doppler ultrasound transducer geometry", [12033])
    },
    "NonIonicIodinatedContrastAgent": {
        "127855": ("Non-ionic iodinated contrast agent", [])
    },
    "NonLesion": {"111102": ("Non-lesion", [6014, 6016, 6054, 6101, 6201])},
    "NonLesionAtBaseline": {"112076": ("Non-Lesion at Baseline", [6145])},
    "NonLesionModifier": {"112037": ("Non-lesion Modifier", [])},
    "NonSpecificVolume": {"130046": ("Non-specific Volume", [9501, 9502, 9580])},
    "NonSynchronizedRoboticTreatment": {
        "130140": ("Non-Synchronized Robotic Treatment", [9523, 9524])
    },
    "NonTargetLesionAtBaseline": {"112075": ("Non-Target Lesion at Baseline", [6145])},
    "NonTargetLesionCompleteResponse": {
        "112045": ("Non-Target Lesion Complete Response", [6143, 6144])
    },
    "NonTargetLesionIncompleteResponseOrStableDisease": {
        "112046": (
            "Non-Target Lesion Incomplete Response or Stable Disease",
            [6143, 6144],
        )
    },
    "NonTargetLesionProgressiveDisease": {
        "112047": ("Non-Target Lesion Progressive Disease", [6143, 6144])
    },
    "NonUterineLeiomyosarcoma": {"130406": ("Non-uterine leiomyosarcoma", [638, 639])},
    "NormalAxillaryNode": {"111251": ("Normal axillary node", [6030, 6031])},
    "NormalBreastTissue": {
        "111287": ("Normal breast tissue", [6030, 6031, 6054, 6057])
    },
    "NormalImplants": {"111503": ("Normal implants", [6072])},
    "NormalIntervalFollowUp": {"111140": ("Normal interval follow-up", [6028, 6029])},
    "NormalMyocardium": {"122112": ("Normal Myocardium", [3704])},
    "NormalRangeAuthority": {"121408": ("Normal Range Authority", [])},
    "NormalRangeDescription": {"121407": ("Normal Range description", [])},
    "Normality": {"121402": ("Normality", [])},
    "NormalizationFactor": {"128522": ("Normalization Factor", [10069])},
    "NormalizedChordLength": {"122450": ("Normalized Chord Length", [])},
    "NormalizedValuesOfVentricularMeasurements": {
        "122609": ("Normalized Values Of Ventricular Measurements", [])
    },
    "NorthAmericanPurebredDogRegistry": {
        "109216": ("North American Purebred Dog Registry", [7481])
    },
    "NoseCone": {"127060": ("Nose cone", [617])},
    "NotANumber": {"114000": ("Not a number", [42, 43])},
    "NotAllAlgorithmsSucceededWithFindings": {
        "111244": ("Not all algorithms succeeded; with findings", [6047])
    },
    "NotAllAlgorithmsSucceededWithoutFindings": {
        "111243": ("Not all algorithms succeeded; without findings", [6047])
    },
    "NotAttempted": {"111225": ("Not Attempted", [6042])},
    "NotForPresentationRenderingDeviceExpectedNotToPresent": {
        "111152": (
            "Not for Presentation: Rendering device expected not to present",
            [6034],
        )
    },
    "NotOptimizedForTheDeviceInstance": {
        "128620": ("Not optimized for the device instance", [800])
    },
    "NotParallel": {"111356": ("Not parallel", [6152])},
    "NotSure": {"111399": ("Not sure", [6164])},
    "NotVisualized": {"122288": ("Not visualized", [3703])},
    "NuclearMedicine": {"NM": ("Nuclear Medicine", [29, 30, 33])},
    "NuclearMedicineImagingSubjectMatter": {
        "128735": ("Nuclear medicine imaging subject matter", [7017])
    },
    "NuclearMedicineProjectionActivity": {
        "110820": ("Nuclear Medicine Projection Activity", [218, 7180, 7469])
    },
    "NuclearMedicineTomographicActivity": {
        "110821": ("Nuclear Medicine Tomographic Activity", [218, 7180, 7469])
    },
    "NumberOfAnimalsWithinSameHousingUnit": {
        "127143": ("Number of animals within same housing unit", [])
    },
    "NumberOfCalcifications": {"111038": ("Number of calcifications", [])},
    "NumberOfDiseasedVesselTerritories": {
        "122762": ("Number of diseased vessel territories", [])
    },
    "NumberOfEctopicBeats": {"122707": ("Number of Ectopic Beats", [])},
    "NumberOfFetuses": {"121038": ("Number of Fetuses", [])},
    "NumberOfFirstDegreeRelativesAffectedByMalignantMelanoma": {
        "130487": (
            "Number of first-degree relatives affected by malignant melanoma",
            [],
        )
    },
    "NumberOfFocalPlanes": {"112707": ("Number of focal planes", [])},
    "NumberOfFractionsCompleted": {"121387": ("Number of Fractions Completed", [])},
    "NumberOfFractionsPlanned": {"121386": ("Number of Fractions Planned", [])},
    "NumberOfFrames": {"121140": ("Number of Frames", [])},
    "NumberOfHousingUnitsPerRack": {"127141": ("Number of housing units per rack", [])},
    "NumberOfImagesUsedForMacularMeasurements": {
        "111691": ("Number of Images Used for Macular Measurements", [])
    },
    "NumberOfInjectorHeads": {"130219": ("Number of Injector Heads", [])},
    "NumberOfLesionInterventionsAttempted": {
        "122175": ("Number of lesion interventions attempted", [])
    },
    "NumberOfLesionInterventionsSuccessful": {
        "122176": ("Number of lesion interventions successful", [])
    },
    "NumberOfMalignantMelanomas": {"130483": ("Number of malignant melanomas", [])},
    "NumberOfMelanomasInSitu": {"130484": ("Number of melanomas in situ", [])},
    "NumberOfNeedlesAroundTarget": {"111439": ("Number of needles around target", [])},
    "NumberOfNodesPositive": {"111474": ("Number of nodes positive", [])},
    "NumberOfNodesRemoved": {"111473": ("Number of nodes removed", [])},
    "NumberOfPasses": {"111436": ("Number of passes", [])},
    "NumberOfPulses": {"113768": ("Number of Pulses", [])},
    "NumberOfRacksPerRoom": {"127140": ("Number of racks per room", [])},
    "NumberOfSamplesUsedPerImage": {"111692": ("Number of Samples Used per Image", [])},
    "NumberOfSimilarFindings": {"111406": ("Number of similar findings", [])},
    "NumberOfSpecimens": {"111437": ("Number of specimens", [])},
    "NumberOfStimulusEvents": {"130494": ("Number of Stimulus Events", [])},
    "NumberOfXRaySources": {"113823": ("Number of X-Ray Sources", [])},
    "NumericalAcousticModel": {"130822": ("Numerical Acoustic Model", [11005])},
    "Nurse": {"121082": ("Nurse", [])},
    "NursingNote": {"121172": ("Nursing Note", [3401])},
    "NursingUnitCancel": {"110511": ("Nursing unit cancel", [9300, 9301])},
    "O10": {"130717": ("O10", [3030])},
    "O9": {"130716": ("O9", [3030])},
    "OBGYNUltrasoundProcedureReport": {
        "125000": ("OB-GYN Ultrasound Procedure Report", [12024])
    },
    "OBGynImagingSpecialty": {"128012": ("OB/Gyn Imaging Specialty", [7449])},
    "OCTAAmplitudeDecorrelation": {"128252": ("OCT-A amplitude decorrelation", [4270])},
    "OCTAComplexVariance": {"128253": ("OCT-A complex variance", [4270])},
    "OCTACorrelationMapping": {"128255": ("OCT-A correlation mapping", [4270])},
    "OCTAOneSidedRatioGreater": {"128305": ("OCT-A one-sided ratio (greater)", [4270])},
    "OCTAOneSidedRatioLesser": {"128304": ("OCT-A one-sided ratio (lesser)", [4270])},
    "OCTASpeckleVariance": {"128254": ("OCT-A speckle variance", [4270])},
    "OCTBScanAnalysis": {"128303": ("OCT B-scan analysis", [7203])},
    "OIQPattern": {"109901": ("OIQ Pattern", [8301])},
    "OLED": {"109994": ("OLED", [8303])},
    "OLINDAEXM": {"113527": ("OLINDA-EXM", [10040])},
    "OSEMAlgorithm": {"122720": ("OSEM algorithm", [3117])},
    "OSLD": {"128706": ("OSLD", [7026, 7027, 7151, 7193])},
    "Obinituzimab89Zr": {"126721": ("Obinituzimab ^89^Zr", [4021])},
    "ObjectRemoved": {"122307": ("Object removed", [3412])},
    "ObjectSecurityAttributesChanged": {
        "110135": ("Object Security Attributes Changed", [401, 403])
    },
    "ObjectSetIncomplete": {"110523": ("Object Set incomplete", [9300, 9302, 9303])},
    "ObjectType": {"111039": ("Object type", [])},
    "ObjectTypesNotSupported": {"110522": ("Object Types not supported", [9300, 9302])},
    "ObjectsIncorrectlyFormatted": {
        "110521": ("Objects incorrectly formatted", [9300, 9302])
    },
    "ObliqueIllumination": {"111746": ("Oblique illumination", [8123])},
    "ObservationByExaminer": {"111846": ("Observation by Examiner", [4253])},
    "ObservationDatetimeQualifier": {"121135": ("Observation DateTime Qualifier", [])},
    "ObserverType": {"121005": ("Observer Type", [])},
    "Ocaratuzumab89Zr": {"126723": ("Ocaratuzumab ^89^Zr", [4021])},
    "OccultBloodTestResult": {"111455": ("Occult blood test result", [])},
    "OccultCarcinomaPresentingWithAxillaryLymphNodeMetastases": {
        "111322": (
            "Occult carcinoma presenting with axillary lymph node metastases",
            [6030, 6033],
        )
    },
    "OcularFixationLight": {"130649": ("Ocular Fixation Light", [9573, 9575])},
    "OcularGazeSetupMethod": {"130635": ("Ocular Gaze Setup Method", [9571])},
    "OfInterest": {"113000": ("Of Interest", [7010])},
    "OfficeCalled": {"122045": ("Office called", [3404])},
    "Offline": {"OFFLINE": ("Offline", [50])},
    "OffsetFactor": {"128523": ("Offset Factor", [10069])},
    "OilCyst": {"111290": ("Oil cyst (fat necrosis cyst)", [6030, 6031])},
    "OldFilmsForComparison": {"111138": ("Old films for comparison", [6028, 6029])},
    "Oligemia": {"112060": ("Oligemia", [6108])},
    "Olsen": {"111765": ("Olsen", [4236])},
    "OnAdmissionToUnit": {"109120": ("On admission to unit", [31])},
    "OnDischarge": {"109121": ("On discharge", [31])},
    "OnDischargeFromUnit": {"109122": ("On discharge from unit", [31])},
    "OncologicImagingSpecialty": {"128013": ("Oncologic Imaging Specialty", [7449])},
    "OncologyMeasurementReport": {"126001": ("Oncology Measurement Report", [7021])},
    "OncologySpecialty": {"128014": ("Oncology Specialty", [7449])},
    "Ongoing": {"111528": ("Ongoing", [])},
    "Online": {"ONLINE": ("Online", [50])},
    "OnsetOfExercise": {"125236": ("Onset of exercise", [12031])},
    "OnsetOfStimulation": {"125238": ("Onset of stimulation", [12031])},
    "Opacity": {"112001": ("Opacity", [6102, 6104])},
    "OpacityDescriptor": {"112027": ("Opacity Descriptor", [])},
    "OperatorDecisionToTerminateTreatment": {
        "130450": ("Operator decision to terminate treatment", [])
    },
    "OperatorError": {"110519": ("Operator Error", [4221, 4222, 9561])},
    "OperatorNarrative": {"109111": ("Operator's narrative", [3000])},
    "OphthalmicAxialMeasurements": {
        "OAM": ("Ophthalmic Axial Measurements", [29, 30, 33])
    },
    "OphthalmicMacularGridProblem": {"111698": ("Ophthalmic Macular Grid Problem", [])},
    "OphthalmicMapping": {"OPM": ("Ophthalmic Mapping", [29, 30, 33])},
    "OphthalmicPhotography": {"OP": ("Ophthalmic Photography", [29, 30, 33])},
    "OphthalmicRefraction": {"OPR": ("Ophthalmic Refraction", [])},
    "OphthalmicTomography": {"OPT": ("Ophthalmic Tomography", [29, 30, 33])},
    "OphthalmicTomographyBScanVolumeAnalysis": {
        "OPTBSV": ("Ophthalmic Tomography B-scan Volume Analysis", [29, 30, 33])
    },
    "OphthalmicTomographyEnFace": {
        "OPTENF": ("Ophthalmic Tomography En Face", [29, 30, 33])
    },
    "OphthalmicVisualField": {"OPV": ("Ophthalmic Visual Field", [29, 30, 33])},
    "OppositeRegionSeverity": {"122461": ("Opposite Region Severity", [])},
    "Optical": {"111752": ("Optical", [])},
    "OpticalCoherenceTomography": {
        "OCT": ("Optical Coherence Tomography", [29, 30, 33])
    },
    "OpticalDistanceMeter": {"130642": ("Optical Distance Meter", [9573, 9575])},
    "OpticalFilterType": {"112712": ("Optical filter type", [])},
    "OpticalFixationMeasurements": {
        "111856": ("Optical Fixation Measurements", [4257])
    },
    "OpticalSurfaceScanner": {
        "OSS": ("Optical Surface Scanner", [29, 30, 33, 9573, 9575])
    },
    "OptimizedForTheDeviceInstance": {
        "128608": ("Optimized for the device instance", [800])
    },
    "OrderRecord": {"110109": ("Order Record", [400])},
    "OrganAtRisk": {"130060": ("Organ At Risk", [9535])},
    "OrganDose": {"113518": ("Organ Dose", [])},
    "OrganDoseInformation": {"113517": ("Organ Dose Information", [])},
    "OrganizationName": {"113873": ("Organization Name", [])},
    "Orientation": {"111354": ("Orientation", [])},
    "OrientationDescriptor": {"112014": ("Orientation Descriptor", [])},
    "OriginalSource": {"111040": ("Original Source", [])},
    "OronaryArteryDiseaseAssessmentModifier": {
        "130721": ("oronary Artery Disease Assessment Modifier", [])
    },
    "OrthogonalLocationArc": {"113669": ("Orthogonal location arc", [6166])},
    "OrthogonalLocationArcInnerMargin": {
        "113670": ("Orthogonal location arc inner margin", [6166])
    },
    "OrthogonalLocationArcOuterMargin": {
        "113671": ("Orthogonal location arc outer margin", [6166])
    },
    "OsmolalityAt37C": {"130184": ("Osmolality at 37C", [])},
    "OsmolarityAt37C": {"130185": ("Osmolarity at 37C", [])},
    "Osseous": {"112053": ("Osseous", [6100])},
    "OsseousModifier": {"112038": ("Osseous Modifier", [])},
    "Other": {"OT": ("Other", [32, 33])},
    "OtherDerivedPlanningData": {"112373": ("Other Derived Planning Data", [])},
    "OtherFailure": {"111220": ("Other failure", [6041, 6135, 7011])},
    "OtherImageOfBiplanePair": {"121314": ("Other image of biplane pair", [7201])},
    "OtherImageOfStereoscopicPair": {
        "121315": ("Other image of stereoscopic pair", [7201])
    },
    "OtherMalignancyType": {"111390": ("Other malignancy type", [6159])},
    "OtherMarker": {"111175": ("Other Marker", [6040, 6401, 7151, 7193])},
    "OtherModality": {"OT": ("Other Modality", [])},
    "OtherPartialViews": {"121313": ("Other partial views", [7201])},
    "OtherProjectionSystem": {"109999": ("Other Projection System", [])},
    "OtherSex": {"121102": ("Other sex", [7455])},
    "OutOfToleranceInterlock": {"130743": ("Out of Tolerance Interlock", [9561, 9568])},
    "OuterLimitsOfFuzzyMargin": {"113663": ("Outer limits of fuzzy margin", [6166])},
    "OuterRetinaStructuralReflectanceMap": {
        "128272": ("Outer retina structural reflectance map", [4271])
    },
    "OuterRetinaVasculatureFlow": {"128271": ("Outer retina vasculature flow", [4271])},
    "OuterSurfaceOfGCL": {"128290": ("Outer surface of GCL", [4273, 7151, 7192, 9514])},
    "OuterSurfaceOfHFL": {"128294": ("Outer surface of HFL", [4273, 7151, 7192, 9514])},
    "OuterSurfaceOfINL": {"128292": ("Outer surface of INL", [4273, 7151, 7192, 9514])},
    "OuterSurfaceOfIPL": {"128291": ("Outer surface of IPL", [4273, 7151, 7192, 9514])},
    "OuterSurfaceOfOPL": {"128293": ("Outer surface of OPL", [4273, 7151, 7192, 9514])},
    "OuterSurfaceOfRNFL": {
        "128289": ("Outer surface of RNFL", [4273, 7151, 7192, 9514])
    },
    "OuterSurfaceOfTheBM": {
        "128300": ("Outer surface of the BM", [4273, 7151, 7192, 9514])
    },
    "OuterSurfaceOfTheCC": {
        "128302": ("Outer surface of the CC", [4273, 7151, 7192, 9514])
    },
    "OutflowAngle": {"122547": ("Outflow Angle", [])},
    "Outline": {"111041": ("Outline", [219])},
    "OutlineOfLobulations": {"113661": ("Outline of lobulations", [6166])},
    "OutlineOfSpiculations": {"113664": ("Outline of spiculations", [6166])},
    "OutputMeasurementPointPosition": {
        "130525": ("Output Measurement Point Position", [])
    },
    "OutsideNormalLimits": {"111847": ("Outside normal limits", [4254])},
    "OutsideOperationalLimitsOfTheDevice": {
        "128619": ("Outside operational limits of the device", [800])
    },
    "OverExposed": {"111212": ("Over exposed", [6041, 6135, 7011])},
    "OverallAssessment": {"111413": ("Overall Assessment", [6052, 6053])},
    "OverallImageQualityEvaluation": {
        "109701": ("Overall image quality evaluation", [8300])
    },
    "OverallImpressionRecommendationAnalysis": {
        "111234": ("Overall Impression/\u200bRecommendation Analysis", [6043])
    },
    "OverallProstateFinding": {"130554": ("Overall Prostate Finding", [])},
    "OverallStudyQuality": {"122739": ("Overall study quality", [])},
    "Overflow": {"114005": ("Overflow", [42, 43])},
    "OxygenAdministrationByVentilator": {
        "121163": ("Oxygen Administration by ventilator", [3531])
    },
    "OxygenAdministrationRate": {"121160": ("Oxygen Administration Rate", [])},
    "OxygenConsumption": {"122239": ("Oxygen Consumption", [])},
    "OxygenExtractionFraction": {
        "126392": ("Oxygen Extraction Fraction", [218, 4108, 7180, 7469])
    },
    "PAPLeakPressure": {"130423": ("PAP Leak Pressure", [3005])},
    "PAPPressure": {"130422": ("PAP Pressure", [3005])},
    "PAPTidalVolume": {"130424": ("PAP Tidal Volume", [3005])},
    "PAS": {"113237": ("PAS", [7261])},
    "PCIDuringThisProcedure": {"122129": ("PCI during this procedure", [])},
    "PETMeasurementReport": {"126003": ("PET Measurement Report", [7021])},
    "PETMyocardialPerfusionRestAndStress": {
        "122793": ("PET Myocardial Perfusion, Rest and Stress", [3106])
    },
    "PETMyocardialPerfusionRestOnly": {
        "122791": ("PET Myocardial Perfusion, Rest only", [3106])
    },
    "PETMyocardialPerfusionStressOnly": {
        "122792": ("PET Myocardial Perfusion, Stress only", [3106])
    },
    "PETMyocardialViabilityRestAndStress": {
        "122797": ("PET Myocardial Viability, Rest and Stress", [3106])
    },
    "PETMyocardialViabilityRestOnly": {
        "122795": ("PET Myocardial Viability, Rest only", [3106])
    },
    "PETMyocardialViabilityStressOnly": {
        "122796": ("PET Myocardial Viability, Stress only", [3106])
    },
    "PETRadionuclideIncubationTime": {
        "126203": ("PET Radionuclide Incubation Time", [])
    },
    "PGreaterThan5Percent": {"111935": ("p>5%", [4265])},
    "PIRADS2Point0ProstateMRIAcquisitionRequirements": {
        "130607": ("PI-RADS 2.0 prostate MRI acquisition requirements", [6326, 6353])
    },
    "PIRADS2Point1ProstateMRIAcquisitionRequirements": {
        "130608": ("PI-RADS 2.1 prostate MRI acquisition requirements", [6326, 6353])
    },
    "PIRADSDCELesionAssessment": {"130563": ("PI-RADS DCE Lesion Assessment", [])},
    "PIRADSDWILesionAssessment": {"130562": ("PI-RADS DWI Lesion Assessment", [])},
    "PIRADSLocalizedAbnormalityAssessment": {
        "130560": ("PI-RADS Localized Abnormality Assessment", [])
    },
    "PIRADST2WILesionAssessment": {"130561": ("PI-RADS T2WI Lesion Assessment", [])},
    "PIRADSV2Point0": {"130564": ("PI-RADS v2.0", [6310])},
    "PIRADSV2Point1": {"130565": ("PI-RADS v2.1", [6310])},
    "PISARadius": {"125331": ("PISA Radius", [12304])},
    "PLesserThan0Point5Percent": {"111939": ("p<0.5%", [4265])},
    "PLesserThan1Percent": {"111938": ("p<1%", [4265])},
    "PLesserThan2Percent": {"111937": ("p<2%", [4265])},
    "PLesserThan5Percent": {"111936": ("p<5%", [4265])},
    "PO10": {"130715": ("PO10", [3030])},
    "PO9": {"130714": ("PO9", [3030])},
    "PSMA1007F18": {"126758": ("PSMA-1007 F^18^", [4021])},
    "PSMA617Ga68": {"126759": ("PSMA-617 Ga^68^", [4021])},
    "PTVNodal": {"130053": ("PTV Nodal", [9534])},
    "PTVPrimary": {"130054": ("PTV Primary", [9534])},
    "PWave": {"109041": ("P wave", [3339])},
    "Paced": {"109083": ("Paced", [3104])},
    "PacedStressTest": {"121438": ("Paced stress test", [])},
    "Pacemaker": {"111171": ("Pacemaker", [])},
    "PacingElectricalStimulusVoltage": {
        "109009": ("Pacing (electrical) stimulus, voltage", [3240])
    },
    "PackageInsert": {"113528": ("Package Insert", [10040])},
    "Paddle": {"111172": ("Paddle", [])},
    "PageSentTo": {"122043": ("Page Sent To", [3404])},
    "PalpationGuided": {"111489": ("Palpation guided", [6058, 6060])},
    "PancreaticStent": {
        "112176": ("Pancreatic stent", [6102, 6138, 6202, 6203, 6404, 7151, 7193])
    },
    "Panitumumab89Zr": {"126736": ("Panitumumab ^89^Zr", [4021])},
    "PanoramicXRay": {"PX": ("Panoramic X-Ray", [29, 30, 33])},
    "PaperBasedBedding": {"127233": ("Paper-based bedding", [605])},
    "PaperDigitized": {"110023": ("Paper Digitized", [7008])},
    "PaperDocument": {"110038": ("Paper Document", [405])},
    "PapillaryCarcinomaInSitu": {"111336": ("Papillary carcinoma in-situ", [])},
    "PapillaryMuscleExcluded": {"122620": ("Papillary Muscle Excluded", [3821])},
    "PapillaryMuscleIncluded": {"122621": ("Papillary Muscle Included", [3821])},
    "PapillaryMuscleIncludedExcluded": {
        "122670": ("Papillary Muscle Included/Excluded", [])
    },
    "Parallel": {"111355": ("Parallel", [6152])},
    "ParallelGrid": {"111644": ("Parallel grid", [10017])},
    "ParallelImaging": {"113228": ("Parallel Imaging", [7260])},
    "Parallelepiped": {"122566": ("Parallelepiped", [3453])},
    "ParaspinalLine": {
        "112091": ("Paraspinal line", [6102, 6109, 6113, 6148, 7151, 7192, 9514])
    },
    "ParenchymalBand": {"112112": ("Parenchymal band", [6102, 6103])},
    "ParentSpecimenIdentifier": {"111705": ("Parent Specimen Identifier", [])},
    "ParentSpecimenType": {"111707": ("Parent specimen type", [])},
    "ParentVesselFinding": {"122686": ("Parent Vessel Finding", [])},
    "PartialFlatteningFilterBeam": {
        "130357": ("Partial Flattening Filter Beam", [9549])
    },
    "PartialRenderingForPresentation": {
        "121333": ("Partial Rendering for Presentation", [7006])
    },
    "PartialReport": {"121363": ("Partial report", [7009])},
    "PartiallyEncapsulated": {"130593": ("Partially encapsulated", [6335, 6341, 6342])},
    "PartiallySucceeded": {"111223": ("Partially Succeeded", [6042])},
    "ParticleDisplacementMethod": {"130756": ("Particle Displacement Method", [12324])},
    "ParticleVelocityMethod": {"130757": ("Particle Velocity Method", [12324])},
    "PatencyTestInjection": {"130247": ("Patency Test Injection", [72])},
    "Path": {"121055": ("Path", [])},
    "PathLength": {"121211": ("Path length", [218, 6165, 7469, 7470])},
    "PathVertex": {"121230": ("Path Vertex", [])},
    "Path_Retired": {"121210": ("Path", [])},
    "Pathology": {"111042": ("Pathology", [])},
    "PathologyResults": {"111468": ("Pathology Results", [])},
    "Patient": {"121025": ("Patient", [271, 272, 7450])},
    "PatientAPDimension": {"128408": ("Patient AP Dimension", [10069])},
    "PatientAdmittedToProcedureRoom": {
        "122002": ("Patient admitted to procedure room", [3402])
    },
    "PatientAdvanceDirectiveGiven": {
        "122005": ("Patient advance directive given", [3402])
    },
    "PatientAlert": {"122025": ("Patient alert", [3402])},
    "PatientAlignmentProcedure": {"130638": ("Patient Alignment Procedure", [9577])},
    "PatientAllergicToMediaContrast": {
        "110503": ("Patient allergic to media/contrast", [64, 9300, 9301])
    },
    "PatientAnatomyModel": {"130067": ("Patient Anatomy Model", [9507])},
    "PatientAsleep": {"122028": ("Patient asleep", [3402])},
    "PatientAssessmentPerformed": {"121165": ("Patient Assessment Performed", [])},
    "PatientAssistedToTable": {"122007": ("Patient assisted to table", [3402])},
    "PatientAttachedDoseControlObject": {
        "130405": ("Patient-Attached Dose Control Object", [9502, 9580])
    },
    "PatientAttenuationCharacteristics": {
        "130529": ("Patient Attenuation Characteristics", [])
    },
    "PatientCalledToProcedureRoom": {
        "122001": ("Patient called to procedure room", [3402])
    },
    "PatientCharacteristics": {"121118": ("Patient Characteristics", [])},
    "PatientClearanceInterlock": {
        "130741": ("Patient Clearance Interlock", [9561, 9568])
    },
    "PatientConditionPreventedContinuing": {
        "110515": ("Patient condition prevented continuing", [60, 9300, 9301, 9561])
    },
    "PatientConnectedToContinuousMonitoring": {
        "122009": ("Patient connected to continuous monitoring", [3402])
    },
    "PatientConsciousness": {"130865": ("Patient Consciousness", [3047])},
    "PatientCoordinateSystemOrigin": {
        "130540": ("Patient Coordinate System Origin", [10074])
    },
    "PatientCoughed": {"122031": ("Patient coughed", [3402])},
    "PatientDataUsedDuringPlanning": {
        "112361": ("Patient Data Used During Planning", [])
    },
    "PatientDecisionToTerminateTreatment": {
        "130451": ("Patient decision to terminate treatment", [])
    },
    "PatientDidNotArrive": {"110507": ("Patient did not arrive", [9300, 9301])},
    "PatientDied": {"110504": ("Patient died", [9300, 9301, 9303, 9561])},
    "PatientDischargedFromDepartment": {
        "122037": ("Patient discharged from department", [3402])
    },
    "PatientDisconnectedFromContinuousMonitoring": {
        "122032": ("Patient disconnected from continuous monitoring", [3402])
    },
    "PatientDisoriented": {"122020": ("Patient disoriented", [3402])},
    "PatientDistractionDevice": {
        "130651": ("Patient Distraction Device", [9573, 9578])
    },
    "PatientElevation": {"130412": ("Patient elevation", [3034])},
    "PatientEquivalentThickness": {"111638": ("Patient Equivalent Thickness", [])},
    "PatientExposureToIonizingRadiation": {
        "121290": ("Patient exposure to ionizing radiation", [])
    },
    "PatientFixationProcedure": {"130637": ("Patient Fixation Procedure", [9577])},
    "PatientGivenPostProcedureInstruction": {
        "122036": ("Patient given post-procedure instruction", [3402])
    },
    "PatientGivenPreProcedureInstruction": {
        "122003": ("Patient given pre-procedure instruction", [3402])
    },
    "PatientHasRespiratoryDifficulty": {
        "122030": ("Patient has respiratory difficulty", [3402])
    },
    "PatientImage": {"112354": ("Patient Image", [])},
    "PatientInformedConsentGiven": {
        "122004": ("Patient informed consent given", [3402])
    },
    "PatientLateralDimension": {"128409": ("Patient Lateral Dimension", [10069])},
    "PatientLocationFiducial": {"128754": ("Patient Location Fiducial", [])},
    "PatientModel": {"113815": ("Patient Model", [])},
    "PatientModelDemographics": {"128427": ("Patient Model Demographics", [])},
    "PatientModelRegistration": {"128456": ("Patient Model Registration", [])},
    "PatientModelType": {"128417": ("Patient Model Type", [])},
    "PatientMotion": {"128553": ("Patient motion", [10034])},
    "PatientMotionInterlock": {"130479": ("Patient Motion Interlock", [9561, 9568])},
    "PatientMotionManagementSetupProcedure": {
        "130639": ("Patient Motion Management Setup Procedure", [9577])
    },
    "PatientMovement": {"110518": ("Patient Movement", [4221, 4222, 9561])},
    "PatientOrientation": {"113743": ("Patient Orientation", [])},
    "PatientOrientationColumn": {"111043": ("Patient Orientation Column", [])},
    "PatientOrientationModifier": {"113744": ("Patient Orientation Modifier", [])},
    "PatientOrientationRow": {"111044": ("Patient Orientation Row", [])},
    "PatientPosition": {"130410": ("Patient position", [3034])},
    "PatientPositioningNote": {"130026": ("Patient Positioning Note", [])},
    "PatientPositioningProblem": {"111209": ("Patient Positioning Problem", [4222])},
    "PatientPositioningProcedureNote": {
        "130035": ("Patient Positioning Procedure Note", [])
    },
    "PatientPregnant": {"110508": ("Patient pregnant", [9300, 9301])},
    "PatientPreppedAndDraped": {"122008": ("Patient prepped and draped", [3402])},
    "PatientPresentation": {"121110": ("Patient Presentation", [])},
    "PatientPronouncedDead": {"122038": ("Patient pronounced dead", [3402])},
    "PatientRadiationDoseModel": {"128500": ("Patient Radiation Dose Model", [])},
    "PatientRadiationDoseModelData": {
        "128425": ("Patient Radiation Dose Model Data", [])
    },
    "PatientRadiationDoseModelReference": {
        "128426": ("Patient Radiation Dose Model Reference", [])
    },
    "PatientRadiationDoseReport": {"128401": ("Patient Radiation Dose Report", [])},
    "PatientRecord": {"110110": ("Patient Record", [400])},
    "PatientRefusedToContinueProcedure": {
        "110505": ("Patient refused to continue procedure", [9300, 9301, 9561])
    },
    "PatientReportsChestPain": {"122023": ("Patient reports chest pain", [3402])},
    "PatientReportsDiscomfort": {"122022": ("Patient reports discomfort", [3402])},
    "PatientReportsNausea": {"122021": ("Patient reports nausea", [3402])},
    "PatientReportsNoPain": {"122024": ("Patient reports no pain", [3402])},
    "PatientRepositioned": {"130455": ("Patient Repositioned", [9563])},
    "PatientRestless": {"122026": ("Patient restless", [3402])},
    "PatientRotationLongitudinal": {
        "130411": ("Patient rotation longitudinal", [3034])
    },
    "PatientSedated": {"122027": ("Patient sedated", [3402])},
    "PatientSegmentedModel": {"128494": ("Patient Segmented Model", [10064])},
    "PatientSetupNote": {"130028": ("Patient Setup Note", [])},
    "PatientSetupPoint": {"130069": ("Patient Setup Point", [9504, 9574])},
    "PatientSetupVerificationInputUsed": {
        "128217": ("Patient Setup Verification Input Used", [7010, 7025])
    },
    "PatientSetupVerificationResult": {
        "128191": ("Patient Setup Verification Result", [7010, 7023])
    },
    "PatientShieldingProcedure": {"130636": ("Patient Shielding Procedure", [9577])},
    "PatientState": {"109054": ("Patient State", [])},
    "PatientStatusOrEvent": {"121123": ("Patient Status or Event", [3401])},
    "PatientSupport": {"128492": ("Patient Support", [10066])},
    "PatientSupportOrigin": {"130538": ("Patient Support Origin", [10074])},
    "PatientTableRelationship": {"113745": ("Patient Table Relationship", [])},
    "PatientTakenForTreatmentOrSurgery": {
        "110506": ("Patient taken for treatment or surgery", [9300, 9301])
    },
    "PatientTransferredFrom": {"122128": ("Patient Transferred From", [])},
    "PatientTransferredToCCU": {"122012": ("Patient transferred to CCU", [3402])},
    "PatientTransferredToHoldingArea": {
        "122010": ("Patient transferred to holding area", [3402])
    },
    "PatientTransferredToMorgue": {"122039": ("Patient transferred to morgue", [3402])},
    "PatientTransferredToSurgery": {
        "122011": ("Patient transferred to surgery", [3402])
    },
    "PatientUnresponsive": {"122029": ("Patient unresponsive", [3402])},
    "PatientWeightExceedsEquipmentLimit": {
        "122764": ("Patient weight exceeds equipment limit", [3205])
    },
    "PatternEvent": {"130860": ("Pattern Event", [3047])},
    "PatternProjection": {"114204": ("Pattern projection", [8201])},
    "PeakActivityWorkload": {"122717": ("Peak activity workload", [])},
    "PeakBloodPressure": {"125329": ("Peak Blood Pressure", [12304])},
    "PeakDoubleProduct": {"122718": ("Peak Double Product", [])},
    "PeakEjectionRate": {"122616": ("Peak Ejection Rate", [])},
    "PeakEjectionTime": {"122617": ("Peak Ejection Time", [])},
    "PeakFillingRate": {"122618": ("Peak Filling Rate", [])},
    "PeakFillingTime": {"122619": ("Peak Filling Time", [])},
    "PeakFlowRateInPhaseActivity": {"130244": ("Peak Flow Rate in Phase Activity", [])},
    "PeakLongitudinalLagrangianStrain": {
        "130671": ("Peak Longitudinal Lagrangian Strain", [12304, 12311])
    },
    "PeakOfThermalCardiacOutputBolus": {
        "109028": ("Peak of thermal cardiac output bolus", [3337])
    },
    "PeakPressureInPhaseActivity": {"130245": ("Peak Pressure in Phase Activity", [])},
    "PeakStressState": {"109095": ("Peak stress state", [])},
    "PeakTissueVelocity": {"125330": ("Peak Tissue Velocity", [12304])},
    "PeakValueWithinROI": {"126031": ("Peak Value Within ROI", [7464])},
    "PectoralMuscleOutline": {"111045": ("Pectoral Muscle Outline", [])},
    "PediatricCardiacUltrasoundReport": {
        "125195": ("Pediatric Cardiac Ultrasound Report", [7000, 12245])
    },
    "PediatricImagingSpecialty": {"128016": ("Pediatric Imaging Specialty", [7449])},
    "PediatricImagingSubjectMatter": {
        "128737": ("Pediatric imaging subject matter", [7017])
    },
    "PediatricRadiology": {"128177": ("Pediatric Radiology", [7030])},
    "Pegdinetanib89Zr": {"126728": ("Pegdinetanib ^89^Zr", [4021])},
    "PelvisAndUterus": {"125011": ("Pelvis and Uterus", [])},
    "PercentFibroglandularTissue": {
        "111046": ("Percent Fibroglandular Tissue", [10069])
    },
    "PercentageOfExpectedBeamOnTimeOfRadiation": {
        "130800": ("Percentage of expected beam-on time of Radiation", [])
    },
    "PercentageOfVascularCalcification": {
        "112198": ("Percentage of Vascular Calcification", [6142])
    },
    "PercentileRankingOfMeasurement": {
        "121415": ("Percentile Ranking of measurement", [221, 227, 7465])
    },
    "PercutaneousCoronaryIntervention": {
        "122061": ("Percutaneous Coronary Intervention", [3739])
    },
    "PercutaneousEntryAction": {"121156": ("Percutaneous Entry Action", [])},
    "PercutaneousSiliconeInjection": {
        "111483": ("Percutaneous silicone injection", [6058, 6059])
    },
    "Perfluorocarbon": {"130605": ("Perfluorocarbon", [6350])},
    "PerformanceOfCTForDetectionOfPulmonaryEmbolismInAdults": {
        "112185": (
            "Performance of CT for Detection of Pulmonary Embolism in Adults",
            [6136],
        )
    },
    "PerformanceOfHighResolutionCTOfTheLungsInAdults": {
        "112186": ("Performance of High-Resolution CT of the Lungs in Adults", [6136])
    },
    "PerformanceOfPediatricAndAdultChestRadiographyACR": {
        "112035": ("Performance of Pediatric and Adult Chest Radiography, ACR", [6136])
    },
    "PerformanceOfPediatricAndAdultThoracicCT": {
        "112184": ("Performance of Pediatric and Adult Thoracic CT", [6136])
    },
    "PerformedImagingAgentAdministration": {
        "130227": ("Performed Imaging Agent Administration", [])
    },
    "PerformedProcedureStep": {"113016": ("Performed Procedure Step", [7012, 10000])},
    "PerformedProcedureStepSOPClassUID": {
        "121127": ("Performed Procedure Step SOP Class UID", [])
    },
    "PerformedProcedureStepSOPInstanceUID": {
        "121126": ("Performed Procedure Step SOP Instance UID", [10001])
    },
    "Performing": {"121094": ("Performing", [7453])},
    "PerformingPhysician": {"121114": ("Performing Physician", [])},
    "Perfusion": {"110831": ("Perfusion", [218, 7180, 7469, 11006])},
    "PerfusionAnalysisByArterialSpinLabelingMRTechnique": {
        "126302": (
            "Perfusion analysis by Arterial Spin Labeling MR technique",
            [4102, 4106],
        )
    },
    "PerfusionAnalysisByIVIodinatedContrastCTTechnique": {
        "126301": (
            "Perfusion analysis by IV Iodinated Contrast CT technique",
            [4102, 4106],
        )
    },
    "PerfusionAnalysisByStableXenonCTTechnique": {
        "126300": ("Perfusion analysis by Stable Xenon CT technique", [4102, 4106])
    },
    "PerfusionAnalysisBySusceptibilityMRTechnique": {
        "126303": ("Perfusion analysis by Susceptibility MR technique", [4102, 4106])
    },
    "PerfusionImageAnalysis": {"129104": ("Perfusion image analysis", [7203])},
    "PeriOperativePhotographicImaging": {
        "112700": ("Peri-operative Photographic Imaging", [8131])
    },
    "PeriProceduralMIOccurred": {"122179": ("Peri-procedural MI occurred", [])},
    "PerilesionalWhiteMatter": {"110706": ("Perilesional White Matter", [7710, 8134])},
    "PerimeterOutline": {"121057": ("Perimeter outline", [])},
    "PerimeterOutline_Retired": {"121213": ("Perimeter Outline", [])},
    "PeripheralDuctPapillomas": {
        "111299": ("Peripheral duct papillomas", [6030, 6032])
    },
    "PeripheralInferior": {"111916": ("Peripheral-inferior", [4207])},
    "PeripheralInferiorNasal": {"111917": ("Peripheral-inferior nasal", [4207])},
    "PeripheralInferiorTemporal": {"111915": ("Peripheral-inferior temporal", [4207])},
    "PeripheralNasal": {"111918": ("Peripheral-nasal", [4207])},
    "PeripheralSuperior": {"111912": ("Peripheral-superior", [4207])},
    "PeripheralSuperiorNasal": {"111919": ("Peripheral-superior nasal", [4207])},
    "PeripheralSuperiorTemporal": {"111913": ("Peripheral-superior temporal", [4207])},
    "PeripheralTemporal": {"111914": ("Peripheral-temporal", [4207])},
    "PerivascularAdiposeTissueFatAttenuationIndex": {
        "130408": ("Perivascular adipose tissue fat attenuation index", [])
    },
    "PersistentDelayedPhaseType1Curve": {
        "130597": ("Persistent delayed phase: Type 1 curve", [6335, 6345, 6346])
    },
    "Person": {"121006": ("Person", [270])},
    "PersonAdministeringDrugContrast": {
        "121152": ("Person administering drug/contrast", [])
    },
    "PersonID": {"113871": ("Person ID", [])},
    "PersonIDIssuer": {"113872": ("Person ID Issuer", [])},
    "PersonName": {"113870": ("Person Name", [])},
    "PersonObserverLoginName": {"128774": ("Person Observer's Login Name", [])},
    "PersonObserverName": {"121008": ("Person Observer Name", [])},
    "PersonObserverOrganizationName": {
        "121009": ("Person Observer's Organization Name", [])
    },
    "PersonObserverRoleInTheOrganization": {
        "121010": ("Person Observer's Role in the Organization", [])
    },
    "PersonObserverRoleInThisProcedure": {
        "121011": ("Person Observer's Role in this Procedure", [])
    },
    "PersonRoleInOrganization": {"113874": ("Person Role in Organization", [])},
    "PersonRoleInProcedure": {"113875": ("Person Role in Procedure", [])},
    "PersonalBreastCancerHistory": {
        "111550": ("Personal breast cancer history", [6081, 6087])
    },
    "PersonalHistoryOfBreastCancerWithBreastConservationTherapy": {
        "111421": (
            "Personal history of breast cancer with breast conservation therapy",
            [6051],
        )
    },
    "PersonalHistoryOfBreastCancerWithMastectomy": {
        "111124": ("Personal history of breast cancer with mastectomy", [6051])
    },
    "PersonnelArrived": {"122041": ("Personnel Arrived", [3404])},
    "PersonnelDeparted": {"122042": ("Personnel Departed", [3404])},
    "Phantom": {"113681": ("Phantom", [])},
    "PhantomTumor": {"112123": ("Phantom tumor (pseudotumor)", [6102, 6104])},
    "PharmacologicAndExerciseStressTest": {
        "121437": ("Pharmacologic and exercise stress test", [])
    },
    "PharmacologicalStressAgentDose": {
        "122715": ("Pharmacological Stress Agent Dose", [])
    },
    "PharmacologicalStressAgentDoseRate": {
        "122705": ("Pharmacological Stress Agent Dose Rate", [])
    },
    "PhaseContrastIllumination": {"111747": ("Phase contrast illumination", [8123])},
    "PhaseOfAnimalHandling": {"127006": ("Phase of animal handling", [])},
    "PhasedBeamSteering": {"125259": ("Phased beam steering", [12034])},
    "Phonocardiogram": {"109114": ("Phonocardiogram", [3000])},
    "Photoacoustic": {"PA": ("Photoacoustic", [29, 30, 33])},
    "PhotogrammetricImaging3DCAMModel": {
        "129020": ("Photogrammetric Imaging 3D CAM model", [7061])
    },
    "PhotonEnergy": {"110857": ("Photon Energy", [7182])},
    "PhotonFixedAperture": {"130344": ("Photon Fixed Aperture", [9541, 9545])},
    "PhotopicLight": {"111629": ("Photopic light", [4203])},
    "PhysicalCompensator": {"130340": ("Physical Compensator", [9542])},
    "PhysicalExaminationResults": {
        "111423": ("Physical Examination Results", [6052, 6053])
    },
    "PhysicalForce": {"109133": ("Physical force", [])},
    "PhysicalSupport": {"128492": ("Physical Support", [])},
    "Physician": {"121081": ("Physician", [])},
    "PhysicianDecisionToTerminateTreatment": {
        "130452": ("Physician decision to terminate treatment", [])
    },
    "PhysicianNote": {"121173": ("Physician Note", [3401])},
    "PhysicianReviewInputUsed": {
        "128213": ("Physician Review Input Used", [7010, 7025])
    },
    "PhysicistReviewInputUsed": {
        "128214": ("Physicist Review Input Used", [7010, 7025])
    },
    "PhysicsAssistant": {"128678": ("Physics Assistant", [9555, 9562])},
    "PhysiologicalAudioSignal": {"109115": ("Physiological audio signal", [3000])},
    "PhysiologicalChallenges": {"109059": ("Physiological challenges", [])},
    "PiezocompositeTransducer": {"130815": ("Piezocomposite Transducer", [11003])},
    "PinatuzumabVedotin89Zr": {"126725": ("Pinatuzumab vedotin ^89^Zr", [4021])},
    "PineChipBedding": {"127234": ("Pine chip bedding", [605])},
    "PineShavingBedding": {"127235": ("Pine shaving bedding", [605])},
    "Pinning": {"112318": ("Pinning", [7310])},
    "PitchFactor": {"113828": ("Pitch Factor", [])},
    "PittsburghCompoundBC11": {"126500": ("Pittsburgh compound B C^11^", [4021])},
    "PixelByPixelAddition": {"113042": ("Pixel by pixel addition", [7203])},
    "PixelByPixelDivision": {"113046": ("Pixel by pixel division", [7203])},
    "PixelByPixelMask": {"113047": ("Pixel by pixel mask", [7203])},
    "PixelByPixelMaximum": {"113048": ("Pixel by pixel Maximum", [7203])},
    "PixelByPixelMean": {"113049": ("Pixel by pixel mean", [7203])},
    "PixelByPixelMinimum": {"113051": ("Pixel by pixel Minimum", [7203])},
    "PixelByPixelMultiplication": {"113053": ("Pixel by pixel multiplication", [7203])},
    "PixelByPixelSubtraction": {"113062": ("Pixel by pixel subtraction", [7203])},
    "PixelDataColumns": {"110911": ("Pixel Data Columns", [])},
    "PixelDataRows": {"110910": ("Pixel Data Rows", [])},
    "PixelFaultsEvaluation": {"109706": ("Pixel faults evaluation", [8300])},
    "PixelatedSpiculations": {"113666": ("Pixelated spiculations", [6166])},
    "PlacerNumber": {"121020": ("Placer Number", [])},
    "Plan": {"PLAN": ("Plan", [32, 33])},
    "PlanComparisonInputUsed": {"128220": ("Plan Comparison Input Used", [7010])},
    "PlanQualityAssuranceInputUsed": {
        "128215": ("Plan Quality Assurance Input Used", [7010, 7025])
    },
    "PlanarMatrixUltrasoundTransducerGeometry": {
        "130808": ("Planar matrix ultrasound transducer geometry", [12033])
    },
    "PlaneA": {"113620": ("Plane A", [10003])},
    "PlaneB": {"113621": ("Plane B", [10003])},
    "PlaneScanPattern": {"125241": ("Plane scan pattern", [12032])},
    "PlaneThroughAnteriorExtent": {"128128": ("Plane through Anterior Extent", [1010])},
    "PlaneThroughCenter": {"128130": ("Plane through Center", [1010])},
    "PlaneThroughDistalExtent": {"128123": ("Plane through Distal Extent", [1010])},
    "PlaneThroughInferiorExtent": {"128121": ("Plane through Inferior Extent", [1010])},
    "PlaneThroughLateralExtent": {"128125": ("Plane through Lateral Extent", [1010])},
    "PlaneThroughLeftmostExtent": {"128126": ("Plane through Leftmost Extent", [1010])},
    "PlaneThroughMedialExtent": {"128124": ("Plane through Medial Extent", [1010])},
    "PlaneThroughPosteriorExtent": {
        "128129": ("Plane through Posterior Extent", [1010])
    },
    "PlaneThroughProximalExtent": {"128122": ("Plane through Proximal Extent", [1010])},
    "PlaneThroughRightmostExtent": {
        "128127": ("Plane through Rightmost Extent", [1010])
    },
    "PlaneThroughSuperiorExtent": {"128120": ("Plane through Superior Extent", [1010])},
    "Planimetry": {"125220": ("Planimetry", [12227, 12229])},
    "PlannedImagingAgentAdministration": {
        "130226": ("Planned Imaging Agent Administration", [])
    },
    "PlannedImagingAgentAdministrationProcedureReport": {
        "130182": ("Planned Imaging Agent Administration Procedure Report", [])
    },
    "PlannedImagingAgentAdministrationSOPInstance": {
        "130236": ("Planned Imaging Agent Administration SOP Instance", [])
    },
    "PlanningImagingNote": {"130030": ("Planning Imaging Note", [])},
    "PlanningInformationForIntraoperativeUsage": {
        "112367": ("Planning Information for Intraoperative Usage", [])
    },
    "PlanningIntent": {"129013": ("Planning Intent", [7064])},
    "PlanningMethod": {"112375": ("Planning Method", [])},
    "PlanningOrganAtRiskVolume": {"130057": ("Planning Organ At Risk Volume", [9535])},
    "PlaqueArea": {"122542": ("Plaque Area", [])},
    "PlaqueBurden": {"122354": ("Plaque Burden", [])},
    "PlaquePlusMediaCrossSectionalArea": {
        "122334": ("Plaque plus Media Cross-Sectional Area", [3482])
    },
    "PlaquePlusMediaEccentricityIndex": {
        "122344": ("Plaque plus Media Eccentricity Index", [3484])
    },
    "PlaquePlusMediaThickness": {"122331": ("Plaque Plus Media Thickness", [3481])},
    "PlaqueRupture": {"122363": ("Plaque Rupture", [3491])},
    "PlasmaDisplay": {"109993": ("Plasma Display", [8303])},
    "PlateauDelayedPhaseType2Curve": {
        "130598": ("Plateau delayed phase: Type 2 curve", [6335, 6345, 6346])
    },
    "Pleonemia": {"112081": ("Pleonemia", [6108])},
    "PointCloudAlgorithmic": {"114208": ("Point Cloud Algorithmic", [8201])},
    "PointShearWaveElastography": {
        "130616": ("Point Shear Wave Elastography", [6058, 12224])
    },
    "PoiseuilleResistance": {"122549": ("Poiseuille Resistance", [])},
    "PolarToRectangularScanConversion": {
        "113093": ("Polar to Rectangular Scan Conversion", [7203])
    },
    "PolarizingFilter": {"111607": ("Polarizing filter", [])},
    "PolatuzumabVedotin89Zr": {"126726": ("Polatuzumab vedotin ^89^Zr", [4021])},
    "PolypStalkWidth": {"112232": ("Polyp stalk width", [6212])},
    "PolyurethaneImplant": {"111482": ("Polyurethane implant", [6058, 6059])},
    "PoorContrastTiming": {"130579": ("Poor contrast timing", [6315, 6317])},
    "PoorFOVSelection": {"130577": ("Poor FOV selection", [6315])},
    "PoorImageQuality": {"122742": ("Poor image quality", [3114])},
    "PoorSNR": {"130578": ("Poor SNR", [6315])},
    "PoorlyDemarcated": {"112141": ("Poorly demarcated", [6120])},
    "PopulationAveragedAIF": {"126361": ("Population Averaged AIF", [4103, 4106])},
    "PopulationDescription": {"121405": ("Population description", [])},
    "PortableMediaImporterEquipment": {
        "MEDIM": ("Portable Media Importer Equipment", [7005])
    },
    "Portacath": {"112172": ("Portacath", [6102, 6138, 6404, 7151, 7193])},
    "PositionFrameOfReference": {"111708": ("Position Frame of Reference", [])},
    "PositionOfDistalBorder": {"122529": ("Position of Distal Border", [])},
    "PositionOfProximalBorder": {"122528": ("Position of Proximal Border", [])},
    "PositionReferenceIndicator": {"127451": ("Position reference indicator", [])},
    "PositionSensor": {"POS": ("Position Sensor", [29, 30, 33])},
    "PositionerIsocenterDetectorRotationAngle": {
        "128759": ("Positioner Isocenter Detector Rotation Angle", [])
    },
    "PositionerIsocenterDetectorRotationEndAngle": {
        "128762": ("Positioner Isocenter Detector Rotation End Angle", [])
    },
    "PositionerIsocenterPrimaryAngle": {
        "128757": ("Positioner Isocenter Primary Angle", [])
    },
    "PositionerIsocenterPrimaryEndAngle": {
        "128760": ("Positioner Isocenter Primary End Angle", [])
    },
    "PositionerIsocenterSecondaryAngle": {
        "128758": ("Positioner Isocenter Secondary Angle", [])
    },
    "PositionerIsocenterSecondaryEndAngle": {
        "128761": ("Positioner Isocenter Secondary End Angle", [])
    },
    "PositionerPrimaryAngle": {"112011": ("Positioner Primary Angle", [])},
    "PositionerPrimaryEndAngle": {"113739": ("Positioner Primary End Angle", [])},
    "PositionerSecondaryAngle": {"112012": ("Positioner Secondary Angle", [])},
    "PositionerSecondaryEndAngle": {"113740": ("Positioner Secondary End Angle", [])},
    "Positioning": {"111209": ("Positioning", [6041, 6135, 7011])},
    "PositiveDCE": {"130600": ("Positive DCE", [6335, 6345, 6346])},
    "PositiveInfinity": {"114002": ("Positive Infinity", [42, 43])},
    "PositronEmissionTomography": {
        "PT": ("Positron emission tomography", [29, 30, 33])
    },
    "PostAdministrationMeasuredActivity": {
        "113509": ("Post-Administration Measured Activity", [])
    },
    "PostCoordinatedMeasurements": {"125302": ("Post-coordinated Measurements", [])},
    "PostHyperventilation": {"130415": ("Post-hyperventilation", [3038])},
    "PostIntervention": {"109124": ("Post-intervention", [31])},
    "PostInterventionStenosisMeasurement": {
        "122108": ("Post-Intervention Stenosis Measurement", [])
    },
    "PostInterventionTIMIFlow": {"122110": ("Post-Intervention TIMI Flow", [])},
    "PostMenopausalPatient": {"111554": ("Post menopausal patient", [6081, 6087])},
    "PostProcedureMammogramsForMarkerPlacement": {
        "111120": ("Post Procedure Mammograms for Marker Placement", [6026])
    },
    "PostReductionMammoplasty": {
        "111291": ("Post reduction mammoplasty", [6030, 6031])
    },
    "PostVoiding": {"109135": ("Post voiding", [91, 9272])},
    "PosteriorAcousticFeatures": {"111366": ("Posterior acoustic features", [])},
    "PosteriorCorneaSurfaceMeasurementMethod": {
        "111759": ("Posterior Cornea Surface Measurement Method", [4242])
    },
    "PosteriorEnhancement": {"111368": ("Posterior enhancement", [6155])},
    "PosteriorFirst": {"126832": ("posterior first", [21])},
    "PosteriorJunctionLine": {
        "112089": ("Posterior junction line", [6102, 6109, 6112, 7151, 7192, 9514])
    },
    "PosteriorLens": {"111779": ("Posterior Lens", [4233])},
    "PosteriorShadowing": {"111369": ("Posterior shadowing", [6155])},
    "PosteriorSurfaceOfTheRPE": {
        "128299": ("Posterior surface of the RPE", [4273, 7151, 7192, 9514])
    },
    "PosteriorToAnterior": {"110871": ("Posterior To Anterior", [7185])},
    "PosteriorTrachealStripe": {
        "112092": (
            "Posterior tracheal stripe",
            [6102, 6109, 6113, 6148, 7151, 7192, 9514],
        )
    },
    "Posttreatment": {"126074": ("Posttreatment", [6146])},
    "PowerDoppler": {"125230": ("Power Doppler", [])},
    "PreAdministrationMeasuredActivity": {
        "113508": ("Pre-Administration Measured Activity", [])
    },
    "PreBiopsyLocalizationOfProstateLesion": {
        "130588": ("Pre-biopsy localization of prostate lesion", [6327])
    },
    "PreCoordinatedMeasurements": {"125301": ("Pre-coordinated Measurements", [])},
    "PreEjectionPeriod": {"125321": ("Pre-ejection Period", [12307])},
    "PreHeatedPad": {"127254": ("Pre-heated pad", [635])},
    "PreIntervention": {"109123": ("Pre-intervention", [31])},
    "PreMedication": {"123012": ("Pre-Medication", [76])},
    "PrePectoralImplant": {"111485": ("Pre-pectoral implant", [6058, 6059])},
    "PrePlanningResult": {"128184": ("Pre-Planning Result", [7010, 7023])},
    "PreProcedureLog": {"122072": ("Pre-procedure log", [3407])},
    "PreSurgicalAnatomy": {"130833": ("Pre-surgical anatomy", [9272])},
    "PreamplifierConnected": {"130889": ("Preamplifier connected", [3039])},
    "PreamplifierDisconnected": {"130890": ("Preamplifier disconnected", [3039])},
    "PreclinicalSmallAnimalImagingAcquisitionContext": {
        "127001": ("Preclinical Small Animal Imaging Acquisition Context", [])
    },
    "PredecessorContainingGroupOfImagingSubjects": {
        "113130": ("Predecessor containing group of imaging subjects", [7202])
    },
    "PregnancyStatus": {"111532": ("Pregnancy Status", [])},
    "PreliminaryReport": {"121362": ("Preliminary report", [7009, 9233])},
    "PreparationForImaging": {"127104": ("Preparation for imaging", [634])},
    "PreparationRoom": {"127371": ("Preparation room", [603])},
    "PrescriptionIdentifier": {"113516": ("Prescription Identifier", [])},
    "PrescriptionRadiationDose": {
        "130009": ("Prescription Radiation Dose", [9500, 9529])
    },
    "PrescriptionVisualAcuity": {"111687": ("Prescription Visual Acuity", [4216])},
    "PresenceOfProperty": {"121052": ("Presence of property", [])},
    "PresenceUndetermined": {"121059": ("Presence Undetermined", [])},
    "Present": {"121053": ("Present", [])},
    "PresentationOptionalRenderingDeviceMayPresent": {
        "111151": ("Presentation Optional: Rendering device may present", [6034])
    },
    "PresentationRequiredRenderingDeviceIsExpectedToPresent": {
        "111150": (
            "Presentation Required: Rendering device is expected to present",
            [6034],
        )
    },
    "PresentationState": {"PR": ("Presentation State", [32, 33])},
    "PressureAboveAdjustmentLimit": {
        "130151": ("Pressure above adjustment limit", [71])
    },
    "PressureAboveWarningLimit": {"130150": ("Pressure above warning limit", [71])},
    "PressureAtDpDtMax": {"122199": ("Pressure at dp/dt max", [3611])},
    "PressureDropAtSFR": {"122551": ("Pressure Drop at SFR", [])},
    "PressureLimit": {"130193": ("Pressure Limit", [])},
    "PressureVsTime": {"130230": ("Pressure vs Time", [])},
    "PreviousFetalMalformationSyndrome": {
        "111570": ("Previous fetal malformation/syndrome", [6087, 6088])
    },
    "PreviousFinding": {"121069": ("Previous Finding", [7002])},
    "PreviousFindings": {"121068": ("Previous Findings", [])},
    "PreviousLBWOrIUGRBirth": {"111569": ("Previous LBW or IUGR birth", [6087, 6088])},
    "PreviousProcedure": {"111531": ("Previous Procedure", [])},
    "PreviousRHNegativeOrBloodDyscrasiaAtBirth": {
        "111571": ("Previous RH negative or blood dyscrasia at birth", [6087, 6088])
    },
    "PreviousReports": {"111549": ("Previous Reports", [])},
    "PreviousTreatmentNote": {"130029": ("Previous Treatment Note", [])},
    "PrimaryComplex": {"112059": ("Primary complex", [6102, 6104])},
    "PrimaryFluenceMonitoringSystemInterlock": {
        "130475": ("Primary Fluence Monitoring System Interlock", [9561, 9568])
    },
    "PrimaryInterventionDevice": {"122111": ("Primary Intervention Device", [])},
    "PrimaryLevel": {"128725": ("Primary level", [7016])},
    "Print": {"110008": ("Print", [9231])},
    "PriorProcedureDescriptions": {"121066": ("Prior Procedure Descriptions", [])},
    "PriorReportForCurrentPatient": {
        "122075": ("Prior report for current patient", [3407])
    },
    "PriorToVoiding": {"109134": ("Prior to voiding", [91, 9272])},
    "Probabilistic": {"113212": ("Probabilistic", [7262])},
    "ProbabilisticTrackingAlgorithm": {
        "113212": ("Probabilistic Tracking Algorithm", [])
    },
    "Probability": {"122157": ("Probability", [])},
    "ProbabilityOfCancer": {"111047": ("Probability of cancer", [6048])},
    "ProcedureActionDuration": {"121128": ("Procedure Action Duration", [])},
    "ProcedureActionItemID": {"121124": ("Procedure Action Item ID", [])},
    "ProcedureCharacteristics": {"130530": ("Procedure Characteristics", [])},
    "ProcedureCode": {"121023": ("Procedure Code", [])},
    "ProcedureDatetime": {"122146": ("Procedure DateTime", [])},
    "ProcedureDescription": {"121065": ("Procedure Description", [7002])},
    "ProcedureEnvironmentalCharacteristics": {
        "122222": ("Procedure Environmental Characteristics", [])
    },
    "ProcedureLogging": {"LOG": ("Procedure Logging", [30])},
    "ProcedureModifier": {"111464": ("Procedure Modifier", [])},
    "ProcedureNote": {"121174": ("Procedure Note", [3401])},
    "ProcedureRecord": {"110111": ("Procedure Record", [400])},
    "ProcedureReported": {"121058": ("Procedure reported", [6052, 6053])},
    "ProcedureResult": {"122177": ("Procedure Result", [])},
    "ProcedureStepNumber": {"109060": ("Procedure Step Number", [])},
    "ProcedureStepToThisPoint": {"113970": ("Procedure Step To This Point", [10000])},
    "ProcedureStudyComponentUID": {"121019": ("Procedure Study Component UID", [])},
    "ProcedureStudyInstanceUID": {"121018": ("Procedure Study Instance UID", [])},
    "ProcedureTimeBase": {"122701": ("Procedure time base", [])},
    "ProcessedKeyObjectSelection": {
        "130753": ("Processed Key Object Selection", [7031])
    },
    "ProcessingEquipment": {"109102": ("Processing Equipment", [7005])},
    "ProcessingStepDescription": {"111703": ("Processing step description", [])},
    "ProcessingType": {"111701": ("Processing type", [])},
    "Profusion": {"112151": ("Profusion", [6123])},
    "ProgesteroneReceptor": {"111476": ("Progesterone receptor", [])},
    "ProgrammableInjectorDevice": {"130218": ("Programmable Injector Device", [])},
    "ProgrammedHoldStarted": {"130266": ("Programmed hold started", [71])},
    "ProjectionEponymousName": {"113946": ("Projection Eponymous Name", [])},
    "ProjectionXRay": {"113704": ("Projection X-Ray", [])},
    "ProliferativeActivity": {"126036": ("Proliferative Activity", [7466])},
    "ProspectiveGating": {"109081": ("Prospective gating", [3104])},
    "ProstateImagingFindings": {"130553": ("Prostate Imaging Findings", [])},
    "ProstateMRIRelevantProcedureInformation": {
        "130552": ("Prostate MRI relevant procedure information", [])
    },
    "ProstateRelationalMeasurements": {
        "130556": ("Prostate relational measurements", [])
    },
    "ProstheticFabrication": {"129015": ("Prosthetic Fabrication", [7064])},
    "ProstheticShaftAugment": {"112316": ("Prosthetic Shaft Augment", [7307, 7308])},
    "ProtocolNotFollowed": {"130570": ("Protocol not followed", [6314])},
    "ProtocolStage": {"109055": ("Protocol Stage", [])},
    "ProtocolTimePointIdentifier": {"126071": ("Protocol Time Point Identifier", [])},
    "ProtonDensity": {"113058": ("Proton Density", [218, 7180, 7469])},
    "ProtonDensityWeightedMRSignalIntensity": {
        "110803": ("Proton Density Weighted MR Signal Intensity", [218, 7180, 7469])
    },
    "ProximalFindingSite": {"121116": ("Proximal Finding Site", [])},
    "ProximalIsovelocitySurfaceArea": {
        "125216": ("Proximal Isovelocity Surface Area", [12227, 12229, 12231])
    },
    "ProximalReference": {"122380": ("Proximal Reference", [3486])},
    "ProximalStentMargin": {"122385": ("Proximal Stent Margin", [3487])},
    "PseudoangiomatousStromalHyperplasia": {
        "111292": ("Pseudoangiomatous stromal hyperplasia", [6030, 6031])
    },
    "Pseudoplaque": {"112068": ("Pseudoplaque", [6102, 6104])},
    "PulmonaryArterialContentFcpa": {
        "122188": ("Pulmonary Arterial Content (FCpa)", [3529])
    },
    "PulmonaryFlow": {"122161": ("Pulmonary Flow", [3618])},
    "PulmonaryImagingSubjectMatter": {
        "128729": ("Pulmonary imaging subject matter", [7017])
    },
    "PulmonaryVenousContentFcpv": {
        "122189": ("Pulmonary Venous Content (FCpv)", [3529])
    },
    "PulseRate": {"113791": ("Pulse Rate", [])},
    "PulseSequenceName": {"128230": ("Pulse Sequence Name", [])},
    "PulseStrength": {"122195": ("Pulse Strength", [])},
    "PulseTrainDefinition": {"109063": ("Pulse train definition", [])},
    "PulseWidth": {"113793": ("Pulse Width", [])},
    "Pulsed": {"113631": ("Pulsed", [10004])},
    "QANode": {"130366": ("QA Node", [9556])},
    "QANodePair": {"130365": ("QA Node Pair", [9556])},
    "QAProcessNote": {"130036": ("QA Process Note", [])},
    "QWave": {"109042": ("Q wave", [3339])},
    "QuadrantLocation": {"111048": ("Quadrant location", [])},
    "QuadraticCellKillFactor": {"130089": ("Quadratic Cell Kill Factor", [])},
    "QualitativeAnalysis": {"122145": ("Qualitative Analysis", [])},
    "QualitativeDifference": {"111049": ("Qualitative Difference", [])},
    "QualityAssessment": {"111050": ("Quality Assessment", [])},
    "QualityControl": {"110002": ("Quality Control", [6051, 6058, 6061, 9231])},
    "QualityControlIntent": {"113680": ("Quality Control Intent", [3629, 7031, 7064])},
    "QualityControlMaterialImage": {
        "130750": ("Quality control material image", [7201])
    },
    "QualityControlStandard": {"111051": ("Quality Control Standard", [])},
    "QualityFinding": {"111052": ("Quality Finding", [])},
    "QualityIssue": {"113010": ("Quality Issue", [7010])},
    "QuantitativeAnalysis": {"122144": ("Quantitative Analysis", [])},
    "QuantitativeArteriographyReport": {
        "122291": ("Quantitative Arteriography Report", [])
    },
    "QuantitativeVentriculographyReport": {
        "122292": ("Quantitative Ventriculography Report", [])
    },
    "QuantityAdministered": {"121382": ("Quantity administered", [3410])},
    "QuantityOfMaterial": {"121146": ("Quantity of Material", [])},
    "Query": {"110112": ("Query", [400])},
    "QuotationMode": {"121001": ("Quotation Mode", [])},
    "QuotedSource": {"121002": ("Quoted Source", [])},
    "R1": {"126393": ("R1", [218, 7180, 7469])},
    "R150789Zr": {"126518": ("R1507 ^89^Zr", [4021])},
    "R2": {"126394": ("R2", [218, 7180, 7469])},
    "R2Coefficient": {"126220": ("R2-Coefficient", [218, 7180, 7469])},
    "R2Star": {"126395": ("R2*", [218, 7180, 7469])},
    "RCARegionInLAOProjection": {"122471": ("RCA Region in LAO Projection", [3461])},
    "RCARegionInRAOProjection": {"122465": ("RCA Region in RAO Projection", [3460])},
    "RCARegionInROAProjection": {"122465": ("RCA Region in ROA Projection", [])},
    "RCoefficient": {"113057": ("R-Coefficient", [218, 7180, 7469])},
    "RDSRFrameOfReferenceDescription": {
        "130507": ("RDSR Frame of Reference Description", [])
    },
    "RDSRFrameOfReferenceOrigin": {"130506": ("RDSR Frame of Reference Origin", [])},
    "RECIST": {"112022": ("RECIST", [])},
    "RECIST1Point0": {"126080": ("RECIST 1.0", [6147])},
    "RECIST1Point1": {"126081": ("RECIST 1.1", [6147])},
    "RGBBComponent": {"110836": ("RGB B Component", [218, 7180, 7186, 7469])},
    "RGBGComponent": {"110835": ("RGB G Component", [218, 7180, 7186, 7469])},
    "RGBRComponent": {"110834": ("RGB R Component", [218, 7180, 7186, 7469])},
    "RO532344189Zr": {"126755": ("RO5323441 ^89^Zr", [4021])},
    "RO54290889Zr": {"126756": ("RO542908 ^89^Zr", [4021])},
    "RO692496311C": {"126719": ("RO6924963 ^11^C", [4021])},
    "RO693164311C": {"126720": ("RO6931643 ^11^C", [4021])},
    "RO695894818F": {"126757": ("RO6958948 ^18^F", [4021])},
    "ROIDepth": {"130613": ("ROI Depth", [12308])},
    "RPLGD": {"128709": ("RPLGD", [7026, 7027, 7151, 7193])},
    "RRInterval": {
        "122182": (
            "R-R interval",
            [
                3613,
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "RTBeamEnergy": {"130034": ("RT Beam Energy", [])},
    "RTBrachyTreatment": {"121735": ("RT Brachy Treatment", [9241])},
    "RTDose": {"RTDOSE": ("RT Dose", [32, 33])},
    "RTDoseCalculationStructure": {
        "130042": ("RT Dose Calculation Structure", [9502, 9503])
    },
    "RTGeometricInformation": {"130043": ("RT Geometric Information", [9502, 9580])},
    "RTImage": {"RTIMAGE": ("RT Image", [29, 30, 33])},
    "RTImageGuidedPatientPositioningAndTreatmentDelivery": {
        "121736": ("RT Image-Guided Patient Positioning and Treatment Delivery", [9241])
    },
    "RTMachineQA": {"121730": ("RT Machine QA", [9241])},
    "RTPatientPositionAcquisitionCTKv": {
        "121707": ("RT Patient Position Acquisition, CT kV", [9242, 9260, 9263, 9266])
    },
    "RTPatientPositionAcquisitionCTMV": {
        "121708": ("RT Patient Position Acquisition, CT MV", [9242, 9260, 9264, 9266])
    },
    "RTPatientPositionAcquisitionConeBeamCTKv": {
        "130785": (
            "RT Patient Position Acquisition, Cone-Beam CT kV",
            [9242, 9260, 9263, 9266],
        )
    },
    "RTPatientPositionAcquisitionConeBeamCTMV": {
        "130787": (
            "RT Patient Position Acquisition, Cone-Beam CT MV",
            [9242, 9260, 9264, 9266],
        )
    },
    "RTPatientPositionAcquisitionConventionalCTKv": {
        "130786": (
            "RT Patient Position Acquisition, Conventional CT kV",
            [9242, 9260, 9263, 9266],
        )
    },
    "RTPatientPositionAcquisitionConventionalCTMV": {
        "130788": (
            "RT Patient Position Acquisition, Conventional CT MV",
            [9242, 9260, 9264, 9266],
        )
    },
    "RTPatientPositionAcquisitionDualPlaneKv": {
        "121705": ("RT Patient Position Acquisition, dual plane kV", [9260])
    },
    "RTPatientPositionAcquisitionDualPlaneKvMV": {
        "121706": ("RT Patient Position Acquisition, dual plane kV/MV", [9260])
    },
    "RTPatientPositionAcquisitionDualPlaneMV": {
        "121703": ("RT Patient Position Acquisition, dual plane MV", [9260])
    },
    "RTPatientPositionAcquisitionFilmCassetteKv": {
        "130784": (
            "RT Patient Position Acquisition, Film Cassette kV",
            [9242, 9260, 9263, 9265],
        )
    },
    "RTPatientPositionAcquisitionFilmCassetteMV": {
        "130783": (
            "RT Patient Position Acquisition, Film Cassette MV",
            [9242, 9260, 9264, 9265],
        )
    },
    "RTPatientPositionAcquisitionIntegratedDoseMV": {
        "130782": (
            "RT Patient Position Acquisition, Integrated Dose MV",
            [9242, 9260, 9264, 9265],
        )
    },
    "RTPatientPositionAcquisitionMR": {
        "121737": ("RT Patient Position Acquisition, MR", [9242, 9273])
    },
    "RTPatientPositionAcquisitionOptical": {
        "121709": ("RT Patient Position Acquisition, Optical", [9242])
    },
    "RTPatientPositionAcquisitionSinglePlaneKv": {
        "121704": (
            "RT Patient Position Acquisition, single plane kV",
            [9242, 9260, 9263, 9265],
        )
    },
    "RTPatientPositionAcquisitionSinglePlaneMV": {
        "121702": (
            "RT Patient Position Acquisition, single plane MV",
            [9242, 9260, 9264, 9265],
        )
    },
    "RTPatientPositionAcquisitionSpatialFiducials": {
        "121711": ("RT Patient Position Acquisition, Spatial Fiducials", [9242])
    },
    "RTPatientPositionAcquisitionUltrasound": {
        "121710": ("RT Patient Position Acquisition, Ultrasound", [9242])
    },
    "RTPatientPositionAdjustment": {
        "121722": ("RT Patient Position Adjustment", [9241])
    },
    "RTPatientPositionInTreatmentSessionReview": {
        "121723": ("RT Patient Position In-treatment-session Review", [9241])
    },
    "RTPatientPositionRegistration2DOn3DReference": {
        "121718": ("RT Patient Position Registration, 2D on 3D reference", [9243])
    },
    "RTPatientPositionRegistration3DCTGeneral": {
        "121714": ("RT Patient Position Registration, 3D CT general", [9243])
    },
    "RTPatientPositionRegistration3DCTMarkerBased": {
        "121715": ("RT Patient Position Registration, 3D CT marker-based", [9243])
    },
    "RTPatientPositionRegistration3DCTVolumeBased": {
        "121716": ("RT Patient Position Registration, 3D CT volume-based", [9243])
    },
    "RTPatientPositionRegistration3DOn2DReference": {
        "121717": ("RT Patient Position Registration, 3D on 2D reference", [9243])
    },
    "RTPatientPositionRegistrationDualPlane": {
        "121713": ("RT Patient Position Registration, dual plane", [9243])
    },
    "RTPatientPositionRegistrationMR": {
        "121738": ("RT Patient Position Registration, MR", [9243])
    },
    "RTPatientPositionRegistrationOptical": {
        "121719": ("RT Patient Position Registration, Optical", [9243])
    },
    "RTPatientPositionRegistrationSinglePlane": {
        "121712": ("RT Patient Position Registration, single plane", [9243])
    },
    "RTPatientPositionRegistrationSpatialFiducials": {
        "121721": ("RT Patient Position Registration, Spatial Fiducials", [9243])
    },
    "RTPatientPositionRegistrationUltrasound": {
        "121720": ("RT Patient Position Registration, Ultrasound", [9243])
    },
    "RTPatientPositioningReferenceImage": {
        "130790": ("RT Patient Positioning Reference Image", [9267])
    },
    "RTPatientSetup": {"121701": ("RT Patient Setup", [9241])},
    "RTPlan": {"RTPLAN": ("RT Plan", [32, 33])},
    "RTPlanLabel": {"121384": ("RT Plan Label", [])},
    "RTPlanSummationInputUsed": {
        "128212": ("RT Plan Summation Input Used", [7010, 7025])
    },
    "RTPlanningResult": {"128189": ("RT Planning Result", [7010, 7023])},
    "RTPreTreatmentConsistencyCheck": {
        "121374": ("RT Pre-Treatment Consistency Check", [701, 702])
    },
    "RTPreTreatmentDoseCheck": {"121373": ("RT Pre-Treatment Dose Check", [701, 702])},
    "RTPrescriptionInputImages": {"130136": ("RT Prescription Input Images", [9509])},
    "RTPrescriptionInputUsed": {"128210": ("RT Prescription Input Used", [7010, 7025])},
    "RTPrescriptionResult": {"128185": ("RT Prescription Result", [7010, 7023])},
    "RTPrescriptionResultForRTTreatmentPlanning": {
        "128186": ("Dose Calculation Image Series", [7010, 7023, 9510])
    },
    "RTRadiationPreviouslyDelivered": {
        "130663": ("RT Radiation previously delivered", [9576])
    },
    "RTRegistrationMark": {"130746": ("RT Registration Mark", [9502, 9503])},
    "RTSegmentMaterial": {"130737": ("RT Segment Material", [])},
    "RTStructureSet": {"RTSTRUCT": ("RT Structure Set", [32, 33])},
    "RTTarget": {"130041": ("RT Target", [9502, 9503, 9580])},
    "RTTreatmentCourseSummary": {
        "128193": ("RT Treatment Course Summary", [7010, 7023])
    },
    "RTTreatmentPlanForThePositionBeingVerified": {
        "121310": ("RT treatment plan for the position being verified", [7022])
    },
    "RTTreatmentPlanningInputImages": {
        "130137": ("RT Treatment Planning Input Images", [9510])
    },
    "RTTreatmentPlanningInputUsed": {
        "128211": ("RT Treatment Planning Input Used", [7010, 7025])
    },
    "RTTreatmentQAByRTIonPlanDifferenceCheck": {
        "121734": ("RT Treatment QA by RT Ion Plan Difference Check", [9241])
    },
    "RTTreatmentQAByRTIonPlanDoseCheck": {
        "121733": ("RT Treatment QA by RT Ion Plan Dose Check", [9241])
    },
    "RTTreatmentQAByRTPlanDifferenceCheck": {
        "121732": ("RT Treatment QA by RT Plan Difference Check", [9241])
    },
    "RTTreatmentQAByRTPlanDoseCheck": {
        "121731": ("RT Treatment QA by RT Plan Dose Check", [9241])
    },
    "RTTreatmentQAResult": {"128194": ("RT Treatment QA Result", [7010, 7023])},
    "RTTreatmentQAWithExternalVerification": {
        "121729": ("RT Treatment QA with External Verification", [9241])
    },
    "RTTreatmentQAWithInternalVerification": {
        "121728": ("RT Treatment QA with Internal Verification", [9241])
    },
    "RTTreatmentQAWithRTIonPlanDifferenceCheck": {
        "121734": ("RT Treatment QA with RT Ion Plan Difference Check", [])
    },
    "RTTreatmentRecord": {"RTRECORD": ("RT Treatment Record", [32, 33])},
    "RTTreatmentSessionResult": {
        "128192": ("RT Treatment Session Result", [7010, 7023])
    },
    "RTTreatmentSimulationResult": {
        "128188": ("RT Treatment Simulation Result", [7010, 7023])
    },
    "RTTreatmentSimulationWithExternalVerification": {
        "121725": ("RT Treatment Simulation with External Verification", [9241])
    },
    "RTTreatmentSimulationWithInternalVerification": {
        "121724": ("RT Treatment Simulation with Internal Verification", [9241])
    },
    "RTTreatmentWithAdHocPlanning": {
        "130444": ("RT Treatment with Ad Hoc Planning", [9241])
    },
    "RTTreatmentWithExternalVerification": {
        "121727": ("RT Treatment with External Verification", [9241])
    },
    "RTTreatmentWithInternalVerification": {
        "121726": ("RT Treatment with Internal Verification", [9241])
    },
    "RTVAudioAndStereoVideoRendition": {
        "130373": ("RTV Audio and Stereo Video Rendition", [7010, 7070])
    },
    "RTVAudioAndVideoRendition": {
        "130371": ("RTV Audio and Video Rendition", [7010, 7070])
    },
    "RTVRendition": {"130370": ("RTV Rendition", [7010, 7070])},
    "RTVStereoVideoRendition": {"130372": ("RTV Stereo Video Rendition", [7010, 7070])},
    "RTWorkflowInputUsed": {"128209": ("RT Workflow Input Used", [7010, 7025])},
    "RWave": {"109043": ("R wave", [3339])},
    "RadialBScanPattern": {"128282": ("Radial B-scan pattern", [4272])},
    "RadialBasedWallMotionAnalysis": {
        "122493": ("Radial Based Wall Motion Analysis", [])
    },
    "RadialDiffusivity": {"113203": ("Radial Diffusivity", [7263])},
    "RadialKurtosis": {"113207": ("Radial Kurtosis", [7263])},
    "RadialLagrangianStrain": {"130675": ("Radial Lagrangian Strain", [12304, 12311])},
    "RadialLagrangianStrainRate": {
        "130676": ("Radial Lagrangian Strain Rate", [12304, 12311])
    },
    "RadialPeripapillaryStructuralReflectanceMap": {
        "128264": ("Radial peripapillary structural reflectance map", [4271])
    },
    "RadialPeripapillaryVasculatureFlow": {
        "128263": ("Radial peripapillary vasculature flow", [4271])
    },
    "RadialScar": {"111293": ("Radial scar", [])},
    "RadialShortening": {"122496": ("Radial Shortening", [])},
    "RadialUltrasoundTransducerGeometry": {
        "125255": ("Radial ultrasound transducer geometry", [12033])
    },
    "RadiationApplied": {"122308": ("Radiation applied", [3412])},
    "RadiationCharacteristicsNote": {"130022": ("Radiation Characteristics Note", [])},
    "RadiationDetectedDuringBeamHoldInterlock": {
        "130828": ("Radiation Detected During Beam Hold Interlock", [9561, 9568])
    },
    "RadiationDoseCompositeParameters": {
        "128436": ("Radiation Dose Composite Parameters", [])
    },
    "RadiationDoseEstimate": {"128402": ("Radiation Dose Estimate", [])},
    "RadiationDoseEstimateMethod": {"128476": ("Radiation Dose Estimate Method", [])},
    "RadiationDoseEstimateMethodReference": {
        "128482": ("Radiation Dose Estimate Method Reference", [])
    },
    "RadiationDoseEstimateMethodType": {
        "128477": ("Radiation Dose Estimate Method Type", [])
    },
    "RadiationDoseEstimateMethodology": {
        "128415": ("Radiation Dose Estimate Methodology", [])
    },
    "RadiationDoseEstimateName": {"128403": ("Radiation Dose Estimate Name", [])},
    "RadiationDoseEstimateParameterType": {
        "128464": ("Radiation Dose Estimate Parameter Type", [])
    },
    "RadiationDoseEstimateParameters": {
        "128434": ("Radiation Dose Estimate Parameters", [])
    },
    "RadiationDoseEstimateRepresentation": {
        "128412": ("Radiation Dose Estimate Representation", [])
    },
    "RadiationDoseNormalizationPoint": {
        "130063": ("Radiation Dose Normalization Point", [9534])
    },
    "RadiationDoseReferencePoint": {
        "130064": ("Radiation Dose Reference Point", [9534])
    },
    "RadiationDoseRepresentationData": {
        "128414": ("Radiation Dose Representation Data", [])
    },
    "RadiationDoseShapingVolume": {"130061": ("Radiation Dose Shaping Volume", [9535])},
    "RadiationExposure": {"113921": ("Radiation Exposure", [])},
    "RadiationExposureAndProtectionInformation": {
        "113923": ("Radiation Exposure and Protection Information", [])
    },
    "RadiationFieldArea": {"130516": ("Radiation Field Area", [])},
    "RadiationFieldOutline": {"130517": ("Radiation Field Outline", [])},
    "RadiationInteractionVolume": {"130066": ("Radiation Interaction Volume", [9535])},
    "RadiationOutput": {"130514": ("Radiation Output", [])},
    "RadiationPhysicist": {"121105": ("Radiation Physicist", [])},
    "RadiationRemoved": {"122309": ("Radiation removed", [3412])},
    "RadiationSourceCharacteristics": {
        "130508": ("Radiation Source Characteristics", [])
    },
    "RadiationTechnique": {"130511": ("Radiation Technique", [])},
    "RadiationTherapyDevice": {"RT": ("Radiation Therapy Device", [30])},
    "RadiationTherapyParticle": {"130033": ("Radiation Therapy Particle", [])},
    "RadiationTransportBasedMethods": {
        "130126": ("Radiation transport-based methods", [9537])
    },
    "RadiationTransportModelType": {"128420": ("Radiation Transport Model Type", [])},
    "RadioFrequencyAblationPower": {
        "109010": ("Radio frequency ablation, power", [3240])
    },
    "RadioactiveMarker": {"130646": ("Radioactive Marker", [9573, 9575])},
    "RadioactiveSubstanceAdministered": {
        "113922": ("Radioactive Substance Administered", [])
    },
    "Radiofluoroscopy": {"RF": ("Radiofluoroscopy", [29, 30, 33])},
    "RadiofrequencyTransponder": {
        "130644": ("Radiofrequency Transponder", [9573, 9575])
    },
    "Radiographer": {"121084": ("Radiographer", [])},
    "RadiographicAnatomy": {"112005": ("Radiographic anatomy", [6101])},
    "RadiographicImaging": {"RG": ("Radiographic imaging", [29, 30, 33])},
    "RadioisotopeGenerator": {"113543": ("Radioisotope Generator", [10041])},
    "RadionuclideResidualSyringeCounts": {
        "123010": ("Radionuclide Residual Syringe Counts", [])
    },
    "RadionuclideSyringeCounts": {"123009": ("Radionuclide Syringe Counts", [])},
    "RadionuclideTotalDose": {"123006": ("Radionuclide Total Dose", [])},
    "RadionuclideVialIdentifier": {"113514": ("Radionuclide Vial Identifier", [])},
    "Radiopharmaceutical": {"123001": ("Radiopharmaceutical", [])},
    "RadiopharmaceuticalAdministration": {
        "113502": ("Radiopharmaceutical Administration", [])
    },
    "RadiopharmaceuticalAdministrationEventUID": {
        "113503": ("Radiopharmaceutical Administration Event UID", [])
    },
    "RadiopharmaceuticalDispenseUnitIdentifier": {
        "113511": ("Radiopharmaceutical Dispense Unit Identifier", [])
    },
    "RadiopharmaceuticalLotIdentifier": {
        "113512": ("Radiopharmaceutical Lot Identifier", [])
    },
    "RadiopharmaceuticalRadiationDoseReport": {
        "113500": ("Radiopharmaceutical Radiation Dose Report", [])
    },
    "RadiopharmaceuticalSpecificActivity": {
        "123007": ("Radiopharmaceutical Specific Activity", [])
    },
    "RadiopharmaceuticalStartDatetime": {
        "123003": ("Radiopharmaceutical Start DateTime", [])
    },
    "RadiopharmaceuticalStopDatetime": {
        "123004": ("Radiopharmaceutical Stop DateTime", [])
    },
    "RadiopharmaceuticalVolume": {"123005": ("Radiopharmaceutical Volume", [])},
    "RadiotherapyApplicator": {"130125": ("Radiotherapy Applicator", [9519, 9520])},
    "RadiotherapyDose": {"RTDOSE": ("Radiotherapy Dose", [])},
    "RadiotherapyDoseRegion": {"130748": ("Radiotherapy Dose Region", [9502, 9580])},
    "RadiotherapyFiducial": {"130666": ("Radiotherapy Fiducial", [9502, 9580])},
    "RadiotherapyImage": {"RTIMAGE": ("Radiotherapy Image", [])},
    "RadiotherapyPlan": {"RTPLAN": ("Radiotherapy Plan", [])},
    "RadiotherapyStructureSet": {"RTSTRUCT": ("Radiotherapy Structure Set", [])},
    "RadiotherapyTreatmentDevice": {
        "130361": ("Radiotherapy Treatment Device", [9551])
    },
    "RadiotherapyTreatmentRecord": {"RTRECORD": ("Radiotherapy Treatment Record", [])},
    "RaisedSkinLesion": {"130486": ("Raised skin lesion", [4407])},
    "Ranibizumab89Zr": {"126742": ("Ranibizumab ^89^Zr", [4021])},
    "RasterBScanPattern": {"128280": ("Raster B-scan pattern", [4272])},
    "RateOfAdministration": {"122094": ("Rate of administration", [3410])},
    "RateOfExposure": {"111579": ("Rate of exposure", [6092])},
    "RatingOfPerceivedExertion": {"122706": ("Rating of Perceived Exertion", [])},
    "RatioOfAchievedToPredictedFunctionalCapacity": {
        "122771": ("Ratio of achieved to predicted functional capacity", [3238])
    },
    "RatioOfAchievedToPredictedMaximalOxygenConsumption": {
        "122770": ("Ratio of achieved to predicted maximal oxygen consumption", [3238])
    },
    "ReaderSpecialty": {"128003": ("Reader Specialty", [])},
    "ReagentVialIdentifier": {"113513": ("Reagent Vial Identifier", [])},
    "RealTimeAcquisition": {"109080": ("Real time acquisition", [3104])},
    "RealWorldValueMap": {"RWV": ("Real World Value Map", [32, 33])},
    "RealWorldValueMapUsedForMeasurement": {
        "126100": ("Real World Value Map used for measurement", [])
    },
    "ReasonForBiosafetyControls": {"127011": ("Reason for biosafety controls", [])},
    "ReasonForProcedure": {"111401": ("Reason for Procedure", [])},
    "ReasonForProceeding": {"113907": ("Reason for Proceeding", [])},
    "ReasonForRejectingAcquisition": {
        "130504": ("Reason for Rejecting Acquisition", [])
    },
    "ReasonForRepeatingAcquisition": {
        "128552": ("Reason for Repeating Acquisition", [])
    },
    "ReasonForStudy": {"122139": ("Reason for Study", [])},
    "RecallForImagingFindings": {"111591": ("Recall for imaging findings", [6051])},
    "RecallForPatientSymptomsClinicalFindings": {
        "111592": ("Recall for patient symptoms/ clinical findings", [6051])
    },
    "RecallForTechnicalReasons": {"111590": ("Recall for technical reasons", [6051])},
    "RecentPhysicalActivity": {"113552": ("Recent Physical Activity", [])},
    "ReciprocatingGrid": {"111643": ("Reciprocating grid", [10017])},
    "Recommendation": {"121075": ("Recommendation", [3419, 6053, 7002])},
    "Recommendations": {"121074": ("Recommendations", [])},
    "RecommendedCADOperatingPoint": {"111092": ("Recommended CAD Operating Point", [])},
    "RecommendedFollowUp": {"111053": ("Recommended Follow-up", [])},
    "RecommendedFollowUpDate": {"111054": ("Recommended Follow-up Date", [])},
    "RecommendedFollowUpInterval": {"111055": ("Recommended Follow-up Interval", [])},
    "Reconstructed": {"122404": ("Reconstructed", [])},
    "ReconstructionAlgorithm": {"113961": ("Reconstruction Algorithm", [])},
    "Recording": {"121097": ("Recording", [7445, 7453])},
    "RecoveryState": {"109096": ("Recovery state", [])},
    "RectalBalloon": {
        "130120": ("Rectal Balloon", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "RecumbentPatientPositionWithRespectToGravity": {
        "112228": ("Recumbent Patient Position with respect to gravity", [])
    },
    "RecurrentMalignancy": {"111338": ("Recurrent malignancy", [6030, 6033])},
    "RedFilter": {"111602": ("Red filter", [])},
    "RedTranslucentIgloo": {"127221": ("Red translucent igloo", [606])},
    "RedistributionState": {"109093": ("Redistribution State", [3101])},
    "ReferenceAcquisitionPoint": {"130072": ("Reference Acquisition Point", [9504])},
    "ReferenceAuthority": {"121406": ("Reference Authority", [])},
    "ReferenceBasis": {"128772": ("Reference Basis", [])},
    "ReferenceEnergy": {"130087": ("Reference Energy", [])},
    "ReferenceGeometry": {"128773": ("Reference Geometry", [])},
    "ReferenceMeasurementGroup": {"130755": ("Reference Measurement Group", [])},
    "ReferenceMethod": {"122430": ("Reference Method", [])},
    "ReferencePoint": {"122611": ("Reference Point", [])},
    "ReferencePointDefinition": {"113780": ("Reference Point Definition", [])},
    "ReferencePointDosimetry": {"130502": ("Reference Point Dosimetry", [])},
    "ReferencePointPosition": {"130526": ("Reference Point Position", [])},
    "ReferencePoints": {"122438": ("Reference Points", [])},
    "ReferenceToUncertaintyDeterminationMethod": {
        "128511": ("Reference to Uncertainty Determination Method", [])
    },
    "ReferencedBeamNumber": {"121389": ("Referenced Beam Number", [])},
    "ReferencedFrames": {"121190": ("Referenced Frames", [7250])},
    "ReferencedImagingAgentAdministrationPhaseIdentifier": {
        "130217": ("Referenced Imaging Agent Administration Phase Identifier", [])
    },
    "ReferencedImagingAgentAdministrationPhaseUID": {
        "130262": ("Referenced Imaging Agent Administration Phase UID", [])
    },
    "ReferencedImagingAgentAdministrationStepUID": {
        "130216": ("Referenced Imaging Agent Administration Step UID", [])
    },
    "ReferencedImagingAgentIdentifier": {
        "130255": ("Referenced Imaging Agent Identifier", [])
    },
    "ReferencedPatientAlignmentReference": {
        "130662": ("Referenced Patient Alignment Reference", [])
    },
    "ReferencedRegionOfInterestIdentifier": {
        "130489": ("Referenced Region of Interest Identifier", [])
    },
    "ReferencedSegment": {"121191": ("Referenced Segment", [7250])},
    "ReferencedSegmentationFrame": {"121214": ("Referenced Segmentation Frame", [])},
    "Referring": {"121095": ("Referring", [])},
    "ReflectionBasedCornealTopographer": {
        "111946": ("Reflection-based corneal topographer", [4210])
    },
    "ReflectionIllumination": {"111742": ("Reflection illumination", [8123])},
    "RefractiveMeasurementsSOPInstance": {
        "111783": ("Refractive Measurements SOP Instance", [4240])
    },
    "RegionInSpace": {"130488": ("Region in Space", [])},
    "RegionalAbnormalWallMotion": {"122417": ("Regional Abnormal Wall Motion", [])},
    "RegionalCerebralBloodFlow": {"113055": ("Regional Cerebral Blood Flow", [])},
    "RegionalCerebralBloodVolume": {"113056": ("Regional Cerebral Blood Volume", [])},
    "RegionalContributionToEjectionFraction": {
        "122495": ("Regional Contribution to Ejection Fraction", [])
    },
    "Registrar": {"121087": ("Registrar", [])},
    "Registration": {"REG": ("Registration", [32, 33])},
    "RegistrationCreatedDuringTreatment": {
        "129211": ("Registration created during Treatment", [7022])
    },
    "RegistrationInput": {"125041": ("Registration Input", [7151, 7165, 7196])},
    "RegistrationMethod": {"128446": ("Registration Method", [])},
    "RegistrationResult": {"128183": ("Registration Result", [7010, 9509, 9510])},
    "RegistrationUsedInPlanning": {"129210": ("Registration used in Planning", [7022])},
    "RegressionOffsetED": {"122432": ("Regression Offset ED", [])},
    "RegressionOffsetES": {"122434": ("Regression Offset ES", [])},
    "RegressionSlopeED": {"122431": ("Regression Slope ED", [])},
    "RegressionSlopeES": {"122433": ("Regression Slope ES", [])},
    "RegressionVolumeExponent": {"122435": ("Regression Volume Exponent", [])},
    "RegurgitationJetArea": {"125332": ("Regurgitation Jet Area", [12304])},
    "RegurgitationJetWidth": {"125333": ("Regurgitation Jet Width", [12304])},
    "ReinjectionState": {"109092": ("Reinjection State", [3101])},
    "RejectedForPatientSafetyReasons": {
        "113037": ("Rejected for Patient Safety Reasons", [7010, 7031])
    },
    "RejectedForQualityReasons": {
        "113001": ("Rejected for Quality Reasons", [7010, 7031])
    },
    "RelatedImplantationReports": {"112365": ("Related Implantation Reports", [])},
    "RelatedOphthalmicThicknessMap": {
        "111933": ("Related ophthalmic thickness map", [4264])
    },
    "RelatedPatientDataNotUsedDuringPlanning": {
        "112364": ("Related Patient Data Not Used During Planning", [])
    },
    "RelatedToNonMacularSensitivity": {
        "111842": ("Related to non macular sensitivity", [4252])
    },
    "RelativeAmountOfExposure": {"111582": ("Relative amount of exposure", [6093])},
    "RelativeAmountOfUse": {"111583": ("Relative amount of use", [6093])},
    "RelativeAnisotropy": {
        "110809": ("Relative Anisotropy", [218, 7180, 7263, 7271, 7469])
    },
    "RelativeDoseAmount": {"111581": ("Relative dose amount", [6093])},
    "RelativeDoseFrequency": {"111584": ("Relative dose frequency", [6094])},
    "RelativeElectronDensity": {"130083": ("Relative Electron Density", [])},
    "RelativeFrequencyOfExposure": {
        "111585": ("Relative frequency of exposure", [6094])
    },
    "RelativeFrequencyOfUse": {"111586": ("Relative frequency of use", [6094])},
    "RelativeLinearStoppingPower": {
        "130086": ("Relative Linear Stopping Power", [218, 7180, 7469])
    },
    "RelativeMassDensity": {"130082": ("Relative Mass Density", [])},
    "RelativePosition": {"122337": ("Relative position", [])},
    "RelativeRegionalBloodFlow": {
        "126397": ("Relative Regional Blood Flow", [218, 4108, 7180, 7469])
    },
    "RelativeRegionalBloodVolume": {
        "126398": ("Relative Regional Blood Volume", [218, 4108, 7180, 7469])
    },
    "RelativeStenosisSeverity": {"122391": ("Relative Stenosis Severity", [])},
    "RelevantIndicatedProblems": {"111514": ("Relevant Indicated Problems", [])},
    "RelevantPatientInformation": {"111517": ("Relevant Patient Information", [])},
    "RelevantPatientInformationForBreastImaging": {
        "111511": ("Relevant Patient Information for Breast Imaging", [])
    },
    "RelevantPreviousProcedures": {"111513": ("Relevant Previous Procedures", [])},
    "RelevantRiskFactors": {"111515": ("Relevant Risk Factors", [])},
    "RemainingConsumableDisposed": {
        "122078": ("Remaining consumable disposed", [3408])
    },
    "RemodelingIndex": {"122345": ("Remodeling Index", [3484])},
    "RemoteServiceOperationStarted": {
        "110139": ("Remote Service Operation Started", [401, 403])
    },
    "RemoteServiceOperationStopped": {
        "110140": ("Remote Service Operation Stopped", [401, 403])
    },
    "RenderingIntent": {"111056": ("Rendering Intent", [])},
    "ReplacedByAlternateRepresentation": {
        "130754": ("Replaced by Alternate Representation", [7031])
    },
    "ReplacedReport": {"121360": ("Replaced report", [7009])},
    "ReportRequested": {"128004": ("Report Requested", [])},
    "ReportVerification": {"110007": ("Report Verification", [9231])},
    "ReportedValueType": {"130510": ("Reported Value Type", [])},
    "ReportingSystem": {"130551": ("Reporting system", [])},
    "RepresentativeOfEthicsCommittee": {
        "128677": ("Representative of Ethics Committee", [7450, 7452])
    },
    "RepresentativeOfProtocolCommittee": {
        "128676": ("Representative of Protocol Committee", [7450, 7452])
    },
    "RepresentativePoint": {"128140": ("Representative point", [219])},
    "Request": {"121062": ("Request", [])},
    "Requesting": {"121096": ("Requesting", [7453])},
    "Resident": {"121086": ("Resident", [])},
    "ResidualVolumeOfImagingAgentInContainer": {
        "130206": ("Residual Volume of Imaging Agent in Container", [])
    },
    "ResolvedByOverridingInterlock": {
        "130454": ("Resolved by overriding Interlock", [])
    },
    "ResolvedByRepositioningPatient": {
        "130455": ("Resolved by repositioning Patient", [])
    },
    "ResourceInadequate": {"110527": ("Resource inadequate", [71, 9300])},
    "ResourcePreEmpted": {"110526": ("Resource pre-empted", [9300, 9303])},
    "RespirationWaveform": {"109117": ("Respiration Waveform", [3005])},
    "RespiratoryEffort": {"130433": ("Respiratory Effort", [3005])},
    "RespiratoryPressure": {"130426": ("Respiratory Pressure", [3005])},
    "RespiratoryWaveform": {"RESP": ("Respiratory Waveform", [29, 30, 33])},
    "ResponseEvaluation": {"112020": ("Response Evaluation", [])},
    "ResponseEvaluationMethod": {"112021": ("Response Evaluation Method", [])},
    "RestTechnetiumStressTechnetium1DayProcedure": {
        "122782": ("Rest technetium/stress technetium 1 day procedure", [3110])
    },
    "RestTechnetiumStressTechnetium2DayProcedure": {
        "122783": ("Rest technetium/stress technetium 2 day procedure", [3110])
    },
    "RestThalliumStressTechnetiumProcedure": {
        "122781": ("Rest thallium/stress technetium procedure", [3110])
    },
    "RestenoticLesion": {"122393": ("Restenotic Lesion", [])},
    "ResultsCommunicated": {"121291": ("Results communicated", [])},
    "ResultsQualityController": {"129004": ("Results Quality Controller", [7453])},
    "ResumeProcedureAction": {"121133": ("Resume Procedure Action", [3421])},
    "ResumeProcedureActionItem": {"121133": ("Resume Procedure Action Item", [])},
    "RetainDeviceIdentityOption": {"113109": ("Retain Device Identity Option", [7050])},
    "RetainInstitutionIdentityOption": {
        "113112": ("Retain Institution Identity Option", [7050])
    },
    "RetainLongitudinalTemporalInformationFullDatesOption": {
        "113106": ("Retain Longitudinal Temporal Information Full Dates Option", [7050])
    },
    "RetainLongitudinalTemporalInformationModifiedDatesOption": {
        "113107": (
            "Retain Longitudinal Temporal Information Modified Dates Option",
            [7050],
        )
    },
    "RetainPatientCharacteristicsOption": {
        "113108": ("Retain Patient Characteristics Option", [7050])
    },
    "RetainSafePrivateOption": {"113111": ("Retain Safe Private Option", [7050])},
    "RetainUidsOption": {"113110": ("Retain UIDs Option", [7050])},
    "Reticle": {"130350": ("Reticle", [9548])},
    "ReticularPattern": {"112113": ("Reticular pattern", [6102, 6103, 6106])},
    "ReticulonodularPattern": {
        "112065": ("Reticulonodular pattern", [6102, 6103, 6104, 6106])
    },
    "RetinaDepthEncodedStructuralReflectanceMap": {
        "128258": ("Retina depth encoded structural reflectance map", [4271])
    },
    "RetinaDepthEncodedVasculatureFlow": {
        "128257": ("Retina depth encoded vasculature flow", [4271])
    },
    "RetinaStructuralReflectanceMap": {
        "128260": ("Retina structural reflectance map", [4271])
    },
    "RetinaVasculatureFlow": {"128259": ("Retina vasculature flow", [4271])},
    "RetinalNerveFiberLayerThickness": {
        "111925": ("Retinal nerve fiber layer thickness", [4262])
    },
    "RetinalTopography": {"111924": ("Retinal topography", [4261])},
    "RetroOrbitalRoute": {"127070": ("Retro-orbital route", [11])},
    "RetroPectoralImplant": {"111486": ("Retro-pectoral implant", [6058, 6059])},
    "RetrospectiveGating": {"109082": ("Retrospective gating", [3104])},
    "Reused": {"127178": ("Reused", [604])},
    "ReverseOsmosisPurifiedHclAcidifiedWater": {
        "127291": ("Reverse osmosis purified, HCl acidified water", [610])
    },
    "ReverseOsmosisPurifiedWater": {
        "127290": ("Reverse osmosis purified water", [610])
    },
    "ReviewOfAnOutsideStudy": {"111418": ("Review of an outside study", [6051, 6327])},
    "RibScaleneTubercle": {"112096": ("Rib Scalene Tubercle", [6115, 8134])},
    "RightContour": {"122508": ("Right Contour", [])},
    "RightEyeRx": {"111688": ("Right Eye Rx", [])},
    "RightFirst": {"126831": ("right first", [21])},
    "RightHemisphereMostAnterior": {
        "125031": ("Right Hemisphere Most Anterior", [7101, 7111])
    },
    "RightHemisphereMostInferior": {
        "125034": ("Right Hemisphere Most Inferior", [7101, 7111])
    },
    "RightHemisphereMostPosterior": {
        "125032": ("Right Hemisphere Most Posterior", [7101, 7111])
    },
    "RightHemisphereMostSuperior": {
        "125033": ("Right Hemisphere Most Superior", [7101, 7111])
    },
    "RightToLeft": {"110866": ("Right To Left", [7185])},
    "RightToLeftFlow": {"122228": ("Right to Left Flow", [])},
    "RightTrachealStripe": {
        "112093": ("Right tracheal stripe", [6102, 6109, 6113, 6148, 7151, 7192, 9514])
    },
    "RightVentricleAnteriorWall": {
        "125319": ("Right Ventricle Anterior Wall", [12305])
    },
    "RightVentricleOutflowTractDistal": {
        "125317": ("Right Ventricle Outflow Tract, Distal", [12305])
    },
    "RightVentricleOutflowTractProximal": {
        "125318": ("Right Ventricle Outflow Tract, Proximal", [12305])
    },
    "RightVentricularEjectionTime": {
        "122213": ("Right Ventricular ejection time", [3613])
    },
    "RightVentricularFillingTime": {
        "122214": ("Right Ventricular filling time", [3613])
    },
    "RingUltrasoundTransducerGeometry": {
        "125256": ("Ring ultrasound transducer geometry", [12033])
    },
    "RiseTime": {"130207": ("Rise Time", [])},
    "RiskFactorModifier": {"111530": ("Risk Factor modifier", [])},
    "Rituximab89Zr": {"126737": ("Rituximab ^89^Zr", [4021])},
    "RobertsonReidTable": {"122253": ("Robertson & Reid table", [3664])},
    "RoleOfPersonReporting": {"111534": ("Role of person reporting", [])},
    "Roledumab89Zr": {"126733": ("Roledumab ^89^Zr", [4021])},
    "RoomBarometricPressure": {"122225": ("Room Barometric pressure", [])},
    "RoomIdentification": {"121121": ("Room identification", [])},
    "RoomLaserPatientSetupPoint": {
        "130070": ("Room Laser Patient Setup Point", [9504, 9574])
    },
    "RoomOrigin": {"130536": ("Room Origin", [10074])},
    "RoomOxygenConcentration": {"122223": ("Room oxygen concentration", [])},
    "RoomTemperature": {"122224": ("Room temperature", [])},
    "RootAngularSecondMomentOfGLCM": {
        "126061": ("Root Angular Second Moment of GLCM", [218, 7467, 7468])
    },
    "RotationAngle": {"130523": ("Rotation Angle", [])},
    "RotationPlaneNormalPoint": {"130522": ("Rotation Plane Normal Point", [])},
    "RotationalAcquisition": {"113613": ("Rotational Acquisition", [10002])},
    "RunEntropy": {"128816": ("Run Entropy", [])},
    "RunLengthNonuniformity": {"128811": ("Run Length Nonuniformity", [])},
    "RunLengthNonuniformityNormalized": {
        "128812": ("Run Length Nonuniformity Normalized", [])
    },
    "RunLengthVariance": {"128815": ("Run Length Variance", [])},
    "RunPercentage": {"128813": ("Run Percentage", [])},
    "RungeKutta": {"113219": ("Runge-Kutta", [7262])},
    "SAR341989Zr": {"126741": ("SAR3419 ^89^Zr", [4021])},
    "SMILE": {"111681": ("SMILE", [4234])},
    "SMPTEPattern": {"109943": ("SMPTE Pattern", [8301])},
    "SOPClassUID": {"110181": ("SOP Class UID", [404])},
    "SPECTCTOfWholeBody": {"127902": ("SPECT CT of whole body", [646])},
    "SPECTOfWholeBody": {"127901": ("SPECT of whole body", [646])},
    "SPhase": {"111477": ("S Phase", [])},
    "SRInstanceUsed": {"128416": ("SR Instance Used", [])},
    "SRKII": {"111766": ("SRKII", [4236])},
    "SRKT": {"111767": ("SRK-T", [4236])},
    "SSDEConversionFactor": {"128410": ("SSDE Conversion Factor", [10069])},
    "SStent": {"130733": ("S - Stent", [3022])},
    "STChangeFromBaseline": {"122099": ("ST change from baseline", [])},
    "STDepressionDownsloping": {"122759": ("ST Depression - Downsloping", [3233])},
    "STDepressionHorizontal": {"122757": ("ST Depression - Horizontal", [3233])},
    "STDepressionUpsloping": {"122758": ("ST Depression - Upsloping", [3233])},
    "STElevationOnsetDatetime": {"122173": ("ST Elevation Onset DateTime", [])},
    "STHRIndex": {"122773": ("ST/HR Index", [3238])},
    "SUVBodySurfaceAreaCalculationMethod": {
        "126412": ("SUV body surface area calculation method", [])
    },
    "SUVBodyWeightCalculationMethod": {
        "126410": ("SUV body weight calculation method", [])
    },
    "SUVIdealBodyWeightCalculationMethod": {
        "126413": ("SUV ideal body weight calculation method", [])
    },
    "SUVLeanBodyMassCalculationJanmahasatianMethod": {
        "126414": ("SUV lean body mass calculation Janmahasatian method", [])
    },
    "SUVLeanBodyMassCalculationMethod": {
        "126411": ("SUV lean body mass calculation method", [])
    },
    "SUVLeanBodyMassCalculationMethodUsing128Multiplier": {
        "126415": ("SUV lean body mass calculation method using 128 multiplier", [])
    },
    "SWave": {"109044": ("S wave", [3339])},
    "Saliency": {"130404": ("Saliency", [217, 218, 7180, 7469])},
    "SalineImplant": {"111481": ("Saline implant", [6058, 6059])},
    "SameAcquisitionAtDifferentFocalDepth": {
        "121352": ("Same acquisition at different focal depth", [8120])
    },
    "SameAcquisitionAtDifferentSpectralBand": {
        "121353": ("Same acquisition at different spectral band", [8120])
    },
    "SameAcquisitionAtHigherResolution": {
        "121351": ("Same acquisition at higher resolution", [8120])
    },
    "SameAcquisitionAtLowerResolution": {
        "121350": ("Same acquisition at lower resolution", [8120])
    },
    "SameAnatomy": {"122401": ("Same Anatomy", [7210])},
    "SameIndication": {"122402": ("Same Indication", [7210])},
    "SampleVolumeDepth": {"125107": ("Sample Volume Depth", [])},
    "SamplingDatetime": {"111469": ("Sampling DateTime", [])},
    "SamplingMethod": {"111704": ("Sampling Method", [])},
    "ScanDelay": {"130198": ("Scan Delay", [])},
    "ScanSpotInterlock": {"130824": ("Scan Spot Interlock", [9561, 9568])},
    "ScanSpotMetersetInterlock": {
        "130827": ("Scan Spot Meterset Interlock", [9561, 9568])
    },
    "ScanSpotPositionInterlock": {
        "130825": ("Scan Spot Position Interlock", [9561, 9568])
    },
    "ScanSpotSizeInterlock": {"130826": ("Scan Spot Size Interlock", [9561, 9568])},
    "ScanningLength": {"113825": ("Scanning Length", [])},
    "ScapularInfraspinatusFossa": {
        "112101": ("Scapular Infraspinatus Fossa", [6115, 8134])
    },
    "ScapularSpine": {"112099": ("Scapular Spine", [6115, 8134])},
    "ScapularSupraspinatusFossa": {
        "112100": ("Scapular Supraspinatus Fossa", [6115, 8134])
    },
    "ScarTissue": {"111168": ("Scar tissue", [])},
    "ScarredMyocardium": {"122113": ("Scarred Myocardium", [3704])},
    "ScheimpflugCamera": {"111626": ("Scheimpflug Camera", [4210])},
    "SclerosingAdenosis": {"111294": ("Sclerosing adenosis", [])},
    "ScopeOfAccumulation": {"113705": ("Scope of Accumulation", [])},
    "ScopeOfFeature": {"111057": ("Scope of Feature", [])},
    "ScotopicLight": {"111627": ("Scotopic light", [4203])},
    "ScrubNurse": {"121090": ("Scrub nurse", [])},
    "SeatPan": {"130855": ("Seat Pan", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "SecondMeasureOfInformationCorrelationOfGLCM": {
        "128799": ("Second Measure of Information Correlation of GLCM", [])
    },
    "SecondaryFluenceMonitoringSystemInterlock": {
        "130476": ("Secondary Fluence Monitoring System Interlock", [9561, 9568])
    },
    "SecondaryPulmonaryLobule": {
        "112054": ("Secondary pulmonary lobule", [6102, 6109, 6110, 7151, 7192, 9514])
    },
    "SectorUltrasoundTransducerGeometry": {
        "125254": ("Sector ultrasound transducer geometry", [12033])
    },
    "SecureDigitalCard": {"110036": ("Secure Digital Card", [405])},
    "SecurityAlert": {"110113": ("Security Alert", [400])},
    "SecurityConfiguration": {"110129": ("Security Configuration", [401, 403])},
    "SecurityRolesChanged": {"110136": ("Security Roles Changed", [401, 403])},
    "SeedPoint": {"128139": ("Seed point", [219])},
    "Segmentation": {"SEG": ("Segmentation", [32, 33])},
    "SegmentationImageDerivation": {"113076": ("Segmentation", [7203])},
    "SegmentationMethod": {"122554": ("Segmentation Method", [])},
    "SegmentationResult": {"128182": ("Segmentation Result", [7010, 9509, 9510])},
    "SegmentedConstrained": {"113269": ("Segmented-Constrained", [7274])},
    "SegmentedUnconstrained": {"113268": ("Segmented-Unconstrained", [7274])},
    "SelectedImplantComponent": {"112346": ("Selected Implant Component", [])},
    "SelectedRegion": {"111099": ("Selected region", [6014, 6016, 6054, 6101, 6201])},
    "SelectedRegionDescription": {"111058": ("Selected Region Description", [])},
    "SelectionStatus": {"121404": ("Selection Status", [])},
    "SentForAnalysis": {"111457": ("Sent for analysis", [])},
    "SeptalDefectIntervention": {
        "122055": ("Septal Defect Intervention", [3405, 3406])
    },
    "SeptalLines": {"112114": ("Septal line(s)", [6102, 6103])},
    "SeptumToWall": {"110876": ("Septum To Wall", [7185])},
    "SeptumWallAxis": {"110865": ("Septum-Wall Axis", [7184])},
    "SequencedAcquisition": {"113804": ("Sequenced Acquisition", [10013])},
    "Series": {"113015": ("Series", [272, 7012, 10000])},
    "SeriesInstanceUID": {"112002": ("Series Instance UID", [10001])},
    "SeriesNumber": {"113607": ("Series Number", [10022])},
    "SeriesOrInstanceUsedForWaterEquivalentDiameterEstimation": {
        "113985": (
            "Series or Instance used for Water Equivalent Diameter estimation",
            [],
        )
    },
    "ServiceDeliveryLocation": {"121434": ("Service Delivery Location", [])},
    "ServicePerformer": {"121435": ("Service Performer", [])},
    "SessionStart": {"110145": ("Session start", [401, 403])},
    "SessionStop": {"110146": ("Session stop", [401, 403])},
    "SevereDistortionInTheAreaOfInterest": {
        "130584": ("Severe distortion in the area of interest", [6315, 6318])
    },
    "SeverelyJaundiced": {"113563": ("Severely Jaundiced", [64, 10045])},
    "SeverityOfCalcification": {"122132": ("Severity of Calcification", [])},
    "SeverityOfComplication": {"111466": ("Severity of Complication", [])},
    "Sewing": {"112319": ("Sewing", [7310])},
    "SexOfAnimalsWithinSameHousingUnit": {
        "127144": ("Sex of animals within same housing unit", [])
    },
    "SexOfHandler": {"127145": ("Sex of handler", [])},
    "Shadow": {"112124": ("Shadow", [6102, 6104])},
    "ShapeFromMotion": {"114206": ("Shape from motion", [8201])},
    "ShapeFromShading": {"114205": ("Shape from shading", [8201])},
    "SharplyDefined": {"112137": ("Sharply defined", [6120])},
    "SharplyDemarcated": {"112140": ("Sharply demarcated", [6120])},
    "ShearWaveDetectionMethod": {"130759": ("Shear Wave Detection Method", [])},
    "ShearWaveDispersionSlope": {"130612": ("Shear Wave Dispersion Slope", [12308])},
    "ShearWaveDispersionSlopeCenterFrequency": {
        "130758": ("Shear Wave Dispersion Slope Center Frequency", [])
    },
    "ShearWaveSpeed": {"130611": ("Shear Wave Speed", [12308])},
    "SheetFilmDigitized": {"110020": ("Sheet Film Digitized", [7008])},
    "ShelterManufacturer": {"127196": ("Shelter manufacturer", [])},
    "ShelterProductCode": {"127198": ("Shelter product code", [])},
    "ShelterProductName": {"127197": ("Shelter product name", [])},
    "ShelterType": {"127195": ("Shelter type", [])},
    "ShinRest": {"130852": ("Shin Rest", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "ShortLabel": {"125309": ("Short Label", [])},
    "ShortRunHighGrayLevelEmphasis": {
        "128806": ("Short Run High Gray Level Emphasis", [])
    },
    "ShortRunLowGrayLevelEmphasis": {
        "128805": ("Short Run Low Gray Level Emphasis", [])
    },
    "ShortRunsEmphasis": {"128801": ("Short Runs Emphasis", [])},
    "ShutterSpeedModelSSM": {"126344": ("Shutter-Speed Model (SSM)", [4101, 4106])},
    "SideOfFamily": {"111540": ("Side of Family", [])},
    "SignalBaselineEnd": {"122639": ("Signal Baseline End", [3836])},
    "SignalBaselineStart": {"122638": ("Signal Baseline Start", [3836])},
    "SignalChange": {"113059": ("Signal change", [218, 7180, 7469])},
    "SignalEarliestPeakTime": {"122631": ("Signal Earliest Peak Time", [3836])},
    "SignalIncreaseStartTime": {"122633": ("Signal Increase Start Time", [3836])},
    "SignalTimeToPeak": {"122634": ("Signal Time to Peak", [3836])},
    "SignalToNoise": {"113060": ("Signal to noise", [218, 7180, 7469])},
    "SignalToNoiseRatio": {"111787": ("Signal to Noise Ratio", [4243])},
    "SignedCompleteAcquisitionContent": {
        "113035": ("Signed Complete Acquisition Content", [7010])
    },
    "SignedCompleteStudyContent": {"113033": ("Signed Complete Study Content", [7010])},
    "SignedManifest": {"113031": ("Signed Manifest", [7010])},
    "SignetRingSign": {"112069": ("Signet-ring sign", [6102, 6104])},
    "SilhouetteSign": {"112152": ("Silhouette sign", [6123])},
    "SiliconeGranuloma": {"111296": ("Silicone granuloma", [6030, 6031])},
    "SiliconeLadenLymphNodes": {"111507": ("Silicone-laden lymph nodes", [6072])},
    "SimpleObjectModel": {"128418": ("Simple Object Model", [10064])},
    "SimplifiedBernoulli": {"125218": ("Simplified Bernoulli", [12227, 12230])},
    "SimulatedKeratometry": {"111755": ("Simulated Keratometry", [4235, 4242])},
    "SimulationIntent": {"129017": ("Simulation Intent", [7064])},
    "SimulationNote": {"130032": ("Simulation Note", [])},
    "SimultaneousArterialPulseWaveform": {
        "121306": ("Simultaneous Arterial Pulse Waveform", [7004])
    },
    "SimultaneousDoppler": {"121301": ("Simultaneous Doppler", [7004])},
    "SimultaneousECG": {"121303": ("Simultaneous ECG", [7004])},
    "SimultaneousHemodynamic": {"121302": ("Simultaneous Hemodynamic", [7004])},
    "SimultaneousPhonocardiographicWaveform": {
        "121307": ("Simultaneous Phonocardiographic Waveform", [7004])
    },
    "SimultaneousRespiratoryWaveform": {
        "121305": ("Simultaneous Respiratory Waveform", [7004])
    },
    "SimultaneousVoiceNarrative": {"121304": ("Simultaneous Voice Narrative", [7004])},
    "SimultaneouslyAcquired": {"122400": ("Simultaneously Acquired", [7210])},
    "SingleFraction": {"130096": ("Single Fraction", [9533])},
    "SingleImageFinding": {"111059": ("Single Image Finding", [])},
    "SingleImageFindingModifier": {"112024": ("Single Image Finding Modifier", [])},
    "SingleLADRegionInRAOProjection": {
        "122466": ("Single LAD Region in RAO Projection", [3461])
    },
    "SingleLeaves": {"130333": ("Single Leaves", [9540, 9541])},
    "SingleLuminance": {"111840": ("Single luminance", [4252])},
    "SingleOrAnteriorLens": {"111778": ("Single or Anterior Lens", [4233])},
    "SinglePhotonEmissionComputedTomography": {
        "ST": ("Single-photon emission computed tomography", [])
    },
    "SinglePlane": {"113622": ("Single Plane", [10003])},
    "SinglePlaneAngiography": {"122059": ("Single plane Angiography", [3748])},
    "SinglePlaneEllipse": {"125226": ("Single Plane Ellipse", [12227, 12228])},
    "SingleRCARegionInRAOProjection": {
        "122467": ("Single RCA Region in RAO Projection", [3461])
    },
    "SingleShotEPI": {"113226": ("Single Shot EPI", [7260])},
    "SingleSideIllumination": {"130810": ("Single-side illumination", [11001])},
    "SingleSubjectExtractedFromGroup": {
        "113132": ("Single subject extracted from group", [7151, 7165, 7196])
    },
    "SingleSubjectSelectedFromGroup": {
        "113132": ("Single subject selected from group", [])
    },
    "SingleTensor": {"113231": ("Single Tensor", [7261])},
    "SingularValueDecompositionSVDDeconvolution": {
        "126311": ("Singular Value Decomposition (SVD) deconvolution", [4105, 4106])
    },
    "SiteInvolvement": {"112008": ("Site involvement", [])},
    "SiteOfLumenMaximum": {"122687": ("Site of Lumen Maximum", [3486])},
    "SiteOfLumenMinimum": {"122382": ("Site of Lumen Minimum", [3486])},
    "SiteOfMaximumLuminal": {"122516": ("Site of Maximum Luminal", [])},
    "SizeDescriptor": {"112025": ("Size Descriptor", [])},
    "SizeSpecificDoseEstimate": {"113930": ("Size Specific Dose Estimate", [])},
    "SizeSpecificDoseEstimateAtLongitudinalPositionZ": {
        "113993": ("Size Specific Dose Estimate At Longitudinal Position Z", [])
    },
    "Skewness": {"126051": ("Skewness", [7464])},
    "SkinAppositionSetupMethod": {"130634": ("Skin Apposition Setup Method", [9571])},
    "SkinConditionClammy": {"122275": ("skin condition Clammy", [3446])},
    "SkinConditionCold": {"122273": ("skin condition Cold", [3446])},
    "SkinConditionCool": {"122272": ("skin condition Cool", [3446])},
    "SkinConditionDiaphoretic": {"122276": ("skin condition Diaphoretic", [3446])},
    "SkinConditionDry": {"122274": ("skin condition Dry", [3446])},
    "SkinConditionFlush": {"122277": ("skin condition Flush", [3446])},
    "SkinConditionMottled": {"122278": ("skin condition Mottled", [3446])},
    "SkinConditionPale": {"122279": ("skin condition Pale", [3446])},
    "SkinConditionWarm": {"122271": ("skin condition Warm", [3446])},
    "SkinDoseMap": {"128485": ("Skin Dose Map", [10063])},
    "SkinLesionColor": {"130832": ("Skin lesion color", [])},
    "SkinOfAnteriorHelixOfEar": {
        "130305": ("Skin of anterior helix of ear", [4029, 8134])
    },
    "SkinOfCaruncleOfEye": {"130306": ("Skin of caruncle of eye", [4029, 8134])},
    "SkinOfInferiorHelixOfEar": {
        "130307": ("Skin of inferior helix of ear", [4029, 8134])
    },
    "SkinOfInferiorPosteriorSurfaceOfThePinna": {
        "130308": ("Skin of inferior posterior surface of the pinna", [4029, 8134])
    },
    "SkinOfInfraalarGroove": {"130312": ("Skin of infraalar groove", [4029, 8134])},
    "SkinOfLateralPartOfDorsumOfFoot": {
        "130309": ("Skin of lateral part of dorsum of foot", [4029, 8134])
    },
    "SkinOfLowerAntihelixOfEar": {
        "130310": ("Skin of lower antihelix of ear", [4029, 8134])
    },
    "SkinOfLowerEyelidMargin": {
        "130311": ("Skin of lower eyelid margin", [4029, 8134])
    },
    "SkinOfLowerParaspinalRegion": {
        "130304": ("Skin of lower paraspinal region", [4029, 8134])
    },
    "SkinOfMedialPartOfDorsumOfFoot": {
        "130313": ("Skin of medial part of dorsum of foot", [4029, 8134])
    },
    "SkinOfMidBack": {"130323": ("Skin of mid back", [4029, 8134])},
    "SkinOfMidParaspinalRegion": {
        "130303": ("Skin of mid paraspinal region", [4029, 8134])
    },
    "SkinOfParanasalCheek": {"130314": ("Skin of paranasal cheek", [4029, 8134])},
    "SkinOfParaspinalAreaOfTheNeck": {
        "130300": ("Skin of paraspinal area of the neck", [4029, 8134])
    },
    "SkinOfParaspinalAreaOfTheSuperiorBack": {
        "130301": ("Skin of paraspinal area of the superior back", [4029, 8134])
    },
    "SkinOfPosteriorHelixOfEar": {
        "130315": ("Skin of posterior helix of ear", [4029, 8134])
    },
    "SkinOfPosteriorLobuleOfTheEar": {
        "130316": ("Skin of posterior lobule of the ear", [4029, 8134])
    },
    "SkinOfSoleOfForefoot": {"130317": ("Skin of sole of forefoot", [4029, 8134])},
    "SkinOfSuperiorAntihelixOfEar": {
        "130318": ("Skin of superior antihelix of ear", [4029, 8134])
    },
    "SkinOfSuperiorPosteriorHelixOfEar": {
        "130319": ("Skin of superior posterior helix of ear", [4029, 8134])
    },
    "SkinOfSuperiorPosteriorSurfaceOfThePinna": {
        "130320": ("Skin of superior posterior surface of the pinna", [4029, 8134])
    },
    "SkinOfUpperAntihelixOfEar": {
        "130321": ("Skin of upper antihelix of ear", [4029, 8134])
    },
    "SkinOfUpperEyelidMargin": {
        "130322": ("Skin of upper eyelid margin", [4029, 8134])
    },
    "SkinOfUpperParaspinalRegion": {
        "130302": ("Skin of upper paraspinal region", [4029, 8134])
    },
    "SleepStageN1": {"130834": ("Sleep Stage N1", [3035, 3046])},
    "SleepStageN2": {"130835": ("Sleep Stage N2", [3035, 3046])},
    "SleepStageN3": {"130836": ("Sleep Stage N3", [3035, 3046])},
    "SliceContributionToEjectionFraction": {
        "122498": ("Slice Contribution to Ejection Fraction", [])
    },
    "SliceThickness": {"112225": ("Slice Thickness", [])},
    "SlideIdentifier": {"121043": ("Slide Identifier", [])},
    "SlideMicroscopy": {"SM": ("Slide Microscopy", [29, 30, 33])},
    "SlideMicroscopyPathologyImaging": {"112702": ("Slide Microscopy", [8131])},
    "SlideUID": {"121044": ("Slide UID", [])},
    "SlidingWindowBeam": {"130106": ("Sliding Window Beam", [9511, 9524, 9583])},
    "SlowDiffusionCoefficient": {
        "113291": ("Slow Diffusion Coefficient", [218, 7180, 7272, 7469])
    },
    "SmallIrregularOpacities": {
        "112125": ("Small irregular opacities", [6102, 6104, 6106])
    },
    "SmallLesserThan32Point0cmLateralThickness": {
        "113601": ("Small: < 32.0 cm lateral thickness", [7041, 7042])
    },
    "SmallRoundedOpacities": {"112126": ("Small rounded opacities", [6102, 6104])},
    "SmallZoneEmphasis": {"128821": ("Small Zone Emphasis", [])},
    "SmallZoneHighGrayLevelEmphasis": {
        "128826": ("Small Zone High Gray Level Emphasis", [])
    },
    "SmallZoneLowGrayLevelEmphasis": {
        "128825": ("Small Zone Low Gray Level Emphasis", [])
    },
    "Smoothing": {"113087": ("Smoothing", [7203])},
    "SoftPlaque": {"122356": ("Soft plaque", [3491])},
    "SoftTissue": {"112144": ("Soft tissue", [])},
    "SoftwareConfiguration": {"110131": ("Software Configuration", [401, 403])},
    "SoftwareFailure": {"111218": ("Software failure", [6041, 6135, 7011])},
    "SolidMass": {"111462": ("Solid mass", [6054, 6064])},
    "Sonazoid": {"125905": ("Sonazoid", [12030])},
    "Sonographer": {"121093": ("Sonographer", [])},
    "Sonologist": {"121092": ("Sonologist", [7450, 7452])},
    "Sonovue": {"125906": ("SonoVue", [12030])},
    "SourceContinuousRollAngle": {"130798": ("Source Continuous Roll Angle", [])},
    "SourceDeformableSpatialRegistration": {
        "125028": ("Source Deformable Spatial Registration", [7013, 7060])
    },
    "SourceDocument": {"121335": ("Source Document", [7006])},
    "SourceDoseForComposingCurrentDose": {
        "121372": ("Source dose for composing current dose", [7221])
    },
    "SourceImage": {"121324": ("Source image", [7060, 7061, 7205])},
    "SourceImageDiffusionBValue": {
        "113240": ("Source image diffusion b-value", [7276, 9000])
    },
    "SourceImageForImageProcessingOperation": {
        "121322": ("Source image for image processing operation", [4264, 7202, 7222])
    },
    "SourceImageForMontage": {"121329": ("Source image for montage", [7202])},
    "SourceImageForSegmentation": {"121233": ("Source image for segmentation", [])},
    "SourceMeasurement": {"128224": ("Source measurement", [7013, 7060])},
    "SourceMedia": {"110155": ("Source Media", [402])},
    "SourceOfDoseInformation": {"113854": ("Source of Dose Information", [])},
    "SourceOfMeasurement": {"121112": ("Source of Measurement", [7003, 7551, 7552])},
    "SourcePositionToleranceViolation": {
        "130473": ("Source Position Tolerance Violation", [9567])
    },
    "SourceRawData": {"128226": ("Source raw data", [7013, 7060])},
    "SourceRealWorldValueMap": {
        "128227": ("Source real world value map", [7013, 7019, 7060])
    },
    "SourceReport": {"128225": ("Source report", [7013, 7060])},
    "SourceRoleID": {"110153": ("Source Role ID", [402])},
    "SourceSeriesForImageProcessingOperation": {
        "121323": ("Source series for image processing operation", [7210])
    },
    "SourceSeriesForSegmentation": {"121232": ("Source series for segmentation", [])},
    "SpacingBetweenSlices": {"112226": ("Spacing between slices", [])},
    "SpatialDisplacementXComponent": {
        "110822": ("Spatial Displacement X Component", [218, 7180, 7469])
    },
    "SpatialDisplacementYComponent": {
        "110823": ("Spatial Displacement Y Component", [218, 7180, 7469])
    },
    "SpatialDisplacementZComponent": {
        "110824": ("Spatial Displacement Z Component", [218, 7180, 7469])
    },
    "SpatialFiducials": {"128447": ("Spatial Fiducials", [])},
    "SpatialFiducialsProducer": {"FID": ("Spatial Fiducials", [32, 33])},
    "SpatialRegistration": {"112353": ("Spatial Registration", [])},
    "SpatialRegistrationReference": {"128444": ("Spatial Registration Reference", [])},
    "SpatialResampling": {"113085": ("Spatial resampling", [7203])},
    "SpatiallyRelatedFramesExtractedFromTheVolume": {
        "113091": ("Spatially-related frames extracted from the volume", [7203])
    },
    "SpecialProcedureNote": {"130025": ("Special Procedure Note", [])},
    "SpecificObservationSubjectClass": {
        "130780": ("Specific observation subject class", [])
    },
    "SpecifiedConformationNumber": {
        "130076": ("Specified Conformation Number", [9500])
    },
    "SpecifiedConformityIndex": {"130074": ("Specified Conformity Index", [9500])},
    "SpecifiedHealthyTissueConformityIndex": {
        "130075": ("Specified Healthy Tissue Conformity Index", [9500])
    },
    "SpecifiedHomogeneityIndex": {"130077": ("Specified Homogeneity Index", [9500])},
    "SpecifiedRadiationDose": {"130019": ("Specified Radiation Dose", [])},
    "SpecifiedVolumePercentage": {"130021": ("Specified Volume Percentage", [])},
    "SpecifiedVolumeSize": {"130020": ("Specified Volume Size", [])},
    "Specimen": {"121027": ("Specimen", [271])},
    "SpecimenAccessionNumber": {"121040": ("Specimen Accession Number", [])},
    "SpecimenContainerIdentifier": {"111700": ("Specimen Container Identifier", [])},
    "SpecimenIdentifier": {"121041": ("Specimen Identifier", [])},
    "SpecimenStorage": {"111729": ("Specimen storage", [8111])},
    "SpecimenType": {"121042": ("Specimen Type", [])},
    "SpecimenUID": {"121039": ("Specimen UID", [])},
    "SpectaclePrescriptionReport": {"111671": ("Spectacle Prescription Report", [])},
    "SpectralDomain": {"111921": ("Spectral domain", [4261])},
    "SpectralFilteredImage": {"121340": ("Spectral filtered image", [7201])},
    "Spectroscopy": {"121317": ("Spectroscopy", [7201])},
    "SpectroscopyDataForWaterPhaseCorrection": {
        "121318": ("Spectroscopy Data for Water Phase Correction", [7215])
    },
    "SpeedOfSound": {"110832": ("Speed of sound", [218, 7180, 7469])},
    "SpeedOfSoundMapCorrection": {"130820": ("Speed of Sound Map Correction", [11004])},
    "Sphere": {"122485": ("Sphere", [8, 3451, 4051, 7111, 7112, 7151, 7193])},
    "SphericalBackProjection": {"130821": ("Spherical Back Projection", [11005])},
    "SphericalDeconvolution": {"113238": ("Spherical Deconvolution", [7261])},
    "SphericalProjection": {"111791": ("Spherical projection", [4245])},
    "Spiculated": {"112136": ("Spiculated", [6119])},
    "SpinTaggingPerfusionMRSignalIntensity": {
        "110800": ("Spin Tagging Perfusion MR Signal Intensity", [218, 7180, 7469])
    },
    "SpontaneousAbortion": {"111566": ("Spontaneous Abortion", [6087, 6088])},
    "SpotMagnificationViews": {"111136": ("Spot magnification view(s)", [6028, 6029])},
    "SquamousCellCarcinomaOfTheNipple": {
        "111340": ("Squamous cell carcinoma of the nipple", [6030, 6033])
    },
    "Stage0": {"111494": ("Stage 0", [6068])},
    "StageI": {"111495": ("Stage I", [6068])},
    "StageIIA": {"111496": ("Stage IIA", [6068])},
    "StageIIB": {"111497": ("Stage IIB", [6068])},
    "StageIIIA": {"111498": ("Stage IIIA", [6068])},
    "StageIIIB": {"111499": ("Stage IIIB", [6068])},
    "StageIIIC": {"111500": ("Stage IIIC", [6068])},
    "StageIV": {"111501": ("Stage IV", [6068])},
    "StageView": {"113017": ("Stage-View", [7012])},
    "StagedMeasurements": {"125310": ("Staged Measurements", [])},
    "StagingPriorToImaging": {"127103": ("Staging prior to imaging", [634])},
    "StandardDeviation": {"113061": ("Standard Deviation", [])},
    "StandardDeviationDistanceOfCalcificationsInACluster": {
        "112201": ("Standard deviation distance of calcifications in a cluster", [6142])
    },
    "StandardDeviationOfAttenuationCoefficient": {
        "112183": ("Standard Deviation of Attenuation Coefficient", [6141])
    },
    "StandardDeviationOfMeasurementsUsed": {
        "111786": ("Standard Deviation of measurements used", [4243])
    },
    "StandardDeviationOfPopulation": {
        "121414": ("Standard deviation of population", [221, 226, 7465])
    },
    "StandardFractionation": {"130097": ("Standard Fractionation", [9533])},
    "StandardToftsModel": {"126340": ("Standard Tofts Model", [4101, 4106])},
    "StandardizedAddedMetabolicActivity": {
        "126037": ("Standardized Added Metabolic Activity", [7466])
    },
    "StandardizedAddedMetabolicActivityBackground": {
        "126038": ("Standardized Added Metabolic Activity Background", [7466])
    },
    "StandardizedAddedMetabolicActivitySAM": {
        "126037": ("Standardized Added Metabolic Activity (SAM)", [])
    },
    "StandardizedAddedMetabolicActivitySAMBackground": {
        "126038": ("Standardized Added Metabolic Activity (SAM) Background", [])
    },
    "StandardizedUptakeValue": {
        "126400": ("Standardized Uptake Value", [218, 7180, 7469])
    },
    "Standby": {"121101": ("Standby", [7453])},
    "StartOfAtrialContraction": {"109045": ("Start of atrial contraction", [3339])},
    "StartOfAtrialContractionSubsequent": {
        "109046": ("Start of atrial contraction (subsequent)", [3339])
    },
    "StartOfContrastAgentAdministration": {
        "125234": ("Start of contrast agent administration", [12031])
    },
    "StartOfDrugDoseAdministration": {
        "125233": ("Start of drug dose administration", [12031])
    },
    "StartOfExpiration": {"109029": ("Start of expiration", [3337])},
    "StartOfInspiration": {"109030": ("Start of inspiration", [3337])},
    "StartOfThermalCO": {"109031": ("Start of thermal CO", [3337])},
    "StartOfThermalCardiacOutputBolus": {
        "109031": ("Start of thermal cardiac output bolus", [])
    },
    "StartOfXRayIrradiation": {"113809": ("Start of X-Ray Irradiation", [])},
    "StartProcedureAction": {"121130": ("Start Procedure Action", [3421])},
    "StartProcedureActionItem": {"121130": ("Start Procedure Action Item", [])},
    "StartingFlowRateOfAdministration": {
        "130208": ("Starting Flow Rate of administration", [])
    },
    "Statement": {"122152": ("Statement", [])},
    "StatementModifier": {"122153": ("Statement Modifier", [])},
    "StaticBeam": {"130102": ("Static Beam", [9511, 9524, 9583])},
    "StationAETitle": {"110119": ("Station AE Title", [])},
    "StationaryAcquisition": {"113611": ("Stationary Acquisition", [10002])},
    "StationaryAcquisitionCT": {"113806": ("Stationary Acquisition", [10013])},
    "StenoticFlowReserve": {"122548": ("Stenotic Flow Reserve", [])},
    "StentComposition": {"122685": ("Stent Composition", [])},
    "StentDiameterRatio": {"122351": ("Stent Diameter Ratio", [3484])},
    "StentDisintegration": {"122684": ("stent disintegration", [3813])},
    "StentExpansionIndex": {"122347": ("Stent Expansion Index", [3484])},
    "StentFracture": {"122683": ("stent fracture", [3813])},
    "StentGap": {"122364": ("Stent Gap", [3483])},
    "StentReExpanded": {"122306": ("Stent re-expanded", [3412])},
    "StentSymmetryIndex": {"122346": ("Stent Symmetry Index", [3484])},
    "StentVolumeObstruction": {"122339": ("Stent Volume Obstruction", [])},
    "StentedRegion": {"122384": ("Entire Pullback", [3487])},
    "StepAndShootBeam": {"130105": ("Step and Shoot Beam", [9511, 9524, 9583])},
    "SteppingAcquisition": {"113612": ("Stepping Acquisition", [10002])},
    "StereometricRelationship": {"SMR": ("Stereometric Relationship", [32, 33])},
    "StereotacticCoordinates": {"127450": ("Stereotactic coordinates", [])},
    "StereotacticSetupMethod": {"130633": ("Stereotactic Setup Method", [9571])},
    "StimulationAtRate1Interval": {
        "109047": ("Stimulation at rate 1 interval", [3339])
    },
    "StimulationAtRate2Interval": {
        "109048": ("Stimulation at rate 2 interval", [3339])
    },
    "StimulationAtRate3Interval": {
        "109049": ("Stimulation at rate 3 interval", [3339])
    },
    "StimulationAtRate4Interval": {
        "109050": ("Stimulation at rate 4 interval", [3339])
    },
    "StimulationMode": {"130491": ("Stimulation Mode", [])},
    "StimulusSamplePosition": {"130492": ("Stimulus Sample Position", [])},
    "StimulusTimeOffset": {"130493": ("Stimulus Time Offset", [])},
    "StorageDetector": {"113950": ("Storage Detector", [10030])},
    "Strain": {"127411": ("Strain", [])},
    "StrainDescription": {"127412": ("Strain description", [])},
    "Streamline": {"113215": ("Streamline", [7262])},
    "StressProtocol": {"109056": ("Stress Protocol", [])},
    "StressTechnetiumRestTechnetium1DayProcedure": {
        "122784": ("Stress technetium/rest technetium 1 day procedure", [3110])
    },
    "StressTestScore": {"122760": ("Stress test score", [])},
    "StretchedExponentialDiffusionModel": {
        "113254": ("Stretched exponential diffusion model", [7273])
    },
    "StripFilter": {"113650": ("Strip filter", [10007])},
    "Stripe": {"112094": ("Stripe", [6102, 6109, 6113, 6148, 7151, 7192, 9514])},
    "StronglyPositive": {"122755": ("Strongly positive", [3231])},
    "StronglyPositiveSTElevation": {
        "122756": ("Strongly positive - ST elevation", [3231])
    },
    "StructuralImageForImageProcessing": {
        "128250": ("Structural image for image processing", [7202])
    },
    "StructureOfTheFindingSite": {"125311": ("Structure of the Finding Site", [12302])},
    "StructuredReportDocument": {"SR": ("Structured Report Document", [32, 33])},
    "StudentTTest": {"113068": ("Student's T-test", [218, 7180, 7469])},
    "StudiesImported": {"110027": ("Studies Imported", [7008])},
    "Study": {"113014": ("Study", [272, 7012, 10000])},
    "StudyDate": {"111060": ("Study Date", [])},
    "StudyInstanceUID": {"110180": ("Study Instance UID", [404, 10001])},
    "StudyPerformedDidNotMatchRequest": {
        "130573": ("Study performed did not match request", [6314])
    },
    "StudyTime": {"111061": ("Study Time", [])},
    "SubjectAge": {"121033": ("Subject Age", [])},
    "SubjectBirthDate": {"121031": ("Subject Birth Date", [])},
    "SubjectBreed": {"121035": ("Subject Breed", [])},
    "SubjectClass": {"121024": ("Subject Class", [])},
    "SubjectID": {"121030": ("Subject ID", [])},
    "SubjectName": {"121029": ("Subject Name", [])},
    "SubjectSex": {"121032": ("Subject Sex", [])},
    "SubjectSpecies": {"121034": ("Subject Species", [])},
    "SubjectTimePointIdentifier": {"126070": ("Subject Time Point Identifier", [])},
    "SubjectUID": {"121028": ("Subject UID", [])},
    "SubjectiveRefraction": {"SRF": ("Subjective Refraction", [29, 30, 33])},
    "SuboptimalContrastTiming": {"128554": ("Suboptimal contrast timing", [10034])},
    "SuboptimalPatientPreparation": {
        "130575": ("Suboptimal patient preparation", [6314])
    },
    "Subpleural": {"112153": ("Subpleural", [])},
    "SubpleuralLine": {"112115": ("Subpleural line", [6102, 6103])},
    "SubscapularFossa": {"112098": ("Subscapular Fossa", [6115, 8134])},
    "SubstanceUseHistory": {"111545": ("Substance Use History", [])},
    "SubtractionImageMissing": {"130581": ("Subtraction image missing", [6315, 6317])},
    "Succeeded": {"111222": ("Succeeded", [6042])},
    "SuccessfulAnalyses": {"111062": ("Successful Analyses", [])},
    "SuccessfulDetections": {"111063": ("Successful Detections", [])},
    "SuggestiveOfMalignancyTakeAppropriateAction": {
        "111146": ("Suggestive of malignancy - take appropriate action", [6028, 6029])
    },
    "SumAverageOfGLCM": {"128787": ("Sum Average of GLCM", [])},
    "SumEntropyOfGLCM": {"128789": ("Sum Entropy of GLCM", [])},
    "SumOfSegmentedVoxelMethodForVolume": {
        "126030": ("Sum of segmented voxel method for volume", [7474])
    },
    "SumVarianceOfGLCM": {"128788": ("Sum Variance of GLCM", [])},
    "Summary": {"121111": ("Summary", [])},
    "SummaryOfAnalyses": {"111065": ("Summary of Analyses", [])},
    "SummaryOfDetections": {"111064": ("Summary of Detections", [])},
    "SuperficialRetinaStructuralReflectanceMap": {
        "128266": ("Superficial retina structural reflectance map", [4271])
    },
    "SuperficialRetinaVasculatureFlow": {
        "128265": ("Superficial retina vasculature flow", [4271])
    },
    "SuperiorLongitudinalFasciculusI": {
        "110703": ("superior longitudinal fasciculus I", [7703, 7710, 8134])
    },
    "SuperiorLongitudinalFasciculusII": {
        "110704": ("superior longitudinal fasciculus II", [7703, 7710, 8134])
    },
    "SuperiorLongitudinalFasciculusIII": {
        "110705": ("superior longitudinal fasciculus III", [7703, 7710, 8134])
    },
    "SuperolateralToInferomedial": {
        "111434": ("Superolateral to inferomedial", [6065])
    },
    "SuperomedialToInferolateral": {
        "111435": ("Superomedial to inferolateral", [6065])
    },
    "SupplementaryData": {"111414": ("Supplementary Data", [])},
    "SupplementaryDataForIntervention": {
        "111463": ("Supplementary Data for Intervention", [])
    },
    "SupportingInformation": {"112359": ("Supporting Information", [])},
    "SurfaceBetweenInnerAndOuterSegmentsOfThePhotoreceptors": {
        "128295": (
            "Surface between Inner and Outer Segments of the photoreceptors",
            [4273, 7151, 7192, 9514],
        )
    },
    "SurfaceContourMapping": {"111792": ("Surface contour mapping", [4245])},
    "SurfaceOfTheCenterOfTheRPE": {
        "128298": ("Surface of the center of the RPE", [4273, 7151, 7192, 9514])
    },
    "SurfaceOfTheChoroidScleraInterface": {
        "128301": ("Surface of the choroid-sclera interface", [4273, 7151, 7192, 9514])
    },
    "SurfaceOfTheInterdigitatingZoneBetweenRetinaAndRPE": {
        "128296": (
            "Surface of the interdigitating zone between retina and RPE",
            [4273, 7151, 7192, 9514],
        )
    },
    "SurfaceRendering": {"113075": ("Surface rendering", [7203])},
    "Surgeon": {"121091": ("Surgeon", [])},
    "SurgeonFactor": {"111773": ("Surgeon Factor", [4237])},
    "SurgicalConsult": {"111410": ("Surgical consult", [6028, 6050, 6083])},
    "Survey": {"111128": ("Survey", [6051, 6058, 6061])},
    "SuspendProcedureAction": {"121132": ("Suspend Procedure Action", [3421])},
    "SuspendProcedureActionItem": {"121132": ("Suspend Procedure Action Item", [])},
    "Suvbsa": {"126403": ("SUVbsa", [218, 7180, 7469])},
    "Suvbw": {"126401": ("SUVbw", [218, 7180, 7469])},
    "Suvibw": {"126404": ("SUVibw", [218, 7180, 7469])},
    "Suvlbm": {"126402": ("SUVlbm", [218, 7180, 7469])},
    "SuvlbmJames128": {"126406": ("SUVlbm(James128)", [218, 7180, 7469])},
    "SuvlbmJanma": {"126405": ("SUVlbm(Janma)", [218, 7180, 7469])},
    "SynchronizedRoboticTreatment": {
        "130139": ("Synchronized Robotic Treatment", [9523, 9524])
    },
    "SynthesizingEquipment": {"109100": ("Synthesizing Equipment", [7005])},
    "SyringeAttached": {"130163": ("Syringe attached", [71])},
    "SyringeDetached": {"130164": ("Syringe detached", [71])},
    "SystemCalculated": {"113940": ("System Calculated", [10020])},
    "SystemicFlow": {"122162": ("Systemic Flow", [3618])},
    "SystolicBloodVelocityMean": {
        "122203": (
            "Systolic blood velocity, mean",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "SystolicBloodVelocityPeak": {
        "122204": (
            "Systolic blood velocity, peak",
            [3612, 12261, 12263, 12264, 12266, 12267, 12268, 12276],
        )
    },
    "SystolicPeakPressure": {"109033": ("Systolic peak pressure", [])},
    "SystolicPressureAverage": {"109032": ("Systolic pressure, average", [])},
    "T1": {"113063": ("T1", [218, 7180, 7469])},
    "T1ByFixedValue": {"126352": ("T1 by Fixed Value", [4100, 4106])},
    "T1ByInversionRecovery": {"126351": ("T1 by Inversion Recovery", [4100, 4106])},
    "T1ByMultipleFlipAngles": {"126350": ("T1 by Multiple Flip Angles", [4100, 4106])},
    "T1UsedForCalculation": {"126353": ("T1 Used For Calculation", [])},
    "T1WeightedDynamicContrastEnhancedMRSignalIntensity": {
        "110816": (
            "T1 Weighted Dynamic Contrast Enhanced MR Signal Intensity",
            [218, 7180, 7469],
        )
    },
    "T1WeightedMRSignalIntensity": {
        "110804": ("T1 Weighted MR Signal Intensity", [218, 6311, 7180, 7469])
    },
    "T1Worst": {"122367": ("T-1 Worst", [3493])},
    "T2": {"113065": ("T2", [218, 7180, 7469])},
    "T2Secondary": {"122368": ("T-2 Secondary", [3493])},
    "T2Star": {"113064": ("T2*", [218, 7180, 7469])},
    "T2StarWeightedDynamicContrastEnhancedMRSignalIntensity": {
        "110818": (
            "T2* Weighted Dynamic Contrast Enhanced MR Signal Intensity",
            [218, 7180, 7469],
        )
    },
    "T2StarWeightedMRSignalIntensity": {
        "110806": ("T2* Weighted MR Signal Intensity", [218, 7180, 7469])
    },
    "T2WeightedDynamicContrastEnhancedMRSignalIntensity": {
        "110817": (
            "T2 Weighted Dynamic Contrast Enhanced MR Signal Intensity",
            [218, 7180, 7469],
        )
    },
    "T2WeightedMRSignalIntensity": {
        "110805": ("T2 Weighted MR Signal Intensity", [218, 6311, 7180, 7469])
    },
    "T3Secondary": {"122369": ("T-3 Secondary", [3493])},
    "T4Secondary": {"122370": ("T-4 Secondary", [3493])},
    "T807F18": {"126502": ("T807 F^18^", [4021])},
    "TBISetupMethod": {"130632": ("TBI Setup Method", [9571])},
    "TEND": {"113216": ("TEND", [7262])},
    "TG18ADPattern": {"109845": ("TG18-AD Pattern", [8301])},
    "TG18AFCPattern": {"109861": ("TG18-AFC Pattern", [8301])},
    "TG18BRPattern": {"109802": ("TG18-BR Pattern", [8301])},
    "TG18CHImage": {"109878": ("TG18-CH Image", [8301])},
    "TG18CTPattern": {"109804": ("TG18-CT Pattern", [8301])},
    "TG18CXPattern": {"109854": ("TG18-CX Pattern", [8301])},
    "TG18GA03Pattern": {"109870": ("TG18-GA03 Pattern", [8301])},
    "TG18GA05Pattern": {"109871": ("TG18-GA05 Pattern", [8301])},
    "TG18GA08Pattern": {"109872": ("TG18-GA08 Pattern", [8301])},
    "TG18GA10Pattern": {"109873": ("TG18-GA10 Pattern", [8301])},
    "TG18GA15Pattern": {"109874": ("TG18-GA15 Pattern", [8301])},
    "TG18GA20Pattern": {"109875": ("TG18-GA20 Pattern", [8301])},
    "TG18GA25Pattern": {"109876": ("TG18-GA25 Pattern", [8301])},
    "TG18GA30Pattern": {"109877": ("TG18-GA30 Pattern", [8301])},
    "TG18GQBPattern": {"109869": ("TG18-GQB Pattern", [8301])},
    "TG18GQNPattern": {"109868": ("TG18-GQN Pattern", [8301])},
    "TG18GQPattern": {"109867": ("TG18-GQ Pattern", [8301])},
    "TG18GVNPattern": {"109866": ("TG18-GVN Pattern", [8301])},
    "TG18GVPattern": {"109865": ("TG18-GV Pattern", [8301])},
    "TG18KNImage": {"109879": ("TG18-KN Image", [8301])},
    "TG18LN1201Pattern": {"109823": ("TG18-LN12-01 Pattern", [8301])},
    "TG18LN1202Pattern": {"109824": ("TG18-LN12-02 Pattern", [8301])},
    "TG18LN1203Pattern": {"109825": ("TG18-LN12-03 Pattern", [8301])},
    "TG18LN1204Pattern": {"109826": ("TG18-LN12-04 Pattern", [8301])},
    "TG18LN1205Pattern": {"109827": ("TG18-LN12-05 Pattern", [8301])},
    "TG18LN1206Pattern": {"109828": ("TG18-LN12-06 Pattern", [8301])},
    "TG18LN1207Pattern": {"109829": ("TG18-LN12-07 Pattern", [8301])},
    "TG18LN1208Pattern": {"109830": ("TG18-LN12-08 Pattern", [8301])},
    "TG18LN1209Pattern": {"109831": ("TG18-LN12-09 Pattern", [8301])},
    "TG18LN1210Pattern": {"109832": ("TG18-LN12-10 Pattern", [8301])},
    "TG18LN1211Pattern": {"109833": ("TG18-LN12-11 Pattern", [8301])},
    "TG18LN1212Pattern": {"109834": ("TG18-LN12-12 Pattern", [8301])},
    "TG18LN1213Pattern": {"109835": ("TG18-LN12-13 Pattern", [8301])},
    "TG18LN1214Pattern": {"109836": ("TG18-LN12-14 Pattern", [8301])},
    "TG18LN1215Pattern": {"109837": ("TG18-LN12-15 Pattern", [8301])},
    "TG18LN1216Pattern": {"109838": ("TG18-LN12-16 Pattern", [8301])},
    "TG18LN1217Pattern": {"109839": ("TG18-LN12-17 Pattern", [8301])},
    "TG18LN1218Pattern": {"109840": ("TG18-LN12-18 Pattern", [8301])},
    "TG18LN801Pattern": {"109805": ("TG18-LN8-01 Pattern", [8301])},
    "TG18LN802Pattern": {"109806": ("TG18-LN8-02 Pattern", [8301])},
    "TG18LN803Pattern": {"109807": ("TG18-LN8-03 Pattern", [8301])},
    "TG18LN804Pattern": {"109808": ("TG18-LN8-04 Pattern", [8301])},
    "TG18LN805Pattern": {"109809": ("TG18-LN8-05 Pattern", [8301])},
    "TG18LN806Pattern": {"109810": ("TG18-LN8-06 Pattern", [8301])},
    "TG18LN807Pattern": {"109811": ("TG18-LN8-07 Pattern", [8301])},
    "TG18LN808Pattern": {"109812": ("TG18-LN8-08 Pattern", [8301])},
    "TG18LN809Pattern": {"109813": ("TG18-LN8-09 Pattern", [8301])},
    "TG18LN810Pattern": {"109814": ("TG18-LN8-10 Pattern", [8301])},
    "TG18LN811Pattern": {"109815": ("TG18-LN8-11 Pattern", [8301])},
    "TG18LN812Pattern": {"109816": ("TG18-LN8-12 Pattern", [8301])},
    "TG18LN813Pattern": {"109817": ("TG18-LN8-13 Pattern", [8301])},
    "TG18LN814Pattern": {"109818": ("TG18-LN8-14 Pattern", [8301])},
    "TG18LN815Pattern": {"109819": ("TG18-LN8-15 Pattern", [8301])},
    "TG18LN816Pattern": {"109820": ("TG18-LN8-16 Pattern", [8301])},
    "TG18LN817Pattern": {"109821": ("TG18-LN8-17 Pattern", [8301])},
    "TG18LN818Pattern": {"109822": ("TG18-LN8-18 Pattern", [8301])},
    "TG18LPH10Pattern": {"109855": ("TG18-LPH10 Pattern", [8301])},
    "TG18LPH50Pattern": {"109856": ("TG18-LPH50 Pattern", [8301])},
    "TG18LPH89Pattern": {"109857": ("TG18-LPH89 Pattern", [8301])},
    "TG18LPV10Pattern": {"109858": ("TG18-LPV10 Pattern", [8301])},
    "TG18LPV50Pattern": {"109859": ("TG18-LPV50 Pattern", [8301])},
    "TG18LPV89Pattern": {"109860": ("TG18-LPV89 Pattern", [8301])},
    "TG18MM1Image": {"109880": ("TG18-MM1 Image", [8301])},
    "TG18MM2Image": {"109881": ("TG18-MM2 Image", [8301])},
    "TG18MPPattern": {"109846": ("TG18-MP Pattern", [8301])},
    "TG18NS10Pattern": {"109862": ("TG18-NS10 Pattern", [8301])},
    "TG18NS50Pattern": {"109863": ("TG18-NS50 Pattern", [8301])},
    "TG18NS89Pattern": {"109864": ("TG18-NS89 Pattern", [8301])},
    "TG18PQCPattern": {"109803": ("TG18-PQC Pattern", [8301])},
    "TG18PXPattern": {"109853": ("TG18-PX Pattern", [8301])},
    "TG18QCPattern": {"109801": ("TG18-QC Pattern", [8301])},
    "TG18RH10Pattern": {"109847": ("TG18-RH10 Pattern", [8301])},
    "TG18RH50Pattern": {"109848": ("TG18-RH50 Pattern", [8301])},
    "TG18RH89Pattern": {"109849": ("TG18-RH89 Pattern", [8301])},
    "TG18RV10Pattern": {"109850": ("TG18-RV10 Pattern", [8301])},
    "TG18RV50Pattern": {"109851": ("TG18-RV50 Pattern", [8301])},
    "TG18RV89Pattern": {"109852": ("TG18-RV89 Pattern", [8301])},
    "TG18UN10Pattern": {"109841": ("TG18-UN10 Pattern", [8301])},
    "TG18UN80Pattern": {"109842": ("TG18-UN80 Pattern", [8301])},
    "TG18UNL10Pattern": {"109843": ("TG18-UNL10 Pattern", [8301, 8302])},
    "TG18UNL80Pattern": {"109844": ("TG18-UNL80 Pattern", [8301, 8302])},
    "THK5351F18": {"126717": ("THK5351 F^18^", [])},
    "TRC10589Zr": {"126749": ("TRC105 ^89^Zr", [4021])},
    "TSPyramid": {"122564": ("TS Pyramid", [3453])},
    "TWave": {"109051": ("T wave", [3339])},
    "Table": {"128459": ("Table", [10066])},
    "TableCore": {"128460": ("Table Core", [10066])},
    "TableCradleTiltAngle": {"113756": ("Table Cradle Tilt Angle", [])},
    "TableCradleTiltEndAngle": {"128765": ("Table Cradle Tilt End Angle", [])},
    "TableHeadTiltAngle": {"113754": ("Table Head Tilt Angle", [])},
    "TableHeadTiltEndAngle": {"128763": ("Table Head Tilt End Angle", [])},
    "TableHeightEndPosition": {"113761": ("Table Height End Position", [10008])},
    "TableHeightPosition": {"113753": ("Table Height Position", [10008])},
    "TableHorizontalRotationAngle": {"113755": ("Table Horizontal Rotation Angle", [])},
    "TableHorizontalRotationEndAngle": {
        "128764": ("Table Horizontal Rotation End Angle", [])
    },
    "TableLateralEndPosition": {"113760": ("Table Lateral End Position", [10008])},
    "TableLateralPosition": {"113752": ("Table Lateral Position", [10008])},
    "TableLongitudinalEndPosition": {
        "113759": ("Table Longitudinal End Position", [10008])
    },
    "TableLongitudinalPosition": {"113751": ("Table Longitudinal Position", [10008])},
    "TableMount": {"113952": ("Table Mount", [10031])},
    "TableOfValues": {"121424": ("Table of Values", [228])},
    "TableOfValuesCitation": {"121422": ("Table of Values Citation", [228])},
    "TableOuterLiner": {"128461": ("Table Outer Liner", [10066])},
    "TablePad": {"128462": ("Table Pad", [10066])},
    "TableXEndPositionToIsocenter": {
        "128769": ("Table X End Position to Isocenter", [10008])
    },
    "TableXPositionToIsocenter": {"128766": ("Table X Position to Isocenter", [10008])},
    "TableYEndPositionToIsocenter": {
        "128770": ("Table Y End Position to Isocenter", [10008])
    },
    "TableYPositionToIsocenter": {"128767": ("Table Y Position to Isocenter", [10008])},
    "TableZEndPositionToIsocenter": {
        "128771": ("Table Z End Position to Isocenter", [10008])
    },
    "TableZPositionToIsocenter": {"128768": ("Table Z Position to Isocenter", [10008])},
    "TabularDataAlgorithm": {"128479": ("Tabular Data Algorithm", [10068])},
    "TapTestBegin": {"130895": ("Tap test begin", [3039])},
    "TapTestEnd": {"130896": ("Tap test end", [3039])},
    "TargestarB": {"125907": ("Targestar-B", [12030])},
    "TargestarP": {"125908": ("Targestar-P", [12030])},
    "Target": {"112162": ("Target", [6132])},
    "TargetCompletelyContainedInTheSpecimen": {
        "111443": ("Target completely contained in the specimen", [])
    },
    "TargetContainedInTheSpecimen": {
        "111443": ("Target contained in the specimen", [6066])
    },
    "TargetContentItemsAreRelatedContraLaterally": {
        "111155": ("Target Content Items are related contra-laterally", [6035])
    },
    "TargetContentItemsAreRelatedSpatially": {
        "111154": ("Target Content Items are related spatially", [6035])
    },
    "TargetContentItemsAreRelatedTemporally": {
        "111153": ("Target Content Items are related temporally", [6035])
    },
    "TargetExposureIndex": {"113846": ("Target Exposure Index", [])},
    "TargetLesionAtBaseline": {"112074": ("Target Lesion at Baseline", [6145])},
    "TargetLesionCompleteResponse": {
        "112041": ("Target Lesion Complete Response", [6143, 6144])
    },
    "TargetLesionPartialResponse": {
        "112042": ("Target Lesion Partial Response", [6143, 6144])
    },
    "TargetLesionProgressiveDisease": {
        "112043": ("Target Lesion Progressive Disease", [6143, 6144])
    },
    "TargetLesionStableDisease": {
        "112044": ("Target Lesion Stable Disease", [6143, 6144])
    },
    "TargetNotInTheSpecimen": {"111445": ("Target not in the specimen", [6066])},
    "TargetPartiallyObtainedInTheSpecimen": {
        "111444": ("Target partially obtained in the specimen", [6066])
    },
    "TargetRegion": {"123014": ("Target Region", [])},
    "Targeted": {"111127": ("Targeted", [6051, 6058, 6061])},
    "Tau": {
        "109072": (
            "Tau",
            [
                3337,
                3613,
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "TauM": {"126330": ("tau_m", [218, 4107, 7180, 7469])},
    "TaxonomicRankOfOrigin": {"127402": ("Taxonomic rank of origin", [])},
    "TeachingFileCategory": {"128723": ("Teaching File Category", [])},
    "TechNote": {"121171": ("Tech Note", [3401])},
    "TechnicalFactorsMissing": {"111194": ("Technical factors missing", [6041])},
    "Technologist": {"121083": ("Technologist", [])},
    "Teichholz": {"125209": ("Teichholz", [12227, 12228])},
    "TeletherapyIsotope": {"130040": ("Teletherapy Isotope", [])},
    "TemperatureEncoded": {"113067": ("Temperature encoded", [218, 7180, 7469])},
    "TemporalDerivativeExceedsThreshold": {
        "126373": ("Temporal Derivative Exceeds Threshold", [4104, 4106])
    },
    "TemporalDerivativeThreshold": {
        "126374": ("Temporal Derivative Threshold", [218, 4109, 7180, 7469])
    },
    "TemporalPredecessor": {"121348": ("Temporal Predecessor", [7201])},
    "TemporalSuccessor": {"121349": ("Temporal Successor", [7201])},
    "TemporallyRelatedFramesExtractedFromTheSetOfVolumes": {
        "113092": (
            "Temporally-related frames extracted from the set of volumes",
            [7203],
        )
    },
    "TerminatedByScanner": {"130177": ("Terminated by scanner", [67, 71])},
    "TerminatedDueToAirDetected": {
        "130176": ("Terminated due to air detected", [67, 71])
    },
    "TerminatedDueToConsumableRemoval": {
        "130179": ("Terminated due to consumable removal", [67, 71])
    },
    "TerminatedDueToCriticalBatteryLevel": {
        "130178": ("Terminated due to critical battery level", [67, 71])
    },
    "TerminatedDueToExcessiveDurationPause": {
        "130158": ("Terminated due to excessive duration pause", [67, 71])
    },
    "TerminatedDueToFlowRateAboveTerminationLimit": {
        "130157": ("Terminated due to flow rate above termination limit", [67, 71])
    },
    "TerminatedDueToInjectorCommunicationLoss": {
        "130159": ("Terminated due to injector communication loss", [67, 71])
    },
    "TerminatedDueToPressureAboveTerminationLimit": {
        "130156": ("Terminated due to pressure above termination limit", [67, 71])
    },
    "TerminatedDueToRequestFromOperator": {
        "130154": ("Terminated due to request from operator", [67, 71])
    },
    "TerminatedDueToUnspecifiedInjectorFailure": {
        "130160": ("Terminated due to unspecified injector failure", [67, 71])
    },
    "TerminatedHoldDueToTimeout": {"130269": ("Terminated hold due to timeout", [71])},
    "TerritoryRegionSeverity": {"122459": ("Territory Region Severity", [])},
    "TextureDescriptor": {"112010": ("Texture Descriptor", [])},
    "TextureMap": {"TEXTUREMAP": ("Texture Map", [32, 33])},
    "ThermalImager": {"130647": ("Thermal Imager", [9573, 9575])},
    "Thermography": {"TG": ("Thermography", [29, 30, 33])},
    "ThickeningAnalysis": {"122607": ("Thickening Analysis", [])},
    "ThicknessDeviationCategoryFromNormativeData": {
        "111931": ("Thickness deviation category from normative data", [4263])
    },
    "ThicknessDeviationFromNormativeData": {
        "111932": ("Thickness deviation from normative data", [4263])
    },
    "ThinningMyocardium": {"122114": ("Thinning Myocardium", [3704])},
    "ThoracicImagingSpecialty": {"128015": ("Thoracic Imaging Specialty", [7449])},
    "ThoracicRadiology": {"128179": ("Thoracic Radiology", [7030])},
    "ThoracicRespiratoryEffort": {"130431": ("Thoracic Respiratory Effort", [3005])},
    "ThoracicRespiratoryInductance": {
        "130427": ("Thoracic Respiratory Inductance", [3005])
    },
    "ThoracicRespiratoryPVDF": {"130429": ("Thoracic Respiratory PVDF", [3005])},
    "ThoraxDiameterSagittal": {"122221": ("Thorax diameter, sagittal", [])},
    "ThreeDimensionalMethod": {"112189": ("Three-dimensional method", [6140])},
    "ThresholdAttenuationCoefficient": {
        "112032": ("Threshold Attenuation Coefficient", [])
    },
    "ThresholdRelated": {"111839": ("Threshold related", [4252])},
    "ThresholdValue": {"122411": ("Threshold Value", [])},
    "ThroughTransducerIllumination": {
        "130813": ("Through-transducer illumination", [11001])
    },
    "Time": {"110858": ("Time", [7182])},
    "TimeAfterStartOfRadiation": {"130799": ("Time after start of Radiation", [])},
    "TimeAfterTheStartOfInjection": {
        "130194": ("Time after the start of injection", [])
    },
    "TimeCourseOfSignal": {"113066": ("Time course of signal", [218, 7180, 7469])},
    "TimeDomain": {"111920": ("Time domain", [4261])},
    "TimeIntervalSinceDetectionOfContrastBolus": {
        "122668": ("Time interval since detection of contrast bolus", [])
    },
    "TimeIntervalSinceInjectionOfContrastMedia": {
        "122665": ("Time interval since injection of contrast media", [])
    },
    "TimeOfFlight": {"114201": ("Time of flight", [8201])},
    "TimeOfFlightAngioMRSignalIntensity": {
        "110802": ("Time Of Flight Angio MR Signal Intensity", [218, 7180, 7469])
    },
    "TimeOfLeadingHalfPeakConcentration": {
        "126372": (
            "Time of Leading Half-Peak Concentration",
            [218, 4104, 4106, 4109, 7180, 7469],
        )
    },
    "TimeOfPeakConcentration": {
        "126370": ("Time of Peak Concentration", [218, 4104, 4106, 4109, 7180, 7469])
    },
    "TimePointOrder": {"126073": ("Time Point Order", [])},
    "TimePointType": {"126072": ("Time Point Type", [])},
    "TimeRelativeToRWavePeak": {"122666": ("Time relative to R-wave peak", [])},
    "TimeSinceStartOfStage": {"122710": ("Time since start of stage", [])},
    "TimeToPeak": {"113069": ("Time To Peak", [218, 4108, 4109, 7180, 7469])},
    "TimeToPeakLongitudinalLagrangianStrain": {
        "130672": ("Time to Peak Longitudinal Lagrangian Strain", [12304, 12311])
    },
    "TimeframeUncertain": {"111399": ("Timeframe uncertain", [])},
    "TimerInterlock": {"130477": ("Timer Interlock", [9561, 9568])},
    "TissueAirRatio": {"128433": ("Tissue Air Ratio", [10069])},
    "TissueFraction": {"128526": ("Tissue Fraction", [10069])},
    "TissueNotOfInterest": {"130751": ("Tissue not of interest", [7151, 7165, 7196])},
    "TissueOfOrigin": {"127401": ("Tissue of origin", [])},
    "TissueSelectionMethod": {"112713": ("Tissue selection method", [])},
    "TissueVelocity": {"110827": ("Tissue Velocity", [218, 7180, 7469])},
    "TitaniumAlloyTitaniumMolybdenum15Zirconium5Aluminium3": {
        "130736": (
            "Titanium alloy (Titanium Molybdenum-15 Zirconium-5 Aluminium-3)",
            [7300, 9579],
        )
    },
    "Title21CFRSection900SubpartB": {
        "111239": ("Title 21 CFR Section 900, Subpart B", [6045, 6326])
    },
    "Tmax": {"113084": ("Tmax", [218, 4108, 7180, 7469])},
    "TooSmall": {"112133": ("Too small", [6118])},
    "ToolFabrication": {"129014": ("Tool Fabrication", [7064])},
    "TopZLocationOfReconstructableVolume": {
        "113895": ("Top Z Location of Reconstructable Volume", [])
    },
    "TopZLocationOfScanningLength": {
        "113897": ("Top Z Location of Scanning Length", [])
    },
    "TopographicBeam": {"130109": ("Topographic Beam", [9512, 9524, 9583])},
    "TotalAcquisitionTime": {"113855": ("Total Acquisition Time", [])},
    "TotalBodyIrradiation": {"130341": ("Total Body Irradiation", [9543])},
    "TotalCholesterolScoreSheetForMen": {
        "122235": ("Total Cholesterol Score Sheet for Men", [3668])
    },
    "TotalCholesterolScoreSheetForWomen": {
        "122236": ("Total Cholesterol Score Sheet for Women", [3668])
    },
    "TotalCorneaPowerMeasurementMethod": {
        "111758": ("Total Cornea Power Measurement Method", [4242])
    },
    "TotalDurationInHousing": {"127150": ("Total duration in housing", [])},
    "TotalDurationOfLightDarkCycle": {
        "127214": ("Total duration of light-dark cycle", [])
    },
    "TotalFluoroTime": {"113730": ("Total Fluoro Time", [10050])},
    "TotalInternalReflectionFluorescence": {
        "111749": ("Total internal reflection fluorescence", [8123])
    },
    "TotalKeepVeinOpenVolumeAdministered": {
        "130165": ("Total Keep Vein Open Volume Administered", [])
    },
    "TotalLesionGlycolysis": {"126033": ("Total Lesion Glycolysis", [7466])},
    "TotalLesionProliferation": {"126035": ("Total Lesion Proliferation", [7466])},
    "TotalNumberOfIrradiationEvents": {
        "113812": ("Total Number of Irradiation Events", [])
    },
    "TotalNumberOfManuallyTriggeredInjections": {
        "130242": ("Total number of manually triggered injections", [])
    },
    "TotalNumberOfRadiographicFrames": {
        "113731": ("Total Number of Radiographic Frames", [10050])
    },
    "TotalPhaseVolumeAdministered": {"130240": ("Total Phase Volume Administered", [])},
    "TotalPlaqueVolume": {"122376": ("Total Plaque Volume", [3485])},
    "TotalPulmonaryResistance": {"122215": ("Total Pulmonary Resistance", [3619])},
    "TotalRetinalThicknessILMToBM": {
        "111929": ("Total retinal thickness (ILM to BM)", [4262])
    },
    "TotalRetinalThicknessILMToISOS": {
        "111927": ("Total retinal thickness (ILM to IS-OS)", [4262])
    },
    "TotalRetinalThicknessILMToRPE": {
        "111928": ("Total retinal thickness (ILM to RPE)", [4262])
    },
    "TotalSkinIrradiation": {"130342": ("Total Skin Irradiation", [9543])},
    "TotalStepVolumeAdministered": {"130241": ("Total Step Volume Administered", [])},
    "TotalVascularResistance": {"122216": ("Total Vascular Resistance", [3619])},
    "Trace": {"113201": ("Trace", [7263])},
    "TracerConcentration": {
        "126377": ("Tracer Concentration", [218, 4109, 7180, 7469])
    },
    "TrackingIdentifier": {"112039": ("Tracking Identifier", [])},
    "TrackingUniqueIdentifier": {"112040": ("Tracking Unique Identifier", [])},
    "TramlineShadow": {"112116": ("Tramline shadow", [6102, 6103])},
    "Transcription": {"110006": ("Transcription", [9231])},
    "TranscriptionOutput": {"110012": ("Transcription", [])},
    "TransesophagealTransducer": {"125262": ("Transesophageal Transducer", [12035])},
    "TransformationMatrix": {"130520": ("Transformation Matrix", [])},
    "TransitTimeTestInjection": {"130248": ("Transit Time Test Injection", [72])},
    "TransmissionIllumination": {"111741": ("Transmission illumination", [8123])},
    "TransmitFrequency": {"122098": ("Transmit Frequency", [3423])},
    "TransponderMarker": {"129306": ("Transponder Marker", [7111, 7112])},
    "Trastuzumab89Zr": {"126512": ("Trastuzumab ^89^Zr", [4021])},
    "TreadmillGradient": {"122703": ("Treadmill gradient", [])},
    "TreadmillSpeed": {"122702": ("Treadmill speed", [])},
    "TreatedVolume": {"130059": ("Treated Volume", [9534])},
    "TreatmentDeliveryType": {"121740": ("Treatment Delivery Type", [9250])},
    "TreatmentExceedsPatientTolerance": {
        "130665": ("Treatment exceeds patient tolerance", [9576])
    },
    "TreatmentMachineIsocenter": {
        "130359": ("Treatment Machine Isocenter", [9544, 9554])
    },
    "TreatmentPlanningNote": {"130024": ("Treatment Planning Note", [])},
    "TreatmentTerminated": {"130453": ("Treatment Terminated", [9563])},
    "TreatmentUnnecessary": {"130664": ("Treatment unnecessary", [9576])},
    "TreeInBudSign": {"112127": ("Tree-in-bud sign", [6102, 6104])},
    "Trend": {"122151": ("Trend", [])},
    "TrigeminalNodeSet": {"130364": ("Trigeminal Node Set", [9556])},
    "TrueLumen": {"122360": ("True Lumen", [3494])},
    "TruncatedGaussianDiffusionModel": {
        "113255": ("Truncated Gaussian diffusion model", [7273])
    },
    "TrustRegion": {"113266": ("Trust-Region", [7274])},
    "TubularShadow": {"112117": ("Tubular shadow", [6102, 6103])},
    "TumorBoardInputUsed": {"128221": ("Tumor Board Input Used", [7010])},
    "TumorGraft": {"127460": ("Tumor Graft", [637])},
    "TumorRegistryInputUsed": {"128222": ("Tumor Registry Input Used", [7010])},
    "TurbulenceResistance": {"122550": ("Turbulence Resistance", [])},
    "TurntableScanMethod": {"114209": ("Turntable Scan Method", [8202])},
    "TwoChamber": {"122565": ("Two Chamber", [3453])},
    "TwoCompartmentExchange2CXModel": {
        "126347": ("Two Compartment Exchange (2CX) Model", [4101, 4106])
    },
    "TwoDimensionalMethod": {"112188": ("Two-dimensional method", [6140])},
    "TypeOfContent": {"112009": ("Type of Content", [])},
    "TypeOfMyocardialInfarction": {"122170": ("Type of Myocardial Infarction", [])},
    "UDI": {"128739": ("UDI", [])},
    "URI": {"110037": ("URI", [405])},
    "US3DCAMModel": {"129018": ("US 3D CAM model", [7061])},
    "USBDiskEmulation": {"110030": ("USB Disk Emulation", [405])},
    "Ublituximab89Zr": {"126739": ("Ublituximab ^89^Zr", [4021])},
    "Ultrasound": {"US": ("Ultrasound", [29, 30, 33])},
    "UltrasoundAttenuation": {"110833": ("Ultrasound Attenuation", [218, 7180, 7469])},
    "UltrasoundBoneDensitometry": {
        "BDUS": ("Ultrasound Bone Densitometry", [29, 30, 33])
    },
    "UltrasoundContact": {"111750": ("Ultrasound Contact", [4230])},
    "UltrasoundImagingSubjectMatter": {
        "128736": ("Ultrasound imaging subject matter", [7017])
    },
    "UltrasoundImmersion": {"111751": ("Ultrasound Immersion", [4230])},
    "Ultrasound_Retired": {"111137": ("Ultrasound", [])},
    "UnavailabilityOfABeamModifier": {
        "130466": ("Unavailability of a Beam Modifier", [9565])
    },
    "Unavailable": {"UNAVAILABLE": ("Unavailable", [50])},
    "UnclassifiedCombination": {"130081": ("Unclassified Combination", [9508])},
    "UnclassifiedVolume": {"130048": ("Unclassified Volume", [9508])},
    "UncompressedPredecessor": {"121320": ("Uncompressed predecessor", [7202])},
    "UnderExposed": {"111211": ("Under exposed", [6041, 6135, 7011])},
    "Underflow": {"114004": ("Underflow", [42, 43])},
    "UndeterminedSex": {"121103": ("Undetermined sex", [7455])},
    "UndilutedDoseAdministered": {"122092": ("Undiluted dose administered", [3410])},
    "Unheated": {"127255": ("Unheated", [635])},
    "UniformSpeedOfSoundCorrection": {
        "130818": ("Uniform Speed of Sound Correction", [11004])
    },
    "Uninvolved": {"111470": ("Uninvolved", [])},
    "UniqueDeviceIdentifiers": {"121000": ("Unique Device Identifiers", [])},
    "UnitSerialIdentifier": {"121148": ("Unit Serial Identifier", [3426])},
    "UnitedAllBreedRegistry": {"109217": ("United All Breed Registry", [7481])},
    "UnitedKennelClub": {"109218": ("United Kennel Club", [7481])},
    "UniversalKennelClubInternational": {
        "109219": ("Universal Kennel Club International", [7481])
    },
    "UnknownFailure": {"111221": ("Unknown failure", [6041, 6135, 7011])},
    "UnknownSex": {"U": ("Unknown sex", [7455])},
    "UnmountedDetector": {"113953": ("Unmounted Detector", [10031])},
    "UnsharpMask": {"113089": ("Unsharp mask", [7203])},
    "Unspecified": {"111176": ("Unspecified", [6040])},
    "UnspecifiedGynecologicalHormone": {
        "111542": ("Unspecified gynecological hormone", [6080])
    },
    "UnspecifiedMethodOfCalculation": {
        "112187": ("Unspecified method of calculation", [6140])
    },
    "UntrackableMeasurement": {"125304": ("Untrackable Measurement", [])},
    "UnusableQualityRendersImageUnusable": {
        "111235": ("Unusable - Quality renders image unusable", [3114, 6044])
    },
    "Unused": {"127177": ("Unused", [604])},
    "UnwantedSubtractionImages": {
        "130582": ("Unwanted subtraction images", [6315, 6317])
    },
    "UprightStandMount": {"113954": ("Upright Stand Mount", [10031])},
    "UsableDoesNotMeetTheQualityControlStandard": {
        "111236": ("Usable - Does not meet the quality control standard", [6044])
    },
    "UsableMeetsTheQualityControlStandard": {
        "111237": ("Usable - Meets the quality control standard", [6044])
    },
    "UseOfRestrictedFunction": {"110132": ("Use of Restricted Function", [401, 403])},
    "UsedSubstanceType": {"111546": ("Used Substance Type", [])},
    "UserAuthentication": {"110114": ("User Authentication", [400])},
    "UserChosenValue": {"121410": ("User chosen value", [224, 4241, 4252, 12301])},
    "UserDefinedAIFROI": {"126362": ("User-defined AIF ROI", [4103, 4106])},
    "UserSecurityAttributesChanged": {
        "110137": ("User Security Attributes Changed", [401, 403])
    },
    "UserSelectedFiducial": {"112356": ("User Selected Fiducial", [])},
    "UserSelectedMethod": {"122575": ("User selected method", [3456])},
    "UterineMalformations": {"111565": ("Uterine malformations", [6087, 6088])},
    "UtricleCyst": {"130569": ("Utricle cyst", [6313])},
    "VMAT": {"130107": ("VMAT", [9511, 9524, 9583])},
    "VMaxMyocardial": {"109073": ("V max myocardial", [3337])},
    "VO2Equals152BSA": {"122250": ("VO2 = 152 * BSA", [3664])},
    "VO2Equals175BSA": {"122251": ("VO2 = 175 * BSA", [3664])},
    "VO2Equals176BSA": {"122252": ("VO2 = 176 * BSA", [3664])},
    "VO2EqualsVestpd10FIO2FE02": {
        "122249": ("VO2 = VeSTPD * 10 * (FIO2 - FE02)", [3664])
    },
    "VVulnerablePlaque": {"130735": ("V - Vulnerable plaque", [3022])},
    "VWave": {"109052": ("V wave", [3339])},
    "VWaveOfNextBeat": {"109053": ("V wave of next beat", [3339])},
    "VWavePeakPressure": {"109034": ("V wave peak pressure", [3337, 3611])},
    "VWavePressureAverage": {"109035": ("V wave pressure, average", [3337])},
    "VacuumAssisted": {"111490": ("Vacuum assisted", [6058, 6060])},
    "VacuumMold": {
        "130118": ("Vacuum Mold", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "VaginalCylinder": {
        "130121": ("Vaginal Cylinder", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "ValueBasedImage": {"129322": ("Value-based Image", [])},
    "ValueIndeterminate": {"114011": ("Value indeterminate", [42, 44])},
    "ValueOutOfRange": {"114009": ("Value out of range", [42, 43])},
    "ValueTiming": {"130518": ("Value Timing", [])},
    "ValueUnknown": {"114010": ("Value unknown", [42, 44])},
    "ValveClose": {"109036": ("Valve close", [3337])},
    "ValveOpen": {"109037": ("Valve open", [3337])},
    "ValvularIntervention": {"122053": ("Valvular Intervention", [3405, 3406])},
    "VariableCircularCollimator": {
        "130332": ("Variable Circular Collimator", [9540, 9541])
    },
    "VascularAndInterventionalImagingSubjectMatter": {
        "128734": ("Vascular and interventional imaging subject matter", [7017])
    },
    "VascularFunctionalAnalysis": {"122606": ("Vascular Functional Analysis", [])},
    "VascularImagingSpecialty": {"128017": ("Vascular Imaging Specialty", [7449])},
    "VascularIntervention": {"122056": ("Vascular Intervention", [3405, 3406])},
    "VascularMorphologicalAnalysis": {
        "122605": ("Vascular Morphological Analysis", [])
    },
    "VascularUltrasoundProcedureReport": {
        "125100": ("Vascular Ultrasound Procedure Report", [])
    },
    "VascularVolumeMeasurementLength": {
        "122336": ("Vascular Volume measurement length", [])
    },
    "Vascularity": {"111372": ("Vascularity", [])},
    "VascularityNotAssessed": {"111374": ("Vascularity not assessed", [6157])},
    "VascularityNotPresent": {"111373": ("Vascularity not present", [6157])},
    "VascularityPresentImmediatelyAdjacentToLesion": {
        "111376": ("Vascularity present immediately adjacent to lesion", [6157])
    },
    "VascularityPresentInLesion": {"111375": ("Vascularity present in lesion", [6157])},
    "Vasoconstriction": {"112077": ("Vasoconstriction", [6107])},
    "Vasodilation": {"112078": ("Vasodilation", [6107])},
    "Ve": {"126314": ("ve", [218, 4107, 7180, 7469])},
    "Vector": {"121225": ("Vector", [7551, 7553])},
    "VeilingGlareEvaluation": {"109707": ("Veiling glare evaluation", [8300])},
    "VelocityEncoded": {"113070": ("Velocity encoded", [218, 7180, 7469])},
    "VelocityEncodingMaximumValue": {"122643": ("Velocity Encoding Maximum Value", [])},
    "VelocityEncodingMinimumValue": {"122642": ("Velocity Encoding Minimum Value", [])},
    "VenaContractaWidth": {"125334": ("Vena Contracta Width", [12304])},
    "VentralDiencephalon": {"110700": ("Ventral Diencephalon", [7140])},
    "VentricularAnalysis": {"122601": ("Ventricular Analysis", [])},
    "VentricularEndDiastolicPressure": {
        "122191": ("Ventricular End Diastolic pressure", [3611])
    },
    "VentricularPressureMeasurements": {
        "122122": ("Ventricular pressure measurements", [])
    },
    "VentricularSystolicBloodPressure": {
        "122194": ("Ventricular Systolic blood pressure", [])
    },
    "Verbal": {"121004": ("Verbal", [])},
    "Verifying": {"121098": ("Verifying", [7453])},
    "VertebralIntervertebralNotch": {
        "112097": ("Vertebral Intervertebral Notch", [6115, 8134])
    },
    "VerticalPixelSpacing": {"111066": ("Vertical Pixel Spacing", [])},
    "VerticalPrismBase": {"111678": ("Vertical Prism Base", [])},
    "VerticalPrismPower": {"111677": ("Vertical Prism Power", [])},
    "VerySmall": {"112132": ("Very small", [6118])},
    "VeryStrongFamilyHistoryOfBreastCancer": {
        "111561": ("Very strong family history of breast cancer", [6081, 6087])
    },
    "VesselBranch": {"125101": ("Vessel Branch", [])},
    "VesselLumenCrossSectionalArea": {
        "122480": ("Vessel Lumen Cross-Sectional Area", [])
    },
    "VesselMorphology": {"122134": ("Vessel Morphology", [])},
    "VideoRecordingOff": {"130888": ("Video recording off", [3039])},
    "VideoRecordingOn": {"130887": ("Video recording on", [3039])},
    "VideoTapeDigitized": {"110022": ("Video Tape Digitized", [7008])},
    "VideoTapeDigitizerEquipment": {"VIDD": ("Video Tape Digitizer Equipment", [7005])},
    "Videofluorography": {"VF": ("Videofluorography", [])},
    "ViewAndLateralityMarkerDoesNotHaveApprovedCodes": {
        "111179": ("View and Laterality Marker does not have approved codes", [6041])
    },
    "ViewAndLateralityMarkerDoesNotHaveBothViewAndLaterality": {
        "111178": (
            "View and Laterality Marker does not have both view and laterality",
            [6041],
        )
    },
    "ViewAndLateralityMarkerIsIncorrect": {
        "111183": ("View and Laterality Marker is incorrect", [6041])
    },
    "ViewAndLateralityMarkerIsMissing": {
        "111177": ("View and Laterality Marker is missing", [6041])
    },
    "ViewAndLateralityMarkerIsNotNearTheAxilla": {
        "111180": ("View and Laterality Marker is not near the axilla", [6041])
    },
    "ViewAndLateralityMarkerIsOffImage": {
        "111184": ("View and Laterality Marker is off image", [6041])
    },
    "ViewAndLateralityMarkerIsPartiallyObscured": {
        "111182": ("View and Laterality Marker is partially obscured", [6041])
    },
    "ViewAndLateralityMarkerOverlapsBreastTissue": {
        "111181": ("View and Laterality Marker overlaps breast tissue", [6041])
    },
    "VirginalHyperplasia": {"111298": ("Virginal hyperplasia", [6030, 6031])},
    "VirtualGrid": {"111640": ("Virtual grid", [10017])},
    "ViscosityAt37C": {"130186": ("Viscosity at 37C", [])},
    "VisibleReflectorMarker": {"129310": ("Visible Reflector Marker", [7111, 7112])},
    "VisualAcuity": {"VA": ("Visual Acuity", [29, 30, 33])},
    "VisualAlignment": {"125025": ("Visual Alignment", [7100])},
    "VisualExplanation": {"130401": ("Visual explanation", [])},
    "VisualField07TestPattern": {"111812": ("Visual Field 07 Test Pattern", [4250])},
    "VisualField10To2TestPattern": {
        "111801": ("Visual Field 10-2 Test Pattern", [4250])
    },
    "VisualField1LTTestStrategy": {
        "111828": ("Visual Field 1-LT Test Strategy", [4251])
    },
    "VisualField24To2TestPattern": {
        "111800": ("Visual Field 24-2 Test Pattern", [4250])
    },
    "VisualField2LTDynamicTestStrategy": {
        "111833": ("Visual Field 2LT-Dynamic Test Strategy", [4251])
    },
    "VisualField2LTNormalTestStrategy": {
        "111834": ("Visual Field 2LT-Normal Test Strategy", [4251])
    },
    "VisualField2LTTestStrategy": {
        "111829": ("Visual Field 2-LT Test Strategy", [4251])
    },
    "VisualField30To2TestPattern": {
        "111802": ("Visual Field 30-2 Test Pattern", [4250])
    },
    "VisualField60To4TestPattern": {
        "111803": ("Visual Field 60-4 Test Pattern", [4250])
    },
    "VisualFieldCLASSStrategy": {"111837": ("Visual Field CLASS Strategy", [4251])},
    "VisualFieldCLIPTestStrategy": {
        "111836": ("Visual Field CLIP Test Strategy", [4251])
    },
    "VisualFieldCentral40PointTestPattern": {
        "111805": ("Visual Field Central 40 Point Test Pattern", [4250])
    },
    "VisualFieldCentral76PointTestPattern": {
        "111806": ("Visual Field Central 76 Point Test Pattern", [4250])
    },
    "VisualFieldCentralTestPattern": {
        "111814": ("Visual Field Central Test Pattern", [4250])
    },
    "VisualFieldDynamicTestStrategy": {
        "111826": ("Visual Field Dynamic Test Strategy", [4251])
    },
    "VisualFieldFastThresholdTestStrategy": {
        "111835": ("Visual Field Fast Threshold Test Strategy", [4251])
    },
    "VisualFieldFastpacTestStrategy": {
        "111819": ("Visual Field FastPac Test Strategy", [4251])
    },
    "VisualFieldFullField120PointTestPattern": {
        "111809": ("Visual Field Full Field 120 Point Test Pattern", [4250])
    },
    "VisualFieldFullField81PointTestPattern": {
        "111808": ("Visual Field Full Field 81 Point Test Pattern", [4250])
    },
    "VisualFieldFullFromPriorTestStrategy": {
        "111820": ("Visual Field Full From Prior Test Strategy", [4251])
    },
    "VisualFieldFullThresholdTestStrategy": {
        "111818": ("Visual Field Full Threshold Test Strategy", [4251])
    },
    "VisualFieldGATETestStrategy": {
        "111831": ("Visual Field GATE Test Strategy", [4251])
    },
    "VisualFieldGTestPattern": {"111810": ("Visual Field G Test Pattern", [4250])},
    "VisualFieldGateiTestStrategy": {
        "111832": ("Visual Field GATEi Test Strategy", [4251])
    },
    "VisualFieldIndex": {"111852": ("Visual Field Index", [4257])},
    "VisualFieldLVCTestPattern": {"111813": ("Visual Field LVC Test Pattern", [4250])},
    "VisualFieldLVSTestStrategy": {
        "111830": ("Visual Field LVS Test Strategy", [4251])
    },
    "VisualFieldLossDueToDiffuseDefect": {
        "111853": ("Visual Field Loss Due to Diffuse Defect", [4257])
    },
    "VisualFieldLossDueToLocalDefect": {
        "111854": ("Visual Field Loss Due to Local Defect", [4257])
    },
    "VisualFieldMTestPattern": {"111811": ("Visual Field M Test Pattern", [4250])},
    "VisualFieldMaculaTestPattern": {
        "111804": ("Visual Field Macula Test Pattern", [4250])
    },
    "VisualFieldNormalTestStrategy": {
        "111827": ("Visual Field Normal Test Strategy", [4251])
    },
    "VisualFieldOptimaTestStrategy": {
        "111821": ("Visual Field Optima Test Strategy", [4251])
    },
    "VisualFieldPeripheral60PointTestPattern": {
        "111807": ("Visual Field Peripheral 60 Point Test Pattern", [4250])
    },
    "VisualFieldQuantifyDefectsTestStrategy": {
        "111824": ("Visual Field Quantify-Defects Test Strategy", [4251])
    },
    "VisualFieldSITAFastTestStrategy": {
        "111817": ("Visual Field SITA-Fast Test Strategy", [4251])
    },
    "VisualFieldSITASWAPTestStrategy": {
        "111816": ("Visual Field SITA-SWAP Test Strategy", [4251])
    },
    "VisualFieldSITAStandardTestStrategy": {
        "111815": ("Visual Field SITA-Standard Test Strategy", [4251])
    },
    "VisualFieldTOPTestStrategy": {
        "111825": ("Visual Field TOP Test Strategy", [4251])
    },
    "VisualFieldThreeZoneTestStrategy": {
        "111823": ("Visual Field Three-Zone Test Strategy", [4251])
    },
    "VisualFieldTwoZoneTestStrategy": {
        "111822": ("Visual Field Two-Zone Test Strategy", [4251])
    },
    "VisualFixationQualityDuringAcquisition": {
        "111696": ("Visual Fixation Quality During Acquisition", [])
    },
    "VisualFixationQualityProblem": {"111697": ("Visual Fixation Quality Problem", [])},
    "VisualMarkingOfSpecimen": {"111723": ("Visual Marking of Specimen", [])},
    "VitreousStructuralReflectanceMap": {
        "128262": ("Vitreous structural reflectance map", [4271])
    },
    "VitreousVasculatureFlow": {"128261": ("Vitreous vasculature flow", [4271])},
    "Vo2femaleEqualsBSA138Point117Point04LogeAge0Point378Hrf": {
        "122248": ("VO2female = BSA (138.1 - 17.04 * loge(age) + 0.378*HRf)", [3664])
    },
    "Vo2maleEqualsBSA138Point111Point49LogeAge0Point378Hrf": {
        "122247": ("VO2male = BSA (138.1 - 11.49 * loge(age) + 0.378*HRf)", [3664])
    },
    "Voice": {"109110": ("Voice", [3000])},
    "VoltageMeasurement": {"109013": ("Voltage measurement", [3240])},
    "VoltageMeasurementByBasketCatheter": {
        "109011": ("Voltage measurement by basket catheter", [3240])
    },
    "VoltageMeasurementByMappingCatheter": {
        "109012": ("Voltage measurement by mapping catheter", [3240])
    },
    "VolumeAdministered": {"122091": ("Volume administered", [3410])},
    "VolumeBasedMethod": {"122652": ("Volume Based Method", [3807])},
    "VolumeCorrespondingToSpatiallyRelatedAcquisitionFrames": {
        "121347": (
            "Volume corresponding to spatially-related acquisition frames",
            [7201],
        )
    },
    "VolumeEditing": {"113077": ("Volume editing", [7203])},
    "VolumeEstimatedFromSingle2DRegion": {
        "121216": ("Volume estimated from single 2D region", [218, 3471, 7469, 7472])
    },
    "VolumeEstimatedFromThreeOrMoreNonCoplanar2DRegions": {
        "121217": (
            "Volume estimated from three or more non-coplanar 2D regions",
            [218, 7469, 7472],
        )
    },
    "VolumeEstimatedFromTwoNonCoplanar2DRegions": {
        "121218": (
            "Volume estimated from two non-coplanar 2D regions",
            [218, 3471, 7469, 7472],
        )
    },
    "VolumeMethod": {"122429": ("Volume Method", [])},
    "VolumeOccupancyImage": {"129328": ("Volume Occupancy Image", [])},
    "VolumeOfBoundingThreeDimensionalRegion": {
        "121219": ("Volume of bounding three dimensional region", [218, 7469, 7472])
    },
    "VolumeOfBreast": {"112193": ("Volume of breast", [6142])},
    "VolumeOfCircumscribedSphere": {
        "121220": ("Volume of circumscribed sphere", [218, 7469, 7472])
    },
    "VolumeOfEllipsoid": {"121221": ("Volume of ellipsoid", [218, 7469, 7472])},
    "VolumeOfParenchymalTissue": {"112192": ("Volume of parenchymal tissue", [6142])},
    "VolumeOfSphere": {"121222": ("Volume of sphere", [218, 7469, 7472])},
    "VolumeOfUse": {"111580": ("Volume of use", [6092])},
    "VolumeOfVascularCalcification": {
        "112197": ("Volume of Vascular Calcification", [6142])
    },
    "VolumeRatio": {"113288": ("Volume Ratio", [218, 7180, 7271, 7469])},
    "VolumeRendering": {"113074": ("Volume rendering", [7203])},
    "VolumeScanPattern": {"125242": ("Volume scan pattern", [12032])},
    "VolumeScoringMethod": {"112056": ("Volume scoring method", [6140])},
    "VolumeSurface": {"121231": ("Volume Surface", [])},
    "VolumeUnadministeredOrDiscarded": {
        "122096": ("Volume unadministered or discarded", [3410])
    },
    "VolumetricDiffusionDxxComponent": {
        "110810": (
            "Volumetric Diffusion Dxx Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VolumetricDiffusionDxyComponent": {
        "110811": (
            "Volumetric Diffusion Dxy Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VolumetricDiffusionDxzComponent": {
        "110812": (
            "Volumetric Diffusion Dxz Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VolumetricDiffusionDyyComponent": {
        "110813": (
            "Volumetric Diffusion Dyy Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VolumetricDiffusionDyzComponent": {
        "110814": (
            "Volumetric Diffusion Dyz Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VolumetricDiffusionDzzComponent": {
        "110815": (
            "Volumetric Diffusion Dzz Component",
            [218, 7180, 7186, 7263, 7270, 7469],
        )
    },
    "VoxelizedRadiationTransportModel": {
        "128422": ("Voxelized Radiation Transport Model", [10065])
    },
    "VoxelwiseSelectionOfBValue": {
        "113285": ("Voxelwise selection of b-value", [7275])
    },
    "Vp": {"126331": ("vp", [218, 4107, 7180, 7469])},
    "VulnerablePlaque": {"122389": ("Vulnerable Plaque", [3491])},
    "WHO": {"112029": ("WHO", [6147])},
    "WSI20XRGB": {"112704": ("WSI 20X RGB", [8131])},
    "WSI40XRGB": {"112705": ("WSI 40X RGB", [8131])},
    "WallMass": {"122447": ("Wall Mass", [12255, 12257, 12258, 12259, 12265, 12271])},
    "WallStress": {"122448": ("Wall Stress", [])},
    "WallThickness": {"122445": ("Wall Thickness", [])},
    "WallThicknessRatioEndSystolicToEndDiastolic": {
        "122624": ("Wall Thickness Ratio end-systolic to end-diastolic", [])
    },
    "WallToSeptum": {"110877": ("Wall To Septum", [7185])},
    "WallVolume": {"122446": ("Wall Volume", [])},
    "WashoutDelayedPhaseType3Curve": {
        "130599": ("Washout delayed phase: Type 3 curve", [6335, 6345, 6346])
    },
    "WaterEquivalentDiameter": {"113980": ("Water Equivalent Diameter", [])},
    "WaterEquivalentDiameterAtLongitudinalPositionZ": {
        "113995": ("Water Equivalent Diameter At Longitudinal Position Z", [])
    },
    "WaterEquivalentDiameterFromLimitedFOVImages": {
        "113992": ("Water Equivalent Diameter From Limited FOV Images", [])
    },
    "WaterEquivalentDiameterFromLocalizer": {
        "113984": ("Water Equivalent Diameter From Localizer", [10024, 10069])
    },
    "WaterEquivalentDiameterFromRawData": {
        "113983": ("Water Equivalent Diameter From Raw Data", [10024, 10069])
    },
    "WaterEquivalentDiameterIntegratedAcrossScanRange": {
        "113982": (
            "Water Equivalent Diameter Integrated Across Scan Range",
            [10024, 10069],
        )
    },
    "WaterEquivalentDiameterRepresentativeValue": {
        "113981": ("Water Equivalent Diameter Representative Value", [10024, 10069])
    },
    "WaterFatInPhase": {"129101": ("Water/fat in phase", [218, 7180, 7469])},
    "WaterFatOutOfPhase": {"129102": ("Water/fat out of phase", [218, 7180, 7469])},
    "WaterFraction": {"129103": ("Water fraction", [218, 7180, 7469])},
    "WaveformAcquired": {"121143": ("Waveform Acquired", [])},
    "WeakFamilyHistoryOfBreastCancer": {
        "111559": ("Weak family history of breast cancer", [6081, 6087])
    },
    "WedgeFilter": {"113651": ("Wedge filter", [10007])},
    "Wedging": {"112321": ("Wedging", [7310])},
    "WeightExceedsEquipmentLimit": {"122764": ("Weight exceeds equipment limit", [])},
    "WellDemarcated": {"112139": ("Well demarcated", [6120])},
    "WhiteMatterT1Hypointensity": {"110701": ("White Matter T1 Hypointensity", [7140])},
    "WhiteMatterT2Hyperintensity": {
        "110702": ("White Matter T2 Hyperintensity", [7140])
    },
    "WhitePattern": {"109941": ("White Pattern", [8301])},
    "WholeBodyPod": {
        "130119": ("Whole Body Pod", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "WholeEyeStructuralReflectanceMap": {
        "128278": ("Whole eye structural reflectance map", [4271])
    },
    "WholeEyeVasculatureFlow": {"128277": ("Whole eye vasculature flow", [4271])},
    "WholeSlideImaging": {"112703": ("Whole Slide Imaging", [8131])},
    "WidthDescriptor": {"112026": ("Width Descriptor", [])},
    "WinstonLutzPointer": {"130353": ("Winston-Lutz Pointer", [9548])},
    "WireMarker": {"129305": ("Wire Marker", [7111, 7112])},
    "WorkingCanineAssociationOfCanada": {
        "109220": ("Working Canine Association of Canada", [7481])
    },
    "WorkitemAssignmentRejectedByAssignedResource": {
        "110530": ("Workitem assignment rejected by assigned resource", [9300, 9303])
    },
    "WorkitemExpired": {"110533": ("Workitem expired", [9300])},
    "Workstation": {"WSD": ("Workstation", [30])},
    "WorldKennelClub": {"109221": ("World Kennel Club", [7481])},
    "WorldWideKennelClub": {"109222": ("World Wide Kennel Club", [7481])},
    "XConcept": {"122698": ("X-Concept", [])},
    "XDescentPressure": {"122208": ("x-descent pressure", [3611])},
    "XOrientation": {"130334": ("X Orientation", [9547])},
    "XRayAngiography": {"XA": ("X-Ray Angiography", [29, 30, 33])},
    "XRayAttenuation": {"110850": ("X-Ray Attenuation", [218, 7180, 7469])},
    "XRayAttenuationCoefficient": {"110851": ("X-Ray Attenuation Coefficient", [])},
    "XRayAttenuatorModelData": {"128470": ("X-Ray Attenuator Model Data", [])},
    "XRayBeamAttenuator": {"128457": ("X-Ray Beam Attenuator", [])},
    "XRayBeamAttenuatorModel": {"128472": ("X-Ray Beam Attenuator Model", [])},
    "XRayBeamAttenuatorModelReference": {
        "128474": ("X-Ray Beam Attenuator Model Reference", [])
    },
    "XRayBeamAttenuatorModelRegistration": {
        "128475": ("X-Ray Beam Attenuator Model Registration", [])
    },
    "XRayDetectorDataAvailable": {"113945": ("X-Ray Detector Data Available", [])},
    "XRayFilterAluminumEquivalent": {
        "113821": ("X-Ray Filter Aluminum Equivalent", [])
    },
    "XRayFilterMaterial": {"113757": ("X-Ray Filter Material", [])},
    "XRayFilterThickness": {"130509": ("X-Ray Filter Thickness", [])},
    "XRayFilterThicknessMaximum": {"113773": ("X-Ray Filter Thickness Maximum", [])},
    "XRayFilterThicknessMinimum": {"113758": ("X-Ray Filter Thickness Minimum", [])},
    "XRayFilterType": {"113772": ("X-Ray Filter Type", [])},
    "XRayFilters": {"113771": ("X-Ray Filters", [10066])},
    "XRayFluoroscopyOnSignal": {"109004": ("X-Ray Fluoroscopy On Signal", [3090])},
    "XRayGrid": {"111635": ("X-Ray Grid", [])},
    "XRayMechanicalDataAvailable": {"113944": ("X-Ray Mechanical Data Available", [])},
    "XRayModulationType": {"113842": ("X-Ray Modulation Type", [])},
    "XRayOnTrigger": {"109005": ("X-Ray On Trigger", [3090])},
    "XRayRadiationDoseReport": {"113701": ("X-Ray Radiation Dose Report", [])},
    "XRayReadingDevice": {"113942": ("X-Ray Reading Device", [7445])},
    "XRaySourceDataAvailable": {"113943": ("X-Ray Source Data Available", [])},
    "XRaySourceReferenceCoordinateSystem": {
        "130519": ("X-Ray Source Reference Coordinate System", [])
    },
    "XRayTubeCurrent": {"113734": ("X-Ray Tube Current", [])},
    "Xmab557489Zr": {"126734": ("XmAb5574 ^89^Zr", [4021])},
    "YBRFULLCBComponent": {
        "110838": ("YBR FULL CB Component", [218, 7180, 7186, 7469])
    },
    "YBRFULLCRComponent": {
        "110839": ("YBR FULL CR Component", [218, 7180, 7186, 7469])
    },
    "YBRFULLYComponent": {"110837": ("YBR FULL Y Component", [218, 7180, 7186, 7469])},
    "YBRICTCBComponent": {"110844": ("YBR ICT CB Component", [218, 7180, 7186, 7469])},
    "YBRICTCRComponent": {"110845": ("YBR ICT CR Component", [218, 7180, 7186, 7469])},
    "YBRICTYComponent": {"110843": ("YBR ICT Y Component", [218, 7180, 7186, 7469])},
    "YBRPARTIALCBComponent": {
        "110841": ("YBR PARTIAL CB Component", [218, 7180, 7186, 7469])
    },
    "YBRPARTIALCRComponent": {
        "110842": ("YBR PARTIAL CR Component", [218, 7180, 7186, 7469])
    },
    "YBRPARTIALYComponent": {
        "110840": ("YBR PARTIAL Y Component", [218, 7180, 7186, 7469])
    },
    "YBRRCTCBComponent": {"110847": ("YBR RCT CB Component", [218, 7180, 7186, 7469])},
    "YBRRCTCRComponent": {"110848": ("YBR RCT CR Component", [218, 7180, 7186, 7469])},
    "YBRRCTYComponent": {"110846": ("YBR RCT Y Component", [218, 7180, 7186, 7469])},
    "YConcept": {"122699": ("Y-Concept", [])},
    "YDescentPressure": {"122209": ("y-descent pressure", [3611])},
    "YOrientation": {"130335": ("Y Orientation", [9547])},
    "YellowGreenFilter": {"111604": ("Yellow-green filter", [])},
    "ZPointPressure": {"122210": ("z-point pressure", [3611])},
    "ZScore": {"113071": ("Z-score", [218, 7180, 7469])},
    "ZScoreOfMeasurement": {"121416": ("Z-Score of measurement", [221, 227, 7465])},
    "ZValueOfLocationOfWaterEquivalentDiameterEstimation": {
        "113986": ("Z value of location of Water Equivalent Diameter estimation", [])
    },
    "ZobelFilter": {"130769": ("Zobel filter", [3042])},
    "ZonePercentage": {"128833": ("Zone Percentage", [])},
    "ZoneSizeEntropy": {"128836": ("Zone Size Entropy", [])},
    "ZoneSizeNonuniformity": {"128831": ("Zone Size Nonuniformity", [])},
    "ZoneSizeNonuniformityNormalized": {
        "128832": ("Zone Size Nonuniformity Normalized", [])
    },
    "ZoneSizeVariance": {"128835": ("Zone Size Variance", [])},
    "_0DocumentedAbsenceOfCAD": {"130723": ("0 - Documented absence of CAD", [3020])},
    "_100cmFromXRaySource": {"113965": ("100cm from X-Ray Source", [10025])},
    "_10X": {"112716": ("10X", [8132])},
    "_10YearCHDRisk": {"122230": ("10 Year CHD Risk", [3667])},
    "_10cmDosimetryPhantom": {"130541": ("10 cm Dosimetry Phantom", [4051, 4052])},
    "_12CarbonNucleus": {"130143": ("^12^Carbon nucleus", [9526])},
    "_152Terbium": {"126606": ("^152^Terbium", [4020])},
    "_15LeadECG": {"122062": ("15-Lead ECG", [])},
    "_15cmFromIsocenterTowardSource": {
        "113860": ("15cm from Isocenter toward Source", [10025])
    },
    "_15cmFromTableCenterline": {"113864": ("15cm from Table Centerline", [10025])},
    "_16OxygenNucleus": {"130144": ("^16^Oxygen nucleus", [9526])},
    "_191mIridium": {"126604": ("^191m^Iridium", [18])},
    "_1MinimalNonObstructiveCAD": {
        "130724": ("1 - Minimal non-obstructive CAD", [3020])
    },
    "_1cmAboveTabletop": {"113862": ("1cm above Tabletop", [10025])},
    "_1stWeek": {"111392": ("1st week", [6163])},
    "_20X": {"112717": ("20X", [8132])},
    "_28H189Zr": {"126752": ("28H1 ^89^Zr", [4021])},
    "_2DShearWaveElastography": {
        "130609": ("2D Shear Wave Elastography", [6058, 12224])
    },
    "_2DSpeckleTracking": {"130679": ("2D Speckle Tracking", [6058, 12224])},
    "_2FAF18": {"126713": ("2FA F^18^", [4021])},
    "_2MildNonObstructiveCAD": {"130725": ("2 - Mild non-obstructive CAD", [3020])},
    "_2SigmaDeviationOfPopulation": {
        "121417": ("2 Sigma deviation of population", [221, 226, 7465])
    },
    "_2ndWeek": {"111393": ("2nd week", [6163])},
    "_30cmAboveTabletop": {"113863": ("30cm above Tabletop", [10025])},
    "_30cmInFrontOfImageInputSurface": {
        "113861": ("30cm in Front of Image Input Surface", [10025])
    },
    "_35PercentOfThermalCO": {"109014": ("35% of thermal CO", [])},
    "_35PercentOfThermalDyeDilutionCO": {
        "109014": ("35% of thermal/dye dilution CO", [3337])
    },
    "_3DDoseMap": {"128487": ("3D Dose Map", [10063])},
    "_3DGel": {"128701": ("3D Gel", [7026])},
    "_3DManufacturingModelingSystem": {
        "M3D": ("3D Manufacturing Modeling System", [30])
    },
    "_3DMode": {"125231": ("3D mode", [])},
    "_3DShearWaveElastography": {
        "130610": ("3D Shear Wave Elastography", [6058, 12224])
    },
    "_3DSpeckleTracking": {"130680": ("3D Speckle Tracking", [6058, 12224])},
    "_3HeliumNucleus": {"130141": ("^3^Helium nucleus", [9526])},
    "_3ModerateStenosis": {"130726": ("3 - Moderate stenosis", [3020])},
    "_3rdWeek": {"111394": ("3rd week", [6163])},
    "_40X": {"112718": ("40X", [8132])},
    "_43Scandium": {"126605": ("^43^Scandium", [4020])},
    "_44Scandium": {"126600": ("^44^Scandium", [4020])},
    "_4ASevereStenosis": {"130728": ("4A - Severe stenosis", [3021])},
    "_4BSevereStenosis": {"130729": ("4B - Severe stenosis", [3021])},
    "_4DRadiationTreatmentNote": {"130027": ("4D Radiation Treatment Note", [])},
    "_4HeliumNucleus": {"130142": ("^4^Helium nucleus", [9526])},
    "_4MonthsTo1YearAgo": {"111397": ("4 months to 1 year ago", [6164])},
    "_4Point2cmAboveBreastSupportSurface": {
        "113865": ("4.2cm above Breast Support Surface", [10025])
    },
    "_4PointSegmentFindingScale": {
        "125223": ("4 Point Segment Finding Scale", [12238])
    },
    "_4SevereStenosis": {"130727": ("4 - Severe stenosis", [3020])},
    "_51Manganese": {"126601": ("^51^Manganese", [4020])},
    "_52mManganese": {"126607": ("^52m^Manganese", [4020])},
    "_5PointSegmentFindingScale": {
        "125224": ("5 Point Segment Finding Scale", [12238])
    },
    "_5PointSegmentFindingScaleWithGradedHypokinesis": {
        "125225": ("5 Point Segment Finding Scale With Graded Hypokinesis", [12238])
    },
    "_5TotalCoronaryOcclusion": {"130730": ("5 - Total coronary occlusion", [3020])},
    "_5X": {"112715": ("5X", [8132])},
    "_70Arsenic": {"126602": ("^70^Arsenic", [4020])},
    "_70PercentOfThermalCO": {"109015": ("70% of thermal CO", [])},
    "_70PercentOfThermalDyeDilutionCO": {
        "109015": ("70% of thermal/dye dilution CO", [3337])
    },
    "_7D1289Zr": {"126751": ("7D12 ^89^Zr", [4021])},
    "_7E1189Zr": {"126750": ("7E11 ^89^Zr", [4021])},
    "_90Niobium": {"126603": ("^90^Niobium", [4020])},
}

concepts["SCT"] = {
    "ACEInhibitor": {"41549009": ("ACE inhibitor", [3760])},
    "AConstant": {"397263007": ("A-Constant", [4237])},
    "AConstantOfIntraocularLens": {
        "397263007": ("A-constant of intraocular lens (observable entity)", [])
    },
    "ASCOBCockerSpanielDogBreed": {
        "30565000": ("A.S.C.O.B. cocker spaniel dog breed", [7480])
    },
    "ATSMCu64": {"422855001": ("ATSM Cu^64^", [4021])},
    "AVFistula": {"128617001": ("AV Fistula", [3413, 3754])},
    "AVGrooveContinuationOfCircumflexArtery": {
        "75902001": (
            "AV groove continuation of Circumflex Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "AVSequentialPacing": {"371910005": ("A-V sequential pacing", [3555])},
    "Abdomen": {
        "818981001": (
            "Abdomen",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                7151,
                7154,
                7192,
                7482,
                7483,
                8134,
                9514,
                12020,
            ],
        )
    },
    "AbdomenAndOrPelvisStructure": {
        "416949008": ("Abdomen and/or pelvis structure (body structure)", [])
    },
    "AbdomenAndPelvis": {
        "818982008": (
            "Abdomen and Pelvis",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AbdominalAorta": {
        "7832008": (
            "Abdominal aorta",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4030,
                4042,
                7151,
                7154,
                7192,
                8134,
                9514,
                12103,
                12112,
                12125,
                12291,
            ],
        )
    },
    "AbdominalAortaStructure": {
        "7832008": ("Abdominal aorta structure (body structure)", [])
    },
    "AbdominalAorticAneurysmStenting": {
        "771453009": ("Abdominal aortic aneurysm stenting", [3721])
    },
    "AbdominalApproach": {"66739002": ("Abdominal approach (qualifier value)", [])},
    "AbdominalCavityStructure": {
        "52731004": ("Abdominal cavity structure (body structure)", [])
    },
    "AbdominalLymphNode": {"818991007": ("abdominal lymph node", [7600, 8134])},
    "AbdominalLymphNodeStructure": {
        "8568009": ("Abdominal lymph node structure (body structure)", [])
    },
    "AbdominalStructure": {"113345001": ("Abdominal structure (body structure)", [])},
    "AbdominalWallMuscle": {
        "1142228004": ("Abdominal wall muscle", [7151, 7154, 7192, 8134, 9514])
    },
    "AbdominopelvicCavity": {"818987002": ("Abdominopelvic cavity", [7154, 8134])},
    "AbducensNerve": {"80622005": ("abducens nerve", [7706, 7710, 8134])},
    "AbducensNerveStructure": {
        "80622005": ("Abducens nerve structure (body structure)", [])
    },
    "Abduction": {"60074003": ("Abduction", [92])},
    "AbductionFunction": {"60074003": ("Abduction, function (observable entity)", [])},
    "AberdeenAngusCattleBreed": {
        "84923006": ("Aberdeen Angus cattle breed (organism)", [])
    },
    "AberdeenAngusCowBreed": {"84923006": ("Aberdeen Angus cow breed", [7480])},
    "AbigarCattleBreed": {"131540009": ("Abigar cattle breed", [7480])},
    "AblationFrequency": {"386132006": ("Ablation frequency", [3425])},
    "AblationOperationForArrhythmia": {
        "233159005": ("Ablation operation for arrhythmia", [3721])
    },
    "AblationPower": {"386131004": ("Ablation power", [3425])},
    "Abnormal": {"263654008": ("Abnormal", [222, 242, 3120])},
    "AbnormalECG": {"102594003": ("Abnormal ECG", [3201, 3221, 3677, 12246])},
    "AbnormalExerciseToleranceTest": {
        "165084003": ("Abnormal exercise tolerance test", [3201])
    },
    "AbnormalExtracardiacTracerUptake": {
        "429576000": ("Abnormal extracardiac tracer uptake (finding)", [])
    },
    "AbnormalExtracardiacUptake": {
        "429576000": ("Abnormal extracardiac uptake", [3116])
    },
    "AbnormalGait": {"22325002": ("Abnormal gait (finding)", [])},
    "AbnormalRenalFunction": {"39539005": ("Abnormal Renal Function", [64, 10045])},
    "AbnormalRespiratoryRhythm": {"248584002": ("abnormal respiratory rhythm", [3416])},
    "AbnormallyHigh": {"371879000": ("Abnormally High", [222])},
    "AbnormallyLow": {"371880002": ("Abnormally Low", [222])},
    "AbnormallySmallCardiacChamber": {
        "373125003": ("Abnormally small cardiac chamber", [3705])
    },
    "AbondanceCattleBreed": {"133551002": ("Abondance cattle breed", [7480])},
    "Abscess": {"44132006": ("Abscess", [6030, 6031, 7151, 7159, 7194])},
    "AbsenceFindings": {"272519000": ("Absence findings (qualifier value)", [])},
    "Absent": {"272519000": ("Absent", [240, 241])},
    "AbtenauerHorseBreed": {"133068005": ("Abtenauer horse breed", [7480])},
    "AbyssinianCat": {"36074003": ("Abyssinian cat", [7480])},
    "AbyssinianHighlandZebuCattleBreed": {
        "133653008": ("Abyssinian Highland Zebu cattle breed", [7480])
    },
    "AbyssinianHorseBreed": {"133122005": ("Abyssinian horse breed", [7480])},
    "AbyssinianShorthornedZebuCattleBreed": {
        "133654002": ("Abyssinian Shorthorned Zebu cattle breed", [7480])
    },
    "Acanthioparietal": {"399242004": ("acanthioparietal", [501, 4010])},
    "AcanthioparietalProjection": {
        "399242004": ("Acanthioparietal projection (qualifier value)", [])
    },
    "Accentuated": {"428691001": ("accentuated", [3210])},
    "AccessoryAtrioventricularBundle": {
        "8225009": ("Accessory atrioventricular bundle", [3011, 8134])
    },
    "AccessoryNerve": {"15119000": ("accessory nerve", [7706, 7710, 8134])},
    "AccessoryNerveStructure": {
        "15119000": ("Accessory nerve structure (body structure)", [])
    },
    "AccessoryRenalArtery": {
        "85383006": (
            "Accessory Renal Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12115, 12125],
        )
    },
    "AccidentAndEmergency": {"225728007": ("Accident and Emergency", [7030])},
    "AccidentAndEmergencyDepartment": {
        "225728007": ("Accident and Emergency department (environment)", [])
    },
    "AcehCattleBreed": {"133655001": ("Aceh cattle breed", [7480])},
    "Acepromazine": {"96218000": ("Acepromazine", [623, 626])},
    "Acetabulum": {"37783008": ("Acetabulum", [1000, 1001, 1005, 8134])},
    "AcetateC11": {"129513004": ("Acetate C^11^", [4021])},
    "AceticAcid": {"2869004": ("Acetic acid", [4412, 8114])},
    "AchhamCattleBreed": {"133656000": ("Achham cattle breed", [7480])},
    "AchillesTendon": {"61352006": ("Achilles Tendon", [12321])},
    "AcidFastStain": {"406976001": ("Acid fast stain (substance)", [])},
    "AcidFastStainMethod": {"67122001": ("Acid fast stain method", [8130])},
    "AcidPhosphataseStain": {"255792001": ("acid phosphatase stain", [8112])},
    "AcousticRadiationsStructure": {
        "410726006": ("Acoustic radiations structure (body structure)", [])
    },
    "AcquiredIncompleteStentApposition": {
        "408710003": ("Acquired Incomplete stent apposition", [3494])
    },
    "AcridineOrangeStain": {"29252006": ("acridine orange stain", [4412, 8112])},
    "AcriflavineStain": {"17693003": ("acriflavine stain", [8112])},
    "AcromioclavicularJoint": {
        "85856004": (
            "Acromioclavicular joint",
            [4, 1000, 1001, 1004, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AcromioclavicularJointStructure": {
        "85856004": ("Acromioclavicular joint structure (body structure)", [])
    },
    "AcromionProcessOfScapula": {
        "31934006": ("Acromion process of scapula", [6115, 8134])
    },
    "AcrylicPolymer": {"261712009": ("Acrylic polymer (substance)", [])},
    "AcrylicResin": {"261712009": ("Acrylic resin", [8115])},
    "ActinicKeratosis": {"201101007": ("Actinic keratosis", [4405])},
    "ActiveIngredient": {"127489000": ("Active Ingredient", [4050])},
    "ActiveProblem": {"394774009": ("Active problem", [3770])},
    "AcuteFebrileMucocutaneousLymphNodeSyndrome": {
        "75053002": ("Acute febrile mucocutaneous lymph node syndrome", [12248, 12249])
    },
    "AcuteHepatitis": {"37871000": ("Acute Hepatitis", [12323])},
    "AcuteMitralRegurgitation": {"373116009": ("Acute mitral regurgitation", [3700])},
    "AcuteMitralRegurgitationFromChordalDysfunction": {
        "371814000": ("Acute mitral regurgitation from chordal dysfunction", [3728])
    },
    "AcuteMitralRegurgitationFromChordalRupture": {
        "371813006": ("Acute mitral regurgitation from chordal rupture", [3728])
    },
    "AcuteMitralRegurgitationFromPapillaryMuscleDysfunction": {
        "371815004": (
            "Acute mitral regurgitation from papillary muscle dysfunction",
            [3728],
        )
    },
    "AcuteMitralRegurgitationFromPapillaryMuscleRupture": {
        "371816003": (
            "Acute mitral regurgitation from papillary muscle rupture",
            [3728],
        )
    },
    "AcuteMyocardialInfarction": {"57054005": ("Acute myocardial infarction", [3700])},
    "AcuteNonSTSegmentElevationMyocardialInfarction": {
        "401314000": (
            "Acute non-ST segment elevation myocardial infarction (disorder)",
            [],
        )
    },
    "AcuteOnset": {"373933003": ("Acute onset (qualifier value)", [])},
    "AcuteSTSegmentElevationMyocardialInfarction": {
        "401303003": ("Acute ST segment elevation myocardial infarction (disorder)", [])
    },
    "AcuteVentricularSeptalRupture": {
        "371817007": ("Acute ventricular septal rupture", [3700, 3728])
    },
    "AdaevHorseBreed": {"133069002": ("Adaev horse breed", [7480])},
    "AdamawaCattleBreed": {"133667004": ("Adamawa cattle breed", [7480])},
    "Adduction": {"11554009": ("Adduction", [92])},
    "AdductionFunction": {"11554009": ("Adduction, function (observable entity)", [])},
    "AdenZebuCattleBreed": {"133668009": ("Aden Zebu cattle breed", [7480])},
    "AdenoAssociatedVirusGroup": {
        "112381006": ("Adeno-associated virus group", [638, 641])
    },
    "Adenocarcinoma": {"1187332001": ("Adenocarcinoma", [638, 639])},
    "AdenocarcinomaNoSubtype": {
        "35917007": ("Adenocarcinoma, no subtype (morphologic abnormality)", [])
    },
    "AdenocarcinomaOfLargeIntestine": {
        "408645001": ("Adenocarcinoma of large intestine", [638, 639])
    },
    "AdenocarcinomaOfLung": {"254626006": ("Adenocarcinoma of lung", [638, 639])},
    "AdenocarcinomaOfPancreas": {
        "700423003": ("Adenocarcinoma of pancreas", [638, 639])
    },
    "AdenocarcinomaOfRectum": {"254582000": ("Adenocarcinoma of rectum", [638, 639])},
    "AdenocarcinomaWithApocrineMetaplasia": {
        "22694002": (
            "Adenocarcinoma with apocrine metaplasia (morphologic abnormality)",
            [],
        )
    },
    "Adenohypophysis": {
        "62818001": ("Adenohypophysis", [7151, 7153, 7192, 8134, 9514])
    },
    "AdenohypophysisStructure": {
        "62818001": ("Adenohypophysis structure (body structure)", [])
    },
    "AdenoidCysticCarcinoma": {"11671000": ("Adenoid cystic carcinoma", [6030, 6033])},
    "AdenoidalStructure": {"55940004": ("Adenoidal structure (body structure)", [])},
    "Adenolipoma": {"22024005": ("Adenolipoma", [6030, 6031])},
    "Adenoma": {"1187375007": ("Adenoma", [6030, 6031])},
    "AdenomaNoSubtype": {
        "32048006": ("Adenoma, no subtype (morphologic abnormality)", [])
    },
    "Adenomyoepithelioma": {"128765009": ("Adenomyoepithelioma", [6030, 6031])},
    "Adenosine": {"108502004": ("Adenosine", [3204])},
    "AdenosineA2ReceptorAgonist": {
        "432062000": ("Adenosine A2 receptor agonist", [3204])
    },
    "AdenosinePreparation": {"108502004": ("Adenosine preparation (product)", [])},
    "AdenosineStressProtocol": {
        "424444005": ("Adenosine Stress protocol", [3261, 12001])
    },
    "Adenosis": {"57597008": ("Adenosis", [6030, 6031])},
    "AdiposeTissue": {"55603005": ("Fat", [7151, 7166, 7191])},
    "AdjunctIntent": {"421974008": ("Adjunct intent", [3629])},
    "AdjuvantIntent": {"373846009": ("Adjuvant intent", [3629])},
    "AdministeredAfterLabVisit": {
        "371899006": ("Administered after lab visit", [3741])
    },
    "AdministeredAfterPercutaneousCoronaryIntervention": {
        "371902009": (
            "Administered After Percutaneous Coronary Intervention",
            [3742, 3743],
        )
    },
    "AdministeredBeforeLabVisit": {
        "371898003": ("Administered before lab visit", [3741])
    },
    "AdministeredBetween3And6HoursBeforePCI": {
        "371897008": ("Administered between 3 and 6 hours before PCI", [3740])
    },
    "AdministeredBetween6HoursAnd7DaysBeforePCI": {
        "371906007": ("Administered between 6 hours and 7 days before PCI", [3740])
    },
    "AdministeredDuringLabVisit": {
        "371905006": ("Administered during lab visit", [3741])
    },
    "AdministeredDuringPercutaneousCoronaryIntervention": {
        "371903004": ("Administered During Percutaneous Coronary Intervention", [3742])
    },
    "AdministeredLessThan3HoursBeforePCI": {
        "371896004": ("Administered less than 3 hours before PCI", [3740])
    },
    "AdministeredLessThan72HoursBeforePCI": {
        "371901002": ("Administered Less than 72 Hours before PCI", [3743])
    },
    "AdministeredPriorToPercutaneousCoronaryIntervention": {
        "371904005": (
            "Administered Prior to Percutaneous Coronary Intervention",
            [3742],
        )
    },
    "AdministrationOfAnesthesia": {
        "399097000": ("Administration of anesthesia (procedure)", [])
    },
    "AdministrationOfMedicationNotDoneDueToContraindication": {
        "373147003": (
            "Administration of medication not done due to contraindication (situation)",
            [],
        )
    },
    "AdministrationOfRadiopharmaceutical": {
        "440252007": ("Administration of radiopharmaceutical (procedure)", [])
    },
    "AdministrationOfSedative": {
        "72641008": ("Administration of sedative (procedure)", [])
    },
    "AdrenalGland": {
        "23451007": (
            "Adrenal gland",
            [
                4,
                1000,
                1001,
                1005,
                4030,
                6202,
                6204,
                7151,
                7154,
                7192,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "AdrenalStructure": {"23451007": ("Adrenal structure (body structure)", [])},
    "AdrenergicAgent": {"86308005": ("Adrenergic agent", [621, 622])},
    "Adventitia": {"30180000": ("Adventitia", [7151, 7156, 7192, 8134, 9514])},
    "AdventitialDegeneration": {"47631006": ("adventitial degeneration", [3817])},
    "Affenpinscher": {"52946002": ("Affenpinscher", [7480])},
    "Afferent": {"49530007": ("Afferent", [2])},
    "AfghanCattleBreed": {"133669001": ("Afghan cattle breed", [7480])},
    "AfghanHound": {"77213006": ("Afghan hound", [7480])},
    "AfricanRace": {"413464008": ("African race", [6099])},
    "AfricanaSheepBreed": {"131768005": ("Africana sheep breed", [7480])},
    "AfricanderCattleBreed": {"131426006": ("Africander cattle breed", [7480])},
    "AfricangnusCattleBreed": {"131541008": ("Africangnus cattle breed", [7480])},
    "AfricangusCattleBreed": {"131541008": ("Africangus cattle breed (organism)", [])},
    "After": {"288563008": ("After", [3600])},
    "AfterMenopause": {"307429007": ("After menopause", [6086])},
    "AfterProcedure": {"303110006": ("After Procedure", [631, 3815, 12002, 12102])},
    "AfterValues": {"288563008": ("After values (qualifier value)", [])},
    "Agar": {"10249006": ("Agar", [8115])},
    "AgeroleseCattleBreed": {"131542001": ("Agerolese cattle breed", [7480])},
    "AggressiveFibromatosis": {
        "47284001": ("Aggressive fibromatosis (morphologic abnormality)", [])
    },
    "AidiDogBreed": {"132406005": ("Aidi dog breed", [7480])},
    "AinuDogBreed": {"132516004": ("Ainu dog breed", [7480])},
    "Air": {"15158005": ("Air", [12, 13, 3850, 6211, 6350, 11002])},
    "AirHeatingPad": {"468192005": ("Air heating pad", [635])},
    "AirHeatingPadSystem": {
        "468192005": ("Air heating pad system (physical object)", [])
    },
    "AirTrapping": {"76171001": ("Air-trapping", [6102, 6105, 6108])},
    "AiredaleTerrier": {"3921008": ("Airedale terrier", [7480])},
    "AirwayManagement": {"386509000": ("Airway management (procedure)", [])},
    "AirwayStructure": {
        "89187006": ("Airway structure", [6102, 6109, 6111, 7151, 7192, 9514])
    },
    "AkbashDogBreed": {"132531008": ("Akbash dog breed", [7480])},
    "AkhaPigBreed": {"133172007": ("Akha pig breed", [7480])},
    "AkhalTekeHorseBreed": {"133121003": ("Akhal-Teke horse breed", [7480])},
    "Akinesis": {"195675009": ("Akinesis", [3703, 3706])},
    "AkitaDog": {"84514002": ("Akita dog", [7480])},
    "AksaîBlackPiedPigBreed": {"132118009": ("Aksaî Black Pied pig breed", [7480])},
    "AlaTauCattleBreed": {"133552009": ("Ala-Tau cattle breed", [7480])},
    "AlambadiCattleBreed": {"133670000": ("Alambadi cattle breed", [7480])},
    "AlanineAminotransferaseSerumGlutamicPyruvicTransaminaseLevelRaised": {
        "409673008": (
            "Alanine aminotransferase (serum glutamic-pyruvic transaminase) level raised (finding)",
            [],
        )
    },
    "AlanoEspañolDogBreed": {"132540007": ("Alano Español dog breed", [7480])},
    "AlapahaBluebloodBulldogBreed": {
        "132532001": ("Alapaha blueblood bullDog breed", [7480])
    },
    "AlaskanKleeKaiDogBreed": {"406725008": ("Alaskan Klee Kai dog breed", [7480])},
    "AlaskanMalamute": {"53228008": ("Alaskan malamute", [7480])},
    "AlbanianDwarfCattleBreed": {"133554005": ("Albanian Dwarf cattle breed", [7480])},
    "AlbanianHorseBreed": {"133070001": ("Albanian horse breed", [7480])},
    "AlbanianIllyrianCattleBreed": {
        "133553004": ("Albanian Illyrian cattle breed", [7480])
    },
    "AlbanianNativePigBreed": {"133209008": ("Albanian Native pig breed", [7480])},
    "AlbersSchonberg": {"399142007": ("Albers-Schonberg", [4012])},
    "AlbersSchönbergProjection": {
        "399142007": ("Albers-Schönberg projection (qualifier value)", [])
    },
    "AlbertStain": {"255793006": ("Albert's stain", [8112])},
    "AlbeseCattleBreed": {"131543006": ("Albese cattle breed", [7480])},
    "AlbèresCattleBreed": {"131555000": ("Albères cattle breed (organism)", [])},
    "AlbèresCattlebreed": {"131555000": ("Albères cattlebreed", [7480])},
    "AlcianBlue8GXStain": {"4656000": ("alcian blue 8GX stain", [8112])},
    "AlcoholConsumption": {"897148007": ("Alcohol consumption", [3774])},
    "AlcoholIntake": {"160573003": ("Alcohol intake (observable entity)", [])},
    "AlcoholSolubleNigrosineStain": {
        "47995002": ("alcohol soluble nigrosine stain", [8112])
    },
    "AlcoholicLiverDamage": {"41309000": ("Alcoholic liver damage (disorder)", [])},
    "AlcoholicLiverDisease": {"41309000": ("Alcoholic liver disease", [12325])},
    "AldehydeFuchsinStain": {"406981005": ("aldehyde fuchsin stain", [8112])},
    "AlentejanaCattleBreed": {"131556004": ("Alentejana cattle breed (organism)", [])},
    "AlentejanaCattlebreed": {"131556004": ("Alentejana cattlebreed", [7480])},
    "AlentejanaPigBreed": {"132086001": ("Alentejana pig breed", [7480])},
    "AlentejoHerderDogBreed": {"132393007": ("Alentejo herder dog breed", [7480])},
    "AleppoCattleBreed": {"133575002": ("Aleppo cattle breed", [7480])},
    "Alexander": {"399237007": ("Alexander", [4012])},
    "AlexanderProjection": {
        "399237007": ("Alexander projection (qualifier value)", [])
    },
    "Alfentanil": {"387560008": ("Alfentanil", [65, 66])},
    "AlgarviaGoatBreed": {"131677009": ("Algarvia goat breed", [7480])},
    "AliabDinkaCattleBreed": {"131441006": ("Aliab Dinka cattle breed", [7480])},
    "AlistanaSanabresaCattleBreed": {
        "133290002": ("Alistana-Sanabresa cattle breed", [7480])
    },
    "Alive": {"438949009": ("Alive", [3772])},
    "AliveAndWell": {"81323004": ("Alive and well", [3772])},
    "AlizarinBlueSStain": {"54432009": ("alizarin blue S stain", [8112])},
    "AlizarinCyanineGreenStain": {"21951008": ("alizarin cyanine green stain", [8112])},
    "AlizarinRedSStain": {"65580004": ("alizarin red S stain", [8112])},
    "AlizarinYellowGGStain": {"27016007": ("alizarin yellow GG stain", [8112])},
    "AlizarinYellowRStain": {"28622002": ("alizarin yellow R stain", [8112])},
    "AlkaliBlue5B4BStain": {"76925007": ("alkali blue 5B (4B) stain", [8112])},
    "AlkaliBlue6BStain": {"63929007": ("alkali blue 6B stain", [8112])},
    "AlkalinePhosphataseStain": {"406971006": ("alkaline phosphatase stain", [8112])},
    "AllLegs": {"42694008": ("All legs", [7482, 7483, 8134])},
    "AllergyAndImmunology": {"309913004": ("Allergy and Immunology", [7030])},
    "AllergyToContrastMedia": {
        "293637006": ("Allergy to contrast media (disorder)", [])
    },
    "AllergyToXRayContrastMedia": {
        "293638001": ("Allergy to X-Ray contrast media (disorder)", [])
    },
    "AlmanzorenaCattleBreed": {"133591004": ("Almanzorena cattle breed", [7480])},
    "AlmostEntirelyFat": {"129716005": ("Almost entirely fat", [6000, 6001])},
    "AlmostEntirelyFatBreastComposition": {
        "129716005": ("Almost entirely fat breast composition (finding)", [])
    },
    "AlongEdge": {"57183005": ("Along edge (qualifier value)", [])},
    "AlphaAdrenergicBlockingAgent": {
        "67440007": ("Alpha-adrenergic blocking agent", [10])
    },
    "Alphachloralose": {"277016007": ("Alphachloralose", [623, 625])},
    "Alphadolone": {"125707004": ("Alphadolone", [623, 625])},
    "AlphadoloneAcetate": {"125707004": ("Alphadolone acetate (substance)", [])},
    "AlpineChamoiseeGoatBreed": {"131674002": ("Alpine Chamoisee goat breed", [7480])},
    "AlpineDachsbrackeDogBreed": {
        "132542004": ("Alpine dachsbracke dog breed", [7480])
    },
    "AlpineGoat": {"70431006": ("Alpine goat (organism)", [])},
    "AlpineGoatBreed": {"70431006": ("Alpine goat breed", [7480])},
    "AltaiHorseBreed": {"133120002": ("Altai horse breed", [7480])},
    "AltaiMountainGoatBreed": {"131618005": ("Altai Mountain goat breed", [7480])},
    "AltaySheepBreed": {"132845007": ("Altay sheep breed", [7480])},
    "AlterRealHorseBreed": {"133071002": ("Alter Real horse breed", [7480])},
    "Aluminum": {"12503006": ("Aluminum", [10006, 10067])},
    "AluminumANDORAluminumCompound": {
        "105830007": ("Aluminum AND/OR aluminum compound (substance)", [])
    },
    "AluminumChloride": {"387372003": ("aluminum chloride", [4412])},
    "AluminumHydroxide": {"273944007": ("Aluminum hydroxide", [4410])},
    "AluminumOxide": {"83881004": ("Aluminum Oxide", [300])},
    "AlurCattleBreed": {"131442004": ("Alur cattle breed", [7480])},
    "AlveolarProcessOfMandible": {
        "38754007": ("alveolar process of mandible", [7601, 8134])
    },
    "AlveolarSoftPartSarcoma": {"88195001": ("Alveolar soft part sarcoma", [638, 639])},
    "AmbuBag": {"371785003": ("Ambu Bag", [3554])},
    "AmbulatoryEEG": {"252735006": ("Ambulatory EEG", [3049])},
    "AmelanoticMelanoma": {"70594002": ("Amelanotic melanoma", [638, 639])},
    "AmericanAlbinoHorse": {"45790002": ("American Albino horse (organism)", [])},
    "AmericanAlbinoHorseBreed": {"45790002": ("American Albino horse breed", [7480])},
    "AmericanBashkirCurlyHorseBreed": {
        "133072009": ("American Bashkir Curly horse breed", [7480])
    },
    "AmericanBerkshirePigBreed": {
        "132046009": ("American Berkshire pig breed", [7480])
    },
    "AmericanBisonXCattleBreed": {
        "133703008": ("American bison X cattle breed", [7480])
    },
    "AmericanBlueGasconHoundDogBreed": {
        "132400004": ("American Blue Gascon Hound dog breed", [7480])
    },
    "AmericanBobtailCatBreed": {"413488005": ("American bobtail cat breed", [7480])},
    "AmericanBobtailLonghairCatBreed": {
        "132658005": ("American Bobtail Longhair cat breed", [7480])
    },
    "AmericanBobtailShorthairCatBreed": {
        "132657000": ("American Bobtail Shorthair cat breed", [7480])
    },
    "AmericanBuckskinHorse": {"90050009": ("American Buckskin horse (organism)", [])},
    "AmericanBuckskinHorseBreed": {
        "90050009": ("American Buckskin horse breed", [7480])
    },
    "AmericanBulldogBreed": {"132534000": ("American Bulldog breed", [7480])},
    "AmericanCashmereGoatBreed": {
        "131617000": ("American Cashmere goat breed", [7480])
    },
    "AmericanCockerSpaniel": {"22697009": ("American cocker spaniel (organism)", [])},
    "AmericanCockerSpanielDogBreed": {
        "22697009": ("American cocker spaniel dog breed", [7480])
    },
    "AmericanCoonhound": {"63269002": ("American coonhound (organism)", [])},
    "AmericanCoonhoundDogBreed": {"63269002": ("American coonhound dog breed", [7480])},
    "AmericanCreamDraftHorseBreed": {
        "133119008": ("American Cream Draft horse breed", [7480])
    },
    "AmericanCreamHorse": {"26837006": ("American cream horse (organism)", [])},
    "AmericanCreamHorseBreed": {"26837006": ("American cream horse breed", [7480])},
    "AmericanCurlCatBreed": {"132659002": ("American Curl cat breed", [7480])},
    "AmericanDraftPonyBreed": {
        "425118005": ("American draft pony breed (organism)", [])
    },
    "AmericanDraftPonyHorseBreed": {
        "425118005": ("American draft pony horse breed", [7480])
    },
    "AmericanEskimoDog": {"31392000": ("American eskimo dog (organism)", [])},
    "AmericanEskimoDogBreed": {"31392000": ("American eskimo dog breed", [7480])},
    "AmericanEskimoMiniatureDogBreed": {
        "132410008": ("American Eskimo, Miniature dog breed", [7480])
    },
    "AmericanEskimoStandardDogBreed": {
        "132409003": ("American Eskimo, standard dog breed", [7480])
    },
    "AmericanEskimoToyDogBreed": {
        "132411007": ("American Eskimo, Toy dog breed", [7480])
    },
    "AmericanEssexPigBreed": {"132030001": ("American Essex pig breed", [7480])},
    "AmericanFoxhound": {"88779009": ("American foxhound", [7480])},
    "AmericanHairlessTerrierDogBreed": {
        "132528007": ("American Hairless Terrier dog breed", [7480])
    },
    "AmericanIndianDogBreed": {"132407001": ("American Indian Dog breed", [7480])},
    "AmericanIndianHorseHorseBreed": {
        "132951001": ("American Indian Horse horse breed", [7480])
    },
    "AmericanIndianOrAlaskaNative": {
        "413490006": ("American Indian or Alaska native", [6099])
    },
    "AmericanLamaleseDogBreed": {"132627009": ("American lamalese dog breed", [7480])},
    "AmericanLandracePigBreed": {"132231009": ("American Landrace pig breed", [7480])},
    "AmericanMerinoSheep": {"73191001": ("American merino sheep (organism)", [])},
    "AmericanMerinoSheepBreed": {"73191001": ("American merino sheep breed", [7480])},
    "AmericanMiniatureHorse": {"54699009": ("American miniature horse (organism)", [])},
    "AmericanMiniatureHorseBreed": {
        "54699009": ("American miniature horse breed", [7480])
    },
    "AmericanMustangHorseBreed": {
        "132952008": ("American Mustang horse breed", [7480])
    },
    "AmericanPaintHorse": {"7623008": ("American paint horse (organism)", [])},
    "AmericanPaintHorseBreed": {"7623008": ("American paint horse breed", [7480])},
    "AmericanPitBullTerrier": {
        "12390000": ("American pit bull terrier (organism)", [])
    },
    "AmericanPitBullTerrierDogBreed": {
        "12390000": ("American pit bull terrier dog breed", [7480])
    },
    "AmericanPony": {"46408008": ("American pony (organism)", [])},
    "AmericanPonyHorseBreed": {"46408008": ("American pony horse breed", [7480])},
    "AmericanQuarterHorseHorseBreed": {
        "132953003": ("American Quarter Horse horse breed", [7480])
    },
    "AmericanSaddlebredHorse": {
        "42724005": ("American saddlebred horse (organism)", [])
    },
    "AmericanSaddlebredHorseBreed": {
        "42724005": ("American saddlebred horse breed", [7480])
    },
    "AmericanShetlandPonyHorseBreed": {
        "132954009": ("American Shetland pony horse breed", [7480])
    },
    "AmericanShorthairCat": {"69855002": ("American shorthair cat", [7480])},
    "AmericanTrotterHorse": {"26973000": ("American trotter horse (organism)", [])},
    "AmericanTrotterHorseBreed": {"26973000": ("American trotter horse breed", [7480])},
    "AmericanTunisHorse": {"72394007": ("American tunis horse (organism)", [])},
    "AmericanTunisHorseBreed": {"72394007": ("American tunis horse breed", [7480])},
    "AmericanTunisSheepBreed": {"132812007": ("American Tunis sheep breed", [7480])},
    "AmericanWalkingPonyHorseBreed": {
        "133117005": ("American Walking Pony horse breed", [7480])
    },
    "AmericanWaterSpaniel": {"31971008": ("American water spaniel (organism)", [])},
    "AmericanWaterSpanielDogBreed": {
        "31971008": ("American water spaniel dog breed", [7480])
    },
    "AmericanWhiteParkCattleBreed": {
        "131557008": ("American White Park cattle breed", [7480])
    },
    "AmericanWirehairedCat": {"21726001": ("American wirehaired cat", [7480])},
    "AmericanYorkshirePigBreed": {
        "132045008": ("American Yorkshire pig breed", [7480])
    },
    "AmerifaxcattleBreed": {"131558003": ("Amerifaxcattle breed", [7480])},
    "AminoglycosideAntibiotic": {"14443002": ("Aminoglycoside antibiotic", [621, 622])},
    "AminoglycosideClassOfAntibiotic": {
        "14443002": ("Aminoglycoside -class of antibiotic- (substance)", [])
    },
    "AmmoniaN13": {"129508003": ("Ammonia N^13^", [4021])},
    "Amniocentesis": {"34536000": ("Amniocentesis", [9])},
    "AmnioinfusionInjectionOfAmnion": {
        "15415002": ("Amnioinfusion [injection of amnion]", [9])
    },
    "AmnioticFluid": {"77012006": ("Amniotic fluid (substance)", [])},
    "AmorphousCalcification": {"129760005": ("Amorphous calcification", [6010, 6011])},
    "Amphetamine": {"703842006": ("Amphetamine", [6089])},
    "AmritMahalZebuCattleBreed": {
        "133594007": ("Amrit Mahal zebu cattle breed", [7480])
    },
    "AmritmahalCattleBreed": {"133594007": ("Amritmahal cattle breed (organism)", [])},
    "Amygdala": {"4958002": ("Amygdala", [7140, 7151, 7153, 7192, 8134, 9514])},
    "AmygdaloidStructure": {"4958002": ("Amygdaloid structure (body structure)", [])},
    "Amyloid": {"37279009": ("Amyloid (tumor)", [6030, 6031])},
    "AmyloidTumor": {"37279009": ("Amyloid tumor (morphologic abnormality)", [])},
    "AnadoluHorseBreed": {"132955005": ("Anadolu horse breed", [7480])},
    "AnalStructure": {"53505006": ("Anal structure (body structure)", [])},
    "Analgesic": {"373265006": ("Analgesic", [621, 622])},
    "AnastomosisOfPulmonarySubclavianArteryByBlalockTaussigOperation": {
        "13662000": (
            "Anastomosis of pulmonary-subclavian artery by Blalock-Taussig operation (procedure)",
            [],
        )
    },
    "Anastrozole": {"386910003": ("Anastrozole", [6080])},
    "AnatolianBlackCattleBreed": {
        "131559006": ("Anatolian Black cattle breed", [7480])
    },
    "AnatolianShepherdDogBreed": {
        "409926004": ("Anatolian shepherd dog breed", [7480])
    },
    "AnatomicStructurePotentiallyInvolvedInEvolutionOfDisease": {
        "413530006": (
            "Anatomic structure potentially involved in evolution of disease (observable entity)",
            [],
        )
    },
    "Anatomical": {"51845000": ("anatomical", [20])},
    "AnatomicalPoint": {"711101009": ("Anatomical point", [7110])},
    "AnatomicalPosition": {"51845000": ("Anatomical position (finding)", [])},
    "AnatomicalReferencePlane": {"243898001": ("Lumbo-Sacral Spine", [7304, 8134])},
    "AnatomicalStructure": {
        "91723000": ("Anatomical Structure", [7150, 9501, 9502, 9580])
    },
    "Anchoring": {"129380009": ("Anchoring", [7310])},
    "AnchoringAction": {"129380009": ("Anchoring - action (qualifier value)", [])},
    "AndalusianBlackCattleBreed": {
        "131560001": ("Andalusian Black cattle breed", [7480])
    },
    "AndalusianBlondCattleBreed": {
        "133291003": ("Andalusian Blond cattle breed", [7480])
    },
    "AndalusianGrayCattleBreed": {
        "131561002": ("Andalusian Gray cattle breed", [7480])
    },
    "AndalusianHorse": {"80777007": ("Andalusian horse (organism)", [])},
    "AndalusianHorseBreed": {"80777007": ("Andalusian horse breed", [7480])},
    "AndeanHorseBreed": {"132956006": ("Andean horse breed", [7480])},
    "AndrogenDeprivationTherapy": {
        "707266006": ("Androgen Deprivation Therapy", [6323])
    },
    "AnesthesiaFinishTime": {
        "398164008": ("Anesthesia finish time (observable entity)", [])
    },
    "AnesthesiaInduction": {"241687005": ("Anesthesia induction", [634])},
    "AnesthesiaStartTime": {
        "398325003": ("Anesthesia start time (observable entity)", [])
    },
    "Anesthesiology": {"309901009": ("Anesthesiology", [7030])},
    "AnestheticDepartment": {"309901009": ("Anesthetic department (environment)", [])},
    "AnestheticFaceMask": {"297120004": ("Anesthetic face mask", [617, 618])},
    "Aneurysm": {"85659009": ("aneurysm", [3810])},
    "AneurysmOfAorticRoot": {"373134008": ("Aneurysm of Aortic Root", [3709])},
    "Aneurysmal": {"255378009": ("Aneurysmal", [3712])},
    "AngelnCattleBreed": {"131562009": ("Angeln cattle breed", [7480])},
    "AngelnSaddlebackPigBreed": {"132252001": ("Angeln Saddleback pig breed", [7480])},
    "Angina": {"194828000": ("Angina (disorder)", [])},
    "AnginaClassI": {"61490001": ("Angina Class I", [3719])},
    "AnginaClassII": {"41334000": ("Angina Class II", [3719])},
    "AnginaClassIII": {"85284003": ("Angina Class III", [3719])},
    "AnginaClassIV": {"89323001": ("Angina Class IV", [3719])},
    "AnginaPectoris": {"194828000": ("Angina pectoris", [64, 3201, 12246])},
    "Angiography": {"77343006": ("Angiography", [100, 3405, 3428])},
    "Angiolipoma": {"73219006": ("Angiolipoma", [6030, 6031])},
    "Angioma": {"400210000": ("angioma", [3810])},
    "Angiomatosis": {"14350002": ("Angiomatosis", [6030, 6031])},
    "AngioplastyInflationDuration": {
        "371852004": ("Angioplasty Inflation duration", [3425])
    },
    "AngioplastyInflationPressure": {
        "371851006": ("Angioplasty Inflation pressure", [3425])
    },
    "Angiosarcoma": {"39000009": ("Angiosarcoma", [6030, 6033])},
    "AngiotensinConvertingEnzymeInhibitorAgent": {
        "41549009": ("Angiotensin-converting enzyme inhibitor agent (product)", [])
    },
    "AngiotensinIIReceptorAntagonist": {
        "96308008": ("Angiotensin II receptor antagonist", [3760])
    },
    "Angle": {"1483009": ("Angle", [12304])},
    "AngleOfRib": {"14510004": ("Angle of rib", [6115, 8134])},
    "AngloArabHorseBreed": {"133044002": ("Anglo-Arab horse breed", [7480])},
    "AngloFrancaisDeMoyenVenerieDogBreed": {
        "132536003": ("Anglo-Francais de moyen venerie dog breed", [7480])
    },
    "AngloFrancaisDePetitVenerieDogBreed": {
        "132537007": ("Anglo-Francais de petit venerie dog breed", [7480])
    },
    "AngloKabardaHorseBreed": {"132957002": ("Anglo-Kabarda horse breed", [7480])},
    "AngloNubianGoat": {"16015002": ("Anglo nubian goat (organism)", [])},
    "AngloNubianGoatBreed": {"16015002": ("Anglo nubian goat breed", [7480])},
    "AngoraGoat": {"64158000": ("Angora goat (organism)", [])},
    "AngoraGoatBreed": {"64158000": ("Angora goat breed", [7480])},
    "Angular": {"1483009": ("Angular (qualifier value)", [])},
    "AnilineBlueStain": {"406990003": ("aniline blue stain", [8112])},
    "AnimalFeed": {"82566005": ("Animal feed (substance)", [])},
    "AnionExchangeResin": {"346322006": ("Anion exchange resin", [3761])},
    "AnkinaCattleBreed": {"131443009": ("Ankina cattle breed", [7480])},
    "AnkleJoint": {
        "70258002": (
            "Ankle joint",
            [4, 1000, 1001, 1006, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "AnkleJointStructure": {"70258002": ("Ankle joint structure (body structure)", [])},
    "AnkoleCattleBreed": {"131427002": ("Ankole cattle breed", [7480])},
    "AnkoleWatusiCattleBreed": {"131428007": ("Ankole-Watusi cattle breed", [7480])},
    "AnnularAbscessOfAorticRoot": {
        "373135009": ("Annular Abscess of Aortic Root", [3709])
    },
    "AnomalousPulmonaryVein": {
        "128585006": (
            "Anomalous pulmonary vein",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AntecubitalVein": {
        "128553008": (
            "Antecubital vein",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AntegradeDirection": {"263677008": ("Antegrade Direction", [12306])},
    "AntegradeFlow": {"263677008": ("Antegrade Flow", [12221])},
    "Anterior": {
        "255549009": ("Anterior", [2, 5, 211, 212, 3019, 4005, 6024, 6025, 12281])
    },
    "AnteriorAuricularLymphNode": {
        "25247006": ("anterior auricular lymph node", [7600, 8134])
    },
    "AnteriorCardiacVein": {
        "194996006": (
            "Anterior cardiac vein",
            [4, 3010, 3827, 3839, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AnteriorCerebralArtery": {
        "60176003": (
            "Anterior Cerebral Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "AnteriorCervicalLymphNode": {
        "5727003": ("anterior cervical lymph node", [7600, 8134])
    },
    "AnteriorChamber": {"31636006": ("Anterior Chamber", [4233])},
    "AnteriorChamberOfEye": {
        "31636006": ("Anterior chamber of eye", [4209, 4211, 8134])
    },
    "AnteriorChamberOfEyeStructure": {
        "31636006": ("Anterior chamber of eye structure (body structure)", [])
    },
    "AnteriorCommissure": {
        "62872008": ("Anterior Commissure", [7101, 7111, 7705, 7710, 8134])
    },
    "AnteriorCommunicatingArtery": {
        "8012006": (
            "Anterior communicating artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12105,
            ],
        )
    },
    "AnteriorCompression": {"441752004": ("Anterior compression", [4015])},
    "AnteriorCompressionOfBreast": {
        "441752004": ("Anterior compression of breast (procedure)", [])
    },
    "AnteriorDescendingBranchOfLeftCoronaryArtery": {
        "59438005": ("Anterior Descending Branch of Left Coronary Artery", [12292])
    },
    "AnteriorEmissiveProjection": {
        "399321004": ("Anterior emissive projection (qualifier value)", [])
    },
    "AnteriorFibromuscularStromaOfProstate": {
        "717025007": ("Anterior fibromuscular stroma of prostate", [6300])
    },
    "AnteriorHornLateralVentricle": {
        "30399003": ("Anterior Horn Lateral Ventricle", [8134, 12022])
    },
    "AnteriorMediastinalLymphNode": {
        "5296000": ("anterior mediastinal lymph node", [7600, 8134])
    },
    "AnteriorMiddleCerebralArteryBifurcation": {
        "397418009": (
            "Anterior-Middle Cerebral Artery Bifurcation",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "AnteriorPosteriorCerebralArteryBifurcation": {
        "397419001": (
            "Anterior-Posterior Cerebral Artery Bifurcation",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "AnteriorProjection": {"399321004": ("Anterior projection", [26, 501])},
    "AnteriorSegmentOfLeftUpperLobe": {
        "22270008": ("Anterior segment of left upper lobe", [6124, 6127])
    },
    "AnteriorSegmentOfRightUpperLobe": {
        "39743006": ("Anterior segment of right upper lobe", [6124, 6127])
    },
    "AnteriorSpinalArtery": {
        "17388009": (
            "Anterior spinal artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AnteriorTibialArtery": {
        "68053000": (
            "Anterior tibial artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "AnteriorTibialLymphNode": {
        "303713004": ("anterior tibial lymph node", [7600, 8134])
    },
    "AnteriorTibialVein": {
        "26703007": (
            "Anterior Tibial Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "AnteriorTriangleOfNeck": {
        "182329002": ("Anterior triangle of neck", [4029, 8134])
    },
    "AnteriorWall": {"263943000": ("Anterior Wall", [12280])},
    "AnteroPosterior": {"399348003": ("antero-posterior", [501, 3466, 4010, 7302])},
    "AnteroPosteriorOblique": {"399312000": ("antero-posterior oblique", [501, 4010])},
    "AnteroPosteriorObliqueProjection": {
        "399312000": ("Antero-posterior oblique projection (qualifier value)", [])
    },
    "AnteroPosteriorProjection": {
        "399348003": ("Antero-posterior projection (qualifier value)", [])
    },
    "Anterolateral": {"37197008": ("Anterolateral", [2])},
    "AntiHeparinAgent": {"3361000": ("Anti-heparin agent", [10, 621, 622])},
    "Antiarrhythmic": {"372813008": ("Antiarrhythmic", [621, 622])},
    "AntiarrhythmicAgent": {"372813008": ("Antiarrhythmic agent (substance)", [])},
    "AntiarrhythmicDrug": {"67507000": ("Antiarrhythmic drug", [10])},
    "Antibiotic": {"255631004": ("Antibiotic", [621, 622, 4410])},
    "Anticholinergic": {"373246003": ("Anticholinergic", [621, 622])},
    "AnticholinergicAgent": {"373246003": ("Anticholinergic agent (substance)", [])},
    "Anticoagulant": {"372862008": ("Anticoagulant", [621, 622])},
    "AnticoagulantAgent": {"81839001": ("Anticoagulant", [10])},
    "Anticonvulsant": {"255632006": ("Anticonvulsant", [621, 622])},
    "AntidiureticHormone": {"77671006": ("Antidiuretic hormone", [621, 622])},
    "Antiemetic": {"52017007": ("Antiemetic", [76, 621, 622])},
    "Antifungal": {"373219008": ("Antifungal", [621, 622, 4410])},
    "AntihemophilicFactorPreparation": {
        "59057006": ("Antihemophilic factor preparation", [10])
    },
    "Antihistamine": {"6425004": ("Antihistamine", [10])},
    "Antihypertensive": {"1182007": ("Antihypertensive", [621, 622])},
    "Anus": {
        "53505006": ("Anus", [4, 645, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "AnusRectumAndSigmoidColon": {
        "110612005": (
            "Anus, rectum and sigmoid colon",
            [4, 4040, 7151, 7192, 8134, 9514],
        )
    },
    "AnusRectumAndSigmoidColonCombinedSite": {
        "110612005": (
            "Anus, rectum and sigmoid colon (combined site) (body structure)",
            [],
        )
    },
    "Anxiety": {"48694002": ("Anxiety", [60, 64, 9300])},
    "Aorta": {
        "15825003": (
            "Aorta",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                4042,
                6129,
                6202,
                6204,
                7151,
                7152,
                7192,
                8134,
                9514,
                12141,
                12223,
                12236,
            ],
        )
    },
    "AortaToPulmonaryArteryConnection": {
        "443096004": ("Aorta to Pulmonary Artery Connection", [12290, 12293])
    },
    "AorticAneurysm": {"67362008": ("Aortic aneurysm", [3700])},
    "AorticArch": {
        "57034009": (
            "Aortic arch",
            [
                4,
                1000,
                1001,
                1004,
                3010,
                3604,
                3606,
                3630,
                4030,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12291,
            ],
        )
    },
    "AorticArchStructure": {"57034009": ("Aortic arch structure (body structure)", [])},
    "AorticBifurcation": {"73166001": ("Aortic Bifurcation", [3837, 7111])},
    "AorticDissection": {"308546005": ("Aortic dissection", [3700])},
    "AorticFistula": {
        "128551005": (
            "Aortic fistula",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "AorticIncompetenceNonRheumatic": {
        "194983005": ("Aortic incompetence, non-rheumatic (disorder)", [])
    },
    "AorticInsufficiency": {"194983005": ("Aortic insufficiency", [3700, 3728])},
    "AorticIsthmus": {
        "88593004": (
            "Aortic isthmus",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514, 12291],
        )
    },
    "AorticLymphNode": {"35783009": ("aortic lymph node", [7600, 8134])},
    "AorticPressureWaveform": {"128444004": ("Aortic pressure waveform", [3003])},
    "AorticPressureWaveformFunction": {
        "128444004": ("Aortic pressure waveform, function (observable entity)", [])
    },
    "AorticRegurgitation": {"60234000": ("Aortic regurgitation", [12248, 12249])},
    "AorticRoot": {"8128003": ("Aortic Root", [12305])},
    "AorticRootFeature": {"364091008": ("Aortic root feature (observable entity)", [])},
    "AorticSinotubularJunction": {
        "443167003": ("Aortic Sinotubular Junction", [12291, 12305])
    },
    "AorticStenosis": {"60573004": ("Aortic stenosis", [64, 3700, 3728, 12248, 12249])},
    "AorticStructure": {"15825003": ("Aortic structure (body structure)", [])},
    "AorticSystolicEjectionPeriod": {
        "371850007": ("Aortic systolic ejection period (observable entity)", [])
    },
    "AorticSystolicEjectionPeriodSepa": {
        "371850007": ("Aortic Systolic Ejection Period (SEPa)", [3616])
    },
    "AorticValve": {
        "34202007": ("Aortic Valve", [3610, 3630, 3701, 12288, 12289, 12305])
    },
    "AorticValveArea": {"251011009": ("Aortic Valve Area", [3614, 3615])},
    "AorticValveDisease": {"8722008": ("Aortic valve disease", [3700])},
    "AorticValveDisorder": {"8722008": ("Aortic valve disorder (disorder)", [])},
    "AorticValveFlow": {"371845001": ("Aortic Valve Flow", [3617])},
    "AorticValvePullbackPressureWaveform": {
        "128453006": ("Aortic valve pullback pressure waveform", [3003])
    },
    "AorticValvePullbackPressureWaveformFunction": {
        "128453006": (
            "Aortic valve pullback pressure waveform, function (observable entity)",
            [],
        )
    },
    "AorticValveRegurgitation": {
        "60234000": ("Aortic valve regurgitation (disorder)", [])
    },
    "AorticValveRing": {"77583004": ("Aortic Valve Ring", [12236, 12242, 12305])},
    "AorticValveStenosis": {"60573004": ("Aortic valve stenosis (disorder)", [])},
    "AorticValveStructure": {
        "34202007": ("Aortic valve structure (body structure)", [])
    },
    "Aortography": {"54640009": ("Aortography", [3405, 3428])},
    "AortopulmonaryReconstructionWithRightVentricleToPulmonaryArterialValvelessConduit": {
        "429616001": (
            "Aortopulmonary reconstruction with right ventricle to pulmonary arterial valveless conduit "
            "(procedure)",
            [],
        )
    },
    "AostaBlackPiedCattleBreed": {
        "133292005": ("Aosta Black Pied cattle breed", [7480])
    },
    "AostaChestnutCattleBreed": {"133293000": ("Aosta Chestnut cattle breed", [7480])},
    "AostaRedPiedCattleBreed": {"133294006": ("Aosta Red Pied cattle breed", [7480])},
    "ApennineSheepBreed": {"132811000": ("Apennine sheep breed", [7480])},
    "ApexOfHeart": {"13383001": ("Apex of heart", [1000, 1001, 1004, 8134])},
    "ApexOfLeftVentricle": {
        "128564006": (
            "Apex of left ventricle",
            [4, 3010, 3011, 3717, 4042, 7151, 7192, 8134, 9514, 12305],
        )
    },
    "ApexOfLung": {
        "86598002": (
            "Apex of Lung",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "ApexOfRightVentricle": {
        "128565007": (
            "Apex of right ventricle",
            [4, 3010, 3011, 4042, 7151, 7192, 8134, 9514, 12305],
        )
    },
    "Aphakia": {"24010005": ("Aphakia (disorder)", [])},
    "Aphakic": {"24010005": ("Aphakic", [4231])},
    "Apical": {"43674008": ("Apical", [2, 5, 6124, 6125])},
    "ApicalAnteriorFibromuscularStromaOfProstate": {
        "716925003": ("Apical anterior fibromuscular stroma of prostate", [6300, 6303])
    },
    "ApicalAxillaryLymphNode": {
        "16051009": ("apical axillary lymph node", [7600, 8134])
    },
    "ApicalFourChamber": {"399214001": ("Apical four chamber", [12226])},
    "ApicalFourChamberView": {
        "399214001": ("Apical four chamber view (qualifier value)", [])
    },
    "ApicalLongAxis": {"399339008": ("Apical long axis", [501, 12226])},
    "ApicalTwoChamber": {"399232001": ("Apical two chamber", [12226])},
    "ApicalTwoChamberView": {
        "399232001": ("Apical two chamber view (qualifier value)", [])
    },
    "ApocrineAdenocarcinoma": {"57141000": ("Apocrine adenocarcinoma", [6030, 6033])},
    "ApocrineMetaplasia": {"81274009": ("Apocrine Metaplasia", [6030, 6031])},
    "AppaloosaHorse": {"55167009": ("Appaloosa horse (organism)", [])},
    "AppaloosaHorseBreed": {"55167009": ("Appaloosa horse breed", [7480])},
    "AppendicealStump": {"441850003": ("Appendiceal stump", [6202, 6204, 8134])},
    "AppendicularLymphNode": {"46157003": ("appendicular lymph node", [7600, 8134])},
    "Appendix": {"66754008": ("Appendix", [6202, 6204, 8134])},
    "AppendixStructure": {"66754008": ("Appendix structure (body structure)", [])},
    "AppenzellGoatBreed": {"131616009": ("Appenzell goat breed", [7480])},
    "AppenzellerDogBreed": {"132538002": ("Appenzeller dog breed", [7480])},
    "Apple": {"422670003": ("Apple", [4012])},
    "AppleProjection": {"422670003": ("Apple projection (qualifier value)", [])},
    "ApproachViaTracheostomy": {
        "180640008": ("Approach via tracheostomy (qualifier value)", [])
    },
    "ApulianPigBreed": {"132097001": ("Apulian pig breed", [7480])},
    "ApulianPodolianCattleBreed": {
        "131444003": ("Apulian Podolian cattle breed", [7480])
    },
    "AraappaloosaHorseBreed": {"131862004": ("AraAppaloosa horse breed", [7480])},
    "ArabiSheepBreed": {"132810004": ("Arabi sheep breed", [7480])},
    "ArabianHorse": {"54098002": ("Arabian horse (organism)", [])},
    "ArabianHorseBreed": {"54098002": ("Arabian horse breed", [7480])},
    "AracenaCattleBreed": {"133295007": ("Aracena cattle breed", [7480])},
    "Arachnoid": {"75042008": ("Arachnoid", [7151, 7153, 7192, 8134, 9514])},
    "ArachnoidStructure": {"75042008": ("Arachnoid structure (body structure)", [])},
    "AradoCattleBreed": {"131445002": ("Arado cattle breed", [7480])},
    "ArapawaIslandGoatBreed": {"131609008": ("Arapawa Island goat breed", [7480])},
    "ArapawaIslandPigBreed": {"132221002": ("Arapawa Island pig breed", [7480])},
    "ArapawaIslandSheepBreed": {"132809009": ("Arapawa Island sheep breed", [7480])},
    "Arcelin": {"399218003": ("Arcelin", [4012])},
    "ArcelinProjection": {"399218003": ("Arcelin projection (qualifier value)", [])},
    "ArchOfVertebra": {"40265002": ("Arch of vertebra", [6115, 8134])},
    "ArchitecturalDistortionOfBreast": {
        "129792006": (
            "Architectural distortion of breast",
            [6014, 6015, 6016, 6054, 6056],
        )
    },
    "ArcuateArteryOfTheKidney": {
        "274231001": (
            "Arcuate Artery of the Kidney",
            [3827, 7151, 7192, 8134, 9514, 12103, 12115],
        )
    },
    "ArdennesHorseBreed": {"133116001": ("Ardennes horse breed", [7480])},
    "Area": {
        "42798000": (
            "Area",
            [
                218,
                7469,
                7471,
                8136,
                12253,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "AreaOfDefinedRegion": {
        "131184002": ("Area of defined region", [218, 7469, 7471, 12308])
    },
    "AreaOfInternalAuditoryCanal": {
        "361078006": ("Area of internal auditory canal (body structure)", [])
    },
    "ArgentineCriolloHorseBreed": {
        "131863009": ("Argentine Criollo horse breed", [7480])
    },
    "ArgentineDogoDogBreed": {"132392002": ("Argentine Dogo dog breed", [7480])},
    "ArgentineFriesianCattleBreed": {
        "133296008": ("Argentine Friesian cattle breed", [7480])
    },
    "ArgentinePoloPonyHorseBreed": {
        "131864003": ("Argentine Polo Pony horse breed", [7480])
    },
    "AriégeoisDogBreed": {"132539005": ("Ariégeois dog breed", [7480])},
    "AriégeoisPonyBreed": {"396488006": ("Ariégeois pony breed (organism)", [])},
    "AriégeoisPonyHorseBreed": {"396488006": ("Ariégeois pony horse breed", [7480])},
    "ArmErgometer": {"429560009": ("Arm ergometer", [3203])},
    "ArmErgometerDevice": {
        "429560009": ("Arm ergometer, device (physical object)", [])
    },
    "ArmantDogBreed": {"132467004": ("Armant dog breed", [7480])},
    "ArmoricanCattleBreed": {"133297004": ("Armorican cattle breed", [7480])},
    "Armrest": {"468115008": ("Backrest", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "ArouquesaCattleBreed": {"133298009": ("Arouquesa cattle breed", [7480])},
    "Arrhythmia": {"698247007": ("Arrhythmia", [3700])},
    "ArsiCattleBreed": {"133631006": ("Arsi cattle breed", [7480])},
    "ArterialAneurysm": {"233981004": ("Arterial aneurysm (disorder)", [])},
    "ArterialBloodStasis": {"408707005": ("Arterial Blood Stasis", [3494])},
    "ArterialDissection": {"710864009": ("Arterial dissection", [3413, 3754, 3810])},
    "ArterialGraftToCitedSegment": {
        "128950003": ("Arterial graft to cited segment (substance)", [])
    },
    "ArterialPressureWaveform": {"128446002": ("Arterial pressure waveform", [3003])},
    "ArterialStasis": {"408707005": ("Arterial stasis (finding)", [])},
    "ArterialStructure": {"51114001": ("Arterial structure (body structure)", [])},
    "ArterialSwitchOperation": {"174826008": ("Arterial switch operation", [12247])},
    "ArterialTrueAneurysm": {"233981004": ("Arterial (True) Aneurysm", [3491])},
    "ArterialVelocity": {"252064005": ("Arterial velocity (observable entity)", [])},
    "Arteriolosclerosis": {"17941002": ("arteriolosclerosis", [3817])},
    "ArteriolosclerosisWithFibrinoidNecrosis": {
        "32651000": (
            "Arteriolosclerosis with fibrinoid necrosis (morphologic abnormality)",
            [],
        )
    },
    "Arteriosclerosis": {"28960008": ("arteriosclerosis", [3817])},
    "ArteriosclerosisWithFibrinoidNecrosis": {
        "32651000": ("arteriosclerosis with fibrinoid necrosis", [3817])
    },
    "ArterioscleroticVascularDisease": {
        "72092001": ("arteriosclerotic vascular disease", [3805])
    },
    "ArteriovenousFistula": {"128617001": ("arteriovenous fistula", [3810])},
    "ArteriovenousFistulaDisorder": {"439470001": ("Arteriovenous Fistula", [12293])},
    "ArteriovenousFistulaOfPulmonaryVessels": {
        "111289009": ("Arteriovenous fistula of pulmonary vessels (disorder)", [])
    },
    "Artery": {
        "51114001": (
            "Artery",
            [3010, 3604, 3606, 3630, 4042, 7151, 7166, 7191, 7192, 8134, 9514],
        )
    },
    "ArtificialLensPresent": {"309523001": ("Pseudophakia", [4231])},
    "ArtificialMenopauseState": {"31351009": ("Artificial menopause state", [6086])},
    "ArtificialRespiration": {"40617009": ("Artificial Respiration", [617, 618])},
    "AsRequired": {"225761000": ("As required", [6091])},
    "Ascending": {"79458005": ("Ascending", [5])},
    "AscendingAorta": {
        "54247002": (
            "Ascending aorta",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12291,
                12305,
            ],
        )
    },
    "AscendingAortaStructure": {
        "54247002": ("Ascending aorta structure (body structure)", [])
    },
    "AscendingColon": {"9040008": ("Ascending colon", [6210, 8134])},
    "AscendingColonStructure": {
        "9040008": ("Ascending colon structure (body structure)", [])
    },
    "AsciticFluid": {"409615008": ("Ascitic fluid", [645, 8134])},
    "AscorbicAcid": {"43706004": ("Ascorbic acid", [4410])},
    "AshantiDwarfPigBreed": {"133178006": ("Ashanti Dwarf pig breed", [7480])},
    "AsianOrPacificIslander": {
        "413581001": ("Asian or Pacific islander (racial group)", [])
    },
    "AsianOrPacificIslanderRace": {
        "413581001": ("Asian or Pacific Islander race", [6099])
    },
    "AsianRace": {"413582008": ("Asian race", [6099])},
    "Asleep": {"248220008": ("Asleep", [3050])},
    "Aspirate": {"119295008": ("Aspirate", [8103])},
    "Aspiration": {"14766002": ("Aspiration", [8109])},
    "AssessmentScales": {"273249006": ("Assessment scales (assessment scale)", [])},
    "AssociatedFinding": {"246090004": ("Associated finding (attribute)", [])},
    "AssociatedMorphology": {"116676008": ("Associated morphology (attribute)", [])},
    "AssociatedProcedure": {"363589002": ("Associated procedure (attribute)", [])},
    "AssociatedWith": {"47429007": ("Associated with (attribute)", [])},
    "Asthenia": {"13791008": ("Asthenia (debility)", [3205])},
    "Asthma": {"195967001": ("Asthma", [64])},
    "AstringentDrug": {"60533005": ("Astringent drug", [10])},
    "Astrocytoma": {"254938000": ("Astrocytoma", [638, 639])},
    "AstrocytomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "38713004": (
            "Astrocytoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "AstrocytomaOfBrain": {"254938000": ("Astrocytoma of brain (disorder)", [])},
    "AsturianMountainCattleBreed": {
        "131563004": ("Asturian Mountain cattle breed", [7480])
    },
    "AsturianTarpanHorseXDomesticHorseBreed": {
        "132992005": ("Asturian tarpan horse X domestic horse breed", [7480])
    },
    "AsturianValleyCattleBreed": {
        "131564005": ("Asturian Valley cattle breed", [7480])
    },
    "AsymmetricBreastTissue": {
        "129790003": ("Asymmetric breast tissue", [6016, 6017, 6054])
    },
    "AsymmetricBreastTissueAnalysis": {
        "133889002": ("Asymmetric breast tissue analysis", [6043])
    },
    "AsymmetricBreastTissueFinding": {
        "129790003": ("Asymmetric breast tissue finding (finding)", [])
    },
    "AsynchronousInvolutionOfBreast": {
        "130963002": ("Asynchronous involution of breast", [6030, 6031])
    },
    "Ataxia": {"20262006": ("Ataxia (finding)", [])},
    "AtaxiaOrIncoordination": {"20262006": ("Ataxia or incoordination", [3205])},
    "AtebrinFSStain": {"84656005": ("atebrin FS stain", [8112])},
    "Atherectomy": {"6832004": ("Atherectomy", [9, 3405])},
    "AtherectomyByLaser": {"76611008": ("Atherectomy by laser", [9, 3405])},
    "AtherectomyByRotaryCutter": {
        "65659003": ("Atherectomy by rotary cutter", [9, 3405])
    },
    "AtherectomyDevice": {"102312002": ("Atherectomy device", [8, 4051, 7151, 7193])},
    "Atheroma": {"48434008": ("atheroma", [3817])},
    "AtheroscleroticFibrousPlaque": {
        "20717008": ("atherosclerotic fibrous plaque", [3817])
    },
    "AtlantalAxialJoint": {"62555009": ("Atlantal-axial joint", [7482, 7483, 8134])},
    "AtlantoOccipitalJoint": {
        "20292002": ("Atlanto-occipital joint", [7482, 7483, 8134])
    },
    "AtlantooccipitalJointStructure": {
        "20292002": ("Atlantooccipital joint structure (body structure)", [])
    },
    "AtopicDermatitis": {"24079001": ("Atopic dermatitis", [4405])},
    "AtpadiMahalCattleBreed": {"133632004": ("Atpadi Mahal cattle breed", [7480])},
    "AtrialArrhythmia": {"17366009": ("atrial arrhythmia", [3826])},
    "AtrialEffectiveRefractoryPeriod": {
        "129087001": ("Atrial effective refractory period (observable entity)", [])
    },
    "AtrialEffectiveRefractoryPeriodEvaluationOf": {
        "129087001": ("Atrial Effective Refractory Period, evaluation of", [3254])
    },
    "AtrialFibrillation": {"49436004": ("Atrial Fibrillation", [3230, 3700])},
    "AtrialInversionOperationUsingAtrialWall": {
        "174822005": ("Atrial inversion operation using atrial wall", [12247])
    },
    "AtrialPacing": {"69158002": ("atrial pacing", [3555])},
    "AtrialPrematureContraction": {
        "284470004": ("Atrial premature contraction", [3230])
    },
    "AtrialSeptalDefect": {
        "70142008": (
            "Atrial Septal defect",
            [
                3610,
                3630,
                3700,
                3728,
                6102,
                6109,
                6113,
                6148,
                7151,
                7192,
                9514,
                12236,
                12244,
                12248,
                12249,
                12284,
            ],
        )
    },
    "AtrialSeptalDefectRepair": {"112811009": ("Atrial septal defect repair", [12247])},
    "AtrialSeptalDefectWithEndocardialCushionDefectPartial": {
        "60732002": (
            "Atrial septal defect with endocardial cushion defect, partial",
            [12248, 12249],
        )
    },
    "AtrialSeptalDefectWithEndocardialCushionDefectPartialType": {
        "60732002": (
            "Atrial septal defect with endocardial cushion defect, partial type (disorder)",
            [],
        )
    },
    "AtrialStructure": {"59652004": ("Atrial structure (body structure)", [])},
    "AtrialSystole": {"59972007": ("Atrial Systole", [12233])},
    "AtrialSystoleAWave": {"59972007": ("Atrial Systole (A-wave)", [12307])},
    "AtrialSystoleFunction": {
        "59972007": ("Atrial systole, function (observable entity)", [])
    },
    "AtrialVein": {"195164009": ("Atrial Vein", [3827, 3839, 7151, 7192, 9514])},
    "AtrioventricularBundle": {"345000": ("Atrioventricular bundle", [3011, 8134])},
    "AtrioventricularBundleStructure": {
        "345000": ("Atrioventricular bundle structure (body structure)", [])
    },
    "AtrioventricularNode": {"25943004": ("Atrioventricular node", [3011, 8134])},
    "AtrioventricularSequentialPacing": {
        "371910005": ("Atrioventricular sequential pacing (procedure)", [])
    },
    "AtrioventricularVein": {
        "195496005": ("Atrioventricular Vein", [3827, 3839, 7151, 7192, 9514])
    },
    "Atrium": {"59652004": ("Atrium", [4, 7151, 7192, 8134, 9514])},
    "Atrophy": {"13331008": ("Atrophy", [6335, 6336, 6337, 6338])},
    "Atropine": {"73949004": ("Atropine", [10, 3204])},
    "AtropineInOcularDoseForm": {"771928002": ("Atropine", [4208])},
    "Attending": {"405279007": ("Attending", [7450, 7452])},
    "AttendingPhysician": {"405279007": ("Attending physician (occupation)", [])},
    "AtypicalAngina": {"371807002": ("Atypical Angina", [3202, 3220])},
    "AtypicalAnginaVariantAngina": {
        "87343002": ("Atypical Angina, Variant Angina", [3700])
    },
    "AtypicalChestPain": {"102589003": ("Atypical chest pain", [3700])},
    "AtypicalIntraductalHyperplasia": {
        "6660000": ("Atypical intraductal hyperplasia", [6030, 6032])
    },
    "AtypicalLobularHyperplasia": {
        "33889003": ("Atypical lobular hyperplasia", [6030, 6032])
    },
    "AtypicalMoleSyndrome": {"254819008": ("Atypical mole syndrome", [4405])},
    "AubracCattleBreed": {"131565006": ("Aubrac cattle breed", [7480])},
    "Audiology": {"309914005": ("Audiology", [7030])},
    "AudiologyDepartment": {"309914005": ("Audiology department (environment)", [])},
    "AuditoryRadiation": {"410726006": ("auditory radiation", [7702, 7710, 8134])},
    "AugmentationMammoplasty": {"22890008": ("Augmentation mammoplasty", [6051])},
    "AulieAtaCattleBreed": {"131566007": ("Aulie-Ata cattle breed", [7480])},
    "Aunt": {"25211005": ("Aunt", [7450, 7451])},
    "AuramineGStain": {"73251007": ("auramine G stain", [8112])},
    "AuramineOStain": {"81397005": ("auramine O stain", [8112])},
    "AuramineStain": {"255794000": ("auramine stain", [8112])},
    "AureEtSaintGironsCattleBreed": {
        "133299001": ("Aure et Saint-Girons cattle breed", [7480])
    },
    "AustralianAborigine": {"413600007": ("Australian aborigine (racial group)", [])},
    "AustralianAborigineRace": {"413600007": ("Australian aborigine race", [6099])},
    "AustralianBrafordCattleBreed": {
        "133704002": ("Australian Braford cattle breed (organism)", [])
    },
    "AustralianBrafordXZebuCattleBreed": {
        "133704002": ("Australian Braford X zebu cattle breed", [7480])
    },
    "AustralianBrumbyHorseBreed": {
        "133131005": ("Australian Brumby horse breed", [7480])
    },
    "AustralianCattleDog": {"11746005": ("Australian cattle dog", [7480])},
    "AustralianFriesianSahiwalCattleBreed": {
        "133715007": ("Australian Friesian Sahiwal cattle breed (organism)", [])
    },
    "AustralianFriesianSahiwalXZebuCattleBreed": {
        "133715007": ("Australian Friesian Sahiwal X zebu cattle breed", [7480])
    },
    "AustralianGoatBreed": {"131608000": ("Australian goat breed", [7480])},
    "AustralianGreyhoundDogBreed": {
        "132468009": ("Australian Greyhound dog breed", [7480])
    },
    "AustralianKelpieDogBreed": {"132541006": ("Australian Kelpie dog breed", [7480])},
    "AustralianLowlineCattleBreed": {
        "131567003": ("Australian Lowline cattle breed", [7480])
    },
    "AustralianMilkingZebuCattleBreed": {
        "133748007": ("Australian Milking Zebu cattle breed (organism)", [])
    },
    "AustralianMilkingZebuXZebuCattleBreed": {
        "133748007": ("Australian Milking Zebu X zebu cattle breed", [7480])
    },
    "AustralianMistCatBreed": {"132660007": ("Australian Mist cat breed", [7480])},
    "AustralianPonyHorseBreed": {"131865002": ("Australian Pony horse breed", [7480])},
    "AustralianShepherdDogBreed": {
        "132521001": ("Australian Shepherd dog breed", [7480])
    },
    "AustralianShorthornCattleBreed": {
        "133472001": ("Australian Shorthorn cattle breed", [7480])
    },
    "AustralianStockHorseHorseBreed": {
        "133132003": ("Australian Stock Horse horse breed", [7480])
    },
    "AustralianTerrier": {"112491001": ("Australian terrier", [7480])},
    "AustralianTerrierRoughCoatedDogBreed": {
        "132469001": ("Australian Terrier, rough-coated dog breed", [7480])
    },
    "AustralianTerrierSilkyDogBreed": {
        "132470000": ("Australian Terrier, silky dog breed", [7480])
    },
    "AustralianWhiteCattleBreed": {
        "133300009": ("Australian White cattle breed", [7480])
    },
    "AustrianHoundDogBreed": {"132471001": ("Austrian Hound dog breed", [7480])},
    "AustrianPinscherDogBreed": {"132408006": ("Austrian Pinscher dog breed", [7480])},
    "AustrianSimmentalCattleBreed": {
        "133301008": ("Austrian Simmental cattle breed", [7480])
    },
    "AustrianSmoothHairedBrackeDogBreed": {
        "132472008": ("Austrian Smooth-Haired Bracke dog breed", [7480])
    },
    "AustrianYellowCattleBreed": {
        "133302001": ("Austrian Yellow cattle breed", [7480])
    },
    "Automated": {"8359006": ("Automated", [7230])},
    "AutomaticImplantableCardiacDefibrillatorInSitu": {
        "443325000": (
            "Automatic implantable cardiac defibrillator in situ (finding)",
            [],
        )
    },
    "AutonomicNerve": {"53520000": ("Autonomic nerve", [7151, 7167, 7192, 8134, 9514])},
    "AutonomicNerveStructure": {
        "53520000": ("Autonomic nerve structure (body structure)", [])
    },
    "AutonomousBreathing": {"45804006": ("autonomous breathing", [3823, 9272])},
    "AutonomousBreathingFunction": {
        "45804006": ("Autonomous breathing, function (observable entity)", [])
    },
    "AuxoisHorseBreed": {"131866001": ("Auxois horse breed", [7480])},
    "AveligneseHorseBreed": {"131867005": ("Avelignese horse breed", [7480])},
    "AverageDiastolicBloodPressure": {
        "314453003": ("Average diastolic blood pressure", [3337, 3611])
    },
    "AverageSystolicBloodPressure": {
        "314440001": ("Average systolic blood pressure", [3337, 3611])
    },
    "Averaged": {"373098007": ("Averaged", [3676])},
    "AveragedHemodynamicMeasurementMethod": {
        "128580001": ("Averaged hemodynamic measurement method", [3241])
    },
    "AveragedHemodynamicMeasurementMethodRegimeTherapy": {
        "128580001": ("Averaged hemodynamic measurement method (regime/therapy)", [])
    },
    "AvetonouCattleBreed": {"133303006": ("Avetonou cattle breed", [7480])},
    "AvilenaBlackIberianCattleBreed": {
        "133305004": ("Avilena-Black Iberian cattle breed", [7480])
    },
    "AvilenaCattleBreed": {"133304000": ("Avilena cattle breed", [7480])},
    "Awake": {"248218005": ("Awake", [3050])},
    "AwassiSheepBreed": {"132808001": ("Awassi sheep breed", [7480])},
    "AweilDinkaCattleBreed": {"131446001": ("Aweil Dinka cattle breed", [7480])},
    "Axial": {"24422004": ("Axial", [2, 26, 501, 6312])},
    "AxialProjection": {"399061009": ("axial", [501, 4010])},
    "Axilla": {"91470000": ("Axilla", [4, 7151, 7192, 8134, 9514])},
    "AxillaPositionOnMammogram": {
        "129791004": ("Axilla position on mammogram (finding)", [])
    },
    "AxillaRegion": {"129791004": ("Axilla region", [6018, 6019])},
    "AxillaryAdenopathy": {
        "127189005": ("Axillary adenopathy", [6014, 6015, 6016, 6054, 6056])
    },
    "AxillaryArtery": {
        "67937003": (
            "Axillary Artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12107,
            ],
        )
    },
    "AxillaryFascia": {
        "368536000": (
            "Axillary Fascia",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "AxillaryLymphNode": {
        "68171009": ("Axillary lymph node", [6014, 6015, 6016, 6054, 6056, 7600, 8134])
    },
    "AxillaryLymphNodeStructure": {
        "68171009": ("Axillary lymph node structure (body structure)", [])
    },
    "AxillaryLymphadenopathy": {
        "127189005": ("Axillary lymphadenopathy (disorder)", [])
    },
    "AxillaryRegionStructure": {
        "91470000": ("Axillary region structure (body structure)", [])
    },
    "AxillaryTail": {"399011000": ("Axillary Tail", [4015])},
    "AxillaryTailMammographyView": {
        "399011000": ("Axillary tail mammography view (qualifier value)", [])
    },
    "AxillaryTailPositionOnMammogram": {
        "129785001": ("Axillary tail position on mammogram (finding)", [])
    },
    "AxillaryTailRegion": {"129785001": ("Axillary tail region", [6018, 6019])},
    "AxillaryTissue": {"442580003": ("Axillary tissue", [4015])},
    "AxillaryTissueMammographyView": {
        "442580003": ("Axillary tissue mammography view (qualifier value)", [])
    },
    "AxillaryVein": {
        "68705008": (
            "Axillary vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12108,
            ],
        )
    },
    "AxillaryVeinLymphNode": {"421624008": ("axillary vein lymph node", [7600, 8134])},
    "AxisOfCylinder": {"251799001": ("Axis of cylinder (observable entity)", [])},
    "AyrshireCattleBreed": {"8989009": ("Ayrshire cattle breed (organism)", [])},
    "AyrshireCowBreed": {"8989009": ("Ayrshire cow breed", [7480])},
    "AzaouakCattleBreed": {"133633009": ("Azaouak cattle breed", [7480])},
    "Azaperone": {"96229001": ("Azaperone", [623, 625])},
    "AzawakhDogBreed": {"132520000": ("Azawakh dog breed", [7480])},
    "AzerbaijanHorseBreed": {"131868000": ("Azerbaijan horse breed", [7480])},
    "AzerbaijanZebuCattleBreed": {
        "133634003": ("Azerbaijan Zebu cattle breed", [7480])
    },
    "AzoBlackStain": {"85066006": ("azo black stain", [8112])},
    "AzocarmineGGXStain": {"76048000": ("azocarmine G (GX) stain", [8112])},
    "AzophloxinStain": {"35609001": ("azophloxin stain", [8112])},
    "AzoresHorseBreed": {"131869008": ("Azores horse breed", [7480])},
    "AzorubinSStain": {"2159007": ("azorubin S stain", [8112])},
    "AztecaHorseBreed": {"133118000": ("Azteca horse breed", [7480])},
    "AzureAStain": {"16836001": ("azure A stain", [8112])},
    "AzureBStain": {"8926000": ("azure B stain", [8112])},
    "AzureCStain": {"11069001": ("azure C stain", [8112])},
    "AzygosVein": {
        "72107004": (
            "Azygos vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                3839,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
            ],
        )
    },
    "BBShotDevice": {"102378009": ("BB shot, device (qualifier value)", [])},
    "BBShotLeadPellet": {
        "102378009": ("BB shot (Lead Pellet)", [6040, 6401, 7111, 7112, 7151, 7193])
    },
    "BKB1PigBreed": {"132277000": ("BKB-1 pig breed", [7480])},
    "BMode": {"399009009": ("B mode", [6058, 12224])},
    "BModeUltrasound": {"399009009": ("B mode ultrasound (qualifier value)", [])},
    "BSIPigBreed": {"132342003": ("BSI pig breed", [7480])},
    "BaXuyenPigBreed": {"132220001": ("Ba Xuyen pig breed", [7480])},
    "BachaurCattleBreed": {"133595008": ("Bachaur cattle breed", [7480])},
    "Back": {"77568009": ("Back", [4, 7151, 7192, 8134, 9514])},
    "BackStructureExcludingNeck": {
        "77568009": ("Back structure, excluding neck (body structure)", [])
    },
    "Backrest": {"20406008": ("Backrest", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "BadenWurttemburgHorseBreed": {
        "132731005": ("Baden Wurttemburg horse breed", [7480])
    },
    "Baffle": {
        "128981007": (
            "Baffle",
            [4, 3010, 3604, 3606, 3608, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "BaffleDevice": {"128981007": ("Baffle, device (physical object)", [])},
    "BaggaraCattleBreed": {"133635002": ("Baggara cattle breed", [7480])},
    "BagotGoatBreed": {"131620008": ("Bagot goat breed", [7480])},
    "BahiaPigBreed": {"132162000": ("Bahia pig breed", [7480])},
    "BahimaCattleBreed": {"131447005": ("Bahima cattle breed", [7480])},
    "BakosiCattleBreed": {"133306003": ("Bakosi cattle breed", [7480])},
    "BakosiPigBreed": {"133187002": ("Bakosi pig breed", [7480])},
    "BakwiriCattleBreed": {"133307007": ("Bakwiri cattle breed", [7480])},
    "BaladiCattleBreed": {"131429004": ("Baladi cattle breed (organism)", [])},
    "BaladicattleCattleBreed": {"131429004": ("Baladicattle cattle breed", [7480])},
    "BalancedCoronaryDominance": {"253730009": ("Balanced Coronary Dominance", [3710])},
    "BalancedCoronarySystem": {
        "253730009": ("Balanced coronary system (disorder)", [])
    },
    "BaldingerSpottedPigBreed": {"132356001": ("Baldinger Spotted pig breed", [7480])},
    "BalearicHorseBreed": {"133140009": ("Balearic horse breed", [7480])},
    "BaliHorseBreed": {"131870009": ("Bali horse breed", [7480])},
    "BalikunHorseBreed": {"131871008": ("Balikun horse breed", [7480])},
    "BalineseCat": {"3653002": ("Balinese cat", [7480])},
    "BalinesePigBreed": {"133175009": ("Balinese pig breed", [7480])},
    "BalkanHoundDogBreed": {"132473003": ("Balkan Hound dog breed", [7480])},
    "BalkeProtocol": {"129097005": ("Balke protocol", [3261, 12001])},
    "BalkhiSheepBreed": {"132804004": ("Balkhi sheep breed", [7480])},
    "BaltaretPigBreed": {"132198008": ("Baltaret pig breed", [7480])},
    "BalticBlackPiedCattleBreed": {
        "133308002": ("Baltic Black Pied cattle breed", [7480])
    },
    "BaluchiHorseBreed": {"133139007": ("Baluchi horse breed", [7480])},
    "BaluchiSheepBreed": {"132861009": ("Baluchi sheep breed", [7480])},
    "BalwenWelshMountainSheepBreed": {
        "132813002": ("Balwen Welsh Mountain sheep breed", [7480])
    },
    "BamaxiangPigBreed": {"132315004": ("Bamaxiang pig breed", [7480])},
    "BambaraCattleBreed": {"133752007": ("Bambara cattle breed (organism)", [])},
    "BambaraXZebuCattleBreed": {"133752007": ("Bambara X zebu cattle breed", [7480])},
    "BambawaCattleBreed": {"133636001": ("Bambawa cattle breed", [7480])},
    "BambeyCattleBreed": {"133763005": ("Bambey cattle breed (organism)", [])},
    "BambeyXZebuCattleBreed": {"133763005": ("Bambey X zebu cattle breed", [7480])},
    "BameiPigBreed": {"132132006": ("Bamei pig breed", [7480])},
    "BamiCattleBreed": {"133637005": ("Bami cattle breed", [7480])},
    "BanEiHorseBreed": {"133137009": ("Ban-ei horse breed", [7480])},
    "BanatWhitePigBreed": {"133197006": ("Banat White pig breed", [7480])},
    "BanjaraGreyhoundDogBreed": {"132474009": ("Banjara greyhound dog breed", [7480])},
    "BankerHorseHorseBreed": {"131873006": ("Banker Horse horse breed", [7480])},
    "BantuPigBreed": {"132227003": ("Bantu pig breed", [7480])},
    "BanyoCattleBreed": {"133638000": ("Banyo cattle breed", [7480])},
    "BaouleCattleBreed": {"133309005": ("Baoule cattle breed", [7480])},
    "BapediCattleBreed": {"131448000": ("Bapedi cattle breed", [7480])},
    "BarbHorseBreed": {"133136000": ("Barb horse breed", [7480])},
    "BarbadoSheepBreed": {"132860005": ("Barbado sheep breed", [7480])},
    "BarbadosBlackbellySheepBreed": {
        "132806002": ("Barbados Blackbelly sheep breed", [7480])
    },
    "BarbadosSheep": {"25660007": ("Barbados sheep (organism)", [])},
    "BarbadosSheepBreed": {"25660007": ("Barbados sheep breed", [7480])},
    "BarbariGoatBreed": {"131645006": ("Barbari goat breed", [7480])},
    "BarbetDogBreed": {"132533006": ("Barbet dog breed", [7480])},
    "Barbiturate": {"372798009": ("Barbiturate", [621, 622])},
    "BardigianoHorseBreed": {"131874000": ("Bardigiano horse breed", [7480])},
    "BargurCattleBreed": {"133639008": ("Bargur cattle breed", [7480])},
    "BariCattleBreed": {"133640005": ("Bari cattle breed", [7480])},
    "BariaVietnamMadagascarCattleBreed": {
        "131449008": ("Baria (Vietnam/Madagascar) cattle breed", [7480])
    },
    "Barium": {"39290007": ("Barium", [13, 300])},
    "BariumSulfate": {"25419009": ("Barium Sulfate", [12, 3850])},
    "BarkaCattleBreed": {"133596009": ("Barka cattle breed (organism)", [])},
    "BarkaZebuCattleBreed": {"133596009": ("Barka zebu cattle breed", [7480])},
    "BarotseCattleBreed": {"131450008": ("Barotse cattle breed", [7480])},
    "BarraDoCuanzoCattleBreed": {"131451007": ("Barra do Cuanzo cattle breed", [7480])},
    "BarrosaCattleBreed": {"133310000": ("Barrosa cattle breed", [7480])},
    "BarrosoCattleBreed": {"133311001": ("Barroso cattle breed", [7480])},
    "BarzonaCattleBreed": {"131568008": ("Barzona cattle breed", [7480])},
    "Basal": {"57195005": ("Basal", [2, 6124, 6125])},
    "BasalAnteriorFibromuscularStromaOfProstate": {
        "716891004": ("Basal anterior fibromuscular stroma of prostate", [6300, 6303])
    },
    "BaseOfTongue": {"47975008": ("base of tongue", [7601, 8134])},
    "BaselineState": {"128974000": ("Baseline state", [3262, 3602])},
    "Basenji": {"47659007": ("Basenji", [7480])},
    "BashiCattleBreed": {"131452000": ("Bashi cattle breed", [7480])},
    "BashkirCurlyHorseBreed": {"133134002": ("Bashkir Curly horse breed", [7480])},
    "BashkirHorseBreed": {"133135001": ("Bashkir horse breed", [7480])},
    "BasilarArtery": {
        "59011009": (
            "Basilar artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12106,
            ],
        )
    },
    "BasilicVein": {
        "19715009": (
            "Basilic vein",
            [3607, 3630, 3827, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "BasqueBlackPiedPigBreed": {"132122004": ("Basque Black Pied pig breed", [7480])},
    "BassetArtésianNormandDogBreed": {
        "132517008": ("Basset Artésian Normand dog breed", [7480])
    },
    "BassetBleuDeGascogneDogBreed": {
        "132414004": ("Basset Bleu de Gascogne dog breed", [7480])
    },
    "BassetFauveDeBretagneDogBreed": {
        "132397008": ("Basset Fauve de Bretagne dog breed", [7480])
    },
    "BassetGriffonVendéenDogBreed": {
        "132412000": ("Basset Griffon Vendéen dog breed", [7480])
    },
    "BassetHound": {"41320000": ("Basset hound", [7480])},
    "BasutoCattleBreed": {"131453005": ("Basuto cattle breed", [7480])},
    "BasutoPonyHorseBreed": {"133133008": ("Basuto Pony horse breed", [7480])},
    "BatakHorseBreed": {"131875004": ("Batak horse breed", [7480])},
    "BatanesBlackCattleBreed": {
        "133764004": ("Batanes Black cattle breed (organism)", [])
    },
    "BatanesBlackXZebuCattleBreed": {
        "133764004": ("Batanes Black X zebu cattle breed", [7480])
    },
    "BatangasCattleBreed": {"131454004": ("Batangas cattle breed", [7480])},
    "BatardDogBreed": {"132413005": ("Batard dog breed", [7480])},
    "BauerChromicAcidLeucofuchsinStain": {
        "406982003": ("bauer's chromic acid leucofuchsin stain", [8112])
    },
    "BavarianForestSheepBreed": {"132805003": ("Bavarian Forest sheep breed", [7480])},
    "BavarianMountainHoundDogBreed": {
        "132518003": ("Bavarian Mountain Hound dog breed", [7480])
    },
    "BavarianWarmbloodHorseBreed": {
        "131876003": ("Bavarian Warmblood horse breed", [7480])
    },
    "BavendaCattleBreed": {"131455003": ("Bavenda cattle breed", [7480])},
    "BazadaisCattleBreed": {"131569000": ("Bazadais cattle breed", [7480])},
    "BaznaPigBreed": {"133198001": ("Bazna pig breed", [7480])},
    "Beagle": {"44696006": ("Beagle", [7480])},
    "BeagleElizabethanDogBreed": {
        "132529004": ("Beagle, Elizabethan dog breed", [7480])
    },
    "BeagleHarrierDogBreed": {"132401000": ("Beagle Harrier dog breed", [7480])},
    "BeagleRoughDogBreed": {"132381003": ("Beagle, Rough dog breed", [7480])},
    "BeagleSmoothDogBreed": {"132380002": ("Beagle, Smooth dog breed", [7480])},
    "BeagleStandardDogBreed": {"132475005": ("Beagle, Standard dog breed", [7480])},
    "BeardedCollie": {"75911001": ("Bearded collie (organism)", [])},
    "BeardedCollieDogBreed": {"75911001": ("Bearded collie dog breed", [7480])},
    "BearnaisCattleBreed": {"133312008": ("Bearnais cattle breed", [7480])},
    "BeauceronDogBreed": {"132519006": ("Beauceron dog breed", [7480])},
    "Beclere": {"399263002": ("Beclere", [4012])},
    "BeclereProjection": {"399263002": ("Beclere projection (qualifier value)", [])},
    "BedlingtonTerrier": {"1514007": ("Bedlington terrier", [7480])},
    "BeefShorthornCattleBreed": {"133313003": ("Beef shorthorn cattle breed", [7480])},
    "BeefSyntheticCattleBreed": {"133314009": ("Beef synthetic cattle breed", [7480])},
    "BeefmakerCattleBreed": {"131570004": ("Beefmaker cattle breed", [7480])},
    "BeetalGoatBreed": {"131659004": ("Beetal goat breed", [7480])},
    "Before": {"272113006": ("Before", [3600])},
    "BeforeMenopause": {"309606002": ("Before menopause", [6086])},
    "BeforeProcedure": {"307153007": ("Before Procedure", [631, 12002, 12102])},
    "BeforeValues": {"272113006": ("Before values (qualifier value)", [])},
    "BeijingBlackPiedCattleBreed": {
        "133315005": ("Beijing Black Pied cattle breed", [7480])
    },
    "BeijingBlackPigBreed": {"132143003": ("Beijing Black pig breed", [7480])},
    "BeiroaCattleBreed": {"133316006": ("Beiroa cattle breed", [7480])},
    "BejaCattleBreed": {"131456002": ("Beja cattle breed", [7480])},
    "BelarusRedCattleBreed": {"131571000": ("Belarus Red cattle breed", [7480])},
    "BelgianArdennaisHorseBreed": {
        "131877007": ("Belgian Ardennais horse breed", [7480])
    },
    "BelgianBlackPiedHolsteinCattleBreed": {
        "133317002": ("Belgian Black Pied Holstein cattle breed", [7480])
    },
    "BelgianBlackPiedHolsteincattleBreed": {
        "133317002": ("Belgian Black Pied Holsteincattle breed (organism)", [])
    },
    "BelgianBlueCattleBreed": {"131572007": ("Belgian Blue cattle breed", [7480])},
    "BelgianGriffonDogBreed": {"132527002": ("Belgian Griffon dog breed", [7480])},
    "BelgianGriffonRoughDogBreed": {
        "132382005": ("Belgian Griffon, Rough dog breed", [7480])
    },
    "BelgianGriffonSmoothDogBreed": {
        "132383000": ("Belgian Griffon, Smooth dog breed", [7480])
    },
    "BelgianGroenendaelDog": {"74536009": ("Belgian groenendael dog", [7480])},
    "BelgianHalfbloodHorseBreed": {
        "131878002": ("Belgian Halfblood horse breed", [7480])
    },
    "BelgianHorse": {"22720009": ("Belgian horse (organism)", [])},
    "BelgianHorseBreed": {"22720009": ("Belgian horse breed", [7480])},
    "BelgianLaekenDog": {"76554006": ("Belgian laeken dog", [7480])},
    "BelgianLandraceBNPigBreed": {
        "132087005": ("Belgian Landrace, BN pig breed", [7480])
    },
    "BelgianMalinoisDog": {"37116003": ("Belgian malinois dog", [7480])},
    "BelgianRedCattleBreed": {"131573002": ("Belgian Red cattle breed", [7480])},
    "BelgianRedPiedCattleBreed": {
        "133318007": ("Belgian Red Pied cattle breed", [7480])
    },
    "BelgianSheepdog": {"85144002": ("Belgian sheepdog", [7480])},
    "BelgianStreetDogBreed": {"132385007": ("Belgian Street Dog breed", [7480])},
    "BelgianTervurenDog": {"27444002": ("Belgian tervuren dog", [7480])},
    "BelgianWarmbloodHorseBreed": {
        "131879005": ("Belgian Warmblood horse breed", [7480])
    },
    "BelgianWhiteAndRedCattleBreed": {
        "133319004": ("Belgian White and Red cattle breed", [7480])
    },
    "BelgianWolfhoundDogBreed": {"132522008": ("Belgian Wolfhound dog breed", [7480])},
    "BelgiumLandracePig": {"10261003": ("Belgium landrace pig (organism)", [])},
    "BelgiumLandracePigBreed": {"10261003": ("Belgium landrace pig breed", [7480])},
    "BelmontAdaptaurCattleBreed": {
        "131574008": ("Belmont Adaptaur cattle breed", [7480])
    },
    "BelmontRedCattleBreed": {"131430009": ("Belmont Red cattle breed", [7480])},
    "BelorusBlackPiedPigBreed": {"132278005": ("Belorus Black Pied pig breed", [7480])},
    "BeltedGallowayCattleBreed": {
        "13544004": ("Belted Galloway cattle breed (organism)", [])
    },
    "BeltedGallowayCowBreed": {"13544004": ("Belted Galloway cow breed", [7480])},
    "BeltedWelshCattleBreed": {"133320005": ("Belted Welsh cattle breed", [7480])},
    "BeltsvillePig": {"48394005": ("Beltsville pig (organism)", [])},
    "BeltsvillePigBreed": {"48394005": ("Beltsville pig breed", [7480])},
    "BenadirGoatBreed": {"131657002": ("Benadir goat breed", [7480])},
    "BengalCatBreed": {"132661006": ("Bengal cat breed", [7480])},
    "BengaliCattleBreed": {"133597000": ("Bengali cattle breed", [7480])},
    "Benign": {"30807003": ("Benign", [6063])},
    "BenignLocalizedHyperplasiaOfProstate": {
        "444808002": ("Benign localized hyperplasia of prostate (disorder)", [])
    },
    "BenignNeoplasmOfNippleOfFemaleBreast": {
        "92248004": ("Benign neoplasm of nipple of female breast (disorder)", [])
    },
    "BenignNeoplasmOfNippleOfFemaleBreastNippleAdenoma": {
        "92248004": (
            "Benign neoplasm of nipple of female breast (Nipple adenoma)",
            [6030, 6031],
        )
    },
    "BenignProstateHyperplasia": {
        "444808002": ("Benign Prostate Hyperplasia", [6313, 6335, 6336, 6337, 6338])
    },
    "BentheimBlackPiedPigBreed": {
        "132355002": ("Bentheim Black Pied pig breed", [7480])
    },
    "BentheimerLandschafSheepBreed": {
        "132859000": ("Bentheimer Landschaf sheep breed", [7480])
    },
    "BenzoFastScarletStain": {"27844007": ("benzo fast scarlet stain", [8112])},
    "Benzodiazepine": {"372664007": ("Benzodiazepine", [621, 622])},
    "BenzodiazepineAntagonist": {
        "372906009": ("Benzodiazepine antagonist", [621, 622])
    },
    "BergamascaSheepBreed": {"132681007": ("Bergamasca sheep breed", [7480])},
    "BergamascoDogBreed": {"132523003": ("Bergamasco dog breed", [7480])},
    "BergerDePicardDogBreed": {"132524009": ("Berger de Picard dog breed", [7480])},
    "BergerDePyreneesDogBreed": {"132525005": ("Berger de Pyrenees dog breed", [7480])},
    "BergerDuLanguedocDogBreed": {
        "132453008": ("Berger du Languedoc dog breed", [7480])
    },
    "BergerShetlandDogBreed": {"132640004": ("Berger Shetland dog breed", [7480])},
    "BerkshirePig": {"112489009": ("Berkshire pig (organism)", [])},
    "BerkshirePigBreed": {"112489009": ("Berkshire pig breed", [7480])},
    "BerneseHoundDogBreed": {"132386008": ("Bernese Hound dog breed", [7480])},
    "BerneseMountainDog": {"33458006": ("Bernese mountain dog", [7480])},
    "BerrendasCattleBreed": {"131575009": ("Berrendas cattle breed", [7480])},
    "Bertel": {"399362005": ("Bertel", [4012])},
    "BertelProjection": {"399362005": ("Bertel projection (qualifier value)", [])},
    "Beryllium": {"88014003": ("Beryllium", [10067])},
    "BestCorrectedVisualAcuity": {
        "419775003": ("Best Corrected Visual Acuity", [4216])
    },
    "BestValue": {"371912002": ("Best value", [3627])},
    "BestValueNumericEstimationTechnique": {
        "371912002": ("Best value - numeric estimation technique (qualifier value)", [])
    },
    "BestuzhevCattleBreed": {"133321009": ("Bestuzhev cattle breed", [7480])},
    "BetaBlocker": {"373254001": ("Beta-blocker", [621, 622])},
    "BetaBlockerContainingProduct": {
        "33252009": ("beta-Adrenergic blocking agent", [10, 3760])
    },
    "BetaBlockingAgent": {"373254001": ("beta-Blocking agent (substance)", [])},
    "BetaGlucuronidaseStain": {"255795004": ("beta-glucuronidase stain", [8112])},
    "BetaLactamAntibiotic": {"373297006": ("Beta-Lactam antibiotic", [621, 622])},
    "BetizuakCattleBreed": {"133322002": ("Betizuak cattle breed", [7480])},
    "BeulahSpeckledFaceSheepBreed": {
        "131716003": ("Beulah Speckled-Face sheep breed", [7480])
    },
    "BhagnariCattleBreed": {"133598005": ("Bhagnari cattle breed", [7480])},
    "BhirumPonyHorseBreed": {"133123000": ("Bhirum Pony horse breed", [7480])},
    "BhujGoatBreed": {"131655005": ("Bhuj goat breed", [7480])},
    "BhutiaHorseBreed": {"131880008": ("Bhutia horse breed", [7480])},
    "BibrikSheepBreed": {"131704009": ("Bibrik sheep breed", [7480])},
    "BichonTeneriffeDogBreed": {"132371002": ("Bichon Teneriffe dog breed", [7480])},
    "BichonsFriseDog": {"41538003": ("Bichons frise dog", [7480])},
    "BicycleErgometer": {"739006": ("Bicycle ergometer", [3203])},
    "BicycleErgometerDevice": {
        "739006": ("Bicycle ergometer, device (physical object)", [])
    },
    "BidirectionalGlennShuntProcedureOfLeftSuperiorVenaCava": {
        "443989003": (
            "Bidirectional Glenn shunt procedure of left superior vena cava (procedure)",
            [],
        )
    },
    "BidirectionalGlennShuntProcedureOfRightSuperiorVenaCava": {
        "444034006": (
            "Bidirectional Glenn shunt procedure of right superior vena cava (procedure)",
            [],
        )
    },
    "BiebrichScarletStain": {"76605005": ("biebrich scarlet stain", [8112])},
    "BielleseSheepBreed": {"131788009": ("Biellese sheep breed", [7480])},
    "BifascicularBlock": {"74021003": ("Bifascicular Block", [3230])},
    "Bifurcation": {"371894001": ("Bifurcation", [3712])},
    "BifurcationLesionOfCoronaryArtery": {
        "371894001": ("Bifurcation lesion of coronary artery (disorder)", [])
    },
    "Bilateral": {"51440002": ("Bilateral", [2, 211, 244, 245])},
    "Bile": {"70150004": ("Bile", [7151, 7197])},
    "BileAcidSequestrant": {"372872006": ("Bile acid sequestrant", [3761])},
    "BileAcidSequestrantAntilipemicAgent": {
        "372872006": ("Bile acid sequestrant antilipemic agent (substance)", [])
    },
    "BileDuct": {
        "28273000": (
            "Bile duct",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7154, 7192, 8134, 9514],
        )
    },
    "BileDuctStructure": {"28273000": ("Bile duct structure (body structure)", [])},
    "BiliaryTract": {
        "34707002": (
            "Biliary tract",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7154, 7192, 8134, 9514],
        )
    },
    "BiliaryTractStructure": {
        "34707002": ("Biliary tract structure (body structure)", [])
    },
    "BillyDogBreed": {"132526006": ("Billy dog breed", [7480])},
    "BimalCattleBreed": {"133641009": ("Bimal cattle breed", [7480])},
    "BiohazardousMaterial": {"409595003": ("Biohazardous material", [602])},
    "Biopsy": {"86273004": ("Biopsy", [9, 8109])},
    "BiopsyOfSkin": {"240977001": ("Biopsy of skin", [4409])},
    "BiosafetyLevel1": {"409600007": ("Biosafety level 1", [601])},
    "BiosafetyLevel2": {"409603009": ("Biosafety level 2", [601])},
    "BiosafetyLevel3": {"409604003": ("Biosafety level 3", [601])},
    "BiosafetyLevel4": {"409605002": ("Biosafety level 4", [601])},
    "BirmanCat": {"43219001": ("Birman cat", [7480])},
    "BisaroPigBreed": {"132174001": ("Bisaro pig breed", [7480])},
    "BismarkBrownRStain": {"44488008": ("bismark brown R stain", [8112])},
    "BismarkBrownYStain": {"85190005": ("bismark brown Y stain", [8112])},
    "Bismuth": {"23172004": ("Bismuth", [300])},
    "BisonBisonXBosTaurusHybrid": {
        "425181009": ("Bison bison X Bos taurus hybrid (organism)", [])
    },
    "BisonBisonXSimmentalHybrid": {
        "424705003": ("Bison bison X Simmental hybrid", [7480])
    },
    "BisonBisonXSimmentalHybridCattleBreed": {
        "424705003": ("Bison bison X Simmental hybrid cattle breed (organism)", [])
    },
    "BiteBlock": {
        "228745001": ("Bite block", [7151, 7157, 7193, 9505, 9513, 9520, 9573])
    },
    "BizanianHoundDogBreed": {"132372009": ("Bizanian Hound dog breed", [7480])},
    "Black": {"371252007": ("Black", [4411])},
    "BlackAndTanCoonhound": {"45561005": ("Black and tan coonhound (organism)", [])},
    "BlackAndTanCoonhoundDogBreed": {
        "45561005": ("Black and tan coonhound dog breed", [7480])
    },
    "BlackAngusCattleBreed": {"409905004": ("Black Angus cattle breed (organism)", [])},
    "BlackAngusCowBreed": {"409905004": ("Black Angus cow breed", [7480])},
    "BlackBaldyCattleBreed": {"133323007": ("Black Baldy cattle breed", [7480])},
    "BlackCockerSpaniel": {"82206008": ("Black cocker spaniel (organism)", [])},
    "BlackCockerSpanielDogBreed": {
        "82206008": ("Black cocker spaniel dog breed", [7480])
    },
    "BlackColor": {"371252007": ("Black color", [6067])},
    "BlackFacedHighlandSheep": {
        "65187008": ("Black faced Highland sheep (organism)", [])
    },
    "BlackFacedHighlandSheepBreed": {
        "65187008": ("Black faced Highland sheep breed", [7480])
    },
    "BlackForestCattleBreed": {
        "133324001": ("Black Forest cattle breed (organism)", [])
    },
    "BlackForestHoundDogBreed": {"132550008": ("Black Forest Hound dog breed", [7480])},
    "BlackForrestCattleBreed": {"133324001": ("Black Forrest cattle breed", [7480])},
    "BlackGrisonneGoatBreed": {"131684001": ("Black Grisonne goat breed", [7480])},
    "BlackHairlessPigBreed": {"132175000": ("Black Hairless pig breed", [7480])},
    "BlackHampshirePigBreed": {"132043001": ("Black Hampshire pig breed", [7480])},
    "BlackIberianCattleBreed": {"133325000": ("Black Iberian cattle breed", [7480])},
    "BlackMangalitsaPigBreed": {"132176004": ("Black Mangalitsa pig breed", [7480])},
    "BlackMouthCurDogBreed": {"132568006": ("Black Mouth Cur dog breed", [7480])},
    "BlackRussianTerrierDogBreed": {
        "132535004": ("Black Russian Terrier dog breed", [7480])
    },
    "BlackSeaHorseHorseBreed": {"131881007": ("Black Sea Horse horse breed", [7480])},
    "BlackSlavonianPigBreed": {"133204003": ("Black Slavonian pig breed", [7480])},
    "BlackWelshCattleBreed": {"28744004": ("Black Welsh cattle breed (organism)", [])},
    "BlackWelshCowBreed": {"28744004": ("Black Welsh cow breed", [7480])},
    "BlackWelshMountainSheepBreed": {
        "131706006": ("Black Welsh Mountain Sheep breed", [7480])
    },
    "BlackettHealy": {"399246001": ("Blackett-Healy", [4012])},
    "BlackettHealyProjection": {
        "399246001": ("Blackett-Healy projection (qualifier value)", [])
    },
    "BlackheadPersianSheepBreed": {
        "131707002": ("Blackhead Persian sheep breed", [7480])
    },
    "BlacksidedTrondheimAndNorlandCattleBreed": {
        "131576005": ("Blacksided Trondheim and Norland cattle breed", [7480])
    },
    "Bladder": {
        "89837001": (
            "Bladder",
            [
                4,
                645,
                4009,
                4030,
                4031,
                4040,
                4042,
                6202,
                6204,
                7151,
                7160,
                7192,
                7482,
                7483,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "BladderAndUrethra": {
        "110837003": ("Bladder and urethra", [4, 4040, 7151, 7192, 8134, 9514])
    },
    "BladderCancer": {"399326009": ("Bladder cancer", [638, 639])},
    "BlalockTaussigShuntPulmonarySubclavianArteryAnastomosis": {
        "13662000": (
            "Blalock-Taussig shunt, pulmonary-subclavian artery anastomosis",
            [12247],
        )
    },
    "BlancDeBouscatRabbitBreed": {
        "132888004": ("Blanc de Bouscat rabbit breed", [7480])
    },
    "BlancaAndaluzaGoatBreed": {"131681009": ("Blanca Andaluza goat breed", [7480])},
    "BlancaCeltibericaGoatBreed": {
        "131682002": ("Blanca Celtiberica goat breed", [7480])
    },
    "BlancoOrejinegroCattleBreed": {
        "131577001": ("Blanco Orejinegro cattle breed", [7480])
    },
    "Bleeding": {"50960005": ("Bleeding", [3413, 3754])},
    "BleedingSkin": {"297968009": ("Bleeding skin", [4406, 4408])},
    "BleuDuMaineSheepBreed": {"131708007": ("Bleu du Maine sheep breed", [7480])},
    "BlockSectioning": {"434472006": ("Block sectioning", [8110])},
    "BlockStepSectioning": {"434475008": ("Block step sectioning", [8110])},
    "BlockSurfaceRecut": {"434474007": ("Block surface recut", [8110])},
    "BlondeDAquitaine": {"62153005": ("Blonde d'Aquitaine (organism)", [])},
    "BlondeDAquitaineCowBreed": {"62153005": ("Blonde d'Aquitaine cow breed", [7480])},
    "BlondeDuCapBonCattleBreed": {
        "131458001": ("Blonde-du Cap Bon cattle breed", [7480])
    },
    "Blood": {"87612001": ("Blood", [7151, 7166, 7191])},
    "BloodClot": {"75753009": ("Blood clot", [7151, 7159, 7194])},
    "BloodCoagulationDisorder": {
        "64779008": ("Blood coagulation disorder (disorder)", [])
    },
    "BloodCoagulationDisorders": {"64779008": ("Blood Coagulation Disorders", [1201])},
    "BloodPressure": {
        "75367002": (
            "Blood Pressure",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "BloodPressureCuffMethod": {"371911009": ("Blood pressure cuff method", [3560])},
    "BloodProduct": {"410652009": ("Blood product", [621, 622])},
    "BloodSamplingFromExtracorporealBloodCircuit": {
        "243776001": ("blood sampling from extracorporeal blood circuit", [3515])
    },
    "BloodVessel": {
        "59820001": (
            "Blood Vessel",
            [6202, 6204, 7151, 7155, 7166, 7191, 7192, 8134, 9514],
        )
    },
    "BloodVesselStructure": {
        "59820001": ("Blood vessel structure (body structure)", [])
    },
    "Bloodhound": {"81529001": ("Bloodhound", [7480])},
    "BloodhoundSouthernHoundDogBreed": {
        "132374005": ("Bloodhound, Southern Hound dog breed", [7480])
    },
    "BloodhoundStHubertDogBreed": {
        "132373004": ("Bloodhound, St. Hubert dog breed", [7480])
    },
    "BloodyNippleDischarge": {"290113009": ("Bloody nipple discharge", [6055])},
    "Blue": {"405738005": ("Blue", [4206, 4255, 4411, 8122])},
    "BlueColor": {"405738005": ("Blue color (qualifier value)", [])},
    "BlueGreenColor": {"405739002": ("Blue green color (qualifier value)", [])},
    "BlueGreenOpticalFilter": {"422915004": ("Blue-green optical filter", [4204])},
    "BlueOpticalFilter": {"445084008": ("Blue optical filter", [4204, 8124])},
    "BlueOpticalFilterOfOpticalMicroscope": {
        "445084008": ("Blue optical filter of optical microscope (physical object)", [])
    },
    "BlueShadeEosinStain": {"1346008": ("blue shade eosin stain", [8112])},
    "BlueTickCoonhound": {"55959002": ("Blue tick coonhound (organism)", [])},
    "BlueTickCoonhoundDogBreed": {
        "55959002": ("Blue tick coonhound dog breed", [7480])
    },
    "BluefacedLeicesterSheepBreed": {
        "131709004": ("Bluefaced Leicester sheep breed", [7480])
    },
    "Blunted": {"428247006": ("blunted", [3210, 3211])},
    "BoXuPigBreed": {"133164005": ("Bo Xu pig breed", [7480])},
    "BoarPowerPig": {"74899005": ("Boar power pig (organism)", [])},
    "BoarPowerPig141": {"68512002": ("Boar power pig 141 (organism)", [])},
    "BoarPowerPig141PigBreed": {"68512002": ("Boar power pig 141 pig breed", [7480])},
    "BoarPowerPig161": {"74970001": ("Boar power pig 161 (organism)", [])},
    "BoarPowerPig161PigBreed": {"74970001": ("Boar power pig 161 pig breed", [7480])},
    "BoarPowerPig27": {"76364003": ("Boar power pig 27 (organism)", [])},
    "BoarPowerPig27PigBreed": {"76364003": ("Boar power pig 27 pig breed", [7480])},
    "BoarPowerPig282": {"87061000": ("Boar power pig 282 (organism)", [])},
    "BoarPowerPig282PigBreed": {"87061000": ("Boar power pig 282 pig breed", [7480])},
    "BoarPowerPig292": {"56084008": ("Boar power pig 292 (organism)", [])},
    "BoarPowerPig292PigBreed": {"56084008": ("Boar power pig 292 pig breed", [7480])},
    "BoarPowerPig414": {"24319000": ("Boar power pig 414 (organism)", [])},
    "BoarPowerPig414PigBreed": {"24319000": ("Boar power pig 414 pig breed", [7480])},
    "BoarPowerPig454": {"43500007": ("Boar power pig 454 (organism)", [])},
    "BoarPowerPig454PigBreed": {"43500007": ("Boar power pig 454 pig breed", [7480])},
    "BoarPowerPig474": {"84315000": ("Boar power pig 474 (organism)", [])},
    "BoarPowerPig474PigBreed": {"84315000": ("Boar power pig 474 pig breed", [7480])},
    "BoarPowerPig48": {"32297006": ("Boar power pig 48 (organism)", [])},
    "BoarPowerPig48PigBreed": {"32297006": ("Boar power pig 48 pig breed", [7480])},
    "BoarPowerPig545": {"61036003": ("Boar power pig 545 (organism)", [])},
    "BoarPowerPig545PigBreed": {"61036003": ("Boar power pig 545 pig breed", [7480])},
    "BoarPowerPig565": {"29223008": ("Boar power pig 565 (organism)", [])},
    "BoarPowerPig565PigBreed": {"29223008": ("Boar power pig 565 pig breed", [7480])},
    "BoarPowerPig59": {"53431006": ("Boar power pig 59 (organism)", [])},
    "BoarPowerPig59PigBreed": {"53431006": ("Boar power pig 59 pig breed", [7480])},
    "BoarPowerPig616": {"33212007": ("Boar power pig 616 (organism)", [])},
    "BoarPowerPig616PigBreed": {"33212007": ("Boar power pig 616 pig breed", [7480])},
    "BoarPowerPig656": {"48470006": ("Boar power pig 656 (organism)", [])},
    "BoarPowerPig656PigBreed": {"48470006": ("Boar power pig 656 pig breed", [7480])},
    "BoarPowerPig72": {"18212001": ("Boar power pig 72 (organism)", [])},
    "BoarPowerPig72PigBreed": {"18212001": ("Boar power pig 72 pig breed", [7480])},
    "BoarPowerPig747": {"84081007": ("Boar power pig 747 (organism)", [])},
    "BoarPowerPig747PigBreed": {"84081007": ("Boar power pig 747 pig breed", [7480])},
    "BoarPowerPig828": {"34595003": ("Boar power pig 828 (organism)", [])},
    "BoarPowerPig828PigBreed": {"34595003": ("Boar power pig 828 pig breed", [7480])},
    "BoarPowerPig84": {"30720007": ("Boar power pig 84 (organism)", [])},
    "BoarPowerPig84PigBreed": {"30720007": ("Boar power pig 84 pig breed", [7480])},
    "BoarPowerPig929": {"25856007": ("Boar power pig 929 (organism)", [])},
    "BoarPowerPig929PigBreed": {"25856007": ("Boar power pig 929 pig breed", [7480])},
    "BoarPowerPigBreed": {"74899005": ("Boar power pig breed", [7480])},
    "Bodmin950PigBreed": {"132217009": ("Bodmin 950 pig breed", [7480])},
    "BodyCavity": {"91806002": ("Body cavity", [7151, 7192, 9514])},
    "BodyCavityStructure": {"91806002": ("Body cavity structure (body structure)", [])},
    "BodyConduit": {
        "91830000": ("Body conduit", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "BodyFatObservable": {"248300009": ("Body fat observable (observable entity)", [])},
    "BodyFluid": {"32457005": ("Body fluid", [7151, 7197])},
    "BodyFluidSample": {"309051001": ("Body fluid sample", [8103])},
    "BodyMassIndex": {"60621009": ("Body mass index (observable entity)", [])},
    "BodyOfSternum": {"52509009": ("Body of sternum", [6115, 8134])},
    "BodyOrganStructure": {"113343008": ("Body organ structure (body structure)", [])},
    "BodyPartStructure": {"38866009": ("Body part structure (body structure)", [])},
    "BodyReferencePointMarker": {"706484002": ("Body reference point marker", [7110])},
    "BodyStructure": {"123037004": ("Body structure (body structure)", [])},
    "BodySubstance": {"91720002": ("Body Substance", [7150, 9502, 9580])},
    "BodySubstanceSample": {"309050000": ("Body substance sample", [8103])},
    "BodySurfacePoint": {"183973000": ("Body surface point", [7110])},
    "BodyTissueStructure": {"85756007": ("Body tissue structure (body structure)", [])},
    "BoerGoatBreed": {"131656006": ("Boer goat breed", [7480])},
    "BoerboelDogBreed": {"416840006": ("Boerboel dog breed", [7480])},
    "BolivianPigBreed": {"132168001": ("Bolivian pig breed", [7480])},
    "BologneseDogBreed": {"132560004": ("Bolognese dog breed", [7480])},
    "BombayCat": {"16528000": ("Bombay cat", [7480])},
    "BondSheepBreed": {"131710009": ("Bond sheep breed", [7480])},
    "Bone": {"272673000": ("Bone", [6202, 6204, 8134])},
    "BoneFormationFunction": {
        "83323007": ("Bone formation, function (observable entity)", [])
    },
    "BoneMarrow": {"14016003": ("Bone Marrow", [644, 8134, 10044, 10060])},
    "BoneMarrowStructure": {"14016003": ("Bone marrow structure (body structure)", [])},
    "BoneMarrowTransplant": {"23719005": ("Bone marrow transplant", [6083, 6085])},
    "BoneMatrix": {"45001002": ("Bone matrix", [6211])},
    "BoneNail": {"63289001": ("Bone Nail", [7307, 7309])},
    "BoneOfLowerLimb": {
        "72001000": (
            "Bone of lower limb",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "BoneOfThorax": {"272710004": ("Bone of thorax", [7151, 7155, 7192, 8134, 9514])},
    "BoneOfUpperLimb": {
        "371195002": (
            "Bone of upper limb",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "BonePin": {"77444004": ("Bone Pin", [7151, 7157, 7193])},
    "BonePinDevice": {"77444004": ("Bone pin, device (physical object)", [])},
    "BonePlate": {"271003": ("Bone Plate", [7307, 7309])},
    "BonePlateDevice": {"271003": ("Bone plate, device (physical object)", [])},
    "BoneScrew": {"68183006": ("Bone Screw", [7151, 7157, 7193])},
    "BoneScrewDevice": {"68183006": ("Bone screw, device (physical object)", [])},
    "BoneStructure": {"272673000": ("Bone structure (body structure)", [])},
    "BoneStructureOfAcetabulum": {
        "37783008": ("Bone structure of acetabulum (body structure)", [])
    },
    "BoneStructureOfAtlas": {
        "14806007": ("Bone structure of atlas (body structure)", [])
    },
    "BoneStructureOfAxis": {
        "39976000": ("Bone structure of axis (body structure)", [])
    },
    "BoneStructureOfC3": {"113205007": ("Bone structure of C3 (body structure)", [])},
    "BoneStructureOfC4": {"5329002": ("Bone structure of C4 (body structure)", [])},
    "BoneStructureOfC5": {"36978003": ("Bone structure of C5 (body structure)", [])},
    "BoneStructureOfC6": {"36054005": ("Bone structure of C6 (body structure)", [])},
    "BoneStructureOfC7": {"87391001": ("Bone structure of C7 (body structure)", [])},
    "BoneStructureOfCalcaneum": {
        "80144004": ("Bone structure of calcaneum (body structure)", [])
    },
    "BoneStructureOfClavicle": {
        "51299004": ("Bone structure of clavicle (body structure)", [])
    },
    "BoneStructureOfCoccyx": {
        "64688005": ("Bone structure of coccyx (body structure)", [])
    },
    "BoneStructureOfCranium": {
        "89546000": ("Bone structure of cranium (body structure)", [])
    },
    "BoneStructureOfDistalFemur": {
        "310652005": ("Bone structure of distal femur (body structure)", [])
    },
    "BoneStructureOfDistalHumerus": {
        "118495001": ("Bone structure of distal humerus (body structure)", [])
    },
    "BoneStructureOfDistalRadius": {
        "75129005": ("Bone structure of distal radius (body structure)", [])
    },
    "BoneStructureOfDistalTibia": {
        "64605006": ("Bone structure of distal tibia (body structure)", [])
    },
    "BoneStructureOfDistalUlna": {
        "91238003": ("Bone structure of distal ulna (body structure)", [])
    },
    "BoneStructureOfFace": {
        "91397008": ("Bone structure of face (body structure)", [])
    },
    "BoneStructureOfFemur": {
        "71341001": ("Bone structure of femur (body structure)", [])
    },
    "BoneStructureOfFibula": {
        "87342007": ("Bone structure of fibula (body structure)", [])
    },
    "BoneStructureOfHand": {
        "24097009": ("Bone structure of hand (body structure)", [])
    },
    "BoneStructureOfHeadAndOrNeck": {
        "312779009": (
            "Bone structure of head and/or neck",
            [4028, 7151, 7192, 8134, 9514],
        )
    },
    "BoneStructureOfHumerus": {
        "85050009": ("Bone structure of humerus (body structure)", [])
    },
    "BoneStructureOfIlium": {
        "22356005": ("Bone structure of ilium (body structure)", [])
    },
    "BoneStructureOfIschium": {
        "85710004": ("Bone structure of ischium (body structure)", [])
    },
    "BoneStructureOfL1": {"66794005": ("Bone structure of L1 (body structure)", [])},
    "BoneStructureOfL2": {"14293000": ("Bone structure of L2 (body structure)", [])},
    "BoneStructureOfL3": {"36470004": ("Bone structure of L3 (body structure)", [])},
    "BoneStructureOfL4": {"11994002": ("Bone structure of L4 (body structure)", [])},
    "BoneStructureOfL5": {"49668003": ("Bone structure of L5 (body structure)", [])},
    "BoneStructureOfLowerLimb": {
        "72001000": ("Bone structure of lower limb (body structure)", [])
    },
    "BoneStructureOfMandible": {
        "91609006": ("Bone structure of mandible (body structure)", [])
    },
    "BoneStructureOfManubrium": {
        "37285002": ("Bone structure of manubrium (body structure)", [])
    },
    "BoneStructureOfMaxilla": {
        "70925003": ("Bone structure of maxilla (body structure)", [])
    },
    "BoneStructureOfMetacarpal": {
        "36455000": ("Bone structure of metacarpal (body structure)", [])
    },
    "BoneStructureOfNavicular": {
        "75772009": ("Bone structure of navicular (body structure)", [])
    },
    "BoneStructureOfPatella": {
        "64234005": ("Bone structure of patella (body structure)", [])
    },
    "BoneStructureOfPelvis": {"118645006": ("Pelvis", [7304, 8134])},
    "BoneStructureOfProximalFemur": {
        "310651003": ("Bone structure of proximal femur (body structure)", [])
    },
    "BoneStructureOfProximalHumerus": {
        "119524001": ("Bone structure of proximal humerus (body structure)", [])
    },
    "BoneStructureOfProximalRadius": {
        "12881000": ("Bone structure of proximal radius (body structure)", [])
    },
    "BoneStructureOfProximalUlna": {
        "34318004": ("Bone structure of proximal ulna (body structure)", [])
    },
    "BoneStructureOfRib": {"113197003": ("Bone structure of rib (body structure)", [])},
    "BoneStructureOfSacrum": {
        "54735007": ("Bone structure of sacrum (body structure)", [])
    },
    "BoneStructureOfScaphoid": {
        "30518006": ("Bone structure of scaphoid (body structure)", [])
    },
    "BoneStructureOfScapula": {
        "79601000": ("Bone structure of scapula (body structure)", [])
    },
    "BoneStructureOfShaftOfFemur": {
        "41111004": ("Bone structure of shaft of femur (body structure)", [])
    },
    "BoneStructureOfShaftOfHumerus": {
        "20760004": ("Bone structure of shaft of humerus (body structure)", [])
    },
    "BoneStructureOfShaftOfRadius": {
        "47728000": ("Bone structure of shaft of radius (body structure)", [])
    },
    "BoneStructureOfShaftOfTibia": {
        "52687003": ("Bone structure of shaft of tibia (body structure)", [])
    },
    "BoneStructureOfShaftOfUlna": {
        "21133008": ("Bone structure of shaft of ulna (body structure)", [])
    },
    "BoneStructureOfSpine": {
        "51282000": ("Bone structure of spine (body structure)", [])
    },
    "BoneStructureOfSternum": {
        "56873002": ("Bone structure of sternum (body structure)", [])
    },
    "BoneStructureOfT1": {"64864005": ("Bone structure of T1 (body structure)", [])},
    "BoneStructureOfT10": {"7610001": ("Bone structure of T10 (body structure)", [])},
    "BoneStructureOfT11": {"12989004": ("Bone structure of T11 (body structure)", [])},
    "BoneStructureOfT12": {"23215003": ("Bone structure of T12 (body structure)", [])},
    "BoneStructureOfT2": {"53733008": ("Bone structure of T2 (body structure)", [])},
    "BoneStructureOfT3": {"1626008": ("Bone structure of T3 (body structure)", [])},
    "BoneStructureOfT4": {"73071006": ("Bone structure of T4 (body structure)", [])},
    "BoneStructureOfT5": {"56401006": ("Bone structure of T5 (body structure)", [])},
    "BoneStructureOfT6": {"45296009": ("Bone structure of T6 (body structure)", [])},
    "BoneStructureOfT7": {"62487009": ("Bone structure of T7 (body structure)", [])},
    "BoneStructureOfT8": {"11068009": ("Bone structure of T8 (body structure)", [])},
    "BoneStructureOfT9": {"82687006": ("Bone structure of T9 (body structure)", [])},
    "BoneStructureOfTalus": {
        "67453005": ("Bone structure of talus (body structure)", [])
    },
    "BoneStructureOfTarsus": {
        "108371006": ("Bone structure of tarsus (body structure)", [])
    },
    "BoneStructureOfThorax": {
        "272710004": ("Bone structure of thorax (body structure)", [])
    },
    "BoneStructureOfTibia": {
        "12611008": ("Bone structure of tibia (body structure)", [])
    },
    "BoneStructureOfUlna": {
        "23416004": ("Bone structure of ulna (body structure)", [])
    },
    "BoneStructureOfUpperLimb": {
        "371195002": ("Bone structure of upper limb (body structure)", [])
    },
    "BoneSurface": {"425647002": ("Bone Surface", [10044, 10060])},
    "BoneTissue": {"3138006": ("Bone", [645, 7151, 7166, 7191, 8134])},
    "BonsmaraCattleBreed": {"131431008": ("Bonsmara cattle breed", [7480])},
    "BonyMetastasis": {"94222008": ("Bony metastasis", [6347])},
    "BooroolaMerinoSheepBreed": {"132735001": ("Booroola Merino sheep breed", [7480])},
    "BootedGoatBreed": {"131648008": ("Booted goat breed", [7480])},
    "BoranCattleBreed": {"133599002": ("Boran cattle breed", [7480])},
    "BordeauxDogBreed": {"132389001": ("Bordeaux Dog breed", [7480])},
    "BorderCollieDogBreed": {"132561000": ("Border Collie dog breed", [7480])},
    "BorderLeicesterSheepBreed": {
        "131711008": ("Border Leicester sheep breed", [7480])
    },
    "BorderTerrier": {"69529009": ("Border terrier", [7480])},
    "BorderlineNormalECG": {"251135002": ("Borderline Normal ECG", [3677])},
    "BorderlineNormalElectrocardiogram": {
        "251135002": ("Borderline normal electrocardiogram (finding)", [])
    },
    "BoreraySheepBreed": {"131712001": ("Boreray sheep breed", [7480])},
    "BorghigianaPigBreed": {"132178003": ("Borghigiana pig breed", [7480])},
    "BorgouCattleBreed": {"133765003": ("Borgou cattle breed (organism)", [])},
    "BorgouXZebuCattleBreed": {"133765003": ("Borgou X zebu cattle breed", [7480])},
    "BorneoZebuCattleBreed": {"133642002": ("Borneo Zebu cattle breed", [7480])},
    "BorzoiDog": {"112492008": ("Borzoi dog", [7480])},
    "Bos": {"388168008": ("Bos", [7454])},
    "BosTaurus": {"34618005": ("Bos taurus", [7454])},
    "BosTaurusIndicus": {"125091004": ("Bos taurus indicus (organism)", [])},
    "BosTaurusIndicusCowBreed": {"125091004": ("Bos taurus indicus cow breed", [7480])},
    "BosTaurusTaurusSubspeciesDomesticEuropeanCattle": {
        "385474004": (
            "Bos taurus taurus subspecies domestic European cattle (organism)",
            [],
        )
    },
    "BosTaurusTaurusSubspeciesDomesticEuropeanCowBreed": {
        "385474004": (
            "Bos taurus taurus subspecies domestic European cow breed",
            [7480],
        )
    },
    "BosTaurusXBisonBisonHybridCowBreed": {
        "425181009": ("Bos taurus X Bison bison hybrid cow breed", [7480])
    },
    "BosnianHorseBreed": {"131882000": ("Bosnian horse breed", [7480])},
    "BosnianHoundDogBreed": {"132573000": ("Bosnian Hound dog breed", [7480])},
    "BostonTerrier": {"79295007": ("Boston terrier", [7480])},
    "BothBreasts": {"63762007": ("Both breasts", [6022, 6023])},
    "BothEyes": {"40638003": ("Both eyes", [4209, 8134])},
    "Bottle": {"68276009": ("Bottle", [68, 69])},
    "BottleDevice": {"68276009": ("Bottle, device (physical object)", [])},
    "BouinFluid": {"433474002": ("Bouin's fluid", [8114])},
    "BoulonnaisHorseBreed": {"131883005": ("Boulonnais horse breed", [7480])},
    "BoundedBy": {"75958009": ("Bounded by", [219])},
    "BouvierDesFlandres": {"66712005": ("Bouvier des Flandres", [7480])},
    "Bovinae": {"107007004": ("Bovinae", [7454])},
    "BovineSpecies": {"79058000": ("Bovine species (organism)", [])},
    "BovskaSheepBreed": {"131713006": ("Bovska sheep breed", [7480])},
    "BoxerDog": {"42250008": ("Boxer dog", [7480])},
    "BoydPerforatingVein": {
        "128548003": (
            "Boyd's perforating vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "BoykinSpanielDogBreed": {"132580003": ("Boykin Spaniel dog breed", [7480])},
    "BraMaineCattleBreed": {"133768001": ("Bra-Maine cattle breed (organism)", [])},
    "BraMaineXZebuCattleBreed": {
        "133768001": ("Bra-Maine X zebu cattle breed", [7480])
    },
    "BraSwissCattleBreed": {"133779006": ("Bra-Swiss cattle breed (organism)", [])},
    "BraSwissXZebuCattleBreed": {
        "133779006": ("Bra-Swiss X zebu cattle breed", [7480])
    },
    "BrabantHorse": {"406711007": ("Brabant horse (organism)", [])},
    "BrabantHorseBreed": {"406711007": ("Brabant horse breed", [7480])},
    "BraccoItalianoDogBreed": {"132562007": ("Bracco Italiano dog breed", [7480])},
    "BrachialArtery": {
        "17137000": (
            "Brachial artery",
            [
                4,
                3010,
                3440,
                3604,
                3606,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12107,
            ],
        )
    },
    "BrachialPlexus": {"36582005": ("Brachial plexus", [8134])},
    "BrachialPlexusStructure": {
        "36582005": ("Brachial plexus structure (body structure)", [])
    },
    "BrachialVein": {
        "20115005": (
            "Brachial vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "BrachiocephalicTrunk": {
        "12691009": ("brachiocephalic trunk", [3604, 3606, 3630, 6102, 6109, 6117])
    },
    "BrachiocephalicVein": {"8887007": ("Brachiocephalic vein", [6102, 6109, 6117])},
    "Brachytherapy": {"384692006": ("Brachytherapy", [6051, 6327])},
    "BrachytherapyDevice": {"228748004": ("Brachytherapy Device", [3411, 3429])},
    "BrachytherapyImplant": {
        "228748004": ("Brachytherapy implant (physical object)", [])
    },
    "BrafordCattleBreed": {"133716008": ("Braford cattle breed (organism)", [])},
    "BrafordXZebuCattleBreed": {"133716008": ("Braford X zebu cattle breed", [7480])},
    "BragadoDoSorraiaCattleBreed": {
        "133327008": ("Bragado do Sorraia cattle breed", [7480])
    },
    "BragancaCattleBreed": {"133328003": ("Braganca cattle breed", [7480])},
    "BrahmaCattleBreed": {"30384003": ("Brahma cattle breed (organism)", [])},
    "BrahmaCowBreed": {"30384003": ("Brahma cow breed", [7480])},
    "BrahmousinCattleBreed": {"133717004": ("Brahmousin cattle breed (organism)", [])},
    "BrahmousinXZebuCattleBreed": {
        "133717004": ("Brahmousin X zebu cattle breed", [7480])
    },
    "BrahornCattleBreed": {"133766002": ("Brahorn cattle breed (organism)", [])},
    "BrahornXZebuCattleBreed": {"133766002": ("Brahorn X zebu cattle breed", [7480])},
    "Brain": {
        "12738006": (
            "Brain",
            [4, 644, 645, 4030, 7151, 7153, 7192, 8134, 9514, 10044, 10060],
        )
    },
    "BrainCerebrospinalFluidPathway": {
        "280371009": (
            "Brain cerebrospinal fluid pathway",
            [7151, 7153, 7192, 8134, 9514],
        )
    },
    "BrainStem": {"119238007": ("Brain Stem", [7140, 7151, 7153, 7192, 8134, 9514])},
    "BrainStemPart": {"119238007": ("Brain stem part (body structure)", [])},
    "BrainStructure": {"12738006": ("Brain structure (body structure)", [])},
    "BrainVentricle": {"35764002": ("Brain ventricle", [7151, 7153, 7192, 8134, 9514])},
    "BrainVentricleStructure": {
        "35764002": ("Brain ventricle structure (body structure)", [])
    },
    "BralersCattleBreed": {"133767006": ("Bralers cattle breed (organism)", [])},
    "BralersXZebuCattleBreed": {"133767006": ("Bralers X zebu cattle breed", [7480])},
    "BrandenburgHorseBreed": {"131884004": ("Brandenburg horse breed", [7480])},
    "BrandlbrackeDogBreed": {"132376007": ("Brandlbracke dog breed", [7480])},
    "BrandroodLjsselveeCattleBreed": {
        "133329006": ("Brandrood ljsselvee cattle breed", [7480])
    },
    "BraqueBelgeDogBreed": {"132384006": ("Braque Belge dog breed", [7480])},
    "BraqueDAriégeDogBreed": {"132377003": ("Braque d'Ariége dog breed", [7480])},
    "BraqueDAuvergneDogBreed": {"132569003": ("Braque d'Auvergne dog breed", [7480])},
    "BraqueDuBourbonnaisDogBreed": {
        "132564008": ("Braque du Bourbonnais dog breed", [7480])
    },
    "BraqueDupuyDogBreed": {"132415003": ("Braque Dupuy dog breed", [7480])},
    "BraqueFrancaisDeGrandTailleDogBreed": {
        "132559009": ("Braque Francais de Grand Taille dog breed (organism)", [])
    },
    "BraqueFrancaisDePetiteTailleDogBreed": {
        "132565009": ("Braque Francais de Petite Taille dog breed", [7480])
    },
    "BraqueFrançaisDeGrandTailleDogBreed": {
        "132559009": ("Braque Français de Grand Taille dog breed", [7480])
    },
    "BraqueSaintGermainDogBreed": {
        "132566005": ("Braque Saint-Germain dog breed", [7480])
    },
    "BraunesBergschafSheepBreed": {
        "131714000": ("Braunes Bergschaf sheep breed", [7480])
    },
    "BraunviehCattleBreed": {"131578006": ("Braunvieh cattle breed", [7480])},
    "BraviaGoatBreed": {"131683007": ("Bravia goat breed", [7480])},
    "BravonCattleBreed": {"133780009": ("Bravon cattle breed (organism)", [])},
    "BravonXZebuCattleBreed": {"133780009": ("Bravon X zebu cattle breed", [7480])},
    "BrazilianDairyHybridCattle": {
        "133781008": ("Brazilian Dairy hybrid cattle (organism)", [])
    },
    "BrazilianDairyHybridXZebuCattleBreed": {
        "133781008": ("Brazilian Dairy Hybrid X zebu cattle breed", [7480])
    },
    "BrazilianPolledCattleBreed": {
        "133330001": ("Brazilian Polled cattle breed", [7480])
    },
    "BrazilianShorthairCatBreed": {
        "132662004": ("Brazilian Shorthair cat breed", [7480])
    },
    "BrazilianSomaliSheepBreed": {
        "131715004": ("Brazilian Somali sheep breed", [7480])
    },
    "BrazilianSportHorseHorseBreed": {
        "131885003": ("Brazilian Sport Horse horse breed", [7480])
    },
    "BrazilinStain": {"41750006": ("brazilin stain", [8112])},
    "Breast": {
        "76752008": (
            "Breast",
            [
                4,
                645,
                4009,
                4013,
                4030,
                4031,
                4042,
                7151,
                7192,
                8134,
                9514,
                10044,
                10060,
                12321,
            ],
        )
    },
    "BreastComposition": {
        "129715009": ("Breast composition", [6014, 6015, 6016, 6052, 6053, 6054, 6056])
    },
    "BreastCompositionAnalysis": {"133890006": ("Breast composition analysis", [6043])},
    "BreastDuctSample": {"309548003": ("breast duct sample", [8103, 8104])},
    "BreastHematoma": {"302924003": ("Breast hematoma", [6056])},
    "BreastImplantTypeNotSpecified": {
        "2282003": ("Breast implant, type not specified", [6058, 6059])
    },
    "BreastImplantation": {"119853006": ("Breast implantation (procedure)", [])},
    "BreastLobularHyperplasia": {
        "6703006": ("Breast lobular hyperplasia", [6030, 6031])
    },
    "BreastLobuleHyperplasia": {
        "6703006": ("Breast lobule hyperplasia (disorder)", [])
    },
    "BreastLump": {"89164003": ("Breast lump", [6055])},
    "BreastNormal": {"290084006": ("Breast normal", [6054])},
    "BreastPain": {"53430007": ("Breast pain", [6055])},
    "BreastProsthesisDevice": {
        "2282003": ("Breast prosthesis, device (physical object)", [])
    },
    "BreastProsthesisInsertion": {
        "302343007": ("Breast prosthesis insertion", [6083, 6084])
    },
    "BreastReconstruction": {"33496007": ("Breast reconstruction", [6083, 6084])},
    "BreastSize": {"248808008": ("Breast size (observable entity)", [])},
    "BreastStructure": {"76752008": ("Breast structure (body structure)", [])},
    "BreastSurgery": {"309968000": ("Breast Surgery", [7030])},
    "BreastSurgeryDepartment": {
        "309968000": ("Breast surgery department (environment)", [])
    },
    "BreastSurgicalBiopsy": {"274331003": ("Breast - surgical biopsy", [6050, 6083])},
    "BreastTruCutBiopsySample": {
        "309058007": ("breast tru-cut biopsy sample", [8103, 8104])
    },
    "BrecknockHillCheviotSheepBreed": {
        "131702008": ("Brecknock Hill Cheviot sheep breed", [7480])
    },
    "BreechVersionObstetricalVersion": {
        "65240009": ("Breech Version [Obstetrical Version]", [9])
    },
    "BreitovPigBreed": {"132108004": ("Breitov pig breed", [7480])},
    "BretonBlackPiedCattleBreed": {
        "133331002": ("Breton Black Pied cattle breed", [7480])
    },
    "BretonHorseBreed": {"133142001": ("Breton horse breed", [7480])},
    "BrewertonProjection": {"260492003": ("Brewerton projection", [4012])},
    "BriardDog": {"10369004": ("Briard dog", [7480])},
    "BrightnessAcuityTestingVisualAcuity": {
        "425141002": ("Brightness Acuity Testing Visual Acuity", [4216])
    },
    "BrillenschafSheepBreed": {"131701001": ("Brillenschaf sheep breed", [7480])},
    "BrilliantCresylBlueStain": {"8342001": ("brilliant cresyl blue stain", [8112])},
    "BrilliantCroceinStain": {"86541009": ("brilliant crocein stain", [8112])},
    "BrilliantOrangeStain": {"8429000": ("brilliant orange stain", [8112])},
    "BrilliantYellowStain": {"57753006": ("brilliant yellow stain", [8112])},
    "BriquetBassetGriffonVendeenDogBreed": {
        "132567001": ("Briquet Basset Griffon Vendeen dog breed", [7480])
    },
    "BriquetBassetGriffonVendéenDogBreed": {
        "132567001": ("Briquet Basset Griffon Vendéen dog breed (organism)", [])
    },
    "BritishAlpineGoatBreed": {"131654009": ("British Alpine goat breed", [7480])},
    "BritishAppaloosaHorseBreed": {
        "131886002": ("British Appaloosa horse breed", [7480])
    },
    "BritishBlueCatBreed": {"132656009": ("British Blue cat breed", [7480])},
    "BritishLandracePig": {"78994007": ("British landrace pig (organism)", [])},
    "BritishLandracePigBreed": {"78994007": ("British landrace pig breed", [7480])},
    "BritishLopPig": {"22506004": ("British lop pig (organism)", [])},
    "BritishLopPigBreed": {"22506004": ("British lop pig breed", [7480])},
    "BritishMilkSheepBreed": {"131700000": ("British Milk Sheep breed", [7480])},
    "BritishRidingPonyHorseBreed": {
        "131887006": ("British Riding Pony horse breed", [7480])
    },
    "BritishSaanenGoatBreed": {"131678004": ("British Saanen goat breed", [7480])},
    "BritishSaddlebackPig": {"15961007": ("British saddleback pig (organism)", [])},
    "BritishSaddlebackPigBreed": {"15961007": ("British saddleback pig breed", [7480])},
    "BritishShorthairedCat": {"70653001": ("British shorthaired cat", [7480])},
    "BritishSpottedPonyHorseBreed": {
        "131888001": ("British Spotted Pony horse breed", [7480])
    },
    "BritishToggenburgGoatBreed": {
        "131679007": ("British Toggenburg goat breed", [7480])
    },
    "BritishWarmbloodHorseBreed": {
        "132732003": ("British Warmblood horse breed", [7480])
    },
    "BritishWhiteCattleBreed": {"131579003": ("British White cattle breed", [7480])},
    "BrittanySpaniel": {"12091005": ("Brittany spaniel (organism)", [])},
    "BrittanySpanielDogBreed": {"12091005": ("Brittany spaniel dog breed", [7480])},
    "BroadLigament": {"34411009": ("Broad ligament", [4, 7151, 7192, 8134, 9514])},
    "BroadLigamentStructure": {
        "34411009": ("Broad ligament structure (body structure)", [])
    },
    "BroadTangentialDirectIllumination": {
        "410463003": ("Broad tangential direct illumination", [4203])
    },
    "BroadTangentialDirectIlluminationAction": {
        "410463003": (
            "Broad tangential direct illumination - action (qualifier value)",
            [],
        )
    },
    "Broden": {"399344001": ("Broden", [4012])},
    "BrodenProjection": {"399344001": ("Broden projection (qualifier value)", [])},
    "Bronchial": {"955009": ("Bronchial", [6129, 8134])},
    "BronchialArtery": {
        "64468002": ("Bronchial artery", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "BronchialStructure": {"955009": ("Bronchial structure (body structure)", [])},
    "Bronchial_Inactive": {"261061003": ("Bronchial (qualifier value)", [])},
    "Bronchodilator": {"372580007": ("Bronchodilator", [621, 622])},
    "BronchopulmonarySegmentStructure": {
        "72674008": ("Bronchopulmonary segment structure (body structure)", [])
    },
    "Bronchospasm": {"4386001": ("Bronchospasm", [60, 9300])},
    "Bronchus": {
        "955009": (
            "Bronchus",
            [4, 4009, 4030, 4031, 4040, 4042, 6102, 6109, 6111, 7151, 7155, 7192, 9514],
        )
    },
    "BroselowLutenBlueZone1923kg": {
        "444474005": ("Broselow Luten Blue Zone (19-23 kg)", [7039, 7040])
    },
    "BroselowLutenGreenZone3036kg": {
        "444503000": ("Broselow Luten Green Zone (30-36 kg)", [7039, 7040])
    },
    "BroselowLutenOrangeZone2429kg": {
        "444496004": ("Broselow Luten Orange Zone (24-29 kg)", [7039, 7040])
    },
    "BroselowLutenPediatricWeightEstimationBlueZone": {
        "444474005": (
            "Broselow Luten pediatric weight estimation blue zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationGreenZone": {
        "444503000": (
            "Broselow Luten pediatric weight estimation green zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationOrangeZone": {
        "444496004": (
            "Broselow Luten pediatric weight estimation orange zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationPinkZone": {
        "444488009": (
            "Broselow Luten pediatric weight estimation pink zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationPurpleZone": {
        "444489001": (
            "Broselow Luten pediatric weight estimation purple zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationRedZone": {
        "444471002": (
            "Broselow Luten pediatric weight estimation red zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationWhiteZone": {
        "444504006": (
            "Broselow Luten pediatric weight estimation white zone (finding)",
            [],
        )
    },
    "BroselowLutenPediatricWeightEstimationYellowZone": {
        "444505007": (
            "Broselow Luten pediatric weight estimation yellow zone (finding)",
            [],
        )
    },
    "BroselowLutenPinkZone67kg": {
        "444488009": ("Broselow Luten Pink Zone (6-7 kg)", [7039, 7040])
    },
    "BroselowLutenPurpleZone1011kg": {
        "444489001": ("Broselow Luten Purple Zone (10-11 kg)", [7039, 7040])
    },
    "BroselowLutenRedZone89kg": {
        "444471002": ("Broselow Luten Red Zone (8-9 kg)", [7039, 7040])
    },
    "BroselowLutenWhiteZone1518kg": {
        "444504006": ("Broselow Luten White Zone (15-18 kg)", [7039, 7040])
    },
    "BroselowLutenYellowZone1214kg": {
        "444505007": ("Broselow Luten Yellow Zone (12-14 kg)", [7039, 7040])
    },
    "Brown": {"371254008": ("Brown", [4411])},
    "BrownAtlasCattleBreed": {"133332009": ("Brown Atlas cattle breed", [7480])},
    "BrownColor": {"371254008": ("Brown color", [6067])},
    "BrownSwissCattleBreed": {"44230005": ("Brown Swiss cattle breed (organism)", [])},
    "BrownSwissCowBreed": {"44230005": ("Brown Swiss cow breed", [7480])},
    "BrownWelshCattleBreed": {"944009": ("Brown Welsh cattle breed (organism)", [])},
    "BrownWelshCowBreed": {"944009": ("Brown Welsh cow breed", [7480])},
    "BruceProtocol": {"129095002": ("Bruce protocol", [3261, 12001])},
    "BrunoDeJuraDogBreed": {"132416002": ("Bruno de Jura dog breed", [7480])},
    "BuccalMucosa": {"16811007": ("buccal mucosa", [7601, 8134])},
    "BuccinatorLymphNode": {"143925009": ("buccinator lymph node", [7600, 8134])},
    "BudyonnyHorseBreed": {"133130006": ("Budyonny horse breed", [7480])},
    "BulboventricularForamen": {"443329006": ("Bulboventricular Foramen", [12286])},
    "BulgarianBrownCattleBreed": {
        "133333004": ("Bulgarian Brown cattle breed", [7480])
    },
    "BulgarianLandracePigBreed": {
        "133192000": ("Bulgarian Landrace pig breed", [7480])
    },
    "BulgarianNativePigBreed": {"132365008": ("Bulgarian Native pig breed", [7480])},
    "BulgarianRedCattleBreed": {"133334005": ("Bulgarian Red cattle breed", [7480])},
    "BulgarianWhitePigBreed": {"133191007": ("Bulgarian White pig breed", [7480])},
    "BullTerrier": {"23995009": ("Bull terrier", [7480])},
    "Bulldog": {"38184008": ("Bulldog", [7480])},
    "Bullet": {
        "86122002": ("Bullet", [6040, 6102, 6138, 6202, 6203, 6401, 6404, 7151, 7193])
    },
    "BulletDevice": {"86122002": ("Bullet, device (physical object)", [])},
    "Bullmastiff": {"71175006": ("Bullmastiff", [7480])},
    "Bunamiodyl": {"90745007": ("Bunamiodyl", [12, 3850])},
    "BuohaiHorseBreed": {"131889009": ("Buohai horse breed", [7480])},
    "Bupivacaine": {"387150008": ("Bupivacaine", [623, 630])},
    "BurlinaCattleBreed": {"133335006": ("Burlina cattle breed", [7480])},
    "Burman": {"422861003": ("Burman", [4012])},
    "BurmanProjection": {"422861003": ("Burman projection (qualifier value)", [])},
    "BurmeseCat": {"89065000": ("Burmese cat", [7480])},
    "BurmeseCattleBreed": {"133782001": ("Burmese cattle breed (organism)", [])},
    "BurmesePigBreed": {"133166007": ("Burmese pig breed", [7480])},
    "BurmeseXZebuCattleBreed": {"133782001": ("Burmese X zebu cattle breed", [7480])},
    "BurnsIntensiveCare": {"426439001": ("Burns Intensive Care", [7030])},
    "BurnsIntensiveCareUnit": {
        "426439001": ("Burns intensive care unit (environment)", [])
    },
    "BurwashCattleBreed": {"133336007": ("Burwash cattle breed", [7480])},
    "BuryatHorseBreed": {"131890000": ("Buryat horse breed", [7480])},
    "BusaCattleBreed": {"133577005": ("Busa cattle breed", [7480])},
    "BushuevCattleBreed": {"133783006": ("Bushuev cattle breed (organism)", [])},
    "BushuevXZebuCattleBreed": {"133783006": ("Bushuev X zebu cattle breed", [7480])},
    "Butabarbital": {"372901004": ("Butabarbital", [623, 625])},
    "ButanaCattleBreed": {"133643007": ("Butana cattle breed", [7480])},
    "ButanolO15": {"422540000": ("Butanol O^15^", [4021])},
    "Buttock": {"46862004": ("Buttock", [4, 7151, 7192, 8134, 9514])},
    "ButtockStructure": {"46862004": ("Buttock structure (body structure)", [])},
    "ButyrateEsteraseStain": {"406955006": ("butyrate esterase stain", [8112])},
    "ByInhalation": {"446406008": ("By inhalation", [11, 613, 614])},
    "ByelorussianHarnessHorseBreed": {
        "133129001": ("Byelorussian Harness horse breed", [7480])
    },
    "ByelorussianRedCattleBreed": {
        "133337003": ("Byelorussian Red cattle breed", [7480])
    },
    "ByelorussianSyntheticCattleBreed": {
        "133338008": ("Byelorussian Synthetic cattle breed", [7480])
    },
    "BypassGraftAngiography": {"252427007": ("Bypass graft angiography", [3405, 3428])},
    "BündnerGoatBreed": {"131680005": ("Bündner goat breed", [7480])},
    "BündnerOberlandSheepBreed": {
        "131699001": ("Bündner Oberland sheep breed", [7480])
    },
    "C1Vertebra": {
        "14806007": (
            "C1 vertebra",
            [1000, 1001, 1003, 7151, 7192, 7602, 7603, 8134, 9514],
        )
    },
    "C2C3Disc": {"1078209008": ("C2/C3 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C2Vertebra": {"39976000": ("C2 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "C3C4Disc": {"58820006": ("C3/C4 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C3Vertebra": {"113205007": ("C3 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "C4C5Disc": {"49400002": ("C4/C5 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C4Vertebra": {"5329002": ("C4 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "C5C6Disc": {"73959003": ("C5/C6 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C5Vertebra": {"36978003": ("C5 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "C6C7Disc": {"75095006": ("C6/C7 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C6Vertebra": {"36054005": ("C6 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "C7T1Disc": {"37414007": ("C7/T1 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "C7Vertebra": {"87391001": ("C7 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "COOPELSO93CattleBreed": {"133359001": ("COOPELSO 93 cattle breed", [7480])},
    "CPFPig": {"24840008": ("CPF pig (organism)", [])},
    "CPFPigBreed": {"24840008": ("CPF pig breed", [7480])},
    "CTAngiographyOfCoronaryArteries": {
        "419545005": ("CT angiography of coronary arteries", [3757])
    },
    "CTArteriographyOfAbdominalAortaWithContrast": {
        "444968003": ("CT arteriography of abdominal aorta with contrast", [3820])
    },
    "CTOfAbdominalAorta": {"241553009": ("CT of abdominal aorta", [3820])},
    "CTOfAbdominalAortaWithoutContrast": {
        "1187246003": ("CT of abdominal aorta without contrast", [3820])
    },
    "CTOfBreast": {"241539009": ("CT of breast", [6050, 6083])},
    "CTV": {"228792002": ("CTV", [9534])},
    "CTWithoutContrast": {"399331006": ("CT without contrast", [10014])},
    "CabanninaCattleBreed": {"133339000": ("Cabannina cattle breed", [7480])},
    "CachenaCattleBreed": {"131580000": ("Cachena cattle breed", [7480])},
    "Cachexia": {"238108007": ("Cachexia", [3205])},
    "Caffeine": {"255641001": ("Caffeine", [6089])},
    "Cahoon": {"399278009": ("Cahoon", [4012])},
    "CahoonProjection": {"399278009": ("Cahoon projection (qualifier value)", [])},
    "CairnTerrier": {"87111007": ("Cairn terrier", [7480])},
    "CaiuaCattleBreed": {"133784000": ("Caiua cattle breed (organism)", [])},
    "CaiuaXZebuCattleBreed": {"133784000": ("Caiua X zebu cattle breed", [7480])},
    "CalabrianCattleBreed": {"131457006": ("Calabrian cattle breed", [7480])},
    "CalabrianHorseBreed": {"131891001": ("Calabrian horse breed", [7480])},
    "CalabrianPigBreed": {"132096005": ("Calabrian pig breed", [7480])},
    "CalascibettaPigBreed": {"132099003": ("Calascibetta pig breed", [7480])},
    "CalasparrenaCattleBreed": {"133593001": ("Calasparrena cattle breed", [7480])},
    "CalcanealTubercle": {"82474009": ("Calcaneal tubercle", [7482, 7483, 8134])},
    "CalcanealTubercleStructure": {
        "82474009": ("Calcaneal tubercle structure (body structure)", [])
    },
    "Calcaneus": {
        "80144004": (
            "Calcaneus",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "CalcificationCluster": {
        "129769006": ("Calcification Cluster", [6014, 6015, 6016, 6054, 6056])
    },
    "CalcificationOfBreast": {"309587003": ("Calcification of breast", [6054])},
    "Calcified": {"237897009": ("Calcified", [3712])},
    "CalcifiedAneurysm": {"125273000": ("calcified aneurysm", [3808])},
    "CalcifiedAtheromatousPlaque": {
        "29483008": ("calcified atheromatous plaque", [3817])
    },
    "CalcifiedHeartValve": {"373136005": ("Calcified Heart Valve", [3711])},
    "CalcifiedSkinOfBreast": {"129757003": ("Calcified skin of breast", [6010, 6011])},
    "CalcifiedSutureMaterial": {
        "129758008": ("Calcified suture material", [6010, 6011])
    },
    "Calcium": {"5540006": ("Calcium", [300, 621, 622])},
    "CalciumChannelBlocker": {"373304005": ("Calcium channel blocker", [621, 622])},
    "CalciumChannelBlockerContainingProduct": {
        "48698004": ("Calcium channel blocker", [3760])
    },
    "CalciumHydroxyapatite": {"256579008": ("Calcium Hydroxyapatite", [300])},
    "Calculated": {"258090004": ("Calculated", [3627, 3745, 10009])},
    "CaldeanoCattleBreed": {"133340003": ("Caldeano cattle breed", [7480])},
    "CaldelanaCattleBreed": {"133341004": ("Caldelana cattle breed", [7480])},
    "Caldwell": {"399358004": ("Caldwell", [4012])},
    "CaldwellProjection": {"399358004": ("Caldwell projection (qualifier value)", [])},
    "CaliforniaRedSheepBreed": {"131795000": ("California Red sheep breed", [7480])},
    "CaliforniaSpangledCatBreed": {
        "132663009": ("California Spangled cat breed", [7480])
    },
    "CaliforniaVariegatedMutantSheepBreed": {
        "131794001": ("California Variegated Mutant sheep breed", [7480])
    },
    "CallithrixJacchus": {"406733009": ("Callithrix jacchus", [7454])},
    "CaloricAgent": {"373530005": ("Caloric agent", [621, 622])},
    "CalvanaCattleBreed": {"133342006": ("Calvana cattle breed", [7480])},
    "Calyx": {"2334006": ("Calyx", [4, 7151, 7192, 8134, 9514])},
    "CamargueCattleBreed": {"133343001": ("Camargue cattle breed", [7480])},
    "CamargueHorseBreed": {"131892008": ("Camargue horse breed", [7480])},
    "CamarronGoat": {"9230001": ("Camarron goat (organism)", [])},
    "CamarronGoatBreed": {"9230001": ("Camarron goat breed", [7480])},
    "CambodianCattleBreed": {"133344007": ("Cambodian cattle breed", [7480])},
    "Camborough12PigBreed": {"132048005": ("Camborough 12 pig breed", [7480])},
    "Camborough15PigBreed": {"132211005": ("Camborough 15 pig breed", [7480])},
    "Camborough22PigBreed": {"132210006": ("Camborough 22 pig breed", [7480])},
    "CamboroughBluePigBreed": {"132047000": ("Camborough Blue pig breed", [7480])},
    "CambridgeSheepBreed": {"131802002": ("Cambridge sheep breed", [7480])},
    "CampCoventry": {"399212002": ("Camp-Coventry", [4012])},
    "CampCoventryProjection": {
        "399212002": ("Camp-Coventry projection (qualifier value)", [])
    },
    "CampanianBarbarySheepBreed": {
        "131793007": ("Campanian Barbary sheep breed", [7480])
    },
    "CampolinaHorseBreed": {"133128009": ("Campolina horse breed", [7480])},
    "CanaanDogBreed": {"132552000": ("Canaan dog breed", [7480])},
    "CanadianArcottSheepBreed": {"132689009": ("Canadian Arcott sheep breed", [7480])},
    "CanadianCattleBreed": {"21921002": ("Canadian cattle breed (organism)", [])},
    "CanadianCowBreed": {"21921002": ("Canadian cow breed", [7480])},
    "CanadianCuttingHorseHorseBreed": {
        "131893003": ("Canadian Cutting Horse horse breed", [7480])
    },
    "CanadianDurocPigBreed": {"132218004": ("Canadian Duroc pig breed", [7480])},
    "CanadianEskimoDog": {"91553005": ("Canadian eskimo dog (organism)", [])},
    "CanadianEskimoDogBreed": {"91553005": ("Canadian eskimo dog breed", [7480])},
    "CanadianHampshirePigBreed": {
        "132219007": ("Canadian Hampshire pig breed", [7480])
    },
    "CanadianHorse": {"47842004": ("Canadian horse (organism)", [])},
    "CanadianHorseBreed": {"47842004": ("Canadian horse breed", [7480])},
    "CanadianLandracePigBreed": {"132034005": ("Canadian Landrace pig breed", [7480])},
    "CanadianRoyalBluePigBreed": {
        "132215001": ("Canadian Royal Blue pig breed", [7480])
    },
    "CanadianRusticPonyHorseBreed": {
        "131894009": ("Canadian Rustic Pony horse breed", [7480])
    },
    "CanadianSportHorseHorseBreed": {
        "131895005": ("Canadian Sport Horse horse breed", [7480])
    },
    "CanadianYorkshirePigBreed": {
        "132035006": ("Canadian Yorkshire pig breed", [7480])
    },
    "CanaryIslandCattleBreed": {"131581001": ("Canary Island cattle breed", [7480])},
    "CanaryIslandGoatBreed": {"131652008": ("Canary Island goat breed", [7480])},
    "CanastraPigBreed": {"132054006": ("Canastra pig breed", [7480])},
    "CanastrinhoPigBreed": {"132065001": ("Canastrinho pig breed", [7480])},
    "CanastrãoCabanoPigBreed": {"132062003": ("Canastrão, Cabano pig breed", [7480])},
    "CanastrãoCapitãoChicoPigBreed": {
        "132060006": ("Canastrão, Capitão Chico pig breed", [7480])
    },
    "CanastrãoJunqueiraPigBreed": {
        "132059001": ("Canastrão, Junqueira pig breed", [7480])
    },
    "CanastrãoPigBreed": {"132058009": ("Canastrão pig breed", [7480])},
    "CanastrãoVermelhoPigBreed": {
        "132063008": ("Canastrão, Vermelho pig breed", [7480])
    },
    "CanastrãoZabumbaPigBreed": {"132061005": ("Canastrão, Zabumba pig breed", [7480])},
    "CancerOfCervix": {"363354003": ("Cancer of cervix", [638, 639])},
    "CancerOfColon": {"363406005": ("Cancer of colon", [638, 639])},
    "CancerOfSalivaryGland": {"255072001": ("Cancer of salivary gland", [638, 639])},
    "CanchimCattleBreed": {"133718009": ("Canchim cattle breed (organism)", [])},
    "CanchimXZebuCattleBreed": {"133718009": ("Canchim X zebu cattle breed", [7480])},
    "CaneCorsoDogBreed": {"132563002": ("Cane Corso dog breed", [7480])},
    "CanikHorseBreed": {"131896006": ("Canik horse breed", [7480])},
    "CanindéGoatBreed": {"131651001": ("Canindé goat breed", [7480])},
    "CanineRegion": {"699510004": ("Canine region", [4017])},
    "CanineRegionOfOralCavity": {
        "699510004": ("Canine region of oral cavity (body structure)", [])
    },
    "CanineSpecies": {"69986009": ("Canine species (organism)", [])},
    "Canis": {"388490000": ("Canis", [7454])},
    "CanisLupus": {"36855005": ("Canis lupus", [7454])},
    "CanisLupusDingo": {"709853007": ("Canis lupus dingo (organism)", [])},
    "CanisLupusFamiliaris": {"448771007": ("Canis lupus familiaris", [7454])},
    "CanisLupusSubspeciesFamiliaris": {
        "448771007": ("Canis lupus subspecies familiaris (organism)", [])
    },
    "Cannabis": {"398705004": ("Cannabis (substance)", [])},
    "CannotReachTargetHeartRate": {
        "429733000": ("Cannot reach target heart rate", [3205])
    },
    "CantonesePigBreed": {"132292004": ("Cantonese pig breed", [7480])},
    "CapeHorseHorseBreed": {"131897002": ("Cape Horse horse breed", [7480])},
    "Capillary": {"20982000": ("Capillary", [7151, 7166, 7191])},
    "Capra": {"388249000": ("Capra", [7454])},
    "CapraHircus": {"125097000": ("Capra hircus", [7454])},
    "CaprineSpecies": {"68552000": ("Caprine species (organism)", [])},
    "Capsular": {"11070000": ("Capsular", [2, 5])},
    "CaptainCookerPigBreed": {"133184009": ("Captain Cooker pig breed", [7480])},
    "CaracuCattleBreed": {"133345008": ("Caracu cattle breed", [7480])},
    "CarazebuCattleBreed": {"133795006": ("Carazebu cattle breed (organism)", [])},
    "CarazebuXZebuCattleBreed": {"133795006": ("Carazebu X zebu cattle breed", [7480])},
    "Carbapenem": {"396345004": ("Carbapenem (substance)", [])},
    "CarbapenemAntibiotic": {"396345004": ("Carbapenem antibiotic", [621, 622])},
    "CarbolFuchsinStain": {"764166003": ("carbol fuchsin stain", [8112])},
    "Carbon14DXylose": {"2942001": ("Carbon^14^ D-xylose", [25])},
    "Carbon14Triolein": {"42417005": ("Carbon^14^ triolein", [25])},
    "CarbonDioxide": {"31811003": ("Carbon dioxide", [12, 13, 623, 624, 3850])},
    "CarbonDioxideO15": {"129507008": ("Carbon dioxide O^15^", [4021])},
    "CarbonFiber": {"256501007": ("Carbon Fiber", [7300, 9579, 10067])},
    "CarbonMonoxideC11": {"129515006": ("Carbon monoxide C^11^", [4021])},
    "CarbonMonoxideO15": {"129506004": ("Carbon monoxide O^15^", [4021])},
    "Carcinogen": {"88376000": ("Carcinogen", [602])},
    "Carcinoma": {"1187425009": ("Carcinoma", [638, 639])},
    "CarcinomaInSituOfMaleBreast": {
        "92652009": ("Carcinoma in situ of male breast", [6030, 6033])
    },
    "CarcinomaNoSubtype": {
        "68453008": ("Carcinoma, no subtype (morphologic abnormality)", [])
    },
    "CarcinomaOfProstate": {"254900004": ("Carcinoma of prostate (disorder)", [])},
    "CarcinomaWithMetaplasia": {
        "22694002": ("Carcinoma with metaplasia", [6030, 6033])
    },
    "Carcinosarcoma": {"63264007": ("Carcinosarcoma", [638, 639, 6030, 6033])},
    "CarcinosarcomaOfUterus": {"702369008": ("Carcinosarcoma of uterus", [638, 639])},
    "CardiacAdrenergicBlockingAgent": {
        "373263004": ("Cardiac adrenergic blocking agent", [10])
    },
    "CardiacAkinesia": {"195675009": ("Cardiac akinesia (finding)", [])},
    "CardiacArrest": {"410429000": ("Cardiac arrest", [3413, 3755])},
    "CardiacArrhythmia": {"698247007": ("Cardiac Arrhythmia", [60, 1201, 9300])},
    "CardiacBloodPoolImaging": {
        "35621002": ("Cardiac blood pool imaging (nuclear)", [3206, 3757])
    },
    "CardiacCT": {"241547009": ("Cardiac CT", [3757])},
    "CardiacCTForCalciumScoring": {
        "426005005": ("Cardiac CT for calcium scoring", [3757])
    },
    "CardiacCathCoronaryAngiogramAndLeftVentriculogram": {
        "418903008": ("Cardiac cath coronary angiogram and left ventriculogram", [3757])
    },
    "CardiacCatheterization": {"41976001": ("Cardiac catheterization", [3757])},
    "CardiacCatheterizationBailoutPhase": {
        "128961006": ("Cardiac catheterization bailout phase", [3250, 3405])
    },
    "CardiacCatheterizationBaselinePhase": {
        "128955008": ("Cardiac catheterization baseline phase", [3250, 3405, 3651])
    },
    "CardiacCatheterizationCoronaryAngiogram": {
        "419416005": ("Cardiac catheterization coronary angiogram", [3757])
    },
    "CardiacCatheterizationGradientAssessmentPhase": {
        "371874005": ("Cardiac catheterization gradient assessment phase", [3651])
    },
    "CardiacCatheterizationImageAcquisitionPhase": {
        "128956009": ("Cardiac catheterization image acquisition phase", [3250, 3405])
    },
    "CardiacCatheterizationInterventionPhase": {
        "128957000": ("Cardiac catheterization intervention phase", [3250, 3405])
    },
    "CardiacCatheterizationLeftHeartTransseptal": {
        "67338003": ("Cardiac catheterization, left heart, transseptal (procedure)", [])
    },
    "CardiacCatheterizationPostContrastPhase": {
        "129083002": ("Cardiac catheterization post contrast phase", [3250, 3405, 3651])
    },
    "CardiacCatheterizationPostInterventionPhase": {
        "128960007": (
            "Cardiac catheterization post-intervention phase",
            [3250, 3405, 3480, 3651],
        )
    },
    "CardiacCatheterizationPreInterventionPhase": {
        "128958005": (
            "Cardiac catheterization pre-intervention phase",
            [3250, 3405, 3480],
        )
    },
    "CardiacCatheterizationProcedurePhase": {
        "129085009": ("Cardiac catheterization procedure phase (qualifier value)", [])
    },
    "CardiacCatheterizationTestChallengePhase": {
        "373105002": (
            "Cardiac catheterization test/challenge phase",
            [3250, 3405, 3651],
        )
    },
    "CardiacCatheterizationTherapyPhase": {
        "128959002": ("Cardiac catheterization therapy phase", [3250, 3405])
    },
    "CardiacComputedTomographyForCalciumScoring": {
        "426005005": ("Cardiac computed tomography for calcium scoring (procedure)", [])
    },
    "CardiacCyclePhase": {"246092007": ("Cardiac cycle phase (attribute)", [])},
    "CardiacDyskinesia": {"25437005": ("Cardiac dyskinesia (finding)", [])},
    "CardiacEjectionFraction": {
        "70822001": (
            "Cardiac ejection fraction",
            [
                12252,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "CardiacEjectionFractionFunction": {
        "70822001": ("Cardiac ejection fraction, function (observable entity)", [])
    },
    "CardiacFunctionTestAbnormal": {
        "165076002": ("cardiac function test abnormal", [3727])
    },
    "CardiacIndex": {
        "54993008": (
            "Cardiac Index",
            [
                12200,
                12207,
                12239,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "CardiacIntensiveCare": {"309907008": ("Cardiac Intensive Care", [7030])},
    "CardiacIntensiveCareUnit": {
        "309907008": ("Cardiac intensive care unit (environment)", [])
    },
    "CardiacMRI": {"241620005": ("Cardiac MRI", [3206, 3757, 3820])},
    "CardiacMRIStress": {"431609005": ("Cardiac MRI stress", [3757])},
    "CardiacMuscle": {"122448007": ("Cardiac muscle", [7710, 8134])},
    "CardiacMuscleTissue": {
        "122448007": ("Cardiac muscle (tissue) (body structure)", [])
    },
    "CardiacOutput": {
        "82799009": (
            "Cardiac Output",
            [
                12200,
                12207,
                12239,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "CardiacOutputMeasurementByThermalBathMethod": {
        "371838001": (
            "Cardiac output measurement by thermal bath method (procedure)",
            [],
        )
    },
    "CardiacOutputMeasurementByThermalDyeDilutionMethod": {
        "373104003": (
            "Cardiac output measurement by thermal dye dilution method (procedure)",
            [],
        )
    },
    "CardiacOutputMeasurementByThermalInlineMethod": {
        "371843008": (
            "Cardiac output measurement by thermal inline method (procedure)",
            [],
        )
    },
    "CardiacPacemaker": {
        "14106009": (
            "Cardiac pacemaker",
            [1000, 3777, 6102, 6138, 6404, 7151, 7157, 7193],
        )
    },
    "CardiacPacemakerDevice": {
        "14106009": ("Cardiac pacemaker, device (physical object)", [])
    },
    "CardiacPacemakerInSitu": {
        "441509002": ("Cardiac pacemaker in situ (finding)", [])
    },
    "CardiacPacemakerLead": {
        "360129009": ("Cardiac pacemaker lead", [6102, 6138, 6404, 7151, 7193])
    },
    "CardiacPacing": {"18590009": ("Cardiac pacing", [12002])},
    "CardiacSeptumProsthesis": {
        "118375008": ("Vascular graft", [4, 7151, 7192, 8134, 9514])
    },
    "CardiacShuntStudy": {"241213007": ("Cardiac shunt study (procedure)", [])},
    "CardiacStressRecoveryState": {
        "432554001": ("Cardiac stress recovery state", [3207, 12002])
    },
    "CardiacStressState": {"432655005": ("Cardiac Stress State", [3101, 3207, 12002])},
    "CardiacSurgery": {"309971008": ("Cardiac Surgery", [7030])},
    "CardiacSurgeryDepartment": {
        "309971008": ("Cardiac surgery department (environment)", [])
    },
    "CardiacTamponade": {"35304003": ("Cardiac tamponade", [3413, 3755])},
    "CardiacValveAnnulus": {"305437000": ("Cardiac valve annulus", [12280])},
    "CardiacValveLeaflet": {"6530003": ("Cardiac Valve Leaflet", [12280])},
    "CardiacValveProsthesisDevice": {
        "25510005": ("Cardiac valve prosthesis, device (physical object)", [])
    },
    "CardiacVentricularStructure": {
        "21814001": ("Cardiac ventricular structure (body structure)", [])
    },
    "CardiacVentriculography": {
        "252426003": ("Cardiac ventriculography", [3405, 3428, 3744])
    },
    "CardiacWallMotionFunction": {
        "60797005": ("Cardiac wall motion, function (observable entity)", [])
    },
    "CardiacWallStructure": {
        "272657006": ("Cardiac wall structure (body structure)", [])
    },
    "CardiganWelshCorgi": {"60517007": ("Cardigan Welsh corgi (organism)", [])},
    "CardiganWelshCorgiDogBreed": {
        "60517007": ("Cardigan Welsh corgi dog breed", [7480])
    },
    "CardiogenicShock": {
        "89138009": ("Cardiogenic shock", [3413, 3700, 3726, 3727, 3755])
    },
    "Cardiology": {"309915006": ("Cardiology", [7030])},
    "CardiologyDepartment": {"309915006": ("Cardiology department (environment)", [])},
    "Cardiomegaly": {"287272002": ("Cardiomegaly", [12248, 12249])},
    "Cardiomyopathy": {"85898001": ("Cardiomyopathy", [64, 3727])},
    "CardiophrenicAngleLymphNode": {
        "371013005": ("cardiophrenic angle lymph node", [7600, 8134])
    },
    "CardiopulmonaryBypass": {"63697000": ("Cardiopulmonary bypass", [3553])},
    "CardiopulmonaryBypassOperation": {
        "63697000": ("Cardiopulmonary bypass operation (procedure)", [])
    },
    "CardiorespiratoryArrest": {"410430005": ("Cardiorespiratory arrest", [60, 9300])},
    "CardiothoracicSurgery": {"309969008": ("Cardiothoracic Surgery", [7030])},
    "CardiothoracicSurgeryDepartment": {
        "309969008": ("Cardiothoracic surgery department (environment)", [])
    },
    "CardiovascularCT": {"303680000": ("cardiovascular CT", [3820])},
    "CardiovascularCyclePoint": {
        "272518008": ("Cardiovascular cycle point (qualifier value)", [])
    },
    "CardiovascularEventRisk": {
        "395112001": ("Cardiovascular event risk (finding)", [])
    },
    "CardiovascularOrificeArea": {
        "399367004": (
            "Cardiovascular Orifice Area",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "CardiovascularOrificeDiameter": {
        "399027007": (
            "Cardiovascular Orifice Diameter",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "CardiovascularPressureGradient": {
        "251081004": ("Cardiovascular pressure gradient (observable entity)", [])
    },
    "CardiovascularStressTestUsingBicycleErgometer": {
        "26046004": (
            "Cardiovascular stress test using bicycle ergometer (procedure)",
            [],
        )
    },
    "CardiovascularStressTestUsingPharmacologicStressAgent": {
        "424064009": (
            "Cardiovascular stress test using pharmacologic stress agent (procedure)",
            [],
        )
    },
    "CardiovascularStressTestUsingTheAdenosineStressTestProtocol": {
        "424444005": (
            "Cardiovascular stress test using the adenosine stress test protocol (procedure)",
            [],
        )
    },
    "CardiovascularStressTestUsingTheDipyridamoleStressTestProtocol": {
        "422685009": (
            "Cardiovascular stress test using the dipyridamole stress test protocol (procedure)",
            [],
        )
    },
    "CardiovascularStressTestUsingTheDobutamineStressTestProtocol": {
        "424225000": (
            "Cardiovascular stress test using the dobutamine stress test protocol (procedure)",
            [],
        )
    },
    "CardiovascularSystem": {
        "113257007": ("Cardiovascular system", [7151, 7156, 7192, 8134, 9514])
    },
    "CareOfTheElderlyDepartment": {
        "309933000": ("Care of the elderly department (environment)", [])
    },
    "CarfentanilC11": {"129511002": ("Carfentanil C^11^", [4021])},
    "Carina": {
        "28700002": (
            "Carina",
            [1000, 1001, 1004, 6102, 6109, 6111, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "CarinthianBlondCattleBreed": {
        "131582008": ("Carinthian Blond cattle breed", [7480])
    },
    "CarmineStain": {"73892005": ("carmine stain", [8112])},
    "CarminicAcidStain": {"432003": ("carminic acid stain", [8112])},
    "CarmoisineAStain": {"37575004": ("carmoisine A stain", [8112])},
    "CarnoyFixative": {"433338005": ("Carnoy fixative (substance)", [])},
    "CarnoyFluid": {"433338005": ("Carnoy's fluid", [8114])},
    "CarolinaDogBreed": {"132544003": ("Carolina Dog breed", [7480])},
    "CarotidArtery": {
        "69105007": (
            "Carotid Artery",
            [
                4,
                1000,
                1001,
                1002,
                3010,
                3440,
                3604,
                3606,
                3630,
                4030,
                4042,
                7151,
                7192,
                8134,
                9514,
            ],
        )
    },
    "CarotidArteryStructure": {
        "69105007": ("Carotid artery structure (body structure)", [])
    },
    "CarotidBifurcation": {
        "80272002": (
            "Carotid Bifurcation",
            [3827, 7151, 7192, 8134, 9514, 12103, 12104],
        )
    },
    "CarotidBody": {
        "51345006": ("Carotid Body", [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514])
    },
    "CarotidBodyStructure": {
        "51345006": ("Carotid body structure (body structure)", [])
    },
    "CarotidBulb": {
        "21479005": ("Carotid Bulb", [3827, 7151, 7192, 8134, 9514, 12103, 12104])
    },
    "CarotidSinusMassageProcedurePhase": {
        "129090007": ("Carotid Sinus Massage procedure phase", [3254])
    },
    "CarotidSiphon": {
        "54409005": ("Carotid Siphon", [3827, 7151, 7192, 8134, 9514, 12103, 12105])
    },
    "CarpatheGoatBreed": {"131687008": ("Carpathe goat breed", [7480])},
    "CarpathianBrownCattleBreed": {
        "133346009": ("Carpathian Brown cattle breed", [7480])
    },
    "CarpathianPonyHorseBreed": {"133138004": ("Carpathian Pony horse breed", [7480])},
    "Carpus": {"8205005": ("Carpus", [7482, 7483, 8134])},
    "CarthusianHorseBreed": {"133127004": ("Carthusian horse breed", [7480])},
    "Cartilage": {"309312004": ("Cartilage", [7151, 7166, 7191])},
    "CartilageTissue": {"309312004": ("Cartilage tissue (body structure)", [])},
    "Cartridge": {"706440002": ("Cartridge", [68])},
    "CarunchinhoPigBreed": {"132164004": ("Carunchinho pig breed", [7480])},
    "CasanarenoCattleBreed": {"133347000": ("Casanareno cattle breed", [7480])},
    "CasertanaPigBreed": {"132255004": ("Casertana pig breed", [7480])},
    "CaspianHorseBreed": {"133126008": ("Caspian horse breed", [7480])},
    "CastlemilkMooritSheepBreed": {
        "131792002": ("Castlemilk Moorit sheep breed", [7480])
    },
    "CatahoulaLeopardDogBreed": {"132545002": ("Catahoula Leopard dog breed", [7480])},
    "CatalanSheepdogBreed": {"132420003": ("Catalan Sheepdog breed", [7480])},
    "CatalanaChickenBreed": {"131998006": ("Catalana chicken breed", [7480])},
    "CatalinaIslandPigBreed": {"132038008": ("Catalina Island pig breed", [7480])},
    "Cataract": {"193570009": ("Cataract (disorder)", [])},
    "CatchingBreath": {"248584002": ("Catching breath (finding)", [])},
    "Catheter": {
        "19923001": (
            "Catheter",
            [
                8,
                69,
                3451,
                4051,
                6040,
                6102,
                6138,
                6202,
                6203,
                6401,
                6404,
                7111,
                7112,
                7151,
                7193,
            ],
        )
    },
    "CatheterDevice": {"19923001": ("Catheter, device (physical object)", [])},
    "CatheterManipulation": {"103712006": ("Catheter manipulation", [9])},
    "CatheterReplacement": {"103713001": ("Catheter replacement", [9])},
    "Catheterization": {"45211000": ("Catheterization", [9])},
    "CatheterizationOfBothLeftAndRightHeartWithGraft": {
        "128952006": (
            "Catheterization of both left and right heart with graft",
            [3250, 3405, 3739],
        )
    },
    "CatheterizationOfBothLeftAndRightHeartWithoutGraft": {
        "128953001": (
            "Catheterization of both left and right heart without graft",
            [3250, 3405, 3739],
        )
    },
    "CatheterizationOfLeftHeart": {
        "67629009": ("Catheterization of left heart", [3250, 3405, 3739])
    },
    "CatheterizationOfRightHeart": {
        "40403005": ("Catheterization of right heart", [3250, 3405, 3739])
    },
    "Caucasian": {"413773004": ("Caucasian (racial group)", [])},
    "CaucasianCattleBreed": {"131583003": ("Caucasian cattle breed", [7480])},
    "CaucasianMountainDogBreed": {
        "132546001": ("Caucasian Mountain Dog breed", [7480])
    },
    "CaucasianRace": {"413773004": ("Caucasian race", [6099])},
    "CaucasianShepherdDogBreed": {
        "132421004": ("Caucasian Shepherd Dog breed", [7480])
    },
    "Caudad": {"399162004": ("caudad", [502, 4011])},
    "Caudal": {"3583002": ("Caudal", [2, 5, 23, 502])},
    "CaudalLAO": {"408724004": ("Caudal LAO", [3466])},
    "CaudalLeftAnteriorObliqueProjection": {
        "408724004": ("Caudal left anterior oblique projection (qualifier value)", [])
    },
    "CaudalRAO": {"408726002": ("Caudal RAO", [3466])},
    "CaudalRightAnteriorObliqueProjection": {
        "408726002": ("Caudal right anterior oblique projection (qualifier value)", [])
    },
    "CaudateNucleus": {
        "11000004": ("Caudate Nucleus", [7140, 7151, 7153, 7192, 8134, 9514])
    },
    "CaudateNucleusStructure": {
        "11000004": ("Caudate nucleus structure (body structure)", [])
    },
    "CaudoCranial": {"399196006": ("caudo-cranial", [501, 4010, 4014])},
    "CaudoCranialProjection": {
        "399196006": ("Caudo-cranial projection (qualifier value)", [])
    },
    "Caudocranial": {"399196006": ("Caudocranial", [7484])},
    "CaudodistalCranioproximalOblique": {
        "442604001": ("Caudodistal-cranioproximal oblique", [7484])
    },
    "CaudodistalCranioproximalObliqueProjection": {
        "442604001": (
            "Caudodistal-cranioproximal oblique projection (qualifier value)",
            [],
        )
    },
    "Causton": {"399065000": ("Causton", [4012])},
    "CaustonProjection": {"399065000": ("Causton projection (qualifier value)", [])},
    "CavalierKingCharlesSpaniel": {
        "66495005": ("Cavalier King Charles spaniel", [7480])
    },
    "CavallinoPigBreed": {"132095009": ("Cavallino pig breed", [7480])},
    "CaviaPorcellus": {"125076001": ("Cavia porcellus", [7454])},
    "CavumOfSeptumPellucidum": {"74968005": ("Cavum of septum pellucidum", [7140])},
    "CayuseHorseBreed": {"133125007": ("Cayuse horse breed", [7480])},
    "Cecum": {"32713005": ("Cecum", [6210, 8134])},
    "CecumStructure": {"32713005": ("Cecum structure (body structure)", [])},
    "CelestineBlueBStain": {"38707008": ("celestine blue B stain", [8112])},
    "CeliacAxis": {"57850000": ("Celiac Axis", [3827, 12103, 12112, 12125])},
    "CeliacLymphNode": {"47985009": ("celiac lymph node", [7600, 8134])},
    "Cell": {"4421005": ("Cell", [8135])},
    "CellStructure": {"4421005": ("Cell structure (cell structure)", [])},
    "CementedComponentFixation": {"257771002": ("Cemented component fixation", [7310])},
    "Central": {"26216008": ("Central", [2, 5, 211, 212, 3019, 4005, 6124, 6125])},
    "CentralAortopulmonaryShuntOperation": {
        "233224003": ("Central aortopulmonary shunt operation", [12247])
    },
    "CentralAsianZebuCattleBreed": {
        "133796007": ("Central Asian Zebu cattle breed (organism)", [])
    },
    "CentralAsianZebuXZebuCattleBreed": {
        "133796007": ("Central Asian Zebu X zebu cattle breed", [7480])
    },
    "CentralAxillaryLymphNode": {
        "283001": ("central axillary lymph node", [7600, 8134])
    },
    "CentralIncisorRegion": {"699453001": ("Central incisor region", [4017])},
    "CentralIncisorRegionOfOralCavity": {
        "699453001": ("Central incisor region of oral cavity (body structure)", [])
    },
    "CentralNervousSystem": {
        "21483005": ("Central nervous system", [645, 7151, 7153, 7192, 8134, 9514])
    },
    "CentralPortionOfBreastPositionOnMammogram": {
        "129786000": ("Central portion of breast position on mammogram (finding)", [])
    },
    "CentralRegionOfBreast": {"129786000": ("Central region of breast", [6018, 6019])},
    "CentralRetinalArtery": {
        "76117006": (
            "Central Retinal Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "CentralRetinalVein": {
        "62869001": (
            "Central Retinal Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "CentralRussianBlackPiedCattleBreed": {
        "133348005": ("Central Russian Black Pied cattle breed", [7480])
    },
    "CentralRussianPigBreed": {"132072000": ("Central Russian pig breed", [7480])},
    "CentralVenousCatheter": {"52124006": ("Central venous catheter", [74])},
    "CentralVenousCatheterDevice": {
        "52124006": ("Central venous catheter, device (physical object)", [])
    },
    "CentralVenousPressureWaveform": {
        "128445003": ("Central venous pressure waveform", [3003])
    },
    "CentralVenousPressureWaveformFunction": {
        "128445003": (
            "Central venous pressure waveform, function (observable entity)",
            [],
        )
    },
    "CentralVenousSystem": {"34340008": ("Central venous system", [3607, 3630])},
    "CentralZoneOfLeftHalfProstate": {
        "716901006": (
            "Central zone of left half prostate",
            [6300, 6301, 6302, 6303, 6304],
        )
    },
    "CentralZoneOfRightHalfProstate": {
        "716900007": (
            "Central zone of right half prostate",
            [6300, 6301, 6302, 6303, 6304],
        )
    },
    "CentrallyActingHypotensiveAgent": {
        "4382004": ("Centrally acting hypotensive agent", [10])
    },
    "Cephalad": {"399196006": ("cephalad", [502, 4011])},
    "Cephalic": {"66787007": ("Cephalic", [2, 23, 502])},
    "CephalicVein": {
        "20699002": (
            "Cephalic vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "CephalosporinAntibiotic": {"764147003": ("Cephalosporin antibiotic", [621, 622])},
    "CephalosporinClassOfAntibiotic": {
        "373262009": ("Cephalosporin -class of antibiotic- (substance)", [])
    },
    "CerbatHorseBreed": {"131898007": ("Cerbat horse breed", [7480])},
    "CerebellarCortex": {"25991003": ("Cerebellar Cortex", [7140])},
    "CerebellarCortexStructure": {
        "25991003": ("Cerebellar cortex structure (body structure)", [])
    },
    "CerebellarStructure": {"113305005": ("Cerebellar structure (body structure)", [])},
    "CerebellarSubarachnoidSpace": {
        "263972004": ("Cerebellar Subarachnoid Space", [7140])
    },
    "CerebellarWhiteMatter": {
        "33060004": (
            "Cerebellar White Matter",
            [7140, 7151, 7153, 7192, 7710, 8134, 9514],
        )
    },
    "CerebellarWhiteMatterStructure": {
        "33060004": ("Cerebellar white matter structure (body structure)", [])
    },
    "Cerebellum": {
        "113305005": (
            "Cerebellum",
            [4, 4030, 7140, 7151, 7153, 7192, 8134, 9514, 12020, 12022],
        )
    },
    "CerebralAqueduct": {
        "80447000": ("Cerebral aqueduct", [7151, 7153, 7192, 8134, 9514])
    },
    "CerebralArtery": {
        "88556005": (
            "Cerebral artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CerebralCortex": {"40146001": ("Cerebral cortex", [7151, 7153, 7192, 8134, 9514])},
    "CerebralFornix": {"87463005": ("Cerebral fornix", [7151, 7153, 7192, 8134, 9514])},
    "CerebralFornixStructure": {
        "87463005": ("Cerebral fornix structure (body structure)", [])
    },
    "CerebralGrayMatter": {"40146001": ("Cerebral Gray Matter", [7140])},
    "CerebralHemisphere": {
        "372073000": ("Cerebral hemisphere", [7151, 7153, 7192, 8134, 9514, 12022])
    },
    "CerebralHemisphereStructure": {
        "372073000": ("Cerebral hemisphere structure (body structure)", [])
    },
    "CerebralWhiteMatter": {
        "68523003": (
            "Cerebral White Matter",
            [7140, 7151, 7153, 7192, 7710, 8134, 9514],
        )
    },
    "CerebralWhiteMatterStructure": {
        "68523003": ("Cerebral white matter structure (body structure)", [])
    },
    "CerebrospinalFluid": {
        "65216001": ("Cerebrospinal Fluid", [7151, 7153, 7192, 8134, 9514])
    },
    "CerebrovascularAccident": {
        "230690007": ("Cerebrovascular accident (disorder)", [])
    },
    "CerebrovascularAccidentOrStroke": {
        "230690007": ("Cerebrovascular Accident or Stroke", [3413, 3755])
    },
    "CervicalCollar": {"63562005": ("Cervical collar", [6102, 6138, 6404, 7151, 7193])},
    "CervicalCollarDevice": {
        "63562005": ("Cervical collar, device (physical object)", [])
    },
    "CervicalLymphNode": {"81105003": ("cervical lymph node", [7600, 8134])},
    "CervicalLymphNodeStructure": {
        "81105003": ("Cervical lymph node structure (body structure)", [])
    },
    "CervicalSpine": {
        "122494005": (
            "Cervical spine",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "CervicoThoracicSpine": {
        "1217257000": (
            "Cervico-thoracic spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "CervicothoracicRegionOfSpineStructure": {
        "297171002": ("Cervicothoracic region of spine structure (body structure)", [])
    },
    "Cervix": {
        "71252005": (
            "Cervix",
            [4, 645, 4040, 6202, 6204, 7151, 7160, 7192, 8134, 9514, 12023],
        )
    },
    "CervixUteriStructure": {
        "71252005": ("Cervix uteri structure (body structure)", [])
    },
    "CeskyFousekDogBreed": {"132547005": ("Cesky Fousek dog breed", [7480])},
    "CeskyTerrierDogBreed": {"132548000": ("Cesky Terrier dog breed", [7480])},
    "CestrumParqui": {"13653002": ("Cestrum parqui", [7480])},
    "Chair": {"706699008": ("Chair", [7151, 7157, 7193, 9505, 9515, 9520, 9573])},
    "ChakouyiHorseBreed": {"131899004": ("Chakouyi horse breed", [7480])},
    "ChamDocCattleBreed": {"131459009": ("Cham-Doc cattle breed", [7480])},
    "Chamberlain": {"399148006": ("Chamberlain", [4012])},
    "ChamberlainProjection": {
        "399148006": ("Chamberlain projection (qualifier value)", [])
    },
    "ChamoisOfTheAlpsGoatBreed": {
        "131685000": ("Chamois of the Alps goat breed", [7480])
    },
    "ChamoiseeGoat": {"89708009": ("Chamoisee goat (organism)", [])},
    "ChamoiseeGoatBreed": {"89708009": ("Chamoisee goat breed", [7480])},
    "ChanDocCattleBreed": {"131459009": ("Chan-Doc cattle breed (organism)", [])},
    "ChanniCattleBreed": {"133600004": ("Channi cattle breed", [7480])},
    "ChantillyTiffanyCatBreed": {"132664003": ("Chantilly/Tiffany cat breed", [7480])},
    "ChaouiaCattleBreed": {"133349002": ("Chaouia cattle breed", [7480])},
    "ChaparGoatBreed": {"131650000": ("Chapar goat breed", [7480])},
    "CharSwissCattleBreed": {"133351003": ("Char-swiss cattle breed", [7480])},
    "CharaHorseHorseBreed": {"131900009": ("Chara Horse horse breed", [7480])},
    "CharbrayCattleBreed": {"133719001": ("Charbray cattle breed (organism)", [])},
    "CharbrayXZebuCattleBreed": {"133719001": ("Charbray X zebu cattle breed", [7480])},
    "CharfordCattleBreed": {"133797003": ("Charford cattle breed (organism)", [])},
    "CharfordXZebuCattleBreed": {"133797003": ("Charford X zebu cattle breed", [7480])},
    "CharnequeriaGoatBreed": {"131686004": ("Charnequeria goat breed", [7480])},
    "CharolaisCattleBreed": {"131584009": ("Charolais cattle breed", [7480])},
    "CharollaisSheepBreed": {"131791009": ("Charollais sheep breed", [7480])},
    "CharollandaisCattleBreed": {"133350002": ("Charollandais cattle breed", [7480])},
    "ChartPolskiDogBreed": {"132549008": ("Chart Polski dog breed", [7480])},
    "ChartreuxCat": {"43529009": ("Chartreux cat", [7480])},
    "ChassardLapinProjection": {
        "399013002": ("Chassard-Lapin projection (qualifier value)", [])
    },
    "ChassardLapine": {"399013002": ("Chassard-Lapine", [4012])},
    "Chausse": {"399355001": ("Chausse", [4012])},
    "ChausseProjection": {"399355001": ("Chausse projection (qualifier value)", [])},
    "Cheek": {"60819002": ("Cheek", [4, 7151, 7192, 8134, 9514])},
    "CheekStructure": {"60819002": ("Cheek structure (body structure)", [])},
    "ChejuHorseBreed": {"133124006": ("Cheju horse breed", [7480])},
    "Chemotherapy": {"367336001": ("Chemotherapy", [6083, 6085])},
    "ChenghuaPigBreed": {"132144009": ("Chenghua pig breed", [7480])},
    "ChernigovCattleBreed": {"131460004": ("Chernigov cattle breed", [7480])},
    "ChesapeakeBayRetriever": {"13248002": ("Chesapeake Bay retriever (organism)", [])},
    "ChesapeakeBayRetrieverDogBreed": {
        "13248002": ("Chesapeake Bay retriever dog breed", [7480])
    },
    "ChesiCattleBreed": {"133353000": ("Chesi cattle breed", [7480])},
    "Chest": {
        "816094009": (
            "Chest",
            [4009, 4030, 4031, 4042, 7151, 7192, 7482, 7483, 8134, 9514],
        )
    },
    "ChestAbdomenAndPelvis": {
        "416775004": (
            "Chest, Abdomen and Pelvis",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "ChestAndAbdomen": {
        "416550000": (
            "Chest and Abdomen",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7482, 7483, 8134, 9514],
        )
    },
    "ChestAndOrAbdomenAndOrPelvisStructure": {
        "416775004": (
            "Chest and/or abdomen and/or pelvis structure (body structure)",
            [],
        )
    },
    "ChestAndOrAbdomenStructure": {
        "416550000": ("Chest and/or abdomen structure (body structure)", [])
    },
    "ChestCavity": {"43799004": ("Chest cavity", [7155, 8134])},
    "ChestCircumference": {
        "248366000": ("Chest circumference (observable entity)", [])
    },
    "ChestDiscomfort": {"279084009": ("Chest discomfort", [3220])},
    "ChestImaging": {"413815006": ("Chest imaging (procedure)", [])},
    "ChestPain": {"29857009": ("Chest Pain", [3201, 3220, 3221, 12246])},
    "ChestPainNotPresent": {
        "161971004": ("Chest pain not present", [3202, 3220, 3719])
    },
    "ChestWall": {"78904004": ("Chest wall", [6018, 6129, 8134])},
    "ChestWallMuscle": {
        "372074006": ("Chest wall muscle", [7151, 7155, 7192, 8134, 9514])
    },
    "ChestWallStructure": {"78904004": ("Chest wall structure (body structure)", [])},
    "ChesterWhitePig": {"69461005": ("Chester white pig (organism)", [])},
    "ChesterWhitePigBreed": {"69461005": ("Chester white pig breed", [7480])},
    "CheurfaCattleBreed": {"133354006": ("Cheurfa cattle breed", [7480])},
    "CheviotSheep": {"50717006": ("Cheviot sheep (organism)", [])},
    "CheviotSheepBreed": {"50717006": ("Cheviot sheep breed", [7480])},
    "ChewingTobacco": {"81911001": ("Chewing tobacco", [6089])},
    "ChiTerrierDogBreed": {"132623008": ("Chi Terrier dog breed", [7480])},
    "ChiangusCattleBreed": {"133578000": ("Chiangus cattle breed", [7480])},
    "ChianinaCattleBreed": {"35229007": ("Chianina cattle breed (organism)", [])},
    "ChianinaCowBreed": {"35229007": ("Chianina cow breed", [7480])},
    "ChianinaPigBreed": {"132179006": ("Chianina pig breed", [7480])},
    "ChickasawHorseBreed": {"131901008": ("Chickasaw horse breed", [7480])},
    "ChienDArtoisDogBreed": {"132551007": ("Chien d'Artois dog breed", [7480])},
    "ChienFrançaisBlancEtNoirDogBreed": {
        "132543009": ("Chien Français Blanc et Noir dog breed", [7480])
    },
    "ChienFrançaisBlancEtOrangeDogBreed": {
        "132558001": ("Chien Français Blanc et Orange dog breed", [7480])
    },
    "ChienFrançaisTricoloreDogBreed": {
        "132553005": ("Chien Français Tricolore dog breed", [7480])
    },
    "ChifordCattleBreed": {"133355007": ("Chiford cattle breed", [7480])},
    "ChihuahuaSuperbreed": {"9761009": ("Chihuahua superbreed (organism)", [])},
    "ChihuahuaSuperbreedDogBreed": {
        "9761009": ("Chihuahua superbreed dog breed", [7480])
    },
    "ChildAndAdolescentPsychiatry": {
        "309959002": ("Child and Adolescent Psychiatry", [7030])
    },
    "ChildAndAdolescentPsychiatryDepartment": {
        "309959002": ("Child and adolescent psychiatry department (environment)", [])
    },
    "ChileanCorraleroHorseBreed": {
        "133141008": ("Chilean Corralero horse breed", [7480])
    },
    "ChillsAndFever": {"274640006": ("Chills and fever", [60, 9300])},
    "ChiloteHorseBreed": {"131902001": ("Chilote horse breed", [7480])},
    "ChimaineCattleBreed": {"133356008": ("Chimaine cattle breed", [7480])},
    "ChinPigBreed": {"133167003": ("Chin pig breed", [7480])},
    "ChinampoCattleBreed": {"133357004": ("Chinampo cattle breed", [7480])},
    "ChincoteaguePonyHorseBreed": {
        "133101004": ("Chincoteague Pony horse breed", [7480])
    },
    "ChineseBlackAndWhiteCattleBreed": {
        "131585005": ("Chinese Black-and-White cattle breed", [7480])
    },
    "ChineseCrestedDogBreed": {"132554004": ("Chinese Crested dog breed", [7480])},
    "ChineseFooDogBreed": {"132555003": ("Chinese Foo Dog breed", [7480])},
    "ChineseImperialChInDogBreed": {
        "132556002": ("Chinese Imperial ch'in dog breed", [7480])
    },
    "ChineseKazakhHorseBreed": {"131903006": ("Chinese Kazakh horse breed", [7480])},
    "ChineseMongolianHorseBreed": {
        "131904000": ("Chinese Mongolian horse breed", [7480])
    },
    "ChinoSantandereanoCattleBreed": {
        "131461000": ("Chino Santandereano cattle breed", [7480])
    },
    "ChinookDogBreed": {"132557006": ("Chinook dog breed", [7480])},
    "ChiosSheepBreed": {"131789001": ("Chios sheep breed", [7480])},
    "ChittagongRedCattleBreed": {"133644001": ("Chittagong Red cattle breed", [7480])},
    "ChloralHydrate": {"273948005": ("Chloral hydrate", [623, 625])},
    "ChloriodizedOil": {"62442005": ("Chloriodized oil", [12, 3850])},
    "ChloroacetateEsteraseStain": {
        "255796003": ("chloroacetate esterase stain", [8112])
    },
    "Chloroform": {"259153006": ("Chloroform", [623, 624, 8114])},
    "Chlorpromazine": {"387258005": ("Chlorpromazine", [623, 626])},
    "Choline": {"65123005": ("Choline", [218, 4032, 4033, 7180, 7186, 7469])},
    "CholinergicAgent": {"421148003": ("Cholinergic agent", [621, 622])},
    "CholinergicReceptorStimulatingAgent": {
        "421148003": ("Cholinergic receptor stimulating agent (substance)", [])
    },
    "CholistaniCattleBreed": {"133601000": ("Cholistani cattle breed", [7480])},
    "CholistaniSheepBreed": {"131703003": ("Cholistani sheep breed", [7480])},
    "Chondroma": {"31186001": ("Chondroma", [6030, 6031])},
    "Chondrosarcoma": {"1163016002": ("Chondrosarcoma", [6030, 6033])},
    "ChondrosarcomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "14990007": (
            "Chondrosarcoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "ChordaeTendineaeCordis": {
        "102298001": (
            "Chordae tendineae cordis",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "ChoroidOfEye": {"68703001": ("Choroid of eye", [4209, 4211, 8134])},
    "ChoroidalStructure": {"68703001": ("Choroidal structure (body structure)", [])},
    "ChowChow": {"28751008": ("Chow Chow", [7480])},
    "ChromeAzurolSStain": {"34128002": ("chrome azurol S stain", [8112])},
    "ChromicAcidStain": {"406986000": ("chromic acid stain", [8112])},
    "ChromicPhosphateP32": {"17069007": ("Chromic phosphate P^32^", [25])},
    "Chromium51Albumin": {"4693006": ("Chromium^51^ albumin", [25])},
    "Chromium51Chloride": {"6973004": ("Chromium^51^ chloride", [25])},
    "ChromiumTrioxide": {"430821002": ("Chromium trioxide", [8114])},
    "Chromotrope2RStain": {"85981002": ("chromotrope 2R stain", [8112])},
    "Chronic": {"90734009": ("Chronic", [3770, 6130])},
    "ChronicSick": {"161901003": ("Chronic sick (finding)", [])},
    "ChronicallyIll": {"27624003": ("Chronically ill", [3772])},
    "ChronotropicIncompetence": {"427989008": ("Chronotropic incompetence", [3213])},
    "ChryakPICPigBreed": {"132214002": ("Chryak PIC pig breed", [7480])},
    "ChrysoidineRStain": {"10247008": ("chrysoidine R stain", [8112])},
    "ChrysoidineYStain": {"16943008": ("chrysoidine Y stain", [8112])},
    "ChumbivilcasHorseBreed": {"131905004": ("Chumbivilcas horse breed", [7480])},
    "ChumyshHorseBreed": {"131906003": ("Chumysh horse breed", [7480])},
    "CigaretteSmokingTobacco": {"66562002": ("Cigarette smoking tobacco", [6089])},
    "CildirCattleBreed": {"133358009": ("Cildir cattle breed", [7480])},
    "CiliaryBody": {"29534007": ("Ciliary body", [4209, 4211, 8134])},
    "CiliaryBodyStructure": {
        "29534007": ("Ciliary body structure (body structure)", [])
    },
    "Cingulum": {"37035000": ("Cingulum", [7151, 7153, 7192, 7704, 7710, 8134, 9514])},
    "CinisaraCattleBreed": {"131462007": ("Cinisara cattle breed", [7480])},
    "CircleOfWillis": {
        "11279006": ("Circle of Willis", [4, 4030, 7151, 7192, 8134, 9514])
    },
    "CirculatingNurse": {"413854007": ("Circulating Nurse", [7453])},
    "Circumference": {
        "74551000": (
            "Circumference",
            [
                218,
                6165,
                7469,
                7470,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
            ],
        )
    },
    "Circumferential": {"255593009": ("Circumferential", [6209])},
    "CircumflexCoronaryArtery": {
        "57396003": (
            "Circumflex Coronary Artery",
            [3015, 3016, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514, 12292],
        )
    },
    "Circumscribed": {"263706005": ("Circumscribed", [6120])},
    "CircumscribedLesion": {
        "129738007": ("Circumscribed lesion", [6006, 6007, 6335, 6341, 6342])
    },
    "CiritHorseBreed": {"131907007": ("Cirit horse breed", [7480])},
    "CirnecoDellEtnaDogBreed": {"132422006": ("Cirneco dell'Etna dog breed", [7480])},
    "CirrhosisOfLiver": {"19943007": ("Cirrhosis of liver", [12325])},
    "CirsoidAneurysm": {"233982006": ("cirsoid aneurysm", [3808])},
    "CisternaMagna": {"54165005": ("Cisterna Magna", [8134, 12020, 12022])},
    "Citrate": {"59351004": ("Citrate", [218, 4032, 4033, 7180, 7186, 7469])},
    "CitricAcid": {"75399008": ("citric acid", [4412])},
    "ClassIAntiarrhythmicAgent": {
        "373260001": ("class I antiarrhythmic agent", [3762])
    },
    "ClassIIAntiarrhythmicAgent": {
        "373278006": ("class II antiarrhythmic agent", [3762])
    },
    "ClassIIIAntiarrhythmicAgent": {
        "372855004": ("class III antiarrhythmic agent", [3762])
    },
    "ClassIVAntiarrhythmicAgent": {
        "372693007": ("class IV antiarrhythmic agent", [3762])
    },
    "Classification": {"278201002": ("Classification (attribute)", [])},
    "Claudication": {"16973004": ("Claudication", [3220, 3221])},
    "Clavicle": {
        "51299004": (
            "Clavicle",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                6102,
                6109,
                6114,
                7151,
                7155,
                7192,
                7304,
                8134,
                9514,
                12021,
            ],
        )
    },
    "ClavicularNotchOfSternum": {
        "75319007": ("Clavicular notch of sternum", [6115, 8134])
    },
    "ClawnPigBreed": {"133181001": ("Clawn pig breed", [7480])},
    "Clear": {"263707001": ("Clear", [6067])},
    "ClearingOfTissueSpecimen": {
        "433452008": ("Clearing of tissue specimen (procedure)", [])
    },
    "Cleavage": {"399161006": ("Cleavage", [4015])},
    "CleavageMammographyView": {
        "399161006": ("Cleavage mammography view (qualifier value)", [])
    },
    "Cleaves": {"399245002": ("Cleaves", [4012])},
    "CleavesProjection": {"399245002": ("Cleaves projection (qualifier value)", [])},
    "Clements": {"399028002": ("Clements", [4012])},
    "ClementsNakayama": {"399320003": ("Clements-Nakayama", [4012])},
    "ClementsNakayamaProjection": {
        "399320003": ("Clements-Nakayama projection (qualifier value)", [])
    },
    "ClementsProjection": {"399028002": ("Clements projection (qualifier value)", [])},
    "ClevelandBayHorse": {"41092008": ("Cleveland bay horse (organism)", [])},
    "ClevelandBayHorseBreed": {"41092008": ("Cleveland bay horse breed", [7480])},
    "ClinicalAllergyDepartment": {
        "309913004": ("Clinical allergy department (environment)", [])
    },
    "ClinicalBiochemistry": {"310076001": ("Clinical Biochemistry", [7030])},
    "ClinicalBiochemistryService": {
        "310076001": ("Clinical biochemistry service (qualifier value)", [])
    },
    "ClinicalFinding": {"404684003": ("Clinical finding (finding)", [])},
    "ClinicalOncology": {"309902002": ("Clinical Oncology", [7030])},
    "ClinicalOncologyDepartment": {
        "309902002": ("Clinical oncology department (environment)", [])
    },
    "ClinicalReport": {"371524004": ("Clinical report (record artifact)", [])},
    "ClinicalTargetVolume": {
        "228792002": ("Clinical target volume (observable entity)", [])
    },
    "Clip": {"77720000": ("Clip", [6040, 6401, 7111, 7112, 7151, 7193])},
    "ClipDevice": {"77720000": ("Clip, device (physical object)", [])},
    "ClosureOfAtrialSeptalDefect": {
        "112811009": ("Closure of atrial septal defect (procedure)", [])
    },
    "ClosureOfDuctusArteriosusWithClip": {
        "233199008": ("Closure of ductus arteriosus with clip", [12247])
    },
    "Clothing": {"272180002": ("Clothing", [7151, 7158, 7193])},
    "ClumberSpaniel": {"67977006": ("Clumber spaniel (organism)", [])},
    "ClumberSpanielDogBreed": {"67977006": ("Clumber spaniel dog breed", [7480])},
    "ClunForestSheep": {"48697009": ("Clun Forest sheep (organism)", [])},
    "ClunForestSheepBreed": {"48697009": ("Clun Forest sheep breed", [7480])},
    "ClydesdaleHorse": {"1247002": ("Clydesdale horse (organism)", [])},
    "ClydesdaleHorseBreed": {"1247002": ("Clydesdale horse breed", [7480])},
    "Cm0": {"1229901006": ("cM0", [6162])},
    "CoPigBreed": {"132350007": ("Co pig breed", [7480])},
    "Coagulant": {"373746004": ("Coagulant", [10])},
    "CoarctationAngioplasty": {
        "308696000": ("Coarctation angioplasty (procedure)", [])
    },
    "CoarctationOfAorta": {"7305005": ("Coarctation of aorta", [3610, 3630, 12291])},
    "CoarctationOfTheAorta": {"7305005": ("Coarctation of the Aorta", [12248, 12249])},
    "CoarctationOfTheAortaAngioplastyWithImplantOfStent": {
        "443829004": (
            "Coarctation of the Aorta Angioplasty with Implant of Stent",
            [12247],
        )
    },
    "CoarctationOfTheAortaBalloonAngioplasty": {
        "308696000": ("Coarctation of the Aorta Balloon Angioplasty", [12247])
    },
    "CoarsePopcornLikeCalcification": {
        "129749001": ("Coarse (popcorn-like) calcification", [6010, 6011, 6132])
    },
    "CoarsePopcornLikeRadiographicCalcification": {
        "129749001": ("Coarse (popcorn-like) radiographic calcification (finding)", [])
    },
    "CobaltChromiumAlloy": {"256526003": ("Cobalt-chromium alloy", [300])},
    "CobbAngle": {"285285000": ("Cobb angle", [218, 7550])},
    "CoburgerFuchsschafSheepBreed": {
        "131736002": ("Coburger Fuchsschaf sheep breed", [7480])
    },
    "Cocaine": {"387085005": ("Cocaine", [6089])},
    "CocaineAbuse": {"78267003": ("Cocaine abuse (disorder)", [])},
    "CoccygealVertebraStructure": {
        "18149002": ("Coccygeal vertebra structure (body structure)", [])
    },
    "CoccygealVertrebrae": {"18149002": ("Coccygeal vertrebrae", [7482, 7483, 8134])},
    "Coccyx": {
        "64688005": ("Coccyx", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "CochinealStain": {"91606004": ("Cochineal stain (substance)", [])},
    "CockerSpanielAnySolidColorOtherThanBlack": {
        "30565000": ("Cocker spaniel, any solid color other than black (organism)", [])
    },
    "CockettPerforatingVein": {
        "128549006": (
            "Cockett's perforating vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "CoiledPosition": {"34108001": ("Coiled position (finding)", [])},
    "CoinLesion": {"308689002": ("Coin lesion", [6102, 6104])},
    "CoinLesionOfLung": {"308689002": ("Coin lesion of lung (finding)", [])},
    "ColNoirDuValaisGoatBreed": {
        "131688003": ("Col Noir du Valais goat breed", [7480])
    },
    "ColcherSussman": {"423091003": ("Colcher-Sussman", [4012])},
    "ColcherSussmanProjection": {
        "423091003": ("Colcher-Sussman projection (qualifier value)", [])
    },
    "ColicLymphNode": {"8356004": ("colic lymph node", [7600, 8134])},
    "CollateralBranchOfVessel": {
        "397406000": ("Collateral Branch of vessel", [3496, 7111, 12117])
    },
    "CollectionOfBloodSpecimenForLaboratory": {
        "82078001": ("collection of blood specimen for laboratory", [3515])
    },
    "Collie": {"19078005": ("Collie", [7480])},
    "Collimator": {"228761004": ("Collimator", [6040, 6401, 7151, 7193])},
    "ColloidalIronStain": {"406952009": ("colloidal iron stain", [8112])},
    "ColombianCriolloHorseBreed": {
        "131804001": ("Colombian Criollo horse breed", [7480])
    },
    "Colon": {
        "71854001": (
            "Colon",
            [
                4,
                645,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7482,
                7483,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "ColonStructure": {"71854001": ("Colon structure (body structure)", [])},
    "ColonicHaustra": {"6533001": ("Colonic haustra", [6201])},
    "ColonicIntraluminalFluid": {
        "442170005": ("Colonic intraluminal fluid (substance)", [])
    },
    "ColorOfFluid": {"250431005": ("Color of fluid (observable entity)", [])},
    "ColoradoRangerHorseBreed": {"133098003": ("Colorado Ranger horse breed", [7480])},
    "ColorectalSurgery": {"309983005": ("Colorectal Surgery", [7030])},
    "ColorectalSurgeryDepartment": {
        "309983005": ("Colorectal surgery department (environment)", [])
    },
    "ColoredBullTerrier": {"86593006": ("Colored bull terrier (organism)", [])},
    "ColoredBullTerrierDogBreed": {
        "86593006": ("Colored bull terrier dog breed", [7480])
    },
    "ColorpointShorthairedCat": {
        "61753003": ("Colorpoint shorthaired cat (organism)", [])
    },
    "ColostomyBag": {"339648008": ("Colostomy bag", [6202, 6203])},
    "ColostomySet": {"341036005": ("Colostomy set (physical object)", [])},
    "ColourpointShorthairedCat": {"61753003": ("Colourpoint shorthaired cat", [7480])},
    "ColumbiaSheepBreed": {"131705005": ("Columbia sheep breed", [7480])},
    "CombinedDiagnosticAndTherapeutic": {
        "371931008": ("Combined diagnostic and therapeutic (procedure)", [])
    },
    "CombinedDiagnosticAndTherapeuticIntent": {
        "1279505009": ("Combined Diagnostic and Therapeutic Intent", [3629])
    },
    "CombinedSmallCellCarcinoma": {
        "21326004": ("Combined small cell carcinoma (morphologic abnormality)", [])
    },
    "ComebackSheepBreed": {"131766009": ("Comeback sheep breed", [7480])},
    "Comedocarcinoma": {"78197004": ("Comedocarcinoma (intraductal)", [6030, 6033])},
    "ComedocarcinomaNoninfiltrating": {
        "78197004": ("Comedocarcinoma, noninfiltrating (morphologic abnormality)", [])
    },
    "ComisanaSheepBreed": {"131765008": ("Comisana sheep breed", [7480])},
    "CommonAtrioventricularCanal": {
        "360481003": ("Common atrioventricular canal", [12248, 12249])
    },
    "CommonAtrium": {
        "253276007": (
            "Common atrium",
            [4, 3010, 3608, 3630, 4042, 7151, 7192, 8134, 9514, 12284],
        )
    },
    "CommonBileDuct": {
        "79741001": (
            "Common bile duct",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CommonBileDuctStructure": {
        "79741001": ("Common bile duct structure (body structure)", [])
    },
    "CommonCarotidArtery": {
        "32062004": (
            "Common carotid artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12104,
            ],
        )
    },
    "CommonCarotidArteryStructure": {
        "32062004": ("Common carotid artery structure (body structure)", [])
    },
    "CommonDuctLymphNode": {"280639005": ("common duct lymph node", [7600, 8134])},
    "CommonFemoralArtery": {
        "181347005": (
            "Common Femoral Artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
            ],
        )
    },
    "CommonFemoralVein": {
        "397363009": (
            "Common Femoral Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "CommonFemoralVeinStructure": {
        "397363009": ("Common femoral vein structure (body structure)", [])
    },
    "CommonHepaticArtery": {
        "66559000": (
            "Common Hepatic Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "CommonIliacArtery": {
        "73634005": (
            "Common iliac artery",
            [
                4,
                3010,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
                12111,
                12125,
                12140,
            ],
        )
    },
    "CommonIliacArteryBifurcation": {
        "413896006": (
            "Common iliac artery bifurcation",
            [1000, 1001, 1005, 3827, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "CommonIliacArteryStructure": {
        "73634005": ("Common iliac artery structure (body structure)", [])
    },
    "CommonIliacBifurcation": {"413896006": ("Common Iliac Bifurcation", [3837, 7111])},
    "CommonIliacVein": {
        "46027005": (
            "Common iliac vein",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110, 12113, 12125],
        )
    },
    "CommonNonMitralNonTricuspidAtrioventricularValveStructure": {
        "312522004": (
            "Common non-mitral non-tricuspid Atrioventricular Valve Structure",
            [12285],
        )
    },
    "CommonTruncusArteriosus": {
        "61959006": ("Common truncus arteriosus (disorder)", [])
    },
    "CommonVentricle": {
        "45503006": (
            "Common ventricle",
            [4, 3010, 3011, 3609, 3630, 4042, 7151, 7192, 8134, 9514, 12287],
        )
    },
    "Complaint": {"1269496002": ("Complaint", [3769])},
    "Complaint_Inactive": {"409586006": ("Complaint (finding)", [])},
    "Complete": {"255594003": ("Complete", [67])},
    "CompleteAtrioventricularBlock": {
        "27885002": ("Complete atrioventricular block (disorder)", [])
    },
    "CompleteTranspositionOfGreatVessels": {
        "26146002": ("Complete transposition of great vessels", [12248, 12249])
    },
    "ComplicatedAtheromatousPlaque": {
        "74937006": ("complicated atheromatous plaque", [3817])
    },
    "ComplicationOfProcedure": {"116224001": ("Complication of Procedure", [7002])},
    "ComponentOfOpticalMicroscope": {
        "445316008": ("Component of optical microscope (physical object)", [])
    },
    "CompositeHemodynamicMeasurementMethod": {
        "128577002": ("Composite hemodynamic measurement method", [3241])
    },
    "CompositeHemodynamicMeasurementMethodRegimeTherapy": {
        "128577002": ("Composite hemodynamic measurement method (regime/therapy)", [])
    },
    "Compression": {"71173004": ("compression", [3805])},
    "CompressionAction": {"263720003": ("Compression - action", [94])},
    "CompressionPaddle": {
        "129460009": ("Compression paddle", [6040, 6401, 7151, 7193])
    },
    "CompressionPaddleDevice": {
        "129460009": ("Compression paddle, device (physical object)", [])
    },
    "ComputedHemodynamicMeasurementMethod": {
        "128576006": ("Computed hemodynamic measurement method", [3241])
    },
    "ComputedHemodynamicMeasurementMethodRegimeTherapy": {
        "128576006": ("Computed hemodynamic measurement method (regime/therapy)", [])
    },
    "ComputedTomography": {"77477000": ("Computed tomography", [10005])},
    "ComputedTomographyAngiographyOfCoronaryArteries": {
        "419545005": (
            "Computed tomography angiography of coronary arteries (procedure)",
            [],
        )
    },
    "ComputedTomographyGuidedBiopsy": {
        "277591006": ("Computed tomography guided biopsy", [6058, 6060])
    },
    "ComputedTomographyOfAbdominalAorta": {
        "241553009": ("Computed tomography of abdominal aorta (procedure)", [])
    },
    "ComputedTomographyOfAbdominalAortaWithContrast": {
        "444968003": (
            "Computed tomography of abdominal aorta with contrast (procedure)",
            [],
        )
    },
    "ComputedTomographyOfBreast": {
        "241539009": ("Computed tomography of breast (procedure)", [])
    },
    "ComputedTomographyOfCardiovascularSystem": {
        "303680000": ("Computed tomography of cardiovascular system (procedure)", [])
    },
    "ComputedTomographyOfHead": {
        "303653007": ("Computed tomography of head (procedure)", [])
    },
    "ComputedTomographyOfHeart": {
        "241547009": ("Computed tomography of heart (procedure)", [])
    },
    "ComputedTomographyWithoutContrast": {
        "399331006": ("Computed tomography without contrast (procedure)", [])
    },
    "ComputerAssistedImageAnalysisForAsymmetricBreastTissue": {
        "133889002": (
            "Computer assisted image analysis for asymmetric breast tissue (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForBreastCompositionAnalysis": {
        "133890006": (
            "Computer assisted image analysis for breast composition analysis (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForFocalAsymmetricDensity": {
        "133888005": (
            "Computer assisted image analysis for focal asymmetric density (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForImageQuality": {
        "133887000": (
            "Computer assisted image analysis for image quality (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForSpatialCollocation": {
        "133884007": (
            "Computer assisted image analysis for spatial collocation (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForSpatialProximity": {
        "133885008": (
            "Computer assisted image analysis for spatial proximity (procedure)",
            [],
        )
    },
    "ComputerAssistedImageAnalysisForTemporalCorrelation": {
        "133886009": (
            "Computer assisted image analysis for temporal correlation (procedure)",
            [],
        )
    },
    "ComputerizedAxialTomography": {
        "77477000": ("Computerized axial tomography (procedure)", [])
    },
    "ComputerizedTomographyService": {
        "310128004": ("Computerized Tomography Service", [7030])
    },
    "ComtoisHorseBreed": {"131805000": ("Comtois horse breed", [7480])},
    "ConcaveContactFundusLens": {"409783000": ("Concave contact fundus lens", [4205])},
    "ConcaveNoncontactFundusLens": {
        "410688004": ("Concave noncontact fundus lens", [4205])
    },
    "Concentric": {"255465008": ("Concentric", [3491, 3806])},
    "CondenserAnnulus": {"445634000": ("Condenser annulus", [8124])},
    "CondenserAnnulusOfOpticalMicroscope": {
        "445634000": ("Condenser annulus of optical microscope (physical object)", [])
    },
    "Condition": {"260905004": ("Condition (attribute)", [])},
    "ConductanceCatheterMethod": {"133910006": ("Conductance catheter method", [3241])},
    "ConductionDisorderOfTheHeart": {
        "44808001": ("Arrhythmia", [3201, 3221, 3413, 3727, 3755, 3756, 12246])
    },
    "ConeBeamAcquisition": {"702569007": ("Cone Beam Acquisition", [10013])},
    "ConeBeamComputedTomography": {
        "717193008": ("Cone beam computed tomography", [10005])
    },
    "ConeBeamComputedTomographyImagingAction": {
        "702569007": (
            "Cone beam computed tomography imaging - action (qualifier value)",
            [],
        )
    },
    "ConfocalScanningLaserOphthalmoscope": {
        "392004000": ("Confocal Scanning Laser Ophthalmoscope", [4210])
    },
    "CongenitalAnomalyOfCardiovascularSystem": {
        "9904008": ("Congenital anomaly of cardiovascular system (disorder)", [])
    },
    "CongenitalCoronaryArteryFistulaToLeftAtrium": {
        "128555001": (
            "Congenital coronary artery fistula to left atrium",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CongenitalCoronaryArteryFistulaToLeftVentricle": {
        "128556000": (
            "Congenital coronary artery fistula to left ventricle",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CongenitalCoronaryArteryFistulaToRightAtrium": {
        "128557009": (
            "Congenital coronary artery fistula to right atrium",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CongenitalCoronaryArteryFistulaToRightVentricle": {
        "128558004": (
            "Congenital coronary artery fistula to right ventricle",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CongenitalHeartDisease": {"13213009": ("Congenital heart disease", [12246])},
    "CongenitalPulmonaryArteryConduit": {
        "128584005": ("Congenital pulmonary artery conduit (disorder)", [])
    },
    "CongenitalPulmonaryVeinConfluence": {
        "128566008": ("Congenital pulmonary vein confluence (disorder)", [])
    },
    "CongenitalPulmonaryVenousAtrium": {
        "128567004": ("Congenital pulmonary venous atrium (disorder)", [])
    },
    "CongenitalStenosisOfAorticValve": {
        "18546004": ("Congenital stenosis of aortic valve", [12248, 12249])
    },
    "CongenitalSystemicVenousAtrium": {
        "128568009": ("Congenital systemic venous atrium (disorder)", [])
    },
    "CongestiveCardiomyopathy": {
        "399020009": ("Congestive cardiomyopathy", [3700, 3728])
    },
    "CongestiveHeartFailure": {
        "42343007": ("Congestive heart failure", [3413, 3700, 3755])
    },
    "CongoRedStain": {"45106005": ("Congo red stain", [8112])},
    "Conjunctiva": {"29445007": ("Conjunctiva", [4209, 4211, 8134])},
    "ConjunctivalStructure": {
        "29445007": ("Conjunctival structure (body structure)", [])
    },
    "ConnectiveTissue": {
        "21793004": ("Connective tissue", [645, 7151, 7166, 7191, 8134])
    },
    "ConnectiveTissueStructure": {
        "21793004": ("Connective tissue structure (body structure)", [])
    },
    "ConnemaraPonyHorseBreed": {"133097008": ("Connemara Pony horse breed", [7480])},
    "ConnorPrairiePig": {"29881002": ("Connor prairie pig (organism)", [])},
    "ConnorPrairiePigBreed": {"29881002": ("Connor prairie pig breed", [7480])},
    "ConstrictedPupil": {"301939004": ("Constricted Pupil", [4222])},
    "ConstrictivePericarditis": {
        "85598007": ("Constrictive pericarditis", [3700, 3728])
    },
    "ConstructionOfConduitRightAtriumToPulmonaryTrunk": {
        "233022006": (
            "Construction of conduit - right atrium to pulmonary trunk (procedure)",
            [],
        )
    },
    "ConstructionOfLVToAortaTunnelWRVToPAValvedConduit": {
        "429620002": (
            "Construction of LV to aorta tunnel w RV to PA valved conduit",
            [12247],
        )
    },
    "ConstructionOfLeftVentricleToAortaTunnelWithRightVentricleToPulmonaryArteryValvedConduit": {
        "429620002": (
            "Construction of left ventricle to aorta tunnel with right ventricle to pulmonary artery "
            "valved conduit (procedure)",
            [],
        )
    },
    "Consultant": {"309390008": ("Consultant", [7450, 7452])},
    "ContactFundusLens": {"410686000": ("Contact fundus lens", [4205])},
    "ContactWith": {"11723008": ("Contact with", [5])},
    "ContactWithContextualQualifier": {
        "11723008": ("Contact with (contextual qualifier) (qualifier value)", [])
    },
    "Continuous": {"255238004": ("Continuous", [6091])},
    "ContinuousElectrocardiogramMonitoring": {
        "266706003": ("Continuous electrocardiogram monitoring (procedure)", [])
    },
    "ContinuousFlowApneicVentilation": {
        "1260411001": ("Continuous flow apneic ventilation", [619, 620])
    },
    "ContinuousFlowVentilation": {
        "243156001": ("Continuous flow ventilation (procedure)", [])
    },
    "Contraceptives": {"108899006": ("Contraceptives", [6080])},
    "Contraindicated": {"373147003": ("Contraindicated", [3741, 3742, 3743])},
    "Contralateral": {"255209002": ("Contralateral", [246])},
    "ContrastAgent": {"7140000": ("Contrast agent", [12, 3850, 6040, 6402, 7198])},
    "ContrastEchocardiography": {"433231002": ("Contrast echocardiography", [12001])},
    "ContrastMedia": {"385420005": ("Contrast media", [6201, 7111, 7112])},
    "ContrastMediaAdverseReaction": {
        "292095005": ("Contrast media adverse reaction", [3413, 3755])
    },
    "ContrastMediaAllergy": {"293637006": ("Contrast Media Allergy", [1200, 1201])},
    "ContrastMediaSeenInPlaque": {
        "413912008": ("Contrast media seen in plaque (finding)", [])
    },
    "ContrastMediumInjectionSystemManifoldKit": {
        "467354001": (
            "Contrast medium injection system manifold kit (physical object)",
            [],
        )
    },
    "ControlledVentilation": {"243147009": ("Controlled Ventilation", [617, 618])},
    "ConvergentGaze": {"408745006": ("Convergent gaze", [4201])},
    "ConvexContactFundusLens": {"410689007": ("Convex contact fundus lens", [4205])},
    "ConvexNoncontactFundusLens": {
        "410687009": ("Convex noncontact fundus lens", [4205])
    },
    "Coonhound": {"73319009": ("Coonhound", [7480])},
    "CoopworthSheepBreed": {"131764007": ("Coopworth sheep breed", [7480])},
    "Copper": {"66925006": ("Copper", [10006, 10067])},
    "Copper62LabeledPyruvaldehydeBisN4MethylThiosemicarbazone": {
        "422789008": (
            "Copper^62^ labeled pyruvaldehyde-bis N-(4)-methyl-thiosemicarbazone (product)",
            [],
        )
    },
    "Copper64Acetate": {"78686003": ("Copper^64^ acetate", [25])},
    "Copper64LabeledDiacetylBisN4Methylthiosemicarbazone": {
        "422855001": (
            "Copper^64^ labeled diacetyl-bis N-(4)-methylthiosemicarbazone (product)",
            [],
        )
    },
    "Copper64Versenate": {"88166005": ("Copper^64^ versenate", [25])},
    "Copper67Ceruloplasmin": {"29460005": ("Copper^67^ ceruloplasmin", [25])},
    "CorTriatriatumOrifice": {"443445009": ("Cor Triatriatum Orifice", [12283])},
    "CorTriloculareBiventriculare": {
        "253276007": ("Cor triloculare biventriculare (disorder)", [])
    },
    "CoracoidProcessOfScapula": {
        "8931003": ("Coracoid process of scapula", [6115, 8134])
    },
    "CoreBodyTemperatureMeasuredInRectum": {
        "307047009": (
            "Core body temperature measured in rectum (observable entity)",
            [],
        )
    },
    "CoreNeedleBiopsyOfBreast": {
        "44578009": ("Core needle biopsy of breast", [6050, 6083])
    },
    "CoreSampleOfTissueBlock": {"430970004": ("Core sample of tissue block", [8103])},
    "CoreSampling": {"434479002": ("Core sampling", [8110])},
    "CoreSamplingOfTissueBlock": {
        "434479002": ("Core sampling of tissue block (procedure)", [])
    },
    "CoriphosphineStain": {"49687009": ("coriphosphine stain", [8112])},
    "CormoSheepBreed": {"131753009": ("Cormo sheep breed", [7480])},
    "Cornea": {
        "28726007": (
            "Cornea",
            [4, 4029, 4209, 4211, 4233, 4266, 7151, 7192, 8134, 9514],
        )
    },
    "CornealEndothelium": {"65431007": ("Corneal endothelium", [4211, 8134])},
    "CornealEpithelium": {"15775008": ("Corneal epithelium", [4211, 8134])},
    "CornealOpacity": {"64634000": ("Corneal Opacity", [4222])},
    "CornealStructure": {"28726007": ("Corneal structure (body structure)", [])},
    "CornishRexCatBreed": {"56917006": ("Cornish rex cat breed", [7480])},
    "Coronal": {"81654009": ("Coronal", [2, 6, 26, 501, 502, 6312])},
    "CoronaryAngiography": {"33367005": ("Coronary angiography (procedure)", [])},
    "CoronaryArteriography": {"33367005": ("Coronary Arteriography", [3405, 3428])},
    "CoronaryArteriosclerosis": {
        "53741008": ("Coronary arteriosclerosis (disorder)", [])
    },
    "CoronaryArtery": {
        "41801008": (
            "Coronary artery",
            [4, 3010, 3604, 3606, 3630, 4030, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CoronaryArteryBypassGraft": {
        "232717009": ("Coronary artery bypass graft", [3721, 3764])
    },
    "CoronaryArteryBypassGrafting": {
        "232717009": ("Coronary artery bypass grafting (procedure)", [])
    },
    "CoronaryArteryCalciumScore": {
        "450360000": ("Coronary artery calcium score", [6142])
    },
    "CoronaryArteryDisease": {
        "53741008": ("Coronary Artery Disease", [3201, 3700, 12246])
    },
    "CoronaryArteryFeature": {
        "364092001": ("Coronary artery feature (observable entity)", [])
    },
    "CoronaryArteryFistulaToRightAtrium": {
        "373095005": ("Coronary artery fistula to right atrium (disorder)", [])
    },
    "CoronaryArteryGraft": {
        "264293000": (
            "Coronary artery graft",
            [
                1000,
                1001,
                1004,
                3015,
                3604,
                3827,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
            ],
        )
    },
    "CoronaryArteryStructure": {
        "41801008": ("Coronary artery structure (body structure)", [])
    },
    "CoronaryBypassGraftAngiography": {
        "252427007": ("Coronary bypass graft angiography (procedure)", [])
    },
    "CoronarySinus": {
        "90219004": (
            "Coronary sinus",
            [4, 3010, 3011, 3608, 3630, 3827, 3839, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "CoronarySinusStructure": {
        "90219004": ("Coronary sinus structure (body structure)", [])
    },
    "CorpusCallosum": {
        "88442005": ("Corpus callosum", [7151, 7153, 7192, 7705, 7710, 8134, 9514])
    },
    "CorpusCallosumStructure": {
        "88442005": ("Corpus callosum structure (body structure)", [])
    },
    "CorpusStriatum": {"31428008": ("Corpus striatum", [7151, 7153, 7192, 8134, 9514])},
    "CorpusStriatumStructure": {
        "31428008": ("Corpus striatum structure (body structure)", [])
    },
    "CorrectedTranspositionOfGreatVessels": {
        "83799000": ("Corrected transposition of great vessels", [12248, 12249])
    },
    "CorrectionOfCongenitalCardiovascularDeformity": {
        "428613004": ("Correction of congenital cardiovascular deformity", [3721])
    },
    "CorrectionOfVentricularSeptalDefect": {
        "76025005": ("Correction of ventricular septal defect", [12247])
    },
    "CorriedaleSheep": {"67515002": ("Corriedale sheep (organism)", [])},
    "CorriedaleSheepBreed": {"67515002": ("Corriedale sheep breed", [7480])},
    "CorrienteCattleBreed": {"131586006": ("Corriente cattle breed", [7480])},
    "CorsicanCattleBreed": {"133361005": ("Corsican cattle breed", [7480])},
    "CorsicanGoatBreed": {"131649000": ("Corsican goat breed", [7480])},
    "CorsicanHorseBreed": {"131806004": ("Corsican horse breed", [7480])},
    "CorsicanPigBreed": {"132123009": ("Corsican pig breed", [7480])},
    "CorticospinalTractInBrainstem": {
        "360568007": ("corticospinal tract in brainstem", [7701, 7710, 8134])
    },
    "CosentinaPigBreed": {"132180009": ("Cosentina pig breed", [7480])},
    "CostaRicanSaddleHorseHorseBreed": {
        "131807008": ("Costa Rican Saddle Horse horse breed", [7480])
    },
    "CostalCartilage": {
        "50016007": (
            "Costal Cartilage",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "CostalGroove": {"17399006": ("Costal groove", [6115, 8134])},
    "CostenoHorseBreed": {"131808003": ("Costeno horse breed", [7480])},
    "CosteñoConCuernosCattleBreed": {
        "131587002": ("Costeño con Cuernos cattle breed", [7480])
    },
    "CostocervicalTrunk": {
        "3159004": ("Costocervical trunk", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "CotonDeTuléarDogBreed": {"132489000": ("Coton de Tuléar dog breed", [7480])},
    "Cotswold16PigBreed": {"132016006": ("Cotswold 16 pig breed", [7480])},
    "Cotswold29PigBreed": {"132017002": ("Cotswold 29 pig breed", [7480])},
    "Cotswold90PigBreed": {"132018007": ("Cotswold 90 pig breed", [7480])},
    "CotswoldGoldPigBreed": {"132014009": ("Cotswold Gold pig breed", [7480])},
    "CotswoldPlatinumPigBreed": {"132015005": ("Cotswold Platinum pig breed", [7480])},
    "CotswoldSheep": {"67414001": ("Cotswold sheep (organism)", [])},
    "CotswoldSheepBreed": {"67414001": ("Cotswold sheep breed", [7480])},
    "Cough": {"49727002": ("Cough", [60, 9300])},
    "CouinaudHepaticSegmentI": {
        "71133005": (
            "Couinaud hepatic segment I",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentII": {
        "277956007": (
            "Couinaud hepatic segment II",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentIII": {
        "277957003": (
            "Couinaud hepatic segment III",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentIV": {
        "277958008": (
            "Couinaud hepatic segment IV",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentV": {
        "277959000": (
            "Couinaud hepatic segment V",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentVI": {
        "277960005": (
            "Couinaud hepatic segment VI",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentVII": {
        "277961009": (
            "Couinaud hepatic segment VII",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "CouinaudHepaticSegmentVIII": {
        "277962002": (
            "Couinaud hepatic segment VIII",
            [7151, 7154, 7170, 7171, 7192, 8134, 9514, 12322],
        )
    },
    "Cranial": {"66787007": ("Cranial", [5])},
    "CranialCavity": {"1101003": ("Cranial Cavity", [7140])},
    "CranialCavityStructure": {
        "1101003": ("Cranial cavity structure (body structure)", [])
    },
    "CranialLAO": {"408723005": ("Cranial LAO", [3466])},
    "CranialLeftAnteriorObliqueProjection": {
        "408723005": ("Cranial left anterior oblique projection (qualifier value)", [])
    },
    "CranialNerve": {"25238003": ("Cranial nerve", [7151, 7167, 7192, 8134, 9514])},
    "CranialNerveStructure": {
        "25238003": ("Cranial nerve structure (body structure)", [])
    },
    "CranialRAO": {"408725003": ("Cranial RAO", [3466])},
    "CranialRightAnteriorObliqueProjection": {
        "408725003": ("Cranial right anterior oblique projection (qualifier value)", [])
    },
    "CranialSubarachnoidSpace": {"33930006": ("Cranial Subarachnoid Space", [7140])},
    "CranialVenousSystem": {
        "128320002": ("Cranial venous system", [4, 4030, 7151, 7192, 8134, 9514])
    },
    "CranioCaudal": {"399162004": ("cranio-caudal", [501, 4010, 4014])},
    "CranioCaudalExaggeratedLaterally": {
        "399192008": ("cranio-caudal exaggerated laterally", [4014])
    },
    "CranioCaudalExaggeratedMedially": {
        "399101009": ("cranio-caudal exaggerated medially", [4014])
    },
    "CranioCaudalProjection": {
        "399162004": ("Cranio-caudal projection (qualifier value)", [])
    },
    "CranioCaudalProjectionExaggeratedLaterally": {
        "399192008": (
            "Cranio-caudal projection exaggerated laterally (qualifier value)",
            [],
        )
    },
    "CranioCaudalProjectionExaggeratedMedially": {
        "399101009": (
            "Cranio-caudal projection exaggerated medially (qualifier value)",
            [],
        )
    },
    "Creatine": {"14804005": ("Creatine", [218, 4032, 4033, 7180, 7186, 7469])},
    "CreationOfConduitBetweenRightVentricleAndPulmonaryArtery": {
        "44777001": (
            "Creation of conduit between right ventricle and pulmonary artery (procedure)",
            [],
        )
    },
    "CreationOfConduitRightAtriumToPulmonaryTrunk": {
        "233022006": ("Creation of conduit right atrium to pulmonary trunk", [12247])
    },
    "CreoleAntillesGoatBreed": {"131658007": ("Creole Antilles goat breed", [7480])},
    "CresylEchtVioletStain": {"406960005": ("cresyl echt violet stain", [8112])},
    "CresylVioletStain": {"406959000": ("cresyl violet stain", [8112])},
    "CretanLowlandCattleBreed": {"133362003": ("Cretan Lowland cattle breed", [7480])},
    "CretanMountainCattleBreed": {
        "133363008": ("Cretan Mountain cattle breed", [7480])
    },
    "CribriformCarcinoma": {
        "30156004": ("Cribriform carcinoma (morphologic abnormality)", [])
    },
    "CriolloCattleBreed": {"83996001": ("Criollo cattle breed (organism)", [])},
    "CriolloCowBreed": {"83996001": ("Criollo cow breed", [7480])},
    "CriolloHorseBreed": {"133095000": ("Criollo horse breed", [7480])},
    "CriolloSheepBreed": {"131752004": ("Criollo sheep breed", [7480])},
    "CriouloHorseBreed": {"133028006": ("Crioulo horse breed", [7480])},
    "CroatianRedCattleBreed": {"133364002": ("Croatian Red cattle breed", [7480])},
    "CryotherapyToSkinLesion": {"302396003": ("Cryotherapy to skin lesion", [4409])},
    "CrystalPonceauStain": {"68459007": ("crystal ponceau stain", [8112])},
    "CrystallineLens": {"247049005": ("Crystalline lens", [4231])},
    "CrystallineLensFinding": {"247049005": ("Crystalline lens finding (finding)", [])},
    "CréoleAntillesGoatBreed": {
        "131658007": ("Créole Antilles goat breed (organism)", [])
    },
    "CréolePigBreed": {"132124003": ("Créole pig breed", [7480])},
    "CubanCriolloCattleBreed": {
        "133798008": ("Cuban Criollo cattle breed (organism)", [])
    },
    "CubanCriolloXZebuCattleBreed": {
        "133798008": ("Cuban Criollo X zebu cattle breed", [7480])
    },
    "CubanPasoHorseBreed": {"131809006": ("Cuban Paso horse breed", [7480])},
    "CubanPintoHorseBreed": {"133105008": ("Cuban Pinto horse breed", [7480])},
    "CubanTrotterHorseBreed": {"131910000": ("Cuban Trotter horse breed", [7480])},
    "CubanZebuCattleBreed": {"133799000": ("Cuban Zebu cattle breed (organism)", [])},
    "CubanZebuXZebuCattleBreed": {
        "133799000": ("Cuban Zebu X zebu cattle breed", [7480])
    },
    "CubitalLymphNode": {"34775006": ("cubital lymph node", [7600, 8134])},
    "CuinoPigBreed": {"132181008": ("Cuino pig breed", [7480])},
    "CukurovaCattleBreed": {"133365001": ("Cukurova cattle breed", [7480])},
    "CukurovaHorseBreed": {"132736000": ("Cukurova horse breed", [7480])},
    "Culprit": {"371895000": ("Culprit", [3712])},
    "CulpritLesion": {"371895000": ("Culprit Lesion", [3487])},
    "CulpritLesionOfCoronaryArtery": {
        "371895000": ("Culprit lesion of coronary artery (finding)", [])
    },
    "CupremHybridCattleBreed": {"131463002": ("Cuprem Hybrid cattle breed", [7480])},
    "CurativeIntent": {"373808002": ("Curative intent", [3629])},
    "CurativeProcedureIntent": {
        "373808002": ("Curative - procedure intent (qualifier value)", [])
    },
    "CurcuminStain": {"89028002": ("curcumin stain", [8112])},
    "CurledUp": {"34108001": ("curled-up", [20])},
    "CurlyCoatedRetriever": {"38449002": ("Curly-coated retriever (organism)", [])},
    "CurlyCoatedRetrieverDogBreed": {
        "38449002": ("Curly-coated retriever dog breed", [7480])
    },
    "CurraleiroCattleBreed": {"133366000": ("Curraleiro cattle breed", [7480])},
    "CurrentSmoker": {"77176002": ("Current Smoker", [3724])},
    "CutchiCattleBreed": {"133645000": ("Cutchi cattle breed", [7480])},
    "CuttingBalloonAngioplastyCBADevice": {
        "371794009": ("Cutting Balloon Angioplasty (CBA) Device", [3411, 3429])
    },
    "CuttingBalloonAngioplastyDevice": {
        "371794009": ("Cutting balloon angioplasty device (physical object)", [])
    },
    "CyanocobalaminCo57": {"187006": ("Cyanocobalamin Co^57^", [25])},
    "CyanocobalaminCo58": {"5692007": ("Cyanocobalamin Co^58^", [25])},
    "CyanocobalaminCo60": {"72159005": ("Cyanocobalamin Co^60^", [25])},
    "Cyanosis": {"3415004": ("Cyanosis", [12248, 12249])},
    "Cyclopentolate": {"8348002": ("Cyclopentolate", [4208])},
    "CylindroidAneurysm": {"52856002": ("cylindroid aneurysm", [3808])},
    "CyprusCattleBreed": {"133367009": ("Cyprus cattle breed", [7480])},
    "Cyst": {"367643001": ("Cyst", [6335, 6336, 6337, 6338, 7151, 7159, 7194])},
    "CystOfBreast": {"399294002": ("Cyst of breast", [6030, 6031, 6054, 6064])},
    "Cystadenocarcinoma": {"21008007": ("Cystadenocarcinoma", [638, 639])},
    "CysticAdventitialDisease": {"234021009": ("cystic adventitial disease", [3805])},
    "CysticLymphNode": {"280556009": ("cystic lymph node", [7600, 8134])},
    "CysticMedialNecrosis": {
        "42182000": ("Cystic medial necrosis (morphologic abnormality)", [])
    },
    "CysticMedicalNecrosis": {"42182000": ("cystic medical necrosis", [3817])},
    "Cytokine": {"75777003": ("Cytokine", [637])},
    "Cytology": {"310200001": ("Cytology", [7030])},
    "CytologyService": {"310200001": ("Cytology service (qualifier value)", [])},
    "CytotoxicAgent": {"373526007": ("Cytotoxic agent", [4410])},
    "CzechColdbloodHorseBreed": {"132737009": ("Czech Coldblood horse breed", [7480])},
    "CzechImprovedWhitePigBreed": {
        "133215008": ("Czech Improved White pig breed", [7480])
    },
    "CzechMeatPigBreed": {"133222000": ("Czech Meat pig breed", [7480])},
    "CzechMiniaturePigBreed": {"133223005": ("Czech Miniature pig breed", [7480])},
    "CzechPiedCattleBreed": {"133368004": ("Czech Pied cattle breed", [7480])},
    "CzechWarmbloodHorseBreed": {"131915005": ("Czech Warmblood horse breed", [7480])},
    "CzechoslovakianSmallRidingHorseHorseBreed": {
        "132738004": ("Czechoslovakian Small Riding Horse horse breed", [7480])
    },
    "CãoDaSerraDeAiresDogBreed": {
        "132417006": ("Cão da Serra de Aires dog breed", [7480])
    },
    "CãoDeCastroLaboreiroDogBreed": {
        "132418001": ("Cão de Castro Laboreiro dog breed", [7480])
    },
    "CãoDeFilaMiguelDogBreed": {"132419009": ("Cão de Fila Miguel dog breed", [7480])},
    "DBIPigBreed": {"132328006": ("DBI pig breed", [7480])},
    "DHSPlate": {"257327003": ("DHS Plate", [7307, 7309])},
    "DKPig30PigBreed": {"41561001": ("DK pig 30 pig breed", [7480])},
    "DKPig31PigBreed": {"36570001": ("DK pig 31 pig breed", [7480])},
    "DKPig33PigBreed": {"6053007": ("DK pig 33 pig breed", [7480])},
    "DKPig51PigBreed": {"8516002": ("DK pig 51 pig breed", [7480])},
    "DKPig61PigBreed": {"61973002": ("DK pig 61 pig breed", [7480])},
    "DKPig63PigBreed": {"112490000": ("DK pig 63 pig breed", [7480])},
    "DKPig77PigBreed": {"11161001": ("DK pig 77 pig breed", [7480])},
    "DKPigBreed": {"74921000": ("DK pig breed", [7480])},
    "DLSSheepBreed": {"132855006": ("DLS sheep breed", [7480])},
    "DRUTMTerminalsPigBreed": {
        "132209001": ("DRU (TM) Terminals pig breed (organism)", [])
    },
    "DRUTerminalsPigBreed": {"132209001": ("DRU™ Terminals pig breed", [7480])},
    "DabieshanCattleBreed": {"131464008": ("Dabieshan cattle breed", [7480])},
    "DachshundMiniatureBreed": {
        "132369002": ("Dachshund, Miniature breed (organism)", [])
    },
    "DachshundMiniatureDogBreed": {
        "132369002": ("Dachshund, Miniature dog breed", [7480])
    },
    "DachshundSuperbreedOfDog": {"2062007": ("Dachshund superbreed of dog", [7480])},
    "DaeraDinPanahGoatBreed": {"131653003": ("Daera Din Panah goat breed", [7480])},
    "DagestanMountainCattleBreed": {
        "133369007": ("Dagestan Mountain cattle breed", [7480])
    },
    "DahePigBreed": {"132152007": ("Dahe pig breed", [7480])},
    "DairyShorthornCattleBreed": {
        "133370008": ("Dairy Shorthorn cattle breed", [7480])
    },
    "DairySyntheticCattleBreed": {
        "133371007": ("Dairy Synthetic cattle breed", [7480])
    },
    "DairyZebuOfUberabaCattleBreed": {
        "133646004": ("Dairy Zebu of Uberaba cattle breed", [7480])
    },
    "DajalCattleBreed": {"133602007": ("Dajal cattle breed", [7480])},
    "DalaSheepBreed": {"131751006": ("Dala sheep breed", [7480])},
    "DalesPonyHorseBreed": {"133099006": ("Dales Pony horse breed", [7480])},
    "DalesbredSheepBreed": {"131750007": ("Dalesbred sheep breed", [7480])},
    "Dalland020PigBreed": {"132206008": ("Dalland 020 pig breed", [7480])},
    "Dalland030PigBreed": {"132050002": ("Dalland 030 pig breed", [7480])},
    "Dalland080PigBreed": {"132171009": ("Dalland 080 pig breed", [7480])},
    "DalmatianDog": {"5916008": ("Dalmatian dog", [7480])},
    "DamaniGoatBreed": {"131689006": ("Damani goat breed", [7480])},
    "DamaniSheepBreed": {"131749007": ("Damani sheep breed", [7480])},
    "DamaraCattleBreed": {"131465009": ("Damara cattle breed", [7480])},
    "DamaraSheepBreed": {"131748004": ("Damara sheep breed", [7480])},
    "DamascusCattleBreed": {"131588007": ("Damascus cattle breed", [7480])},
    "DamiettaCattleBreed": {"131432001": ("Damietta cattle breed", [7480])},
    "DamusStanselKayeOperation": {
        "233134001": ("Damus-Stansel-Kaye operation", [12247])
    },
    "DanakilCattleBreed": {"131466005": ("Danakil cattle breed", [7480])},
    "DandieDinmontTerrier": {"3347005": ("Dandie dinmont terrier", [7480])},
    "DaneliusMiller": {"424811006": ("Danelius-Miller", [4012])},
    "DaneliusMillerProjection": {
        "424811006": ("Danelius-Miller projection (qualifier value)", [])
    },
    "DangiCattleBreed": {"133603002": ("Dangi cattle breed", [7480])},
    "DanishBroholmerDogBreed": {"132491008": ("Danish Broholmer dog breed", [7480])},
    "DanishDurocPigBreed": {"132265005": ("Danish Duroc pig breed", [7480])},
    "DanishHampshirePigBreed": {"132266006": ("Danish Hampshire pig breed", [7480])},
    "DanishJerseyCattleBreed": {"133580006": ("Danish Jersey cattle breed", [7480])},
    "DanishLandraceGoatBreed": {"131661008": ("Danish Landrace goat breed", [7480])},
    "DanishLandracePig": {"84528008": ("Danish landrace pig (organism)", [])},
    "DanishLandracePigBreed": {"84528008": ("Danish landrace pig breed", [7480])},
    "DanishLandraceSheepBreed": {"131737006": ("Danish Landrace sheep breed", [7480])},
    "DanishLargeWhitePigBreed": {"132264009": ("Danish Large White pig breed", [7480])},
    "DanishOldenborgHorseBreed": {
        "132779003": ("Danish Oldenborg horse breed", [7480])
    },
    "DanishRedCattleBreed": {"131589004": ("Danish Red cattle breed", [7480])},
    "DanishRedPiedCattleBreed": {"133372000": ("Danish Red Pied cattle breed", [7480])},
    "DanishSportPonyHorseBreed": {
        "132757008": ("Danish Sport Pony horse breed", [7480])
    },
    "DanishWarmbloodHorseBreed": {
        "131819000": ("Danish Warmblood horse breed", [7480])
    },
    "DanubeWhitePigBreed": {"133193005": ("Danube White pig breed", [7480])},
    "DanubianHorseBreed": {"131919004": ("Danubian horse breed", [7480])},
    "DarkfieldStop": {"445624009": ("Darkfield stop", [8124])},
    "DarkfieldStopOfOpticalMicroscope": {
        "445624009": ("Darkfield stop of optical microscope (physical object)", [])
    },
    "DartmoorPonyHorseBreed": {"133027001": ("Dartmoor Pony horse breed", [7480])},
    "DartmoorSheepBreed": {"131717007": ("Dartmoor sheep breed", [7480])},
    "DashtiariCattleBreed": {"133647008": ("Dashtiari cattle breed", [7480])},
    "DatongHorseBreed": {"131935006": ("Datong horse breed", [7480])},
    "DaweiziPigBreed": {"132295002": ("Daweizi pig breed", [7480])},
    "DeSénarmontCompensator": {"445663002": ("de Sénarmont compensator", [8124])},
    "DeSénarmontCompensatorOfOpticalMicroscope": {
        "445663002": (
            "de Sénarmont compensator of optical microscope (physical object)",
            [],
        )
    },
    "Dead": {"419099009": ("Dead (finding)", [])},
    "DebouilletSheep": {"44835005": ("Debouillet sheep (organism)", [])},
    "DebouilletSheepBreed": {"44835005": ("Debouillet sheep breed", [7480])},
    "Deceased": {"419099009": ("Deceased", [3772])},
    "DeciduousMandibularLeftCanineTooth": {
        "43281008": ("Deciduous mandibular left canine tooth", [4019, 4026])
    },
    "DeciduousMandibularLeftCentralIncisorTooth": {
        "89552004": ("Deciduous mandibular left central incisor tooth", [4019, 4026])
    },
    "DeciduousMandibularLeftFirstMolarTooth": {
        "38896004": ("Deciduous mandibular left first molar tooth", [4019, 4026])
    },
    "DeciduousMandibularLeftLateralIncisorTooth": {
        "14770005": ("Deciduous mandibular left lateral incisor tooth", [4019, 4026])
    },
    "DeciduousMandibularLeftSecondMolarTooth": {
        "49330006": ("Deciduous mandibular left second molar tooth", [4019, 4026])
    },
    "DeciduousMandibularRightCanineTooth": {
        "6062009": ("Deciduous mandibular right canine tooth", [4019, 4026])
    },
    "DeciduousMandibularRightCentralIncisorTooth": {
        "67834006": ("Deciduous mandibular right central incisor tooth", [4019, 4026])
    },
    "DeciduousMandibularRightFirstMolarTooth": {
        "58646007": ("Deciduous mandibular right first molar tooth", [4019, 4026])
    },
    "DeciduousMandibularRightLateralIncisorTooth": {
        "22445006": ("Deciduous mandibular right lateral incisor tooth", [4019, 4026])
    },
    "DeciduousMandibularRightSecondMolarTooth": {
        "61868007": ("Deciduous mandibular right second molar tooth", [4019, 4026])
    },
    "DeciduousMaxillaryLeftCanineTooth": {
        "73937000": ("Deciduous maxillary left canine tooth", [4019, 4026])
    },
    "DeciduousMaxillaryLeftCentralIncisorTooth": {
        "51678005": ("Deciduous maxillary left central incisor tooth", [4019, 4026])
    },
    "DeciduousMaxillaryLeftFirstMolarTooth": {
        "45234009": ("Deciduous maxillary left first molar tooth", [4019, 4026])
    },
    "DeciduousMaxillaryLeftLateralIncisorTooth": {
        "43622005": ("Deciduous maxillary left lateral incisor tooth", [4019, 4026])
    },
    "DeciduousMaxillaryLeftSecondMolarTooth": {
        "51943008": ("Deciduous maxillary left second molar tooth", [4019, 4026])
    },
    "DeciduousMaxillaryRightCanineTooth": {
        "30618001": ("Deciduous maxillary right canine tooth", [4019, 4026])
    },
    "DeciduousMaxillaryRightCentralIncisorTooth": {
        "88824007": ("Deciduous maxillary right central incisor tooth", [4019, 4026])
    },
    "DeciduousMaxillaryRightFirstMolarTooth": {
        "17505006": ("Deciduous maxillary right first molar tooth", [4019, 4026])
    },
    "DeciduousMaxillaryRightLateralIncisorTooth": {
        "65624003": ("Deciduous maxillary right lateral incisor tooth", [4019, 4026])
    },
    "DeciduousMaxillaryRightSecondMolarTooth": {
        "27855007": ("Deciduous maxillary right second molar tooth", [4019, 4026])
    },
    "Decongestant": {"96328007": ("Decongestant", [621, 622])},
    "DecongestantPreparation": {"96328007": ("Decongestant preparation (product)", [])},
    "DecreaseInNumberOfCalcifications": {
        "129727007": ("Decrease in number of calcifications", [6002, 6003])
    },
    "DecreaseInNumberOfCalcificationsSincePreviousMammogram": {
        "129727007": (
            "Decrease in number of calcifications since previous mammogram (finding)",
            [],
        )
    },
    "DecreaseInSize": {"19776001": ("Decrease in size", [6002, 6003, 6134])},
    "Decreased": {"1250004": ("Decreased", [3642])},
    "DecreasedMyocardialIschemiaComparedToPriorStudy": {
        "429232006": (
            "Decreased myocardial ischemia compared to prior study (finding)",
            [],
        )
    },
    "DecreasedSize": {"19776001": ("Decreased size (finding)", [])},
    "DecreasedTolerance": {"102460003": ("Decreased tolerance", [3236])},
    "Deep": {"795002": ("Deep", [2, 3489])},
    "DeepAnteriorCervicalLymphNode": {
        "168360002": ("deep anterior cervical lymph node", [7600, 8134])
    },
    "DeepCervicalLymphNode": {"279145002": ("deep cervical lymph node", [7600, 8134])},
    "DeepInguinalLymphNode": {"65266007": ("deep inguinal lymph node", [7600, 8134])},
    "DeepIntraparotidLymphNode": {
        "75040000": ("deep intraparotid lymph node", [7600, 8134])
    },
    "DeepLateralCervicalLymphNode": {
        "167864002": ("deep lateral cervical lymph node", [7600, 8134])
    },
    "DeepLymphNode": {"60996007": ("deep lymph node", [7600, 8134])},
    "DeepPalmarArchOfRadialArtery": {
        "10119003": (
            "Deep Palmar Arch of Radial Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12107],
        )
    },
    "DeepPalmarVenousArch": {
        "368481004": (
            "Deep Palmar Venous Arch",
            [3827, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "DeepParotidLymphNode": {"279142004": ("deep parotid lymph node", [7600, 8134])},
    "DeepPoplitealLymphNode": {"35721009": ("deep popliteal lymph node", [7600, 8134])},
    "Defibrillator": {"72506001": ("Defibrillator", [7151, 7157, 7193])},
    "DefiniteThrombus": {"373142009": ("Definite Thrombus", [3714])},
    "DegreeOfBloodVesselLumenCrossSectionalAreaReduction": {
        "408714007": (
            "Degree of blood vessel lumen cross sectional area reduction (observable entity)",
            [],
        )
    },
    "DegreeOfBloodVesselLumenDiameterReduction": {
        "408715008": (
            "Degree of blood vessel lumen diameter reduction (observable entity)",
            [],
        )
    },
    "DehydrationOfTissueSpecimen": {
        "433470006": ("Dehydration of tissue specimen (procedure)", [])
    },
    "DekalbHybridPigLine": {"74921000": ("Dekalb hybrid pig line (organism)", [])},
    "DekalbHybridPigLine30": {"41561001": ("Dekalb hybrid pig line 30 (organism)", [])},
    "DekalbHybridPigLine31": {"36570001": ("Dekalb hybrid pig line 31 (organism)", [])},
    "DekalbHybridPigLine33": {"6053007": ("Dekalb hybrid pig line 33 (organism)", [])},
    "DekalbHybridPigLine51": {"8516002": ("Dekalb hybrid pig line 51 (organism)", [])},
    "DekalbHybridPigLine61": {"61973002": ("Dekalb hybrid pig line 61 (organism)", [])},
    "DekalbHybridPigLine63": {
        "112490000": ("Dekalb hybrid pig line 63 (organism)", [])
    },
    "DekalbHybridPigLine77": {"11161001": ("Dekalb hybrid pig line 77 (organism)", [])},
    "DelaineMerinoSheep": {"46392004": ("Delaine merino sheep (organism)", [])},
    "DelaineMerinoSheepBreed": {"46392004": ("Delaine merino sheep breed", [7480])},
    "DelibozHorseBreed": {"133026005": ("Deliboz horse breed", [7480])},
    "DeliveredRadiationDose": {"371892002": ("Delivered Radiation Dose", [3425])},
    "DelphianLymphNode": {"167664004": ("delphian lymph node", [7600, 8134])},
    "DeltoidMuscle": {
        "35259002": ("Deltoid muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "DengchuanCattleBreed": {"133373005": ("Dengchuan cattle breed", [7480])},
    "DentalProsthesis": {"27606000": ("Dental Prosthesis", [7151, 7157, 7193])},
    "DentalProsthesisDevice": {
        "27606000": ("Dental prosthesis, device (physical object)", [])
    },
    "DentalRadiography": {"1290849002": ("Dental radiography", [10005])},
    "DentalSurgery": {"309972001": ("Dental Surgery", [7030])},
    "DentalSurgeryDepartment": {
        "309972001": ("Dental surgery department (environment)", [])
    },
    "DeogirCattleBreed": {"133701005": ("Deogir cattle breed", [7480])},
    "DeoniCattleBreed": {"133604008": ("Deoni cattle breed", [7480])},
    "DependenceOnEnablingMachineOrDevice": {
        "105501005": ("Dependence on enabling machine or device", [3205])
    },
    "DepolarizingAgent": {"373250005": ("Depolarizing agent (substance)", [])},
    "DepressionOfLeftVentricularSystolicFunction": {
        "371862006": ("Depression of left ventricular systolic function", [3728])
    },
    "Depth": {"131197000": ("Depth", [218, 6165, 7469, 7470])},
    "DepthOfVesselFromSurface": {
        "413975003": ("Depth of vessel from surface (observable entity)", [])
    },
    "DerbyshireGritstoneSheepBreed": {
        "131735003": ("Derbyshire Gritstone sheep breed", [7480])
    },
    "DerivedFlowNonValve": {"371839009": ("Derived Flow, Non-Valve", [3617])},
    "DerivedPeriodNonValve": {"371853009": ("Derived Period, Non-Valve", [3616])},
    "DermantsiPiedPigBreed": {"133194004": ("Dermantsi Pied pig breed", [7480])},
    "Dermatology": {"309923008": ("Dermatology", [7030])},
    "DermatologyDepartment": {
        "309923008": ("Dermatology department (environment)", [])
    },
    "DermoscopicPhotography": {
        "446078004": ("Dermoscopic photography (procedure)", [])
    },
    "DesFossesCommunesDeLOuestGoatBreed": {
        "131690002": ("Des Fosses (Communes de l'Ouest) goat breed", [7480])
    },
    "Descending": {"75294000": ("Descending", [5])},
    "DescendingAorta": {
        "281130003": (
            "Descending aorta",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514, 12141],
        )
    },
    "DescendingAortaStructure": {
        "281130003": ("Descending aorta structure (body structure)", [])
    },
    "DescendingColon": {"32622004": ("Descending colon", [6210, 8134])},
    "DescendingColonStructure": {
        "32622004": ("Descending colon structure (body structure)", [])
    },
    "Desflurane": {"386841003": ("Desflurane", [623, 624])},
    "DetachableBalloon": {"102320000": ("Detachable balloon", [8, 4051, 7151, 7193])},
    "DetachableBalloonDevice": {
        "102320000": ("Detachable balloon, device (physical object)", [])
    },
    "DeuteriumOxide": {"12977001": ("Deuterium oxide", [11002])},
    "DeutscheBrackeDogBreed": {"132498002": ("Deutsche bracke dog breed", [7480])},
    "DeutschesBlaukoepfigesFleischschafSheepBreed": {
        "131734004": ("Deutsches Blaukoepfiges Fleischschaf sheep breed", [7480])
    },
    "DeviceAppliedToPatient": {"373061006": ("Device applied to patient", [3422])},
    "DeviceAtSiteOfInterest": {"371876007": ("Device at site of interest", [3422])},
    "DeviceCrossedSeptum": {"386125002": ("Device crossed septum", [3422])},
    "DeviceInsertedIntoSheath": {"371877003": ("Device inserted into sheath", [3422])},
    "DeviceUsed": {"373062004": ("Device used", [3422])},
    "DeviceWithdrawnAndOrRemoved": {
        "371875006": ("Device withdrawn and / or removed (finding)", [])
    },
    "DeviceWithdrawnRemoved": {"371875006": ("Device withdrawn / removed", [3422])},
    "DevonCattleBreed": {"131590008": ("Devon cattle breed", [7480])},
    "DevonClosewoolSheepBreed": {"131733005": ("Devon Closewool sheep breed", [7480])},
    "DevonRexCatBreed": {"51692004": ("Devon rex cat breed", [7480])},
    "DexamethasoneSodiumPhosphate": {
        "396017000": ("Dexamethasone sodium phosphate (substance)", [])
    },
    "DexamethasoneSodiumPhosphate_Inactive": {
        "49992008": ("Dexamethasone sodium sulfate", [65])
    },
    "DexterCattleBreed": {"53031002": ("Dexter cattle breed (organism)", [])},
    "DexterCowBreed": {"53031002": ("Dexter cow breed", [7480])},
    "DexterKerryCattleBreed": {"133374004": ("Dexter-Kerry cattle breed", [7480])},
    "Dextran": {"13132007": ("Dextran", [10, 70, 3850])},
    "Dextrocardia": {"27637000": ("Dextrocardia", [12248, 12249])},
    "DhanniCattleBreed": {"133605009": ("Dhanni cattle breed", [7480])},
    "DiabeticOnDietOnly": {"170745003": ("Diabetic on diet only (finding)", [])},
    "DiabeticOnDietaryTreatment": {
        "170745003": ("Diabetic on Dietary Treatment", [3722])
    },
    "DiabeticOnInsulin": {"170747006": ("Diabetic on Insulin", [3722])},
    "DiabeticOnOralTreatment": {"170746002": ("Diabetic on Oral Treatment", [3722])},
    "DiabeticRetinopathyStudyField1": {
        "408734008": ("Diabetic Retinopathy Study field 1", [4207])
    },
    "DiabeticRetinopathyStudyField2": {
        "410434001": ("Diabetic Retinopathy Study field 2", [4207])
    },
    "DiabeticRetinopathyStudyField3": {
        "410435000": ("Diabetic Retinopathy Study field 3", [4207])
    },
    "DiabeticRetinopathyStudyField4": {
        "410436004": ("Diabetic Retinopathy Study field 4", [4207])
    },
    "DiabeticRetinopathyStudyField5": {
        "410437008": ("Diabetic Retinopathy Study field 5", [4207])
    },
    "DiabeticRetinopathyStudyField6": {
        "410438003": ("Diabetic Retinopathy Study field 6", [4207])
    },
    "DiabeticRetinopathyStudyField7": {
        "410439006": ("Diabetic Retinopathy Study field 7", [4207])
    },
    "Diagnostic": {"261004008": ("Diagnostic", [4256, 6051, 6058, 6061])},
    "DiagnosticAspirationOfBreastCyst": {
        "287572003": ("Diagnostic aspiration of breast cyst", [6050, 6083])
    },
    "DiagnosticImaging": {"441662001": ("Diagnostic Imaging", [7030])},
    "DiagnosticImagingDepartment": {
        "441662001": ("Diagnostic imaging department (environment)", [])
    },
    "DiagnosticIntent": {"261004008": ("Diagnostic Intent", [3629, 6319, 7064])},
    "DiagnosticProcedure": {"103693007": ("Diagnostic procedure (procedure)", [])},
    "DiagnosticProcedureOnBloodVessel": {
        "83422003": ("Diagnostic procedure on blood vessel (procedure)", [])
    },
    "DiagnosticRadiographyStereotacticLocalization": {
        "64318009": ("Diagnostic radiography, stereotactic localization", [6058, 6060])
    },
    "DiagnosticRadiographyWithContrastMedia": {
        "27483000": ("Diagnostic radiography with contrast media", [10014])
    },
    "DiagnosticRadioisotope": {"17600005": ("Diagnostic radioisotope", [25])},
    "DiagnosticUltrasonography": {
        "16310003": ("Diagnostic ultrasonography", [6028, 6029])
    },
    "DialiCattleBreed": {"133648003": ("Diali cattle breed", [7480])},
    "Diameter": {
        "81827009": (
            "Diameter",
            [
                218,
                3423,
                6165,
                7469,
                7470,
                12008,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
                12304,
            ],
        )
    },
    "DiameterOfCircumscribedCircle": {
        "131192006": ("Diameter of circumscribed circle", [218, 6165, 7469, 7470])
    },
    "DiamondBlackStain": {"72572003": ("diamond black stain", [8112])},
    "DianiPigBreed": {"133176005": ("Diani pig breed", [7480])},
    "Diaphragm": {
        "5798000": (
            "Diaphragm",
            [1000, 1001, 1004, 6102, 6109, 6116, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "DiaphragmStructure": {"5798000": ("Diaphragm structure (body structure)", [])},
    "DiaphragmaticLymphNode": {"196751009": ("diaphragmatic lymph node", [7600, 8134])},
    "Diastasis": {"444469002": ("Diastasis", [12233, 12307])},
    "DiastasisOfCardiacCycle": {
        "444469002": ("Diastasis of cardiac cycle (qualifier value)", [])
    },
    "Diastole": {"90892000": ("Diastole", [12233, 12307])},
    "DiastoleFunction": {"90892000": ("Diastole, function (observable entity)", [])},
    "Diastolic": {"90892000": ("Diastolic", [3472])},
    "DiastolicBloodPressure": {
        "271650006": ("Diastolic blood pressure (observable entity)", [])
    },
    "DiastolicPressure": {"271650006": ("Diastolic Pressure", [3641])},
    "DiastolicPressureEqualization": {
        "413985002": ("Diastolic pressure equalization (finding)", [])
    },
    "DiastolicRapidInflow": {"444392003": ("Diastolic Rapid Inflow", [12233])},
    "DiastolicRapidInflowEWave": {
        "444392003": ("Diastolic Rapid Inflow (E-wave)", [12307])
    },
    "Diatrizoate": {"12335007": ("Diatrizoate", [12, 3850])},
    "Diazepam": {"387264003": ("Diazepam", [623, 625])},
    "DibromofluoresceinStain": {"17172002": ("dibromofluorescein stain", [8112])},
    "DichroicBeamsplitter": {"445316008": ("Dichroic beamsplitter", [8124])},
    "DidingaCattleBreed": {"133649006": ("Didinga cattle breed", [7480])},
    "Diencephalon": {"87563008": ("Diencephalon", [7151, 7153, 7192, 8134, 9514])},
    "DiethylEther": {"259170003": ("Diethyl ether", [623, 624])},
    "DifferenceInAttenuation": {"442707000": ("Difference in attenuation", [6207])},
    "DifferenceInBorderDefinition": {
        "442688001": ("Difference in border definition", [6134])
    },
    "DifferenceInBorderShape": {"442755000": ("Difference in border shape", [6134])},
    "DifferenceInDistribution": {"442704007": ("Difference in distribution", [6134])},
    "DifferenceInLocation": {"129808005": ("Difference in location", [6037])},
    "DifferenceInMargin": {"129812004": ("Difference in margin", [6038])},
    "DifferenceInNumberOfCalcifications": {
        "129810007": ("Difference in number of calcifications", [6037])
    },
    "DifferenceInOpacity": {"129807000": ("Difference in opacity", [6037])},
    "DifferenceInShape": {"129811006": ("Difference in shape", [6038])},
    "DifferenceInSiteInvolvement": {
        "442711006": ("Difference in site involvement", [6134])
    },
    "DifferenceInSize": {"129806009": ("Difference in size", [6037])},
    "DifferenceInSpatialProximity": {
        "129809002": ("Difference in spatial proximity", [6037])
    },
    "DifferenceInSubstance": {"442691001": ("Difference in substance", [6134])},
    "DifferenceInSymmetry": {"129813009": ("Difference in symmetry", [6038])},
    "DifferenceInTexture": {"442700003": ("Difference in Texture", [6134])},
    "Diffuse": {"19648000": ("Diffuse", [6128, 6335, 6336, 6337])},
    "DiffuseCalcificationDistribution": {
        "129764001": ("Diffuse calcification distribution", [6012, 6013])
    },
    "DiffuseDisease": {"371915000": ("Diffuse Disease", [3712])},
    "DiffuseDiseaseOfCoronaryArtery": {
        "371915000": ("Diffuse disease of coronary artery (finding)", [])
    },
    "DiffuseFibroelastosis": {"125358004": ("diffuse fibroelastosis", [3817])},
    "DiffuseInflammatoryErythema": {
        "402603005": ("Diffuse inflammatory erythema", [60, 9300])
    },
    "Digit": {"82680008": ("Digit", [7482, 7483, 8134])},
    "DigitStructure": {"82680008": ("Digit structure (body structure)", [])},
    "DigitalArteryOfHand": {
        "40254007": (
            "Digital artery of hand",
            [3827, 7151, 7192, 8134, 9514, 12103, 12107],
        )
    },
    "DigitalExaminationOfRectum": {
        "410006001": ("Digital Examination of Rectum", [6321])
    },
    "DigitalImager": {"468440006": ("Digital imager", [7026, 7027, 7151, 7193])},
    "DigitalImagerRadiationTherapy": {
        "468440006": ("Digital imager, radiation therapy", [9268, 9271])
    },
    "DigitalImagingScannerComputedRadiography": {
        "468886001": ("Digital imaging scanner, computed radiography", [9268])
    },
    "Digoxin": {"796001": ("Digoxin", [10])},
    "Dilatation": {"25322007": ("dilatation", [3810])},
    "DilatedPortionOfSegment": {"413996005": ("Dilated portion of segment", [12116])},
    "DilationAndCurettage": {"13091001": ("Dilation and curettage", [6082])},
    "DingPigBreed": {"132134007": ("Ding pig breed", [7480])},
    "DingoDogBreed": {"709853007": ("Dingo dog breed", [7480])},
    "Diphenhydramine": {"372682005": ("Diphenhydramine", [65])},
    "Dipyridamole": {"66859009": ("Dipyridamole", [3204])},
    "DipyridamoleStressProtocol": {
        "422685009": ("Dipyridamole Stress protocol", [3261, 12001])
    },
    "DirectOphthalmoscope": {"409900009": ("Direct Ophthalmoscope", [4202])},
    "DirectionOfFlow": {"260674002": ("Direction of flow (attribute)", [])},
    "DirectionalCoronaryAtherectomyDCADevice": {
        "371796006": ("Directional Coronary Atherectomy (DCA) Device", [3411, 3429])
    },
    "DirectionalCoronaryAtherectomyDevice": {
        "371796006": ("Directional coronary atherectomy device (physical object)", [])
    },
    "Disability": {"21134002": ("Disability (finding)", [])},
    "DisabilitySevere": {"161045001": ("Disability - severe (finding)", [])},
    "Disabled": {"21134002": ("Disabled", [3772])},
    "Discoid": {"255282008": ("Discoid", [6128])},
    "DiscoidAtelectasis": {"40779009": ("Discoid atelectasis (disorder)", [])},
    "DiscolorationOfSkinOfBreast": {
        "290069002": ("Discoloration of skin of breast", [6055])
    },
    "Disease": {"64572001": ("Disease", [3769])},
    "DiseaseConditionDeterminationWellControlled": {
        "1194003": ("Disease condition determination, well controlled (finding)", [])
    },
    "DishtyCattleBreed": {"133800001": ("Dishty cattle breed (organism)", [])},
    "DishtyXZebuCattleBreed": {"133800001": ("Dishty X zebu cattle breed", [7480])},
    "DisorderOfBreastImplant": {"271989003": ("Disorder of breast implant", [6055])},
    "DisorderOfEyelid": {"60113004": ("Disorder of eyelid (disorder)", [])},
    "DisorderOfLung": {"19829001": ("Disorder of lung (disorder)", [])},
    "DisorderOfPericardium": {"55855009": ("Disorder of pericardium (disorder)", [])},
    "DisorderOfRefraction": {"39021009": ("Disorder of refraction (disorder)", [])},
    "DisplacementOfBreastImplant": {
        "399209000": ("Displacement of breast implant (procedure)", [])
    },
    "DissectingAneurysm": {"710864009": ("dissecting aneurysm", [3808])},
    "Dissection": {"122459003": ("Dissection", [8110])},
    "DissectionOfAorta": {"308546005": ("Dissection of aorta (disorder)", [])},
    "DissectionOfArtery": {"710864009": ("Dissection of artery (disorder)", [])},
    "DissectionProcedure": {"122459003": ("Dissection procedure (procedure)", [])},
    "Disseminated": {"65709003": ("Disseminated", [6128])},
    "DisseminatedMalignancyOfUnknownPrimary": {
        "285645000": ("Disseminated malignancy of unknown primary (disorder)", [])
    },
    "Distal": {"46053002": ("Distal", [2, 5, 211, 212, 3019, 12116, 12281])},
    "DistalCircumflexCoronaryArtery": {
        "6511003": (
            "Distal Circumflex Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514, 12292],
        )
    },
    "DistalFemur": {"310652005": ("Distal Femur", [7304, 8134])},
    "DistalHumerus": {"118495001": ("Distal Humerus", [7304, 8134])},
    "DistalLeftAnteriorDescendingCoronaryArtery": {
        "36672000": (
            "Distal Left Anterior Descending Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "DistalRadius": {"75129005": ("Distal Radius", [7304, 8134])},
    "DistalRightCoronaryArtery": {
        "41879009": (
            "Distal Right Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "DistalTibia": {"64605006": ("Distal Tibia", [7304, 8134])},
    "DistalUlna": {"91238003": ("Distal Ulna", [7304, 8134])},
    "DistilledWater": {"444883009": ("Distilled water", [610])},
    "Diuretic": {"372695000": ("Diuretic", [621, 622])},
    "DiureticAgent": {"30492008": ("Diuretic", [10, 3760])},
    "Diverticulum": {"31113003": ("diverticulum", [3810, 6201])},
    "Dizziness": {"404640003": ("Dizziness", [3220])},
    "DizzinessDueToDrug": {"473188002": ("Dizziness due to drug (disorder)", [])},
    "DjakoreCattleBreed": {"133801002": ("Djakore cattle breed (organism)", [])},
    "DjakoreXZebuCattleBreed": {"133801002": ("Djakore X zebu cattle breed", [7480])},
    "DjermaHorseBreed": {"133025009": ("Djerma horse breed", [7480])},
    "DnepropetrovskPigBreed": {"132075003": ("Dnepropetrovsk pig breed", [7480])},
    "DnieperCattleBreed": {"131467001": ("Dnieper cattle breed", [7480])},
    "DnieperPigBreed": {"132078001": ("Dnieper pig breed", [7480])},
    "DoayoCattleBreed": {"131468006": ("Doayo cattle breed", [7480])},
    "DobermanPinscher": {"47075006": ("Doberman pinscher", [7480])},
    "DobrogeaBlackPigBreed": {"133199009": ("Dobrogea Black pig breed", [7480])},
    "Dobutamine": {"26523005": ("Dobutamine", [3204])},
    "DobutamineStressProtocol": {
        "424225000": ("Dobutamine Stress protocol", [3261, 12001])
    },
    "DoddPerforatingVein": {
        "128554002": (
            "Dodd's perforating vein",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "DogueDeBordeauxDogBreed": {
        "132389001": ("Dogue de Bordeaux dog breed (organism)", [])
    },
    "DomesticLeopardCat": {"73271003": ("Domestic leopard cat", [7480])},
    "DomesticLonghairedCat": {"8419007": ("Domestic longhaired cat", [7480])},
    "DomesticMediumHairedCat": {"409914009": ("Domestic medium-haired cat", [7480])},
    "DomesticShorthairedCat": {"15020009": ("Domestic shorthaired cat", [7480])},
    "DonGoatBreed": {"131669005": ("Don goat breed", [7480])},
    "DonHorseBreed": {"132761002": ("Don horse breed", [7480])},
    "DonPigBreed": {"132103008": ("Don pig breed", [7480])},
    "DongolaCattleBreed": {"133650006": ("Dongola cattle breed", [7480])},
    "DongolaHorseBreed": {"133023002": ("Dongola horse breed", [7480])},
    "Doppler": {"420717009": ("Doppler", [3442])},
    "DopplerCatheterMethod": {"133911005": ("Doppler catheter method", [3241])},
    "DopplerColorFlow": {"261197005": ("Doppler Color Flow", [6058, 12224])},
    "DopplerContinuousWave": {"261198000": ("Doppler Continuous Wave", [6058, 12224])},
    "DopplerPulsed": {"261199008": ("Doppler Pulsed", [6058, 12224])},
    "DopplerUltrasonographyOfHeartTissue": {
        "439858009": ("Doppler ultrasonography of heart tissue (procedure)", [])
    },
    "DoranCattleBreed": {"133375003": ("Doran cattle breed", [7480])},
    "DornaCattleBreed": {"133376002": ("Dorna cattle breed", [7480])},
    "DorperSheepBreed": {"131732000": ("Dorper sheep breed", [7480])},
    "Dorsal20DegreeRostralVentrocaudalOblique": {
        "442745004": ("Dorsal 20 degree rostral-ventrocaudal oblique", [7484])
    },
    "Dorsal20DegreeRostralVentrocaudalObliqueProjection": {
        "442745004": (
            "Dorsal 20 degree rostral-ventrocaudal oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal35DegreeLateralPalmaromedialOblique": {
        "442746003": ("Dorsal 35 degree lateral-palmaromedial oblique", [7484])
    },
    "Dorsal35DegreeLateralPalmaromedialObliqueProjection": {
        "442746003": (
            "Dorsal 35 degree lateral-palmaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal35DegreeLateralPlantaromedialOblique": {
        "442606004": ("Dorsal 35 degree lateral-plantaromedial oblique", [7484])
    },
    "Dorsal35DegreeLateralPlantaromedialObliqueProjection": {
        "442606004": (
            "Dorsal 35 degree lateral-plantaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal35DegreeMedialPalmarolateralOblique": {
        "442583001": ("Dorsal 35 degree medial-palmarolateral oblique", [7484])
    },
    "Dorsal35DegreeMedialPalmarolateralObliqueProjection": {
        "442583001": (
            "Dorsal 35 degree medial-palmarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal35DegreeMedialPlantarolateralOblique": {
        "442608003": ("Dorsal 35 degree medial-plantarolateral oblique", [7484])
    },
    "Dorsal35DegreeMedialPlantarolateralObliqueProjection": {
        "442608003": (
            "Dorsal 35 degree medial-plantarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal40DegreeLateralPlantaromedialOblique": {
        "442585008": ("Dorsal 40 degree lateral-plantaromedial oblique", [7484])
    },
    "Dorsal40DegreeLateralPlantaromedialObliqueProjection": {
        "442585008": (
            "Dorsal 40 degree lateral-plantaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal40DegreeMedialPalmarolateralOblique": {
        "442621005": ("Dorsal 40 degree medial-palmarolateral oblique", [7484])
    },
    "Dorsal40DegreeMedialPalmarolateralObliqueProjection": {
        "442621005": (
            "Dorsal 40 degree medial-palmarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal40DegreeMedialPlantarolateralOblique": {
        "442622003": ("Dorsal 40 degree medial-plantarolateral oblique", [7484])
    },
    "Dorsal40DegreeMedialPlantarolateralObliqueProjection": {
        "442622003": (
            "Dorsal 40 degree medial-plantarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal45DegreeLateralPalmaromedialOblique": {
        "442597009": ("Dorsal 45 degree lateral-palmaromedial oblique", [7484])
    },
    "Dorsal45DegreeLateralPalmaromedialObliqueProjection": {
        "442597009": (
            "Dorsal 45 degree lateral-palmaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal45DegreeLateralPlantaromedialOblique": {
        "442643002": ("Dorsal 45 degree lateral-plantaromedial oblique", [7484])
    },
    "Dorsal45DegreeLateralPlantaromedialObliqueProjection": {
        "442643002": (
            "Dorsal 45 degree lateral-plantaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal45DegreeMedialPalmarolateral": {
        "442623008": ("Dorsal 45 degree medial-palmarolateral", [7484])
    },
    "Dorsal45DegreeMedialPalmarolateralProjection": {
        "442623008": (
            "Dorsal 45 degree medial-palmarolateral projection (qualifier value)",
            [],
        )
    },
    "Dorsal45DegreeMedialPlantarolateralOblique": {
        "442600005": ("Dorsal 45 degree medial-plantarolateral oblique", [7484])
    },
    "Dorsal45DegreeMedialPlantarolateralObliqueProjection": {
        "442600005": (
            "Dorsal 45 degree medial-plantarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal60DegreeLateralPalmaromedialOblique": {
        "442639001": ("Dorsal 60 degree lateral-palmaromedial oblique", [7484])
    },
    "Dorsal60DegreeLateralPalmaromedialObliqueProjection": {
        "442639001": (
            "Dorsal 60 degree lateral-palmaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal60DegreeLateralPlantaromedialOblique": {
        "442601009": ("Dorsal 60 degree lateral-plantaromedial oblique", [7484])
    },
    "Dorsal60DegreeLateralPlantaromedialObliqueProjection": {
        "442601009": (
            "Dorsal 60 degree lateral-plantaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal60DegreeMedialPalmarolateralOblique": {
        "442602002": ("Dorsal 60 degree medial-palmarolateral oblique", [7484])
    },
    "Dorsal60DegreeMedialPalmarolateralObliqueProjection": {
        "442602002": (
            "Dorsal 60 degree medial-palmarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal60DegreeMedialPlantarolateralOblique": {
        "442641000": ("Dorsal 60 degree medial-plantarolateral oblique", [7484])
    },
    "Dorsal60DegreeMedialPlantarolateralObliqueProjection": {
        "442641000": (
            "Dorsal 60 degree medial-plantarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal65DegreeProximalPalmarodistalOblique": {
        "442609006": ("Dorsal 65 degree proximal-palmarodistal oblique", [7484])
    },
    "Dorsal65DegreeProximalPalmarodistalObliqueProjection": {
        "442609006": (
            "Dorsal 65 degree proximal-palmarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsal65DegreeProximalPlantarodistalOblique": {
        "442624002": ("Dorsal 65 degree proximal-plantarodistal oblique", [7484])
    },
    "Dorsal65DegreeProximalPlantarodistalObliqueProjection": {
        "442624002": (
            "Dorsal 65 degree proximal-plantarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "DorsalAspectOfScapula": {"51698000": ("Dorsal aspect of scapula", [6115, 8134])},
    "DorsalFuniculus": {"59752008": ("dorsal funiculus", [7707, 7710, 8134])},
    "DorsalFuniculusStructure": {
        "59752008": ("Dorsal funiculus structure (body structure)", [])
    },
    "DorsalScapularArtery": {
        "91732003": (
            "Dorsal scapular artery",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "DorsalisPedisArtery": {
        "86547008": (
            "Dorsalis Pedis Artery",
            [3440, 3827, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "DorsetDownSheepBreed": {"131722007": ("Dorset Down sheep breed", [7480])},
    "DorsetSheepSuperbreed": {"25327001": ("Dorset sheep superbreed", [7480])},
    "DorsoVentral": {"441672003": ("Dorso-ventral", [7484])},
    "DorsoVentralProjection": {
        "441672003": ("Dorso-ventral projection (qualifier value)", [])
    },
    "DorsolateralPalmaromedialOblique": {
        "442657000": ("Dorsolateral-palmaromedial oblique", [7484])
    },
    "DorsolateralPalmaromedialObliqueProjection": {
        "442657000": (
            "Dorsolateral-palmaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "DorsolateralPlantaromedialOblique": {
        "442610001": ("Dorsolateral-plantaromedial oblique", [7484])
    },
    "DorsolateralPlantaromedialObliqueProjection": {
        "442610001": (
            "Dorsolateral-plantaromedial oblique projection (qualifier value)",
            [],
        )
    },
    "DorsomedialPalmarolateral": {"442729001": ("Dorsomedial-palmarolateral", [7484])},
    "DorsomedialPalmarolateralProjection": {
        "442729001": ("Dorsomedial-palmarolateral projection (qualifier value)", [])
    },
    "DorsomedialPlantarolateralOblique": {
        "442611002": ("Dorsomedial-plantarolateral oblique", [7484])
    },
    "DorsomedialPlantarolateralObliqueProjection": {
        "442611002": (
            "Dorsomedial-plantarolateral oblique projection (qualifier value)",
            [],
        )
    },
    "Dorsopalmar": {"441505008": ("Dorsopalmar", [7484])},
    "DorsopalmarProjection": {
        "441505008": ("Dorsopalmar projection (qualifier value)", [])
    },
    "Dorsoplantar": {"399335002": ("dorsoplantar", [501, 4010, 7484])},
    "DorsoplantarProjection": {
        "399335002": ("Dorsoplantar projection (qualifier value)", [])
    },
    "DorsoproximalPalmarodistalOblique": {
        "442744000": ("Dorsoproximal-palmarodistal oblique", [7484])
    },
    "DorsoproximalPalmarodistalObliqueProjection": {
        "442744000": (
            "Dorsoproximal-palmarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "DorsoproximalPlantarodistalOblique": {
        "442592003": ("Dorsoproximal-plantarodistal oblique", [7484])
    },
    "DorsoproximalPlantarodistalObliqueProjection": {
        "442592003": (
            "Dorsoproximal-plantarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "DorsorostralVentrocaudalOblique": {
        "442659002": ("Dorsorostral-ventrocaudal oblique", [7484])
    },
    "DorsorostralVentrocaudalObliqueProjection": {
        "442659002": (
            "Dorsorostral-ventrocaudal oblique projection (qualifier value)",
            [],
        )
    },
    "DortyolCattleBreed": {"133377006": ("Dortyol cattle breed", [7480])},
    "Dosage": {"260911001": ("Dosage", [6092])},
    "Dosimeter": {"15869005": ("Dosimeter", [10010, 10020])},
    "DosimeterDevice": {"15869005": ("Dosimeter, device (physical object)", [])},
    "DotatateGa68": {"724025002": ("Dotatate Ga^68^", [4021])},
    "DoubleCoronaryVesselDisease": {
        "194843003": ("Double coronary vessel disease (disorder)", [])
    },
    "DoubleVesselCoronaryArteryDisease": {
        "194843003": ("Double vessel coronary artery disease.", [3728])
    },
    "Down": {"255518004": ("Down", [4215])},
    "Downgaze": {"255521002": ("Downgaze", [4201])},
    "DownwardGaze": {"255521002": ("Downward gaze (qualifier value)", [])},
    "DraftPonySuperbreed": {"425253007": ("Draft pony superbreed (organism)", [])},
    "DraftPonySuperbreedHorseBreed": {
        "425253007": ("Draft pony superbreed horse breed", [7480])
    },
    "DrakensbergerCattleBreed": {"131551009": ("Drakensberger cattle breed", [7480])},
    "DrentsHeideschaapSheepBreed": {
        "132685003": ("Drents Heideschaap sheep breed", [7480])
    },
    "DrentsePatrijshondDogBreed": {
        "132493006": ("Drentse Patrijshond dog breed", [7480])
    },
    "DreverDog": {"56984005": ("Drever dog", [7480])},
    "DropShaped": {"1312007": ("Drop-shaped", [6335, 6339, 6340])},
    "Droperidol": {"387146001": ("Droperidol", [623, 625])},
    "DroughtmasterCattleBreed": {
        "133720007": ("Droughtmaster cattle breed (organism)", [])
    },
    "DroughtmasterXZebuCattleBreed": {
        "133720007": ("Droughtmaster X zebu cattle breed", [7480])
    },
    "Drowsy": {"271782001": ("Drowsy", [3035, 3050])},
    "DrugDiluent": {"74626007": ("Drug diluent", [621, 622])},
    "DrugInducedDizziness": {"473188002": ("Drug Induced Dizziness", [60, 9300])},
    "DrugInducedFlushing": {"403618004": ("Drug induced Flushing", [60, 9300])},
    "DrugInducedHypotension": {"234171009": ("Drug-induced hypotension", [60, 9300])},
    "DrugInfusion": {"133882006": ("Drug infusion", [91, 3271])},
    "DrugInfusionChallenge": {
        "133882006": ("Drug Infusion Challenge", [3250, 3405, 3651])
    },
    "DrugMisuseBehavior": {"228366006": ("Drug misuse behavior", [3774])},
    "DrugRash": {"28926001": ("Drug Rash", [60, 9300])},
    "DryEyes": {"162290004": ("Dry eyes (finding)", [])},
    "DryEyesProblem": {"162290004": ("Dry Eyes Problem", [4222])},
    "DrysdaleSheepBreed": {"131721000": ("Drysdale sheep breed", [7480])},
    "DualCatheterMethod": {"128573003": ("Dual catheter method", [3241])},
    "DualDiffuseDirectIllumination": {
        "410461001": ("Dual diffuse direct illumination", [4203])
    },
    "DualDiffuseDirectIlluminationAction": {
        "410461001": ("Dual diffuse direct illumination - action (qualifier value)", [])
    },
    "DualEnergyXRayAbsorptiometry": {
        "241686001": ("Dual energy X-Ray absorptiometry", [10005])
    },
    "DualEnergyXRayPhotonAbsorptiometry": {
        "241686001": ("Dual energy X-ray photon absorptiometry (procedure)", [])
    },
    "DuctalHyperplasiaUsual": {"67617000": ("Ductal hyperplasia, Usual", [6030, 6031])},
    "DukeCoronaryArteryDiseaseScore": {
        "304915008": ("Duke's coronary artery disease score (assessment scale)", [])
    },
    "DukeTreadmillScore": {"304915008": ("Duke treadmill score", [3238])},
    "DulmenPonyHorseBreed": {"131937003": ("Dulmen Pony horse breed", [7480])},
    "DunkerDogBreed": {"132494000": ("Dunker dog breed", [7480])},
    "Dunlap": {"399303002": ("Dunlap", [4012])},
    "DunlapProjection": {"399303002": ("Dunlap projection (qualifier value)", [])},
    "DuodenalStructure": {"38848004": ("Duodenal structure (body structure)", [])},
    "Duodenum": {
        "38848004": ("Duodenum", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "DuraMater": {"18545000": ("Dura mater", [5, 7151, 7153, 7192, 8134, 9514])},
    "DuraMaterStructure": {"18545000": ("Dura mater structure (body structure)", [])},
    "Duration": {"103335007": ("Duration (attribute)", [])},
    "DurazolRedStain": {"11780008": ("durazol red stain", [8112])},
    "During": {"272114000": ("During", [3600])},
    "DuringMenopause": {"303111005": ("During menopause", [6086])},
    "DuringProcedure": {"307154001": ("During Procedure", [631, 12002, 12102])},
    "DuringValues": {"272114000": ("During values (qualifier value)", [])},
    "DurocPig": {"3260001": ("Duroc pig (organism)", [])},
    "DurocPigBreed": {"3260001": ("Duroc pig breed", [7480])},
    "DutchBeltedCattleBreed": {"131592000": ("Dutch Belted cattle breed", [7480])},
    "DutchDraftHorseBreed": {"133016003": ("Dutch Draft horse breed", [7480])},
    "DutchFriesianCattleBreed": {"131593005": ("Dutch Friesian cattle breed", [7480])},
    "DutchKooikerDogBreed": {"132495004": ("Dutch Kooiker Dog breed", [7480])},
    "DutchLandracePig": {"58311005": ("Dutch landrace pig (organism)", [])},
    "DutchLandracePigBreed": {"58311005": ("Dutch landrace pig breed", [7480])},
    "DutchShepherdDogBreed": {"132496003": ("Dutch Shepherd dog breed", [7480])},
    "DutchTuigpaardHorseBreed": {"131947000": ("Dutch Tuigpaard horse breed", [7480])},
    "DutchWarmbloodHorseBreed": {"133022007": ("Dutch Warmblood horse breed", [7480])},
    "DutchYorkshirePigBreed": {"132270003": ("Dutch Yorkshire pig breed", [7480])},
    "DyeDilution": {"373104003": ("Dye Dilution", [3628])},
    "DyeDilutionCardiacOutputWaveform": {
        "128455004": ("Dye dilution cardiac output waveform", [3003])
    },
    "DyeDilutionCardiacOutputWaveformFunction": {
        "128455004": (
            "Dye dilution cardiac output waveform, function (observable entity)",
            [],
        )
    },
    "DynamicHipScrewPlate": {
        "257327003": ("Dynamic hip screw plate (physical object)", [])
    },
    "DynamicMagneticResonanceImagingOfKnee": {
        "433139009": ("Dynamic magnetic resonance imaging of knee", [100])
    },
    "DynamicMagneticResonanceImagingOfPelvis": {
        "446315002": ("Dynamic magnetic resonance imaging of pelvis", [100])
    },
    "Dyskinesis": {"25437005": ("Dyskinesis", [3703])},
    "Dyspnea": {"267036007": ("Dyspnea", [60, 3201, 3220, 3221, 9300, 12246])},
    "DystrophicCalcification": {
        "129750001": ("Dystrophic calcification", [6010, 6011])
    },
    "DystrophicRadiographicCalcification": {
        "129750001": ("Dystrophic radiographic calcification (finding)", [])
    },
    "DzumaliaPigBreed": {"133207005": ("Dzumalia pig breed", [7480])},
    "DølafeCattleBreed": {"131591007": ("Dølafe cattle breed", [7480])},
    "DøleHorseBreed": {"133024008": ("Døle horse breed", [7480])},
    "ECGAnalysis": {"258181008": ("ECG analysis", [3215])},
    "ECGEquivocal": {"370359005": ("ECG Equivocal", [3677])},
    "ECGNormal": {"164854000": ("Normal", [3230, 3677])},
    "EDTAGa68": {"423498000": ("EDTA Ga^68^", [4021])},
    "EEG": {"54550000": ("EEG", [3049])},
    "ELMExternalLimitingMembrane": {
        "76710003": ("ELM - External limiting membrane", [4273, 7151, 7192, 9514])
    },
    "Ear": {"117590005": ("Ear", [4, 7151, 7192, 8134, 9514])},
    "EarNoseAndThroatDepartment": {
        "309978002": ("Ear, nose and throat department (environment)", [])
    },
    "EarStructure": {"117590005": ("Ear structure (body structure)", [])},
    "EarlyDiastole": {"444389002": ("Early Diastole", [12233])},
    "EarlyDiastoleOfCardiacCycle": {
        "444389002": ("Early diastole of cardiac cycle (qualifier value)", [])
    },
    "EarlyRepolarization": {"428417006": ("Early repolarization", [3230])},
    "EastAnatolianRedCattleBreed": {
        "133378001": ("East Anatolian Red cattle breed", [7480])
    },
    "EastAndSoutheastAnadoluHorseBreed": {
        "131948005": ("East and Southeast Anadolu horse breed", [7480])
    },
    "EastBalkanPigBreed": {"132366009": ("East Balkan pig breed", [7480])},
    "EastBulgarianHorseBreed": {"132765006": ("East Bulgarian horse breed", [7480])},
    "EastFinnishCattleBreed": {"133379009": ("East Finnish cattle breed", [7480])},
    "EastFriesianOldTypeHorseBreed": {
        "132766007": ("East Friesian (Old Type) horse breed", [7480])
    },
    "EastFriesianSheepBreed": {"132853004": ("East Friesian sheep breed", [7480])},
    "EastFriesianWarmbloodModernTypeHorseBreed": {
        "132767003": ("East Friesian Warmblood (Modern Type) horse breed", [7480])
    },
    "EastMacedonianCattleBreed": {
        "133380007": ("East Macedonian cattle breed", [7480])
    },
    "EastSiberianLaikaDogBreed": {
        "132497007": ("East Siberian Laika dog breed", [7480])
    },
    "EasternNuerCattleBreed": {"131469003": ("Eastern Nuer cattle breed", [7480])},
    "Eccentric": {"255380003": ("Eccentric", [3491, 3806])},
    "EccentricFixation": {"251786004": ("Eccentric Fixation", [4221, 4222])},
    "Echocardiography": {"40701008": ("Echocardiography", [3206, 3744, 3757, 12001])},
    "EchocardiographyForDeterminingVentricularContraction": {
        "35757004": (
            "Echocardiography for determining ventricular contraction (procedure)",
            [],
        )
    },
    "Ectatic": {"386140000": ("Ectatic", [3712])},
    "EctaticCoronaryArtery": {"386140000": ("Ectatic coronary artery (finding)", [])},
    "EctopicAccessoryBreastTissue": {
        "1896004": ("Ectopic (accessory) breast tissue", [6030, 6031])
    },
    "EctopicBreastTissue": {"1896004": ("Ectopic breast tissue (disorder)", [])},
    "Eczema": {"43116000": ("Eczema", [4405])},
    "EdelschweinPigBreed": {"132353009": ("Edelschwein pig breed", [7480])},
    "Edema": {
        "79654002": (
            "Edema",
            [6014, 6015, 6016, 6030, 6031, 6054, 6056, 7151, 7159, 7168, 7169, 7194],
        )
    },
    "EdemaOfFetalChestWall": {"443115002": ("Edema of fetal chest wall", [12249])},
    "EdemaOfFetalScalp": {"443168008": ("Edema of fetal scalp", [12249])},
    "EdemaOfLarynx": {"51599000": ("Edema of larynx (disorder)", [])},
    "Edge": {"57183005": ("Edge", [2])},
    "Efferent": {"33843005": ("Efferent", [2])},
    "Effusion": {"41699000": ("Effusion", [12280])},
    "EggshellCalcification": {
        "129751002": ("Eggshell calcification", [6010, 6011, 6132])
    },
    "EggshellRadiographicCalcification": {
        "129751002": ("Eggshell radiographic calcification (finding)", [])
    },
    "EgyptianCattleBreed": {"131470002": ("Egyptian cattle breed", [7480])},
    "EgyptianHorseBreed": {"133017007": ("Egyptian horse breed", [7480])},
    "EgyptianMauCat": {"21637005": ("Egyptian mau cat", [7480])},
    "EighthIntercostalSpace": {"117140004": ("Eighth Intercostal Space", [5])},
    "ElasticVascularSclerosis": {"19952003": ("elastic vascular sclerosis", [3817])},
    "ElbowJoint": {
        "16953009": (
            "Elbow joint",
            [
                4,
                1000,
                1001,
                1006,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "ElbowJointStructure": {"16953009": ("Elbow joint structure (body structure)", [])},
    "Elective": {"103390000": ("Elective (qualifier value)", [])},
    "ElectiveProcedure": {"103390000": ("Elective Procedure", [3414])},
    "ElectricBlanket": {"79811009": ("Electric blanket", [635])},
    "ElectricBlanketDevice": {
        "79811009": ("Electric blanket, device (physical object)", [])
    },
    "ElectricHeatingPad": {"27812008": ("Electric heating pad", [635])},
    "ElectricHeatingPadDevice": {
        "27812008": ("Electric heating pad, device (physical object)", [])
    },
    "ElectrocardiogramAbnormal": {
        "102594003": ("Electrocardiogram abnormal (finding)", [])
    },
    "ElectrocardiogramAnalysis": {
        "258181008": ("Electrocardiogram analysis (qualifier value)", [])
    },
    "ElectrocardiogramEquivocal": {
        "370359005": ("Electrocardiogram equivocal (finding)", [])
    },
    "ElectrocardiogramFinding": {
        "271921002": ("Electrocardiogram finding (observable entity)", [])
    },
    "ElectrocardiogramSTIntervalNormal": {
        "164929001": ("Electrocardiogram: ST interval normal (finding)", [])
    },
    "ElectrocardiogramWithExerciseTest": {
        "46136006": ("Electrocardiogram with exercise test (procedure)", [])
    },
    "Electron": {"46602004": ("Electron", [9525])},
    "ElectronMicroscopyGrid": {"434533009": ("Electron microscopy grid", [8101, 8102])},
    "ElectrophysiologyMappingPhase": {
        "129092004": ("Electrophysiology Mapping phase", [3254])
    },
    "ElectrophysiologyProcedureBaselinePhase": {
        "129082007": ("Electrophysiology procedure baseline phase", [3254])
    },
    "ElegantWarmbloodHorseBreed": {
        "131961005": ("Elegant Warmblood horse breed", [7480])
    },
    "EleiaHorseBreed": {"132742001": ("Eleia horse breed", [7480])},
    "Elevated": {"75540009": ("Elevated", [3642])},
    "EleventhIntercostalSpace": {"117143002": ("Eleventh Intercostal Space", [5])},
    "EllestadProtocol": {"129098000": ("Ellestad protocol", [3261, 12001])},
    "ElliottdaleSheepBreed": {"131720004": ("Elliottdale sheep breed", [7480])},
    "Embolism": {"55584005": ("embolism", [3805, 3810, 3815])},
    "EmbolizationBall": {"102315000": ("Embolization ball", [8, 4051, 7151, 7193])},
    "EmbolizationBallDevice": {
        "102315000": ("Embolization ball, device (physical object)", [])
    },
    "EmbolizationCoil": {"102314001": ("Embolization coil", [8, 4051, 7151, 7193])},
    "EmbolizationCoilDevice": {
        "102314001": ("Embolization coil, device (physical object)", [])
    },
    "EmbolizationParticulate": {
        "102316004": ("Embolization particulate", [8, 4051, 7151, 7193])
    },
    "EmbolizationParticulateDevice": {
        "102316004": ("Embolization particulate, device (physical object)", [])
    },
    "Embolus": {"55584005": ("Embolus", [7151, 7159, 7194])},
    "EmbryonicVascularStructure": {
        "51852003": ("Embryonic vascular structure (body structure)", [])
    },
    "Emergency": {"25876001": ("Emergency (qualifier value)", [])},
    "EmergencyCoronaryArteryBypass": {
        "414088005": ("Emergency Coronary Artery Bypass", [3413, 3755])
    },
    "EmergencyCoronaryArteryBypassGraft": {
        "414088005": ("Emergency coronary artery bypass graft (procedure)", [])
    },
    "EmergencyDepartment": {"50849002": ("Emergency Department", [3729])},
    "EmergencyDepartmentPatientVisit": {
        "4525004": ("Emergency Department patient visit", [3671])
    },
    "EmergencyPercutaneousCoronaryIntervention": {
        "414089002": ("Emergency Percutaneous Coronary Intervention", [3413, 3755])
    },
    "EmergencyProcedure": {"373110003": ("Emergency procedure", [3671])},
    "EmergencyRoomAdmission": {
        "50849002": ("Emergency room admission (procedure)", [])
    },
    "EmergentProcedure": {"25876001": ("Emergent Procedure", [3414])},
    "EmptyRectum": {"249599008": ("Empty Rectum", [9272])},
    "EmptyVitreous": {"232077005": ("Empty vitreous (disorder)", [])},
    "Encapsulated": {"59135002": ("Encapsulated", [6335, 6341, 6342])},
    "EndDiastole": {"416190007": ("End diastole", [3337, 12233, 12307])},
    "EndOfProtocol": {"255253007": ("End of Protocol", [3221])},
    "EndSystole": {"416430001": ("End Systole", [12233, 12307])},
    "Ended": {"385656004": ("Ended", [3773])},
    "EnderbyIslandCattleBreed": {"133581005": ("Enderby Island cattle breed", [7480])},
    "EndoArterial": {"51114001": ("Endo-arterial", [4])},
    "EndoCardiac": {"80891009": ("Endo-cardiac", [4])},
    "EndoEsophageal": {"32849002": ("Endo-esophageal", [4])},
    "EndoNasal": {"53342003": ("Endo-nasal", [4, 7151, 7192, 9514])},
    "EndoNasopharyngeal": {
        "18962004": ("Endo-nasopharyngeal", [4, 7151, 7192, 8134, 9514])
    },
    "EndoRectal": {"34402009": ("Endo-rectal", [4])},
    "EndoRenal": {"64033007": ("Endo-renal", [4])},
    "EndoUreteric": {"87953007": ("Endo-ureteric", [4, 8134])},
    "EndoUrethral": {"13648007": ("Endo-urethral", [4])},
    "EndoVaginal": {"76784001": ("Endo-vaginal", [4])},
    "EndoVascular": {"59820001": ("Endo-vascular", [4])},
    "EndoVenous": {"29092000": ("Endo-venous", [4])},
    "EndoVesical": {"48367006": ("Endo-vesical", [4, 7151, 7192, 8134, 9514])},
    "Endocardial": {"304059001": ("endocardial", [3843])},
    "EndocrineSurgery": {"309979005": ("Endocrine Surgery", [7030])},
    "EndocrineSurgeryDepartment": {
        "309979005": ("Endocrine surgery department (environment)", [])
    },
    "Endocrinology": {"309925001": ("Endocrinology", [7030])},
    "EndocrinologyDepartment": {
        "309925001": ("Endocrinology department (environment)", [])
    },
    "EndometrialBiopsy": {"386802000": ("Endometrial biopsy", [6082])},
    "EndometrialStructure": {"2739003": ("Endometrial structure (body structure)", [])},
    "Endometrium": {"2739003": ("Endometrium", [8134, 12023])},
    "Endoscopy": {"310030000": ("Endoscopy", [7030])},
    "EndoscopyService": {"310030000": ("Endoscopy service (qualifier value)", [])},
    "EndotrachealTube": {
        "26412008": ("Endotracheal tube", [6102, 6138, 6404, 7151, 7193])
    },
    "EndotrachealTubeDevice": {
        "26412008": ("Endotracheal tube, device (physical object)", [])
    },
    "EnemaTubeDevice": {"67966000": ("Enema tube, device (physical object)", [])},
    "Enflurane": {"387176008": ("Enflurane", [623, 624])},
    "EnglishBulldogBreed": {"132388009": ("English Bulldog breed", [7480])},
    "EnglishCobHorseBreed": {"132743006": ("English Cob horse breed", [7480])},
    "EnglishCockerSpaniel": {"21418008": ("English cocker spaniel (organism)", [])},
    "EnglishCockerSpanielDogBreed": {
        "21418008": ("English cocker spaniel dog breed", [7480])
    },
    "EnglishCoonhound": {"31281003": ("English coonhound (organism)", [])},
    "EnglishCoonhoundDogBreed": {"31281003": ("English coonhound dog breed", [7480])},
    "EnglishFoxhound": {"59975009": ("English foxhound", [7480])},
    "EnglishGoatBreed": {"131691003": ("English goat breed", [7480])},
    "EnglishGuernseyGoatBreed": {"131692005": ("English Guernsey goat breed", [7480])},
    "EnglishHackHorseBreed": {"131964002": ("English Hack horse breed", [7480])},
    "EnglishHunterHorseBreed": {"132746003": ("English Hunter horse breed", [7480])},
    "EnglishLonghornCattleBreed": {
        "131594004": ("English Longhorn cattle breed", [7480])
    },
    "EnglishSetter": {"84367001": ("English setter (organism)", [])},
    "EnglishSetterDogBreed": {"84367001": ("English setter dog breed", [7480])},
    "EnglishShepherdDogBreed": {"132492001": ("English Shepherd dog breed", [7480])},
    "EnglishSpringerSpaniel": {"62228004": ("English Springer spaniel (organism)", [])},
    "EnglishSpringerSpanielDogBreed": {
        "62228004": ("English Springer spaniel dog breed", [7480])
    },
    "EnglishToySpaniel": {"67088002": ("English toy spaniel", [7480])},
    "EnglishToyTerrierDogBreed": {
        "132423001": ("English Toy Terrier dog breed", [7480])
    },
    "Enlarged": {"260376009": ("Enlarged", [6107, 6118])},
    "EnlargedAorticRoot": {"373133002": ("Enlarged Aortic Root", [3709])},
    "EnlargedSeptalForamenOfHeart": {
        "287272002": ("Enlarged septal foramen of heart (disorder)", [])
    },
    "Entire": {"255503000": ("Entire", [6347])},
    "EntireAnulusFibrosusOfMitralOrifice": {
        "279174006": ("Entire anulus fibrosus of mitral orifice (body structure)", [])
    },
    "EntireAnulusFibrosusOfTricuspidOrifice": {
        "279170002": (
            "Entire anulus fibrosus of tricuspid orifice (body structure)",
            [],
        )
    },
    "EntireBloodVessel": {"361097006": ("Entire blood vessel (body structure)", [])},
    "EntireBody": {
        "38266002": (
            "Entire body",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7482,
                7483,
                8103,
                8134,
                9514,
                10060,
            ],
        )
    },
    "EntireBodyAsAWhole": {"38266002": ("Entire body as a whole (body structure)", [])},
    "EntireBreast": {"181131000": ("Entire breast (body structure)", [])},
    "EntireDeciduousMandibularLeftCanineTooth": {
        "245639007": (
            "Entire deciduous mandibular left canine tooth (body structure)",
            [],
        )
    },
    "EntireDeciduousMandibularRightFirstMolarTooth": {
        "245631005": (
            "Entire deciduous mandibular right first molar tooth (body structure)",
            [],
        )
    },
    "EntireDeciduousMaxillaryRightCentralIncisorTooth": {
        "245620002": (
            "Entire deciduous maxillary right central incisor tooth (body structure)",
            [],
        )
    },
    "EntireDeciduousMaxillaryRightFirstMolarTooth": {
        "245616001": (
            "Entire deciduous maxillary right first molar tooth (body structure)",
            [],
        )
    },
    "EntireDeciduousMaxillaryRightLateralIncisorTooth": {
        "245619008": (
            "Entire deciduous maxillary right lateral incisor tooth (body structure)",
            [],
        )
    },
    "EntireGastrocnemiusVein": {
        "264481007": ("Entire gastrocnemius vein (body structure)", [])
    },
    "EntirePulmonaryVeinGreatVessel": {
        "53603007": ("Entire pulmonary vein (great vessel) (body structure)", [])
    },
    "EntireSkin": {"181469002": ("Entire skin (body structure)", [])},
    "EntireVessel": {"361097006": ("Entire Vessel", [3019])},
    "EntlebucherDogBreed": {"132486007": ("Entlebucher dog breed", [7480])},
    "EntorhinalCortex": {
        "3937002": ("Entorhinal Cortex", [7151, 7153, 7192, 8134, 9514])
    },
    "Entrapment": {"363563002": ("entrapment", [3805])},
    "EpagneulBleuDePicardieDogBreed": {
        "132478007": ("Epagneul Bleu de Picardie dog breed", [7480])
    },
    "EpagneulPicardDogBreed": {"132477002": ("Epagneul Picard dog breed", [7480])},
    "EpagneulPontAudemerDogBreed": {
        "132480001": ("Epagneul Pont-Audemer dog breed", [7480])
    },
    "Ephedrine": {"387358007": ("Ephedrine", [65])},
    "Epicardial": {"261073003": ("epicardial", [3843])},
    "EpicardialEchocardiography": {
        "433232009": ("Epicardial echocardiography", [3405, 3428, 12001])
    },
    "Epicardium": {"6871001": ("Epicardium", [3011, 8134])},
    "EpidermalInclusionCyst": {"419670003": ("Epidermal inclusion cyst", [6030, 6031])},
    "EpidermoidCyst": {"419670003": ("Epidermoid cyst (morphologic abnormality)", [])},
    "Epididymis": {"87644002": ("Epididymis", [8134])},
    "EpididymisStructure": {"87644002": ("Epididymis structure (body structure)", [])},
    "EpiduralAnesthesia": {"18946005": ("Epidural anesthesia", [611, 612])},
    "EpigastricLymphNode": {"60965003": ("epigastric lymph node", [7600, 8134])},
    "EpigastricRegion": {
        "27947004": ("Epigastric region", [4, 7151, 7192, 8134, 9514])
    },
    "EpigastricRegionStructure": {
        "27947004": ("Epigastric region structure (body structure)", [])
    },
    "Epilepsy": {"84757009": ("Epilepsy (disorder)", [])},
    "EpilepticConvulsions": {"84757009": ("Epileptic convulsions", [60, 9300])},
    "Epinephrine": {"387362001": ("Epinephrine", [10])},
    "EpirusCattleBreed": {"133381006": ("Epirus cattle breed", [7480])},
    "EpitrochlearLymphNode": {"28870006": ("epitrochlear lymph node", [7600, 8134])},
    "EpoxyResin": {"65345002": ("Epoxy resin", [8115])},
    "Equal": {"9726003": ("Equal", [75])},
    "EqualDensityIsodenseLesion": {
        "129745007": ("Equal density (isodense) lesion", [6008, 6009])
    },
    "EquipmentTemperature": {
        "250881009": ("Equipment temperature (observable entity)", [])
    },
    "Equivocal": {"42425007": ("Equivocal", [3120])},
    "Equus": {"388445009": ("Equus", [7454])},
    "EquusCaballus": {"35354009": ("Equus caballus", [7454])},
    "EquusCaballusGmelini": {"125084002": ("Equus caballus gmelini (organism)", [])},
    "EquusCaballusGmeliniHorseBreed": {
        "125084002": ("Equus caballus gmelini horse breed", [7480])
    },
    "ErasoModification": {"424655003": ("Eraso Modification", [4012])},
    "ErasoModificationProjection": {
        "424655003": ("Eraso Modification projection (qualifier value)", [])
    },
    "ErectorSpinaeMuscle": {
        "44947003": (
            "Erector spinae muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "ErhulianPigBreed": {"132317007": ("Erhulian pig breed", [7480])},
    "ErieGarnetStain": {"40076005": ("erie garnet stain", [8112])},
    "EriochromeBlueBlackSEStain": {
        "58631000": ("eriochrome blue black SE stain", [8112])
    },
    "EriskayPonyHorseBreed": {"132747007": ("Eriskay Pony horse breed", [7480])},
    "ErlunchunHorseBreed": {"131976002": ("Erlunchun horse breed", [7480])},
    "EruptionCausedByDrug": {"28926001": ("Eruption caused by drug (disorder)", [])},
    "EruptionOfSkin": {"271807003": ("Eruption of skin (disorder)", [])},
    "Erythema": {"247441003": ("Erythema", [4406, 4408, 6055])},
    "ErythrosinBStain": {"7434003": ("erythrosin B stain", [8112])},
    "ErythrosinYStain": {"5043000": ("erythrosin Y stain", [8112])},
    "ErzgebirgGoatBreed": {"131667007": ("Erzgebirg goat breed", [7480])},
    "EskimoDog": {"89450005": ("Eskimo dog", [7480])},
    "EsophagealApertureOfDiaphragm": {
        "280062008": ("Esophageal aperture of diaphragm (body structure)", [])
    },
    "EsophagealArtery": {
        "206034008": ("Esophageal artery", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "EsophagealHiatus": {
        "280062008": (
            "Esophageal Hiatus",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "EsophagealLymphNode": {"11899006": ("esophageal lymph node", [7600, 8134])},
    "EsophagealStructure": {"32849002": ("Esophageal structure (body structure)", [])},
    "Esophagus": {
        "32849002": (
            "Esophagus",
            [
                4009,
                4030,
                4031,
                4042,
                6102,
                6109,
                6113,
                6149,
                7151,
                7155,
                7192,
                7482,
                7483,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "EsophagusStomachAndDuodenum": {
        "110861005": (
            "Esophagus, stomach and duodenum",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "EsophagusStomachAndDuodenumCombinedSite": {
        "110861005": (
            "Esophagus, stomach and duodenum (combined site) (body structure)",
            [],
        )
    },
    "Estimated": {"414135002": ("Estimated", [3627, 3745, 6140, 10009])},
    "EstonianBaconPigBreed": {"132274007": ("Estonian Bacon pig breed", [7480])},
    "EstonianBlackPiedCattleBreed": {
        "133382004": ("Estonian Black Pied cattle breed", [7480])
    },
    "EstonianDraftHorseBreed": {"132749005": ("Estonian Draft horse breed", [7480])},
    "EstonianHoundDogBreed": {"132479004": ("Estonian Hound dog breed", [7480])},
    "EstonianNativeCattleBreed": {
        "133389008": ("Estonian Native cattle breed", [7480])
    },
    "EstonianNativeHorseBreed": {"133018002": ("Estonian Native horse breed", [7480])},
    "EstonianRedCattleBreed": {"131595003": ("Estonian Red cattle breed", [7480])},
    "EstrelaMountainDogBreed": {"132476006": ("Estrela Mountain Dog breed", [7480])},
    "Estrogen": {"41598000": ("Estrogen (substance)", [])},
    "EstrogenProduct": {"61946003": ("Estrogen product", [6080])},
    "EstrogenReceptorAgonist": {"1052201004": ("Estrogen", [621, 622])},
    "EstrogenicPreparation": {"61946003": ("Estrogenic preparation (product)", [])},
    "Ethanol": {"419442005": ("Ethanol", [10, 8114])},
    "EthmoidBone": {"52374004": ("Ethmoid bone", [4028, 7151, 7192, 8134, 9514])},
    "EthmoidBoneStructure": {
        "52374004": ("Ethmoid bone structure (body structure)", [])
    },
    "EthylAlcohol": {"419442005": ("Ethyl alcohol", [6089])},
    "EthylVioletStain": {"65445001": ("ethyl violet stain", [8112])},
    "EthylenediamineTetraAcetateGallium68": {
        "423498000": ("Ethylenediamine tetra-acetate gallium^68^ (product)", [])
    },
    "Etiology": {"134198009": ("Etiology (attribute)", [])},
    "Etomidate": {"387218008": ("Etomidate", [623, 625])},
    "EurasianDogBreed": {"132481002": ("Eurasian dog breed", [7480])},
    "EurasierDogBreed": {"132387004": ("Eurasier dog breed", [7480])},
    "EurolinePigBreed": {"132026004": ("Euroline pig breed", [7480])},
    "Europium": {"767775001": ("Europium", [10006, 10067])},
    "EustachianCanalStructure": {
        "91207004": ("Eustachian canal structure (body structure)", [])
    },
    "EvaluationOfMurmur": {"373112006": ("Evaluation of murmur", [3671])},
    "EvaluationProcedure": {"386053000": ("Evaluation procedure", [6028])},
    "EvansBlueStain": {"22931006": ("Evans blue stain", [8112])},
    "EvolèneCattleBreed": {"131596002": ("Evolène cattle breed", [7480])},
    "EwingSarcoma": {"76909002": ("Ewing's sarcoma", [638, 639])},
    "EwingSarcomaPeripheralNeuroectodermalTumor": {
        "703707001": ("Ewing sarcoma / peripheral neuroectodermal tumor", [638, 639])
    },
    "ExSmoker": {"8517006": ("Ex-smoker (finding)", [])},
    "ExaggeratedCranioCaudalProjection": {
        "399265009": ("Exaggerated cranio-caudal projection (qualifier value)", [])
    },
    "ExaminationOfBreast": {"46662001": ("Examination of breast", [6050, 6083])},
    "Excision": {"65801008": ("Excision", [8109])},
    "ExcisionOfSkin": {"440258006": ("Excision of skin", [4409])},
    "Exercise": {"256235009": ("Exercise", [3774])},
    "ExerciseChallenge": {
        "128967005": ("Exercise challenge", [91, 3250, 3271, 3405, 3651])
    },
    "ExerciseECGAbnormal": {"165084003": ("Exercise ECG abnormal", [3208])},
    "ExerciseECGEquivocal": {"370367002": ("Exercise ECG equivocal", [3208])},
    "ExerciseECGNormal": {"165082004": ("Exercise ECG normal", [3208])},
    "ExerciseInducedAngina": {"300995000": ("Exercise-induced angina (disorder)", [])},
    "ExerciseState": {"128976003": ("Exercise state", [3262, 3602])},
    "ExerciseStressECGTest": {"46136006": ("Exercise stress ECG test", [3261, 12001])},
    "ExerciseStressEchocardiography": {
        "433233004": ("Exercise Stress echocardiography", [3757, 12001])
    },
    "ExerciseStressTest": {"165079009": ("Exercise stress test", [3200])},
    "ExerciseTolerance": {"248243004": ("Exercise tolerance (observable entity)", [])},
    "ExerciseToleranceTest": {"165079009": ("Exercise Tolerance Test", [3757])},
    "ExerciseToleranceTestAbnormal": {
        "165084003": ("Exercise tolerance test abnormal (finding)", [])
    },
    "ExerciseToleranceTestEquivocal": {
        "370367002": ("Exercise tolerance test equivocal (finding)", [])
    },
    "ExerciseToleranceTestNormal": {
        "165082004": ("Exercise tolerance test normal (finding)", [])
    },
    "ExerciseToleranceTestRefused": {
        "408551003": ("Exercise tolerance test refused (situation)", [])
    },
    "ExerciserDevice": {"111045004": ("Exerciser, device (physical object)", [])},
    "ExertionalHypertension": {"429198000": ("Exertional hypertension", [3213])},
    "ExertionalHypotension": {"429561008": ("Exertional hypotension", [3213])},
    "ExmoorHornSheepBreed": {"131719005": ("Exmoor Horn sheep breed", [7480])},
    "ExmoorPonyHorseBreed": {"133019005": ("Exmoor Pony horse breed", [7480])},
    "ExoticShorthairedCat": {"26057009": ("Exotic shorthaired cat", [7480])},
    "ExpandingAneurysm": {"125272005": ("expanding aneurysm", [3808])},
    "Expiration": {"58322009": ("expiration", [3823, 9272, 12234])},
    "Extension": {"24154002": ("Extension", [92])},
    "Extent": {"260858005": ("Extent (attribute)", [])},
    "ExtentOfCardiacPerfusionDefect": {
        "427751006": ("Extent of cardiac perfusion defect (observable entity)", [])
    },
    "ExtentOfMyocardialStressIschemia": {
        "429162008": ("Extent of myocardial stress ischemia (observable entity)", [])
    },
    "Extents": {"255590007": ("Extents (qualifier value)", [])},
    "External": {"261074009": ("External", [2, 5, 6139])},
    "ExternalAuditoryCanal": {
        "84301002": ("External auditory canal", [4, 4040, 7151, 7192, 8134, 9514])
    },
    "ExternalAuditoryCanalStructure": {
        "84301002": ("External auditory canal structure (body structure)", [])
    },
    "ExternalAuditoryMeatus": {
        "84301002": ("External Auditory Meatus", [1000, 1001, 1002])
    },
    "ExternalCamera": {"409903006": ("External Camera", [4202])},
    "ExternalCapsule": {"10517005": ("external capsule", [7702, 7710, 8134])},
    "ExternalCapsuleOfBrain": {
        "10517005": ("External capsule of brain (body structure)", [])
    },
    "ExternalCarotidArtery": {
        "22286001": (
            "External carotid artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12104],
        )
    },
    "ExternalCarotidArteryStructure": {
        "22286001": ("External carotid artery structure (body structure)", [])
    },
    "ExternalCounterPulsation": {"371790000": ("External Counter-Pulsation", [3553])},
    "ExternalCounterPulsationCirculatorySupportDevice": {
        "371790000": (
            "External counter-pulsation circulatory support device (physical object)",
            [],
        )
    },
    "ExternalEar": {"28347008": ("External ear", [4028, 7151, 7192, 8134, 9514])},
    "ExternalEarStructure": {
        "28347008": ("External ear structure (body structure)", [])
    },
    "ExternalElasticMembrane": {"414165007": ("External Elastic Membrane", [3497])},
    "ExternalElasticMembraneOfArtery": {
        "414165007": ("External elastic membrane of artery (body structure)", [])
    },
    "ExternalIliacArtery": {
        "113269004": (
            "External iliac artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "ExternalIliacLymphNode": {"65349008": ("external iliac lymph node", [7600, 8134])},
    "ExternalIliacVein": {
        "63507001": (
            "External iliac vein",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "ExternalIntercostalMuscle": {
        "53967007": (
            "External intercostal muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "ExternalJugularVein": {
        "71585003": (
            "External jugular vein",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "ExternalMammaryLymphNode": {
        "421988007": ("external mammary lymph node", [7600, 8134])
    },
    "ExternalProsthesisForSonographicProcedureDevice": {
        "102322008": (
            "External prosthesis for sonographic procedure, device (physical object)",
            [],
        )
    },
    "ExternalProsthesisForSonographicProcedureStandOff": {
        "102322008": ("External prosthesis for sonographic procedure [Stand-off]", [5])
    },
    "ExternalRotation": {"52019005": ("External rotation", [92])},
    "ExternalRotationFunction": {
        "52019005": ("External rotation, function (observable entity)", [])
    },
    "ExtraAbdominalDesmoid": {"47284001": ("Extra abdominal desmoid", [6030, 6031])},
    "ExtraArticular": {"87687004": ("Extra-articular", [2])},
    "ExtraCorporealCirculation": {"182744004": ("Extra-corporeal circulation", [3553])},
    "ExtracorporealCirculationProcedure": {
        "182744004": ("Extracorporeal circulation procedure (procedure)", [])
    },
    "ExtrafovealOcularFixation": {
        "251786004": ("Extrafoveal ocular fixation (finding)", [])
    },
    "ExtrapulmonaryLymphNodeOfLung": {
        "363537007": ("extrapulmonary lymph node of lung", [7600, 8134])
    },
    "ExtremelyDense": {"129719003": ("Extremely dense", [6000, 6001])},
    "ExtremelyDenseBreastComposition": {
        "129719003": ("Extremely dense breast composition (finding)", [])
    },
    "Extremity": {
        "66019005": ("Extremity", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "Eye": {
        "81745001": ("Eye", [4, 4009, 4030, 4031, 4042, 4209, 7151, 7192, 8134, 9514])
    },
    "EyeLenses": {"78076003": ("Eye lenses", [10044, 10060])},
    "EyeLubricant": {"470091001": ("Eye lubricant (physical object)", [])},
    "EyeRadiationShield": {"469266003": ("Eye radiation shield", [9572, 9573])},
    "EyeRegion": {
        "371398005": (
            "Eye region",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "EyeRegionStructure": {"371398005": ("Eye region structure (body structure)", [])},
    "Eyeball": {"79652003": ("Eyeball", [4028, 7151, 7192, 8134, 9514])},
    "Eyelash": {"85803001": ("Eyelash", [4029, 8134])},
    "Eyelid": {"80243003": ("Eyelid", [4, 4209, 7151, 7192, 8134, 9514])},
    "EyelidDisease": {"60113004": ("Eyelid Disease", [4222])},
    "EyelidStructure": {"80243003": ("Eyelid structure (body structure)", [])},
    "FHCElitePig1": {"45635003": ("FHC elite pig 1 (organism)", [])},
    "FHCElitePig1PigBreed": {"45635003": ("FHC elite pig 1 pig breed", [7480])},
    "FHCElitePig2": {"59667000": ("FHC elite pig 2 (organism)", [])},
    "FHCElitePig2PigBreed": {"59667000": ("FHC elite pig 2 pig breed", [7480])},
    "FHCElitePig3": {"24111007": ("FHC elite pig 3 (organism)", [])},
    "FHCElitePig3PigBreed": {"24111007": ("FHC elite pig 3 pig breed", [7480])},
    "FHCElitePig4": {"47795006": ("FHC elite pig 4 (organism)", [])},
    "FHCElitePig4PigBreed": {"47795006": ("FHC elite pig 4 pig breed", [7480])},
    "FHCElitePig5": {"67720004": ("FHC elite pig 5 (organism)", [])},
    "FHCElitePig5PigBreed": {"67720004": ("FHC elite pig 5 pig breed", [7480])},
    "FHCElitePig6": {"49462008": ("FHC elite pig 6 (organism)", [])},
    "FHCElitePig6PigBreed": {"49462008": ("FHC elite pig 6 pig breed", [7480])},
    "FHCElitePig7": {"32683006": ("FHC elite pig 7 (organism)", [])},
    "FHCElitePig7PigBreed": {"32683006": ("FHC elite pig 7 pig breed", [7480])},
    "FHCElitePig8": {"73005003": ("FHC elite pig 8 (organism)", [])},
    "FHCElitePig8PigBreed": {"73005003": ("FHC elite pig 8 pig breed", [7480])},
    "FHCElitePig9": {"14063001": ("FHC elite pig 9 (organism)", [])},
    "FHCElitePig9PigBreed": {"14063001": ("FHC elite pig 9 pig breed", [7480])},
    "FHCPig": {"89928000": ("FHC pig (organism)", [])},
    "FHCPigBreed": {"89928000": ("FHC pig breed", [7480])},
    "FRSCattleBreed": {"133393002": ("FRS cattle breed", [7480])},
    "FabrianeseSheepBreed": {"131718002": ("Fabrianese sheep breed", [7480])},
    "Face": {"89545001": ("Face", [4, 7151, 7192, 8134, 9514])},
    "FaceStructure": {"89545001": ("Face structure (body structure)", [])},
    "FacialArtery": {
        "23074001": (
            "Facial artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "FacialBones": {
        "91397008": (
            "Facial bones",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "FacialLymphNode": {"48918001": ("facial lymph node", [7600, 8134])},
    "FacialNerve": {"56052001": ("facial nerve", [7706, 7710, 8134])},
    "FacialNerveStructure": {
        "56052001": ("Facial nerve structure (body structure)", [])
    },
    "FacialSwelling": {"278528006": ("Facial Swelling", [60, 9300])},
    "FaeroesCattleBreed": {"133390004": ("Faeroes cattle breed", [7480])},
    "FaeroesIslandHorseHorseBreed": {
        "133020004": ("Faeroes Island Horse horse breed", [7480])
    },
    "FaeroesSheepBreed": {"132846008": ("Faeroes sheep breed", [7480])},
    "FailedAttemptedProcedure": {"103709008": ("Failed attempted procedure", [9])},
    "FalabellaHorseBreed": {"133021000": ("Falabella horse breed", [7480])},
    "FallopianTube": {"31435000": ("Fallopian tube", [7151, 7160, 7192, 8134, 9514])},
    "FallopianTubeStructure": {
        "31435000": ("Fallopian tube structure (body structure)", [])
    },
    "FamilyHistoryCardiovascularDisease": {
        "266894000": ("Family history: Cardiovascular disease (situation)", [])
    },
    "FamilyHistoryDiabetesMellitus": {
        "160303001": ("Family history: Diabetes mellitus (situation)", [])
    },
    "FamilyHistoryMyocardialInfarction": {
        "266897007": ("Family history: Myocardial infarction (situation)", [])
    },
    "FamilyHistoryOfBreastCancer": {
        "429740004": ("Family history of breast cancer", [6081, 6087])
    },
    "FamilyHistoryOfCardiovascularDisease": {
        "266894000": ("Family history of cardiovascular disease", [3758])
    },
    "FamilyHistoryOfClinicalFinding": {
        "416471007": ("Family history of clinical finding (situation)", [])
    },
    "FamilyHistoryOfCoronaryArteriosclerosis": {
        "430091005": ("Family history of coronary arteriosclerosis", [3758])
    },
    "FamilyHistoryOfDiabetesMellitus": {
        "160303001": ("Family history of diabetes mellitus", [3758])
    },
    "FamilyHistoryOfMalignantMelanoma": {
        "427858005": ("Family history of malignant melanoma (situation)", [])
    },
    "FamilyHistoryOfMalignantNeoplasmOfBreast": {
        "429740004": ("Family history of malignant neoplasm of breast (situation)", [])
    },
    "FamilyHistoryOfMyocardialInfarction": {
        "266897007": ("Family history of myocardial infarction", [3758])
    },
    "FamilyHistoryUnknown": {"407559004": ("Family history unknown", [3758, 6322])},
    "FamilyHistoryWithExplicitContext": {
        "57177007": ("Family history with explicit context (situation)", [])
    },
    "FannongSpottedPigBreed": {"132332000": ("Fannong Spotted pig breed", [7480])},
    "FascialLayer": {
        "120576005": ("Fascial layer", [6102, 6109, 6113, 6148, 7151, 7192, 9514])
    },
    "FastBlueBBSaltStain": {"91295002": ("fast blue BB salt stain", [8112])},
    "FastBlueBSaltStain": {"34700000": ("fast blue B salt stain", [8112])},
    "FastBlueRRSaltStain": {"64112001": ("fast blue RR salt stain", [8112])},
    "FastGarnetGBCSaltStain": {"89148006": ("fast garnet GBC salt stain", [8112])},
    "FastGreenFCFStain": {"24167004": ("fast green FCF stain", [8112])},
    "FastRedBSaltStain": {"40718007": ("fast red B salt stain", [8112])},
    "FastRedITRStain": {"47486002": ("fast red ITR stain", [8112])},
    "FastRedTRSaltStain": {"76633005": ("fast red TR salt stain", [8112])},
    "FastSulfonBlackFStain": {"88660000": ("fast sulfon black F stain", [8112])},
    "FastVioletBSaltStain": {"72371006": ("fast violet B salt stain", [8112])},
    "Fat": {"256674009": ("Fat", [218, 300, 6122, 6211, 7180, 7469])},
    "FatContainingRadiolucentLesion": {
        "129747004": ("Fat containing (radiolucent) lesion", [6008, 6009])
    },
    "FatNecrosisOfBreast": {"21381006": ("Fat necrosis of breast", [6030, 6031])},
    "FatRed7BStain": {"76439002": ("fat red 7B stain", [8112])},
    "Fatal": {"399166001": ("Fatal", [3716])},
    "Fatigue": {"84229001": ("Fatigue", [3220, 3221])},
    "FattyDegeneration": {"29185008": ("fatty degeneration", [3802])},
    "FattyStreaks": {"53151000": ("fatty streaks", [3817])},
    "FauveDeBretagneDogBreed": {"132426009": ("Fauve de Bretagne dog breed", [7480])},
    "Feces": {"39477002": ("Feces", [6201, 7151, 7197])},
    "FeedingPatientRegimeTherapy": {
        "75118006": ("Feeding patient (regime/therapy)", [])
    },
    "FeedingTube": {"25062003": ("Feeding tube", [6102, 6138, 6404, 7151, 7193])},
    "FeedingTubeDevice": {"25062003": ("Feeding tube, device (physical object)", [])},
    "FeetFirst": {"102541007": ("feet-first", [21])},
    "FeetfirstPosition": {"102541007": ("Feetfirst position (finding)", [])},
    "FelineSpecies": {"23826000": ("Feline species (organism)", [])},
    "Felis": {"388626009": ("Felis", [7454])},
    "FelisCatus": {"448169003": ("Felis catus", [7454])},
    "FellPonyHorseBreed": {"132996008": ("Fell Pony horse breed", [7480])},
    "FellTerrierDogBreed": {"132482009": ("Fell Terrier dog breed", [7480])},
    "FellataCattleBreed": {"133651005": ("Fellata cattle breed", [7480])},
    "FemaleExternalGenitaliaStructure": {
        "86969008": ("Female external genitalia structure (body structure)", [])
    },
    "FemaleExternalUrethralOrifice": {
        "279479008": ("Female external urethral orifice", [4029, 8134])
    },
    "FemaleFirstCousin": {"270002": ("Female first cousin", [7450, 7451])},
    "FemaleGenitalLymphNode": {
        "314736006": ("female genital lymph node", [7600, 8134])
    },
    "FemaleInternalGenitalia": {
        "699886009": ("Female internal genitalia", [7151, 7160, 7192, 8134, 9514])
    },
    "FemaleInternalGenitaliaStructure": {
        "87759004": ("Female internal genitalia structure (body structure)", [])
    },
    "FemoralArtery": {
        "7657000": (
            "Femoral artery",
            [4, 3010, 3440, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "FemoralArteryPressureWaveform": {
        "128442000": ("Femoral artery pressure waveform", [3003])
    },
    "FemoralHead": {"2812003": ("Femoral head", [1000, 1001, 1005, 8134])},
    "FemoralHeadProsthesis": {"304121006": ("Femoral Head Prosthesis", [7307, 7308])},
    "FemoralVein": {
        "83419000": (
            "Femoral vein",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "FemoralVeinStructure": {
        "83419000": ("Femoral vein structure (body structure)", [])
    },
    "Femur": {
        "71341001": (
            "Femur",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
                12020,
                12021,
            ],
        )
    },
    "FengjingPigBreed": {"132233007": ("Fengjing pig breed", [7480])},
    "Ferguson": {"399372008": ("Ferguson", [4012])},
    "FergusonProjection": {"399372008": ("Ferguson projection (qualifier value)", [])},
    "FerrandaisCattleBreed": {"133383009": ("Ferrandais cattle breed", [7480])},
    "FerricHexacyanoferrateII": {
        "406452004": ("Ferric hexacyanoferrate-II (substance)", [])
    },
    "FerrousChlorideFe59": {"31192007": ("Ferrous chloride Fe^59^", [25])},
    "FerrousCitrateFe59": {"87958003": ("Ferrous citrate Fe^59^", [25])},
    "FerrousSulfateFe59": {"125001": ("Ferrous sulfate Fe^59^", [25])},
    "FetalEchocardiography": {"433235006": ("Fetal echocardiography", [12001])},
    "FetalUmbilicalVein": {"13576009": ("Umbilical Vein", [12140])},
    "FetlockOfForelimb": {"13190002": ("Fetlock of forelimb", [7482, 7483, 8134])},
    "FetlockOfHindlimb": {"113351006": ("Fetlock of hindlimb", [7482, 7483, 8134])},
    "FetlockRegionOfForelimb": {
        "13190002": ("Fetlock region of forelimb (body structure)", [])
    },
    "FetlockRegionOfHindlimb": {
        "113351006": ("Fetlock region of hindlimb (body structure)", [])
    },
    "Fetocide": {"133874006": ("Fetocide (selective reduction)", [9])},
    "FeulgenReactionStain": {"255797007": ("Feulgen reaction stain", [8112])},
    "Fever": {"386661006": ("Fever", [12246])},
    "FeverWithChills": {"274640006": ("Fever with chills (finding)", [])},
    "FiberopticCatheter": {"371801001": ("Fiberoptic catheter", [3429])},
    "FiberopticCatheterMethod": {"133912003": ("Fiberoptic catheter method", [3241])},
    "Fibril": {"88921000": ("Fibril", [637])},
    "FibrinogenI123": {"71636003": ("Fibrinogen I^123^", [25])},
    "Fibrinolysin": {"764170006": ("Fibrinolysin", [10])},
    "Fibroadenoma": {"254845004": ("Fibroadenoma", [6030, 6031])},
    "FibroadenomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "65877006": (
            "Fibroadenoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "FibroadenomaOfBreast": {"254845004": ("Fibroadenoma of breast (disorder)", [])},
    "FibrocysticBreastChanges": {
        "27431007": ("Fibrocystic breast changes (finding)", [])
    },
    "FibrocysticDiseaseOfBreast": {
        "27431007": ("Fibrocystic disease of breast", [6030, 6031])
    },
    "Fibroelastosis": {"72166006": ("fibroelastosis", [3817])},
    "Fibromatosis": {"19928005": ("Fibromatosis", [6030, 6031])},
    "FibromuscularDysplasia": {"31653004": ("fibromuscular dysplasia", [3810])},
    "Fibrosarcoma": {"53654007": ("Fibrosarcoma", [638, 639, 6030, 6033])},
    "FibrosingAdenosis": {
        "50916005": ("Fibrosing adenosis (morphologic abnormality)", [])
    },
    "Fibrosis": {"112674009": ("Fibrosis", [6335, 6336, 6337, 6338])},
    "Fibrous": {"255423002": ("fibrous", [3802, 3805])},
    "FibrousHistiocytomaMalignant": {
        "34360000": ("Fibrous histiocytoma, malignant (morphologic abnormality)", [])
    },
    "FibrousPlaque": {"40772000": ("Fibrous Plaque", [3491, 3495, 3496, 3497, 7111])},
    "Fibula": {
        "87342007": (
            "Fibula",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514, 12021],
        )
    },
    "FibularLymphNode": {"31171007": ("fibular lymph node", [7600, 8134])},
    "FieldSpaniel": {"27385008": ("Field spaniel (organism)", [])},
    "FieldSpanielDogBreed": {"27385008": ("Field spaniel dog breed", [7480])},
    "FieldStain": {"255798002": ("field's stain", [8112])},
    "FifthIntercostalSpace": {"117137004": ("Fifth Intercostal Space", [5])},
    "FifthLumbarArtery": {
        "397411003": (
            "Fifth Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "FifthPercentile": {"371888009": ("Fifth percentile (qualifier value)", [])},
    "FifthVentricle": {"180933005": ("Fifth ventricle (body structure)", [])},
    "FightingBullCattleBreed": {"131597006": ("Fighting Bull cattle breed", [7480])},
    "FilaBrasileiroDogBreed": {"132483004": ("Fila Brasileiro dog breed", [7480])},
    "Finding": {"404684003": ("Finding", [3769])},
    "FindingByInspectionSimpleObservation": {
        "118243007": ("Finding by inspection (simple observation) (finding)", [])
    },
    "FindingByPalpation": {"118242002": ("Finding by palpation (finding)", [])},
    "FindingLessWellDefinedSincePreviousMammogram": {
        "129728002": (
            "Finding less well defined since previous mammogram (finding)",
            [],
        )
    },
    "FindingMoreDefinedSincePreviousMammogram": {
        "129729005": ("Finding more defined since previous mammogram (finding)", [])
    },
    "FindingOfChangeSincePreviousMammogram": {
        "129720009": ("Finding of change since previous mammogram (finding)", [])
    },
    "FindingOfDifferenceInBorderDefinitionComparedToPreviousRadiologicExamination": {
        "442688001": (
            "Finding of difference in border definition compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfDifferenceInBorderShapeComparedToPreviousRadiologicExamination": {
        "442755000": (
            "Finding of difference in border shape compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfDifferenceInDistributionComparedToPreviousRadiologicExamination": {
        "442704007": (
            "Finding of difference in distribution compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfDifferenceInLocation": {
        "442726008": ("Difference in location", [6133, 6207])
    },
    "FindingOfDifferenceInRadiographicAttenuationComparedToPreviousRadiologicExamination": {
        "442707000": (
            "Finding of difference in radiographic attenuation compared to previous radiologic examination "
            "(finding)",
            [],
        )
    },
    "FindingOfDifferenceInSiteInvolvementComparedToPreviousRadiologicExamination": {
        "442711006": (
            "Finding of difference in site involvement compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfDifferenceInSize": {"442714003": ("Difference in size", [6133, 6207])},
    "FindingOfDifferenceInSubstanceComparedToPreviousRadiologicExamination": {
        "442691001": (
            "Finding of difference in substance compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfDifferenceInTextureComparedToPreviousRadiologicExamination": {
        "442700003": (
            "Finding of difference in texture compared to previous radiologic examination (finding)",
            [],
        )
    },
    "FindingOfElectrocardiogramSTSegment": {
        "365416000": ("Finding of electrocardiogram ST segment (finding)", [])
    },
    "FindingOfFunctionalPerformanceAndActivity": {
        "248536006": ("Finding of functional performance and activity (finding)", [])
    },
    "FindingOfGestationalSac": {
        "300571009": ("Finding of gestational sac (finding)", [])
    },
    "FindingOfLeftVentricularFunction": {
        "366188009": ("Finding of left ventricular function (finding)", [])
    },
    "FindingOfLesion": {"300577008": ("Finding of lesion (finding)", [])},
    "FindingOfTobaccoSmokingBehavior": {
        "365981007": ("Finding of tobacco smoking behavior (finding)", [])
    },
    "FindingPartiallyRemoved": {
        "129722001": ("Finding partially removed", [6002, 6003, 6134])
    },
    "FindingPartiallyRemovedSincePreviousMammogram": {
        "129722001": (
            "Finding partially removed since previous mammogram (finding)",
            [],
        )
    },
    "FindingRelatingToDrugMisuseBehavior": {
        "228366006": ("Finding relating to drug misuse behavior (finding)", [])
    },
    "FindingReportedByPatientInformant": {
        "418799008": ("Finding reported by patient/informant", [3769])
    },
    "FindingReportedBySubjectOrHistoryProvider": {
        "418799008": ("Finding reported by subject or history provider (finding)", [])
    },
    "FindingSite": {"363698007": ("Finding Site", [9000])},
    "FindingsCategoryType": {
        "276214006": ("Findings category type (qualifier value)", [])
    },
    "FindingsValues": {"260245000": ("Findings values (qualifier value)", [])},
    "FineLinearBranchingCastingCalcification": {
        "129762002": ("Fine linear, branching (casting) calcification", [6010, 6011])
    },
    "FineLinearBranchingCastingRadiographicCalcification": {
        "129762002": (
            "Fine, linear, branching (casting) radiographic calcification (finding)",
            [],
        )
    },
    "FineLinearCastingCalcification": {
        "129761009": ("Fine, linear (casting) calcification", [6010, 6011])
    },
    "FineLinearCastingRadiographicCalcification": {
        "129761009": (
            "Fine, linear, (casting) radiographic calcification (finding)",
            [],
        )
    },
    "FineMerinoSheepBreed": {"132722005": ("Fine Merino sheep breed", [7480])},
    "FineNeedleAspirationOfBreast": {
        "387736007": ("Fine needle aspiration of breast", [6050, 6083])
    },
    "FineSlitBeamDirectIllumination": {
        "410462008": ("Fine slit beam direct illumination", [4203])
    },
    "FineSlitBeamDirectIlluminationAction": {
        "410462008": (
            "Fine slit beam direct illumination - action (qualifier value)",
            [],
        )
    },
    "Finger": {
        "7569003": ("Finger", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "FingerJoint": {"125682004": ("Finger Joint", [7304, 8134])},
    "FingerJointStructure": {
        "125682004": ("Finger joint structure (body structure)", [])
    },
    "FingerStructure": {"7569003": ("Finger structure (body structure)", [])},
    "FinnhorseDraftHorseBreed": {"131993002": ("Finnhorse Draft horse breed", [7480])},
    "FinnhorseHorseBreed": {"133029003": ("Finnhorse horse breed", [7480])},
    "FinnishAyrshireCattleBreed": {
        "133384003": ("Finnish Ayrshire cattle breed", [7480])
    },
    "FinnishHoundDogBreed": {"132484005": ("Finnish Hound dog breed", [7480])},
    "FinnishLandraceGoatBreed": {"131666003": ("Finnish Landrace goat breed", [7480])},
    "FinnishLandracePigBreed": {"132234001": ("Finnish Landrace pig breed", [7480])},
    "FinnishLandraceSheep": {"72329005": ("Finnish landrace sheep (organism)", [])},
    "FinnishLandraceSheepBreed": {"72329005": ("Finnish landrace sheep breed", [7480])},
    "FinnishLapphundDogBreed": {"132485006": ("Finnish Lapphund dog breed", [7480])},
    "FinnishSpitzDog": {"83504004": ("Finnish spitz dog", [7480])},
    "FinnishYorkshirePigBreed": {"132269004": ("Finnish Yorkshire pig breed", [7480])},
    "FirstDegreeAtrioventricularBlock": {
        "270492004": ("First degree Atrioventricular block", [3230])
    },
    "FirstIntercostalSpace": {"117133000": ("First Intercostal Space", [5])},
    "FirstLumbarArtery": {
        "397407009": (
            "First Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "FirstMolarRegion": {"699507006": ("First molar region", [4017])},
    "FirstMolarRegionOfOralCavity": {
        "699507006": ("First molar region of oral cavity (body structure)", [])
    },
    "FirstPremolarRegion": {"699509009": ("First premolar region", [4017])},
    "FirstPremolarRegionOfOralCavity": {
        "699509009": ("First premolar region of oral cavity (body structure)", [])
    },
    "FishOils": {"346441008": ("Fish oils", [3761])},
    "Fisk": {"424962005": ("Fisk", [4012])},
    "FiskProjection": {"424962005": ("Fisk projection (qualifier value)", [])},
    "FissureOfLung": {
        "278983006": ("Fissure of lung", [6102, 6109, 6112, 7151, 7192, 9514])
    },
    "FistulaCoronaryToLeftAtrium": {
        "128555001": ("Fistula coronary to left atrium", [3604, 3606, 3630])
    },
    "FistulaCoronaryToLeftVentricle": {
        "128556000": ("Fistula coronary to left ventricle", [3604, 3606, 3630])
    },
    "FistulaCoronaryToRightAtrium": {
        "373095005": ("Fistula coronary to right atrium", [3604, 3606, 3630])
    },
    "FistulaCoronaryToRightVentricle": {
        "128558004": ("Fistula coronary to right ventricle", [3604, 3606, 3630])
    },
    "FitzpatrickClassificationOfSkinType": {
        "443635002": ("Fitzpatrick classification of skin type (assessment scale)", [])
    },
    "FiveChamber": {"398994001": ("Five chamber", [6, 501, 502])},
    "FiveChamberView": {"398994001": ("Five chamber view (qualifier value)", [])},
    "FixedMyocardialPerfusionDefect": {
        "251057006": ("Fixed myocardial perfusion defect", [3113])
    },
    "FjallCattleBreed": {"131598001": ("Fjall cattle breed", [7480])},
    "FjordHorse": {"89648005": ("Fjord horse (organism)", [])},
    "FjordHorseBreed": {"89648005": ("Fjord horse breed", [7480])},
    "FlagellarStain": {"255799005": ("Flagellar stain", [8112])},
    "Flank": {"58602004": ("Flank", [4, 644, 7151, 7192, 8134, 9514])},
    "FlankStructure": {"58602004": ("Flank structure (body structure)", [])},
    "Flat": {"6041008": ("Flat", [6209])},
    "FlatCoatedRetriever": {"9528004": ("Flat-coated retriever (organism)", [])},
    "FlatCoatedRetrieverDogBreed": {
        "9528004": ("Flat-coated retriever dog breed", [7480])
    },
    "Fleischner": {"399281004": ("Fleischner", [4012])},
    "FleischnerProjection": {
        "399281004": ("Fleischner projection (qualifier value)", [])
    },
    "FlemishCattleBreed": {"133385002": ("Flemish cattle breed", [7480])},
    "FleuveHorseBreed": {"133106009": ("Fleuve horse breed", [7480])},
    "Flexion": {"9964006": ("Flexion", [92])},
    "FlexionFunction": {"9964006": ("Flexion, function (observable entity)", [])},
    "Floor": {"709280007": ("Floor", [7151, 7157, 7193, 9505, 9515, 9520, 9573])},
    "FloorOfMouth": {"36360002": ("floor of mouth", [7601, 8134])},
    "FloorOfMouthStructure": {
        "36360002": ("Floor of mouth structure (body structure)", [])
    },
    "Florbetaben18F": {"712736002": ("Florbetaben [18F] (substance)", [])},
    "FlorbetabenF18": {"712736002": ("Florbetaben F^18^", [4021])},
    "FlorbetapirF18": {"456995000": ("Florbetapir F^18^", [4021])},
    "FloresHorseBreed": {"131978001": ("Flores horse breed", [7480])},
    "FloridaCrackerHorseBreed": {"133114003": ("Florida Cracker horse breed", [7480])},
    "FloridaCrackerPineywoodsCattleBreed": {
        "131599009": ("Florida Cracker/Pineywoods cattle breed", [7480])
    },
    "FlortaucipirF18": {"879808000": ("Flortaucipir F^18^", [4021])},
    "FluciclatideF18": {"456999006": ("Fluciclatide F^18^", [4021])},
    "FluciclovineF18": {"457000009": ("Fluciclovine F^18^", [4021])},
    "FluidFilledCatheter": {"371798007": ("Fluid filled catheter", [3429])},
    "FluidFilledCatheterHemodynamicMeasurementMethodRegimeTherapy": {
        "128581002": (
            "Fluid filled catheter hemodynamic measurement method (regime/therapy)",
            [],
        )
    },
    "FluidFilledCatheterMethod": {
        "128581002": ("Fluid filled catheter method", [3241])
    },
    "FlumazenilC11": {"423543007": ("Flumazenil C^11^", [4021])},
    "FlumazenilF18": {"422975006": ("Flumazenil F^18^", [4021])},
    "Fluorescein": {"350086004": ("Fluorescein", [4200])},
    "FluoresceinProduct": {"350086004": ("Fluorescein product (product)", [])},
    "FluoresceinSodiumStain": {"25351006": ("Fluorescein sodium stain", [8112])},
    "FluoresceinStain": {"85596006": ("fluorescein stain", [4412, 8112])},
    "FluorescentStain": {"35352008": ("fluorescent stain", [8112])},
    "FluorethyltyrosinF18": {"424708001": ("Fluorethyltyrosin F^18^", [4021])},
    "FluorexonStain": {"108880002": ("fluorexon stain", [8112])},
    "FluoroLDopaF18": {"5811000122108": ("Fluoro-L-dopa F^18^", [4021])},
    "FluorobenzothiazoleF18": {"423546004": ("Fluorobenzothiazole F^18^", [4021])},
    "FluorocholineF18": {"456992002": ("Fluorocholine F^18^", [4021])},
    "FluorodeoxyglucoseF18": {
        "35321007": ("Fluorodeoxyglucose F^18^", [25, 3107, 4021])
    },
    "Fluorodopa18F": {"5811000122108": ("Fluorodopa [18F] (substance)", [])},
    "FluoroestradiolF18": {"926372005": ("Fluoroestradiol F^18^", [4021])},
    "FluoromethaneF18": {"422763008": ("Fluoromethane F^18^", [4021])},
    "FluoromisonidazoleF18": {"422598008": ("Fluoromisonidazole F^18^", [4021])},
    "FluoroscopicAngiographyOfCoronaryArteries": {
        "419416005": ("Fluoroscopic angiography of coronary arteries (procedure)", [])
    },
    "FluoroscopicAngiographyOfLeftVentricleAndCoronaryArteries": {
        "418903008": (
            "Fluoroscopic angiography of left ventricle and coronary arteries (procedure)",
            [],
        )
    },
    "Fluoroscopy": {"44491008": ("Fluoroscopy", [10002, 10005])},
    "FluoroscopyAndRadiography": {
        "169014003": ("Fluoroscopy and radiography", [10005])
    },
    "FluorouracilF18": {"425236000": ("Fluorouracil F^18^", [4021])},
    "Flushing": {"238810007": ("Flushing", [3220])},
    "FlutemetamolF18": {"456997008": ("Flutemetamol F^18^", [4021])},
    "Focal": {"87017008": ("Focal", [6128, 6335, 6336, 6337])},
    "FocalAsymmetricBreastTissue": {
        "129789007": ("Focal asymmetric breast tissue", [6016, 6017, 6054])
    },
    "FocalAsymmetricBreastTissueFinding": {
        "129789007": ("Focal asymmetric breast tissue finding (finding)", [])
    },
    "FocalAsymmetricDensityAnalysis": {
        "133888005": ("Focal asymmetric density analysis", [6043])
    },
    "FocalFibroelastosis": {"125357009": ("focal fibroelastosis", [3817])},
    "FocalFibrosis": {"45559001": ("Focal fibrosis", [6030, 6031])},
    "FogeraCattleBreed": {"131471003": ("Fogera cattle breed", [7480])},
    "Folio": {"425157002": ("Folio", [4012])},
    "FolioProjection": {"425157002": ("Folio projection (qualifier value)", [])},
    "FontanBaffleLeak": {"443461006": ("Fontan Baffle Leak", [12282, 12284])},
    "FontanFenestration": {"443724003": ("Fontan Fenestration", [12282, 12284])},
    "FontanInferiorVenaCavaPathway": {
        "443326004": ("Fontan Inferior Vena Cava Pathway", [12282])
    },
    "FontanPathway": {"443298009": ("Fontan Pathway", [12282])},
    "FontanPulmonaryArteryConnection": {
        "443625008": ("Fontan Pulmonary Artery Connection", [12282])
    },
    "FontanelOfSkull": {"79361005": ("Fontanel of skull", [4, 7151, 7192, 8134, 9514])},
    "FonthillMerinoSheepBreed": {"132718000": ("Fonthill Merino sheep breed", [7480])},
    "Foot": {
        "56459004": (
            "Foot",
            [
                4,
                1000,
                1001,
                1006,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                8134,
                9514,
                12020,
            ],
        )
    },
    "FootStructure": {"56459004": ("Foot structure (body structure)", [])},
    "ForamenMagnum": {"24532009": ("Foramen Magnum", [1000, 1001, 1002, 8134])},
    "ForcedAirWarmingBlanket": {"420572009": ("Forced air warming blanket", [635])},
    "ForcepsMajor": {"80049006": ("forceps major", [7705, 7710, 8134])},
    "ForcepsMinor": {"42932006": ("forceps minor", [7705, 7710, 8134])},
    "Forearm": {
        "14975008": ("Forearm", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "ForearmStructure": {"14975008": ("Forearm structure (body structure)", [])},
    "Forefoot": {"419176008": ("Forefoot", [7482, 7483, 8134])},
    "ForefootOfQuadruped": {
        "419176008": ("Forefoot of quadruped (body structure)", [])
    },
    "ForeignBody": {"19227008": ("Foreign body", [6054, 6064, 7151, 7158, 7193])},
    "ForeignBodyGiantCellGranuloma": {
        "37058002": ("Foreign body (reaction)", [6030, 6031])
    },
    "ForeignMaterialIodizedOilMercuryTalc": {
        "19227008": ("Foreign material (iodized oil, mercury,talc)", [6122])
    },
    "ForensicIntent": {"447295008": ("Forensic Intent", [3629])},
    "ForestMountainPigBreed": {"132077006": ("Forest Mountain pig breed", [7480])},
    "Formaldehyde": {"111095003": ("Formaldehyde", [8114])},
    "Formalin": {"431510009": ("Formalin", [8114])},
    "FormerSmoker": {"8517006": ("Former Smoker", [3724])},
    "FormolSublimate": {"434295000": ("Formol sublimate", [8114])},
    "Fornix": {"87463005": ("fornix", [7704, 7710])},
    "FortCrossCattleBreed": {"133387005": ("Fort Cross cattle breed", [7480])},
    "FouchetStain": {"406980006": ("fouchet stain", [8112])},
    "FourChamber": {"399214001": ("Four chamber", [6, 501, 502])},
    "FourthIntercostalSpace": {"117136008": ("Fourth Intercostal Space", [5])},
    "FourthLumbarArtery": {
        "397410002": (
            "Fourth Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "FourthVentricle": {
        "35918002": ("Fourth Ventricle", [7140, 7151, 7153, 7192, 8134, 9514])
    },
    "FourthVentricleStructure": {
        "35918002": ("Fourth ventricle structure (body structure)", [])
    },
    "FoutaHorseBreed": {"133113009": ("Fouta horse breed", [7480])},
    "FoveaCentralis": {"67046006": ("Fovea centralis", [4209, 4266, 8134])},
    "FoxTerrierSuperbreed": {"35802007": ("Fox terrier superbreed (organism)", [])},
    "FoxTerrierSuperbreedDogBreed": {
        "35802007": ("Fox terrier superbreed dog breed", [7480])
    },
    "Foxhound": {"90101001": ("Foxhound", [7480])},
    "FractionalFlowReserve": {"371842003": ("Fractional flow reserve", [3620, 3621])},
    "FractionalFlowReserveUsingIntracoronaryBolus": {
        "371835003": ("Fractional Flow Reserve using intracoronary bolus", [3620, 3621])
    },
    "FractionalFlowReserveUsingIntravenousInfusion": {
        "371841005": (
            "Fractional Flow Reserve using intravenous infusion",
            [3620, 3621],
        )
    },
    "FractureOfLowerLimb": {"46866001": ("Fracture of lower limb", [3205])},
    "FratiCattleBreed": {"133388000": ("Frati cattle breed", [7480])},
    "FrederiksborgHorseBreed": {"133112004": ("Frederiksborg horse breed", [7480])},
    "FreeOfSymptoms": {"162467007": ("Free of symptoms (situation)", [])},
    "FreibergHorseBreed": {"131979009": ("Freiberg horse breed", [7480])},
    "FrenchAlpineGoat": {"5438004": ("French alpine goat (organism)", [])},
    "FrenchAlpineGoatBreed": {"5438004": ("French alpine goat breed", [7480])},
    "FrenchArdennaisHorseBreed": {
        "132734002": ("French Ardennais horse breed", [7480])
    },
    "FrenchBrownCattleBreed": {"133391000": ("French Brown cattle breed", [7480])},
    "FrenchBulldog": {"59643008": ("French bulldog", [7480])},
    "FrenchCobHorseBreed": {"131990004": ("French Cob horse breed", [7480])},
    "FrenchGuardDogBreed": {"132487003": ("French Guard Dog breed", [7480])},
    "FrenchLandracePig": {"8970009": ("French landrace pig (organism)", [])},
    "FrenchLandracePigBreed": {"8970009": ("French landrace pig breed", [7480])},
    "FrenchLargeWhitePigBreed": {"132088000": ("French Large White pig breed", [7480])},
    "FrenchSaddlePonyHorseBreed": {
        "131991000": ("French Saddle pony horse breed", [7480])
    },
    "FrenchSpanielDogBreed": {"132488008": ("French Spaniel dog breed", [7480])},
    "FrenchTrotterHorseBreed": {"132774008": ("French Trotter horse breed", [7480])},
    "FrenulumOfLabiaMinora": {"279867004": ("Frenulum of labia minora", [4029, 8134])},
    "Frequencies": {"272123002": ("Frequencies (qualifier value)", [])},
    "Frequent": {"70232002": ("Frequent", [6091])},
    "Friedman": {"399103007": ("Friedman", [4012])},
    "FriedmanProjection": {"399103007": ("Friedman projection (qualifier value)", [])},
    "Friend": {"113163005": ("Friend", [7450])},
    "FriesianHorseBreed": {"133111006": ("Friesian horse breed", [7480])},
    "FriesianMilkSheepBreed": {"131725009": ("Friesian Milk Sheep breed", [7480])},
    "FrijolilloCattleBreed": {"133392007": ("Frijolillo cattle breed", [7480])},
    "FriuliBlackPigBreed": {"132182001": ("Friuli Black pig breed", [7480])},
    "Frog": {"34296003": ("frog", [20])},
    "FrogLikePosture": {"34296003": ("Frog-like posture (finding)", [])},
    "Frontal": {"399033003": ("frontal", [501, 4010])},
    "FrontalBone": {"74872008": ("Frontal bone", [4028, 7151, 7192, 8134, 9514])},
    "FrontalBoneStructure": {
        "74872008": ("Frontal bone structure (body structure)", [])
    },
    "FrontalLobe": {"83251001": ("Frontal lobe", [7151, 7153, 7192, 8134, 9514])},
    "FrontalLobeStructure": {
        "83251001": ("Frontal lobe structure (body structure)", [])
    },
    "FrontalOblique": {"399160007": ("frontal oblique", [501, 4010])},
    "FrontalObliqueAxial": {"399132005": ("frontal-oblique axial", [501, 4010])},
    "FrontalObliqueAxialProjection": {
        "399132005": ("Frontal-oblique axial projection (qualifier value)", [])
    },
    "FrontalObliqueProjection": {
        "399160007": ("Frontal oblique projection (qualifier value)", [])
    },
    "FrontalProjection": {"399033003": ("Frontal projection (qualifier value)", [])},
    "FrontalSinus": {
        "55060009": ("Frontal sinus", [1000, 1001, 1002, 7482, 7483, 8134])
    },
    "FrontalSinusStructure": {
        "55060009": ("Frontal sinus structure (body structure)", [])
    },
    "FroxfieldPygmyPigBreed": {"132263003": ("Froxfield Pygmy pig breed", [7480])},
    "FrozenSectionBreastSample": {
        "309059004": ("frozen section breast sample", [8103, 8104])
    },
    "FrozenSectionEmbeddingMedium": {
        "433469005": ("Frozen section embedding medium (substance)", [])
    },
    "FuanSpottedPigBreed": {"132283002": ("Fuan Spotted pig breed", [7480])},
    "Fuchs": {"399073009": ("Fuchs", [4012])},
    "FuchsProjection": {"399073009": ("Fuchs projection (qualifier value)", [])},
    "FuchsinAcidStain": {"60920007": ("fuchsin acid stain", [8112])},
    "FuchsinBasicStain": {"50062004": ("fuchsin basic stain", [8112])},
    "FujianSmallPigBreed": {"132281000": ("Fujian Small pig breed", [7480])},
    "FullRectum": {"249602003": ("Full Rectum", [9272])},
    "FullSpectrum": {"414298005": ("Full Spectrum", [4206, 8122])},
    "FullSpectrumColor": {"414298005": ("Full spectrum color (qualifier value)", [])},
    "FumatiPigBreed": {"132183006": ("Fumati pig breed", [7480])},
    "Function": {"246464006": ("Function", [7150, 9502, 9580])},
    "FunctionalCapacity": {
        "429160000": ("Functional capacity (observable entity)", [])
    },
    "FunctionalPerformanceAndActivity": {
        "248536006": ("Functional performance and activity", [3769])
    },
    "FunctionalSingleVentricle": {
        "443379009": ("Functional Single Ventricle", [12248, 12249])
    },
    "FundusCamera": {"409898007": ("Fundus Camera", [4202])},
    "FuriosoHorseBreed": {"132775009": ("Furioso horse breed", [7480])},
    "FusiformAneurysm": {"85431000": ("fusiform aneurysm", [3808])},
    "Fusion": {"129379006": ("Fusion", [7310])},
    "FusionAction": {"129379006": ("Fusion - action (qualifier value)", [])},
    "FuzhouBlackPigBreed": {"132285009": ("Fuzhou Black pig breed", [7480])},
    "G1Grade": {"54102005": ("G1 grade (finding)", [])},
    "G2Grade": {"1663004": ("G2 grade (finding)", [])},
    "G3Grade": {"61026006": ("G3 grade (finding)", [])},
    "G4Grade": {"258245003": ("G4 grade (finding)", [])},
    "GTV": {"228791009": ("GTV", [9534])},
    "GackoCattleBreed": {"133394008": ("Gacko cattle breed", [7480])},
    "GadoDaTerraCattleBreed": {"133395009": ("Gado da Terra cattle breed", [7480])},
    "GadobenateDimeglumine": {"792865009": ("Gadobenate dimeglumine", [12, 3850])},
    "Gadobutrol": {"407976008": ("Gadobutrol", [12, 3850])},
    "Gadodiamide": {"354088005": ("Gadodiamide", [12, 3850])},
    "Gadofosveset": {"770982005": ("Gadofosveset", [12, 3850])},
    "Gadolinium": {"58281002": ("Gadolinium", [13, 300])},
    "GadoliniumContainingProduct": {"768763008": ("Gadolinium", [12, 3850])},
    "GadopentetateDimeglumine": {
        "109216000": ("Gadopentetate dimeglumine", [12, 3850])
    },
    "GadopentetateDimeglumine_Inactive": {
        "404846007": ("Gadopentetate dimeglumine (product)", [])
    },
    "GadoterateMeglumine": {"712714000": ("Gadoterate meglumine", [12, 3850])},
    "Gadoteridol": {"353946009": ("Gadoteridol", [12, 3850])},
    "Gadoversetamide": {"409477004": ("Gadoversetamide", [12, 3850])},
    "GadoxetateDisodium": {"440223009": ("Gadoxetate disodium", [12, 3850])},
    "GaitProblem": {"22325002": ("Gait problem", [3205])},
    "Galactocele": {"42385006": ("Galactocele", [6030, 6031])},
    "GalicenoHorse": {"6220006": ("Galiceno horse (organism)", [])},
    "GalicenoHorseBreed": {"6220006": ("Galiceno horse breed", [7480])},
    "GalicianBlondCattleBreed": {"131600007": ("Galician Blond cattle breed", [7480])},
    "GalicianPigBreed": {"132184000": ("Galician pig breed", [7480])},
    "GalicianPonyHorseBreed": {"133110007": ("Galician Pony horse breed", [7480])},
    "Gallbladder": {
        "28231008": (
            "Gallbladder",
            [
                4,
                4009,
                4030,
                4031,
                4040,
                4042,
                7151,
                7154,
                7192,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "GallbladderStructure": {
        "28231008": ("Gallbladder structure (body structure)", [])
    },
    "Gallium67Citrate": {"73065000": ("Gallium^67^ citrate", [25])},
    "GallocyanineStain": {"8836009": ("gallocyanine stain", [8112])},
    "GallowayCattleBreed": {"66911005": ("Galloway cattle breed (organism)", [])},
    "GallowayCowBreed": {"66911005": ("Galloway cow breed", [7480])},
    "GalwaySheepBreed": {"406660008": ("Galway sheep breed", [7480])},
    "GambianNDamaCattleBreed": {
        "133802009": ("Gambian N'Dama cattle breed (organism)", [])
    },
    "GambianNDamaXZebuCattleBreed": {
        "133802009": ("Gambian N'Dama X zebu cattle breed", [7480])
    },
    "GansuAlpineFineWoolSheepBreed": {
        "131726005": ("Gansu Alpine Fine-wool sheep breed", [7480])
    },
    "GanzhongnanSpottedPigBreed": {
        "132300000": ("Ganzhongnan Spotted pig breed", [7480])
    },
    "GanzhouWhitePigBreed": {"132336002": ("Ganzhou White pig breed", [7480])},
    "GaolaoCattleBreed": {"133606005": ("Gaolao cattle breed", [7480])},
    "GarfagninaCattleBreed": {"131472005": ("Garfagnina cattle breed", [7480])},
    "GarganicaGoatBreed": {"131695007": ("Garganica goat breed", [7480])},
    "Garment": {"272180002": ("Garment (physical object)", [])},
    "GarranoTarpanHorseXDomesticHorseBreed": {
        "132990002": ("Garrano tarpan horse X domestic horse breed", [7480])
    },
    "GarreCattleBreed": {"133657009": ("Garre cattle breed", [7480])},
    "Garth": {"425188003": ("Garth", [4012])},
    "GarthProjection": {"425188003": ("Garth projection (qualifier value)", [])},
    "Gas": {"74947009": ("Gas", [7151, 7197])},
    "GasInVitreousCavity": {"247094004": ("Gas in vitreous cavity", [4232])},
    "GasaraCattleBreed": {"133658004": ("Gasara cattle breed", [7480])},
    "GasconCattleBreed": {"131601006": ("Gascon cattle breed", [7480])},
    "GasconyPigBreed": {"132125002": ("Gascony pig breed", [7480])},
    "GaseousSubstance": {"74947009": ("Gaseous substance (substance)", [])},
    "GaspingForBreath": {"23141003": ("Gasping for breath (finding)", [])},
    "GaspingRespiration": {"23141003": ("gasping respiration", [3416])},
    "GastricArtery": {
        "23771002": (
            "Gastric Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "GastricVein": {
        "110568007": (
            "Gastric vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12113,
                12125,
            ],
        )
    },
    "GastroOmentalLymphNode": {"83380007": ("gastro-omental lymph node", [7600, 8134])},
    "GastrocnemiusVein": {
        "264481007": (
            "Gastrocnemius vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "GastroduodenalArtery": {
        "37274004": (
            "Gastroduodenal Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "GastrointestinalProkinetic": {
        "372607002": ("Gastrointestinal prokinetic", [621, 622])
    },
    "GastrointestinalSurgery": {"309980008": ("Gastrointestinal Surgery", [7030])},
    "GastrointestinalSurgeryDepartment": {
        "309980008": ("Gastrointestinal surgery department (environment)", [])
    },
    "GaunlingCattleBreed": {"131474006": ("Gaunling cattle breed", [7480])},
    "Gavage": {"61420007": ("Gavage", [609])},
    "GayalCattleBreed": {"133702003": ("Gayal cattle breed", [7480])},
    "GaynorHart": {"399082003": ("Gaynor-Hart", [4012])},
    "GaynorHartProjection": {
        "399082003": ("Gaynor-Hart projection (qualifier value)", [])
    },
    "GazeDownAndLeft": {"255523004": ("Gaze down and left (qualifier value)", [])},
    "GazeDownAndRight": {"255524005": ("Gaze down and right (qualifier value)", [])},
    "GazeUpAndLeft": {"255525006": ("Gaze up and left (qualifier value)", [])},
    "GazeUpAndRight": {"255526007": ("Gaze up and right (qualifier value)", [])},
    "GelProsthesisForImagingProcedure": {
        "102325005": ("Gel prosthesis for imaging procedure", [5])
    },
    "GelProsthesisForImagingProcedureDevice": {
        "102325005": (
            "Gel prosthesis for imaging procedure, device (physical object)",
            [],
        )
    },
    "GelbrayCattleBreed": {"133731004": ("Gelbray cattle breed (organism)", [])},
    "GelbrayXZebuCattleBreed": {"133731004": ("Gelbray X zebu cattle breed", [7480])},
    "GelbveihCattleBreed": {"76497003": ("Gelbveih cattle breed (organism)", [])},
    "GelbveihCowBreed": {"76497003": ("Gelbveih cow breed", [7480])},
    "GelderlandHorseBreed": {"133109002": ("Gelderland horse breed", [7480])},
    "GeneralAnesthesia": {"50697003": ("General anesthesia", [611, 612])},
    "GeneralAnesthetic": {"373288007": ("General anesthetic", [621, 622])},
    "GeneralMedicalDepartment": {
        "309927009": ("General medical department (environment)", [])
    },
    "GeneralMedicine": {"309927009": ("General Medicine", [7030])},
    "GeneralSurgery": {"309984004": ("General Surgery", [7030])},
    "GeneralSurgicalDepartment": {
        "309984004": ("General surgical department (environment)", [])
    },
    "Generalized": {"60132005": ("Generalized", [6128])},
    "GeneticSample": {"258562007": ("Genetic sample", [8103])},
    "GenicularArtery": {
        "128559007": ("Genicular artery", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "GeniculateArtery": {"128559007": ("geniculate artery", [3604, 3606, 3630])},
    "GentianViolet": {"387239001": ("Gentian violet (substance)", [])},
    "GentileDiPugliaSheepBreed": {
        "131740006": ("Gentile di Puglia sheep breed", [7480])
    },
    "GenuOfCorpusCallosum": {
        "70215001": ("genu of corpus callosum", [7705, 7710, 8134])
    },
    "GenusBos": {"388168008": ("Genus Bos (organism)", [])},
    "GenusCanis": {"388490000": ("Genus Canis (organism)", [])},
    "GenusCapra": {"388249000": ("Genus Capra (organism)", [])},
    "GenusDependovirus": {"112381006": ("Genus Dependovirus (organism)", [])},
    "GenusEquus": {"388445009": ("Genus Equus (organism)", [])},
    "GenusFelis": {"388626009": ("Genus Felis (organism)", [])},
    "GenusMus": {"447482001": ("Genus Mus (organism)", [])},
    "GenusOvis": {"388254009": ("Genus Ovis (organism)", [])},
    "GenusRattus": {"371564000": ("Genus Rattus (organism)", [])},
    "GenusSus": {"388393002": ("Genus Sus (organism)", [])},
    "GeorgianMountainCattleBreed": {
        "133396005": ("Georgian Mountain cattle breed", [7480])
    },
    "GeriatricMedicine": {"309933000": ("Geriatric Medicine", [7030])},
    "GermanAngusCattleBreed": {"133582003": ("German Angus cattle breed", [7480])},
    "GermanBerkshirePigBreed": {"132185004": ("German Berkshire pig breed", [7480])},
    "GermanBlackPiedCattleBreed": {
        "133397001": ("German Black Pied cattle breed", [7480])
    },
    "GermanBlackPiedDairyCattleBreed": {
        "133398006": ("German Black Pied Dairy cattle breed", [7480])
    },
    "GermanBlackheadedMuttonSheepBreed": {
        "131723002": ("German Blackheaded Mutton sheep breed", [7480])
    },
    "GermanBrownCattleBreed": {"133405000": ("German Brown cattle breed", [7480])},
    "GermanColoredGoatBreed": {"131693000": ("German colored goat breed", [7480])},
    "GermanCornwallPigBreed": {"132358000": ("German Cornwall pig breed", [7480])},
    "GermanFleckViehCattleBreed": {
        "67448000": ("German Fleck-Vieh cattle breed (organism)", [])
    },
    "GermanFleckViehCowBreed": {"67448000": ("German Fleck-Vieh cow breed", [7480])},
    "GermanKoolieDogBreed": {"132464006": ("German Koolie dog breed", [7480])},
    "GermanLandraceBPigBreed": {"132352004": ("German Landrace B pig breed", [7480])},
    "GermanLandracePig": {"8763002": ("German landrace pig (organism)", [])},
    "GermanLandracePigBreed": {"8763002": ("German landrace pig breed", [7480])},
    "GermanLonghairedPointer": {
        "1420005": ("German longhaired pointer (organism)", [])
    },
    "GermanLonghairedPointerDogBreed": {
        "1420005": ("German longhaired pointer dog breed", [7480])
    },
    "GermanMountainSheepBreed": {"131741005": ("German Mountain sheep breed", [7480])},
    "GermanMuttonMerinoSheepBreed": {
        "132716001": ("German Mutton Merino sheep breed", [7480])
    },
    "GermanPasturePigBreed": {"132159003": ("German Pasture pig breed", [7480])},
    "GermanPinscherDogBreed": {"132510005": ("German Pinscher dog breed", [7480])},
    "GermanRedPiedCattleBreed": {"131602004": ("German Red Pied cattle breed", [7480])},
    "GermanRedPiedPigBreed": {"132357005": ("German Red Pied pig breed", [7480])},
    "GermanRexCatBreed": {"132666001": ("German Rex cat breed", [7480])},
    "GermanRidingPonyHorseBreed": {
        "131972000": ("German Riding Pony horse breed", [7480])
    },
    "GermanShepherdDog": {"42252000": ("German shepherd dog", [7480])},
    "GermanShorthairedPointer": {
        "86767001": ("German shorthaired pointer (organism)", [])
    },
    "GermanShorthairedPointerDogBreed": {
        "86767001": ("German shorthaired pointer dog breed", [7480])
    },
    "GermanShorthornCattleBreed": {
        "133406004": ("German Shorthorn cattle breed", [7480])
    },
    "GermanSpanielDogBreed": {"132505006": ("German Spaniel dog breed", [7480])},
    "GermanSpitzDogBreed": {"132424007": ("German Spitz dog breed", [7480])},
    "GermanWhiteheadedMuttonSheepBreed": {
        "131727001": ("German Whiteheaded Mutton sheep breed", [7480])
    },
    "GermanWirehairedPointer": {
        "25264009": ("German wirehaired pointer (organism)", [])
    },
    "GermanWirehairedPointerDogBreed": {
        "25264009": ("German wirehaired pointer dog breed", [7480])
    },
    "GermaniumGe68": {"53315004": ("Germanium Ge^68^", [4021])},
    "GhanaSangaCattleBreed": {"133803004": ("Ghana Sanga cattle breed (organism)", [])},
    "GhanaSangaXZebuCattleBreed": {
        "133803004": ("Ghana Sanga X zebu cattle breed", [7480])
    },
    "GhanaShorthornCattleBreed": {
        "133407008": ("Ghana Shorthorn cattle breed", [7480])
    },
    "GhoriPigBreed": {"132186003": ("Ghori pig breed", [7480])},
    "GiacominiVein": {
        "397437000": ("Giacomini vein", [3827, 7151, 7192, 8134, 9514, 12103, 12110])
    },
    "GiantFibroadenoma": {"34882000": ("Giant fibroadenoma", [6030, 6031])},
    "GiantSchnauzer": {"57947002": ("Giant schnauzer (organism)", [])},
    "GiantSchnauzerDogBreed": {"57947002": ("Giant schnauzer dog breed", [7480])},
    "GidranHorseBreed": {"133108005": ("Gidran horse breed", [7480])},
    "GiemsaStain": {"373646006": ("giemsa stain", [8112])},
    "GirCattleBreed": {"32938007": ("Gir cattle breed (organism)", [])},
    "GirCowBreed": {"32938007": ("Gir cow breed", [7480])},
    "GirgentanaGoatBreed": {"131696008": ("Girgentana goat breed", [7480])},
    "GirolandoCattleBreed": {"133804005": ("Girolando cattle breed (organism)", [])},
    "GirolandoXZebuCattleBreed": {
        "133804005": ("Girolando X zebu cattle breed", [7480])
    },
    "GlanCattleBreed": {"131603009": ("Glan cattle breed", [7480])},
    "GlanDonnersbergCattleBreed": {
        "133408003": ("Glan-Donnersberg cattle breed", [7480])
    },
    "GlenOfImaalTerrierDogBreed": {
        "132644008": ("Glen of Imaal Terrier dog breed", [7480])
    },
    "GlennPathway": {"443789005": ("Glenn Pathway", [12282])},
    "GlenoidCavityOfScapula": {"46385009": ("Glenoid cavity of scapula", [6115, 8134])},
    "GlenoidStructure": {"46385009": ("Glenoid structure (body structure)", [])},
    "Glioblastoma": {"1163375002": ("Glioblastoma", [638, 639])},
    "GlioblastomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "63634009": (
            "Glioblastoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "GlobusPallidus": {
        "14738005": ("Globus Pallidus", [7140, 7151, 7153, 7192, 8134, 9514])
    },
    "GlobusPallidusStructure": {
        "14738005": ("Globus pallidus structure (body structure)", [])
    },
    "GlomerularFiltrationRate": {
        "80274001": ("Glomerular filtration rate (observable entity)", [])
    },
    "GlossopharyngealNerve": {
        "21161002": ("glossopharyngeal nerve", [7706, 7710, 8134])
    },
    "GlossopharyngealNerveStructure": {
        "21161002": ("Glossopharyngeal nerve structure (body structure)", [])
    },
    "Glottis": {"1307006": ("glottis", [7601, 8134])},
    "GlottisStructure": {"1307006": ("Glottis structure (body structure)", [])},
    "GloucesterCattleBreed": {"131604003": ("Gloucester cattle breed", [7480])},
    "GloucesterOldSpotPig": {"90885005": ("Gloucester old spot pig (organism)", [])},
    "GloucesterOldSpotPigBreed": {
        "90885005": ("Gloucester old spot pig breed", [7480])
    },
    "GlucagonPreparation": {"10712001": ("Glucagon preparation", [10])},
    "GlucagonProduct": {"10712001": ("Glucagon product (product)", [])},
    "Glue": {"57126000": ("Glue", [8, 4051, 7151, 7193])},
    "Gluing": {"360038009": ("Gluing", [7310])},
    "GluingAction": {"360038009": ("Gluing - action (qualifier value)", [])},
    "GlutamateN13": {"129509006": ("Glutamate N^13^", [4021])},
    "Glutamine": {"25761002": ("Glutamine", [218, 4032, 4033, 7180, 7186, 7469])},
    "GlutealLymphNode": {"80867000": ("gluteal lymph node", [7600, 8134])},
    "GlycogenRichCarcinoma": {"74280008": ("Glycogen-rich carcinoma", [6030, 6033])},
    "GobraCattleBreed": {"133659007": ("Gobra cattle breed", [7480])},
    "Gold": {"2309006": ("Gold", [300])},
    "GoldAlloy": {"256496006": ("Gold Alloy", [7300, 9579])},
    "GoldenAmericanSaddlebredHorseBreed": {
        "133107000": ("Golden American Saddlebred horse breed", [7480])
    },
    "GoldenGuernseyGoatBreed": {"131660009": ("Golden Guernsey goat breed", [7480])},
    "GoldenRetriever": {"58108001": ("Golden retriever (organism)", [])},
    "GoldenRetrieverDogBreed": {"58108001": ("Golden retriever dog breed", [7480])},
    "GoleCattleBreed": {"133409006": ("Gole cattle breed", [7480])},
    "GolpayeganiCattleBreed": {"133410001": ("Golpayegani cattle breed", [7480])},
    "GonadRadiationShield": {"470204007": ("Gonad radiation shield", [9572, 9573])},
    "Goniolens": {"389156006": ("Goniolens", [4205])},
    "GoomsurCattleBreed": {"133660002": ("Goomsur cattle breed", [7480])},
    "GorbatovRedCattleBreed": {"133411002": ("Gorbatov Red cattle breed", [7480])},
    "GordonSetter": {"57849000": ("Gordon setter (organism)", [])},
    "GordonSetterDogBreed": {"57849000": ("Gordon setter dog breed", [7480])},
    "GorlinSyndrome": {"69408002": ("Gorlin syndrome", [4405])},
    "GorynCattleBreed": {"133412009": ("Goryn cattle breed", [7480])},
    "GotlandHorseBreed": {"133100003": ("Gotland horse breed", [7480])},
    "Grade1WellDifferentiated": {"54102005": ("Grade 1: well differentiated", [6070])},
    "Grade2ModeratelyDifferentiated": {
        "1663004": ("Grade 2: moderately differentiated", [6070])
    },
    "Grade3PoorlyDifferentiated": {
        "61026006": ("Grade 3: poorly differentiated", [6070])
    },
    "Grade4Undifferentiated": {"258245003": ("Grade 4: undifferentiated", [6070])},
    "GraftMaterial": {"246345001": ("Graft material", [637])},
    "GraftToCitedBodySegment": {
        "128947001": ("Graft to cited body segment (substance)", [])
    },
    "GraftToDistalAnastomosis": {"128948006": ("Graft to distal anastomosis", [3019])},
    "GraftToProximalAnastomosis": {
        "128949003": ("Graft to proximal anastomosis", [3019])
    },
    "GramStain": {"385484003": ("gram stain", [8112])},
    "GrandAngloFrançaisDogBreed": {
        "132506007": ("Grand Anglo-Français dog breed", [7480])
    },
    "GrandBassettGriffonVendeenDogBreed": {
        "132507003": ("Grand Bassett Griffon Vendeen dog breed", [7480])
    },
    "GrandBassettGriffonVendéenDogBreed": {
        "132507003": ("Grand Bassett Griffon Vendéen dog breed (organism)", [])
    },
    "GrandBleuDeGascogneDogBreed": {
        "132508008": ("Grand Bleu de Gascogne dog breed", [7480])
    },
    "GrandGasconSaintongeoisDogBreed": {
        "132509000": ("Grand Gascon-Saintongeois dog breed", [7480])
    },
    "GrandGriffonVendeenDogBreed": {
        "132515000": ("Grand Griffon Vendeen dog breed", [7480])
    },
    "GrandGriffonVendéenDogBreed": {
        "132515000": ("Grand Griffon Vendéen dog breed (organism)", [])
    },
    "Grandy": {"399311007": ("Grandy", [4012])},
    "GrandyProjection": {"399311007": ("Grandy projection (qualifier value)", [])},
    "GranularCellTumor": {"12169001": ("Granular cell tumor", [6030, 6031])},
    "Grashey": {"399146005": ("Grashey", [4012])},
    "GrasheyProjection": {"399146005": ("Grashey projection (qualifier value)", [])},
    "GratiCattleBreed": {"131473000": ("Grati cattle breed", [7480])},
    "GraueGehoernteHeidschnuckeSheepBreed": {
        "131728006": ("Graue Gehoernte Heidschnucke sheep breed", [7480])
    },
    "Gray": {"371253002": ("Gray", [4411])},
    "GrayAlpineCattleBreed": {"133422003": ("Gray Alpine cattle breed", [7480])},
    "GrayColor": {"371253002": ("Gray color", [6067])},
    "GrayMatter": {"389081007": ("Gray Matter", [7151, 7153, 7192, 8134, 9514])},
    "GrayMatterStructureOfCentralNervousSystem": {
        "389081007": (
            "Gray matter structure of central nervous system (body structure)",
            [],
        )
    },
    "GreatCardiacVein": {
        "5928000": (
            "Great cardiac vein",
            [4, 3010, 3011, 3827, 3839, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "GreatCardiacVeinStructure": {
        "5928000": ("Great cardiac vein structure (body structure)", [])
    },
    "GreatDaneDog": {"27615007": ("Great dane dog", [7480])},
    "GreatMünsterländerDogBreed": {
        "132379000": ("Great Münsterländer dog breed", [7480])
    },
    "GreatPyreneeDog": {"32670005": ("Great Pyrenee dog", [7480])},
    "GreatSaphenousVein": {
        "60734001": (
            "Great saphenous vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "GreatSaphenousVeinOfCalf": {
        "414368000": (
            "Great Saphenous Vein of Calf",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "GreatSaphenousVeinOfThigh": {
        "414369008": (
            "Great Saphenous Vein of Thigh",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "GreatSaphenousVeinStructure": {
        "60734001": ("Great saphenous vein structure (body structure)", [])
    },
    "GreaterCaucasusCattleBreed": {
        "133413004": ("Greater Caucasus cattle breed", [7480])
    },
    "GreaterSwissMountainDogBreed": {
        "132511009": ("Greater Swiss Mountain Dog breed", [7480])
    },
    "GreaterSwissMountainHoundDogBreed": {
        "132649003": ("Greater Swiss Mountain Hound dog breed", [7480])
    },
    "GreekLocalPigBreed": {"132253006": ("Greek Local pig breed", [7480])},
    "GreekShorthornCattleBreed": {
        "133420006": ("Greek Shorthorn cattle breed", [7480])
    },
    "GreekSteppeCattleBreed": {"133421005": ("Greek Steppe cattle breed", [7480])},
    "Green": {"371246006": ("Green", [4206, 4255, 8122])},
    "GreenColor": {"371246006": ("Green color", [6067])},
    "GreenOpticalFilter": {"445465004": ("Green optical filter", [4204, 8124])},
    "GreenOpticalFilterOfOpticalMircroscope": {
        "445465004": (
            "Green optical filter of optical mircroscope (physical object)",
            [],
        )
    },
    "GreenlandDogBreed": {"132512002": ("Greenland Dog breed", [7480])},
    "Greyhound": {"112494009": ("Greyhound", [7480])},
    "GriffonDog": {"55058007": ("Griffon dog", [7480])},
    "GriffonFauveDeBretegneDogBreed": {
        "132513007": ("Griffon Fauve de Bretegne dog breed", [7480])
    },
    "GriffonNivernaisDogBreed": {"132514001": ("Griffon Nivernais dog breed", [7480])},
    "GrisonsStripedGoatBreed": {"131664000": ("Grisons Striped goat breed", [7480])},
    "GroinSkinCrease": {"280387007": ("Groin skin crease", [4029, 8134])},
    "GromarkSheepBreed": {"131730008": ("Gromark sheep breed", [7480])},
    "GroningenHorseBreed": {"133104007": ("Groningen horse breed", [7480])},
    "GroningenWhiteheadedCattleBreed": {
        "131605002": ("Groningen Whiteheaded cattle breed", [7480])
    },
    "GrossPathologyRequest": {"168456004": ("Gross pathology request", [8131])},
    "GrossSpecimen": {"430861001": ("Gross specimen", [8103])},
    "GrossTumorVolume": {"228791009": ("Gross tumor volume (observable entity)", [])},
    "GroupedCalcificationDistribution": {
        "129766004": ("Grouped calcification distribution", [6012, 6013])
    },
    "GuadarramaGoatBreed": {"131694006": ("Guadarrama goat breed", [7480])},
    "GuadeloupeCreolePigBreed": {"132032009": ("Guadeloupe Creole pig breed", [7480])},
    "GuadeloupeCréolePigBreed": {
        "132032009": ("Guadeloupe Créole pig breed (organism)", [])
    },
    "GuadianaSpottedCattleBreed": {
        "133423008": ("Guadiana Spotted cattle breed", [7480])
    },
    "GuangxiHorseBreed": {"133115002": ("Guangxi horse breed", [7480])},
    "GuangxiWhitePigBreed": {"132337006": ("Guangxi White pig breed", [7480])},
    "GuanlingPigBreed": {"132148007": ("Guanling pig breed", [7480])},
    "GuanzhongHorseBreed": {"131973005": ("Guanzhong horse breed", [7480])},
    "GuelmaCattleBreed": {"133424002": ("Guelma cattle breed", [7480])},
    "GuernseyCattleBreed": {"84839000": ("Guernsey cattle breed (organism)", [])},
    "GuernseyCowBreed": {"84839000": ("Guernsey cow breed", [7480])},
    "GuidanceIntent": {"363675004": ("Guidance Intent", [3629])},
    "GuideWire": {"272224001": ("Guide Wire", [3411, 3429])},
    "GuidingCatheter": {
        "102317008": ("Guiding catheter", [8, 3411, 3429, 4051, 7151, 7193])
    },
    "GuidingCatheterDevice": {
        "102317008": ("Guiding catheter, device (physical object)", [])
    },
    "GuineaHogPigBreed": {"132235000": ("Guinea Hog pig breed", [7480])},
    "GuizhouHorseBreed": {"131974004": ("Guizhou horse breed", [7480])},
    "GujamavuCattleBreed": {"133661003": ("Gujamavu cattle breed", [7480])},
    "GujaratiCattleBreed": {"112485003": ("Gujarati cattle breed (organism)", [])},
    "GujaratiCowBreed": {"112485003": ("Gujarati cow breed", [7480])},
    "GulfCoastNativeSheepBreed": {
        "131731007": ("Gulf Coast Native sheep breed", [7480])
    },
    "GuoxiaHorseBreed": {"131975003": ("Guoxia horse breed", [7480])},
    "GurktalPigBreed": {"133203009": ("Gurktal pig breed", [7480])},
    "GuteSheepBreed": {"131738001": ("Gute sheep breed", [7480])},
    "Guttate": {"1312007": ("Guttate (qualifier value)", [])},
    "Gutter": {"68493006": ("Gutter", [2])},
    "GuzerandoCattleBreed": {"133805006": ("Guzerando cattle breed (organism)", [])},
    "GuzerandoXZebuCattleBreed": {
        "133805006": ("Guzerando X zebu cattle breed", [7480])
    },
    "GuzeratCattleBreed": {"133699005": ("Guzerat cattle breed", [7480])},
    "GynecologicalHistory": {
        "267011001": ("Gynecological history (observable entity)", [])
    },
    "Gynecology": {"309943002": ("Gynecology", [7030])},
    "GynecologyDepartment": {"309943002": ("Gynecology department (environment)", [])},
    "Gynecomastia": {"4754008": ("Gynecomastia", [6030, 6031])},
    "GypsyVannerHorse": {"406714004": ("Gypsy Vanner horse (organism)", [])},
    "GypsyVannerHorseBreed": {"406714004": ("Gypsy Vanner horse breed", [7480])},
    "GöttingenMiniaturePigBreed": {
        "132359008": ("Göttingen Miniature pig breed", [7480])
    },
    "GüssingForestPigPigBreed": {"132100006": ("Güssing Forest Pig pig breed", [7480])},
    "H1Antihistamine": {"373228009": ("H-1 Antihistamine", [65])},
    "HIVPositive": {"165816005": ("HIV Positive", [3756])},
    "HaBacPigBreed": {"132348004": ("Ha Bac pig breed", [7480])},
    "Haas": {"399341009": ("Haas", [4012])},
    "HaasProjection": {"399341009": ("Haas projection (qualifier value)", [])},
    "HabenularCommissure": {"6866008": ("habenular commissure", [7705, 7710, 8134])},
    "HackneyHorse": {"112488001": ("Hackney horse (organism)", [])},
    "HackneyHorseBreed": {"112488001": ("Hackney horse breed", [7480])},
    "HackneyPonyHorseBreed": {"132748002": ("Hackney Pony horse breed", [7480])},
    "HaemostaticAgent": {"372681003": ("Hemostatic agent", [621, 622, 4410])},
    "HaflingerHorse": {"54447000": ("Haflinger horse (organism)", [])},
    "HaflingerHorseBreed": {"54447000": ("Haflinger horse breed", [7480])},
    "Hafnium": {"50672002": ("Hafnium", [300])},
    "HailumPigBreed": {"133169000": ("Hailum pig breed", [7480])},
    "HainanLingaoPigBreed": {"132155009": ("Hainan, Lingao pig breed", [7480])},
    "HainanPigBreed": {"133162009": ("Hainan pig breed", [7480])},
    "HainanTunchangPigBreed": {"132156005": ("Hainan, Tunchang pig breed", [7480])},
    "HainanWenchangPigBreed": {"132157001": ("Hainan, Wenchang pig breed", [7480])},
    "Hair": {"386045008": ("Hair", [4029, 8134])},
    "HairStructure": {"386045008": ("Hair structure (body structure)", [])},
    "HaitiCreolePigBreed": {"132009005": ("Haiti Creole pig breed", [7480])},
    "HaitiCréolePigBreed": {"132009005": ("Haiti Créole pig breed (organism)", [])},
    "HalfBrother": {"45929001": ("Half-brother", [7450, 7451])},
    "HalfLifeOfRadiopharmaceutical": {
        "304283002": ("Half-life of radiopharmaceutical (qualifier value)", [])
    },
    "HalfSaddlebredHorseBreed": {"131977006": ("Half Saddlebred horse breed", [7480])},
    "HalfSister": {"2272004": ("Half-sister", [7450, 7451])},
    "HalhinGolCattleBreed": {"131475007": ("Halhin Gol cattle breed", [7480])},
    "HallCatheter": {"371799004": ("Hall catheter (physical object)", [])},
    "HallCatheterMethod": {"133913008": ("Hall catheter method (procedure)", [])},
    "HallikarCattleBreed": {"133607001": ("Hallikar cattle breed", [7480])},
    "HallingdalCattleBreed": {"133579008": ("Hallingdal cattle breed", [7480])},
    "Halothane": {"387351001": ("Halothane", [623, 624])},
    "Hamartoma": {"51398009": ("Hamartoma", [6030, 6031])},
    "HamiltonstövareDogBreed": {"132490009": ("Hamiltonstövare dog breed", [7480])},
    "HamlinePigBreed": {"132011001": ("Hamline pig breed", [7480])},
    "HampenPigBreed": {"132019004": ("Hampen pig breed", [7480])},
    "HamplinePigBreed": {"132025000": ("Hampline pig breed", [7480])},
    "HampshireDownSheep": {"82440005": ("Hampshire Down sheep (organism)", [])},
    "HampshireDownSheepBreed": {"82440005": ("Hampshire Down sheep breed", [7480])},
    "HampshirePig": {"20280002": ("Hampshire pig (organism)", [])},
    "HampshirePigBreed": {"20280002": ("Hampshire pig breed", [7480])},
    "HampshireSheepBreed": {"131739009": ("Hampshire sheep breed", [7480])},
    "HamrocPigBreed": {"132208009": ("Hamroc pig breed", [7480])},
    "HanSheepBreed": {"131729003": ("Han sheep breed", [7480])},
    "Hand": {"85562004": ("Hand", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])},
    "HandBone": {"24097009": ("Hand bone", [7304, 8134])},
    "HandGrip": {"128965002": ("Hand grip", [12002])},
    "HandGripPhysiologicChallenge": {
        "128965002": ("Hand grip physiologic challenge (procedure)", [])
    },
    "HandStructure": {"85562004": ("Hand structure (body structure)", [])},
    "HandSurgery": {"309985003": ("Hand Surgery", [7030])},
    "HandSurgeryDepartment": {
        "309985003": ("Hand surgery department (environment)", [])
    },
    "Handgrip": {"128965002": ("Handgrip", [91, 3271])},
    "HanfordMiniaturePigBreed": {"132042006": ("Hanford Miniature pig breed", [7480])},
    "HangPigBreed": {"132301001": ("Hang pig breed", [7480])},
    "HanjiangBlackPigBreed": {"132133001": ("Hanjiang Black pig breed", [7480])},
    "HanoverianHorse": {"66168008": ("Hanoverian horse (organism)", [])},
    "HanoverianHorseBreed": {"66168008": ("Hanoverian horse breed", [7480])},
    "HanoverianHoundDogBreed": {"132499005": ("Hanoverian Hound dog breed", [7480])},
    "HanselStain": {"406983008": ("hansel stain", [8112])},
    "HanzhongWhitePigBreed": {"132338001": ("Hanzhong White pig breed", [7480])},
    "HarbinWhitePigBreed": {"132127005": ("Harbin White pig breed", [7480])},
    "HarianaCattleBreed": {"133608006": ("Hariana cattle breed", [7480])},
    "HarlequinCatBreed": {"132650003": ("Harlequin cat breed", [7480])},
    "HarnaiSheepBreed": {"131775006": ("Harnai sheep breed", [7480])},
    "HarrierDog": {"76724004": ("Harrier dog", [7480])},
    "HarrisBeathAxialProjection": {
        "260493008": ("Harris Beath axial projection", [4012])
    },
    "HartónCattleBreed": {"131606001": ("Hartón cattle breed", [7480])},
    "HarvestingOfDonorMaterial": {
        "53958007": ("Harvesting of donor material (procedure)", [])
    },
    "HarvestingOfTissue": {"53958007": ("Harvesting of tissue", [8109])},
    "HarzRedCattleBreed": {"133425001": ("Harz Red cattle breed", [7480])},
    "HasActiveIngredient": {"127489000": ("Has active ingredient (attribute)", [])},
    "HasIntent": {"363703001": ("Has intent (attribute)", [])},
    "HasInterpretation": {"363713009": ("Has interpretation (attribute)", [])},
    "HashtNagriSheepBreed": {"131762006": ("Hasht Nagri sheep breed", [7480])},
    "HattonCattleBreed": {"133806007": ("Hatton cattle breed (organism)", [])},
    "HattonXZebuCattleBreed": {"133806007": ("Hatton X zebu cattle breed", [7480])},
    "HavanaBrownCat": {"3354004": ("Havana brown cat", [7480])},
    "HavaneseDogBreed": {"132626000": ("Havanese dog breed", [7480])},
    "HawaiianWildCattleBreed": {"133426000": ("Hawaiian wild cattle breed", [7480])},
    "HaysConverterCattleBreed": {
        "23629009": ("Hays converter cattle breed (organism)", [])
    },
    "HaysConverterCowBreed": {"23629009": ("Hays converter cow breed", [7480])},
    "HazaragieSheepBreed": {"131763001": ("Hazaragie sheep breed", [7480])},
    "Head": {
        "69536005": ("Head", [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "HeadAndNeck": {
        "774007": (
            "Head and Neck",
            [4, 645, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "HeadAndNeckStructure": {
        "774007": ("Head and neck structure (body structure)", [])
    },
    "HeadOfFemur": {"2812003": ("Head of Femur", [7304])},
    "HeadOfRib": {"12872006": ("Head of rib", [6115, 8134])},
    "HeadOfRibStructure": {"12872006": ("Head of rib structure (body structure)", [])},
    "HeadStructure": {"69536005": ("Head structure (body structure)", [])},
    "HeadUp": {"128963009": ("Head up", [91, 3271])},
    "HeadUpPhysiologicChallenge": {
        "128963009": ("Head up physiologic challenge (procedure)", [])
    },
    "Headache": {"25064002": ("Headache", [60, 9300])},
    "Headfirst": {"102540008": ("headfirst", [21])},
    "HeadfirstPosition": {"102540008": ("Headfirst position (finding)", [])},
    "Headrest": {"706683002": ("Headrest", [7151, 7157, 7193, 9505, 9513, 9520, 9573])},
    "HealthcareAssociatedInfection": {
        "408678008": ("Healthcare associated infection", [6062])
    },
    "HealthcareAssociatedInfectiousDisease": {
        "408678008": ("Healthcare associated infectious disease (disorder)", [])
    },
    "HealthcareProfessional": {"223366009": ("Healthcare professional", [7450])},
    "Heart": {
        "80891009": (
            "Heart",
            [
                1000,
                1001,
                1004,
                4009,
                4030,
                4031,
                4042,
                6100,
                6102,
                6109,
                6113,
                6149,
                7151,
                7152,
                7192,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "HeartDisease": {"56265001": ("heart disease", [3727])},
    "HeartDiseaseCongenital": {
        "13213009": ("heart disease, congenital", [3700, 3727, 3728])
    },
    "HeartDiseaseRiskFactors": {
        "171224000": ("Heart disease risk factors", [3201, 12246])
    },
    "HeartFailure": {"84114007": ("Heart failure", [3201])},
    "HeartRateRecoveryTime": {
        "429157007": ("Heart rate recovery time (observable entity)", [])
    },
    "HeartRateResponse": {"428531008": ("Heart rate response (observable entity)", [])},
    "HeartStructure": {"80891009": ("Heart structure (body structure)", [])},
    "HeartTransplant": {"32413006": ("Heart transplant", [3721, 3727])},
    "HeartValveCalcification": {
        "373136005": ("Heart valve calcification (finding)", [])
    },
    "HeartValveDisorder": {"368009": ("Heart valve disorder (disorder)", [])},
    "HeartValveProsthesis": {
        "25510005": ("Heart valve prosthesis", [6102, 6138, 6404, 7151, 7193])
    },
    "HeartValveReplacementProsthesis": {
        "275035006": ("Heart valve replacement - prosthesis (procedure)", [])
    },
    "HeartValveStenosis": {"44241007": ("Stenosis", [3711])},
    "HebrideanSheepBreed": {"131770001": ("Hebridean sheep breed", [7480])},
    "HeidschnuckeSheepBreed": {"131771002": ("Heidschnucke sheep breed", [7480])},
    "HeiheHorseBreed": {"132750005": ("Heihe horse breed", [7480])},
    "HeilongjiangSpottedPigBreed": {
        "132128000": ("Heilongjiang Spotted pig breed", [7480])
    },
    "HeilongkaingHorseBreed": {"132751009": ("Heilongkaing horse breed", [7480])},
    "HeldInspiration": {"128969008": ("Held inspiration", [91, 3271])},
    "HeldInspirationPhysiologicChallenge": {
        "128969008": ("Held inspiration physiologic challenge (procedure)", [])
    },
    "HeldVentilation": {"128970009": ("Held ventilation", [91, 3271])},
    "HeldVentilationPhysiologicChallenge": {
        "128970009": ("Held ventilation physiologic challenge (procedure)", [])
    },
    "HellenicHoundDogBreed": {"132427000": ("Hellenic Hound dog breed", [7480])},
    "HellyFixative": {"433471005": ("Helly fixative (substance)", [])},
    "HellyFluid": {"433471005": ("Helly's fluid", [8114])},
    "Hemangioma": {"400210000": ("Hemangioma", [6030, 6031])},
    "HemangiomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "2099007": (
            "Hemangioma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "HemangiomaOfSubcutaneousTissue": {
        "93473009": ("Hemangioma of subcutaneous tissue", [6030, 6031])
    },
    "HemangiomaVenous": {"56468002": ("Hemangioma - venous", [6030, 6031])},
    "Hemangiopericytoma": {"128736003": ("Hemangiopericytoma", [6030, 6033])},
    "HemangiopericytomaMorphologic": {
        "36060005": ("Hemangiopericytoma (morphologic abnormality)", [])
    },
    "Hemangiosarcoma": {"39000009": ("Hemangiosarcoma (morphologic abnormality)", [])},
    "HemateinStain": {"75956008": ("hematein stain", [8112])},
    "Hematology": {"309954007": ("Hematology", [7030])},
    "HematologyDepartment": {"309954007": ("Hematology department (environment)", [])},
    "Hematoma": {"35566002": ("Hematoma", [6030, 6031])},
    "HematomaPostoperative": {"213262007": ("Hematoma - postoperative", [6062])},
    "HematoxylinAndEosinStainMethod": {
        "104210008": ("Hematoxylin and eosin stain method", [8130])
    },
    "HematoxylinStain": {"12710003": ("hematoxylin stain", [8112])},
    "HemiFontanOperation": {"233230003": ("Hemi-Fontan operation", [12247])},
    "HemiFontanPathway": {"443327008": ("Hemi-Fontan Pathway", [12282])},
    "HemodynamicFlowWaveform": {"128434001": ("Hemodynamic flow waveform", [3003])},
    "HemodynamicFlowWaveformFunction": {
        "128434001": ("Hemodynamic flow waveform, function (observable entity)", [])
    },
    "HemodynamicImpedanceWaveform": {
        "128552003": ("Hemodynamic impedance waveform", [3003])
    },
    "HemodynamicImpedanceWaveformFunction": {
        "128552003": (
            "Hemodynamic impedance waveform, function (observable entity)",
            [],
        )
    },
    "HemodynamicMeasurementViaDualCatheterMethodRegimeTherapy": {
        "128573003": (
            "Hemodynamic measurement via dual catheter method (regime/therapy)",
            [],
        )
    },
    "HemodynamicMeasurementViaPullbackMethodRegimeTherapy": {
        "128575005": (
            "Hemodynamic measurement via pullback method (regime/therapy)",
            [],
        )
    },
    "HemodynamicMeasurementViaWedgeMethodRegimeTherapy": {
        "128579004": ("Hemodynamic measurement via wedge method (regime/therapy)", [])
    },
    "HemodynamicMeasurements": {"44324008": ("Hemodynamic Measurements", [12302])},
    "HemodynamicOxygenSaturationWaveform": {
        "128435000": ("Hemodynamic oxygen saturation waveform", [3003])
    },
    "HemodynamicOxygenSaturationWaveformFunction": {
        "128435000": (
            "Hemodynamic oxygen saturation waveform, function (observable entity)",
            [],
        )
    },
    "HemodynamicPressureWaveform": {
        "128433007": ("Hemodynamic pressure waveform", [3003])
    },
    "HemodynamicPressureWaveformFunction": {
        "128433007": ("Hemodynamic pressure waveform, function (observable entity)", [])
    },
    "HemodynamicWaveform": {"128432002": ("Hemodynamic waveform", [3003])},
    "HemodynamicWaveformFunction": {
        "128432002": ("Hemodynamic waveform, function (observable entity)", [])
    },
    "Hemoglobin": {"38082009": ("Hemoglobin", [11006])},
    "HemolymphNode": {"18457007": ("hemolymph node", [7600, 8134])},
    "Hemoptysis": {"66857006": ("Hemoptysis", [12246])},
    "Hemorrhage": {
        "50960005": ("Hemorrhage", [6335, 6336, 6337, 6338, 7151, 7159, 7194])
    },
    "HemorrhagePostprocedure": {"110265006": ("Hemorrhage postprocedure", [6062])},
    "HemostaticAgent": {"26370007": ("Hemostatic agent", [10])},
    "Henschen": {"399199004": ("Henschen", [4012])},
    "HenschenProjection": {"399199004": ("Henschen projection (qualifier value)", [])},
    "Heparin": {"84812008": ("Heparin", [10])},
    "HepaticArtery": {
        "76015000": (
            "Hepatic artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "HepaticCongestion": {"76281005": ("Hepatic Congestion", [12323])},
    "HepaticFlexureOfColon": {"48338005": ("Hepatic flexure of colon", [6210, 8134])},
    "HepaticLymphNode": {"61492009": ("hepatic lymph node", [7600, 8134])},
    "HepaticVein": {
        "8993003": (
            "Hepatic vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12114,
                12125,
                12282,
            ],
        )
    },
    "Hepatitis": {"128241005": ("Hepatitis", [12325])},
    "HepatitisB": {"66071002": ("Hepatitis B", [12325])},
    "HepatitisC": {"50711007": ("Hepatitis C", [12325])},
    "HepatobiliarySurgery": {"310158005": ("Hepatobiliary Surgery", [7030])},
    "HepatobiliarySurgicalService": {
        "310158005": ("Hepatobiliary surgical service (qualifier value)", [])
    },
    "Hepatomegaly": {"80515008": ("Hepatomegaly", [12325])},
    "HequHorseBreed": {"133096004": ("Hequ horse breed", [7480])},
    "HerdwickSheepBreed": {"131772009": ("Herdwick sheep breed", [7480])},
    "HerefordCattleSuperbreed": {"125074003": ("Hereford cattle superbreed", [7480])},
    "HerefordPig": {"19770007": ("Hereford pig (organism)", [])},
    "HerefordPigBreed": {"19770007": ("Hereford pig breed", [7480])},
    "HerelandCattleBreed": {"133427009": ("Hereland cattle breed", [7480])},
    "HerensCattleBreed": {"133231000": ("Herens cattle breed", [7480])},
    "Heroin": {"387341002": ("Heroin", [6089])},
    "HessenHorseBreed": {"131980007": ("Hessen horse breed", [7480])},
    "HetaoLopEarPigBreed": {"132139002": ("Hetao Lop-Ear pig breed", [7480])},
    "HeterogeneousCalcification": {
        "129763007": ("Heterogeneous calcification", [6010, 6011])
    },
    "HeterogeneousRadiographicCalcification": {
        "129763007": ("Heterogeneous radiographic calcification (finding)", [])
    },
    "HeterogeneouslyDense": {"129718006": ("Heterogeneously dense", [6000, 6001])},
    "HeterogeneouslyDenseBreastComposition": {
        "129718006": ("Heterogeneously dense breast composition (finding)", [])
    },
    "HezuoPigBreed": {"132236004": ("Hezuo pig breed", [7480])},
    "Hickey": {"399277004": ("Hickey", [4012])},
    "HickeyProjection": {"399277004": ("Hickey projection (qualifier value)", [])},
    "High": {"75540009": ("High", [75])},
    "HighConformationWhitePigBreed": {
        "132021009": ("High Conformation White pig breed", [7480])
    },
    "HighDensityLesion": {"129744006": ("High density lesion", [6008, 6009])},
    "HighFrequencyJetVentilation": {
        "243154003": ("High frequency jet ventilation (procedure)", [])
    },
    "HighFrequencyVentilation": {
        "243154003": ("High frequency ventilation", [619, 620])
    },
    "HighPowerNonImmersionLens": {
        "445621001": ("High power non-immersion lens", [8121])
    },
    "HighPowerNonimmersionLensOfOpticalMicroscope": {
        "445621001": (
            "High power nonimmersion lens of optical microscope (physical object)",
            [],
        )
    },
    "HighRightAtrium": {"128591008": ("High right atrium", [3011, 8134])},
    "HighRisk": {"15508007": ("High risk", [3118])},
    "HighRiskOfContextualQualifier": {
        "15508007": ("High risk of (contextual qualifier) (qualifier value)", [])
    },
    "HighRiskTumor": {"258270003": ("High risk tumor", [6063])},
    "HighestMediastinalLymphNode": {
        "127926002": ("highest mediastinal lymph node", [7600, 8134])
    },
    "HighlandPonyHorseBreed": {"133103001": ("Highland Pony horse breed", [7480])},
    "HighlySignificant": {"371926006": ("Highly significant", [220])},
    "Hilar": {"32381004": ("Hilar", [2])},
    "HilarArtery": {
        "397405001": ("Hilar Artery", [3827, 7151, 7192, 8134, 9514, 12103, 12115])
    },
    "HilarLymphNode": {"53074004": ("hilar lymph node", [7600, 8134])},
    "HilarRenalArtery": {"397405001": ("Hilar renal artery (body structure)", [])},
    "HillRadnorSheepBreed": {"131773004": ("Hill Radnor sheep breed", [7480])},
    "HilumOfLung": {
        "46750007": ("Hilum of lung", [6102, 6109, 6111, 6129, 7151, 7192, 8134, 9514])
    },
    "Hindfoot": {"416804009": ("Hindfoot", [7482, 7483, 8134])},
    "HindfootOfQuadruped": {
        "416804009": ("Hindfoot of quadruped (body structure)", [])
    },
    "HinisHorseBreed": {"131981006": ("Hinis horse breed", [7480])},
    "HinterwaldCattleBreed": {"133232007": ("Hinterwald cattle breed", [7480])},
    "Hip": {"29836001": ("Hip", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])},
    "HipJoint": {
        "24136001": (
            "Hip joint",
            [
                4,
                1000,
                1001,
                1005,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "HipJointReconstruction": {"119614000": ("Hip joint reconstruction", [7301])},
    "HipJointStructure": {"24136001": ("Hip joint structure (body structure)", [])},
    "HipRegionStructure": {"29836001": ("Hip region structure (body structure)", [])},
    "HippocampalStructure": {"5366008": ("Hippocampal structure (body structure)", [])},
    "Hippocampus": {"5366008": ("Hippocampus", [7140, 7151, 7153, 7192, 8134, 9514])},
    "HirtzModification": {"424086005": ("Hirtz Modification", [4012])},
    "HirtzModificationProjection": {
        "424086005": ("Hirtz Modification projection (qualifier value)", [])
    },
    "HirzaiHorseBreed": {"131982004": ("Hirzai horse breed", [7480])},
    "Hispanic": {"414408004": ("Hispanic", [6099])},
    "HissarCattleBreed": {"133663000": ("Hissar cattle breed", [7480])},
    "HistologicGrade": {"371469007": ("Histologic grade (observable entity)", [])},
    "HistologicTest": {"714797009": ("Histologic test (procedure)", [])},
    "HistologicalGradeFinding": {
        "373372005": ("Histological grade finding (finding)", [])
    },
    "Histology": {"714797009": ("Histology", [8131])},
    "HistomineReceptorAntagonist": {"372806008": ("Antihistamine", [621, 622])},
    "Histopathology": {"252416005": ("Histopathology", [8131])},
    "HistopathologyDepartment": {"441950002": ("Histopathology", [7030])},
    "HistopathologyTest": {"252416005": ("Histopathology test (procedure)", [])},
    "HistoryOfBetaAdrenergicReceptorBlockingAgentTherapy": {
        "440935004": (
            "History of beta adrenergic receptor blocking agent therapy (situation)",
            [],
        )
    },
    "HistoryOfBetaBlockingAgentsTherapy": {
        "440935004": ("History of Beta-blocking agents therapy", [64])
    },
    "HistoryOfCABG": {"399261000": ("History of CABG", [3201])},
    "HistoryOfCardiovascularDisease": {
        "266995000": ("History of - cardiovascular disease", [6087, 6088])
    },
    "HistoryOfCerebrovascularDisease": {
        "308064009": ("History of cerebrovascular disease", [3756])
    },
    "HistoryOfChronicLungDisease": {
        "414415007": ("History of chronic lung disease", [3756])
    },
    "HistoryOfCongestiveHeartFailure": {
        "161505003": ("History of congestive heart failure", [64, 3756])
    },
    "HistoryOfCoronaryArteryBypassGrafting": {
        "399261000": ("History of coronary artery bypass grafting (situation)", [])
    },
    "HistoryOfDiabetes": {"161445009": ("History of Diabetes", [3756])},
    "HistoryOfDiabetesMellitus": {
        "161445009": ("History of diabetes mellitus", [64, 6087, 6088])
    },
    "HistoryOfEclampsia": {"161806007": ("History of - eclampsia", [6087, 6088])},
    "HistoryOfEctopicPregnancy": {
        "161763005": ("History of - ectopic pregnancy", [6087, 6088])
    },
    "HistoryOfFemaleInfertility": {
        "161798008": ("History of female infertility (situation)", [])
    },
    "HistoryOfHeartFailure": {
        "161505003": ("History of heart failure (situation)", [])
    },
    "HistoryOfHypercholesterolemia": {
        "414416008": ("History of hypercholesterolemia", [3756])
    },
    "HistoryOfHypertension": {
        "161501007": ("History of Hypertension", [64, 3756, 6087, 6088])
    },
    "HistoryOfInfertility": {"161798008": ("History of infertility", [6087, 6088])},
    "HistoryOfKidneyDisease": {
        "275552000": ("History of kidney disease (situation)", [])
    },
    "HistoryOfLowerLimbAmputation": {
        "161622006": ("History of lower limb amputation (situation)", [])
    },
    "HistoryOfMalignantBasalCellNeoplasmOfSkin": {
        "428053000": ("History of malignant basal cell neoplasm of skin", [4404])
    },
    "HistoryOfMalignantMelanoma": {
        "161432005": ("History of malignant melanoma", [4402])
    },
    "HistoryOfMalignantMelanomaOfTheSkin": {
        "321000119108": ("History of malignant melanoma of the skin", [4402])
    },
    "HistoryOfMalignantNeoplasmOfSkinExcludingMelanoma": {
        "443895001": (
            "History of malignant neoplasm of skin excluding melanoma",
            [4404],
        )
    },
    "HistoryOfMelanomaInSituOfSkin": {
        "1251000119106": ("History of melanoma in situ of skin (situation)", [])
    },
    "HistoryOfMelanomaInSituOfTheSkin": {
        "1251000119106": ("History of melanoma in situ of the skin", [4403])
    },
    "HistoryOfMyocardialInfarction": {
        "399211009": ("History of Myocardial Infarction", [3201, 3756])
    },
    "HistoryOfObesity": {"161453001": ("History of - obesity", [6087, 6088])},
    "HistoryOfPrematureDelivery": {
        "161765003": ("History of - premature delivery", [6087, 6088])
    },
    "HistoryOfRadiationTherapy": {
        "429479009": ("History of radiation therapy", [6051])
    },
    "HistoryOfRegularMedication": {
        "161656000": ("History of - regular medication", [6087, 6088])
    },
    "HistoryOfRenalFailure": {"414417004": ("History of renal failure", [64, 3756])},
    "HistoryOfSeverePreEclampsia": {
        "161807003": ("History of - severe pre-eclampsia", [6087, 6088])
    },
    "HistoryOfSquamousCellCarcinomaOfSkin": {
        "429024007": ("History of squamous cell carcinoma of skin", [4404])
    },
    "HistoryOfSubstanceAbuse": {
        "371422002": ("History of substance abuse", [6087, 6088])
    },
    "HodgkinLymphoma": {"1163005009": ("Hodgkin's lymphoma", [6030, 6033])},
    "HodgkinLymphomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "14537002": (
            "Hodgkin lymphoma, no International Classification of Diseases for Oncology subtype (morphologic "
            "abnormality)",
            [],
        )
    },
    "HoffmanModulator": {"445635004": ("Hoffman modulator", [8124])},
    "HoffmanModulatorOfOpticalMicroscope": {
        "445635004": ("Hoffman modulator of optical microscope (physical object)", [])
    },
    "HogIslandSheepBreed": {"131787004": ("Hog Island Sheep breed", [7480])},
    "HokkaidoHorseBreed": {"133102006": ("Hokkaido horse breed", [7480])},
    "HolgusCattleBreed": {"133428004": ("Holgus cattle breed", [7480])},
    "HollandShepherdDogBreed": {"132428005": ("Holland Shepherd dog breed", [7480])},
    "Holly": {"399129007": ("Holly", [4012])},
    "HollyProjection": {"399129007": ("Holly projection (qualifier value)", [])},
    "Holmblad": {"399285008": ("Holmblad", [4012])},
    "HolmbladProjection": {"399285008": ("Holmblad projection (qualifier value)", [])},
    "HolmongerCattleBreed": {"131476008": ("Holmonger cattle breed", [7480])},
    "HolsteinFriesianCattleBreed": {
        "26105007": ("Holstein-Friesian cattle breed (organism)", [])
    },
    "HolsteinFriesianCowBreed": {"26105007": ("Holstein-Friesian cow breed", [7480])},
    "HolsteinerHorse": {"25813002": ("Holsteiner horse (organism)", [])},
    "HolsteinerHorseBreed": {"25813002": ("Holsteiner horse breed", [7480])},
    "Homatropine": {"82264009": ("Homatropine", [4208])},
    "HomoSapiens": {"337915000": ("Homo sapiens", [7454])},
    "HomoSapiensLivingOrganism": {
        "30996001": ("Homo sapiens (living organism) (organism)", [])
    },
    "HondurasSwitchTailPigBreed": {
        "132066000": ("Honduras Switch-Tail pig breed", [7480])
    },
    "Horizontal": {"24020000": ("Horizontal", [2, 6121])},
    "HorizontalLongAxis": {"131186000": ("Horizontal Long Axis", [26, 27, 501])},
    "HormelMiniaturePig": {"86694007": ("Hormel miniature pig (organism)", [])},
    "HormelMiniaturePigBreed": {"86694007": ("Hormel miniature pig breed", [7480])},
    "HormoneTherapy": {"169413002": ("Hormone therapy", [6083, 6085])},
    "HornDorsetSheepBreed": {"86920006": ("Horn dorset sheep breed", [7480])},
    "HornedDorsetSheepBreed": {
        "86920006": ("Horned dorset sheep breed (organism)", [])
    },
    "HornedHereford": {"7843000": ("Horned Hereford (organism)", [])},
    "HornedHerefordCowBreed": {"7843000": ("Horned Hereford cow breed", [7480])},
    "HorroCattleBreed": {"131433006": ("Horro cattle breed", [7480])},
    "HospitalAdmissionElective": {"8715000": ("Elective", [3729])},
    "HospitalAdmissionTransferFromOtherHospitalOrHealthCareFacility": {
        "4563007": (
            "Hospital admission, transfer from other hospital or health care facility (procedure)",
            [],
        )
    },
    "HospitalConsultant": {"309390008": ("Hospital consultant (occupation)", [])},
    "HospitalIsolationRoom": {
        "409688003": ("Hospital isolation room (environment)", [])
    },
    "HospitalRegistrar": {"158971006": ("Hospital registrar (occupation)", [])},
    "Hough": {"399168000": ("Hough", [4012])},
    "HoughProjection": {"399168000": ("Hough projection (qualifier value)", [])},
    "Hound": {"25097001": ("Hound", [7480])},
    "HovawartDogBreed": {"132500001": ("Hovawart dog breed", [7480])},
    "HrbineckyCattleBreed": {"133429007": ("Hrbinecky cattle breed", [7480])},
    "Hsieh": {"399083008": ("Hsieh", [4012])},
    "HsiehProjection": {"399083008": ("Hsieh projection (qualifier value)", [])},
    "HuaiPigBreed": {"132135008": ("Huai pig breed", [7480])},
    "HuangHuaiHaiBlackPigBreed": {
        "132131004": ("Huang-Huai-Hai Black pig breed", [7480])
    },
    "HuangHuaiHaiBlackShenxianPigBreed": {
        "132130003": ("Huang-Huai-Hai Black, Shenxian pig breed", [7480])
    },
    "HuazhongTwoEndBlackJianliPigBreed": {
        "132297005": ("Huazhong Two-End Black, Jianli pig breed", [7480])
    },
    "HuazhongTwoEndBlackPigBreed": {
        "132296001": ("Huazhong Two-End Black pig breed", [7480])
    },
    "HuazhongTwoEndBlackSatzelingPigBreed": {
        "132299008": ("Huazhong Two-End Black, Satzeling pig breed", [7480])
    },
    "HuazhongTwoEndBlackTongchengPigBreed": {
        "132298000": ("Huazhong Two-End Black, Tongcheng pig breed", [7480])
    },
    "HubeiWhitePigBreed": {"132323002": ("Hubei White pig breed", [7480])},
    "HuchuanMountainPigBreed": {"132149004": ("Huchuan Mountain pig breed", [7480])},
    "HuculHorseBreed": {"131861006": ("Hucul horse breed", [7480])},
    "HuertanaCattleBreed": {"133436008": ("Huertana cattle breed", [7480])},
    "Hughston": {"399003005": ("Hughston", [4012])},
    "HughstonProjection": {"399003005": ("Hughston projection (qualifier value)", [])},
    "HumanFibrinogen": {"418326009": ("Human fibrinogen", [10])},
    "HumanImmunodeficiencyVirusPositive": {
        "165816005": ("Human immunodeficiency virus positive (finding)", [])
    },
    "Humerus": {
        "85050009": (
            "Humerus",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                6102,
                6109,
                6114,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "HungahybPigBreed": {"132364007": ("Hungahyb pig breed", [7480])},
    "HungairanColdbloodHorseBreed": {
        "131983009": ("Hungairan Coldblood horse breed", [7480])
    },
    "HungarianColdbloodHorseBreed": {
        "131983009": ("Hungarian Coldblood horse breed (organism)", [])
    },
    "HungarianDunHorseBreed": {"131984003": ("Hungarian Dun horse breed", [7480])},
    "HungarianGrayCattleBreed": {"133233002": ("Hungarian Gray cattle breed", [7480])},
    "HungarianPiedCattleBreed": {"133437004": ("Hungarian Pied cattle breed", [7480])},
    "HungarianSportHorseHorseBreed": {
        "131985002": ("Hungarian Sport Horse horse breed", [7480])
    },
    "HungarianWhitePigBreed": {"132363001": ("Hungarian White pig breed", [7480])},
    "HungarofriesCattleBreed": {"133438009": ("Hungarofries cattle breed", [7480])},
    "HunterHorse": {"19356005": ("Hunter horse (organism)", [])},
    "HunterHorseBreed": {"19356005": ("Hunter horse breed", [7480])},
    "HunterianPerforatingVein": {
        "128560002": (
            "Hunterian perforating vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "HydrocortisoneSodiumSuccinate": {
        "109066000": ("Hydrocortisone sodium succinate", [65])
    },
    "HydroxymethylglutarylCoenzymeAReductaseInhibitor": {
        "96302009": (
            "Hydroxymethylglutaryl-coenzyme A reductase inhibitor (product)",
            [],
        )
    },
    "HyoidBone": {"21387005": ("Hyoid bone", [4028, 7151, 7192, 8134, 9514])},
    "HyoidBoneStructure": {"21387005": ("Hyoid bone structure (body structure)", [])},
    "HyperLargeWhitePigBreed": {"132089008": ("Hyper Large White pig breed", [7480])},
    "Hyperkinesis": {"371855002": ("Hyperkinesis", [3706])},
    "HyperkinesisOfRegionOfCardiacWall": {
        "373123005": ("Hyperkinesis of region of cardiac wall (finding)", [])
    },
    "HyperkineticRegion": {"373123005": ("Hyperkinetic region", [3703])},
    "HyperkineticVentricularWall": {
        "371855002": ("Hyperkinetic ventricular wall (finding)", [])
    },
    "Hypermetropia": {"38101003": ("Hypermetropia (disorder)", [])},
    "Hyperopia": {"38101003": ("Hyperopia", [4238])},
    "Hyperplasia": {"76197007": ("hyperplasia", [3802, 3815])},
    "HyperplasiaUsual": {"76197007": ("Hyperplasia, usual", [6030, 6031])},
    "Hypertension": {"38341003": ("Hypertension", [60, 3201, 9300, 12246])},
    "Hypertensive": {"38341003": ("Hypertensive", [3211])},
    "HypertensiveDisorderSystemicArterial": {
        "38341003": ("Hypertensive disorder, systemic arterial (disorder)", [])
    },
    "HypertensiveEpisode": {"443482000": ("Hypertensive episode", [3221])},
    "HypertensiveHeartDisease": {
        "64715009": ("Hypertensive heart disease", [3700, 3728])
    },
    "HypertensiveUrgency": {"443482000": ("Hypertensive urgency (disorder)", [])},
    "HypertrophicCardiomyopathyWithObstruction": {
        "45227007": ("Hypertrophic cardiomyopathy with obstruction", [3728])
    },
    "HypertrophicCardiomyopathyWithoutObstruction": {
        "195020003": ("Hypertrophic cardiomyopathy without obstruction", [3728])
    },
    "HypertrophicObstructiveCardiomyopathy": {
        "45227007": ("Hypertrophic obstructive cardiomyopathy", [3700])
    },
    "Hyperventilation": {"68978004": ("Hyperventilation", [3038, 3207, 12002])},
    "HypochondriacRegion": {"19695001": ("Subcostal", [4, 7151, 7192, 8134, 9514])},
    "HypogastricLymphNode": {"69255009": ("hypogastric lymph node", [7600, 8134])},
    "HypogastricRegion": {
        "11708003": ("Hypogastric region", [4, 7151, 7192, 8134, 9514])
    },
    "HypogastricRegionStructure": {
        "11708003": ("Hypogastric region structure (body structure)", [])
    },
    "HypoglossalNerve": {"37899009": ("hypoglossal nerve", [7706, 7710, 8134])},
    "HypoglossalNerveStructure": {
        "37899009": ("Hypoglossal nerve structure (body structure)", [])
    },
    "Hypoglycemic": {"312064005": ("Hypoglycemic", [621, 622])},
    "HypoglycemicAgent": {"312064005": ("Hypoglycemic agent (product)", [])},
    "Hypokinesis": {"37706002": ("Hypokinesis", [3703, 3706])},
    "HypokinesisOfCardiacWall": {
        "37706002": ("Hypokinesis of cardiac wall (finding)", [])
    },
    "HypopharyngealStructure": {
        "81502006": ("Hypopharyngeal structure (body structure)", [])
    },
    "Hypopharynx": {"81502006": ("Hypopharynx", [4, 7151, 7192, 7601, 8134, 9514])},
    "Hypoplasia": {"55199003": ("hypoplasia", [3813])},
    "HypoplasticLeftHeartSyndrome": {
        "62067003": ("Hypoplastic left heart syndrome", [12248, 12249])
    },
    "Hypotension": {"45007003": ("Hypotension", [60, 9300])},
    "Hypotensive": {"45007003": ("Hypotensive", [3211])},
    "HypotensiveAgent": {"1182007": ("Hypotensive agent", [10])},
    "HypotensiveEpisode": {"67763001": ("Hypotensive episode", [3221])},
    "Hysterectomy": {"236886002": ("Hysterectomy", [6082])},
    "IDPlate": {"129467007": ("ID Plate", [6040, 6401, 7151, 7193])},
    "ILMInternalLimitingMembrane": {
        "280677004": ("ILM - Internal limiting membrane", [4273, 7151, 7192, 9514])
    },
    "INRA95CattleBreed": {"133440004": ("INRA 95 cattle breed", [7480])},
    "IbageCattleBreed": {"133807003": ("Ibage cattle breed (organism)", [])},
    "IbageXZebuCattleBreed": {"133807003": ("Ibage X zebu cattle breed", [7480])},
    "IbanPigBreed": {"132844006": ("Iban pig breed", [7480])},
    "IberianBlackIberianPigBreed": {
        "132082004": ("Iberian, Black Iberian pig breed", [7480])
    },
    "IberianExtremaduraRedPigBreed": {
        "132080007": ("Iberian, Extremadura Red pig breed", [7480])
    },
    "IberianJabugoSpottedPigBreed": {
        "132081006": ("Iberian, Jabugo Spotted pig breed", [7480])
    },
    "IberianPigBreed": {"132079009": ("Iberian pig breed", [7480])},
    "IbizanHound": {"10842007": ("Ibizan hound", [7480])},
    "IcelandicCattleBreed": {"133234008": ("Icelandic cattle breed", [7480])},
    "IcelandicHorse": {"70457009": ("Icelandic horse (organism)", [])},
    "IcelandicHorseBreed": {"70457009": ("Icelandic horse breed", [7480])},
    "IcelandicPigBreed": {"132254000": ("Icelandic pig breed", [7480])},
    "IcelandicSheepBreed": {"131774005": ("Icelandic sheep breed", [7480])},
    "IcelandicSheepdogBreed": {"132501002": ("Icelandic Sheepdog breed", [7480])},
    "IdentificationPlateDevice": {
        "129467007": ("Identification plate, device (physical object)", [])
    },
    "IdiopathicHypertrophicSubaorticStenosis": {
        "360465008": ("Idiopathic hypertrophic subaortic stenosis (disorder)", [])
    },
    "IleDeFranceSheepBreed": {"131761004": ("Ile-de-France sheep breed", [7480])},
    "IlealStructure": {"34516001": ("Ileal structure (body structure)", [])},
    "IlealVein": {
        "397439002": ("Ileal vein", [3827, 7151, 7192, 8134, 9514, 12103, 12113, 12125])
    },
    "IlealVeinStructure": {"397439002": ("Ileal vein structure (body structure)", [])},
    "IleocecalValve": {"23153004": ("Ileocecal valve", [6201])},
    "IleocecalValveStructure": {
        "23153004": ("Ileocecal valve structure (body structure)", [])
    },
    "IleocolicLymphNode": {"281676003": ("ileocolic lymph node", [7600, 8134])},
    "IleostomyBag": {"417136005": ("Ileostomy bag", [6202, 6203])},
    "IleostomySet": {"342706005": ("Ileostomy set (physical object)", [])},
    "Ileum": {
        "34516001": ("Ileum", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "IliacAndOrFemoralArtery": {
        "299716001": ("Iliac and/or femoral artery", [4, 4030, 7151, 7192, 8134, 9514])
    },
    "IliacAndOrFemoralArteryStructures": {
        "299716001": ("Iliac and/or femoral artery structures (body structure)", [])
    },
    "IliacArtery": {
        "10293006": (
            "Iliac artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "IliacCrest": {"29850006": ("Iliac Crest", [1000, 1001, 1005, 8134])},
    "IliacCrestStructure": {"29850006": ("Iliac crest structure (body structure)", [])},
    "IliacLymphNode": {"84219008": ("iliac lymph node", [7600, 8134])},
    "IliacVein": {"244411005": ("Iliac vein", [4, 3010, 4042, 7151, 7192, 8134, 9514])},
    "IliacVeinStructure": {"244411005": ("Iliac vein structure (body structure)", [])},
    "IliocostalisMuscle": {
        "57651003": ("Iliocostalis muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "Ilium": {
        "22356005": ("Ilium", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "IllawarraCattleBreed": {"133235009": ("Illawarra cattle breed", [7480])},
    "IlocosCattleBreed": {"131477004": ("Ilocos cattle breed", [7480])},
    "IloiloCattleBreed": {"131478009": ("Iloilo cattle breed", [7480])},
    "ImageAnalysis": {"24587005": ("Image analysis", [3215])},
    "ImageMode": {"399264008": ("Image mode (observable entity)", [])},
    "ImageQualityAnalysis": {"133887000": ("Image quality analysis", [6043, 6137])},
    "Imaging": {"363679005": ("Imaging (procedure)", [])},
    "ImagingFinding": {"365853002": ("Imaging finding (finding)", [])},
    "ImagingGuidance": {"413815006": ("Imaging guidance", [12246])},
    "ImagingProcedure": {"363679005": ("Imaging procedure", [100, 634])},
    "ImagingResultAbnormal": {"408574004": ("Imaging result abnormal", [3209])},
    "ImagingResultEquivocal": {"408379005": ("Imaging result equivocal", [3209])},
    "ImagingResultNormal": {"408573005": ("Imaging result normal", [3209])},
    "ImmobileHeartValve": {"373137001": ("Immobile Heart Valve", [3711])},
    "ImmunofluorescentStain": {"255800009": ("immunofluorescent stain", [4412, 8112])},
    "Immunohistochemistry": {"117617002": ("Immunohistochemistry", [8131])},
    "ImmunohistochemistryProcedure": {
        "117617002": ("Immunohistochemistry procedure (procedure)", [])
    },
    "Immunomodulator": {"372558009": ("Immunomodulator", [4410])},
    "ImpairedLeftVentricularFunction": {
        "275514001": ("Impaired left ventricular function", [3119])
    },
    "ImpairedRenalFunction": {"236423003": ("Impaired Renal Function", [1200, 1201])},
    "Implant": {"40388003": ("Implant", [6040, 6054, 6401, 7151, 7193])},
    "ImplantDevice": {"40388003": ("Implant, device", [1000])},
    "ImplantDisplaced": {"399209000": ("Implant Displaced", [4015])},
    "ImplantRevisedSincePreviousMammogram": {
        "129731001": ("Implant revised since previous mammogram", [6002, 6003])
    },
    "ImplantableDefibrillator": {"72506001": ("Implantable defibrillator", [3777])},
    "ImplantableDefibrillatorDevice": {
        "72506001": ("Implantable defibrillator, device (physical object)", [])
    },
    "ImplantableVenousAccessPort": {
        "398013009": ("Implantable venous access port", [74])
    },
    "ImplantationOfAutomaticCardiacDefibrillator": {
        "233170003": ("Implantation of automatic cardiac defibrillator", [3721])
    },
    "ImplantationOfBaffleAtrialOrInteratrial": {
        "245544005": ("Implantation of baffle, atrial or interatrial (procedure)", [])
    },
    "ImplantationOfCardiacPacemaker": {
        "307280005": ("Implantation of cardiac pacemaker", [3721])
    },
    "ImplantationOfHeartValveProsthesisOrSyntheticDevice": {
        "47432005": (
            "Implantation of heart valve prosthesis or synthetic device",
            [12247],
        )
    },
    "ImplantationOfHeartValveWithTissueGraft": {
        "37153009": ("Implantation of heart valve with tissue graft", [12247])
    },
    "ImplantationOfRadioactiveSeedIntoProstate": {
        "425767004": ("Implantation of radioactive seed into prostate", [6323])
    },
    "ImprovedRodopiCattleBreed": {
        "133439001": ("Improved Rodopi cattle breed", [7480])
    },
    "ImprovementOfLeftVentricularWallMotionComparedToPriorStudy": {
        "428825004": (
            "Improvement of left ventricular wall motion compared to prior study (finding)",
            [],
        )
    },
    "ImprovementOfWallMotion": {"428825004": ("Improvement of wall motion", [3237])},
    "InProgress": {"385651009": ("In progress", [3773])},
    "InRemission": {"765205004": ("In remission", [3772])},
    "InactiveProblem": {"394775005": ("Inactive problem", [3770])},
    "IncaHairlessDogBreed": {"132502009": ("Inca Hairless Dog breed", [7480])},
    "IncisionOfTrachea": {"48387007": ("Incision of trachea (procedure)", [])},
    "IncompetenceOfAnyValvularStructure": {
        "10337008": ("Incompetence of any valvular structure (finding)", [])
    },
    "IncompleteArterialStentAppositionAtTimeOfPlacement": {
        "408709008": (
            "Incomplete arterial stent apposition at time of placement (finding)",
            [],
        )
    },
    "IncompleteArterialStentAppositionSubsequentToPlacement": {
        "408710003": (
            "Incomplete arterial stent apposition subsequent to placement (finding)",
            [],
        )
    },
    "IncompleteLeftBundleBranchBlock": {
        "251120003": ("Incomplete Left bundle branch block", [3230])
    },
    "IncompleteRightBundleBranchBlock": {
        "251124007": ("Incomplete Right bundle branch block", [3230])
    },
    "IncompleteStentApposition": {"408709008": ("Incomplete Stent apposition", [3494])},
    "IncreaseInNumberOfCalcifications": {
        "129726003": ("Increase in number of calcifications", [6002, 6003])
    },
    "IncreaseInNumberOfCalcificationsSincePreviousMammogram": {
        "129726003": (
            "Increase in number of calcifications since previous mammogram (finding)",
            [],
        )
    },
    "IncreaseInSize": {"15454001": ("Increase in size", [6002, 6003, 6134])},
    "IncreasedLungTracerUptake": {
        "428920008": ("Increased lung tracer uptake (finding)", [])
    },
    "IncreasedLungUptake": {"428920008": ("Increased lung uptake", [3116])},
    "IncreasedMyocardialIschemiaComparedToPriorStudy": {
        "429477006": (
            "Increased myocardial ischemia compared to prior study (finding)",
            [],
        )
    },
    "IncreasedSize": {"15454001": ("Increased size (finding)", [])},
    "IncreasedTolerance": {"102459008": ("Increased tolerance", [3236])},
    "Indeterminate": {"82334004": ("Indeterminate", [4220])},
    "IndeterminateResult": {"280416009": ("Indeterminate result", [6063])},
    "IndiaInkStain": {"406972004": ("India ink stain", [8112])},
    "Indian": {"414481008": ("Indian (racial group)", [])},
    "IndianRace": {"414481008": ("Indian race", [6099])},
    "IndicationForProcedure": {
        "432678004": ("Indication for procedure (observable entity)", [])
    },
    "IndigoCarmine": {"412548006": ("indigo carmine", [8112])},
    "IndigoCarmineStain": {"45475000": ("Indigo carmine stain (substance)", [])},
    "IndigotindisulfonateSodium": {
        "412548006": ("Indigotindisulfonate sodium (substance)", [])
    },
    "IndirectIrisTransillumination": {
        "410467002": ("Indirect iris transillumination", [4203])
    },
    "IndirectIrisTransilluminationAction": {
        "410467002": ("Indirect iris transillumination - action (qualifier value)", [])
    },
    "IndirectOphthalmoscope": {"409901008": ("Indirect Ophthalmoscope", [4202])},
    "IndirectOphthalmoscopyLens": {
        "409897002": ("Indirect ophthalmoscopy lens", [4205])
    },
    "IndirectRetroilluminationFromTheIris": {
        "410465005": ("Indirect retroillumination from the iris", [4203])
    },
    "IndirectRetroilluminationFromTheIrisAction": {
        "410465005": (
            "Indirect retroillumination from the iris - action (qualifier value)",
            [],
        )
    },
    "IndirectRetroilluminationFromTheRetina": {
        "410466006": ("Indirect retroillumination from the retina", [4203])
    },
    "IndirectRetroilluminationFromTheRetinaAction": {
        "410466006": (
            "Indirect retroillumination from the retina - action (qualifier value)",
            [],
        )
    },
    "IndirectScleroticScatterIllumination": {
        "410464009": ("Indirect sclerotic scatter illumination", [4203])
    },
    "IndirectScleroticScatterIlluminationAction": {
        "410464009": (
            "Indirect sclerotic scatter illumination - action (qualifier value)",
            [],
        )
    },
    "IndistinctLesion": {
        "129741003": ("Indistinct lesion", [6006, 6007, 6335, 6341, 6342])
    },
    "Indium111CapromabPendetide": {
        "446871009": ("Indium^111 Capromab Pendetide", [25])
    },
    "Indium111Chloride": {"395742005": ("Indium^111 Chloride", [25])},
    "Indium111FeOHGreaterThan3LesserThan": {
        "6516008": ("Indium^111^-Fe(OH) >3<", [25])
    },
    "Indium111Oxyquinoline": {"424570009": ("Indium^111^oxyquinoline", [25])},
    "Indium111Pentetate": {"29218008": ("Indium^111^ pentetate", [25])},
    "Indium111Pentetreotide": {"446800006": ("Indium^111 Pentetreotide", [25])},
    "Indium111RedCellLabel": {"81621007": ("Indium^111^ red cell label", [25])},
    "Indium111Transferrin": {"78570003": ("Indium^111^ transferrin", [25])},
    "Indium111inOxyquinoline": {
        "424570009": ("Indium[111In]oxyquinoline (product)", [])
    },
    "Indium113Bleomycin": {"90617008": ("Indium^113^ bleomycin (substance)", [])},
    "Indium113Pentetate": {"42728008": ("Indium^113^ pentetate (substance)", [])},
    "Indium113mBleomycin": {"90617008": ("Indium^113m^ bleomycin", [25])},
    "Indium113mChloride": {"767418009": ("Indium^113m^ chloride", [25])},
    "Indium113mPentetate": {"42728008": ("Indium^113m^ pentetate", [25])},
    "IndiumChloride111in": {"395742005": ("Indium chloride[111In] (substance)", [])},
    "IndividualCalcification": {
        "129770007": ("Individual Calcification", [6014, 6015, 6016, 6054, 6056])
    },
    "IndoBrazilianCattleBreed": {"133609003": ("Indo-Brazilian cattle breed", [7480])},
    "IndocyanineGreen": {"7292004": ("Indocyanine green", [4200])},
    "IndophenolFromNaphtholStain": {
        "11727009": ("indophenol from naphthol stain", [8112])
    },
    "InductionOfGeneralAnesthesia": {
        "241687005": ("Induction of general anesthesia (procedure)", [])
    },
    "InductionRoom": {"414485004": ("Induction room", [603])},
    "InfarctionOfBreast": {"77296004": ("Infarction of breast", [6030, 6031])},
    "InfectiousDisease": {"309934006": ("Infectious Disease", [7030])},
    "InfectiousDiseasesDepartment": {
        "309934006": ("Infectious diseases department (environment)", [])
    },
    "Inferior": {
        "261089000": ("Inferior", [2, 5, 211, 212, 3019, 4005, 6065, 12117, 12281])
    },
    "InferiorArticularFacetOfAxis": {
        "181901007": ("Inferior articular facet of axis", [6115, 8134])
    },
    "InferiorArticularProcessOfVertebra": {
        "317766009": ("Inferior articular process of vertebra", [6115, 8134])
    },
    "InferiorAuricularLymphNode": {
        "155237005": ("inferior auricular lymph node", [7600, 8134])
    },
    "InferiorCardiacVein": {
        "195416006": ("Inferior cardiac vein", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "InferiorCerebellarPeduncle": {
        "67701001": (
            "Inferior cerebellar peduncle",
            [7151, 7153, 7192, 7701, 7710, 8134, 9514],
        )
    },
    "InferiorCerebellarPeduncleStructure": {
        "67701001": ("Inferior cerebellar peduncle structure (body structure)", [])
    },
    "InferiorFrontoOccipitalFasciculus": {
        "35664009": ("inferior fronto-occipital fasciculus", [7703, 7710, 8134])
    },
    "InferiorGlutealLymphNode": {
        "40684008": ("inferior gluteal lymph node", [7600, 8134])
    },
    "InferiorHornOfLateralVentricle": {
        "53118009": ("Inferior Horn of Lateral Ventricle", [7140])
    },
    "InferiorInguinalLymphNode": {
        "85380009": ("inferior inguinal lymph node", [7600, 8134])
    },
    "InferiorLeftPulmonaryVein": {
        "51249003": (
            "Inferior left pulmonary vein",
            [4, 3010, 3011, 3827, 3840, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "InferiorLongitudinalFasciculus": {
        "55233005": (
            "Inferior longitudinal fasciculus",
            [7151, 7153, 7192, 7703, 7710, 8134, 9514],
        )
    },
    "InferiorMesentericArtery": {
        "33795007": (
            "Inferior mesenteric artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "InferiorMesentericLymphNode": {
        "113336002": ("inferior mesenteric lymph node", [7600, 8134])
    },
    "InferiorMesentericVein": {
        "32859001": (
            "Inferior Mesenteric Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "InferiorPancreaticLymphNode": {
        "280915003": ("inferior pancreatic lymph node", [7600, 8134])
    },
    "InferiorPancreaticoduodenalLymphNode": {
        "281227003": ("inferior pancreaticoduodenal lymph node", [7600, 8134])
    },
    "InferiorPhrenicArtery": {
        "29660000": (
            "Inferior phrenic artery",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "InferiorRightHepaticVein": {
        "397425002": (
            "Inferior Right Hepatic Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "InferiorRightHepaticVeinStructure": {
        "397425002": ("Inferior right hepatic vein structure (body structure)", [])
    },
    "InferiorRightPulmonaryVein": {
        "113273001": (
            "Inferior right pulmonary vein",
            [4, 3010, 3011, 3827, 3840, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "InferiorRollingOfBreast": {
        "414493004": ("Inferior rolling of breast (procedure)", [])
    },
    "InferiorVenaCava": {
        "64131007": (
            "Inferior vena cava",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                6202,
                6204,
                7151,
                7154,
                7192,
                8134,
                9514,
                12103,
                12114,
                12125,
                12282,
                12305,
            ],
        )
    },
    "InferiorVenaCavaStructure": {
        "64131007": ("Inferior vena cava structure (body structure)", [])
    },
    "InferomedialToSuperolateralOblique": {
        "441555000": ("inferomedial to superolateral oblique", [4014])
    },
    "InferomedialToSuperolateralObliqueView": {
        "441555000": (
            "Inferomedial to superolateral oblique view (qualifier value)",
            [],
        )
    },
    "InfiltratingDuctCarcinoma": {
        "82711006": ("Infiltrating duct carcinoma", [6030, 6033])
    },
    "InfiltratingDuctCarcinomaOfBreast": {
        "408643008": ("Infiltrating duct carcinoma of breast (disorder)", [])
    },
    "InfiltratingDuctalCarcinomaOfBreast": {
        "408643008": ("Infiltrating ductal carcinoma of breast", [638, 639])
    },
    "Inflammation": {"409774005": ("Inflammation", [6030, 6031, 7151, 7159, 7194])},
    "InflammationMorphologic": {
        "23583003": ("Inflammation (morphologic abnormality)", [])
    },
    "InflammatoryAbdominalAorticAneurysm": {
        "314186008": ("inflammatory abdominal aortic aneurysm", [3808])
    },
    "InflammatoryCarcinoma": {"32968003": ("Inflammatory carcinoma", [6030, 6033])},
    "InflammatoryDiseaseOfLiver": {
        "128241005": ("Inflammatory disease of liver (disorder)", [])
    },
    "InflammatoryMorphology": {
        "409774005": ("Inflammatory morphology (morphologic abnormality)", [])
    },
    "InfraMammaryFold": {"442593008": ("Infra-mammary fold", [4015])},
    "InfraMammaryFoldMammographyView": {
        "442593008": ("Infra-mammary fold mammography view (qualifier value)", [])
    },
    "InfraRedColor": {"414497003": ("Infra-red color (qualifier value)", [])},
    "InfraRenalAorta": {
        "28205006": (
            "Infra-renal Aorta",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125, 12291],
        )
    },
    "InfraclavicularLymphNode": {
        "9659009": ("infraclavicular lymph node", [7600, 8134])
    },
    "Infrared": {"414497003": ("Infrared", [4206, 8122])},
    "InfraredOpticalFilter": {"445169002": ("Infrared optical filter", [4204, 8124])},
    "InfraredOpticalFilterOfOpticalMicroscope": {
        "445169002": (
            "Infrared optical filter of optical microscope (physical object)",
            [],
        )
    },
    "InfraspinatusMuscle": {
        "72573008": ("Infraspinatus muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "InfraspinatusMuscleStructure": {
        "72573008": ("Infraspinatus muscle structure (body structure)", [])
    },
    "Infrequent": {"27789000": ("Infrequent", [6091])},
    "IngessanaCattleBreed": {"133664006": ("Ingessana cattle breed", [7480])},
    "InguinalLymphNode": {"8928004": ("inguinal lymph node", [7600, 8134])},
    "InguinalLymphNodeStructure": {
        "8928004": ("Inguinal lymph node structure (body structure)", [])
    },
    "InguinalRegion": {
        "26893007": ("Inguinal region", [4, 4040, 7151, 7192, 8134, 9514])
    },
    "InguinalRegionStructure": {
        "26893007": ("Inguinal region structure (body structure)", [])
    },
    "InhalationAnesthesiaMachineSystemClosedNoRebreathingOfPrimaryAgent": {
        "44812007": (
            "Inhalation anesthesia, machine system, closed, no rebreathing of primary agent (procedure)",
            [],
        )
    },
    "InhalationAnesthesiaMachineSystemClosedRebreathingOfPrimaryAgent": {
        "112987001": (
            "Inhalation anesthesia, machine system, closed, rebreathing of primary agent (procedure)",
            [],
        )
    },
    "InhalationAnesthesiaSystemClosedNoRebreathingPrimaryAgent": {
        "44812007": (
            "Inhalation anesthesia system closed no rebreathing primary agent",
            [615, 616],
        )
    },
    "InhalationAnesthesiaSystemClosedRebreathingPrimaryAgent": {
        "112987001": (
            "Inhalation anesthesia system closed rebreathing primary agent",
            [615, 616],
        )
    },
    "InhalationTechnique": {
        "446406008": ("Inhalation technique (qualifier value)", [])
    },
    "InjectionOfAmnion": {"15415002": ("Injection of amnion (procedure)", [])},
    "InjectionOfProstaglandin": {
        "133875007": ("Injection of prostaglandin (procedure)", [])
    },
    "InjectionSiteAbscess": {"95382004": ("Injection site abscess", [60, 9300, 10043])},
    "InjectionSiteAnesthesia": {
        "95398006": ("Injection site anesthesia", [60, 9300, 10043])
    },
    "InjectionSiteAtrophy": {"95404001": ("Injection site atrophy", [60, 9300, 10043])},
    "InjectionSiteBruising": {
        "95401009": ("Injection site bruising", [60, 9300, 10043])
    },
    "InjectionSiteBurning": {"95389008": ("Injection site burning", [60, 9300, 10043])},
    "InjectionSiteCyst": {"95396005": ("Injection site cyst", [60, 9300, 10043])},
    "InjectionSiteDermatitis": {
        "95393002": ("Injection site dermatitis", [60, 9300, 10043])
    },
    "InjectionSiteDisorder": {
        "95376002": ("Injection site disorder", [60, 9300, 10043])
    },
    "InjectionSiteEdema": {"95392007": ("Injection site edema", [60, 9300, 10043])},
    "InjectionSiteExtravasation": {
        "95384003": ("Injection Site Extravasation", [60, 9300, 9301])
    },
    "InjectionSiteFibrosis": {
        "95403007": ("Injection site fibrosis", [60, 9300, 10043])
    },
    "InjectionSiteGranuloma": {
        "24389009": ("Injection site granuloma", [60, 9300, 10043])
    },
    "InjectionSiteHemorrhage": {
        "95385002": ("Injection site hemorrhage", [60, 9300, 10043])
    },
    "InjectionSiteHypersensitivity": {
        "95378001": ("Injection site hypersensitivity", [60, 9300, 10043])
    },
    "InjectionSiteInduration": {
        "95402002": ("Injection site induration", [60, 9300, 10043])
    },
    "InjectionSiteInfection": {
        "95381006": ("Injection site infection", [60, 9300, 10043])
    },
    "InjectionSiteInflammation": {
        "95391000": ("Injection site inflammation", [60, 9300, 10043])
    },
    "InjectionSiteIrritation": {
        "95379009": ("Injection site irritation", [60, 9300, 10043])
    },
    "InjectionSiteMalabsorption": {
        "95387005": ("Injection site malabsorption", [60, 9300, 10043])
    },
    "InjectionSiteMass": {"95395009": ("Injection site mass", [60, 9300, 10043])},
    "InjectionSiteNecrosis": {
        "95397001": ("Injection site necrosis", [60, 9300, 10043])
    },
    "InjectionSiteNerveDamage": {
        "95390004": ("Injection site nerve damage", [60, 9300, 10043])
    },
    "InjectionSitePain": {"95388000": ("Injection site pain", [60, 9300, 10043])},
    "InjectionSiteParesthesia": {
        "95399003": ("Injection site paresthesia", [60, 9300, 10043])
    },
    "InjectionSitePigmentationChange": {
        "95380007": ("Injection site pigmentation change", [60, 9300, 10043])
    },
    "InjectionSiteReaction": {"95377006": ("Injection site reaction (disorder)", [])},
    "InjectionSiteScar": {"111017005": ("Injection site scar", [60, 9300, 10043])},
    "InjectionSiteSterileAbscess": {
        "95383009": ("Injection site sterile abscess", [60, 9300, 10043])
    },
    "InjectionSiteThrombosis": {
        "95386001": ("Injection site thrombosis", [60, 9300, 10043])
    },
    "InjectionSiteUlcer": {"95400005": ("Injection site ulcer", [60, 9300, 10043])},
    "InjectionSiteUrticaria": {
        "95394008": ("Injection site urticaria", [60, 9300, 10043])
    },
    "InkukuCattleBreed": {"131479001": ("Inkuku cattle breed", [7480])},
    "InlayDentalRestoration": {
        "272287005": ("Inlay Dental Restoration", [7151, 7157, 7193])
    },
    "InnerEar": {"22945000": ("Inner ear", [4028, 7151, 7192, 8134, 9514])},
    "InnerEarStructure": {"22945000": ("Inner ear structure (body structure)", [])},
    "InnermostIntercostalMuscles": {
        "24062007": (
            "Innermost intercostal muscles",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "InnominateArtery": {
        "12691009": (
            "Innominate artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12107, 12291],
        )
    },
    "InnominateLymphNode": {"196821008": ("innominate lymph node", [7600, 8134])},
    "InnominateVein": {
        "8887007": (
            "Innominate vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "InobutaInterSpeciesHybridPigBreed": {
        "133182008": ("Inobuta (inter-species hybrid) pig breed", [7480])
    },
    "Inositol": {"72164009": ("Inositol", [218, 4032, 4033, 7180, 7186, 7469])},
    "InotropicAgent": {"111139005": ("Inotropic agent", [10, 621, 622])},
    "InsertionOfAtrialBaffle": {"1179121008": ("Insertion of atrial baffle", [12247])},
    "InsertionOfCatheter": {"45211000": ("Insertion of catheter", [6050, 6083])},
    "InsertionOfCoronaryArteryStent": {
        "36969009": ("Insertion of coronary artery stent", [3764])
    },
    "InsertionOfHipProsthesis": {"398010007": ("Insertion of hip prosthesis", [7301])},
    "InsertionOfInteratrialBaffle": {
        "261537001": ("Insertion of interatrial baffle", [12247])
    },
    "InsertionOfProsthesisForBreast": {
        "302343007": ("Insertion of prosthesis for breast (procedure)", [])
    },
    "InsolubleBerlinBlueStain": {"47030008": ("insoluble berlin blue stain", [8112])},
    "Inspiration": {"14910006": ("inspiration", [3823, 9272, 12234])},
    "InstrumentDevice": {"57134006": ("Instrument, device (physical object)", [])},
    "InsufficientSample": {"281268007": ("Insufficient sample", [6063])},
    "Insula": {"36169008": ("Insula", [7151, 7153, 7192, 8134, 9514])},
    "InsularStructure": {"36169008": ("Insular structure (body structure)", [])},
    "InsulinDependentMotherIDM": {
        "444161008": ("Insulin dependent mother (IDM)", [3756])
    },
    "InsulinPump": {"69805005": ("Insulin pump", [3777])},
    "InsulinPumpDevice": {"69805005": ("Insulin pump, device (physical object)", [])},
    "IntegratedRayTraceTriangulationAcquisitionLaserScanningDeviceWithConventionalFundusImaging": {
        "416567007": (
            "Integrated ray-trace triangulation acquisition laser scanning device with conventional "
            "fundus imaging (physical object)",
            [],
        )
    },
    "IntensiveCare": {"309904001": ("Intensive Care", [7030])},
    "IntensiveCareUnit": {"309904001": ("Intensive care unit (environment)", [])},
    "IntentsNatureOfProcedureValues": {
        "363675004": ("Intents (nature of procedure values) (qualifier value)", [])
    },
    "InteratrialSeptum": {
        "58095006": ("Interatrial septum", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "InteratrialSeptumStructure": {
        "58095006": ("Interatrial Septum Structure", [12284])
    },
    "InteratrialTranspositionOfVenousReturn": {
        "40250003": ("Interatrial transposition of venous return (procedure)", [])
    },
    "Intercostal": {"1197041002": ("Intercostal", [5])},
    "IntercostalArtery": {
        "281134007": ("Intercostal artery", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "IntercostalLymphNode": {"64038003": ("intercostal lymph node", [7600, 8134])},
    "InterferonGamma": {"420303002": ("Interferon gamma", [638, 642])},
    "InteriliacLymphNode": {"279271008": ("interiliac lymph node", [7600, 8134])},
    "InterlobarArteryOfKidney": {
        "274143007": (
            "Interlobar Artery of Kidney",
            [3827, 7151, 7192, 8134, 9514, 12103, 12115],
        )
    },
    "InterlobarLymphNodeOfTheLung": {
        "127919002": ("interlobar lymph node of the lung", [7600, 8134])
    },
    "InterlobularArteryOfKidney": {
        "274329007": (
            "Interlobular Artery of Kidney",
            [3827, 7151, 7192, 8134, 9514, 12103, 12115],
        )
    },
    "Intermediate": {"11896004": ("Intermediate", [2])},
    "IntermediateArtery": {"244252004": ("Intermediate artery (body structure)", [])},
    "IntermediateArteryRamus": {
        "244252004": (
            "Intermediate Artery (Ramus)",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "IntermediateCommonIliacLymphNode": {
        "64556009": ("intermediate common iliac lymph node", [7600, 8134])
    },
    "IntermediateExternalIliacLymphNode": {
        "50193000": ("intermediate external iliac lymph node", [7600, 8134])
    },
    "Intermittent": {"7087005": ("Intermittent", [3770])},
    "Internal": {"260521003": ("Internal", [2, 5, 6139])},
    "InternalAuditoryCanal": {
        "361078006": (
            "Internal Auditory Canal",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "InternalAuditoryMeatus": {
        "61671002": ("Internal Auditory Meatus", [1000, 1001, 1002, 8134])
    },
    "InternalCapsule": {"85637007": ("internal capsule", [7702, 7710, 8134])},
    "InternalCapsuleStructureOfBrain": {
        "85637007": ("Internal capsule structure of brain (body structure)", [])
    },
    "InternalCarotidArtery": {
        "86117002": (
            "Internal carotid artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12104,
                12105,
            ],
        )
    },
    "InternalCarotidArteryC5Segment": {
        "415637004": (
            "Internal Carotid Artery C5 segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "InternalCarotidArteryC6Segment": {
        "698348000": (
            "Internal Carotid Artery C6 segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "InternalCarotidArteryStructure": {
        "86117002": ("Internal carotid artery structure (body structure)", [])
    },
    "InternalFixationUsingInternalFixatorSystem": {
        "257837004": ("Internal fixation using internal fixator system (procedure)", [])
    },
    "InternalFixationUsingPlate": {
        "257835007": ("Internal fixation using plate", [7310])
    },
    "InternalFixationUsingScrew": {
        "257834006": ("Internal fixation using screw", [7310])
    },
    "InternalFixationUsingStaple": {
        "257833000": ("Internal fixation using staple", [7310])
    },
    "InternalIliacArtery": {
        "90024005": (
            "Internal iliac artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "InternalIliacVein": {
        "40300007": (
            "Internal iliac vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "InternalIntercostalMuscle": {
        "41313007": (
            "Internal intercostal muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "InternalJugularVein": {
        "12123001": (
            "Internal jugular vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12108,
            ],
        )
    },
    "InternalJugularVeinStructure": {
        "12123001": ("Internal jugular vein structure (body structure)", [])
    },
    "InternalLimitingMembraneOfRetina": {
        "280677004": ("Internal limiting membrane of retina (body structure)", [])
    },
    "InternalMammaryArtery": {
        "69327007": (
            "Internal mammary artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "InternalNose": {"53342003": ("Internal nose", [8134])},
    "InternalNoseStructure": {
        "53342003": ("Internal nose structure (body structure)", [])
    },
    "InternalRotation": {"12852001": ("Internal rotation", [92])},
    "InternalRotationFunction": {
        "12852001": ("Internal rotation, function (observable entity)", [])
    },
    "InternalSkeletalFixation": {"118470002": ("Internal skeletal fixation", [7310])},
    "InternalStructureOfEyeball": {
        "79652003": ("Internal structure of eyeball (body structure)", [])
    },
    "InternalThoracicArtery": {
        "69327007": ("Internal thoracic artery", [6102, 6109, 6117])
    },
    "InternationalStripedHorseHorseBreed": {
        "131986001": ("International Striped Horse horse breed", [7480])
    },
    "InterpectoralLymphNode": {"420800007": ("interpectoral lymph node", [7600, 8134])},
    "InterruptedAorticArch": {"218728005": ("Interrupted Aortic Arch", [12248, 12249])},
    "InterstitialTissue": {"85293002": ("Interstitial tissue", [6129, 8134])},
    "IntertarsalJointStructure": {
        "27949001": ("Intertarsal joint structure (body structure)", [])
    },
    "Interval": {"385673002": ("Interval", [12304])},
    "InterventionalRadiologyService": {
        "708174004": ("Interventional Radiology Service", [7030])
    },
    "InterventricularSeptum": {
        "589001": (
            "Interventricular septum",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514, 12286, 12305],
        )
    },
    "InterventricularSeptumStructure": {
        "589001": ("Interventricular septum structure (body structure)", [])
    },
    "IntervertebralDiscStructureOfEighthThoracicVertebra": {
        "9188009": (
            "Intervertebral disc structure of eighth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfEleventhThoracicVertebra": {
        "1537001": (
            "Intervertebral disc structure of eleventh thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFifthCervicalVertebra": {
        "73959003": (
            "Intervertebral disc structure of fifth cervical vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFifthLumbarVertebra": {
        "75782005": (
            "Intervertebral disc structure of fifth lumbar vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFifthThoracicVertebra": {
        "72692000": (
            "Intervertebral disc structure of fifth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFirstLumbarVertebra": {
        "80064006": (
            "Intervertebral disc structure of first lumbar vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFirstThoracicVertebra": {
        "40908007": (
            "Intervertebral disc structure of first thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFourthCervicalVertebra": {
        "49400002": (
            "Intervertebral disc structure of fourth cervical vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFourthLumbarVertebra": {
        "84020006": (
            "Intervertebral disc structure of fourth lumbar vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfFourthThoracicVertebra": {
        "82965004": (
            "Intervertebral disc structure of fourth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfNinthThoracicVertebra": {
        "113209001": (
            "Intervertebral disc structure of ninth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSecondLumbarVertebra": {
        "67459009": (
            "Intervertebral disc structure of second lumbar vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSecondThoracicVertebra": {
        "6004007": (
            "Intervertebral disc structure of second thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSeventhCervicalVertebra": {
        "37414007": (
            "Intervertebral disc structure of seventh cervical vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSeventhThoracicVertebra": {
        "28693002": (
            "Intervertebral disc structure of seventh thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSixthCervicalVertebra": {
        "75095006": (
            "Intervertebral disc structure of sixth cervical vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfSixthThoracicVertebra": {
        "74401007": (
            "Intervertebral disc structure of sixth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfTenthThoracicVertebra": {
        "34959001": (
            "Intervertebral disc structure of tenth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfThirdCervicalVertebra": {
        "58820006": (
            "Intervertebral disc structure of third cervical vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfThirdLumbarVertebra": {
        "62551000": (
            "Intervertebral disc structure of third lumbar vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfThirdThoracicVertebra": {
        "2620004": (
            "Intervertebral disc structure of third thoracic vertebra (body structure)",
            [],
        )
    },
    "IntervertebralDiscStructureOfTwelfthThoracicVertebra": {
        "76206002": (
            "Intervertebral disc structure of twelfth thoracic vertebra (body structure)",
            [],
        )
    },
    "IntestinalLymphNode": {"36251007": ("intestinal lymph node", [7600, 8134])},
    "Intima": {"87483006": ("Intima", [7151, 7156, 7192, 8134, 9514])},
    "IntraAbdominopelvic": {
        "818987002": ("Intra-abdominopelvic", [4, 4040, 7151, 7192, 9514])
    },
    "IntraAorticBalloonPump": {"129113006": ("Intra-Aortic Balloon Pump", [3553])},
    "IntraAorticBalloonPumpDevice": {
        "129113006": ("Intra-aortic balloon pump, device (physical object)", [])
    },
    "IntraAorticBalloonPumpIABP": {
        "129113006": ("Intra-Aortic Balloon Pump (IABP)", [3429])
    },
    "IntraArterialInfusionOfThrombolyticAgent": {
        "69245005": ("Intra-arterial infusion of thrombolytic agent", [9])
    },
    "IntraArterialRoute": {"58100008": ("Intra-arterial route", [11])},
    "IntraArterialWaveform": {
        "128446002": ("Intra-arterial waveform (observable entity)", [])
    },
    "IntraArticular": {
        "131183008": ("Intra-articular", [2, 4, 7151, 7192, 8134, 9514])
    },
    "IntraArticularRoute": {"12130007": ("Intra-articular route", [11])},
    "IntraAtrialPacing": {"69158002": ("Intra-atrial pacing (procedure)", [])},
    "IntraPelvic": {"816989007": ("Intra-pelvic", [4, 4040, 7151, 7192, 9514])},
    "IntraThoracic": {"43799004": ("Intra-thoracic", [4, 4040, 7151, 7192, 9514])},
    "IntracardiacEchocardiography": {
        "252421008": ("Intracardiac echocardiography", [3405, 3428, 12001])
    },
    "IntracardiacRoute": {"372460008": ("Intracardiac route", [11])},
    "IntracavitaryBrachytherapy": {
        "384692006": ("Intracavitary brachytherapy (procedure)", [])
    },
    "IntracavitaryEchocardiography": {
        "252421008": ("Intracavitary echocardiography (procedure)", [])
    },
    "IntracoronaryDopplerGuideWire": {
        "371788001": ("Intracoronary Doppler guide wire", [3411, 3429])
    },
    "IntracoronaryPressureGuideWire": {
        "371789009": ("Intracoronary pressure guide wire", [3411, 3429])
    },
    "IntracoronaryRoute": {"372463005": ("Intracoronary route", [11])},
    "IntracorpusCavernosumRoute": {
        "1259221004": ("Intracorpus cavernosum route", [11])
    },
    "IntracorpusCavernosumRoute_Inactive": {
        "445769006": ("Intracorpus cavernosum route (qualifier value)", [])
    },
    "Intracranial": {"1101003": ("Intracranial", [4, 7151, 7192, 8134, 9514])},
    "IntracranialHemorrhage": {"1386000": ("Intracranial hemorrhage (disorder)", [])},
    "IntracranialStructure": {"128319008": ("Intracranial structure", [7140])},
    "IntracutaneousRoute": {"372464004": ("Intracutaneous route", [11])},
    "IntracysticPapillaryAdenoma": {
        "47488001": ("Intracystic papillary adenoma (morphologic abnormality)", [])
    },
    "IntracysticPapilloma": {"47488001": ("Intracystic papilloma", [6030, 6031])},
    "IntradermalRoute": {"372464004": ("Intradermal route (qualifier value)", [])},
    "IntraductalCarcinomaMicroPapillary": {
        "128696009": ("Intraductal carcinoma micro-papillary", [6030, 6033])
    },
    "IntraductalCarcinomaNoninfiltratingNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "86616005": (
            "Intraductal carcinoma, noninfiltrating, no International Classification of Diseases "
            "for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "IntraductalHyperplasia": {
        "67617000": ("Intraductal hyperplasia (morphologic abnormality)", [])
    },
    "IntraductalMicropapillaryCarcinoma": {
        "128696009": (
            "Intraductal micropapillary carcinoma (morphologic abnormality)",
            [],
        )
    },
    "IntraductalPapilloma": {"5244003": ("Intraductal papilloma", [6030, 6031])},
    "IntraglandularParotidLymphNode": {
        "143824007": ("intraglandular parotid lymph node", [7600, 8134])
    },
    "Intraluminal": {"264045001": ("Intraluminal", [5])},
    "IntraluminalFluid": {"442170005": ("Intraluminal fluid", [6201])},
    "IntraluminalRoute": {"37737002": ("Intraluminal route", [11])},
    "IntramammaryLymphNode": {
        "443808008": (
            "Intramammary lymph node",
            [6014, 6015, 6016, 6030, 6031, 6054, 6056, 7600, 8134],
        )
    },
    "IntramedullaryRoute": {"60213007": ("Intramedullary route", [11])},
    "IntramuralHematoma": {"54493002": ("Intramural hematoma", [3492])},
    "IntramuscularRoute": {"78421000": ("Intramuscular route", [11, 613, 614])},
    "Intraoperative": {"277671009": ("Intraoperative", [31, 631, 632])},
    "IntraoperativeEchocardiography": {
        "429884006": ("Intraoperative echocardiography", [12001])
    },
    "IntraperitonealRoute": {"38239002": ("Intraperitoneal route", [11, 613])},
    "IntrapulmonaryArteriovenousFistula": {
        "253639004": ("Intrapulmonary arteriovenous fistula (disorder)", [])
    },
    "IntrapulmonaryLymphNode": {
        "196662004": ("intrapulmonary lymph node", [7600, 8134])
    },
    "IntrathecalRoute": {"72607000": ("Intrathecal route", [11])},
    "IntratumorRoute": {"447122006": ("Intratumor route", [11])},
    "IntrauterineCordocentesis": {"6708002": ("Intrauterine cordocentesis", [9])},
    "IntrauterineTransfusion": {"45460008": ("Intrauterine transfusion", [9])},
    "IntravascularEchocardiography": {
        "252420009": ("Intravascular echocardiography", [12001])
    },
    "IntravascularOpticalCoherenceTomographyDevice": {
        "445282004": ("Intravascular Optical Coherence Tomography Device", [3411, 3429])
    },
    "IntravascularOpticalCoherenceTomographyImagingDevice": {
        "445282004": (
            "Intravascular optical coherence tomography imaging device (physical object)",
            [],
        )
    },
    "IntravascularUltrasound": {
        "241466007": ("Intravascular ultrasound", [3405, 3428])
    },
    "IntravascularUltrasoundDevice": {
        "371795005": ("Intravascular ultrasound device (physical object)", [])
    },
    "IntravascularUltrasoundIVUSDevice": {
        "371795005": ("Intravascular Ultrasound (IVUS) Device", [3411, 3429])
    },
    "IntravascularUltrasoundScan": {
        "241466007": ("Intravascular ultrasound scan (procedure)", [])
    },
    "IntravenousRoute": {"47625008": ("Intravenous route", [11, 613, 614])},
    "IntraventricularConductionDefect": {
        "4554005": ("Intraventricular conduction defect (disorder)", [])
    },
    "IntraventricularConductionDisturbance": {
        "4554005": ("Intraventricular conduction disturbance", [3230])
    },
    "IntraventricularPacing": {
        "344994008": ("Intraventricular pacing (procedure)", [])
    },
    "IntraventricularRouteCardiac": {
        "420287000": ("Intraventricular route - cardiac", [11])
    },
    "IntubationOfRespiratoryTract": {
        "447996002": ("Intubation of respiratory tract", [617, 618])
    },
    "InvasiveCribriformCarcinoma": {
        "30156004": ("Invasive cribriform carcinoma", [6030, 6033])
    },
    "InvasiveLobularCarcinoma": {
        "89740008": ("Invasive lobular carcinoma", [6030, 6033])
    },
    "InverseTrendelenburg": {"26527006": ("inverse Trendelenburg", [20])},
    "InverseTrendelenburgPosition": {
        "26527006": ("Inverse Trendelenburg position (finding)", [])
    },
    "InvertedDiverticulum": {"441901008": ("Inverted diverticulum", [6201])},
    "Inward": {"255460003": ("Inward", [4214])},
    "IodamideMeglumine": {"12801003": ("Iodamide meglumine", [12, 3850])},
    "IodinatedI125Albumin": {"72015003": ("Iodinated I^125^ albumin", [25])},
    "IodinatedI125HumanSerumAlbumin": {
        "64488003": ("Iodinated I^125^ human serum albumin", [25])
    },
    "IodinatedI125Liothyronine": {
        "33785000": ("Iodinated I^125^ liothyronine (substance)", [])
    },
    "IodinatedI125Povidone": {"69839009": ("Iodinated I^125^ povidone", [25])},
    "IodinatedI125SodiumIodine": {"70154008": ("Iodinated I^125^ sodium iodine", [25])},
    "IodinatedI131AggregatedAlbumin": {
        "55814006": ("Iodinated I^131^ aggregated albumin", [25])
    },
    "IodinatedI131Albumin": {"39200002": ("Iodinated I^131^ albumin", [25])},
    "IodinatedI131GammaGlobulin": {
        "52408003": ("Iodinated I^131^ gamma globulin", [25])
    },
    "Iodine": {"44588005": ("Iodine", [13, 300])},
    "Iodine123154Iodophenyl3RSMethylpentadecanoicAcid": {
        "447134003": (
            "Iodine^123 15-(4-Iodophenyl)-3(R,S)-Methylpentadecanoic Acid",
            [25],
        )
    },
    "Iodine1233IodobenzylguanidineMIBG": {
        "395787009": ("Iodine^123 3-Iodobenzylguanidine MIBG", [25])
    },
    "Iodine1313IodobenzylguanidineMIBG": {
        "395789007": ("Iodine^131 3-Iodobenzylguanidine MIBG", [25])
    },
    "Iodine131Polyvinylpyrrolidone": {
        "35884005": ("Iodine^131^ polyvinylpyrrolidone", [25])
    },
    "Iodipamide": {"73212002": ("Iodipamide", [12, 3850])},
    "Iodixanol": {"353962003": ("Iodixanol", [12, 3850])},
    "IodizedOil": {"89595000": ("Iodized oil", [12, 3850])},
    "IodoalphionicAcid": {"86584005": ("Iodoalphionic acid", [12, 3850])},
    "IodocholesterolI131": {"68967007": ("Iodocholesterol I^131^", [25])},
    "IodohippurateI123Sodium": {"33785000": ("Iodohippurate I^123^ sodium", [25])},
    "IodohippurateI125Sodium": {"36900006": ("Iodohippurate I^125^ sodium", [25])},
    "IodohippurateI131Sodium": {"33271006": ("Iodohippurate I^131^ sodium", [25])},
    "Iodophthalein": {"74554008": ("Iodophthalein", [12, 3850])},
    "Iodopyracet": {"40710000": ("Iodopyracet", [12, 3850])},
    "IofetamineI123Hydrochloride": {
        "78481003": ("Iofetamine I^123^ hydrochloride", [25])
    },
    "Ioflupane123I": {"429296007": ("Ioflupane[123I] (product)", [])},
    "IoflupaneI123": {"429296007": ("Ioflupane I^123^", [25])},
    "Iohexol": {"109218004": ("Iohexol", [12, 3850])},
    "IomudHorseBreed": {"133047009": ("Iomud horse breed", [7480])},
    "Iopamidol": {"109219007": ("Iopamidol", [12, 3850])},
    "IopanoicAcid": {"76155001": ("Iopanoic acid", [12, 3850])},
    "Iophendylate": {"28121005": ("Iophendylate", [12, 3850])},
    "IophenoxicAcid": {"23053002": ("Iophenoxic acid", [12, 3850])},
    "Iopromide": {"353903006": ("Iopromide", [12, 3850])},
    "Iothalamate": {"353912008": ("Iothalamate", [12, 3850])},
    "IothalamateSodiumI125": {"55673009": ("Iothalamate sodium I^125^", [25])},
    "Ioversol": {"109222009": ("Ioversol", [12, 3850])},
    "Ioxaglate": {"412228003": ("Ioxaglate", [12, 3850])},
    "IoxaglateMeglumine": {"412228003": ("Ioxaglate meglumine (substance)", [])},
    "Ioxilan": {"409484007": ("Ioxilan", [12, 3850])},
    "Ipodate": {"87445005": ("Ipodate", [12, 3850])},
    "Ipsilateral": {"255208005": ("Ipsilateral", [246])},
    "IranianZebuCattleBreed": {"133686007": ("Iranian Zebu cattle breed", [7480])},
    "IraqiCattleBreed": {"133808008": ("Iraqi cattle breed (organism)", [])},
    "IraqiXZebuCattleBreed": {"133808008": ("Iraqi X zebu cattle breed", [7480])},
    "Iris": {"41296002": ("Iris", [4029, 4209, 4211, 8134])},
    "IrisStructure": {"41296002": ("Iris structure (body structure)", [])},
    "IrishCobHorseBreed": {"131987005": ("Irish Cob horse breed", [7480])},
    "IrishDraftHorseBreed": {"131908002": ("Irish Draft horse breed", [7480])},
    "IrishGoatBreed": {"131663006": ("Irish goat breed", [7480])},
    "IrishHunterHorseBreed": {"131909005": ("Irish Hunter horse breed", [7480])},
    "IrishMoiledCattleBreed": {"133236005": ("Irish Moiled cattle breed", [7480])},
    "IrishRedAndWhiteSetterDogBreed": {
        "132503004": ("Irish Red and White Setter dog breed", [7480])
    },
    "IrishSetter": {"11477006": ("Irish setter (organism)", [])},
    "IrishSetterDogBreed": {"11477006": ("Irish setter dog breed", [7480])},
    "IrishTerrier": {"75494002": ("Irish terrier", [7480])},
    "IrishWaterSpaniel": {"34870009": ("Irish water spaniel (organism)", [])},
    "IrishWaterSpanielDogBreed": {
        "34870009": ("Irish water spaniel dog breed", [7480])
    },
    "IrishWolfhound": {"52952001": ("Irish wolfhound", [7480])},
    "Iron": {"3829006": ("Iron", [13, 300])},
    "IronFe59LabeledDextran": {"60459006": ("Iron Fe^59^ labeled dextran", [25])},
    "IrradiatedVolume": {"228790005": ("Irradiated Volume", [9534])},
    "Irregular": {
        "49608001": (
            "Irregular",
            [3715, 6004, 6005, 6119, 6335, 6339, 6340, 6341, 6342],
        )
    },
    "IrregularBreathing": {"248585001": ("irregular breathing", [3416])},
    "Irrigation": {"67889009": ("Irrigation", [9])},
    "IrrigationFollowingInsertionOfCatheter": {
        "8592001": ("Irrigation following insertion of catheter (procedure)", [])
    },
    "IschemicHeartDisease": {"414545008": ("ischemic heart disease", [3727])},
    "IscherwoodProjection": {
        "399025004": ("Ischerwood projection (qualifier value)", [])
    },
    "Ischium": {"85710004": ("Ischium", [1000, 1001, 1005, 8134])},
    "Isherwood": {"399025004": ("Isherwood", [4012])},
    "IskarCattleBreed": {"131480003": ("Iskar cattle breed", [7480])},
    "Isoflurane": {"387368002": ("Isoflurane", [623, 624])},
    "IsolationRoom": {"409688003": ("Isolation room", [603])},
    "IsraeliHolsteinCattleBreed": {
        "133237001": ("Israeli Holstein cattle breed", [7480])
    },
    "IsraeliHorseBreed": {"132733008": ("Israeli horse breed", [7480])},
    "IsraeliRedCattleBreed": {"133583008": ("Israeli Red cattle breed", [7480])},
    "IstobenCattleBreed": {"133238006": ("Istoben cattle breed", [7480])},
    "IstrianCattleBreed": {"131481004": ("Istrian cattle breed", [7480])},
    "IstrianPramenkaSheepBreed": {
        "131776007": ("Istrian Pramenka sheep breed", [7480])
    },
    "ItalianBrownCattleBreed": {"133441000": ("Italian Brown cattle breed", [7480])},
    "ItalianGreyhound": {"30347000": ("Italian greyhound", [7480])},
    "ItalianHeavyDraftHorseBreed": {
        "131911001": ("Italian Heavy Draft horse breed", [7480])
    },
    "ItalianLandracePig": {"71923001": ("Italian landrace pig (organism)", [])},
    "ItalianLandracePigBreed": {"71923001": ("Italian landrace pig breed", [7480])},
    "ItalianRedPiedCattleBreed": {
        "133442007": ("Italian Red Pied cattle breed", [7480])
    },
    "Itching": {"418363000": ("Itching", [60, 4406, 9300])},
    "ItchingOfSkin": {"418363000": ("Itching of skin (finding)", [])},
    "JWire": {"129463006": ("J Wire", [6040, 6401, 7151, 7193])},
    "JWireDevice": {"129463006": ("J wire, device (physical object)", [])},
    "JabeHorseBreed": {"131912008": ("Jabe horse breed", [7480])},
    "JackRusselTerrier": {"6103004": ("Jack Russel terrier", [7480])},
    "JackRussellTerrierDogBreed": {
        "6103004": ("Jack Russell terrier dog breed (organism)", [])
    },
    "JacobSheepBreed": {"131777003": ("Jacob sheep breed", [7480])},
    "JagdterrierDogBreed": {"132504005": ("Jagdterrier dog breed", [7480])},
    "JamaicaBlackCattleBreed": {
        "133732006": ("Jamaica Black cattle breed (organism)", [])
    },
    "JamaicaBlackXZebuCattleBreed": {
        "133732006": ("Jamaica Black X zebu cattle breed", [7480])
    },
    "JamaicaBrahmanCattleBreed": {
        "133665007": ("Jamaica Brahman cattle breed", [7480])
    },
    "JamaicaHopeCattleBreed": {
        "133733001": ("Jamaica Hope cattle breed (organism)", [])
    },
    "JamaicaHopeXZebuCattleBreed": {
        "133733001": ("Jamaica Hope X zebu cattle breed", [7480])
    },
    "JamaicaRedCattleBreed": {"133734007": ("Jamaica Red cattle breed (organism)", [])},
    "JamaicaRedXZebuCattleBreed": {
        "133734007": ("Jamaica Red X zebu cattle breed", [7480])
    },
    "JanusGreenBStain": {"68263003": ("janus green B stain", [8112])},
    "JapaneseBlackCattleBreed": {"133443002": ("Japanese Black cattle breed", [7480])},
    "JapaneseBobtailCat": {"26382003": ("Japanese bobtail cat", [7480])},
    "JapaneseBrownCattleBreed": {"133444008": ("Japanese Brown cattle breed", [7480])},
    "JapaneseChinDog": {"53922000": ("Japanese chin dog", [7480])},
    "JapanesePointerDogBreed": {"132530009": ("Japanese Pointer dog breed", [7480])},
    "JapanesePollCattleBreed": {"133445009": ("Japanese Poll cattle breed", [7480])},
    "JapaneseRetrieverDogBreed": {
        "132398003": ("Japanese Retriever dog breed", [7480])
    },
    "JapaneseShorthornCattleBreed": {
        "133452006": ("Japanese Shorthorn cattle breed", [7480])
    },
    "JapaneseSpaniel": {"23159000": ("Japanese spaniel", [7480])},
    "JapaneseSpitzDogBreed": {"132429002": ("Japanese Spitz dog breed", [7480])},
    "JarmelistaCattleBreed": {"133453001": ("Jarmelista cattle breed", [7480])},
    "JaulanCattleBreed": {"133239003": ("Jaulan cattle breed", [7480])},
    "Jaundice": {"18165001": ("Jaundice", [12325])},
    "JavaHorseBreed": {"131913003": ("Java horse breed", [7480])},
    "JavaneseCat": {"10701001": ("Javanese cat", [7480])},
    "JavaneseOngoleCattleBreed": {
        "131482006": ("Javanese Ongole cattle breed", [7480])
    },
    "JavaneseZebuCattleBreed": {"131483001": ("Javanese Zebu cattle breed", [7480])},
    "JawRegion": {
        "661005": (
            "Jaw region",
            [4, 4009, 4016, 4028, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "JawRegionStructure": {"661005": ("Jaw region structure (body structure)", [])},
    "JejunalStructure": {"21306003": ("Jejunal structure (body structure)", [])},
    "JejunostomyTube": {
        "126065006": ("Jejunostomy tube", [6102, 6138, 6202, 6203, 6404, 7151, 7193])
    },
    "JejunostomyTubeDevice": {
        "126065006": ("Jejunostomy tube, device (physical object)", [])
    },
    "Jejunum": {
        "21306003": ("Jejunum", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "JellicutCattleBreed": {"133666008": ("Jellicut cattle breed", [7480])},
    "JemJemZebuCattleBreed": {"132771000": ("Jem-Jem Zebu cattle breed", [7480])},
    "JennerGiemsaStain": {"255801008": ("Jenner-Giemsa stain", [8112])},
    "JenubiCattleBreed": {"133674009": ("Jenubi cattle breed", [7480])},
    "JerdiCattleBreed": {"133809000": ("Jerdi cattle breed (organism)", [])},
    "JerdiXZebuCattleBreed": {"133809000": ("Jerdi X zebu cattle breed", [7480])},
    "JerseyCattleBreed": {"51937006": ("Jersey cattle breed (organism)", [])},
    "JerseyCowBreed": {"51937006": ("Jersey cow breed", [7480])},
    "JersindCattleBreed": {"133810005": ("Jersind cattle breed (organism)", [])},
    "JersindXZebuCattleBreed": {"133810005": ("Jersind X zebu cattle breed", [7480])},
    "Jewelry": {"80919006": ("Jewelry", [6102, 6138, 6202, 6203, 6404, 7151, 7193])},
    "JezerskosolcavskaSheepBreed": {
        "131778008": ("Jezerskosolcavska sheep breed", [7480])
    },
    "JianchangHorseBreed": {"132739007": ("Jianchang horse breed", [7480])},
    "JianliPigBreed": {"132187007": ("Jianli pig breed", [7480])},
    "JiaoxiPigBreed": {"132321000": ("Jiaoxi pig breed", [7480])},
    "JiaxingBlackPigBreed": {"132318002": ("Jiaxing Black pig breed", [7480])},
    "JidduCattleBreed": {"133675005": ("Jiddu cattle breed", [7480])},
    "JielinHorseBreed": {"132740009": ("Jielin horse breed", [7480])},
    "JijjigaZebuCattleBreed": {"133676006": ("Jijjiga Zebu cattle breed", [7480])},
    "JindoDogBreed": {"132431006": ("Jindo dog breed", [7480])},
    "JinhongHorseBreed": {"131916006": ("Jinhong horse breed", [7480])},
    "JinhuaDongyangPigBreed": {"132293009": ("Jinhua, Dongyang pig breed", [7480])},
    "JinhuaPigBreed": {"132244004": ("Jinhua pig breed", [7480])},
    "JinhuaYongkangPigBreed": {"132294003": ("Jinhua, Yongkang pig breed", [7480])},
    "JiningGrayGoatBreed": {"131665004": ("Jining Gray goat breed", [7480])},
    "JinnanCattleBreed": {"131484007": ("Jinnan cattle breed", [7480])},
    "JinzhouHorseBreed": {"131917002": ("Jinzhou horse breed", [7480])},
    "Joint": {"39352004": ("Joint", [4, 4040, 7151, 7192, 8134, 9514])},
    "JointStructure": {"39352004": ("Joint structure (body structure)", [])},
    "JonicaGoatBreed": {"131697004": ("Jonica goat breed", [7480])},
    "JotkoCattleBreed": {"133811009": ("Jotko cattle breed (organism)", [])},
    "JotkoXZebuCattleBreed": {"133811009": ("Jotko X zebu cattle breed", [7480])},
    "Judd": {"399201002": ("Judd", [4012])},
    "JuddProjection": {"399201002": ("Judd projection (qualifier value)", [])},
    "JudetProjection": {"260496000": ("Judet projection", [4012])},
    "JugularLymphNode": {"58130000": ("jugular lymph node", [7600, 8134])},
    "JutlandHorseBreed": {"133048004": ("Jutland horse breed", [7480])},
    "JuvenileCarcinomaOfTheBreast": {
        "41919003": ("Juvenile carcinoma of the breast (morphologic abnormality)", [])
    },
    "JuvenileFibroadenoma": {"46212000": ("Juvenile fibroadenoma", [6030, 6031])},
    "JuxtaintestinalLymphNode": {
        "279609001": ("juxtaintestinal lymph node", [7600, 8134])
    },
    "JuxtaposedAppendage": {"128563000": ("Juxtaposed appendage", [3608, 3630])},
    "JuxtaposedAtrialAppendage": {
        "128563000": (
            "Juxtaposed atrial appendage",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "JämthundDogBreed": {"132430007": ("Jämthund dog breed", [7480])},
    "KM1PigBreed": {"132117004": ("KM-1 pig breed", [7480])},
    "KabardaHorseBreed": {"132758003": ("Kabarda horse breed", [7480])},
    "KabotaCattleBreed": {"133677002": ("Kabota cattle breed", [7480])},
    "KabyleCattleBreed": {"133454007": ("Kabyle cattle breed", [7480])},
    "KachchaSiriCattleBreed": {"133678007": ("Kachcha Siri cattle breed", [7480])},
    "KachhiSheepBreed": {"131779000": ("Kachhi sheep breed", [7480])},
    "KaghaniGoatBreed": {"131662001": ("Kaghani goat breed", [7480])},
    "KaiDogBreed": {"132399006": ("Kai Dog breed", [7480])},
    "KaiKenDogBreed": {"132390005": ("Kai Ken dog breed", [7480])},
    "KajliSheepBreed": {"131786008": ("Kajli sheep breed", [7480])},
    "KakhetianPigBreed": {"132768008": ("Kakhetian pig breed", [7480])},
    "KalakheriCattleBreed": {"133679004": ("Kalakheri cattle breed", [7480])},
    "KalmykCattleBreed": {"131485008": ("Kalmyk cattle breed", [7480])},
    "KalmykHorseBreed": {"132759006": ("Kalmyk horse breed", [7480])},
    "KamanPigBreed": {"133177001": ("Kaman pig breed", [7480])},
    "KamdhinoCattleBreed": {"133680001": ("Kamdhino cattle breed", [7480])},
    "KameroenSheepBreed": {"132686002": ("Kameroen sheep breed", [7480])},
    "KamoriGoatBreed": {"131668002": ("Kamori goat breed", [7480])},
    "KandahariCattleBreed": {"133681002": ("Kandahari cattle breed", [7480])},
    "Kandel": {"399152006": ("Kandel", [4012])},
    "KandelProjection": {"399152006": ("Kandel projection (qualifier value)", [])},
    "KanemCattleBreed": {"133812002": ("Kanem cattle breed (organism)", [])},
    "KanemXZebuCattleBreed": {"133812002": ("Kanem X zebu cattle breed", [7480])},
    "KangalDogBreed": {"132402007": ("Kangal Dog breed", [7480])},
    "KangarooIslandPigBreed": {"133183003": ("Kangaroo Island pig breed", [7480])},
    "KangayamCattleBreed": {"133610008": ("Kangayam cattle breed", [7480])},
    "KaninganCattleBreed": {"133682009": ("Kaningan cattle breed", [7480])},
    "KankrejCattleBreed": {"133611007": ("Kankrej cattle breed", [7480])},
    "KaokoveldCattleBreed": {"131486009": ("Kaokoveld cattle breed", [7480])},
    "KappiliyanCattleBreed": {"133689000": ("Kappiliyan cattle breed", [7480])},
    "KapsikiCattleBreed": {"133455008": ("Kapsiki cattle breed", [7480])},
    "KarabairHorseBreed": {"133049007": ("Karabair horse breed", [7480])},
    "KarabakhHorseBreed": {"133050007": ("Karabakh horse breed", [7480])},
    "KarachaiHorseBreed": {"131920005": ("Karachai horse breed", [7480])},
    "KarakacanHorseBreed": {"131921009": ("Karakacan horse breed", [7480])},
    "KarakulSheep": {"64591001": ("Karakul sheep (organism)", [])},
    "KarakulSheepBreed": {"64591001": ("Karakul sheep breed", [7480])},
    "KaramajongCattleBreed": {"133690009": ("Karamajong cattle breed", [7480])},
    "KaranFriesCattleBreed": {"133735008": ("Karan Fries cattle breed (organism)", [])},
    "KaranFriesXZebuCattleBreed": {
        "133735008": ("Karan Fries X zebu cattle breed", [7480])
    },
    "KaranSwissCattleBreed": {"133736009": ("Karan Swiss cattle breed (organism)", [])},
    "KaranSwissXZebuCattleBreed": {
        "133736009": ("Karan Swiss X zebu cattle breed", [7480])
    },
    "KarelianBearDog": {"84660008": ("Karelian bear dog", [7480])},
    "KareloFinnishLaikaDogBreed": {
        "132432004": ("Karelo-Finnish Laika dog breed", [7480])
    },
    "Kasabach": {"399280003": ("Kasabach", [4012])},
    "KasabachProjection": {"399280003": ("Kasabach projection (qualifier value)", [])},
    "KatahdinSheepBreed": {"131743008": ("Katahdin sheep breed", [7480])},
    "KateriniCattleBreed": {"133456009": ("Katerini cattle breed", [7480])},
    "KathiawariHorseBreed": {"131922002": ("Kathiawari horse breed", [7480])},
    "KazakhCattleBreed": {"133240001": ("Kazakh cattle breed", [7480])},
    "KazakhHorseBreed": {"133051006": ("Kazakh horse breed", [7480])},
    "KazakhWhiteheadCattleBreed": {
        "131487000": ("Kazakh Whitehead cattle breed", [7480])
    },
    "KeErQinHorseBreed": {"131923007": ("Ke-Er-Qin horse breed", [7480])},
    "KedahKelantanCattleBreed": {"131488005": ("Kedah-Kelantan cattle breed", [7480])},
    "Keeshond": {"81607005": ("Keeshond", [7480])},
    "KelePigBreed": {"132238003": ("Kele pig breed", [7480])},
    "KemerovoPigBreed": {"132116008": ("Kemerovo pig breed", [7480])},
    "KempHarper": {"399227002": ("Kemp Harper", [4012])},
    "KempHarperProjection": {
        "399227002": ("Kemp Harper projection (qualifier value)", [])
    },
    "KenacidBlueRStain": {"29342009": ("kenacid blue R stain", [8112])},
    "KenanaCattleBreed": {"133691008": ("Kenana cattle breed", [7480])},
    "KenkathaCattleBreed": {"133612000": ("Kenkatha cattle breed", [7480])},
    "KenranCattleBreed": {"133457000": ("Kenran cattle breed", [7480])},
    "KentuckyRedBerkshirePig": {
        "33551003": ("Kentucky red berkshire pig (organism)", [])
    },
    "KentuckyRedBerkshirePigBreed": {
        "33551003": ("Kentucky red berkshire pig breed", [7480])
    },
    "KenyaBoranCattleBreed": {"133692001": ("Kenya Boran cattle breed", [7480])},
    "KenyaZebuCattleBreed": {"133693006": ("Kenya Zebu cattle breed", [7480])},
    "Keratolytic": {"280906005": ("Keratolytic (product)", [])},
    "KeratolyticAgent": {"280906005": ("Keratolytic agent", [4410])},
    "Keratoscope": {"397522002": ("Keratoscope", [4202])},
    "KerryBlueTerrier": {"32591006": ("Kerry blue terrier", [7480])},
    "KerryCattleBreed": {"133241002": ("Kerry cattle breed", [7480])},
    "KerryHillSheep": {"11967001": ("Kerry Hill sheep (organism)", [])},
    "KerryHillSheepBreed": {"11967001": ("Kerry Hill sheep breed", [7480])},
    "Ketamine": {"373464007": ("Ketamine", [623, 625])},
    "KetekuCattleBreed": {"133813007": ("Keteku cattle breed (organism)", [])},
    "KetekuXZebuCattleBreed": {"133813007": ("Keteku X zebu cattle breed", [7480])},
    "KhamalaCattleBreed": {"133694000": ("Khamala cattle breed", [7480])},
    "KharkovPigBreed": {"132074004": ("Kharkov pig breed", [7480])},
    "KherigarhCattleBreed": {"133613005": ("Kherigarh cattle breed", [7480])},
    "KhevsurianCattleBreed": {"133458005": ("Khevsurian cattle breed", [7480])},
    "KhillariCattleBreed": {"133614004": ("Khillari cattle breed", [7480])},
    "KholmogoryCattleBreed": {"133242009": ("Kholmogory cattle breed", [7480])},
    "KhorsanCattleBreed": {"133687003": ("Khorsan cattle breed", [7480])},
    "KhurasaniZebuCattleBreed": {"133695004": ("Khurasani zebu cattle breed", [7480])},
    "Kidney": {
        "64033007": (
            "Kidney",
            [
                645,
                1000,
                1001,
                1005,
                4030,
                4040,
                6202,
                6204,
                7151,
                7154,
                7192,
                8134,
                9514,
                10044,
                10060,
                12020,
                12321,
            ],
        )
    },
    "KidneyStructure": {"64033007": ("Kidney structure (body structure)", [])},
    "KigerMustangHorseBreed": {"133062006": ("Kiger Mustang horse breed", [7480])},
    "KigeziCattleBreed": {"131489002": ("Kigezi cattle breed", [7480])},
    "KikoGoatBreed": {"131670006": ("Kiko goat breed", [7480])},
    "KilaraCattleBreed": {"133696003": ("Kilara cattle breed", [7480])},
    "KilisCattleBreed": {"133459002": ("Kilis cattle breed", [7480])},
    "KinderGoatBreed": {"131671005": ("Kinder goat breed", [7480])},
    "KingShepherdDogBreed": {"132433009": ("King Shepherd dog breed", [7480])},
    "KinniyaCattleBreed": {"133697007": ("Kinniya cattle breed", [7480])},
    "KirdiPonyHorseBreed": {"133053009": ("Kirdi Pony horse breed", [7480])},
    "KirgizHorseBreed": {"131924001": ("Kirgiz horse breed", [7480])},
    "KirhizDogBreed": {"132435002": ("Kirhiz dog breed", [7480])},
    "KisantuCattleBreed": {"131490006": ("Kisantu cattle breed", [7480])},
    "KisberFelverHorseBreed": {"133043008": ("Kisber Felver horse breed", [7480])},
    "KishuDogBreed": {"132434003": ("Kishu dog breed", [7480])},
    "KisoHorseBreed": {"133054003": ("Kiso horse breed", [7480])},
    "Kite": {"425030002": ("Kite", [4012])},
    "KiteProjection": {"425030002": ("Kite projection (qualifier value)", [])},
    "KladrubyHorseBreed": {"133055002": ("Kladruby horse breed", [7480])},
    "KleenLeenBlackPig": {"36111002": ("Kleen leen black pig (organism)", [])},
    "KleenLeenBlackPigBreed": {"36111002": ("Kleen leen black pig breed", [7480])},
    "KleenLeenPig": {"69602006": ("Kleen leen pig (organism)", [])},
    "KleenLeenPigBreed": {"69602006": ("Kleen leen pig breed", [7480])},
    "KleenLeenRedPig": {"84232003": ("Kleen leen red pig (organism)", [])},
    "KleenLeenRedPigBreed": {"84232003": ("Kleen leen red pig breed", [7480])},
    "KleenLeenWhitePig": {"57613003": ("Kleen leen white pig (organism)", [])},
    "KleenLeenWhitePigBreed": {"57613003": ("Kleen leen white pig breed", [7480])},
    "KnabstrupHorseBreed": {"133056001": ("Knabstrup horse breed", [7480])},
    "Knee": {
        "72696002": (
            "Knee",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "KneeChest": {"23242002": ("knee-chest", [20])},
    "KneeChestPosition": {"23242002": ("Knee-chest position (finding)", [])},
    "KneeJoint": {"49076000": ("Knee joint", [1000, 1001, 1006, 8134])},
    "KneeJointStructure": {"49076000": ("Knee joint structure (body structure)", [])},
    "KneeRegionStructure": {"72696002": ("Knee region structure (body structure)", [])},
    "Kneeling": {"55864004": ("kneeling", [20])},
    "KnownAbsent": {"410516002": ("Known absent", [3770])},
    "KochiCattleBreed": {"133460007": ("Kochi cattle breed", [7480])},
    "KolbroekPigBreed": {"133189004": ("Kolbroek pig breed", [7480])},
    "KolubaraCattleBreed": {"131491005": ("Kolubara cattle breed", [7480])},
    "KomondorDog": {"46239008": ("Komondor dog", [7480])},
    "KonariCattleBreed": {"133698002": ("Konari cattle breed", [7480])},
    "KoninkTarpanHorseXDomesticHorseBreed": {
        "132991003": ("Konink tarpan horse X domestic horse breed", [7480])
    },
    "KooikerhondjeDogBreed": {"132495004": ("Kooikerhondje dog breed (organism)", [])},
    "KookaSheepBreed": {"131724008": ("Kooka sheep breed", [7480])},
    "KoratCat": {"27125003": ("Korat cat", [7480])},
    "KoreanBlackCattleBreed": {"133352005": ("Korean Black cattle breed", [7480])},
    "KoreanImprovedPigBreed": {"132141001": ("Korean Improved pig breed", [7480])},
    "KoreanNativeCattleBreed": {"133461006": ("Korean Native cattle breed", [7480])},
    "KoreanNativePigBreed": {"132140000": ("Korean Native pig breed", [7480])},
    "KoronadalPigBreed": {"133179003": ("Koronadal pig breed", [7480])},
    "KostromaCattleBreed": {"133468000": ("Kostroma cattle breed", [7480])},
    "Kovacs": {"399318001": ("Kovacs", [4012])},
    "KovacsProjection": {"399318001": ("Kovacs projection (qualifier value)", [])},
    "KraskyOvcarDogBreed": {"132624002": ("Krasky Ovcar dog breed", [7480])},
    "KravarskyCattleBreed": {"133469008": ("Kravarsky cattle breed", [7480])},
    "KrishnaValleyCattleBreed": {"133615003": ("Krishna Valley cattle breed", [7480])},
    "KrishnagariCattleBreed": {"133705001": ("Krishnagari cattle breed", [7480])},
    "KrolevetsPigBreed": {"133228001": ("Krolevets pig breed", [7480])},
    "KromfohrländerDogBreed": {"132625001": ("Kromfohrländer dog breed", [7480])},
    "KrskopoljePigBreed": {"132239006": ("Krskopolje pig breed", [7480])},
    "Krypton81m": {"61716009": ("Krypton^81m", [25])},
    "Kuchendorf": {"399080006": ("Kuchendorf", [4012])},
    "KuchendorfProjection": {
        "399080006": ("Kuchendorf projection (qualifier value)", [])
    },
    "KuchinoshimaCattleBreed": {"133470009": ("Kuchinoshima cattle breed", [7480])},
    "KuiMlkDogBreed": {"132391009": ("Kui Mlk dog breed", [7480])},
    "KulaPigBreed": {"132367000": ("Kula pig breed", [7480])},
    "KumamotoCattleBreed": {"133473006": ("Kumamoto cattle breed", [7480])},
    "KumauniCattleBreed": {"133706000": ("Kumauni cattle breed", [7480])},
    "KunekunePigBreed": {"132240008": ("Kunekune pig breed", [7480])},
    "KurdiCattleBreed": {"133252008": ("Kurdi cattle breed", [7480])},
    "KurganCattleBreed": {"131492003": ("Kurgan cattle breed", [7480])},
    "KuriCattleBreed": {"131434000": ("Kuri cattle breed", [7480])},
    "Kurzbauer": {"399332004": ("Kurzbauer", [4012])},
    "KurzbauerProjection": {
        "399332004": ("Kurzbauer projection (qualifier value)", [])
    },
    "KushumHorseBreed": {"133057005": ("Kushum horse breed", [7480])},
    "KustanaiHorseBreed": {"133058000": ("Kustanai horse breed", [7480])},
    "KuvaszDog": {"84548001": ("Kuvasz dog", [7480])},
    "KuznetHorseBreed": {"131925000": ("Kuznet horse breed", [7480])},
    "KwaiPigBreed": {"133170004": ("Kwai pig breed", [7480])},
    "KyiApsoDogBreed": {"132631003": ("Kyi Apso dog breed", [7480])},
    "KyiLeoDogBreed": {"132612009": ("Kyi-Leo dog breed", [7480])},
    "KyogaCattleBreed": {"131493008": ("Kyoga cattle breed", [7480])},
    "L1L2Disc": {"80064006": ("L1/L2 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "L1Vertebra": {"66794005": ("L1 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "L2L3Disc": {"67459009": ("L2/L3 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "L2Vertebra": {"14293000": ("L2 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "L3L4Disc": {"62551000": ("L3/L4 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "L3Vertebra": {"36470004": ("L3 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "L4L5Disc": {"84020006": ("L4/L5 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "L4Vertebra": {"11994002": ("L4 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "L5S1Disc": {"715056000": ("L5/S1 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "L5Vertebra": {"49668003": ("L5 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "LASEK": {"414582004": ("LASEK", [4234])},
    "LASIK": {"312965008": ("LASIK", [4234])},
    "LVPeakDiastolicTissueVelocityDuringAtrialSystole": {
        "399007006": (
            "LV Peak Diastolic Tissue Velocity During Atrial Systole",
            [12200, 12203],
        )
    },
    "LaManchaGoat": {"55530007": ("La Mancha goat (organism)", [])},
    "LaManchaGoatBreed": {"55530007": ("La Mancha goat breed", [7480])},
    "LaVelasquezCattleBreed": {"133477007": ("La Velasquez cattle breed", [7480])},
    "LaboratoryBiosafetyLevel": {
        "409599009": ("Laboratory biosafety level (observable entity)", [])
    },
    "LabradorRetriever": {"62137007": ("Labrador retriever (organism)", [])},
    "LabradorRetrieverDogBreed": {"62137007": ("Labrador retriever dog breed", [7480])},
    "LacmoidStain": {"35724001": ("lacmoid stain", [8112])},
    "LacombePig": {"30448006": ("Lacombe pig (organism)", [])},
    "LacombePigBreed": {"30448006": ("Lacombe pig breed", [7480])},
    "LaconiePigBreed": {"132093002": ("Laconie pig breed", [7480])},
    "LacrimalArtery": {
        "59749000": (
            "Lacrimal artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LacrimalArteryOfRightEye": {
        "128979005": (
            "Lacrimal artery of right eye",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LacrimalArteryRight": {
        "128979005": ("Lacrimal artery, right (body structure)", [])
    },
    "LacrimalBone": {"6229007": ("Lacrimal bone", [4028, 7151, 7192, 8134, 9514])},
    "LacrimalBoneStructure": {
        "6229007": ("Lacrimal bone structure (body structure)", [])
    },
    "LacrimalCaruncle": {"43045000": ("Lacrimal caruncle", [4209, 8134])},
    "LacrimalCaruncleStructure": {
        "43045000": ("Lacrimal caruncle structure (body structure)", [])
    },
    "LacrimalGland": {"13561001": ("Lacrimal gland", [4209, 8134])},
    "LacrimalGlandStructure": {
        "13561001": ("Lacrimal gland structure (body structure)", [])
    },
    "LacrimalSac": {"3954005": ("Lacrimal sac", [4209, 8134])},
    "LacrimalSacStructure": {
        "3954005": ("Lacrimal sac structure (body structure)", [])
    },
    "Lactate": {"83036002": ("Lactate", [218, 4032, 4033, 7180, 7186, 7469])},
    "LactatedRingerSolution": {
        "347379006": ("Lactated Ringer's solution (product)", [])
    },
    "LactatingAdenoma": {"128651002": ("Lactating adenoma", [6030, 6031])},
    "Lactocele": {"42385006": ("Lactocele (disorder)", [])},
    "LacunarLymphNode": {"360993001": ("lacunar lymph node", [7600, 8134])},
    "LadakhiCattleBreed": {"133707009": ("Ladakhi cattle breed", [7480])},
    "LaguneCattleBreed": {"133474000": ("Lagune cattle breed", [7480])},
    "LakelandTerrier": {"78214003": ("Lakeland terrier", [7480])},
    "LakenvelderCattleBreed": {"133475004": ("Lakenvelder cattle breed", [7480])},
    "LaminaOfVertebra": {"89340005": ("Lamina of vertebra", [6115, 8134])},
    "Laminar": {"88446008": ("Laminar (qualifier value)", [])},
    "Laminated": {"88446008": ("Laminated", [6132])},
    "LanYuPigBreed": {"132306006": ("Lan-Yu pig breed", [7480])},
    "LandaisHorseBreed": {"131926004": ("Landais horse breed", [7480])},
    "LandracePig": {"80131009": ("Landrace pig (organism)", [])},
    "LandracePigBreed": {"80131009": ("Landrace pig breed", [7480])},
    "LangHongPigBreed": {"132344002": ("Lang Hong pig breed", [7480])},
    "LangheSheepBreed": {"131756001": ("Langhe sheep breed", [7480])},
    "LantangPigBreed": {"132287001": ("Lantang pig breed", [7480])},
    "LaoshanPigBreed": {"132333005": ("Laoshan pig breed", [7480])},
    "LapermLonghairCatBreed": {"132668000": ("LaPerm Longhair cat breed", [7480])},
    "LapermShorthairCatBreed": {"132667005": ("LaPerm Shorthair cat breed", [7480])},
    "LaquerrierePierquin": {"399156009": ("Laquerriere-Pierquin", [4012])},
    "LaquerrierePierquinProjection": {
        "399156009": ("Laquerriere-Pierquin projection (qualifier value)", [])
    },
    "Large": {"255509001": ("Large", [252, 6118])},
    "LargeBlackPig": {"21021000": ("Large black pig (organism)", [])},
    "LargeBlackPigBreed": {"21021000": ("Large black pig breed", [7480])},
    "LargeBlackWhitePigBreed": {"132241007": ("Large Black-White pig breed", [7480])},
    "LargeCellCarcinoma": {"22687000": ("Large cell carcinoma", [638, 639])},
    "LargeIntestinalStructure": {
        "14742008": ("Large intestinal structure (body structure)", [])
    },
    "LargeIntestine": {
        "14742008": (
            "Large intestine",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LargeLiver": {"80515008": ("Large liver (disorder)", [])},
    "LargePolishLongEaredPigBreed": {
        "133230004": ("Large Polish Long-Eared pig breed", [7480])
    },
    "LargeRodLikeCalcification": {
        "129752009": ("Large rod-like calcification", [6010, 6011])
    },
    "LargeRodLikeRadiographicCalcification": {
        "129752009": ("Large rod-like radiographic calcification (finding)", [])
    },
    "LargeSpanishHoundDogBreed": {
        "132613004": ("Large Spanish Hound dog breed", [7480])
    },
    "LargeVWave": {"414576002": ("Large v-wave (finding)", [])},
    "LargeWhitePig": {"77236002": ("Large white pig (organism)", [])},
    "LargeWhitePigBreed": {"77236002": ("Large white pig breed", [7480])},
    "LaryngealEdema": {"51599000": ("Laryngeal edema", [60, 9300])},
    "LaryngealMaskAirwayInsertion": {
        "424979004": ("Laryngeal mask airway insertion (procedure)", [])
    },
    "LaryngealMaskAirwayLMA": {
        "424979004": ("Laryngeal Mask Airway (LMA)", [617, 618])
    },
    "LaryngealStructure": {"4596009": ("Laryngeal structure (body structure)", [])},
    "LaryngealVoiceFunction": {
        "43914001": ("Laryngeal voice function (observable entity)", [])
    },
    "Larynx": {
        "4596009": (
            "Larynx",
            [4, 645, 4009, 4028, 4030, 4031, 4040, 4042, 7151, 7192, 7601, 8134, 9514],
        )
    },
    "Laser": {"38586004": ("Laser", [8, 3411, 3429, 4051, 7151, 7193])},
    "LaserAssistedInSituKeratomileusis": {
        "312965008": ("Laser assisted in situ keratomileusis (procedure)", [])
    },
    "LaserAssistedSubepithelialKeratomileusis": {
        "414582004": ("Laser assisted subepithelial keratomileusis (procedure)", [])
    },
    "LaserDevice": {"122456005": ("Laser", [8125])},
    "LaserGeneratedElectromagneticRadiation": {
        "38586004": ("Laser-generated electromagnetic radiation (physical force)", [])
    },
    "LaserMicrodissection": {"433454009": ("Laser microdissection", [8110])},
    "LaserProcedureOnSkin": {"445907001": ("Laser procedure on skin", [4409])},
    "Lateral": {
        "49370004": (
            "Lateral",
            [2, 5, 26, 211, 212, 501, 3019, 4005, 6065, 12117, 12281],
        )
    },
    "Lateral45DegDorsal50DegProximalMediopalmarodistalOblique": {
        "442582006": (
            "Lateral 45 deg dorsal 50 deg proximal-mediopalmarodistal oblique",
            [7484],
        )
    },
    "Lateral45DegDorsal50DegProximalMedioplantarodistalObliq": {
        "442661006": (
            "Lateral 45 deg dorsal 50 deg proximal-medioplantarodistal obliq",
            [7484],
        )
    },
    "Lateral45DegreeDorsal50DegreeProximalMediopalmarodistalObliqueProjection": {
        "442582006": (
            "Lateral 45 degree dorsal 50 degree proximal-mediopalmarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "Lateral45DegreeDorsal50DegreeProximalMedioplantarodistalObliqueProjection": {
        "442661006": (
            "Lateral 45 degree dorsal 50 degree proximal-medioplantarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "LateralAxillaryLymphNode": {
        "33770006": ("lateral axillary lymph node", [7600, 8134])
    },
    "LateralCalfPerforator": {
        "714754004": (
            "Lateral calf perforator",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "LateralCanthus": {"61242005": ("Lateral Canthus", [1000, 1001, 1002, 8134])},
    "LateralCanthusStructure": {
        "61242005": ("Lateral canthus structure (body structure)", [])
    },
    "LateralCavalFontanProcedure": {
        "427886002": ("Lateral-Caval Fontan procedure", [12247])
    },
    "LateralCervicalLymphNode": {
        "68915008": ("lateral cervical lymph node", [7600, 8134])
    },
    "LateralCommonIliacLymphNode": {
        "41145006": ("lateral common iliac lymph node", [7600, 8134])
    },
    "LateralCorticospinalTract": {
        "461002": ("Lateral corticospinal tract", [7151, 7153, 7192, 8134, 9514])
    },
    "LateralDecubitus": {"32185000": ("lateral decubitus", [20])},
    "LateralDecubitusPosition": {
        "32185000": ("Lateral decubitus position (finding)", [])
    },
    "LateralExternalIliacLymphNode": {
        "40242007": ("lateral external iliac lymph node", [7600, 8134])
    },
    "LateralFuniculus": {"14892003": ("lateral funiculus", [7707, 7710, 8134])},
    "LateralFuniculusStructure": {
        "14892003": ("Lateral funiculus structure (body structure)", [])
    },
    "LateralHighRightAtrium": {
        "128592001": ("Lateral high right atrium", [3011, 8134])
    },
    "LateralIncisorRegion": {"699511000": ("Lateral incisor region", [4017])},
    "LateralIncisorRegionOfOralCavity": {
        "699511000": ("Lateral incisor region of oral cavity (body structure)", [])
    },
    "LateralJugularLymphNode": {
        "168159002": ("lateral jugular lymph node", [7600, 8134])
    },
    "LateralLemniscus": {"86136007": ("lateral lemniscus", [7701, 7710, 8134])},
    "LateralLemniscusStructure": {
        "86136007": ("Lateral lemniscus structure (body structure)", [])
    },
    "LateralMedial": {"399352003": ("lateral-medial", [501, 4010, 7302])},
    "LateralMedialEmissiveProjection": {"399300004": ("Lateral-medial", [26, 501])},
    "LateralMedialProjection": {
        "399352003": ("Lateral-medial projection (qualifier value)", [])
    },
    "LateralMitralAnnulus": {
        "399086000": ("Lateral Mitral Annulus", [12235, 12236, 12305])
    },
    "LateralMitralAnnulusStructure": {
        "399086000": ("Lateral mitral annulus structure (body structure)", [])
    },
    "LateralOblique": {"260427002": ("lateral oblique", [501, 4010])},
    "LateralPericardialLymphNode": {
        "196587000": ("lateral pericardial lymph node", [7600, 8134])
    },
    "LateralPlantarArtery": {
        "44830000": ("lateral plantar artery", [3604, 3606, 3630])
    },
    "LateralProjection": {"399067008": ("lateral", [501, 4010])},
    "LateralRetropharyngealLymphNode": {
        "167464007": ("lateral retropharyngeal lymph node", [7600, 8134])
    },
    "LateralRollingOfBreast": {
        "399197002": ("Lateral rolling of breast (procedure)", [])
    },
    "LateralVentricle": {
        "66720007": ("Lateral Ventricle", [7140, 7151, 7153, 7192, 8134, 9514, 12022])
    },
    "LateralVentricleStructure": {
        "66720007": ("Lateral ventricle structure (body structure)", [])
    },
    "LateralVesicularLymphNode": {
        "278672000": ("lateral vesicular lymph node", [7600, 8134])
    },
    "Laterality": {"272741003": ("Laterality", [210])},
    "LateroMedial": {"399352003": ("latero-medial", [4014])},
    "LateroMedialOblique": {"399099002": ("latero-medial oblique", [501, 4010, 4014])},
    "LateroMedialObliqueEmissiveProjection": {
        "399159002": ("Latero-medial oblique", [26, 501])
    },
    "LateroMedialObliqueProjection": {
        "399099002": ("Latero-medial oblique projection (qualifier value)", [])
    },
    "LaterodorsoproximalMediopalmarodistalOblique": {
        "442630002": ("Laterodorsoproximal-mediopalmarodistal oblique", [7484])
    },
    "LaterodorsoproximalMediopalmarodistalObliqueProjection": {
        "442630002": (
            "Laterodorsoproximal-mediopalmarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "LaterodorsoproximalMedioplantarodistalOblique": {
        "442631003": ("Laterodorsoproximal-medioplantarodistal oblique", [7484])
    },
    "LaterodorsoproximalMedioplantarodistalObliqueProjection": {
        "442631003": (
            "Laterodorsoproximal-medioplantarodistal oblique projection (qualifier value)",
            [],
        )
    },
    "Lateromedial": {"399352003": ("Lateromedial", [7484])},
    "LatiSheepBreed": {"131754003": ("Lati sheep breed", [7480])},
    "LatissimusDorsiMuscle": {
        "15665001": (
            "Latissimus dorsi muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "LatukaCattleBreed": {"133708004": ("Latuka cattle breed", [7480])},
    "LatvianBlueRoanCattleBreed": {
        "133476003": ("Latvian Blue Roan cattle breed", [7480])
    },
    "LatvianBrownCattleBreed": {"133243004": ("Latvian Brown cattle breed", [7480])},
    "LatvianHorseBreed": {"133059008": ("Latvian horse breed", [7480])},
    "LatvianWhitePigBreed": {"132275008": ("Latvian White pig breed", [7480])},
    "Lauenstein": {"399169008": ("Lauenstein", [4012])},
    "LauensteinProjection": {
        "399169008": ("Lauenstein projection (qualifier value)", [])
    },
    "LaviniaCattleBreed": {"133814001": ("Lavinia cattle breed (organism)", [])},
    "LaviniaXZebuCattleBreed": {"133814001": ("Lavinia X zebu cattle breed", [7480])},
    "Law": {"399206007": ("Law", [4012])},
    "LawProjection": {"399206007": ("Law projection (qualifier value)", [])},
    "Lawrence": {"399179005": ("Lawrence", [4012])},
    "LawrenceProjection": {"399179005": ("Lawrence projection (qualifier value)", [])},
    "Lead": {"88488004": ("Lead", [10006, 10067])},
    "LeakageOfConduitFromRightAtriumToPulmonaryArtery": {
        "443461006": (
            "Leakage of conduit from right atrium to pulmonary artery (disorder)",
            [],
        )
    },
    "LebaneseCattleBreed": {"133484004": ("Lebanese cattle breed", [7480])},
    "LebedinCattleBreed": {"133485003": ("Lebedin cattle breed", [7480])},
    "LeeSungPigBreed": {"132305005": ("Lee-Sung pig breed", [7480])},
    "Left": {"7771000": ("Left", [2, 5, 211, 244, 245, 247, 3019, 12117])},
    "Left20DegreeDorsalRightVentralOblique": {
        "442748002": ("Left 20 degree dorsal-right ventral oblique", [7484])
    },
    "Left20DegreeDorsalRightVentralObliqueProjection": {
        "442748002": (
            "Left 20 degree dorsal-right ventral oblique projection (qualifier value)",
            [],
        )
    },
    "Left20DegreeRostralRightCaudalOblique": {
        "442632005": ("Left 20 degree rostral-right caudal oblique", [7484])
    },
    "Left20DegreeRostralRightCaudalObliqueProjection": {
        "442632005": (
            "Left 20 degree rostral-right caudal oblique projection (qualifier value)",
            [],
        )
    },
    "Left20DegreeVentralRightDorsalOblique": {
        "442636008": ("Left 20 degree ventral-right dorsal oblique", [7484])
    },
    "Left20DegreeVentralRightDorsalObliqueProjection": {
        "442636008": (
            "Left 20 degree ventral-right dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "Left30DegreeCaudalRightRostralOblique": {
        "442596000": ("Left 30 degree caudal-right rostral oblique", [7484])
    },
    "Left30DegreeCaudalRightRostralObliqueProjection": {
        "442596000": (
            "Left 30 degree caudal-right rostral oblique projection (qualifier value)",
            [],
        )
    },
    "Left45DegreeDorsalRightVentralOblique": {
        "442598004": ("Left 45 degree dorsal-right ventral oblique", [7484])
    },
    "Left45DegreeDorsalRightVentralObliqueProjection": {
        "442598004": (
            "Left 45 degree dorsal-right ventral oblique projection (qualifier value)",
            [],
        )
    },
    "Left45DegreeVentralRightDorsalOblique": {
        "442599007": ("Left 45 degree ventral-right dorsal oblique", [7484])
    },
    "Left45DegreeVentralRightDorsalObliqueProjection": {
        "442599007": (
            "Left 45 degree ventral-right dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "LeftAnteriorApicalPeripheralZoneOfProstate": {
        "716937001": (
            "Left anterior apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftAnteriorApicalTransitionZoneOfProstate": {
        "716931000": (
            "Left anterior apical transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftAnteriorBasalPeripheralZoneOfProstate": {
        "716905002": (
            "Left anterior basal peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftAnteriorBasalTransitionZoneOfProstate": {
        "716897000": (
            "Left anterior basal transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftAnteriorDescendingCoronaryArtery": {
        "59438005": (
            "Left Anterior Descending Coronary Artery",
            [3015, 3016, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "LeftAnteriorDivisionOfLeftBranchAtrioventricularBundle": {
        "84654008": (
            "Left anterior division of left branch atrioventricular bundle",
            [3011, 8134],
        )
    },
    "LeftAnteriorFascicularBlock": {
        "37760005": ("Left anterior fascicular block", [3230])
    },
    "LeftAnteriorMiddlePeripheralZoneOfProstate": {
        "716920008": (
            "Left anterior middle peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftAnteriorMiddleTransitionZoneOfProstate": {
        "716914007": (
            "Left anterior middle transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftAnteriorOblique": {"399135007": ("left anterior oblique", [501, 3466, 4010])},
    "LeftAnteriorObliqueEmissiveProjection": {
        "399074003": ("Left anterior oblique", [26, 501])
    },
    "LeftAnteriorObliqueProjection": {
        "399135007": ("Left anterior oblique projection (qualifier value)", [])
    },
    "LeftApicalAnteriorFibromuscularStromaOfProstate": {
        "716927006": (
            "Left apical anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftApicalPeripheralZoneOfProstate": {
        "716935009": ("Left apical peripheral zone of prostate", [6300, 6302])
    },
    "LeftApicalTransitionZoneOfProstate": {
        "716929009": ("Left apical transition zone of prostate", [6300, 6302, 6303])
    },
    "LeftAtrialStructure": {"82471001": ("Left atrial structure (body structure)", [])},
    "LeftAtrium": {
        "82471001": (
            "Left atrium",
            [
                4,
                3010,
                3011,
                3462,
                3608,
                3630,
                4042,
                7151,
                7192,
                8134,
                9514,
                12284,
                12305,
            ],
        )
    },
    "LeftAtriumPressureWaveform": {
        "128441007": ("Left atrium pressure waveform", [3003])
    },
    "LeftAtriumPressureWaveformFunction": {
        "128441007": ("Left atrium pressure waveform, function (observable entity)", [])
    },
    "LeftAtriumSystolicVolume": {"399235004": ("Left Atrium Systolic Volume", [12205])},
    "LeftAuricularAppendage": {
        "33626005": (
            "Left auricular appendage",
            [4, 3010, 3011, 4042, 7151, 7192, 8134, 9514, 12284],
        )
    },
    "LeftBasalAnteriorFibromuscularStromaOfProstate": {
        "716893001": (
            "Left basal anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftBasalPartTransitionZoneOfProstate": {
        "716895008": ("Left basal part transition zone of prostate", [6300, 6302, 6303])
    },
    "LeftBasalPeripheralZoneOfProstate": {
        "716903009": ("Left basal peripheral zone of prostate", [6300, 6302])
    },
    "LeftBranchOfAtrioventricularBundle": {
        "74031005": ("Left branch of atrioventricular bundle", [3011, 8134])
    },
    "LeftBranchOfHepaticArtery": {
        "21807003": (
            "Left Branch of Hepatic Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "LeftBreast": {"80248007": ("Left breast", [6022, 6023])},
    "LeftBreastStructure": {"80248007": ("Left breast structure (body structure)", [])},
    "LeftBundleBranchBlock": {
        "63467002": ("Left bundle branch block", [3201, 3205, 3230, 12246])
    },
    "LeftBundleBranchStructure": {
        "74031005": ("Left bundle branch structure (body structure)", [])
    },
    "LeftCaudalRightRostralOblique": {
        "442638009": ("Left caudal-right rostral oblique", [7484])
    },
    "LeftCaudalRightRostralObliqueProjection": {
        "442638009": (
            "Left caudal-right rostral oblique projection (qualifier value)",
            [],
        )
    },
    "LeftCommonCarotidArtery": {"113263003": ("Left Common Carotid Artery", [12291])},
    "LeftCommonCarotidArteryStructure": {
        "113263003": ("Left common carotid artery structure (body structure)", [])
    },
    "LeftCoronaryArtery": {
        "50018008": (
            "Left Coronary Artery",
            [3015, 3016, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "LeftCoronaryArteryMainStem": {
        "3227004": ("Left coronary artery main stem (body structure)", [])
    },
    "LeftCoronaryArteryStructure": {
        "50018008": ("Left coronary artery structure (body structure)", [])
    },
    "LeftCoronaryDominance": {"253729004": ("Left Coronary Dominance", [3710])},
    "LeftDominantCoronarySystem": {
        "253729004": ("Left dominant coronary system (disorder)", [])
    },
    "LeftDorsalRightVentralOblique": {
        "442644008": ("Left dorsal-right ventral oblique", [7484])
    },
    "LeftDorsalRightVentralObliqueProjection": {
        "442644008": (
            "Left dorsal-right ventral oblique projection (qualifier value)",
            [],
        )
    },
    "LeftDowngaze": {"255523004": ("Left downgaze", [4201])},
    "LeftFemoralArtery": {
        "113270003": (
            "Left femoral artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LeftGaze": {"255530005": ("Left gaze", [4201])},
    "LeftGlennShuntProcedure": {"444178004": ("Left Glenn shunt procedure", [12247])},
    "LeftHepaticVein": {
        "273202007": (
            "Left Hepatic Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "LeftHypochondriacRegion": {
        "133945003": ("Left hypochondriac region", [4, 7151, 7192, 8134, 9514])
    },
    "LeftHypochondriacRegionStructure": {
        "133945003": ("Left hypochondriac region structure (body structure)", [])
    },
    "LeftInferiorPulmonaryVein": {
        "51249003": ("Left Inferior Pulmonary Vein", [12280])
    },
    "LeftInguinalRegion": {
        "85119005": ("Left inguinal region", [4, 7151, 7192, 8134, 9514])
    },
    "LeftInguinalRegionStructure": {
        "85119005": ("Left inguinal region structure (body structure)", [])
    },
    "LeftLateral": {"399173006": ("left lateral", [501, 3466, 4010])},
    "LeftLateralDecubitus": {"102536004": ("left lateral decubitus", [20, 6206])},
    "LeftLateralDecubitusPosition": {
        "102536004": ("Left lateral decubitus position (finding)", [])
    },
    "LeftLateralEmissiveProjection": {
        "399118008": ("Left lateral projection", [26, 501])
    },
    "LeftLateralMiddlePeripheralZoneOfProstate": {
        "716918005": ("Left lateral middle peripheral zone of prostate", [6300, 6302])
    },
    "LeftLateralProjection": {
        "399173006": ("Left lateral projection (qualifier value)", [])
    },
    "LeftLobeOfThyroid": {"79163004": ("Left lobe of thyroid", [12131])},
    "LeftLowerQuadrantOfAbdomen": {
        "68505006": ("Left lower quadrant of abdomen", [4, 7151, 7192, 8134, 9514])
    },
    "LeftLowerSegment": {"264068005": ("Left Lower Segment", [12219, 12237])},
    "LeftLumbarRegion": {
        "1017210004": ("Left lumbar region", [4, 7151, 7192, 8134, 9514])
    },
    "LeftLumbarRegion_Inactive": {
        "133943005": ("Left lumbar region (body structure)", [])
    },
    "LeftMainBranchOfPortalVein": {
        "70253006": (
            "Left Main Branch of Portal Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "LeftMainBronchus": {
        "75245000": (
            "Left main bronchus",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "LeftMainBronchusStructure": {
        "75245000": ("Left main bronchus structure (body structure)", [])
    },
    "LeftMainCoronaryArtery": {
        "3227004": (
            "Left Main Coronary Artery",
            [
                3014,
                3015,
                3016,
                3604,
                3827,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12292,
            ],
        )
    },
    "LeftMainCoronaryArteryDisease": {
        "371804009": ("Left main coronary artery disease", [3728])
    },
    "LeftMainCoronaryArteryOstium": {
        "1256091001": (
            "Left Main Coronary Artery Ostium",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "LeftMiddleAnteriorFibromuscularStromaOfProstate": {
        "716910003": (
            "Left middle anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftMiddleTransitionZoneOfProstate": {
        "716912006": ("Left middle transition zone of prostate", [6300, 6302, 6303])
    },
    "LeftOblique": {"399184004": ("left oblique", [501, 4010])},
    "LeftObliqueProjection": {
        "399184004": ("Left oblique projection (qualifier value)", [])
    },
    "LeftPleura": {"53727004": ("Left Pleura", [12294])},
    "LeftPleuraStructure": {"53727004": ("Left pleura structure (body structure)", [])},
    "LeftPosteriorApicalTransitionZoneOfProstate": {
        "716933002": (
            "Left posterior apical transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftPosteriorBasalTransitionZoneOfProstate": {
        "716899002": (
            "Left posterior basal transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftPosteriorDescendingCircumflexCoronaryArtery": {
        "91760001": (
            "Left Posterior Descending Circumflex Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "LeftPosteriorDivisionOfLeftBranchAtrioventricularBundle": {
        "91085002": (
            "Left posterior division of left branch atrioventricular bundle",
            [3011, 8134],
        )
    },
    "LeftPosteriorFascicularBlock": {
        "62026008": ("Left posterior fascicular block", [3230])
    },
    "LeftPosteriorMiddleTransitionZoneOfProstate": {
        "716916009": (
            "Left posterior middle transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "LeftPosteriorOblique": {"399006002": ("left posterior oblique", [501, 4010])},
    "LeftPosteriorObliqueEmissiveProjection": {
        "399136008": ("Left posterior oblique", [26, 501])
    },
    "LeftPosteriorObliqueProjection": {
        "399006002": ("Left posterior oblique projection (qualifier value)", [])
    },
    "LeftPosterolateralApicalPeripheralZoneOfProstate": {
        "716939003": (
            "Left posterolateral apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftPosterolateralBasalPeripheralZoneOfProstate": {
        "716907005": (
            "Left posterolateral basal peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftPosterolateralCircumflexCoronaryArtery": {
        "57823005": (
            "Left Posterolateral Circumflex Coronary Artery",
            [3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "LeftPosterolateralMiddlePeripheralZoneOfProstate": {
        "716922000": (
            "Left posterolateral middle peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftPosteromedialApicalPeripheralZoneOfProstate": {
        "716941002": (
            "Left posteromedial apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "LeftPosteromedialBasalPeripheralZoneOfProstate": {
        "836427009": (
            "Left posteromedial basal peripheral zone of prostate",
            [6300, 6304],
        )
    },
    "LeftPosteromedialMiddlePeripheralZoneOfProstate": {
        "716924004": (
            "Left posteromedial middle peripheral zone of prostate",
            [6300, 6301, 6302, 6303, 6304],
        )
    },
    "LeftPulmonaryArtery": {
        "50408007": (
            "Left pulmonary artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3829,
                4042,
                7151,
                7192,
                8134,
                9514,
                12280,
                12305,
            ],
        )
    },
    "LeftPulmonaryVein": {
        "27706005": ("Left Pulmonary Vein", [3827, 3840, 7151, 7192, 9514])
    },
    "LeftRightLateral": {"399198007": ("Left-right lateral", [7484])},
    "LeftRostralRightCaudalOblique": {
        "442645009": ("Left rostral-right caudal oblique", [7484])
    },
    "LeftRostralRightCaudalObliqueProjection": {
        "442645009": (
            "Left rostral-right caudal oblique projection (qualifier value)",
            [],
        )
    },
    "LeftSeminalVesicle": {
        "42320003": ("Left seminal vesicle", [6300, 6301, 6302, 6303, 6304])
    },
    "LeftSidedBidirectionalGlennShuntProcedure": {
        "443989003": ("Left-sided bidirectional Glenn shunt procedure", [12247])
    },
    "LeftSinusOfValsalva": {"36371001": ("Left Sinus of Valsalva", [12291])},
    "LeftSubclavianArtery": {"85235006": ("Left Subclavian Artery", [12291])},
    "LeftSuperiorPulmonaryVein": {
        "43863001": ("Left Superior Pulmonary Vein", [12280])
    },
    "LeftSuperiorVenaCava": {"9642004": ("Left Superior Vena Cava", [12282])},
    "LeftToRightCardiovascularShunt": {
        "66130006": ("Left to right cardiovascular shunt", [12221])
    },
    "LeftUpgaze": {"255525006": ("Left upgaze", [4201])},
    "LeftUpperQuadrantOfAbdomen": {
        "86367003": ("Left upper quadrant of abdomen", [4, 7151, 7192, 8134, 9514])
    },
    "LeftUpperSegment": {"255482005": ("Left Upper Segment", [12219, 12237])},
    "LeftVentralRightDorsalOblique": {
        "442739007": ("Left ventral-right dorsal oblique", [7484])
    },
    "LeftVentralRightDorsalObliqueProjection": {
        "442739007": (
            "Left ventral-right dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "LeftVentricle": {
        "87878005": (
            "Left ventricle",
            [
                4,
                3010,
                3011,
                3462,
                3463,
                3609,
                3630,
                4042,
                7151,
                7152,
                7192,
                8134,
                9514,
                12223,
                12236,
                12287,
                12288,
                12305,
            ],
        )
    },
    "LeftVentricleAnteriorSegment": {
        "284355001": ("Left ventricle anterior segment", [3232])
    },
    "LeftVentricleApex": {"128564006": ("Left ventricle apex", [3609, 3630])},
    "LeftVentricleApicalAnteriorSegment": {
        "264844003": (
            "left ventricle apical anterior segment",
            [3717, 3781, 3784, 3785, 12305],
        )
    },
    "LeftVentricleApicalInferiorSegment": {
        "264849008": (
            "left ventricle apical inferior segment",
            [3717, 3781, 3784, 3785, 12305],
        )
    },
    "LeftVentricleApicalLateralSegment": {
        "264853005": ("left ventricle apical lateral segment", [3717, 3784, 12305])
    },
    "LeftVentricleApicalSegment": {
        "128564006": ("Left ventricle apical segment", [3232])
    },
    "LeftVentricleApicalSeptalSegment": {
        "264845002": ("left ventricle apical septal segment", [3717, 3718, 3784, 12305])
    },
    "LeftVentricleBasalAnteriorSegment": {
        "264850008": (
            "left ventricle basal anterior segment",
            [3717, 3718, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalAnterolateralSegment": {
        "396654005": (
            "left ventricle basal anterolateral segment",
            [3717, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalAnteroseptalSegment": {
        "396482007": (
            "left ventricle basal anteroseptal segment",
            [3717, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalInferiorSegment": {
        "264846001": (
            "left ventricle basal inferior segment",
            [3717, 3718, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalInferolateralSegment": {
        "396652009": (
            "left ventricle basal inferolateral segment",
            [3717, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalInferoseptalSegment": {
        "396646008": (
            "left ventricle basal inferoseptal segment",
            [3717, 3781, 3782, 12305],
        )
    },
    "LeftVentricleBasalLateralSegment": {
        "277631004": ("Left ventricle basal lateral segment (body structure)", [])
    },
    "LeftVentricleEpicardialDiastolicAreaPsaxPapView": {
        "399293008": (
            "Left Ventricle Epicardial Diastolic Area, psax pap view",
            [12200, 12240],
        )
    },
    "LeftVentricleInferiorSegment": {
        "284357009": ("Left ventricle inferior segment", [3232])
    },
    "LeftVentricleInflow": {
        "70238003": (
            "Left ventricle inflow",
            [4, 3010, 3011, 3609, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LeftVentricleLateralSegment": {
        "284358004": ("Left ventricle lateral segment", [3232])
    },
    "LeftVentricleMidAnteriorSegment": {
        "264848000": ("left ventricle mid anterior segment", [3717, 3781, 3783, 12305])
    },
    "LeftVentricleMidAnterolateralSegment": {
        "396656007": (
            "left ventricle mid anterolateral segment",
            [3717, 3781, 3783, 12305],
        )
    },
    "LeftVentricleMidAnteroseptalSegment": {
        "396647004": (
            "left ventricle mid anteroseptal segment",
            [3717, 3781, 3783, 12305],
        )
    },
    "LeftVentricleMidInferiorSegment": {
        "264847005": ("left ventricle mid inferior segment", [3717, 3781, 3783, 12305])
    },
    "LeftVentricleMidInferolateralSegment": {
        "396655006": (
            "left ventricle mid inferolateral segment",
            [3717, 3781, 3783, 12305],
        )
    },
    "LeftVentricleMidInferoseptalSegment": {
        "396649001": (
            "left ventricle mid inferoseptal segment",
            [3717, 3781, 3783, 12305],
        )
    },
    "LeftVentricleMyocardium": {"49848007": ("Left Ventricle Myocardium", [12305])},
    "LeftVentricleOutflowTract": {
        "13418002": (
            "Left ventricle outflow tract",
            [
                4,
                3010,
                3011,
                3609,
                3610,
                3630,
                3701,
                4042,
                7151,
                7192,
                8134,
                9514,
                12223,
                12236,
                12243,
                12280,
                12305,
            ],
        )
    },
    "LeftVentriclePressureWaveform": {
        "128438003": ("Left ventricle pressure waveform", [3003])
    },
    "LeftVentriclePressureWaveformFunction": {
        "128438003": (
            "Left ventricle pressure waveform, function (observable entity)",
            [],
        )
    },
    "LeftVentricleSemiMajorAxisDiastolicDimension": {
        "399063007": (
            "Left Ventricle Semi-major Axis Diastolic Dimension",
            [12200, 12201],
        )
    },
    "LeftVentricleSeptalSegment": {
        "284356000": ("Left ventricle septal segment", [3232])
    },
    "LeftVentricleToPulmonaryArteryConduitAnastomosis": {
        "443696003": ("Left Ventricle to Pulmonary Artery Conduit Anastomosis", [12288])
    },
    "LeftVentricleTruncatedSemiMajorAxisDiastolicDimension": {
        "399309003": (
            "Left Ventricle Truncated Semi-major Axis Diastolic Dimension",
            [12200, 12201],
        )
    },
    "LeftVentricularAreaFractionalChange": {
        "399287000": ("Left ventricular area fractional change (observable entity)", [])
    },
    "LeftVentricularAssistDevice": {
        "360066001": ("Left Ventricular Assist Device", [3553, 3777, 7151, 7157, 7193])
    },
    "LeftVentricularBasalSeptalSegment": {
        "277630003": ("Left ventricular basal septal segment (body structure)", [])
    },
    "LeftVentricularCavitySize": {
        "250929008": ("Left ventricular cavity size (observable entity)", [])
    },
    "LeftVentricularDiastolicArea": {
        "399109006": ("Left Ventricular Diastolic Area", [12200, 12240])
    },
    "LeftVentricularEndDiastolicPressure": {
        "276781007": ("Left Ventricular End-Diastolic Pressure", [3641])
    },
    "LeftVentricularEpicardialDiastolicAreaPsaxPapView": {
        "399293008": (
            "Left ventricular epicardial diastolic area, psax pap view (observable entity)",
            [],
        )
    },
    "LeftVentricularFractionalAreaChange": {
        "399287000": ("Left Ventricular Fractional Area Change", [12200, 12240])
    },
    "LeftVentricularFunction": {
        "250907009": ("Left ventricular function (observable entity)", [])
    },
    "LeftVentricularIndexOfMyocardialPerformance": {
        "399266005": (
            "Left Ventricular Index of Myocardial Performance",
            [12200, 12203],
        )
    },
    "LeftVentricularIndexOfMyocardiumPerformance": {
        "399266005": (
            "Left ventricular index of myocardium performance (observable entity)",
            [],
        )
    },
    "LeftVentricularIsovolumicContractionTime": {
        "399051002": ("Left Ventricular Isovolumic Contraction Time", [12200, 12203])
    },
    "LeftVentricularPeakDiastolicTissueVelocityDuringAtrialSystole": {
        "399007006": (
            "Left ventricular peak diastolic tissue velocity during atrial systole (observable entity)",
            [],
        )
    },
    "LeftVentricularPeakEarlyDiastolicTissueVelocity": {
        "399133000": (
            "Left Ventricular Peak Early Diastolic Tissue Velocity",
            [12200, 12203],
        )
    },
    "LeftVentricularPeakSystolicTissueVelocity": {
        "399167005": ("Left Ventricular Peak Systolic Tissue Velocity", [12200, 12203])
    },
    "LeftVentricularPosterobasalSegment": {
        "408720008": ("Left ventricular posterobasal segment (body structure)", [])
    },
    "LeftVentricularSemiMajorAxisDiastolicDimension": {
        "399063007": (
            "Left ventricular semi-major axis diastolic dimension (observable entity)",
            [],
        )
    },
    "LeftVentricularSize": {
        "250924003": ("Left ventricular size (observable entity)", [])
    },
    "LeftVentricularStructure": {
        "87878005": ("Left ventricular structure (body structure)", [])
    },
    "LeftVentricularSystolicArea": {
        "399030000": ("Left Ventricular Systolic Area", [12200, 12240])
    },
    "LeftVentricularSystolicPressure": {
        "276780008": ("Left Ventricular Systolic Pressure", [3641])
    },
    "LeftVentricularTruncatedSemiMajorAxisDiastolicDimension": {
        "399309003": (
            "Left ventricular truncated semi-major axis diastolic dimension (observable entity)",
            [],
        )
    },
    "LeftVentricularWallMotion": {
        "250909007": ("Left ventricular wall motion (observable entity)", [])
    },
    "LeftVentriculogram": {"265484009": ("Left ventriculogram (procedure)", [])},
    "LeftVentriculography": {"265484009": ("Left Ventriculography", [3405, 3428])},
    "LegUp": {"128964003": ("Leg up", [91, 3271])},
    "LegUpPhysiologicChallenge": {
        "128964003": ("Leg up physiologic challenge (procedure)", [])
    },
    "LeicesterSheep": {"6431001": ("Leicester sheep (organism)", [])},
    "LeicesterSheepBreed": {"6431001": ("Leicester sheep breed", [7480])},
    "LeicomaPigBreed": {"132361004": ("Leicoma pig breed", [7480])},
    "LeineschafSheepBreed": {"131744002": ("Leineschaf sheep breed", [7480])},
    "Leiomyoma": {"1162890002": ("Leiomyoma", [6030, 6031])},
    "LeiomyomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "44598004": (
            "Leiomyoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "Leiomyosarcoma": {"1163022006": ("Leiomyosarcoma", [6030, 6033])},
    "LeiomyosarcomaNoSubtype": {
        "51549004": ("Leiomyosarcoma, no subtype (morphologic abnormality)", [])
    },
    "LeiqiongCattleBreed": {"133662005": ("Leiqiong cattle breed", [7480])},
    "LeishmanStain": {"255802001": ("Leishman stain", [8112])},
    "Length": {
        "410668003": (
            "Length",
            [
                218,
                3423,
                6165,
                7469,
                7470,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
                12304,
            ],
        )
    },
    "LengthOfSegment": {"414599003": ("Length of Segment", [12119, 12122])},
    "LengthOfVesselSegment": {
        "414599003": ("Length of vessel segment (observable entity)", [])
    },
    "LengthProperty": {"410668003": ("Length property (qualifier value)", [])},
    "Lens": {"78076003": ("Lens", [4209, 4211, 8134])},
    "LensOpacity": {"193570009": ("Lens Opacity", [4222])},
    "LeonardGeorge": {"398996004": ("Leonard-George", [4012])},
    "LeonardGeorgeProjection": {
        "398996004": ("Leonard-George projection (qualifier value)", [])
    },
    "LeonbergerDogBreed": {"132622003": ("Leonberger dog breed (organism)", [])},
    "LeopardCurDogBreed": {"132403002": ("Leopard Cur dog breed", [7480])},
    "LepingPigBreed": {"132302008": ("Leping pig breed", [7480])},
    "Lesion": {
        "52988006": ("Lesion", [3487, 4266, 6335, 6336, 6337, 7151, 7159, 7194])
    },
    "LesionOfBrain": {"301766008": ("Lesion of Brain", [7151, 7194, 7199])},
    "LesionWithCircumscribedMargin": {
        "129738007": ("Lesion with circumscribed margin (finding)", [])
    },
    "LesionWithEqualIsodenseRadiographicDensity": {
        "129745007": ("Lesion with equal (isodense) radiographic density (finding)", [])
    },
    "LesionWithFatContainingRadiolucentDensity": {
        "129747004": ("Lesion with fat containing (radiolucent) density (finding)", [])
    },
    "LesionWithHighRadiographicDensity": {
        "129744006": ("Lesion with high radiographic density (finding)", [])
    },
    "LesionWithIndistinctMargin": {
        "129741003": ("Lesion with indistinct margin (finding)", [])
    },
    "LesionWithLowRadiographicDensityNotContainingFat": {
        "129746008": (
            "Lesion with low radiographic density (not containing fat) (finding)",
            [],
        )
    },
    "LesionWithMicrolobulatedMargin": {
        "129739004": ("Lesion with microlobulated margin (finding)", [])
    },
    "LesionWithObscuredMargin": {
        "129740002": ("Lesion with obscured margin (finding)", [])
    },
    "LesionWithSpiculatedMargin": {
        "129742005": ("Lesion with spiculated margin (finding)", [])
    },
    "LessDefined": {"129728002": ("Less defined", [6002, 6003, 6134])},
    "LessIschemia": {"429232006": ("Less ischemia", [3235])},
    "LesserCaucasusCattleBreed": {
        "133486002": ("Lesser Caucasus cattle breed", [7480])
    },
    "LesserSaphenousVein": {
        "26805005": (
            "Lesser Saphenous Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "LesserTrochanter": {"55499008": ("Lesser trochanter", [1000, 1001, 1005, 8134])},
    "Leukemia": {"93143009": ("Leukemia", [638, 639])},
    "LeukemiaDisease": {"93143009": ("Leukemia, disease (disorder)", [])},
    "LevatoresCostarumMuscles": {
        "73930003": (
            "Levatores costarum muscles",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "LevelOfC2C3IntervertebralDisc": {
        "243902007": ("Level of C2/C3 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfC3C4IntervertebralDisc": {
        "243903002": ("Level of C3/C4 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfC4C5IntervertebralDisc": {
        "243904008": ("Level of C4/C5 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfC5C6IntervertebralDisc": {
        "243905009": ("Level of C5/C6 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfC6C7IntervertebralDisc": {
        "243906005": ("Level of C6/C7 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfC7T1IntervertebralDisc": {
        "243925008": ("Level of C7/T1 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfL1L2IntervertebralDisc": {
        "243920003": ("Level of L1/L2 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfL2L3IntervertebralDisc": {
        "243921004": ("Level of L2/L3 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfL3L4IntervertebralDisc": {
        "243922006": ("Level of L3/L4 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfL4L5IntervertebralDisc": {
        "243923001": ("Level of L4/L5 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfL5S1IntervertebralDisc": {
        "243927000": ("Level of L5/S1 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT10T11IntervertebralDisc": {
        "243917006": ("Level of T10/T11 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT11T12IntervertebralDisc": {
        "243918001": ("Level of T11/T12 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT12L1IntervertebralDisc": {
        "243926009": ("Level of T12/L1 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT1T2IntervertebralDisc": {
        "243908006": ("Level of T1/T2 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT2T3IntervertebralDisc": {
        "243909003": ("Level of T2/T3 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT3T4IntervertebralDisc": {
        "243910008": ("Level of T3/T4 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT4T5IntervertebralDisc": {
        "243911007": ("Level of T4/T5 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT5T6IntervertebralDisc": {
        "243912000": ("Level of T5/T6 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT6T7IntervertebralDisc": {
        "243913005": ("Level of T6/T7 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT7T8IntervertebralDisc": {
        "243914004": ("Level of T7/T8 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT8T9IntervertebralDisc": {
        "243915003": ("Level of T8/T9 intervertebral disc", [1000, 1001, 1003])
    },
    "LevelOfT9T10IntervertebralDisc": {
        "243916002": ("Level of T9/T10 intervertebral disc", [1000, 1001, 1003])
    },
    "Lewis": {"399037002": ("Lewis", [4012])},
    "LewisProjection": {"399037002": ("Lewis projection (qualifier value)", [])},
    "LewitzerHorseBreed": {"131927008": ("Lewitzer horse breed", [7480])},
    "LhasaApso": {"36438004": ("Lhasa apso", [7480])},
    "LiangGuangSmallSpottedGuangdongSmallEarPigBreed": {
        "132288006": (
            "Liang Guang Small Spotted, Guangdong Small Ear pig breed",
            [7480],
        )
    },
    "LiangGuangSmallSpottedLuchuanPigBreed": {
        "132280004": ("Liang Guang Small Spotted, Luchuan pig breed", [7480])
    },
    "LiangGuangSmallSpottedPigBreed": {
        "132158006": ("Liang Guang Small Spotted pig breed", [7480])
    },
    "LiaoningBlackPigBreed": {"132129008": ("Liaoning Black pig breed", [7480])},
    "LiberianDwarfCattleBreed": {"133487006": ("Liberian Dwarf cattle breed", [7480])},
    "LibyanCattleBreed": {"133488001": ("Libyan cattle breed", [7480])},
    "LichuanHorseBreed": {"131928003": ("Lichuan horse breed", [7480])},
    "Lidocaine": {"387480006": ("Lidocaine", [623, 630])},
    "LidocaineContainingProduct": {"82573000": ("Lidocaine", [10])},
    "LidocaineHydrochloride": {"61773008": ("Lidocaine hydrochloride", [10])},
    "LidocainePrilocaine": {"346553009": ("Lidocaine + Prilocaine", [623, 630])},
    "Ligament": {"52082005": ("Ligament", [7151, 7166, 7191])},
    "LigamentumArteriosum": {
        "2160002": (
            "Ligamentum arteriosum",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "LightEmittingDiode": {"445683001": ("Light emitting diode", [8125])},
    "LightEmittingDiodeLampMicroscopeIlluminator": {
        "445683001": (
            "Light emitting diode lamp microscope illuminator (physical object)",
            [],
        )
    },
    "LightGreenSFStain": {"89139001": ("light green SF stain", [8112])},
    "LightMicroscopy": {"104157003": ("Light microscopy", [8131])},
    "LijiangHorseBreed": {"131929006": ("Lijiang horse breed", [7480])},
    "Lilienfeld": {"399342002": ("Lilienfeld", [4012])},
    "LilienfeldProjection": {
        "399342002": ("Lilienfeld projection (qualifier value)", [])
    },
    "LimCattleBreed": {"133489009": ("Lim cattle breed", [7480])},
    "LimbStructure": {"66019005": ("Limb structure (body structure)", [])},
    "LimbicLobe": {"279215006": ("Limbic lobe", [7151, 7153, 7192, 8134, 9514])},
    "LimbusOfFossaOvalis": {"84712000": ("Limbus of Fossa Ovalis", [12284])},
    "LimianaCattleBreed": {"133490000": ("Limiana cattle breed", [7480])},
    "LimousinCattleBreed": {"48702000": ("Limousin cattle breed (organism)", [])},
    "LimousinCowBreed": {"48702000": ("Limousin cow breed", [7480])},
    "LimousinPigBreed": {"132126001": ("Limousin pig breed", [7480])},
    "Limping": {"16973004": ("Limping (finding)", [])},
    "LimpurgerCattleBreed": {"133491001": ("Limpurger cattle breed", [7480])},
    "LincolnLongwoolSheepBreed": {
        "131745001": ("Lincoln Longwool sheep breed", [7480])
    },
    "LincolnRedCattleBreed": {"3216001": ("Lincoln red cattle breed (organism)", [])},
    "LincolnRedCowBreed": {"3216001": ("Lincoln red cow breed", [7480])},
    "LincolnRedShorthornCattleBreed": {
        "133244005": ("Lincoln Red Shorthorn cattle breed", [7480])
    },
    "LincolnSheep": {"65492002": ("Lincoln sheep (organism)", [])},
    "LincolnSheepBreed": {"65492002": ("Lincoln sheep breed", [7480])},
    "Lincomycin": {"372677003": ("Lincomycin (substance)", [])},
    "LincomycinAntibiotic": {"372677003": ("Lincomycin antibiotic", [621, 622])},
    "Lindblom": {"399308006": ("Lindblom", [4012])},
    "LindblomProjection": {"399308006": ("Lindblom projection (qualifier value)", [])},
    "Line21PigBreed": {"132023007": ("Line 21 pig breed", [7480])},
    "Line32PigBreed": {"132022002": ("Line 32 pig breed", [7480])},
    "Line500DurocPigBreed": {"132216000": ("Line 500 Duroc pig breed", [7480])},
    "LinearCalcificationDistribution": {
        "129765000": ("Linear calcification distribution", [6012, 6013])
    },
    "LinebackCattleBreed": {"133584002": ("Lineback cattle breed", [7480])},
    "Linezolid": {"387056004": ("Linezolid (substance)", [])},
    "LinezolidAntibiotic": {"387056004": ("Linezolid antibiotic", [621, 622])},
    "LingualArtery": {
        "113264009": (
            "Lingual artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LingualTonsil": {"2048000": ("lingual tonsil", [7601, 8134])},
    "LingualTonsilStructure": {
        "2048000": ("Lingual tonsil structure (body structure)", [])
    },
    "Lip": {"48477009": ("Lip", [4028, 7151, 7192, 7601, 8134, 9514])},
    "LipStructure": {"48477009": ("Lip structure (body structure)", [])},
    "LipetskPigBreed": {"132070008": ("Lipetsk pig breed", [7480])},
    "Lipid": {"70106000": ("Lipid", [218, 4032, 4033, 7180, 7186, 7469, 11006])},
    "LipidRichCarcinoma": {
        "3839000": ("Lipid-rich carcinoma (morphologic abnormality)", [])
    },
    "LipidRichLipidSecretingCarcinoma": {
        "3839000": ("Lipid-rich (lipid-secreting) carcinoma", [6030, 6033])
    },
    "LipizzanerHorse": {"41754002": ("Lipizzaner horse (organism)", [])},
    "LipizzanerHorseBreed": {"41754002": ("Lipizzaner horse breed", [7480])},
    "Lipoadenoma": {"22024005": ("Lipoadenoma (morphologic abnormality)", [])},
    "Lipoma": {"134328007": ("Lipoma", [6030, 6031, 6201])},
    "LipomaMorphology": {
        "134328007": ("Lipoma morphology (morphologic abnormality)", [])
    },
    "LipomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "46720004": (
            "Lipoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "Lipopolysaccharide": {"3325005": ("Lipopolysaccharide", [638, 643])},
    "Liposarcoma": {"1162815008": ("Liposarcoma", [6030, 6033])},
    "LiposarcomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "49430005": (
            "Liposarcoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "LiquidBasedCytologicMaterial": {
        "430346005": ("Liquid based cytologic material", [8103])
    },
    "LissamineFastRedBStain": {"6701008": ("lissamine fast red B stain", [8112])},
    "LissamineFastYellowStain": {"25079009": ("lissamine fast yellow stain", [8112])},
    "LissamineGreenBStain": {"38543004": ("lissamine green B stain", [8112])},
    "LissamineRhodamineStain": {"111102009": ("lissamine rhodamine stain", [8112])},
    "Lithotomy": {"14205002": ("lithotomy", [20])},
    "LithotomyPosition": {"14205002": ("Lithotomy position (finding)", [])},
    "LithuanianHeavyDraftHorseBreed": {
        "133060003": ("Lithuanian Heavy Draft horse breed", [7480])
    },
    "LithuanianNativePigBreed": {"132242000": ("Lithuanian Native pig breed", [7480])},
    "LithuanianRedCattleBreed": {"133245006": ("Lithuanian Red cattle breed", [7480])},
    "LithuanianWhitePigBreed": {"132276009": ("Lithuanian White pig breed", [7480])},
    "Liver": {
        "10200004": (
            "Liver",
            [
                4,
                644,
                1000,
                1001,
                1005,
                4030,
                6202,
                6204,
                7151,
                7154,
                7192,
                8134,
                9514,
                10044,
                10060,
                12321,
            ],
        )
    },
    "LiverAndBiliaryStructure": {
        "303270005": (
            "Liver and biliary structure",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "LiverAndOrBiliaryStructure": {
        "303270005": ("Liver and/or biliary structure (body structure)", [])
    },
    "LiverStructure": {"10200004": ("Liver structure (body structure)", [])},
    "LivnyPigBreed": {"132109007": ("Livny pig breed", [7480])},
    "LlaneroHorseBreed": {"131930001": ("Llanero horse breed", [7480])},
    "LlanwenogSheepBreed": {"131746000": ("Llanwenog sheep breed", [7480])},
    "LleynSheepBreed": {"131747009": ("Lleyn sheep breed", [7480])},
    "LoashanGoatBreed": {"131635009": ("Loashan goat breed", [7480])},
    "LobarArtery": {
        "274060004": ("Lobar Artery", [3827, 7151, 7192, 8134, 9514, 12103, 12115])
    },
    "LobarLymphNodeOfTheLung": {
        "127920008": ("lobar lymph node of the lung", [7600, 8134])
    },
    "LobeOfLung": {"31094006": ("Lobe of lung", [6102, 6109, 6110, 7151, 7192, 9514])},
    "LobeOfThyroid": {"119281005": ("Lobe of thyroid", [12130])},
    "LobiCattleBreed": {"133492008": ("Lobi cattle breed", [7480])},
    "Lobular": {"40266001": ("Lobular", [6004, 6005])},
    "LobularCarcinoma": {
        "89740008": ("Lobular carcinoma (morphologic abnormality)", [])
    },
    "LobularCarcinomaInSituOfBreast": {
        "109888004": ("Lobular carcinoma in situ of breast", [6030, 6032])
    },
    "LocalAnesthesia": {"386761002": ("Local anesthesia", [611, 612])},
    "LocalAnesthetic": {"373477003": ("Local anesthetic", [621, 622])},
    "LocalAnestheticIntrathecalBlock": {
        "231249005": ("Local anesthetic intrathecal block (procedure)", [])
    },
    "LocalIndianDairyCattleBreed": {
        "133815000": ("Local Indian Dairy cattle breed (organism)", [])
    },
    "LocalIndianDairyXZebuCattleBreed": {
        "133815000": ("Local Indian Dairy X zebu cattle breed", [7480])
    },
    "LoenbergerDogBreed": {"132622003": ("Loenberger dog breed", [7480])},
    "LohaniCattleBreed": {"133616002": ("Lohani cattle breed", [7480])},
    "LohiSheepBreed": {"131760003": ("Lohi sheep breed", [7480])},
    "LokaiHorseBreed": {"133061004": ("Lokai horse breed", [7480])},
    "LombokHorseBreed": {"131931002": ("Lombok horse breed", [7480])},
    "Long": {"255511005": ("long", [3831])},
    "LongAndShortCoatChihuahua": {
        "57349006": ("Long and short coat chihuahua (organism)", [])
    },
    "LongAndShortCoatChihuahuaDogBreed": {
        "57349006": ("Long and short coat chihuahua dog breed", [7480])
    },
    "LongAxis": {
        "103339001": ("Long axis", [6, 218, 501, 502, 6165, 6166, 7469, 7470])
    },
    "LongCoatChihuahua": {"36611001": ("Long coat chihuahua (organism)", [])},
    "LongCoatChihuahuaDogBreed": {
        "36611001": ("Long coat chihuahua dog breed", [7480])
    },
    "LonghairedManx": {"40547002": ("Longhaired manx", [7480])},
    "LonghairedMiniatureDachshund": {
        "57429001": ("Longhaired miniature dachshund (organism)", [])
    },
    "LonghairedMiniatureDachshundDogBreed": {
        "57429001": ("Longhaired miniature dachshund dog breed", [7480])
    },
    "LonghairedStandardDachshund": {
        "59492009": ("Longhaired standard dachshund (organism)", [])
    },
    "LonghairedStandardDachshundDogBreed": {
        "59492009": ("Longhaired standard dachshund dog breed", [7480])
    },
    "LonghornCattleBreed": {"66314009": ("Longhorn cattle breed (organism)", [])},
    "LonghornCowBreed": {"66314009": ("Longhorn cow breed", [7480])},
    "LongissimusMuscle": {
        "88340001": ("Longissimus muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "Longitudinal": {"38717003": ("Longitudinal", [2, 6, 501, 502, 12118])},
    "LonglinPigBreed": {"132289003": ("Longlin pig breed", [7480])},
    "LongyouBlackPigBreed": {"132303003": ("Longyou Black pig breed", [7480])},
    "LonkSheepBreed": {"131755002": ("Lonk sheep breed", [7480])},
    "Lordotic": {"260450008": ("lordotic", [20])},
    "LordoticProjection": {"260450008": ("Lordotic projection (qualifier value)", [])},
    "Lorenz": {"399251007": ("Lorenz", [4012])},
    "LorenzProjection": {"399251007": ("Lorenz projection (qualifier value)", [])},
    "LorquinaCattleBreed": {"133592006": ("Lorquina cattle breed", [7480])},
    "LossOfDistalPulse": {"414617007": ("Loss of distal pulse", [3413, 3754])},
    "LostToFollowUp": {"399307001": ("Lost to follow-up", [3772])},
    "LourdaisCattleBreed": {"133493003": ("Lourdais cattle breed", [7480])},
    "Low": {"62482003": ("Low", [75])},
    "LowBeer": {"399327000": ("Low-Beer", [4012])},
    "LowBeerProjection": {"399327000": ("Low-Beer projection (qualifier value)", [])},
    "LowBloodPressure": {"45007003": ("Low blood pressure (disorder)", [])},
    "LowDensityNotContainingFatLesion": {
        "129746008": ("Low density (not containing fat) lesion", [6008, 6009])
    },
    "LowMolecularWeightHeparin": {
        "373294004": ("Low Molecular Weight Heparin", [621, 622])
    },
    "LowPowerScanningLensOfOpticalMicroscope": {
        "445601002": (
            "Low power scanning lens of optical microscope (physical object)",
            [],
        )
    },
    "LowRightAtrium": {"128594000": ("Low right atrium", [3011, 8134])},
    "LowRisk": {"75976002": ("Low risk", [3118])},
    "LowToModerateRisk": {"429551001": ("Low to moderate risk", [3118])},
    "LowerAlveolarRidgeStructure": {
        "288546009": ("Lower alveolar ridge structure (body structure)", [])
    },
    "LowerDeepCervicalLymphNode": {
        "285429007": ("lower deep cervical lymph node", [7600, 8134])
    },
    "LowerDentalArchStructure": {
        "88176008": ("Lower dental arch structure (body structure)", [])
    },
    "LowerEyelid": {"62736007": ("Lower Eyelid", [4209, 8134])},
    "LowerEyelidStructure": {
        "62736007": ("Lower eyelid structure (body structure)", [])
    },
    "LowerInnerQuadrantOfBreast": {
        "19100000": (
            "Lower inner quadrant of breast",
            [4, 6020, 6021, 7151, 7192, 8134, 9514],
        )
    },
    "LowerJugularLymphNode": {"245323006": ("lower jugular lymph node", [7600, 8134])},
    "LowerLeg": {
        "30021000": ("Lower leg", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "LowerLegStructure": {"30021000": ("Lower leg structure (body structure)", [])},
    "LowerLimb": {
        "61685007": ("Lower limb", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "LowerLimbAmputation": {"161622006": ("Lower limb amputation", [3205])},
    "LowerLimbLymphNode": {"4942000": ("lower limb lymph node", [7600, 8134])},
    "LowerLimbLymphNodeStructure": {
        "4942000": ("Lower limb lymph node structure (body structure)", [])
    },
    "LowerLimbStructure": {"61685007": ("Lower limb structure (body structure)", [])},
    "LowerLimitOfReferenceRange": {
        "385524004": ("Lower limit of reference range (qualifier value)", [])
    },
    "LowerLobeOfLung": {
        "90572001": (
            "Lower lobe of lung",
            [6124, 6126, 6170, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "LowerOuterQuadrantOfBreast": {
        "33564002": (
            "Lower outer quadrant of breast",
            [4, 6020, 6021, 7151, 7192, 8134, 9514],
        )
    },
    "LowerParatrachealIncludingAzygousLymphNode": {
        "127932007": ("lower paratracheal (including azygous) lymph node", [7600, 8134])
    },
    "LowerZoneOfLung": {"281394001": ("Lower zone of lung", [6124, 6126, 6171])},
    "LucanianCattleBreed": {"131494002": ("Lucanian cattle breed", [7480])},
    "LucanianPigBreed": {"132188002": ("Lucanian pig breed", [7480])},
    "LucentCenteredCalcification": {
        "129754005": ("Lucent-centered calcification", [6010, 6011])
    },
    "LucentCenteredRadiographicCalcification": {
        "129754005": ("Lucent-centered radiographic calcification (finding)", [])
    },
    "LucernaCattleBreed": {"133500008": ("Lucerna cattle breed", [7480])},
    "LuciePig": {"80084005": ("Lucie pig (organism)", [])},
    "LuciePigBreed": {"80084005": ("Lucie pig breed", [7480])},
    "LugwareCattleBreed": {"133709007": ("Lugware cattle breed", [7480])},
    "LuingCattleBreed": {"21553004": ("Luing cattle breed (organism)", [])},
    "LuingCowBreed": {"21553004": ("Luing cow breed", [7480])},
    "LumbarArtery": {
        "34635009": (
            "Lumbar artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12111,
                12125,
            ],
        )
    },
    "LumbarLymphNode": {"8334002": ("lumbar lymph node", [7600, 8134])},
    "LumbarRegion": {"52612000": ("Lumbar region", [4, 7151, 7192, 8134, 9514])},
    "LumbarRegionBackStructure": {
        "52612000": ("Lumbar region back structure (body structure)", [])
    },
    "LumbarSpine": {
        "122496007": (
            "Lumbar spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "LumbarSpineStructure": {
        "122496007": ("Lumbar spine structure (body structure)", [])
    },
    "LumboSacralSpine": {
        "1217253001": (
            "Lumbo-sacral spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7482, 7483, 8134, 9514],
        )
    },
    "LumbosacralRegionOfSpineStructure": {
        "297173004": ("Lumbosacral region of spine structure (body structure)", [])
    },
    "Lumen": {"113342003": ("Lumen", [5])},
    "LumenAreaStenosis": {"408714007": ("Lumen Area Stenosis", [12119, 12121])},
    "LumenDiameterStenosis": {"408715008": ("Lumen Diameter Stenosis", [12119, 12121])},
    "LumenOfArtery": {"67170007": ("Lumen of artery", [3497])},
    "LumenOfBloodVessel": {
        "91747007": ("Lumen of blood vessel", [4, 4040, 7151, 7156, 7192, 8134, 9514])
    },
    "LuminalIrregularities": {"371873004": ("Luminal Irregularities", [3712])},
    "LuminalIrregularitiesOfCoronaryArtery": {
        "371873004": ("Luminal irregularities of coronary artery (finding)", [])
    },
    "Lumpectomy": {"392021009": ("Lumpectomy", [6058, 6060])},
    "LumpectomyBreastSample": {"309546004": ("lumpectomy breast sample", [8103, 8104])},
    "LumpectomyOfBreast": {"392021009": ("Lumpectomy of breast (procedure)", [])},
    "LundehundDogBreed": {"132614005": ("Lundehund dog breed", [7480])},
    "LundyPonyHorseBreed": {"131932009": ("Lundy Pony horse breed", [7480])},
    "Lung": {
        "39607008": (
            "Lung",
            [
                4,
                645,
                1000,
                1001,
                1004,
                6100,
                6129,
                6202,
                6204,
                7151,
                7155,
                7192,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "LungAndTracheobronchialLymphNodes": {
        "110550009": ("lung and tracheobronchial lymph nodes", [7600, 8134])
    },
    "LungAndTracheobronchialLymphNodesCombinedSite": {
        "110550009": (
            "Lung and tracheobronchial lymph nodes (combined site) (body structure)",
            [],
        )
    },
    "LungCancer": {"93880001": ("Lung cancer", [638, 639])},
    "LungStructure": {"39607008": ("Lung structure (body structure)", [])},
    "LupusErythematosus": {"200936003": ("Lupus erythematosus", [4405])},
    "LurcherHoundDogBreed": {"132615006": ("Lurcher Hound dog breed", [7480])},
    "LusitanoHorseBreed": {"133035003": ("Lusitano horse breed", [7480])},
    "LutaiWhitePigBreed": {"132339009": ("Lutai White pig breed", [7480])},
    "Lutetium177DOTATATE": {"781259000": ("Lutetium^177^ DOTATATE", [25])},
    "LuxiCattleBreed": {"133501007": ("Luxi cattle breed", [7480])},
    "LuxolFastBlueStain": {"406958008": ("luxol fast blue stain", [8112])},
    "LuzeinSheepBreed": {"131742003": ("Luzein sheep breed", [7480])},
    "Lymph": {"38000004": ("Lymph", [645, 8134])},
    "LymphNode": {
        "59441001": (
            "Lymph Node",
            [
                645,
                6030,
                6031,
                6054,
                6064,
                6102,
                6109,
                6113,
                6149,
                6348,
                7151,
                7166,
                7191,
                7192,
                7600,
                8134,
                9514,
                10044,
                10060,
            ],
        )
    },
    "LymphNodeOfAorticArch": {"127937001": ("lymph node of aortic arch", [7600, 8134])},
    "LymphNodeOfAortopulmonaryWindow": {
        "127938006": ("lymph node of aortopulmonary window", [7600, 8134])
    },
    "LymphNodeOfEpiploicForamen": {
        "68878000": ("lymph node of epiploic foramen", [7600, 8134])
    },
    "LymphNodeOfGreaterCurvatureOfStomach": {
        "76878005": ("lymph node of greater curvature of stomach", [7600, 8134])
    },
    "LymphNodeOfHead": {"13482005": ("lymph node of head", [7600, 8134])},
    "LymphNodeOfHeadAndNeck": {
        "312501005": ("lymph node of head and neck", [7600, 8134])
    },
    "LymphNodeOfLesserCurvatureOfStomach": {
        "279784003": ("lymph node of lesser curvature of stomach", [7600, 8134])
    },
    "LymphNodeOfLimb": {"312503008": ("lymph node of limb", [7600, 8134])},
    "LymphNodeOfMesentery": {"279795009": ("lymph node of mesentery", [7600, 8134])},
    "LymphNodeOfStomach": {"314730000": ("lymph node of stomach", [7600, 8134])},
    "LymphNodeOfThePulmonaryLigament": {
        "127941002": ("lymph node of the pulmonary ligament", [7600, 8134])
    },
    "LymphNodeOfThorax": {"47109002": ("lymph node of thorax", [7600, 8134])},
    "LymphNodeOfTrunk": {"312502003": ("lymph node of trunk", [7600, 8134])},
    "LymphNodeRingOfCardiaOfStomach": {
        "279866008": ("lymph node ring of cardia of stomach", [7600, 8134])
    },
    "LymphNodeStructureOfLimb": {
        "312503008": ("Lymph node structure of limb (body structure)", [])
    },
    "LymphNodeStructureOfTrunk": {
        "312502003": ("Lymph node structure of trunk (body structure)", [])
    },
    "Lymphadenopathy": {"30746006": ("Lymphadenopathy", [6347])},
    "LymphaticSystem": {"89890002": ("Lymphatic system", [7151, 7166, 7191])},
    "LymphaticTissue": {"181768009": ("Lymphatic tissue (body structure)", [])},
    "LymphaticVessel": {"83555006": ("Lymphatic vessel", [7151, 7166, 7191])},
    "LymphocyticTissueStructure": {
        "6969002": ("Lymphocytic tissue structure (body structure)", [])
    },
    "LymphoidTissue": {"6969002": ("Lymphoid tissue", [645, 7151, 7166, 7191, 8134])},
    "Lymphoma": {"1163043007": ("Lymphoma", [6030, 6033])},
    "LysergicAcidDiethlyamide": {"15698006": ("Lysergic acid diethlyamide", [6089])},
    "LysergicAcidDiethylamide": {
        "15698006": ("Lysergic acid diethylamide (substance)", [])
    },
    "Lysholm": {"399370000": ("Lysholm", [4012])},
    "LysholmProjection": {"399370000": ("Lysholm projection (qualifier value)", [])},
    "Lysolecithin": {"54446009": ("Lysolecithin (substance)", [])},
    "Lysophosphatidylcholine": {"54446009": ("Lysophosphatidylcholine", [638, 643])},
    "LöwchenDogBreed": {"132607008": ("Löwchen dog breed", [7480])},
    "MBayarHorseBreed": {"133034004": ("M'Bayar horse breed", [7480])},
    "MMode": {"399155008": ("M mode", [6058, 12224])},
    "MModeUltrasound": {"399155008": ("M-mode ultrasound (qualifier value)", [])},
    "MOSFETRadiationTherapyDosimetrySystemDosimeter": {
        "701933006": (
            "MOSFET radiation therapy dosimetry system dosimeter",
            [7026, 7027, 7151, 7193],
        )
    },
    "MRIOfBreast": {"241615005": ("MRI of breast", [6028, 6029, 6050, 6083])},
    "MX": {"399593007": ("MX", [6162])},
    "MXCategory": {"27167007": ("MX category (finding)", [])},
    "MacauPigBreed": {"132052005": ("Macau pig breed", [7480])},
    "MacedonianBusaCattleBreed": {
        "133502000": ("Macedonian Busa cattle breed", [7480])
    },
    "MacedonianPigBreed": {"133208000": ("Macedonian pig breed", [7480])},
    "MacrolideAntibiotic": {"372480009": ("Macrolide antibiotic", [621, 622])},
    "MacrolideClassOfAntibioticSubstance": {
        "372480009": ("Macrolide (class of antibiotic, substance) (substance)", [])
    },
    "MacroscopicTissueSpecimen": {
        "430861001": ("Macroscopic tissue specimen (specimen)", [])
    },
    "MadagascarZebuCattleBreed": {
        "133710002": ("Madagascar Zebu cattle breed", [7480])
    },
    "MadaripurCattleBreed": {"133711003": ("Madaripur cattle breed", [7480])},
    "MadonieSicilianPigBreed": {"132256003": ("Madonie-Sicilian pig breed", [7480])},
    "MaduraCattleBreed": {"133842004": ("Madura cattle breed (organism)", [])},
    "MaduraWildJavanXZebuCattleBreed": {
        "133842004": ("Madura wild javan X zebu cattle breed", [7480])
    },
    "MagalCattleBreed": {"133712005": ("Magal cattle breed", [7480])},
    "Magnesium": {"72717003": ("Magnesium", [621, 622])},
    "MagnetInducedPacing": {"371909000": ("Magnet induced pacing (procedure)", [])},
    "MagneticResonanceAngiography": {
        "241663008": ("Magnetic resonance angiography", [3757])
    },
    "MagneticResonanceImagingGuidance": {
        "258177008": ("Magnetic resonance imaging guidance", [100])
    },
    "MagneticResonanceImagingGuidedBiopsy": {
        "277592004": ("Magnetic resonance imaging guided biopsy", [6058, 6060])
    },
    "MagneticResonanceImagingOfBreast": {
        "241615005": ("Magnetic resonance imaging of breast (procedure)", [])
    },
    "MagneticResonanceImagingOfHeart": {
        "241620005": ("Magnetic resonance imaging of heart (procedure)", [])
    },
    "MagneticResonanceImagingOfVessels": {
        "241663008": ("Magnetic resonance imaging of vessels (procedure)", [])
    },
    "MagneticResonanceImagingOfWholeBody": {
        "426252008": ("Magnetic resonance imaging of whole body (procedure)", [])
    },
    "MagneticResonanceImagingService": {
        "310127009": ("Magnetic Resonance Imaging Service", [7030])
    },
    "MagneticResonanceImagingStressStudyOfCardiacFunction": {
        "431609005": (
            "Magnetic resonance imaging stress study of cardiac function (procedure)",
            [],
        )
    },
    "MagneticResonanceImagingUnit": {
        "90003000": ("Magnetic resonance imaging unit", [9268])
    },
    "Magnification": {"399163009": ("Magnification", [4015])},
    "MagnificationViews": {"399163009": ("Magnification views", [6028, 6029])},
    "MagnifiedProjection": {
        "399163009": ("Magnified projection (qualifier value)", [])
    },
    "MagyarAgårDogBreed": {"132436001": ("Magyar Agår dog breed", [7480])},
    "Main": {"63161005": ("Main", [12117])},
    "MainPulmonaryArteryPeakVelocity": {
        "399048009": ("Main Pulmonary Artery Peak Velocity", [12210])
    },
    "MaineAnjouCattleBreed": {"45284002": ("Maine Anjou cattle breed (organism)", [])},
    "MaineAnjouCowBreed": {"45284002": ("Maine Anjou cow breed", [7480])},
    "MaineCoonCat": {"81866001": ("Maine coon cat", [7480])},
    "MaintenanceOfGeneralAnesthesia": {
        "241695009": ("Maintenance of general anesthesia (procedure)", [])
    },
    "MaintenanceTherapy": {"314122007": ("Maintenance therapy", [6146])},
    "Major": {"255603008": ("Major", [251])},
    "MajorAxis": {
        "131187009": (
            "Major Axis",
            [
                218,
                6165,
                7469,
                7470,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
            ],
        )
    },
    "MakaweliCattleBreed": {"133503005": ("Makaweli cattle breed", [7480])},
    "MalachiteGreenStain": {"27120008": ("malachite green stain", [8112])},
    "MalagueñaGoatBreed": {"131676000": ("Malagueña goat breed", [7480])},
    "MalakanHorseBreed": {"131933004": ("Malakan horse breed", [7480])},
    "MalawiZebuCattleBreed": {"133713000": ("Malawi Zebu cattle breed", [7480])},
    "MaleExternalGenitalia": {
        "90418005": ("Male external genitalia", [7151, 7160, 7192, 8134, 9514])
    },
    "MaleExternalGenitaliaStructure": {
        "90418005": ("Male external genitalia structure (body structure)", [])
    },
    "MaleExternalUrethralOrifice": {
        "279478000": ("Male external urethral orifice", [4029, 8134])
    },
    "MaleExternalUrethralSphincter": {
        "717027004": (
            "Male external urethral sphincter",
            [6300, 6301, 6302, 6303, 6304, 6348],
        )
    },
    "MaleFirstCousin": {"11993008": ("Male first cousin", [7450, 7451])},
    "MaleInternalGenitalia": {
        "38242008": ("Male internal genitalia", [7151, 7160, 7192, 8134, 9514])
    },
    "MaleInternalGenitaliaStructure": {
        "38242008": ("Male internal genitalia structure (body structure)", [])
    },
    "Malignant": {"21594007": ("Malignant", [6063])},
    "MalignantEpithelialNeoplasmOfThyroid": {
        "448216007": ("Malignant epithelial neoplasm of thyroid", [64])
    },
    "MalignantFibrousHistiocytoma": {
        "34360000": ("Malignant fibrous histiocytoma", [6030, 6033])
    },
    "MalignantLymphomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "21964009": (
            "Malignant lymphoma, no International Classification of Diseases for Oncology subtype (morphologic "
            "abnormality)",
            [],
        )
    },
    "MalignantMelanoma": {"1162635006": ("Malignant melanoma", [638, 639])},
    "MalignantMelanomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "2092003": (
            "Malignant melanoma, no International Classification of Diseases for Oncology subtype (morphologic "
            "abnormality)",
            [],
        )
    },
    "MalignantMesothelioma": {"62064005": ("Malignant mesothelioma", [638, 639])},
    "MalignantNeoplasmPrimary": {
        "86049000": ("Malignant neoplasm, primary (morphologic abnormality)", [])
    },
    "MalignantPeripheralNerveSheathTumor": {
        "19897006": ("Malignant peripheral nerve sheath tumor", [638, 639])
    },
    "MalignantTumorOfCervix": {
        "363354003": ("Malignant tumor of cervix (disorder)", [])
    },
    "MalignantTumorOfColon": {"363406005": ("Malignant tumor of colon (disorder)", [])},
    "MalignantTumorOfOvary": {"363443007": ("Malignant tumor of ovary (disorder)", [])},
    "MalignantTumorOfSalivaryGland": {
        "255072001": ("Malignant tumor of salivary gland (disorder)", [])
    },
    "MalignantTumorOfUrinaryBladder": {
        "399326009": ("Malignant tumor of urinary bladder (disorder)", [])
    },
    "MalignantTumourOfUnknownOrigin": {
        "255052006": ("Disseminated malignancy of unknown primary", [6055])
    },
    "MalleolarStructureOfTibia": {
        "314796009": ("Malleolar structure of tibia", [1000, 1001, 1006, 8134])
    },
    "MalloryBleachStain": {"406953004": ("Mallory bleach stain", [8112])},
    "MalnadGiddaCattleBreed": {"133714006": ("Malnad Gidda cattle breed", [7480])},
    "MalopolskiHorseBreed": {"131934005": ("Malopolski horse breed", [7480])},
    "MalteseCatBreed": {"132652006": ("Maltese cat breed", [7480])},
    "MalteseDog": {"39348004": ("Maltese dog", [7480])},
    "MalteseGoatBreed": {"131610003": ("Maltese goat breed", [7480])},
    "MalviCattleBreed": {"133617006": ("Malvi cattle breed", [7480])},
    "MammaryDuctEctasia": {
        "22049009": ("Mammary duct ectasia", [6030, 6031, 6054, 6057, 6064])
    },
    "MammaryDuctogram": {"18102001": ("Mammary ductogram", [6028, 6029, 6050, 6083])},
    "MammographicArchitecturalDistortionOfBreast": {
        "129792006": ("Mammographic architectural distortion of breast (finding)", [])
    },
    "MammographicBreastCompositionFinding": {
        "129715009": ("Mammographic breast composition finding (finding)", [])
    },
    "MammographicBreastCompositionShowingScatteredFibroglandularDensities": {
        "129717001": (
            "Mammographic breast composition showing scattered fibroglandular densities (finding)",
            [],
        )
    },
    "MammographicBreastDensity": {
        "129793001": ("Mammographic breast density (finding)", [])
    },
    "MammographicBreastMass": {
        "129788004": ("Mammographic breast mass", [6016, 6017, 6054])
    },
    "MammographicBreastMassFinding": {
        "129788004": ("Mammographic breast mass finding (finding)", [])
    },
    "MammographicDifferenceInLocation": {
        "129808005": ("Mammographic difference in location (finding)", [])
    },
    "MammographicDifferenceInMargin": {
        "129812004": ("Mammographic difference in margin (finding)", [])
    },
    "MammographicDifferenceInNumberOfCalcifications": {
        "129810007": (
            "Mammographic difference in number of calcifications (finding)",
            [],
        )
    },
    "MammographicDifferenceInOpacity": {
        "129807000": ("Mammographic difference in opacity (finding)", [])
    },
    "MammographicDifferenceInShape": {
        "129811006": ("Mammographic difference in shape (finding)", [])
    },
    "MammographicDifferenceInSize": {
        "129806009": ("Mammographic difference in size (finding)", [])
    },
    "MammographicDifferenceInSpatialProximity": {
        "129809002": ("Mammographic difference in spatial proximity (finding)", [])
    },
    "MammographicDifferenceInSymmetry": {
        "129813009": ("Mammographic difference in symmetry (finding)", [])
    },
    "MammographicSkinRetractionOfBreast": {
        "129796009": ("Mammographic skin retraction of breast (finding)", [])
    },
    "MammographicSkinThickeningOfBreast": {
        "129797000": ("Mammographic skin thickening of breast (finding)", [])
    },
    "MammographicTrabecularThickeningOfBreast": {
        "129795008": ("Mammographic trabecular thickening of breast (finding)", [])
    },
    "Mammography": {"71651007": ("Mammography", [6058, 6060, 10005])},
    "MammographyAssessmentCategory0NeedAdditionalImagingEvaluation": {
        "397138000": (
            "Mammography assessment (Category 0) - Need additional imaging evaluation (finding)",
            [],
        )
    },
    "MammographyAssessmentCategory1Negative": {
        "397140005": ("Mammography assessment (Category 1) - Negative (finding)", [])
    },
    "MammographyAssessmentCategory2BenignFinding": {
        "397141009": (
            "Mammography assessment (Category 2) - Benign finding (finding)",
            [],
        )
    },
    "MammographyAssessmentCategory3ProbablyBenignFindingShortIntervalFollowUp": {
        "397143007": (
            "Mammography assessment (Category 3) - Probably benign finding, short interval follow-up (finding)",
            [],
        )
    },
    "MammographyAssessmentCategory4SuspiciousAbnormalityBiopsyShouldBeConsidered": {
        "397144001": (
            "Mammography assessment (Category 4) - Suspicious abnormality, biopsy should be considered (finding)",
            [],
        )
    },
    "MammographyAssessmentCategory5HighlySuggestiveOfMalignancy": {
        "397145000": (
            "Mammography assessment (Category 5) - Highly suggestive of malignancy (finding)",
            [],
        )
    },
    "MammographyBreastDensity": {
        "129793001": ("Mammography breast density", [6014, 6015, 6016, 6054, 6056])
    },
    "MampatiCattleBreed": {"133721006": ("Mampati cattle breed", [7480])},
    "ManagraPigBreed": {"132033004": ("Managra pig breed", [7480])},
    "ManapariCattleBreed": {"133722004": ("Manapari cattle breed", [7480])},
    "ManchesterTerrierSuperbreed": {
        "5306005": ("Manchester terrier superbreed (organism)", [])
    },
    "ManchesterTerrierSuperbreedDogBreed": {
        "5306005": ("Manchester terrier superbreed dog breed", [7480])
    },
    "MandalongCattleBreed": {"133747002": ("Mandalong cattle breed (organism)", [])},
    "MandalongXZebuCattleBreed": {
        "133747002": ("Mandalong X zebu cattle breed", [7480])
    },
    "MandiPigBreed": {"132165003": ("Mandi pig breed", [7480])},
    "Mandible": {
        "91609006": (
            "Mandible",
            [
                4,
                1000,
                1001,
                1002,
                4009,
                4016,
                4028,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "MandibularDentalArch": {
        "88176008": ("Mandibular dental arch", [7482, 7483, 8134])
    },
    "MandibularIncisorTeeth": {
        "442274007": ("Mandibular incisor teeth", [7482, 7483, 8134])
    },
    "MandibularLeftCanineTooth": {
        "860782001": ("Mandibular left canine tooth", [4018, 4026])
    },
    "MandibularLeftCentralIncisorTooth": {
        "425106001": ("Mandibular left central incisor tooth", [4018, 4026])
    },
    "MandibularLeftFirstMolarTooth": {
        "866006002": ("Mandibular left first molar tooth", [4018, 4026])
    },
    "MandibularLeftFirstPremolarTooth": {
        "2400006": ("Mandibular left first premolar tooth", [4018, 4026])
    },
    "MandibularLeftLateralTooth": {
        "423331005": ("Mandibular left lateral tooth", [4018, 4026])
    },
    "MandibularLeftSecondMolarTooth": {
        "863898000": ("Mandibular left second molar tooth", [4018, 4026])
    },
    "MandibularLeftSecondPremolarTooth": {
        "24573005": ("Mandibular left second premolar tooth", [4018, 4026])
    },
    "MandibularLeftThirdMolarTooth": {
        "74344005": ("Mandibular left third molar tooth", [4018, 4026])
    },
    "MandibularLymphNode": {"155338003": ("mandibular lymph node", [7600, 8134])},
    "MandibularRightCanineTooth": {
        "860785004": ("Mandibular right canine tooth", [4018, 4026])
    },
    "MandibularRightCentralIncisorTooth": {
        "424575004": ("Mandibular right central incisor tooth", [4018, 4026])
    },
    "MandibularRightFirstMolarTooth": {
        "866005003": ("Mandibular right first molar tooth", [4018, 4026])
    },
    "MandibularRightFirstPremolarTooth": {
        "80140008": ("Mandibular right first premolar tooth", [4018, 4026])
    },
    "MandibularRightLateralIncisorTooth": {
        "423937004": ("Mandibular right lateral incisor tooth", [4018, 4026])
    },
    "MandibularRightSecondMolarTooth": {
        "863899008": ("Mandibular right second molar tooth", [4018, 4026])
    },
    "MandibularRightSecondPremolarTooth": {
        "8873007": ("Mandibular right second premolar tooth", [4018, 4026])
    },
    "MandibularRightThirdMolarTooth": {
        "38994002": ("Mandibular right third molar tooth", [4018, 4026])
    },
    "MangalargaHorseBreed": {"133052004": ("Mangalarga horse breed", [7480])},
    "MangalargaMarchadorHorseBreed": {
        "132760001": ("Mangalarga Marchador horse breed", [7480])
    },
    "MangalargaPaulistaHorseBreed": {
        "131936007": ("Mangalarga Paulista horse breed", [7480])
    },
    "MangalistaPigBreed": {"132085002": ("Mangalista pig breed", [7480])},
    "MangaliztaPigBreed": {"132085002": ("Mangalizta pig breed (organism)", [])},
    "ManipulationOfCatheter": {
        "103712006": ("Manipulation of catheter (procedure)", [])
    },
    "ManipulationOfJoint": {"118745001": ("Manipulation of joint", [93])},
    "ManipuriHorseBreed": {"132762009": ("Manipuri horse breed", [7480])},
    "ManorHybridPigBreed": {"132010000": ("Manor Hybrid pig breed", [7480])},
    "ManorMeishanPigBreed": {"132013003": ("Manor Meishan pig breed", [7480])},
    "ManorRangerPigBreed": {"132012008": ("Manor Ranger pig breed", [7480])},
    "MantiqueiraCattleBreed": {
        "133816004": ("Mantiqueira cattle breed (organism)", [])
    },
    "MantiqueiraXZebuCattleBreed": {
        "133816004": ("Mantiqueira X zebu cattle breed", [7480])
    },
    "Manual": {"87982008": ("Manual", [7230])},
    "ManubriumOfSternum": {"37285002": ("Manubrium of sternum", [6115, 8134])},
    "Manx": {"3995008": ("Manx", [7480])},
    "ManxLoaghtanSheepBreed": {"131757005": ("Manx Loaghtan sheep breed", [7480])},
    "ManxameseCatBreed": {"132651004": ("Manxamese cat breed", [7480])},
    "MarchigianaCattleBreed": {"65344003": ("Marchigiana cattle breed (organism)", [])},
    "MarchigianaCowBreed": {"65344003": ("Marchigiana cow breed", [7480])},
    "MaremmaSheepdogsDogBreed": {"132616007": ("Maremma Sheepdogs dog breed", [7480])},
    "MaremmanaCattleBreed": {"131495001": ("Maremmana cattle breed", [7480])},
    "MaremmanaHorseBreed": {"131938008": ("Maremmana horse breed", [7480])},
    "MaremmanaPigBreed": {"132189005": ("Maremmana pig breed", [7480])},
    "MarfanSyndrome": {"19346006": ("Marfan's Syndrome", [3809])},
    "Marginal": {"112233002": ("Marginal", [2])},
    "MarginalCoronaryArtery": {
        "22765000": (
            "Marginal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "MarianasCattleBreed": {"131496000": ("Marianas cattle breed", [7480])},
    "Marijuana": {"398705004": ("Marijuana", [6089])},
    "MarinhoaCattleBreed": {"133504004": ("Marinhoa cattle breed", [7480])},
    "MarkedlyEnlargedCardiacChamber": {
        "373128001": ("Markedly enlarged cardiac chamber", [3122, 3705])
    },
    "MaronesaCattleBreed": {"133505003": ("Maronesa cattle breed", [7480])},
    "MartiusYellowStain": {"46139004": ("martius yellow stain", [8112])},
    "MarwariHorseBreed": {"131939000": ("Marwari horse breed", [7480])},
    "MarylandPig": {"60958006": ("Maryland pig (organism)", [])},
    "MarylandPigBreed": {"60958006": ("Maryland pig breed", [7480])},
    "MaryutiCattleBreed": {"131497009": ("Maryuti cattle breed", [7480])},
    "MasaiCattleBreed": {"409908002": ("Masai cattle breed (organism)", [])},
    "MasaiCowBreed": {"409908002": ("Masai cow breed", [7480])},
    "MasaiSheepBreed": {"131758000": ("Masai sheep breed", [7480])},
    "MashenPigBreed": {"132137000": ("Mashen pig breed", [7480])},
    "MashonaCattleBreed": {"133246007": ("Mashona cattle breed", [7480])},
    "Mass": {"4147007": ("Mass", [6102, 6104, 6335, 6336, 6337, 7151, 7159, 7194])},
    "MassQuantity": {"118538004": ("Mass", [12304])},
    "MassifCentralGoatBreed": {"131675001": ("Massif Central goat breed", [7480])},
    "Mastectomy": {"5884001": ("Mastectomy (procedure)", [])},
    "MastergiltPigBreed": {"132067009": ("Mastergilt pig breed", [7480])},
    "MastiffDog": {"48524002": ("Mastiff dog", [7480])},
    "MastoidBone": {
        "59066005": (
            "Mastoid bone",
            [4, 1000, 1001, 1002, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "MastoidCellsAndAntra": {
        "91716001": ("Mastoid cells and antra", [1000, 1001, 1002, 8134])
    },
    "MastoidCellsAndAntraStructure": {
        "91716001": ("Mastoid cells and antra structure (body structure)", [])
    },
    "MastoidLymphNode": {"279143009": ("mastoid lymph node", [7600, 8134])},
    "MastoidStructure": {"59066005": ("Mastoid structure (body structure)", [])},
    "MaternalHistoryOfInsulinDependence": {
        "444161008": ("Maternal history of insulin dependence (situation)", [])
    },
    "MaureCattleBreed": {"133723009": ("Maure cattle breed", [7480])},
    "MauritiusCreoleCattleBreed": {
        "131498004": ("Mauritius Creole cattle breed", [7480])
    },
    "MauritiusCréoleCattleBreed": {
        "131498004": ("Mauritius Créole cattle breed (organism)", [])
    },
    "Maxilla": {
        "70925003": (
            "Maxilla",
            [4, 4009, 4016, 4028, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "MaxillaryDentalArch": {"39481002": ("Maxillary dental arch", [7482, 7483, 8134])},
    "MaxillaryIncisorTeeth": {
        "442100006": ("Maxillary incisor teeth", [7482, 7483, 8134])
    },
    "MaxillarySinus": {"15924003": ("Maxillary sinus", [1000, 1001, 1002, 7601, 8134])},
    "MaxillarySinusStructure": {
        "15924003": ("Maxillary sinus structure (body structure)", [])
    },
    "Maximal": {"56851009": ("Maximal (qualifier value)", [])},
    "Maximum": {"56851009": ("Maximum", [3488, 3838, 7464, 10072, 12301])},
    "MaximumDiastolicBloodPressure": {
        "314452008": ("Maximum diastolic blood pressure (observable entity)", [])
    },
    "MaximumHeartRateAchievedDuringCourseOfProcedure": {
        "428630002": (
            "Maximum heart rate achieved during course of procedure (observable entity)",
            [],
        )
    },
    "MaximumSystolicBloodPressure": {
        "314439003": ("Maximum systolic blood pressure", [3337, 3611])
    },
    "May": {"399024000": ("May", [4012])},
    "MayGrunwaldGiemsaStain": {"255803006": ("may-Grunwald giemsa stain", [8112])},
    "MayProjection": {"399024000": ("May projection (qualifier value)", [])},
    "Mayer": {"399000008": ("Mayer", [4012])},
    "MayerProjection": {"399000008": ("Mayer projection (qualifier value)", [])},
    "MazandaraniCattleBreed": {"133724003": ("Mazandarani cattle breed", [7480])},
    "MazuryCattleBreed": {"133506002": ("Mazury cattle breed", [7480])},
    "McnabDogBreed": {"132617003": ("McNab dog breed", [7480])},
    "Mean": {"373098007": ("Mean", [3488, 3627, 3838, 7464, 10072])},
    "MeanBloodPressure": {"6797001": ("Mean blood pressure", [3337, 3611, 12304])},
    "MeanNumericEstimationTechnique": {
        "373098007": ("Mean - numeric estimation technique (qualifier value)", [])
    },
    "MeanValueOfPopulation": {
        "373098007": ("Mean Value of population", [221, 226, 7465])
    },
    "Measured": {"258104002": ("Measured", [3627, 10009])},
    "MeasurementMethod": {"370129005": ("Measurement Method", [9000])},
    "MeasurementOfBloodPressureUsingCuffMethod": {
        "371911009": ("Measurement of blood pressure using cuff method (procedure)", [])
    },
    "MeasurementOfCardiacOutput": {
        "117610000": ("Measurement of cardiac output (procedure)", [])
    },
    "MeasurementOfSystemicArterialPressureRegimeTherapy": {
        "73002000": ("Measurement of systemic arterial pressure (regime/therapy)", [])
    },
    "MeasurementOfVenousPressure": {
        "31724009": ("Measurement of venous pressure (procedure)", [])
    },
    "MeasuringRuler": {"102304005": ("Measuring ruler", [8, 3451, 4051, 7151, 7193])},
    "MeasuringRulerDevice": {
        "102304005": ("Measuring ruler, device (physical object)", [])
    },
    "MeatlinePigBreed": {"132024001": ("Meatline pig breed", [7480])},
    "MecklenburgHorseBreed": {"131994008": ("Mecklenburg horse breed", [7480])},
    "Media": {"61695000": ("Media", [7151, 7156, 7192, 8134, 9514])},
    "Medial": {"255561001": ("Medial", [2, 5, 4005, 6065, 12117, 12281])},
    "MedialCommonIliacLymphNode": {
        "34625003": ("medial common iliac lymph node", [7600, 8134])
    },
    "MedialExternalIliacLymphNode": {
        "42472007": ("medial external iliac lymph node", [7600, 8134])
    },
    "MedialLacunarLymphNode": {"23198005": ("medial lacunar lymph node", [7600, 8134])},
    "MedialLateralEmissiveProjection": {"399012007": ("Medial-lateral", [26, 501])},
    "MedialLemniscus": {
        "30114003": ("Medial Lemniscus", [7151, 7153, 7192, 7701, 7710, 8134, 9514])
    },
    "MedialLemniscusStructure": {
        "30114003": ("Medial lemniscus structure (body structure)", [])
    },
    "MedialLongitudinalFasciculus": {
        "28390009": ("medial longitudinal fasciculus", [7701, 7710, 8134])
    },
    "MedialLongitudinalFasciculusStructure": {
        "28390009": ("Medial longitudinal fasciculus structure (body structure)", [])
    },
    "MedialMitralAnnulus": {
        "399093001": ("Medial Mitral Annulus", [12235, 12236, 12305])
    },
    "MedialMitralAnnulusStructure": {
        "399093001": ("Medial mitral annulus structure (body structure)", [])
    },
    "MedialOblique": {"260426006": ("medial oblique", [501, 4010])},
    "MedialPlantarArtery": {"74156002": ("medial plantar artery", [3604, 3606, 3630])},
    "MedialRollingOfBreast": {
        "399226006": ("Medial rolling of breast (procedure)", [])
    },
    "Median": {"373099004": ("Median", [3627, 3676, 7464, 10072])},
    "MedianCubitalVein": {
        "49852007": (
            "Median Cubital vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "MedianNumericEstimationTechnique": {
        "373099004": ("Median - numeric estimation technique (qualifier value)", [])
    },
    "MedianRetropharyngealLymphNode": {
        "167364008": ("median retropharyngeal lymph node", [7600, 8134])
    },
    "MedianValueOfPopulation": {
        "373099004": ("Median Value of population", [221, 226, 7465])
    },
    "MediastinalEmphysema": {"16838000": ("Mediastinal emphysema (disorder)", [])},
    "MediastinalLymphNode": {"62683002": ("mediastinal lymph node", [7600, 8134])},
    "MediastinalLymphNodeStructure": {
        "62683002": ("Mediastinal lymph node structure (body structure)", [])
    },
    "MediastinalStructure": {
        "72410000": ("Mediastinal structure (body structure)", [])
    },
    "Mediastinum": {
        "72410000": (
            "Mediastinum",
            [4, 4009, 4030, 4031, 4040, 4042, 6100, 6129, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "MedicalAir": {"417696007": ("Medical air", [623, 629])},
    "MedicalIntensiveCare": {"441994008": ("Medical Intensive Care", [7030])},
    "MedicalIntensiveCareUnit": {
        "441994008": ("Medical intensive care unit (environment)", [])
    },
    "MedicalMicrobiology": {"309956009": ("Medical Microbiology", [7030])},
    "MedicalMicrobiologyDepartment": {
        "309956009": ("Medical microbiology department (environment)", [])
    },
    "MedicalOphthalmologyDepartment": {
        "309935007": ("Medical ophthalmology department (environment)", [])
    },
    "MedicalPractitioner": {
        "158965000": ("Medical Practitioner", [7450, 7452, 9536, 9555, 9562])
    },
    "MedicalRadiographer": {"159016003": ("Medical radiographer (occupation)", [])},
    "MedicalXRayFilm": {"706247001": ("Medical X-Ray film", [7026])},
    "MedicationAdministeredAfterLabVisit": {
        "371899006": ("Medication administered after lab visit (situation)", [])
    },
    "MedicationAdministeredAfterPercutaneousCoronaryIntervention": {
        "371902009": (
            "Medication administered after percutaneous coronary intervention (situation)",
            [],
        )
    },
    "MedicationAdministeredBeforeLabVisit": {
        "371898003": ("Medication administered before lab visit (situation)", [])
    },
    "MedicationAdministeredDuringLabVisit": {
        "371905006": ("Medication administered during lab visit (situation)", [])
    },
    "MedicationAdministeredDuringPercutaneousCoronaryIntervention": {
        "371903004": (
            "Medication administered during percutaneous coronary intervention (situation)",
            [],
        )
    },
    "MedicationAdministeredLessThan72HoursBeforePercutaneousCoronaryIntervention": {
        "371901002": (
            "Medication administered less than 72 hours before percutaneous coronary intervention (situation)",
            [],
        )
    },
    "MedicationAdministeredPriorToPercutaneousCoronaryIntervention": {
        "371904005": (
            "Medication administered prior to percutaneous coronary intervention (situation)",
            [],
        )
    },
    "MedicationGiven": {"182833002": ("Medication given (situation)", [])},
    "MedicationNotAdministered": {
        "371900001": ("Medication not administered (situation)", [])
    },
    "MedioLateralObliqueEmissiveProjection": {
        "399268006": ("Medio-lateral oblique", [26, 501])
    },
    "MedioLateralObliqueProjection": {
        "399368009": ("medio-lateral oblique", [501, 4010, 4014, 7302])
    },
    "MedioLateralProjection": {
        "399260004": ("medial-lateral", [501, 4010, 4014, 7484])
    },
    "Mediolateral": {"261129000": ("Mediolateral (qualifier value)", [])},
    "Medium": {"255508009": ("Medium", [252, 6118])},
    "MediumWoolMerinoSheepBreed": {
        "132717005": ("Medium-Wool Merino sheep breed", [7480])
    },
    "MedullaryCarcinoma": {"32913002": ("Medullary carcinoma", [6030, 6033])},
    "MegezhHorseBreed": {"131940003": ("Megezh horse breed", [7480])},
    "MeglumineDiatrizoate": {"47192000": ("Meglumine diatrizoate", [12, 3850])},
    "MeglumineIodipamide": {"69783005": ("Meglumine iodipamide", [12, 3850])},
    "MegrelHorseBreed": {"131941004": ("Megrel horse breed", [7480])},
    "MeishanPigBreed": {"132243005": ("Meishan pig breed", [7480])},
    "MeixinPigBreed": {"132330008": ("Meixin pig breed", [7480])},
    "Melanin": {"59094002": ("Melanin", [11006])},
    "MeldolaBlueStain": {"24516006": ("meldola blue stain", [8112])},
    "Membranous": {"94150003": ("Membranous", [3707])},
    "MembranousVentricularSeptumDefect": {
        "94150003": ("Membranous ventricular septum defect (disorder)", [])
    },
    "MengguCattleBreed": {"131554001": ("Menggu cattle breed", [7480])},
    "Meninges": {"1231004": ("Meninges", [7151, 7153, 7192, 8134, 9514])},
    "MeningesStructure": {"1231004": ("Meninges structure (body structure)", [])},
    "Meniscus": {"74135004": ("Meniscus", [7151, 7166, 7191])},
    "MeniscusStructureOfJoint": {
        "74135004": ("Meniscus structure of joint (body structure)", [])
    },
    "MenstrualBleedingPresent": {
        "289894009": ("Menstrual bleeding present (finding)", [])
    },
    "MenstruationPresent": {"289894009": ("Menstruation present", [6163])},
    "MentalState": {"363871006": ("Mental state (observable entity)", [])},
    "MenufiCattleBreed": {"131499007": ("Menufi cattle breed", [7480])},
    "MeoPigBreed": {"132346000": ("Meo pig breed", [7480])},
    "MeraukeCattleBreed": {"133725002": ("Merauke cattle breed", [7480])},
    "Merchant": {"399284007": ("Merchant", [4012])},
    "MerchantProjection": {"399284007": ("Merchant projection (qualifier value)", [])},
    "MercuricChloride": {"11496005": ("Mercuric chloride", [8114])},
    "MercuryArcLamp": {"445685008": ("Mercury arc lamp", [8125])},
    "MercuryArcLampMicroscopeIlluminator": {
        "445685008": ("Mercury arc lamp microscope illuminator (physical object)", [])
    },
    "MerensHorseBreed": {"131942006": ("Merens horse breed", [7480])},
    "MergellandSheepBreed": {"132687006": ("Mergelland sheep breed", [7480])},
    "MerinoSheepSuperbreed": {"125101009": ("Merino sheep superbreed", [7480])},
    "MerinolandschafSheepBreed": {"131759008": ("Merinolandschaf sheep breed", [7480])},
    "MerkelCellCarcinoma": {"253001006": ("Merkel cell carcinoma", [638, 639])},
    "MertolengaCattleBreed": {"133585001": ("Mertolenga cattle breed", [7480])},
    "Mescaline": {"373780001": ("Mescaline", [6089])},
    "MesentericArtery": {
        "86570000": (
            "Mesenteric artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "MesentericArteryLymphNode": {
        "299993000": ("mesenteric artery lymph node", [7600, 8134])
    },
    "MesentericVein": {
        "128583004": (
            "Mesenteric vein",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "Mesocardia": {"16567006": ("Mesocardia", [12248, 12249])},
    "MesotheliomaMalignant": {
        "62064005": ("Mesothelioma, malignant (morphologic abnormality)", [])
    },
    "Mesothelium": {"71400007": ("Mesothelium", [645, 8134])},
    "MesotheliumStructure": {
        "71400007": ("Mesothelium structure (body structure)", [])
    },
    "MespiperoneC11": {"424789007": ("Mespiperone C^11^", [4021])},
    "MessaoriaCattleBreed": {"133507006": ("Messaoria cattle breed", [7480])},
    "MessaraHorseBreed": {"131943001": ("Messara horse breed", [7480])},
    "Metacarpal": {"36455000": ("Metacarpal", [1000, 1001, 1006])},
    "Metacarpus": {"36455000": ("Metacarpus", [7482, 7483, 8134])},
    "Metaiodobenzylguanidine123I": {
        "395787009": ("Metaiodobenzylguanidine[123I] (substance)", [])
    },
    "Metaiodobenzylguanidine131I": {
        "395789007": ("Metaiodobenzylguanidine[131I] (substance)", [])
    },
    "MetalOxideSemiconductorFieldEffectTransistorRadiationTherapyDosimetrySystemRadiationTherapyDosimetrySystemDosimeter": {
        "701933006": (
            "Metal-oxide semiconductor field-effect transistor radiation "
            "therapy dosimetry system radiation therapy dosimetry system "
            "dosimeter (physical object)",
            [],
        )
    },
    "MetalStent": {"257363003": ("metal stent", [3814])},
    "MetanilYellowStain": {"54791001": ("metanil yellow stain", [8112])},
    "Metastasis": {"128462008": ("Metastasis", [645, 8134])},
    "MetastasisCategoryFinding": {
        "385380006": ("Metastasis category finding (finding)", [])
    },
    "Metatarsus": {"280711000": ("Metatarsus", [7482, 7483, 8134])},
    "Methadone": {"387286002": ("Methadone", [6089])},
    "Methanol": {"259221006": ("Methanol", [8114])},
    "MethionineC11": {"129510001": ("Methionine C^11^", [4021])},
    "Methlyphenidate": {"373337007": ("Methlyphenidate", [6089])},
    "Methohexital": {"373488009": ("Methohexital", [623, 625])},
    "Methoxyflurane": {"11136004": ("Methoxyflurane", [623, 624])},
    "MethylBlueStain": {"9010006": ("methyl blue stain", [4412, 8112])},
    "MethylGreenPyroninStain": {"255804000": ("methyl green pyronin stain", [8112])},
    "MethylGreenStain": {"22021002": ("methyl green stain", [8112])},
    "MethylOrangeStain": {"42248000": ("methyl orange stain", [8112])},
    "MethylRedStain": {"13744001": ("methyl red stain", [8112])},
    "MethylViolet2BStain": {"15896008": ("methyl violet 2B stain", [8112])},
    "MethylViolet6BStain": {"14544006": ("methyl violet 6B stain", [8112])},
    "MethylVioletStain": {"387239001": ("methyl violet stain", [8112])},
    "MethyleneBlue": {"6725000": ("Methylene blue", [10])},
    "MethyleneBlueStain": {"6725000": ("methylene blue stain", [8112])},
    "MethyleneVioletStain": {"406961009": ("methylene violet stain", [8112])},
    "MethyleneVioletStainBernthsen": {
        "31260003": ("methylene violet stain (Bernthsen)", [8112])
    },
    "Methylphenidate": {"373337007": ("Methylphenidate (substance)", [])},
    "Methylprednisolone": {"116593003": ("Methylprednisolone", [65])},
    "MethylprednisoloneSodiumPhosphate": {
        "412248005": ("Methylprednisolone sodium phosphate", [65])
    },
    "MethylprednisoloneSodiumSuccinate": {
        "412248005": ("Methylprednisolone sodium succinate (substance)", [])
    },
    "MethylthioniniumChlorideContainingProduct": {
        "354064008": ("Methylene blue", [4200])
    },
    "MetohijaRedCattleBreed": {"133508001": ("Metohija Red cattle breed", [7480])},
    "Metrizamide": {"90733003": ("Metrizamide", [12, 3850])},
    "Metrizoate": {"354094002": ("Metrizoate", [12, 3850])},
    "Metronidazole": {"372602008": ("Metronidazole (substance)", [])},
    "MetronidazoleAntibiotic": {"372602008": ("Metronidazole antibiotic", [621, 622])},
    "MeusseRhineIjsselCattleBreed": {
        "6112002": ("Meusse-Rhine-Ijssel cattle breed (organism)", [])
    },
    "MeusseRhineIjsselCowBreed": {"6112002": ("Meusse-Rhine-Ijssel cow breed", [7480])},
    "MewatiCattleBreed": {"133618001": ("Mewati cattle breed", [7480])},
    "MexicanHairlessDog": {"78246003": ("Mexican hairless dog", [7480])},
    "MexicanWattledPigBreed": {"132170005": ("Mexican Wattled pig breed", [7480])},
    "MezenHorseBreed": {"131988000": ("Mezen horse breed", [7480])},
    "MezohegyesSportHorseHorseBreed": {
        "131989008": ("Mezohegyes Sport Horse horse breed", [7480])
    },
    "MezzalinaCattleBreed": {"131500003": ("Mezzalina cattle breed", [7480])},
    "MhaswadCattleBreed": {"133727005": ("Mhaswad cattle breed", [7480])},
    "MiKiDogBreed": {"132438000": ("Mi-Ki dog breed", [7480])},
    "MiPigBreed": {"132319005": ("Mi pig breed", [7480])},
    "MiamiPigBreed": {"132190001": ("Miami pig breed", [7480])},
    "MichelMedium": {"430028007": ("Michel's medium", [8114])},
    "MichelTransportMedium": {"430028007": ("Michel transport medium (substance)", [])},
    "MicrocalcificationsOfTheBreast": {
        "44771000": ("Microcalcifications of the breast", [6010, 6011])
    },
    "MicrodissectionOfTissueSpecimenUsingLaser": {
        "433454009": ("Microdissection of tissue specimen using laser (procedure)", [])
    },
    "MicrolobulatedLesion": {"129739004": ("Microlobulated lesion", [6006, 6007])},
    "MicroscopeSlide": {"433466003": ("Microscope slide", [8101, 8102])},
    "MicroscopeSlideCoverSlip": {"433472003": ("Microscope slide cover slip", [8102])},
    "MicroscopeSlideCoverslip": {
        "433472003": ("Microscope slide coverslip (physical object)", [])
    },
    "MicroscopeSlideMountingMedia": {
        "430862008": ("Microscope slide mounting media", [8102])
    },
    "MicroscopeSlideMountingMedium": {
        "430862008": ("Microscope slide mounting medium (substance)", [])
    },
    "Microscopy": {"117259009": ("Microscopy", [8131])},
    "MicrowaveHeatingOfTissueSpecimen": {
        "433455005": ("Microwave heating of tissue specimen (procedure)", [])
    },
    "MidCircumflexCoronaryArtery": {
        "91753007": (
            "Mid Circumflex Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514, 12292],
        )
    },
    "MidFrequency": {"255218000": ("Mid-frequency", [6091])},
    "MidLeftAnteriorDescendingCoronaryArtery": {
        "91748002": (
            "Mid Left Anterior Descending Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "MidLongitudinal": {"103342007": ("Mid-longitudinal", [5, 211, 212, 3019])},
    "MidRightAtrium": {"128593006": ("Mid right atrium", [3011, 8134])},
    "MidRightCoronaryArtery": {
        "450960006": (
            "Mid Right Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "Midazolam": {"373476007": ("Midazolam", [65, 66, 623, 625])},
    "Midbrain": {"61962009": ("Midbrain", [7151, 7153, 7192, 8134, 9514])},
    "MidbrainStructure": {"61962009": ("Midbrain structure (body structure)", [])},
    "MidcolicLymphNode": {"282031000": ("midcolic lymph node", [7600, 8134])},
    "MiddleAnteriorFibromuscularStromaOfProstate": {
        "716908000": ("Middle anterior fibromuscular stroma of prostate", [6300, 6303])
    },
    "MiddleAsianOvtcharkaDogBreed": {
        "132437005": ("Middle Asian Ovtcharka dog breed", [7480])
    },
    "MiddleCardiacVein": {
        "73580002": ("Middle cardiac vein", [3011, 3827, 3839, 7151, 7192, 8134, 9514])
    },
    "MiddleCerebellarPeduncle": {
        "33723005": (
            "Middle cerebellar peduncle",
            [7151, 7153, 7192, 7701, 7710, 8134, 9514],
        )
    },
    "MiddleCerebralArtery": {
        "17232002": (
            "Middle Cerebral Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105, 12141],
        )
    },
    "MiddleCerebralArteryM1Segment": {
        "414722000": (
            "Middle Cerebral Artery M1 Segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "MiddleCerebralArteryM2Segment": {
        "414723005": (
            "Middle Cerebral Artery M2 Segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "MiddleDeepCervicalLymphNode": {
        "285427009": ("middle deep cervical lymph node", [7600, 8134])
    },
    "MiddleEar": {"25342003": ("Middle ear", [4028, 7151, 7192, 8134, 9514])},
    "MiddleEarStructure": {"25342003": ("Middle ear structure (body structure)", [])},
    "MiddleHepaticVein": {
        "273099000": (
            "Middle Hepatic Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "MiddleJugularLymphNode": {
        "245322001": ("middle jugular lymph node", [7600, 8134])
    },
    "MiddleLobeOfRightLung": {
        "72481006": (
            "Middle lobe of right lung",
            [6124, 6126, 6170, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "MiddleWhitePig": {"82909008": ("Middle white pig (organism)", [])},
    "MiddleWhitePigBreed": {"82909008": ("Middle white pig breed", [7480])},
    "MiddleZoneOfLung": {"281393007": ("Middle zone of lung", [6124, 6126, 6171])},
    "Midline": {"261129000": ("Midline", [5])},
    "MigrationOfImplantOrInternalDevice": {
        "370512004": ("migration of implant or internal device", [3813])
    },
    "Mild": {"255604002": ("Mild", [3716, 3832, 6130])},
    "MildHypokinesis": {"371868005": ("Mild Hypokinesis", [3703])},
    "MildHypokinesisOfCardiacWall": {
        "371868005": ("Mild hypokinesis of cardiac wall (finding)", [])
    },
    "MildIntimalCoronaryIrregularitiesNoSignificantStenoses": {
        "371861004": (
            "Mild intimal coronary irregularities, no significant stenoses",
            [3728],
        )
    },
    "MildIntimalCoronaryIrregularitiesWithoutSignificantStenoses": {
        "371861004": (
            "Mild intimal coronary irregularities without significant stenoses (finding)",
            [],
        )
    },
    "MildRiskOfContextualQualifier": {
        "75976002": ("Mild risk of (contextual qualifier) (qualifier value)", [])
    },
    "MildToModerate": {"371923003": ("Mild to Moderate", [3716])},
    "MildToModerateRiskOfContextualQualifier": {
        "429551001": (
            "Mild to moderate risk of (contextual qualifier) (qualifier value)",
            [],
        )
    },
    "MildlyEnlargedCardiacChamber": {
        "373126002": ("Mildly enlarged cardiac chamber", [3122, 3705])
    },
    "MiliaryAneurysm": {"43299000": ("miliary aneurysm", [3808])},
    "MilkOfCalciumCalcification": {
        "129753004": ("Milk of calcium calcification", [6010, 6011])
    },
    "MilkOfCalciumRadiographicCalcification": {
        "129753004": ("Milk of calcium radiographic calcification (finding)", [])
    },
    "MilkingDevonCattleBreed": {"133247003": ("Milking Devon cattle breed", [7480])},
    "MilkingShorthornCattleBreed": {
        "78541007": ("Milking Shorthorn cattle breed (organism)", [])
    },
    "MilkingShorthornCowBreed": {"78541007": ("Milking Shorthorn cow breed", [7480])},
    "Miller": {"399005003": ("Miller", [4012])},
    "MillerProjection": {"399005003": ("Miller projection (qualifier value)", [])},
    "MinPeiDogBreed": {"132440005": ("Min-pei dog breed", [7480])},
    "MinPigBreed": {"132120007": ("Min pig breed", [7480])},
    "MinbeiSpottedPigBreed": {"132286005": ("Minbei Spotted pig breed", [7480])},
    "MingrelianRedCattleBreed": {"133509009": ("Mingrelian Red cattle breed", [7480])},
    "MinhotaCattleBreed": {"133516005": ("Minhota cattle breed", [7480])},
    "MiniatureAustralianShepherdDogBreed": {
        "132439008": ("Miniature Australian Shepherd dog breed", [7480])
    },
    "MiniatureBullTerrierDogBreed": {
        "132618008": ("Miniature Bull Terrier dog breed", [7480])
    },
    "MiniatureHerefordCattleBreed": {
        "132770004": ("Miniature Hereford cattle breed", [7480])
    },
    "MiniaturePinscherDog": {"12131006": ("Miniature pinscher dog", [7480])},
    "MiniaturePoodle": {"40121001": ("Miniature poodle (organism)", [])},
    "MiniaturePoodleDogBreed": {"40121001": ("Miniature poodle dog breed", [7480])},
    "MiniatureSchnauzer": {"300004": ("Miniature schnauzer (organism)", [])},
    "MiniatureSchnauzerDogBreed": {"300004": ("Miniature schnauzer dog breed", [7480])},
    "MiniatureZebuCattleBreed": {"133728000": ("Miniature Zebu cattle breed", [7480])},
    "Minimal": {"255605001": ("Minimal (qualifier value)", [])},
    "Minimum": {"255605001": ("Minimum", [3488, 3838, 7464, 10072, 12301])},
    "MinimumDiastolicBloodPressure": {
        "314451001": ("Minimum diastolic blood pressure", [3337, 3611])
    },
    "MinisibPigBreed": {"132112005": ("Minisib pig breed", [7480])},
    "MinnesotaPig": {"61083001": ("Minnesota pig (organism)", [])},
    "MinnesotaPigBreed": {"61083001": ("Minnesota pig breed", [7480])},
    "Minor": {"255606000": ("Minor", [251])},
    "MinorAxis": {
        "131188004": (
            "Minor Axis",
            [
                218,
                6165,
                7469,
                7470,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
                12304,
            ],
        )
    },
    "MinorcanCattleBreed": {"133517001": ("Minorcan cattle breed", [7480])},
    "MinusinHorseBreed": {"132772007": ("Minusin horse breed", [7480])},
    "MinzhuPigBreed": {"132248001": ("Minzhu pig breed", [7480])},
    "MirandesaCattleBreed": {"133248008": ("Mirandesa cattle breed", [7480])},
    "MirgorodPigBreed": {"132279002": ("Mirgorod pig breed", [7480])},
    "MisakiHorseBreed": {"133012001": ("Misaki horse breed", [7480])},
    "MishimaCattleBreed": {"133518006": ("Mishima cattle breed", [7480])},
    "MissouriFoxTrottingHorse": {
        "12360007": ("Missouri fox trotting horse (organism)", [])
    },
    "MissouriFoxTrottingHorseBreed": {
        "12360007": ("Missouri fox trotting horse breed", [7480])
    },
    "MitralAnnulus": {"65197004": ("Mitral Annulus", [12235, 12236, 12305])},
    "MitralDiastolicFillingPeriod": {
        "371849007": ("Mitral diastolic filling period (observable entity)", [])
    },
    "MitralDiastolicFillingPeriodDfpm": {
        "371849007": ("Mitral Diastolic Filling Period (DFPm)", [3616])
    },
    "MitralRegurgitation": {
        "48724000": ("Mitral regurgitation", [3700, 3728, 12248, 12249])
    },
    "MitralRing": {"65197004": ("Mitral ring", [3011, 8134])},
    "MitralStenosis": {"79619009": ("Mitral stenosis", [3700, 3728, 12248, 12249])},
    "MitralValve": {
        "91134007": (
            "Mitral Valve",
            [
                3610,
                3630,
                3701,
                6102,
                6109,
                6113,
                6149,
                7151,
                7192,
                8134,
                9514,
                12223,
                12236,
                12285,
                12305,
            ],
        )
    },
    "MitralValveATDTRatio": {"399062002": ("Mitral Valve AT/DT Ratio", [12207])},
    "MitralValveAWaveDuration": {
        "399229004": ("Mitral Valve A-Wave Duration", [12207])
    },
    "MitralValveArea": {"251012002": ("Mitral Valve Area", [3615])},
    "MitralValveClosureToOpeningTime": {
        "399104001": ("Mitral Valve Closure to Opening Time", [12207])
    },
    "MitralValveDisease": {"11851006": ("Mitral valve disease", [3700])},
    "MitralValveDisorder": {"11851006": ("Mitral valve disorder (disorder)", [])},
    "MitralValveEWaveDecelerationTime": {
        "399354002": ("Mitral Valve E-Wave Deceleration Time", [12207])
    },
    "MitralValveFlow": {"371837006": ("Mitral Valve Flow", [3617])},
    "MitralValveProlapse": {
        "409712001": ("Mitral valve prolapse", [3700, 3728, 12248, 12249])
    },
    "MitralValvePullbackPressureFunction": {
        "128450009": (
            "Mitral valve pullback pressure, function (observable entity)",
            [],
        )
    },
    "MitralValvePullbackPressureWaveform": {
        "128450009": ("Mitral valve pullback pressure waveform", [3003])
    },
    "MitralValveRegurgitation": {
        "48724000": ("Mitral valve regurgitation (disorder)", [])
    },
    "MitralValveStenosis": {"79619009": ("Mitral valve stenosis (disorder)", [])},
    "MitralValveStructure": {
        "91134007": ("Mitral valve structure (body structure)", [])
    },
    "Mixed": {"26242008": ("mixed", [3802])},
    "MixedAneurysm": {"85726003": ("mixed aneurysm", [3808])},
    "MixedBreedCat": {"132653001": ("Mixed breed cat", [7480, 7486])},
    "MixedBreedCattle": {"409906003": ("Mixed breed cattle", [7480, 7486])},
    "MixedBreedChicken": {"406723001": ("Mixed breed chicken", [7480, 7486])},
    "MixedBreedDog": {"132619000": ("Mixed breed dog", [7480, 7486])},
    "MixedBreedGoat": {"131607005": ("Mixed breed goat", [7480, 7486])},
    "MixedBreedHorse": {"406721004": ("Mixed breed horse", [7480, 7486])},
    "MixedBreedPig": {"417012009": ("Mixed breed pig", [7480, 7486])},
    "MixedBreedSheep": {"406722006": ("Mixed breed sheep", [7480, 7486])},
    "MixedDairyCattleBreed": {"133249000": ("Mixed dairy cattle breed", [7480])},
    "MixedIschemiaAndInfarction": {
        "428196007": ("Mixed Ischemia and Infarction", [3121])
    },
    "MixedMyocardialIschemiaAndInfarction": {
        "428196007": ("Mixed myocardial ischemia and infarction (disorder)", [])
    },
    "MixedPerijaneroCattleBreed": {
        "133821001": ("Mixed Perijanero cattle breed (organism)", [])
    },
    "MixedPerijaneroXZebuCattleBreed": {
        "133821001": ("Mixed Perijanero X zebu cattle breed", [7480])
    },
    "MixedRacialGroup": {"414752008": ("Mixed racial group", [6099])},
    "MixedSmallCellCarcinoma": {"21326004": ("Mixed small cell carcinoma", [638, 639])},
    "MixedVenousBlood": {"116176007": ("Mixed Venous Blood", [3520])},
    "Mixture": {"272163001": ("Mixture (qualifier value)", [])},
    "MiyakoHorseBreed": {"133013006": ("Miyako horse breed", [7480])},
    "Mode": {"373100007": ("Mode", [3627, 7464, 10072])},
    "ModeNumericEstimationTechnique": {
        "373100007": ("Mode - numeric estimation technique (qualifier value)", [])
    },
    "ModeneseCattleBreed": {"133519003": ("Modenese cattle breed", [7480])},
    "Moderate": {"6736007": ("Moderate", [3716, 3832, 6130])},
    "ModerateHypokinesis": {"371869002": ("Moderate Hypokinesis", [3703])},
    "ModerateHypokinesisOfCardiacWall": {
        "371869002": ("Moderate hypokinesis of cardiac wall (finding)", [])
    },
    "ModerateRisk": {"25594002": ("Moderate risk", [3118])},
    "ModerateRiskOfContextualQualifier": {
        "25594002": ("Moderate risk of (contextual qualifier) (qualifier value)", [])
    },
    "ModerateSeverityModifier": {
        "6736007": ("Moderate (severity modifier) (qualifier value)", [])
    },
    "ModerateToHighRisk": {"429557002": ("Moderate to high risk", [3118])},
    "ModerateToHighRiskOfContextualQualifier": {
        "429557002": (
            "Moderate to high risk of (contextual qualifier) (qualifier value)",
            [],
        )
    },
    "ModerateToSevere": {"371924009": ("Moderate to Severe", [3716])},
    "ModeratelyEnlargedCardiacChamber": {
        "373127006": ("Moderately enlarged cardiac chamber", [3122, 3705])
    },
    "ModicanaCattleBreed": {"131501004": ("Modicana cattle breed", [7480])},
    "ModicanaLowlandCattleBreed": {
        "131552002": ("Modicana lowland cattle breed", [7480])
    },
    "ModifiedBruceProtocol": {"129096001": ("Modified Bruce protocol", [3261, 12001])},
    "ModifiedNaughtonProtocol": {
        "129102008": ("Modified Naughton protocol", [3261, 12001])
    },
    "ModifiedTrichromeStain": {"406991004": ("modified trichrome stain", [8112])},
    "MoghaniSheepBreed": {"132788007": ("Moghani sheep breed", [7480])},
    "MoiCattleBreed": {"131502006": ("Moi cattle breed", [7480])},
    "MolesPerUnitVolume": {
        "282258000": ("moles per unit volume (qualifier value)", [])
    },
    "Molybdenum": {"71128006": ("Molybdenum", [10006, 10016, 10067])},
    "MonarchPigBreed": {"132173007": ("Monarch pig breed", [7480])},
    "MonchinaCattleBreed": {"133520009": ("Monchina cattle breed", [7480])},
    "MongCaiPigBreed": {"132343008": ("Mong Cai pig breed", [7480])},
    "MongallaCattleBreed": {"133729008": ("Mongalla cattle breed", [7480])},
    "MongolianCattleBreed": {"133250000": ("Mongolian cattle breed", [7480])},
    "MongolianHorseBreed": {"133014000": ("Mongolian horse breed", [7480])},
    "MonitoredAnesthesiaCareMAC": {
        "398239001": ("Monitored Anesthesia Care (MAC)", [611, 612])
    },
    "MonitoredAnesthesiaCareSedation": {
        "398239001": ("Monitored anesthesia care sedation (procedure)", [])
    },
    "MonitoringOfBloodPressureTemperaturePulseRateAndRespiratoryRateRegimeTherapy": {
        "304495004": (
            "Monitoring of blood pressure, temperature, pulse rate and respiratory rate (regime/therapy)",
            [],
        )
    },
    "MonitoringOfElectrocardiogramAtSurgery": {
        "91096005": ("Monitoring of electrocardiogram at surgery (procedure)", [])
    },
    "MonitoringOfRespirationRegimeTherapy": {
        "53617003": ("Monitoring of respiration (regime/therapy)", [])
    },
    "MonoclonalAntibodyI124": {"423249007": ("Monoclonal antibody I^124^", [4021])},
    "MonoclonalGammopathyOfUndeterminedSignificance": {
        "35601003": (
            "Monoclonal gammopathy of undetermined significance (morphologic abnormality)",
            [],
        )
    },
    "MontafonCattleBreed": {"133521008": ("Montafon cattle breed", [7480])},
    "MontanaPig": {"74517004": ("Montana pig (organism)", [])},
    "MontanaPigBreed": {"74517004": ("Montana pig breed", [7480])},
    "MontanaraCattleBreed": {"133590003": ("Montanara cattle breed", [7480])},
    "MontbeliardCattleBreed": {"133522001": ("Montbeliard cattle breed", [7480])},
    "MontdaleSheep": {"5164003": ("Montdale sheep (organism)", [])},
    "MontdaleSheepBreed": {"5164003": ("Montdale sheep breed", [7480])},
    "MontmorillonPigBreed": {"132191002": ("Montmorillon pig breed", [7480])},
    "Moore": {"422568001": ("Moore", [4012])},
    "MooreProjection": {"422568001": ("Moore projection (qualifier value)", [])},
    "MoraRomagnolaPigBreed": {"132246002": ("Mora Romagnola pig breed", [7480])},
    "MorabHorseBreed": {"133031007": ("Morab horse breed", [7480])},
    "MoradaNovaSheepBreed": {"132803005": ("Morada Nova sheep breed", [7480])},
    "MorangCattleBreed": {"133730003": ("Morang cattle breed", [7480])},
    "MoravaPigBreed": {"133206001": ("Morava pig breed", [7480])},
    "MoravianLargeYorkshirePigBreed": {
        "133216009": ("Moravian Large Yorkshire pig breed", [7480])
    },
    "MoreDefined": {"129729005": ("More defined", [6002, 6003, 6134])},
    "MoreIschemia": {"429477006": ("More ischemia", [3235])},
    "MorenasDelNoroesteCattleBreed": {
        "133523006": ("Morenas del Noroeste cattle breed", [7480])
    },
    "MorganHorse": {"21295007": ("Morgan horse (organism)", [])},
    "MorganHorseBreed": {"21295007": ("Morgan horse breed", [7480])},
    "Moribund": {"271593001": ("Moribund (finding)", [])},
    "MorochucoHorseBreed": {"132773002": ("Morochuco horse breed", [7480])},
    "Morphine": {"373529000": ("Morphine", [6089])},
    "MorphologicallyAbnormalStructure": {
        "49755003": (
            "Morphologically Abnormal Structure",
            [1000, 3487, 4266, 7150, 9502, 9580],
        )
    },
    "MorphologyWithinNormalLimits": {
        "125112009": ("Morphology within normal limits (finding)", [])
    },
    "MorticeProjection": {"260497009": ("Mortice projection", [4012])},
    "MoruchaCattleBreed": {"133251001": ("Morucha cattle breed", [7480])},
    "MoscowLonghairedToyTerrierDogBreed": {
        "132442002": ("Moscow Longhaired Toy Terrier dog breed", [7480])
    },
    "MostSignificant": {"371925005": ("Most significant", [220])},
    "MotilityStimulant": {"372607002": ("Motility stimulant (substance)", [])},
    "MouflonSheep": {"45690005": ("Mouflon sheep (organism)", [])},
    "MouflonSheepBreed": {"45690005": ("Mouflon sheep breed", [7480])},
    "MountainCurDogBreed": {"132441009": ("Mountain Cur dog breed", [7480])},
    "MouraPigBreed": {"132053000": ("Moura pig breed", [7480])},
    "Mouth": {"123851003": ("Mouth", [4, 645, 7151, 7192, 8134, 9514])},
    "MouthClosed": {"286866000": ("Mouth closed", [502, 4011])},
    "MouthRegionStructure": {
        "123851003": ("Mouth region structure (body structure)", [])
    },
    "MoxotóGoatBreed": {"131622000": ("Moxotó goat breed", [7480])},
    "MoyleHorseBreed": {"133032000": ("Moyle horse breed", [7480])},
    "MozambiqueAngoniCattleBreed": {
        "133737000": ("Mozambique Angoni cattle breed", [7480])
    },
    "MpwapwaCattleBreed": {"133738005": ("Mpwapwa cattle breed", [7480])},
    "MubanPigBreed": {"132843000": ("Muban pig breed", [7480])},
    "MucicarmineStain": {"406964001": ("mucicarmine stain", [8112])},
    "MucinousAdenocarcinoma": {
        "72495009": ("Mucinous adenocarcinoma (morphologic abnormality)", [])
    },
    "MucinousAdenocarcinomaColloidCarcinoma": {
        "72495009": ("Mucinous adenocarcinoma (Colloid carcinoma)", [6030, 6033])
    },
    "MucosaOfFloorOfMouth": {"36152006": ("Mucosa of floor of mouth", [4029, 8134])},
    "MucosaOfLowerLip": {"46353006": ("Mucosa of lower lip", [4029, 8134])},
    "MucosaOfMandibularGingiva": {
        "245823002": ("Mucosa of mandibular gingiva", [4029, 8134])
    },
    "MucosaOfMaxillaryGingiva": {
        "245814000": ("Mucosa of maxillary gingiva", [4029, 8134])
    },
    "MucosaOfTipOfTongue": {"245831007": ("Mucosa of tip of tongue", [4029, 8134])},
    "MucosaOfTongue": {"8001006": ("Mucosa of tongue", [4029, 8134])},
    "MucosaOfUpperLip": {"18444004": ("Mucosa of upper lip", [4029, 8134])},
    "MudiDogBreed": {"132620006": ("Mudi dog breed", [7480])},
    "MukotaPigBreed": {"132247006": ("Mukota pig breed", [7480])},
    "MulefootPigBreed": {"132250009": ("Mulefoot pig breed", [7480])},
    "MultiVesselCoronaryArteryDisease": {
        "371803003": ("Multi vessel coronary artery disease.", [3728])
    },
    "Multifocal": {"524008": ("Multifocal", [6128, 6335, 6336, 6337])},
    "MultifocalPrematureVentricularComplexes": {
        "10626002": ("Multifocal premature ventricular complexes (disorder)", [])
    },
    "MultifocalPvcs": {"10626002": ("Multifocal PVCs", [3234])},
    "MultiparametricMRIOfProstate": {
        "719178004": ("Multiparametric MRI of prostate", [100])
    },
    "MultiparametricMagneticResonanceImagingOfProstate": {
        "719178004": (
            "Multiparametric magnetic resonance imaging of prostate (procedure)",
            [],
        )
    },
    "MultipleAneurysm": {"125274006": ("multiple aneurysm", [3808])},
    "MultipleAneurysms": {
        "125274006": ("Multiple aneurysms (morphologic abnormality)", [])
    },
    "MultipleIrregularities": {"371922008": ("Multiple Irregularities", [3715])},
    "MultipleMyeloma": {"1162576007": ("Multiple myeloma", [64])},
    "MultipleMyelomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "55921005": (
            "Multiple myeloma, no International Classification of Diseases for Oncology subtype (morphologic "
            "abnormality)",
            [],
        )
    },
    "MultiplePregnancy": {"16356006": ("Multiple pregnancy", [6087, 6088])},
    "MunchkinLonghairCatBreed": {"132670009": ("Munchkin Longhair cat breed", [7480])},
    "MunchkinShorthairCatBreed": {
        "132669008": ("Munchkin Shorthair cat breed", [7480])
    },
    "MunichMiniaturePigBreed": {"132360003": ("Munich Miniature pig breed", [7480])},
    "MunsterLanderPointerDogBreed": {
        "132621005": ("Munster Lander Pointer dog breed", [7480])
    },
    "MuongKhuongPigBreed": {"132345001": ("Muong Khuong pig breed", [7480])},
    "MurakozHorseBreed": {"131992007": ("Murakoz horse breed", [7480])},
    "MurbodenCattleBreed": {"133268002": ("Murboden cattle breed", [7480])},
    "MurcianCattleBreed": {"133524000": ("Murcian cattle breed", [7480])},
    "MurcianPigBreed": {"132094008": ("Murcian pig breed", [7480])},
    "MurcianaGranadinaGoatBreed": {
        "131698009": ("Murciana-Granadina goat breed", [7480])
    },
    "MurgeseHorse": {"406715003": ("Murgese horse (organism)", [])},
    "MurgeseHorseBreed": {"406715003": ("Murgese horse breed", [7480])},
    "MurgheseHorseBreed": {"132776005": ("Murghese horse breed", [7480])},
    "MurinePoliovirus": {"42024000": ("Murine poliovirus (organism)", [])},
    "MurleCattleBreed": {"133739002": ("Murle cattle breed", [7480])},
    "Murmur": {"373112006": ("Murmur", [12246])},
    "MurnauWerdenfelsCattleBreed": {
        "133525004": ("Murnau-Werdenfels cattle breed", [7480])
    },
    "MurrayGrayCattleBreed": {"133471008": ("Murray Gray cattle breed", [7480])},
    "MusGenus": {"447482001": ("Mus genus", [7454])},
    "MusMusculus": {"447612001": ("Mus musculus", [7454])},
    "Muscle": {"71616004": ("Muscle", [6202, 6204, 8134, 10044, 10060])},
    "MuscleBridge": {"424045003": ("Muscle Bridge", [3712])},
    "MuscleOfHead": {"22688005": ("Muscle of head", [4028, 7151, 7192, 8134, 9514])},
    "MuscleOfLowerLimb": {
        "102292000": (
            "Muscle of lower limb",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "MuscleOfNeck": {"81727001": ("Muscle of neck", [4028, 7151, 7192, 8134, 9514])},
    "MuscleOfUpperLimb": {
        "30608006": (
            "Muscle of upper limb",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "MuscleTissue": {"91727004": ("Muscle", [7151, 7166, 7191])},
    "Muscular": {"71616004": ("Muscular", [6100])},
    "Muscular_Inactive": {"263816006": ("Muscular (qualifier value)", [])},
    "MusculoskeletalExtensionFunction": {
        "24154002": ("Musculoskeletal extension, function (observable entity)", [])
    },
    "MusculoskeletalTorsion": {
        "51795009": ("Musculoskeletal torsion (observable entity)", [])
    },
    "MustangHorseBreed": {"133033005": ("Mustang horse breed", [7480])},
    "MustardOperation": {"40250003": ("Mustard operation", [12247])},
    "MustardOrSenningCommonSystemicVenousPathway": {
        "443809000": ("Mustard or Senning Common Systemic Venous Pathway", [12282])
    },
    "MustardOrSenningInferiorVenaCavaPathway": {
        "444329004": ("Mustard or Senning Inferior Vena Cava Pathway", [12282])
    },
    "MustardOrSenningPulmonaryVenousPathway": {
        "443907004": ("Mustard or Senning Pulmonary Venous Pathway", [12283])
    },
    "MustardOrSenningSuperiorVenaCavaPathway": {
        "444177009": ("Mustard or Senning Superior Vena Cava Pathway", [12282])
    },
    "MustelaPutoriusFuro": {"449310008": ("Mustela putorius furo", [7454])},
    "MustelaPutoriusSubspeciesFuro": {
        "449310008": ("Mustela putorius subspecies furo (organism)", [])
    },
    "MycoticAneurysm": {"51668007": ("mycotic aneurysm", [3808])},
    "MyeloperoxidaseStain": {"255805004": ("myeloperoxidase stain", [8112])},
    "MyocardialBridgeOfCoronaryArtery": {
        "424045003": ("Myocardial bridge of coronary artery (disorder)", [])
    },
    "MyocardialInfarction": {"22298006": ("Myocardial Infarction", [3121])},
    "MyocardialIschemia": {"414795007": ("Myocardial ischemia", [3121])},
    "MyocardialIschemiaManifestOnStressTestStatusPostMyocardialInfarction": {
        "371824008": (
            "Myocardial ischemia manifest on stress test status post myocardial infarction (finding)",
            [],
        )
    },
    "MyocardialPerfusion": {
        "251053005": ("Myocardial perfusion (observable entity)", [])
    },
    "MyocardialPerfusionNormal": {"301121007": ("Myocardial perfusion normal", [3120])},
    "MyocardialPerfusionStressImagingUsingThallium201": {
        "431511008": (
            "Myocardial perfusion stress imaging using Thallium 201 (procedure)",
            [],
        )
    },
    "Myocarditis": {"50920009": ("Myocarditis", [3700])},
    "MyocardiumOfAnterolateralRegion": {
        "73050001": ("myocardium of anterolateral region", [3718])
    },
    "MyocardiumOfApexOfHeart": {"47962008": ("myocardium of apex of heart", [3718])},
    "MyocardiumOfDiaphragmaticRegion": {
        "72542009": ("myocardium of diaphragmatic region", [3718])
    },
    "MyocardiumOfInferolateralRegion": {
        "16239001": ("myocardium of inferolateral region", [3718])
    },
    "MyocardiumOfPosterolateralRegion": {
        "33272004": ("myocardium of posterolateral region", [3718])
    },
    "Myofibroblastoma": {"128738002": ("Myofibroblastoma", [6030, 6031])},
    "Myopia": {"57190000": ("Myopia", [4238])},
    "MyotonicGoatBreed": {"131623005": ("Myotonic goat breed", [7480])},
    "MytileneHorseBreed": {"132777001": ("Mytilene horse breed", [7480])},
    "N3MetastasisToIpsilateralInternalMammaryLymphNodes": {
        "369991007": (
            "N3: Metastasis to ipsilateral internal mammary lymph node(s) (finding)",
            [],
        )
    },
    "NAcetylLAspartate": {"115391007": ("N-acetyl-L-aspartate (substance)", [])},
    "NAcetylaspartate": {
        "115391007": ("N-acetylaspartate", [218, 4032, 4033, 7180, 7186, 7469])
    },
    "NDamaCattleBreed": {"133253003": ("N'dama cattle breed", [7480])},
    "NDamaSangaCattleBreed": {"131505008": ("N'Dama Sanga cattle breed", [7480])},
    "NGabouCattleBreed": {"133534009": ("N'Gabou cattle breed", [7480])},
    "NGaoundereCattleBreed": {"133742008": ("N'Gaoundere cattle breed", [7480])},
    "NSAID": {"372665008": ("NSAID", [621, 622])},
    "NYHAClassI": {"420300004": ("NYHA Class I", [3736])},
    "NYHAClassII": {"421704003": ("NYHA Class II", [3736])},
    "NYHAClassIII": {"420913000": ("NYHA Class III", [3736])},
    "NYHAClassIV": {"422293003": ("NYHA Class IV", [3736])},
    "NachiGoatBreed": {"131624004": ("Nachi goat breed", [7480])},
    "NadbuzanskaPigBreed": {"133226002": ("Nadbuzanska pig breed", [7480])},
    "NagoriCattleBreed": {"133619009": ("Nagori cattle breed", [7480])},
    "NailUnitOfFifthToe": {"770820003": ("Nail unit of fifth toe", [4029, 8134])},
    "NailUnitOfFinger": {"770809003": ("Nail unit of finger", [4029, 8134])},
    "NailUnitOfFourthToe": {"770821004": ("Nail unit of fourth toe", [4029, 8134])},
    "NailUnitOfGreatToe": {"770822006": ("Nail unit of great toe", [4029, 8134])},
    "NailUnitOfIndexFinger": {"770815003": ("Nail unit of index finger", [4029, 8134])},
    "NailUnitOfLittleFinger": {
        "770818001": ("Nail unit of little finger", [4029, 8134])
    },
    "NailUnitOfMiddleFinger": {
        "770816002": ("Nail unit of middle finger", [4029, 8134])
    },
    "NailUnitOfRingFinger": {"770817006": ("Nail unit of ring finger", [4029, 8134])},
    "NailUnitOfSecondToe": {"770823001": ("Nail unit of second toe", [4029, 8134])},
    "NailUnitOfThirdToe": {"770825008": ("Nail unit of third toe", [4029, 8134])},
    "NailUnitOfThumb": {"770810008": ("Nail unit of thumb", [4029, 8134])},
    "NailUnitOfToe": {"770805009": ("Nail unit of toe", [4029, 8134])},
    "NakaliCattleBreed": {"133740000": ("Nakali cattle breed", [7480])},
    "NamaCattleBreed": {"131503001": ("Nama cattle breed", [7480])},
    "NamibDesertHorseHorseBreed": {
        "132778006": ("Namib Desert Horse horse breed", [7480])
    },
    "NanjingBlackPigBreed": {"132334004": ("Nanjing Black pig breed", [7480])},
    "NantaisCattleBreed": {"133532008": ("Nantais cattle breed", [7480])},
    "NanyangCattleBreed": {"131504007": ("Nanyang cattle breed", [7480])},
    "NaphthaleneBlack12BStain": {"16788000": ("naphthalene black 12B stain", [8112])},
    "NaphtholASDChloracetateEsteraseStain": {
        "406973009": ("naphthol-AS-D-chloracetate esterase stain", [8112])
    },
    "NaphtholGreenBStain": {"14958002": ("naphthol green B stain", [8112])},
    "NaphtholYellowSStain": {"111101002": ("naphthol yellow S stain", [8112])},
    "NarcoticOpiateAntagonist": {
        "372656001": ("Narcotic (opiate) antagonist", [621, 622])
    },
    "Narrow": {"134223000": ("Narrow", [6107])},
    "NarymHorseBreed": {"132960009": ("Narym horse breed", [7480])},
    "NasalBone": {
        "74386004": (
            "Nasal bone",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "NasalBoneStructure": {"74386004": ("Nasal bone structure (body structure)", [])},
    "NasalCavity": {"279549004": ("nasal cavity", [7601, 8134])},
    "NasalCavityStructure": {
        "279549004": ("Nasal cavity structure (body structure)", [])
    },
    "NasalCongestion": {"68235000": ("Nasal Congestion", [60, 9300])},
    "NasalRoute": {"46713006": ("Nasal route", [11])},
    "NasalSinus": {"2095001": ("Nasal sinus", [7482, 7483, 8134])},
    "NasalSinusStructure": {"2095001": ("Nasal sinus structure (body structure)", [])},
    "NasalStructure": {"45206002": ("Nasal structure (body structure)", [])},
    "NasolabialLymphNode": {"144026003": ("nasolabial lymph node", [7600, 8134])},
    "Nasopharyngeal": {"71836000": ("nasopharynx", [7601, 8134])},
    "Nasopharynx": {"360955006": ("Nasopharynx", [4, 4040, 7151, 7192, 8134, 9514])},
    "NationalShowHorseHorseBreed": {
        "132997004": ("National Show Horse horse breed", [7480])
    },
    "NationalSpottedSaddleHorseHorseBreed": {
        "132961008": ("National Spotted Saddle Horse horse breed", [7480])
    },
    "NaturalBrother": {"60614009": ("Natural brother", [7450, 7451])},
    "NaturalDaughter": {"83420006": ("Natural daughter", [7450, 7451])},
    "NaturalFather": {"9947008": ("Natural father", [7450, 7451])},
    "NaturalGrandFather": {"62296006": ("Natural grand-father", [7450, 7451])},
    "NaturalGrandMother": {"17945006": ("Natural grand-mother", [7450, 7451])},
    "NaturalGrandfather": {"62296006": ("Natural grandfather (person)", [])},
    "NaturalGrandmother": {"17945006": ("Natural grandmother (person)", [])},
    "NaturalMother": {"65656005": ("Natural mother", [7450, 7451])},
    "NaturalSister": {"73678001": ("Natural sister", [7450, 7451])},
    "NaturalSon": {"113160008": ("Natural son", [7450, 7451])},
    "NaughtonProtocol": {"129101001": ("Naughton protocol", [3261, 12001])},
    "Nausea": {"422587007": ("Nausea", [3220])},
    "NauseaAndVomiting": {"16932000": ("Nausea and vomiting", [60, 9300])},
    "NavajoChurroSheepBreed": {"131838009": ("Navajo-Churro sheep breed", [7480])},
    "NavajoSheep": {"59210004": ("Navajo sheep (organism)", [])},
    "NavajoSheepBreed": {"59210004": ("Navajo sheep breed", [7480])},
    "NavicularOfForefoot": {"30518006": ("Navicular of forefoot", [7482, 7483, 8134])},
    "NavicularOfHindfoot": {"75772009": ("Navicular of hindfoot", [7482, 7483, 8134])},
    "NdaguCattleBreed": {"133817008": ("Ndagu cattle breed (organism)", [])},
    "NdaguXZebuCattleBreed": {"133817008": ("Ndagu X zebu cattle breed", [7480])},
    "NebelungCatBreed": {"132671008": ("Nebelung cat breed", [7480])},
    "Neck": {
        "45048000": ("Neck", [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "NeckAndChest": {
        "417437006": (
            "Neck and Chest",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "NeckAndOrChestAndOrAbdomenAndOrPelvis": {
        "416319003": (
            "Neck and/or chest and/or abdomen and/or pelvis (body structure)",
            [],
        )
    },
    "NeckAndOrChestAndOrAbdomenStructure": {
        "416152001": ("Neck and/or chest and/or abdomen structure (body structure)", [])
    },
    "NeckAndOrChestStructure": {
        "417437006": ("Neck and/or chest structure (body structure)", [])
    },
    "NeckChestAbdomenAndPelvis": {
        "416319003": (
            "Neck, Chest, Abdomen and Pelvis",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "NeckChestAndAbdomen": {
        "416152001": (
            "Neck, Chest and Abdomen",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "NeckOfRib": {"72184008": ("Neck of rib", [6115, 8134])},
    "NeckStructure": {"45048000": ("Neck structure (body structure)", [])},
    "Necrosis": {"6574001": ("Necrosis", [7151, 7159, 7168, 7194])},
    "Needle": {
        "79068005": ("Needle", [69, 6102, 6138, 6202, 6203, 6404, 7151, 7157, 7193])
    },
    "NeedleDevice": {"79068005": ("Needle, device (physical object)", [])},
    "Neer": {"422795009": ("Neer", [4012])},
    "NeerProjection": {"422795009": ("Neer projection (qualifier value)", [])},
    "Negative": {"260385009": ("Negative", [250])},
    "NegativeLowerBodyPressure": {
        "128966001": ("Negative lower body pressure", [91, 3271])
    },
    "NegativeLowerBodyPressurePhysiologicChallenge": {
        "128966001": (
            "Negative lower body pressure physiologic challenge (procedure)",
            [],
        )
    },
    "NegraSerranaGoatBreed": {"131612006": ("Negra Serrana goat breed", [7480])},
    "NeijiangPigBreed": {"132249009": ("Neijiang pig breed", [7480])},
    "NejdiCattleBreed": {"133533003": ("Nejdi cattle breed", [7480])},
    "NelloreCattleBreed": {"76604009": ("Nellore cattle breed (organism)", [])},
    "NelloreCowBreed": {"76604009": ("Nellore cow breed", [7480])},
    "NeloreCattleBreed": {"133620003": ("Nelore cattle breed", [7480])},
    "NeoAdjuvantIntent": {"373847000": ("Neo-adjuvant intent", [3629])},
    "NeoAorta": {"14944004": ("Neo-aorta (primitive aorta)", [3604, 3606, 3630])},
    "NeoaorticRoot": {"443726001": ("Neoaortic Root", [12289])},
    "NeoaorticValve": {"443283007": ("Neoaortic Valve", [12289])},
    "NeonatalIntensiveCare": {"405269005": ("Neonatal Intensive Care", [7030])},
    "NeonatalIntensiveCareUnit": {
        "405269005": ("Neonatal intensive care unit (environment)", [])
    },
    "NeonatalPulmonaryArteryPrimitivePA": {
        "91707000": ("Neonatal pulmonary artery (primitive PA)", [3604, 3606, 3630])
    },
    "Neoplasm": {"108369006": ("Neoplasm", [7151, 7159, 7194])},
    "NeoplasmMetastatic": {
        "14799000": ("Neoplasm, metastatic (morphologic abnormality)", [])
    },
    "NeoplasmOfColon": {"126838000": ("Neoplasm of colon (disorder)", [])},
    "NeoplasmOfSkinOfBreast": {
        "126510002": ("Neoplasm of skin of breast (disorder)", [])
    },
    "NeoplasmOfTheMammarySkin": {
        "126510002": ("Neoplasm of the mammary skin", [6030, 6033])
    },
    "NeoplasmPrimary": {"372087000": ("Neoplasm, Primary", [7151, 7159, 7194])},
    "NeoplasmSecondary": {"14799000": ("Neoplasm, Secondary", [7151, 7159, 7194])},
    "NeopolitanMastiffDogBreed": {
        "132599007": ("Neopolitan Mastiff dog breed", [7480])
    },
    "NepaleseHillZebuCattleBreed": {
        "133741001": ("Nepalese Hill Zebu cattle breed", [7480])
    },
    "Nephrology": {"309936008": ("Nephrology", [7030])},
    "NephrologyDepartment": {"309936008": ("Nephrology department (environment)", [])},
    "Nerve": {"91728009": ("Nerve", [7151, 7166, 7191])},
    "NervePart": {"119410002": ("Nerve part (body structure)", [])},
    "NeuroendocrineTumor": {"255046005": ("Neuroendocrine tumor", [638, 639])},
    "Neurofibroma": {"404029005": ("Neurofibroma", [6030, 6031])},
    "NeurofibromaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "89084002": (
            "Neurofibroma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "Neurofibromatosis": {"81669005": ("Neurofibromatosis", [6030, 6031])},
    "Neurohypophysis": {
        "37512009": ("Neurohypophysis", [7151, 7153, 7192, 8134, 9514])
    },
    "NeurohypophysisStructure": {
        "37512009": ("Neurohypophysis structure (body structure)", [])
    },
    "Neurology": {"309937004": ("Neurology", [7030])},
    "NeurologyDepartment": {"309937004": ("Neurology department (environment)", [])},
    "NeuromuscularBlockingNMBDepolarizing": {
        "373250005": ("NeuroMuscular Blocking (NMB) - depolarizing", [621, 622])
    },
    "NeuromuscularBlockingNMBNonDepolarizing": {
        "372790002": ("NeuroMuscular Blocking (NMB) - non depolarizing", [621, 622])
    },
    "Neurosurgery": {"310159002": ("Neurosurgery", [7030])},
    "NeurosurgicalService": {
        "310159002": ("Neurosurgical service (qualifier value)", [])
    },
    "NeutralBufferedFormalin": {"434162003": ("Neutral Buffered Formalin", [8114])},
    "NeutralRedStain": {"67956008": ("neutral red stain", [8112])},
    "NeutrophilAlkalinePhosphataseStain": {
        "255806003": ("neutrophil alkaline phosphatase stain", [8112])
    },
    "NeverSmokedTobacco": {"266919005": ("Never smoked tobacco (finding)", [])},
    "NewFinding": {"129721008": ("New finding", [6002, 6003])},
    "NewFindingSincePreviousMammogram": {
        "129721008": ("New finding since previous mammogram (finding)", [])
    },
    "NewForestPony": {"26699009": ("New Forest pony (organism)", [])},
    "NewForestPonyHorseBreed": {"26699009": ("New Forest pony horse breed", [7480])},
    "NewFuchsinStain": {"31714001": ("new fuchsin stain", [8112])},
    "NewHuaiPigBreed": {"132136009": ("New Huai pig breed", [7480])},
    "NewInfarction": {"429391004": ("New infarction", [3235])},
    "NewIschemia": {"428927006": ("New ischemia", [3235])},
    "NewKirgizHorseBreed": {"133010009": ("New Kirgiz horse breed", [7480])},
    "NewLeftVentricularWallMotionAbnormalityComparedToPriorStudy": {
        "429058004": (
            "New left ventricular wall motion abnormality compared to prior study (finding)",
            [],
        )
    },
    "NewMyocardialInfarctionComparedToPriorStudy": {
        "429391004": ("New myocardial infarction compared to prior study (finding)", [])
    },
    "NewMyocardialIschemiaComparedToPriorStudy": {
        "428927006": ("New myocardial ischemia compared to prior study (finding)", [])
    },
    "NewWallMotionAbnormality": {"429058004": ("New wall motion abnormality", [3237])},
    "NewYorkHeartAssociationClassificationClassI": {
        "420300004": (
            "New York Heart Association Classification - Class I (finding)",
            [],
        )
    },
    "NewYorkHeartAssociationClassificationClassII": {
        "421704003": (
            "New York Heart Association Classification - Class II (finding)",
            [],
        )
    },
    "NewYorkHeartAssociationClassificationClassIII": {
        "420913000": (
            "New York Heart Association Classification - Class III (finding)",
            [],
        )
    },
    "NewYorkHeartAssociationClassificationClassIV": {
        "422293003": (
            "New York Heart Association Classification - Class IV (finding)",
            [],
        )
    },
    "NewYorkRedPigBreed": {"132268007": ("New York Red pig breed", [7480])},
    "NewZealandHeadingDogBreed": {
        "132463000": ("New Zealand Heading Dog breed", [7480])
    },
    "NewZealandRabbitBreed": {"132901006": ("New Zealand rabbit breed", [7480])},
    "NewfoundlandDog": {"52253003": ("Newfoundland dog", [7480])},
    "NewfoundlandPonyHorseBreed": {
        "133036002": ("Newfoundland Pony horse breed", [7480])
    },
    "NewfoundlandSheepBreed": {"132701004": ("Newfoundland sheep breed", [7480])},
    "NgandaCattleBreed": {"131506009": ("Nganda cattle breed", [7480])},
    "NghiaBinhPigBreed": {"132368005": ("Nghia Binh pig breed", [7480])},
    "NguniCattleBreed": {"131435004": ("Nguni cattle breed", [7480])},
    "NickelCobaltChromium": {"261249004": ("Nickel cobalt chromium", [300])},
    "NickelTitanium": {"261250004": ("Nickel Titanium", [7300, 9579])},
    "Nifedipine": {"85272000": ("Nifedipine", [10])},
    "NigerianDwarfGoatBreed": {"131625003": ("Nigerian Dwarf goat breed", [7480])},
    "NigerianHorseBreed": {"132962001": ("Nigerian horse breed", [7480])},
    "NigerianPigBreed": {"133186006": ("Nigerian pig breed", [7480])},
    "NightBlueStain": {"406965000": ("night blue stain", [8112])},
    "NileBlueStain": {"77073008": ("nile blue stain", [4412, 8112])},
    "NiloCanastraPigBreed": {"132057004": ("Nilo-Canastra pig breed", [7480])},
    "NiloPigBreed": {"132161007": ("Nilo pig breed", [7480])},
    "NiloticSangaCattleBreed": {"131507000": ("Nilotic Sanga cattle breed", [7480])},
    "NimariCattleBreed": {"133621004": ("Nimari cattle breed", [7480])},
    "NinetiethPercentile": {
        "371887004": ("Ninetieth percentile (qualifier value)", [])
    },
    "NinetyFifthPercentile": {
        "371889001": ("Ninety-fifth percentile (qualifier value)", [])
    },
    "NingxiangPigBreed": {"132245003": ("Ningxiang pig breed", [7480])},
    "NinthIntercostalSpace": {"117141000": ("Ninth Intercostal Space", [5])},
    "Niobium": {"767776000": ("Niobium", [10006, 10067])},
    "Nipple": {"24142002": ("Nipple", [6014, 6016, 6054])},
    "NippleDischargeSymptom": {"162164007": ("Nipple discharge symptom", [6055])},
    "NippleInProfile": {"442581004": ("Nipple in profile", [4015])},
    "NippleInProfileMammographyView": {
        "442581004": ("Nipple in profile mammography view (qualifier value)", [])
    },
    "NippleProblem": {"290119008": ("Nipple problem", [6055])},
    "NippleRetraction": {"271955004": ("Nipple retraction", [6039, 6056])},
    "NippleStructure": {"24142002": ("Nipple structure (body structure)", [])},
    "NitraHybridPigBreed": {"133219002": ("Nitra Hybrid pig breed", [7480])},
    "NitrateBasedVasodilatingAgent": {
        "372700007": ("Nitrate-based vasodilating agent (substance)", [])
    },
    "NitrateVasodilator": {"372700007": ("Nitrate vasodilator", [3760])},
    "NitrazineYellowStain": {"86750008": ("nitrazine yellow stain", [8112])},
    "Nitroglycerin": {"71759000": ("Nitroglycerin", [10])},
    "NitrousOxide": {"111132001": ("Nitrous oxide", [623, 624])},
    "NkediCattleBreed": {"133743003": ("Nkedi cattle breed", [7480])},
    "NkoneCattleBreed": {"131508005": ("Nkone cattle breed", [7480])},
    "No": {"373067005": ("No", [230, 231])},
    "NoChange": {"260388006": ("No change", [3235, 3236, 3237])},
    "NoFamilyHistoryBreastCarcinoma": {
        "313376005": ("No family history: breast carcinoma (situation)", [])
    },
    "NoFamilyHistoryCardiovascularDisease": {
        "160270001": ("No family history: Cardiovascular disease (situation)", [])
    },
    "NoFamilyHistoryDiabetes": {
        "160274005": ("No family history diabetes (situation)", [])
    },
    "NoFamilyHistoryOfBreastCarcinoma": {
        "313376005": ("No family history of breast carcinoma", [6081, 6087])
    },
    "NoFamilyHistoryOfCardiovascularDisease": {
        "160270001": ("No family history of cardiovascular disease", [3758])
    },
    "NoFamilyHistoryOfDiabetes": {
        "160274005": ("No family history of diabetes", [3758])
    },
    "NoHistoryOfSmoking": {"266919005": ("No History of Smoking", [3724])},
    "NoMotorResponseToCommand": {
        "298336006": ("No motor response to command", [60, 9300])
    },
    "NoSignificantChangeSincePreviousMammogram": {
        "129723006": ("No significant change since previous mammogram (finding)", [])
    },
    "NoSignificantChangesInTheFinding": {
        "129723006": ("No significant changes in the finding", [6002, 6003, 6134])
    },
    "NoStatusChange": {"260388006": ("No status change (qualifier value)", [])},
    "NoTailSheep": {"112486002": ("No-tail sheep (organism)", [])},
    "NoTailSheepBreed": {"112486002": ("No-tail sheep breed", [7480])},
    "NoThrombus": {"373140001": ("No Thrombus", [3714])},
    "NodeCategoryFinding": {"385382003": ("Node category finding (finding)", [])},
    "Nodular": {"255288007": ("Nodular", [6132])},
    "Nodule": {
        "27925004": ("Nodule", [6102, 6104, 6335, 6336, 6337, 7151, 7159, 7194])
    },
    "Nolke": {"399002000": ("Nolke", [4012])},
    "NolkeProjection": {"399002000": ("Nolke projection (qualifier value)", [])},
    "NomaHorseBreed": {"133037006": ("Noma horse breed", [7480])},
    "NomarskiPrism": {"445633006": ("Nomarski prism", [8124])},
    "NomarskiPrismOfOpticalMicroscope": {
        "445633006": ("Nomarski prism of optical microscope (physical object)", [])
    },
    "Nominal": {"117362005": ("Nominal", [10072])},
    "NominalValue": {"117362005": ("Nominal value (qualifier value)", [])},
    "NonAlcoholicFattyLiver": {
        "197315008": ("Non-alcoholic fatty liver (disorder)", [])
    },
    "NonCalcified": {"17589002": ("non-calcified", [3802])},
    "NonCardiacChestPain": {"274668005": ("Non-cardiac chest pain (finding)", [])},
    "NonCoronarySinus": {"24865005": ("Non-coronary Sinus", [12291])},
    "NonElectricHeatingPad": {"39790008": ("Non-electric heating pad", [635])},
    "NonElectricHeatingPadDevice": {
        "39790008": ("Non-electric heating pad, device (physical object)", [])
    },
    "NonHodgkinLymphoma": {"1172592001": ("Non-Hodgkin's lymphoma", [6030, 6033])},
    "NonHodgkinLymphomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "1929004": (
            "Non-Hodgkin lymphoma, no International Classification of Diseases for Oncology subtype (morphologic "
            "abnormality)",
            [],
        )
    },
    "NonInfiltratingIntraductalCarcinoma": {
        "1162814007": ("Non-infiltrating intraductal carcinoma", [6159])
    },
    "NonRadiopaqueMedium": {"43538006": ("Non radiopaque medium", [12, 3850])},
    "NonRestrictive": {"373131000": ("Non-restrictive", [3707])},
    "NonRestrictiveVentricularSeptalDefect": {
        "373131000": ("Non-restrictive ventricular septal defect (disorder)", [])
    },
    "NonSTElevationMyocardialInfarction": {
        "401314000": ("Non ST Elevation Myocardial Infarction", [3723])
    },
    "NonSmallCellCarcinoma": {"128632008": ("Non-small cell carcinoma", [638, 639])},
    "NonSmallCellLungCancer": {"254637007": ("Non-small cell lung cancer", [638, 639])},
    "NonSteroidalAntiInflammatoryAgent": {
        "372665008": ("Non-steroidal anti-inflammatory agent (substance)", [])
    },
    "NonSustainedVentricularTachycardia": {
        "66657009": ("Non-sustained ventricular tachycardia", [3230])
    },
    "NonalcoholicFattyLiverDiseaseNAFLD": {
        "1231824009": ("Nonalcoholic fatty liver disease (NAFLD)", [12325])
    },
    "NonalcoholicFattyLiverNAFL": {
        "197315008": ("Nonalcoholic fatty liver (NAFL)", [12325])
    },
    "NonalcoholicSteatohepatitis": {
        "442685003": ("Nonalcoholic steatohepatitis (disorder)", [])
    },
    "NonalcoholicSteatohepatitisNASH": {
        "442685003": ("Nonalcoholic steatohepatitis (NASH)", [12325])
    },
    "NoncardiacChestPain": {"274668005": ("Noncardiac Chest Pain", [3202, 3220])},
    "NoncontactFundusLens": {"410685001": ("Noncontact fundus lens", [4205])},
    "NondepolarizingAgent": {"372790002": ("Nondepolarizing agent (substance)", [])},
    "None_": {"260413007": ("None", [606, 3707, 3716, 4253])},
    "NonionicIodinatedContrastMedia": {
        "96388005": ("Nonionic iodinated contrast media (product)", [])
    },
    "NoniusHorseBreed": {"133045001": ("Nonius horse breed", [7480])},
    "NonspecificEsteraseStain": {"255807007": ("nonspecific esterase stain", [8112])},
    "NonspecificSTTAbnormality": {
        "428750005": ("Nonspecific ST-T abnormality", [3230])
    },
    "NonspecificSTTAbnormalityOnElectrocardiogram": {
        "428750005": ("Nonspecific ST-T abnormality on electrocardiogram (finding)", [])
    },
    "NooitgedachtHorseBreed": {"133046000": ("Nooitgedacht horse breed", [7480])},
    "NooitgedachtPonyHorseBreed": {
        "133038001": ("Nooitgedacht Pony horse breed", [7480])
    },
    "NoordhollanderSheepBreed": {"132690000": ("Noordhollander sheep breed", [7480])},
    "NorbottenspetsDogBreed": {"132592003": ("Norbottenspets dog breed", [7480])},
    "NordlandHorseBreed": {"133039009": ("Nordland horse breed", [7480])},
    "NorfolkHornSheepBreed": {"131837004": ("Norfolk Horn sheep breed", [7480])},
    "NorfolkTerrier": {"62790004": ("Norfolk terrier", [7480])},
    "Norgaard": {"399157000": ("Norgaard", [4012])},
    "NorgaardProjection": {
        "399157000": ("Norgaard's projection (qualifier value)", [])
    },
    "NoricHorseBreed": {"133040006": ("Noric horse breed", [7480])},
    "NorlinePigBreed": {"132027008": ("Norline pig breed", [7480])},
    "Normal": {
        "17621005": (
            "Normal",
            [222, 242, 3119, 3210, 3211, 7151, 7159, 7165, 7168, 7169, 7194, 7196],
        )
    },
    "NormalAorticRoot": {"373132007": ("Normal Aortic Root", [3709])},
    "NormalCoronaryArteries": {"371860003": ("Normal coronary arteries", [3728])},
    "NormalExtracardiacTracerUptake": {
        "428552000": ("Normal extracardiac tracer uptake (finding)", [])
    },
    "NormalExtracardiacUptake": {"428552000": ("Normal extracardiac uptake", [3116])},
    "NormalGeneralBodyFunction": {
        "81323004": ("Normal general body function (finding)", [])
    },
    "NormalLeftAndRightHeartHemodynamics": {
        "371858000": ("Normal left and right heart hemodynamics", [3728])
    },
    "NormalLeftHeartHemodynamics": {
        "371856001": ("Normal left heart hemodynamics", [3728])
    },
    "NormalLeftVentricularSystolicFunctionAndWallMotion": {
        "371857005": (
            "Normal left ventricular systolic function and wall motion",
            [3728],
        )
    },
    "NormalOverallCardiacContractility": {
        "373129009": ("Normal overall cardiac contractility (finding)", [])
    },
    "NormalRange": {"260395002": ("Normal Range", [3642])},
    "NormalRangeLowerLimit": {"385524004": ("Normal Range Lower Limit", [223])},
    "NormalRangeUpperLimit": {"371933006": ("Normal Range Upper Limit", [223])},
    "NormalRespiratoryRhythm": {"5467003": ("normal respiratory rhythm", [3416])},
    "NormalRightHeartHemodynamics": {
        "371859008": ("Normal right heart hemodynamics", [3728])
    },
    "NormalRisk": {"427986001": ("Normal risk", [3118])},
    "NormalRiskOfContextualQualifier": {
        "427986001": ("Normal risk of (contextual qualifier) (qualifier value)", [])
    },
    "NormalSaline": {"262003004": ("Normal saline (product)", [])},
    "NormalShape": {"31842008": ("Normal shape", [6039])},
    "NormalSinusRhythm": {"64730000": ("normal sinus rhythm", [3826])},
    "NormalSizeCardiacChamber": {
        "373124004": ("Normal size cardiac chamber", [3122, 3705])
    },
    "NormalVentricularWallMotion": {
        "373122000": ("Normal ventricular wall motion (finding)", [])
    },
    "NormalWallContractility": {"373129009": ("Normal wall contractility", [3706])},
    "NormalWallMotion": {"373122000": ("Normal wall motion", [3703])},
    "NormalityUndetermined": {"371934000": ("Normality Undetermined", [222, 242])},
    "NormanCoachHorse": {"39532001": ("Norman coach horse (organism)", [])},
    "NormanCoachHorseBreed": {"39532001": ("Norman coach horse breed", [7480])},
    "NormandPigBreed": {"132251008": ("Normand pig breed", [7480])},
    "NormandeCattleBreed": {"81267004": ("Normande cattle breed (organism)", [])},
    "NormandieCowBreed": {"81267004": ("Normandie cow breed", [7480])},
    "NormanzuCattleBreed": {"133818003": ("Normanzu cattle breed (organism)", [])},
    "NormanzuXZebuCattleBreed": {"133818003": ("Normanzu X zebu cattle breed", [7480])},
    "NorthAmericanShepherdDogBreed": {
        "132630002": ("North American Shepherd dog breed", [7480])
    },
    "NorthBangladeshGrayCattleBreed": {
        "133744009": ("North Bangladesh Gray cattle breed", [7480])
    },
    "NorthCaucasusPigBreed": {"132102003": ("North Caucasus pig breed", [7480])},
    "NorthCountyCheviotSheep": {
        "87962009": ("North County cheviot sheep (organism)", [])
    },
    "NorthCountyCheviotSheepBreed": {
        "87962009": ("North County cheviot sheep breed", [7480])
    },
    "NorthEastChinaSpottedPigBreed": {
        "132331007": ("North East China Spotted pig breed", [7480])
    },
    "NorthFinncattleCattleBreed": {
        "133535005": ("North Finncattle cattle breed", [7480])
    },
    "NorthFujianBlackAndWhitePigBreed": {
        "132282007": ("North Fujian Black-and-White pig breed", [7480])
    },
    "NorthMalawiAngoniCattleBreed": {
        "131509002": ("North Malawi Angoni cattle breed", [7480])
    },
    "NorthSiberianPigBreed": {"132114006": ("North Siberian pig breed", [7480])},
    "NorthSomaliZebuCattleBreed": {
        "133745005": ("North Somali Zebu cattle breed", [7480])
    },
    "NorthSwedishHorseHorseBreed": {
        "133041005": ("North Swedish Horse horse breed", [7480])
    },
    "NorthSwedishTrotterHorseBreed": {
        "132963006": ("North Swedish Trotter horse breed", [7480])
    },
    "NortheasternHorseBreed": {"133042003": ("Northeastern horse breed", [7480])},
    "NorthernArdennaisHorseBreed": {
        "133067000": ("Northern Ardennais horse breed", [7480])
    },
    "NorthernBlueCattleBreed": {"133326004": ("Northern Blue cattle breed", [7480])},
    "NorwegianDunkerhoundDogBreed": {
        "132593008": ("Norwegian Dunkerhound dog breed", [7480])
    },
    "NorwegianElkhound": {"76994004": ("Norwegian elkhound", [7480])},
    "NorwegianForestCatBreed": {"132672001": ("Norwegian Forest cat breed", [7480])},
    "NorwegianFurSheepBreed": {"131836008": ("Norwegian Fur sheep breed", [7480])},
    "NorwegianGoatBreed": {"131631000": ("Norwegian goat breed", [7480])},
    "NorwegianLandracePig": {"42948007": ("Norwegian landrace pig (organism)", [])},
    "NorwegianLandracePigBreed": {"42948007": ("Norwegian landrace pig breed", [7480])},
    "NorwegianLundehundDogBreed": {
        "132629007": ("Norwegian Lundehund dog breed", [7480])
    },
    "NorwegianRedCattleBreed": {"133254009": ("Norwegian Red cattle breed", [7480])},
    "NorwichTerrier": {"26332008": ("Norwich terrier", [7480])},
    "NorwoodTypeOperation": {"233139006": ("Norwood type operation", [12247])},
    "Nose": {"45206002": ("Nose", [4, 7151, 7192, 8134, 9514])},
    "NotAdministered": {"371900001": ("Not Administered", [3742, 3743])},
    "NotEvaluated": {"373121007": ("Not Evaluated", [3703])},
    "NotFasting": {"440565004": ("Not Fasting", [12323])},
    "NotPerformed": {"262008008": ("Not performed", [3208, 3209])},
    "NotPregnant": {"60001007": ("not pregnant", [6096])},
    "NotSignificant": {"371928007": ("Not significant", [220])},
    "NotSteady": {"103361006": ("Not Steady", [4220])},
    "NottinghamCombinedGrade": {"372276001": ("Nottingham Combined Grade", [6071])},
    "NottinghamCombinedGradeCannotBeDetermined": {
        "384668003": ("Nottingham Combined Grade cannot be determined", [6069])
    },
    "NottinghamCombinedGradeI3To5Points": {
        "369790002": ("Nottingham Combined Grade I: 3-5 points", [6069])
    },
    "NottinghamCombinedGradeII6To7Points": {
        "369791003": ("Nottingham Combined Grade II: 6-7 points", [6069])
    },
    "NottinghamCombinedGradeIII8To9Points": {
        "369792005": ("Nottingham Combined Grade III: 8-9 points", [6069])
    },
    "NovaScotiaDuckTollingRetriever": {
        "26229008": ("Nova Scotia duck tolling retriever (organism)", [])
    },
    "NovaScotiaDuckTollingRetrieverDogBreed": {
        "26229008": ("Nova Scotia duck tolling retriever dog breed", [7480])
    },
    "NubaMountainCattleBreed": {
        "133819006": ("Nuba Mountain cattle breed (organism)", [])
    },
    "NubaMountainXZebuCattleBreed": {
        "133819006": ("Nuba Mountain X zebu cattle breed", [7480])
    },
    "NuclearFastRedStain": {"78869007": ("nuclear fast red stain", [8112])},
    "NuclearMedicine": {"309938009": ("Nuclear Medicine", [7030])},
    "NuclearMedicineCardiovascularStudy": {
        "108294005": ("Nuclear medicine cardiovascular study", [3108, 3206, 3757])
    },
    "NuclearMedicineDepartment": {
        "309938009": ("Nuclear medicine department (environment)", [])
    },
    "NuclearMedicineDiagnosticProcedureOnCardiovascularSystem": {
        "108294005": (
            "Nuclear medicine diagnostic procedure on cardiovascular system (procedure)",
            [],
        )
    },
    "NuclearMedicineDiagnosticProcedureOnNervousSystem": {
        "108300008": (
            "Nuclear medicine diagnostic procedure on nervous system (procedure)",
            [],
        )
    },
    "NuclearMedicineProcedure": {"371572003": ("Nuclear medicine procedure", [6028])},
    "Nucleus": {"84640000": ("Nucleus", [8135])},
    "NucleusAccumbens": {
        "427667007": ("Nucleus Accumbens", [7140, 7151, 7153, 7192, 8134, 9514])
    },
    "NuerCattleBreed": {"131510007": ("Nuer cattle breed", [7480])},
    "Nulliparous": {"102877006": ("Nulliparous", [6081, 6087])},
    "NumberOfLesions": {"246206008": ("Number of lesions (observable entity)", [])},
    "NumberOfOccurrences": {
        "246432004": ("Number of occurrences (qualifier value)", [])
    },
    "NumberOfUmbilicalArteries": {
        "249192005": ("Number of umbilical arteries", [12279])
    },
    "NurasCattleBreed": {"131511006": ("Nuras cattle breed", [7480])},
    "Nurse": {"106292003": ("Nurse", [7450, 7452])},
    "Nutrition": {"364393001": ("Nutrition", [3774])},
    "NutritionalObservable": {
        "364393001": ("Nutritional observable (observable entity)", [])
    },
    "NyoroCattleBreed": {"131512004": ("Nyoro cattle breed", [7480])},
    "OEAxillaryLymphadenopathy": {
        "164150006": ("O/E - axillary lymphadenopathy", [6055])
    },
    "OEBreastLumpPalpated": {"268951004": ("O/E - Breast lump palpated", [6055])},
    "OELymphadenopathy": {"274303007": ("O/E - lymphadenopathy", [6055])},
    "OICPig": {"9135003": ("OIC pig (organism)", [])},
    "OICPigBreed": {"9135003": ("OIC pig breed", [7480])},
    "OberhasliGoatBreed": {"131632007": ("Oberhasli goat breed", [7480])},
    "Oblique": {"21114003": ("Oblique", [6, 501, 502, 6121, 6312])},
    "ObliqueAxial": {"399004004": ("oblique axial", [501, 4010])},
    "ObliqueAxialEmissiveProjection": {"399089007": ("Oblique axial", [26, 501])},
    "ObliqueAxialProjection": {
        "399004004": ("Oblique axial projection (qualifier value)", [])
    },
    "ObliqueBodyPosition": {"399366008": ("oblique", [21])},
    "ObliqueCaudoCranial": {"399225005": ("oblique caudo-cranial", [501, 4010])},
    "ObliqueCaudoCranialProjection": {
        "399225005": ("Oblique caudo-cranial projection (qualifier value)", [])
    },
    "ObliqueCranioCaudal": {"399288005": ("oblique cranio-caudal", [501, 4010])},
    "ObliqueCranioCaudalProjection": {
        "399288005": ("Oblique cranio-caudal projection (qualifier value)", [])
    },
    "ObliqueLateral": {"260427002": ("Oblique lateral (qualifier value)", [])},
    "ObliqueProjection": {"399182000": ("oblique", [501, 4010])},
    "ObscuredLesion": {
        "129740002": ("Obscured lesion", [6006, 6007, 6335, 6341, 6342])
    },
    "ObstetricHistory": {"248983002": ("Obstetric history (observable entity)", [])},
    "ObstetricUSScan": {"268445003": ("Obstetric US scan", [12024])},
    "ObstetricalVersion": {"65240009": ("Obstetrical version (procedure)", [])},
    "Obstetrics": {"309944008": ("Obstetrics", [7030])},
    "ObstetricsAndGynecology": {"309942007": ("Obstetrics and Gynecology", [7030])},
    "ObstetricsAndGynecologyDepartment": {
        "309942007": ("Obstetrics and gynecology department (environment)", [])
    },
    "ObstetricsDepartment": {"309944008": ("Obstetrics department (environment)", [])},
    "Obstruction": {"26036001": ("Obstruction (morphologic abnormality)", [])},
    "ObturatorLymphNode": {"36086000": ("obturator lymph node", [7600, 8134])},
    "OccipitalArtery": {
        "31145008": (
            "Occipital artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "OccipitalBone": {"31640002": ("Occipital bone", [4028, 7151, 7192, 8134, 9514])},
    "OccipitalBoneStructure": {
        "31640002": ("Occipital bone structure (body structure)", [])
    },
    "OccipitalLobe": {"31065004": ("Occipital lobe", [7151, 7153, 7192, 8134, 9514])},
    "OccipitalLobeStructure": {
        "31065004": ("Occipital lobe structure (body structure)", [])
    },
    "OccipitalLymphNode": {"3916005": ("occipital lymph node", [7600, 8134])},
    "OccipitalRegionOfScalp": {
        "700032006": ("Occipital region of scalp", [8134, 12022])
    },
    "OccipitalVein": {
        "32114007": ("Occipital vein", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "OcclusalProjection": {"260499007": ("Occlusal projection", [501, 4010])},
    "Occlusion": {"26036001": ("occlusion", [3810])},
    "OcclusionOfArtery": {"1386000": ("Occlusion of artery", [3413, 3754])},
    "OcclusionOfCatheter": {"103714007": ("Occlusion of catheter", [9])},
    "OcclusionOfPatentDuctusArteriosusUsingEmbolizationCoil": {
        "441676000": (
            "Occlusion of patent ductus arteriosus using embolization coil (procedure)",
            [],
        )
    },
    "OccupationalRequirement": {"429060002": ("Occupational requirement", [3201])},
    "Ocicat": {"63972001": ("Ocicat", [7480])},
    "OcularLubricant": {"470091001": ("Ocular Lubricant", [621, 622])},
    "OculomotorNerve": {"56193007": ("oculomotor nerve", [7706, 7710, 8134])},
    "OculomotorNerveStructure": {
        "56193007": ("Oculomotor nerve structure (body structure)", [])
    },
    "OffAxis": {"103341000": ("Off axis", [6, 501, 502])},
    "OhminiPigBreed": {"133180000": ("Ohmini pig breed", [7480])},
    "OilImmersionLens": {"445622008": ("Oil immersion lens", [8121])},
    "OilImmersionLensOfOpticalMicroscope": {
        "445622008": ("Oil immersion lens of optical microscope (physical object)", [])
    },
    "OilInVitreousCavity": {"247095003": ("Oil in vitreous cavity (finding)", [])},
    "OilRedOStain": {"40808006": ("oil red O stain", [8112])},
    "OldDanishBirdDogBreed": {"132594002": ("Old Danish Bird Dog breed", [7480])},
    "OldEnglishSheepdog": {"87029004": ("Old English sheepdog", [7480])},
    "OldFarmCollieDogBreed": {"132461003": ("Old Farm Collie dog breed", [7480])},
    "OldFormatDachsundDogBreed": {
        "132595001": ("Old Format Dachsund dog breed", [7480])
    },
    "OldFormatManchesterTerrierDogBreed": {
        "132596000": ("Old Format Manchester Terrier dog breed", [7480])
    },
    "OldFormatMinToyPoodleDogBreed": {
        "132597009": ("Old Format Min/Toy Poodle dog breed", [7480])
    },
    "OldFormatSheepBreed": {"131835007": ("Old Format Sheep breed", [7480])},
    "OldFormatWelshCorgiDogBreed": {
        "132598004": ("Old Format Welsh Corgi dog breed", [7480])
    },
    "OldGermanShepherdDogBreed": {
        "132462005": ("Old German Shepherd dog breed", [7480])
    },
    "OldNorwegianSheepBreed": {"131834006": ("Old Norwegian sheep breed", [7480])},
    "OldSwedishSpottedPigBreed": {
        "132192009": ("Old Swedish Spotted pig breed", [7480])
    },
    "OldeEnglishBulldoggeDogBreed": {
        "132606004": ("Olde English Bulldogge dog breed", [7480])
    },
    "OldenburgHorseBreed": {"133011008": ("Oldenburg horse breed", [7480])},
    "OleicAcidI125": {"22979004": ("Oleic acid I^125^", [25])},
    "OlfactoryTract": {"3960005": ("olfactory tract", [7706, 7710, 8134])},
    "OlfactoryTractStructure": {
        "3960005": ("Olfactory tract structure (body structure)", [])
    },
    "OliventinaPigBreed": {"132193004": ("Oliventina pig breed", [7480])},
    "OmentalBursa": {"113346000": ("Omental bursa", [4, 7151, 7192, 8134, 9514])},
    "OmentalBursaStructure": {
        "113346000": ("Omental bursa structure (body structure)", [])
    },
    "Omentum": {"27398004": ("Omentum", [4, 7151, 7154, 7192, 8134, 9514])},
    "OmentumStructure": {"27398004": ("Omentum structure (body structure)", [])},
    "OnAdmission": {"278307001": ("On admission", [31])},
    "OnExaminationAxillaryLymphadenopathy": {
        "164150006": ("On examination - axillary lymphadenopathy (disorder)", [])
    },
    "OnExaminationBreastLumpPalpated": {
        "268951004": ("On examination - breast lump palpated (finding)", [])
    },
    "OnExaminationLens": {"309649001": ("On examination - lens (finding)", [])},
    "OnExaminationLymphNodes": {
        "274303007": ("On examination - lymph nodes (finding)", [])
    },
    "OnExaminationPoorVisualFixation": {
        "314348007": ("On examination - poor visual fixation (finding)", [])
    },
    "OneStandardDeviationAboveMean": {
        "371917008": ("One standard deviation above mean (qualifier value)", [])
    },
    "OneStandardDeviationBelowMean": {
        "371919006": ("One standard deviation below mean (qualifier value)", [])
    },
    "OpaqueMarker": {"262301009": ("Opaque Marker", [6040, 6401, 7151, 7193])},
    "OpenWoundOfLowerLimb": {"26947005": ("Open wound of lower limb", [3205])},
    "OperatingMicroscope": {"102321001": ("Operating Microscope", [4202])},
    "OperatingMicroscopeDevice": {
        "102321001": ("Operating microscope, device (physical object)", [])
    },
    "OperationOnHeartValve": {"73544002": ("Operation on heart valve", [3721])},
    "OperationOnPelvicRegionOfTrunk": {
        "609638001": ("Operation on pelvic region of trunk", [6320])
    },
    "OperativeProcedureOnPelvis": {
        "12745006": ("Operative procedure on pelvis (procedure)", [])
    },
    "OperativeSite": {"43526002": ("Operative Site", [6201])},
    "OphthalmicArtery": {
        "53549008": (
            "Ophthalmic artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                4209,
                7151,
                7192,
                8134,
                9514,
                12103,
                12105,
            ],
        )
    },
    "OphthalmicEndoscope": {"409902001": ("Ophthalmic Endoscope", [4202])},
    "OphthalmicPhenylephrine": {
        "386693003": ("Ophthalmic phenylephrine (product)", [])
    },
    "Ophthalmology": {"309935007": ("Ophthalmology", [7030])},
    "OpiateAntagonist": {"372656001": ("Opiate antagonist (substance)", [])},
    "OpticCanal": {
        "55024004": (
            "Optic canal",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "OpticCanalStructure": {"55024004": ("Optic canal structure (body structure)", [])},
    "OpticChiasm": {"244453006": ("Optic chiasm", [7151, 7153, 7192, 8134, 9514])},
    "OpticChiasma": {"244453006": ("Optic chiasma (body structure)", [])},
    "OpticDiscStructure": {"81016008": ("Optic disc structure (body structure)", [])},
    "OpticForamenProjection": {
        "272466003": ("Optic foramen projection (qualifier value)", [])
    },
    "OpticNerve": {"18234004": ("optic nerve", [7706, 7710, 8134])},
    "OpticNerveHead": {"81016008": ("Optic nerve head", [4209, 4211, 4266, 8134])},
    "OpticNerveStructure": {"18234004": ("Optic nerve structure (body structure)", [])},
    "OpticRadiation": {
        "70105001": ("Optic radiation", [7151, 7153, 7192, 7702, 7710, 8134, 9514])
    },
    "OpticTract": {"53238003": ("Optic tract", [7151, 7153, 7192, 8134, 9514])},
    "OpticTractStructure": {"53238003": ("Optic tract structure (body structure)", [])},
    "OpticalCoherenceTomographyScanner": {
        "392012008": ("Optical Coherence Tomography Scanner", [4210])
    },
    "Optometry": {"310105000": ("Optometry", [7030])},
    "OptometryService": {"310105000": ("Optometry service (qualifier value)", [])},
    "OralCavity": {"74262004": ("oral cavity", [7601, 8134])},
    "OralCavityStructure": {"74262004": ("Oral cavity structure (body structure)", [])},
    "OralMucosa": {"113277000": ("Oral mucosa", [4029, 8134, 10044, 10060])},
    "OralMucousMembraneStructure": {
        "113277000": ("Oral mucous membrane structure (body structure)", [])
    },
    "OralRoute": {"26643006": ("Oral route", [11])},
    "OralSurgery": {"309974000": ("Oral Surgery", [7030])},
    "OralSurgeryDepartment": {
        "309974000": ("Oral surgery department (environment)", [])
    },
    "Orange": {"371242008": ("Orange", [4411])},
    "OrangeColor": {"371242008": ("Orange color (qualifier value)", [])},
    "OrangeGStain": {"54221006": ("orange G stain", [8112])},
    "OrangeIIStain": {"25941002": ("orange II stain", [8112])},
    "Orbit": {"363654007": ("Orbit", [12022])},
    "OrbitalStructure": {
        "363654007": (
            "Orbital structure",
            [
                4,
                1000,
                1001,
                1002,
                4009,
                4028,
                4030,
                4031,
                4042,
                7151,
                7192,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "OrbitoParietalProjection": {
        "399351005": ("Orbito-parietal projection (qualifier value)", [])
    },
    "Orbitoparietal": {"399351005": ("orbitoparietal", [501, 4010])},
    "OrceinStain": {"406966004": ("orcein stain", [8112])},
    "OregonRexCatBreed": {"396505009": ("Oregon rex cat breed", [7480])},
    "OrehlaDeColherPigBreed": {"132166002": ("Orehla de Colher pig breed", [7480])},
    "Organ": {"91772007": ("Organ", [7151, 7166, 7191])},
    "OrganParenchyma": {"91772007": ("Organ parenchyma (body structure)", [])},
    "OrientalHorseHorseBreed": {"132964000": ("Oriental Horse horse breed", [7480])},
    "OrientalLonghairCatBreed": {"132673006": ("Oriental Longhair cat breed", [7480])},
    "OrientalShorthairedCat": {"24967003": ("Oriental shorthaired cat", [7480])},
    "OriginOfVessel": {"397421006": ("Origin of vessel", [12116])},
    "OrkneySheepBreed": {"131833000": ("Orkney sheep breed", [7480])},
    "OrlovTrotterHorseBreed": {"133066009": ("Orlov Trotter horse breed", [7480])},
    "OrobicaGoatBreed": {"131613001": ("Orobica goat breed", [7480])},
    "OropaCattleBreed": {"133536006": ("Oropa cattle breed", [7480])},
    "OropharyngealStructure": {
        "31389004": ("Oropharyngeal structure (body structure)", [])
    },
    "OropharyngealTonsil": {"17861009": ("oropharyngeal tonsil", [7601, 8134])},
    "Oropharynx": {"31389004": ("oropharynx", [7601, 8134])},
    "OrthopedicDepartment": {"309989009": ("Orthopedic department (environment)", [])},
    "OrthopedicDevice": {"16349000": ("Orthopedic device", [1000])},
    "OrthopedicSurgery": {"309989009": ("Orthopedic Surgery", [7030])},
    "OrthostaticBodyPosition": {
        "10904000": ("Orthostatic body position (finding)", [])
    },
    "OryctolagusCuniculus": {"36571002": ("Oryctolagus cuniculus", [7454])},
    "OsmiumTetroxide": {"13931001": ("Osmium tetroxide", [8114])},
    "Osmolality": {"56953008": ("Osmolality (observable entity)", [])},
    "OssabawIslandPigBreed": {"132237008": ("Ossabaw Island pig breed", [7480])},
    "Ossification": {"83323007": ("Ossification", [6132])},
    "OssimiSheepBreed": {"132858008": ("Ossimi sheep breed", [7480])},
    "Osteosarcoma": {"1163405004": ("Osteosarcoma", [638, 639, 6030, 6033])},
    "OsteosarcomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "21708004": (
            "Osteosarcoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "Ostium": {"264114003": ("Ostium", [3019, 3837, 7111])},
    "OstiumOfCoronarySinus": {"71271007": ("Ostium of coronary sinus", [3011, 8134])},
    "Otorhinolaryngology": {"309978002": ("Otorhinolaryngology", [7030])},
    "OtterHound": {"58116005": ("Otter hound", [7480])},
    "Ottonello": {"399171008": ("Ottonello", [4012])},
    "OttonelloProjection": {
        "399171008": ("Ottonello projection (qualifier value)", [])
    },
    "OuessantSheepBreed": {"132688001": ("Ouessant sheep breed", [7480])},
    "OulmesBlondCattleBreed": {"133537002": ("Oulmes Blond cattle breed", [7480])},
    "OutaouaisArcottSheepBreed": {
        "132857003": ("Outaouais Arcott sheep breed", [7480])
    },
    "OutpatientProcedure": {"371883000": ("Outpatient procedure", [3671])},
    "Outward": {"255543005": ("Outward", [4214])},
    "OvalShapedLesion": {"129734009": ("Oval shaped lesion (finding)", [])},
    "OvamboCattleBreed": {"131513009": ("Ovambo cattle breed", [7480])},
    "OvarianArtery": {
        "12052000": (
            "Ovarian Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125, 12140],
        )
    },
    "OvarianCancer": {"363443007": ("Ovarian cancer", [638, 639])},
    "OvarianFollicleStructure": {
        "24162005": ("Ovarian follicle structure (body structure)", [])
    },
    "OvarianStructure": {"15497006": ("Ovarian structure (body structure)", [])},
    "OvarianVein": {
        "976004": (
            "Ovarian vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12113, 12125, 12140],
        )
    },
    "Ovary": {
        "15497006": (
            "Ovary",
            [4, 645, 6202, 6204, 7151, 7160, 7192, 8134, 9514, 10044, 10060],
        )
    },
    "OverbedTrapezeDevice": {
        "422915004": ("Overbed trapeze device (physical object)", [])
    },
    "OvineSpecies": {"36295001": ("Ovine species (organism)", [])},
    "Ovis": {"388254009": ("Ovis", [7454])},
    "OvisAries": {"125099002": ("Ovis aries", [7454])},
    "OvoidShape": {"84360004": ("Ovoid shape", [6335, 6339, 6340])},
    "OvoidShapeOval": {"84360004": ("Ovoid shape (Oval)", [6004, 6005])},
    "OwczarekPodhalandskiDogBreed": {
        "132591005": ("Owczarek Podhalandski dog breed", [7480])
    },
    "OxfordDownSheep": {"53360003": ("Oxford Down sheep (organism)", [])},
    "OxfordDownSheepBreed": {"53360003": ("Oxford Down sheep breed", [7480])},
    "OxfordSandyBlockPig": {"5227002": ("Oxford sandy block pig (organism)", [])},
    "OxfordSandyBlockPigBreed": {"5227002": ("Oxford sandy block pig breed", [7480])},
    "Oxygen": {"24099007": ("Oxygen", [12, 13, 3850])},
    "OxygenAdministrationByMask": {
        "371908008": ("Oxygen Administration by mask", [3531])
    },
    "OxygenAdministrationByNasalCannula": {
        "371907003": ("Oxygen Administration by nasal cannula", [3531])
    },
    "OxygenGas": {"320917000": ("Oxygen gas", [623, 629])},
    "OxygenO14": {"424875009": ("Oxygen O^14^ (substance)", [])},
    "OxygenO15": {"129504001": ("Oxygen O^15^", [4021])},
    "OxygenTherapy": {"57485005": ("Oxygen Therapy", [617, 618])},
    "OxygenWaterO15": {"129505000": ("Oxygen-water O^15^", [4021])},
    "Oxytocic": {"410937004": ("Oxytocic", [621, 622])},
    "OxytocicAgent": {"410937004": ("Oxytocic agent (substance)", [])},
    "PETBrainStudy": {"764666002": ("PET brain study", [3108])},
    "PETBreastStudy": {"416323006": ("PET breast study", [3108])},
    "PETCTFDGImagingOfWholeBody": {
        "443271005": ("PET/CT FDG imaging of whole body", [100])
    },
    "PETCTMETImagingOfWholeBody": {
        "764704008": ("PET/CT MET imaging of whole body", [100])
    },
    "PETHeartStudy": {"241439007": ("PET heart study", [3108, 3206, 3757])},
    "PETStudyForLocalizationOfTumor": {
        "241443006": ("PET study for localization of tumor", [3108])
    },
    "PR1050PigBreed": {"132212003": ("PR 1050 pig breed", [7480])},
    "PR1075PigBreed": {"132213008": ("PR 1075 pig breed", [7480])},
    "PRK": {"397516006": ("PRK", [4234])},
    "PTENHamartomaTumorSyndrome": {
        "722859001": ("PTEN hamartoma tumor syndrome", [4405])
    },
    "PTSMCu62": {"422789008": ("PTSM Cu^62^", [4021])},
    "PTV": {"228793007": ("PTV", [9534])},
    "PabnaCattleBreed": {"133820000": ("Pabna cattle breed (organism)", [])},
    "PabnaXZebuCattleBreed": {"133820000": ("Pabna X zebu cattle breed", [7480])},
    "PacedStressTest": {"428685003": ("Paced stress test", [3200])},
    "PacemakerPulseGenerator": {
        "118378005": ("Cardiac Pacemaker", [6040, 6401, 7151, 7193])
    },
    "Pacing": {"18590009": ("Pacing", [91, 3271, 3555])},
    "PacingWithMagnet": {"371909000": ("pacing with magnet", [3555])},
    "PageBlue83Stain": {"5442001": ("page blue 83 stain", [8112])},
    "PageBlueG90Stain": {"2088005": ("page blue G-90 stain", [8112])},
    "PagetDiseaseMammary": {
        "2985005": ("Paget's disease, mammary (of the nipple)", [6030, 6033])
    },
    "PagliarolaSheepBreed": {"131830002": ("Pagliarola sheep breed", [7480])},
    "PainManagement": {"309949003": ("Pain Management", [7030])},
    "PainManagementDepartment": {
        "309949003": ("Pain management department (environment)", [])
    },
    "PainOfBreast": {"53430007": ("Pain of breast (finding)", [])},
    "PainScore": {"225908003": ("Pain score (observable entity)", [])},
    "PainfulSkin": {"403598008": ("Painful skin", [4406])},
    "PajunaCattleBreed": {"133538007": ("Pajuna cattle breed", [7480])},
    "PalatineBone": {"51283005": ("Palatine bone", [4028, 7151, 7192, 8134, 9514])},
    "PalatineBoneStructure": {
        "51283005": ("Palatine bone structure (body structure)", [])
    },
    "PalatineTonsil": {"75573002": ("palatine tonsil", [7601, 8134])},
    "PaleComplexion": {"398979000": ("Pale complexion (finding)", [])},
    "PalliativeCare": {"309939001": ("Palliative Care", [7030])},
    "PalliativeCareDepartment": {
        "309939001": ("Palliative care department (environment)", [])
    },
    "PalliativeIntent": {"363676003": ("Palliative Intent", [3629])},
    "PalliativeProcedureIntent": {
        "363676003": ("Palliative - procedure intent (qualifier value)", [])
    },
    "PallorPaleComplexion": {"398979000": ("Pallor (Pale Complexion)", [60, 9300])},
    "Palmar45DegreeMedialDorsolateral": {
        "442740009": ("Palmar 45 degree medial-dorsolateral", [7484])
    },
    "Palmar45DegreeMedialDorsolateralProjection": {
        "442740009": (
            "Palmar 45 degree medial-dorsolateral projection (qualifier value)",
            [],
        )
    },
    "Palmar75DegreeProximalDorsodistalOblique": {
        "442591005": ("Palmar 75 degree proximal-dorsodistal oblique", [7484])
    },
    "Palmar75DegreeProximalDorsodistalObliqueProjection": {
        "442591005": (
            "Palmar 75 degree proximal-dorsodistal oblique projection (qualifier value)",
            [],
        )
    },
    "PalmaromedialDorsolateral": {"442742001": ("Palmaromedial-dorsolateral", [7484])},
    "PalmaromedialDorsolateralProjection": {
        "442742001": ("Palmaromedial-dorsolateral projection (qualifier value)", [])
    },
    "PalmaroproximalDorsodistalOblique": {
        "442674000": ("Palmaroproximal-dorsodistal oblique", [7484])
    },
    "PalmaroproximalDorsodistalObliqueProjection": {
        "442674000": (
            "Palmaroproximal-dorsodistal oblique projection (qualifier value)",
            [],
        )
    },
    "PalmeraCattleBreed": {"133539004": ("Palmera cattle breed", [7480])},
    "PalmitateC11": {"129514005": ("Palmitate C^11^", [4021])},
    "PalominoHorse": {"41738000": ("Palomino horse (organism)", [])},
    "PalominoHorseBreed": {"41738000": ("Palomino horse breed", [7480])},
    "PalousePig": {"49240006": ("Palouse pig (organism)", [])},
    "PalousePigBreed": {"49240006": ("Palouse pig breed", [7480])},
    "Palpation": {"113011001": ("Palpation", [3442])},
    "Palpitations": {"80313002": ("Palpitations", [3201, 12246])},
    "PanamaSheep": {"13934009": ("Panama sheep (organism)", [])},
    "PanamaSheepBreed": {"13934009": ("Panama sheep breed", [7480])},
    "Pancreas": {
        "15776009": (
            "Pancreas",
            [
                4,
                645,
                1000,
                1001,
                1005,
                4009,
                4030,
                4031,
                4042,
                7151,
                7154,
                7192,
                8134,
                9514,
                10044,
                10060,
                12321,
            ],
        )
    },
    "PancreaticDuct": {
        "69930009": (
            "Pancreatic duct",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7154, 7192, 8134, 9514],
        )
    },
    "PancreaticDuctAndBileDuctSystems": {
        "110621006": (
            "Pancreatic duct and bile duct systems",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7154, 7192, 8134, 9514],
        )
    },
    "PancreaticDuctAndBileDuctSystemsCombinedSite": {
        "110621006": (
            "Pancreatic duct and bile duct systems (combined site) (body structure)",
            [],
        )
    },
    "PancreaticDuctStructure": {
        "69930009": ("Pancreatic duct structure (body structure)", [])
    },
    "PancreaticLymphNode": {"77778009": ("pancreatic lymph node", [7600, 8134])},
    "PancreaticStructure": {"15776009": ("Pancreatic structure (body structure)", [])},
    "PancreaticoduodenalLymphNode": {
        "76659008": ("pancreaticoduodenal lymph node", [7600, 8134])
    },
    "PancreaticosplenicLymphNode": {
        "16050005": ("pancreaticosplenic lymph node", [7600, 8134])
    },
    "Pancuronium": {"373738000": ("Pancuronium", [623, 627])},
    "PanjeHorseBreed": {"132971005": ("Panje horse breed", [7480])},
    "PankotaRedCattleBreed": {"133540002": ("Pankota Red cattle breed", [7480])},
    "PantaneiroCattleBreed": {"133673003": ("Pantaneiro cattle breed", [7480])},
    "PantaneiroHorseBreed": {"133065008": ("Pantaneiro horse breed", [7480])},
    "PantelleriaCattleBreed": {"131514003": ("Pantelleria cattle breed", [7480])},
    "Papaverine": {"372784001": ("Papaverine", [10, 65])},
    "PaphosCattleBreed": {"133541003": ("Paphos cattle breed", [7480])},
    "PapillaryCarcinoma": {
        "25910003": ("Papillary carcinoma (invasive)", [6030, 6033])
    },
    "PapillaryCarcinomaInSitu": {
        "10376009": ("Papillary carcinoma in-situ", [6030, 6033])
    },
    "PapillaryThyroidCarcinoma": {
        "255029007": ("Papillary thyroid carcinoma", [638, 639])
    },
    "Papilloma": {"711329002": ("Papilloma", [6030, 6031])},
    "PapillomaNoInternationalClassificationOfDiseasesForOncologySubtypeExceptPapillomaOfBladderM81201": {
        "23730008": (
            "Papilloma, no International Classification of Diseases for Oncology subtype "
            "(except papilloma of bladder M-81201) (morphologic abnormality)",
            [],
        )
    },
    "PapillonDog": {"41263004": ("Papillon dog", [7480])},
    "ParaAorticLymphNodeOfTheAnteriorMediastinum": {
        "127939003": (
            "para-aortic lymph node of the anterior mediastinum",
            [7600, 8134],
        )
    },
    "ParaesophagealLymphNodeBelowCarina": {
        "127940001": ("paraesophageal lymph node below carina", [7600, 8134])
    },
    "Paraffin": {"255667006": ("Paraffin (substance)", [])},
    "ParaffinWax": {"311731000": ("Paraffin wax", [8115])},
    "Paraformaldehyde": {"52836003": ("Paraformaldehyde", [8114])},
    "ParalyticSyndrome": {"29426003": ("Paralytic syndrome", [3205])},
    "ParamammaryLymphNode": {"368550005": ("paramammary lymph node", [7600, 8134])},
    "ParametrialLymphNode": {"3243006": ("parametrial lymph node", [7600, 8134])},
    "ParanasalSinus": {
        "2095001": (
            "Paranasal sinus",
            [4, 4009, 4028, 4030, 4031, 4040, 4042, 7151, 7192, 7601, 9514],
        )
    },
    "Paraproteinemia": {"35601003": ("Paraproteinemia", [64])},
    "PararectalLymphNode": {"21875007": ("pararectal lymph node", [7600, 8134])},
    "Parasagittal": {"103343002": ("Parasagittal", [5, 6, 501, 502])},
    "Parasternal": {"91691001": ("Parasternal", [4, 7151, 7192, 8134, 9514])},
    "ParasternalLongAxis": {"399139001": ("Parasternal long axis", [501, 12226])},
    "ParasternalLongAxisView": {
        "399139001": ("Parasternal long axis view (qualifier value)", [])
    },
    "ParasternalLongAxisViewOfRightVentricularInflowTract": {
        "443082005": (
            "Parasternal long axis view of right ventricular inflow tract (qualifier value)",
            [],
        )
    },
    "ParasternalLongAxisViewOfRightVentricularOutflowTract": {
        "443083000": (
            "Parasternal long axis view of right ventricular outflow tract (qualifier value)",
            [],
        )
    },
    "ParasternalLongAxisViewOfTheRVInflowTract": {
        "443082005": ("Parasternal long axis view of the RV inflow tract", [501, 12226])
    },
    "ParasternalLongAxisViewOfTheRVOutflowTract": {
        "443083000": (
            "Parasternal long axis view of the RV outflow tract",
            [501, 12226],
        )
    },
    "ParasternalLymphNode": {"82365008": ("parasternal lymph node", [7600, 8134])},
    "ParasternalShortAxis": {"399306005": ("Parasternal short axis", [501, 12226])},
    "ParasternalShortAxisAtTheAorticValveLevel": {
        "399239005": ("Parasternal short axis at the aortic valve level", [501, 12226])
    },
    "ParasternalShortAxisAtTheLevelOfTheMitralChords": {
        "399371001": (
            "Parasternal short axis at the level of the mitral chords",
            [501, 12226],
        )
    },
    "ParasternalShortAxisAtTheMitralValveLevel": {
        "399036006": ("Parasternal short axis at the Mitral Valve level", [501, 12226])
    },
    "ParasternalShortAxisAtThePapillaryMuscleLevel": {
        "399271003": (
            "Parasternal short axis at the Papillary Muscle level",
            [501, 12226],
        )
    },
    "ParasternalShortAxisView": {
        "399306005": ("Parasternal short axis view (qualifier value)", [])
    },
    "ParasternalShortAxisViewAtTheAorticValveLevel": {
        "399239005": (
            "Parasternal short axis view at the aortic valve level (qualifier value)",
            [],
        )
    },
    "ParasternalShortAxisViewAtTheLevelOfTheMitralChords": {
        "399371001": (
            "Parasternal short axis view at the level of the mitral chords (qualifier value)",
            [],
        )
    },
    "ParasternalShortAxisViewAtTheMitralValveLevel": {
        "399036006": (
            "Parasternal short axis view at the mitral valve level (qualifier value)",
            [],
        )
    },
    "ParasternalShortAxisViewAtThePapillaryMuscleLevel": {
        "399271003": (
            "Parasternal short axis view at the papillary muscle level (qualifier value)",
            [],
        )
    },
    "Parathyroid": {"111002": ("Parathyroid", [4, 4030, 7151, 7192, 8134, 9514])},
    "ParathyroidStructure": {"111002": ("Parathyroid structure (body structure)", [])},
    "ParatrachealLymphNode": {"65690001": ("paratracheal lymph node", [7600, 8134])},
    "ParavaginalLymphNode": {"16228004": ("paravaginal lymph node", [7600, 8134])},
    "ParavesicularLymphNode": {"1439000": ("paravesicular lymph node", [7600, 8134])},
    "Parenchyma": {"91772007": ("Parenchyma", [5])},
    "ParenteralEnteralSolutionBag": {
        "464557001": ("Parenteral/enteral solution bag", [68])
    },
    "ParietalBone": {"24924006": ("Parietal bone", [4028, 7151, 7192, 8134, 9514])},
    "ParietalBoneStructure": {
        "24924006": ("Parietal bone structure (body structure)", [])
    },
    "ParietalLobe": {"16630005": ("Parietal lobe", [7151, 7153, 7192, 8134, 9514])},
    "ParietalLobeStructure": {
        "16630005": ("Parietal lobe structure (body structure)", [])
    },
    "ParietoOrbital": {"399316002": ("parieto-orbital", [501, 4010])},
    "ParietoOrbitalProjection": {
        "399316002": ("Parieto-orbital projection (qualifier value)", [])
    },
    "Parietoacanthial": {"399272005": ("parietoacanthial", [501, 4010])},
    "ParietoacanthialProjection": {
        "399272005": ("Parietoacanthial projection (qualifier value)", [])
    },
    "ParmensePigBreed": {"132194005": ("Parmense pig breed", [7480])},
    "ParotidGland": {
        "45289007": (
            "Parotid gland",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "ParotidGlandStructure": {
        "45289007": ("Parotid gland structure (body structure)", [])
    },
    "ParotidLymphNode": {"10209003": ("parotid lymph node", [7600, 8134])},
    "ParoxysmalVentricularTachycardia": {
        "66657009": ("Paroxysmal ventricular tachycardia (disorder)", [])
    },
    "ParthenaisCattleBreed": {"133255005": ("Parthenais cattle breed", [7480])},
    "PartiColorCockerSpaniel": {
        "58888001": ("Parti-color cocker spaniel (organism)", [])
    },
    "PartiColorCockerSpanielDogBreed": {
        "58888001": ("Parti-color cocker spaniel dog breed", [7480])
    },
    "Partial": {"255609007": ("Partial (qualifier value)", [])},
    "PartialAnomalousPulmonaryVenousConnection": {
        "68237008": ("Partial anomalous pulmonary venous connection", [12248, 12249])
    },
    "PartialAnomalousPulmonaryVenousConnectionOperation": {
        "174900004": (
            "Partial anomalous pulmonary venous connection operation",
            [12247],
        )
    },
    "PartiallyReversibleMyocardialPerfusionDefect": {
        "251056002": ("Partially Reversible myocardial perfusion defect", [3113])
    },
    "PasoFinoHorse": {"56086005": ("Paso Fino horse (organism)", [])},
    "PasoFinoHorseBreed": {"56086005": ("Paso Fino horse breed", [7480])},
    "PassiveMovement": {"21278004": ("Passive movement", [93])},
    "PastHistoryOfProcedure": {
        "416940007": ("Past history of procedure (situation)", [])
    },
    "PasternOfForefoot": {"31329001": ("Pastern of forefoot", [7482, 7483, 8134])},
    "PasternOfHindfoot": {"18525008": ("Pastern of hindfoot", [7482, 7483, 8134])},
    "PasternRegionOfHindfoot": {
        "18525008": ("Pastern region of hindfoot (body structure)", [])
    },
    "Patella": {
        "64234005": (
            "Patella",
            [
                4,
                1000,
                1001,
                1006,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "PatellarTendon": {"18033002": ("Patellar Tendon", [12321])},
    "PatentBlueVSodiumSaltStain": {
        "48540004": ("patent blue V sodium salt stain", [4412, 8112])
    },
    "PatentDuctusArteriosus": {
        "83330001": (
            "Patent ductus arteriosus",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514, 12293],
        )
    },
    "PatentDuctusArteriosusCoilOrDeviceClosure": {
        "441676000": ("Patent ductus arteriosus coil or device closure", [12247])
    },
    "PatentForamenOvale": {"204317008": ("Patent Foramen Ovale", [12284])},
    "Paternal": {"224944003": ("Paternal", [6097])},
    "PathologicCalcification": {
        "18115005": ("pathologic calcification", [3802, 6335, 6336, 6337, 6338])
    },
    "PathologicCalcificationCalcifiedStructure": {
        "18115005": (
            "Pathologic calcification, calcified structure (morphologic abnormality)",
            [],
        )
    },
    "Pathology": {"309950003": ("Pathology", [7030])},
    "PathologyDepartment": {"309950003": ("Pathology department (environment)", [])},
    "PatibarcinaHorseBreed": {"132972003": ("Patibarcina horse breed", [7480])},
    "PatientCurrentlyPregnant": {
        "77386006": ("Patient currently pregnant", [1200, 1201, 6096])
    },
    "PatientHasPacemaker": {"441509002": ("Patient has pacemaker", [3205, 12323])},
    "PatientImmunocompromised": {"370388006": ("Patient immunocompromised", [602])},
    "PatientMentalStateAssessment": {
        "363871006": ("Patient mental state assessment", [3441])
    },
    "PatientRefusedExerciseTest": {
        "408551003": ("Patient Refused exercise test", [3221])
    },
    "PatterdaleTerrierDogBreed": {
        "132404008": ("Patterdale Terrier dog breed", [7480])
    },
    "Pawlow": {"399181007": ("Pawlow", [4012])},
    "PawlowProjection": {"399181007": ("Pawlow projection (qualifier value)", [])},
    "PeacockGoatBreed": {"131633002": ("Peacock goat breed", [7480])},
    "PeakCardiacStressState": {
        "434161005": ("Peak cardiac stress state", [3207, 12002])
    },
    "PeakSystolic": {"255236000": ("Peak Systolic", [12233])},
    "PeakSystolicFunction": {
        "255236000": ("Peak systolic, function (observable entity)", [])
    },
    "PeakToPeak": {"371914001": ("Peak to peak", [3627])},
    "PeakToPeakNumericEstimationTechnique": {
        "371914001": (
            "Peak to peak - numeric estimation technique (qualifier value)",
            [],
        )
    },
    "Pearson": {"399365007": ("Pearson", [4012])},
    "PearsonProjection": {"399365007": ("Pearson projection (qualifier value)", [])},
    "PeauDOrangeSurfaceOfBreast": {
        "87386002": ("Peau d'orange surface of breast", [6055])
    },
    "PechoraCattleBreed": {"133399003": ("Pechora cattle breed", [7480])},
    "PechoraHorseBreed": {"132973008": ("Pechora horse breed", [7480])},
    "PectoralAxillaryLymphNode": {
        "69691007": ("pectoral axillary lymph node", [7600, 8134])
    },
    "PectoralGirdle": {"26444007": ("Pectoral girdle", [6115, 8134])},
    "PectoralisMajorMuscle": {
        "60005003": (
            "Pectoralis major muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "PectoralisMajorMuscleStructure": {
        "60005003": ("Pectoralis major muscle structure (body structure)", [])
    },
    "PectoralisMinorMuscle": {
        "18686000": (
            "Pectoralis minor muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "PectoralisMinorMuscleStructure": {
        "18686000": ("Pectoralis minor muscle structure (body structure)", [])
    },
    "PediatricEchocardiography": {"431852008": ("Pediatric echocardiography", [12001])},
    "PediatricIntensiveCare": {"309910001": ("Pediatric Intensive Care", [7030])},
    "PediatricIntensiveCareUnit": {
        "309910001": ("Pediatric intensive care unit (environment)", [])
    },
    "PediatricMedicine": {"420223003": ("Pediatric Medicine", [7030])},
    "PediatricMedicineDepartment": {
        "420223003": ("Pediatric medicine department (environment)", [])
    },
    "PediatricOncology": {"309948006": ("Pediatric Oncology", [7030])},
    "PediatricOncologyDepartment": {
        "309948006": ("Pediatric oncology department (environment)", [])
    },
    "PediatricSurgery": {"309991001": ("Pediatric Surgery", [7030])},
    "PediatricSurgicalDepartment": {
        "309991001": ("Pediatric surgical department (environment)", [])
    },
    "PedicleOfVertebra": {"78972004": ("Pedicle of vertebra", [6115, 8134])},
    "Pedunculated": {"25126001": ("Pedunculated", [6209])},
    "PeeWeeCattleBreed": {"133400005": ("Pee Wee cattle breed", [7480])},
    "Peeling": {"271767006": ("Peeling", [4406])},
    "PeelingOfSkin": {"271767006": ("Peeling of skin (finding)", [])},
    "PekingeseDog": {"67684001": ("Pekingese dog", [7480])},
    "PelibüeySheepBreed": {"132802000": ("Pelibüey sheep breed", [7480])},
    "PeloponnesusCattleBreed": {"133401009": ("Peloponnesus cattle breed", [7480])},
    "PelvicCavityFalse": {
        "816990003": ("Pelvic cavity, false", [7151, 7160, 7192, 8134, 9514])
    },
    "PelvicCavityFalseAndOrTrue": {
        "816989007": ("Pelvic cavity, false and/or true", [7160, 8134])
    },
    "PelvicCavityStructure": {
        "21844003": ("Pelvic cavity structure (body structure)", [])
    },
    "PelvicCavityTrue": {
        "816991004": ("Pelvic cavity, true", [7151, 7160, 7192, 8134, 9514])
    },
    "PelvicLymphNode": {"54268001": ("pelvic lymph node", [7600, 8134])},
    "PelvicLymphNodeStructure": {
        "54268001": ("Pelvic lymph node structure (body structure)", [])
    },
    "PelvicStructure": {"12921003": ("Pelvic structure (body structure)", [])},
    "PelvicVascularStructure": {
        "281496003": ("Pelvic vascular structure (body structure)", [])
    },
    "Pelvis": {
        "816092008": (
            "Pelvis",
            [4, 4009, 4030, 4031, 4042, 7151, 7160, 7192, 7482, 7483, 8134, 9514],
        )
    },
    "PelvisAndLowerExtremities": {
        "1231522001": (
            "Pelvis and lower extremities",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PelvisAndOrLowerExtremityStructure": {
        "416631005": ("Pelvis and/or lower extremity structure (body structure)", [])
    },
    "PelónPigBreed": {"132169009": ("Pelón pig breed", [7480])},
    "PembrokeWelshCorgi": {"46725009": ("Pembroke Welsh corgi (organism)", [])},
    "PembrokeWelshCorgiDogBreed": {
        "46725009": ("Pembroke Welsh corgi dog breed", [7480])
    },
    "PenArLan77PigBreed": {"132091000": ("Pen ar Lan 77 pig breed", [7480])},
    "PenbukPigBreed": {"132142008": ("Penbuk pig breed", [7480])},
    "PeneiaHorseBreed": {"132974002": ("Peneia horse breed", [7480])},
    "PenicillinAntibiotic": {"373270004": ("Penicillin antibiotic", [621, 622])},
    "PenicillinClassOfAntibiotic": {
        "373270004": ("Penicillin -class of antibiotic- (substance)", [])
    },
    "PenileStructure": {"18911002": ("Penile structure (body structure)", [])},
    "Penis": {"18911002": ("Penis", [4, 7151, 7192, 8134, 9514])},
    "Penner": {"399138009": ("Penner", [4012])},
    "PennerProjection": {"399138009": ("Penner projection (qualifier value)", [])},
    "PenshirePigBreed": {"132092007": ("Penshire pig breed", [7480])},
    "PentetateCalciumTrisodiumYb169": {
        "29348008": ("Pentetate calcium trisodium Yb^169^", [25])
    },
    "Pentobarbital": {"372703009": ("Pentobarbital", [623, 625])},
    "PepperProtocol": {"129100000": ("Pepper protocol", [3261, 12001])},
    "PerRectum": {"37161004": ("Per rectum", [11, 613, 614])},
    "PercheronHorse": {"1006005": ("Percheron horse (organism)", [])},
    "PercheronHorseBreed": {"1006005": ("Percheron horse breed", [7480])},
    "PercutaneousCoronaryIntervention": {
        "415070008": ("Percutaneous coronary intervention", [3721, 3764])
    },
    "PercutaneousInsertionOfIntravascularFilter": {
        "105373008": ("Percutaneous insertion of intravascular filter", [9, 3405])
    },
    "PercutaneousProstheticClosureOfAtrialSeptalDefect": {
        "30123000": ("Percutaneous prosthetic closure of atrial septal defect", [12247])
    },
    "PercutaneousRemovalOfEndovascularForeignBody": {
        "240946003": (
            "Percutaneous removal of endovascular foreign body (procedure)",
            [],
        )
    },
    "PercutaneousRemovalOfIntravascularForeignBody": {
        "240946003": ("Percutaneous removal of intravascular foreign body", [9, 3405])
    },
    "PercutaneousTransluminalAngioplastyBalloon": {
        "102319006": (
            "Percutaneous transluminal angioplasty balloon",
            [8, 3411, 3429, 4051, 7151, 7193],
        )
    },
    "PercutaneousTransluminalAngioplastyBalloonDevice": {
        "102319006": (
            "Percutaneous transluminal angioplasty balloon, device (physical object)",
            [],
        )
    },
    "PercutaneousTransluminalBalloonAngioplasty": {
        "68457009": ("Percutaneous transluminal balloon angioplasty", [9, 3405])
    },
    "PercutaneousTransluminalBalloonAngioplastyOfCoarctationOfAortaWithInsertionOfStent": {
        "443829004": (
            "Percutaneous transluminal balloon angioplasty of coarctation of aorta with insertion of stent "
            "(procedure)",
            [],
        )
    },
    "PerdigueiroPortugueseDogBreed": {
        "132443007": ("Perdigueiro Portuguese dog breed", [7480])
    },
    "PerdigueroDeBurgosDogBreed": {
        "132600005": ("Perdiguero de Burgos dog breed", [7480])
    },
    "PerdigueroNavarroDogBreed": {
        "132601009": ("Perdiguero Navarro dog breed", [7480])
    },
    "PerendaleSheep": {"41706005": ("Perendale sheep (organism)", [])},
    "PerendaleSheepBreed": {"41706005": ("Perendale sheep breed", [7480])},
    "PerforatingArteryOfKidney": {
        "15763003": (
            "Perforating Artery of Kidney",
            [3827, 7151, 7192, 8134, 9514, 12103, 12115],
        )
    },
    "Perfusion": {"371863001": ("Perfusion", [7151, 7161, 7195])},
    "PerfusionFinding": {"371863001": ("Perfusion finding (finding)", [])},
    "PerfusionImaging": {"35202002": ("Perfusion imaging (nuclear)", [3757])},
    "PerianganHorseBreed": {"132975001": ("Periangan horse breed", [7480])},
    "PericardialCavity": {
        "25489000": ("Pericardial cavity", [7151, 7152, 7192, 8134, 9514])
    },
    "PericardialCavityStructure": {
        "25489000": ("Pericardial cavity structure (body structure)", [])
    },
    "PericardialDisease": {"55855009": ("Pericardial disease", [3700])},
    "PericardialEffusion": {"373945007": ("Pericardial effusion", [12236])},
    "PericardialStructure": {
        "76848001": ("Pericardial structure (body structure)", [])
    },
    "PericardialTamponade": {"35304003": ("Pericardial tamponade", [3700, 3728])},
    "PericardiophrenicArtery": {
        "3924000": (
            "Pericardiophrenic Artery",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "Pericardium": {"76848001": ("Pericardium", [7151, 7152, 7192, 8134, 9514, 12294])},
    "PerigastricLymphNode": {"245344006": ("perigastric lymph node", [7600, 8134])},
    "Perimeter": {"131191004": ("Perimeter", [218, 6165, 7469, 7470])},
    "PerinealStructure": {"38864007": ("Perineal structure (body structure)", [])},
    "Perineum": {"38864007": ("Perineum", [4, 7151, 7192, 8134, 9514])},
    "PeriodOfCollection": {"260867005": ("Period of collection (qualifier value)", [])},
    "PeriodicAcidSchiffStain": {"255808002": ("periodic acid Schiff stain", [8112])},
    "PeripancreaticLymphNode": {
        "245346008": ("peripancreatic lymph node", [7600, 8134])
    },
    "Peripheral": {"14414005": ("Peripheral", [2, 5, 6124, 6125])},
    "PeripheralAngiography": {"271993009": ("peripheral angiography", [3820])},
    "PeripheralIntravenousCatheter": {
        "82449006": ("Peripheral intravenous catheter", [74])
    },
    "PeripheralIntravenousCatheterDevice": {
        "82449006": ("Peripheral intravenous catheter, device (physical object)", [])
    },
    "PeripheralNerve": {
        "84782009": ("Peripheral nerve", [7151, 7167, 7192, 7710, 8134, 9514])
    },
    "PeripheralNerveMyelinSheath": {
        "61005006": ("Peripheral nerve myelin sheath", [645, 8134])
    },
    "PeripheralNerveStructure": {
        "84782009": ("Peripheral nerve structure (body structure)", [])
    },
    "PeripheralNervousSystem": {
        "3058005": ("Peripheral nervous system", [7151, 7167, 7192, 8134, 9514])
    },
    "PeripheralNervousSystemStructure": {
        "3058005": ("Peripheral nervous system structure (body structure)", [])
    },
    "PeripheralVascularDisease": {
        "400047006": ("Peripheral vascular disease", [3205, 3756])
    },
    "PeripheralVascularUltrasoundReport": {
        "48526000": ("Peripheral Vascular Ultrasound Report", [7000, 12100])
    },
    "PeripheralZoneOfTheProstate": {
        "279706003": ("Peripheral zone of the prostate", [6300])
    },
    "Peritioneum": {"15425007": ("Peritioneum", [7151, 7154, 7192, 8134, 9514])},
    "PeritonealCavity": {
        "83670000": ("Peritoneal cavity", [7151, 7154, 7192, 8134, 9514])
    },
    "PeritonealCavityStructure": {
        "83670000": ("Peritoneal cavity structure (body structure)", [])
    },
    "PeritoneumSerousMembraneStructure": {
        "15425007": ("Peritoneum (serous membrane) structure (body structure)", [])
    },
    "PermanentMaxillaryLeftCanineTooth": {
        "860780009": ("Permanent maxillary left canine tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftCentralIncisorTooth": {
        "424399000": ("Permanent maxillary left central incisor tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftFirstMolarTooth": {
        "865988009": ("Permanent maxillary left first molar tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftFirstPremolarTooth": {
        "61897005": ("Permanent maxillary left first premolar tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftLateralIncisorTooth": {
        "423185002": ("Permanent maxillary left lateral incisor tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftSecondMolarTooth": {
        "863901004": ("Permanent maxillary left second molar tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftSecondPremolarTooth": {
        "23226009": ("Permanent maxillary left second premolar tooth", [4018, 4026])
    },
    "PermanentMaxillaryLeftThirdMolarTooth": {
        "87704003": ("Permanent maxillary left third molar tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightCanineTooth": {
        "860767006": ("Permanent maxillary right canine tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightCentralIncisorTooth": {
        "422653006": ("Permanent maxillary right central incisor tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightFirstMolarTooth": {
        "865995000": ("Permanent maxillary right first molar tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightFirstPremolarTooth": {
        "57826002": ("Permanent maxillary right first premolar tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightLateralIncisorTooth": {
        "424877001": ("Permanent maxillary right lateral incisor tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightSecondMolarTooth": {
        "863902006": ("Permanent maxillary right second molar tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightSecondPremolarTooth": {
        "36492000": ("Permanent maxillary right second premolar tooth", [4018, 4026])
    },
    "PermanentMaxillaryRightThirdMolarTooth": {
        "68085002": ("Permanent maxillary right third molar tooth", [4018, 4026])
    },
    "Permethrin": {"410457007": ("permethrin", [8112])},
    "PermethrinStain": {"333111009": ("Permethrin stain (substance)", [])},
    "PernaCurtaPigBreed": {"132163005": ("Perna-Curta pig breed", [7480])},
    "PeronealArtery": {
        "8821006": (
            "Peroneal artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
            ],
        )
    },
    "PeronealVein": {
        "71758008": ("Peroneal Vein", [3827, 7151, 7192, 8134, 9514, 12103, 12110])
    },
    "PeroxidaseStain": {"406974003": ("peroxidase stain", [8112])},
    "PerpendicularAxis": {"131189007": ("Perpendicular Axis", [218, 6165, 7469, 7470])},
    "PersianArabHorseBreed": {"132976000": ("Persian Arab horse breed", [7480])},
    "PersianCat": {"68086001": ("Persian cat", [7480])},
    "PersistentPainFollowingProcedure": {
        "279047007": ("Persistent pain following procedure", [6062])
    },
    "PersonalHistoryOfBreastCancer": {
        "415076002": ("Personal history of breast cancer", [6051])
    },
    "PersonalHistoryOfPrimaryMalignantNeoplasmOfBreast": {
        "415076002": (
            "Personal history of primary malignant neoplasm of breast (situation)",
            [],
        )
    },
    "PeruvianIncaOrchidDogBreed": {
        "132602002": ("Peruvian Inca Orchid dog breed", [7480])
    },
    "PeruvianPasoHorse": {"4960000": ("Peruvian Paso horse (organism)", [])},
    "PeruvianPasoHorseBreed": {"4960000": ("Peruvian Paso horse breed", [7480])},
    "PesterCattleBreed": {"133402002": ("Pester cattle breed", [7480])},
    "PetisoArgentinoHorseBreed": {
        "132977009": ("Petiso Argentino horse breed", [7480])
    },
    "PetitBassetGriffonVendeenDog": {
        "47542005": ("Petit basset griffon vendeen dog", [7480])
    },
    "PetitBassetGriffonVendéenDog": {
        "47542005": ("Petit Basset Griffon Vendéen dog (organism)", [])
    },
    "PetitBleuDeGascogneDogBreed": {
        "132603007": ("Petit Bleu de Gascogne dog breed", [7480])
    },
    "PetitBrabaçonDogBreed": {"132405009": ("Petit Brabaçon dog breed", [7480])},
    "PetitGasconSaintongeoisDogBreed": {
        "132604001": ("Petit Gascon-Saintongeois dog breed", [7480])
    },
    "PetitGriffonBleuDeGascogneDogBreed": {
        "132605000": ("Petit Griffon Bleu de Gascogne dog breed", [7480])
    },
    "PhakicIOL": {"397559001": ("Phakic IOL", [4231])},
    "PhakicIntraocularLensImplant": {
        "397559001": ("Phakic intraocular lens implant (physical object)", [])
    },
    "Phantom": {
        "706342009": (
            "Phantom",
            [
                4,
                4009,
                4030,
                4031,
                4042,
                4051,
                4052,
                7151,
                7192,
                7710,
                8134,
                9514,
                10060,
            ],
        )
    },
    "PharaohHound": {"14876008": ("Pharaoh hound", [7480])},
    "PharmacologicAndExerciseStressTest": {
        "428813002": (
            "Pharmacologic and exercise stress test",
            [3200, 3261, 3757, 12001],
        )
    },
    "PharmacologicStressProtocol": {
        "424064009": ("Pharmacologic Stress protocol", [3261, 12001])
    },
    "PharmacologicStressTest": {"424064009": ("Pharmacologic stress test", [3200])},
    "PharmacologicalStressTest": {"424064009": ("Pharmacological stress test", [3757])},
    "PharmacologicalStressUsed": {
        "246489000": ("Pharmacological stress used (attribute)", [])
    },
    "PharyngealRecess": {"1849007": ("pharyngeal recess", [7601, 8134])},
    "PharyngealStructure": {"54066008": ("Pharyngeal structure (body structure)", [])},
    "PharyngealTonsil": {"55940004": ("pharyngeal tonsil", [7601, 8134])},
    "PharyngotympanicTube": {"91207004": ("pharyngotympanic tube", [7601, 8134])},
    "Pharynx": {"54066008": ("Pharynx", [4, 4040, 7151, 7192, 8134, 9514])},
    "PharynxAndLarynx": {
        "312535008": ("Pharynx and larynx", [4, 4040, 7151, 7192, 8134, 9514])
    },
    "PharynxAndOrLarynxStructures": {
        "312535008": ("Pharynx and/or larynx structures (body structure)", [])
    },
    "PhaseContrastPlate": {"445625005": ("Phase contrast plate", [8124])},
    "PhaseContrastPlateOfOpticalMicroscope": {
        "445625005": (
            "Phase contrast plate of optical microscope (physical object)",
            [],
        )
    },
    "Phencyclidine": {"9721008": ("Phencyclidine", [6089])},
    "Phentolamine": {"372863003": ("Phentolamine", [10])},
    "Phenylephrine": {"386693003": ("Phenylephrine", [4208])},
    "PhilippineGoatBreed": {"131634008": ("Philippine goat breed", [7480])},
    "PhilippineNativeCattleBreed": {
        "131436003": ("Philippine Native cattle breed", [7480])
    },
    "PhilippineNativeIlocosPigBreed": {
        "132083009": ("Philippine Native, Ilocos pig breed", [7480])
    },
    "PhilippineNativeJalajalaPigBreed": {
        "132084003": ("Philippine Native, Jalajala pig breed", [7480])
    },
    "PhilippineNativePigBreed": {"132223004": ("Philippine Native pig breed", [7480])},
    "Phlebosclerosis": {"18016009": ("phlebosclerosis", [3817])},
    "PhloxinBStain": {"71957009": ("phloxin B stain", [8112])},
    "Phonation": {"43914001": ("Phonation", [91])},
    "PhosphotungsticAcidHematoxylinStain": {
        "406967008": ("phosphotungstic acid-hematoxylin stain", [8112])
    },
    "PhotodynamicTherapyOfSkin": {
        "428604001": ("Photodynamic therapy of skin", [4409])
    },
    "PhotographyMicroscopic": {
        "60287000": ("Photography, microscopic (procedure)", [])
    },
    "PhotographyOfGrossOrgan": {"52432005": ("Photography of gross organ", [8131])},
    "PhotographyOfPatient": {"197157006": ("Photography of patient", [8131])},
    "Photomicrography": {"60287000": ("Photomicrography", [8131])},
    "Photon": {"290006006": ("Photon", [9525])},
    "PhotorefractiveKeratectomy": {
        "397516006": ("Photorefractive keratectomy (procedure)", [])
    },
    "PhyllodesTumor": {"71232009": ("Phyllodes tumor", [6030, 6032])},
    "PhyllodesTumorBorderline": {
        "71232009": ("Phyllodes tumor, borderline (morphologic abnormality)", [])
    },
    "PhyllodesTumorMalignant": {
        "87913009": ("Phyllodes tumor, malignant", [6030, 6033])
    },
    "PhysicalObject": {"260787004": ("Physical object", [7150, 9501, 9502, 9580])},
    "Physician": {"309343006": ("Physician", [7450, 7452, 9536, 9555, 9562])},
    "PhysiologicalMonitoringRegimeRegimeTherapy": {
        "281691001": ("Physiological monitoring regime (regime/therapy)", [])
    },
    "Physiotherapy": {"310464005": ("Physiotherapy", [7030])},
    "PhysiotherapyDepartment": {
        "310464005": ("Physiotherapy department (environment)", [])
    },
    "PiaMater": {"23180006": ("Pia mater", [5, 7151, 7153, 7192, 8134, 9514])},
    "PiaMaterStructure": {"23180006": ("Pia mater structure (body structure)", [])},
    "PiauCarunchoPiauPigBreed": {
        "132064002": ("Piau, Caruncho Piau pig breed", [7480])
    },
    "PiauPigBreed": {"132056008": ("Piau pig breed", [7480])},
    "PiauSorocabaPigBreed": {"132160008": ("Piau, Sorocaba pig breed", [7480])},
    "PicCambourghPig": {"17717005": ("Pic Cambourgh pig (organism)", [])},
    "PicCambourghPigBreed": {"17717005": ("Pic Cambourgh pig breed", [7480])},
    "PicLinePig24": {"86440008": ("Pic line pig 24 (organism)", [])},
    "PicLinePig24PigBreed": {"86440008": ("Pic line pig 24 pig breed", [7480])},
    "PicLinePig26": {"29235007": ("Pic line pig 26 (organism)", [])},
    "PicLinePig26PigBreed": {"29235007": ("Pic line pig 26 pig breed", [7480])},
    "PicPig": {"75709004": ("Pic pig (organism)", [])},
    "PicPigBreed": {"75709004": ("Pic pig breed", [7480])},
    "PicricAcid": {"24215009": ("Picric acid", [8114])},
    "PieRougeDeLEstCattleBreed": {
        "133403007": ("Pie Rouge de l'Est cattle breed", [7480])
    },
    "PieRougeDesPlainesCattleBreed": {
        "133563007": ("Pie Rouge des Plaines cattle breed", [7480])
    },
    "PietrainPig": {"20044005": ("Pietrain pig (organism)", [])},
    "PietrainPigBreed": {"20044005": ("Pietrain pig breed", [7480])},
    "PigghamPigBreed": {"132267002": ("Piggham pig breed", [7480])},
    "PiggybackIOL": {"370951003": ("Piggyback IOL", [4231])},
    "PiggybackIntraocularLens": {
        "370951003": ("Piggyback intraocular lens (finding)", [])
    },
    "Pin": {"77444004": ("Pin", [6102, 6138, 6202, 6203, 6404])},
    "PindosHorseBreed": {"132984001": ("Pindos horse breed", [7480])},
    "PindosPonyBreed": {"424111008": ("Pindos pony breed (organism)", [])},
    "PindosPonyHorseBreed": {"424111008": ("Pindos pony horse breed", [7480])},
    "PinealGland": {"45793000": ("Pineal Gland", [7151, 7153, 7192, 8134, 9514])},
    "PinealStructure": {"45793000": ("Pineal structure (body structure)", [])},
    "PineywoodsPigBreed": {"132037003": ("Pineywoods pig breed", [7480])},
    "PinholeVisualAcuity": {"419475002": ("Pinhole Visual Acuity", [4216])},
    "Pink": {"371243003": ("Pink", [4411])},
    "PinkColor": {"371243003": ("Pink color (qualifier value)", [])},
    "PintabianHorseBreed": {"133064007": ("Pintabian horse breed", [7480])},
    "Pinto": {"58264006": ("Pinto (organism)", [])},
    "PintoHorseBreed": {"58264006": ("Pinto horse breed", [7480])},
    "PinzgauerCattleBreed": {"400003": ("Pinzgauer cattle breed (organism)", [])},
    "PinzgauerCowBreed": {"400003": ("Pinzgauer cow breed", [7480])},
    "PinzhouCattleBreed": {"131515002": ("Pinzhou cattle breed", [7480])},
    "PinziritaSheepBreed": {"132851002": ("Pinzirita sheep breed", [7480])},
    "PiquiraPonyHorseBreed": {"132985000": ("Piquira Pony horse breed", [7480])},
    "PirapetingaPigBreed": {"132055007": ("Pirapetinga pig breed", [7480])},
    "Pirie": {"399022001": ("Pirie", [4012])},
    "PirieProjection": {"399022001": ("Pirie projection (qualifier value)", [])},
    "PisanaCattleBreed": {"133404001": ("Pisana cattle breed", [7480])},
    "PitangueirasCattleBreed": {
        "133822008": ("Pitangueiras cattle breed (organism)", [])
    },
    "PitangueirasXZebuCattleBreed": {
        "133822008": ("Pitangueiras X zebu cattle breed", [7480])
    },
    "PitmanMooreMiniaturePigBreed": {
        "132040003": ("Pitman-Moore Miniature pig breed", [7480])
    },
    "PittIslandSheepBreed": {"132849001": ("Pitt Island sheep breed", [7480])},
    "Pituitary": {"56329008": ("Pituitary", [7151, 7153, 7192, 8134, 9514])},
    "PituitaryFossa": {"42575006": ("Pituitary Fossa", [1000, 1001, 1002])},
    "PituitaryStructure": {"56329008": ("Pituitary structure (body structure)", [])},
    "PixieBobCatBreed": {"417277001": ("Pixie-bob cat breed", [7480])},
    "PlacementOfStent": {"103716009": ("Placement of stent (procedure)", [])},
    "PlacementOfStentInCoronaryArtery": {
        "36969009": ("Placement of stent in coronary artery (procedure)", [])
    },
    "Placenta": {"78067005": ("Placenta", [8134])},
    "PlacentalStructure": {"78067005": ("Placental structure (body structure)", [])},
    "PlacingRestraint": {"68894007": ("Placing restraint", [9577])},
    "PlainRadiography": {"168537006": ("Plain radiography", [10005])},
    "PlanningTargetVolume": {
        "228793007": ("Planning target volume (observable entity)", [])
    },
    "Plantar60DegreeLateralDorsomedialOblique": {
        "442675004": ("Plantar 60 degree lateral-dorsomedial oblique", [7484])
    },
    "Plantar60DegreeLateralDorsomedialObliqueProjection": {
        "442675004": (
            "Plantar 60 degree lateral-dorsomedial oblique projection (qualifier value)",
            [],
        )
    },
    "Plantar75DegreeProximalDorsodistalOblique": {
        "442603007": ("Plantar 75 degree proximal-dorsodistal oblique", [7484])
    },
    "Plantar75DegreeProximalDorsodistalObliqueProjection": {
        "442603007": (
            "Plantar 75 degree proximal-dorsodistal oblique projection (qualifier value)",
            [],
        )
    },
    "PlantarArterialArch": {
        "83018002": (
            "Plantar Arterial Arch",
            [3827, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "PlantarArterialArchStructure": {
        "83018002": ("Plantar arterial arch structure (body structure)", [])
    },
    "PlantarolateralDorsomedialOblique": {
        "442658005": ("Plantarolateral-dorsomedial oblique", [7484])
    },
    "PlantarolateralDorsomedialObliqueProjection": {
        "442658005": (
            "Plantarolateral-dorsomedial oblique projection (qualifier value)",
            [],
        )
    },
    "PlantaroproximalDorsodistalOblique": {
        "442625001": ("Plantaroproximal-dorsodistal oblique", [7484])
    },
    "PlantaroproximalDorsodistalObliqueProjection": {
        "442625001": (
            "Plantaroproximal-dorsodistal oblique projection (qualifier value)",
            [],
        )
    },
    "Plantodorsal": {"399071006": ("plantodorsal", [501, 4010])},
    "PlantodorsalProjection": {
        "399071006": ("Plantodorsal projection (qualifier value)", [])
    },
    "Plaque": {"1522000": ("plaque", [3810])},
    "PlaqueUlceration": {"62189002": ("Plaque Ulceration", [3491])},
    "PlasmaExpander": {"372578001": ("Plasma Expander", [621, 622])},
    "Plasmacytoma": {"1187107005": ("Plasmacytoma", [6030, 6033])},
    "Plastic": {"61088005": ("Plastic", [8115])},
    "PlasticStent": {"257362008": ("plastic stent", [3814])},
    "PlasticSurgery": {"309992008": ("Plastic Surgery", [7030])},
    "PlasticSurgeryDepartment": {
        "309992008": ("Plastic surgery department (environment)", [])
    },
    "PlateLikeAtelectasis": {"40779009": ("Plate-like atelectasis", [6102, 6103])},
    "Platinum": {"84847000": ("Platinum", [300])},
    "PleomorphicAdenoma": {"8360001": ("Pleomorphic adenoma", [6030, 6031])},
    "Pleura": {"3120008": ("Pleura", [7151, 7155, 7192, 9514, 12294])},
    "PleuralEffusion": {"60046008": ("Pleural effusion", [645, 8134])},
    "PleuralMembraneStructure": {
        "3120008": ("Pleural membrane structure (body structure)", [])
    },
    "PleuralStructure": {"3120008": ("Pleural structure", [6100, 6129, 8134])},
    "PlevenHorseBreed": {"132986004": ("Pleven horse breed", [7480])},
    "PlottHound": {"40400008": ("Plott hound", [7480])},
    "Pm0NoDistantMetastasisBreast": {
        "373169005": ("pM0: No distant metastasis (breast) (finding)", [])
    },
    "Pm1": {"1229916009": ("pM1", [6162])},
    "Pm1DistantMetastasisBreast": {
        "373171005": ("pM1: Distant metastasis (breast) (finding)", [])
    },
    "PmxDistantMetastasisCannotBeAssessedBreast": {
        "373170006": (
            "pMX: Distant metastasis cannot be assessed (breast) (finding)",
            [],
        )
    },
    "Pn0": {"1229947003": ("pN0", [6161])},
    "Pn0NoRegionalLymphNodeMetastasisHistologicallyIENoneGreaterThan0Point2mmNoAdditionalExaminationForIsolatedTumorCellsBreast": {
        "373151001": (
            "pN0: No regional lymph node metastasis histologically "
            "(i.e., none greater than 0.2 mm), no additional "
            "examination for isolated tumor cells (breast) (finding)",
            [],
        )
    },
    "Pn1": {"1229951001": ("pN1", [6161])},
    "Pn1MetastasisIn1To3AxillaryLymphNodesAndOrInInternalMammaryNodesWithMicroscopicDiseaseDetectedBySentinelLymphNodeDissectionButNotClinicallyApparentBreast": {
        "373156006": (
            "pN1: Metastasis in 1 to "
            "3 axillary lymph nodes, "
            "and/or in internal "
            "mammary nodes with "
            "microscopic disease "
            "detected by sentinel "
            "lymph node dissection "
            "but not clinically "
            "apparent (breast) "
            "(finding)",
            [],
        )
    },
    "Pn2": {"1229957002": ("pN2", [6161])},
    "Pn2MetastasisIn4To9AxillaryLymphNodesOrInClinicallyApparentInternalMammaryLymphNodesInTheAbsenceOfAxillaryLymphNodeMetastasisBreast": {
        "373162001": (
            "pN2: Metastasis in 4 to 9 axillary lymph "
            "nodes, or in clinically apparent internal "
            "mammary lymph nodes in the absence of axillary "
            "lymph node metastasis (breast) (finding)",
            [],
        )
    },
    "Pn2a": {"1229959004": ("pN2a", [6161])},
    "Pn2aMetastasisIn4To9AxillaryLymphNodesAtLeastOneTumorDepositGreaterThan2Point0mmBreast": {
        "373163006": (
            "pN2a: Metastasis in 4 to 9 axillary lymph nodes (at least one tumor deposit greater than "
            "2.0 mm) (breast) (finding)",
            [],
        )
    },
    "Pn2b": {"1229960009": ("pN2b", [6161])},
    "Pn2bMetastasisInClinicallyApparentInternalMammaryLymphNodesInTheAbsenceOfAxillaryLymphNodeMetastasisBreast": {
        "373164000": (
            "pN2b: Metastasis in clinically apparent internal mammary lymph nodes in "
            "the absence of axillary lymph node metastasis (breast) (finding)",
            [],
        )
    },
    "Pn3": {"1229962001": ("pN3", [6161])},
    "Pn3a": {"1229963006": ("pN3a", [6161])},
    "Pn3aMetastasisIn10OrMoreAxillaryLymphNodesAtLeastOneTumorDepositGreaterThan2Point0mmOrMetastasisToInfraclavicularLymphNodesBreast": {
        "373165004": (
            "pN3a: Metastasis in 10 or more axillary lymph "
            "nodes (at least one tumor deposit greater than "
            "2.0 mm), or metastasis to infraclavicular lymph "
            "nodes (breast) (finding)",
            [],
        )
    },
    "Pn3b": {"1229964000": ("pN3b", [6161])},
    "Pn3bTumorOfBreastWithMetastasisAsPerAmericanJointCommitteeOnCancer6thEditionDefinitionBreast": {
        "373167007": (
            "pN3b: Tumor of breast with metastasis as per American Joint Committee on Cancer 6th "
            "Edition definition (breast) (finding)",
            [],
        )
    },
    "Pn3c": {"1229965004": ("pN3c", [6161])},
    "Pn3cMetastasisInIpsilateralSupraclavicularLymphNodesBreast": {
        "373166003": (
            "pN3c: Metastasis in ipsilateral supraclavicular lymph nodes (breast) (finding)",
            [],
        )
    },
    "Pneumomediastinum": {"16838000": ("Pneumomediastinum", [6102, 6105])},
    "Pneumothorax": {"36118008": ("Pneumothorax", [6062, 6102, 6105])},
    "Pnx": {"1229945006": ("pNX", [6161])},
    "PnxRegionalLymphNodesCannotBeAssessedEGPreviouslyRemovedOrNotRemovedForPathologicStudyBreast": {
        "373150000": (
            "pNX: Regional lymph nodes cannot be assessed (e.g., previously removed, or not "
            "removed for pathologic study) (breast) (finding)",
            [],
        )
    },
    "PodengoCanarioDogBreed": {"132444001": ("Podengo Canario dog breed", [7480])},
    "PodengoPequenoDogBreed": {"132445000": ("Podengo Pequeno dog breed", [7480])},
    "PointSourceMeasurement": {"371913007": ("Point source measurement", [3627])},
    "PointSourceNumericEstimationTechnique": {
        "371913007": (
            "Point source - numeric estimation technique (qualifier value)",
            [],
        )
    },
    "Pointer": {"73318001": ("Pointer", [7480])},
    "PoitevinDogBreed": {"132610001": ("Poitevin dog breed", [7480])},
    "PoitouGoatBreed": {"131646007": ("Poitou goat breed", [7480])},
    "PoitouMuleProducerHorseBreed": {
        "133073004": ("Poitou Mule Producer horse breed", [7480])
    },
    "PolandChinaPig": {"79814001": ("Poland China pig (organism)", [])},
    "PolandChinaPigBreed": {"79814001": ("Poland China pig breed", [7480])},
    "PolarizingOpticalFilter": {
        "445391002": ("Polarizing optical filter", [4204, 8124])
    },
    "PolarizingOpticalFilterOfOpticalMicroscope": {
        "445391002": (
            "Polarizing optical filter of optical microscope (physical object)",
            [],
        )
    },
    "PolesianHorseBreed": {"133074005": ("Polesian horse breed", [7480])},
    "PolesianPigBreed": {"133225003": ("Polesian pig breed", [7480])},
    "PolishBlackAndWhiteLowlandCattleBreed": {
        "133414005": ("Polish Black and White Lowland cattle breed", [7480])
    },
    "PolishDraftHorseBreed": {"132978004": ("Polish Draft horse breed", [7480])},
    "PolishHoundDogBreed": {"132609006": ("Polish Hound dog breed", [7480])},
    "PolishLandracePigBreed": {"132273001": ("Polish Landrace pig breed", [7480])},
    "PolishMarshPigBreed": {"133229009": ("Polish Marsh pig breed", [7480])},
    "PolishRedCattleBreed": {"133256006": ("Polish Red cattle breed", [7480])},
    "PolishSimmentalCattleBreed": {
        "133415006": ("Polish Simmental cattle breed", [7480])
    },
    "PollMerinoSheepBreed": {"132721003": ("Poll Merino sheep breed", [7480])},
    "PolledGirCattleBreed": {"133688008": ("Polled Gir cattle breed", [7480])},
    "PolledGuzeratCattleBreed": {"133746006": ("Polled Guzerat cattle breed", [7480])},
    "PolledHereford": {"9277006": ("Polled Hereford (organism)", [])},
    "PolledHerefordCowBreed": {"9277006": ("Polled Hereford cow breed", [7480])},
    "PolledJerseyCattleBreed": {"133416007": ("Polled Jersey cattle breed", [7480])},
    "PolledLincolnRedCattleBreed": {
        "133417003": ("Polled Lincoln Red cattle breed", [7480])
    },
    "PolledNeloreCattleBreed": {"133753002": ("Polled Nelore cattle breed", [7480])},
    "PolledShorthornUSCattleBreed": {
        "133418008": ("Polled Shorthorn (US) cattle breed", [7480])
    },
    "PolledShorthornUnitedStatesOfAmericaCattleBreed": {
        "133418008": (
            "Polled Shorthorn (United States of America) cattle breed (organism)",
            [],
        )
    },
    "PolledSimmentalCattleBreed": {
        "133419000": ("Polled Simmental cattle breed", [7480])
    },
    "PolledSussexCattleBreed": {"133430002": ("Polled Sussex cattle breed", [7480])},
    "PolledWelshBlackCattleBreed": {
        "133431003": ("Polled Welsh Black cattle breed", [7480])
    },
    "PolskiOwczarekNizinnyDogBreed": {
        "132608003": ("Polski Owczarek Nizinny dog breed", [7480])
    },
    "PoltavaPigBreed": {"132069007": ("Poltava pig breed", [7480])},
    "PolwarthSheepBreed": {"132705008": ("Polwarth sheep breed", [7480])},
    "Polymer": {"412155002": ("Polymer", [7300, 9579])},
    "PolypOfColon": {"68496003": ("Polyp of colon", [6201])},
    "PolypSizeLargestDimension": {
        "373197004": ("Polyp size, largest dimension", [6212])
    },
    "PolypStalkLength": {"395511002": ("Polyp stalk length", [6212])},
    "PolypaySheepBreed": {"131823008": ("Polypay sheep breed", [7480])},
    "PomeranianCoarsewoolSheepBreed": {
        "131831003": ("Pomeranian Coarsewool sheep breed", [7480])
    },
    "PomeranianDog": {"10040000": ("Pomeranian dog", [7480])},
    "PomeranianPigBreed": {"132272006": ("Pomeranian pig breed", [7480])},
    "Ponceau3RStain": {"65730007": ("ponceau 3R stain", [8112])},
    "PonceauSStain": {"89856006": ("ponceau S stain", [8112])},
    "PonceauXylidineStain": {"70520000": ("ponceau xylidine stain", [8112])},
    "PontamineSkyBlue5BXStain": {"89577003": ("pontamine sky blue 5BX stain", [8112])},
    "PontamineSkyBlue6BXStain": {"80305003": ("pontamine sky blue 6BX stain", [8112])},
    "PontremoleseCattleBreed": {"133432005": ("Pontremolese cattle breed", [7480])},
    "PonwarCattleBreed": {"133622006": ("Ponwar cattle breed", [7480])},
    "Pony": {"3997000": ("Pony (organism)", [])},
    "PonyHorseBreed": {"3997000": ("Pony horse breed", [7480])},
    "PonyOfTheAmericasHorseBreed": {
        "133063001": ("Pony of the Americas horse breed", [7480])
    },
    "PoodleSuperbreed": {"15171008": ("Poodle superbreed (organism)", [])},
    "PoodleSuperbreedDogBreed": {"15171008": ("Poodle superbreed dog breed", [7480])},
    "PoorVisualFixation": {"314348007": ("Poor Visual Fixation", [4222])},
    "PoorlyDefined": {"300841009": ("Poorly defined", [6120])},
    "PoplitealArtery": {
        "43899006": (
            "Popliteal artery",
            [
                4,
                3010,
                3440,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
            ],
        )
    },
    "PoplitealFossa": {"32361000": ("Popliteal fossa", [4, 7151, 7192, 8134, 9514])},
    "PoplitealFossaStructure": {
        "32361000": ("Popliteal fossa structure (body structure)", [])
    },
    "PoplitealLymphNode": {"47471008": ("popliteal lymph node", [7600, 8134])},
    "PoplitealLymphNodeStructure": {
        "47471008": ("Popliteal lymph node structure (body structure)", [])
    },
    "PoplitealVein": {
        "56849005": ("Popliteal Vein", [3827, 7151, 7192, 8134, 9514, 12103, 12110])
    },
    "PorcelaineDogBreed": {"132578009": ("Porcelaine dog breed", [7480])},
    "PorcineSpecies": {"42018006": ("Porcine species (organism)", [])},
    "Portal": {"32381004": ("Portal (qualifier value)", [])},
    "PortalVein": {
        "32764006": (
            "Portal vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12114,
                12125,
            ],
        )
    },
    "PortalVeinStructure": {"32764006": ("Portal vein structure (body structure)", [])},
    "PortlandSheepBreed": {"132682000": ("Portland sheep breed", [7480])},
    "PortoAmboimCattleBreed": {"131516001": ("Porto Amboim cattle breed", [7480])},
    "PortugueseGuardDogBreed": {"132378008": ("Portuguese Guard Dog breed", [7480])},
    "PortuguesePointerDogBreed": {
        "132577004": ("Portuguese Pointer dog breed", [7480])
    },
    "PortugueseWaterDog": {"63390008": ("Portuguese water dog", [7480])},
    "PosavinaCattleBreed": {"131517005": ("Posavina cattle breed", [7480])},
    "PositionOfJoint": {"276334009": ("Position of joint (finding)", [])},
    "Positive": {"10828004": ("Positive", [250, 3231])},
    "PositronEmissionTomographyBreastStudy": {
        "416323006": ("Positron emission tomography breast study (procedure)", [])
    },
    "PositronEmissionTomographyHeartStudy": {
        "241439007": ("Positron emission tomography heart study (procedure)", [])
    },
    "PositronEmissionTomographyOfWholeBody": {
        "702767007": ("Positron emission tomography of whole body (procedure)", [])
    },
    "PositronEmissionTomographyStudyForLocalizationOfTumor": {
        "241443006": (
            "Positron emission tomography study for localization of tumor (procedure)",
            [],
        )
    },
    "PositronEmissionTomographyWithComputedTomographyFluorodeoxyglucoseImagingOfWholeBody": {
        "443271005": (
            "Positron emission tomography with computed tomography fluorodeoxyglucose imaging of whole "
            "body (procedure)",
            [],
        )
    },
    "PossiblePregnancy": {"146799005": ("possible pregnancy", [6096])},
    "PossiblePregnancy_Inactive": {"102874004": ("Possible pregnancy (finding)", [])},
    "PossibleThrombus": {"373141002": ("Possible Thrombus", [3714])},
    "PostAblationPhase": {"129093009": ("Post-ablation phase", [3254])},
    "PostChemotherapy": {"262502001": ("Post-chemotherapy", [6146])},
    "PostDefibrillationProcedurePhase": {
        "129091006": ("Post-defibrillation procedure phase", [3254])
    },
    "PostDose": {"255566006": ("Post-dose", [31])},
    "PostExerciseState": {"128977007": ("Post-exercise state", [3262, 3602])},
    "PostInfarctAngina": {"314116003": ("Post infarct angina (disorder)", [])},
    "PostInfarctionAngina": {"314116003": ("Post-infarction angina", [3700])},
    "PostOperative": {"262061000": ("Post-operative", [31, 6146])},
    "PostPTCA": {"371822007": ("Post PTCA", [3201])},
    "PostPercutaneousTransluminalCoronaryAngioplasty": {
        "373108000": (
            "Post percutaneous transluminal coronary angioplasty (finding)",
            [],
        )
    },
    "PostRadiation": {"264908009": ("Post-radiation", [6146])},
    "PostStenoticDilation": {"371872009": ("Post Stenotic Dilation", [3709])},
    "PostSurgicalAnatomy": {"245849007": ("Post-surgical anatomy", [9272])},
    "PostVitrectomy": {"232077005": ("Post-Vitrectomy", [4232])},
    "PostVolumeChallenge": {"128971008": ("Post volume challenge", [91, 3271])},
    "PostartificialMenopausalSyndrome": {
        "31351009": ("Postartificial menopausal syndrome (disorder)", [])
    },
    "PostauricularLymphNode": {"245328002": ("postauricular lymph node", [7600, 8134])},
    "Postaxial": {"60583000": ("Postaxial", [2])},
    "PostductalRegionOfAorticArch": {
        "60835009": ("Postductal region of aortic arch", [12280])
    },
    "Posterior": {
        "255551008": ("Posterior", [2, 5, 211, 212, 3019, 4005, 6024, 6025, 12281])
    },
    "PosteriorArchVein": {
        "397435008": (
            "Posterior arch vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "PosteriorAuricularLymphNode": {
        "30793004": ("posterior auricular lymph node", [7600, 8134])
    },
    "PosteriorCerebralArtery": {
        "70382005": (
            "Posterior Cerebral Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "PosteriorCerebralArteryP1Segment": {
        "415144009": (
            "Posterior Cerebral Artery P1 Segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "PosteriorCerebralArteryP2Segment": {
        "415145005": (
            "Posterior Cerebral Artery P2 Segment",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "PosteriorCerebralCommissure": {
        "279336005": ("Posterior cerebral commissure", [7151, 7153, 7192, 8134, 9514])
    },
    "PosteriorCommissure": {
        "279336005": ("Posterior Commissure", [7101, 7111, 7705, 7710])
    },
    "PosteriorCommissureOfLabiumMajorum": {
        "4019005": ("Posterior commissure of labium majorum", [4029, 8134])
    },
    "PosteriorCommunicatingArtery": {
        "43119007": (
            "posterior communicating artery",
            [3604, 3606, 3630, 3827, 12103, 12105],
        )
    },
    "PosteriorCommunicationArtery": {
        "43119007": (
            "Posterior communication artery",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PosteriorDescendingCoronaryArtery": {
        "443113009": ("Posterior Descending Coronary Artery", [12292])
    },
    "PosteriorDescendingRightCoronaryArtery": {
        "53655008": (
            "Posterior Descending Right Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "PosteriorEmissiveProjection": {
        "399001007": ("Posterior emissive projection (qualifier value)", [])
    },
    "PosteriorHornLateralVentricle": {
        "52943005": ("Posterior Horn Lateral Ventricle", [8134, 12022])
    },
    "PosteriorMedialTributary": {
        "128569001": (
            "Posterior medial tributary",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PosteriorMedialTributaryOfSuperficialVenousSystemOfLowerExtremity": {
        "128569001": (
            "Posterior medial tributary of superficial venous system of lower extremity (body structure)",
            [],
        )
    },
    "PosteriorMediastinalLymphNode": {
        "25447008": ("posterior mediastinal lymph node", [7600, 8134])
    },
    "PosteriorProjection": {"399001007": ("Posterior projection", [26, 501])},
    "PosteriorSegmentOfRightUpperLobe": {
        "3236000": ("Posterior segment of right upper lobe", [6124, 6127])
    },
    "PosteriorTibialArtery": {
        "13363002": (
            "Posterior tibial artery",
            [4, 3010, 3440, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12109],
        )
    },
    "PosteriorTibialLymphNode": {
        "303623000": ("posterior tibial lymph node", [7600, 8134])
    },
    "PosteriorTibialVein": {
        "4258007": (
            "Posterior Tibial Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "PosteriorTriangleCervicalLymphNode": {
        "245324000": ("posterior triangle cervical lymph node", [7600, 8134])
    },
    "PosteriorWall": {"264159006": ("Posterior Wall", [12280])},
    "PosteroAnterior": {"272479007": ("postero-anterior", [501, 4010])},
    "PosteroAnteriorOblique": {"399059000": ("postero-anterior oblique", [501, 4010])},
    "PosteroAnteriorObliqueProjection": {
        "399059000": ("Postero-anterior oblique projection (qualifier value)", [])
    },
    "PosteroanteriorProjection": {
        "272479007": ("Posteroanterior projection (qualifier value)", [])
    },
    "Posterolateral": {"90069004": ("Posterolateral", [2])},
    "PosterolateralBranchOfRightCoronaryArtery": {
        "17269004": (
            "Posterolateral branch of right Coronary Artery",
            [3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "Postoperative": {"262061000": ("Postoperative", [631, 632])},
    "PostoperativeAnesthesiaCareUnit": {
        "398161000": ("Postoperative anesthesia care unit (environment)", [])
    },
    "PostoperativeHematomaFormation": {
        "213262007": ("Postoperative hematoma formation (disorder)", [])
    },
    "PostoperativeHemorrhage": {
        "110265006": ("Postoperative hemorrhage (disorder)", [])
    },
    "PostoperativePeriod": {
        "262061000": ("Postoperative period (qualifier value)", [])
    },
    "Postprandial": {"24863003": ("Postprandial (qualifier value)", [])},
    "PostproceduralPeriod": {
        "303110006": ("Postprocedural period (qualifier value)", [])
    },
    "PostsurgicalMenopause": {"371036001": ("Postsurgical menopause", [6086])},
    "PostvesicularLymphNode": {"278571002": ("postvesicular lymph node", [7600, 8134])},
    "Potassium": {"88480006": ("Potassium", [621, 622])},
    "PotassiumCarbonateK42": {"111161000": ("Potassium carbonate K^42^", [25])},
    "PotassiumChlorideK42": {"36641004": ("Potassium chloride K^42^", [25])},
    "PotassiumChlorideK43": {"47729008": ("Potassium chloride K^43^", [25])},
    "PotassiumDichromate": {"19893005": ("Potassium dichromate", [8114])},
    "PotassiumHydroxide": {"34763001": ("Potassium hydroxide (substance)", [])},
    "PotassiumHydroxideStain": {"34763001": ("potassium hydroxide stain", [8112])},
    "PotentialAcuityMeterVisualAcuity": {
        "424622008": ("Potential Acuity Meter Visual Acuity", [4216])
    },
    "PottokTarpanHorseXDomesticHorseBreed": {
        "132993000": ("Pottok tarpan horse X domestic horse breed", [7480])
    },
    "PowerDoppler": {"425704008": ("Power Doppler", [218, 6058, 7180, 7469, 12224])},
    "PowerDopplerUltrasound": {
        "425704008": ("Power doppler ultrasound (qualifier value)", [])
    },
    "PowerOfCylinder": {"251797004": ("Power of cylinder (observable entity)", [])},
    "PowerOfSphere": {"251795007": ("Power of sphere (observable entity)", [])},
    "PreAdmission": {"281379000": ("Pre-admission", [31])},
    "PreBiopsyLocalizationOfBreastLesion": {
        "237380007": ("Pre-biopsy localization of breast lesion", [6050, 6083])
    },
    "PreDose": {"255235001": ("Pre-dose", [31])},
    "PreExerciseState": {"128977007": ("Pre-exercise state (finding)", [])},
    "PreOperative": {"262068006": ("Pre-operative", [31, 3201, 6146])},
    "PreSurgeryTesting": {"110467000": ("Pre-Surgery testing", [3671])},
    "Preaxial": {"32400000": ("Preaxial", [2])},
    "PrececalLymphNode": {"281765006": ("prececal lymph node", [7600, 8134])},
    "Prednisone": {"116602009": ("Prednisone", [65])},
    "PreductalRegionOfAorticArch": {
        "75397005": ("Preductal region of aortic arch", [12280])
    },
    "PrefemoralLymphNode": {"48193007": ("prefemoral lymph node", [7600, 8134])},
    "PregnancyObservable": {
        "364320009": ("Pregnancy observable (observable entity)", [])
    },
    "PreinfarctionSyndrome": {"4557003": ("Preinfarction syndrome (disorder)", [])},
    "PrelaryngealLymphNode": {"74203007": ("prelaryngeal lymph node", [7600, 8134])},
    "PrematureAtrialContraction": {
        "284470004": ("Premature atrial contraction (disorder)", [])
    },
    "PremierPigBreed": {"132028003": ("Premier pig breed", [7480])},
    "Preoperative": {"262068006": ("Preoperative", [631, 632])},
    "PrepericardialLymphNode": {
        "196516004": ("prepericardial lymph node", [7600, 8134])
    },
    "PresaCanarioDogBreed": {"132576008": ("Presa Canario dog breed", [7480])},
    "Present": {"52101004": ("Present", [240, 241])},
    "PressaMallorquinDogBreed": {"132446004": ("Pressa Mallorquin dog breed", [7480])},
    "PressurePhysicalAgent": {
        "279046003": ("Pressure - physical agent (physical force)", [])
    },
    "PressureSupportVentilator": {"371786002": ("Pressure Support Ventilator", [3554])},
    "PresticePigBreed": {"133213001": ("Prestice pig breed", [7480])},
    "PresymphysialLymphNode": {"6413002": ("presymphysial lymph node", [7600, 8134])},
    "PretaCattleBreed": {"133433000": ("Preta cattle breed", [7480])},
    "PretrachealLymphNode": {"168460001": ("pretracheal lymph node", [7600, 8134])},
    "PrevascularRetrotrachealLymphNode": {
        "127930004": ("prevascular/retrotracheal lymph node", [7600, 8134])
    },
    "PreventiveIntent": {"129428001": ("Preventive intent", [3629])},
    "PreventiveProcedureIntent": {
        "129428001": ("Preventive - procedure intent (qualifier value)", [])
    },
    "PrevertebralLymphNode": {"196446004": ("prevertebral lymph node", [7600, 8134])},
    "PrevesicularLymphNode": {"11740004": ("prevesicular lymph node", [7600, 8134])},
    "PrewakwaCattleBreed": {"133754008": ("Prewakwa cattle breed", [7480])},
    "PrianganSheepBreed": {"132814008": ("Priangan sheep breed", [7480])},
    "PrimaryCareDepartment": {"441480003": ("Primary Care Department", [7030])},
    "PrimaryGaze": {"408744005": ("Primary gaze", [4201])},
    "PrimaryMalignantNeoplasmOfLung": {
        "93880001": ("Primary malignant neoplasm of lung (disorder)", [])
    },
    "PrimaryRestrictiveCardiomyopathy": {
        "90828009": ("Primary restrictive cardiomyopathy (disorder)", [])
    },
    "PrimarySquamousCellCarcinomaOfAnus": {
        "723265000": ("Primary squamous cell carcinoma of anus (disorder)", [])
    },
    "PrimarySquamousCellCarcinomaOfLaryngealCartilage": {
        "707357005": (
            "Primary squamous cell carcinoma of laryngeal cartilage (disorder)",
            [],
        )
    },
    "PrimitiveAorta": {
        "14944004": ("Primitive aorta", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "PrimitiveAorticStructure": {
        "14944004": ("Primitive aortic structure (body structure)", [])
    },
    "PrimitivePulmonaryArtery": {
        "91707000": (
            "Primitive pulmonary artery",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "Principal": {"63161005": ("Principal (qualifier value)", [])},
    "PrinzmetalAngina": {"87343002": ("Prinzmetal angina (disorder)", [])},
    "PriobHorseBreed": {"132979007": ("Priob horse breed", [7480])},
    "Priority": {"260870009": ("Priority (attribute)", [])},
    "PrisheksninskPigBreed": {"132107009": ("Prisheksninsk pig breed", [7480])},
    "Problem": {"55607006": ("Problem", [3769])},
    "ProblemResolved": {"413322009": ("Problem resolved", [3770])},
    "Procedure": {"71388002": ("Procedure (procedure)", [])},
    "ProcedureContext": {"408730004": ("Procedure context (attribute)", [])},
    "ProcedureInCardiacCareUnit": {
        "373111004": ("Procedure in Cardiac Care Unit", [3671])
    },
    "ProcedureInCoronaryCareUnit": {
        "373111004": ("Procedure in coronary care unit (procedure)", [])
    },
    "ProcedureOnJoint": {"118745001": ("Procedure on joint (procedure)", [])},
    "ProcedurePhase": {"128954007": ("Procedure phase (qualifier value)", [])},
    "ProcedureSite": {"363704007": ("Procedure site (attribute)", [])},
    "ProcedureToMeetOccupationalRequirement": {
        "429060002": ("Procedure to meet occupational requirement (procedure)", [])
    },
    "ProcionBrilliantBlueMRSStain": {
        "24900003": ("procion brilliant blue MRS stain", [8112])
    },
    "ProfessionalNurse": {"106292003": ("Professional nurse (occupation)", [])},
    "ProfundaFemorisArtery": {
        "31677005": (
            "Profunda Femoris Artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
            ],
        )
    },
    "ProfundaFemorisVein": {
        "23438002": (
            "Profunda Femoris Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "ProgesteronePreparation": {"50318003": ("Progesterone preparation (product)", [])},
    "ProgesteroneProduct": {"50318003": ("Progesterone product", [6080])},
    "ProkineticAgent": {"116532005": ("Prokinetic Agent (product)", [])},
    "PromontoryCommonIliacLymphNode": {
        "279189002": ("promontory common iliac lymph node", [7600, 8134])
    },
    "Pronation": {"88241000": ("Pronation", [92])},
    "PronationFunction": {"88241000": ("Pronation, function (observable entity)", [])},
    "Prone": {"1240000": ("prone", [20, 6206])},
    "ProneBodyPosition": {"1240000": ("Prone body position (finding)", [])},
    "ProperHepaticArtery": {
        "18112008": (
            "Proper Hepatic Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "ProphylacticIntent": {"360271000": ("Prophylactic intent", [3629])},
    "ProphylaxisProcedureIntent": {
        "360271000": ("Prophylaxis - procedure intent (qualifier value)", [])
    },
    "Propofol": {"387423006": ("Propofol", [65, 66, 623, 625])},
    "Propranolol": {"55745002": ("Propranolol", [10])},
    "Propyliodone": {"111158001": ("Propyliodone", [12, 3850])},
    "ProstaglandinE1": {"48988008": ("Prostaglandin E1", [10])},
    "ProstaglandinInjection": {"133875007": ("Prostaglandin injection", [9])},
    "ProstaglandinPGE1": {"48988008": ("Prostaglandin PGE1 (substance)", [])},
    "Prostate": {
        "41216001": (
            "Prostate",
            [
                4,
                645,
                4009,
                4030,
                4031,
                4042,
                6202,
                6204,
                6300,
                7151,
                7160,
                7192,
                8134,
                9514,
                10044,
                10060,
                12321,
            ],
        )
    },
    "ProstateCancer": {"254900004": ("Prostate cancer", [6313])},
    "ProstateCancerAntigenMeasurement": {
        "63476009": ("Prostate Cancer Antigen Measurement", [6321])
    },
    "ProstateSpecificAntigenMeasurement": {
        "63476009": ("Prostate specific antigen measurement (procedure)", [])
    },
    "Prostatectomy": {"90470006": ("Prostatectomy", [6320, 6323])},
    "ProstaticStructure": {"41216001": ("Prostatic structure (body structure)", [])},
    "ProstaticUrethra": {"71553001": ("Prostatic Urethra", [6300])},
    "ProstaticUrethraStructure": {
        "71553001": ("Prostatic urethra structure (body structure)", [])
    },
    "Prostatitis": {"9713002": ("Prostatitis", [6313, 6335, 6336, 6337, 6338])},
    "Prosthesis": {
        "53350007": ("Prosthesis", [6102, 6138, 6202, 6203, 6404, 7151, 7193])
    },
    "ProsthesisDevice": {"53350007": ("Prosthesis, device (physical object)", [])},
    "ProstheticReplacementOfHeartValve": {
        "307279007": ("Prosthetic replacement of heart valve (procedure)", [])
    },
    "ProtamineSulfate": {"64520006": ("Protamine sulfate", [10])},
    "ProtargolSStain": {"406993001": ("protargol S stain", [8112])},
    "ProteaseDigestionOfTissueSpecimen": {
        "433456006": ("Protease digestion of tissue specimen", [8113])
    },
    "Proton": {"89177007": ("Proton", [9526])},
    "ProvençaleGoatBreed": {"131611004": ("Provençale goat breed", [7480])},
    "Proximal": {"40415009": ("Proximal", [2, 5, 211, 212, 3019, 12116, 12281])},
    "ProximalCircumflexCoronaryArtery": {
        "52433000": (
            "Proximal Circumflex Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514, 12292],
        )
    },
    "ProximalFemur": {"310651003": ("Proximal Femur", [7304, 8134])},
    "ProximalHumerus": {"119524001": ("Proximal Humerus", [7304, 8134])},
    "ProximalLeftAnteriorDescendingCoronaryArtery": {
        "68787002": (
            "Proximal Left Anterior Descending Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "ProximalRadius": {"12881000": ("Proximal Radius", [7304, 8134])},
    "ProximalRightCoronaryArtery": {
        "91083009": (
            "Proximal Right Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "ProximalUlna": {"34318004": ("Proximal Ulna", [7304, 8134])},
    "ProximoDistal": {"442626000": ("Proximo-distal", [7484])},
    "ProximoDistalProjection": {
        "442626000": ("Proximo-distal projection (qualifier value)", [])
    },
    "PrussianBlueStain": {"406452004": ("Prussian blue stain", [8112])},
    "PseudoAneurysm": {"22036004": ("Pseudo Aneurysm", [3491])},
    "Pseudoaneurysm": {"22036004": ("Pseudoaneurysm", [3413, 3754, 3810])},
    "Pseudophakia": {"95217000": ("Pseudophakia (disorder)", [])},
    "Psoriasis": {"9014002": ("Psoriasis", [4405])},
    "Psychiatry": {"309958005": ("Psychiatry", [7030])},
    "PsychiatryDepartment": {"309958005": ("Psychiatry department (environment)", [])},
    "Psychology": {"310123008": ("Psychology", [7030])},
    "PsychologyService": {"310123008": ("Psychology service (qualifier value)", [])},
    "Pt0": {"1228951007": ("pT0", [6160])},
    "Pt0NoEvidenceOfPrimaryTumorBreast": {
        "373174002": ("pT0: No evidence of primary tumor (breast) (finding)", [])
    },
    "Pt1": {"1228957006": ("pT1", [6160])},
    "Pt1Tumor2cmOrLessInGreatestDimensionBreast": {
        "373172003": (
            "pT1: Tumor 2 cm or less in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt1a": {"1228959009": ("pT1a", [6160])},
    "Pt1aTumorMoreThan0Point1cmButNotMoreThan0Point5cmInGreatestDimensionBreast": {
        "373180005": (
            "pT1a: Tumor more than 0.1 cm but not more than 0.5 cm in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt1b": {"1228962007": ("pT1b", [6160])},
    "Pt1bTumorMoreThan0Point5cmButNotMoreThan1cmInGreatestDimensionBreast": {
        "373204007": (
            "pT1b: Tumor more than 0.5 cm but not more than 1 cm in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt1c": {"1229846008": ("pT1c", [6160])},
    "Pt1cTumorMoreThan1cmButNotMoreThan2cmInGreatestDimensionBreast": {
        "373183007": (
            "pT1c: Tumor more than 1 cm but not more than 2 cm in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt1mi": {"1228958001": ("pT1mi", [6160])},
    "Pt1micMicroinvasion0Point1cmOrLessInGreatestDimensionBreast": {
        "373179007": (
            "pT1mic: Microinvasion 0.1 cm or less in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt2": {"1229852009": ("pT2", [6160])},
    "Pt2TumorMoreThan2cmButNotMoreThan5cmInGreatestDimensionBreast": {
        "373182002": (
            "pT2: Tumor more than 2 cm but not more than 5 cm in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt3": {"1229859000": ("pT3", [6160])},
    "Pt3TumorMoreThan5cmInGreatestDimensionBreast": {
        "373184001": (
            "pT3: Tumor more than 5 cm in greatest dimension (breast) (finding)",
            [],
        )
    },
    "Pt4": {"1229864001": ("pT4", [6160])},
    "Pt4TumorOfAnySizeWithDirectExtensionToChestWallOrSkinBreast": {
        "373185000": (
            "pT4: Tumor of any size with direct extension to chest wall or skin (breast) (finding)",
            [],
        )
    },
    "Pt4a": {"1229865000": ("pT4a", [6160])},
    "Pt4aTumorOfAnySizeWithExtensionToChestWallNotIncludingPectoralisMuscleBreast": {
        "373186004": (
            "pT4a: Tumor of any size with extension to chest wall, not including pectoralis muscle (breast) "
            "(finding)",
            [],
        )
    },
    "Pt4b": {"1229866004": ("pT4b", [6160])},
    "Pt4bTumorOfAnySizeWithEdemaIncludingPeauDOrangeOrUlcerationOfTheSkinOfTheBreastOrSatelliteSkinNodulesConfinedToTheSameBreastBreast": {
        "373187008": (
            "pT4b: Tumor of any size with edema (including "
            "peau d'orange) or ulceration of the skin of the "
            "breast or satellite skin nodules confined to "
            "the same breast (breast) (finding)",
            [],
        )
    },
    "Pt4c": {"1229867008": ("pT4c", [6160])},
    "Pt4cTumorOfAnySizeWithDirectExtensionToChestWallNotIncludingPectoralisMuscleAndEdemaIncludingPeauDOrangeOrUlcerationOfTheSkinOfTheBreastOrSatelliteSkinNodulesConfinedToTheSameBreast": {
        "373189006": (
            "pT4c: "
            "Tumor "
            "of "
            "any "
            "size "
            "with "
            "direct "
            "extension "
            "to "
            "chest "
            "wall "
            "(not "
            "including "
            "pectoralis "
            "muscle) "
            "and "
            "edema "
            "(including "
            "peau "
            "d'orange) "
            "or "
            "ulceration "
            "of "
            "the "
            "skin "
            "of "
            "the "
            "breast "
            "or "
            "satellite "
            "skin "
            "nodules "
            "confined "
            "to "
            "the "
            "same "
            "breast "
            "(finding)",
            [],
        )
    },
    "Pt4d": {"1229868003": ("pT4d", [6160])},
    "Pt4dInflammatoryCarcinomaBreast": {
        "373190002": ("pT4d: Inflammatory carcinoma (breast) (finding)", [])
    },
    "PtCategoryFinding": {"385385001": ("pT category finding (finding)", [])},
    "Ptis": {"1228953005": ("pTis", [6160])},
    "PtisCarcinomaInSituBreast": {
        "373175001": ("pTis: Carcinoma in situ (breast) (finding)", [])
    },
    "PtisDCIS": {"1228954004": ("pTis (DCIS)", [6160])},
    "PtisDuctalCarcinomaInSituBreast": {
        "373176000": ("pTis: Ductal carcinoma in situ (breast) (finding)", [])
    },
    "PtisLobularCarcinomaInSituBreast": {
        "373177009": ("pTis: Lobular carcinoma in situ (breast) (finding)", [])
    },
    "PtisPaget": {"1228956002": ("pTis (Paget)", [6160])},
    "PtisPagetDiseaseWithoutInvasiveCarcinomaBreast": {
        "373178004": (
            "pTis: Paget disease without invasive carcinoma (breast) (finding)",
            [],
        )
    },
    "Ptx": {"1228950008": ("pTX", [6160])},
    "PtxPrimaryTumorCannotBeAssessedBreast": {
        "373173008": ("pTX: Primary tumor cannot be assessed (breast) (finding)", [])
    },
    "Pudelpointer": {"61286000": ("Pudelpointer", [7480])},
    "PuertoRicanCriolloCattleBreed": {
        "133434006": ("Puerto Rican Criollo cattle breed", [7480])
    },
    "PugDog": {"60252000": ("Pug dog", [7480])},
    "PulMBorCattleBreed": {"133755009": ("Pul-M'bor cattle breed", [7480])},
    "PulawyPigBreed": {"132271004": ("Pulawy pig breed", [7480])},
    "PuliDog": {"21039009": ("Puli dog", [7480])},
    "PullbackMethod": {"128575005": ("Pullback method", [3241])},
    "PulmonaryArterialWaveform": {
        "128443005": ("Pulmonary arterial waveform (observable entity)", [])
    },
    "PulmonaryArteriovenousFistula": {
        "111289009": (
            "Pulmonary arteriovenous fistula",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryArtery": {
        "81040000": (
            "Pulmonary artery",
            [
                4,
                3010,
                3604,
                3606,
                3610,
                3630,
                4030,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12141,
                12290,
                12305,
            ],
        )
    },
    "PulmonaryArteryBlood": {"371953005": ("Pulmonary Artery Blood", [3520])},
    "PulmonaryArteryCatheter": {
        "397755005": ("Pulmonary artery catheter (physical object)", [])
    },
    "PulmonaryArteryConduit": {
        "128584005": (
            "Pulmonary artery conduit",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryArteryOxygenSaturationWaveform": {
        "128447006": ("Pulmonary artery oxygen saturation waveform", [3003])
    },
    "PulmonaryArteryOxygenSaturationWaveformFunction": {
        "128447006": (
            "Pulmonary artery oxygen saturation waveform, function (observable entity)",
            [],
        )
    },
    "PulmonaryArteryPressure": {"250767002": ("Pulmonary Artery Pressure", [3641])},
    "PulmonaryArteryPressureWaveform": {
        "128443005": ("Pulmonary artery pressure waveform", [3003])
    },
    "PulmonaryArteryStructure": {
        "81040000": ("Pulmonary artery structure (body structure)", [])
    },
    "PulmonaryArteryWedge": {"128449009": ("Pulmonary artery wedge", [3608, 3630])},
    "PulmonaryArteryWedgePressure": {
        "118433006": ("Pulmonary artery wedge pressure (observable entity)", [])
    },
    "PulmonaryArteryWedgePressureWaveform": {
        "128449009": ("Pulmonary artery wedge pressure waveform", [3003])
    },
    "PulmonaryAtresiaWithIntactVentricularSeptum": {
        "253590009": (
            "Pulmonary atresia with intact ventricular septum",
            [12248, 12249],
        )
    },
    "PulmonaryAtresiaWithVentricularSeptalDefect": {
        "253591008": (
            "Pulmonary atresia with ventricular septal defect",
            [12248, 12249],
        )
    },
    "PulmonaryCapillaryWedge": {
        "128448001": ("Pulmonary capillary wedge", [3608, 3630])
    },
    "PulmonaryCapillaryWedgeMethod": {
        "128448001": ("Pulmonary capillary wedge method", [3241])
    },
    "PulmonaryCapillaryWedgePressure": {
        "118433006": ("Pulmonary Capillary Wedge Pressure", [3641])
    },
    "PulmonaryCapillaryWedgePressureWaveform": {
        "128448001": ("Pulmonary capillary wedge pressure waveform", [3003])
    },
    "PulmonaryCapillaryWedgePressureWaveformFunction": {
        "128448001": (
            "Pulmonary capillary wedge pressure waveform, function (observable entity)",
            [],
        )
    },
    "PulmonaryChamberInCorTriatriatum": {
        "128586007": ("Pulmonary chamber in cor triatriatum", [3608, 3630])
    },
    "PulmonaryChamberOfCorTriatriatum": {
        "128586007": (
            "Pulmonary chamber of cor triatriatum",
            [4, 3010, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryDisease": {"19829001": ("Pulmonary disease", [3205])},
    "PulmonaryEdema": {"19242006": ("Pulmonary edema", [3700])},
    "PulmonaryEmbolism": {"59282003": ("Pulmonary embolism", [6102, 6104])},
    "PulmonaryHypertension": {
        "70995007": ("Pulmonary hypertension", [64, 3640, 3700, 3728])
    },
    "PulmonaryMetastasis": {"94391008": ("Pulmonary metastasis", [645, 8134])},
    "PulmonarySystemicFlowRatio": {
        "251050008": ("Pulmonary/Systemic Flow Ratio", [3620])
    },
    "PulmonarySystolicEjectionPeriod": {
        "371848004": ("Pulmonary systolic ejection period (observable entity)", [])
    },
    "PulmonarySystolicEjectionPeriodSepp": {
        "371848004": ("Pulmonary Systolic Ejection Period (SEPp)", [3616])
    },
    "PulmonaryToSystemicFlowRatio": {
        "251050008": ("Pulmonary to systemic flow ratio (observable entity)", [])
    },
    "PulmonaryTrunk": {
        "45341000": ("Pulmonary trunk", [6102, 6109, 6117, 8134, 12290])
    },
    "PulmonaryValve": {
        "39057004": (
            "Pulmonary valve",
            [3610, 3630, 3701, 6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryValveArea": {
        "251013007": ("Pulmonary valve area (observable entity)", [])
    },
    "PulmonaryValveDisorder": {"76267008": ("Pulmonary valve disorder (disorder)", [])},
    "PulmonaryValveFlow": {"371846000": ("Pulmonary Valve Flow", [3617])},
    "PulmonaryValvePullbackPressureWaveform": {
        "128452001": ("Pulmonary valve pullback pressure waveform", [3003])
    },
    "PulmonaryValveSinuses": {
        "90315007": ("pulmonary valve sinuses", [3829, 7151, 7192, 9514])
    },
    "PulmonaryValveStructure": {
        "39057004": ("Pulmonary valve structure (body structure)", [])
    },
    "PulmonaryVascularResistance": {
        "276901002": ("Pulmonary Vascular Resistance", [3619, 3641])
    },
    "PulmonaryVein": {
        "122972007": (
            "Pulmonary vein",
            [
                4,
                3010,
                3011,
                3607,
                3630,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12141,
                12283,
            ],
        )
    },
    "PulmonaryVeinAWaveDuration": {
        "399070007": ("Pulmonary Vein A-Wave Duration", [12214])
    },
    "PulmonaryVeinBlood": {"371954004": ("Pulmonary Vein Blood", [3520])},
    "PulmonaryVeinCommonLeftSegment": {
        "443705001": ("Pulmonary Vein Common Left Segment", [12280])
    },
    "PulmonaryVeinCommonRightSegment": {
        "443591004": ("Pulmonary Vein Common Right Segment", [12280])
    },
    "PulmonaryVeinConfluence": {
        "128566008": (
            "Pulmonary vein confluence",
            [4, 3010, 3607, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryVeinConfluenceToAtriumConnection": {
        "443208000": ("Pulmonary Vein confluence to Atrium Connection", [12280])
    },
    "PulmonaryVeinDWaveVelocityTimeIntegral": {
        "399039004": ("Pulmonary Vein D-Wave Velocity Time Integral", [12214])
    },
    "PulmonaryVeinGreatVessel": {
        "430757002": ("Pulmonary Vein", [3827, 3840, 7151, 7192, 9514, 12283, 12305])
    },
    "PulmonaryVeinSWaveVelocityTimeIntegral": {
        "399267001": ("Pulmonary Vein S-Wave Velocity Time Integral", [12214])
    },
    "PulmonaryVeinToAtriumConnection": {
        "443297004": ("Pulmonary Vein to Atrium Connection", [12283])
    },
    "PulmonaryVeinWedge": {"371829003": ("Pulmonary vein wedge", [3604, 3606, 3630])},
    "PulmonaryVeinWedgePressure": {
        "371829003": ("Pulmonary vein wedge pressure (observable entity)", [])
    },
    "PulmonaryVenousAtrium": {
        "128567004": (
            "Pulmonary venous atrium",
            [4, 3010, 3608, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "PulmonaryVenousStructure": {
        "122972007": ("Pulmonary venous structure (body structure)", [])
    },
    "PulmonicRing": {"90318009": ("Pulmonic ring", [3011, 8134, 12305])},
    "PulmonicValve": {"39057004": ("Pulmonic Valve", [12288, 12289, 12305])},
    "PulmonicValveArea": {"251013007": ("Pulmonic Valve Area", [3614, 3615])},
    "PulmonicValveDisease": {"76267008": ("Pulmonic valve disease", [3700])},
    "PulmonicValveStenosis": {"56786000": ("Pulmonic valve stenosis", [12248, 12249])},
    "Pulmonology": {"309918008": ("Pulmonology", [7030])},
    "PulsatilityIndex": {"252068008": ("Pulsatility Index", [3620])},
    "PulsatilityIndexArterialVelocityWaveform": {
        "252068008": (
            "Pulsatility index, arterial velocity waveform (observable entity)",
            [],
        )
    },
    "PumiDogBreed": {"132575007": ("Pumi dog breed", [7480])},
    "PunctateCalcification": {"129755006": ("Punctate calcification", [6010, 6011])},
    "PunctateRadiographicCalcification": {
        "129755006": ("Punctate radiographic calcification (finding)", [])
    },
    "PunganurCattleBreed": {"133756005": ("Punganur cattle breed", [7480])},
    "Pupillograph": {"420827006": ("Pupillograph", [4202])},
    "PurkinjeFibers": {"13050003": ("Purkinje fibers", [3011, 8134])},
    "Purple": {"371250004": ("Purple", [4411])},
    "PurpleColor": {"371250004": ("Purple color (qualifier value)", [])},
    "Putamen": {"89278009": ("Putamen", [7140, 7151, 7153, 7192, 8134, 9514])},
    "PutianPigBreed": {"132284008": ("Putian pig breed", [7480])},
    "PygmyGoat": {"684003": ("Pygmy goat (organism)", [])},
    "PygmyGoatBreed": {"684003": ("Pygmy goat breed", [7480])},
    "PygoraGoatBreed": {"131672003": ("Pygora goat breed", [7480])},
    "PyloricLymphNode": {"24889003": ("pyloric lymph node", [7600, 8134])},
    "PyreneanCattleBreed": {"133435007": ("Pyrenean cattle breed", [7480])},
    "PyreneanGoatBreed": {"131619002": ("Pyrenean goat breed", [7480])},
    "PyreneanMastiffDogBreed": {"132447008": ("Pyrenean Mastiff dog breed", [7480])},
    "PyriformSinus": {"6217003": ("pyriform sinus", [7601, 8134])},
    "PyronineBStain": {"76001002": ("pyronine B stain", [8112])},
    "PyronineGStain": {"43106008": ("pyronine G stain", [8112])},
    "QinchuanCattleBreed": {"133446005": ("Qinchuan cattle breed", [7480])},
    "QingpingPigBreed": {"132313006": ("Qingping pig breed", [7480])},
    "QuadrantectomyOfBreast": {"172049005": ("Quadrantectomy of breast", [6058, 6060])},
    "Quantity": {"246205007": ("Quantity", [9000])},
    "QuarabHorseBreed": {"133159006": ("Quarab horse breed", [7480])},
    "QuarterHorse": {"76467006": ("Quarter horse (organism)", [])},
    "QuarterHorseBreed": {"76467006": ("Quarter horse breed", [7480])},
    "QuarterPonyHorseBreed": {"133158003": ("Quarter Pony horse breed", [7480])},
    "QuasahCattleBreed": {"133823003": ("Quasah cattle breed (organism)", [])},
    "QuasahXZebuCattleBreed": {"133823003": ("Quasah X zebu cattle breed", [7480])},
    "QuinacrineFluorescentStain": {
        "406968003": ("quinacrine fluorescent stain", [8112])
    },
    "Quinidine": {"31306009": ("Quinidine", [10])},
    "QuinoloneAntibiotic": {"372722000": ("Quinolone antibiotic", [621, 622])},
    "QuinoloneClassOfAntibiotic": {
        "372722000": ("Quinolone -class of antibiotic- (substance)", [])
    },
    "RK": {"51683002": ("RK", [4234])},
    "RVCardiacIndex": {"427990004": ("RV Cardiac Index", [12204])},
    "RVCardiacOutput": {"428628004": ("RV Cardiac Output", [12204])},
    "RVStrokeIndex": {"429619008": ("RV Stroke Index", [12204])},
    "RVStrokeVolume": {"429483009": ("RV Stroke Volume", [12204])},
    "RX3CattleBreed": {"133266003": ("RX3 cattle breed", [7480])},
    "RaadPigBreed": {"133171000": ("Raad pig breed", [7480])},
    "RaboLargoSheepBreed": {"132815009": ("Rabo Largo sheep breed", [7480])},
    "RacemoseAneurysm": {"14156004": ("racemose aneurysm", [3808])},
    "RacialGroup": {"415229000": ("Racial group (racial group)", [])},
    "RackaSheepBreed": {"131851004": ("Racka sheep breed", [7480])},
    "RackingHorseHorseBreed": {"133157008": ("Racking Horse horse breed", [7480])},
    "RacloprideC11": {"129512009": ("Raclopride C^11^", [4021])},
    "RadialArtery": {
        "45631007": (
            "Radial artery",
            [
                4,
                3010,
                3440,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12107,
            ],
        )
    },
    "RadialKeratotomy": {"51683002": ("Radial keratotomy (procedure)", [])},
    "RadialScar": {"133855003": ("Radial scar", [6030, 6031])},
    "RadialVein": {
        "52359001": ("Radial vein", [3827, 7151, 7192, 8134, 9514, 12103, 12108])
    },
    "RadiationOncologyANDORRadiotherapy": {
        "108290001": ("Radiation oncology AND/OR radiotherapy (procedure)", [])
    },
    "RadiationTherapist": {
        "3430008": ("Radiation Therapist", [7450, 7452, 9536, 9562])
    },
    "RadiationTherapy": {"108290001": ("Radiation therapy", [6083, 6085])},
    "RadiationTherapyProcedureOrService": {
        "53438000": ("Radiation therapy procedure or service (procedure)", [])
    },
    "RadicalAortopulmonaryReconstructWRVToPAValvelessConduit": {
        "429616001": (
            "Radical aortopulmonary reconstruct w RV to PA valveless conduit",
            [12247],
        )
    },
    "RadioactiveImplant": {
        "19443004": ("Radioactive implant", [7111, 7112, 7151, 7157, 7193])
    },
    "RadioactiveImplantDevice": {
        "19443004": ("Radioactive implant, device (physical object)", [])
    },
    "RadioactiveIsotope": {"89457008": ("Radioactive isotope (substance)", [])},
    "RadiofrequencyAblation": {"879916008": ("Radiofrequency ablation", [4409])},
    "RadiofrequencyAblationDevice": {
        "371791001": ("Radiofrequency Ablation Device", [3411, 3429])
    },
    "RadiofrequencyAblationProcedurePhase": {
        "129089003": ("Radiofrequency Ablation procedure phase", [3254])
    },
    "RadiographicCalcificationFinding": {
        "129748009": ("Radiographic calcification finding", [6055])
    },
    "RadiographicCalcificationWithClusteredDistribution": {
        "129769006": (
            "Radiographic calcification with clustered distribution (finding)",
            [],
        )
    },
    "RadiographicCalcificationWithDiffuseDistribution": {
        "129764001": (
            "Radiographic calcification with diffuse distribution (finding)",
            [],
        )
    },
    "RadiographicCalcificationWithGroupedDistribution": {
        "129766004": (
            "Radiographic calcification with grouped distribution (finding)",
            [],
        )
    },
    "RadiographicCalcificationWithLinearDistribution": {
        "129765000": (
            "Radiographic calcification with linear distribution (finding)",
            [],
        )
    },
    "RadiographicCalcificationWithRegionalDistribution": {
        "129767008": (
            "Radiographic calcification with regional distribution (finding)",
            [],
        )
    },
    "RadiographicCalcificationWithSegmentalDistribution": {
        "129768003": (
            "Radiographic calcification with segmental distribution (finding)",
            [],
        )
    },
    "RadiographicContrastMedia": {
        "7140000": ("Radiographic contrast media (product)", [])
    },
    "RadiographicFindingOfAmorphousCalcification": {
        "129760005": ("Radiographic finding of amorphous calcification (finding)", [])
    },
    "RadiographicFindingOfCalcifiedSkinOfBreast": {
        "129757003": ("Radiographic finding of calcified skin of breast (finding)", [])
    },
    "RadiographicFindingOfCalcifiedSutureMaterial": {
        "129758008": ("Radiographic finding of calcified suture material (finding)", [])
    },
    "RadiographicIndividualCalcification": {
        "129770007": ("Radiographic individual calcification (finding)", [])
    },
    "RadiographicLesionMarginCharacteristics": {
        "129737002": ("Radiographic lesion margin characteristics (finding)", [])
    },
    "RadiographyOfTeeth": {"22891007": ("Radiography of teeth (procedure)", [])},
    "RadioisotopeBrainImaging": {
        "41440006": ("Radioisotope brain imaging (procedure)", [])
    },
    "RadioisotopeScanOfLymphaticSystem": {
        "169167001": ("Radioisotope scan of lymphatic system", [6050, 6083])
    },
    "RadioisotopeScanOfTotalBody": {
        "229033006": ("Radioisotope scan of total body (procedure)", [])
    },
    "RadioisotopeStudyOfEndocrineSystem": {
        "7562007": ("Radioisotope study of endocrine system", [3108])
    },
    "RadioisotopeStudyOfGastrointestinalSystem": {
        "53585008": ("Radioisotope study of gastrointestinal system", [3108])
    },
    "RadioisotopeStudyOfGenitourinarySystem": {
        "76927004": ("Radioisotope study of genitourinary system", [3108])
    },
    "RadioisotopeStudyOfHematopoieticSystem": {
        "41842006": ("Radioisotope study of hematopoietic system", [3108])
    },
    "RadioisotopeStudyOfMusculoskeletalSystem": {
        "68796002": ("Radioisotope study of musculoskeletal system", [3108])
    },
    "RadioisotopeStudyOfRespiratorySystem": {
        "19086005": ("Radioisotope study of respiratory system", [3108])
    },
    "RadiologicTechnologist": {
        "159016003": ("Radiologic Technologist", [7450, 7452, 9536])
    },
    "Radiology": {"309964003": ("Radiology", [7030])},
    "RadiologyDepartment": {"309964003": ("Radiology department (environment)", [])},
    "RadionuclideAngiocardiography": {
        "426940008": ("Radionuclide angiocardiography", [3757])
    },
    "RadionuclideCardiacVentriculography": {
        "85606007": ("Radionuclide cardiac ventriculography (procedure)", [])
    },
    "RadionuclideLocalizationOfTumor": {
        "45316007": ("Radionuclide localization of tumor", [3108])
    },
    "RadionuclideLocalizationOfTumorLimitedArea": {
        "66377006": ("Radionuclide localization of tumor, limited area", [6050, 6083])
    },
    "RadionuclideLymphogram": {
        "169167001": ("Radionuclide lymphogram (procedure)", [])
    },
    "RadionuclideMyocardialPerfusionStudy": {
        "252432008": ("Radionuclide myocardial perfusion study", [3757])
    },
    "RadionuclideStudyForLocalizationOfInflammatoryDisease": {
        "252680004": (
            "Radionuclide study for localization of inflammatory disease",
            [3108],
        )
    },
    "RadionuclideStudyForLocalizationOfTumorOrInflammatoryDisease": {
        "252680004": (
            "Radionuclide study for localization of tumor or inflammatory disease (procedure)",
            [],
        )
    },
    "RadionuclideVentriculography": {
        "85606007": ("Radionuclide ventriculography", [3744])
    },
    "RadiopharmaceuticalAdverseReaction": {
        "292094009": ("Radiopharmaceutical Adverse Reaction", [9300, 9301])
    },
    "Radiopharmaceuticals": {"349358000": ("Radiopharmaceuticals (product)", [])},
    "Radiotherapy": {"309903007": ("Radiotherapy", [7030])},
    "RadiotherapyDepartment": {
        "309903007": ("Radiotherapy department (environment)", [])
    },
    "Radium223Dichloride": {"724024003": ("Radium^223^ dichloride", [25])},
    "Radius": {
        "131190003": (
            "Radius",
            [
                218,
                6165,
                7469,
                7470,
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
            ],
        )
    },
    "RadiusANDUlnaCombinedSite": {
        "110535000": ("radius AND ulna (combined site) (body structure)", [])
    },
    "RadiusAndUlna": {"110535000": ("Radius and ulna", [7482, 7483, 8134])},
    "RadiusBone": {"62413002": ("Radius", [7304, 8134, 12021])},
    "RaferProjection": {"423720000": ("Rafer projection (qualifier value)", [])},
    "Rafert": {"423720000": ("Rafert", [4012])},
    "RafertLong": {"422534007": ("Rafert-Long", [4012])},
    "RafertLongProjection": {
        "422534007": ("Rafert-Long projection (qualifier value)", [])
    },
    "RagamuffinCatBreed": {"132675004": ("Ragamuffin cat breed", [7480])},
    "RagdollCatBreed": {"132654007": ("Ragdoll cat breed", [7480])},
    "RahvanHorseBreed": {"132980005": ("Rahvan horse breed", [7480])},
    "Raloxifene": {"109029006": ("Raloxifene", [6080])},
    "RambouilletSheep": {"2124007": ("Rambouillet sheep (organism)", [])},
    "RambouilletSheepBreed": {"2124007": ("Rambouillet sheep breed", [7480])},
    "RamgarhiCattleBreed": {"133757001": ("Ramgarhi cattle breed", [7480])},
    "RamoGrandeCattleBreed": {"133447001": ("Ramo Grande cattle breed", [7480])},
    "RampProtocol": {"129099008": ("Ramp protocol", [3261, 12001])},
    "RanaCattle": {"133824009": ("Rana cattle (organism)", [])},
    "RanaXZebuCattleBreed": {"133824009": ("Rana X zebu cattle breed", [7480])},
    "RandallLinebackCattleBreed": {
        "133448006": ("Randall Lineback cattle breed", [7480])
    },
    "RangeOfLowerMeasurementUncertainty": {
        "371885007": ("-, range of lower measurement uncertainty", [225])
    },
    "RangeOfMeasurementUncertainty": {
        "371884006": ("+/-, range of measurement uncertainty", [225])
    },
    "RangeOfUpperMeasurementUncertainty": {
        "371886008": ("+, range of upper measurement uncertainty", [225])
    },
    "RangerCattle": {"133825005": ("Ranger cattle (organism)", [])},
    "RangerXZebuCattleBreed": {"133825005": ("Ranger X zebu cattle breed", [7480])},
    "RasNLansaPigBreed": {"132039000": ("Ras-n-Lansa pig breed", [7480])},
    "RasaAragonesaSheepBreed": {"131852006": ("Rasa Aragonesa sheep breed", [7480])},
    "Rash": {"271807003": ("Rash", [6062])},
    "RastelliInterventricularTunnel": {
        "443260009": ("Rastelli Interventricular Tunnel", [12288])
    },
    "RastelliOperation": {"44777001": ("Rastelli operation", [12247])},
    "RastreadorBrasileiroDogBreed": {
        "132448003": ("Rastreador Brasileiro dog breed", [7480])
    },
    "RatTerrierDogBreed": {"132574006": ("Rat Terrier dog breed", [7480])},
    "RathCattleBreed": {"133623001": ("Rath cattle breed", [7480])},
    "RathiCattleBreed": {"133624007": ("Rathi cattle breed", [7480])},
    "Ratio": {"118586006": ("Ratio", [12303])},
    "RatioOfAorticValveAccelerationTimeToAorticValveEjectionTime": {
        "399058008": (
            "Ratio of aortic valve acceleration time to aortic valve ejection time (observable entity)",
            [],
        )
    },
    "RatioOfAorticValveAccelerationTimeToEjectionTime": {
        "399058008": (
            "Ratio of Aortic Valve Acceleration Time to Ejection Time",
            [12211],
        )
    },
    "RatioOfMVPeakVelocityToLVPeakTissueVelocityEWave": {
        "399140004": (
            "Ratio of MV Peak Velocity to LV Peak Tissue Velocity E-Wave",
            [12200, 12203, 12257, 12259, 12271],
        )
    },
    "RatioOfMitralValveAccelerationTimeToMitralValveDecelerationTime": {
        "399062002": (
            "Ratio of mitral valve acceleration time to mitral valve deceleration time (observable entity)",
            [],
        )
    },
    "RatioOfMitralValvePeakVelocityToLeftVentricularPeakTissueVelocityEWave": {
        "399140004": (
            "Ratio of mitral valve peak velocity to left ventricular peak tissue velocity e-wave (observable entity)",
            [],
        )
    },
    "RatioOfPulmonicValveAccelerationTimeToEjectionTime": {
        "399238002": (
            "Ratio of Pulmonic Valve Acceleration Time to Ejection Time",
            [12209],
        )
    },
    "RatioOfPulmonicValveAccelerationTimeToPulmonicValveEjectionTime": {
        "399238002": (
            "Ratio of pulmonic valve acceleration time to pulmonic valve ejection time (observable entity)",
            [],
        )
    },
    "RatioProperty": {"118586006": ("Ratio (property) (qualifier value)", [])},
    "Rattus": {"371564000": ("Rattus", [7454])},
    "RattusNorvegicus": {"371565004": ("Rattus norvegicus", [7454])},
    "RaynaudDisease": {"195295006": ("Raynaud's disease", [3805])},
    "RazorBackPigBreed": {"132051003": ("Razor-Back pig breed", [7480])},
    "ReasonForStoppingTest": {
        "246101005": ("Reason for stopping test (attribute)", [])
    },
    "ReceivingOfSpecimenInLaboratory": {
        "428995007": ("Receiving of specimen in laboratory (procedure)", [])
    },
    "RecentMyocardialInfarction": {
        "428752002": ("Recent Myocardial infarction", [3205])
    },
    "ReconstructionOfBreast": {
        "33496007": ("Reconstruction of breast (procedure)", [])
    },
    "RecoveryRoom": {"398161000": ("Recovery room", [603])},
    "RectalCatheter": {"705541005": ("Rectal Catheter", [74])},
    "RectalMass": {"248523006": ("Rectal mass", [6201])},
    "RectalRoute": {"37161004": ("Rectal route (qualifier value)", [])},
    "RectalTemperature": {"307047009": ("Rectal temperature", [636])},
    "RectalTube": {"67966000": ("Rectal tube", [6202, 6203])},
    "Rectum": {
        "34402009": (
            "Rectum",
            [
                645,
                4009,
                4030,
                4031,
                4040,
                4042,
                6202,
                6204,
                6210,
                6348,
                7151,
                7160,
                7192,
                8134,
                9514,
            ],
        )
    },
    "RectumStructure": {"34402009": ("Rectum structure (body structure)", [])},
    "Recumbent": {"102538003": ("recumbent", [19])},
    "RecumbentBodyPosition": {"102538003": ("Recumbent body position (finding)", [])},
    "RecurrenceOfTumor": {"25173007": ("Recurrence of tumor", [6146])},
    "Recurrent": {"255227004": ("Recurrent", [3770])},
    "RecurrentAnginaPostCABG": {"371810009": ("Recurrent angina Post-CABG", [3700])},
    "RecurrentAnginaPostDCA": {"371812001": ("Recurrent angina Post-DCA", [3700])},
    "RecurrentAnginaPostPTCA": {"371808007": ("Recurrent angina Post-PTCA", [3700])},
    "RecurrentAnginaPostRotationalAtherectomy": {
        "371811008": ("Recurrent angina Post-Rotational Atherectomy", [3700])
    },
    "RecurrentAnginaPostStent": {"371809004": ("Recurrent angina Post-Stent", [3700])},
    "RecurrentAnginaStatusPostCoronaryArteryBypassGraft": {
        "371810009": (
            "Recurrent angina status post coronary artery bypass graft (disorder)",
            [],
        )
    },
    "RecurrentAnginaStatusPostCoronaryStentPlacement": {
        "371809004": (
            "Recurrent angina status post coronary stent placement (disorder)",
            [],
        )
    },
    "RecurrentAnginaStatusPostDirectionalCoronaryAtherectomy": {
        "371812001": (
            "Recurrent angina status post directional coronary atherectomy (disorder)",
            [],
        )
    },
    "RecurrentAnginaStatusPostPercutaneousTransluminalCoronaryAngioplasty": {
        "371808007": (
            "Recurrent angina status post percutaneous transluminal coronary angioplasty (disorder)",
            [],
        )
    },
    "RecurrentAnginaStatusPostRotationalAtherectomy": {
        "371811008": (
            "Recurrent angina status post rotational atherectomy (disorder)",
            [],
        )
    },
    "RecurrentTumor": {"25173007": ("Recurrent tumor (finding)", [])},
    "Red": {"371240000": ("Red", [4206, 4255, 4411, 8122])},
    "RedAndWhiteCattleBreed": {"133258007": ("Red and White cattle breed", [7480])},
    "RedAngusCattleBreed": {"133259004": ("Red Angus cattle breed", [7480])},
    "RedBororoCattleBreed": {"133758006": ("Red Bororo cattle breed", [7480])},
    "RedBrangusCattleBreed": {"133749004": ("Red Brangus cattle breed (organism)", [])},
    "RedBrangusXZebuCattleBreed": {
        "133749004": ("Red Brangus X zebu cattle breed", [7480])
    },
    "RedColor": {"371240000": ("Red color (qualifier value)", [])},
    "RedDesertCattleBreed": {"133759003": ("Red Desert cattle breed", [7480])},
    "RedEngadineSheepBreed": {"131853001": ("Red Engadine sheep breed", [7480])},
    "RedFlemishCattleBreed": {"133386001": ("Red Flemish cattle breed", [7480])},
    "RedFree": {"405739002": ("Red free", [4206])},
    "RedFriesianCattleBreed": {"133586000": ("Red Friesian cattle breed", [7480])},
    "RedGallowayCattleBreed": {"133449003": ("Red Galloway cattle breed", [7480])},
    "RedHampracePigBreed": {"132044007": ("Red Hamprace pig breed", [7480])},
    "RedKandhariCattleBreed": {"133760008": ("Red Kandhari cattle breed", [7480])},
    "RedOpticalFilter": {"445279009": ("Red optical filter", [4204, 8124])},
    "RedOpticalFilterOfOpticalMicroscope": {
        "445279009": ("Red optical filter of optical microscope (physical object)", [])
    },
    "RedPollCattleBreed": {"88807001": ("Red Poll cattle breed (organism)", [])},
    "RedPollCowBreed": {"88807001": ("Red Poll cow breed", [7480])},
    "RedPolledØstlandCattleBreed": {
        "133260009": ("Red Polled Østland cattle breed", [7480])
    },
    "RedSindhiCattleBreed": {"133625008": ("Red Sindhi cattle breed", [7480])},
    "RedSteppeCattleBreed": {"133261008": ("Red Steppe cattle breed", [7480])},
    "RedWattlePig": {"74568001": ("Red wattle pig (organism)", [])},
    "RedWattlePigBreed": {"74568001": ("Red wattle pig breed", [7480])},
    "RedboneCoonhound": {"25171009": ("Redbone coonhound (organism)", [])},
    "RedboneCoonhoundDogBreed": {"25171009": ("Redbone coonhound dog breed", [7480])},
    "ReductionMammoplasty": {"59214008": ("Reduction mammoplasty", [6051, 6083, 6084])},
    "RefractiveError": {"39021009": ("Refractive Error", [4222])},
    "ReggianaCattleBreed": {"133262001": ("Reggiana cattle breed", [7480])},
    "RegionOfMetatarsal": {"280711000": ("Region of metatarsal (body structure)", [])},
    "RegionOfNasopharynx": {
        "360955006": ("Region of nasopharynx (body structure)", [])
    },
    "Regional": {"410674003": ("Regional", [6335, 6336, 6337])},
    "RegionalAnesthesia": {"27372005": ("Regional anesthesia", [611, 612])},
    "RegionalCalcificationDistribution": {
        "129767008": ("Regional calcification distribution", [6012, 6013])
    },
    "RegionalLymphNode": {"312500006": ("regional lymph node", [7600, 8134])},
    "RegionalLymphNodeStructure": {
        "312500006": ("Regional lymph node structure (body structure)", [])
    },
    "Registrar": {"158971006": ("Registrar", [7450, 7452])},
    "RegurgitantFlow": {"397417004": ("Regurgitant Flow", [12221])},
    "RegurgitantFraction": {
        "399301000": (
            "Regurgitant Fraction",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "Regurgitation": {"10337008": ("Regurgitation", [3711])},
    "RegusCattleBreed": {"133450003": ("Regus cattle breed", [7480])},
    "Rehabilitation": {"309940004": ("Rehabilitation", [7030])},
    "RehabilitationDepartment": {
        "309940004": ("Rehabilitation department (environment)", [])
    },
    "RelativeTimeProperty": {
        "118578006": ("Relative time (property) (qualifier value)", [])
    },
    "Remifentanil": {"386839004": ("Remifentanil", [65, 66])},
    "RemovalOfBreastImplant": {"27315000": ("Removal of breast implant", [6083, 6084])},
    "RemovalOfCatheter": {"103715008": ("Removal of catheter", [9])},
    "RemovalOfDevice": {"128538000": ("Removal of device", [8109])},
    "RemovalOfForeignBody": {"10849003": ("Removal of foreign body", [9])},
    "RemovalOfImplantSincePreviousMammogram": {
        "129730000": ("Removal of implant since previous mammogram", [6002, 6003])
    },
    "RenalArtery": {
        "2841007": (
            "Renal artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                3837,
                4030,
                4042,
                7111,
                7151,
                7192,
                8134,
                9514,
                12103,
                12115,
            ],
        )
    },
    "RenalCellCarcinoma": {"702391001": ("Renal cell carcinoma", [638, 639])},
    "RenalFailure": {"42399005": ("Renal failure", [3413, 3755])},
    "RenalFailureSyndrome": {"42399005": ("Renal failure syndrome (disorder)", [])},
    "RenalHilarLymphNode": {"249708006": ("renal hilar lymph node", [7600, 8134])},
    "RenalImpairment": {"236423003": ("Renal impairment (disorder)", [])},
    "RenalPelvis": {"25990002": ("Renal pelvis", [4, 7151, 7192, 8134, 9514])},
    "RenalPelvisStructure": {
        "25990002": ("Renal pelvis structure (body structure)", [])
    },
    "RenalStone": {"386103008": ("Renal stone", [300])},
    "RenalVein": {
        "56400007": (
            "Renal vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12115],
        )
    },
    "RendenaCattleBreed": {"133451004": ("Rendena cattle breed", [7480])},
    "ReniteloCattleBreed": {"133826006": ("Renitelo cattle breed (organism)", [])},
    "ReniteloXZebuCattleBreed": {"133826006": ("Renitelo X zebu cattle breed", [7480])},
    "RepairByNailing": {"35860002": ("Repair by nailing", [7310])},
    "RepairOfAtrialSeptalDefectWithProsthesisByClosedHeartTechnique": {
        "30123000": (
            "Repair of atrial septal defect with prosthesis by closed heart technique (procedure)",
            [],
        )
    },
    "RepairOfCoarctationOfAorta": {
        "274022008": ("Repair of coarctation of aorta", [12247])
    },
    "RepairOfDefectOfTheAtrioventricularSeptum": {
        "174836000": ("Repair of defect of the atrioventricular septum", [12247])
    },
    "RepairOfPartialAnomalousPulmonaryVenousConnection": {
        "174900004": (
            "Repair of partial anomalous pulmonary venous connection (procedure)",
            [],
        )
    },
    "RepairOfTotalAnomalousPulmonaryVenousConnection": {
        "174830006": ("Repair of total anomalous pulmonary venous connection", [12247])
    },
    "RepairOfVentricularSeptalDefectWithProsthesis": {
        "89814007": (
            "Repair of ventricular septal defect with prosthesis (procedure)",
            [],
        )
    },
    "RepartidaGoatBreed": {"131647003": ("Repartida goat breed", [7480])},
    "ReplacementOfCatheter": {"103713001": ("Replacement of catheter (procedure)", [])},
    "Representative": {"371916004": ("Representative", [3676])},
    "RepresentativeNumericEstimationTechnique": {
        "371916004": (
            "Representative - numeric estimation technique (qualifier value)",
            [],
        )
    },
    "RequestByPhysician": {"103321005": ("Request by Physician", [3205])},
    "RequestByPhysicianContextualQualifier": {
        "103321005": (
            "Request by physician (contextual qualifier) (qualifier value)",
            [],
        )
    },
    "ResavaPigBreed": {"133205002": ("Resava pig breed", [7480])},
    "Resident": {"405277009": ("Resident", [7450, 7452, 9536, 9555, 9562])},
    "ResidentPhysician": {"405277009": ("Resident physician (occupation)", [])},
    "ResolutionOfIschemia": {"428824000": ("Resolution of ischemia", [3235])},
    "ResolutionOfMyocardialIschemia": {
        "428824000": ("Resolution of myocardial ischemia (situation)", [])
    },
    "RespirationAssessment": {"364062005": ("Respiration assessment", [3441])},
    "RespirationImpedanceWaveform": {
        "128436004": ("Respiration impedance waveform", [3003])
    },
    "RespirationImpedanceWaveformFunction": {
        "128436004": (
            "Respiration impedance waveform, function (observable entity)",
            [],
        )
    },
    "RespirationIntermittent": {"271824009": ("respiration intermittent", [3416])},
    "RespirationObservable": {
        "364062005": ("Respiration observable (observable entity)", [])
    },
    "RespiratoryCyclePoint": {
        "272517003": ("Respiratory cycle point (qualifier value)", [])
    },
    "RespiratoryMedicineDepartment": {
        "309918008": ("Respiratory medicine department (environment)", [])
    },
    "RespiratoryRate": {"86290005": ("Respiratory rate (observable entity)", [])},
    "RespiratoryStimulant": {"418760000": ("Respiratory stimulant", [621, 622])},
    "Restenotic": {"371893007": ("Restenotic", [3712])},
    "RestenoticLesionOfCoronaryArtery": {
        "371893007": ("Restenotic lesion of coronary artery (finding)", [])
    },
    "RestingState": {
        "128975004": (
            "Resting State",
            [3101, 3102, 3207, 3250, 3262, 3405, 3602, 3651, 10045, 12002],
        )
    },
    "Restrictive": {"253551005": ("Restrictive", [3707])},
    "RestrictiveCardiomyopathy": {
        "90828009": ("Restrictive cardiomyopathy", [3700, 3728])
    },
    "RestrictiveVentricularSeptalDefect": {
        "253551005": ("Restrictive ventricular septal defect (disorder)", [])
    },
    "ResurfacingOfHeadOfFemur": {
        "445185007": ("Resurfacing of head of femur (procedure)", [])
    },
    "ResurfacingOfTheFemoralHead": {
        "445185007": ("Resurfacing of the femoral head", [7301])
    },
    "ResurfacingOfThePatella": {"239503002": ("Resurfacing of the patella", [7301])},
    "Retina": {"5665001": ("Retina", [4029, 4209, 4211, 8134])},
    "RetinalStructure": {"5665001": ("Retinal structure (body structure)", [])},
    "RetinalThicknessAnalyzer": {"416567007": ("Retinal Thickness Analyzer", [4210])},
    "RetintaCattleBreed": {"133263006": ("Retinta cattle breed", [7480])},
    "RetintaExtremenaGoatBreed": {
        "131615008": ("Retinta Extremena goat breed", [7480])
    },
    "RetractionOfNipple": {"31845005": ("Retraction of nipple (disorder)", [])},
    "Retriever": {"1974006": ("Retriever", [7480])},
    "RetrocecalLymphNode": {"281847004": ("retrocecal lymph node", [7600, 8134])},
    "RetrogradeDirection": {"312004007": ("Retrograde Direction", [12306])},
    "RetrogradeFlow": {"312004007": ("Retrograde Flow", [12221])},
    "RetromolarTrigone": {"85816001": ("retromolar trigone", [7601, 8134])},
    "RetroperitonealCompartmentStructure": {
        "82849001": ("Retroperitoneal compartment structure (body structure)", [])
    },
    "RetroperitonealLymphNode": {
        "91394001": ("retroperitoneal lymph node", [7600, 8134])
    },
    "RetroperitonealSpace": {
        "82849001": ("Retroperitoneal space", [7151, 7154, 7192, 8134, 9514])
    },
    "Retroperitoneum": {"82849001": ("Retroperitoneum", [4])},
    "RetropharyngealLymphNode": {
        "25683005": ("retropharyngeal lymph node", [7600, 8134])
    },
    "RetropyloricLymphNode": {"280402004": ("retropyloric lymph node", [7600, 8134])},
    "RetrotrachealLymphNode": {
        "127931000": ("retrotracheal lymph node (mediastinal)", [7600, 8134])
    },
    "RetrotrachealLymphNodeMediastinal": {
        "127931000": ("Retrotracheal lymph node (mediastinal) (body structure)", [])
    },
    "ReversibleMyocardialPerfusionDefect": {
        "251055003": ("Reversible myocardial perfusion defect", [3113])
    },
    "RexCatBreed": {"1809004": ("Rex cat breed", [7480])},
    "RhaetianGrayCattleBreed": {"133462004": ("Rhaetian Gray cattle breed", [7480])},
    "RheinbergFilter": {"445623003": ("Rheinberg filter", [8124])},
    "RheinbergFilterOfOpticalMicroscope": {
        "445623003": ("Rheinberg filter of optical microscope (physical object)", [])
    },
    "Rhenium186re": {"395865006": ("Rhenium[186Re] (substance)", [])},
    "Rhese": {"399234000": ("Rhese", [4012])},
    "RheseProjection": {"399234000": ("Rhese projection (qualifier value)", [])},
    "RheumaticFever": {"58718002": ("Rheumatic Fever", [12248, 12249])},
    "Rheumatology": {"309941000": ("Rheumatology", [7030])},
    "RheumatologyDepartment": {
        "309941000": ("Rheumatology department (environment)", [])
    },
    "RhinelandHeavyDraftHorseBreed": {
        "132965004": ("Rhineland Heavy Draft horse breed", [7480])
    },
    "RhodamineBStain": {"27671009": ("rhodamine B stain", [8112])},
    "RhodamineStain": {"406977005": ("rhodamine stain", [8112])},
    "RhodesianRidgebackDog": {"74173000": ("Rhodesian ridgeback dog", [7480])},
    "Rhodium": {"59801003": ("Rhodium", [10006, 10016, 10067])},
    "RhoenschafSheepBreed": {"131854007": ("Rhoenschaf sheep breed", [7480])},
    "Rib": {
        "113197003": (
            "Rib",
            [4, 4009, 4030, 4031, 4042, 6102, 6109, 6114, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "RideauArcottSheepBreed": {"132703001": ("Rideau Arcott sheep breed", [7480])},
    "Right": {"24028007": ("Right", [2, 5, 211, 244, 245, 247, 3019, 12117])},
    "Right20DegreeDorsalLeftVentralOblique": {
        "442627009": ("Right 20 degree dorsal-left ventral oblique", [7484])
    },
    "Right20DegreeDorsalLeftVentralObliqueProjection": {
        "442627009": (
            "Right 20 degree dorsal-left ventral oblique projection (qualifier value)",
            [],
        )
    },
    "Right20DegreeRostralLeftCaudalOblique": {
        "442743006": ("Right 20 degree rostral-left caudal oblique", [7484])
    },
    "Right20DegreeRostralLeftCaudalObliqueProjection": {
        "442743006": (
            "Right 20 degree rostral-left caudal oblique projection (qualifier value)",
            [],
        )
    },
    "Right20DegreeVentralLeftDorsalOblique": {
        "442612009": ("Right 20 degree ventral-left dorsal oblique", [7484])
    },
    "Right20DegreeVentralLeftDorsalObliqueProjection": {
        "442612009": (
            "Right 20 degree ventral-left dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "Right30DegreeCaudalLeftRostralOblique": {
        "442605000": ("Right 30 degree caudal-left rostral oblique", [7484])
    },
    "Right30DegreeCaudalLeftRostralObliqueProjection": {
        "442605000": (
            "Right 30 degree caudal-left rostral oblique projection (qualifier value)",
            [],
        )
    },
    "Right45DegreeDorsalLeftVentralOblique": {
        "442660007": ("Right 45 degree dorsal-left ventral oblique", [7484])
    },
    "Right45DegreeDorsalLeftVentralObliqueProjection": {
        "442660007": (
            "Right 45 degree dorsal-left ventral oblique projection (qualifier value)",
            [],
        )
    },
    "Right45DegreeVentralLeftDorsalOblique": {
        "442628004": ("Right 45 degree ventral-left dorsal oblique", [7484])
    },
    "Right45DegreeVentralLeftDorsalObliqueProjection": {
        "442628004": (
            "Right 45 degree ventral-left dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "RightAndLeft": {"51440002": ("Right and left (qualifier value)", [])},
    "RightAnteriorApicalPeripheralZoneOfProstate": {
        "716936005": (
            "Right anterior apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightAnteriorApicalTransitionZoneOfProstate": {
        "716930004": (
            "Right anterior apical transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightAnteriorBasalPeripheralZoneOfProstate": {
        "716904003": (
            "Right anterior basal peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightAnteriorBasalTransitionZoneOfProstate": {
        "716896009": (
            "Right anterior basal transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightAnteriorMiddlePeripheralZoneOfProstate": {
        "716919002": (
            "Right anterior middle peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightAnteriorMiddleTransitionZoneOfProstate": {
        "716913001": (
            "Right anterior middle transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightAnteriorOblique": {
        "399356000": ("right anterior oblique", [501, 3466, 4010])
    },
    "RightAnteriorObliqueEmissiveProjection": {
        "399108003": ("Right anterior oblique", [26, 501])
    },
    "RightAnteriorObliqueProjection": {
        "399356000": ("Right anterior oblique projection (qualifier value)", [])
    },
    "RightApicalAnteriorFibromuscularStromaOfProstate": {
        "716926002": (
            "Right apical anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightApicalPeripheralZoneOfProstate": {
        "716934008": ("Right apical peripheral zone of prostate", [6300, 6302])
    },
    "RightApicalTransitionZoneOfProstate": {
        "716928001": ("Right apical transition zone of prostate", [6300, 6302, 6303])
    },
    "RightAtrialPressure": {"276755008": ("Right Atrial Pressure", [3641])},
    "RightAtrialStructure": {
        "73829009": ("Right atrial structure (body structure)", [])
    },
    "RightAtrioventricularOstium": {
        "90561006": ("Right atrioventricular ostium", [3011, 8134])
    },
    "RightAtrioventricularOstiumStructure": {
        "90561006": ("Right atrioventricular ostium structure (body structure)", [])
    },
    "RightAtrium": {
        "73829009": (
            "Right atrium",
            [
                4,
                3010,
                3011,
                3462,
                3608,
                3630,
                4042,
                7151,
                7192,
                8134,
                9514,
                12284,
                12305,
            ],
        )
    },
    "RightAtriumPressureWaveform": {
        "128440008": ("Right atrium pressure waveform", [3003])
    },
    "RightAtriumPressureWaveformFunction": {
        "128440008": (
            "Right atrium pressure waveform, function (observable entity)",
            [],
        )
    },
    "RightAuricularAppendage": {
        "68300000": (
            "Right auricular appendage",
            [4, 3010, 3011, 4042, 7151, 7192, 8134, 9514, 12284],
        )
    },
    "RightBasalAnteriorFibromuscularStromaOfProstate": {
        "716892006": (
            "Right basal anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightBasalPeripheralZoneOfProstate": {
        "716902004": ("Right basal peripheral zone of prostate", [6300, 6302])
    },
    "RightBasalTransitionZoneOfProstate": {
        "716894007": ("Right basal transition zone of prostate", [6300, 6302, 6303])
    },
    "RightBranchOfAtrioventricularBundle": {
        "57383004": ("Right branch of Atrioventricular bundle", [3011, 8134])
    },
    "RightBranchOfHepaticArtery": {
        "69421009": (
            "Right Branch of Hepatic Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "RightBreast": {"73056007": ("Right breast", [6022, 6023])},
    "RightBreastStructure": {
        "73056007": ("Right breast structure (body structure)", [])
    },
    "RightBundleBranchBlock": {"59118001": ("Right bundle branch block", [3230])},
    "RightCaudalLeftRostralOblique": {
        "442629007": ("Right caudal-left rostral oblique", [7484])
    },
    "RightCaudalLeftRostralObliqueProjection": {
        "442629007": (
            "Right caudal-left rostral oblique projection (qualifier value)",
            [],
        )
    },
    "RightCommonCarotidArtery": {"65355003": ("Right Common Carotid Artery", [12291])},
    "RightCommonCarotidArteryStructure": {
        "65355003": ("Right common carotid artery structure (body structure)", [])
    },
    "RightCoronaryArtery": {
        "13647002": (
            "Right Coronary Artery",
            [3015, 3016, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514, 12292],
        )
    },
    "RightCoronaryArteryOstium": {
        "56789007": (
            "Right Coronary Artery Ostium",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "RightCoronaryArteryStructure": {
        "13647002": ("Right coronary artery structure (body structure)", [])
    },
    "RightCoronaryDominance": {"253728007": ("Right Coronary Dominance", [3710])},
    "RightDominantCoronarySystem": {
        "253728007": ("Right dominant coronary system (disorder)", [])
    },
    "RightDorsalLeftVentralOblique": {
        "442607008": ("Right dorsal-left ventral oblique", [7484])
    },
    "RightDorsalLeftVentralObliqueProjection": {
        "442607008": (
            "Right dorsal-left ventral oblique projection (qualifier value)",
            [],
        )
    },
    "RightDowngaze": {"255524005": ("Right downgaze", [4201])},
    "RightFemoralArtery": {
        "69833005": (
            "Right femoral artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "RightGaze": {"255531009": ("Right gaze", [4201])},
    "RightGlennShuntProcedure": {"444001009": ("Right Glenn shunt procedure", [12247])},
    "RightHepaticVein": {
        "272998002": (
            "Right Hepatic Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "RightHypochondriacRegion": {
        "133946002": ("Right hypochondriac region", [4, 7151, 7192, 8134, 9514])
    },
    "RightHypochondriacRegionStructure": {
        "133946002": ("Right hypochondriac region structure (body structure)", [])
    },
    "RightInferiorPulmonaryVein": {
        "113273001": ("Right Inferior Pulmonary Vein", [12280])
    },
    "RightInguinalRegion": {
        "37117007": ("Right inguinal region", [4, 7151, 7192, 8134, 9514])
    },
    "RightInguinalRegionStructure": {
        "37117007": ("Right inguinal region structure (body structure)", [])
    },
    "RightLateral": {"399198007": ("right lateral", [501, 4010])},
    "RightLateralDecubitus": {"102535000": ("right lateral decubitus", [20, 6206])},
    "RightLateralDecubitusPosition": {
        "102535000": ("Right lateral decubitus position (finding)", [])
    },
    "RightLateralEmissiveProjection": {
        "399297009": ("Right lateral projection", [26, 501])
    },
    "RightLateralMiddlePeripheralZoneOfProstate": {
        "716917000": ("Right lateral middle peripheral zone of prostate", [6300, 6302])
    },
    "RightLateralProjection": {
        "399198007": ("Right lateral projection (qualifier value)", [])
    },
    "RightLeftLateral": {"399173006": ("Right-left lateral", [7484])},
    "RightLobeOfThyroid": {"29565003": ("Right lobe of thyroid", [12131])},
    "RightLowerQuadrantOfAbdomen": {
        "48544008": ("Right lower quadrant of abdomen", [4, 7151, 7192, 8134, 9514])
    },
    "RightLowerSegment": {"255496004": ("Right Lower Segment", [12219, 12237])},
    "RightLumbarRegion": {
        "1017211000": ("Right lumbar region", [4, 7151, 7192, 8134, 9514])
    },
    "RightLumbarRegion_Inactive": {
        "133944004": ("Right lumbar region (body structure)", [])
    },
    "RightMainBranchOfPortalVein": {
        "73931004": (
            "Right Main Branch of Portal Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "RightMainBronchus": {
        "70074004": (
            "Right main bronchus",
            [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "RightMainBronchusStructure": {
        "70074004": ("Right main bronchus structure (body structure)", [])
    },
    "RightMiddleAnteriorFibromuscularStromaOfProstate": {
        "716909008": (
            "Right middle anterior fibromuscular stroma of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightMiddleTransitionZoneOfProstate": {
        "716911004": ("Right middle transition zone of prostate", [6300, 6302, 6303])
    },
    "RightOblique": {"399236003": ("right oblique", [501, 4010])},
    "RightObliqueProjection": {
        "399236003": ("Right oblique projection (qualifier value)", [])
    },
    "RightPleura": {"44788007": ("Right Pleura", [12294])},
    "RightPleuraStructure": {
        "44788007": ("Right pleura structure (body structure)", [])
    },
    "RightPosteriorAVCoronaryArtery": {
        "12800002": (
            "Right posterior AV Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "RightPosteriorApicalTransitionZoneOfProstate": {
        "716932007": (
            "Right posterior apical transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightPosteriorBasalTransitionZoneOfProstate": {
        "716898005": (
            "Right posterior basal transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightPosteriorMiddleTransitionZoneOfProstate": {
        "716915008": (
            "Right posterior middle transition zone of prostate",
            [6300, 6301, 6304],
        )
    },
    "RightPosteriorOblique": {"399038007": ("right posterior oblique", [501, 4010])},
    "RightPosteriorObliqueEmissiveProjection": {
        "399075002": ("Right posterior oblique", [26, 501])
    },
    "RightPosteriorObliqueProjection": {
        "399038007": ("Right posterior oblique projection (qualifier value)", [])
    },
    "RightPosterolateralApicalPeripheralZoneOfProstate": {
        "716938006": (
            "Right posterolateral apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightPosterolateralBasalPeripheralZoneOfProstate": {
        "716906001": (
            "Right posterolateral basal peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightPosterolateralMiddlePeripheralZoneOfProstate": {
        "716921007": (
            "Right posterolateral middle peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightPosteromedialApicalPeripheralZoneOfProstate": {
        "716940001": (
            "Right posteromedial apical peripheral zone of prostate",
            [6300, 6301, 6303, 6304],
        )
    },
    "RightPosteromedialBasalPeripheralZoneOfProstate": {
        "836428004": (
            "Right posteromedial basal peripheral zone of prostate",
            [6300, 6304],
        )
    },
    "RightPosteromedialMiddlePeripheralZoneOfProstate": {
        "716923005": (
            "Right posteromedial middle peripheral zone of prostate",
            [6300, 6301, 6302, 6303, 6304],
        )
    },
    "RightPulmonaryArtery": {
        "78480002": (
            "Right pulmonary artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3829,
                4042,
                7151,
                7192,
                8134,
                9514,
                12280,
                12305,
            ],
        )
    },
    "RightPulmonaryVein": {
        "91539005": ("Right Pulmonary Vein", [3827, 3840, 7151, 7192, 9514])
    },
    "RightRostralLeftCaudalOblique": {
        "442747007": ("Right rostral-left caudal oblique", [7484])
    },
    "RightRostralLeftCaudalObliqueProjection": {
        "442747007": (
            "Right rostral-left caudal oblique projection (qualifier value)",
            [],
        )
    },
    "RightSeminalVesicle": {
        "74308000": ("Right seminal vesicle", [6300, 6301, 6302, 6303, 6304])
    },
    "RightSidedBidirectionalGlennShuntProcedure": {
        "444034006": ("Right-sided bidirectional Glenn shunt procedure", [12247])
    },
    "RightSinusOfValsalva": {"89093001": ("Right Sinus of Valsalva", [12291])},
    "RightSubclavianArtery": {"29700009": ("Right Subclavian Artery", [12291])},
    "RightSuperiorPulmonaryVein": {
        "8629005": ("Right Superior Pulmonary Vein", [12280])
    },
    "RightSuperiorVenaCava": {"443444008": ("Right Superior Vena Cava", [12282])},
    "RightToLeftCardiovascularShunt": {
        "79692001": ("Right to left cardiovascular shunt", [12221])
    },
    "RightUpgaze": {"255526007": ("Right upgaze", [4201])},
    "RightUpperQuadrantOfAbdomen": {
        "50519007": ("Right upper quadrant of abdomen", [4, 7151, 7192, 8134, 9514])
    },
    "RightUpperSegment": {"255499006": ("Right Upper Segment", [12219, 12237])},
    "RightVentralLeftDorsalOblique": {
        "442595001": ("Right ventral-left dorsal oblique", [7484])
    },
    "RightVentralLeftDorsalObliqueProjection": {
        "442595001": (
            "Right ventral-left dorsal oblique projection (qualifier value)",
            [],
        )
    },
    "RightVentricle": {
        "53085002": (
            "Right ventricle",
            [
                4,
                3010,
                3011,
                3462,
                3463,
                3609,
                3630,
                4042,
                7151,
                7152,
                7192,
                8134,
                9514,
                12287,
                12288,
                12305,
            ],
        )
    },
    "RightVentricleApex": {"128565007": ("Right ventricle apex", [3609, 3630])},
    "RightVentricleBasalSegment": {
        "277635008": ("Right Ventricle Basal Segment", [12305])
    },
    "RightVentricleInflow": {
        "8017000": (
            "Right ventricle inflow",
            [4, 3010, 3011, 3609, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "RightVentricleMidventricularSegment": {
        "277634007": ("Right Ventricle Midventricular Segment", [12305])
    },
    "RightVentricleOutflowTract": {
        "44627009": (
            "Right ventricle outflow tract",
            [
                4,
                3010,
                3011,
                3609,
                3610,
                3630,
                4042,
                7151,
                7192,
                8134,
                9514,
                12223,
                12236,
                12280,
                12305,
            ],
        )
    },
    "RightVentriclePressureWaveform": {
        "128439006": ("Right ventricle pressure waveform", [3003])
    },
    "RightVentriclePressureWaveformFunction": {
        "128439006": (
            "Right ventricle pressure waveform, function (observable entity)",
            [],
        )
    },
    "RightVentricleToPulmonaryArteryConduitAnastomosis": {
        "443328003": (
            "Right Ventricle to Pulmonary Artery Conduit Anastomosis",
            [12288],
        )
    },
    "RightVentricularCardiacIndex": {
        "427990004": ("Right ventricular cardiac index (observable entity)", [])
    },
    "RightVentricularCardiacOutput": {
        "428628004": ("Right ventricular cardiac output (observable entity)", [])
    },
    "RightVentricularCavitySize": {
        "250964004": ("Right ventricular cavity size (observable entity)", [])
    },
    "RightVentricularEndDiastolicPressure": {
        "276774000": ("Right Ventricular End-Diastolic Pressure", [3641])
    },
    "RightVentricularIndexOfMyocardialPerformance": {
        "399154007": ("Right Ventricular Index of Myocardial Performance", [12204])
    },
    "RightVentricularInflowTractView": {
        "398998003": ("Right Ventricular Inflow Tract View", [501, 12226])
    },
    "RightVentricularOutflowTractView": {
        "399195005": ("Right Ventricular Outflow Tract View", [501, 12226])
    },
    "RightVentricularPeakSystolicPressure": {
        "399023006": ("Right Ventricular Peak Systolic Pressure", [12204])
    },
    "RightVentricularStrokeIndex": {
        "429619008": ("Right ventricular stroke index (observable entity)", [])
    },
    "RightVentricularStrokeVolume": {
        "429483009": ("Right ventricular stroke volume (observable entity)", [])
    },
    "RightVentricularStructure": {
        "53085002": ("Right ventricular structure (body structure)", [])
    },
    "RightVentricularSystolicPressure": {
        "276772001": ("Right Ventricular Systolic Pressure", [3641])
    },
    "RightVentricularWallMotion": {
        "250949004": ("Right ventricular wall motion (observable entity)", [])
    },
    "RightVentriculogram": {"265483003": ("Right ventriculogram (procedure)", [])},
    "RightVentriculography": {"265483003": ("Right Ventriculography", [3405, 3428])},
    "RijnlamASheepBreed": {"132697002": ("Rijnlam-A sheep breed", [7480])},
    "RioLimonDairyCriolloCattleBreed": {
        "133463009": ("Rio Limon Dairy Criollo cattle breed", [7480])
    },
    "RiopardenzeCattleBreed": {
        "133827002": ("Riopardenze cattle breed (organism)", [])
    },
    "RiopardenzeXZebuCattleBreed": {
        "133827002": ("Riopardenze X zebu cattle breed", [7480])
    },
    "RiskFactor": {"80943009": ("Risk factor (observable entity)", [])},
    "RiskFactorsPresentAtHeartDiseaseScreening": {
        "171224000": (
            "Risk factors present at heart disease screening (observable entity)",
            [],
        )
    },
    "Robert": {"425035007": ("Robert", [4012])},
    "RobertProjection": {"425035007": ("Robert projection (qualifier value)", [])},
    "RockAlpineGoat": {"74745008": ("Rock alpine goat (organism)", [])},
    "RockAlpineGoatBreed": {"74745008": ("Rock alpine goat breed", [7480])},
    "RockyMountainHorseHorseBreed": {
        "133156004": ("Rocky Mountain Horse horse breed", [7480])
    },
    "RodopiCattleBreed": {"133464003": ("Rodopi cattle breed", [7480])},
    "RojhanCattleBreed": {"133626009": ("Rojhan cattle breed", [7480])},
    "RolledInferior": {"414493004": ("Rolled Inferior", [4015])},
    "RolledLateral": {"399197002": ("Rolled Lateral", [4015])},
    "RolledMedial": {"399226006": ("Rolled Medial", [4015])},
    "RolledSuperior": {"415670009": ("Rolled Superior", [4015])},
    "RomagnolaCattleBreed": {"131437007": ("Romagnola cattle breed", [7480])},
    "RomagnolaPigBreed": {"132195006": ("Romagnola pig breed", [7480])},
    "RomanianBrownCattleBreed": {"133466001": ("Romanian Brown cattle breed", [7480])},
    "RomanianLargeWhitePigBreed": {
        "133201006": ("Romanian Large White pig breed", [7480])
    },
    "RomanianMeatPigPigBreed": {"133202004": ("Romanian Meat Pig pig breed", [7480])},
    "RomanianNativeBaltaretPigBreed": {
        "133196002": ("Romanian Native, Baltaret pig breed", [7480])
    },
    "RomanianNativeStocliPigBreed": {
        "133195003": ("Romanian Native, Stocli pig breed", [7480])
    },
    "RomanianRedCattleBreed": {"133465002": ("Romanian Red cattle breed", [7480])},
    "RomanianSaddleHorseHorseBreed": {
        "132966003": ("Romanian Saddle Horse horse breed", [7480])
    },
    "RomanianSteppeCattleBreed": {
        "131518000": ("Romanian Steppe cattle breed", [7480])
    },
    "RomanovSheep": {"32145006": ("Romanov sheep (organism)", [])},
    "RomanovSheepBreed": {"32145006": ("Romanov sheep breed", [7480])},
    "RomanowskyStain": {"255810000": ("Romanowsky stain", [8112])},
    "RomboSyndrome": {"721904001": ("Rombo syndrome", [4405])},
    "RomedaleSheep": {"79603002": ("Romedale sheep (organism)", [])},
    "RomedaleSheepBreed": {"79603002": ("Romedale sheep breed", [7480])},
    "RomneletSheep": {"112487006": ("Romnelet sheep (organism)", [])},
    "RomneletSheepBreed": {"112487006": ("Romnelet sheep breed", [7480])},
    "RomneyMarshSheep": {"3099004": ("Romney marsh sheep (organism)", [])},
    "RomneyMarshSheepBreed": {"3099004": ("Romney marsh sheep breed", [7480])},
    "RomneySheepBreed": {"132807006": ("Romney sheep breed", [7480])},
    "RomosinuanoCattleBreed": {"133264000": ("Romosinuano cattle breed", [7480])},
    "RongchangPigBreed": {"132150004": ("Rongchang pig breed", [7480])},
    "RootOfAorta": {"8128003": ("Root of Aorta", [12289, 12291])},
    "Rosacea": {"398909004": ("Rosacea", [4405])},
    "RoseBengal": {"408742009": ("Rose bengal (substance)", [])},
    "RoseBengalContainingProduct": {"330888007": ("Rose Bengal", [4200])},
    "RoseBengalSodiumI131": {"111159009": ("Rose Bengal sodium I^131^", [25])},
    "RoseBengalStain": {"408742009": ("rose bengal stain", [8112])},
    "Rosenberg": {"425042007": ("Rosenberg", [4012])},
    "RosenbergProjection": {
        "425042007": ("Rosenberg projection (qualifier value)", [])
    },
    "RosolicAcidSodiumSaltStain": {
        "15529003": ("rosolic acid sodium salt stain", [8112])
    },
    "RostovPigBreed": {"132104002": ("Rostov pig breed", [7480])},
    "Rostral20DegreeDorsalCaudoventralOblique": {
        "442730006": ("Rostral 20 degree dorsal-caudoventral oblique", [7484])
    },
    "Rostral20DegreeDorsalCaudoventralObliqueProjection": {
        "442730006": (
            "Rostral 20 degree dorsal-caudoventral oblique projection (qualifier value)",
            [],
        )
    },
    "Rostral30DegreeVentralCaudodorsal": {
        "442588005": ("Rostral 30 degree ventral-caudodorsal", [7484])
    },
    "Rostral30DegreeVentralCaudodorsalProjection": {
        "442588005": (
            "Rostral 30 degree ventral-caudodorsal projection (qualifier value)",
            [],
        )
    },
    "Rostrocaudal": {"442690000": ("Rostrocaudal", [7484])},
    "RostrocaudalProjection": {
        "442690000": ("Rostrocaudal projection (qualifier value)", [])
    },
    "RostrodorsalCaudoventralOblique": {
        "442637004": ("Rostrodorsal-caudoventral oblique", [7484])
    },
    "RostrodorsalCaudoventralObliqueProjection": {
        "442637004": (
            "Rostrodorsal-caudoventral oblique projection (qualifier value)",
            [],
        )
    },
    "RostroventralCaudodorsal": {"442586009": ("Rostroventral-caudodorsal", [7484])},
    "RostroventralCaudodorsalProjection": {
        "442586009": ("Rostroventral-caudodorsal projection (qualifier value)", [])
    },
    "RotationAction": {"257912008": ("Rotation - action", [94])},
    "RotationalAtherectomyDevice": {
        "102313007": (
            "Rotational atherectomy device",
            [8, 3411, 3429, 4051, 7151, 7193],
        )
    },
    "RotationalAtherectomySpeed": {
        "371854003": ("Rotational Atherectomy Speed", [3425])
    },
    "RottalHorseBreed": {"132967007": ("Rottal horse breed", [7480])},
    "RottweilerDog": {"14245006": ("Rottweiler dog", [7480])},
    "RougeDeLQuestSheepBreed": {"132789004": ("Rouge de l'Quest sheep breed", [7480])},
    "RoughAndSmoothCollie": {"58341007": ("Rough and smooth collie (organism)", [])},
    "RoughAndSmoothDogBreed": {"58341007": ("Rough and smooth dog breed", [7480])},
    "RoughCollie": {"31377001": ("Rough collie (organism)", [])},
    "RoughCollieDogBreed": {"31377001": ("Rough collie dog breed", [7480])},
    "RoughFellSheepBreed": {"131816007": ("Rough Fell sheep breed", [7480])},
    "Round": {"42700002": ("Round", [6335, 6339, 6340])},
    "RoundShape": {"42700002": ("Round shape", [6004, 6005, 6119])},
    "RoundShapedCalcification": {
        "129756007": ("Round shaped calcification", [6010, 6011])
    },
    "RoundShapedRadiographicCalcification": {
        "129756007": ("Round shaped radiographic calcification (finding)", [])
    },
    "RouteOfAdministration": {"410675002": ("Route of administration (attribute)", [])},
    "Routine": {"373115008": ("Routine", [3676])},
    "RoutineNumericEstimationTechnique": {
        "373115008": ("Routine - numeric estimation technique (qualifier value)", [])
    },
    "RoutineProcedure": {"373113001": ("Routine procedure", [3671])},
    "RoveGoatBreed": {"131639003": ("Rove goat breed", [7480])},
    "RoyaVesubieGoatBreed": {"131614007": ("Roya-Vesubie goat breed", [7480])},
    "RoyalCanadianMountedPoliceHorseHorseBreed": {
        "132968002": ("Royal Canadian Mounted Police Horse horse breed", [7480])
    },
    "RubidiumChlorideRb82": {"129503007": ("Rubidium chloride Rb^82^", [4021])},
    "RuptureOfBreastImplant": {"237473006": ("Rupture of breast implant", [6072])},
    "RupturedAneurysm": {"22039006": ("ruptured aneurysm", [3808])},
    "RupturedSinusOfValsalva": {"21379009": ("Ruptured Sinus of Valsalva", [3709])},
    "RussianBlackPiedCattleBreed": {
        "133265004": ("Russian Black Pied cattle breed", [7480])
    },
    "RussianBlueCat": {"84797007": ("Russian blue cat", [7480])},
    "RussianBrownCattleBreed": {"133467005": ("Russian Brown cattle breed", [7480])},
    "RussianHeavyDraftHorseBreed": {
        "133155000": ("Russian Heavy Draft horse breed", [7480])
    },
    "RussianLargeWhitePigBreed": {
        "132076002": ("Russian Large White pig breed", [7480])
    },
    "RussianLongEaredWhitePigBreed": {
        "132105001": ("Russian Long-Eared White pig breed", [7480])
    },
    "RussianSaddleHorseHorseBreed": {
        "132969005": ("Russian Saddle Horse horse breed", [7480])
    },
    "RussianShortEaredWhitePigBreed": {
        "132106000": ("Russian Short-Eared White pig breed", [7480])
    },
    "RussianTrotterHorseBreed": {"132994006": ("Russian Trotter horse breed", [7480])},
    "RussianWhiteGoatBreed": {"131621007": ("Russian White goat breed", [7480])},
    "RussoLaikaDogBreed": {"132572005": ("Russo-Laika dog breed", [7480])},
    "RustaqiCattleBreed": {"133828007": ("Rustaqi cattle breed (organism)", [])},
    "RustaqiXZebuCattleBreed": {"133828007": ("Rustaqi X zebu cattle breed", [7480])},
    "RutherfordMorrisonPouch": {
        "243977002": ("Rutherford Morrison's pouch (body structure)", [])
    },
    "RyaSheepBreed": {"132787002": ("Rya sheep breed", [7480])},
    "RyelandSheepBreed": {"132706009": ("Ryeland sheep breed", [7480])},
    "RygjaSheepBreed": {"132786006": ("Rygja sheep breed", [7480])},
    "RätienGrayCattleBreed": {"133257002": ("Rätien Gray cattle breed", [7480])},
    "SL96PigBreed": {"133221007": ("SL96 pig breed", [7480])},
    "SPECT": {"105371005": ("SPECT", [3206, 3757])},
    "SPMIPositiveStressForIschemia": {
        "371824008": ("s/p MI positive stress for ischemia", [3700])
    },
    "SPMPigBreed": {"132020005": ("SPM pig breed", [7480])},
    "SRDGoatBreed": {"131640001": ("SRD goat breed", [7480])},
    "STDepression": {"429622005": ("ST Depression (observable entity)", [])},
    "STElevation": {"164931005": ("ST elevation (observable entity)", [])},
    "STElevationMyocardialInfarction": {
        "401303003": ("ST Elevation Myocardial Infarction", [3723])
    },
    "STIntervalNormal": {"164929001": ("ST Interval Normal", [3231])},
    "STSegmentDepression": {"26141007": ("ST depression", [3230, 3233])},
    "STSegmentElevation": {"76388001": ("ST elevation", [3230, 3233])},
    "SaanenGoat": {"21208000": ("Saanen goat (organism)", [])},
    "SaanenGoatBreed": {"21208000": ("Saanen goat breed", [7480])},
    "SableIslandHorseHorseBreed": {
        "132970006": ("Sable Island Horse horse breed", [7480])
    },
    "SabreCattleBreed": {"133829004": ("Sabre cattle breed (organism)", [])},
    "SabreXZebuCattleBreed": {"133829004": ("Sabre X zebu cattle breed", [7480])},
    "SabuesoEspañolDogBreed": {
        "132449006": ("Sabueso Español dog breed (organism)", [])
    },
    "SabuesosEspañolesDogBreed": {
        "132449006": ("Sabuesos Españoles dog breed", [7480])
    },
    "SaccularAneurysm": {"54002007": ("saccular aneurysm", [3808])},
    "SacralLymphNode": {"79926007": ("sacral lymph node", [7600, 8134])},
    "SacroCoccygealSpine": {
        "1217254007": (
            "Sacro-coccygeal Spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "SacrococcygealRegionOfSpineStructure": {
        "297174005": ("Sacrococcygeal region of spine structure (body structure)", [])
    },
    "SacroiliacJoint": {
        "39723000": (
            "Sacroiliac joint",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SacroiliacJointStructure": {
        "39723000": ("Sacroiliac joint structure (body structure)", [])
    },
    "Sacrum": {
        "54735007": (
            "Sacrum",
            [
                4,
                1000,
                1001,
                1005,
                4009,
                4030,
                4031,
                4042,
                7151,
                7192,
                7482,
                7483,
                8134,
                9514,
            ],
        )
    },
    "SaddlebackPigBreed": {"132225006": ("Saddleback pig breed", [7480])},
    "SaddlebredHorseSuperbreed": {
        "427136006": ("Saddlebred horse superbreed (organism)", [])
    },
    "SaddlebredHorseSuperbreedHorseBreed": {
        "427136006": ("Saddlebred horse superbreed horse breed", [7480])
    },
    "SaffronStain": {"38271009": ("saffron stain", [8112])},
    "SafraninStain": {"406988004": ("safranin stain", [8112])},
    "Sagittal": {"30730003": ("Sagittal", [2, 6, 26, 501, 502, 4010, 6312])},
    "SagittalObliqueAxial": {"399325008": ("sagittal-oblique axial", [501, 4010])},
    "SagittalObliqueAxialProjection": {
        "399325008": ("Sagittal-oblique axial projection (qualifier value)", [])
    },
    "SagittalObliqueAxisEmissiveProjection": {
        "399273000": ("Sagittal-oblique axial", [26, 501])
    },
    "SahelTypeSheepBreed": {"132785005": ("Sahel-type sheep breed", [7480])},
    "SahfordCattleBreed": {"133830009": ("Sahford cattle breed (organism)", [])},
    "SahfordXZebuCattleBreed": {"133830009": ("Sahford X zebu cattle breed", [7480])},
    "SahiwalCattleBreed": {"133627000": ("Sahiwal cattle breed", [7480])},
    "SaidiCattleBreed": {"131519008": ("Saidi cattle breed", [7480])},
    "SaintBernardDog": {"83236005": ("Saint Bernard dog (organism)", [])},
    "SaintBernardLongHairedDogBreed": {
        "132394001": ("Saint Bernard, Long-haired dog breed", [7480])
    },
    "SaintBernardShortHairedDogBreed": {
        "132395000": ("Saint Bernard, Short-haired dog breed", [7480])
    },
    "SaintHubertBloodhoundDogBreed": {
        "132373004": ("Saint Hubert bloodhound dog breed (organism)", [])
    },
    "SakhalinWhitePigBreed": {"132113000": ("Sakhalin White pig breed", [7480])},
    "SalernoHorseBreed": {"132981009": ("Salerno horse breed", [7480])},
    "SalersCattleBreed": {"90612002": ("Salers cattle breed (organism)", [])},
    "SalersCowBreed": {"90612002": ("Salers cow breed", [7480])},
    "Saline": {"373757009": ("Saline", [70, 3850])},
    "SalineBagProsthesisForImagingProcedure": {
        "102324009": ("Saline bag prosthesis for imaging procedure", [5])
    },
    "SalineBagProsthesisForImagingProcedureDevice": {
        "102324009": (
            "Saline bag prosthesis for imaging procedure, device (physical object)",
            [],
        )
    },
    "SalineThrombectomy": {"371797002": ("Saline Thrombectomy", [3411, 3429])},
    "SalineThrombectomyDevice": {
        "371797002": ("Saline thrombectomy device (physical object)", [])
    },
    "SalivaryGland": {
        "385294005": ("Salivary gland", [645, 4028, 7151, 7192, 7601, 8134, 9514])
    },
    "SalivaryGlandStructure": {
        "385294005": ("Salivary gland structure (body structure)", [])
    },
    "SalivaryGlands": {"385294005": ("Salivary Glands", [10044, 10060])},
    "SalornCattleBreed": {"133267007": ("Salorn cattle breed", [7480])},
    "SalukiDog": {"59528003": ("Saluki dog", [7480])},
    "SalvageProcedure": {"257950002": ("Salvage Procedure", [3414])},
    "Samarium153": {"419804008": ("Samarium 153 (substance)", [])},
    "SamoyedDog": {"69474004": ("Samoyed dog", [7480])},
    "SamplingForSmear": {"448895004": ("Sampling for smear (procedure)", [])},
    "SamplingOfTissueSpecimen": {"433465004": ("Sampling of tissue specimen", [8111])},
    "SanClementeGoatBreed": {"131636005": ("San Clemente goat breed", [7480])},
    "SanMartineroCattleBreed": {"133269005": ("San Martinero cattle breed", [7480])},
    "SanPierrePig": {"80979001": ("San Pierre pig (organism)", [])},
    "SanPierrePigBreed": {"80979001": ("San Pierre pig breed", [7480])},
    "SandalwoodHorseBreed": {"132982002": ("Sandalwood horse breed", [7480])},
    "SandanHorseBreed": {"132983007": ("Sandan horse breed", [7480])},
    "SanfratelloHorseBreed": {"133030008": ("Sanfratello horse breed", [7480])},
    "SanheCattleBreed": {"131438002": ("Sanhe cattle breed", [7480])},
    "SanheHorseBreed": {"133154001": ("Sanhe horse breed", [7480])},
    "SanjiangWhitePigBreed": {"132121006": ("Sanjiang White pig breed", [7480])},
    "SanoProcedure": {"442123009": ("Sano procedure", [12247])},
    "SantaCruzCattleBreed": {"133750004": ("Santa Cruz cattle breed (organism)", [])},
    "SantaCruzSheepBreed": {"131790005": ("Santa Cruz sheep breed", [7480])},
    "SantaCruzXZebuCattleBreed": {
        "133750004": ("Santa Cruz X zebu cattle breed", [7480])
    },
    "SantaInêsSheepBreed": {"132784009": ("Santa Inês sheep breed", [7480])},
    "SaphenofemoralJunction": {
        "128587003": (
            "Saphenofemoral junction",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "SaphenopoplitealJunction": {
        "244415001": (
            "Saphenopopliteal junction",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "SaphenousVein": {
        "362072009": (
            "Saphenous vein",
            [4, 3010, 3607, 3630, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "SaphenousVeinStructure": {
        "362072009": ("Saphenous vein structure (body structure)", [])
    },
    "SarPlaninaSheepBreed": {"132783003": ("Sar Planina sheep breed", [7480])},
    "SarabiCattleBreed": {"133270006": ("Sarabi cattle breed", [7480])},
    "Sarcoma": {"1187396000": ("Sarcoma", [638, 639])},
    "SarcomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "2424003": (
            "Sarcoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "SarcomaOfSoftTissue": {"424952003": ("Sarcoma of soft tissue", [638, 639])},
    "SardaGoatBreed": {"131626002": ("Sarda goat breed", [7480])},
    "SardinianAngloArabHorseBreed": {
        "133075006": ("Sardinian Anglo-Arab horse breed", [7480])
    },
    "SardinianBrownCattleBreed": {
        "133479005": ("Sardinian brown cattle breed", [7480])
    },
    "SardinianCattleBreed": {"133478002": ("Sardinian cattle breed", [7480])},
    "SardinianPigBreed": {"132257007": ("Sardinian pig breed", [7480])},
    "SardinianPonyHorseBreed": {"133076007": ("Sardinian Pony horse breed", [7480])},
    "SardinianSheepBreed": {"132852009": ("Sardinian sheep breed", [7480])},
    "SardoModicanaCattleBreed": {"131520002": ("Sardo-Modicana cattle breed", [7480])},
    "SarnyPigBreed": {"133227006": ("Sarny pig breed", [7480])},
    "SarplaninacDogBreed": {"132571003": ("Sarplaninac dog breed", [7480])},
    "SarvarHorseBreed": {"133077003": ("Sarvar horse breed", [7480])},
    "SavinjaGrayCattleBreed": {"133480008": ("Savinja Gray cattle breed", [7480])},
    "SayaguesaCattleBreed": {"133481007": ("Sayaguesa cattle breed", [7480])},
    "ScaleneLymphNode": {"81132008": ("scalene lymph node", [7600, 8134])},
    "ScalenousAnteriorMuscle": {
        "50755001": (
            "Scalenous anterior muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "Scalp": {"41695006": ("Scalp", [4, 7151, 7192, 8134, 9514])},
    "ScalpEEG": {"252721009": ("Scalp EEG", [3049])},
    "ScalpStructure": {"41695006": ("Scalp structure (body structure)", [])},
    "ScanningLaserOphthalmoscope": {
        "392001008": ("Scanning Laser Ophthalmoscope", [4202])
    },
    "ScanningLaserPolarimeter": {"392007007": ("Scanning Laser Polarimeter", [4210])},
    "ScanningOrImagingWithVascularFlow": {
        "35202002": ("Scanning or imaging with vascular flow (procedure)", [])
    },
    "Scaphoid": {"30518006": ("Scaphoid", [1000, 1001, 1006])},
    "Scapula": {
        "79601000": (
            "Scapula",
            [
                4,
                1000,
                1001,
                1004,
                4009,
                4030,
                4031,
                4042,
                6102,
                6109,
                6114,
                7151,
                7192,
                8134,
                9514,
            ],
        )
    },
    "Scar": {"12402003": ("Scar (morphologic abnormality)", [])},
    "ScarTissue": {
        "12402003": (
            "Scar tissue",
            [6030, 6031, 6040, 6102, 6138, 6202, 6203, 6403, 6405, 7151, 7191],
        )
    },
    "ScatteredFibroglandularDensities": {
        "129717001": ("Scattered fibroglandular densities", [6000, 6001])
    },
    "SchapendoesDogBreed": {"132570002": ("Schapendoes dog breed", [7480])},
    "SchillerHoundDogBreed": {"132450006": ("Schiller Hound dog breed", [7480])},
    "SchipperkeDog": {"21150005": ("Schipperke dog", [7480])},
    "SchleswigHorseBreed": {"133078008": ("Schleswig horse breed", [7480])},
    "SchnauzerSuperbreed": {"91429002": ("Schnauzer superbreed (organism)", [])},
    "SchnauzerSuperbreedDogBreed": {
        "91429002": ("Schnauzer superbreed dog breed", [7480])
    },
    "SchoonebekerSheepBreed": {"132698007": ("Schoonebeker sheep breed", [7480])},
    "Schuller": {"399290006": ("Schuller", [4012])},
    "SchwarzwalderFuchseHorseBreed": {
        "133079000": ("Schwarzwalder Fuchse horse breed", [7480])
    },
    "SchwerfurtMeatPigBreed": {"132362006": ("Schwerfurt Meat pig breed", [7480])},
    "SchwyzCattleBreed": {"133576001": ("Schwyz cattle breed", [7480])},
    "SchwyzZeboidCattleBreed": {
        "133831008": ("Schwyz-Zeboid cattle breed (organism)", [])
    },
    "SchwyzZeboidXZebuCattleBreed": {
        "133831008": ("Schwyz-Zeboid X zebu cattle breed", [7480])
    },
    "SchüllerProjection": {"399290006": ("Schüller projection (qualifier value)", [])},
    "Sclera": {"18619003": ("Sclera", [4, 4029, 4209, 4211, 7151, 7192, 8134, 9514])},
    "ScleralStructure": {"18619003": ("Scleral structure (body structure)", [])},
    "SclerosingAdenosis": {"50916005": ("Sclerosing adenosis", [6030, 6031])},
    "Score": {"246262008": ("Score (attribute)", [])},
    "ScottishDeerhound": {"54858000": ("Scottish deerhound", [7480])},
    "ScottishFoldCat": {"73049001": ("Scottish fold cat", [7480])},
    "ScottishHighlandCattleBreed": {
        "83173002": ("Scottish Highland cattle breed (organism)", [])
    },
    "ScottishHighlandCowBreed": {"83173002": ("Scottish Highland cow breed", [7480])},
    "ScottishTerrier": {"61405001": ("Scottish terrier", [7480])},
    "Scraping": {"56757003": ("Scraping", [8109])},
    "Screening": {"360156006": ("Screening", [4256, 6051, 6058, 6061])},
    "ScreeningIntent": {"360156006": ("Screening Intent", [3629])},
    "ScreeningProcedureIntent": {
        "360156006": ("Screening - procedure intent (qualifier value)", [])
    },
    "Screw": {"68183006": ("Screw", [7307, 7309])},
    "ScrotalStructure": {"20233005": ("Scrotal structure (body structure)", [])},
    "Scrotum": {"20233005": ("Scrotum", [4, 7151, 7192, 8134, 9514])},
    "ScrubNurse": {"415506007": ("Scrub nurse", [7450, 7452])},
    "SealyhamTerrier": {"34752004": ("Sealyham terrier", [7480])},
    "SebaceousCystOfSkinOfBreast": {
        "76649007": ("Sebaceous cyst of skin of breast", [6054, 6064])
    },
    "SecondDegreeAtrioventricularBlock": {
        "195042002": ("Second degree Atrioventricular block", [3230])
    },
    "SecondIntercostalSpace": {"117134006": ("Second Intercostal Space", [5])},
    "SecondLumbarArtery": {
        "397408004": (
            "Second Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "SecondMolarRegion": {"699505003": ("Second molar region", [4017])},
    "SecondMolarRegionOfOralCavity": {
        "699505003": ("Second molar region of oral cavity (body structure)", [])
    },
    "SecondPremolarRegion": {"699508001": ("Second premolar region", [4017])},
    "SecondPremolarRegionOfOralCavity": {
        "699508001": ("Second premolar region of oral cavity (body structure)", [])
    },
    "SecondaryMalignantNeoplasmOfBone": {
        "94222008": ("Secondary malignant neoplasm of bone (disorder)", [])
    },
    "SecondaryMalignantNeoplasmOfLung": {
        "94391008": ("Secondary malignant neoplasm of lung (disorder)", [])
    },
    "SecondaryMalignantNeoplasticDisease": {
        "128462008": ("Secondary malignant neoplastic disease (disorder)", [])
    },
    "SecondarySTTAbnormality": {"428549008": ("Secondary ST-T abnormality", [3230])},
    "SecondarySTTAbnormalityOnElectrocardiogram": {
        "428549008": ("Secondary ST-T abnormality on electrocardiogram (finding)", [])
    },
    "SecretoryJuvenileCarcinomaOfTheBreast": {
        "41919003": ("Secretory (juvenile) carcinoma of the breast", [6030, 6033])
    },
    "SectioningOfTissueBlock": {
        "434472006": ("Sectioning of tissue block (procedure)", [])
    },
    "Sedation": {"72641008": ("Sedation", [611, 612, 9577])},
    "Sedative": {"372614000": ("Sedative", [76])},
    "SeferihisarCattleBreed": {"133482000": ("Seferihisar cattle breed", [7480])},
    "SegmentOfLung": {
        "72674008": ("Segment of lung", [6102, 6109, 6110, 7151, 7192, 9514])
    },
    "Segmental": {"62372003": ("Segmental", [6128])},
    "SegmentalArtery": {
        "120234003": ("Segmental Artery", [3827, 7151, 7192, 8134, 9514, 12103, 12115])
    },
    "SegmentalCalcificationDistribution": {
        "129768003": ("Segmental calcification distribution", [6012, 6013])
    },
    "SegmentalLymphNodeOfTheLung": {
        "127921007": ("segmental lymph node of the lung", [7600, 8134])
    },
    "SegmentectomyBreastSample": {
        "309547008": ("segmentectomy breast sample", [8103, 8104])
    },
    "SelectiveEmbolizationOfArtery": {
        "57238002": ("Selective embolization of artery", [9, 3405])
    },
    "SelectiveFetalReduction": {
        "133874006": ("Selective fetal reduction (procedure)", [])
    },
    "Selenium75HCAT": {"395894004": ("Selenium^75^ HCAT", [25])},
    "SelenomethionineSe75": {"88473009": ("Selenomethionine Se^75^ (substance)", [])},
    "SelenomethionioneSe75": {"88473009": ("Selenomethionione Se^75^", [25])},
    "SelkirkRexCatBreed": {"132676003": ("Selkirk Rex cat breed", [7480])},
    "SellaTurcica": {
        "42575006": (
            "Sella turcica",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SelleFrancaisHorseBreed": {"133153007": ("Selle Francais horse breed", [7480])},
    "SemiErect": {"102539006": ("semi-erect", [19])},
    "SemiErectBodyPosition": {"102539006": ("Semi-erect body position (finding)", [])},
    "SemiProne": {"34026001": ("semi-prone", [20])},
    "SeminalVesicle": {
        "64739004": (
            "Seminal Vesicle",
            [6202, 6204, 6300, 6348, 7151, 7160, 7192, 8134, 9514],
        )
    },
    "SeminalVesicleStructure": {
        "64739004": ("Seminal vesicle structure (body structure)", [])
    },
    "SemipronePosition": {"34026001": ("Semiprone position (finding)", [])},
    "SemirechenskPigBreed": {"132119001": ("Semirechensk pig breed", [7480])},
    "SenegusCattleBreed": {"133587009": ("Senegus cattle breed", [7480])},
    "SenepolCattleBreed": {"132781001": ("Senepol cattle breed", [7480])},
    "SengologaCattleBreed": {"131521003": ("Sengologa cattle breed", [7480])},
    "SenneHorseBreed": {"133080002": ("Senne horse breed", [7480])},
    "SensationOfBeingWarm": {
        "724232004": ("Sensation of being warm (finding)", [60, 9300])
    },
    "SentinelLymphNodeBiopsy": {
        "396487001": ("Sentinel lymph node biopsy", [6050, 6083])
    },
    "Sepsis": {"91302008": ("Sepsis", [1201])},
    "SeptalArtery": {"244251006": ("Septal Artery", [12292])},
    "SerereCattleBreed": {"131522005": ("Serere cattle breed", [7480])},
    "Seroma": {"56021002": ("Seroma", [6030, 6031])},
    "SerpentinaGoatBreed": {"131627006": ("Serpentina goat breed", [7480])},
    "SerpentineAneurysm": {"70984001": ("serpentine aneurysm", [3808])},
    "SerranaGoatBreed": {"131628001": ("Serrana goat breed", [7480])},
    "SerratusAnteriorMuscle": {
        "18346003": (
            "Serratus anterior muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "SerumAlanineAminotransferaseLevelRaised": {
        "409673008": ("Serum alanine aminotransferase level raised", [12325])
    },
    "SerumAlbuminBelowReferenceRange": {
        "115347700": ("Serum albumin below reference range", [12325])
    },
    "SerumBilirubinRaised": {"166612004": ("Serum bilirubin raised", [12325])},
    "SesamoidBonesOfFoot": {
        "58742003": (
            "Sesamoid bones of foot",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SeshagaCattleBreed": {"131523000": ("Seshaga cattle breed", [7480])},
    "Sessile": {"5712003": ("Sessile", [6209])},
    "Settegast": {"399243009": ("Settegast", [4012])},
    "SettegastProjection": {
        "399243009": ("Settegast projection (qualifier value)", [])
    },
    "Setter": {"37453003": ("Setter", [7480])},
    "SeventhIntercostalSpace": {"117139001": ("Seventh Intercostal Space", [5])},
    "Severe": {"24484000": ("Severe", [3716, 3832, 6130])},
    "SevereHypokinesis": {"371870001": ("Severe Hypokinesis", [3703])},
    "SevereHypokinesisOfCardiacWall": {
        "371870001": ("Severe hypokinesis of cardiac wall (finding)", [])
    },
    "SevereSeverityModifier": {
        "24484000": ("Severe (severity modifier) (qualifier value)", [])
    },
    "SevereThrombus": {"373143004": ("Severe Thrombus", [3714])},
    "SeverelyDisabled": {"161045001": ("Severely disabled", [3772])},
    "SeverelyElevated": {"260360000": ("Severely Elevated", [3642])},
    "SeverelyIll": {"271593001": ("Severely ill", [3772])},
    "Severity": {"246112005": ("Severity (attribute)", [])},
    "Sevoflurane": {"386842005": ("Sevoflurane", [623, 624])},
    "ShaftOfFemur": {"41111004": ("Shaft of Femur", [7304, 8134])},
    "ShaftOfHumerus": {"20760004": ("Shaft of Humerus", [7304, 8134])},
    "ShaftOfRadius": {"47728000": ("Shaft of Radius", [7304, 8134])},
    "ShaftOfRib": {"41601005": ("Shaft of rib", [6115, 8134])},
    "ShaftOfTibia": {"52687003": ("Shaft of Tibia", [7304, 8134])},
    "ShaftOfUlna": {"21133008": ("Shaft of Ulna", [7304, 8134])},
    "ShagyaHorseBreed": {"133152002": ("Shagya horse breed", [7480])},
    "ShahutouPigBreed": {"132320004": ("Shahutou pig breed", [7480])},
    "ShakhansurriCattleBreed": {"133761007": ("Shakhansurri cattle breed", [7480])},
    "Shaking": {"26079004": ("Shaking", [60, 9300])},
    "ShallowBreathing": {"386616007": ("shallow breathing", [3823, 9272])},
    "ShanHorseBreed": {"133081003": ("Shan horse breed", [7480])},
    "ShanghaiWhitePigBreed": {"132322007": ("Shanghai White pig breed", [7480])},
    "ShanxiBlackPigBreed": {"132335003": ("Shanxi Black pig breed", [7480])},
    "Shape": {"300842002": ("Shape", [6333, 6334])},
    "ShapeFinding": {"107644003": ("Shape finding (qualifier value)", [])},
    "SharPeiDogBreed": {"132589002": ("Shar-pei dog breed", [7480])},
    "SharabiCattleBreed": {"133271005": ("Sharabi cattle breed", [7480])},
    "Sheath": {"371793003": ("Sheath", [3429])},
    "SheathContraception": {"268461001": ("Sheath contraception (finding)", [])},
    "SheepBreedUndeterminedSheepBreed": {
        "131832005": ("Sheep, Breed Undetermined sheep breed", [7480])
    },
    "ShekoCattleBreed": {"133762000": ("Sheko cattle breed", [7480])},
    "ShendiCattleBreed": {"133769009": ("Shendi cattle breed", [7480])},
    "ShengxianSpottedPigBreed": {"132312001": ("Shengxian Spotted pig breed", [7480])},
    "ShetlandCattleBreed": {"133272003": ("Shetland cattle breed", [7480])},
    "ShetlandPony": {"69067004": ("Shetland pony (organism)", [])},
    "ShetlandPonyHorseBreed": {"69067004": ("Shetland pony horse breed", [7480])},
    "ShetlandSheepBreed": {"131801009": ("Shetland sheep breed", [7480])},
    "ShetlandSheepdog": {"50125003": ("Shetland sheepdog", [7480])},
    "ShibaInuDogBreed": {"132587000": ("Shiba Inu dog breed", [7480])},
    "ShieldingBlock": {
        "228739009": ("Shielding Block", [9517, 9520, 9541, 9545, 10066])
    },
    "ShihTzuDog": {"31077009": ("Shih tzu dog", [7480])},
    "ShillukCattleBreed": {"132782008": ("Shilluk cattle breed", [7480])},
    "ShilohShepherdDogBreed": {"132586009": ("Shiloh Shepherd dog breed", [7480])},
    "ShireHorse": {"13487004": ("Shire horse (organism)", [])},
    "ShireHorseBreed": {"13487004": ("Shire horse breed", [7480])},
    "ShkodraPigBreed": {"133210003": ("Shkodra pig breed", [7480])},
    "ShkodraRedCattleBreed": {"133483005": ("Shkodra Red cattle breed", [7480])},
    "Short": {"367450005": ("short", [3831])},
    "ShortAxis": {
        "103340004": ("Short axis", [6, 26, 27, 218, 501, 502, 6165, 6166, 7469, 7470])
    },
    "ShortCoatChihuahua": {"15966002": ("Short coat chihuahua (organism)", [])},
    "ShortCoatChihuahuaDogBreed": {
        "15966002": ("Short coat chihuahua dog breed", [7480])
    },
    "ShorthairCatBreed": {"132665002": ("Shorthair cat breed", [7480])},
    "ShorthornCattleBreed": {"80835003": ("Shorthorn cattle breed (organism)", [])},
    "ShorthornCowBreed": {"80835003": ("Shorthorn cow breed", [7480])},
    "Shoulder": {
        "16982005": (
            "Shoulder",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "ShoulderGirdleStructure": {
        "26444007": ("Shoulder girdle structure (body structure)", [])
    },
    "ShoulderRegionStructure": {
        "16982005": ("Shoulder region structure", [1000, 1001, 1004])
    },
    "ShropshireSheep": {"4574003": ("Shropshire sheep (organism)", [])},
    "ShropshireSheepBreed": {"4574003": ("Shropshire sheep breed", [7480])},
    "ShropshireTerrierDogBreed": {
        "132579001": ("Shropshire Terrier dog breed", [7480])
    },
    "ShuwaCattleBreed": {"133770005": ("Shuwa cattle breed", [7480])},
    "SiameseCat": {"65694005": ("Siamese cat", [7480])},
    "SiamesePigBreed": {"133168008": ("Siamese pig breed", [7480])},
    "SiberianBlackPiedCattleBreed": {
        "131524006": ("Siberian Black Pied cattle breed", [7480])
    },
    "SiberianBlackPiedPigBreed": {
        "132115007": ("Siberian Black Pied pig breed", [7480])
    },
    "SiberianCatBreed": {"132677007": ("Siberian cat breed", [7480])},
    "SiberianHuskie": {"3674001": ("Siberian huskie", [7480])},
    "SiberianPigBreed": {"132196007": ("Siberian pig breed", [7480])},
    "SiboneyCattleBreed": {"133751000": ("Siboney cattle breed (organism)", [])},
    "SiboneyXZebuCattleBreed": {"133751000": ("Siboney X zebu cattle breed", [7480])},
    "SicilianBarbarySheepBreed": {
        "131767000": ("Sicilian Barbary sheep breed", [7480])
    },
    "SicilianPigBreed": {"132258002": ("Sicilian pig breed", [7480])},
    "SienaBeltedPigBreed": {"132098006": ("Siena Belted pig breed", [7480])},
    "SigmoidColon": {
        "60184004": ("Sigmoid colon", [4, 4040, 6210, 7151, 7192, 8134, 9514])
    },
    "SigmoidColonStructure": {
        "60184004": ("Sigmoid colon structure (body structure)", [])
    },
    "SigmoidLymphNode": {"30024008": ("sigmoid lymph node", [7600, 8134])},
    "SignetRingCellCarcinoma": {
        "87737001": ("Signet ring cell carcinoma", [6030, 6033])
    },
    "SignificanceUndetermined": {"386135008": ("Significance Undetermined", [220])},
    "Significant": {"386134007": ("Significant", [220])},
    "SignificantCoronaryBypassGraftDisease": {
        "371805005": ("Significant coronary bypass graft disease", [3728])
    },
    "SilentIschemia": {"233823002": ("Silent ischemia", [3700])},
    "SilentMyocardialIschemia": {
        "233823002": ("Silent myocardial ischemia (disorder)", [])
    },
    "SilesianHorseBreed": {"133082005": ("Silesian horse breed", [7480])},
    "Silicon": {"51420009": ("Silicon", [300])},
    "Silicone": {"13652007": ("Silicone", [300])},
    "SiliconeGelFilledBreastImplant": {
        "465380004": ("Silicone gel-filled breast implant (physical object)", [])
    },
    "SiliconeGelImplant": {"465380004": ("Silicone gel implant", [6058, 6059])},
    "SiliconeOil": {"247095003": ("Silicone Oil", [4232])},
    "SilkyTerrier": {"39882003": ("Silky terrier", [7480])},
    "Silver": {"41967008": ("Silver", [300, 10006, 10067])},
    "SilverNitrateStain": {"406985001": ("silver nitrate stain", [8112])},
    "SilverStain": {"406951002": ("silver stain", [8112])},
    "SimbrahCattleBreed": {"133273008": ("Simbrah cattle breed", [7480])},
    "SimmentalCattleBreed": {"28483003": ("Simmental cattle breed (organism)", [])},
    "SimmentalCowBreed": {"28483003": ("Simmental cow breed", [7480])},
    "SinclairMiniaturePigBreed": {
        "132224005": ("Sinclair Miniature pig breed", [7480])
    },
    "SingapuraCat": {"10136006": ("Singapura cat", [7480])},
    "SingleBeats": {"371871002": ("Single Beats", [3676])},
    "SingleBeatsNumericEstimationTechnique": {
        "371871002": (
            "Single beats - numeric estimation technique (qualifier value)",
            [],
        )
    },
    "SingleCoronaryVesselDisease": {
        "194842008": ("Single coronary vessel disease (disorder)", [])
    },
    "SingleEvent": {"307486002": ("Single event", [6091])},
    "SingleFootingHorseHorseBreed": {
        "133160001": ("Single-Footing Horse horse breed", [7480])
    },
    "SinglePhotonEmissionComputerizedTomography": {
        "105371005": ("Single photon emission computerized tomography (procedure)", [])
    },
    "SingleVesselCoronaryArteryDisease": {
        "194842008": ("Single vessel coronary artery disease.", [3728])
    },
    "SinhalaCattleBreed": {"133771009": ("Sinhala cattle breed", [7480])},
    "SiniHorseBreed": {"133083000": ("Sini horse breed", [7480])},
    "SinoAtrialNode": {"88210001": ("Sino-atrial node", [3011, 8134])},
    "SinoGasconyPigBreed": {"132031002": ("Sino-Gascony pig breed", [7480])},
    "SinoVietnamesePigBreed": {"133163004": ("Sino-Vietnamese pig breed", [7480])},
    "SinusNodeRecoveryPhase": {
        "129086005": ("Sinus node recovery phase (qualifier value)", [])
    },
    "SinusNodeRecoveryTimeEvaluationOf": {
        "129086005": ("Sinus Node Recovery Time, evaluation of", [3254])
    },
    "SiriZebuCattleBreed": {"133628005": ("Siri zebu cattle breed", [7480])},
    "SiriusRedF3BStain": {"51567006": ("sirius red F3B stain", [8112])},
    "SiskaPigBreed": {"132261001": ("Siska pig breed", [7480])},
    "SistaniCattleBreed": {"133772002": ("Sistani cattle breed", [7480])},
    "SiteOf": {"272737002": ("Site of (attribute)", [])},
    "Sitting": {"33586001": ("sitting", [20])},
    "SittingPosition": {"33586001": ("Sitting position (finding)", [])},
    "SixthIntercostalSpace": {"117138009": ("Sixth Intercostal Space", [5])},
    "SixthLumbarArtery": {
        "397412005": (
            "Sixth Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "SkeletalAndOrSmoothMuscleStructure": {
        "71616004": ("Skeletal and/or smooth muscle structure (body structure)", [])
    },
    "SkeletalMuscle": {"127954009": ("Skeletal muscle", [7710, 8134])},
    "SkeletalMuscleRelaxant": {"372666009": ("Skeletal muscle relaxant", [621, 622])},
    "SkeletalMuscleStructure": {
        "127954009": ("Skeletal muscle structure (body structure)", [])
    },
    "SkeletalMuscleStructureOfChestWall": {
        "372074006": ("Skeletal muscle structure of chest wall (body structure)", [])
    },
    "SkeletalMuscleStructureOfHead": {
        "22688005": ("Skeletal muscle structure of head (body structure)", [])
    },
    "SkeletalMuscleStructureOfLowerLimb": {
        "102292000": ("Skeletal muscle structure of lower limb (body structure)", [])
    },
    "SkeletalMuscleStructureOfNeck": {
        "81727001": ("Skeletal muscle structure of neck (body structure)", [])
    },
    "SkeletalMuscleStructureOfUpperLimb": {
        "30608006": ("Skeletal muscle structure of upper limb (body structure)", [])
    },
    "Skin": {"39937001": ("Skin", [645, 4029, 7151, 7166, 7191, 8134, 10044, 10060])},
    "SkinAcheSyndrome": {"403598008": ("Skin-ache syndrome (finding)", [])},
    "SkinCareTopicalTreatmentsRegimeTherapy": {
        "386439008": ("Skin care: topical treatments (regime/therapy)", [])
    },
    "SkinConditionAssessment": {"364528001": ("Skin condition assessment", [3441])},
    "SkinLesion": {"95324001": ("Skin lesion", [6014, 6015, 6016, 6054, 6056])},
    "SkinObservable": {"364528001": ("Skin observable (observable entity)", [])},
    "SkinOfAbdomen": {
        "75093004": ("Skin of abdomen", [4029, 7151, 7154, 7192, 8134, 9514])
    },
    "SkinOfAlaNasi": {"68598004": ("Skin of ala nasi", [4029, 8134])},
    "SkinOfAntecubitalFossa": {"17957002": ("Skin of antecubital fossa", [4029, 8134])},
    "SkinOfAnteriorPortionOfNeck": {
        "11584001": ("Skin of anterior portion of neck", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfForearm": {
        "70559009": ("Skin of anterior surface of forearm", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfKnee": {
        "181553006": ("Skin of anterior surface of knee", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfLowerLeg": {
        "25763004": ("Skin of anterior surface of lower leg", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfThigh": {
        "61248009": ("Skin of anterior surface of thigh", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfThorax": {
        "244106003": ("Skin of anterior surface of thorax", [4029, 8134])
    },
    "SkinOfAnteriorSurfaceOfUpperArm": {
        "45981001": ("Skin of anterior surface of upper arm", [4029, 8134])
    },
    "SkinOfAnteriorTrunk": {"181491009": ("Skin of anterior trunk", [4029, 8134])},
    "SkinOfAntitragus": {"38407007": ("Skin of antitragus", [4029, 8134])},
    "SkinOfAnus": {"59112000": ("Skin of anus", [4029, 8134])},
    "SkinOfAreola": {"72005009": ("Skin of areola", [4029, 8134])},
    "SkinOfAxilla": {"76261009": ("Skin of axilla", [4029, 8134])},
    "SkinOfBack": {"66643007": ("Skin of back", [4029, 8134])},
    "SkinOfBackOfUpperThoracicRegion": {
        "699893008": ("Skin of back of upper thoracic region", [4029, 8134])
    },
    "SkinOfButtock": {"22180002": ("Skin of buttock", [4029, 8134])},
    "SkinOfCavityOfConcha": {"51098001": ("Skin of cavity of concha", [4029, 8134])},
    "SkinOfCheek": {"36141000": ("Skin of cheek", [4029, 8134])},
    "SkinOfChest": {"74160004": ("Skin of chest", [7151, 7155, 7192, 8134, 9514])},
    "SkinOfChin": {"23747009": ("Skin of chin", [4029, 8134])},
    "SkinOfClitoris": {"29353003": ("Skin of clitoris", [4029, 8134])},
    "SkinOfCrusOfHelix": {"57726007": ("Skin of crus of helix", [4029, 8134])},
    "SkinOfDigitOfHand": {"244169007": ("Skin of digit of hand", [4029, 8134])},
    "SkinOfDorsalAreaOfWrist": {
        "52876008": ("Skin of dorsal area of wrist", [4029, 8134])
    },
    "SkinOfEar": {"1902009": ("Skin of ear", [4029, 8134])},
    "SkinOfEarLobule": {"2059009": ("Skin of ear lobule", [4029, 8134])},
    "SkinOfEpigastricArea": {"30598005": ("Skin of epigastric area", [4029, 8134])},
    "SkinOfExternalAuditoryCanal": {
        "86409001": ("Skin of external auditory canal", [4029, 8134])
    },
    "SkinOfExternalGenitalia": {
        "60944009": ("Skin of external genitalia", [4029, 8134])
    },
    "SkinOfEyeRegion": {"362916000": ("Skin of eye region", [4029, 8134])},
    "SkinOfEyebrow": {"367577003": ("Skin of eyebrow", [4029, 8134])},
    "SkinOfFace": {"73897004": ("Skin of face", [4029, 8134])},
    "SkinOfFingersAndThumb": {
        "244169007": ("Skin of fingers and thumb (body structure)", [])
    },
    "SkinOfFoot": {"60496002": ("Skin of foot", [4029, 8134])},
    "SkinOfForehead": {"68698007": ("Skin of forehead", [4029, 8134])},
    "SkinOfGlansPenis": {"7991003": ("Skin of glans penis", [4029, 8134])},
    "SkinOfGlutealFold": {"63029009": ("Skin of gluteal fold", [4029, 8134])},
    "SkinOfHand": {"33712006": ("Skin of hand", [4029, 8134])},
    "SkinOfHead": {"70762009": ("Skin of head", [4029, 8134])},
    "SkinOfHeel": {"84607009": ("Skin of heel", [4029, 8134])},
    "SkinOfHelixOfEar": {"79313003": ("Skin of helix of ear", [4029, 8134])},
    "SkinOfHypogastricRegion": {
        "367578008": ("Skin of hypogastric region", [4029, 8134])
    },
    "SkinOfHypothenarRegionOfPalm": {
        "89784008": ("Skin of hypothenar region of palm", [4029, 8134])
    },
    "SkinOfInfraclavicularRegion": {
        "66288003": ("Skin of infraclavicular region", [4029, 8134])
    },
    "SkinOfInguinalRegion": {"39687006": ("Skin of inguinal region", [4029, 8134])},
    "SkinOfIntertragalIncisure": {
        "45591000": ("Skin of intertragal incisure", [4029, 8134])
    },
    "SkinOfJawline": {"244097004": ("Skin of jawline", [4029, 8134])},
    "SkinOfLabium": {"73058008": ("Skin of labium", [4029, 8134])},
    "SkinOfLabiumMajus": {"128252004": ("Skin of labium majus", [4029, 8134])},
    "SkinOfLabiumMinus": {"128253009": ("Skin of labium minus", [4029, 8134])},
    "SkinOfLateralAspectOfAnkle": {
        "181564009": ("Skin of lateral aspect of ankle", [4029, 8134])
    },
    "SkinOfLateralBorderOfSoleOfFoot": {
        "35739000": ("Skin of lateral border of sole of foot", [4029, 8134])
    },
    "SkinOfLateralCanthus": {"37671003": ("Skin of lateral canthus", [4029, 8134])},
    "SkinOfLateralNoseStructure": {
        "314395006": ("Skin of lateral nose structure (body structure)", [])
    },
    "SkinOfLateralPartOfHeel": {
        "699909001": ("Skin of lateral part of heel", [4029, 8134])
    },
    "SkinOfLateralPortionOfNeck": {
        "5272005": ("Skin of lateral portion of neck", [4029, 8134])
    },
    "SkinOfLip": {"88089004": ("Skin of lip", [4029, 8134])},
    "SkinOfLowerAbdomen": {"699914002": ("Skin of lower abdomen", [4029, 8134])},
    "SkinOfLowerBack": {"113182001": ("Skin of lower back", [4029, 8134])},
    "SkinOfLowerChestWall": {"699915001": ("Skin of lower chest wall", [4029, 8134])},
    "SkinOfLowerExtremity": {"371304004": ("Skin of lower extremity", [4029, 8134])},
    "SkinOfLowerEyelid": {"40069000": ("Skin of lower eyelid", [4029, 8134])},
    "SkinOfLowerLip": {"66934001": ("Skin of lower lip", [4029, 8134])},
    "SkinOfMedialAspectOfAnkle": {
        "181563003": ("Skin of medial aspect of ankle", [4029, 8134])
    },
    "SkinOfMedialBorderOfSoleOfFoot": {
        "52953006": ("Skin of medial border of sole of foot", [4029, 8134])
    },
    "SkinOfMedialCanthus": {"27887005": ("Skin of medial canthus", [4029, 8134])},
    "SkinOfMedialPartOfHeel": {
        "699919007": ("Skin of medial part of heel", [4029, 8134])
    },
    "SkinOfMedialSurfaceOfThigh": {
        "73958006": ("Skin of medial surface of thigh", [4029, 8134])
    },
    "SkinOfNasolabialFold": {"37108007": ("Skin of nasolabial fold", [4029, 8134])},
    "SkinOfNeck": {"43081002": ("Skin of neck", [4029, 8134])},
    "SkinOfNipple": {"54468004": ("Skin of nipple", [4029, 8134])},
    "SkinOfNose": {"113179006": ("Skin of nose", [4029, 8134])},
    "SkinOfNuchalRegion": {"4658004": ("Skin of nuchal region", [4029, 8134])},
    "SkinOfOccipitalRegion": {"79951008": ("Skin of occipital region", [4029, 8134])},
    "SkinOfPalmOfHand": {"70887009": ("Skin of palm of hand", [4029, 8134])},
    "SkinOfPalmarAreaOfWrist": {
        "24527008": ("Skin of palmar area of wrist", [4029, 8134])
    },
    "SkinOfParietalRegion": {"21672008": ("Skin of parietal region", [4029, 8134])},
    "SkinOfPartOfDorsalSurfaceOfHand": {
        "281642007": ("Skin of part of dorsal surface of hand", [4029, 8134])
    },
    "SkinOfPenis": {"35900000": ("Skin of penis", [4029, 8134])},
    "SkinOfPerineum": {"48014002": ("Skin of perineum", [4029, 8134])},
    "SkinOfPerioralRegionOfFace": {
        "110488009": ("Skin of perioral region of face", [4029, 8134])
    },
    "SkinOfPhiltrum": {"84365009": ("Skin of philtrum", [4029, 8134])},
    "SkinOfPoplitealFossa": {"84507004": ("Skin of popliteal fossa", [4029, 8134])},
    "SkinOfPostauricularRegion": {
        "24483006": ("Skin of postauricular region", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfElbow": {
        "181536004": ("Skin of posterior surface of elbow", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfForearm": {
        "41550009": ("Skin of posterior surface of forearm", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfLowerLeg": {
        "47224004": ("Skin of posterior surface of lower leg", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfThigh": {
        "4578000": ("Skin of posterior surface of thigh", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfThorax": {
        "244111001": ("Skin of posterior surface of thorax", [4029, 8134])
    },
    "SkinOfPosteriorSurfaceOfUpperArm": {
        "72939005": ("Skin of posterior surface of upper arm", [4029, 8134])
    },
    "SkinOfPreauricularRegion": {
        "86719006": ("Skin of preauricular region", [4029, 8134])
    },
    "SkinOfPrepuceOfClitoris": {
        "76723005": ("Skin of prepuce of clitoris", [4029, 8134])
    },
    "SkinOfRootOfPenis": {"244117002": ("Skin of root of penis", [4029, 8134])},
    "SkinOfScalp": {"43067004": ("Skin of scalp", [4029, 8134])},
    "SkinOfScrotum": {"81992007": ("Skin of scrotum", [4029, 8134])},
    "SkinOfShaftOfPenis": {"244118007": ("Skin of shaft of penis", [4029, 8134])},
    "SkinOfSideOfNose": {"314395006": ("Skin of side of nose", [4029, 8134])},
    "SkinOfSideOfNose_Inactive": {
        "244089006": ("Skin of side of nose (body structure)", [])
    },
    "SkinOfStructureOfLabiumPudendum": {
        "73058008": ("Skin of structure of labium pudendum (body structure)", [])
    },
    "SkinOfSubmentalArea": {"34926004": ("Skin of submental area", [4029, 8134])},
    "SkinOfSupraclavicularRegionOfNeck": {
        "76072005": ("Skin of supraclavicular region of neck", [4029, 8134])
    },
    "SkinOfTemporalRegion": {"16621002": ("Skin of temporal region", [4029, 8134])},
    "SkinOfThenarRegionOfPalm": {
        "26795005": ("Skin of thenar region of palm", [4029, 8134])
    },
    "SkinOfTipOfNose": {"79283007": ("Skin of tip of nose", [4029, 8134])},
    "SkinOfToe": {"52034004": ("Skin of toe", [4029, 8134])},
    "SkinOfTragus": {"79502000": ("Skin of tragus", [4029, 8134])},
    "SkinOfUmbilicus": {"315003": ("Skin of umbilicus", [4029, 8134])},
    "SkinOfUpperAbdomen": {"699935000": ("Skin of upper abdomen", [4029, 8134])},
    "SkinOfUpperExtremity": {"371311000": ("Skin of upper extremity", [4029, 8134])},
    "SkinOfUpperEyelid": {"41310005": ("Skin of upper eyelid", [4029, 8134])},
    "SkinOfUpperLip": {"16251004": ("Skin of upper lip", [4029, 8134])},
    "SkinOfUpperTrunk": {"54440003": ("Skin of upper trunk", [4029, 8134])},
    "SkinOfVertex": {"61719002": ("Skin of vertex (body structure)", [])},
    "SkinOfVertexOfScalp": {"61719002": ("Skin of vertex of scalp", [4029, 8134])},
    "SkinRetractionOfBreast": {
        "129796009": ("Skin retraction of breast", [6014, 6015, 6016, 6054, 6055, 6056])
    },
    "SkinStructure": {"39937001": ("Skin structure (body structure)", [])},
    "SkinStructureOfAbdomen": {
        "75093004": ("Skin structure of abdomen (body structure)", [])
    },
    "SkinStructureOfAlaNasi": {
        "68598004": ("Skin structure of ala nasi (body structure)", [])
    },
    "SkinStructureOfAntecubitalFossa": {
        "17957002": ("Skin structure of antecubital fossa (body structure)", [])
    },
    "SkinStructureOfAnteriorPortionOfNeck": {
        "11584001": ("Skin structure of anterior portion of neck (body structure)", [])
    },
    "SkinStructureOfAnteriorSurfaceOfForearm": {
        "70559009": (
            "Skin structure of anterior surface of forearm (body structure)",
            [],
        )
    },
    "SkinStructureOfAnteriorSurfaceOfThigh": {
        "61248009": ("Skin structure of anterior surface of thigh (body structure)", [])
    },
    "SkinStructureOfAnteriorSurfaceOfUpperArm": {
        "45981001": (
            "Skin structure of anterior surface of upper arm (body structure)",
            [],
        )
    },
    "SkinStructureOfAnteriorTrunk": {
        "181491009": ("Skin structure of anterior trunk (body structure)", [])
    },
    "SkinStructureOfAntitragus": {
        "38407007": ("Skin structure of antitragus (body structure)", [])
    },
    "SkinStructureOfAnus": {
        "59112000": ("Skin structure of anus (body structure)", [])
    },
    "SkinStructureOfAreola": {
        "72005009": ("Skin structure of areola (body structure)", [])
    },
    "SkinStructureOfAxilla": {
        "76261009": ("Skin structure of axilla (body structure)", [])
    },
    "SkinStructureOfAxillaryFossa": {
        "699891005": ("Skin structure of axillary fossa (body structure)", [])
    },
    "SkinStructureOfBack": {
        "66643007": ("Skin structure of back (body structure)", [])
    },
    "SkinStructureOfBackOfUpperThoracicRegion": {
        "699893008": (
            "Skin structure of back of upper thoracic region (body structure)",
            [],
        )
    },
    "SkinStructureOfButtock": {
        "22180002": ("Skin structure of buttock (body structure)", [])
    },
    "SkinStructureOfCavityOfConcha": {
        "51098001": ("Skin structure of cavity of concha (body structure)", [])
    },
    "SkinStructureOfCheek": {
        "36141000": ("Skin structure of cheek (body structure)", [])
    },
    "SkinStructureOfChest": {
        "74160004": ("Skin structure of chest (body structure)", [])
    },
    "SkinStructureOfChin": {
        "23747009": ("Skin structure of chin (body structure)", [])
    },
    "SkinStructureOfClitoris": {
        "29353003": ("Skin structure of clitoris (body structure)", [])
    },
    "SkinStructureOfCruroVulvarFold": {
        "87706001": ("Skin structure of cruro-vulvar fold (body structure)", [])
    },
    "SkinStructureOfCrusOfHelix": {
        "57726007": ("Skin structure of crus of helix (body structure)", [])
    },
    "SkinStructureOfDorsalAreaOfWrist": {
        "52876008": ("Skin structure of dorsal area of wrist (body structure)", [])
    },
    "SkinStructureOfEar": {"1902009": ("Skin structure of ear (body structure)", [])},
    "SkinStructureOfEarLobule": {
        "2059009": ("Skin structure of ear lobule (body structure)", [])
    },
    "SkinStructureOfEpigastricArea": {
        "30598005": ("Skin structure of epigastric area (body structure)", [])
    },
    "SkinStructureOfExternalGenitalia": {
        "60944009": ("Skin structure of external genitalia (body structure)", [])
    },
    "SkinStructureOfEyebrow": {
        "367577003": ("Skin structure of eyebrow (body structure)", [])
    },
    "SkinStructureOfFace": {
        "73897004": ("Skin structure of face (body structure)", [])
    },
    "SkinStructureOfFoot": {
        "60496002": ("Skin structure of foot (body structure)", [])
    },
    "SkinStructureOfForehead": {
        "68698007": ("Skin structure of forehead (body structure)", [])
    },
    "SkinStructureOfFrontalRegionOfScalp": {
        "699900002": ("Skin structure of frontal region of scalp (body structure)", [])
    },
    "SkinStructureOfGlansPenis": {
        "7991003": ("Skin structure of glans penis (body structure)", [])
    },
    "SkinStructureOfGlutealFold": {
        "63029009": ("Skin structure of gluteal fold (body structure)", [])
    },
    "SkinStructureOfHand": {
        "33712006": ("Skin structure of hand (body structure)", [])
    },
    "SkinStructureOfHead": {
        "70762009": ("Skin structure of head (body structure)", [])
    },
    "SkinStructureOfHeel": {
        "84607009": ("Skin structure of heel (body structure)", [])
    },
    "SkinStructureOfHelix": {
        "79313003": ("Skin structure of helix (body structure)", [])
    },
    "SkinStructureOfHypogastricRegion": {
        "367578008": ("Skin structure of hypogastric region (body structure)", [])
    },
    "SkinStructureOfHypothenarRegionOfPalm": {
        "89784008": ("Skin structure of hypothenar region of palm (body structure)", [])
    },
    "SkinStructureOfInfraclavicularRegion": {
        "66288003": ("Skin structure of infraclavicular region (body structure)", [])
    },
    "SkinStructureOfInguinalRegion": {
        "39687006": ("Skin structure of inguinal region (body structure)", [])
    },
    "SkinStructureOfIntertragalIncisure": {
        "45591000": ("Skin structure of intertragal incisure (body structure)", [])
    },
    "SkinStructureOfLabiumMajus": {
        "128252004": ("Skin structure of labium majus (body structure)", [])
    },
    "SkinStructureOfLabiumMinus": {
        "128253009": ("Skin structure of labium minus (body structure)", [])
    },
    "SkinStructureOfLateralBorderOfSoleOfFoot": {
        "35739000": (
            "Skin structure of lateral border of sole of foot (body structure)",
            [],
        )
    },
    "SkinStructureOfLateralCanthus": {
        "37671003": ("Skin structure of lateral canthus (body structure)", [])
    },
    "SkinStructureOfLateralPartOfHeel": {
        "699909001": ("Skin structure of lateral part of heel (body structure)", [])
    },
    "SkinStructureOfLateralPortionOfNeck": {
        "5272005": ("Skin structure of lateral portion of neck (body structure)", [])
    },
    "SkinStructureOfLip": {"88089004": ("Skin structure of lip (body structure)", [])},
    "SkinStructureOfLowerAbdomen": {
        "699914002": ("Skin structure of lower abdomen (body structure)", [])
    },
    "SkinStructureOfLowerBack": {
        "113182001": ("Skin structure of lower back (body structure)", [])
    },
    "SkinStructureOfLowerChestWall": {
        "699915001": ("Skin structure of lower chest wall (body structure)", [])
    },
    "SkinStructureOfLowerEyelid": {
        "40069000": ("Skin structure of lower eyelid (body structure)", [])
    },
    "SkinStructureOfLowerLimb": {
        "371304004": ("Skin structure of lower limb (body structure)", [])
    },
    "SkinStructureOfLowerLip": {
        "66934001": ("Skin structure of lower lip (body structure)", [])
    },
    "SkinStructureOfMedialBorderOfSoleOfFoot": {
        "52953006": (
            "Skin structure of medial border of sole of foot (body structure)",
            [],
        )
    },
    "SkinStructureOfMedialCanthus": {
        "27887005": ("Skin structure of medial canthus (body structure)", [])
    },
    "SkinStructureOfMedialPartOfHeel": {
        "699919007": ("Skin structure of medial part of heel (body structure)", [])
    },
    "SkinStructureOfMedialSurfaceOfThigh": {
        "73958006": ("Skin structure of medial surface of thigh (body structure)", [])
    },
    "SkinStructureOfNasolabialFold": {
        "37108007": ("Skin structure of nasolabial fold (body structure)", [])
    },
    "SkinStructureOfNeck": {
        "43081002": ("Skin structure of neck (body structure)", [])
    },
    "SkinStructureOfNipple": {
        "54468004": ("Skin structure of nipple (body structure)", [])
    },
    "SkinStructureOfNose": {
        "113179006": ("Skin structure of nose (body structure)", [])
    },
    "SkinStructureOfNuchalRegion": {
        "4658004": ("Skin structure of nuchal region (body structure)", [])
    },
    "SkinStructureOfPalmarAreaOfHand": {
        "70887009": ("Skin structure of palmar area of hand (body structure)", [])
    },
    "SkinStructureOfPalmarAreaOfWrist": {
        "24527008": ("Skin structure of palmar area of wrist (body structure)", [])
    },
    "SkinStructureOfPerineum": {
        "48014002": ("Skin structure of perineum (body structure)", [])
    },
    "SkinStructureOfPhiltrum": {
        "84365009": ("Skin structure of philtrum (body structure)", [])
    },
    "SkinStructureOfPoplitealFossa": {
        "84507004": ("Skin structure of popliteal fossa (body structure)", [])
    },
    "SkinStructureOfPostauricularRegion": {
        "24483006": ("Skin structure of postauricular region (body structure)", [])
    },
    "SkinStructureOfPosteriorSurfaceOfForearm": {
        "41550009": (
            "Skin structure of posterior surface of forearm (body structure)",
            [],
        )
    },
    "SkinStructureOfPosteriorSurfaceOfThigh": {
        "4578000": ("Skin structure of posterior surface of thigh (body structure)", [])
    },
    "SkinStructureOfPosteriorSurfaceOfUpperArm": {
        "72939005": (
            "Skin structure of posterior surface of upper arm (body structure)",
            [],
        )
    },
    "SkinStructureOfPreauricularRegion": {
        "86719006": ("Skin structure of preauricular region (body structure)", [])
    },
    "SkinStructureOfPrepuceOfClitoris": {
        "76723005": ("Skin structure of prepuce of clitoris (body structure)", [])
    },
    "SkinStructureOfScalp": {
        "43067004": ("Skin structure of scalp (body structure)", [])
    },
    "SkinStructureOfScrotum": {
        "81992007": ("Skin structure of scrotum (body structure)", [])
    },
    "SkinStructureOfSupraclavicularRegionOfNeck": {
        "76072005": (
            "Skin structure of supraclavicular region of neck (body structure)",
            [],
        )
    },
    "SkinStructureOfTemporalRegion": {
        "16621002": ("Skin structure of temporal region (body structure)", [])
    },
    "SkinStructureOfThenarRegionOfPalm": {
        "26795005": ("Skin structure of thenar region of palm (body structure)", [])
    },
    "SkinStructureOfTipOfNose": {
        "79283007": ("Skin structure of tip of nose (body structure)", [])
    },
    "SkinStructureOfToe": {"52034004": ("Skin structure of toe (body structure)", [])},
    "SkinStructureOfTragus": {
        "79502000": ("Skin structure of tragus (body structure)", [])
    },
    "SkinStructureOfUmbilicus": {
        "315003": ("Skin structure of umbilicus (body structure)", [])
    },
    "SkinStructureOfUpperAbdomen": {
        "699935000": ("Skin structure of upper abdomen (body structure)", [])
    },
    "SkinStructureOfUpperEyelid": {
        "41310005": ("Skin structure of upper eyelid (body structure)", [])
    },
    "SkinStructureOfUpperLimb": {
        "371311000": ("Skin structure of upper limb (body structure)", [])
    },
    "SkinStructureOfUpperLip": {
        "16251004": ("Skin structure of upper lip (body structure)", [])
    },
    "SkinStructureOfUpperTrunk": {
        "54440003": ("Skin structure of upper trunk (body structure)", [])
    },
    "SkinThickeningOfBreast": {
        "129797000": ("Skin thickening of breast", [6014, 6015, 6016, 6054, 6055, 6056])
    },
    "SkuddeSheepBreed": {"131799006": ("Skudde sheep breed", [7480])},
    "Skull": {
        "89546000": (
            "Skull",
            [
                4,
                1000,
                1001,
                1002,
                4009,
                4028,
                4030,
                4031,
                4042,
                7151,
                7192,
                7304,
                7482,
                7483,
                8134,
                9514,
                12020,
            ],
        )
    },
    "SkyeTerrier": {"24299002": ("Skye terrier", [7480])},
    "SkyrosHorseBreed": {"133084006": ("Skyros horse breed", [7480])},
    "SkyrosPonyBreed": {"423926000": ("Skyros pony breed (organism)", [])},
    "SkyrosPonyHorseBreed": {"423926000": ("Skyros pony horse breed", [7480])},
    "SleepEEG": {"18648009": ("Sleep EEG", [3049])},
    "Slide": {"1179252003": ("Slide", [8103])},
    "SlideOverviewLens": {"445601002": ("Slide overview lens", [8121])},
    "Slide_Inactive": {"258661006": ("Slide (specimen)", [])},
    "SlitLampBiomicroscope": {"397247004": ("Slit Lamp Biomicroscope", [4202])},
    "SloughiDogBreed": {"132590006": ("Sloughi dog breed", [7480])},
    "SlovakCuvakDogBreed": {"132585008": ("Slovak Cuvak dog breed", [7480])},
    "SlovakWarmbloodHorseBreed": {
        "133085007": ("Slovak Warmblood horse breed", [7480])
    },
    "SlovakianBlackPiedPigBreed": {
        "133214007": ("Slovakian Black Pied pig breed", [7480])
    },
    "SlovakianPiedCattleBreed": {"133494009": ("Slovakian Pied cattle breed", [7480])},
    "SlovakianPinzgauCattleBreed": {
        "133495005": ("Slovakian Pinzgau cattle breed", [7480])
    },
    "SlovakianWhitePigBreed": {"133217000": ("Slovakian White pig breed", [7480])},
    "SlovenianBrownCattleBreed": {
        "133496006": ("Slovenian Brown cattle breed", [7480])
    },
    "SlovenianWhitePigBreed": {"133211004": ("Slovenian White pig breed", [7480])},
    "Slovhyb1PigBreed": {"133218005": ("Slovhyb-1 pig breed", [7480])},
    "Small": {"255507004": ("Small", [252, 6118])},
    "SmallCardiacVein": {
        "49082002": ("Small Cardiac Vein", [3827, 3839, 7151, 7192, 9514])
    },
    "SmallCellCarcinoma": {"74364000": ("Small cell carcinoma", [638, 639])},
    "SmallEastAfricanZebuCattleBreed": {
        "133773007": ("Small East African Zebu cattle breed", [7480])
    },
    "SmallIntestinalStructure": {
        "30315005": ("Small intestinal structure (body structure)", [])
    },
    "SmallIntestine": {
        "30315005": (
            "Small intestine",
            [4, 4009, 4030, 4031, 4042, 7151, 7154, 7192, 8134, 9514, 10044, 10060],
        )
    },
    "SmallMünsterländerDogBreed": {
        "132584007": ("Small Münsterländer dog breed", [7480])
    },
    "SmallPolishPrickEaredPigBreed": {
        "133224004": ("Small Polish Prick-Eared pig breed", [7480])
    },
    "SmallSpanishHoundDogBreed": {
        "132583001": ("Small Spanish Hound dog breed", [7480])
    },
    "SmallWhitePigBreed": {"132197003": ("Small White pig breed", [7480])},
    "SmallestCardiacVein": {
        "195073003": ("Smallest Cardiac Vein", [3827, 3839, 7151, 7192, 9514])
    },
    "SmearProcedure": {"448895004": ("Smear procedure", [8110])},
    "SmearSample": {"258433009": ("Smear sample", [8103])},
    "SmithfieldDogBreed": {"132465007": ("Smithfield dog breed", [7480])},
    "Smoker": {"77176002": ("Smoker (finding)", [])},
    "Smooth": {"82280004": ("Smooth", [3715])},
    "SmoothCollie": {"10544000": ("Smooth collie (organism)", [])},
    "SmoothCollieDogBreed": {"10544000": ("Smooth collie dog breed", [7480])},
    "SmoothFoxTerrier": {"8351009": ("Smooth fox terrier (organism)", [])},
    "SmoothFoxTerrierDogBreed": {"8351009": ("Smooth fox terrier dog breed", [7480])},
    "SmoothHairedVizsla": {"90444005": ("Smooth haired vizsla (organism)", [])},
    "SmoothHairedVizslaDogBreed": {
        "90444005": ("Smooth haired vizsla dog breed", [7480])
    },
    "SmoothMiniatureDachshund": {
        "112493003": ("Smooth miniature dachshund (organism)", [])
    },
    "SmoothMiniatureDachshundDogBreed": {
        "112493003": ("Smooth miniature dachshund dog breed", [7480])
    },
    "SmoothStandardDachshund": {
        "69862006": ("Smooth standard dachshund (organism)", [])
    },
    "SmoothStandardDachshundDogBreed": {
        "69862006": ("Smooth standard dachshund dog breed", [7480])
    },
    "SnowshoeCatBreed": {"132678002": ("Snowshoe cat breed", [7480])},
    "SoaySheepBreed": {"132790008": ("Soay sheep breed", [7480])},
    "SocialPersonalHistoryObservable": {
        "160476009": ("Social / personal history observable (observable entity)", [])
    },
    "SocotraCattleBreed": {"131525007": ("Socotra cattle breed", [7480])},
    "SodiumAcetrizoate": {"32836007": ("Sodium acetrizoate", [12, 3850])},
    "SodiumChlorideNa22": {"6257000": ("Sodium chloride Na^22^", [25])},
    "SodiumChlorideNa24": {"31527000": ("Sodium chloride Na^24^", [25])},
    "SodiumChlorideSolution": {
        "373757009": ("Sodium chloride solution (substance)", [])
    },
    "SodiumChromateCr51": {"62517004": ("Sodium chromate Cr^51^", [25])},
    "SodiumDiatrizoate": {"24891006": ("Sodium diatrizoate (product)", [])},
    "SodiumDiprotrizoate": {"83423008": ("Sodium diprotrizoate", [12, 3850])},
    "SodiumFluorideF18": {"129501009": ("Sodium fluoride F^18^", [25, 4021])},
    "SodiumIodideI123": {"67690002": ("Sodium iodide I^123^", [25])},
    "SodiumIodideI124": {"422980002": ("Sodium iodide I^124^", [4021])},
    "SodiumIodideI131": {"111160004": ("Sodium iodide I^131^", [25])},
    "SodiumIodipamide": {"925002": ("Sodium iodipamide", [12, 3850])},
    "SodiumIodomethamate": {"38344006": ("Sodium iodomethamate", [12, 3850])},
    "SodiumNa22": {"71633006": ("Sodium Na^22^", [4021])},
    "SodiumPertechnetateTc99m": {"19495007": ("Sodium pertechnetate Tc^99m^", [25])},
    "SodiumPhosphateP32": {"10781003": ("Sodium phosphate P^32^", [25])},
    "SodiumTyropanoate": {"109212003": ("Sodium tyropanoate", [12, 3850])},
    "SoftCoatedWheatenTerrier": {"47699005": ("Soft-coated wheaten terrier", [7480])},
    "SoftPalate": {"49460000": ("Soft palate", [4028, 7151, 7192, 8134, 9514])},
    "SoftPalateStructure": {"49460000": ("Soft palate structure (body structure)", [])},
    "SoftTissue": {
        "87784001": ("Soft tissue", [645, 6122, 6211, 7151, 7166, 7191, 8134])
    },
    "SoftTissues": {"87784001": ("Soft tissues (body structure)", [])},
    "SokokeCatBreed": {"132679005": ("Sokoke cat breed", [7480])},
    "SokolkaHorseBreed": {"133086008": ("Sokolka horse breed", [7480])},
    "SokotoGudaliCattleBreed": {"133774001": ("Sokoto Gudali cattle breed", [7480])},
    "SolarDegeneration": {"43982006": ("Solar degeneration", [4405])},
    "SolealVein": {
        "397427005": ("Soleal vein", [3827, 7151, 7192, 8134, 9514, 12103, 12110])
    },
    "SolitaryFibrousTumor": {
        "128736003": ("Solitary fibrous tumor (morphologic abnormality)", [])
    },
    "SolitaryPlasmacytomaOfBone": {
        "10639003": ("Solitary plasmacytoma of bone (morphologic abnormality)", [])
    },
    "SolochromeAzurineBSStain": {"43549000": ("solochrome azurine (BS) stain", [8112])},
    "SolochromeBlack6BStain": {"11201005": ("solochrome black 6B stain", [8112])},
    "SolochromeCyanineRStain": {"25091000": ("solochrome cyanine R stain", [8112])},
    "SolochromeDarkBlueStain": {"38902009": ("solochrome dark blue stain", [8112])},
    "SolognoteSheepBreed": {"131803007": ("Solognote sheep breed", [7480])},
    "SolubleBerlinBlueStain": {"64991008": ("soluble berlin blue stain", [8112])},
    "SomaliCat": {"4042003": ("Somali cat", [7480])},
    "SomaliCattleBreed": {"133775000": ("Somali cattle breed", [7480])},
    "SomaliGoatBreed": {"131637001": ("Somali goat breed", [7480])},
    "SomaliPonyHorseBreed": {"133150005": ("Somali Pony horse breed", [7480])},
    "SomaliSheepBreed": {"131797008": ("Somali sheep breed", [7480])},
    "SombaCattleBreed": {"133497002": ("Somba cattle breed", [7480])},
    "SonValleyCattleBreed": {"133777008": ("Son Valley cattle breed", [7480])},
    "SonkheriCattleBreed": {"133776004": ("Sonkheri cattle breed", [7480])},
    "SopravissanaSheepBreed": {"131796004": ("Sopravissana sheep breed", [7480])},
    "SorraiaHorseBreed": {"133149005": ("Sorraia horse breed", [7480])},
    "Source": {"260753009": ("Source (attribute)", [])},
    "SouthAfricanBrownSwissCattleBreed": {
        "133498007": ("South African Brown Swiss cattle breed", [7480])
    },
    "SouthAfricanLandracePigBreed": {
        "133190008": ("South African Landrace pig breed", [7480])
    },
    "SouthAfricanMerinoSheepBreed": {
        "132723000": ("South African Merino sheep breed", [7480])
    },
    "SouthAfricanMiniatureHorseBreed": {
        "133087004": ("South African Miniature horse breed", [7480])
    },
    "SouthAfricanMuttonMerinoSheepBreed": {
        "132719008": ("South African Mutton Merino sheep breed", [7480])
    },
    "SouthAnatolianRedCattleBreed": {
        "133499004": ("South Anatolian Red cattle breed", [7480])
    },
    "SouthChinaBlackPigBreed": {"133174008": ("South China Black pig breed", [7480])},
    "SouthChinaPigBreed": {"133173002": ("South China pig breed", [7480])},
    "SouthChinaZebuCattleBreed": {
        "133778003": ("South China Zebu cattle breed", [7480])
    },
    "SouthDevonCattleBreed": {"133274002": ("South Devon cattle breed", [7480])},
    "SouthGermanColdbloodHorseBreed": {
        "133088009": ("South German Coldblood horse breed", [7480])
    },
    "SouthMalawiZebuCattleBreed": {
        "133785004": ("South Malawi Zebu cattle breed", [7480])
    },
    "SouthRussianOvcharkaDogBreed": {
        "132582006": ("South Russian Ovcharka dog breed", [7480])
    },
    "SouthRussianSteppeHoundDogBreed": {
        "132451005": ("South Russian Steppe Hound dog breed", [7480])
    },
    "SouthSuffolkSheepBreed": {"132791007": ("South Suffolk sheep breed", [7480])},
    "SouthWalesMountainSheepBreed": {
        "132792000": ("South Wales Mountain sheep breed", [7480])
    },
    "SouthYunnanShortEaredPigBreed": {
        "132154008": ("South Yunnan Short-Eared pig breed", [7480])
    },
    "SouthdownSheep": {"3566006": ("Southdown sheep (organism)", [])},
    "SouthdownSheepBreed": {"3566006": ("Southdown sheep breed", [7480])},
    "SouthernBlackmouthCurDogBreed": {
        "132581004": ("Southern Blackmouth Cur dog breed", [7480])
    },
    "SouthernCriouloCattleBreed": {
        "133588004": ("Southern Crioulo cattle breed", [7480])
    },
    "SouthernTswanaCattleBreed": {
        "131526008": ("Southern Tswana cattle breed", [7480])
    },
    "SouthernUkrainianCattleBreed": {
        "133510004": ("Southern Ukrainian cattle breed (organism)", [])
    },
    "SouthernUkranianCattleBreed": {
        "133510004": ("Southern Ukranian cattle breed", [7480])
    },
    "SouthwestSpanishMustangHorseBreed": {
        "133089001": ("Southwest Spanish Mustang horse breed", [7480])
    },
    "SovereignPigBreed": {"132068004": ("Sovereign pig breed", [7480])},
    "SovietHeavyDraftHorseBreed": {
        "133148002": ("Soviet Heavy Draft horse breed", [7480])
    },
    "SovietMeatPigBreed": {"132071007": ("Soviet Meat pig breed", [7480])},
    "Spaniel": {"45625009": ("Spaniel", [7480])},
    "SpanishAmericanHorseHorseBreed": {
        "133090005": ("Spanish-American Horse horse breed", [7480])
    },
    "SpanishAngloArabHorseBreed": {
        "133091009": ("Spanish Anglo-Arab horse breed", [7480])
    },
    "SpanishBarbHorseBreed": {"133147007": ("Spanish Barb horse breed", [7480])},
    "SpanishBrownAlpineCattleBreed": {
        "133511000": ("Spanish Brown Alpine cattle breed", [7480])
    },
    "SpanishColonialHorseHorseBreed": {
        "133092002": ("Spanish Colonial Horse horse breed", [7480])
    },
    "SpanishGoatBreed": {"131638006": ("Spanish goat breed", [7480])},
    "SpanishGreyhoundDogBreed": {"132466008": ("Spanish Greyhound dog breed", [7480])},
    "SpanishMastiffDogBreed": {"132639001": ("Spanish Mastiff dog breed", [7480])},
    "SpanishMustangHorse": {"76302002": ("Spanish mustang horse (organism)", [])},
    "SpanishMustangHorseBreed": {"76302002": ("Spanish mustang horse breed", [7480])},
    "SpanishNormanHorseBreed": {"133146003": ("Spanish-Norman horse breed", [7480])},
    "SpanishPointerDogBreed": {"132611002": ("Spanish Pointer dog breed", [7480])},
    "SpatialAndRelationalConcept": {
        "309825002": ("Spatial and Relational Concept", [7150, 9502, 9580])
    },
    "SpatialAndRelationalConcepts": {
        "309825002": ("Spatial and relational concepts (qualifier value)", [])
    },
    "SpatialCollocationAnalysis": {
        "133884007": ("Spatial collocation analysis", [6043, 6137])
    },
    "SpatialProximityAnalysis": {
        "133885008": ("Spatial proximity analysis", [6043, 6137])
    },
    "SpecimenClearing": {"433452008": ("Specimen clearing", [8113])},
    "SpecimenCollection": {"17636008": ("specimen collection", [3515, 8109, 8111])},
    "SpecimenContainer": {"434711009": ("Specimen container", [8101, 8102])},
    "SpecimenContainerLid": {"434473001": ("Specimen container lid", [8102])},
    "SpecimenDehydration": {"433470006": ("Specimen dehydration", [8113])},
    "SpecimenFreezing": {"27872000": ("Specimen freezing", [8113])},
    "SpecimenFromBreastByIncisionalBiopsyOfBreastMass": {
        "122739003": (
            "specimen from breast by incisional biopsy of breast mass",
            [8103, 8104],
        )
    },
    "SpecimenFromBreastObtainedByCoreNeedleBiopsy": {
        "122737001": (
            "specimen from breast obtained by core needle biopsy",
            [8103, 8104],
        )
    },
    "SpecimenFromBreastObtainedByExcision": {
        "397199005": ("specimen from breast obtained by excision", [8103, 8104])
    },
    "SpecimenFromBreastObtainedByImageGuidedCoreBiopsy": {
        "373102004": (
            "specimen from breast obtained by image guided core biopsy",
            [8103, 8104],
        )
    },
    "SpecimenFromBreastObtainedByIncisionalBiopsyOfBreastMass": {
        "122739003": (
            "Specimen from breast obtained by incisional biopsy of breast mass (specimen)",
            [],
        )
    },
    "SpecimenFromBreastObtainedByTotalMastectomy": {
        "122595009": ("specimen from breast obtained by total mastectomy", [8103, 8104])
    },
    "SpecimenFromBreastStereotacticallyGuidedCoreNeedleBiopsy": {
        "122738006": (
            "specimen from breast, stereotactically guided core needle biopsy",
            [8103, 8104],
        )
    },
    "SpecimenMammography": {"80865008": ("Specimen mammography (procedure)", [])},
    "SpecimenMicrowaveHeating": {"433455005": ("Specimen microwave heating", [8113])},
    "SpecimenObtainedByAspiration": {
        "119295008": ("Specimen obtained by aspiration (specimen)", [])
    },
    "SpecimenObtainedFromBreastByStereotacticallyGuidedCoreNeedleBiopsy": {
        "122738006": (
            "Specimen obtained from breast by stereotactically guided core needle biopsy (specimen)",
            [],
        )
    },
    "SpecimenProcessing": {"9265001": ("Specimen processing", [8111])},
    "SpecimenRadiographyOfBreast": {
        "80865008": ("Specimen radiography of breast", [6050, 6083])
    },
    "SpecimenReceiving": {"428995007": ("Specimen receiving", [8111])},
    "SpecimenSteamHeating": {"433457002": ("Specimen steam heating", [8113])},
    "SpecimenType": {"371439000": ("Specimen type (observable entity)", [])},
    "SpecimenVial": {"434746001": ("Specimen vial", [8101, 8102])},
    "SpecimenWell": {"434822004": ("Specimen well", [8101, 8102])},
    "SpecularMicroscope": {"409899004": ("Specular Microscope", [4202])},
    "SpeechAndLanguageTherapy": {"310101009": ("Speech and Language Therapy", [7030])},
    "SpeechAndLanguageTherapyService": {
        "310101009": ("Speech and language therapy service (qualifier value)", [])
    },
    "SpeedOfBloodPressureResponse": {
        "427732000": ("Speed of blood pressure response (observable entity)", [])
    },
    "SphenoidBone": {"73117003": ("Sphenoid bone", [4028, 7151, 7192, 8134, 9514])},
    "SphenoidBoneStructure": {
        "73117003": ("Sphenoid bone structure (body structure)", [])
    },
    "SphynxCatBreed": {"132680008": ("Sphynx cat breed", [7480])},
    "SpiculatedLesion": {
        "129742005": ("Spiculated lesion", [6006, 6007, 6335, 6341, 6342])
    },
    "SpiegelSheepBreed": {"132794004": ("Spiegel sheep breed", [7480])},
    "SpinalAnesthesia": {"231249005": ("Spinal anesthesia", [611, 612])},
    "SpinalCanalStructure": {
        "61853006": ("Spinal canal structure (body structure)", [])
    },
    "SpinalCerebrospinalFluidPathway": {
        "280401006": (
            "Spinal cerebrospinal fluid pathway",
            [7151, 7153, 7192, 8134, 9514],
        )
    },
    "SpinalCord": {"2748008": ("Spinal cord", [4, 7151, 7153, 7192, 8134, 9514])},
    "SpinalCordGrayMatter": {
        "12958003": ("Spinal cord gray matter", [7151, 7153, 7192, 8134, 9514])
    },
    "SpinalCordGrayMatterStructure": {
        "12958003": ("Spinal cord gray matter structure (body structure)", [])
    },
    "SpinalCordStructure": {"2748008": ("Spinal cord structure (body structure)", [])},
    "SpinalCordWhiteMatter": {
        "27088001": ("Spinal cord white matter", [7151, 7153, 7192, 7710, 8134, 9514])
    },
    "SpinalCordWhiteMatterStructure": {
        "27088001": ("Spinal cord white matter structure (body structure)", [])
    },
    "SpinalisMuscle": {
        "4317002": ("Spinalis muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "SpindleCellCarcinoma": {"65692009": ("Spindle cell carcinoma", [638, 639])},
    "SpindleCellNodule": {"110451006": ("Spindle cell nodule (tumor)", [6030, 6033])},
    "Spine": {
        "421060004": (
            "Spine",
            [4, 4009, 4030, 4031, 4040, 4042, 6102, 6109, 6114, 7151, 7192, 8134, 9514],
        )
    },
    "SpinoneItalianoDogBreed": {"132638009": ("Spinone Italiano dog breed", [7480])},
    "SpinousProcessOfVertebra": {
        "55678000": ("Spinous process of vertebra", [6115, 8134])
    },
    "SpiperoneF18": {"129499001": ("Spiperone F^18^", [4021])},
    "SpiralAcquisition": {"116152004": ("Spiral Acquisition", [10013])},
    "SpiralComputedTomographyScan": {
        "116152004": ("Spiral computed tomography scan (procedure)", [])
    },
    "SpiritSolubleAnilineBlueStain": {
        "11645004": ("spirit soluble aniline blue stain", [8112])
    },
    "SpiritSolubleEosinStain": {"83600004": ("spirit soluble eosin stain", [8112])},
    "SpitiHorseBreed": {"133093007": ("Spiti horse breed", [7480])},
    "Spleen": {
        "78961009": (
            "Spleen",
            [4, 4030, 6202, 6204, 7151, 7154, 7192, 8134, 9514, 10044, 10060, 12321],
        )
    },
    "SplenicArtery": {
        "22083002": (
            "Splenic artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "SplenicFlexureOfColon": {"72592005": ("Splenic flexure of colon", [6210, 8134])},
    "SplenicLymphNode": {"280824006": ("splenic lymph node", [7600, 8134])},
    "SplenicStructure": {"78961009": ("Splenic structure (body structure)", [])},
    "SplenicVein": {
        "35819009": (
            "Splenic vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12114,
                12125,
            ],
        )
    },
    "SpleniumOfCorpusCallosum": {
        "23347006": ("splenium of corpus callosum", [7705, 7710, 8134])
    },
    "SporeStain": {"255811001": ("spore stain", [8112])},
    "SpotCompression": {"399055006": ("Spot Compression", [4015, 6028, 6029])},
    "SpotCompressionOfBreast": {
        "399055006": ("Spot compression of breast (procedure)", [])
    },
    "SpottedPig": {"36187006": ("Spotted pig (organism)", [])},
    "SpottedPigBreed": {"36187006": ("Spotted pig breed", [7480])},
    "SprecaCattleBreed": {"131527004": ("Spreca cattle breed", [7480])},
    "SpælsauSheepBreed": {"132793005": ("Spælsau sheep breed", [7480])},
    "SquamousCellCarcinoma": {
        "1162767002": ("Squamous cell carcinoma", [638, 639, 6030, 6033])
    },
    "SquamousCellCarcinomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "28899001": (
            "Squamous cell carcinoma, no International Classification of Diseases for Oncology subtype "
            "(morphologic abnormality)",
            [],
        )
    },
    "SquamousCellCarcinomaOfAnus": {
        "723265000": ("Squamous cell carcinoma of anus", [638, 639])
    },
    "SquamousCellCarcinomaOfHeadAndNeck": {
        "716659002": ("Squamous cell carcinoma of head and neck", [638, 639])
    },
    "SquamousCellCarcinomaOfLaryngealCartilage": {
        "707357005": ("Squamous cell carcinoma of laryngeal cartilage", [638, 639])
    },
    "SquamousCellCarcinomaOfLung": {
        "254634000": ("Squamous cell carcinoma of lung", [638, 639])
    },
    "SquamousCellCarcinomaOfMouth": {
        "307502000": ("Squamous cell carcinoma of mouth", [638, 639])
    },
    "SquamousCellCarcinomaOfSkin": {
        "254651007": ("Squamous cell carcinoma of skin", [638, 639])
    },
    "StBernardDog": {"83236005": ("St. Bernard dog", [7480])},
    "StCroixSheepBreed": {"132795003": ("St. Croix sheep breed", [7480])},
    "StableAngina": {"233819005": ("Stable Angina", [3202, 3220, 3700])},
    "StabyhounDogBreed": {"132637004": ("Stabyhoun dog breed", [7480])},
    "StaffordshireBullTerrier": {
        "83216009": ("Staffordshire bull terrier (organism)", [])
    },
    "StaffordshireBullTerrierDogBreed": {
        "83216009": ("Staffordshire bull terrier dog breed", [7480])
    },
    "Stage": {"258214002": ("Stage (attribute)", [])},
    "Stage0": {"261613009": ("Stage 0", [3778])},
    "Stage1": {"258215001": ("Stage 1", [3778])},
    "Stage2": {"258219007": ("Stage 2", [3778])},
    "Stage3": {"258224005": ("Stage 3", [3778])},
    "Stage4": {"258228008": ("Stage 4", [3778])},
    "Stage5": {"261617005": ("Stage 5", [3778])},
    "Staging": {"373825000": ("Staging", [6327])},
    "StagingIntent": {"373825000": ("Staging intent", [3629])},
    "StagingProcedureIntent": {
        "373825000": ("Staging - procedure intent (qualifier value)", [])
    },
    "Staining": {"127790008": ("Staining", [8111, 8130])},
    "StainingMethod": {"127790008": ("Staining method (procedure)", [])},
    "StainlessSteelMaterial": {"256506002": ("Stainless Steel Material", [7300, 9579])},
    "StandardDachshund": {"416885007": ("Standard dachshund (organism)", [])},
    "StandardDachshundDogBreed": {
        "416885007": ("Standard dachshund dog breed", [7480])
    },
    "StandardDeviation": {"386136009": ("Standard Deviation", [7464])},
    "StandardManchesterTerrier": {"9131007": ("Standard Manchester terrier", [7480])},
    "StandardPoodle": {"507002": ("Standard poodle (organism)", [])},
    "StandardPoodleDogBreed": {"507002": ("Standard poodle dog breed", [7480])},
    "StandardSchnauzer": {"69592005": ("Standard schnauzer (organism)", [])},
    "StandardSchnauzerDogBreed": {"69592005": ("Standard schnauzer dog breed", [7480])},
    "StandardbredHorse": {"34200004": ("Standardbred horse (organism)", [])},
    "StandardbredHorseBreed": {"34200004": ("Standardbred horse breed", [7480])},
    "Standing": {"10904000": ("standing", [20])},
    "Staple": {
        "56353002": ("Staple", [6040, 6102, 6138, 6202, 6203, 6401, 6404, 7151, 7193])
    },
    "StapleDevice": {"56353002": ("Staple, device (physical object)", [])},
    "StartTime": {"398201009": ("Start time (qualifier value)", [])},
    "StaticCatheterHemodynamicMeasurementMethodRegimeTherapy": {
        "128578007": (
            "Static catheter hemodynamic measurement method (regime/therapy)",
            [],
        )
    },
    "StaticCatheterMethod": {"128578007": ("Static catheter method", [3241])},
    "Statins": {"96302009": ("Statins", [3761])},
    "Staunig": {"399098005": ("Staunig", [4012])},
    "StaunigProjection": {"399098005": ("Staunig projection (qualifier value)", [])},
    "Steady": {"55011004": ("Steady", [4220])},
    "SteamHeatingOfTissueSpecimen": {
        "433457002": ("Steam heating of tissue specimen (procedure)", [])
    },
    "Stecher": {"399292003": ("Stecher", [4012])},
    "StecherProjection": {"399292003": ("Stecher projection (qualifier value)", [])},
    "SteigarSheepBreed": {"132796002": ("Steigar sheep breed", [7480])},
    "SteinschafSheepBreed": {"132797006": ("Steinschaf sheep breed", [7480])},
    "Stenosis": {"415582006": ("stenosis", [3810, 3813])},
    "Stenotic": {"386139002": ("Stenotic", [3712])},
    "StenoticCoronaryArtery": {"386139002": ("Stenotic coronary artery (finding)", [])},
    "StenoticLesionLength": {"408716009": ("Stenotic Lesion Length", [3483])},
    "Stent": {
        "65818007": ("Stent", [8, 3411, 3429, 3496, 3810, 4051, 7111, 7151, 7157, 7193])
    },
    "StentCrossSectionalArea": {"408705002": ("Stent Cross-Sectional Area", [3482])},
    "StentDevice": {"65818007": ("Stent, device", [1000])},
    "StentDiameter": {"408706001": ("Stent Diameter", [3481])},
    "StentLength": {"408703009": ("Stent Length", [3483])},
    "StentPlacement": {"103716009": ("Stent placement", [9, 3405])},
    "StentVolume": {"408704003": ("Stent Volume", [3485])},
    "Stented": {"386138005": ("Stented", [3712])},
    "StentedCoronaryArtery": {"386138005": ("Stented coronary artery (finding)", [])},
    "StenverProjection": {"399349006": ("Stenver's projection (qualifier value)", [])},
    "Stenvers": {"399349006": ("Stenvers", [4012])},
    "StepSectioningOfTissueBlock": {
        "434475008": ("Step sectioning of tissue block (procedure)", [])
    },
    "SteppeMeatPigBreed": {"132073005": ("Steppe Meat pig breed", [7480])},
    "SterlingPigBreed": {"132200002": ("Sterling pig breed", [7480])},
    "SternalAngle": {"44612009": ("Sternal angle", [6115, 8134])},
    "SternalSkin": {"244107007": ("Sternal skin", [4029, 8134])},
    "SternoclavicularJoint": {
        "7844006": (
            "Sternoclavicular joint",
            [4, 1000, 1001, 1004, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SternoclavicularJointStructure": {
        "7844006": ("Sternoclavicular joint structure (body structure)", [])
    },
    "SternocleidomastoidMuscle": {
        "22823000": (
            "Sternocleidomastoid muscle",
            [6102, 6109, 6116, 7151, 7192, 8134, 9514],
        )
    },
    "Sternum": {
        "56873002": (
            "Sternum",
            [
                4,
                1000,
                1001,
                1004,
                4009,
                4030,
                4031,
                4042,
                6102,
                6109,
                6114,
                7151,
                7155,
                7192,
                8134,
                9514,
            ],
        )
    },
    "Steroid": {"116566001": ("Steroid", [621, 622, 4410])},
    "Stiffle": {"116010006": ("Stiffle", [7482, 7483, 8134])},
    "StifleJoint": {"116010006": ("Stifle joint (body structure)", [])},
    "StomaAppliance": {"407744008": ("Stoma appliance", [6202, 6203])},
    "Stomach": {
        "69695003": (
            "Stomach",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514, 10044, 10060],
        )
    },
    "StomachStructure": {"69695003": ("Stomach structure (body structure)", [])},
    "StoopedOver": {"87068006": ("stooped-over", [20])},
    "StoopedOverPosition": {"87068006": ("Stooped-over position (finding)", [])},
    "StopTime": {"397898000": ("Stop time (qualifier value)", [])},
    "StreiPigBreed": {"133200007": ("Strei pig breed", [7480])},
    "Streptokinase": {"395889004": ("Streptokinase", [10])},
    "StressTestUsingBicycleErgometer": {
        "26046004": ("Stress test using Bicycle Ergometer", [3261, 12001])
    },
    "StressTestUsingCardiacPacing": {
        "428685003": ("Stress test using cardiac pacing", [3261, 3757, 12001])
    },
    "StressThalliumProcedure": {"431511008": ("Stress thallium procedure", [3110])},
    "Stretcher": {
        "89149003": ("Stretcher", [7151, 7157, 7193, 9505, 9515, 9520, 9573])
    },
    "StretcherDevice": {"89149003": ("Stretcher, device (physical object)", [])},
    "Stroke": {"309966001": ("Stroke", [7030])},
    "StrokeIndex": {
        "277381004": (
            "Stroke Index",
            [
                12200,
                12207,
                12239,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "StrokeUnit": {"309966001": ("Stroke unit (environment)", [])},
    "StrokeVolume": {
        "90096001": (
            "Stroke Volume",
            [
                3835,
                12200,
                12207,
                12239,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "StrongWoolMerinoSheepBreed": {
        "132720002": ("Strong Wool Merino sheep breed", [7480])
    },
    "Strontium89Chloride": {"125701003": ("Strontium-89 Chloride (substance)", [])},
    "StrontiumChlorideSr85": {"69076006": ("Strontium chloride Sr^85^", [25])},
    "StrontiumChlorideSr87": {"38424001": ("Strontium chloride Sr^87^", [25])},
    "StrontiumChlorideSr89": {"125701003": ("Strontium chloride Sr^89^", [25])},
    "StrontiumNitrateSr85": {"8858006": ("Strontium nitrate Sr^85^", [25])},
    "StrontiumNitrateSr87": {"31953001": ("Strontium nitrate Sr^87^", [25])},
    "StructureOfAbdominalVein": {
        "84421000": ("Structure of abdominal vein (body structure)", [])
    },
    "StructureOfAbdominalWallMuscle": {
        "195879000": ("Structure of abdominal wall muscle (body structure)", [])
    },
    "StructureOfAchillesTendon": {
        "61352006": ("Structure of achilles tendon (body structure)", [])
    },
    "StructureOfAcromion": {"31934006": ("Structure of acromion (body structure)", [])},
    "StructureOfAlveolarProcessOfMandible": {
        "38754007": ("Structure of alveolar process of mandible (body structure)", [])
    },
    "StructureOfAmnion": {"70847004": ("Structure of amnion (body structure)", [])},
    "StructureOfAngleOfRib": {
        "14510004": ("Structure of angle of rib (body structure)", [])
    },
    "StructureOfAntecubitalVein": {
        "128553008": ("Structure of antecubital vein (body structure)", [])
    },
    "StructureOfAnteriorApicalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716937001": (
            "Structure of anterior apical part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorApicalPartOfPeripheralZoneOfRightHalfProstate": {
        "716936005": (
            "Structure of anterior apical part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorApicalPartOfTransitionZoneOfLeftHalfProstate": {
        "716931000": (
            "Structure of anterior apical part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorApicalPartOfTransitionZoneOfRightHalfProstate": {
        "716930004": (
            "Structure of anterior apical part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorAuricularLymphNode": {
        "25247006": ("Structure of anterior auricular lymph node (body structure)", [])
    },
    "StructureOfAnteriorBasalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716905002": (
            "Structure of anterior basal part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorBasalPartOfPeripheralZoneOfRightHalfProstate": {
        "716904003": (
            "Structure of anterior basal part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorBasalPartOfTransitionZoneOfLeftHalfProstate": {
        "716897000": (
            "Structure of anterior basal part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorBasalPartOfTransitionZoneOfRightHalfProstate": {
        "716896009": (
            "Structure of anterior basal part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorCardiacVein": {
        "194996006": ("Structure of anterior cardiac vein (body structure)", [])
    },
    "StructureOfAnteriorCerebralArtery": {
        "60176003": ("Structure of anterior cerebral artery (body structure)", [])
    },
    "StructureOfAnteriorCervicalLymphNode": {
        "5727003": ("Structure of anterior cervical lymph node (body structure)", [])
    },
    "StructureOfAnteriorCommissure": {
        "62872008": ("Structure of anterior commissure (body structure)", [])
    },
    "StructureOfAnteriorCommunicatingArtery": {
        "8012006": ("Structure of anterior communicating artery (body structure)", [])
    },
    "StructureOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "59438005": (
            "Structure of anterior descending branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfAnteriorFibromuscularStromaOfProstate": {
        "717025007": (
            "Structure of anterior fibromuscular stroma of prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorHornOfLateralVentricle": {
        "30399003": (
            "Structure of anterior horn of lateral ventricle (body structure)",
            [],
        )
    },
    "StructureOfAnteriorMediastinalLymphNode": {
        "5296000": ("Structure of anterior mediastinal lymph node (body structure)", [])
    },
    "StructureOfAnteriorMiddleRegionalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716920008": (
            "Structure of anterior middle regional part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorMiddleRegionalPartOfPeripheralZoneOfRightHalfProstate": {
        "716919002": (
            "Structure of anterior middle regional part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorMiddleRegionalPartOfTransitionZoneOfLeftHalfProstate": {
        "716914007": (
            "Structure of anterior middle regional part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorMiddleRegionalPartOfTransitionZoneOfRightHalfProstate": {
        "716913001": (
            "Structure of anterior middle regional part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAnteriorSegmentOfRightUpperLobeOfLung": {
        "39743006": (
            "Structure of anterior segment of right upper lobe of lung (body structure)",
            [],
        )
    },
    "StructureOfAnteriorSegmentOfUpperDivisionOfLeftUpperLobeOfLung": {
        "22270008": (
            "Structure of anterior segment of upper division of left upper lobe of lung (body structure)",
            [],
        )
    },
    "StructureOfAnteriorSpinalArtery": {
        "17388009": ("Structure of anterior spinal artery (body structure)", [])
    },
    "StructureOfAnteriorTibialArtery": {
        "68053000": ("Structure of anterior tibial artery (body structure)", [])
    },
    "StructureOfAnteriorTibialLymphNode": {
        "303713004": ("Structure of anterior tibial lymph node (body structure)", [])
    },
    "StructureOfAnteriorTibialVein": {
        "26703007": ("Structure of anterior tibial vein (body structure)", [])
    },
    "StructureOfAnulusFibrosusOfAorta": {
        "77583004": ("Structure of anulus fibrosus of aorta (body structure)", [])
    },
    "StructureOfAnulusFibrosusOfMitralOrifice": {
        "65197004": (
            "Structure of anulus fibrosus of mitral orifice (body structure)",
            [],
        )
    },
    "StructureOfAnulusFibrosusOfPulmonaryArtery": {
        "90318009": (
            "Structure of anulus fibrosus of pulmonary artery (body structure)",
            [],
        )
    },
    "StructureOfAnulusFibrosusOfTricuspidOrifice": {
        "113259005": (
            "Structure of anulus fibrosus of tricuspid orifice (body structure)",
            [],
        )
    },
    "StructureOfAorticBifurcation": {
        "73166001": ("Structure of aortic bifurcation (body structure)", [])
    },
    "StructureOfAorticIsthmus": {
        "88593004": ("Structure of aortic isthmus (body structure)", [])
    },
    "StructureOfAorticLymphNode": {
        "35783009": ("Structure of aortic lymph node (body structure)", [])
    },
    "StructureOfAorticSinotubularJunction": {
        "443167003": ("Structure of aortic sinotubular junction (body structure)", [])
    },
    "StructureOfApexOfHeart": {
        "13383001": ("Structure of apex of heart (body structure)", [])
    },
    "StructureOfApexOfLeftVentricle": {
        "128564006": ("Structure of apex of left ventricle (body structure)", [])
    },
    "StructureOfApexOfLung": {
        "86598002": ("Structure of apex of lung (body structure)", [])
    },
    "StructureOfApexOfRightVentricle": {
        "128565007": ("Structure of apex of right ventricle (body structure)", [])
    },
    "StructureOfApicalAxillaryLymphNode": {
        "16051009": ("Structure of apical axillary lymph node (body structure)", [])
    },
    "StructureOfApicalPartOfAnteriorFibromuscularStromaOfLeftHalfProstate": {
        "716927006": (
            "Structure of apical part of anterior fibromuscular stroma of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfAnteriorFibromuscularStromaOfProstate": {
        "716925003": (
            "Structure of apical part of anterior fibromuscular stroma of prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfAnteriorFibromuscularStromaOfRightHalfProstate": {
        "716926002": (
            "Structure of apical part of anterior fibromuscular stroma of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716935009": (
            "Structure of apical part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfPeripheralZoneOfRightHalfProstate": {
        "716934008": (
            "Structure of apical part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfTransitionZoneOfLeftHalfProstate": {
        "716929009": (
            "Structure of apical part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfApicalPartOfTransitionZoneOfRightHalfProstate": {
        "716928001": (
            "Structure of apical part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfAppendicularLymphNode": {
        "46157003": ("Structure of appendicular lymph node (body structure)", [])
    },
    "StructureOfArchOfVertebra": {
        "40265002": ("Structure of arch of vertebra (body structure)", [])
    },
    "StructureOfArcuateArteryOfKidney": {
        "274231001": ("Structure of arcuate artery of kidney (body structure)", [])
    },
    "StructureOfArteryOfAbdomen": {
        "118634008": ("Structure of artery of abdomen (body structure)", [])
    },
    "StructureOfArteryOfLowerExtremity": {
        "70791007": ("Structure of artery of lower extremity (body structure)", [])
    },
    "StructureOfArteryOfNeck": {
        "119568004": ("Structure of artery of neck (body structure)", [])
    },
    "StructureOfArteryOfUpperExtremity": {
        "75531005": ("Structure of artery of upper extremity (body structure)", [])
    },
    "StructureOfAtlantoaxialJoint": {
        "62555009": ("Structure of atlantoaxial joint (body structure)", [])
    },
    "StructureOfAtrialVein": {
        "195164009": ("Structure of atrial vein (body structure)", [])
    },
    "StructureOfAtrioventricularBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "75902001": (
            "Structure of atrioventricular branch of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfAtrioventricularNode": {
        "25943004": ("Structure of atrioventricular node (body structure)", [])
    },
    "StructureOfAtrioventricularNodeBranchOfRightCoronaryArtery": {
        "12800002": (
            "Structure of atrioventricular node branch of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfAtrioventricularVein": {
        "195496005": ("Structure of atrioventricular vein (body structure)", [])
    },
    "StructureOfAxillaryArtery": {
        "67937003": ("Structure of axillary artery (body structure)", [])
    },
    "StructureOfAxillaryFascia": {
        "368536000": ("Structure of axillary fascia (body structure)", [])
    },
    "StructureOfAxillaryVein": {
        "68705008": ("Structure of axillary vein (body structure)", [])
    },
    "StructureOfAxillaryVeinLymphNode": {
        "421624008": ("Structure of axillary vein lymph node (body structure)", [])
    },
    "StructureOfAzygousVein": {
        "72107004": ("Structure of azygous vein (body structure)", [])
    },
    "StructureOfBasalPartOfAnteriorFibromuscularStromaOfLeftHalfProstate": {
        "716893001": (
            "Structure of basal part of anterior fibromuscular stroma of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfAnteriorFibromuscularStromaOfProstate": {
        "716891004": (
            "Structure of basal part of anterior fibromuscular stroma of prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfAnteriorFibromuscularStromaOfRightHalfProstate": {
        "716892006": (
            "Structure of basal part of anterior fibromuscular stroma of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716903009": (
            "Structure of basal part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfPeripheralZoneOfRightHalfProstate": {
        "716902004": (
            "Structure of basal part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfTransitionZoneOfLeftHalfProstate": {
        "716895008": (
            "Structure of basal part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasalPartOfTransitionZoneOfRightHalfProstate": {
        "716894007": (
            "Structure of basal part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfBasilarArtery": {
        "59011009": ("Structure of basilar artery (body structure)", [])
    },
    "StructureOfBasilicVein": {
        "19715009": ("Structure of basilic vein (body structure)", [])
    },
    "StructureOfBodyConduit": {
        "91830000": ("Structure of body conduit (body structure)", [])
    },
    "StructureOfBodyOfSternum": {
        "52509009": ("Structure of body of sternum (body structure)", [])
    },
    "StructureOfBothEyes": {
        "40638003": ("Structure of both eyes (body structure)", [])
    },
    "StructureOfBrachialArtery": {
        "17137000": ("Structure of brachial artery (body structure)", [])
    },
    "StructureOfBrachialVein": {
        "20115005": ("Structure of brachial vein (body structure)", [])
    },
    "StructureOfBrachiocephalicArtery": {
        "12691009": ("Structure of brachiocephalic artery (body structure)", [])
    },
    "StructureOfBrachiocephalicVein": {
        "8887007": ("Structure of brachiocephalic vein (body structure)", [])
    },
    "StructureOfBronchialArtery": {
        "64468002": ("Structure of bronchial artery (body structure)", [])
    },
    "StructureOfBuccinatorLymphNode": {
        "143925009": ("Structure of buccinator lymph node (body structure)", [])
    },
    "StructureOfCalfOfLeg": {
        "53840002": ("Structure of calf of leg (body structure)", [])
    },
    "StructureOfCalyx": {"2334006": ("Structure of calyx (body structure)", [])},
    "StructureOfCapillaryBloodVesselOrgan": {
        "20982000": ("Structure of capillary blood vessel (organ) (body structure)", [])
    },
    "StructureOfCardiacValveLeaflet": {
        "6530003": ("Structure of cardiac valve leaflet (body structure)", [])
    },
    "StructureOfCardiacVein": {
        "85439003": ("Structure of cardiac vein (body structure)", [])
    },
    "StructureOfCardiovascularSystem": {
        "113257007": ("Structure of cardiovascular system (body structure)", [])
    },
    "StructureOfCarina": {"28700002": ("Structure of carina (body structure)", [])},
    "StructureOfCarotidBifurcation": {
        "80272002": ("Structure of carotid bifurcation (body structure)", [])
    },
    "StructureOfCarotidSinus": {
        "21479005": ("Structure of carotid sinus (body structure)", [])
    },
    "StructureOfCarotidSiphon": {
        "54409005": ("Structure of carotid siphon (body structure)", [])
    },
    "StructureOfCaudateLobeOfLiver": {
        "71133005": ("Structure of caudate lobe of liver (body structure)", [])
    },
    "StructureOfCavumSeptiPellucidi": {
        "74968005": ("Structure of cavum septi pellucidi (body structure)", [])
    },
    "StructureOfCeliacArtery": {
        "57850000": ("Structure of celiac artery (body structure)", [])
    },
    "StructureOfCeliacLymphNode": {
        "47985009": ("Structure of celiac lymph node (body structure)", [])
    },
    "StructureOfCentralAxillaryLymphNode": {
        "283001": ("Structure of central axillary lymph node (body structure)", [])
    },
    "StructureOfCentralNervousSystem": {
        "21483005": ("Structure of central nervous system (body structure)", [])
    },
    "StructureOfCentralRetinalArtery": {
        "76117006": ("Structure of central retinal artery (body structure)", [])
    },
    "StructureOfCentralVeinOfTheRetina": {
        "62869001": ("Structure of central vein of the retina (body structure)", [])
    },
    "StructureOfCentralZoneOfLeftHalfProstate": {
        "716901006": (
            "Structure of central zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfCentralZoneOfRightHalfProstate": {
        "716900007": (
            "Structure of central zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfCephalicVein": {
        "20699002": ("Structure of cephalic vein (body structure)", [])
    },
    "StructureOfCerebralAqueduct": {
        "80447000": ("Structure of cerebral aqueduct (body structure)", [])
    },
    "StructureOfCerebralArtery": {
        "88556005": ("Structure of cerebral artery (body structure)", [])
    },
    "StructureOfCerebralCortex": {
        "40146001": ("Structure of cerebral cortex (body structure)", [])
    },
    "StructureOfCervicalVertebralColumn": {
        "122494005": ("Structure of cervical vertebral column (body structure)", [])
    },
    "StructureOfChordaeTendineaeCordis": {
        "102298001": ("Structure of chordae tendineae cordis (body structure)", [])
    },
    "StructureOfChoroidPlexus": {
        "80621003": ("Structure of choroid plexus (body structure)", [])
    },
    "StructureOfCingulum": {"37035000": ("Structure of cingulum (body structure)", [])},
    "StructureOfCircleOfWillis": {
        "11279006": ("Structure of circle of Willis (body structure)", [])
    },
    "StructureOfCircumflexBranchOfLeftCoronaryArtery": {
        "57396003": (
            "Structure of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfCisternaMagna": {
        "54165005": ("Structure of cisterna magna (body structure)", [])
    },
    "StructureOfClavicularNotchOfSternum": {
        "75319007": ("Structure of clavicular notch of sternum (body structure)", [])
    },
    "StructureOfClinoidPortionOfInternalCarotidArtery": {
        "415637004": (
            "Structure of clinoid portion of internal carotid artery (body structure)",
            [],
        )
    },
    "StructureOfColicLymphNode": {
        "8356004": ("Structure of colic lymph node (body structure)", [])
    },
    "StructureOfColonicHaustra": {
        "6533001": ("Structure of colonic haustra (body structure)", [])
    },
    "StructureOfCommonDuctLymphNode": {
        "280639005": ("Structure of common duct lymph node (body structure)", [])
    },
    "StructureOfCommonHepaticArtery": {
        "66559000": ("Structure of common hepatic artery (body structure)", [])
    },
    "StructureOfCommonIliacVein": {
        "46027005": ("Structure of common iliac vein (body structure)", [])
    },
    "StructureOfCommonLeftPulmonaryVein": {
        "443705001": ("Structure of common left pulmonary vein (body structure)", [])
    },
    "StructureOfCommonRightPulmonaryVein": {
        "443591004": ("Structure of common right pulmonary vein (body structure)", [])
    },
    "StructureOfCoracoidProcessOfScapula": {
        "8931003": ("Structure of coracoid process of scapula (body structure)", [])
    },
    "StructureOfCornealEndothelium": {
        "65431007": ("Structure of corneal endothelium (body structure)", [])
    },
    "StructureOfCornealEpithelium": {
        "15775008": ("Structure of corneal epithelium (body structure)", [])
    },
    "StructureOfCoronarySinusOfLeftAtrialSeptum": {
        "31162003": (
            "Structure of coronary sinus of left atrial septum (body structure)",
            [],
        )
    },
    "StructureOfCoronarySinusOstium": {
        "71271007": ("Structure of coronary sinus ostium (body structure)", [])
    },
    "StructureOfCostalCartilage": {
        "50016007": ("Structure of costal cartilage (body structure)", [])
    },
    "StructureOfCostalGroove": {
        "17399006": ("Structure of costal groove (body structure)", [])
    },
    "StructureOfCostocervicalTrunk": {
        "3159004": ("Structure of costocervical trunk (body structure)", [])
    },
    "StructureOfCubitalLymphNode": {
        "34775006": ("Structure of cubital lymph node (body structure)", [])
    },
    "StructureOfCysticLymphNode": {
        "280556009": ("Structure of cystic lymph node (body structure)", [])
    },
    "StructureOfDeciduousMandibularLeftCanineTooth": {
        "43281008": (
            "Structure of deciduous mandibular left canine tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularLeftCentralIncisorTooth": {
        "89552004": (
            "Structure of deciduous mandibular left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularLeftFirstMolarTooth": {
        "38896004": (
            "Structure of deciduous mandibular left first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularLeftLateralIncisorTooth": {
        "14770005": (
            "Structure of deciduous mandibular left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularLeftSecondMolarTooth": {
        "49330006": (
            "Structure of deciduous mandibular left second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularRightCanineTooth": {
        "6062009": (
            "Structure of deciduous mandibular right canine tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularRightCentralIncisorTooth": {
        "67834006": (
            "Structure of deciduous mandibular right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularRightFirstMolarTooth": {
        "58646007": (
            "Structure of deciduous mandibular right first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularRightLateralIncisorTooth": {
        "22445006": (
            "Structure of deciduous mandibular right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMandibularRightSecondMolarTooth": {
        "61868007": (
            "Structure of deciduous mandibular right second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryLeftCanineTooth": {
        "73937000": (
            "Structure of deciduous maxillary left canine tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryLeftCentralIncisorTooth": {
        "51678005": (
            "Structure of deciduous maxillary left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryLeftFirstMolarTooth": {
        "45234009": (
            "Structure of deciduous maxillary left first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryLeftLateralIncisorTooth": {
        "43622005": (
            "Structure of deciduous maxillary left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryLeftSecondMolarTooth": {
        "51943008": (
            "Structure of deciduous maxillary left second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryRightCanineTooth": {
        "30618001": (
            "Structure of deciduous maxillary right canine tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryRightCentralIncisorTooth": {
        "88824007": (
            "Structure of deciduous maxillary right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryRightFirstMolarTooth": {
        "17505006": (
            "Structure of deciduous maxillary right first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryRightLateralIncisorTooth": {
        "65624003": (
            "Structure of deciduous maxillary right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfDeciduousMaxillaryRightSecondMolarTooth": {
        "27855007": (
            "Structure of deciduous maxillary right second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfDeepAnteriorCervicalLymphNode": {
        "168360002": (
            "Structure of deep anterior cervical lymph node (body structure)",
            [],
        )
    },
    "StructureOfDeepInguinalLymphNode": {
        "65266007": ("Structure of deep inguinal lymph node (body structure)", [])
    },
    "StructureOfDeepLateralCervicalLymphNode": {
        "167864002": (
            "Structure of deep lateral cervical lymph node (body structure)",
            [],
        )
    },
    "StructureOfDeepLymphNode": {
        "60996007": ("Structure of deep lymph node (body structure)", [])
    },
    "StructureOfDeepPalmarVenousArch": {
        "368481004": ("Structure of deep palmar venous arch (body structure)", [])
    },
    "StructureOfDeepPoplitealLymphNode": {
        "35721009": ("Structure of deep popliteal lymph node (body structure)", [])
    },
    "StructureOfDeepVolarArchOfRadialArtery": {
        "10119003": (
            "Structure of deep volar arch of radial artery (body structure)",
            [],
        )
    },
    "StructureOfDelphianLymphNode": {
        "167664004": ("Structure of delphian lymph node (body structure)", [])
    },
    "StructureOfDeltoidMuscle": {
        "35259002": ("Structure of deltoid muscle (body structure)", [])
    },
    "StructureOfDescendingThoracicAorta": {
        "32672002": (
            "Descending aorta",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "StructureOfDiaphragmaticLymphNode": {
        "196751009": ("Structure of diaphragmatic lymph node (body structure)", [])
    },
    "StructureOfDiencephalon": {
        "87563008": ("Structure of diencephalon (body structure)", [])
    },
    "StructureOfDigitalArteryOfHand": {
        "40254007": ("Structure of digital artery of hand (body structure)", [])
    },
    "StructureOfDistalPortionOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "36672000": (
            "Structure of distal portion of anterior descending branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfDistalPortionOfCircumflexBranchOfLeftCoronaryArtery": {
        "6511003": (
            "Structure of distal portion of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfDistalPortionOfRightCoronaryArtery": {
        "41879009": (
            "Structure of distal portion of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfDorsalAspectOfScapula": {
        "51698000": ("Structure of dorsal aspect of scapula (body structure)", [])
    },
    "StructureOfDorsalScapularArtery": {
        "91732003": ("Structure of dorsal scapular artery (body structure)", [])
    },
    "StructureOfDorsalisPedisArtery": {
        "86547008": ("Structure of dorsalis pedis artery (body structure)", [])
    },
    "StructureOfDuctusArteriosus": {
        "4432005": ("Structure of ductus arteriosus (body structure)", [])
    },
    "StructureOfDuctusVenosus": {
        "367624001": ("Structure of ductus venosus (body structure)", [])
    },
    "StructureOfEighthIntercostalSpace": {
        "117140004": ("Structure of eighth intercostal space (body structure)", [])
    },
    "StructureOfEleventhIntercostalSpace": {
        "117143002": ("Structure of eleventh intercostal space (body structure)", [])
    },
    "StructureOfEntorhinalCortex": {
        "3937002": ("Structure of entorhinal cortex (body structure)", [])
    },
    "StructureOfEpigastricLymphNode": {
        "60965003": ("Structure of epigastric lymph node (body structure)", [])
    },
    "StructureOfEpitrochlearLymphNode": {
        "28870006": ("Structure of epitrochlear lymph node (body structure)", [])
    },
    "StructureOfErectorSpinaeMuscle": {
        "44947003": ("Structure of erector spinae muscle (body structure)", [])
    },
    "StructureOfEsophagealArtery": {
        "206034008": ("Structure of esophageal artery (body structure)", [])
    },
    "StructureOfEsophagealLymphNode": {
        "11899006": ("Structure of esophageal lymph node (body structure)", [])
    },
    "StructureOfExternalIliacArtery": {
        "113269004": ("Structure of external iliac artery (body structure)", [])
    },
    "StructureOfExternalIliacLymphNode": {
        "65349008": ("Structure of external iliac lymph node (body structure)", [])
    },
    "StructureOfExternalIliacVein": {
        "63507001": ("Structure of external iliac vein (body structure)", [])
    },
    "StructureOfExternalIntercostalMuscle": {
        "53967007": ("Structure of external intercostal muscle (body structure)", [])
    },
    "StructureOfExternalJugularVein": {
        "71585003": ("Structure of external jugular vein (body structure)", [])
    },
    "StructureOfExternalLimitingMembraneOfRetina": {
        "76710003": (
            "Structure of external limiting membrane of retina (body structure)",
            [],
        )
    },
    "StructureOfExternalMammaryLymphNode": {
        "421988007": ("Structure of external mammary lymph node (body structure)", [])
    },
    "StructureOfExtrapulmonaryLymphNodeOfLung": {
        "363537007": (
            "Structure of extrapulmonary lymph node of lung (body structure)",
            [],
        )
    },
    "StructureOfEyeProper": {
        "81745001": ("Structure of eye proper (body structure)", [])
    },
    "StructureOfEyelash": {"85803001": ("Structure of eyelash (body structure)", [])},
    "StructureOfFacialArtery": {
        "23074001": ("Structure of facial artery (body structure)", [])
    },
    "StructureOfFacialLymphNode": {
        "48918001": ("Structure of facial lymph node (body structure)", [])
    },
    "StructureOfFemaleInternalGenitalOrgan": {
        "699886009": ("Structure of female internal genital organ (body structure)", [])
    },
    "StructureOfFemoralArtery": {
        "7657000": ("Structure of femoral artery (body structure)", [])
    },
    "StructureOfFibularLymphNode": {
        "31171007": ("Structure of fibular lymph node (body structure)", [])
    },
    "StructureOfFifthIntercostalSpace": {
        "117137004": ("Structure of fifth intercostal space (body structure)", [])
    },
    "StructureOfFifthLumbarArtery": {
        "397411003": ("Structure of fifth lumbar artery (body structure)", [])
    },
    "StructureOfFirstDiagonalBranchOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "91750005": (
            "Structure of first diagonal branch of anterior descending branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfFirstIntercostalSpace": {
        "117133000": ("Structure of first intercostal space (body structure)", [])
    },
    "StructureOfFirstLeftPosterolateralBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91757008": (
            "Structure of first left posterolateral branch of circumflex branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfFirstLumbarArtery": {
        "397407009": ("Structure of first lumbar artery (body structure)", [])
    },
    "StructureOfFirstObtuseMarginalBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91754001": (
            "Structure of first obtuse marginal branch of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfFirstRightPosterolateralBranchOfThePosteriorDescendingBranchOfRightCoronaryArtery": {
        "91761002": (
            "Structure of first right posterolateral branch of the posterior descending branch of "
            "right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfFontanelOfSkull": {
        "79361005": ("Structure of fontanel of skull (body structure)", [])
    },
    "StructureOfForamenMagnum": {
        "24532009": ("Structure of foramen magnum (body structure)", [])
    },
    "StructureOfForcepsMajor": {
        "80049006": ("Structure of forceps major (body structure)", [])
    },
    "StructureOfForcepsMinor": {
        "42932006": ("Structure of forceps minor (body structure)", [])
    },
    "StructureOfFourthIntercostalSpace": {
        "117136008": ("Structure of fourth intercostal space (body structure)", [])
    },
    "StructureOfFourthLumbarArtery": {
        "397410002": ("Structure of fourth lumbar artery (body structure)", [])
    },
    "StructureOfFoveaCentralis": {
        "67046006": ("Structure of fovea centralis (body structure)", [])
    },
    "StructureOfGastricVein": {
        "110568007": ("Structure of gastric vein (body structure)", [])
    },
    "StructureOfGastroOmentalLymphNode": {
        "83380007": ("Structure of gastro-omental lymph node (body structure)", [])
    },
    "StructureOfGastroduodenalArtery": {
        "37274004": ("Structure of gastroduodenal artery (body structure)", [])
    },
    "StructureOfGenicularArtery": {
        "128559007": ("Structure of genicular artery (body structure)", [])
    },
    "StructureOfGenuOfCorpusCallosum": {
        "70215001": ("Structure of genu of corpus callosum (body structure)", [])
    },
    "StructureOfGlutealLymphNode": {
        "80867000": ("Structure of gluteal lymph node (body structure)", [])
    },
    "StructureOfHabenularCommissure": {
        "6866008": ("Structure of habenular commissure (body structure)", [])
    },
    "StructureOfHeadOfFemur": {
        "2812003": ("Structure of head of femur (body structure)", [])
    },
    "StructureOfHemolymphNode": {
        "18457007": ("Structure of hemolymph node (body structure)", [])
    },
    "StructureOfHepaticArteryProper": {
        "18112008": ("Structure of hepatic artery proper (body structure)", [])
    },
    "StructureOfHepaticLymphNode": {
        "61492009": ("Structure of hepatic lymph node (body structure)", [])
    },
    "StructureOfHepaticVein": {
        "8993003": ("Structure of hepatic vein (body structure)", [])
    },
    "StructureOfHighRightAtrium": {
        "128591008": ("Structure of high right atrium (body structure)", [])
    },
    "StructureOfHilarLymphNode": {
        "53074004": ("Structure of hilar lymph node (body structure)", [])
    },
    "StructureOfHilumOfLung": {
        "46750007": ("Structure of hilum of lung (body structure)", [])
    },
    "StructureOfHypogastricLymphNode": {
        "69255009": ("Structure of hypogastric lymph node (body structure)", [])
    },
    "StructureOfIleocolicLymphNode": {
        "281676003": ("Structure of ileocolic lymph node (body structure)", [])
    },
    "StructureOfIliacArtery": {
        "10293006": ("Structure of iliac artery (body structure)", [])
    },
    "StructureOfIliacLymphNode": {
        "84219008": ("Structure of iliac lymph node (body structure)", [])
    },
    "StructureOfIliocostalisMuscle": {
        "57651003": ("Structure of iliocostalis muscle (body structure)", [])
    },
    "StructureOfInferiorArticularProcessOfVertebra": {
        "317766009": (
            "Structure of inferior articular process of vertebra (body structure)",
            [],
        )
    },
    "StructureOfInferiorAuricularLymphNode": {
        "155237005": ("Structure of inferior auricular lymph node (body structure)", [])
    },
    "StructureOfInferiorFrontoOccipitalFasciculus": {
        "35664009": (
            "Structure of inferior fronto-occipital fasciculus (body structure)",
            [],
        )
    },
    "StructureOfInferiorGlutealLymphNode": {
        "40684008": ("Structure of inferior gluteal lymph node (body structure)", [])
    },
    "StructureOfInferiorHornOfLateralVentricle": {
        "53118009": (
            "Structure of inferior horn of lateral ventricle (body structure)",
            [],
        )
    },
    "StructureOfInferiorInguinalLymphNode": {
        "85380009": ("Structure of inferior inguinal lymph node (body structure)", [])
    },
    "StructureOfInferiorLeftPulmonaryVein": {
        "51249003": ("Structure of inferior left pulmonary vein (body structure)", [])
    },
    "StructureOfInferiorLongitudinalFasciculus": {
        "55233005": (
            "Structure of inferior longitudinal fasciculus (body structure)",
            [],
        )
    },
    "StructureOfInferiorMesentericArtery": {
        "33795007": ("Structure of inferior mesenteric artery (body structure)", [])
    },
    "StructureOfInferiorMesentericLymphNode": {
        "113336002": (
            "Structure of inferior mesenteric lymph node (body structure)",
            [],
        )
    },
    "StructureOfInferiorMesentericVein": {
        "32859001": ("Structure of inferior mesenteric vein (body structure)", [])
    },
    "StructureOfInferiorPancreaticLymphNode": {
        "280915003": (
            "Structure of inferior pancreatic lymph node (body structure)",
            [],
        )
    },
    "StructureOfInferiorPancreaticoduodenalLymphNode": {
        "281227003": (
            "Structure of inferior pancreaticoduodenal lymph node (body structure)",
            [],
        )
    },
    "StructureOfInferiorPhrenicArtery": {
        "29660000": ("Structure of inferior phrenic artery (body structure)", [])
    },
    "StructureOfInferiorRightPulmonaryVein": {
        "113273001": ("Structure of inferior right pulmonary vein (body structure)", [])
    },
    "StructureOfInflowTractOfLeftVentricle": {
        "70238003": ("Structure of inflow tract of left ventricle (body structure)", [])
    },
    "StructureOfInflowTractOfRightVentricle": {
        "8017000": ("Structure of inflow tract of right ventricle (body structure)", [])
    },
    "StructureOfInfrarenalAorta": {
        "28205006": ("Structure of infrarenal aorta (body structure)", [])
    },
    "StructureOfInfrasternalAngle": {
        "5076001": ("Structure of infrasternal angle (body structure)", [])
    },
    "StructureOfInnermostIntercostalMuscle": {
        "24062007": ("Structure of innermost intercostal muscle (body structure)", [])
    },
    "StructureOfInnominateLymphNode": {
        "196821008": ("Structure of innominate lymph node (body structure)", [])
    },
    "StructureOfInteriliacLymphNode": {
        "279271008": ("Structure of interiliac lymph node (body structure)", [])
    },
    "StructureOfInterlobarArteryOfKidney": {
        "274143007": ("Structure of interlobar artery of kidney (body structure)", [])
    },
    "StructureOfInterlobularArteryOfKidney": {
        "274329007": ("Structure of interlobular artery of kidney (body structure)", [])
    },
    "StructureOfIntermediateCommonIliacLymphNode": {
        "64556009": (
            "Structure of intermediate common iliac lymph node (body structure)",
            [],
        )
    },
    "StructureOfInternalAcousticMeatusOfTemporalBone": {
        "61671002": (
            "Structure of internal acoustic meatus of temporal bone (body structure)",
            [],
        )
    },
    "StructureOfInternalIliacArtery": {
        "90024005": ("Structure of internal iliac artery (body structure)", [])
    },
    "StructureOfInternalIliacVein": {
        "40300007": ("Structure of internal iliac vein (body structure)", [])
    },
    "StructureOfInternalIntercostalMuscle": {
        "41313007": ("Structure of internal intercostal muscle (body structure)", [])
    },
    "StructureOfInternalThoracicArtery": {
        "69327007": ("Structure of internal thoracic artery (body structure)", [])
    },
    "StructureOfInterpectoralLymphNode": {
        "420800007": ("Structure of interpectoral lymph node (body structure)", [])
    },
    "StructureOfIntervertebralDiscOfL5AndS1": {
        "715056000": (
            "Structure of intervertebral disc of L5 and S1 (body structure)",
            [],
        )
    },
    "StructureOfIntestinalLymphNode": {
        "36251007": ("Structure of intestinal lymph node (body structure)", [])
    },
    "StructureOfIntracranialVein": {
        "128320002": ("Structure of intracranial vein (body structure)", [])
    },
    "StructureOfIntraglandularParotidLymphNode": {
        "143824007": (
            "Structure of intraglandular parotid lymph node (body structure)",
            [],
        )
    },
    "StructureOfIntramammaryLymphNode": {
        "443808008": ("Structure of intramammary lymph node (body structure)", [])
    },
    "StructureOfIntrapulmonaryLymphNode": {
        "196662004": ("Structure of intrapulmonary lymph node (body structure)", [])
    },
    "StructureOfIsthmusOfThyroidGland": {
        "40867004": ("Structure of isthmus of thyroid gland (body structure)", [])
    },
    "StructureOfJugularLymphNode": {
        "58130000": ("Structure of jugular lymph node (body structure)", [])
    },
    "StructureOfJugularNotchOfSternum": {
        "26493002": ("Structure of jugular notch of sternum (body structure)", [])
    },
    "StructureOfJuxtaintestinalLymphNode": {
        "279609001": ("Structure of juxtaintestinal lymph node (body structure)", [])
    },
    "StructureOfLacrimalArtery": {
        "59749000": ("Structure of lacrimal artery (body structure)", [])
    },
    "StructureOfLaminaOfVertebra": {
        "89340005": ("Structure of lamina of vertebra (body structure)", [])
    },
    "StructureOfLateralAxillaryLymphNode": {
        "33770006": ("Structure of lateral axillary lymph node (body structure)", [])
    },
    "StructureOfLateralCalfPerforator": {
        "714754004": ("Structure of lateral calf perforator (body structure)", [])
    },
    "StructureOfLateralCervicalLymphNode": {
        "68915008": ("Structure of lateral cervical lymph node (body structure)", [])
    },
    "StructureOfLateralCommonIliacLymphNode": {
        "41145006": (
            "Structure of lateral common iliac lymph node (body structure)",
            [],
        )
    },
    "StructureOfLateralCorticospinalTract": {
        "461002": ("Structure of lateral corticospinal tract (body structure)", [])
    },
    "StructureOfLateralExternalIliacLymphNode": {
        "40242007": (
            "Structure of lateral external iliac lymph node (body structure)",
            [],
        )
    },
    "StructureOfLateralJugularLymphNode": {
        "168159002": ("Structure of lateral jugular lymph node (body structure)", [])
    },
    "StructureOfLateralMiddleRegionalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716918005": (
            "Structure of lateral middle regional part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfLateralMiddleRegionalPartOfPeripheralZoneOfRightHalfProstate": {
        "716917000": (
            "Structure of lateral middle regional part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfLateralPericardialLymphNode": {
        "196587000": (
            "Structure of lateral pericardial lymph node (body structure)",
            [],
        )
    },
    "StructureOfLateralPlantarArtery": {
        "44830000": ("Structure of lateral plantar artery (body structure)", [])
    },
    "StructureOfLateralRetropharyngealLymphNode": {
        "167464007": (
            "Structure of lateral retropharyngeal lymph node (body structure)",
            [],
        )
    },
    "StructureOfLateralVesicularLymphNode": {
        "278672000": ("Structure of lateral vesicular lymph node (body structure)", [])
    },
    "StructureOfLatissimusDorsiMuscle": {
        "15665001": ("Structure of latissimus dorsi muscle (body structure)", [])
    },
    "StructureOfLeftAnteriorDivisionOfLeftBranchOfAtrioventricularBundle": {
        "84654008": (
            "Structure of left anterior division of left branch of atrioventricular bundle (body structure)",
            [],
        )
    },
    "StructureOfLeftAuricularAppendage": {
        "33626005": ("Structure of left auricular appendage (body structure)", [])
    },
    "StructureOfLeftBranchOfHepaticArtery": {
        "21807003": ("Structure of left branch of hepatic artery (body structure)", [])
    },
    "StructureOfLeftCarotidSinus": {
        "9775002": ("Structure of left carotid sinus (body structure)", [])
    },
    "StructureOfLeftColicFlexure": {
        "72592005": ("Structure of left colic flexure (body structure)", [])
    },
    "StructureOfLeftFemoralArtery": {
        "113270003": ("Structure of left femoral artery (body structure)", [])
    },
    "StructureOfLeftGastricArtery": {
        "23771002": ("Structure of left gastric artery (body structure)", [])
    },
    "StructureOfLeftHepaticVein": {
        "273202007": ("Structure of left hepatic vein (body structure)", [])
    },
    "StructureOfLeftLobeOfThyroidGland": {
        "79163004": ("Structure of left lobe of thyroid gland (body structure)", [])
    },
    "StructureOfLeftLowerQuadrantOfAbdomen": {
        "68505006": ("Structure of left lower quadrant of abdomen (body structure)", [])
    },
    "StructureOfLeftMainBranchOfPortalVein": {
        "70253006": (
            "Structure of left main branch of portal vein (body structure)",
            [],
        )
    },
    "StructureOfLeftPosteriorDivisionOfLeftBranchOfAtrioventricularBundle": {
        "91085002": (
            "Structure of left posterior division of left branch of atrioventricular bundle (body structure)",
            [],
        )
    },
    "StructureOfLeftPosteriorLateralBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "57823005": (
            "Structure of left posterior lateral branch of circumflex branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfLeftPulmonaryArtery": {
        "50408007": ("Structure of left pulmonary artery (body structure)", [])
    },
    "StructureOfLeftPulmonaryVein": {
        "27706005": ("Structure of left pulmonary vein (body structure)", [])
    },
    "StructureOfLeftSeminalVesicle": {
        "42320003": ("Structure of left seminal vesicle (body structure)", [])
    },
    "StructureOfLeftSinusOfValsalva": {
        "36371001": ("Structure of left sinus of Valsalva (body structure)", [])
    },
    "StructureOfLeftSubclavianArtery": {
        "85235006": ("Structure of left subclavian artery (body structure)", [])
    },
    "StructureOfLeftUpperQuadrantOfAbdomen": {
        "86367003": ("Structure of left upper quadrant of abdomen (body structure)", [])
    },
    "StructureOfLensOfEye": {
        "78076003": ("Structure of lens of eye (body structure)", [])
    },
    "StructureOfLesserTrochanterOfFemur": {
        "55499008": ("Structure of lesser trochanter of femur (body structure)", [])
    },
    "StructureOfLevatorCostaeMuscle": {
        "73930003": ("Structure of levator costae muscle (body structure)", [])
    },
    "StructureOfLigament": {"52082005": ("Structure of ligament (body structure)", [])},
    "StructureOfLigamentumArteriosum": {
        "2160002": ("Structure of ligamentum arteriosum (body structure)", [])
    },
    "StructureOfLimbusOfFossaOvalis": {
        "84712000": ("Structure of limbus of fossa ovalis (body structure)", [])
    },
    "StructureOfLingualArtery": {
        "113264009": ("Structure of lingual artery (body structure)", [])
    },
    "StructureOfLobarArteryOfKidney": {
        "274060004": ("Structure of lobar artery of kidney (body structure)", [])
    },
    "StructureOfLobeOfLung": {
        "31094006": ("Structure of lobe of lung (body structure)", [])
    },
    "StructureOfLobeOfThyroidGland": {
        "119281005": ("Structure of lobe of thyroid gland (body structure)", [])
    },
    "StructureOfLongissimusMuscle": {
        "88340001": ("Structure of longissimus muscle (body structure)", [])
    },
    "StructureOfLowRightAtrium": {
        "128594000": ("Structure of low right atrium (body structure)", [])
    },
    "StructureOfLowerInnerQuadrantOfBreast": {
        "19100000": ("Structure of lower inner quadrant of breast (body structure)", [])
    },
    "StructureOfLowerLobeOfLung": {
        "90572001": ("Structure of lower lobe of lung (body structure)", [])
    },
    "StructureOfLowerOuterQuadrantOfBreast": {
        "33564002": ("Structure of lower outer quadrant of breast (body structure)", [])
    },
    "StructureOfLumbarArtery": {
        "34635009": ("Structure of lumbar artery (body structure)", [])
    },
    "StructureOfLumbarLymphNode": {
        "8334002": ("Structure of lumbar lymph node (body structure)", [])
    },
    "StructureOfLumenOfBodySystem": {
        "113342003": ("Structure of lumen of body system (body structure)", [])
    },
    "StructureOfLungAndOrMediastinum": {
        "310787001": ("Structure of lung and/or mediastinum (body structure)", [])
    },
    "StructureOfLymphNode": {
        "59441001": ("Structure of lymph node (body structure)", [])
    },
    "StructureOfLymphNodeOfEpiploicForamen": {
        "68878000": ("Structure of lymph node of epiploic foramen (body structure)", [])
    },
    "StructureOfLymphNodeOfGreaterCurvatureOfStomach": {
        "76878005": (
            "Structure of lymph node of greater curvature of stomach (body structure)",
            [],
        )
    },
    "StructureOfLymphNodeOfHead": {
        "13482005": ("Structure of lymph node of head (body structure)", [])
    },
    "StructureOfLymphNodeOfHeadAndNeck": {
        "312501005": ("Structure of lymph node of head and neck (body structure)", [])
    },
    "StructureOfLymphNodeOfLesserCurvatureOfStomach": {
        "279784003": (
            "Structure of lymph node of lesser curvature of stomach (body structure)",
            [],
        )
    },
    "StructureOfLymphNodeOfMesentery": {
        "279795009": ("Structure of lymph node of mesentery (body structure)", [])
    },
    "StructureOfLymphNodeOfThorax": {
        "47109002": ("Structure of lymph node of thorax (body structure)", [])
    },
    "StructureOfLymphNodeRingOfCardiaOfStomach": {
        "279866008": (
            "Structure of lymph node ring of cardia of stomach (body structure)",
            [],
        )
    },
    "StructureOfLymphaticSystem": {
        "89890002": ("Structure of lymphatic system (body structure)", [])
    },
    "StructureOfLymphaticVessel": {
        "83555006": ("Structure of lymphatic vessel (body structure)", [])
    },
    "StructureOfMaleExternalUrethralSphincter": {
        "717027004": (
            "Structure of male external urethral sphincter (body structure)",
            [],
        )
    },
    "StructureOfMandibularIncisorTooth": {
        "442274007": ("Structure of mandibular incisor tooth (body structure)", [])
    },
    "StructureOfMandibularLeftCanineTooth": {
        "39844006": ("Structure of mandibular left canine tooth (body structure)", [])
    },
    "StructureOfMandibularLeftCentralIncisorTooth": {
        "113278005": (
            "Structure of mandibular left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftFirstMolarTooth": {
        "89625000": (
            "Structure of mandibular left first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftFirstPremolarTooth": {
        "2400006": (
            "Structure of mandibular left first premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftLateralIncisorTooth": {
        "77130001": (
            "Structure of mandibular left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftSecondMolarTooth": {
        "48402004": (
            "Structure of mandibular left second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftSecondPremolarTooth": {
        "24573005": (
            "Structure of mandibular left second premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLeftThirdMolarTooth": {
        "74344005": (
            "Structure of mandibular left third molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularLymphNode": {
        "155338003": ("Structure of mandibular lymph node (body structure)", [])
    },
    "StructureOfMandibularRightCanineTooth": {
        "47055002": ("Structure of mandibular right canine tooth (body structure)", [])
    },
    "StructureOfMandibularRightCentralIncisorTooth": {
        "15422005": (
            "Structure of mandibular right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightFirstMolarTooth": {
        "28480000": (
            "Structure of mandibular right first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightFirstPremolarTooth": {
        "80140008": (
            "Structure of mandibular right first premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightLateralIncisorTooth": {
        "82628004": (
            "Structure of mandibular right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightSecondMolarTooth": {
        "40005008": (
            "Structure of mandibular right second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightSecondPremolarTooth": {
        "8873007": (
            "Structure of mandibular right second premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMandibularRightThirdMolarTooth": {
        "38994002": (
            "Structure of mandibular right third molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMarginalBranchOfRightCoronaryArtery": {
        "22765000": (
            "Structure of marginal branch of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryIncisorTooth": {
        "442100006": ("Structure of maxillary incisor tooth (body structure)", [])
    },
    "StructureOfMaxillaryLeftCanineTooth": {
        "72876007": ("Structure of maxillary left canine tooth (body structure)", [])
    },
    "StructureOfMaxillaryLeftCentralIncisorTooth": {
        "31982000": (
            "Structure of maxillary left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftFirstMolarTooth": {
        "23427002": (
            "Structure of maxillary left first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftFirstPremolarTooth": {
        "61897005": (
            "Structure of maxillary left first premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftLateralIncisorTooth": {
        "25748002": (
            "Structure of maxillary left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftSecondMolarTooth": {
        "66303006": (
            "Structure of maxillary left second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftSecondPremolarTooth": {
        "23226009": (
            "Structure of maxillary left second premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryLeftThirdMolarTooth": {
        "87704003": (
            "Structure of maxillary left third molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightCanineTooth": {
        "80647007": ("Structure of maxillary right canine tooth (body structure)", [])
    },
    "StructureOfMaxillaryRightCentralIncisorTooth": {
        "22120004": (
            "Structure of maxillary right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightFirstMolarTooth": {
        "5140004": (
            "Structure of maxillary right first molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightFirstPremolarTooth": {
        "57826002": (
            "Structure of maxillary right first premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightLateralIncisorTooth": {
        "11712009": (
            "Structure of maxillary right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightSecondMolarTooth": {
        "7121006": (
            "Structure of maxillary right second molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightSecondPremolarTooth": {
        "36492000": (
            "Structure of maxillary right second premolar tooth (body structure)",
            [],
        )
    },
    "StructureOfMaxillaryRightThirdMolarTooth": {
        "68085002": (
            "Structure of maxillary right third molar tooth (body structure)",
            [],
        )
    },
    "StructureOfMedialCommonIliacLymphNode": {
        "34625003": ("Structure of medial common iliac lymph node (body structure)", [])
    },
    "StructureOfMedialExternalIliacLymphNode": {
        "42472007": (
            "Structure of medial external iliac lymph node (body structure)",
            [],
        )
    },
    "StructureOfMedialLacunarLymphNode": {
        "23198005": ("Structure of medial lacunar lymph node (body structure)", [])
    },
    "StructureOfMedialPlantarArtery": {
        "74156002": ("Structure of medial plantar artery (body structure)", [])
    },
    "StructureOfMedianCubitalVein": {
        "49852007": ("Structure of median cubital vein (body structure)", [])
    },
    "StructureOfMedianRetropharyngealLymphNode": {
        "167364008": (
            "Structure of median retropharyngeal lymph node (body structure)",
            [],
        )
    },
    "StructureOfMesentericArtery": {
        "86570000": ("Structure of mesenteric artery (body structure)", [])
    },
    "StructureOfMesentericArteryLymphNode": {
        "299993000": ("Structure of mesenteric artery lymph node (body structure)", [])
    },
    "StructureOfMesentericVein": {
        "128583004": ("Structure of mesenteric vein (body structure)", [])
    },
    "StructureOfMidPortionOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "91748002": (
            "Structure of mid portion of anterior descending branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfMidPortionOfCircumflexBranchOfLeftCoronaryArtery": {
        "91753007": (
            "Structure of mid portion of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfMidPortionOfRightCoronaryArtery": {
        "450960006": (
            "Structure of mid portion of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfMidRightAtrium": {
        "128593006": ("Structure of mid right atrium (body structure)", [])
    },
    "StructureOfMidcolicLymphNode": {
        "282031000": ("Structure of midcolic lymph node (body structure)", [])
    },
    "StructureOfMiddleCardiacVein": {
        "73580002": ("Structure of middle cardiac vein (body structure)", [])
    },
    "StructureOfMiddleCerebellarPeduncle": {
        "33723005": ("Structure of middle cerebellar peduncle (body structure)", [])
    },
    "StructureOfMiddleCerebralArtery": {
        "17232002": ("Structure of middle cerebral artery (body structure)", [])
    },
    "StructureOfMiddleHepaticVein": {
        "273099000": ("Structure of middle hepatic vein (body structure)", [])
    },
    "StructureOfMiddleLobeOfLung": {
        "40020002": ("Structure of middle lobe of lung (body structure)", [])
    },
    "StructureOfMiddleLobeOfRightLung": {
        "72481006": ("Structure of middle lobe of right lung (body structure)", [])
    },
    "StructureOfMiddleRegionalPartOfAnteriorFibromuscularStromaOfLeftHalfProstate": {
        "716910003": (
            "Structure of middle regional part of anterior fibromuscular stroma of left half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfMiddleRegionalPartOfAnteriorFibromuscularStromaOfProstate": {
        "716908000": (
            "Structure of middle regional part of anterior fibromuscular stroma of prostate (body structure)",
            [],
        )
    },
    "StructureOfMiddleRegionalPartOfAnteriorFibromuscularStromaOfRightHalfProstate": {
        "716909008": (
            "Structure of middle regional part of anterior fibromuscular stroma of right half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfMiddleRegionalPartOfTransitionZoneOfLeftHalfProstate": {
        "716912006": (
            "Structure of middle regional part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfMiddleRegionalPartOfTransitionZoneOfRightHalfProstate": {
        "716911004": (
            "Structure of middle regional part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfMucousMembraneOfFloorOfMouth": {
        "36152006": (
            "Structure of mucous membrane of floor of mouth (body structure)",
            [],
        )
    },
    "StructureOfMucousMembraneOfLowerLip": {
        "46353006": ("Structure of mucous membrane of lower lip (body structure)", [])
    },
    "StructureOfMucousMembraneOfTongue": {
        "8001006": ("Structure of mucous membrane of tongue (body structure)", [])
    },
    "StructureOfMucousMembraneOfUpperLip": {
        "18444004": ("Structure of mucous membrane of upper lip (body structure)", [])
    },
    "StructureOfMyocardiumOfAnterolateralRegion": {
        "73050001": (
            "Structure of myocardium of anterolateral region (body structure)",
            [],
        )
    },
    "StructureOfMyocardiumOfApexOfHeart": {
        "47962008": ("Structure of myocardium of apex of heart (body structure)", [])
    },
    "StructureOfMyocardiumOfDiaphragmaticRegion": {
        "72542009": (
            "Structure of myocardium of diaphragmatic region (body structure)",
            [],
        )
    },
    "StructureOfMyocardiumOfInferolateralRegion": {
        "16239001": (
            "Structure of myocardium of inferolateral region (body structure)",
            [],
        )
    },
    "StructureOfMyocardiumOfLeftVentricle": {
        "49848007": ("Structure of myocardium of left ventricle (body structure)", [])
    },
    "StructureOfMyocardiumOfPosterolateralRegion": {
        "33272004": (
            "Structure of myocardium of posterolateral region (body structure)",
            [],
        )
    },
    "StructureOfNasolabialLymphNode": {
        "144026003": ("Structure of nasolabial lymph node (body structure)", [])
    },
    "StructureOfNasopharyngealCavity": {
        "18962004": ("Structure of nasopharyngeal cavity (body structure)", [])
    },
    "StructureOfNeckOfRib": {
        "72184008": ("Structure of neck of rib (body structure)", [])
    },
    "StructureOfNinthIntercostalSpace": {
        "117141000": ("Structure of ninth intercostal space (body structure)", [])
    },
    "StructureOfObturatorLymphNode": {
        "36086000": ("Structure of obturator lymph node (body structure)", [])
    },
    "StructureOfOccipitalArtery": {
        "31145008": ("Structure of occipital artery (body structure)", [])
    },
    "StructureOfOccipitalLymphNode": {
        "3916005": ("Structure of occipital lymph node (body structure)", [])
    },
    "StructureOfOccipitalRegionOfScalp": {
        "700032006": ("Structure of occipital region of scalp (body structure)", [])
    },
    "StructureOfOccipitalVein": {
        "32114007": ("Structure of occipital vein (body structure)", [])
    },
    "StructureOfOphthalmicArtery": {
        "53549008": ("Structure of ophthalmic artery (body structure)", [])
    },
    "StructureOfOphthalmicSegmentOfInternalCarotidArtery": {
        "698348000": (
            "Structure of ophthalmic segment of internal carotid artery (body structure)",
            [],
        )
    },
    "StructureOfOpticRadiation": {
        "70105001": ("Structure of optic radiation (body structure)", [])
    },
    "StructureOfOrbitProper": {
        "363654007": ("Structure of orbit proper (body structure)", [])
    },
    "StructureOfOstiumOfLeftCoronaryArtery": {
        "76862008": ("Structure of ostium of left coronary artery (body structure)", [])
    },
    "StructureOfOstiumOfRightCoronaryArtery": {
        "56789007": (
            "Structure of ostium of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfOutflowTractOfLeftVentricle": {
        "13418002": (
            "Structure of outflow tract of left ventricle (body structure)",
            [],
        )
    },
    "StructureOfOutflowTractOfRightVentricle": {
        "44627009": (
            "Structure of outflow tract of right ventricle (body structure)",
            [],
        )
    },
    "StructureOfOvarianArtery": {
        "12052000": ("Structure of ovarian artery (body structure)", [])
    },
    "StructureOfOvarianVein": {
        "976004": ("Structure of ovarian vein (body structure)", [])
    },
    "StructureOfPancreaticLymphNode": {
        "77778009": ("Structure of pancreatic lymph node (body structure)", [])
    },
    "StructureOfPancreaticoduodenalLymphNode": {
        "76659008": ("Structure of pancreaticoduodenal lymph node (body structure)", [])
    },
    "StructureOfPancreaticosplenicLymphNode": {
        "16050005": ("Structure of pancreaticosplenic lymph node (body structure)", [])
    },
    "StructureOfParamammaryLymphNode": {
        "368550005": ("Structure of paramammary lymph node (body structure)", [])
    },
    "StructureOfParametrialLymphNode": {
        "3243006": ("Structure of parametrial lymph node (body structure)", [])
    },
    "StructureOfPararectalLymphNode": {
        "21875007": ("Structure of pararectal lymph node (body structure)", [])
    },
    "StructureOfParasternalLymphNode": {
        "82365008": ("Structure of parasternal lymph node (body structure)", [])
    },
    "StructureOfParasternalRegion": {
        "91691001": ("Structure of parasternal region (body structure)", [])
    },
    "StructureOfParatrachealLymphNode": {
        "65690001": ("Structure of paratracheal lymph node (body structure)", [])
    },
    "StructureOfParavaginalLymphNode": {
        "16228004": ("Structure of paravaginal lymph node (body structure)", [])
    },
    "StructureOfParavesicularLymphNode": {
        "1439000": ("Structure of paravesicular lymph node (body structure)", [])
    },
    "StructureOfParotidLymphNode": {
        "10209003": ("Structure of parotid lymph node (body structure)", [])
    },
    "StructureOfPatellarLigament": {
        "18033002": ("Structure of patellar ligament (body structure)", [])
    },
    "StructureOfPectoralAxillaryLymphNode": {
        "69691007": ("Structure of pectoral axillary lymph node (body structure)", [])
    },
    "StructureOfPedicleOfVertebra": {
        "78972004": ("Structure of pedicle of vertebra (body structure)", [])
    },
    "StructureOfPenileArtery": {
        "282044005": ("Structure of penile artery (body structure)", [])
    },
    "StructureOfPerforatingArteryOfKidney": {
        "15763003": ("Structure of perforating artery of kidney (body structure)", [])
    },
    "StructureOfPericardiophrenicArtery": {
        "3924000": ("Structure of pericardiophrenic artery (body structure)", [])
    },
    "StructureOfPeripheralGlandularZoneOfProstate": {
        "279706003": (
            "Structure of peripheral glandular zone of prostate (body structure)",
            [],
        )
    },
    "StructureOfPermanentMandibularLeftCentralIncisorTooth": {
        "425106001": (
            "Structure of permanent mandibular left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMandibularLeftLateralIncisorTooth": {
        "423331005": (
            "Structure of permanent mandibular left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMandibularRightCentralIncisorTooth": {
        "424575004": (
            "Structure of permanent mandibular right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMandibularRightLateralIncisorTooth": {
        "423937004": (
            "Structure of permanent mandibular right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMaxillaryLeftCentralIncisorTooth": {
        "424399000": (
            "Structure of permanent maxillary left central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMaxillaryLeftLateralIncisorTooth": {
        "423185002": (
            "Structure of permanent maxillary left lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMaxillaryRightCentralIncisorTooth": {
        "422653006": (
            "Structure of permanent maxillary right central incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPermanentMaxillaryRightLateralIncisorTooth": {
        "424877001": (
            "Structure of permanent maxillary right lateral incisor tooth (body structure)",
            [],
        )
    },
    "StructureOfPeronealArtery": {
        "8821006": ("Structure of peroneal artery (body structure)", [])
    },
    "StructureOfPeronealVein": {
        "71758008": ("Structure of peroneal vein (body structure)", [])
    },
    "StructureOfPharyngealRecess": {
        "1849007": ("Structure of pharyngeal recess (body structure)", [])
    },
    "StructureOfPiriformRecess": {
        "6217003": ("Structure of piriform recess (body structure)", [])
    },
    "StructureOfPituitaryFossa": {
        "42575006": ("Structure of pituitary fossa (body structure)", [])
    },
    "StructureOfPoplitealArtery": {
        "43899006": ("Structure of popliteal artery (body structure)", [])
    },
    "StructureOfPoplitealVein": {
        "56849005": ("Structure of popliteal vein (body structure)", [])
    },
    "StructureOfPostductalRegionOfAorticArch": {
        "60835009": (
            "Structure of postductal region of aortic arch (body structure)",
            [],
        )
    },
    "StructureOfPosteriorApicalPartOfTransitionZoneOfLeftHalfProstate": {
        "716933002": (
            "Structure of posterior apical part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorApicalPartOfTransitionZoneOfRightHalfProstate": {
        "716932007": (
            "Structure of posterior apical part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorBasalPartOfTransitionZoneOfLeftHalfProstate": {
        "716899002": (
            "Structure of posterior basal part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorBasalPartOfTransitionZoneOfRightHalfProstate": {
        "716898005": (
            "Structure of posterior basal part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorCerebralArtery": {
        "70382005": ("Structure of posterior cerebral artery (body structure)", [])
    },
    "StructureOfPosteriorCommissureOfLabiaMajora": {
        "4019005": (
            "Structure of posterior commissure of labia majora (body structure)",
            [],
        )
    },
    "StructureOfPosteriorCommunicatingArtery": {
        "43119007": ("Structure of posterior communicating artery (body structure)", [])
    },
    "StructureOfPosteriorDescendingBranchOfRightCoronaryArtery": {
        "53655008": (
            "Structure of posterior descending branch of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfPosteriorDescendingCoronaryArtery": {
        "443113009": (
            "Structure of posterior descending coronary artery (body structure)",
            [],
        )
    },
    "StructureOfPosteriorHornOfLateralVentricle": {
        "52943005": (
            "Structure of posterior horn of lateral ventricle (body structure)",
            [],
        )
    },
    "StructureOfPosteriorLateralBranchOfRightCoronaryArtery": {
        "17269004": (
            "Structure of posterior lateral branch of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfPosteriorMediastinalLymphNode": {
        "25447008": (
            "Structure of posterior mediastinal lymph node (body structure)",
            [],
        )
    },
    "StructureOfPosteriorMiddleRegionalPartOfTransitionZoneOfLeftHalfProstate": {
        "716916009": (
            "Structure of posterior middle regional part of transition zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorMiddleRegionalPartOfTransitionZoneOfRightHalfProstate": {
        "716915008": (
            "Structure of posterior middle regional part of transition zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteriorSegmentOfRightUpperLobeOfLung": {
        "3236000": (
            "Structure of posterior segment of right upper lobe of lung (body structure)",
            [],
        )
    },
    "StructureOfPosteriorSinusOfValsalva": {
        "24865005": ("Structure of posterior sinus of Valsalva (body structure)", [])
    },
    "StructureOfPosteriorTibialArtery": {
        "13363002": ("Structure of posterior tibial artery (body structure)", [])
    },
    "StructureOfPosteriorTibialLymphNode": {
        "303623000": ("Structure of posterior tibial lymph node (body structure)", [])
    },
    "StructureOfPosteriorTibialVein": {
        "4258007": ("Structure of posterior tibial vein (body structure)", [])
    },
    "StructureOfPosteriorVeinOfLeftVentricle": {
        "195416006": (
            "Structure of posterior vein of left ventricle (body structure)",
            [],
        )
    },
    "StructureOfPosterolateralApicalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716939003": (
            "Structure of posterolateral apical part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosterolateralApicalPartOfPeripheralZoneOfRightHalfProstate": {
        "716938006": (
            "Structure of posterolateral apical part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosterolateralBasalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716907005": (
            "Structure of posterolateral basal part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosterolateralBasalPartOfPeripheralZoneOfRightHalfProstate": {
        "716906001": (
            "Structure of posterolateral basal part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosterolateralMiddleRegionalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716922000": (
            "Structure of posterolateral middle regional part of peripheral zone of left half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfPosterolateralMiddleRegionalPartOfPeripheralZoneOfRightHalfProstate": {
        "716921007": (
            "Structure of posterolateral middle regional part of peripheral zone of right half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfPosteromedialApicalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716941002": (
            "Structure of posteromedial apical part of peripheral zone of left half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteromedialApicalPartOfPeripheralZoneOfRightHalfProstate": {
        "716940001": (
            "Structure of posteromedial apical part of peripheral zone of right half prostate (body structure)",
            [],
        )
    },
    "StructureOfPosteromedialMiddleRegionalPartOfPeripheralZoneOfLeftHalfProstate": {
        "716924004": (
            "Structure of posteromedial middle regional part of peripheral zone of left half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfPosteromedialMiddleRegionalPartOfPeripheralZoneOfRightHalfProstate": {
        "716923005": (
            "Structure of posteromedial middle regional part of peripheral zone of right half prostate (body "
            "structure)",
            [],
        )
    },
    "StructureOfPostvesicularLymphNode": {
        "278571002": ("Structure of postvesicular lymph node (body structure)", [])
    },
    "StructureOfPrececalLymphNode": {
        "281765006": ("Structure of prececal lymph node (body structure)", [])
    },
    "StructureOfPreductalRegionOfAorticArch": {
        "75397005": (
            "Structure of preductal region of aortic arch (body structure)",
            [],
        )
    },
    "StructureOfPrelaryngealLymphNode": {
        "74203007": ("Structure of prelaryngeal lymph node (body structure)", [])
    },
    "StructureOfPrepericardialLymphNode": {
        "196516004": ("Structure of prepericardial lymph node (body structure)", [])
    },
    "StructureOfPresymphysialLymphNode": {
        "6413002": ("Structure of presymphysial lymph node (body structure)", [])
    },
    "StructureOfPretrachealLymphNode": {
        "168460001": ("Structure of pretracheal lymph node (body structure)", [])
    },
    "StructureOfPrevertebralLymphNode": {
        "196446004": ("Structure of prevertebral lymph node (body structure)", [])
    },
    "StructureOfPrevesicularLymphNode": {
        "11740004": ("Structure of prevesicular lymph node (body structure)", [])
    },
    "StructureOfPrimitivePulmonaryArtery": {
        "91707000": ("Structure of primitive pulmonary artery (body structure)", [])
    },
    "StructureOfProfundaFemorisArtery": {
        "31677005": ("Structure of profunda femoris artery (body structure)", [])
    },
    "StructureOfProfundaFemorisVein": {
        "23438002": ("Structure of profunda femoris vein (body structure)", [])
    },
    "StructureOfPromontoryCommonIliacLymphNode": {
        "279189002": (
            "Structure of promontory common iliac lymph node (body structure)",
            [],
        )
    },
    "StructureOfProximalPortionOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "68787002": (
            "Structure of proximal portion of anterior descending branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfProximalPortionOfCircumflexBranchOfLeftCoronaryArtery": {
        "52433000": (
            "Structure of proximal portion of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfProximalPortionOfRightCoronaryArtery": {
        "91083009": (
            "Structure of proximal portion of right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfPurkinjeFibers": {
        "13050003": ("Structure of purkinje fibers (body structure)", [])
    },
    "StructureOfPutamen": {"89278009": ("Structure of putamen (body structure)", [])},
    "StructureOfPyloricLymphNode": {
        "24889003": ("Structure of pyloric lymph node (body structure)", [])
    },
    "StructureOfRadialArtery": {
        "45631007": ("Structure of radial artery (body structure)", [])
    },
    "StructureOfRadialVein": {
        "52359001": ("Structure of radial vein (body structure)", [])
    },
    "StructureOfRectouterinePouch": {
        "53843000": ("Structure of rectouterine pouch (body structure)", [])
    },
    "StructureOfRenalArtery": {
        "2841007": ("Structure of renal artery (body structure)", [])
    },
    "StructureOfRenalHilarLymphNode": {
        "249708006": ("Structure of renal hilar lymph node (body structure)", [])
    },
    "StructureOfRenalVein": {
        "56400007": ("Structure of renal vein (body structure)", [])
    },
    "StructureOfRetrocecalLymphNode": {
        "281847004": ("Structure of retrocecal lymph node (body structure)", [])
    },
    "StructureOfRetromolarAreaOfMouth": {
        "85816001": ("Structure of retromolar area of mouth (body structure)", [])
    },
    "StructureOfRetroperitonealLymphNode": {
        "91394001": ("Structure of retroperitoneal lymph node (body structure)", [])
    },
    "StructureOfRetropharyngealLymphNode": {
        "25683005": ("Structure of retropharyngeal lymph node (body structure)", [])
    },
    "StructureOfRetropyloricLymphNode": {
        "280402004": ("Structure of retropyloric lymph node (body structure)", [])
    },
    "StructureOfRightAuricularAppendage": {
        "68300000": ("Structure of right auricular appendage (body structure)", [])
    },
    "StructureOfRightBranchOfAtrioventricularBundle": {
        "57383004": (
            "Structure of right branch of atrioventricular bundle (body structure)",
            [],
        )
    },
    "StructureOfRightBranchOfHepaticArtery": {
        "69421009": ("Structure of right branch of hepatic artery (body structure)", [])
    },
    "StructureOfRightColicFlexure": {
        "48338005": ("Structure of right colic flexure (body structure)", [])
    },
    "StructureOfRightFemoralArtery": {
        "69833005": ("Structure of right femoral artery (body structure)", [])
    },
    "StructureOfRightHepaticVein": {
        "272998002": ("Structure of right hepatic vein (body structure)", [])
    },
    "StructureOfRightLobeOfThyroidGland": {
        "29565003": ("Structure of right lobe of thyroid gland (body structure)", [])
    },
    "StructureOfRightLowerQuadrantOfAbdomen": {
        "48544008": (
            "Structure of right lower quadrant of abdomen (body structure)",
            [],
        )
    },
    "StructureOfRightMainBranchOfPortalVein": {
        "73931004": (
            "Structure of right main branch of portal vein (body structure)",
            [],
        )
    },
    "StructureOfRightMiddlePulmonaryVein": {
        "443714006": ("Structure of right middle pulmonary vein (body structure)", [])
    },
    "StructureOfRightPulmonaryArtery": {
        "78480002": ("Structure of right pulmonary artery (body structure)", [])
    },
    "StructureOfRightPulmonaryVein": {
        "91539005": ("Structure of right pulmonary vein (body structure)", [])
    },
    "StructureOfRightSeminalVesicle": {
        "74308000": ("Structure of right seminal vesicle (body structure)", [])
    },
    "StructureOfRightSinusOfValsalva": {
        "89093001": ("Structure of right sinus of Valsalva (body structure)", [])
    },
    "StructureOfRightSubclavianArtery": {
        "29700009": ("Structure of right subclavian artery (body structure)", [])
    },
    "StructureOfRightUpperQuadrantOfAbdomen": {
        "50519007": (
            "Structure of right upper quadrant of abdomen (body structure)",
            [],
        )
    },
    "StructureOfRootOfTongue": {
        "47975008": ("Structure of root of tongue (body structure)", [])
    },
    "StructureOfSacralLymphNode": {
        "79926007": ("Structure of sacral lymph node (body structure)", [])
    },
    "StructureOfSaphenofemoralJunction": {
        "128587003": ("Structure of saphenofemoral junction (body structure)", [])
    },
    "StructureOfScaleneLymphNode": {
        "81132008": ("Structure of scalene lymph node (body structure)", [])
    },
    "StructureOfScalenusAnteriorMuscle": {
        "50755001": ("Structure of scalenus anterior muscle (body structure)", [])
    },
    "StructureOfSecondDiagonalBranchOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "91751009": (
            "Structure of second diagonal branch of anterior descending branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfSecondIntercostalSpace": {
        "117134006": ("Structure of second intercostal space (body structure)", [])
    },
    "StructureOfSecondLeftPosterolateralBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91758003": (
            "Structure of second left posterolateral branch of circumflex branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfSecondLumbarArtery": {
        "397408004": ("Structure of second lumbar artery (body structure)", [])
    },
    "StructureOfSecondObtuseMarginalBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91755000": (
            "Structure of second obtuse marginal branch of circumflex branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfSecondRightPosterolateralBranchOfThePosteriorDescendingBranchOfRightCoronaryArtery": {
        "91762009": (
            "Structure of second right posterolateral branch of the posterior descending branch of "
            "right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfSegmentalBranchOfRenalArtery": {
        "120234003": (
            "Structure of segmental branch of renal artery (body structure)",
            [],
        )
    },
    "StructureOfSerratusAnteriorMuscle": {
        "18346003": ("Structure of serratus anterior muscle (body structure)", [])
    },
    "StructureOfSesamoidBoneOfFoot": {
        "58742003": ("Structure of sesamoid bone of foot (body structure)", [])
    },
    "StructureOfSeventhIntercostalSpace": {
        "117139001": ("Structure of seventh intercostal space (body structure)", [])
    },
    "StructureOfShaftOfRib": {
        "41601005": ("Structure of shaft of rib (body structure)", [])
    },
    "StructureOfSigmoidLymphNode": {
        "30024008": ("Structure of sigmoid lymph node (body structure)", [])
    },
    "StructureOfSinoatrialNode": {
        "88210001": ("Structure of sinoatrial node (body structure)", [])
    },
    "StructureOfSinusOfValsalva": {
        "81128002": ("Structure of sinus of Valsalva (body structure)", [])
    },
    "StructureOfSixthIntercostalSpace": {
        "117138009": ("Structure of sixth intercostal space (body structure)", [])
    },
    "StructureOfSixthLumbarArtery": {
        "397412005": ("Structure of sixth lumbar artery (body structure)", [])
    },
    "StructureOfSmallCardiacVein": {
        "49082002": ("Structure of small cardiac vein (body structure)", [])
    },
    "StructureOfSmallSaphenousVein": {
        "26805005": ("Structure of small saphenous vein (body structure)", [])
    },
    "StructureOfSmallestCardiacVein": {
        "195073003": ("Structure of smallest cardiac vein (body structure)", [])
    },
    "StructureOfSpinalisMuscle": {
        "4317002": ("Structure of spinalis muscle (body structure)", [])
    },
    "StructureOfSpinousProcessOfVertebra": {
        "55678000": ("Structure of spinous process of vertebra (body structure)", [])
    },
    "StructureOfSplenicArtery": {
        "22083002": ("Structure of splenic artery (body structure)", [])
    },
    "StructureOfSplenicLymphNode": {
        "280824006": ("Structure of splenic lymph node (body structure)", [])
    },
    "StructureOfSplenicVein": {
        "35819009": ("Structure of splenic vein (body structure)", [])
    },
    "StructureOfSpleniumOfCorpusCallosum": {
        "23347006": ("Structure of splenium of corpus callosum (body structure)", [])
    },
    "StructureOfSternalAngle": {
        "44612009": ("Structure of sternal angle (body structure)", [])
    },
    "StructureOfSternocleidomastoidMuscle": {
        "22823000": ("Structure of sternocleidomastoid muscle (body structure)", [])
    },
    "StructureOfSubarachnoidSpaceOfBrain": {
        "33930006": ("Structure of subarachnoid space of brain (body structure)", [])
    },
    "StructureOfSubcarinalLymphNode": {
        "28330007": ("Structure of subcarinal lymph node (body structure)", [])
    },
    "StructureOfSubclavianArtery": {
        "36765005": ("Structure of subclavian artery (body structure)", [])
    },
    "StructureOfSubclavianLymphNode": {
        "421861001": ("Structure of subclavian lymph node (body structure)", [])
    },
    "StructureOfSubclavianVein": {
        "9454009": ("Structure of subclavian vein (body structure)", [])
    },
    "StructureOfSubcostalMuscle": {
        "64658001": ("Structure of subcostal muscle (body structure)", [])
    },
    "StructureOfSubmandibularLymphNode": {
        "59503006": ("Structure of submandibular lymph node (body structure)", [])
    },
    "StructureOfSubmentalLymphNode": {
        "46055009": ("Structure of submental lymph node (body structure)", [])
    },
    "StructureOfSubpyloricLymphNode": {
        "280314006": ("Structure of subpyloric lymph node (body structure)", [])
    },
    "StructureOfSubscapularAxillaryLymphNode": {
        "12196003": (
            "Structure of subscapular axillary lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperficialAnteriorCervicalLymphNode": {
        "168557005": (
            "Structure of superficial anterior cervical lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperficialInguinalLymphNode": {
        "113340006": (
            "Structure of superficial inguinal lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperficialLateralCervicalLymphNode": {
        "167965000": (
            "Structure of superficial lateral cervical lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperficialLymphNode": {
        "90606007": ("Structure of superficial lymph node (body structure)", [])
    },
    "StructureOfSuperficialPalmarVenousArch": {
        "368479001": (
            "Structure of superficial palmar venous arch (body structure)",
            [],
        )
    },
    "StructureOfSuperficialPoplitealLymphNode": {
        "12728001": (
            "Structure of superficial popliteal lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperficialTemporalArtery": {
        "15672000": ("Structure of superficial temporal artery (body structure)", [])
    },
    "StructureOfSuperiorArticularProcessOfVertebra": {
        "317665004": (
            "Structure of superior articular process of vertebra (body structure)",
            [],
        )
    },
    "StructureOfSuperiorCerebellarPeduncle": {
        "11089000": ("Structure of superior cerebellar peduncle (body structure)", [])
    },
    "StructureOfSuperiorFrontoOccipitalFasciculus": {
        "13958008": (
            "Structure of superior fronto-occipital fasciculus (body structure)",
            [],
        )
    },
    "StructureOfSuperiorGlutealLymphNode": {
        "76290003": ("Structure of superior gluteal lymph node (body structure)", [])
    },
    "StructureOfSuperiorLeftPulmonaryVein": {
        "43863001": ("Structure of superior left pulmonary vein (body structure)", [])
    },
    "StructureOfSuperiorLongitudinalFasciculus": {
        "89202009": (
            "Structure of superior longitudinal fasciculus (body structure)",
            [],
        )
    },
    "StructureOfSuperiorMesentericLymphNode": {
        "49394004": ("Structure of superior mesenteric lymph node (body structure)", [])
    },
    "StructureOfSuperiorMesentericVein": {
        "90771006": ("Structure of superior mesenteric vein (body structure)", [])
    },
    "StructureOfSuperiorPancreaticLymphNode": {
        "280999005": (
            "Structure of superior pancreatic lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperiorPancreaticoduodenalLymphNode": {
        "281320004": (
            "Structure of superior pancreaticoduodenal lymph node (body structure)",
            [],
        )
    },
    "StructureOfSuperiorPhrenicArtery": {
        "38991005": ("Structure of superior phrenic artery (body structure)", [])
    },
    "StructureOfSuperiorRectalLymphNode": {
        "68881005": ("Structure of superior rectal lymph node (body structure)", [])
    },
    "StructureOfSuperiorRightPulmonaryVein": {
        "8629005": ("Structure of superior right pulmonary vein (body structure)", [])
    },
    "StructureOfSuperiorThyroidArtery": {
        "72021004": ("Structure of superior thyroid artery (body structure)", [])
    },
    "StructureOfSuperiorTracheobronchialLymphNode": {
        "67941004": (
            "Structure of superior tracheobronchial lymph node (body structure)",
            [],
        )
    },
    "StructureOfSupraclavicularLymphNode": {
        "76838003": ("Structure of supraclavicular lymph node (body structure)", [])
    },
    "StructureOfSupraclavicularRegionOfNeck": {
        "77621008": ("Structure of supraclavicular region of neck (body structure)", [])
    },
    "StructureOfSuprapyloricLymphNode": {
        "280216006": ("Structure of suprapyloric lymph node (body structure)", [])
    },
    "StructureOfSuprarenalAorta": {
        "1918003": ("Structure of suprarenal aorta (body structure)", [])
    },
    "StructureOfSurfaceOfBone": {
        "425647002": ("Structure of surface of bone (body structure)", [])
    },
    "StructureOfTapetumOfCorpusCallosum": {
        "60105000": ("Structure of tapetum of corpus callosum (body structure)", [])
    },
    "StructureOfTeethGumsAndSupportingStructures": {
        "28035005": (
            "Structure of teeth, gums, and supporting structures (body structure)",
            [],
        )
    },
    "StructureOfTelencephalon": {
        "11628009": ("Structure of telencephalon (body structure)", [])
    },
    "StructureOfTendonOfTodaro": {
        "128595004": ("Structure of tendon of Todaro (body structure)", [])
    },
    "StructureOfTenthIntercostalSpace": {
        "117142007": ("Structure of tenth intercostal space (body structure)", [])
    },
    "StructureOfTeresMajorMuscle": {
        "1193009": ("Structure of teres major muscle (body structure)", [])
    },
    "StructureOfTeresMinorMuscle": {
        "51159009": ("Structure of teres minor muscle (body structure)", [])
    },
    "StructureOfTerminalPortionOfInternalCarotidArtery": {
        "415646005": (
            "Structure of terminal portion of internal carotid artery (body structure)",
            [],
        )
    },
    "StructureOfTesticularArtery": {
        "27175001": ("Structure of testicular artery (body structure)", [])
    },
    "StructureOfTesticularVein": {
        "31688004": ("Structure of testicular vein (body structure)", [])
    },
    "StructureOfThighPerforator": {
        "714759009": ("Structure of thigh perforator (body structure)", [])
    },
    "StructureOfThirdDiagonalBranchOfAnteriorDescendingBranchOfLeftCoronaryArtery": {
        "91752002": (
            "Structure of third diagonal branch of anterior descending branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfThirdIntercostalSpace": {
        "117135007": ("Structure of third intercostal space (body structure)", [])
    },
    "StructureOfThirdLeftPosterolateralBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91759006": (
            "Structure of third left posterolateral branch of circumflex branch of left coronary artery (body "
            "structure)",
            [],
        )
    },
    "StructureOfThirdLumbarArtery": {
        "397409007": ("Structure of third lumbar artery (body structure)", [])
    },
    "StructureOfThirdObtuseMarginalBranchOfCircumflexBranchOfLeftCoronaryArtery": {
        "91756004": (
            "Structure of third obtuse marginal branch of circumflex branch of left coronary artery (body structure)",
            [],
        )
    },
    "StructureOfThirdRightPosterolateralBranchOfThePosteriorDescendingBranchOfRightCoronaryArtery": {
        "91763004": (
            "Structure of third right posterolateral branch of the posterior descending branch of "
            "right coronary artery (body structure)",
            [],
        )
    },
    "StructureOfThoracicInlet": {
        "42973007": ("Structure of thoracic inlet (body structure)", [])
    },
    "StructureOfThyrocervicalTrunk": {
        "6538005": ("Structure of thyrocervical trunk (body structure)", [])
    },
    "StructureOfTibialLymphNode": {
        "80769008": ("Structure of tibial lymph node (body structure)", [])
    },
    "StructureOfTransitionZoneOfProstate": {
        "399384005": ("Structure of transition zone of prostate (body structure)", [])
    },
    "StructureOfTransverseAorticArch": {
        "443281009": ("Structure of transverse aortic arch (body structure)", [])
    },
    "StructureOfTransverseProcessOfVertebra": {
        "73400003": ("Structure of transverse process of vertebra (body structure)", [])
    },
    "StructureOfTransverseThoracisMuscle": {
        "88454005": ("Structure of transverse thoracis muscle (body structure)", [])
    },
    "StructureOfTrapeziusMuscle": {
        "31764008": ("Structure of trapezius muscle (body structure)", [])
    },
    "StructureOfTrunkOfPulmonaryArtery": {
        "45341000": ("Structure of trunk of pulmonary artery (body structure)", [])
    },
    "StructureOfTubalTonsil": {
        "21058000": ("Structure of tubal tonsil (body structure)", [])
    },
    "StructureOfUlnarArtery": {
        "44984001": ("Structure of ulnar artery (body structure)", [])
    },
    "StructureOfUlnarVein": {
        "17623008": ("Structure of ulnar vein (body structure)", [])
    },
    "StructureOfUmbilicalArtery": {
        "50536004": ("Structure of umbilical artery (body structure)", [])
    },
    "StructureOfUmbilicalPortionOfPortalVein": {
        "284639000": (
            "Structure of umbilical portion of portal vein (body structure)",
            [],
        )
    },
    "StructureOfUmbilicalVein": {
        "367567000": ("Structure of umbilical vein (body structure)", [])
    },
    "StructureOfUncinateFasciculus": {
        "26230003": ("Structure of uncinate fasciculus (body structure)", [])
    },
    "StructureOfUpperInnerQuadrantOfBreast": {
        "77831004": ("Structure of upper inner quadrant of breast (body structure)", [])
    },
    "StructureOfUpperLobeOfLeftLung": {
        "44714003": ("Structure of upper lobe of left lung (body structure)", [])
    },
    "StructureOfUpperLobeOfLung": {
        "45653009": ("Structure of upper lobe of lung (body structure)", [])
    },
    "StructureOfUpperOuterQuadrantOfBreast": {
        "76365002": ("Structure of upper outer quadrant of breast (body structure)", [])
    },
    "StructureOfUpperUrinaryTractProper": {
        "431491007": ("Structure of upper urinary tract proper (body structure)", [])
    },
    "StructureOfUrinaryBladderCavity": {
        "48367006": ("Structure of urinary bladder cavity (body structure)", [])
    },
    "StructureOfUrinaryTractProper": {
        "431938005": ("Structure of urinary tract proper (body structure)", [])
    },
    "StructureOfUterineArtery": {
        "91079009": ("Structure of uterine artery (body structure)", [])
    },
    "StructureOfUterineParacervicalLymphNode": {
        "5394000": ("Structure of uterine paracervical lymph node (body structure)", [])
    },
    "StructureOfUterineVein": {
        "60028002": ("Structure of uterine vein (body structure)", [])
    },
    "StructureOfVeinOfLowerExtremity": {
        "122774002": ("Structure of vein of lower extremity (body structure)", [])
    },
    "StructureOfVeinOfUpperExtremity": {
        "122775001": ("Structure of vein of upper extremity (body structure)", [])
    },
    "StructureOfVenousNetwork": {
        "34340008": ("Structure of venous network (body structure)", [])
    },
    "StructureOfVentricularVein": {
        "195328002": ("Structure of ventricular vein (body structure)", [])
    },
    "StructureOfVertebralArtery": {
        "85234005": ("Structure of vertebral artery (body structure)", [])
    },
    "StructureOfVertebralColumn": {
        "421060004": ("Structure of vertebral column (body structure)", [])
    },
    "StructureOfVerticalOccipitalFasciculus": {
        "80434005": ("Structure of vertical occipital fasciculus (body structure)", [])
    },
    "StructureOfVisceralPericardium": {
        "6871001": ("Structure of visceral pericardium (body structure)", [])
    },
    "StructureOfVitellineArteryOfPlacenta": {
        "256779006": ("Structure of vitelline artery of placenta (body structure)", [])
    },
    "StructureOfVitellineVeinOfPlacenta": {
        "256875007": ("Structure of vitelline vein of placenta (body structure)", [])
    },
    "StructureOfXiphoidProcessOfSternum": {
        "20298003": ("Structure of xiphoid process of sternum (body structure)", [])
    },
    "StructureSinusOfValsalva": {"81128002": ("Structure Sinus of Valsalva", [12291])},
    "Stryker": {"422954003": ("Stryker", [4012])},
    "StrykerProjection": {"422954003": ("Stryker projection (qualifier value)", [])},
    "StumpyTailCattleDogBreed": {"132636008": ("Stumpy Tail Cattle Dog breed", [7480])},
    "StyrianMountainDogBreed": {"132452003": ("Styrian Mountain dog breed", [7480])},
    "Subacute": {"19939008": ("Subacute", [6130])},
    "SubacuteBacterialEndocarditis": {
        "73774007": ("Subacute bacterial endocarditis", [3700])
    },
    "SubaorticCommonIliacLymphNode": {
        "60227002": ("subaortic common iliac lymph node", [7600, 8134])
    },
    "SubarachnoidSpace": {
        "35951006": ("Subarachnoid space", [7151, 7153, 7192, 8134, 9514])
    },
    "SubarachnoidSpaceStructure": {
        "35951006": ("Subarachnoid space structure (body structure)", [])
    },
    "SubareolarPositionOnMammogram": {
        "129784002": ("Subareolar position on mammogram (finding)", [])
    },
    "SubareolarRegion": {"129784002": ("Subareolar region", [6018, 6019])},
    "Subcapsular": {"61397002": ("Subcapsular", [2])},
    "SubcarinalLymphNode": {"28330007": ("subcarinal lymph node", [7600, 8134])},
    "SubclavianArtery": {
        "36765005": (
            "Subclavian artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12104,
                12107,
            ],
        )
    },
    "SubclavianLymphNode": {"421861001": ("subclavian lymph node", [7600, 8134])},
    "SubclavianVein": {
        "9454009": (
            "Subclavian vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12108,
            ],
        )
    },
    "Subcostal": {"264208000": ("Subcostal", [5])},
    "SubcostalLongAxis": {"399310008": ("Subcostal long axis", [501, 12226])},
    "SubcostalLongAxisView": {
        "399310008": ("Subcostal long axis view (qualifier value)", [])
    },
    "SubcostalMuscle": {
        "64658001": ("Subcostal muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "SubcostalObliqueCoronalView": {
        "443640005": ("Subcostal oblique coronal view", [501, 12226])
    },
    "SubcostalShortAxis": {"399200001": ("Subcostal short axis", [501, 12226])},
    "SubcostalShortAxisView": {
        "399200001": ("Subcostal short axis view (qualifier value)", [])
    },
    "SubcostalShortAxisViewAtAorticValveLevel": {
        "443609003": ("Subcostal short axis view at aortic valve level", [501, 12226])
    },
    "SubcostalShortAxisViewAtMitralValveLevel": {
        "443499004": ("Subcostal short axis view at mitral valve level", [501, 12226])
    },
    "SubcostalShortAxisViewAtPapillaryMuscleLevel": {
        "443160001": (
            "Subcostal short axis view at papillary muscle level",
            [501, 12226],
        )
    },
    "SubcostalShortAxisViewAtVenousInflowLevel": {
        "443500008": ("Subcostal short axis view at venous inflow level", [501, 12226])
    },
    "SubcostalViewOfCardiacOutletsDirectedAnteriorly": {
        "443100003": (
            "Subcostal view of cardiac outlets directed anteriorly",
            [501, 12226],
        )
    },
    "SubcutaneousRoute": {"34206005": ("Subcutaneous route", [11])},
    "SubcutaneousTissue": {"71966008": ("Subcutaneous tissue", [5, 7151, 7166, 7191])},
    "SubcutaneousTissueStructure": {
        "71966008": ("Subcutaneous tissue structure (body structure)", [])
    },
    "SubdiaphragmaticTracerUptake": {
        "429382003": ("Subdiaphragmatic tracer uptake (finding)", [])
    },
    "SubdiaphragmaticUptake": {"429382003": ("Subdiaphragmatic uptake", [3115])},
    "SubfamilyBovinae": {"107007004": ("Subfamily Bovinae (organism)", [])},
    "SubjectRelationshipContext": {
        "408732007": ("Subject relationship context (attribute)", [])
    },
    "SubmandibularArea": {"5713008": ("Submandibular area", [4, 7151, 7192, 9514])},
    "SubmandibularGland": {
        "54019009": (
            "Submandibular gland",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SubmandibularLymphNode": {"59503006": ("submandibular lymph node", [7600, 8134])},
    "SubmandibularSalivaryApparatus": {
        "54019009": ("Submandibular salivary apparatus (body structure)", [])
    },
    "SubmandibularTriangle": {"5713008": ("Submandibular triangle", [4029, 8134])},
    "SubmandibularTriangleStructure": {
        "5713008": ("Submandibular triangle structure (body structure)", [])
    },
    "Submental": {"170887008": ("Submental", [4, 7151, 7192, 8134, 9514])},
    "SubmentalLymphNode": {"46055009": ("submental lymph node", [7600, 8134])},
    "SubmentalTriangleStructure": {
        "170887008": ("Submental triangle structure (body structure)", [])
    },
    "Submentovertical": {"399255003": ("submentovertical", [501, 4010])},
    "SubmentoverticalProjection": {
        "399255003": ("Submentovertical projection (qualifier value)", [])
    },
    "SuboticaWhitePigBreed": {"133212006": ("Subotica White pig breed", [7480])},
    "Subpleural": {"68599007": ("Subpleural", [6124, 6126])},
    "SubpyloricLymphNode": {"280314006": ("subpyloric lymph node", [7600, 8134])},
    "SubscapularAxillaryLymphNode": {
        "12196003": ("subscapular axillary lymph node", [7600, 8134])
    },
    "SubscapularisMuscle": {
        "90588001": ("Subscapularis muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "SubscapularisMuscleStructure": {
        "90588001": ("Subscapularis muscle structure (body structure)", [])
    },
    "SubsegmentalLymphNodeOfTheLung": {
        "127922000": ("subsegmental lymph node of the lung", [7600, 8134])
    },
    "Substance": {"105590001": ("Substance", [7150, 9502, 9580])},
    "SubstantiaNigra": {
        "70007007": ("Substantia nigra", [7151, 7153, 7192, 8134, 9514])
    },
    "SubstantiaNigraStructure": {
        "70007007": ("Substantia nigra structure (body structure)", [])
    },
    "Subxiphoid": {"5076001": ("Subxiphoid", [4, 7151, 7192, 8134, 9514])},
    "Succinylcholine": {"372724004": ("Succinylcholine", [623, 627])},
    "SudanBlackBStain": {"36572009": ("Sudan black B stain", [8112])},
    "SudanBlackStain": {"310805002": ("Sudan black stain", [8112])},
    "SudanBlueStain": {"10740006": ("Sudan blue stain", [8112])},
    "SudanCountryBredHorseBreed": {
        "133145004": ("Sudan Country-Bred horse breed", [7480])
    },
    "SudanIIIStain": {"39777001": ("Sudan III stain", [8112])},
    "SudanIIStain": {"12030009": ("Sudan II stain", [8112])},
    "SudanIVStain": {"69133007": ("Sudan IV stain", [8112])},
    "SudanStain": {"314900004": ("Sudan stain", [8112])},
    "SudaneseFulaniCattleBreed": {
        "133786003": ("Sudanese Fulani cattle breed", [7480])
    },
    "SuddenOnset": {"385315009": ("Acute onset", [6130])},
    "Sufentanil": {"49998007": ("Sufentanil", [65, 66])},
    "SuffolkCattleBreed": {"133275001": ("Suffolk cattle breed", [7480])},
    "SuffolkHorse": {"53567001": ("Suffolk horse (organism)", [])},
    "SuffolkHorseBreed": {"53567001": ("Suffolk horse breed", [7480])},
    "SuffolkSheep": {"72648002": ("Suffolk sheep (organism)", [])},
    "SuffolkSheepBreed": {"72648002": ("Suffolk sheep breed", [7480])},
    "SuiaCattleBreed": {"133832001": ("Suia cattle breed (organism)", [])},
    "SuiaXZebuCattleBreed": {"133832001": ("Suia X zebu cattle breed", [7480])},
    "SuisbuCattleBreed": {"133833006": ("Suisbu cattle breed (organism)", [])},
    "SuisbuXZebuCattleBreed": {"133833006": ("Suisbu X zebu cattle breed", [7480])},
    "SuksunCattleBreed": {"133512007": ("Suksun cattle breed", [7480])},
    "SulawesiHorseBreed": {"133094001": ("Sulawesi horse breed", [7480])},
    "SulfonamideAntibiotic": {"372788003": ("Sulfonamide antibiotic", [621, 622])},
    "SulfonamideClassOfAntibiotic": {
        "372788003": ("Sulfonamide -class of antibiotic- (substance)", [])
    },
    "SumadijaPigBreed": {"132262008": ("Sumadija pig breed", [7480])},
    "SumbaHorseBreed": {"131944007": ("Sumba horse breed", [7480])},
    "SumbawaHorseBreed": {"131945008": ("Sumbawa horse breed", [7480])},
    "SunandiniCattleBreed": {"133834000": ("Sunandini cattle breed (organism)", [])},
    "SunandiniXZebuCattleBreed": {
        "133834000": ("Sunandini X zebu cattle breed", [7480])
    },
    "SunkumaCattleBreed": {"131528009": ("Sunkuma cattle breed", [7480])},
    "SunsetYellowFCFStain": {"22968009": ("sunset yellow FCF stain", [8112])},
    "Superficial": {"26283006": ("Superficial", [2, 3489])},
    "SuperficialAnteriorCervicalLymphNode": {
        "168557005": ("superficial anterior cervical lymph node", [7600, 8134])
    },
    "SuperficialCervicalLymphNode": {
        "279144003": ("superficial cervical lymph node", [7600, 8134])
    },
    "SuperficialFemoralArtery": {
        "181349008": (
            "Superficial Femoral Artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12109,
            ],
        )
    },
    "SuperficialFemoralVein": {
        "397364003": (
            "Superficial Femoral Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12110],
        )
    },
    "SuperficialFemoralVeinStructure": {
        "397364003": ("Superficial femoral vein structure (body structure)", [])
    },
    "SuperficialInguinalLymphNode": {
        "113340006": ("superficial inguinal lymph node", [7600, 8134])
    },
    "SuperficialIntraparotidLymphNode": {
        "68339009": ("superficial intraparotid lymph node", [7600, 8134])
    },
    "SuperficialLateralCervicalLymphNode": {
        "167965000": ("superficial lateral cervical lymph node", [7600, 8134])
    },
    "SuperficialLymphNode": {"90606007": ("superficial lymph node", [7600, 8134])},
    "SuperficialPalmarArch": {
        "26818002": (
            "Superficial Palmar Arch",
            [3827, 7151, 7192, 8134, 9514, 12103, 12107],
        )
    },
    "SuperficialPalmarArchStructure": {
        "26818002": ("Superficial palmar arch structure (body structure)", [])
    },
    "SuperficialPalmarVenousArch": {
        "368479001": (
            "Superficial Palmar Venous Arch",
            [3827, 7151, 7192, 8134, 9514, 12103, 12108],
        )
    },
    "SuperficialParotidLymphNode": {
        "279141006": ("superficial parotid lymph node", [7600, 8134])
    },
    "SuperficialPoplitealLymphNode": {
        "12728001": ("superficial popliteal lymph node", [7600, 8134])
    },
    "SuperficialTemporalArtery": {
        "15672000": (
            "Superficial temporal artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SuperfineMerinoSheepBreed": {
        "132724006": ("Superfine Merino sheep breed", [7480])
    },
    "Superior": {"264217000": ("Superior", [2, 5, 211, 212, 3019, 4005, 6065, 12117])},
    "SuperiorArticularFacetOfAxis": {
        "181900008": ("Superior articular facet of axis", [6115, 8134])
    },
    "SuperiorArticularProcessOfVertebra": {
        "317665004": ("Superior articular process of vertebra", [6115, 8134])
    },
    "SuperiorCerebellarPeduncle": {
        "11089000": (
            "Superior cerebellar peduncle",
            [7151, 7153, 7192, 7701, 7710, 8134, 9514],
        )
    },
    "SuperiorFrontoOccipitalFasciculus": {
        "13958008": ("superior fronto-occipital fasciculus", [7703, 7710, 8134])
    },
    "SuperiorGlutealLymphNode": {
        "76290003": ("superior gluteal lymph node", [7600, 8134])
    },
    "SuperiorLateralInguinalLymphNode": {
        "76704003": ("superior lateral inguinal lymph node", [7600, 8134])
    },
    "SuperiorLeftPulmonaryVein": {
        "43863001": (
            "Superior left pulmonary vein",
            [4, 3010, 3011, 3827, 3840, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SuperiorLongitudinalFasciculus": {
        "89202009": ("Superior longitudinal fasciculus", [7151, 7153, 7192, 8134, 9514])
    },
    "SuperiorMedialInguinalLymphNode": {
        "52554005": ("superior medial inguinal lymph node", [7600, 8134])
    },
    "SuperiorMediastinalLymphNode": {
        "127925003": ("superior mediastinal lymph node", [7600, 8134])
    },
    "SuperiorMesentericArtery": {
        "42258001": (
            "Superior mesenteric artery",
            [4, 3010, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12112, 12125],
        )
    },
    "SuperiorMesentericArteryStructure": {
        "42258001": ("Superior mesenteric artery structure (body structure)", [])
    },
    "SuperiorMesentericLymphNode": {
        "49394004": ("superior mesenteric lymph node", [7600, 8134])
    },
    "SuperiorMesentericVein": {
        "90771006": (
            "Superior Mesenteric Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "SuperiorPancreaticLymphNode": {
        "280999005": ("superior pancreatic lymph node", [7600, 8134])
    },
    "SuperiorPancreaticoduodenalLymphNode": {
        "281320004": ("superior pancreaticoduodenal lymph node", [7600, 8134])
    },
    "SuperiorPhrenicArtery": {
        "38991005": (
            "Superior phrenic artery",
            [6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "SuperiorRectalLymphNode": {
        "68881005": ("superior rectal lymph node", [7600, 8134])
    },
    "SuperiorRightPulmonaryVein": {
        "8629005": (
            "Superior right pulmonary vein",
            [4, 3010, 3011, 3827, 3840, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SuperiorRollingOfBreast": {
        "415670009": ("Superior rolling of breast (procedure)", [])
    },
    "SuperiorThyroidArtery": {
        "72021004": (
            "Superior thyroid artery",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SuperiorTracheobronchialLymphNode": {
        "67941004": ("superior tracheobronchial lymph node", [7600, 8134])
    },
    "SuperiorVenaCava": {
        "48345005": (
            "Superior vena cava",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12108,
                12282,
            ],
        )
    },
    "SuperiorVenaCavaStructure": {
        "48345005": ("Superior vena cava structure (body structure)", [])
    },
    "SupernumeraryDeciduousMandibularLeftCanineTooth": {
        "2211000124103": (
            "Supernumerary deciduous mandibular left canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularLeftCanineTooth_Inactive": {
        "707029006": (
            "Supernumerary deciduous mandibular left canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularLeftCentralIncisorTooth": {
        "2261000124100": (
            "Supernumerary deciduous mandibular left central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularLeftCentralIncisorTooth_Inactive": {
        "707026004": (
            "Supernumerary deciduous mandibular left central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularLeftFirstMolarTooth": {
        "2181000124102": (
            "Supernumerary deciduous mandibular left first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularLeftFirstMolarTooth_Inactive": {
        "707030001": (
            "Supernumerary deciduous mandibular left first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularLeftLateralIncisorTooth": {
        "2171000124100": (
            "Supernumerary deciduous mandibular left lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularLeftLateralIncisorTooth_Inactive": {
        "707028003": (
            "Supernumerary deciduous mandibular left lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularLeftSecondMolarTooth": {
        "2241000124104": (
            "Supernumerary deciduous mandibular left second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularLeftSecondMolarTooth_Inactive": {
        "707031002": (
            "Supernumerary deciduous mandibular left second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularRightCanineTooth": {
        "2221000124106": (
            "Supernumerary deciduous mandibular right canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularRightCanineTooth_Inactive": {
        "707023007": (
            "Supernumerary deciduous mandibular right canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularRightCentralIncisorTooth": {
        "2191000124104": (
            "Supernumerary deciduous mandibular right central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularRightCentralIncisorTooth_Inactive": {
        "707025000": (
            "Supernumerary deciduous mandibular right central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularRightFirstMolarTooth": {
        "2231000124109": (
            "Supernumerary deciduous mandibular right first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularRightFirstMolarTooth_Inactive": {
        "707022002": (
            "Supernumerary deciduous mandibular right first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularRightLateralIncisorTooth": {
        "2201000124101": (
            "Supernumerary deciduous mandibular right lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularRightLateralIncisorTooth_Inactive": {
        "707024001": (
            "Supernumerary deciduous mandibular right lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMandibularRightSecondMolarTooth": {
        "2251000124102": (
            "Supernumerary deciduous mandibular right second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMandibularRightSecondMolarTooth_Inactive": {
        "707021009": (
            "Supernumerary deciduous mandibular right second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftCanineTooth": {
        "2361000124105": (
            "Supernumerary deciduous maxillary left canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftCanineTooth_Inactive": {
        "707016006": (
            "Supernumerary deciduous maxillary left canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftCentralIncisorTooth": {
        "2351000124108": (
            "Supernumerary deciduous maxillary left central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftCentralIncisorTooth_Inactive": {
        "707014009": (
            "Supernumerary deciduous maxillary left central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftFirstMolarTooth": {
        "2331000124101": (
            "Supernumerary deciduous maxillary left first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftFirstMolarTooth_Inactive": {
        "707017002": (
            "Supernumerary deciduous maxillary left first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftLateralIncisorTooth": {
        "2271000124107": (
            "Supernumerary deciduous maxillary left lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftLateralIncisorTooth_Inactive": {
        "707015005": (
            "Supernumerary deciduous maxillary left lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftSecondMolarTooth": {
        "2311000124107": (
            "Supernumerary deciduous maxillary left second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryLeftSecondMolarTooth_Inactive": {
        "707018007": (
            "Supernumerary deciduous maxillary left second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightCanineTooth": {
        "2291000124108": (
            "Supernumerary deciduous maxillary right canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightCanineTooth_Inactive": {
        "707011001": (
            "Supernumerary deciduous maxillary right canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightCentralIncisorTooth": {
        "2301000124109": (
            "Supernumerary deciduous maxillary right central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightCentralIncisorTooth_Inactive": {
        "707013003": (
            "Supernumerary deciduous maxillary right central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightFirstMolarTooth": {
        "2321000124104": (
            "Supernumerary deciduous maxillary right first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightFirstMolarTooth_Inactive": {
        "707010000": (
            "Supernumerary deciduous maxillary right first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightLateralIncisorTooth": {
        "2281000124105": (
            "Supernumerary deciduous maxillary right lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightLateralIncisorTooth_Inactive": {
        "707012008": (
            "Supernumerary deciduous maxillary right lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightSecondMolarTooth": {
        "2341000124106": (
            "Supernumerary deciduous maxillary right second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryDeciduousMaxillaryRightSecondMolarTooth_Inactive": {
        "707009005": (
            "Supernumerary deciduous maxillary right second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftCanineTooth": {
        "2441000124103": (
            "Supernumerary permanent mandibular left canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftCanineTooth_Inactive": {
        "707058009": (
            "Supernumerary permanent mandibular left canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftCentralIncisorTooth": {
        "2481000124109": (
            "Supernumerary permanent mandibular left central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftCentralIncisorTooth_Inactive": {
        "707060006": (
            "Supernumerary permanent mandibular left central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftFirstMolarTooth": {
        "2431000124108": (
            "Supernumerary permanent mandibular left first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftFirstMolarTooth_Inactive": {
        "707055007": (
            "Supernumerary permanent mandibular left first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftFirstPremolarTooth": {
        "2451000124101": (
            "Supernumerary permanent mandibular left first premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftFirstPremolarTooth_Inactive": {
        "707057004": (
            "Supernumerary permanent mandibular left first premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftLateralIncisorTooth": {
        "2381000124100": (
            "Supernumerary permanent mandibular left lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftLateralIncisorTooth_Inactive": {
        "707059001": (
            "Supernumerary permanent mandibular left lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftSecondMolarTooth": {
        "2371000124103": (
            "Supernumerary permanent mandibular left second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftSecondMolarTooth_Inactive": {
        "707054006": (
            "Supernumerary permanent mandibular left second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftSecondPremolarTooth": {
        "2471000124106": (
            "Supernumerary permanent mandibular left second premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftSecondPremolarTooth_Inactive": {
        "707056008": (
            "Supernumerary permanent mandibular left second premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularLeftThirdMolarTooth": {
        "2391000124102": (
            "Supernumerary permanent mandibular left third molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularLeftThirdMolarTooth_Inactive": {
        "707052005": (
            "Supernumerary permanent mandibular left third molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightCanineTooth": {
        "2501000124104": (
            "Supernumerary permanent mandibular right canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightCanineTooth_Inactive": {
        "707063008": (
            "Supernumerary permanent mandibular right canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightCentralIncisorTooth": {
        "2511000124101": (
            "Supernumerary permanent mandibular right central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightCentralIncisorTooth_Inactive": {
        "707061005": (
            "Supernumerary permanent mandibular right central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightFirstMolarTooth": {
        "2491000124107": (
            "Supernumerary permanent mandibular right first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightFirstMolarTooth_Inactive": {
        "707066000": (
            "Supernumerary permanent mandibular right first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightFirstPremolarTooth": {
        "2461000124104": (
            "Supernumerary permanent mandibular right first premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightFirstPremolarTooth_Inactive": {
        "707064002": (
            "Supernumerary permanent mandibular right first premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightLateralIncisorTooth": {
        "2521000124109": (
            "Supernumerary permanent mandibular right lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightLateralIncisorTooth_Inactive": {
        "707062003": (
            "Supernumerary permanent mandibular right lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightSecondMolarTooth": {
        "2401000124100": (
            "Supernumerary permanent mandibular right second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightSecondMolarTooth_Inactive": {
        "707067009": (
            "Supernumerary permanent mandibular right second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightSecondPremolarTooth": {
        "2421000124105": (
            "Supernumerary permanent mandibular right second premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightSecondPremolarTooth_Inactive": {
        "707065001": (
            "Supernumerary permanent mandibular right second premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMandibularRightThirdMolarTooth": {
        "2411000124102": (
            "Supernumerary permanent mandibular right third molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMandibularRightThirdMolarTooth_Inactive": {
        "707068004": (
            "Supernumerary permanent mandibular right third molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftCanineTooth": {
        "2611000124100": (
            "Supernumerary permanent maxillary left canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftCanineTooth_Inactive": {
        "707044007": (
            "Supernumerary permanent maxillary left canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftCentralIncisorTooth": {
        "2541000124102": (
            "Supernumerary permanent maxillary left central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftCentralIncisorTooth_Inactive": {
        "707042006": (
            "Supernumerary permanent maxillary left central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftFirstMolarTooth": {
        "2661000124102": (
            "Supernumerary permanent maxillary left first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftFirstMolarTooth_Inactive": {
        "707047000": (
            "Supernumerary permanent maxillary left first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftFirstPremolarTooth": {
        "2641000124101": (
            "Supernumerary permanent maxillary left first premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftFirstPremolarTooth_Inactive": {
        "707045008": (
            "Supernumerary permanent maxillary left first premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftLateralIncisorTooth": {
        "2651000124104": (
            "Supernumerary permanent maxillary left lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftLateralIncisorTooth_Inactive": {
        "707043001": (
            "Supernumerary permanent maxillary left lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftSecondMolarTooth": {
        "2621000124108": (
            "Supernumerary permanent maxillary left second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftSecondMolarTooth_Inactive": {
        "707048005": (
            "Supernumerary permanent maxillary left second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftSecondPremolarTooth": {
        "2681000124107": (
            "Supernumerary permanent maxillary left second premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftSecondPremolarTooth_Inactive": {
        "707046009": (
            "Supernumerary permanent maxillary left second premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftThirdMolarTooth": {
        "2581000124108": (
            "Supernumerary permanent maxillary left third molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryLeftThirdMolarTooth_Inactive": {
        "707049002": (
            "Supernumerary permanent maxillary left third molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightCanineTooth": {
        "2601000124103": (
            "Supernumerary permanent maxillary right canine tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightCanineTooth_Inactive": {
        "707038008": (
            "Supernumerary permanent maxillary right canine tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightCentralIncisorTooth": {
        "2551000124100": (
            "Supernumerary permanent maxillary right central incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightCentralIncisorTooth_Inactive": {
        "707041004": (
            "Supernumerary permanent maxillary right central incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightFirstMolarTooth": {
        "2591000124106": (
            "Supernumerary permanent maxillary right first molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightFirstMolarTooth_Inactive": {
        "707035006": (
            "Supernumerary permanent maxillary right first molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightFirstPremolarTooth": {
        "2671000124109": (
            "Supernumerary permanent maxillary right first premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightFirstPremolarTooth_Inactive": {
        "707037003": (
            "Supernumerary permanent maxillary right first premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightLateralIncisorTooth": {
        "2571000124105": (
            "Supernumerary permanent maxillary right lateral incisor tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightLateralIncisorTooth_Inactive": {
        "707039000": (
            "Supernumerary permanent maxillary right lateral incisor tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightSecondMolarTooth": {
        "2631000124106": (
            "Supernumerary permanent maxillary right second molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightSecondMolarTooth_Inactive": {
        "707033004": (
            "Supernumerary permanent maxillary right second molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightSecondPremolarTooth": {
        "2561000124103": (
            "Supernumerary permanent maxillary right second premolar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightSecondPremolarTooth_Inactive": {
        "707036007": (
            "Supernumerary permanent maxillary right second premolar tooth (morphologic abnormality)",
            [],
        )
    },
    "SupernumeraryPermanentMaxillaryRightThirdMolarTooth": {
        "2531000124107": (
            "Supernumerary permanent maxillary right third molar tooth",
            [4025, 4026],
        )
    },
    "SupernumeraryPermanentMaxillaryRightThirdMolarTooth_Inactive": {
        "707032009": (
            "Supernumerary permanent maxillary right third molar tooth (morphologic abnormality)",
            [],
        )
    },
    "SuperolateralToInferomedialOblique": {
        "399188001": ("superolateral to inferomedial oblique", [4014])
    },
    "SuperolateralToInferomedialObliqueProjection": {
        "399188001": (
            "Superolateral to inferomedial oblique projection (qualifier value)",
            [],
        )
    },
    "Supination": {"14502000": ("Supination", [92])},
    "SupinationFunction": {
        "14502000": ("Supination, function (observable entity)", [])
    },
    "Supine": {"40199007": ("supine", [20, 6206])},
    "SupineBodyPosition": {"40199007": ("Supine body position", [3262, 3602])},
    "SupportiveIntent": {"399707004": ("Supportive intent", [3629])},
    "SupportiveProcedureIntent": {
        "399707004": ("Supportive - procedure intent (qualifier value)", [])
    },
    "SupraRenalAorta": {
        "1918003": (
            "Supra-renal Aorta",
            [3827, 7151, 7192, 8134, 9514, 12103, 12112, 12125, 12291],
        )
    },
    "SupraaorticValveAreaStructure": {
        "8128003": ("Supraaortic valve area structure (body structure)", [])
    },
    "SupraclavicularLymphNode": {
        "76838003": ("supraclavicular lymph node", [7600, 8134])
    },
    "SupraclavicularRegionOfNeck": {
        "77621008": ("Supraclavicular region of neck", [4, 7151, 7192, 8134, 9514])
    },
    "Supraglottis": {"119255006": ("supraglottis", [7601, 8134])},
    "SupraglottisStructure": {
        "119255006": ("Supraglottis structure (body structure)", [])
    },
    "SuprapulmonicValveArea": {
        "79142001": ("Suprapulmonic valve area", [3829, 7151, 7192, 9514])
    },
    "SuprapulmonicValveAreaStructure": {
        "79142001": ("Suprapulmonic valve area structure (body structure)", [])
    },
    "SuprapyloricLymphNode": {"280216006": ("suprapyloric lymph node", [7600, 8134])},
    "SupraspinatusMuscle": {
        "6423006": ("Supraspinatus muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "SupraspinatusMuscleStructure": {
        "6423006": ("Supraspinatus muscle structure (body structure)", [])
    },
    "SuprasternalCoronalView": {
        "443162009": ("Suprasternal coronal view", [501, 12226])
    },
    "SuprasternalLongAxis": {"399106004": ("Suprasternal long axis", [501, 12226])},
    "SuprasternalLongAxisView": {
        "399106004": ("Suprasternal long axis view (qualifier value)", [])
    },
    "SuprasternalLongAxisViewOfAorticArch": {
        "443562002": ("Suprasternal long axis view of aortic arch", [501, 12226])
    },
    "SuprasternalNotch": {
        "26493002": (
            "Suprasternal notch",
            [4, 1000, 1001, 1004, 6115, 7151, 7192, 8134, 9514],
        )
    },
    "SuprasternalSagittalView": {
        "443163004": ("Suprasternal sagittal view", [501, 12226])
    },
    "SuprasternalShortAxis": {"399145009": ("Suprasternal short axis", [501, 12226])},
    "SuprasternalShortAxisView": {
        "399145009": ("Suprasternal short axis view (qualifier value)", [])
    },
    "SupraventricularTachycardia": {
        "6456007": ("Supraventricular tachycardia", [3201, 3230, 12246])
    },
    "Surface": {"410679008": ("Surface", [2, 5])},
    "SurfaceBolus": {"228736002": ("Surface Bolus", [9516, 9520])},
    "SurfaceRecuttingOfTissueBlock": {
        "434474007": ("Surface recutting of tissue block (procedure)", [])
    },
    "Surgeon": {"304292004": ("Surgeon", [7450, 7452])},
    "Surgery": {"309967005": ("Surgery", [7030])},
    "Surgical": {"83578000": ("surgical", [3815])},
    "SurgicalBiopsyOfBreast": {
        "274331003": ("Surgical biopsy of breast (procedure)", [])
    },
    "SurgicalDepartment": {"309967005": ("Surgical department (environment)", [])},
    "SurgicalIntensiveCare": {"418433008": ("Surgical Intensive Care", [7030])},
    "SurgicalIntensiveCareUnit": {
        "418433008": ("Surgical intensive care unit (environment)", [])
    },
    "SurgicalMarginObservable": {
        "396631001": ("Surgical margin observable (observable entity)", [])
    },
    "SurgicalMetalNailDevice": {
        "63289001": ("Surgical metal nail, device (physical object)", [])
    },
    "SurgicalProcedure": {"387713003": ("Surgical procedure", [6319])},
    "SurgicalScar": {"63130001": ("Surgical scar", [6056])},
    "SurgicalSutureDevice": {
        "27065002": ("Surgical suture, device (physical object)", [])
    },
    "SurgicalTransplantDepartment": {
        "309993003": ("Surgical transplant department (environment)", [])
    },
    "SurgicallyConstructedBidirectionalPathwayFromSuperiorVenaCavaToPulmonaryArtery": {
        "443789005": (
            "Surgically constructed bidirectional pathway from superior vena cava to pulmonary artery "
            "(morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConnectionOfAortaToPulmonaryArtery": {
        "443096004": (
            "Surgically constructed connection of aorta to pulmonary artery (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConnectionOfLeftVentricleToPulmonaryArteryConduit": {
        "443696003": (
            "Surgically constructed connection of left ventricle to pulmonary artery conduit (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConnectionOfPulmonaryVeinToLeftAtrium": {
        "443297004": (
            "Surgically constructed connection of pulmonary vein to left atrium (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConnectionOfRightVentricleToPulmonaryArteryConduit": {
        "443328003": (
            "Surgically constructed connection of right ventricle to pulmonary artery conduit (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConnectionOfSystemicVenousReturnToPulmonaryArtery": {
        "443625008": (
            "Surgically constructed connection of systemic venous return to pulmonary artery (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedConvergenceOfInferiorAndSuperiorVenaCavaPathwaysIntoCommonPathwaySuperiorToAtrioventricularValve": {
        "443809000": (
            "Surgically constructed convergence of inferior and superior "
            "vena cava pathways into common pathway superior to "
            "atrioventricular valve (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedOrificeBetweenSystemicVenousPathwayAndLeftAtrium": {
        "443724003": (
            "Surgically constructed orifice between systemic venous pathway and left atrium (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedOutflowTractOfLeftVentricleAcrossVentricularSeptalDefectToAorta": {
        "443260009": (
            "Surgically constructed outflow tract of left ventricle across ventricular septal defect to "
            "aorta (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayFromInferiorVenaCavaToMitralValve": {
        "444329004": (
            "Surgically constructed pathway from inferior vena cava to mitral valve (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayFromInferiorVenaCavaToPulmonaryArtery": {
        "443326004": (
            "Surgically constructed pathway from inferior vena cava to pulmonary artery (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayFromPulmonaryVeinsToTricuspidValve": {
        "443907004": (
            "Surgically constructed pathway from pulmonary veins to tricuspid valve (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayFromSuperiorVenaCavaToMitralValve": {
        "444177009": (
            "Surgically constructed pathway from superior vena cava to mitral valve (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayFromSuperiorVenaCavaToPulmonaryArtery": {
        "443327008": (
            "Surgically constructed pathway from superior vena cava to pulmonary artery (morphologic abnormality)",
            [],
        )
    },
    "SurgicallyConstructedPathwayThroughHeartFromInferiorVenaCavaAndSuperiorVenaCavaToPulmonaryArtery": {
        "443298009": (
            "Surgically constructed pathway through heart from inferior vena cava and superior "
            "vena cava to pulmonary artery (morphologic abnormality)",
            [],
        )
    },
    "Sus": {"388393002": ("Sus", [7454])},
    "SusScrofa": {"78678003": ("Sus scrofa", [7454])},
    "SusScrofaDomesticPigXJapaneseWildBoarIntragenusHybrid": {
        "133182008": (
            "Sus scrofa domestic pig X Japanese wild boar intragenus hybrid (organism)",
            [],
        )
    },
    "Suspected": {"415684004": ("Suspected", [3770])},
    "Suspended": {"385655000": ("Suspended", [3773])},
    "SussexCattleBreed": {"133276000": ("Sussex cattle breed", [7480])},
    "SussexSpaniel": {"80576000": ("Sussex spaniel (organism)", [])},
    "SussexSpanielDogBreed": {"80576000": ("Sussex spaniel dog breed", [7480])},
    "Suture": {
        "27065002": ("Suture", [6040, 6102, 6138, 6202, 6203, 6401, 6404, 7151, 7193])
    },
    "SwabianHallPigBreed": {"132354003": ("Swabian-Hall pig breed", [7480])},
    "SwaledaleSheepBreed": {"131822003": ("Swaledale sheep breed", [7480])},
    "SwallowBeliedMangalitzaPigBreed": {
        "132232002": ("Swallow Belied Mangalitza pig breed", [7480])
    },
    "SwannGanzCatheter": {"397755005": ("Swann-Ganz catheter", [3429])},
    "Swanson": {"399313005": ("Swanson", [4012])},
    "SwansonProjection": {"399313005": ("Swanson projection (qualifier value)", [])},
    "Sweating": {"415690000": ("Sweating", [60, 9300])},
    "SwedishArdennesHorseBreed": {
        "131946009": ("Swedish Ardennes horse breed", [7480])
    },
    "SwedishFurSheepBreed": {"132799009": ("Swedish Fur Sheep breed", [7480])},
    "SwedishLandraceGoatBreed": {"131641002": ("Swedish Landrace goat breed", [7480])},
    "SwedishLandracePig": {"12407009": ("Swedish landrace pig (organism)", [])},
    "SwedishLandracePigBreed": {"12407009": ("Swedish landrace pig breed", [7480])},
    "SwedishLapphundDogBreed": {"132632005": ("Swedish Lapphund dog breed", [7480])},
    "SwedishRedPolledCattleBreed": {
        "133277009": ("Swedish Red Polled cattle breed", [7480])
    },
    "SwedishVallhundDogBreed": {"132635007": ("Swedish Vallhund dog breed", [7480])},
    "SwedishWarmbloodHorseBreed": {
        "133144000": ("Swedish Warmblood horse breed", [7480])
    },
    "Swelling": {"65124004": ("Swelling", [6062])},
    "SwissAlpineGoat": {"28360002": ("Swiss alpine goat (organism)", [])},
    "SwissAlpineGoatBreed": {"28360002": ("Swiss alpine goat breed", [7480])},
    "SwissBlackPiedCattleBreed": {
        "133513002": ("Swiss Black Pied cattle breed", [7480])
    },
    "SwissEdelschweinPigBreed": {"132101005": ("Swiss Edelschwein pig breed", [7480])},
    "SwissImprovedLandracePigBreed": {
        "132351006": ("Swiss Improved Landrace pig breed", [7480])
    },
    "SwissMountainDogBreed": {"132642007": ("Swiss Mountain Dog breed", [7480])},
    "SwissWarmbloodHorseBreed": {"132763004": ("Swiss Warmblood horse breed", [7480])},
    "SychevkaCattleBreed": {"133514008": ("Sychevka cattle breed", [7480])},
    "SykiaCattleBreed": {"133515009": ("Sykia cattle breed", [7480])},
    "SympatheticTrunk": {
        "44909008": ("Sympathetic trunk", [7151, 7167, 7192, 8134, 9514])
    },
    "SympatheticTrunkStructure": {
        "44909008": ("Sympathetic trunk structure (body structure)", [])
    },
    "SympathomimeticAgent": {"86308005": ("Sympathomimetic agent (product)", [])},
    "SymphysisPubis": {"82561000": ("Symphysis pubis", [1000, 1001, 1005, 8134])},
    "SymphysisPubisStructure": {
        "82561000": ("Symphysis pubis structure (body structure)", [])
    },
    "SymptomFree": {"162467007": ("Symptom free", [3772])},
    "SymptomHasChanged": {"162499001": ("Symptom has changed", [4406])},
    "Syncope": {"271594007": ("Syncope", [3201, 3220, 3221, 12246])},
    "SyntheticSL98PigBreed": {"133220008": ("Synthetic SL98 pig breed", [7480])},
    "Syringe": {"61968008": ("Syringe", [69, 4051])},
    "SyringeDevice": {"61968008": ("Syringe, device (physical object)", [])},
    "SyringeUnit": {"733020007": ("Syringe", [68])},
    "Systemic": {"31099001": ("Systemic", [6128])},
    "SystemicArterialBlood": {"371952000": ("Systemic arterial blood (substance)", [])},
    "SystemicArterialHypertension": {
        "38341003": ("Systemic arterial hypertension", [3640])
    },
    "SystemicArterialStructure": {
        "281159003": ("Systemic arterial structure (body structure)", [])
    },
    "SystemicArtery": {
        "281159003": ("Systemic artery", [7151, 7156, 7192, 8134, 9514])
    },
    "SystemicArteryBlood": {"371952000": ("Systemic Artery Blood", [3520])},
    "SystemicCollateralArteryToLung": {
        "128589000": (
            "Systemic collateral artery to lung",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SystemicVascularResistance": {
        "386530009": ("Systemic Vascular Resistance", [3619])
    },
    "SystemicVascularStructure": {"281157001": ("Systemic vascular structure", [6100])},
    "SystemicVein": {"360592004": ("Systemic vein", [7151, 7156, 7192, 8134, 9514])},
    "SystemicVenousAtrium": {
        "128568009": (
            "Systemic venous atrium",
            [4, 3010, 3608, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "SystemicVenousStructure": {
        "360592004": ("Systemic venous structure (body structure)", [])
    },
    "Systole": {"111973004": ("Systole", [12233, 12307])},
    "SystoleFunction": {"111973004": ("Systole, function (observable entity)", [])},
    "Systolic": {"111973004": ("Systolic", [3472])},
    "SystolicBloodPressure": {
        "271649006": ("Systolic blood pressure (observable entity)", [])
    },
    "T10T11Disc": {"34959001": ("T10/T11 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T10Vertebra": {"7610001": ("T10 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T11T12Disc": {"1537001": ("T11/T12 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T11Vertebra": {"12989004": ("T11 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T12L1Disc": {"76206002": ("T12/L1 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T12Vertebra": {"23215003": ("T12 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T1T2Disc": {"40908007": ("T1/T2 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T1Vertebra": {"64864005": ("T1 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T2T3Disc": {"6004007": ("T2/T3 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T2Vertebra": {"53733008": ("T2 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T3T4Disc": {"2620004": ("T3/T4 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T3Vertebra": {"1626008": ("T3 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T4T5Disc": {"82965004": ("T4/T5 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T4Vertebra": {"73071006": ("T4 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T5T6Disc": {"72692000": ("T5/T6 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T5Vertebra": {"56401006": ("T5 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T6T7Disc": {"74401007": ("T6/T7 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T6Vertebra": {"45296009": ("T6 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T7T8Disc": {"28693002": ("T7/T8 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T7Vertebra": {"62487009": ("T7 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T8T9Disc": {"9188009": ("T8/T9 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T8Vertebra": {"11068009": ("T8 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "T9T10Disc": {"113209001": ("T9/T10 disc", [7151, 7192, 7602, 7604, 8134, 9514])},
    "T9Vertebra": {"82687006": ("T9 vertebra", [7151, 7192, 7602, 7603, 8134, 9514])},
    "TSSH1CattleBreed": {"133839005": ("TSSH-1 cattle breed (organism)", [])},
    "TSSH1XZebuCattleBreed": {"133839005": ("TSSH-1 X zebu cattle breed", [7480])},
    "TWaveAlternans": {"428550008": ("T wave alternans (observable entity)", [])},
    "TabapuaCattleBreed": {"133787007": ("Tabapua cattle breed", [7480])},
    "Table": {"86407004": ("Table", [7151, 7157, 7158, 7193, 9505, 9515, 9520, 9573])},
    "TableDevice": {"86407004": ("Table, device (physical object)", [])},
    "TachycardiaBradycardia": {"74615001": ("Tachycardia-bradycardia", [60, 9300])},
    "TadzhikZebuCattleBreed": {"133700006": ("Tadzhik zebu cattle breed", [7480])},
    "TagilCattleBreed": {"133526003": ("Tagil cattle breed", [7480])},
    "TahltanBearDog": {"61320006": ("Tahltan bear dog", [7480])},
    "TaihuPigBreed": {"132316003": ("Taihu pig breed", [7480])},
    "TainoCattleBreed": {"133835004": ("Taino cattle breed (organism)", [])},
    "TainoXZebuCattleBreed": {"133835004": ("Taino X zebu cattle breed", [7480])},
    "TaishuhHorseBreed": {"133143006": ("Taishuh horse breed", [7480])},
    "TaiwanSmallBlackPigBreed": {"132146006": ("Taiwan Small Black pig breed", [7480])},
    "TaiwanSmallRedPigBreed": {"132147002": ("Taiwan Small Red pig breed", [7480])},
    "TaiwanYellowCattleBreed": {"131553007": ("Taiwan Yellow cattle breed", [7480])},
    "TaiwanZebuCattleBreed": {"131529001": ("Taiwan Zebu cattle breed", [7480])},
    "TajmaCattleBreed": {"133527007": ("Tajma cattle breed", [7480])},
    "TakingOfSwab": {"285570007": ("Taking of swab", [8109])},
    "TakingPatientVitalSigns": {
        "61746007": ("Taking patient vital signs (procedure)", [])
    },
    "Talus": {"67453005": ("Talus", [1000, 1001, 1006, 7304, 8134])},
    "TamankaduwaCattleBreed": {"133788002": ("Tamankaduwa cattle breed", [7480])},
    "TambovRedCattleBreed": {"133528002": ("Tambov Red cattle breed", [7480])},
    "Tamoxifen": {"75959001": ("Tamoxifen", [6080])},
    "TamworthPig": {"30634003": ("Tamworth pig (organism)", [])},
    "TamworthPigBreed": {"30634003": ("Tamworth pig breed", [7480])},
    "Tangential": {"399110001": ("tangential", [501, 4010, 4015])},
    "TangentialProjection": {
        "399110001": ("Tangential projection (qualifier value)", [])
    },
    "Tantalum": {"45215009": ("Tantalum", [300, 10006, 10067])},
    "TanzanianZebuCattleBreed": {"133789005": ("Tanzanian Zebu cattle breed", [7480])},
    "TaoyuanPigBreed": {"132145005": ("Taoyuan pig breed", [7480])},
    "TapWater": {"444923006": ("Tap water", [610])},
    "TapetumOfCorpusCallosum": {
        "60105000": ("tapetum of corpus callosum", [7705, 7710, 8134])
    },
    "TaraiCattleBreed": {"133790001": ("Tarai cattle breed", [7480])},
    "TarentaiseCattleBreed": {"50959000": ("Tarentaise cattle breed (organism)", [])},
    "TarentaiseCowBreed": {"50959000": ("Tarentaise cow breed", [7480])},
    "TargetHeartRate": {"428420003": ("Target heart rate (observable entity)", [])},
    "TargetHeartRateAchieved": {"258153002": ("Target Heart Rate Achieved", [3221])},
    "TargheeSheep": {"89665001": ("Targhee sheep (organism)", [])},
    "TargheeSheepBreed": {"89665001": ("Targhee sheep breed", [7480])},
    "TarinaCattleBreed": {"133529005": ("Tarina cattle breed", [7480])},
    "Tarrant": {"399247005": ("Tarrant", [4012])},
    "TarrantProjection": {"399247005": ("Tarrant projection (qualifier value)", [])},
    "TarsalJoint": {
        "27949001": (
            "Tarsal joint",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "Tarsus": {"108371006": ("Tarsus", [7482, 7483, 8134])},
    "TartrazineStain": {"21592006": ("tartrazine stain", [8112])},
    "TasteSenseAltered": {"271801002": ("Taste sense altered", [60, 9300])},
    "Taurine": {"10944007": ("Taurine (substance)", [])},
    "TauroselcholicAcid75se": {
        "395894004": ("Tauroselcholic acid[75Se] (substance)", [])
    },
    "TavdaHorseBreed": {"132764005": ("Tavda horse breed", [7480])},
    "TawleedHorseBreed": {"133001001": ("Tawleed horse breed", [7480])},
    "Taylor": {"399296000": ("Taylor", [4012])},
    "TaylorProjection": {"399296000": ("Taylor projection (qualifier value)", [])},
    "Tc99mSestamibi": {"424299003": ("Tc-99m sestamibi", [3111])},
    "Tc99mTetrofosmin": {"424118002": ("Tc-99m tetrofosmin", [3111])},
    "Technetium99mGalactosylHumanSerumAlbuminDiethylenetriamine": {
        "446534003": (
            "Technetium^99m Galactosyl Human Serum Albumin Diethylenetriamine",
            [25],
        )
    },
    "Technetium99mLabeledCarbon": {
        "447125008": ("Technetium^99m labeled carbon", [25])
    },
    "Technetium99mMercaptoacetylTriglycineMAG3": {
        "446536001": ("Technetium^99m Mercaptoacetyl triglycine MAG3", [25])
    },
    "Technetium99mNPyridoxyl5Methyltryptophan": {
        "447126009": ("Technetium^99m N-pyridoxyl-5-methyltryptophan", [25])
    },
    "Technetium99mPhytate": {"447127000": ("Technetium^99m Phytate", [25])},
    "Technetium99mStannousColloid": {
        "447128005": ("Technetium^99m Stannous Colloid", [25])
    },
    "Technetium99mtcEthylCysteinateDimerInjection": {
        "404713008": (
            "Technetium[99mTc] ethyl cysteinate dimer injection (product)",
            [],
        )
    },
    "TechnetiumTc99NSubstitutedIminodiacetate": {
        "87410002": ("Technetium Tc^99^ N-substituted iminodiacetate (substance)", [])
    },
    "TechnetiumTc99TaggedRedCells": {
        "89818005": ("Technetium Tc^99^ tagged red cells (substance)", [])
    },
    "TechnetiumTc99cAlbuminMicrospheres": {
        "55494003": ("Technetium Tc^99c^ albumin microspheres", [25])
    },
    "TechnetiumTc99mAggregatedAlbumin": {
        "85693008": ("Technetium Tc^99m^ aggregated albumin", [25])
    },
    "TechnetiumTc99mAlbuminMicrospheres": {
        "55494003": ("Technetium Tc^99m^ albumin microspheres (substance)", [])
    },
    "TechnetiumTc99mBicisate": {"766886003": ("Technetium Tc^99m^ bicisate", [25])},
    "TechnetiumTc99mDepreotide": {"415704007": ("Technetium Tc^99m^ depreotide", [25])},
    "TechnetiumTc99mDisofenin": {"3040004": ("Technetium Tc^99m^ disofenin", [25])},
    "TechnetiumTc99mExametazime": {
        "77313009": ("Technetium Tc^99m^ exametazime", [25])
    },
    "TechnetiumTc99mGalactosylHumanSerumAlbuminDiethylenetriaminePentaaceticAcid": {
        "446534003": (
            "Technetium Tc^99m^ galactosyl human serum albumin diethylenetriamine pentaacetic acid (substance)",
            [],
        )
    },
    "TechnetiumTc99mIronAscorbate": {
        "87853006": ("Technetium Tc^99m^ iron ascorbate", [25])
    },
    "TechnetiumTc99mLabeledCarbon": {
        "447125008": ("Technetium Tc^99m^ labeled carbon (substance)", [])
    },
    "TechnetiumTc99mLidofenin": {"7281000": ("Technetium Tc^99m^ lidofenin", [25])},
    "TechnetiumTc99mMebrofenin": {"4832001": ("Technetium Tc^99m^ mebrofenin", [25])},
    "TechnetiumTc99mMedronate": {"96390006": ("Technetium Tc^99m^ medronate", [25])},
    "TechnetiumTc99mMercaptoacetyltriglycine": {
        "446536001": ("Technetium Tc^99m^ mercaptoacetyltriglycine (substance)", [])
    },
    "TechnetiumTc99mMicroaggregatedAlbumin": {
        "81761004": ("Technetium Tc^99m^ microaggregated albumin", [25])
    },
    "TechnetiumTc99mNPyridoxyl5Methyltryptophan": {
        "447126009": (
            "Technetium Tc^99m^ N-pyridoxyl-5-methyltryptophan (substance)",
            [],
        )
    },
    "TechnetiumTc99mNSubstitutedIminodiacetate": {
        "87410002": ("Technetium Tc^99m^ N-substituted iminodiacetate", [25])
    },
    "TechnetiumTc99mOxidronate": {"53951001": ("Technetium Tc^99m^ oxidronate", [25])},
    "TechnetiumTc99mPentetate": {"430276001": ("Technetium Tc^99m^ pentetate", [25])},
    "TechnetiumTc99mPhytate": {
        "447127000": ("Technetium Tc^99m^ phytate (substance)", [])
    },
    "TechnetiumTc99mPyrophosphate": {
        "764821009": ("Technetium Tc^99m^ pyrophosphate", [25])
    },
    "TechnetiumTc99mSerumAlbumin": {
        "79610008": ("Technetium Tc^99m^ serum albumin", [25])
    },
    "TechnetiumTc99mSestamibi": {"424299003": ("Technetium Tc^99m^ sestamibi", [25])},
    "TechnetiumTc99mSodiumGlucoheptonate": {
        "45849009": ("Technetium Tc^99m^ sodium glucoheptonate", [25])
    },
    "TechnetiumTc99mStannousColloid": {
        "447128005": ("Technetium Tc^99m^ stannous colloid (substance)", [])
    },
    "TechnetiumTc99mStannousEtidronate": {
        "111162007": ("Technetium Tc^99m^ stannous etidronate", [25])
    },
    "TechnetiumTc99mSuccimer": {"24511001": ("Technetium Tc^99m^ succimer", [25])},
    "TechnetiumTc99mSulfurColloid": {
        "5931004": ("Technetium Tc^99m^ sulfur colloid", [25])
    },
    "TechnetiumTc99mTaggedRedCells": {
        "89818005": ("Technetium Tc^99m^ tagged red cells", [25])
    },
    "TechnetiumTc99mTeboroxime": {"424318009": ("Technetium Tc^99m^Teboroxime", [25])},
    "TechnetiumTc99mTetrofosmin": {
        "424118002": ("Technetium Tc^99m^Tetrofosmin", [25])
    },
    "TeddyRooseveltTerrierDogBreed": {
        "132454002": ("Teddy Roosevelt Terrier dog breed", [7480])
    },
    "TeeswaterSheepBreed": {"132800008": ("Teeswater sheep breed", [7480])},
    "TeethGumsAndSupportingStructures": {
        "28035005": ("Teeth, gums and supporting structures", [4016])
    },
    "TelangiectasiaCutaneousCancerSyndromeFamilial": {
        "782823001": ("Telangiectasia, cutaneous, cancer syndrome, familial", [4405])
    },
    "TelemarkCattleBreed": {"133278004": ("Telemark cattle breed", [7480])},
    "Telencephalon": {"11628009": ("Telencephalon", [7151, 7153, 7192, 8134, 9514])},
    "TelomainDogBreed": {"132634006": ("Telomain dog breed", [7480])},
    "TelomianDogBreed": {"132634006": ("Telomian dog breed (organism)", [])},
    "TemperatureWaveform": {"128437008": ("Temperature waveform", [3003])},
    "TemperatureWaveformFunction": {
        "128437008": ("Temperature waveform, function (observable entity)", [])
    },
    "TemporalBone": {
        "60911003": ("Temporal Bone", [1000, 1001, 1002, 4028, 7151, 7192, 8134, 9514])
    },
    "TemporalBoneStructure": {
        "60911003": ("Temporal bone structure (body structure)", [])
    },
    "TemporalCorrelation": {"133886009": ("Temporal correlation", [6043, 6137])},
    "TemporalLobe": {"78277001": ("Temporal lobe", [7151, 7153, 7192, 8134, 9514])},
    "TemporalLobeStructure": {
        "78277001": ("Temporal lobe structure (body structure)", [])
    },
    "TemporalPeriodsRelatingToFeedingAndEating": {
        "309602000": (
            "Temporal periods relating to feeding and eating (qualifier value)",
            [],
        )
    },
    "TemporalPeriodsRelatingToProcedure": {
        "307152002": ("Temporal periods relating to procedure (qualifier value)", [])
    },
    "TemporaryTranscutaneousPacing": {
        "59218006": ("temporary transcutaneous pacing", [3555])
    },
    "TemporomandibularJoint": {
        "53620006": (
            "Temporomandibular joint",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "TemporomandibularJointStructure": {
        "53620006": ("Temporomandibular joint structure (body structure)", [])
    },
    "Tendon": {"13024002": ("Tendon", [7151, 7166, 7191])},
    "TendonOfRotatorCuffOfShoulder": {
        "245100002": ("Tendon of Rotator Cuff of Shoulder", [12321])
    },
    "TendonOfTodaro": {"128595004": ("Tendon of Todaro", [3011, 8134])},
    "TendonStructure": {"13024002": ("Tendon structure (body structure)", [])},
    "TennesseeWalkingHorse": {"51023000": ("Tennessee walking horse (organism)", [])},
    "TennesseeWalkingHorseBreed": {
        "51023000": ("Tennessee walking horse breed", [7480])
    },
    "TenthIntercostalSpace": {"117142007": ("Tenth Intercostal Space", [5])},
    "TenthPercentile": {"371890005": ("Tenth percentile (qualifier value)", [])},
    "TeresMajorMuscle": {
        "1193009": ("Teres major muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "TeresMinorMuscle": {
        "51159009": ("Teres minor muscle", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "TerminalDeoxynucleotidylTransferaseStain": {
        "406975002": ("terminal deoxynucleotidyl transferase stain", [8112])
    },
    "TerminalInternalCarotidArtery": {
        "415646005": (
            "Terminal internal carotid artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12105],
        )
    },
    "Terrier": {"606003": ("Terrier", [7480])},
    "TerskHorseBreed": {"133151009": ("Tersk horse breed", [7480])},
    "TestDuration": {"252129004": ("Test duration (observable entity)", [])},
    "TestNotDone": {"373121007": ("Test not done (qualifier value)", [])},
    "TesticularArtery": {
        "27175001": (
            "Testicular Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "TesticularVein": {
        "31688004": (
            "Testicular Vein",
            [3827, 7151, 7192, 8134, 9514, 12103, 12113, 12125],
        )
    },
    "Testis": {
        "40689003": (
            "Testis",
            [4, 4030, 6202, 6204, 7151, 7160, 7192, 8134, 9514, 10044, 10060, 12321],
        )
    },
    "TestisStructure": {"40689003": ("Testis structure (body structure)", [])},
    "TetracyclineAntibiotic": {"373206009": ("Tetracycline antibiotic", [621, 622])},
    "TetracyclineClassOfAntibioticSubstance": {
        "373206009": ("Tetracycline (class of antibiotic, substance) (substance)", [])
    },
    "TetralogyOfFallot": {"86299006": ("Tetralogy of Fallot", [12248, 12249])},
    "Teufel": {"399127009": ("Teufel", [4012])},
    "TeufelProjection": {"399127009": ("Teufel projection (qualifier value)", [])},
    "TexasLonghornCattleBreed": {"133279007": ("Texas Longhorn cattle breed", [7480])},
    "TexelSheepBreed": {"132801007": ("Texel sheep breed", [7480])},
    "TexonCattleBreed": {"133280005": ("Texon cattle breed", [7480])},
    "ThaiBinhPigBreed": {"132349007": ("Thai Binh pig breed", [7480])},
    "ThaiCattleBreed": {"131530006": ("Thai cattle breed", [7480])},
    "ThaiPonyHorseBreed": {"131949002": ("Thai Pony horse breed", [7480])},
    "ThaiRidgebackDogBreed": {"132641000": ("Thai Ridgeback dog breed", [7480])},
    "ThailandFightingZebuCattleBreed": {
        "131531005": ("Thailand Fighting Zebu cattle breed", [7480])
    },
    "ThalamicStructure": {"42695009": ("Thalamic structure (body structure)", [])},
    "Thalamus": {"42695009": ("Thalamus", [7151, 7153, 7192, 8134, 9514])},
    "ThalamusPart": {"119406000": ("Thalamus", [7140])},
    "ThalliSheepBreed": {"132707000": ("Thalli sheep breed", [7480])},
    "Thallium201": {"353842007": ("Thallium-201", [3111])},
    "ThallousChlorideTl201": {"73685002": ("Thallous chloride Tl^201^", [25])},
    "ThanhHoaCattleBreed": {"131532003": ("Thanh-Hoa cattle breed", [7480])},
    "TharparkarCattleBreed": {"133629002": ("Tharparkar cattle breed", [7480])},
    "TheilerMurineEncephalomyelitisVirus": {
        "42024000": ("Theiler's murine encephalomyelitis virus", [638, 641])
    },
    "TherapeuticIntent": {"262202000": ("Therapeutic Intent", [3629])},
    "TherapeuticProcedure": {"277132007": ("Therapeutic procedure", [6319])},
    "TherapeuticRadioisotope": {"439007": ("Therapeutic radioisotope", [25])},
    "ThermalBath": {"371838001": ("Thermal Bath", [3628])},
    "ThermalCardiacOutputWaveform": {
        "128454000": ("Thermal cardiac output waveform", [3003])
    },
    "ThermalCardiacOutputWaveformFunction": {
        "128454000": (
            "Thermal cardiac output waveform, function (observable entity)",
            [],
        )
    },
    "ThermalInline": {"371843008": ("Thermal Inline", [3628])},
    "ThermistorCatheter": {"371800000": ("Thermistor catheter", [3429])},
    "ThermistorCatheterMethod": {"133914002": ("Thermistor catheter method", [3241])},
    "ThermoluminescentRadiationDosimeter": {
        "464983000": ("Thermoluminescent radiation dosimeter", [7026, 7027, 7151, 7193])
    },
    "ThessalonianHorseBreed": {"131950002": ("Thessalonian horse breed", [7480])},
    "ThessalyCattleBreed": {"133530000": ("Thessaly cattle breed", [7480])},
    "Thiamylal": {"40342009": ("Thiamylal", [623, 625])},
    "ThiamylalSodium": {"40342009": ("Thiamylal sodium (substance)", [])},
    "ThibarCattleBreed": {"133836003": ("Thibar cattle breed (organism)", [])},
    "ThibarXZebuCattleBreed": {"133836003": ("Thibar X zebu cattle breed", [7480])},
    "Thigh": {
        "68367000": ("Thigh", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "ThighPerforator": {
        "714759009": ("Thigh perforator", [3827, 7151, 7192, 8134, 9514, 12103, 12110])
    },
    "ThighStructure": {"68367000": ("Thigh structure (body structure)", [])},
    "ThillariCattleBreed": {"133791002": ("Thillari cattle breed", [7480])},
    "ThioflavineSStain": {"406995008": ("thioflavine S stain", [8112])},
    "ThioflavineTStain": {"61068006": ("thioflavine T stain", [8112])},
    "ThioninStain": {"406969006": ("Thionin stain (substance)", [])},
    "ThionineStain": {"12001002": ("thionine stain", [8112])},
    "Thiopental": {"387448009": ("Thiopental", [623, 625])},
    "ThiopentalSodium": {"387448009": ("Thiopental sodium (substance)", [])},
    "ThirdDegreeAtrioventricularBlock": {
        "27885002": ("Third degree Atrioventricular block", [3230])
    },
    "ThirdIntercostalSpace": {"117135007": ("Third Intercostal Space", [5])},
    "ThirdLumbarArtery": {
        "397409007": (
            "Third Lumbar Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125],
        )
    },
    "ThirdMolarRegion": {"699503005": ("Third molar region", [4017])},
    "ThirdMolarRegionOfOralCavity": {
        "699503005": ("Third molar region of oral cavity (body structure)", [])
    },
    "ThirdVentricle": {
        "49841001": ("Third ventricle", [7140, 7151, 7153, 7192, 8134, 9514])
    },
    "ThirdVentricleStructure": {
        "49841001": ("Third ventricle structure (body structure)", [])
    },
    "Thoracentesis": {"91602002": ("Thoracentesis", [9])},
    "ThoracicAorta": {
        "113262008": (
            "Thoracic aorta",
            [4, 3010, 3604, 3606, 3630, 4030, 4042, 7151, 7192, 8134, 9514, 12291],
        )
    },
    "ThoracicAortaStructure": {
        "113262008": ("Thoracic aorta structure (body structure)", [])
    },
    "ThoracicAorticCoarctation": {
        "253678000": ("Thoracic Aortic Coarctation", [12236])
    },
    "ThoracicCavityStructure": {
        "43799004": ("Thoracic cavity structure (body structure)", [])
    },
    "ThoracicDuct": {
        "1732005": ("Thoracic Duct", [6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514])
    },
    "ThoracicDuctStructure": {
        "1732005": ("Thoracic duct structure (body structure)", [])
    },
    "ThoracicInlet": {"42973007": ("Thoracic Inlet", [1000, 1001, 1004, 8134])},
    "ThoracicSpine": {
        "122495006": (
            "Thoracic spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7155, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "ThoracicSpineStructure": {
        "122495006": ("Thoracic spine structure (body structure)", [])
    },
    "ThoracicStructure": {"51185008": ("Thoracic structure (body structure)", [])},
    "ThoracicSurgery": {"309970009": ("Thoracic Surgery", [7030])},
    "ThoracicSurgeryDepartment": {
        "309970009": ("Thoracic surgery department (environment)", [])
    },
    "ThoracoLumbarSpine": {
        "1217256009": (
            "Thoraco-lumbar spine",
            [4, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 7482, 7483, 8134, 9514],
        )
    },
    "ThoracolumbarRegionOfSpineStructure": {
        "297172009": ("Thoracolumbar region of spine structure (body structure)", [])
    },
    "Thorax": {"816094009": ("Thorax", [4, 7155, 12020])},
    "ThraceCattleBreed": {"133360006": ("Thrace cattle breed", [7480])},
    "Thrombin": {"36176003": ("Thrombin (substance)", [])},
    "ThrombinPreparation": {"36176003": ("Thrombin preparation", [10])},
    "ThrombolysisInMyocardialInfarctionGrade0NoPerfusion": {
        "371867000": (
            "Thrombolysis in Myocardial Infarction grade 0: no perfusion (finding)",
            [],
        )
    },
    "ThrombolysisInMyocardialInfarctionGrade1PenetrationWithoutPerfusion": {
        "371866009": (
            "Thrombolysis in Myocardial Infarction grade 1: penetration without perfusion (finding)",
            [],
        )
    },
    "ThrombolysisInMyocardialInfarctionGrade2PartialPerfusion": {
        "371864007": (
            "Thrombolysis in Myocardial Infarction grade 2: partial perfusion (finding)",
            [],
        )
    },
    "ThrombolysisInMyocardialInfarctionGrade3CompletePerfusion": {
        "371865008": (
            "Thrombolysis in Myocardial Infarction grade 3: complete perfusion (finding)",
            [],
        )
    },
    "Thrombolytic": {"303960004": ("Thrombolytic (product)", [])},
    "ThrombolyticAgent": {"303960004": ("Thrombolytic agent", [10])},
    "ThrombolyticAgentAdministeredBetween3And6HoursBeforePercutaneousCoronaryIntervention": {
        "371897008": (
            "Thrombolytic agent administered between 3 and 6 hours before percutaneous coronary "
            "intervention (situation)",
            [],
        )
    },
    "ThrombolyticAgentAdministeredBetween6HoursAnd7DaysBeforePercutaneousCoronaryIntervention": {
        "371906007": (
            "Thrombolytic agent administered between 6 hours and 7 days before percutaneous coronary "
            "intervention (situation)",
            [],
        )
    },
    "ThrombolyticAgentAdministeredLessThan3HoursBeforePercutaneousCoronaryIntervention": {
        "371896004": (
            "Thrombolytic agent administered less than 3 hours before percutaneous coronary intervention "
            "(situation)",
            [],
        )
    },
    "ThrombolyticAgentContraindicated": {"373148008": ("Contraindicated", [3740])},
    "ThrombolyticTherapy": {"426347000": ("Thrombolytic therapy", [3764])},
    "ThrombophlebitisOfBreast": {
        "69954004": ("Thrombophlebitis of breast (disorder)", [])
    },
    "ThrombophlebitisOfBreastMondorDisease": {
        "69954004": ("Thrombophlebitis of breast (Mondor's disease)", [6030, 6031])
    },
    "Thromboplastin": {"387124009": ("Thromboplastin", [10])},
    "ThrombosedAneurysm": {"125271003": ("thrombosed aneurysm", [3808])},
    "Thrombosis": {"264579008": ("thrombosis", [3805])},
    "ThromboticDisorder": {"118927008": ("thrombosis", [3815])},
    "Thrombus": {
        "396339007": (
            "Thrombus",
            [3491, 3495, 3497, 3712, 3810, 7151, 7156, 7192, 8134, 9514],
        )
    },
    "Thumb": {
        "76505004": ("Thumb", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "ThumbStructure": {"76505004": ("Thumb structure (body structure)", [])},
    "ThuocNhieuPigBreed": {"133165006": ("Thuoc Nhieu pig breed", [7480])},
    "ThuringianGoatBreed": {"131642009": ("Thuringian goat breed", [7480])},
    "ThymidineF18": {"129502002": ("Thymidine F^18^", [4021])},
    "Thymus": {"9875009": ("Thymus", [4, 4030, 7151, 7155, 7192, 9514, 10044, 10060])},
    "ThymusGland": {"9875009": ("Thymus Gland", [6102, 6109, 6113, 6149, 8134])},
    "ThymusGlandStructure": {
        "9875009": ("Thymus gland structure (body structure)", [])
    },
    "ThyrocervicalTrunk": {
        "6538005": ("Thyrocervical trunk", [6102, 6109, 6117, 7151, 7192, 8134, 9514])
    },
    "Thyroid": {
        "69748006": (
            "Thyroid",
            [
                4,
                645,
                4030,
                6102,
                6109,
                6113,
                6149,
                7151,
                7192,
                8134,
                9514,
                10044,
                10060,
                12321,
            ],
        )
    },
    "ThyroidHormone": {"18220004": ("Thyroid hormone", [621, 622])},
    "ThyroidIsthmus": {"40867004": ("Thyroid isthmus", [12130])},
    "ThyroidNodule": {"237495005": ("Thyroid Nodule", [12322])},
    "ThyroidPart": {"362892003": ("Thyroid part", [12130])},
    "ThyroidStructure": {"69748006": ("Thyroid structure (body structure)", [])},
    "TiaMeslanPigBreed": {"132090004": ("Tia Meslan pig breed", [7480])},
    "TibetanCattleBreed": {"131533008": ("Tibetan cattle breed", [7480])},
    "TibetanHorseBreed": {"131951003": ("Tibetan horse breed", [7480])},
    "TibetanMastiffDogBreed": {"132643002": ("Tibetan Mastiff dog breed", [7480])},
    "TibetanPigBreed": {"132228008": ("Tibetan pig breed", [7480])},
    "TibetanSpaniel": {"87219003": ("Tibetan spaniel", [7480])},
    "TibetanTerrier": {"17663009": ("Tibetan terrier", [7480])},
    "Tibia": {"12611008": ("Tibia", [7304, 8134, 12021])},
    "TibiaAndFibula": {"110536004": ("Tibia and fibula", [7482, 7483, 8134])},
    "TibiaAndFibulaCombinedSite": {
        "110536004": ("Tibia and fibula (combined site) (body structure)", [])
    },
    "TibialArtery": {"181351007": ("tibial artery", [3604, 3606, 3630])},
    "TibialLymphNode": {"80769008": ("tibial lymph node", [7600, 8134])},
    "TibialPlateau": {"306783000": ("Tibial Plateau", [1000, 1001, 1006, 8134])},
    "TibialPlateauStructure": {
        "306783000": ("Tibial plateau structure (body structure)", [])
    },
    "TielingHorseBreed": {"131952005": ("Tieling horse breed", [7480])},
    "Tiletamine": {"96265006": ("Tiletamine", [623, 625])},
    "TiletamineHydrochloride": {
        "96265006": ("Tiletamine hydrochloride (substance)", [])
    },
    "TimeFromStartOfTest": {
        "252131008": ("Time from start of test (observable entity)", [])
    },
    "TimorHorseBreed": {"131953000": ("Timor horse breed", [7480])},
    "Tin": {"12597001": ("Tin", [10006, 10067])},
    "TinimaCattleBreed": {"133531001": ("Tinima cattle breed", [7480])},
    "TinosCattleBreed": {"133542005": ("Tinos cattle breed", [7480])},
    "TipManometer": {"371802008": ("Tip manometer", [3429])},
    "TipManometerHemodynamicMeasurementMethodRegimeTherapy": {
        "128582009": (
            "Tip manometer hemodynamic measurement method (regime/therapy)",
            [],
        )
    },
    "TipManometerMethod": {"128582009": ("Tip manometer method", [3241])},
    "Tissue": {
        "85756007": ("Tissue", [7150, 7151, 7165, 7166, 7191, 7196, 9502, 9580])
    },
    "TissueCassette": {"434464009": ("Tissue cassette", [8101, 8102])},
    "TissueCassetteForMicroarray": {
        "434708008": ("Tissue cassette for microarray (physical object)", [])
    },
    "TissueDopplerImaging": {"439858009": ("Tissue Doppler Imaging", [6058, 12224])},
    "TissueEmbeddingMedium": {"430863003": ("Tissue embedding medium (substance)", [])},
    "TissueFixative": {"430864009": ("Tissue fixative (substance)", [])},
    "TissueFreezingMedium": {"433469005": ("Tissue freezing medium", [8115])},
    "TissueMicroarrayCassette": {
        "434708008": ("Tissue microarray cassette", [8101, 8102])
    },
    "TissueSection": {"430856003": ("Tissue section", [8103])},
    "TissueSpecimen": {"119376003": ("tissue specimen", [501, 4010, 8103])},
    "TissueSpecimenFromBreast": {"127457009": ("tissue specimen from breast", [4014])},
    "TissueSpot": {"431196006": ("Tissue spot", [8103])},
    "TitanYellowStain": {"84217005": ("titan yellow stain", [8112])},
    "Titanium": {"1166006": ("Titanium", [300])},
    "Titterington": {"399241006": ("Titterington", [4012])},
    "TitteringtonProjection": {
        "399241006": ("Titterington projection (qualifier value)", [])
    },
    "TobaccoSmokingBehavior": {"365981007": ("Tobacco Smoking Behavior", [3774])},
    "Toe": {
        "29707007": (
            "Toe",
            [4, 1000, 1001, 1006, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "ToeStructure": {"29707007": ("Toe structure (body structure)", [])},
    "ToggenburgGoat": {"30089001": ("Toggenburg goat (organism)", [])},
    "ToggenburgGoatBreed": {"30089001": ("Toggenburg goat breed", [7480])},
    "TokaraHorseBreed": {"133009004": ("Tokara horse breed", [7480])},
    "TolazolineHydrochloride": {
        "19041007": ("Tolazoline hydrochloride (substance)", [])
    },
    "TolazolineHydrocholoride": {"19041007": ("Tolazoline hydrocholoride", [10])},
    "ToluidineBlueStain": {"29522004": ("toluidine blue stain", [4412, 8112])},
    "TongConPigBreed": {"132347009": ("Tong Con pig breed", [7480])},
    "TongSheepBreed": {"132708005": ("Tong sheep breed", [7480])},
    "TongaCattleBreed": {"131534002": ("Tonga cattle breed", [7480])},
    "Tongue": {"21974007": ("Tongue", [4, 4028, 7151, 7192, 7601, 8134, 9514])},
    "TongueStructure": {"21974007": ("Tongue structure (body structure)", [])},
    "TonkineseCat": {"44855006": ("Tonkinese cat", [7480])},
    "TonsilAndAdenoid": {"303337002": ("tonsil and adenoid", [7601, 8134])},
    "TonsilAndAdenoidStructure": {
        "303337002": ("Tonsil and adenoid structure (body structure)", [])
    },
    "TonsillarStructurePalatine": {
        "75573002": ("Tonsillar structure (palatine) (body structure)", [])
    },
    "Tooth": {"38199008": ("Tooth", [4028, 4029, 7151, 7192, 8134, 9514])},
    "ToothStructure": {"38199008": ("Tooth structure (body structure)", [])},
    "TopicalChemotherapyForMalignantNeoplasm": {
        "24977001": ("Topical chemotherapy for malignant neoplasm", [4409])
    },
    "TopicalLocalAnesthesia": {"386760001": ("Topical local anesthesia", [611, 612])},
    "TopicalLocalAnesthetic": {
        "386760001": ("Topical local anesthetic (procedure)", [])
    },
    "TopicalRoute": {"6064005": ("Topical route", [11])},
    "TopographicalModifier": {"106233006": ("Topographical modifier", [210])},
    "ToposaCattleBreed": {"133792009": ("Toposa cattle breed", [7480])},
    "ToriHorseBreed": {"133008007": ("Tori horse breed", [7480])},
    "ToroCattleBreed": {"131535001": ("Toro cattle breed", [7480])},
    "ToronkeCattleBreed": {"133793004": ("Toronke cattle breed", [7480])},
    "Torsion": {"51795009": ("Torsion", [92])},
    "Tortuosity": {"15690004": ("tortuosity", [3810])},
    "Tortuous": {"386137000": ("Tortuous", [3712])},
    "TortuousCoronaryArtery": {"386137000": ("Tortuous coronary artery (finding)", [])},
    "TosaInuDogBreed": {"132645009": ("Tosa Inu dog breed", [7480])},
    "Total": {"255619001": ("Total", [7464])},
    "TotalBloodVolume": {"268384009": ("Total blood volume (observable entity)", [])},
    "TotalCavopulmonaryConnectionWithLateralAtrialTunnel": {
        "427886002": (
            "Total cavopulmonary connection with lateral atrial tunnel (procedure)",
            [],
        )
    },
    "TotalExerciseTime": {"252130009": ("Total exercise time (observable entity)", [])},
    "TouabireSheepBreed": {"132709002": ("Touabire sheep breed", [7480])},
    "ToubouCattleBreed": {"133837007": ("Toubou cattle breed (organism)", [])},
    "ToubouXZebuCattleBreed": {"133837007": ("Toubou X zebu cattle breed", [7480])},
    "TouchPreparation": {"430854000": ("Touch preparation (procedure)", [8110])},
    "TouchPreparationCytologicMaterial": {
        "430855004": ("Touch preparation cytologic material", [8103])
    },
    "TouchPreparationOfSpecimen": {
        "430854000": ("Touch preparation of specimen (procedure)", [])
    },
    "ToupouriCattleBreed": {"133794005": ("Toupouri cattle breed", [7480])},
    "Towne": {"399270002": ("Towne", [4012])},
    "TowneProjection": {"399270002": ("Towne's projection (qualifier value)", [])},
    "Toxin": {"80917008": ("Toxin", [637])},
    "ToyFoxTerrier": {"26639007": ("Toy fox terrier (organism)", [])},
    "ToyFoxTerrierDogBreed": {"26639007": ("Toy fox terrier dog breed", [7480])},
    "ToyHavaneseTerrierDogBreed": {
        "132646005": ("Toy Havanese Terrier dog breed", [7480])
    },
    "ToyManchesterTerrier": {"13942005": ("Toy Manchester terrier", [7480])},
    "ToyPoodle": {"25243005": ("Toy poodle (organism)", [])},
    "ToyPoodleDogBreed": {"25243005": ("Toy poodle dog breed", [7480])},
    "TrabeculaeCarnae": {
        "118755002": ("Trabeculae carnae", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "TrabeculaeCarneaeCordis": {
        "118755002": ("Trabeculae carneae cordis (body structure)", [])
    },
    "TrabecularThickeningOfBreast": {
        "129795008": ("Trabecular thickening of breast", [6014, 6015, 6016, 6054, 6056])
    },
    "Trachea": {
        "44567001": (
            "Trachea",
            [
                4,
                4009,
                4028,
                4030,
                4031,
                4042,
                6102,
                6109,
                6113,
                6149,
                7151,
                7155,
                7192,
                8134,
                9514,
            ],
        )
    },
    "TracheaAndBronchus": {
        "110726009": ("Trachea and bronchus", [4, 4040, 7151, 7192, 8134, 9514])
    },
    "TracheaAndBronchusCombinedSite": {
        "110726009": ("Trachea and bronchus (combined site) (body structure)", [])
    },
    "TrachealStructure": {"44567001": ("Tracheal structure (body structure)", [])},
    "TracheobronchialLymphNode": {
        "245341003": ("tracheobronchial lymph node", [7600, 8134])
    },
    "TracheobronchialLymphNodeLocatedNearCarina": {
        "89858007": ("tracheobronchial lymph node, located near carina", [7600, 8134])
    },
    "Tracheotomy": {"48387007": ("Tracheotomy", [6102, 6138, 6404, 7151, 7193])},
    "TractionAction": {"129411004": ("Traction - action", [94])},
    "TrakehnerHorse": {"1789009": ("Trakehner horse (organism)", [])},
    "TrakehnerHorseBreed": {"1789009": ("Trakehner horse breed", [7480])},
    "TrakyaHorseBreed": {"131954006": ("Trakya horse breed", [7480])},
    "TransAbdominal": {"66739002": ("Trans-abdominal", [7, 12025])},
    "TransEsophageal": {"103383005": ("Trans-esophageal", [7])},
    "TransGastric": {"103353001": ("Trans-gastric", [7])},
    "TransHepatic": {"103381007": ("Trans-hepatic", [7])},
    "TransMural": {"103354007": ("Trans-mural", [7])},
    "TransOrbital": {"129226004": ("Trans-orbital", [7])},
    "TransPancreatic": {"103349003": ("Trans-pancreatic", [7])},
    "TransPleural": {"103348006": ("Trans-pleural", [7])},
    "TransRectal": {"6706003": ("Trans-rectal", [7])},
    "TransRenal": {"103347001": ("Trans-renal", [7])},
    "TransSplenic": {"103346005": ("Trans-splenic", [7])},
    "TransTemporal": {"103382000": ("Trans-temporal", [7])},
    "TransThecal": {"103345009": ("Trans-thecal", [7])},
    "TransUrethral": {"57257006": ("Trans-urethral", [7])},
    "TransVaginal": {"54300008": ("Trans-vaginal", [7, 12025])},
    "TransVesical": {"103344008": ("Trans-vesical", [7])},
    "TransarterialApproach": {
        "103387006": ("Transarterial approach (qualifier value)", [])
    },
    "TranscatheterDeploymentOfDetachableBalloon": {
        "105372003": ("Transcatheter deployment of detachable balloon", [9, 3405])
    },
    "TranscatheterTherapyForEmbolization": {
        "16736007": ("Transcatheter therapy for embolization", [9, 3405])
    },
    "TransesophagealApproach": {
        "103383005": ("Transesophageal approach (qualifier value)", [])
    },
    "TransesophagealEchocardiography": {
        "105376000": ("Transesophageal echocardiography", [3405, 3428, 3757, 12001])
    },
    "TransesophagealShortAxisView": {
        "443698002": ("Transesophageal short axis view", [501, 12226])
    },
    "Transfer": {"4563007": ("Transfer", [3729])},
    "Transforamenal": {"272466003": ("transforamenal", [502, 4011])},
    "Transgastric": {"103353001": ("Transgastric (qualifier value)", [])},
    "TranshepaticApproach": {
        "103381007": ("Transhepatic approach (qualifier value)", [])
    },
    "TransientIschemicDilatationOfLeftVentricularCavity": {
        "429710001": (
            "Transient ischemic dilatation of left ventricular cavity (disorder)",
            [],
        )
    },
    "TransientIschemicDilatationRatioOfLeftVentricularCavity": {
        "428832008": (
            "Transient ischemic dilatation ratio of left ventricular cavity (observable entity)",
            [],
        )
    },
    "TransientLimbParalysis": {"274662006": ("Transient limb paralysis", [3205])},
    "TransitionZoneOfTheProstate": {
        "399384005": ("Transition zone of the prostate", [6300])
    },
    "TransitionalCellCarcinoma": {
        "27090000": ("Transitional cell carcinoma", [638, 639])
    },
    "TransjugularIntrahepaticPortosystemicShunt": {
        "397423009": (
            "Transjugular Intrahepatic Portosystemic Shunt",
            [3827, 7151, 7192, 8134, 9514, 12103, 12114, 12125],
        )
    },
    "TransluminalExtractionCatheterDevice": {
        "21870002": ("Transluminal extraction catheter, device (physical object)", [])
    },
    "TransluminalExtractionCatheterTEC": {
        "21870002": ("Transluminal Extraction Catheter (TEC)", [3411, 3429])
    },
    "Transmural": {"103354007": ("Transmural (qualifier value)", [])},
    "Transoral": {"118438002": ("transoral", [502, 4011])},
    "TransoralApproach": {"118438002": ("Transoral approach (qualifier value)", [])},
    "Transorbital": {"278318001": ("transorbital", [502, 4011])},
    "TransorbitalApproach": {
        "129226004": ("Transorbital approach (qualifier value)", [])
    },
    "TransorbitalProjection": {
        "278318001": ("Transorbital projection (qualifier value)", [])
    },
    "Transpancreatic": {"103349003": ("Transpancreatic (qualifier value)", [])},
    "TransplantSurgery": {"309993003": ("Transplant Surgery", [7030])},
    "TransplantationOfBoneMarrow": {
        "23719005": ("Transplantation of bone marrow (procedure)", [])
    },
    "TransplantationOfHeart": {
        "32413006": ("Transplantation of heart (procedure)", [])
    },
    "Transpleural": {"103348006": ("Transpleural (qualifier value)", [])},
    "TransrectalApproach": {"6706003": ("Transrectal approach (qualifier value)", [])},
    "Transrenal": {"103347001": ("Transrenal (qualifier value)", [])},
    "TransseptalCatheter": {"386124003": ("Transseptal catheter", [3429])},
    "TransseptalCatheterization": {
        "67338003": ("Transseptal catheterization", [3250, 3405])
    },
    "Transsplenic": {"103346005": ("Transsplenic (qualifier value)", [])},
    "TranstemporalApproach": {
        "103382000": ("Transtemporal approach (qualifier value)", [])
    },
    "Transthecal": {"103345009": ("Transthecal (qualifier value)", [])},
    "Transthoracic": {"272476000": ("transthoracic", [502, 4011])},
    "TransthoracicEchocardiography": {
        "433236007": ("Transthoracic echocardiography", [3405, 3428, 3757, 12001])
    },
    "TransthoracicEchocardiography_Inactive": {
        "252418006": ("Transthoracic echocardiography (procedure)", [])
    },
    "TransthoracicProjection": {
        "272476000": ("Transthoracic projection (qualifier value)", [])
    },
    "TranstrachealJetVentilation": {
        "448442005": ("Transtracheal jet ventilation", [619, 620])
    },
    "TransurethralApproach": {
        "57257006": ("Transurethral approach (qualifier value)", [])
    },
    "TransvenousApproach": {
        "103386002": ("Transvenous approach (qualifier value)", [])
    },
    "Transverse": {"62824007": ("Transverse", [2, 5, 6, 26, 501, 502, 3838, 12118])},
    "TransverseAorticArch": {"443281009": ("Transverse Aortic Arch", [12280])},
    "TransverseBodyPosition": {"399220000": ("transverse", [21])},
    "TransverseColon": {"485005": ("Transverse colon", [6210, 8134])},
    "TransverseColonStructure": {
        "485005": ("Transverse colon structure (body structure)", [])
    },
    "TransverseProcessOrVertebra": {
        "73400003": ("Transverse process or vertebra", [6115, 8134])
    },
    "TransversusThoracis": {
        "88454005": ("Transversus thoracis", [6102, 6109, 6116, 7151, 7192, 8134, 9514])
    },
    "Transvesical": {"103344008": ("Transvesical (qualifier value)", [])},
    "TransylvanianHoundDogBreed": {
        "132455001": ("Transylvanian Hound dog breed", [7480])
    },
    "TransylvanianPinzgauCattleBreed": {
        "133543000": ("Transylvanian Pinzgau cattle breed (organism)", [])
    },
    "TransylvanianPinzguaCattleBreed": {
        "133543000": ("Transylvanian Pinzgua cattle breed", [7480])
    },
    "TrapeziusMuscle": {
        "31764008": (
            "Trapezius muscle",
            [6102, 6109, 6113, 6116, 6149, 7151, 7192, 8134, 9514],
        )
    },
    "Trauma": {"417746004": ("trauma", [3815])},
    "TraumaSurgery": {"309994009": ("Trauma Surgery", [7030])},
    "TraumaSurgeryDepartment": {
        "309994009": ("Trauma surgery department (environment)", [])
    },
    "TraumaticAbnormality": {"417746004": ("Traumatic Abnormality", [3809])},
    "TraumaticAbnormalityMorphologic": {
        "19130008": ("Traumatic abnormality (morphologic abnormality)", [])
    },
    "TraumaticAneurysm": {"110421000": ("traumatic aneurysm", [3808])},
    "TraumaticInjury": {"417746004": ("Traumatic injury (disorder)", [])},
    "Treadmill": {"1211003": ("Treadmill", [3203])},
    "TreadmillDevice": {"1211003": ("Treadmill, device (physical object)", [])},
    "TreeingCurDogBreed": {"132647001": ("Treeing Cur dog breed", [7480])},
    "TreeingFeistDogBreed": {"132648006": ("Treeing Feist dog breed", [7480])},
    "TreeingTennesseeBrindleDogBreed": {
        "132633000": ("Treeing Tennessee Brindle dog breed", [7480])
    },
    "TreeingWalkerCoonhound": {"57120006": ("Treeing walker coonhound (organism)", [])},
    "TreeingWalkerCoonhoundDogBreed": {
        "57120006": ("Treeing walker coonhound dog breed", [7480])
    },
    "Tremor": {"26079004": ("Tremor (finding)", [])},
    "Trendelenburg": {"34106002": ("Trendelenburg", [20])},
    "TrendelenburgPosition": {"34106002": ("Trendelenburg position (finding)", [])},
    "Tretinoin": {"387305002": ("Tretinoin", [4410])},
    "TribredPigBreed": {"132029006": ("Tribred pig breed", [7480])},
    "Tribromoethanol": {"84386009": ("Tribromoethanol", [623, 625])},
    "TrichromeStain": {"406989007": ("trichrome stain", [8112])},
    "TricuspidAnnulus": {"113259005": ("Tricuspid Annulus", [12236, 12241, 12305])},
    "TricuspidDiastolicFillingPeriod": {
        "371847009": ("Tricuspid diastolic filling period (observable entity)", [])
    },
    "TricuspidDiastolicFillingPeriodDfpt": {
        "371847009": ("Tricuspid Diastolic Filling Period (DFPt)", [3616, 12267, 12268])
    },
    "TricuspidRegurgitation": {"111287006": ("Tricuspid regurgitation", [3700])},
    "TricuspidRing": {"113259005": ("Tricuspid ring", [3011, 8134])},
    "TricuspidValve": {
        "46030003": (
            "Tricuspid valve",
            [
                3610,
                3630,
                3701,
                6102,
                6109,
                6113,
                6149,
                7151,
                7192,
                8134,
                9514,
                12285,
                12305,
            ],
        )
    },
    "TricuspidValveArea": {"251014001": ("Tricuspid Valve Area", [3614, 3615])},
    "TricuspidValveClosureToOpeningTime": {
        "399282006": ("Tricuspid Valve Closure to Opening Time", [12208])
    },
    "TricuspidValveDisease": {"20721001": ("Tricuspid valve disease", [3700])},
    "TricuspidValveDisorder": {"20721001": ("Tricuspid valve disorder (disorder)", [])},
    "TricuspidValveFlow": {"371840006": ("Tricuspid Valve Flow", [3617])},
    "TricuspidValvePullbackPressureWaveform": {
        "128451008": ("Tricuspid valve pullback pressure waveform", [3003])
    },
    "TricuspidValvePullbackPressureWaveformFunction": {
        "128451008": (
            "Tricuspid valve pullback pressure waveform, function (observable entity)",
            [],
        )
    },
    "TricuspidValveRegurgitation": {
        "111287006": ("Tricuspid valve regurgitation (disorder)", [])
    },
    "TricuspidValveStructure": {
        "46030003": ("Tricuspid valve structure (body structure)", [])
    },
    "TrigeminalNerve": {"27612005": ("trigeminal nerve", [7706, 7710, 8134])},
    "TrigeminalNerveStructure": {
        "27612005": ("Trigeminal nerve structure (body structure)", [])
    },
    "TriggHoundDogBreed": {"132456000": ("Trigg Hound dog breed", [7480])},
    "TripleVesselCoronaryArteryDisease": {
        "233817007": ("Triple vessel coronary artery disease.", [3728])
    },
    "TripleVesselDiseaseOfTheHeart": {
        "233817007": ("Triple vessel disease of the heart (disorder)", [])
    },
    "TrochlearNerve": {"39322007": ("trochlear nerve", [7706, 7710, 8134])},
    "TrochlearNerveStructure": {
        "39322007": ("Trochlear nerve structure (body structure)", [])
    },
    "TropaeolinOOStain": {"53511009": ("tropaeolin OO stain", [8112])},
    "TropaeolinOStain": {"35094004": ("tropaeolin O stain", [8112])},
    "TropicalCattleBreed": {"133838002": ("Tropical cattle breed (organism)", [])},
    "TropicalDairyCattleCattleBreed": {
        "133544006": ("Tropical Dairy Cattle cattle breed", [7480])
    },
    "TropicalMedicine": {"441548002": ("Tropical Medicine", [7030])},
    "TropicalMedicineDepartment": {
        "441548002": ("Tropical medicine department (environment)", [])
    },
    "TropicalXZebuCattleBreed": {"133838002": ("Tropical X zebu cattle breed", [7480])},
    "Tropicamide": {"9190005": ("Tropicamide", [4208])},
    "TropicanaCattleBreed": {"133545007": ("Tropicana cattle breed", [7480])},
    "TroteEnGallopeHorseBreed": {"131955007": ("Trote en Gallope horse breed", [7480])},
    "TruncalValveStructure": {"279317000": ("Truncal Valve Structure", [12289])},
    "TruncusArteriosusCommunis": {
        "61959006": (
            "Truncus arteriosus communis",
            [4, 3010, 3604, 3606, 3630, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "TruncusCoeliacus": {
        "57850000": ("Truncus coeliacus", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "Trunk": {"22943007": ("Trunk", [8134, 12020])},
    "TrunkAngiography": {"303827001": ("trunk angiography", [3820])},
    "TrunkOfPulmonaryArtery": {
        "45341000": ("Trunk of pulmonary artery", [3829, 7151, 7192, 9514, 12305])
    },
    "TrunkStructure": {"22943007": ("Trunk structure (body structure)", [])},
    "TrypanBlue": {"60441008": ("Trypan blue", [4200])},
    "TrypanBlueStain": {"60441008": ("trypan blue stain", [8112])},
    "TsivilskPigBreed": {"132110002": ("Tsivilsk pig breed", [7480])},
    "TswanaCattleBreed": {"131439005": ("Tswana cattle breed", [7480])},
    "Tuarine": {"10944007": ("Tuarine", [218, 4032, 4033, 7180, 7186, 7469])},
    "TubalTonsil": {"21058000": ("tubal tonsil", [7601, 8134])},
    "Tube": {"83059008": ("Tube", [69])},
    "TubeDevice": {"83059008": ("Tube, device (physical object)", [])},
    "TubeFeedingOfPatientRegimeTherapy": {
        "61420007": ("Tube feeding of patient (regime/therapy)", [])
    },
    "TubercleOfRib": {"113198008": ("Tubercle of rib", [6115, 8134])},
    "TubercleOfRibStructure": {
        "113198008": ("Tubercle of rib structure (body structure)", [])
    },
    "TubularAdenocarcinoma": {"4631006": ("Tubular adenocarcinoma", [6030, 6033])},
    "TubularAdenoma": {"444408007": ("Tubular adenoma", [6030, 6031])},
    "TubularAdenomaNoInternationalClassificationOfDiseasesForOncologySubtype": {
        "19665009": (
            "Tubular adenoma, no International Classification of Diseases for Oncology subtype (morphologic abnormality)",
            [],
        )
    },
    "TubularDensity": {
        "129794007": ("Tubular density", [6014, 6015, 6016, 6054, 6056])
    },
    "TubularShapedDensityOfBreast": {
        "129794007": ("Tubular shaped density of breast (finding)", [])
    },
    "TudancaCattleBreed": {"133546008": ("Tudanca cattle breed", [7480])},
    "TukidaleSheepBreed": {"132704007": ("Tukidale sheep breed", [7480])},
    "TuliCattleBreed": {"131440007": ("Tuli cattle breed", [7480])},
    "Tumor": {"108369006": ("Tumor", [644, 3815, 8134])},
    "TumorInvasionLimitedToSkin": {
        "409783000": ("Tumor invasion limited to skin (finding)", [])
    },
    "TumorMarginStatus": {"372249001": ("Tumor margin status (observable entity)", [])},
    "TumorNecrosisFactorAlpha": {
        "39525005": ("Tumor necrosis factor alpha", [638, 642])
    },
    "TumorOfColon": {"126838000": ("Tumor of colon", [6201])},
    "TumorStageFinding": {"385356007": ("Tumor stage finding (finding)", [])},
    "TunchangPigBreed": {"132199000": ("Tunchang pig breed (organism)", [])},
    "TungchangPigBreed": {"132199000": ("Tungchang pig breed", [7480])},
    "Tungsten": {"26194003": ("Tungsten", [10016])},
    "TungstenHalogenLamp": {"445679001": ("Tungsten halogen lamp", [8125])},
    "TungstenHalogenLampMicroscopeIlluminator": {
        "445679001": (
            "Tungsten halogen lamp microscope illuminator (physical object)",
            [],
        )
    },
    "TuniCattleBreed": {"131536000": ("Tuni cattle breed", [7480])},
    "TunicaAdventitia": {"30180000": ("Tunica adventitia (body structure)", [])},
    "TunicaIntima": {"8361002": ("Tunica intima (body structure)", [])},
    "TunicaIntimaOfVessel": {
        "87483006": ("Tunica intima of vessel (body structure)", [])
    },
    "TunicaMediaVasorum": {"61695000": ("Tunica media vasorum (body structure)", [])},
    "TunisSheepBreed": {"132710007": ("Tunis sheep breed", [7480])},
    "TurinoCattleBreed": {"133547004": ("Turino cattle breed", [7480])},
    "TurkishAngoraCat": {"50441005": ("Turkish angora cat", [7480])},
    "TurkishBrownCattleBreed": {"133548009": ("Turkish Brown cattle breed", [7480])},
    "TurkishGraySteppeCattleBreed": {
        "131537009": ("Turkish Gray Steppe cattle breed", [7480])
    },
    "TurkishVanCatBreed": {"132655008": ("Turkish van cat breed", [7480])},
    "TurkmenZebuCattleBreed": {"133652003": ("Turkmen zebu cattle breed", [7480])},
    "TurkomanHorseBreed": {"131956008": ("Turkoman horse breed", [7480])},
    "TuropoljePigBreed": {"132229000": ("Turopolje pig breed", [7480])},
    "TushinHorseBreed": {"131957004": ("Tushin horse breed", [7480])},
    "TuvaHorseBreed": {"131958009": ("Tuva horse breed", [7480])},
    "TuxZillertalCattleBreed": {"133549001": ("Tux-Zillertal cattle breed", [7480])},
    "TuyHoaCattleBreed": {"131538004": ("Tuy-Hoa cattle breed", [7480])},
    "TuyHoaHairlessPigBreed": {"133161002": ("Tuy Hoa Hairless pig breed", [7480])},
    "Twining": {"399125001": ("Twining", [4012])},
    "TwiningProjection": {"399125001": ("Twining projection (qualifier value)", [])},
    "TwoChamber": {"399232001": ("Two chamber", [6, 501, 502])},
    "TwoStandardDeviationsAboveMean": {
        "371920000": ("Two standard deviations above mean (qualifier value)", [])
    },
    "TwoStandardDeviationsBelowMean": {
        "371918003": ("Two standard deviations below mean (qualifier value)", [])
    },
    "TypeOfStenosis": {"246244004": ("Type of stenosis (attribute)", [])},
    "TypicalAngina": {"429559004": ("Typical Angina", [3202, 3220])},
    "TyrolGrayCattleBreed": {"133550001": ("Tyrol Gray cattle breed", [7480])},
    "TyrolMountainSheepBreed": {"132711006": ("Tyrol Mountain sheep breed", [7480])},
    "TyroleanHoundDogBreed": {"132457009": ("Tyrolean Hound dog breed", [7480])},
    "TyropanoateSodium": {"109212003": ("Tyropanoate sodium (substance)", [])},
    "UdaSheepBreed": {"132712004": ("Uda sheep breed", [7480])},
    "UjumqinCattleBreed": {"131539007": ("Ujumqin cattle breed", [7480])},
    "UjumqinSheepBreed": {"132854005": ("Ujumqin sheep breed", [7480])},
    "UkrainianGrayCattleBreed": {"131544000": ("Ukrainian Gray cattle breed", [7480])},
    "UkrainianSaddleHorseHorseBreed": {
        "133007002": ("Ukrainian Saddle Horse horse breed", [7480])
    },
    "UkrainianSteppeWhitePig": {
        "406663005": ("Ukrainian steppe white pig (organism)", [])
    },
    "UkrainianSteppeWhitePigBreed": {
        "406663005": ("Ukrainian steppe white pig breed", [7480])
    },
    "UkrainianWhiteheadedCattleBreed": {
        "133555006": ("Ukrainian Whiteheaded cattle breed", [7480])
    },
    "Ulcer": {"56208002": ("Ulcer", [6209])},
    "Ulcerated": {"373138006": ("Ulcerated", [3712])},
    "UlceratedAtheromatousPlaque": {
        "62189002": ("ulcerated atheromatous plaque", [3817])
    },
    "UlceratedLesionOfCoronaryArtery": {
        "373138006": ("Ulcerated lesion of coronary artery (finding)", [])
    },
    "Ulcerative": {"255321001": ("Ulcerative", [3715])},
    "Ulna": {"23416004": ("Ulna", [7304, 8134, 12021])},
    "UlnarArtery": {
        "44984001": (
            "Ulnar artery",
            [4, 3010, 3440, 3827, 4042, 7151, 7192, 8134, 9514, 12103, 12107],
        )
    },
    "UlnarVein": {
        "17623008": ("Ulnar vein", [3827, 7151, 7192, 8134, 9514, 12103, 12108])
    },
    "UltraVioletColor": {"415770004": ("Ultra-violet color (qualifier value)", [])},
    "UltrasonicGuidanceForAmniocentesis": {
        "65388005": ("Ultrasonic guidance for amniocentesis", [9])
    },
    "UltrasonicGuidanceProcedure": {
        "61593002": ("Ultrasonic guidance procedure", [6058, 6060])
    },
    "Ultrasonography": {"310169008": ("Ultrasonography", [7030])},
    "UltrasonographyOfBreast": {
        "47079000": ("Ultrasonography of breast", [6050, 6083])
    },
    "UltrasonographyOfTotalBody": {
        "24135002": ("Ultrasonography of total body", [646])
    },
    "UltrasonographyService": {
        "310169008": ("Ultrasonography service (qualifier value)", [])
    },
    "UltrasoundCouplingGel": {"1004163002": ("Ultrasound coupling gel", [11002])},
    "UltrasoundPeripheralVascularFlowStudy": {
        "48526000": ("Ultrasound peripheral vascular flow study (procedure)", [])
    },
    "UltrasoundScanNormal": {"169254007": ("Ultrasound scan normal", [6054, 6064])},
    "UltrasoundScanObstetric": {
        "268445003": ("Ultrasound scan - obstetric (procedure)", [])
    },
    "Ultraviolet": {"415770004": ("Ultraviolet", [4206, 8122])},
    "UltravioletOpticalFilter": {"445254006": ("Ultraviolet optical filter", [8124])},
    "UltravioletOpticalFilterOfOpticalMicroscope": {
        "445254006": (
            "Ultraviolet optical filter of optical microscope (physical object)",
            [],
        )
    },
    "UmbilicalArtery": {
        "50536004": (
            "Umbilical artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12111,
                12125,
                12140,
            ],
        )
    },
    "UmbilicalCord": {"29870000": ("Umbilical cord", [8134])},
    "UmbilicalCordStructure": {
        "29870000": ("Umbilical cord structure (body structure)", [])
    },
    "UmbilicalRegion": {"90290004": ("Umbilical region", [4, 7151, 7192, 8134, 9514])},
    "UmbilicalRegionStructure": {
        "90290004": ("Umbilical region structure (body structure)", [])
    },
    "UmbilicalVein": {
        "284639000": (
            "Umbilical vein",
            [
                4,
                3010,
                3607,
                3630,
                3827,
                4042,
                7151,
                7192,
                8134,
                9514,
                12103,
                12114,
                12125,
            ],
        )
    },
    "UmblacheryCattleBreed": {"133671001": ("Umblachery cattle breed", [7480])},
    "UnableToAchieveTargetHeartRate": {
        "429733000": ("Unable to achieve target heart rate (finding)", [])
    },
    "UncementedComponentFixation": {
        "304367000": ("Uncemented component fixation", [7310])
    },
    "Uncertain": {"64957009": ("Uncertain (qualifier value)", [])},
    "UncertainRisk": {"64957009": ("Uncertain risk", [3118])},
    "UncinateFasciculus": {
        "26230003": ("Uncinate fasciculus", [7151, 7153, 7192, 7703, 7710, 8134, 9514])
    },
    "Uncle": {"38048003": ("Uncle", [7450, 7451])},
    "UncorrectedVisualAcuity": {"420050001": ("Uncorrected Visual Acuity", [4216])},
    "Undetermined": {"373068000": ("Undetermined", [230, 240])},
    "UnifocalPrematureVentricularComplexes": {
        "27337007": ("Unifocal premature ventricular complexes (disorder)", [])
    },
    "UnifocalPvcs": {"27337007": ("Unifocal PVCs", [3234])},
    "Unilateral": {"66459002": ("Unilateral", [2, 211, 244, 245])},
    "Unknown": {"261665006": ("Unknown", [3703, 6096])},
    "UnstableAngina": {"4557003": ("Unstable Angina", [3202, 3220])},
    "UnstableAnginaProgressiveAngina": {
        "4557003": ("Unstable Angina, Progressive Angina", [3700])
    },
    "Unsteady": {"103361006": ("Unsteady (qualifier value)", [])},
    "Up": {"255532002": ("Up", [4215])},
    "UpperAbdomen": {"80581009": ("Upper abdomen", [6129, 8134])},
    "UpperAbdomenStructure": {
        "80581009": ("Upper abdomen structure (body structure)", [])
    },
    "UpperArm": {
        "40983000": ("Upper arm", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "UpperArmStructure": {"40983000": ("Upper arm structure (body structure)", [])},
    "UpperDeepCervicalLymphNode": {
        "285425001": ("upper deep cervical lymph node", [7600, 8134])
    },
    "UpperDentalArchStructure": {
        "39481002": ("Upper dental arch structure (body structure)", [])
    },
    "UpperEyelid": {"38934000": ("Upper Eyelid", [4209, 8134])},
    "UpperEyelidStructure": {
        "38934000": ("Upper eyelid structure (body structure)", [])
    },
    "UpperGastroIntestinalTract": {
        "62834003": ("Upper gastro-intestinal tract", [7482, 7483, 8134])
    },
    "UpperGastrointestinalTractStructure": {
        "62834003": ("Upper gastrointestinal tract structure (body structure)", [])
    },
    "UpperInnerQuadrantOfBreast": {
        "77831004": (
            "Upper inner quadrant of breast",
            [4, 6020, 6021, 7151, 7192, 8134, 9514],
        )
    },
    "UpperJugularLymphNode": {"245321008": ("upper jugular lymph node", [7600, 8134])},
    "UpperLimb": {
        "53120007": ("Upper limb", [4, 4009, 4030, 4031, 4042, 7151, 7192, 8134, 9514])
    },
    "UpperLimbLymphNode": {"44914007": ("upper limb lymph node", [7600, 8134])},
    "UpperLimbLymphNodeStructure": {
        "44914007": ("Upper limb lymph node structure (body structure)", [])
    },
    "UpperLimbStructure": {"53120007": ("Upper limb structure (body structure)", [])},
    "UpperLimitOfReferenceRange": {
        "371933006": ("Upper limit of reference range (qualifier value)", [])
    },
    "UpperLobeOfLung": {
        "45653009": (
            "Upper lobe of lung",
            [6124, 6126, 6170, 7151, 7155, 7192, 8134, 9514],
        )
    },
    "UpperOuterQuadrantOfBreast": {
        "76365002": (
            "Upper outer quadrant of breast",
            [4, 6020, 6021, 7151, 7192, 8134, 9514],
        )
    },
    "UpperParatrachealLymphNode": {
        "127927006": ("upper paratracheal lymph node (mediastinal)", [7600, 8134])
    },
    "UpperParatrachealLymphNodeMediastinal": {
        "127927006": (
            "Upper paratracheal lymph node (mediastinal) (body structure)",
            [],
        )
    },
    "UpperUrinaryTract": {
        "431491007": (
            "Upper urinary tract",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "UpperZoneOfLung": {"281392002": ("Upper zone of lung", [6124, 6126, 6171])},
    "UpwardGaze": {"255533007": ("Upward gaze", [4201])},
    "UralBlackPiedCattleBreed": {"133556007": ("Ural Black Pied cattle breed", [7480])},
    "Ureter": {
        "87953007": ("Ureter", [4009, 4030, 4031, 4040, 4042, 7151, 7192, 9514])
    },
    "UreteralStent": {"286558002": ("Ureteral stent", [6202, 6203])},
    "UretericStent": {"286558002": ("Ureteric stent", [6102, 6138, 6404, 7151, 7193])},
    "UretericStructure": {"87953007": ("Ureteric structure (body structure)", [])},
    "Urethan": {"873008": ("Urethan (substance)", [])},
    "Urethane": {"873008": ("Urethane (ethyl carbamate)", [623, 625])},
    "Urethra": {
        "13648007": (
            "Urethra",
            [4009, 4030, 4031, 4042, 7151, 7192, 7482, 7483, 8134, 9514],
        )
    },
    "UrethralCatheter": {"34759008": ("Urethral catheter", [6202, 6203])},
    "UrethralCatheterDevice": {
        "34759008": ("Urethral catheter, device (physical object)", [])
    },
    "UrethralStructure": {"13648007": ("Urethral structure (body structure)", [])},
    "Urgency": {"103391001": ("Urgency (qualifier value)", [])},
    "UrgentProcedure": {"103391001": ("Urgent Procedure", [3414])},
    "UricAcid": {"1710001": ("Uric Acid", [300])},
    "UrinaryBladderAndUrethraCombinedSite": {
        "110837003": (
            "Urinary bladder and urethra (combined site) (body structure)",
            [],
        )
    },
    "UrinaryBladderStructure": {
        "89837001": ("Urinary bladder structure (body structure)", [])
    },
    "UrinarySystem": {"122489005": ("Urinary system", [7151, 7154, 7192, 8134, 9514])},
    "UrinarySystemStructure": {
        "122489005": ("Urinary system structure (body structure)", [])
    },
    "UrinaryTract": {"431938005": ("Urinary tract", [7482, 7483, 8134])},
    "Urine": {"78014005": ("Urine", [7151, 7197])},
    "Urokinase": {"59082006": ("Urokinase (substance)", [])},
    "UrokinasePreparation": {"59082006": ("Urokinase preparation", [10])},
    "Urology": {"309995005": ("Urology", [7030])},
    "UrologyDepartment": {"309995005": ("Urology department (environment)", [])},
    "UrostomyBag": {"344088002": ("Urostomy bag", [6202, 6203])},
    "UrostomySet": {"344575009": ("Urostomy set (physical object)", [])},
    "UrzhumPigBreed": {"132111003": ("Urzhum pig breed", [7480])},
    "UsesEquipment": {"116682006": ("Uses equipment (attribute)", [])},
    "UsingSubstance": {"424361007": ("Using substance (attribute)", [])},
    "UterineArtery": {
        "91079009": (
            "Uterine Artery",
            [3827, 7151, 7192, 8134, 9514, 12103, 12111, 12125, 12140],
        )
    },
    "UterineLeiomyoma": {"95315005": ("Uterine leiomyoma (disorder)", [])},
    "UterineParacervicalLymphNode": {
        "5394000": ("uterine paracervical lymph node", [7600, 8134])
    },
    "UterineStructure": {"35039007": ("Uterine structure (body structure)", [])},
    "UterineVein": {"60028002": ("Uterine Vein", [12140])},
    "Uterus": {
        "35039007": (
            "Uterus",
            [
                4,
                645,
                4030,
                6202,
                6204,
                7151,
                7160,
                7192,
                8134,
                9514,
                10044,
                10060,
                12023,
            ],
        )
    },
    "UterusAndFallopianTubes": {
        "110639002": (
            "Uterus and fallopian tubes",
            [4, 4009, 4030, 4031, 4040, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "UterusAndFallopianTubesCombinedSite": {
        "110639002": ("Uterus and fallopian tubes (combined site) (body structure)", [])
    },
    "Uvula": {"26140008": ("uvula", [7601, 8134])},
    "UvulaPalatinaStructure": {
        "26140008": ("Uvula palatina structure (body structure)", [])
    },
    "UzbekBlackGoatBreed": {"131643004": ("Uzbek Black goat breed", [7480])},
    "UzunyaylaHorseBreed": {"131959001": ("Uzunyayla horse breed", [7480])},
    "VagalStimulation": {"128968000": ("Vagal stimulation", [91, 3271])},
    "VagalStimulationPhysiologicChallenge": {
        "128968000": ("Vagal stimulation physiologic challenge (procedure)", [])
    },
    "Vagina": {"76784001": ("Vagina", [7151, 7160, 7192, 8134, 9514])},
    "VaginalApproach": {"54300008": ("Vaginal approach (qualifier value)", [])},
    "VaginalIntroitus": {"18857001": ("Vaginal introitus", [4029, 8134])},
    "VaginalIntroitusStructure": {
        "18857001": ("Vaginal introitus structure (body structure)", [])
    },
    "VaginalRoute": {"16857009": ("Vaginal route", [11])},
    "VaginalStructure": {"76784001": ("Vaginal structure (body structure)", [])},
    "VagusNerve": {
        "88882009": ("Vagus nerve", [7151, 7167, 7192, 7706, 7710, 8134, 9514])
    },
    "VagusNerveStructure": {"88882009": ("Vagus nerve structure (body structure)", [])},
    "Valdini": {"399330007": ("Valdini", [4012])},
    "ValdiniProjection": {"399330007": ("Valdini projection (qualifier value)", [])},
    "ValdresCattleBreed": {"133557003": ("Valdres cattle breed", [7480])},
    "ValsalvaManeuver": {
        "261039008": ("Valsalva maneuver", [91, 3271, 3823, 9272, 12002])
    },
    "ValvularEndocarditis": {"89736004": ("Valvular endocarditis", [3711])},
    "ValvularHeartDisease": {"368009": ("Valvular heart disease", [3201, 3727, 12246])},
    "VanGiesonStain": {"406957003": ("Van Gieson stain", [8112])},
    "VanRosenProjection": {"260506009": ("Van Rosen projection", [4012])},
    "Vancomycin": {"372735009": ("Vancomycin (substance)", [])},
    "VancomycinAntibiotic": {"372735009": ("Vancomycin antibiotic", [621, 622])},
    "VaricoseAneurysm": {"57754000": ("varicose aneurysm", [3808])},
    "VasDeferens": {"57671007": ("Vas deferens", [7151, 7160, 7192, 8134, 9514])},
    "VasDeferensStructure": {
        "57671007": ("Vas deferens structure (body structure)", [])
    },
    "VascularCalcification": {
        "237897009": ("Vascular Calcification", [3491, 3495, 3496, 3497, 7111])
    },
    "VascularCalcificationRadiographicFinding": {
        "129759000": ("Vascular calcification", [6010, 6011])
    },
    "VascularCoiling": {"416061003": ("vascular coiling", [3810])},
    "VascularConstrictionFunction": {
        "32318003": ("Vascular constriction, function (observable entity)", [])
    },
    "VascularDilatationFunction": {
        "30017007": ("Vascular dilatation, function (observable entity)", [])
    },
    "VascularEndothelialGrowthFactor": {
        "417324009": ("Vascular endothelial growth factor", [638, 642])
    },
    "VascularFlowFunction": {
        "58190003": ("Vascular flow, function (observable entity)", [])
    },
    "VascularGraft": {"312288001": ("Vascular graft (morphologic abnormality)", [])},
    "VascularMRI": {"241663008": ("vascular MRI", [3820])},
    "VascularResistance": {"251271006": ("Vascular Resistance", [12304])},
    "VascularResistanceFunction": {"88619007": ("Vascular Resistance", [3641])},
    "VascularSclerosis": {
        "107671003": ("vascular sclerosis", [3810, 7151, 7156, 7192, 8134, 9514])
    },
    "VascularStentCrossSectionalArea": {
        "408705002": ("Vascular stent cross sectional area (observable entity)", [])
    },
    "VascularStentDiameter": {
        "408706001": ("Vascular stent diameter (observable entity)", [])
    },
    "VascularStentLength": {
        "408703009": ("Vascular stent length (observable entity)", [])
    },
    "VascularStentVolume": {
        "408704003": ("Vascular stent volume (observable entity)", [])
    },
    "VascularStructureOfHead": {
        "281231009": ("Vascular structure of head (body structure)", [])
    },
    "VascularStructureOfKidney": {
        "303402001": ("Vascular structure of kidney (body structure)", [])
    },
    "VascularSurgery": {"309996006": ("Vascular Surgery", [7030])},
    "VascularSurgeryDepartment": {
        "309996006": ("Vascular surgery department (environment)", [])
    },
    "VascularWallDegeneration": {"33593002": ("vascular wall degeneration", [3817])},
    "Vasculitis": {"31996006": ("vasculitis", [3805, 3815])},
    "Vasoconstriction": {"32318003": ("Vasoconstriction", [9])},
    "Vasoconstrictor": {"372881000": ("Vasoconstrictor (substance)", [])},
    "Vasodilatation": {"30017007": ("Vasodilatation", [9])},
    "Vasodilator": {"372787008": ("Vasodilator", [10, 621, 622])},
    "Vasopressor": {"870406003": ("Vasoconstrictor", [621, 622])},
    "VasovagalSyncope": {"398665005": ("Vasovagal syncope", [6062])},
    "VaynolCattleBreed": {"133558008": ("Vaynol cattle breed", [7480])},
    "Vein": {
        "29092000": (
            "Vein",
            [3010, 3496, 3607, 3630, 4042, 7111, 7151, 7166, 7191, 7192, 8134, 9514],
        )
    },
    "VenaAnonyma": {"371951007": ("Vena anonyma (body structure)", [])},
    "VenaCava": {"35532006": ("Vena cava", [4, 3010, 4042, 7151, 7192, 8134, 9514])},
    "VenaCavaFilter": {
        "257409000": ("Vena cava filter", [6102, 6138, 6202, 6203, 6404, 7151, 7193])
    },
    "VenaCavalStructure": {"35532006": ("Vena caval structure (body structure)", [])},
    "VenaContracta": {"443501007": ("Vena Contracta", [12280])},
    "VendéenSheepBreed": {"131914009": ("Vendéen sheep breed", [7480])},
    "VenezuelanBlackPigBreed": {"132167006": ("Venezuelan Black pig breed", [7480])},
    "VenezuelanZebuCattleBreed": {
        "133672008": ("Venezuelan Zebu cattle breed", [7480])
    },
    "Venipuncture": {"22778000": ("Venipuncture", [8109])},
    "VenousGraftToCitedSegment": {
        "128951004": ("Venous graft to cited segment (substance)", [])
    },
    "VenousHemangioma": {
        "56468002": ("Venous hemangioma (morphologic abnormality)", [])
    },
    "VenousNetwork": {
        "34340008": ("Venous network", [4, 3010, 4042, 7151, 7192, 8134, 9514])
    },
    "VenousStructure": {"29092000": ("Venous structure (body structure)", [])},
    "Ventral20DegreeRostralDorsocaudalOblique": {
        "442721003": ("Ventral 20 degree rostral-dorsocaudal oblique", [7484])
    },
    "Ventral20DegreeRostralDorsocaudalObliqueProjection": {
        "442721003": (
            "Ventral 20 degree rostral-dorsocaudal oblique projection (qualifier value)",
            [],
        )
    },
    "Ventral30DegreeLeftDorsalRightOblique": {
        "442589002": ("Ventral 30 degree left-dorsal right oblique", [7484])
    },
    "Ventral30DegreeLeftDorsalRightObliqueProjection": {
        "442589002": (
            "Ventral 30 degree left-dorsal right oblique projection (qualifier value)",
            [],
        )
    },
    "Ventral30DegreeRightDorsalLeftOblique": {
        "442587000": ("Ventral 30 degree right-dorsal left oblique", [7484])
    },
    "Ventral30DegreeRightDorsalLeftObliqueProjection": {
        "442587000": (
            "Ventral 30 degree right-dorsal left oblique projection (qualifier value)",
            [],
        )
    },
    "VentralFuniculus": {"31701002": ("ventral funiculus", [7707, 7710, 8134])},
    "VentralFuniculusStructure": {
        "31701002": ("Ventral funiculus structure (body structure)", [])
    },
    "VentralLeftDorsalRightOblique": {
        "442590006": ("Ventral left-dorsal right oblique", [7484])
    },
    "VentralLeftDorsalRightObliqueProjection": {
        "442590006": (
            "Ventral left-dorsal right oblique projection (qualifier value)",
            [],
        )
    },
    "VentralRightDorsalLeftOblique": {
        "442738004": ("Ventral right-dorsal left oblique", [7484])
    },
    "VentralRightDorsalLeftObliqueProjection": {
        "442738004": (
            "Ventral right-dorsal left oblique projection (qualifier value)",
            [],
        )
    },
    "Ventricle": {
        "21814001": ("Ventricle", [4, 6102, 6109, 6113, 6149, 7151, 7192, 8134, 9514])
    },
    "VentricularAneurysm": {"90539001": ("Ventricular Aneurysm", [3704])},
    "VentricularArrhythmia": {"44103008": ("ventricular arrhythmia", [3826])},
    "VentricularBigeminy": {"11157007": ("Ventricular bigeminy", [3234])},
    "VentricularEffectiveRefractoryPeriod": {
        "129088006": ("Ventricular effective refractory period (observable entity)", [])
    },
    "VentricularEffectiveRefractoryPeriodEvaluationOf": {
        "129088006": ("Ventricular Effective Refractory Period, evaluation of", [3254])
    },
    "VentricularEjection": {"444371003": ("Ventricular Ejection", [12233])},
    "VentricularEjectionSWave": {
        "444371003": ("Ventricular Ejection (S-wave)", [12307])
    },
    "VentricularFibrillation": {"71908006": ("Ventricular fibrillation", [3230, 3700])},
    "VentricularIsovolumicContraction": {
        "444379001": ("Ventricular Isovolumic Contraction", [12233, 12307])
    },
    "VentricularIsovolumicRelaxation": {
        "444361000": ("Ventricular Isovolumic Relaxation", [12233, 12307])
    },
    "VentricularPacing": {"344994008": ("ventricular pacing", [3555])},
    "VentricularPreExcitation": {"195060002": ("Ventricular pre-excitation", [3230])},
    "VentricularPrematureBeats": {
        "17338001": ("Ventricular premature beats (disorder)", [])
    },
    "VentricularPrematureComplex": {
        "251175005": ("Ventricular premature complex (disorder)", [])
    },
    "VentricularPrematureContraction": {
        "251175005": ("Ventricular premature contraction", [3230])
    },
    "VentricularSeptalDefect": {
        "30288003": (
            "Ventricular Septal defect",
            [3610, 3630, 3700, 3728, 12236, 12244, 12248, 12249, 12286],
        )
    },
    "VentricularSeptalDefectDeviceClosure": {
        "89814007": ("Ventricular septal defect device closure", [12247])
    },
    "VentricularSystole": {"8997002": ("Ventricular Systole", [12233])},
    "VentricularSystoleFunction": {
        "8997002": ("Ventricular systole, function (observable entity)", [])
    },
    "VentricularTachycardia": {
        "25569003": ("Ventricular tachycardia", [3230, 3234, 3700])
    },
    "VentricularTachycardiaPolymorphic": {
        "251159007": ("Ventricular tachycardia, polymorphic", [3234])
    },
    "VentricularVein": {
        "195328002": ("Ventricular Vein", [3827, 3839, 7151, 7192, 9514])
    },
    "VentroDorsal": {"442441009": ("Ventro-dorsal", [7484])},
    "VentroDorsalProjection": {
        "442441009": ("Ventro-dorsal projection (qualifier value)", [])
    },
    "VentrorostralDorsocaudalOblique": {
        "442741008": ("Ventrorostral-dorsocaudal oblique", [7484])
    },
    "VentrorostralDorsocaudalObliqueProjection": {
        "442741008": (
            "Ventrorostral-dorsocaudal oblique projection (qualifier value)",
            [],
        )
    },
    "Verapamil": {"47898004": ("Verapamil", [10])},
    "VerataGoatBreed": {"131629009": ("Verata goat breed", [7480])},
    "VerhoeffHematoxylinStain": {"406992006": ("verhoeff's hematoxylin stain", [8112])},
    "VerinesaCattleBreed": {"133559000": ("Verinesa cattle breed", [7480])},
    "Vertebra": {"51282000": ("Vertebra", [6102, 6109, 6114, 7151, 7192, 9514])},
    "VertebralArtery": {
        "85234005": (
            "Vertebral artery",
            [
                4,
                3010,
                3604,
                3606,
                3630,
                3827,
                4042,
                6102,
                6109,
                6117,
                7151,
                7192,
                8134,
                9514,
                12103,
                12104,
            ],
        )
    },
    "VertebralCanal": {"61853006": ("Vertebral canal", [6115, 8134])},
    "VertebralColumnAndCranium": {
        "110517009": (
            "Vertebral column and cranium",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "VertebralColumnAndCraniumCombinedSite": {
        "110517009": (
            "Vertebral column and cranium (combined site) (body structure)",
            [],
        )
    },
    "VertebralForamen": {"280734009": ("Vertebral foramen", [6115, 8134])},
    "VertexOfHead": {"88986008": ("Vertex of Head", [1000, 1001, 1002, 8134])},
    "VertexStructure": {"88986008": ("Vertex structure (body structure)", [])},
    "Vertical": {"33096000": ("Vertical", [2, 6121])},
    "VerticalLongAxis": {"131185001": ("Vertical Long Axis", [26, 27, 501])},
    "VerticalOccipitalFasciculus": {
        "80434005": ("vertical occipital fasciculus", [7703, 7710, 8134])
    },
    "Verticosubmental": {"399360002": ("verticosubmental", [501, 4010])},
    "VerticosubmentalProjection": {
        "399360002": ("Verticosubmental projection (qualifier value)", [])
    },
    "VeryHigh": {"260360000": ("Very high (qualifier value)", [])},
    "VerzascaBlackGoatBreed": {
        "131630004": ("Verzasca black goat breed (organism)", [])
    },
    "VerzascaGoatBreed": {"131630004": ("Verzasca goat breed", [7480])},
    "VesicularLymphNode": {"360992006": ("vesicular lymph node", [7600, 8134])},
    "VesselDepthFromSurface": {
        "413975003": ("Vessel depth from surface", [12119, 12122])
    },
    "VesselIntimalCrossSectionalDiameter": {
        "415814008": ("Vessel Intimal Cross-Sectional Diameter", [12119, 12122])
    },
    "VesselIntimalDiameter": {"415815009": ("Vessel Intimal Diameter", [12119, 12122])},
    "VesselLumenCrossSectionalArea": {
        "397415007": ("Vessel lumen cross-sectional area", [3482, 12119, 12122])
    },
    "VesselLumenCrossSectionalAreaIncrease": {
        "415816005": (
            "Vessel lumen cross-sectional area increase (observable entity)",
            [],
        )
    },
    "VesselLumenCrossSectionalDiameterIncrease": {
        "415817001": (
            "Vessel lumen cross-sectional diameter increase (observable entity)",
            [],
        )
    },
    "VesselLumenDiameter": {
        "397413000": ("Vessel lumen diameter", [3481, 12119, 12122])
    },
    "VesselOrigin": {"397421006": ("Vessel Origin", [3496, 7111])},
    "VesselOutsideDiameter": {"397414006": ("Vessel outside diameter", [12119, 12122])},
    "VestibulocochlearNerve": {
        "8598002": ("vestibulocochlear nerve", [7706, 7710, 8134])
    },
    "VestibulocochlearNerveStructure": {
        "8598002": ("Vestibulocochlear nerve structure (body structure)", [])
    },
    "VestlandFjordCattleBreed": {"133281009": ("Vestland Fjord cattle breed", [7480])},
    "VestlandRedPolledCattleBreed": {
        "133282002": ("Vestland Red Polled cattle breed", [7480])
    },
    "ViaArmVein": {"261459001": ("Via arm vein", [3403, 3746])},
    "ViaArtery": {"103387006": ("Via artery", [3403, 3746])},
    "ViaBrachialArtery": {"260585005": ("Via brachial artery", [3403, 3746])},
    "ViaFemoralArtery": {"260590008": ("Via femoral artery", [3403, 3746])},
    "ViaFemoralVein": {"260601006": ("Via femoral vein", [3403, 3746])},
    "ViaRadialArtery": {"444850002": ("Via radial artery", [3403, 3746])},
    "ViaTracheostomy": {"180640008": ("Via tracheostomy", [617, 618])},
    "ViaVein": {"103386002": ("Via vein", [3403, 3746])},
    "VianesaCattleBreed": {"133560005": ("Vianesa cattle breed", [7480])},
    "VichPigBreed": {"132201003": ("Vich pig breed", [7480])},
    "VictoriaBlue4RStain": {"82682000": ("victoria blue 4R stain", [8112])},
    "VictoriaBlueBStain": {"22749001": ("victoria blue B stain", [8112])},
    "VictoriaCattleBreed": {"133840007": ("Victoria cattle breed (organism)", [])},
    "VictoriaXZebuCattleBreed": {"133840007": ("Victoria X zebu cattle breed", [7480])},
    "VictorianBulldogge": {"426571006": ("Victorian Bulldogge (organism)", [])},
    "VictorianBulldoggeDogBreed": {
        "426571006": ("Victorian Bulldogge dog breed", [7480])
    },
    "VideoEEG": {"252738008": ("Video EEG", [3049])},
    "VietnamesePigBreed": {"132202005": ("Vietnamese pig breed", [7480])},
    "VietnamesePotBelliedPigPigBreed": {
        "132230005": ("Vietnamese Pot-Bellied Pig pig breed", [7480])
    },
    "VietnameseYellowCattleBreed": {
        "131545004": ("Vietnamese Yellow cattle breed", [7480])
    },
    "VietnameseYorkshirePigBreed": {
        "132307002": ("Vietnamese Yorkshire pig breed", [7480])
    },
    "VikingHorse": {"1118004": ("Viking horse (organism)", [])},
    "VikingHorseBreed": {"1118004": ("Viking horse breed", [7480])},
    "VillardDeLansCattleBreed": {"133561009": ("Villard-de-Lans cattle breed", [7480])},
    "VioletOpticalFilter": {"445278001": ("Violet optical filter", [8124])},
    "VioletOpticalFilterOfOpticalMicroscope": {
        "445278001": (
            "Violet optical filter of optical microscope (physical object)",
            [],
        )
    },
    "ViralHepatitisTypeB": {"66071002": ("Viral hepatitis type B (disorder)", [])},
    "ViralHepatitisTypeC": {"50711007": ("Viral hepatitis type C (disorder)", [])},
    "Virus": {"49872002": ("Virus", [637])},
    "VisualEstimation": {"258083009": ("Visual estimation", [3627])},
    "VitaVetLabMinipigPigBreed": {
        "132041004": ("Vita Vet Lab Minipig pig breed", [7480])
    },
    "VitalNewRedStain": {"20230008": ("vital new red stain", [8112])},
    "VitalSignsMonitoring": {"304495004": ("Vital signs monitoring", [9577])},
    "Vitamin": {"87708000": ("Vitamin", [621, 622])},
    "VitellineArteryOfPlacenta": {
        "256779006": ("Vitelline Artery of Placenta", [12140])
    },
    "VitellineVeinOfPlacenta": {"256875007": ("Vitelline Vein of Placenta", [12140])},
    "VitoriaPigBreed": {"132203000": ("Vitoria pig breed", [7480])},
    "Vitreous": {"26386000": ("Vitreous", [4211, 8134])},
    "VitreousBodyPart": {"372242005": ("Vitreous body part (body structure)", [])},
    "VitreousCavity": {"26386000": ("Vitreous Cavity", [4233])},
    "VitreousCavityStructure": {
        "26386000": ("Vitreous cavity structure (body structure)", [])
    },
    "VitreousOnly": {"372242005": ("Vitreous Only", [4232])},
    "VitreousOpacities": {"422061002": ("Vitreous opacities (disorder)", [])},
    "VitreousOpacity": {"422061002": ("Vitreous Opacity", [4222])},
    "VizslaSuperbreed": {"52105008": ("Vizsla superbreed (organism)", [])},
    "VizslaSuperbreedDogBreed": {"52105008": ("Vizsla superbreed dog breed", [7480])},
    "VlaamperdHorseBreed": {"133006006": ("Vlaamperd horse breed", [7480])},
    "VladimirHeavyDraftHorseBreed": {
        "133005005": ("Vladimir Heavy Draft horse breed", [7480])
    },
    "VogelsbergCattleBreed": {"133562002": ("Vogelsberg cattle breed", [7480])},
    "Volume": {
        "118565006": (
            "Volume",
            [218, 3423, 4050, 7469, 7472, 12257, 12258, 12259, 12265, 12271, 12304],
        )
    },
    "VolumeProperty": {"118565006": ("Volume (property) (qualifier value)", [])},
    "VolumeSupportVentilator": {"371787006": ("Volume Support Ventilator", [3554])},
    "VolynskCattleBreed": {"132780000": ("Volynsk cattle breed", [7480])},
    "VomerBone": {"87166008": ("Vomer bone", [4028, 7151, 7192, 8134, 9514])},
    "VomerBoneStructure": {"87166008": ("Vomer bone structure (body structure)", [])},
    "VorderwaldCattleBreed": {"133564001": ("Vorderwald cattle breed", [7480])},
    "VoronezhCoachHorseHorseBreed": {
        "131960006": ("Voronezh Coach Horse horse breed", [7480])
    },
    "VosgesCattleBreed": {"133589007": ("Vosges cattle breed", [7480])},
    "Vulva": {"45292006": ("Vulva", [4, 7151, 7160, 7192, 8134, 9514])},
    "VulvalStructure": {"45292006": ("Vulval structure (body structure)", [])},
    "VulvalVestibule": {"23213005": ("Vulval vestibule", [4029, 8134])},
    "VulvalVestibuleStructure": {
        "23213005": ("Vulval vestibule structure (body structure)", [])
    },
    "VyatkaHorseBreed": {"133004009": ("Vyatka horse breed", [7480])},
    "WagyuCattleBreed": {"133283007": ("Wagyu cattle breed", [7480])},
    "WaiChowPigBreed": {"132204006": ("Wai Chow pig breed", [7480])},
    "WakwaCattleBreed": {"133683004": ("Wakwa cattle breed", [7480])},
    "WalachenschafSheepBreed": {"132856007": ("Walachenschaf sheep breed", [7480])},
    "WaldeyerRingStructure": {
        "17861009": ("Waldeyer's ring structure (body structure)", [])
    },
    "WalerHorseBreed": {"133015004": ("Waler horse breed", [7480])},
    "WalkingSurfaceOfRoom": {
        "709280007": ("Walking surface of room (physical object)", [])
    },
    "Wall": {"224727009": ("Wall", [7151, 7157, 7193, 9505, 9515, 9520, 9573])},
    "WallisBlacknosedSheepBreed": {
        "132699004": ("Wallis Blacknosed Sheep breed", [7480])
    },
    "WallisCountrySheepBreed": {"132702006": ("Wallis Country Sheep breed", [7480])},
    "Walls": {"224727009": ("Walls (physical object)", [])},
    "WanzheSpottedChunanSpottedPigBreed": {
        "132310009": ("Wanzhe Spotted, Chunan Spotted pig breed", [7480])
    },
    "WanzheSpottedPigBreed": {"132309004": ("Wanzhe Spotted pig breed", [7480])},
    "WanzheSpottedWannanSpottedPigBreed": {
        "132311008": ("Wanzhe Spotted, Wannan Spotted pig breed", [7480])
    },
    "Warfarin": {"48603004": ("Warfarin", [10])},
    "WarmbloodHorseBreed": {"407402001": ("Warmblood horse breed", [7480])},
    "WarmerDevice": {"71384000": ("Warmer device", [635])},
    "WarmingBlanket": {"421335007": ("Warming blanket", [635])},
    "WasteMaterial": {"289925000": ("Waste Material", [7151, 7165, 7196])},
    "Water": {
        "11713004": ("Water", [12, 13, 218, 300, 3850, 6350, 7180, 7469, 11002, 11006])
    },
    "WaterBagProsthesisForImagingProcedure": {
        "102323003": ("Water bag prosthesis for imaging procedure", [5])
    },
    "WaterBagProsthesisForImagingProcedureDevice": {
        "102323003": (
            "Water bag prosthesis for imaging procedure, device (physical object)",
            [],
        )
    },
    "WaterSolubleAnilineBlueStain": {
        "88625006": ("water soluble aniline blue stain", [8112])
    },
    "WaterSolubleAnthraceneBrownStain": {
        "58755002": ("water soluble anthracene brown stain", [8112])
    },
    "WaterSolubleEosinStain": {"36879007": ("water soluble eosin stain", [8112])},
    "WaterSolubleNigrosineStain": {
        "12119009": ("water soluble nigrosine stain", [8112])
    },
    "Waters": {"260473000": ("Waters", [4012])},
    "Waters35DegreeTiltToRadiographicBaseline": {
        "260473000": (
            "Waters - 35 degree tilt to radiographic baseline (qualifier value)",
            [],
        )
    },
    "WatusiUSACattleBreed": {"131546003": ("Watusi (USA) cattle breed", [7480])},
    "WatusiUnitedStatesOfAmericaCattleBreed": {
        "131546003": ("Watusi (United States of America) cattle breed (organism)", [])
    },
    "Waveform": {"260893003": ("Waveform", [272])},
    "WaxolineBlueStain": {"60739006": ("waxoline blue stain", [8112])},
    "WaysonStain": {"409549005": ("wayson stain", [8112])},
    "WaziriHorseBreed": {"131872001": ("Waziri horse breed", [7480])},
    "WaziriSheepBreed": {"131800005": ("Waziri sheep breed", [7480])},
    "WeaklyPositive": {"260408008": ("Weakly positive", [3231])},
    "Weal": {"247472004": ("Weal", [6062])},
    "WedgeMethod": {"128579004": ("Wedge method", [3241])},
    "WedgeShape": {"20446002": ("Wedge shape (qualifier value)", [])},
    "WedgeShaped": {"20446002": ("Wedge-shaped", [6335, 6339, 6340])},
    "WeightBearing": {"87731000": ("Weight bearing", [91])},
    "WeightBearingFunctionOfBone": {
        "87731000": ("Weight bearing function of bone (observable entity)", [])
    },
    "Weimaraner": {"69249004": ("Weimaraner", [7480])},
    "WeisseHornloseHeidschnuckeSheepBreed": {
        "132684004": ("Weisse Hornlose Heidschnucke sheep breed", [7480])
    },
    "WeleraPonyHorseBreed": {"133003003": ("Welera Pony horse breed", [7480])},
    "WellControlled": {"1194003": ("Well controlled", [3770])},
    "WellDefined": {"260409000": ("Well defined", [6120])},
    "WelshCobHorseBreed": {"131962003": ("Welsh Cob horse breed", [7480])},
    "WelshCorgiSuperbreed": {"37024005": ("Welsh corgi superbreed (organism)", [])},
    "WelshCorgiSuperbreedDogBreed": {
        "37024005": ("Welsh corgi superbreed dog breed", [7480])
    },
    "WelshHillSpeckledFaceSheepBreed": {
        "131798003": ("Welsh Hill Speckled Face sheep breed", [7480])
    },
    "WelshMountainBadgerFacedSheepBreed": {
        "131769002": ("Welsh Mountain Badger Faced sheep breed", [7480])
    },
    "WelshMountainPonyHorseBreed": {
        "131963008": ("Welsh Mountain Pony horse breed", [7480])
    },
    "WelshMountainSheepBreed": {"132798001": ("Welsh Mountain sheep breed", [7480])},
    "WelshPig": {"54232006": ("Welsh pig (organism)", [])},
    "WelshPigBreed": {"54232006": ("Welsh pig breed", [7480])},
    "WelshPonyHorseBreed": {"132744000": ("Welsh Pony horse breed", [7480])},
    "WelshPonyOfCobTypeHorseBreed": {
        "132745004": ("Welsh Pony of Cob Type horse breed", [7480])
    },
    "WelshSheepdogBreed": {"132588005": ("Welsh Sheepdog breed", [7480])},
    "WelshSpringerSpaniel": {"40898002": ("Welsh Springer spaniel (organism)", [])},
    "WelshSpringerSpanielDogBreed": {
        "40898002": ("Welsh Springer spaniel dog breed", [7480])
    },
    "WelshTerrier": {"49421002": ("Welsh terrier", [7480])},
    "WelshWalkingHorse": {"8089006": ("Welsh walking horse (organism)", [])},
    "WelshWalkingHorseBreed": {"8089006": ("Welsh walking horse breed", [7480])},
    "WenshanCattleBreed": {"131547007": ("Wenshan cattle breed", [7480])},
    "WensleydaleSheepBreed": {"131780002": ("Wensleydale sheep breed", [7480])},
    "WessexSaddlebackPig": {"73648005": ("Wessex saddleback pig (organism)", [])},
    "WessexSaddlebackPigBreed": {"73648005": ("Wessex saddleback pig breed", [7480])},
    "WestAfricanBarbHorseBreed": {
        "132995007": ("West African Barb horse breed", [7480])
    },
    "WestAfricanDwarfSheepBreed": {
        "131781003": ("West African Dwarf sheep breed", [7480])
    },
    "WestAfricanDwarfShorthornCattleBreed": {
        "133565000": ("West African Dwarf Shorthorn cattle breed", [7480])
    },
    "WestAfricanPigBreed": {"133185005": ("West African pig breed", [7480])},
    "WestFinnishCattleBreed": {"133566004": ("West Finnish cattle breed", [7480])},
    "WestFrenchWhitePigBreed": {"132769000": ("West French White pig breed", [7480])},
    "WestHighlandWhiteTerrier": {"40727008": ("West Highland white terrier", [7480])},
    "WestMacedonianCattleBreed": {
        "133567008": ("West Macedonian cattle breed", [7480])
    },
    "WestPoint": {"399130002": ("West Point", [4012])},
    "WestPointProjection": {
        "399130002": ("West Point projection (qualifier value)", [])
    },
    "WestSiberianLaikaDogBreed": {
        "132396004": ("West Siberian Laika dog breed", [7480])
    },
    "WesternSudanPonyHorseBreed": {
        "133002008": ("Western Sudan Pony horse breed", [7480])
    },
    "WestphalianHorse": {"25369002": ("Westphalian horse (organism)", [])},
    "WestphalianHorseBreed": {"25369002": ("Westphalian horse breed", [7480])},
    "WestrainPigBreed": {"132049002": ("Westrain pig breed", [7480])},
    "WhippetDog": {"76351004": ("Whippet dog", [7480])},
    "White": {"371251000": ("White", [4255, 4411])},
    "WhiteBullTerrier": {"42902003": ("White bull terrier (organism)", [])},
    "WhiteBullTerrierDogBreed": {"42902003": ("White bull terrier dog breed", [7480])},
    "WhiteColor": {"371251000": ("White color", [6067])},
    "WhiteCáceresCattleBreed": {"133284001": ("White Cáceres cattle breed", [7480])},
    "WhiteFulaniCattleBreed": {"133684005": ("White Fulani cattle breed", [7480])},
    "WhiteGallowayCattleBreed": {"133569006": ("White Galloway cattle breed", [7480])},
    "WhiteMatter": {"389080008": ("White Matter", [7151, 7153, 7192, 8134, 9514])},
    "WhiteMatterOfBrainAndSpinalCord": {
        "389080008": ("White matter of brain and spinal cord", [7710])
    },
    "WhiteMatterStructureOfBrainAndSpinalCord": {
        "389080008": (
            "White matter structure of brain and spinal cord (body structure)",
            [],
        )
    },
    "WhiteParkCattleBreed": {"26525003": ("White Park cattle breed (organism)", [])},
    "WhiteParkCowBreed": {"26525003": ("White Park cow breed", [7480])},
    "WhiteShepherdDogBreed": {"132458004": ("White Shepherd dog breed", [7480])},
    "WhiteSuffolkSheepBreed": {"131782005": ("White Suffolk sheep breed", [7480])},
    "WhiteWelshCattleBreed": {"133570007": ("White Welsh cattle breed", [7480])},
    "WhitebredShorthornCattleBreed": {
        "133568003": ("Whitebred Shorthorn cattle breed", [7480])
    },
    "WhitefaceDartmoorSheepBreed": {
        "131783000": ("Whiteface Dartmoor sheep breed", [7480])
    },
    "WhitefaceWoodlandSheepBreed": {
        "131784006": ("Whiteface Woodland sheep breed", [7480])
    },
    "Width": {"103355008": ("Width", [218, 6165, 7469, 7470])},
    "WielkopolskiHorseBreed": {"132741008": ("Wielkopolski horse breed", [7480])},
    "WigbyTaylor": {"399215000": ("Wigby-Taylor", [4012])},
    "WigbyTaylorProjection": {
        "399215000": ("Wigby-Taylor projection (qualifier value)", [])
    },
    "WiltshireHornSheep": {"39855006": ("Wiltshire horn sheep (organism)", [])},
    "WiltshireHornSheepBreed": {"39855006": ("Wiltshire horn sheep breed", [7480])},
    "WiltshirePigBreed": {"132207004": ("Wiltshire pig breed", [7480])},
    "WindsnyerPigBreed": {"133188007": ("Windsnyer pig breed", [7480])},
    "Wing": {"53036007": ("Wing", [7482, 7483, 8134])},
    "WingStructure": {"53036007": ("Wing structure (body structure)", [])},
    "WireFoxTerrier": {"41584008": ("Wire fox terrier (organism)", [])},
    "WireFoxTerrierDogBreed": {"41584008": ("Wire fox terrier dog breed", [7480])},
    "WirehairStyrianMountainDogBreed": {
        "132459007": ("Wirehair Styrian mountain dog breed", [7480])
    },
    "WirehairedMiniatureDachshund": {
        "56243001": ("Wirehaired miniature dachshund (organism)", [])
    },
    "WirehairedMiniatureDachshundDogBreed": {
        "56243001": ("Wirehaired miniature dachshund dog breed", [7480])
    },
    "WirehairedPointingGriffonDog": {
        "33401005": ("Wirehaired pointing griffon dog", [7480])
    },
    "WirehairedStandardDachshund": {
        "36274006": ("Wirehaired standard dachshund (organism)", [])
    },
    "WirehairedStandardDachshundDogBreed": {
        "36274006": ("Wirehaired standard dachshund dog breed", [7480])
    },
    "WirehairedVizsla": {"583000": ("Wirehaired vizsla (organism)", [])},
    "WirehairedVizslaDogBreed": {"583000": ("Wirehaired vizsla dog breed", [7480])},
    "WithinNormalLimits": {"125112009": ("Within normal limits", [4254])},
    "WitrikCattleBreed": {"133571006": ("Witrik cattle breed", [7480])},
    "WokalupCattleBreed": {"133841006": ("Wokalup cattle breed (organism)", [])},
    "WokalupXZebuCattleBreed": {"133841006": ("Wokalup X zebu cattle breed", [7480])},
    "Wolf": {"82676003": ("Wolf", [7480])},
    "WolfProjection": {"422996004": ("Wolf", [4012])},
    "WoodenLegGoatBreed": {"131673008": ("Wooden Leg goat breed", [7480])},
    "WrightStain": {"373682001": ("wright stain", [8112])},
    "WristJoint": {
        "74670003": (
            "Wrist joint",
            [4, 1000, 1001, 1006, 4009, 4030, 4031, 4042, 7151, 7192, 7304, 8134, 9514],
        )
    },
    "WristJointStructure": {"74670003": ("Wrist joint structure (body structure)", [])},
    "WristRegionStructure": {
        "8205005": ("Wrist region structure (body structure)", [])
    },
    "WujinPigBreed": {"132151000": ("Wujin pig breed", [7480])},
    "WurttembergHorseBreed": {"131965001": ("Wurttemberg horse breed", [7480])},
    "WuyiBlackPigBreed": {"132304009": ("Wuyi Black pig breed", [7480])},
    "WuzhishanPigBreed": {"132222009": ("Wuzhishan pig breed", [7480])},
    "XRayFilmCassetteManual": {
        "466556008": ("X-Ray film cassette, manual", [9268, 9271])
    },
    "XRayShield": {"65577000": ("X-Ray shield", [10066])},
    "XRayShieldDevice": {"65577000": ("X-Ray shield, device (physical object)", [])},
    "Xenon": {"83598005": ("Xenon", [13])},
    "Xenon133Gas": {"780783002": ("Xenon^133^ gas", [25])},
    "XenonArcLamp": {"445671003": ("Xenon arc lamp", [8125])},
    "XenonArcLampMicroscopeIlluminator": {
        "445671003": ("Xenon arc lamp microscope illuminator (physical object)", [])
    },
    "XiangPigBreed": {"132291006": ("Xiang pig breed", [7480])},
    "XiangxiBlackPigBreed": {"132314000": ("Xiangxi Black pig breed", [7480])},
    "XilingolHorseBreed": {"131966000": ("Xilingol horse breed", [7480])},
    "XinjiangBrownCattleBreed": {"133285000": ("Xinjiang Brown cattle breed", [7480])},
    "XinjiangFinewoolSheepBreed": {
        "131785007": ("Xinjiang Finewool sheep breed", [7480])
    },
    "XinjiangWhitePigBreed": {"132341005": ("Xinjiang White pig breed", [7480])},
    "XinjinJilinBlackPigBreed": {
        "132325009": ("Xinjin, Jilin Black pig breed", [7480])
    },
    "XinjinNingAnPigBreed": {"132326005": ("Xinjin, Ning-an pig breed", [7480])},
    "XinjinPigBreed": {"132324008": ("Xinjin pig breed", [7480])},
    "XinjinXinjinPigBreed": {"132329003": ("Xinjin, Xinjin pig breed", [7480])},
    "XiphoidProcess": {"20298003": ("Xiphoid Process", [1000, 1001, 1004])},
    "XiphoidProcessOfSternum": {
        "20298003": ("Xiphoid process of sternum", [6115, 8134])
    },
    "Xylazine": {"96230006": ("Xylazine", [623, 625])},
    "XylazineHydrochloride": {"96230006": ("Xylazine hydrochloride (substance)", [])},
    "XyleneCyanolFFStain": {"55831004": ("xylene cyanol FF stain", [8112])},
    "YacumentoCattleBreed": {"133572004": ("Yacumento cattle breed", [7480])},
    "YakutCattleBreed": {"131548002": ("Yakut cattle breed", [7480])},
    "YakutHorseBreed": {"133000000": ("Yakut horse breed", [7480])},
    "YananPigBreed": {"132153002": ("Yanan pig breed", [7480])},
    "YanbianCattleBreed": {"133286004": ("Yanbian cattle breed", [7480])},
    "YanqiHorseBreed": {"131967009": ("Yanqi horse breed", [7480])},
    "YaroslavlCattleBreed": {"133573009": ("Yaroslavl cattle breed", [7480])},
    "Yellow": {"371244009": ("Yellow", [4255, 4411])},
    "YellowColor": {"371244009": ("Yellow color", [6067])},
    "YellowGreenOpticalFilter": {"445340000": ("Yellow-green optical filter", [4204])},
    "YellowGreenOpticalFilterOfOpticalMicroscope": {
        "445340000": (
            "Yellow-green optical filter of optical microscope (physical object)",
            [],
        )
    },
    "YemeniHorsesHorseBreed": {"131968004": ("Yemeni Horses horse breed", [7480])},
    "YemeniZebuCattleBreed": {"133685006": ("Yemeni Zebu cattle breed", [7480])},
    "Yes": {"373066001": ("Yes", [230, 231, 10072])},
    "YiliHorseBreed": {"131969007": ("Yili horse breed", [7480])},
    "YiliWhitePigBreed": {"132340006": ("Yili White pig breed", [7480])},
    "YimengBlackPigBreed": {"132138005": ("Yimeng Black pig breed", [7480])},
    "YiwuHorseBreed": {"131970008": ("Yiwu horse breed", [7480])},
    "YonaguniHorseBreed": {"132999001": ("Yonaguni horse breed", [7480])},
    "YorkshireBlueAndWhitePigBreed": {
        "132205007": ("Yorkshire Blue and White pig breed", [7480])
    },
    "YorkshireCoachHorse": {"31633003": ("Yorkshire coach horse (organism)", [])},
    "YorkshireCoachHorseBreed": {"31633003": ("Yorkshire coach horse breed", [7480])},
    "YorkshirePig": {"85315007": ("Yorkshire pig (organism)", [])},
    "YorkshirePigBreed": {"85315007": ("Yorkshire pig breed", [7480])},
    "YorkshireTerrier": {"13284009": ("Yorkshire terrier", [7480])},
    "Yttrium": {"63754004": ("Yttrium", [300])},
    "Yttrium90Microspheres": {"764678003": ("Yttrium^90^ microspheres", [25])},
    "Yttrium90Microspheres_Inactive": {
        "409105003": ("Yttrium-90 microspheres (product)", [])
    },
    "YucaPig": {"15443006": ("Yuca pig (organism)", [])},
    "YucaPigBreed": {"15443006": ("Yuca pig breed", [7480])},
    "YucatanMinaturePigBreed": {"132226007": ("Yucatan Minature pig breed", [7480])},
    "YucatanMiniaturePigBreed": {
        "132226007": ("Yucatan Miniature pig breed (organism)", [])
    },
    "YuedongBlackPigBreed": {"132290007": ("Yuedong Black pig breed", [7480])},
    "YugoslavianHoundDogBreed": {"132460002": ("Yugoslavian Hound dog breed", [7480])},
    "YujiangPigBreed": {"132308007": ("Yujiang pig breed", [7480])},
    "YunnanHorseBreed": {"131971007": ("Yunnan horse breed", [7480])},
    "YunnanZebuCattleBreed": {"131549005": ("Yunnan Zebu cattle breed", [7480])},
    "YurinoCattleBreed": {"133574003": ("Yurino cattle breed", [7480])},
    "ZambiaAngoniCattleBreed": {"131550005": ("Zambia Angoni cattle breed", [7480])},
    "Zanelli": {"399026003": ("Zanelli", [4012])},
    "ZanelliProjection": {"399026003": ("Zanelli projection (qualifier value)", [])},
    "ZanzibarZebuCattleBreed": {"133630007": ("Zanzibar Zebu cattle breed", [7480])},
    "ZaobeiCattleBreed": {"133287008": ("Zaobei cattle breed", [7480])},
    "ZavotCattleBreed": {"133288003": ("Zavot cattle breed", [7480])},
    "ZenkerFixative": {"433473008": ("Zenker fixative (substance)", [])},
    "ZenkerFluid": {"433473008": ("Zenker's fluid", [8114])},
    "ZhemaichuHorseBreed": {"132998009": ("Zhemaichu horse breed", [7480])},
    "ZhongweiGoatBreed": {"131644005": ("Zhongwei goat breed", [7480])},
    "ZiehlNeelsenStain": {"255813003": ("Ziehl-Neelsen stain", [8112])},
    "ZlotnikiSpottedPigBreed": {"132259005": ("Zlotniki Spotted pig breed", [7480])},
    "ZlotnikiWhitePigBreed": {"132260000": ("Zlotniki White pig breed", [7480])},
    "ZnamenskCattleBreed": {"133289006": ("Znamensk cattle breed", [7480])},
    "Zolazepam": {"96227004": ("Zolazepam", [623, 625])},
    "ZolazepamHydrochloride": {"96227004": ("Zolazepam hydrochloride (substance)", [])},
    "Zygoma": {
        "13881006": (
            "Zygoma",
            [4, 4009, 4028, 4030, 4031, 4042, 7151, 7192, 8134, 9514],
        )
    },
    "ZygomaticBoneStructure": {
        "13881006": ("Zygomatic bone structure (body structure)", [])
    },
    "_0NoPerfusion": {"371867000": ("0: No Perfusion", [3713])},
    "_103Palladium": {"9351000": ("^103^Palladium", [9528])},
    "_106Ruthenium": {"8227001": ("^106^Ruthenium", [9528])},
    "_10OClockPosition": {"129781005": ("10 o'clock position", [6018, 6019, 6205])},
    "_10OClockPositionOnMammogram": {
        "129781005": ("10 o'clock position on mammogram (finding)", [])
    },
    "_10thPercentileValueOfPopulation": {
        "371890005": ("10th Percentile Value of population", [221, 226, 7465])
    },
    "_111Indium": {"56609000": ("^111^Indium", [18])},
    "_113mIndium": {"48895003": ("^113m^Indium", [18])},
    "_11Carbon": {"40565003": ("^11^Carbon", [4020])},
    "_11OClockPosition": {"129782003": ("11 o'clock position", [6018, 6019, 6205])},
    "_11OClockPositionOnMammogram": {
        "129782003": ("11 o'clock position on mammogram (finding)", [])
    },
    "_123Iodine": {"21572004": ("^123^Iodine", [18])},
    "_124Iodine": {"40937006": ("^124^Iodine", [4020])},
    "_125Iodine": {"68630002": ("^125^Iodine", [18, 9528])},
    "_127Xenon": {"27081007": ("^127^Xenon", [18])},
    "_12LeadECG": {"268400002": ("12-Lead ECG", [3670])},
    "_12LeadElectrocardiogram": {
        "268400002": ("12 lead electrocardiogram (procedure)", [])
    },
    "_12OClockPosition": {"129783008": ("12 o'clock position", [6018, 6019, 6205])},
    "_12OClockPositionOnMammogram": {
        "129783008": ("12 o'clock position on mammogram (finding)", [])
    },
    "_131Cesium": {"13237009": ("^131^Cesium", [9528])},
    "_131Iodine": {"1368003": ("^131^Iodine", [18])},
    "_133Barium": {"3027009": ("^133^Barium", [18])},
    "_133Xenon": {"80751004": ("^133^Xenon", [18])},
    "_137Cesium": {"55117002": ("^137^Cesium", [9528])},
    "_13Nitrogen": {"21576001": ("^13^Nitrogen", [3107, 4020])},
    "_14Carbon": {"71647005": ("^14^Carbon", [18])},
    "_14Oxygen": {"424875009": ("^14^Oxygen", [4020])},
    "_153Gadolinium": {"14529005": ("^153^Gadolinium", [18])},
    "_153Samarium": {"419804008": ("^153^Samarium", [18])},
    "_15LeadECG": {"429163003": ("15-Lead ECG", [3670])},
    "_15LeadElectrocardiographicMonitoring": {
        "429163003": ("15 lead electrocardiographic monitoring (procedure)", [])
    },
    "_15Oxygen": {"129504001": ("^15^Oxygen", [4020])},
    "_169Ytterbium": {"41758004": ("^169^Ytterbium", [18, 9528])},
    "_177Lutetium": {"447553000": ("^177^Lutetium", [18])},
    "_178Tantalum": {"6301006": ("^178^Tantalum", [18])},
    "_186Rhenium": {"395865006": ("^186^Rhenium", [18])},
    "_188Rhenium": {"423578007": ("^188^Rhenium", [18])},
    "_18FFluoroethylflumazenil": {
        "422975006": ("[(18)F]fluoroethylflumazenil (product)", [])
    },
    "_18Fluorine": {"77004003": ("^18^Fluorine", [18, 4020])},
    "_18LeadECG": {"425808002": ("18-Lead ECG", [3670])},
    "_18LeadElectrocardiographicMonitoring": {
        "425808002": ("18 lead electrocardiographic monitoring (procedure)", [])
    },
    "_192Iridium": {"48341001": ("^192^Iridium", [9528])},
    "_198Gold": {"24301009": ("^198^Gold", [18, 9528])},
    "_199Gold": {"70544003": ("^199^Gold", [18])},
    "_1OClockPosition": {"129772004": ("1 o'clock position", [6018, 6019, 6205])},
    "_1OClockPositionOnMammogram": {
        "129772004": ("1 o'clock position on mammogram (finding)", [])
    },
    "_1PenetrationWithoutPerfusion": {
        "371866009": ("1: Penetration without Perfusion", [3713])
    },
    "_1Phenylpropan2Amine": {"703842006": ("1-phenylpropan-2-amine (substance)", [])},
    "_1SigmaLowerValueOfPopulation": {
        "371919006": ("1 Sigma Lower Value of population", [221, 226, 7465])
    },
    "_1SigmaUpperValueOfPopulation": {
        "371917008": ("1 Sigma Upper Value of population", [221, 226, 7465])
    },
    "_1stDiagonalCoronaryArtery": {
        "91750005": (
            "1st Diagonal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stLeftPosterolateralCoronaryArtery": {
        "91757008": (
            "1st Left Posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stMarginalCoronaryArtery": {
        "91754001": (
            "1st Marginal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stRightPosterolateralCoronaryArtery": {
        "91761002": (
            "1st Right posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stSeptalCoronaryArtery": {
        "244251006": (
            "1st Septal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_201Thallium": {"60057003": ("^201^Thallium", [18])},
    "_203Lead": {"47588004": ("^203^Lead", [18])},
    "_222Radon": {"51800004": ("^222^Radon", [9528])},
    "_223Radium": {"24853006": ("^223^Radium", [18])},
    "_226Radium": {"28243009": ("^226^Radium", [9528])},
    "_22Sodium": {"71633006": ("^22^Sodium", [18, 4020])},
    "_24Sodium": {"58541008": ("^24^Sodium", [18])},
    "_252Californium": {"35978008": ("^252^Californium", [9528])},
    "_2DMode": {"399064001": ("2D mode", [6058, 12224])},
    "_2DModeUltrasound": {"399064001": ("2D mode ultrasound (qualifier value)", [])},
    "_2OClockPosition": {"129773009": ("2 o'clock position", [6018, 6019, 6205])},
    "_2OClockPositionOnMammogram": {
        "129773009": ("2 o'clock position on mammogram (finding)", [])
    },
    "_2PartialPerfusion": {"371864007": ("2: Partial Perfusion", [3713])},
    "_2SigmaLowerValueOfPopulation": {
        "371918003": ("2 Sigma Lower Value of population", [221, 226, 7465])
    },
    "_2SigmaUpperValueOfPopulation": {
        "371920000": ("2 Sigma Upper Value of population", [221, 226, 7465])
    },
    "_2ndDiagonalCoronaryArtery": {
        "91751009": (
            "2nd Diagonal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_2ndLeftPosterolateralCoronaryArtery": {
        "91758003": (
            "2nd Left Posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_2ndMarginalCoronaryArtery": {
        "91755000": (
            "2nd Marginal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_2ndRightPosterolateralCoronaryArtery": {
        "91762009": (
            "2nd Right posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_32Phosphorus": {"32505007": ("^32^Phosphorus", [18, 9528])},
    "_38Potassium": {"423764008": ("^38^Potassium", [4020])},
    "_3CompletePerfusion": {"371865008": ("3: Complete Perfusion", [3713])},
    "_3DMode": {"426865009": ("3D mode", [6058, 12224])},
    "_3DModeUltrasound": {"426865009": ("3D mode ultrasound (qualifier value)", [])},
    "_3OClockPosition": {"129774003": ("3 o'clock position", [6018, 6019, 6205])},
    "_3OClockPositionOnMammogram": {
        "129774003": ("3 o'clock position on mammogram (finding)", [])
    },
    "_3rdDiagonalCoronaryArtery": {
        "91752002": (
            "3rd diagonal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_3rdLeftPosterolateralCoronaryArtery": {
        "91759006": (
            "3rd Left Posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_3rdMarginalCoronaryArtery": {
        "91756004": (
            "3rd Marginal Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_3rdRightPosterolateralCoronaryArtery": {
        "91763004": (
            "3rd Right posterolateral Coronary Artery",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_42Potassium": {"59844004": ("^42^Potassium", [18])},
    "_43Potassium": {"8202008": ("^43^Potassium", [18])},
    "_45Titanium": {"75696008": ("^45^Titanium", [4020])},
    "_4OClockPosition": {"129775002": ("4 o'clock position", [6018, 6019, 6205])},
    "_4OClockPositionOnMammogram": {
        "129775002": ("4 o'clock position on mammogram (finding)", [])
    },
    "_51Chromium": {"52745005": ("^51^Chromium", [18])},
    "_52Iron": {"69089000": ("^52^Iron", [4020])},
    "_52Manganese": {"37225000": ("^52^Manganese", [4020])},
    "_57Cobalt": {"27054007": ("^57^Cobalt", [18])},
    "_58Cobalt": {"89272005": ("^58^Cobalt", [18])},
    "_59Iron": {"68580003": ("^59^Iron", [18])},
    "_5OClockPosition": {"129776001": ("5 o'clock position", [6018, 6019, 6205])},
    "_5OClockPositionOnMammogram": {
        "129776001": ("5 o'clock position on mammogram (finding)", [])
    },
    "_5thPercentileValueOfPopulation": {
        "371888009": ("5th Percentile Value of population", [221, 226, 7465])
    },
    "_60Cobalt": {"5405008": ("^60^Cobalt", [18, 9527, 9528])},
    "_60Copper": {"425364008": ("^60^Copper", [4020])},
    "_61Copper": {"71425003": ("^61^Copper", [4020])},
    "_62Copper": {"422934004": ("^62^Copper", [4020])},
    "_62Zinc": {"65054007": ("^62^Zinc", [4020])},
    "_64Copper": {"3932008": ("^64^Copper", [18, 4020])},
    "_66Gallium": {"79477007": ("^66^Gallium", [4020])},
    "_67Copper": {"53700003": ("^67^Copper", [18])},
    "_67Gallium": {"2008008": ("^67^Gallium", [18])},
    "_68Gallium": {"35337001": ("^68^Gallium", [4020])},
    "_68Germanium": {"53315004": ("^68^Germanium", [4020])},
    "_6OClockPosition": {"129777005": ("6 o'clock position", [6018, 6019, 6205])},
    "_6OClockPositionOnMammogram": {
        "129777005": ("6 o'clock position on mammogram (finding)", [])
    },
    "_72Arsenic": {"2705002": ("^72^Arsenic", [4020])},
    "_73Selenium": {"87437000": ("^73^Selenium", [4020])},
    "_75Bromine": {"17910003": ("^75^Bromine", [4020])},
    "_75Selenium": {"43239002": ("^75^Selenium", [18])},
    "_76Bromine": {"79523006": ("^76^Bromine", [4020])},
    "_77Bromine": {"86521004": ("^77^Bromine", [4020])},
    "_7OClockPosition": {"129778000": ("7 o'clock position", [6018, 6019, 6205])},
    "_7OClockPositionOnMammogram": {
        "129778000": ("7 o'clock position on mammogram (finding)", [])
    },
    "_81mKrypton": {"61716009": ("^81m^Krypton", [18])},
    "_82Rubidium": {"79197006": ("^82^Rubidium", [3107, 4020])},
    "_85Krypton": {"34127007": ("^85^Krypton", [18])},
    "_85Strontium": {"111084009": ("^85^Strontium", [18])},
    "_86Yttrium": {"10738001": ("^86^Yttrium", [4020])},
    "_87mStrontium": {"78023008": ("^87m^Strontium", [18])},
    "_89Strontium": {"7770004": ("^89^Strontium", [18])},
    "_89Zirconium": {"63360001": ("^89^Zirconium", [4020])},
    "_8OClockPosition": {"129779008": ("8 o'clock position", [6018, 6019, 6205])},
    "_8OClockPositionOnMammogram": {
        "129779008": ("8 o'clock position on mammogram (finding)", [])
    },
    "_90Strontium": {"14071002": ("^90^Strontium", [9528])},
    "_90Yttrium": {"14691008": ("^90^Yttrium", [18, 4020, 9528])},
    "_90thPercentileValueOfPopulation": {
        "371887004": ("90th Percentile Value of population", [221, 226, 7465])
    },
    "_94mTechnetium": {"424079002": ("^94m^Technetium", [4020])},
    "_95thPercentileValueOfPopulation": {
        "371889001": ("95th Percentile Value of population", [221, 226, 7465])
    },
    "_97Ruthenium": {"23788009": ("^97^Ruthenium", [18])},
    "_99mTechnetium": {"72454006": ("^99m^Technetium", [18])},
    "_9OClockPosition": {"129780006": ("9 o'clock position", [6018, 6019, 6205])},
    "_9OClockPositionOnMammogram": {
        "129780006": ("9 o'clock position on mammogram (finding)", [])
    },
    "ÍPigBreed": {"132327001": ("Í pig breed", [7480])},
}

concepts["NCIt"] = {
    "Abnormality": {"C9440": ("Abnormality", [6333, 6334])},
    "AdLibitum": {"C64636": ("ad libitum", [609])},
    "Adjudicator": {"C96561": ("Adjudicator", [7453])},
    "BronchioloalveolarAdenocarcinoma": {
        "C2923": ("Bronchioloalveolar adenocarcinoma", [638, 639])
    },
    "CardiotonicAgent": {"C78322": ("Cardiotonic agent", [10])},
    "Cryoablation": {"C15215": ("Cryoablation", [6320])},
    "DCFBCF18": {"C96234": ("DCFBC F^18^", [4021])},
    "Dosimetrist": {"C93176": ("Dosimetrist", [9555, 9562])},
    "EnhancingLesion": {
        "C113842": ("Enhancing Lesion", [7151, 7159, 7168, 7169, 7194])
    },
    "Enrollment": {"C37948": ("Enrollment", [280])},
    "Erect": {"C86043": ("erect", [19])},
    "ExtraluminalRoute": {"C38213": ("Extraluminal route", [11])},
    "Fibrate": {"C98150": ("Fibrate", [3761])},
    "FocusedUltrasoundAblation": {"C68681": ("Focused ultrasound ablation", [6320])},
    "Image": {"C48179": ("Image", [272])},
    "ImagingRegionOfInterest": {"C85402": ("Imaging Region of Interest", [272])},
    "IndexLesion": {"C110961": ("Index lesion", [6335, 6336, 6337])},
    "IntraepithelialRoute": {"C38244": ("Intraepithelial route", [11])},
    "Ion": {"C597": ("Ion", [9526])},
    "IrreversibleElectroporation": {
        "C131483": ("Irreversible electroporation", [6320])
    },
    "Middle": {"C25569": ("Middle", [211, 212, 3019, 6024, 6025, 12116, 12117, 12281])},
    "MouseMammaryFatPad": {"C22550": ("Mouse mammary fat pad", [644, 8134])},
    "NativeHawaiianOrOtherPacificIslander": {
        "C41219": ("Native Hawaiian or other Pacific Islander", [6099])
    },
    "NoEvidenceOfDisease": {"C40413": ("No Evidence of Disease", [6098])},
    "NonEnhancingLesion": {
        "C81175": ("Non-Enhancing Lesion", [7151, 7159, 7168, 7169, 7194])
    },
    "PSMA11Ga68": {"C118961": ("PSMA-11 Ga^68^", [4021])},
    "PiflufolastatF18": {"C116352": ("Piflufolastat F^18^", [4021])},
    "RANO": {"C114879": ("RANO", [6147])},
    "Reader": {"C28747": ("Reader", [7453])},
    "RecurrentDisease": {"C38155": ("Recurrent Disease", [6098])},
    "ReferenceRegion": {"C94970": ("Reference Region", [7151, 7165, 7196, 9000])},
    "Reviewer": {"C54634": ("Reviewer", [7453])},
    "SarcosineC11": {"C122684": ("Sarcosine C^11^", [4021])},
    "SoftTissueSarcomaExcludingRhabdomyosarcoma": {
        "C148457": ("Soft tissue sarcoma, excluding rhabdomyosarcoma", [638, 639])
    },
    "Technetium99mTrofolastat": {"C116887": ("Technetium^99m Trofolastat", [25])},
    "TransluminalRoute": {"C38306": ("Transluminal route", [11])},
    "UnitConversionFactor": {"C70774": ("Unit Conversion Factor", [10069])},
    "_166Holmium": {"C1943": ("^166^Holmium", [18])},
    "_4kscore": {"C142184": ("4Kscore", [6352])},
}

concepts["RXNORM"] = {
    "MangafodipirTrisodium": {"236987": ("Mangafodipir trisodium", [12, 3850])}
}

concepts["RADLEX"] = {
    "ACRCategory1ActionableFinding": {
        "RID49480": ("ACR Category 1 Actionable Finding", [7035])
    },
    "ACRCategory2ActionableFinding": {
        "RID49481": ("ACR Category 2 Actionable Finding", [7035])
    },
    "ACRCategory3ActionableFinding": {
        "RID49482": ("ACR Category 3 Actionable Finding", [7035])
    },
    "BariumSulfateSuspension": {"RID11588": ("Barium sulfate suspension", [6350])},
    "BeamHardeningArtifact": {"RID11327": ("Beam-hardening artifact", [6135])},
    "ChemicalShiftArtifact": {"RID11374": ("Chemical shift artifact", [6315, 6316])},
    "DelayedPhase": {"RID11081": ("Delayed phase", [6335, 6345, 6346])},
    "DiagnosticQuality": {"RID12": ("Diagnostic quality", [7036])},
    "DiffusionWeightedHypersensitivity": {
        "RID39536": ("Diffusion-weighted hypersensitivity", [6335, 6343, 6344])
    },
    "EarlyPhaseWashIn": {"RID49530": ("Early phase wash-in", [6335, 6345, 6346])},
    "EndorectalCoil": {"RID10809": ("Endorectal coil", [6349])},
    "EnhancementPattern": {"RID6058": ("Enhancement pattern", [6333, 6334])},
    "ErasedCharcoalSign": {"RID49497": ("Erased charcoal sign", [6335, 6341, 6342])},
    "FerromagneticImplantMagneticSusceptibilityEffect": {
        "RID11370": (
            "Ferromagnetic implant magnetic susceptibility effect",
            [6315, 6316],
        )
    },
    "Ghost": {"RID11426": ("Ghost", [6315, 6316])},
    "Heterogeneous": {"RID6060": ("Heterogeneous", [6335, 6343, 6344])},
    "Hyperintense": {"RID35805": ("Hyperintense", [6335, 6343, 6344])},
    "Hypointense": {"RID35804": ("Hypointense", [6335, 6343, 6344])},
    "IonicIodinatedContrastAgent": {
        "RID11585": ("Ionic iodinated contrast agent", [12, 3850])
    },
    "Isointense": {"RID6053": ("Isointense", [6335, 6343, 6344])},
    "Lenticular": {"RID49495": ("Lenticular", [6335, 6339, 6340])},
    "LimitedQuality": {"RID13": ("Limited quality", [7036])},
    "MarkedlyHypointense": {"RID49500": ("Markedly hypointense", [6335, 6343, 6344])},
    "MotionRelatedArtifact": {"RID11423": ("Motion related artifact", [6315, 6316])},
    "NMHeadPerfusionBrainPETCTAV45": {
        "RPID5427": ("NM head perfusion brain PET-CT AV-45", [100])
    },
    "NeurovascularBundleOfProstate": {
        "RID43492": ("Neurovascular bundle of prostate", [6348])
    },
    "NonCircumscribedMargin": {
        "RID34355": ("Non-circumscribed margin", [6335, 6341, 6342])
    },
    "NonDiagnosticQuality": {"RID14": ("Non-diagnostic quality", [7036])},
    "NonIonicIodinatedContrastAgent": {
        "RID38696": ("Non-ionic iodinated contrast agent", [12, 3850])
    },
    "NonUniformSignalIntensityArtifact": {
        "RID11408": ("Non-uniform signal intensity artifact", [6315, 6316])
    },
    "OrganizedChaos": {"RID49499": ("Organized chaos", [6335, 6343, 6344])},
    "PIRADS1DWIVeryLow": {"RID50314": ("PI-RADS 1 - DWI Very low", [6331])},
    "PIRADS1T2WIPZVeryLow": {"RID50302": ("PI-RADS 1 - T2WI PZ Very low", [6329])},
    "PIRADS1T2WITZVeryLow": {"RID50308": ("PI-RADS 1 - T2WI TZ Very low", [6330])},
    "PIRADS1VeryLow": {"RID50289": ("PI-RADS 1 - Very low", [6324, 6325])},
    "PIRADS1VeryLowLesion": {"RID50296": ("PI-RADS 1 - Very low (lesion)", [6328])},
    "PIRADS2DWILow": {"RID50315": ("PI-RADS 2 - DWI Low", [6331])},
    "PIRADS2Low": {"RID50290": ("PI-RADS 2 - Low", [6324, 6325])},
    "PIRADS2LowLesion": {"RID50297": ("PI-RADS 2 - Low (lesion)", [6328])},
    "PIRADS2T2WIPZLow": {"RID50303": ("PI-RADS 2 - T2WI PZ Low", [6329])},
    "PIRADS2T2WITZLow": {"RID50309": ("PI-RADS 2 - T2WI TZ Low", [6330])},
    "PIRADS3DWIIntermediate": {"RID50316": ("PI-RADS 3 - DWI Intermediate", [6331])},
    "PIRADS3Intermediate": {"RID50291": ("PI-RADS 3 - Intermediate", [6324, 6325])},
    "PIRADS3IntermediateLesion": {
        "RID50298": ("PI-RADS 3 - Intermediate (lesion)", [6328])
    },
    "PIRADS3T2WIPZIntermediate": {
        "RID50304": ("PI-RADS 3 - T2WI PZ Intermediate", [6329])
    },
    "PIRADS3T2WITZIntermediate": {
        "RID50310": ("PI-RADS 3 - T2WI TZ Intermediate", [6330])
    },
    "PIRADS4DWIHigh": {"RID50317": ("PI-RADS 4 - DWI High", [6331])},
    "PIRADS4High": {"RID50292": ("PI-RADS 4 - High", [6324, 6325])},
    "PIRADS4HighLesion": {"RID50299": ("PI-RADS 4 - High (lesion)", [6328])},
    "PIRADS4T2WIPZHigh": {"RID50305": ("PI-RADS 4 - T2WI PZ High", [6329])},
    "PIRADS4T2WITZHigh": {"RID50311": ("PI-RADS 4 - T2WI TZ High", [6330])},
    "PIRADS5DWIVeryHigh": {"RID50318": ("PI-RADS 5 - DWI Very high", [6331])},
    "PIRADS5T2WIPZVeryHigh": {"RID50306": ("PI-RADS 5 - T2WI PZ Very high", [6329])},
    "PIRADS5T2WITZVeryHigh": {"RID50312": ("PI-RADS 5 - T2WI TZ Very high", [6330])},
    "PIRADS5VeryHigh": {"RID50293": ("PI-RADS 5 - Very high", [6324, 6325])},
    "PIRADS5VeryHighLesion": {"RID50300": ("PI-RADS 5 - Very high (lesion)", [6328])},
    "PIRADSXDCEInadequateOrAbsent": {
        "RID50327": ("PI-RADS X - DCE Inadequate or absent", [6332])
    },
    "PIRADSXDWIInadequateOrAbsent": {
        "RID50326": ("PI-RADS X - DWI Inadequate or absent", [6331])
    },
    "PIRADSXInadequateOrAbsent": {
        "RID50322": ("PI-RADS X - Inadequate or absent", [6324, 6325])
    },
    "PIRADSXInadequateOrAbsentLesion": {
        "RID50323": ("PI-RADS X - Inadequate or absent (lesion)", [6328])
    },
    "PIRADSXT2WIPZInadequateOrAbsent": {
        "RID50324": ("PI-RADS X - T2WI PZ Inadequate or absent", [6329])
    },
    "PIRADSXT2WITZInadequateOrAbsent": {
        "RID50325": ("PI-RADS X - T2WI TZ Inadequate or absent", [6330])
    },
    "PIRADS_DCENegative": {"RID50320": ("PI-RADS DCE -ve", [6332])},
    "PIRADS_DCEPositive": {"RID50321": ("PI-RADS DCE +ve", [6332])},
    "ParallelImagingArtifact": {
        "RID11464": ("Parallel imaging artifact", [6315, 6316])
    },
    "PersistentDelayedPhase": {
        "RID34331": ("Persistent delayed phase", [6335, 6345, 6346])
    },
    "PhaseWraparoundArtifact": {
        "RID11395": ("Phase wraparound artifact", [6315, 6316])
    },
    "PlateauDelayedPhase": {"RID34332": ("Plateau delayed phase", [6335, 6345, 6346])},
    "RestrictedDiffusion": {"RID43349": ("Restricted diffusion", [6335, 6343, 6344])},
    "SignalCharacteristic": {"RID6049": ("Signal characteristic", [6333, 6334])},
    "Smearing": {"RID11424": ("Smearing", [6315, 6316])},
    "SuboptimalPatientPositioning": {
        "RID11281": ("Suboptimal patient positioning", [6314])
    },
    "T2Hyperintensity": {"RID39467": ("T2 Hyperintensity", [6335, 6343, 6344])},
    "T2Hypointensity": {"RID49501": ("T2 hypointensity", [6335, 6343, 6344])},
    "TissueMagneticSusceptibilityEffect": {
        "RID11372": ("Tissue magnetic susceptibility effect", [6315, 6316])
    },
    "WashoutDelayedPhase": {"RID34333": ("Washout delayed phase", [6335, 6345, 6346])},
}

concepts["MSH"] = {
    "LactatedRingerS": {"D000077325": ("Lactated Ringer's", [70, 3850])},
    "MonteCarloMethod": {"D009010": ("Monte Carlo Method", [10068])},
    "Technetium99mTRODAT1": {"C104196": ("Technetium^99m^ TRODAT-1", [25])},
}

concepts["UCUM"] = {
    "ArbitraryUnit": {"[arb'U]": ("arbitrary unit", [7181])},
    "Becquerel": {"Bq": ("becquerel", [3083])},
    "BecquerelsPerMilliliter": {"Bq/ml": ("Becquerels/milliliter", [83, 84, 7181])},
    "Centimeter": {"cm": ("cm", [7063, 7181, 7183, 7460])},
    "CentimeterPerSecond": {"cm/s": ("centimeter/second", [7181])},
    "Counts": {"{counts}": ("Counts", [83, 84, 7181])},
    "CountsPerSecond": {"{counts}/s": ("Counts per second", [83, 84, 7181])},
    "CubicCentimeter": {"cm3": ("cubic centimeter", [7181, 7462])},
    "CubicDecimeter": {"dm3": ("cubic decimeter", [7181, 7462])},
    "CubicMicrometer": {"um3": ("cubic micrometer", [7181, 7462])},
    "CubicMillimeter": {"mm3": ("cubic millimeter", [7181, 7462])},
    "Day": {"d": ("day", [6046, 7456])},
    "Decibel": {"dB": ("decibel", [7181])},
    "DecibelsMicrovolt": {"dB[uV]": ("dB(µV)", [3045])},
    "DecibelsMillivolt": {"dB[mV]": ("dB(mV)", [3045])},
    "Degree": {"deg": ("Degree", [7183])},
    "DegreesCelsius": {"Cel": ("degrees Celsius", [7181])},
    "Diopters": {"diop": ("diopters", [4267])},
    "DynScm5": {"dyn.s.cm-5": ("dyn.s.cm-5", [3502, 7181])},
    "DynScm5M2": {"dyn.s.cm-5/m2": ("dyn.s.cm-5/m2", [3503, 7181])},
    "ElectronDensity": {"10*23/ml": ("Electron Density", [301])},
    "French": {"[Ch]": ("french", [3510])},
    "GrayPerSecond": {"Gy/s": ("Gy/s", [9558, 9560])},
    "Gy": {"Gy": ("Gy", [7181, 10071])},
    "Hertz": {"Hz": ("Hertz", [7181])},
    "HounsfieldUnit": {"[hnsf'U]": ("Hounsfield unit", [83, 301, 7181])},
    "Hour": {"h": ("hour", [7456])},
    "Hz": {"Hz": ("Hz", [3044])},
    "KilometersPerHour": {"km/h": ("km/h", [3212])},
    "Kilopascal": {"kPa": ("kPa", [3500, 7181])},
    "Kilovolt": {"kV": ("Kilovolt", [9521])},
    "MegaElectronVolt": {"MeV": ("Megaelectronvolt", [9521])},
    "Megabecquerel": {"MBq": ("megabecquerel", [3083])},
    "Megavolt": {"MV": ("Megavolt", [9521])},
    "Meter": {"m": ("m", [7063])},
    "Micrometer": {"um": ("micrometer", [4260, 4267, 7063, 7181, 7183, 7460])},
    "MicromolePerMilliliter": {"umol/ml": ("Micromole/milliliter", [83, 84, 7181])},
    "MicromolePerMinutePerMilliliter": {
        "umol/min/ml": ("Micromole/minute/milliliter", [83, 84, 7181])
    },
    "Microvolt": {"uV": ("µV", [3045])},
    "MilesPerHour": {"[mi_i]/h": ("mph", [3212])},
    "Millicurie": {"mCi": ("millicurie", [3083])},
    "MilligramsPerCubicCentimeter": {"mg/cm3": ("mg/cm^3", [301])},
    "MilligramsPerMilliliter": {"mg/ml": ("mg/ml", [301, 4244])},
    "MilligramsPerMinutePerMilliliter": {
        "mg/min/ml": ("Milligrams/minute/milliliter", [83, 84, 7181])
    },
    "MilliliterPer100GramPerMinute": {
        "ml/[100]g/min": ("milliliter per 100 gram per minute", [7181])
    },
    "MilliliterPer100Milliliter": {
        "ml/[100]ml": ("milliliter per 100 milliliter", [7181])
    },
    "MilliliterPerGram": {"ml/g": ("Milliliter/gram", [83, 84, 7181])},
    "MilliliterPerMinute": {"ml/min": ("milliliter per minute", [7181])},
    "MilliliterPerMinutePerGram": {
        "ml/min/g": ("Milliliter/minute/gram", [83, 84, 7181])
    },
    "MilliliterPerSecond": {"ml/s": ("milliliter per second", [7181])},
    "Millimeter": {"mm": ("mm", [3510, 4267, 7063, 7181, 7183, 7460])},
    "MillimeterPerSecond": {"mm/s": ("millimeter/second", [7181])},
    "MillimetersHg": {"mm[Hg]": ("mmHg", [3500, 7181])},
    "MillimolesPerKilogramWetWeight": {
        "mmol/kg{WetWeight}": ("millimoles per kg wet weight", [7181])
    },
    "Millisecond": {"ms": ("millisecond", [7181, 7183])},
    "Millitesla": {"mT": ("milliTesla", [7181])},
    "Millivolt": {"mV": ("mV", [3045])},
    "Minute": {"min": ("minute", [7456])},
    "MonitorUnits": {"{MU}": ("Monitor Units", [9269, 9552, 9557, 9559])},
    "MonitorUnitsPerSecond": {"{MU}/s": ("Monitor Units/Second", [9550, 9558])},
    "Month": {"mo": ("month", [6046, 7456])},
    "NoUnits": {"1": ("no units", [7181])},
    "NumberParticlesPer100GramOfTissue": {
        "{Particles}/[100]g{Tissue}": (
            "number particles per 100 gram of tissue",
            [7181],
        )
    },
    "PRU": {"[PRU]": ("P.R.U.", [3502, 7181])},
    "PRUPerSquareMeter": {"[PRU]/m2": ("P.R.U./m2", [3503, 7181])},
    "PerCentimeter": {"/cm": ("/Centimeter", [83, 84, 7181])},
    "PerMinute": {"/min": ("/min", [7181])},
    "PerSecond": {"/s": ("/s", [7181])},
    "Percent": {"%": ("Percent", [83, 84, 301, 4244, 7181])},
    "Ppm": {"ppm": ("ppm", [7181])},
    "ProportionalToCounts": {
        "{propcounts}": ("Proportional to counts", [83, 84, 7181])
    },
    "ProportionalToCountsPerSecond": {
        "{propcounts}/s": ("Proportional to counts per second", [83, 84, 7181])
    },
    "Radian": {"rad": ("Radian", [7183])},
    "RadiansPerSecond": {"rad/s": ("rad/s", [3044])},
    "Ratio": {"{ratio}": ("ratio", [7181])},
    "Second": {"s": ("second", [7181, 7183, 9269, 9557])},
    "SecondPerSquareMillimeter": {"s/mm2": ("second per square millimeter", [7181])},
    "SquareCentimeter": {"cm2": ("Centimeter**2", [83, 84, 7181, 7461])},
    "SquareCentimeterPerMilliliter": {
        "cm2/ml": ("Centimeter**2/milliliter", [83, 84, 7181])
    },
    "SquareMicrometer": {"um2": ("square micrometer", [7181, 7461])},
    "SquareMicrometerPerMillisecond": {"um2/ms": ("um2/ms", [7181, 7277])},
    "SquareMicrometerPerSecond": {"um2/s": ("um2/s", [7181, 7277])},
    "SquareMillimeter": {"mm2": ("square millimeter", [7181, 7461])},
    "SquareMillimeterPerSecond": {"mm2/s": ("mm2/s", [7181, 7277])},
    "StandardizedUptakeValueBodySurfaceArea": {
        "cm2/ml{SUVbsa}": (
            "Standardized Uptake Value body surface area",
            [83, 84, 85, 7181],
        )
    },
    "StandardizedUptakeValueBodyWeight": {
        "g/ml{SUVbw}": ("Standardized Uptake Value body weight", [83, 84, 85, 7181])
    },
    "StandardizedUptakeValueIdealBodyWeight": {
        "g/ml{SUVibw}": (
            "Standardized Uptake Value ideal body weight",
            [83, 84, 85, 7181],
        )
    },
    "StandardizedUptakeValueLeanBodyMassJames": {
        "g/ml{SUVlbm}": (
            "Standardized Uptake Value lean body mass (James)",
            [83, 84, 85, 7181],
        )
    },
    "StandardizedUptakeValueLeanBodyMassJames128Multiplier": {
        "g/ml{SUVlbm(James128)}": (
            "Standardized Uptake Value lean body mass (James 128 multiplier)",
            [83, 84, 85, 7181],
        )
    },
    "StandardizedUptakeValueLeanBodyMassJanma": {
        "g/ml{SUVlbm(Janma)}": (
            "Standardized Uptake Value lean body mass (Janma)",
            [83, 84, 85, 7181],
        )
    },
    "Sv": {"Sv": ("Sv", [7181, 10071])},
    "Week": {"wk": ("week", [6046, 7456])},
    "WoodU": {"[wood'U]": ("Wood U", [3502, 7181])},
    "WoodUM2": {"[wood'U]/m2": ("Wood U/m2", [3503, 7181])},
    "Year": {"a": ("year", [6046, 7456])},
    "_10EMinus6SquareMillimetersPerSecond": {
        "10-6.mm2/s": ("10-6.mm2/s", [7181, 7277])
    },
}

concepts["LN"] = {
    "ACASUM2000": {"33072-0": ("AC, ASUM 2000", [12012, 12013])},
    "ACByGAASUM2000": {"33145-4": ("AC by GA, ASUM 2000", [12012, 12015])},
    "ACByGAHadlock1984": {"33146-2": ("AC by GA, Hadlock 1984", [12012, 12015])},
    "ACByGAMerz1988": {"33148-8": ("AC by GA, Merz 1988", [12012, 12015])},
    "ACByGAShinozuka1996": {"33149-6": ("AC by GA, Shinozuka 1996", [12012, 12015])},
    "ACCampbell1975": {"11889-3": ("AC, Campbell 1975", [12012, 12013])},
    "ACDerivedByGAChitty1994": {
        "33546-3": ("AC (derived) by GA, Chitty 1994", [12012, 12015])
    },
    "ACHadlock1984": {"11892-7": ("AC, Hadlock 1984", [12012, 12013])},
    "ACHansmann1985": {"33073-8": ("AC, Hansmann1985", [12012, 12013])},
    "ACInterval": {
        "59103-2": (
            "A-C Interval",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "ACJeanty1982": {"33537-2": ("AC, Jeanty 1982", [12012, 12013])},
    "ACJeanty1984": {"11893-5": ("AC, Jeanty 1984", [12012, 12013])},
    "ACLessoway1998": {"33074-6": ("AC, Lessoway 1998", [12012, 12013])},
    "ACMeasuredByGAChitty1994": {
        "33147-0": ("AC (measured) by GA, Chitty 1994", [12012, 12015])
    },
    "ACMertz1988": {"33075-3": ("AC, Mertz 1988", [12012, 12013])},
    "ACShinozuka1996": {"33076-1": ("AC, Shinozuka 1996", [12012, 12013])},
    "APADTAD": {"33191-8": ("APAD * TAD", [12005])},
    "APAbdominalDiameterLessoway1998": {
        "33077-9": ("A-P Abdominal Diameter, Lessoway 1998", [12012, 12013])
    },
    "AWaveAmplitude": {"59100-8": ("A-Wave Amplitude", [12262, 12263, 12271, 12272])},
    "AWaveDuration": {
        "59105-7": (
            "A-Wave Duration",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "AWavePeakVelocity": {
        "59081-0": (
            "A-Wave Peak Velocity",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "AbdominalCircumference": {"11979-2": ("Abdominal Circumference", [12005, 12279])},
    "AccelerationIndex": {"20167-3": ("Acceleration Index", [12119, 12121])},
    "AccelerationSlope": {
        "20167-3": (
            "Acceleration Slope",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "AccelerationTime": {
        "20168-1": (
            "Acceleration Time",
            [
                12119,
                12122,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "Addendum": {"55107-7": ("Addendum", [6052, 7001])},
    "AliasVelocity": {
        "59130-5": (
            "Alias velocity",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "AmnioticFluidIndex": {"11627-7": ("Amniotic Fluid Index", [12008])},
    "AnteriorHornLateralVentricularWidth": {
        "33197-5": ("Anterior Horn Lateral ventricular width", [12007])
    },
    "AnteriorPosteriorAbdominalDiameter": {
        "11818-2": ("Anterior-Posterior Abdominal Diameter", [12005, 12279])
    },
    "AnteriorPosteriorTrunkDiameter": {
        "11819-0": ("Anterior-Posterior Trunk Diameter", [12005, 12279])
    },
    "AorticAnnulusDiameter": {"79940-3": ("Aortic annulus diameter", [12300])},
    "AorticArchDiameter": {"18011-7": ("Aortic Arch Diameter", [12212])},
    "AorticIsthmusDiameter": {"18014-1": ("Aortic Isthmus Diameter", [12212])},
    "AorticRegurgitantFlow": {"79941-1": ("Aortic regurgitant flow", [12300])},
    "AorticRegurgitantFraction": {"79942-9": ("Aortic regurgitant fraction", [12300])},
    "AorticRegurgitantJetAreaLVOTAreaPercent": {
        "79943-7": ("Aortic regurgitant jet area/LVOT area %", [12300])
    },
    "AorticRegurgitantJetWidthLVOTWidthPercent": {
        "79944-5": ("Aortic regurgitant jet width/LVOT width %", [12300])
    },
    "AorticRegurgitationPISARadius": {
        "79945-2": ("Aortic regurgitation PISA radius", [12300])
    },
    "AorticRegurgitationPISAVelocity": {
        "79946-0": ("Aortic regurgitation PISA velocity", [12300])
    },
    "AorticRegurgitationPressureHalfTime": {
        "79947-8": ("Aortic regurgitation pressure half-time", [12300])
    },
    "AorticRegurgitationVTI": {"79952-8": ("Aortic regurgitation VTI", [12300])},
    "AorticRegurgitationVenaContractaWidth": {
        "79948-6": ("Aortic regurgitation vena contracta width", [12300])
    },
    "AorticRegurgitationVmax": {"79949-4": ("Aortic regurgitation Vmax", [12300])},
    "AorticRegurgitationVolumeContinuityVTI": {
        "79950-2": ("Aortic regurgitation volume (Continuity VTI)", [12300])
    },
    "AorticRegurgitationVolumePISA": {
        "79951-0": ("Aortic regurgitation volume (PISA)", [12300])
    },
    "AorticRootAnnuloAorticJunctionDiameter": {
        "18015-8": ("Aortic Root Diameter", [12212])
    },
    "AorticRootDiameter": {"79953-6": ("Aortic root diameter", [12300])},
    "AorticRootDiameterBSA": {"79954-4": ("Aortic root diameter / BSA", [12300])},
    "AorticRootOxygenContent": {"10232-7": ("Aortic Root Oxygen content", [3525])},
    "AorticSinotubularJunctionDimension": {
        "79955-1": ("Aortic sinotubular junction dimension", [12300])
    },
    "AorticSinotubularJunctionToAorticRootRatio": {
        "59116-4": ("Aortic Sinotubular Junction to Aortic Root Ratio", [12273, 12274])
    },
    "AorticValveAreaContinuityVTI": {
        "79958-5": ("Aortic valve area (Continuity VTI)", [12300])
    },
    "AorticValveAreaContinuityVTIBSA": {
        "79959-3": ("Aortic valve area (Continuity VTI) / BSA", [12300])
    },
    "AorticValveAreaContinuityVmax": {
        "79956-9": ("Aortic valve area (Continuity Vmax)", [12300])
    },
    "AorticValveAreaContinuityVmaxBSA": {
        "79957-7": ("Aortic valve area (Continuity Vmax) / BSA", [12300])
    },
    "AorticValveCuspSeparation": {
        "17996-0": ("Aortic Valve Cusp Separation", [12211, 12270, 12271, 12272, 12274])
    },
    "AorticValveEffectiveRegurgitantOrificeArea": {
        "79960-1": ("Aortic valve effective regurgitant orifice area", [12300])
    },
    "AorticValveEjectionTime": {"18041-4": ("Aortic Valve Ejection Time", [12211])},
    "AorticValveMeanBloodVelocity": {
        "79961-9": ("Aortic valve mean blood velocity", [12300])
    },
    "AorticValveMeanGradient": {"79962-7": ("Aortic valve mean gradient", [12300])},
    "AorticValvePeakInstantaneousGradient": {
        "79963-5": ("Aortic valve peak instantaneous gradient", [12300])
    },
    "AorticValveVTI": {"79965-0": ("Aortic valve VTI", [12300])},
    "AorticValveVmax": {"79964-3": ("Aortic valve Vmax", [12300])},
    "ArterialBloodBaseExcess": {"1925-7": ("Arterial Blood Base Excess", [3527])},
    "ArterialBloodCarbonDioxideContent": {
        "2026-3": ("Arterial Blood Carbon dioxide content", [3525])
    },
    "ArterialBloodCarbonDioxidePartialPressure": {
        "2019-8": ("Arterial Blood Carbon dioxide partial pressure", [3524])
    },
    "ArterialBloodPh": {"2744-1": ("Arterial Blood pH", [3528])},
    "ArterialContentFca": {"19218-7": ("Arterial Content (FCa)", [3529])},
    "ArterialOxygenContent": {"19218-7": ("Arterial Oxygen content", [3525])},
    "ArterialOxygenPartialPressure": {
        "2703-7": ("Arterial Oxygen partial pressure", [3524])
    },
    "ArterialOxygenPartialPressureAt50PercentSaturation": {
        "19214-6": ("Arterial Oxygen partial pressure at 50% saturation", [3524])
    },
    "ArterialOxygenSaturation": {"2708-6": ("Arterial Oxygen saturation", [3526])},
    "AscendingAortaDimension": {"79966-8": ("Ascending Aorta Dimension", [12300])},
    "AscendingAorticDiameter": {"18012-5": ("Ascending Aortic Diameter", [12212])},
    "AverageUltrasoundAge": {"11884-4": ("Average Ultrasound Age", [12012, 12013])},
    "AxtByGAShinozuka1996": {"33150-4": ("AxT by GA, Shinozuka 1996", [12012, 12015])},
    "AxtShinozuka1996": {"33078-7": ("AxT, Shinozuka 1996", [12012, 12013])},
    "BCSlope": {"59126-3": ("B-C Slope", [12262, 12263, 12271, 12272])},
    "BDJeanty1982": {"33545-5": ("BD, Jeanty 1982", [12012, 12013])},
    "BPDASUM1989": {"33079-5": ("BPD, ASUM 1989", [12012, 12013])},
    "BPDAreaCorrected": {"11824-0": ("BPD area corrected", [12005, 12279])},
    "BPDByGAASUM2000": {"33151-2": ("BPD by GA, ASUM 2000", [12012, 12015])},
    "BPDByGAHadlock1984": {"33198-3": ("BPD by GA, Hadlock 1984", [12012, 12015])},
    "BPDByGAJeanty1982": {"33153-8": ("BPD by GA, Jeanty 1982", [12012, 12015])},
    "BPDByGAMerz1988": {"33154-6": ("BPD by GA, Merz 1988", [12012, 12015])},
    "BPDByGARempen1991": {"33155-3": ("BPD by GA, Rempen 1991", [12012, 12015])},
    "BPDByGAShinozuka1996": {"33156-1": ("BPD by GA, Shinozuka 1996", [12012, 12015])},
    "BPDDoubilet1993": {"11900-8": ("BPD, Doubilet 1993", [12012, 12013])},
    "BPDHadlock1984": {"11902-4": ("BPD, Hadlock 1984", [12012, 12013])},
    "BPDHansmann1985": {"11903-2": ("BPD, Hansmann 1985", [12012, 12013])},
    "BPDHansmann1986": {"33538-0": ("BPD, Hansmann 1986", [12012, 12013])},
    "BPDJeanty1982": {"33539-8": ("BPD, Jeanty 1982", [12012, 12013])},
    "BPDJeanty1984": {"11905-7": ("BPD, Jeanty 1984", [12012, 12013])},
    "BPDKurtz1980": {"11906-5": ("BPD, Kurtz 1980", [12012, 12013])},
    "BPDLessoway1998": {"33080-3": ("BPD, Lessoway 1998", [12012, 12013])},
    "BPDMertz1988": {"33081-1": ("BPD, Mertz 1988", [12012, 12013])},
    "BPDOiChitty1997": {"33086-0": ("BPD-oi, Chitty 1997", [12012, 12013])},
    "BPDOoChitty1997": {"33087-8": ("BPD-oo, Chitty 1997", [12012, 12013])},
    "BPDOsaka1989": {"33082-9": ("BPD, Osaka 1989", [12012, 12013])},
    "BPDOuterInnerByGAChitty1994": {
        "33556-2": ("BPD outer-inner by GA, Chitty 1994", [12012, 12015])
    },
    "BPDOuterOuterByGAChitty1994": {
        "33152-0": ("BPD outer-outer by GA, Chitty 1994", [12012, 12015])
    },
    "BPDRempen1991": {"33083-7": ("BPD, Rempen 1991", [12012, 12013])},
    "BPDSabbagha1978": {"11907-3": ("BPD, Sabbagha 1978", [12012, 12013])},
    "BPDShinozuka1996": {"33084-5": ("BPD, Shinozuka 1996", [12012, 12013])},
    "BPDTokyo1986": {"33085-2": ("BPD, Tokyo 1986", [12012, 12013])},
    "BSA": {"8277-6": ("BSA", [3455])},
    "BiparietalDiameter": {"11820-8": ("Biparietal Diameter", [12005, 12279])},
    "BloodBaseExcess": {"11555-0": ("Blood Base Excess", [3527])},
    "BloodCarbonDioxideContent": {"20565-8": ("Blood Carbon dioxide content", [3525])},
    "BloodCarbonDioxidePartialPressure": {
        "11557-6": ("Blood Carbon dioxide partial pressure", [3524])
    },
    "BloodOxygenPartialPressure": {
        "11556-8": ("Blood Oxygen partial pressure", [3524])
    },
    "BloodOxygenSaturation": {"20564-1": ("Blood Oxygen saturation", [3526])},
    "BloodPh": {"11558-4": ("Blood pH", [3528])},
    "BpdaHadlock1982": {"11901-6": ("BPDa, Hadlock 1982", [12012, 12013])},
    "BrainMRI": {"24590-2": ("Brain MRI", [646])},
    "BrainPET": {"44138-6": ("Brain PET", [646])},
    "BrainSPECT": {"39632-5": ("Brain SPECT", [646])},
    "BreastBilateralMammogram": {
        "36626-0": ("breast - bilateral mammogram", [6050, 6083])
    },
    "BreastBilateralMr": {"30795-9": ("breast - bilateral mr", [6050, 6083])},
    "BreastBilateralMrWContrastIv": {
        "36150-1": ("breast - bilateral mr w contrast iv", [6050, 6083])
    },
    "BreastBilateralMrWoAndWContrastIv": {
        "36277-2": ("breast - bilateral mr wo and w contrast iv", [6050, 6083])
    },
    "BreastFfdMammogram": {"46342-2": ("breast ffd mammogram", [6050, 6083])},
    "BreastLeftMammogram": {"36627-8": ("breast - left mammogram", [6050, 6083])},
    "BreastLeftMr": {"35954-7": ("breast - left mr", [6050, 6083])},
    "BreastLeftMrWContrastIv": {
        "36151-9": ("breast - left mr w contrast iv", [6050, 6083])
    },
    "BreastLeftMrWoAndWContrastIv": {
        "36278-0": ("breast - left mr wo and w contrast iv", [6050, 6083])
    },
    "BreastMrWContrastIv": {"36149-3": ("breast mr w contrast iv", [6050, 6083])},
    "BreastMrWoAndWContrastIv": {
        "36276-4": ("breast mr wo and w contrast iv", [6050, 6083])
    },
    "BreastRightMammogram": {"37774-7": ("breast - right mammogram", [6050, 6083])},
    "BreastRightMr": {"35955-4": ("breast - right mr", [6050, 6083])},
    "BreastRightMrWContrastIv": {
        "36152-7": ("breast - right mr w contrast iv", [6050, 6083])
    },
    "BreastRightMrWoAndWContrastIv": {
        "36279-8": ("breast - right mr wo and w contrast iv", [6050, 6083])
    },
    "BreastUnilateralMammogram": {
        "46339-8": ("breast - unilateral mammogram", [6050, 6083])
    },
    "BreastUnilateralMr": {"46299-4": ("breast - unilateral mr", [6050, 6083])},
    "BreastUnilateralMrWContrastIv": {
        "46323-2": ("breast - unilateral mr w contrast iv", [6050, 6083])
    },
    "BreastUnilateralMrWoAndWContrastIv": {
        "43528-9": ("breast - unilateral mr wo and w contrast iv", [6050, 6083])
    },
    "BreastUnilateralMrWoContrast": {
        "46333-1": ("breast - unilateral mr wo contrast", [6050, 6083])
    },
    "CEDistance": {
        "59122-2": (
            "C-E Distance",
            [12251, 12262, 12263, 12266, 12267, 12268, 12271, 12272],
        )
    },
    "CKDEPIFormulaEstimationOfGFR": {
        "62238-1": ("CKD-EPI Formula estimation of GFR", [10046])
    },
    "CRLASUM1991": {"33089-4": ("CRL, ASUM 1991", [12012, 12013])},
    "CRLASUM2000": {"33090-2": ("CRL, ASUM 2000", [12012, 12013])},
    "CRLByGAASUM2000": {"33159-5": ("CRL by GA ASUM 2000", [12012, 12015])},
    "CRLByGARempen1991": {"33160-3": ("CRL by GA, Rempen1991", [12012, 12015])},
    "CRLByGAShinozuka1996": {"33161-1": ("CRL by GA, Shinozuka 1996", [12012, 12015])},
    "CRLDaya1993": {"33091-0": ("CRL, Daya 1993", [12012, 12013])},
    "CRLHadlock1992": {"11910-7": ("CRL, Hadlock 1992", [12012, 12013])},
    "CRLHansmann1985": {"11911-5": ("CRL, Hansmann 1985", [12012, 12013])},
    "CRLHansmann1986": {"33540-6": ("CRL, Hansmann 1986", [12012, 12013])},
    "CRLJeanty1982": {"33092-8": ("CRL, Jeanty 1982", [12012, 12013])},
    "CRLJeanty1984": {"11917-2": ("CRL, Jeanty 1984", [12012, 12013])},
    "CRLNelson1981": {"11913-1": ("CRL, Nelson 1981", [12012, 12013])},
    "CRLOsaka1989": {"33093-6": ("CRL, Osaka 1989", [12012, 12013])},
    "CRLRempen1991": {"33094-4": ("CRL, Rempen 1991", [12012, 12013])},
    "CRLRobinson1975": {"11914-9": ("CRL, Robinson 1975", [12012, 12013])},
    "CRLShinozuka1996": {"33095-1": ("CRL, Shinozuka 1996", [12012, 12013])},
    "CRLTokyo1986": {"33096-9": ("CRL, Tokyo 1986", [12012, 12013])},
    "CT3DCAMModel": {"85040-4": ("CT 3D CAM model", [7061])},
    "CTAbdomenReport": {"41806-1": ("CT Abdomen Report", [7000])},
    "CTChestReport": {"24627-2": ("CT Chest Report", [7000])},
    "CTHeadReport": {"24725-4": ("CT Head Report", [7000])},
    "CTPerfusionHeadWithContrastIV": {
        "39142-5": ("CT perfusion head with contrast IV", [100])
    },
    "CTReport": {"25045-6": ("CT Report", [7000])},
    "CTUnspecifiedBodyRegion": {"25045-6": ("CT unspecified body region", [100])},
    "CapillaryBloodOxygenSaturation": {
        "2709-4": ("Capillary Blood Oxygen Saturation", [3526])
    },
    "CapillaryBloodOxygenSaturationByOximetry": {
        "2710-2": ("Capillary Blood Oxygen Saturation, by Oximetry", [3526])
    },
    "CardiacCatheterizationReport": {
        "18745-0": ("Cardiac Catheterization Report", [7000])
    },
    "CardiacCircumferenceTransverseByUS": {
        "59073-7": ("Cardiac Circumference, transverse by US", [12279])
    },
    "CardiacCrossSectionalAreaTransverseByUS": {
        "59075-2": ("Cardiac Cross-sectional Area, transverse by US", [12279])
    },
    "CardiacElectrophysiologyReport": {
        "18750-0": ("Cardiac Electrophysiology Report", [7000])
    },
    "CardiacRhythm": {"8884-9": ("Cardiac Rhythm", [3441])},
    "CardiothoracicAreaRatio": {"59076-0": ("Cardiothoracic Area Ratio", [12279])},
    "CardiothoracicCircumferenceRatio": {
        "59074-5": ("Cardiothoracic Circumference Ratio", [12279])
    },
    "CephalicIndex": {"11823-2": ("Cephalic Index", [12004, 12279])},
    "CephalicIndexByGAChitty1994": {
        "33157-9": ("Cephalic Index by GA, Chitty 1994", [12012, 12015])
    },
    "CephalicIndexByGAHadlock1981": {
        "33158-7": ("Cephalic Index by GA, Hadlock 1981", [12012, 12015])
    },
    "CervixLength": {"11961-0": ("Cervix Length", [12011])},
    "CisternaMagnaLength": {
        "11860-4": ("Cisterna Magna Length", [12005, 12007, 12279])
    },
    "ClavicleLength": {"11962-8": ("Clavicle length", [12006])},
    "ClavicleLengthYarkoni1985": {
        "33088-6": ("Clavicle length, Yarkoni 1985", [12012, 12013])
    },
    "ClinicalInformation": {"55752-0": ("Clinical Information", [7001])},
    "ClosureToOpeningTime": {
        "59082-8": (
            "Closure to Opening Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "CockroftGaultFormulaEstimationOfGFR": {
        "35591-7": ("Cockroft-Gault Formula estimation of GFR", [10046])
    },
    "CommunicationOfCriticalResults": {
        "73568-8": ("Communication of Critical Results", [7001])
    },
    "Complications": {"55109-3": ("Complications", [7001])},
    "CompositeUltrasoundAge": {"11888-5": ("Composite Ultrasound Age", [12019])},
    "ConceptionDate": {"33067-0": ("Conception Date", [12003])},
    "Conclusions": {"55110-1": ("Conclusions", [6052, 7001])},
    "CrownRumpLength": {"11957-8": ("Crown Rump Length", [12009])},
    "CurrentProcedureDescriptions": {
        "55111-9": ("Current Procedure Descriptions", [7001])
    },
    "DEExcursion": {
        "59091-9": (
            "D-E Excursion",
            [12251, 12262, 12263, 12266, 12267, 12268, 12271, 12272],
        )
    },
    "DESlope": {
        "59127-1": (
            "D-E Slope",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "DEXASkeletalSystemStudyReport": {
        "38269-7": ("DEXA Skeletal System Study Report", [7000])
    },
    "DecelerationSlope": {
        "20216-8": (
            "Deceleration Slope",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "DecelerationTime": {
        "20217-6": (
            "Deceleration Time",
            [
                12119,
                12122,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "DescendingAorticDiameter": {"18013-3": ("Descending Aortic Diameter", [12212])},
    "DiagnosticImagingReport": {"18748-4": ("Diagnostic Imaging Report", [7000])},
    "DiagnosticInterventionalRadiologyReport": {
        "47048-4": ("Diagnostic Interventional Radiology Report", [7000])
    },
    "DpDtByUS": {
        "59120-6": (
            "dP/dt by US",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "ECGReport": {"11524-0": ("ECG Report", [7000])},
    "EDD": {"11778-8": ("EDD", [12003])},
    "EDDFromAverageUltrasoundAge": {
        "11781-2": ("EDD from average ultrasound age", [12003])
    },
    "EDDFromLMP": {"11779-6": ("EDD from LMP", [12003])},
    "EDDFromOvulationDate": {"11780-4": ("EDD from ovulation date", [12003])},
    "EFSlope": {
        "59128-9": (
            "E-F Slope",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "EFW1ByShinozuka1996": {"33141-3": ("EFW1 by Shinozuka 1996", [12012, 12014])},
    "EFW2ByShinozuka1996": {"33142-1": ("EFW2 by Shinozuka 1996", [12012, 12014])},
    "EFW3ByShinozuka1996": {"33143-9": ("EFW3 by Shinozuka 1996", [12012, 12014])},
    "EFWByACAndBPDShepard1982": {
        "11739-0": ("EFW by AC and BPD, Shepard 1982", [12012, 12014])
    },
    "EFWByACBPDFLHCHadlock1985": {
        "11732-5": ("EFW by AC, BPD, FL, HC, Hadlock 1985", [12012, 12014])
    },
    "EFWByACBPDFLHadlock1984": {
        "11734-1": ("EFW by AC, BPD, FL, Hadlock 1984", [12012, 12014])
    },
    "EFWByACBPDFLHadlock1985": {
        "11735-8": ("EFW by AC, BPD, FL, Hadlock 1985", [12012, 12014])
    },
    "EFWByACBPDHadlock1984": {
        "11738-2": ("EFW by AC, BPD, Hadlock 1984", [12012, 12014])
    },
    "EFWByACCampbell1975": {"11756-4": ("EFW by AC, Campbell 1975", [12012, 12014])},
    "EFWByACFLHCHadlock1985": {
        "11746-5": ("EFW by AC, FL, HC, Hadlock 1985", [12012, 12014])
    },
    "EFWByACFLHadlock1984": {
        "11750-7": ("EFW by AC, FL, Hadlock 1984", [12012, 12014])
    },
    "EFWByACFLHadlock1985": {
        "11751-5": ("EFW by AC, FL, Hadlock 1985", [12012, 12014])
    },
    "EFWByACHCHadlock1984": {"11754-9": ("EFW by AC, HC Hadlock 1984", [12012, 12014])},
    "EFWByBPDAPADTADFLTokyo1987": {
        "33144-7": ("EFW by BPD, APAD, TAD, FL, Tokyo 1987", [12012, 12014])
    },
    "EFWByBPDFTAFLOsaka1990": {
        "33140-5": ("EFW by BPD, FTA, FL, Osaka 1990", [12012, 12014])
    },
    "EFWByBPDTTDHansmann1986": {
        "33139-7": ("EFW by BPD, TTD, Hansmann 1986", [12012, 12014])
    },
    "EFWByGAHadlock1991": {"33162-9": ("EFW by GA, Hadlock 1991", [12012, 12015])},
    "EFWByGAHansmann1986": {"33163-7": ("EFW by GA, Hansmann 1986", [12012, 12015])},
    "EFWDiscordance": {"98031-8": ("EFW Discordance", [12019])},
    "EFWPercentileRank": {"11767-1": ("EFW percentile rank", [12019])},
    "EFWRatio": {"98032-6": ("EFW Ratio", [12019])},
    "EVelocityToAnnulusEVelocityRatio": {
        "59111-5": (
            "E Velocity to Annulus E Velocity Ratio",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "EWavePeakVelocity": {
        "59080-2": (
            "E-Wave Peak Velocity",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "EchocardiographyReport": {"42148-7": ("Echocardiography Report", [7000])},
    "EjectionTime": {
        "20222-6": (
            "Ejection Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "EndDiastolicVelocity": {
        "11653-3": (
            "End Diastolic Velocity",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "EndocardialArea": {
        "59094-3": ("Endocardial Area", [12255, 12257, 12258, 12259, 12265, 12271])
    },
    "EndometriumThickness": {"12145-9": ("Endometrium Thickness", [12011])},
    "EnvelopeDuration": {
        "59108-1": (
            "Envelope Duration",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "EpicardialArea": {
        "59093-5": (
            "Epicardial Area",
            [12255, 12257, 12258, 12259, 12265, 12271, 12304],
        )
    },
    "EstimatedLMPByEDD": {"33066-2": ("Estimated LMP by EDD", [12003])},
    "EstimatedWeight": {"11727-5": ("Estimated Weight", [12019])},
    "ExerciseStressTestReport": {"18752-6": ("Exercise Stress Test Report", [7000])},
    "FLAC": {"11871-1": ("FL/AC", [12004, 12279])},
    "FLBPD": {"11872-9": ("FL/BPD", [12004, 12279])},
    "FLByGAASUM2000": {"33165-2": ("FL by GA, ASUM 2000", [12012, 12015])},
    "FLByGAChitty1994": {"33167-8": ("FL by GA, Chitty 1994", [12012, 12015])},
    "FLByGAHadlock1984": {"33166-0": ("FL by GA, Hadlock 1984", [12012, 12015])},
    "FLByGAJeanty1982": {"33168-6": ("FL by GA, Jeanty 1982", [12012, 12015])},
    "FLByGAMerz1988": {"33169-4": ("FL by GA, Merz 1988", [12012, 12015])},
    "FLByGAShinozuka1996": {"33170-2": ("FL by GA, Shinozuka 1996", [12012, 12015])},
    "FLChitty1997": {"33098-5": ("FL, Chitty 1997", [12012, 12013])},
    "FLHC": {"11873-7": ("FL/HC", [12004, 12279])},
    "FLHadlock1984": {"11920-6": ("FL, Hadlock 1984", [12012, 12013])},
    "FLHansmann1985": {"11921-4": ("FL, Hansmann 1985", [12012, 12013])},
    "FLHansmann1986": {"33541-4": ("FL, Hansmann 1986", [12012, 12013])},
    "FLHohler1982": {"11922-2": ("FL, Hohler 1982", [12012, 12013])},
    "FLJeanty1982": {"33099-3": ("FL, Jeanty 1982", [12012, 12013])},
    "FLJeanty1984": {"11923-0": ("FL, Jeanty 1984", [12012, 12013])},
    "FLLessoway1998": {"33100-9": ("FL, Lessoway 1998", [12012, 12013])},
    "FLMerz1987": {"11924-8": ("FL, Merz 1987", [12012, 12013])},
    "FLMerz1988": {"33542-2": ("FL, Merz 1988", [12012, 12013])},
    "FLOsaka1989": {"33101-7": ("FL, Osaka 1989", [12012, 12013])},
    "FLShinozuka1996": {"33102-5": ("FL, Shinozuka 1996", [12012, 12013])},
    "FLTokyo1986": {"33103-3": ("FL, Tokyo 1986", [12012, 12013])},
    "FWPByGAAlexander1996": {"33185-0": ("FWP by GA, Alexander, 1996", [12012, 12016])},
    "FWPByGABrenner1976": {"33189-2": ("FWP by GA, Brenner 1976", [12012, 12016])},
    "FWPByGAHadlock1985": {"33190-0": ("FWP by GA, Hadlock 1985", [12012, 12016])},
    "FWPByGAWilliams1982": {"33184-3": ("FWP by GA, Williams, 1982", [12012, 12016])},
    "FemaleSingletonBWPByGAArbuckle1993": {
        "33187-6": ("Female Singleton BWP by GA, Arbuckle 1993", [12012, 12016])
    },
    "FemaleTwinsBWPByGAArbuckle1993": {
        "33188-4": ("Female Twins BWP by GA, Arbuckle 1993", [12012, 12016])
    },
    "FemurLength": {"11963-6": ("Femur Length", [12005, 12006, 12279])},
    "FetalHeartRate": {"11948-7": ("Fetal Heart Rate", [12019])},
    "FetalTrunkCrossSectionalAreaOsaka1989": {
        "33138-9": ("Fetal Trunk Cross-Sectional Area, Osaka 1989", [12012, 12013])
    },
    "FibulaByGAJeanty1983": {"33164-5": ("Fibula by GA, Jeanty 1983", [12012, 12015])},
    "FibulaJeanty1983": {"33097-7": ("Fibula, Jeanty 1983", [12012, 12013])},
    "FibulaLength": {"11964-4": ("Fibula length", [12006])},
    "FibulaMerz1987": {"11918-0": ("Fibula, Merz 1987", [12012, 12013])},
    "FillingTime": {
        "59119-8": (
            "Filling Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "Findings": {"59776-5": ("Findings", [6052, 7001])},
    "FirstQuadrantDiameter": {"11624-4": ("First Quadrant Diameter", [12008])},
    "FitzpatrickSkinTypeI": {"C74569": ("Fitzpatrick Skin Type I", [4401])},
    "FitzpatrickSkinTypeII": {"C74570": ("Fitzpatrick Skin Type II", [4401])},
    "FitzpatrickSkinTypeIII": {"C74571": ("Fitzpatrick Skin Type III", [4401])},
    "FitzpatrickSkinTypeIV": {"C74572": ("Fitzpatrick Skin Type IV", [4401])},
    "FitzpatrickSkinTypeV": {"C74573": ("Fitzpatrick Skin Type V", [4401])},
    "FitzpatrickSkinTypeVI": {"C74574": ("Fitzpatrick Skin Type VI", [4401])},
    "FlowArea": {
        "20226-7": (
            "Flow Area",
            [
                12253,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "FlowRadius": {
        "59102-4": (
            "Flow Radius",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "FluoroscopyStudyReport": {"49512-7": ("Fluoroscopy Study Report", [7000])},
    "FootLength": {"11965-1": ("Foot length", [12005])},
    "FootLengthMercer1987": {"11926-3": ("Foot Length, Mercer 1987", [12012, 12013])},
    "ForamenOvaleDiameterAorticRootDiameter": {
        "59077-8": ("Foramen Ovale Diameter/Aortic Root Diameter", [12279])
    },
    "FourthQuadrantDiameter": {"11623-6": ("Fourth Quadrant Diameter", [12008])},
    "FractionalShortening": {
        "59132-1": (
            "Fractional Shortening",
            [
                12252,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
                12304,
            ],
        )
    },
    "GSByGARempen1991": {"33171-0": ("GS by GA, Rempen 1991", [12012, 12015])},
    "GSDaya1991": {"33104-1": ("GS, Daya 1991", [12012, 12013])},
    "GSHansmann1979": {"33105-8": ("GS, Hansmann 1979", [12012, 12013])},
    "GSHansmann1982": {"33106-6": ("GS, Hansmann 1982", [12012, 12013])},
    "GSHellman1969": {"11928-9": ("GS, Hellman 1969", [12012, 12013])},
    "GSNyberg1992": {"33107-4": ("GS, Nyberg 1992", [12012, 12013])},
    "GSRempen1991": {"11929-7": ("GS, Rempen 1991", [12012, 12013])},
    "GSTokyo1986": {"33108-2": ("GS, Tokyo 1986", [12012, 12013])},
    "GestationalAge": {"18185-9": ("Gestational Age", [12019])},
    "GestationalAgeByLMP": {
        "11885-1": ("Gestational Age by LMP", [12012, 12013, 12019])
    },
    "GestationalAgeByOvulationDate": {
        "11886-9": ("Gestational Age by ovulation date", [12018])
    },
    "GestationalSacDiameter": {"11850-5": ("Gestational Sac Diameter", [12009])},
    "GlomerularFiltrationRateBlackMDRD": {
        "48643-1": ("Glomerular Filtration Rate black (MDRD)", [10046])
    },
    "GlomerularFiltrationRateCreatinineBasedFormulaSchwartz": {
        "50384-7": (
            "Glomerular Filtration Rate Creatinine-based formula (Schwartz)",
            [10046],
        )
    },
    "GlomerularFiltrationRateCystatinBasedFormula": {
        "50210-4": ("Glomerular Filtration Rate Cystatin-based formula", [10046])
    },
    "GlomerularFiltrationRateFemaleMDRD": {
        "50044-7": ("Glomerular Filtration Rate female (MDRD)", [10046])
    },
    "GlomerularFiltrationRateMDRD": {
        "33914-3": ("Glomerular Filtration Rate (MDRD)", [10046])
    },
    "GlomerularFiltrationRateNonBlackMDRD": {
        "48642-3": ("Glomerular Filtration Rate non-black (MDRD)", [10046])
    },
    "HCAC": {"11947-9": ("HC/AC", [12004, 12279])},
    "HCACByGACampbell1977": {"80416-1": ("HC/AC by GA, Campbell 1977", [12012, 12015])},
    "HCASUM2000": {"33109-0": ("HC, ASUM 2000", [12012, 12013])},
    "HCByGAASUM2000": {"33172-8": ("HC by GA, ASUM 2000", [12012, 12015])},
    "HCByGAHadlock1984": {"33173-6": ("HC by GA, Hadlock 1984", [12012, 12015])},
    "HCByGAJeanty1982": {"33175-1": ("HC by GA, Jeanty 1982", [12012, 12015])},
    "HCByGAMerz1988": {"33176-9": ("HC by GA, Merz 1988", [12012, 12015])},
    "HCDerivedByGAChitty1994": {
        "33174-4": ("HC derived by GA, Chitty 1994", [12012, 12015])
    },
    "HCDerivedChitty1997": {"33111-6": ("HC derived, Chitty 1997", [12012, 12013])},
    "HCHadlock1984": {"11932-1": ("HC, Hadlock 1984", [12012, 12013])},
    "HCHansmann1985": {"33112-4": ("HC, Hansmann 1985", [12012, 12013])},
    "HCHansmann1986": {"33543-0": ("HC, Hansmann 1986", [12012, 12013])},
    "HCJeanty1982": {"33113-2": ("HC, Jeanty 1982", [12012, 12013])},
    "HCJeanty1984": {"11934-7": ("HC, Jeanty 1984", [12012, 12013])},
    "HCLessoway1998": {"33114-0": ("HC, Lessoway 1998", [12012, 12013])},
    "HCMeasuredChitty1997": {"33110-8": ("HC measured, Chitty 1997", [12012, 12013])},
    "HCMerz1988": {"33115-7": ("HC Merz, 1988", [12012, 12013])},
    "HRCorrectedMeanVelocityOfCircumferentialFiberShortening": {
        "59118-0": (
            "HR-Corrected Mean Velocity of Circumferential Fiber Shortening",
            [
                12252,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "HeadCT": {"24725-4": ("Head CT", [646])},
    "HeadCircumference": {"11984-2": ("Head Circumference", [12005, 12279])},
    "HeartRate": {
        "8867-4": (
            "Heart rate",
            [
                12200,
                12204,
                12205,
                12206,
                12207,
                12208,
                12209,
                12210,
                12211,
                12212,
                12214,
                12215,
                12216,
                12217,
                12218,
                12220,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "HeartRateCorrectedEjectionTime": {
        "59086-9": (
            "Heart Rate-Corrected Ejection Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "HeartRateCorrectedPreEjectionPeriod": {
        "59087-7": (
            "Heart Rate-Corrected Pre-Ejection Period",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "Hematocrit": {"20570-8": ("Hematocrit", [4110])},
    "HepaticVeinAtrialContractionReversalPeakVelocity": {
        "29474-4": ("Hepatic Vein Atrial Contraction Reversal Peak Velocity", [12216])
    },
    "HepaticVeinDiastolicPeakVelocity": {
        "29472-8": ("Hepatic Vein Diastolic Peak Velocity", [12216])
    },
    "HepaticVeinSystolicPeakVelocity": {
        "29471-0": ("Hepatic Vein Systolic Peak Velocity", [12216])
    },
    "HepaticVeinSystolicToDiastolicRatio": {
        "29473-6": ("Hepatic Vein Systolic to Diastolic Ratio", [12216])
    },
    "History": {"11329-0": ("History", [7001, 7002])},
    "HolterStudyReport": {"18754-2": ("Holter Study Report", [7000])},
    "HumerusJeanty1984": {"11936-2": ("Humerus, Jeanty 1984", [12012, 12013])},
    "HumerusLength": {"11966-9": ("Humerus length", [12006])},
    "HumerusLengthASUM2000": {"33116-5": ("Humerus Length, ASUM 2000", [12012, 12013])},
    "HumerusLengthByGAASUM2000": {
        "33177-7": ("Humerus Length by GA, ASUM 2000", [12012, 12015])
    },
    "HumerusLengthOsaka1989": {
        "33117-3": ("Humerus Length, Osaka 1989", [12012, 12013])
    },
    "HumerusMerz1987": {"11937-0": ("Humerus, Merz 1987", [12012, 12013])},
    "ICACCAVelocityRatio": {"33868-1": ("ICA/CCA velocity ratio", [12123])},
    "IODMayden1982": {"33122-3": ("IOD, Mayden 1982", [12012, 12013])},
    "IODTrout1994": {"33123-1": ("IOD, Trout 1994", [12012, 12013])},
    "Impressions": {"19005-8": ("Impressions", [6052, 7001])},
    "IndicationsForProcedure": {
        "18785-6": ("Indications for Procedure", [6052, 6053, 7001])
    },
    "InferiorVenaCavaDiameter": {"18006-7": ("Inferior Vena Cava Diameter", [12215])},
    "InferiorVenaCavaDiameterAtEndExpiration": {
        "79967-6": ("Inferior vena cava diameter", [12300])
    },
    "InferiorVenaCavaPercentCollapse": {
        "18050-5": ("Inferior Vena Cava % Collapse", [12215])
    },
    "InnerOrbitalDiameter": {"33070-4": ("Inner Orbital Diameter", [12007])},
    "InternalDimension": {"59090-1": ("Internal Dimension", [12304])},
    "InterventricularSeptumDiastolicDimension2D": {
        "79969-2": ("Interventricular septum diastolic dimension 2D", [12300])
    },
    "InterventricularSeptumDiastolicDimensionMM": {
        "79968-4": ("Interventricular septum diastolic dimension MM", [12300])
    },
    "InterventricularSeptumDiastolicThickness": {
        "18154-5": ("Interventricular Septum Diastolic Thickness", [12200, 12201])
    },
    "InterventricularSeptumPercentThickening": {
        "18054-7": ("Interventricular Septum % Thickening", [12200, 12201])
    },
    "InterventricularSeptumSystolicDimension2D": {
        "79971-8": ("Interventricular septum systolic dimension 2D", [12300])
    },
    "InterventricularSeptumSystolicDimensionMM": {
        "79970-0": ("Interventricular septum systolic dimension MM", [12300])
    },
    "InterventricularSeptumSystolicThickness": {
        "18158-6": ("Interventricular Septum Systolic Thickness", [12200, 12201])
    },
    "InterventricularSeptumTimeToPeakDisplacement": {
        "79972-6": ("Interventricular septum time to peak displacement", [12300])
    },
    "InterventricularSeptumToPosteriorWallThicknessRatio": {
        "18155-2": (
            "Interventricular Septum to Posterior Wall Thickness Ratio",
            [12200, 12201, 12257, 12259, 12271],
        )
    },
    "IntravascularDiastolicBloodPressure": {
        "8462-4": ("Intravascular diastolic blood pressure", [3611])
    },
    "IntravascularSystolicBloodPressure": {
        "8480-6": ("Intravascular Systolic Blood pressure", [3611])
    },
    "IsovolumicContractionTime": {
        "59084-4": (
            "Isovolumic Contraction Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "IsovolumicRelaxationTime": {
        "59083-6": (
            "Isovolumic Relaxation Time",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "JetArea": {
        "59123-0": (
            "Jet Area",
            [
                12253,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "KeyImages": {"55113-5": ("Key Images", [7001])},
    "LMP": {"8665-2": ("LMP", [12003])},
    "LVBasalAnteriorTimeToSVmaxTsBasalAnterior": {
        "80036-7": ("LV basal anterior time to S Vmax (Ts-basal anterior)", [12300])
    },
    "LVBasalAnteroseptalTimeToSVmaxTSBasalAnteroseptal": {
        "80037-5": (
            "LV basal anteroseptal time to S Vmax (TS-basal anteroseptal)",
            [12300],
        )
    },
    "LVBasalInferiorTimeToSVmaxTsBasalInferior": {
        "80038-3": ("LV basal inferior time to S Vmax (Ts-basal inferior)", [12300])
    },
    "LVBasalLateralTimeToSVmaxTsBasalLateral": {
        "80039-1": ("LV basal lateral time to S Vmax (Ts-basal lateral)", [12300])
    },
    "LVBasalPosteriorTimeToSVmaxTsBasalPosterior": {
        "80040-9": ("LV basal posterior time to S Vmax (Ts-basal posterior)", [12300])
    },
    "LVBasalSeptalTimeToSVmaxTsBasalSeptal": {
        "80041-7": ("LV basal septal time to S Vmax (Ts-basal septal)", [12300])
    },
    "LVMidAnteriorTimeToSVmaxTsMidAnterior": {
        "80042-5": ("LV mid anterior time to S Vmax (Ts-mid anterior)", [12300])
    },
    "LVMidAnteroseptalTimeToSVmaxTsMidAnteroseptal": {
        "80043-3": ("LV mid anteroseptal time to S Vmax (Ts-mid anteroseptal)", [12300])
    },
    "LVMidInferiorTimeToSVmaxTsMidInferior": {
        "80044-1": ("LV mid inferior time to S Vmax (Ts-mid inferior)", [12300])
    },
    "LVMidLateralTimeToSVmaxTsMidLateral": {
        "80045-8": ("LV mid lateral time to S Vmax (Ts-mid lateral)", [12300])
    },
    "LVMidPosteriorTimeToSVmaxTsMidPosterior": {
        "80046-6": ("LV mid posterior time to S Vmax (Ts-mid posterior)", [12300])
    },
    "LVMidSeptalTimeToSVmaxTsMidSeptal": {
        "80047-4": ("LV mid septal time to S Vmax (Ts-mid septal)", [12300])
    },
    "LVTsSDDyssynchronyIndex": {"80048-2": ("LV Ts-SD (Dyssynchrony Index)", [12300])},
    "LateralVentricleWidth": {"12171-5": ("Lateral Ventricle width", [12007])},
    "LeafletSeparation": {
        "59109-9": (
            "Leaflet Separation",
            [12251, 12262, 12263, 12266, 12267, 12268, 12271, 12272],
        )
    },
    "LeafletThickness": {
        "59110-7": (
            "Leaflet Thickness",
            [12251, 12262, 12263, 12266, 12267, 12268, 12271, 12272],
        )
    },
    "LeftAtrialAppendagePeakVelocity": {
        "29486-8": ("Left Atrial Appendage Peak Velocity", [12205])
    },
    "LeftAtrialEndSystolicArea2C": {
        "79973-4": ("Left atrial end systolic area 2C", [12300])
    },
    "LeftAtrialEndSystolicArea4C": {
        "79974-2": ("Left atrial end systolic area 4C", [12300])
    },
    "LeftAtrialEndSystolicDiameterAP2D": {
        "79975-9": ("Left atrial end systolic diameter (AP) 2D", [12300])
    },
    "LeftAtrialEndSystolicDiameterAP2DBSA": {
        "79976-7": ("Left atrial end systolic diameter (AP) 2D / BSA", [12300])
    },
    "LeftAtrialEndSystolicDiameterAPMM": {
        "79977-5": ("Left atrial end systolic diameter (AP) MM", [12300])
    },
    "LeftAtrialEndSystolicDiameterAPMMBSA": {
        "79978-3": ("Left atrial end systolic diameter (AP) MM / BSA", [12300])
    },
    "LeftAtrialEndSystolicLength2C": {
        "79979-1": ("Left atrial end systolic length 2C", [12300])
    },
    "LeftAtrialEndSystolicLength4C": {
        "79980-9": ("Left atrial end systolic length 4C", [12300])
    },
    "LeftAtrialEndSystolicVolumeBiplaneAreaLength": {
        "79981-7": ("Left atrial end systolic volume biplane (area-length)", [12300])
    },
    "LeftAtrialEndSystolicVolumeBiplaneAreaLengthBSA": {
        "79982-5": (
            "Left atrial end systolic volume biplane (area-length) / BSA",
            [12300],
        )
    },
    "LeftAtrialEndSystolicVolumeBiplaneMOD": {
        "79983-3": ("Left atrial end systolic volume biplane (MOD)", [12300])
    },
    "LeftAtrialEndSystolicVolumeBiplaneMODBSA": {
        "79984-1": ("Left atrial end systolic volume biplane (MOD) / BSA", [12300])
    },
    "LeftAtrialEndSystolicVolumeSinglePlane2CMOD": {
        "79985-8": ("Left atrial end systolic volume single plane 2C (MOD)", [12300])
    },
    "LeftAtrialEndSystolicVolumeSinglePlane4CMOD": {
        "79986-6": ("Left atrial end systolic volume single plane 4C (MOD)", [12300])
    },
    "LeftAtriumAnteroPosteriorSystolicDimension": {
        "29469-4": ("Left Atrium Antero-posterior Systolic Dimension", [12205])
    },
    "LeftAtriumAreaA4CView": {"17977-0": ("Left Atrium Area A4C view", [12205])},
    "LeftAtriumToAorticRootRatio": {
        "17985-3": ("Left Atrium to Aortic Root Ratio", [12205, 12265])
    },
    "LeftAtriumVolumeToRightAtriumVolumeRatio": {
        "59131-3": ("Left Atrium Volume to Right Atrium Volume Ratio", [12265])
    },
    "LeftFetalEarLength": {"53684-7": ("Left Fetal Ear Length", [12005])},
    "LeftKidneyLength": {"11834-9": ("Left Kidney length", [12005])},
    "LeftKidneyThickness": {"11853-9": ("Left Kidney thickness", [12005])},
    "LeftKidneyWidth": {"11825-7": ("Left Kidney width", [12005])},
    "LeftPulmonaryArteryDiameter": {
        "18019-0": ("Left Pulmonary Artery Diameter", [12210])
    },
    "LeftPulmonaryArteryDiameterAtEndSystole": {
        "79987-4": ("Left pulmonary artery diameter", [12300])
    },
    "LeftVenticularPosteriorWallTimeToPeakDisplacement": {
        "79988-2": ("Left venticular posterior wall time to peak displacement", [12300])
    },
    "LeftVentricleDiastolicMajorAxis": {
        "18077-8": ("Left Ventricle diastolic major axis", [12200, 12201])
    },
    "LeftVentricleEToATissueVelocityRatio": {
        "59129-7": (
            "Left Ventricle E to A Tissue Velocity Ratio",
            [12257, 12259, 12271],
        )
    },
    "LeftVentricleInternalEndDiastolicDimension": {
        "29436-3": ("Left Ventricle Internal End Diastolic Dimension", [12200, 12201])
    },
    "LeftVentricleInternalSystolicDimension": {
        "29438-9": ("Left Ventricle Internal Systolic Dimension", [12200, 12201])
    },
    "LeftVentricleMass": {"18087-7": ("Left Ventricle Mass", [12200, 12203])},
    "LeftVentricleMeridionalWallStress": {
        "59097-6": ("Left Ventricle Meridional Wall Stress", [12257, 12259, 12271])
    },
    "LeftVentriclePosteriorWallDiastolicThickness": {
        "18152-9": ("Left Ventricle Posterior Wall Diastolic Thickness", [12200, 12201])
    },
    "LeftVentriclePosteriorWallPercentThickening": {
        "18053-9": ("Left Ventricle Posterior Wall % Thickening", [12200, 12201])
    },
    "LeftVentriclePosteriorWallSystolicThickness": {
        "18156-0": ("Left Ventricle Posterior Wall Systolic Thickness", [12200, 12201])
    },
    "LeftVentricleRightVentricleDiameterRatio": {
        "59078-6": ("Left Ventricle/Right Ventricle Diameter Ratio", [12279])
    },
    "LeftVentricleSystolicMajorAxis": {
        "18076-0": ("Left Ventricle systolic major axis", [12200, 12201])
    },
    "LeftVentricularEDVolume": {"8821-1": ("Left Ventricular ED Volume", [3468, 3835])},
    "LeftVentricularESVolume": {"8823-7": ("Left Ventricular ES Volume", [3469, 3835])},
    "LeftVentricularEjectionFraction3D": {
        "79990-8": ("Left ventricular ejection fraction 3D", [12300])
    },
    "LeftVentricularEjectionFractionBiplaneMOD": {
        "79991-6": ("Left ventricular ejection fraction biplane (MOD)", [12300])
    },
    "LeftVentricularEjectionFractionByAngiography": {
        "8808-8": ("Left Ventricular Ejection Fraction by Angiography", [3467])
    },
    "LeftVentricularEjectionFractionByCT": {
        "8810-4": ("Left ventricular ejection fraction by CT", [3833])
    },
    "LeftVentricularEjectionFractionByMR": {
        "8811-2": ("Left ventricular ejection fraction by MR", [3833])
    },
    "LeftVentricularEjectionFractionByUS": {
        "18043-0": ("Left Ventricular Ejection Fraction by US", [12200, 12202])
    },
    "LeftVentricularEjectionFractionSinglePlane2CMOD": {
        "79992-4": ("Left ventricular ejection fraction single plane 2C (MOD)", [12300])
    },
    "LeftVentricularEjectionFractionSinglePlane4CMOD": {
        "79993-2": ("Left ventricular ejection fraction single plane 4C (MOD)", [12300])
    },
    "LeftVentricularEjectionFractionTeichholz2D": {
        "77891-0": ("Left ventricular ejection fraction (Teichholz) 2D", [12300])
    },
    "LeftVentricularEjectionFractionTeichholzMM": {
        "18049-7": ("Left ventricular ejection fraction (Teichholz) MM", [12300])
    },
    "LeftVentricularEndDiastolicLength4C": {
        "79994-0": ("Left ventricular end diastolic length 4C", [12300])
    },
    "LeftVentricularEndDiastolicVolume": {
        "18026-5": ("Left Ventricular End Diastolic Volume", [12200, 12202])
    },
    "LeftVentricularEndDiastolicVolume3D": {
        "79995-7": ("Left ventricular end diastolic volume (3D)", [12300])
    },
    "LeftVentricularEndDiastolicVolumeBiplaneMOD": {
        "79996-5": ("Left ventricular end diastolic volume biplane (MOD)", [12300])
    },
    "LeftVentricularEndDiastolicVolumeBiplaneMODBSA": {
        "79997-3": (
            "Left ventricular end diastolic volume biplane (MOD) / BSA",
            [12300],
        )
    },
    "LeftVentricularEndDiastolicVolumeSinglePlane2CMOD": {
        "79998-1": (
            "Left ventricular end diastolic volume single plane 2C (MOD)",
            [12300],
        )
    },
    "LeftVentricularEndDiastolicVolumeSinglePlane4CMOD": {
        "79999-9": (
            "Left ventricular end diastolic volume single plane 4C (MOD)",
            [12300],
        )
    },
    "LeftVentricularEndSystolicVolume": {
        "18148-7": ("Left Ventricular End Systolic Volume", [12200, 12202])
    },
    "LeftVentricularEndSystolicVolume3D": {
        "80000-3": ("Left ventricular end systolic volume (3D)", [12300])
    },
    "LeftVentricularEndSystolicVolumeBiplaneMOD": {
        "80001-1": ("Left ventricular end systolic volume biplane (MOD)", [12300])
    },
    "LeftVentricularEndSystolicVolumeBiplaneMODBSA": {
        "80002-9": ("Left ventricular end systolic volume biplane (MOD) / BSA", [12300])
    },
    "LeftVentricularEndSystolicVolumeSinglePlane2CMOD": {
        "80003-7": (
            "Left ventricular end systolic volume single plane 2C (MOD)",
            [12300],
        )
    },
    "LeftVentricularEndSystolicVolumeSinglePlane4CMOD": {
        "80004-5": (
            "Left ventricular end systolic volume single plane 4C (MOD)",
            [12300],
        )
    },
    "LeftVentricularEndocardialAreaSAXPMLevel": {
        "80005-2": ("Left ventricular endocardial area SAX PM level", [12300])
    },
    "LeftVentricularEpicardialAreaSAXPMLevel": {
        "80006-0": ("Left ventricular epicardial area SAX PM level", [12300])
    },
    "LeftVentricularFractionalShortening": {
        "18051-3": ("Left Ventricular Fractional Shortening", [12200, 12201])
    },
    "LeftVentricularFractionalShorteningOfMinorAxis2D": {
        "29434-8": (
            "Left ventricular fractional shortening (of minor axis) (2D)",
            [12300],
        )
    },
    "LeftVentricularFractionalShorteningOfMinorAxisMM": {
        "29435-5": (
            "Left ventricular fractional shortening (of minor axis) (MM)",
            [12300],
        )
    },
    "LeftVentricularInternalDiastolicDimension2D": {
        "80007-8": ("Left ventricular internal diastolic dimension - 2D", [12300])
    },
    "LeftVentricularInternalDiastolicDimensionByUS2DBSA": {
        "80010-2": ("Left ventricular internal diastolic dimension / BSA", [12300])
    },
    "LeftVentricularInternalDiastolicDimensionByUSMModeBSA": {
        "80009-4": ("Left ventricular internal diastolic dimension / BSA", [12300])
    },
    "LeftVentricularInternalDiastolicDimensionMM": {
        "80008-6": ("Left ventricular internal diastolic dimension - MM", [12300])
    },
    "LeftVentricularInternalSystolicDimension2D": {
        "80011-0": ("Left ventricular internal systolic dimension - 2D", [12300])
    },
    "LeftVentricularInternalSystolicDimensionByUS2DBSA": {
        "80014-4": ("Left ventricular internal systolic dimension / BSA", [12300])
    },
    "LeftVentricularInternalSystolicDimensionByUSMModeBSA": {
        "80013-6": ("Left ventricular internal systolic dimension / BSA", [12300])
    },
    "LeftVentricularInternalSystolicDimensionMM": {
        "80012-8": ("Left ventricular internal systolic dimension - MM", [12300])
    },
    "LeftVentricularIsovolumicRelaxationTime": {
        "18071-1": ("Left Ventricular Isovolumic Relaxation Time", [12200, 12203])
    },
    "LeftVentricularIsovolumicRelaxationTimeByDoppler": {
        "18071-1": ("Left ventricular isovolumic relaxation time by Doppler", [12300])
    },
    "LeftVentricularIsovolumicRelaxationTimeByTDI": {
        "80015-1": ("Left ventricular isovolumic relaxation time by TDI", [12300])
    },
    "LeftVentricularMass": {
        "80025-0": ("Left ventricular mass (truncated ellipse)", [12300])
    },
    "LeftVentricularMassAreaLength": {
        "80016-9": ("Left ventricular mass (area-length)", [12300])
    },
    "LeftVentricularMassAreaLengthBSA": {
        "80017-7": ("Left ventricular mass (area-length) / BSA", [12300])
    },
    "LeftVentricularMassAreaLengthHeight2Point7": {
        "80018-5": ("Left ventricular mass (area-length) / height^^2.7", [12300])
    },
    "LeftVentricularMassDimensionMethod2D": {
        "80019-3": ("Left ventricular mass (dimension method) 2D", [12300])
    },
    "LeftVentricularMassDimensionMethod2DBSA": {
        "80020-1": ("Left ventricular mass (dimension method) 2D / BSA", [12300])
    },
    "LeftVentricularMassDimensionMethod2DHeight2Point7": {
        "80021-9": (
            "Left ventricular mass (dimension method) 2D / height^^2.7",
            [12300],
        )
    },
    "LeftVentricularMassDimensionMethodMM": {
        "80022-7": ("Left ventricular mass (dimension method) MM", [12300])
    },
    "LeftVentricularMassDimensionMethodMMBSA": {
        "80023-5": ("Left ventricular mass (dimension method) MM / BSA", [12300])
    },
    "LeftVentricularMassDimensionMethodMMHeight2Point7": {
        "80024-3": (
            "Left ventricular mass (dimension method) MM / height^^2.7",
            [12300],
        )
    },
    "LeftVentricularMassTruncatedEllipseBSA": {
        "80026-8": ("Left ventricular mass (truncated ellipse) / BSA", [12300])
    },
    "LeftVentricularMassTruncatedEllipseHeight2Point7": {
        "80027-6": ("Left ventricular mass (truncated ellipse) / height^^2.7", [12300])
    },
    "LeftVentricularOutflowTractDimension2D": {
        "80028-4": ("Left ventricular outflow tract dimension (2D)", [12300])
    },
    "LeftVentricularOutflowTractVTI": {
        "80030-0": ("Left ventricular outflow tract VTI", [12300])
    },
    "LeftVentricularOutflowTractVmax": {
        "80029-2": ("Left ventricular outflow tract Vmax", [12300])
    },
    "LeftVentricularPosteriorWallDiastolicThicknessByUS2D": {
        "80032-6": ("Left ventricular posterior wall diastolic thickness", [12300])
    },
    "LeftVentricularPosteriorWallDiastolicThicknessByUSMMode": {
        "80031-8": ("Left ventricular posterior wall diastolic thickness", [12300])
    },
    "LeftVentricularPosteriorWallSystolicThicknessByUS2D": {
        "80034-2": ("Left ventricular posterior wall systolic thickness", [12300])
    },
    "LeftVentricularPosteriorWallSystolicThicknessByUSMMode": {
        "80033-4": ("Left ventricular posterior wall systolic thickness", [12300])
    },
    "LeftVentricularPreEjectionPeriod": {
        "79989-0": ("Left ventricular pre-ejection period", [12300])
    },
    "LeftVentricularStrokeVolume3D": {
        "80035-9": ("Left ventricular stroke volume 3D", [12300])
    },
    "LengthOfVertebraTokyo1986": {
        "33118-1": ("Length of Vertebra, Tokyo 1986", [12012, 12013])
    },
    "MR3DCAMModel": {"85041-2": ("MR 3D CAM model", [7061])},
    "MRIHeadReport": {"24590-2": ("MRI Head Report", [7000])},
    "MRIReport": {"25056-3": ("MRI Report", [7000])},
    "MRISpineReport": {"18756-7": ("MRI Spine Report", [7000])},
    "MRIUnspecifiedBodyRegion": {"25056-3": ("MRI unspecified body region", [100])},
    "MRIWholeBody": {"46358-8": ("MRI whole body", [646])},
    "MainPulmonaryArteryDiameter": {
        "18020-8": ("Main Pulmonary Artery Diameter", [12210])
    },
    "MainPulmonaryArteryDiameterAtEndSystole": {
        "80049-0": ("Main pulmonary artery diameter", [12300])
    },
    "MainPulmonaryArteryVmax": {"": ("Main pulmonary artery Vmax", [12300])},
    "MaleSingletonBWPByGAArbuckle1993": {
        "33186-8": ("Male Singleton BWP by GA, Arbuckle 1993", [12012, 12016])
    },
    "MaleTwinsBWPByGAArbuckle1993": {
        "33199-1": ("Male Twins BWP by GA, Arbuckle 1993", [12012, 12016])
    },
    "MammographyScreeningReport": {"24606-6": ("Mammography Screening Report", [7000])},
    "MeanBloodVelocity": {"20352-1": ("Mean Blood Velocity", [12304])},
    "MeanGradient": {
        "20256-4": (
            "Mean Gradient",
            [
                12119,
                12122,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "MeanGradientPressureByDoppler": {
        "20256-4": ("Mean Gradient [Pressure] by Doppler", [12304])
    },
    "MeanVelocityOfCircumferentialFiberShorteningMeanVcfv": {
        "59117-2": (
            "Mean Velocity of Circumferential Fiber Shortening (Mean VcFv)",
            [
                12252,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "MedicationsAdministered": {"29549-3": ("Medications Administered", [7001])},
    "MinimumDiastolicVelocity": {
        "11665-7": ("Minimum Diastolic Velocity", [12119, 12120])
    },
    "MitralAnnulusDiastolicDiameterA2C": {
        "80050-8": ("Mitral annulus diastolic diameter - A2C", [12300])
    },
    "MitralAnnulusDiastolicDiameterA4C": {
        "80051-6": ("Mitral annulus diastolic diameter - A4C", [12300])
    },
    "MitralAnnulusDiastolicDiameterPLAX": {
        "80052-4": ("Mitral annulus diastolic diameter - PLAX", [12300])
    },
    "MitralAnnulusVTI": {"80053-2": ("Mitral annulus VTI", [12300])},
    "MitralLateralEPrimeVmax": {"80054-0": ("Mitral lateral e-prime Vmax", [12300])},
    "MitralRegurgitantFlowPISA": {
        "80057-3": ("Mitral regurgitant flow (PISA)", [12300])
    },
    "MitralRegurgitantFractionContinuityVTI": {
        "80055-7": ("Mitral regurgitant fraction (Continuity VTI)", [12300])
    },
    "MitralRegurgitantFractionPISA": {
        "80056-5": ("Mitral regurgitant fraction (PISA)", [12300])
    },
    "MitralRegurgitationDpDtDerivedFromMitralRegVelocity": {
        "18035-6": (
            "Mitral Regurgitation dP/dt derived from Mitral Reg. velocity",
            [12207, 12266, 12268],
        )
    },
    "MitralRegurgitationPISARadius": {
        "80059-9": ("Mitral regurgitation PISA radius", [12300])
    },
    "MitralRegurgitationPISAVelocity": {
        "80060-7": ("Mitral regurgitation PISA velocity", [12300])
    },
    "MitralRegurgitationPeakGradient": {
        "80058-1": ("Mitral regurgitation peak gradient", [12300])
    },
    "MitralRegurgitationVenaContractaWidth": {
        "80061-5": ("Mitral regurgitation vena contracta width", [12300])
    },
    "MitralRegurgitationVmax": {"80062-3": ("Mitral regurgitation Vmax", [12300])},
    "MitralRegurgitationVolumeContinuityVTI": {
        "80063-1": ("Mitral regurgitation volume (Continuity VTI)", [12300])
    },
    "MitralRegurgitationVolumePISA": {
        "80064-9": ("Mitral regurgitation volume (PISA)", [12300])
    },
    "MitralSeptalEPrimeVmax": {"79911-4": ("Mitral septal e-prime Vmax", [12300])},
    "MitralValveAWaveDuration": {"80065-6": ("Mitral valve A-wave duration", [12300])},
    "MitralValveAWavePeakVelocity": {
        "17978-8": ("Mitral Valve A-Wave Peak Velocity", [12207])
    },
    "MitralValveAWaveVmax": {"80066-4": ("Mitral valve A-wave Vmax", [12300])},
    "MitralValveAreaPISA": {"80067-2": ("Mitral valve area (PISA)", [12300])},
    "MitralValveAreaPlanimetry": {
        "80068-0": ("Mitral valve area (Planimetry)", [12300])
    },
    "MitralValveAreaPressureHalfTime": {
        "80069-8": ("Mitral valve area (Pressure Half-Time)", [12300])
    },
    "MitralValveDecelerationTime": {
        "78191-4": ("Mitral valve deceleration time", [12300])
    },
    "MitralValveDiastolicPeakInstantaneousGradient": {
        "18057-0": ("Mitral Valve Diastolic Peak Instantaneous Gradient", [12207])
    },
    "MitralValveEFSlopeByMMode": {
        "18040-6": ("Mitral Valve E-F Slope by M-Mode", [12207])
    },
    "MitralValveEPSSEWave": {
        "18036-4": ("Mitral Valve EPSS, E wave", [12207, 12266, 12268])
    },
    "MitralValveESeptalSeparation": {
        "59098-4": ("Mitral Valve E-septal Separation", [12266, 12268])
    },
    "MitralValveEToARatio": {"18038-0": ("Mitral Valve E to A Ratio", [12207, 12300])},
    "MitralValveEWavePeakVelocity": {
        "18037-2": ("Mitral Valve E-Wave Peak Velocity", [12207])
    },
    "MitralValveEWaveVmax": {"80070-6": ("Mitral valve E-wave Vmax", [12300])},
    "MitralValveEffectiveRegurgitantOrificeAreaPISA": {
        "80071-4": ("Mitral valve effective regurgitant orifice area (PISA)", [12300])
    },
    "MitralValveFlowPropagationVelocityVp": {
        "80072-2": ("Mitral valve flow propagation velocity (Vp)", [12300])
    },
    "MitralValveMeanGradient": {"80073-0": ("Mitral valve mean gradient", [12300])},
    "MitralValvePeakInstantaneousGradient": {
        "80074-8": ("Mitral valve peak instantaneous gradient", [12300])
    },
    "MitralValvePressureHalfTime": {
        "79912-2": ("Mitral valve pressure half-time", [12300])
    },
    "MitralValveVTI": {"79914-8": ("Mitral valve VTI", [12300])},
    "MitralValveVmax": {"79913-0": ("Mitral valve Vmax", [12300])},
    "MyocardialPerformanceIndexTei": {
        "59099-2": (
            "Myocardial Performance Index (Tei)",
            [12255, 12257, 12258, 12259, 12265, 12271, 12304],
        )
    },
    "NMUnspecifiedBodyRegion": {"49118-3": ("NM unspecified body region", [100])},
    "NuchalFoldThickness": {"12146-7": ("Nuchal Fold thickness", [12007])},
    "NuchalTranslucency": {"33069-6": ("Nuchal Translucency", [12007, 12009, 12279])},
    "NuclearMedicineReport": {"49118-3": ("Nuclear Medicine Report", [7000])},
    "NuclearMedicineThyroidScanReport": {
        "30695-1": ("Nuclear Medicine Thyroid Scan Report", [7000])
    },
    "NumberOfFetusesByUS": {"11878-6": ("Number of Fetuses by US", [12018])},
    "OFDASUM2000": {"33119-9": ("OFD, ASUM 2000", [12012, 12013])},
    "OFDByGAASUM2000": {"33178-5": ("OFD by GA, ASUM 2000", [12012, 12015])},
    "OFDByGAChitty1994": {"33179-3": ("OFD by GA, Chitty 1994", [12012, 12015])},
    "OFDHansmann1985": {"33544-8": ("OFD, Hansmann 1985", [12012, 12013])},
    "OFDHansmann1986": {"33120-7": ("OFD, Hansmann 1986", [12012, 12013])},
    "OFDLessoway1998": {"33121-5": ("OFD, Lessoway 1998", [12012, 12013])},
    "OODMayden1982": {"33124-9": ("OOD, Mayden, 1982", [12012, 12013])},
    "OODTrout1994": {"33125-6": ("OOD, Trout 1994", [12012, 12013])},
    "OccipitalFrontalDiameter": {
        "11851-3": ("Occipital-Frontal Diameter", [12005, 12279])
    },
    "OuterOrbitalDiameter": {"11629-3": ("Outer Orbital Diameter", [12007])},
    "OvulationDate": {"11976-8": ("Ovulation date", [12003])},
    "OxygenPartialPressureAt50PercentSaturationP50": {
        "19217-9": ("Oxygen partial pressure at 50% saturation (P50)", [3524])
    },
    "PETScanReport": {"44136-0": ("PET Scan Report", [7000])},
    "PETUnspecifiedBodyRegion": {"44136-0": ("PET unspecified body region", [100])},
    "PETWholeBody": {"44139-4": ("PET whole body", [100, 646])},
    "PatientPresentation": {"55108-5": ("Patient Presentation", [7001, 7002])},
    "PeakBloodVelocity": {
        "20355-4": (
            "Peak Blood Velocity",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "PeakEWavePeakAWaveByUS": {
        "59104-0": (
            "Peak E wave/Peak A wave by US",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "PeakGradient": {
        "20247-3": (
            "Peak Gradient",
            [
                12119,
                12122,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "PeakGradientPressure": {"20247-3": ("Peak Gradient [Pressure]", [12304])},
    "PeakInstantaneousFlowRate": {
        "34141-2": (
            "Peak Instantaneous Flow Rate",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "PeakReversalVelocityDuringAtrialContraction": {
        "59079-4": (
            "Peak Reversal Velocity during Atrial Contraction",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "PeakSystolicVelocity": {
        "11726-7": (
            "Peak Systolic Velocity",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "PeakTissueVelocity": {"59133-9": ("Peak Tissue Velocity", [12257, 12259, 12271])},
    "PercentThickening": {
        "59092-7": (
            "% Thickening",
            [
                12252,
                12257,
                12259,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "PosteriorHornLateralVentricularWidth": {
        "33196-7": ("Posterior Horn Lateral ventricular width", [12007])
    },
    "PreEjectionPeriod": {
        "59085-1": (
            "Pre-Ejection Period",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "PreEjectionPeriodEjectionTimeRatio": {
        "59088-5": (
            "Pre-Ejection Period/Ejection Time Ratio",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "PressureHalfTime": {
        "20280-4": (
            "Pressure Half-Time",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "PreviousFindings": {"18834-2": ("Previous Findings", [7001])},
    "PriorProcedureDescriptions": {"55114-3": ("Prior Procedure Descriptions", [7001])},
    "ProstateCancerAntigen": {"2857-1": ("Prostate Cancer Antigen", [6352])},
    "ProstateSpecificAntigenDensity": {
        "15325-4": ("Prostate specific Antigen Density", [6352])
    },
    "PulmVeinADurationToMVADurationDifference": {
        "59114-9": ("Pulm Vein A duration to MV A duration difference", [12261, 12263])
    },
    "PulmonaryArteryMainOxygenContent": {
        "10245-9": ("Pulmonary Artery Main Oxygen content", [3525])
    },
    "PulmonaryArteryPressureUsingAccelTime": {
        "59101-6": (
            "Pulmonary Artery Pressure using Accel Time",
            [12262, 12263, 12271, 12272],
        )
    },
    "PulmonaryToSystemicShuntFlowRatio": {
        "29462-9": ("Pulmonary-to-Systemic Shunt Flow Ratio", [12217, 12276])
    },
    "PulmonaryVeinADurationMitralValveADurationRatio": {
        "59112-3": (
            "Pulmonary Vein A Duration Mitral Valve A Duration Ratio",
            [12261, 12263],
        )
    },
    "PulmonaryVeinAVTIToMitralValveVTIRatio": {
        "59113-1": ("Pulmonary Vein A VTI to Mitral Valve VTI Ratio", [12261, 12263])
    },
    "PulmonaryVeinAWaveDuration": {
        "78184-9": ("Pulmonary vein A-wave duration", [12300])
    },
    "PulmonaryVeinAWaveVmax": {"79915-5": ("Pulmonary vein A-wave Vmax", [12300])},
    "PulmonaryVeinAtrialContractionReversalPeakVelocity": {
        "29453-8": ("Pulmonary Vein Atrial Contraction Reversal Peak Velocity", [12214])
    },
    "PulmonaryVeinDWaveVmax": {"79916-3": ("Pulmonary vein D-wave Vmax", [12300])},
    "PulmonaryVeinDiastolicPeakVelocity": {
        "29451-2": ("Pulmonary Vein Diastolic Peak Velocity", [12214])
    },
    "PulmonaryVeinSWaveVmax": {"79917-1": ("Pulmonary vein S-wave Vmax", [12300])},
    "PulmonaryVeinSystolicPeakVelocity": {
        "29450-4": ("Pulmonary Vein Systolic Peak Velocity", [12214])
    },
    "PulmonaryVeinSystolicToDiastolicRatio": {
        "29452-0": ("Pulmonary Vein Systolic to Diastolic Ratio", [12214])
    },
    "PulmonaryWedgeOxygenContent": {
        "10247-5": ("Pulmonary Wedge Oxygen content", [3525])
    },
    "PulmonicAnnulusDiameter": {"79909-8": ("Pulmonic annulus diameter", [12300])},
    "PulmonicRegurgitationEndDiastolicPeakGradient": {
        "79934-6": ("Pulmonic regurgitation end diastolic peak gradient", [12300])
    },
    "PulmonicRegurgitationEndDiastolicVelocity": {
        "79918-9": ("Pulmonic regurgitation end diastolic velocity", [12300])
    },
    "PulmonicRegurgitationVmax": {"79919-7": ("Pulmonic regurgitation Vmax", [12300])},
    "PulmonicValveAccelerationTime": {
        "79928-8": ("Pulmonic valve acceleration time", [12300])
    },
    "PulmonicValveAreaByContinuity": {
        "18096-8": ("Pulmonic Valve Area by continuity", [12209])
    },
    "PulmonicValveEjectionTime": {
        "18042-2": ("Pulmonic Valve Ejection Time", [12209, 12300])
    },
    "PulmonicValvePeakGradient": {"79935-3": ("Pulmonic valve peak gradient", [12300])},
    "PulmonicValveVTI": {"79910-6": ("Pulmonic valve VTI", [12300])},
    "PulmonicValveVmax": {"79920-5": ("Pulmonic valve Vmax", [12300])},
    "PulsatilityIndex": {"12008-9": ("Pulsatility Index", [12119, 12121])},
    "ROIInternalDimensionByUS": {
        "59090-1": (
            "ROI Internal Dimension by US",
            [
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
            ],
        )
    },
    "ROIThicknessByUS": {
        "59089-3": (
            "ROI Thickness by US",
            [
                12218,
                12250,
                12251,
                12257,
                12258,
                12259,
                12260,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12277,
            ],
        )
    },
    "RadiationExposureAndProtectionInformation": {
        "73569-6": ("Radiation Exposure and Protection Information", [7001])
    },
    "RadiologyReport": {"11528-7": ("Radiology Report", [7000])},
    "RadionuclideBrainScan": {"24730-4": ("Radionuclide brain scan", [646])},
    "RadionuclideScanOfWholeBody": {
        "42175-0": ("Radionuclide scan of whole body", [646])
    },
    "RadiusByGAJeanty1983": {"33180-1": ("Radius by GA, Jeanty 1983", [12012, 12015])},
    "RadiusJeanty1983": {"33126-4": ("Radius, Jeanty 1983", [12012, 12013])},
    "RadiusLength": {"11967-7": ("Radius length", [12006])},
    "RadiusMerz1987": {"11939-6": ("Radius, Merz 1987", [12012, 12013])},
    "Recommendations": {"18783-1": ("Recommendations", [6052, 7001])},
    "RelevantDiagnosticTestsAndOrLaboratoryData": {
        "30954-2": ("Relevant Diagnostic Tests and/or Laboratory Data", [9509])
    },
    "RenalArteryAortaVelocityRatio": {
        "33869-9": ("Renal Artery/Aorta velocity ratio", [12124])
    },
    "Request": {"55115-0": ("Request", [7001, 7002])},
    "ResistivityIndex": {"12023-8": ("Resistivity Index", [12119, 12121])},
    "RespirationRhythm": {"9304-7": ("Respiration Rhythm", [3441])},
    "RightAtrialEndSystolicArea4C": {
        "80075-5": ("Right atrial end systolic area 4C", [12300])
    },
    "RightAtrialMajorAxisDimension4C": {
        "80076-3": ("Right atrial major axis dimension 4C", [12300])
    },
    "RightAtrialMinorAxisDimension4C": {
        "80077-1": ("Right atrial minor axis dimension 4C", [12300])
    },
    "RightAtrialMinorAxisDimension4CBSA": {
        "80078-9": ("Right atrial minor axis dimension 4C / BSA", [12300])
    },
    "RightAtriumAreaA4CView": {"17988-7": ("Right Atrium Area A4C view", [12206])},
    "RightAtriumSystolicPressure": {
        "18070-3": ("Right Atrium Systolic Pressure", [12206])
    },
    "RightFetalEarLength": {"53668-0": ("Right Fetal Ear Length", [12005])},
    "RightKidneyLength": {"11836-4": ("Right Kidney length", [12005])},
    "RightKidneyThickness": {"11855-4": ("Right Kidney thickness", [12005])},
    "RightKidneyWidth": {"11827-3": ("Right Kidney width", [12005])},
    "RightPulmonaryArteryDiameter": {
        "18021-6": ("Right Pulmonary Artery Diameter", [12210])
    },
    "RightPulmonaryArteryDiameterAtEndSystole": {
        "80079-7": ("Right pulmonary artery diameter", [12300])
    },
    "RightVentricularAnteriorWallDiastolicThickness": {
        "18153-7": ("Right Ventricular Anterior Wall Diastolic Thickness", [12204])
    },
    "RightVentricularAnteriorWallSystolicThickness": {
        "18157-8": ("Right Ventricular Anterior Wall Systolic Thickness", [12204])
    },
    "RightVentricularBasalDimension4C": {
        "80080-5": ("Right ventricular basal dimension 4C", [12300])
    },
    "RightVentricularEDVolume": {
        "8822-9": ("Right Ventricular ED Volume", [3468, 3835])
    },
    "RightVentricularESVolume": {
        "8824-5": ("Right Ventricular ES Volume", [3469, 3835])
    },
    "RightVentricularEjectionFractionByAngiography": {
        "8815-3": ("Right Ventricular Ejection Fraction by Angiography", [3467])
    },
    "RightVentricularEjectionFractionByCT": {
        "8817-9": ("Right ventricular ejection fraction by CT", [3833])
    },
    "RightVentricularEjectionFractionByMR": {
        "8818-7": ("Right ventricular ejection fraction by MR", [3833])
    },
    "RightVentricularEjectionTime": {
        "79929-6": ("Right ventricular ejection time", [12300])
    },
    "RightVentricularEndDiastolicArea4C": {
        "80081-3": ("Right ventricular end diastolic area 4C", [12300])
    },
    "RightVentricularEndSystolicArea4C": {
        "80082-1": ("Right ventricular end systolic area 4C", [12300])
    },
    "RightVentricularFractionalAreaChange": {
        "79936-1": ("Right ventricular fractional area change", [12300])
    },
    "RightVentricularFreeWallThickness2D": {
        "80083-9": ("Right ventricular free wall thickness 2D", [12300])
    },
    "RightVentricularFreeWallThicknessMM": {
        "80084-7": ("Right ventricular free wall thickness MM", [12300])
    },
    "RightVentricularInternalDiastolicDimension": {
        "20304-2": ("Right Ventricular Internal Diastolic Dimension", [12204])
    },
    "RightVentricularInternalSystolicDimension": {
        "20305-9": ("Right Ventricular Internal Systolic Dimension", [12204])
    },
    "RightVentricularMidCavityDimension4C": {
        "80085-4": ("Right ventricular mid-cavity dimension 4C", [12300])
    },
    "RightVentricularMyocardialPerformanceIndex": {
        "80086-2": ("Right ventricular myocardial performance index", [12300])
    },
    "RightVentricularOutflowTractDiameterAtPulmonicValveRVOTDistal": {
        "80087-0": (
            "Right ventricular outflow tract diameter at pulmonic valve (RVOT-Distal)",
            [12300],
        )
    },
    "RightVentricularOutflowTractDiameterAtSubvalvularLevelRVOTProximal": {
        "80088-8": (
            "Right ventricular outflow tract diameter at subvalvular level (RVOT-Proximal)",
            [12300],
        )
    },
    "RightVentricularOutflowTractVTI": {
        "80089-6": ("Right ventricular outflow tract VTI", [12300])
    },
    "RightVentricularPreEjectionPeriod": {
        "80090-4": ("Right ventricular pre-ejection period", [12300])
    },
    "SPECTBrain": {"39632-5": ("SPECT brain", [100])},
    "SecondQuadrantDiameter": {"11626-9": ("Second Quadrant Diameter", [12008])},
    "SpineLength": {"33071-2": ("Spine Length", [12009])},
    "SpineLengthTokyo1989": {"33127-2": ("Spine Length, Tokyo, 1989", [12012, 12013])},
    "StenosisPeakGradient": {
        "59106-5": (
            "Stenosis Peak Gradient",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "StenosisPeakVelocity": {
        "59107-3": (
            "Stenosis Peak Velocity",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "Summary": {"55112-7": ("Summary", [7001, 7002])},
    "SystolicToDiastolicVelocityRatio": {
        "12144-2": (
            "Systolic to Diastolic Velocity Ratio",
            [
                12119,
                12121,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "TADEriksen1985": {"33128-0": ("TAD, Eriksen 1985", [12012, 12013])},
    "TADHansmann1979": {"33129-8": ("TAD Hansmann, 1979", [12012, 12013])},
    "TADTokyo1986": {"33130-6": ("TAD, Tokyo 1986", [12012, 12013])},
    "TCDByGAGoldstein1987": {"33181-9": ("TCD by GA Goldstein 1987", [12012, 12015])},
    "TCDChitty1994": {"33132-2": ("TCD, Chitty 1994", [12012, 12013])},
    "TCDGoldstein1987": {"33133-0": ("TCD, Goldstein 1987", [12012, 12013])},
    "TCDHill1990": {"33134-8": ("TCD, Hill 1990", [12012, 12013])},
    "TCDNimrod1986": {"33135-5": ("TCD, Nimrod 1986", [12012, 12013])},
    "ThcChitkara1987": {"33131-4": ("ThC, Chitkara 1987", [12012, 12013])},
    "Thickness": {"59089-3": ("Thickness", [12304])},
    "ThirdQuadrantDiameter": {"11625-1": ("Third Quadrant Diameter", [12008])},
    "ThoracicAortaCoarctationSystolicPeakInstantaneousGradient": {
        "17995-2": (
            "Thoracic Aorta Coarctation Systolic Peak Instantaneous Gradient",
            [12212],
        )
    },
    "ThoracicAortaCoarctationSystolicPeakVelocity": {
        "29460-3": ("Thoracic Aorta Coarctation Systolic Peak Velocity", [12212])
    },
    "ThoracicArea": {"33068-8": ("Thoracic Area", [12005, 12279])},
    "ThoracicCircumference": {"11988-3": ("Thoracic Circumference", [12005, 12279])},
    "TibiaJeanty1984": {"11941-2": ("Tibia, Jeanty 1984", [12012, 12013])},
    "TibiaLength": {"11968-5": ("Tibia length", [12006])},
    "TibialBrachialIndex": {"8581-1": ("Tibial/brachial index", [3620])},
    "TimeAveragedMeanVelocity": {
        "20352-1": (
            "Time averaged mean velocity",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "TimeAveragedPeakVelocity": {
        "11692-1": (
            "Time averaged peak velocity",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "TimeFromQWaveToPulmonicValveCloses": {
        "20295-2": (
            "Time from Q wave to Pulmonic Valve Closes",
            [12209, 12262, 12263, 12271, 12272],
        )
    },
    "TimeFromQWaveToTricuspidValveOpens": {
        "20296-0": ("Time from Q wave to Tricuspid Valve Opens", [12208, 12267, 12268])
    },
    "TimeToLeftVentricleETissueVelocity": {
        "59096-8": ("Time to Left Ventricle E Tissue Velocity", [12257, 12259, 12271])
    },
    "TimeToLeftVentricleSTissueVelocity": {
        "59095-0": ("Time to Left Ventricle S Tissue Velocity", [12257, 12259, 12271])
    },
    "TimeToPeakByUS": {
        "59121-4": (
            "Time to Peak by US",
            [
                12254,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12266,
                12267,
                12268,
                12270,
                12271,
                12272,
                12274,
            ],
        )
    },
    "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleAWave": {
        "59125-5": (
            "Tissue Velocity Time Integral (VTI) for the area under Left Ventricle A wave",
            [12257, 12259, 12271],
        )
    },
    "TissueVelocityTimeIntegralVTIForTheAreaUnderLeftVentricleEWave": {
        "59124-8": (
            "Tissue Velocity Time Integral (VTI) for the area under Left Ventricle E wave",
            [12257, 12259, 12271],
        )
    },
    "TransverseAbdominalDiameter": {
        "11862-0": ("Transverse Abdominal Diameter", [12005, 12279])
    },
    "TransverseCerebellarDiameter": {
        "11863-8": ("Transverse Cerebellar Diameter", [12005, 12007, 12279])
    },
    "TransverseThoracicDiameter": {
        "11864-6": ("Transverse Thoracic Diameter", [12005, 12279])
    },
    "TransverseThoracicDiameterHansmann1985": {
        "33136-3": ("Transverse Thoracic Diameter, Hansmann 1985", [12012, 12013])
    },
    "TransverseThoracicDiameterLessoway1998": {
        "33137-1": ("Transverse Thoracic Diameter, Lessoway 1998", [12012, 12013])
    },
    "TricuspidAnnularPlaneSystolicExcursionTAPSE": {
        "77903-3": ("Tricuspid Annular Plane Systolic Excursion (TAPSE)", [12300])
    },
    "TricuspidAnnulusDiameter": {"80091-2": ("Tricuspid annulus diameter", [12300])},
    "TricuspidRegurgitationDpDt": {
        "18034-9": ("Tricuspid Regurgitation dP/dt", [12208])
    },
    "TricuspidRegurgitationPISARadius": {
        "79932-0": ("Tricuspid regurgitation PISA radius", [12300])
    },
    "TricuspidRegurgitationPeakGradient": {
        "79937-9": ("Tricuspid regurgitation peak gradient", [12300])
    },
    "TricuspidRegurgitationVenaContractaWidth": {
        "79933-8": ("Tricuspid regurgitation vena contracta width", [12300])
    },
    "TricuspidRegurgitationVmax": {
        "79921-3": ("Tricuspid regurgitation Vmax", [12300])
    },
    "TricuspidValveAPrimeVmax": {"79922-1": ("Tricuspid valve a-prime Vmax", [12300])},
    "TricuspidValveAWavePeakVelocity": {
        "18030-7": ("Tricuspid Valve A Wave Peak Velocity", [12208])
    },
    "TricuspidValveAWaveVmax": {"79923-9": ("Tricuspid valve A-wave Vmax", [12300])},
    "TricuspidValveClosureToOpeningTime": {
        "79930-4": ("Tricuspid valve closure to opening time", [12300])
    },
    "TricuspidValveDecelerationTime": {
        "79931-2": ("Tricuspid valve deceleration time", [12300])
    },
    "TricuspidValveDiastolicVTI": {
        "18175-0": ("Tricuspid valve diastolic VTI", [12300])
    },
    "TricuspidValveEPrimeVmax": {"79924-7": ("Tricuspid valve e-prime Vmax", [12300])},
    "TricuspidValveEToARatio": {"18039-8": ("Tricuspid Valve E to A Ratio", [12208])},
    "TricuspidValveEWavePeakVelocity": {
        "18031-5": ("Tricuspid Valve E Wave Peak Velocity", [12208])
    },
    "TricuspidValveEWaveVmax": {"79925-4": ("Tricuspid valve E-wave Vmax", [12300])},
    "TricuspidValveMeanGradient": {
        "79938-7": ("Tricuspid valve mean gradient", [12300])
    },
    "TricuspidValvePeakGradient": {
        "79939-5": ("Tricuspid valve peak gradient", [12300])
    },
    "TricuspidValvePressureHalfTime": {
        "18032-3": ("Tricuspid valve pressure half-time", [12300])
    },
    "TricuspidValveSPrimeVmax": {"79926-2": ("Tricuspid valve s-prime Vmax", [12300])},
    "TricuspidValveVmax": {"79927-0": ("Tricuspid valve Vmax", [12300])},
    "USArteryReport": {"46375-2": ("US Artery Report", [7000, 12100])},
    "USBreastReport": {"24601-7": ("US Breast Report", [7000, 12320])},
    "USCarotidArteriesReport": {
        "24616-5": ("US Carotid Arteries Report", [7000, 12100])
    },
    "USDopplerAbdominalVesselsReport": {
        "24534-0": ("US Doppler Abdominal Vessels Report", [7000, 12100])
    },
    "USDopplerExtremityVesselsReport": {
        "43771-5": ("US Doppler Extremity Vessels Report", [7000, 12100])
    },
    "USDopplerHeadVesselsReport": {
        "24733-8": ("US Doppler Head Vessels Report", [7000, 12100])
    },
    "USDopplerLowerExtremityVesselsReport": {
        "44174-1": ("US Doppler Lower Extremity Vessels Report", [7000, 12100])
    },
    "USDopplerUpperExtremityVesselsReport": {
        "39448-6": ("US Doppler Upper Extremity Vessels Report", [7000, 12100])
    },
    "USDopplerVesselsReport": {"39445-2": ("US Doppler Vessels Report", [7000, 12100])},
    "USKidneyReport": {"38036-0": ("US Kidney Report", [7000, 12320])},
    "USLiverReport": {"28614-6": ("US Liver Report", [7000, 12320])},
    "USPancreasReport": {"24859-1": ("US Pancreas Report", [7000, 12320])},
    "USPelvis": {"24869-0": ("US Pelvis", [12024])},
    "USProstateTransrectalReport": {
        "24884-9": ("US Prostate (transrectal) Report", [7000, 12320])
    },
    "USScrotumAndTesticleReport": {
        "25002-7": ("US Scrotum and Testicle Report", [7000, 12320])
    },
    "USShoulderReport": {"24907-8": ("US Shoulder Report", [7000, 12320])},
    "USSpleenReport": {"24990-4": ("US Spleen Report", [7000, 12320])},
    "USTendonReport": {"39453-6": ("US Tendon Report", [7000, 12320])},
    "USThyroidReport": {"25010-0": ("US Thyroid Report", [7000, 12320])},
    "USVeinReport": {"39036-9": ("US Vein Report", [7000, 12100])},
    "UlnaJeanty1984": {"11944-6": ("Ulna, Jeanty 1984", [12012, 12013])},
    "UlnaLength": {"11969-3": ("Ulna length", [12006])},
    "UlnaMerz1987": {"11945-3": ("Ulna, Merz 1987", [12012, 12013])},
    "UltrasoundObstetricAndGynReport": {
        "11525-3": ("Ultrasound Obstetric and Gyn Report", [7000])
    },
    "UltrasoundReport": {"25061-3": ("Ultrasound Report", [7000, 12320])},
    "VelocityOfFlowPropagation": {
        "59115-6": (
            "Velocity of Flow Propagation",
            [
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "VelocityRatio": {"33867-3": ("Velocity ratio", [12119, 12121])},
    "VelocityTimeIntegral": {
        "20354-7": (
            "Velocity Time Integral",
            [
                12119,
                12120,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
                12304,
            ],
        )
    },
    "VenousBloodBaseExcess": {"1927-3": ("Venous Blood Base Excess", [3527])},
    "VenousBloodCarbonDioxideContent": {
        "2027-1": ("Venous Blood Carbon dioxide content", [3525])
    },
    "VenousBloodCarbonDioxidePartialPressure": {
        "2021-4": ("Venous Blood Carbon dioxide partial pressure", [3524])
    },
    "VenousBloodPh": {"2746-6": ("Venous Blood pH", [3528])},
    "VenousContentFcv": {"19220-3": ("Venous Content (FCv)", [3529])},
    "VenousOxygenContent": {"19220-3": ("Venous Oxygen content", [3525])},
    "VenousOxygenPartialPressure": {
        "2705-2": ("Venous Oxygen partial pressure", [3524])
    },
    "VenousOxygenPartialPressureAt50PercentSaturation": {
        "19216-1": ("Venous Oxygen partial pressure at 50% saturation", [3524])
    },
    "VenousOxygenSaturation": {"2711-0": ("Venous Oxygen saturation", [3526])},
    "VolumeFlow": {
        "33878-0": (
            "Volume flow",
            [
                12119,
                12122,
                12200,
                12204,
                12207,
                12208,
                12209,
                12211,
                12218,
                12222,
                12257,
                12258,
                12259,
                12260,
                12261,
                12262,
                12263,
                12264,
                12265,
                12266,
                12267,
                12268,
                12269,
                12270,
                12271,
                12272,
                12273,
                12274,
                12275,
                12276,
            ],
        )
    },
    "VolumeFlowRate": {"33878-0": ("Volume Flow Rate", [12304])},
    "WholeBodyCT": {"46305-9": ("Whole body CT", [646])},
    "WholeBodyCt": {"46305-9": ("whole body ct", [6050, 6083])},
    "WholeBodyPtWRncIv": {"44139-4": ("whole body pt w rnc iv", [6050, 6083])},
    "WidthOfHemisphere": {"12170-7": ("Width of Hemisphere", [12007])},
    "XRUnspecifiedBodyRegion": {"43468-8": ("XR unspecified body region", [100])},
    "XRayReport": {"43468-8": ("X-Ray Report", [7000])},
    "YolkSacLength": {"11816-6": ("Yolk Sac length", [12009])},
}

concepts["IBSI"] = {
    "AngularSecondMomentOfGLCM": {
        "8ZQL": ("Angular Second Moment of GLCM", [218, 7467, 7468])
    },
    "AreaDensityInApproximateEnclosingEllipsoid": {
        "RDD2": ("Area Density in Approximate Enclosing Ellipsoid", [218, 7477])
    },
    "AreaDensityInConvexHull": {"7T7F": ("Area Density in Convex Hull", [218, 7477])},
    "AreaDensityInFrameOfReferenceAxisAlignedBoundingBox": {
        "R59B": (
            "Area Density in Frame of Reference Axis Aligned Bounding Box",
            [218, 7477],
        )
    },
    "AreaDensityInMinimumVolumeEnclosingEllipsoid": {
        "BRI8": ("Area Density in Minimum Volume Enclosing Ellipsoid", [218, 7477])
    },
    "AreaDensityInOrientedMinimumBoundingBox": {
        "IQYR": ("Area Density in Oriented Minimum Bounding Box", [218, 7477])
    },
    "Asphericity": {"25C7": ("Asphericity", [218, 7477])},
    "AutocorrelationOfGLCM": {"QWB0": ("Autocorrelation of GLCM", [218, 7467, 7468])},
    "BusynessOfNGTDM": {"NQ30": ("Busyness of NGTDM", [218, 7468, 7500])},
    "CentreOfMassShift": {"KLMA": ("Centre of Mass Shift", [218, 7477])},
    "ClusterProminenceOfGLCM": {
        "AE86": ("Cluster Prominence of GLCM", [218, 7467, 7468])
    },
    "ClusterShadeOfGLCM": {"7NFM": ("Cluster Shade of GLCM", [218, 7467, 7468])},
    "ClusterTendencyOfGLCM": {"DG8W": ("Cluster Tendency of GLCM", [218, 7467, 7468])},
    "CoarsenessOfNGTDM": {"QCDE": ("Coarseness of NGTDM", [218, 7468, 7500])},
    "Compactness1": {"SKGS": ("Compactness 1", [218, 7477])},
    "Compactness2": {"BQWJ": ("Compactness 2", [218, 7477])},
    "ComplexityOfNGTDM": {"HDEZ": ("Complexity of NGTDM", [218, 7468, 7500])},
    "ContrastOfGLCM": {"ACUI": ("Contrast of GLCM", [218, 7467, 7468])},
    "ContrastOfNGTDM": {"65HE": ("Contrast of NGTDM", [218, 7468, 7500])},
    "CorrelationOfGLCM": {"NI2N": ("Correlation of GLCM", [218, 7467, 7468])},
    "DependenceCountEnergy": {"CAS9": ("Dependence count energy", [218, 7468, 7501])},
    "DependenceCountEntropy": {"FCBV": ("Dependence count entropy", [218, 7468, 7501])},
    "DependenceCountNonUniformity": {
        "Z87G": ("Dependence count non-uniformity", [218, 7468, 7501])
    },
    "DependenceCountNonUniformityNormalized": {
        "OKJI": ("Dependence count non-uniformity normalized", [218, 7468, 7501])
    },
    "DependenceCountPercentage": {
        "6XV8": ("Dependence count percentage", [218, 7468, 7501])
    },
    "DependenceCountVariance": {
        "DNX2": ("Dependence count variance", [218, 7468, 7501])
    },
    "DifferenceAverageOfGLCM": {
        "TF7R": ("Difference Average of GLCM", [218, 7467, 7468])
    },
    "DifferenceEntropyOfGLCM": {
        "NTRS": ("Difference Entropy of GLCM", [218, 7467, 7468])
    },
    "DifferenceVarianceOfGLCM": {
        "D3YU": ("Difference Variance of GLCM", [218, 7467, 7468])
    },
    "DissimilarityOfGLCM": {"8S9J": ("Dissimilarity of GLCM", [218, 7467, 7468])},
    "DistanceZonePercentage": {"VIWW": ("Distance Zone Percentage", [218, 7468, 7479])},
    "Elongation": {"Q3CK": ("Elongation", [218, 7477])},
    "FirstMeasureOfInformationCorrelationOfGLCM": {
        "R8DG": ("First Measure of Information Correlation of GLCM", [218, 7467, 7468])
    },
    "Flatness": {"N17B": ("Flatness", [218, 7477])},
    "GearyCMeasure": {"NPT7": ("Geary's C Measure", [218, 7477])},
    "GrayLevelNonuniformityInRuns": {
        "R5YN": ("Gray Level Nonuniformity in Runs", [218, 7468, 7475])
    },
    "GrayLevelNonuniformityOfSizeZoneCounts": {
        "JNSA": ("Gray Level Nonuniformity of Size Zone Counts", [218, 7468, 7476])
    },
    "GrayLevelVarianceInRuns": {
        "8CE5": ("Gray Level Variance in Runs", [218, 7468, 7475])
    },
    "GrayLevelVarianceInSizeZones": {
        "BYLV": ("Gray Level Variance in Size Zones", [218, 7468, 7476])
    },
    "GreyLevelNonUniformityOfDistanceZoneCounts": {
        "VFT7": ("Grey Level Non-uniformity of Distance Zone Counts", [218, 7468, 7479])
    },
    "GreyLevelNonUniformityOfNGLDM": {
        "FP8K": ("Grey level non-uniformity of NGLDM", [218, 7468, 7501])
    },
    "GreyLevelVarianceInDistanceZones": {
        "QK93": ("Grey Level Variance in Distance Zones", [218, 7468, 7479])
    },
    "GreyLevelVarianceOfNGLDM": {
        "1PFV": ("Grey level variance of NGLDM", [218, 7468, 7501])
    },
    "HighDependenceEmphasis": {"IMOQ": ("High dependence emphasis", [218, 7468, 7501])},
    "HighDependenceHighGreyLevelEmphasis": {
        "9QMG": ("High dependence high grey level emphasis", [218, 7468, 7501])
    },
    "HighDependenceLowGreyLevelEmphasis": {
        "NBZI": ("High dependence low grey level emphasis", [218, 7468, 7501])
    },
    "HighGrayLevelRunEmphasis": {
        "G3QZ": ("High Gray Level Run Emphasis", [218, 7468, 7475])
    },
    "HighGrayLevelZoneEmphasis": {
        "5GN9": ("High Gray Level Zone Emphasis", [218, 7468, 7476])
    },
    "HighGreyLevelCountEmphasis": {
        "OAE7": ("High grey level count emphasis", [218, 7468, 7501])
    },
    "HighGreyLevelZoneEmphasis": {
        "K26C": ("High Grey Level Zone Emphasis", [218, 7468, 7479])
    },
    "IntegratedIntensity": {"99N0": ("Integrated Intensity", [218, 7477])},
    "IntensityHistogram10thPercentile": {
        "GPMT": ("Intensity Histogram 10th Percentile", [218, 7468, 7478])
    },
    "IntensityHistogram90thPercentile": {
        "OZ0C": ("Intensity Histogram 90th Percentile", [218, 7468, 7478])
    },
    "IntensityHistogramCoefficientOfVariation": {
        "CWYJ": ("Intensity Histogram Coefficient of Variation", [218, 7468, 7478])
    },
    "IntensityHistogramEntropy": {
        "TLU2": ("Intensity Histogram Entropy", [218, 7468, 7478])
    },
    "IntensityHistogramInterquartileRange": {
        "WR0O": ("Intensity Histogram Interquartile Range", [218, 7468, 7478])
    },
    "IntensityHistogramKurtosis": {
        "C3I7": ("Intensity Histogram Kurtosis", [218, 7468, 7478])
    },
    "IntensityHistogramMaximumGradient": {
        "12CE": ("Intensity Histogram Maximum Gradient", [218, 7468, 7478])
    },
    "IntensityHistogramMaximumGradientGrayLevel": {
        "8E6O": ("Intensity Histogram Maximum Gradient Gray Level", [218, 7468, 7478])
    },
    "IntensityHistogramMaximumGrayLevel": {
        "3NCY": ("Intensity Histogram Maximum Gray Level", [218, 7468, 7478])
    },
    "IntensityHistogramMean": {"X6K6": ("Intensity Histogram Mean", [218, 7468, 7478])},
    "IntensityHistogramMeanAbsoluteDeviation": {
        "D2ZX": ("Intensity Histogram Mean Absolute Deviation", [218, 7468, 7478])
    },
    "IntensityHistogramMedian": {
        "WIFQ": ("Intensity Histogram Median", [218, 7468, 7478])
    },
    "IntensityHistogramMedianAbsoluteDeviation": {
        "4RNL": ("Intensity Histogram Median Absolute Deviation", [218, 7468, 7478])
    },
    "IntensityHistogramMinimumGradient": {
        "VQB3": ("Intensity Histogram Minimum Gradient", [218, 7468, 7478])
    },
    "IntensityHistogramMinimumGradientGrayLevel": {
        "RHQZ": ("Intensity Histogram Minimum Gradient Gray Level", [218, 7468, 7478])
    },
    "IntensityHistogramMinimumGrayLevel": {
        "1PR8": ("Intensity Histogram Minimum Gray Level", [218, 7468, 7478])
    },
    "IntensityHistogramMode": {"AMMC": ("Intensity Histogram Mode", [218, 7468, 7478])},
    "IntensityHistogramQuartileCoefficientOfDispersion": {
        "SLWD": (
            "Intensity Histogram Quartile Coefficient of Dispersion",
            [218, 7468, 7478],
        )
    },
    "IntensityHistogramRange": {
        "5Z3W": ("Intensity Histogram Range", [218, 7468, 7478])
    },
    "IntensityHistogramRobustMeanAbsoluteDeviation": {
        "WRZB": (
            "Intensity Histogram Robust Mean Absolute Deviation",
            [218, 7468, 7478],
        )
    },
    "IntensityHistogramSkewness": {
        "88K1": ("Intensity Histogram Skewness", [218, 7468, 7478])
    },
    "IntensityHistogramUniformity": {
        "BJ5W": ("Intensity Histogram Uniformity", [218, 7468, 7478])
    },
    "IntensityHistogramVariance": {
        "CH89": ("Intensity Histogram Variance", [218, 7468, 7478])
    },
    "InverseDifferenceMomentOfGLCM": {
        "WF0Z": ("Inverse Difference Moment of GLCM", [218, 7467, 7468])
    },
    "InverseDifferenceOfGLCM": {
        "IB1Z": ("Inverse Difference of GLCM", [218, 7467, 7468])
    },
    "InverseVarianceOfGLCM": {"E8JP": ("Inverse Variance of GLCM", [218, 7467, 7468])},
    "JointAverageOfGLCM": {"60VM": ("Joint Average of GLCM", [218, 7467, 7468])},
    "JointEntropyOfGLCM": {"TU9B": ("Joint Entropy of GLCM", [218, 7467, 7468])},
    "JointMaximumOfGLCM": {"GYBY": ("Joint Maximum of GLCM", [218, 7467, 7468])},
    "JointVarianceOfGLCM": {"UR99": ("Joint Variance of GLCM", [218, 7467, 7468])},
    "LargeDistanceEmphasis": {"MB4I": ("Large Distance Emphasis", [218, 7468, 7479])},
    "LargeDistanceHighGreyLevelEmphasis": {
        "KLTH": ("Large Distance High Grey Level Emphasis", [218, 7468, 7479])
    },
    "LargeDistanceLowGreyLevelEmphasis": {
        "A7WM": ("Large Distance Low Grey Level Emphasis", [218, 7468, 7479])
    },
    "LargeZoneEmphasis": {"48P8": ("Large Zone Emphasis", [218, 7468, 7476])},
    "LargeZoneHighGrayLevelEmphasis": {
        "J17V": ("Large Zone High Gray Level Emphasis", [218, 7468, 7476])
    },
    "LargeZoneLowGrayLevelEmphasis": {
        "YH51": ("Large Zone Low Gray Level Emphasis", [218, 7468, 7476])
    },
    "LeastAxisIn3DLength": {
        "7J51": ("Least Axis in 3D Length", [218, 6165, 7469, 7470, 7477])
    },
    "LongRunHighGrayLevelEmphasis": {
        "3KUM": ("Long Run High Gray Level Emphasis", [218, 7468, 7475])
    },
    "LongRunLowGrayLevelEmphasis": {
        "IVPO": ("Long Run Low Gray Level Emphasis", [218, 7468, 7475])
    },
    "LongRunsEmphasis": {"W4KF": ("Long Runs Emphasis", [218, 7468, 7475])},
    "LowDependenceEmphasis": {"SODN": ("Low dependence emphasis", [218, 7468, 7501])},
    "LowDependenceHighGreyLevelEmphasis": {
        "JA6D": ("Low dependence high grey level emphasis", [218, 7468, 7501])
    },
    "LowDependenceLowGreyLevelEmphasis": {
        "EQ3F": ("Low dependence low grey level emphasis", [218, 7468, 7501])
    },
    "LowGrayLevelRunEmphasis": {
        "V3SW": ("Low Gray Level Run Emphasis", [218, 7468, 7475])
    },
    "LowGrayLevelZoneEmphasis": {
        "XMSY": ("Low Gray Level Zone Emphasis", [218, 7468, 7476])
    },
    "LowGreyLevelCountEmphasis": {
        "TL9H": ("Low grey level count emphasis", [218, 7468, 7501])
    },
    "LowGreyLevelZoneEmphasis": {
        "S1RA": ("Low Grey Level Zone Emphasis", [218, 7468, 7479])
    },
    "MajorAxisIn3DLength": {
        "TDIC": ("Major Axis in 3D Length", [218, 6165, 7469, 7470, 7477])
    },
    "Maximum3DDiameterOfAMesh": {
        "L0JK": ("Maximum 3D Diameter of a Mesh", [218, 6165, 7469, 7470, 7477])
    },
    "MinorAxisIn3DLength": {
        "P9VJ": ("Minor Axis in 3D Length", [218, 6165, 7469, 7470, 7477])
    },
    "MoranIIndex": {"N365": ("Moran's I Index", [218, 7477])},
    "NormalizedGrayLevelNonuniformityInRuns": {
        "OVBL": ("Normalized Gray Level Nonuniformity in Runs", [218, 7468, 7475])
    },
    "NormalizedGrayLevelNonuniformityOfSizeZoneCounts": {
        "Y1RO": (
            "Normalized Gray Level Nonuniformity of Size Zone Counts",
            [218, 7468, 7476],
        )
    },
    "NormalizedGreyLevelNonUniformityOfDistanceZoneCounts": {
        "7HP3": (
            "Normalized Grey Level Non-uniformity of Distance Zone Counts",
            [218, 7468, 7479],
        )
    },
    "NormalizedGreyLevelNonUniformityOfNGLDM": {
        "5SPA": ("Normalized grey level non-uniformity of NGLDM", [218, 7468, 7501])
    },
    "NormalizedInverseDifferenceMomentOfGLCM": {
        "1QCO": ("Normalized Inverse Difference Moment of GLCM", [218, 7467, 7468])
    },
    "NormalizedInverseDifferenceOfGLCM": {
        "NDRX": ("Normalized Inverse Difference of GLCM", [218, 7467, 7468])
    },
    "NormalizedRunLengthNonuniformity": {
        "IC23": ("Normalized Run Length Nonuniformity", [218, 7468, 7475])
    },
    "NormalizedZoneDistanceNonUniformity": {
        "IATH": ("Normalized Zone Distance Non-uniformity", [218, 7468, 7479])
    },
    "NormalizedZoneSizeNonuniformity": {
        "VB3A": ("Normalized Zone Size Nonuniformity", [218, 7468, 7476])
    },
    "RunEntropy": {"HJ9O": ("Run Entropy", [218, 7468, 7475])},
    "RunLengthNonuniformity": {"W92Y": ("Run Length Nonuniformity", [218, 7468, 7475])},
    "RunLengthVariance": {"SXLW": ("Run Length Variance", [218, 7468, 7475])},
    "RunPercentage": {"9ZK5": ("Run Percentage", [218, 7468, 7475])},
    "SecondMeasureOfInformationCorrelationOfGLCM": {
        "JN9H": ("Second Measure of Information Correlation of GLCM", [218, 7467, 7468])
    },
    "ShortRunHighGrayLevelEmphasis": {
        "GD3A": ("Short Run High Gray Level Emphasis", [218, 7468, 7475])
    },
    "ShortRunLowGrayLevelEmphasis": {
        "HTZT": ("Short Run Low Gray Level Emphasis", [218, 7468, 7475])
    },
    "ShortRunsEmphasis": {"22OV": ("Short Runs Emphasis", [218, 7468, 7475])},
    "SizeZonePercentage": {"P30P": ("Size Zone Percentage", [218, 7468, 7476])},
    "SmallDistanceEmphasis": {"0GBI": ("Small Distance Emphasis", [218, 7468, 7479])},
    "SmallDistanceHighGreyLevelEmphasis": {
        "DKNJ": ("Small Distance High Grey Level Emphasis", [218, 7468, 7479])
    },
    "SmallDistanceLowGreyLevelEmphasis": {
        "RUVG": ("Small Distance Low Grey Level Emphasis", [218, 7468, 7479])
    },
    "SmallZoneEmphasis": {"5QRC": ("Small Zone Emphasis", [218, 7468, 7476])},
    "SmallZoneHighGrayLevelEmphasis": {
        "HW1V": ("Small Zone High Gray Level Emphasis", [218, 7468, 7476])
    },
    "SmallZoneLowGrayLevelEmphasis": {
        "5RAI": ("Small Zone Low Gray Level Emphasis", [218, 7468, 7476])
    },
    "SphericalDisproportion": {"KRCK": ("Spherical Disproportion", [218, 7477])},
    "Sphericity": {"QCFX": ("Sphericity", [218, 7477])},
    "StrengthOfNGTDM": {"1X9X": ("Strength of NGTDM", [218, 7468, 7500])},
    "SumAverageOfGLCM": {"ZGXS": ("Sum Average of GLCM", [218, 7467, 7468])},
    "SumEntropyOfGLCM": {"P6QZ": ("Sum Entropy of GLCM", [218, 7467, 7468])},
    "SumVarianceOfGLCM": {"OEEB": ("Sum Variance of GLCM", [218, 7467, 7468])},
    "SurfaceAreaOfMesh": {"C0JK": ("Surface Area of Mesh", [218, 7469, 7471, 7477])},
    "SurfaceToVolumeRatio": {"2PR5": ("Surface to Volume Ratio", [218, 7477])},
    "VolumeDensityInApproximateEnclosingEllipsoid": {
        "6BDE": ("Volume Density in Approximate Enclosing Ellipsoid", [218, 7477])
    },
    "VolumeDensityInConvexHull": {
        "R3ER": ("Volume Density in Convex Hull", [218, 7477])
    },
    "VolumeDensityInFrameOfReferenceAxisAlignedBoundingBox": {
        "PBX1": (
            "Volume Density in Frame of Reference Axis Aligned Bounding Box",
            [218, 7477],
        )
    },
    "VolumeDensityInMinimumVolumeEnclosingEllipsoid": {
        "SWZ1": ("Volume Density in Minimum Volume Enclosing Ellipsoid", [218, 7477])
    },
    "VolumeDensityInOrientedMinimumBoundingBox": {
        "ZH1A": ("Volume Density in Oriented Minimum Bounding Box", [218, 7477])
    },
    "VolumeFromVoxelSummation": {
        "YEKZ": ("Volume from Voxel Summation", [218, 7469, 7472, 7477])
    },
    "VolumeOfMesh": {"RNU0": ("Volume of Mesh", [218, 7469, 7472, 7477])},
    "ZoneDistanceEntropy": {"GBDU": ("Zone Distance Entropy", [218, 7468, 7479])},
    "ZoneDistanceNonUniformity": {
        "V294": ("Zone Distance Non-uniformity", [218, 7468, 7479])
    },
    "ZoneDistanceVariance": {"7WT1": ("Zone Distance Variance", [218, 7468, 7479])},
    "ZoneSizeEntropy": {"GU8N": ("Zone Size Entropy", [218, 7468, 7476])},
    "ZoneSizeNonuniformity": {"4JP3": ("Zone Size Nonuniformity", [218, 7468, 7476])},
    "ZoneSizeVariance": {"3NSA": ("Zone Size Variance", [218, 7468, 7476])},
}

concepts["UMLS"] = {
    "Alphaxalone": {"C0051482": ("Alphaxalone", [623, 625])},
    "AnesthesiaRecoveryPeriod": {"C0002908": ("Anesthesia recovery period", [634])},
    "Antihypoglycemic": {"C1579431": ("Antihypoglycemic", [621, 622])},
    "Baseline": {"C1442488": ("Baseline", [6146])},
    "CoefficientOfVariance": {"C0681921": ("Coefficient of Variance", [7464])},
    "CommercialProduct": {"C1547887": ("Commercial product", [608])},
    "ConsultingPhysician": {
        "C1441532": ("Consulting Physician", [7450, 7452, 9536, 9555, 9562])
    },
    "DistalPhalanx": {"C3669027": ("Distal phalanx", [7482, 7483, 8134])},
    "EdotreotideGa68": {"C2713594": ("Edotreotide Ga^68^", [4021])},
    "EthidiumBromide": {"C0019873": ("Ethidium Bromide", [638, 643])},
    "Fluanisone": {"C0060473": ("Fluanisone", [623, 625])},
    "FluoroetanidazoleF18": {"C1541539": ("Fluoroetanidazole F^18^", [4021])},
    "FluoropropylDihydrotetrabenazineF18": {
        "C2934038": ("Fluoropropyl-dihydrotetrabenazine F^18^", [4021])
    },
    "H2Antagonist": {"C0019593": ("H2 antagonist", [621, 622])},
    "HeatLamp": {"C0181514": ("Heat lamp", [635])},
    "ISO1F18": {"C2981788": ("ISO-1 F^18^", [4021])},
    "InfantOfMotherWithGestationalDiabetes": {
        "C0456029": ("Infant of mother with gestational diabetes", [3756])
    },
    "Intern": {"C1144859": ("Intern", [7450, 7452])},
    "InverseRatioVentilation": {"C2223982": ("Inverse ratio ventilation", [619, 620])},
    "KevlarAramidFiber": {"C0064329": ("Kevlar Aramid Fiber", [10067])},
    "MK6240F18": {"C4506764": ("MK-6240 F^18^", [4021])},
    "Mastectomy": {"C0024881": ("Mastectomy", [6058, 6060])},
    "MedicalPhysicist": {
        "C1708969": ("Medical Physicist", [7450, 7452, 9536, 9555, 9562])
    },
    "Metomidate": {"C0025856": ("Metomidate", [623, 625])},
    "MiddleEasternOrNorthAfrican": {
        "C5690844": ("Middle Eastern or North African", [6099])
    },
    "MobileSkinLesion": {"C2071496": ("Mobile skin lesion", [4407])},
    "Nadir": {"C1708760": ("Nadir", [6146])},
    "NarcoticAnalgesic": {"C0027409": ("Narcotic analgesic", [621, 622])},
    "Neuroradiology": {"C2183225": ("Neuroradiology", [7030])},
    "PK1119511C": {"C1609883": ("PK11195 ^11^C", [4021])},
    "Pretreatment": {"C3539075": ("Pretreatment", [6146])},
    "RadiationPhysicist": {
        "C2985483": ("Radiation Physicist", [7450, 7452, 9536, 9555, 9562])
    },
    "Referring": {"C1709880": ("Referring", [7453])},
    "RestrictedDiet": {"C0425422": ("Restricted diet", [609])},
    "RoomAir": {"C3846005": ("Room air", [623, 629])},
    "RootMeanSquare": {"C2347976": ("Root Mean Square", [7464])},
    "Sonographer": {"C1954848": ("Sonographer", [7450, 7452])},
    "THK5317F18": {"C4550127": ("THK5317 F^18^", [4021])},
    "THK5351F18": {"C4279748": ("THK5351 F^18^", [4021])},
    "Thermography": {"C0039810": ("Thermography", [636])},
    "TransgenicAnimal": {"C0003069": ("Transgenic animal", [602])},
    "UCBJC11": {"C4506788": ("UCB-J C^11^", [4021])},
    "UnknownPrimaryNeoplasiaSite": {
        "C0221297": ("unknown primary neoplasia site", [7601, 8134])
    },
    "Unscheduled": {"C1699701": ("Unscheduled", [6146])},
    "Variance": {"C1711260": ("Variance", [7464])},
}

concepts["PUBCHEM_CID"] = {
    "_6Hydroxydopamine": {"4624": ("6-hydroxydopamine", [638, 643])}
}

concepts["FMA"] = {
    "ArcuateFasciculus": {
        "276650": ("Arcuate Fasciculus", [7151, 7153, 7192, 8134, 9514])
    },
    "Bregma": {"264776": ("Bregma", [647])},
    "Lambda": {"264773": ("Lambda", [647])},
    "LumenOfGallbladder": {
        "17891": ("Lumen of gallbladder", [7151, 7154, 7192, 8134, 9514])
    },
    "MucosaOfDorsumOfOralPartOfTongue": {
        "281534": ("Mucosa of dorsum of oral part of tongue", [4029, 8134])
    },
    "MucosaOfDorsumOfPharyngealPartOfTongue": {
        "281537": ("Mucosa of dorsum of pharyngeal part of tongue", [4029, 8134])
    },
    "MucosaOfOralSegmentOfHardPalate": {
        "289677": ("Mucosa of oral segment of hard palate", [4029, 8134])
    },
    "MucosaOfPalatoglossalArch": {
        "60031": ("Mucosa of palatoglossal arch", [4029, 8134])
    },
    "MucosaOfPharynx": {"55031": ("Mucosa of pharynx", [4029, 8134])},
    "MucosaOfPosteriorWallOfOropharynx": {
        "55060": ("Mucosa of posterior wall of oropharynx", [4029, 8134])
    },
    "MucosaOfUvula": {"60030": ("Mucosa of uvula", [4029, 8134])},
    "Nasion": {"264779": ("Nasion", [1000, 1001, 1002, 8134])},
    "SkinOfBackOfTrunk": {"49943": ("Skin of back of trunk", [4029, 8134])},
    "SkinOfDorsalPartOfFifthToe": {
        "37885": ("Skin of dorsal part of fifth toe", [4029, 8134])
    },
    "SkinOfDorsalPartOfFourthToe": {
        "37882": ("Skin of dorsal part of fourth toe", [4029, 8134])
    },
    "SkinOfDorsalPartOfGreatToe": {
        "37873": ("Skin of dorsal part of great toe", [4029, 8134])
    },
    "SkinOfDorsalPartOfIndexFinger": {
        "38324": ("Skin of dorsal part of index finger", [4029, 8134])
    },
    "SkinOfDorsalPartOfLittleFinger": {
        "38333": ("Skin of dorsal part of little finger", [4029, 8134])
    },
    "SkinOfDorsalPartOfMiddleFinger": {
        "38327": ("Skin of dorsal part of middle finger", [4029, 8134])
    },
    "SkinOfDorsalPartOfRingFinger": {
        "38330": ("Skin of dorsal part of ring finger", [4029, 8134])
    },
    "SkinOfDorsalPartOfSecondToe": {
        "37876": ("Skin of dorsal part of second toe", [4029, 8134])
    },
    "SkinOfDorsalPartOfThirdToe": {
        "37879": ("Skin of dorsal part of third toe", [4029, 8134])
    },
    "SkinOfDorsalPartOfThumb": {
        "38321": ("Skin of dorsal part of thumb", [4029, 8134])
    },
    "SkinOfDorsumOfNose": {"59532": ("Skin of dorsum of nose", [4029, 8134])},
    "SkinOfLowerInnerQuadrantOfBreast": {
        "61427": ("Skin of lower inner quadrant of breast", [4029, 8134])
    },
    "SkinOfLowerOuterQuadrantOfBreast": {
        "61423": ("Skin of lower outer quadrant of breast", [4029, 8134])
    },
    "SkinOfPalmarPartOfIndexFinger": {
        "38344": ("Skin of palmar part of index finger", [4029, 8134])
    },
    "SkinOfPalmarPartOfLittleFinger": {
        "38357": ("Skin of palmar part of little finger", [4029, 8134])
    },
    "SkinOfPalmarPartOfMiddleFinger": {
        "38347": ("Skin of palmar part of middle finger", [4029, 8134])
    },
    "SkinOfPalmarPartOfRingFinger": {
        "38354": ("Skin of palmar part of ring finger", [4029, 8134])
    },
    "SkinOfPalmarPartOfThumb": {
        "38341": ("Skin of palmar part of thumb", [4029, 8134])
    },
    "SkinOfPlantarPartOfFifthToe": {
        "38119": ("Skin of plantar part of fifth toe", [4029, 8134])
    },
    "SkinOfPlantarPartOfFourthToe": {
        "38116": ("Skin of plantar part of fourth toe", [4029, 8134])
    },
    "SkinOfPlantarPartOfGreatToe": {
        "38107": ("Skin of plantar part of great toe", [4029, 8134])
    },
    "SkinOfPlantarPartOfSecondToe": {
        "38110": ("Skin of plantar part of second toe", [4029, 8134])
    },
    "SkinOfPlantarPartOfThirdToe": {
        "38113": ("Skin of plantar part of third toe", [4029, 8134])
    },
    "SkinOfUpperInnerQuadrantOfBreast": {
        "61426": ("Skin of upper inner quadrant of breast", [4029, 8134])
    },
    "SkinOfUpperOuterQuadrantOfLeftBreast": {
        "61439": ("Skin of upper outer quadrant of left breast", [4029, 8134])
    },
    "SkinOfVermilionProperOfLowerLip": {
        "312651": ("Skin of vermilion proper of lower lip", [4029, 8134])
    },
    "SkinOfVermilionProperOfUpperLip": {
        "312647": ("Skin of vermilion proper of upper lip", [4029, 8134])
    },
    "SubcorticalGrayMatter": {
        "223151": ("Subcortical gray matter", [7151, 7153, 7192, 8134, 9514])
    },
    "SubiliacLymphNode": {"323407": ("subiliac lymph node", [7600, 8134])},
    "SupramammaryLymphNode": {"12785": ("supramammary lymph node", [7600, 8134])},
    "TorusOfPharyngotympanicTube": {
        "54993": ("torus of pharyngotympanic tube", [7601, 8134])
    },
    "WallOfGallbladder": {
        "14657": ("Wall of gallbladder", [7151, 7154, 7192, 8134, 9514])
    },
}

concepts["MDC"] = {
    "A1": {"7:1289": ("A1", [3030])},
    "A2": {"7:1290": ("A2", [3030])},
    "AF3": {"7:1217": ("AF3", [3030])},
    "AF4": {"7:1222": ("AF4", [3030])},
    "AF7": {"7:1225": ("AF7", [3030])},
    "AF8": {"7:1230": ("AF8", [3030])},
    "AVDissociation": {"10:9712": ("AV Dissociation", [3415])},
    "AVDissociationWithInterference": {
        "10:9728": ("AV dissociation with interference", [3415])
    },
    "AVJunctionalNodalEscapeRhythm": {
        "10:9552": ("AV junctional (nodal) escape rhythm", [3415])
    },
    "AVJunctionalNodalRhythm": {"10:9536": ("AV junctional (nodal) rhythm", [3415])},
    "AVReciprocatingTachycardia": {"10:9600": ("AV reciprocating tachycardia", [3415])},
    "AberratedAtrialPrematureBeatAshmanBeat": {
        "10:8304": ("Aberrated atrial premature beat (Ashman beat)", [3335])
    },
    "AbnormalBeat": {"10:8224": ("Abnormal beat", [3335])},
    "AcceleratedAVJunctionalNodalRhythm": {
        "10:9568": ("Accelerated AV junctional (nodal) rhythm", [3415])
    },
    "AcceleratedIdioventricularRhythm": {
        "10:9888": ("Accelerated idioventricular rhythm", [3415])
    },
    "Afz": {"7:1004": ("AFz", [3030])},
    "AlphadeltaSleep": {"2:23728": ("Alphadelta Sleep", [3035, 3046])},
    "AmplitudeAt20msIntoSTSegmentPerLead": {
        "2:14848": ("Amplitude at 20 ms into ST segment, per lead", [3688])
    },
    "AmplitudeAt40msIntoSTSegmentPerLead": {
        "2:15104": ("Amplitude at 40 ms into ST segment, per lead", [3688])
    },
    "AmplitudeAt60msIntoSTSegmentPerLead": {
        "2:14336": ("Amplitude at 60 ms into ST segment, per lead", [3688])
    },
    "AmplitudeAt80msIntoSTSegmentPerLead": {
        "2:14592": ("Amplitude at 80 ms into ST segment, per lead", [3688])
    },
    "AntiTachycardiaPacing": {"10:10304": ("Anti-Tachycardia Pacing", [3415])},
    "AntibradycardiaPaceSpike": {"10:4096": ("Antibradycardia pace spike", [3335])},
    "AntitachycardiaPaceSpike": {"10:6144": ("Antitachycardia pace spike", [3335])},
    "Apnea": {"3:3072": ("Apnea", [3038])},
    "Apnea15Sec": {"3:3284": ("Apnea 15 sec", [3038])},
    "Apnea30Sec": {"3:3292": ("Apnea 30 sec", [3038])},
    "Arousal": {"2:22952": ("Arousal", [3040])},
    "ArrhythmiaEvent": {"3:3266": ("Arrhythmia event", [3038])},
    "Artifact": {"3:432": ("Artifact", [3039])},
    "ArtifactualActivity": {"2:24200": ("Artifactual activity", [3035])},
    "Asystole": {"3:3076": ("Asystole", [3038])},
    "AsystoleRhythm": {"10:10336": ("Asystole", [3415])},
    "AtrialBigeminy": {"10:9376": ("Atrial Bigeminy", [3415])},
    "AtrialDemandModePacing": {"10:10176": ("Atrial Demand Mode Pacing", [3415])},
    "AtrialEctopicRhythm": {"10:9360": ("Atrial Ectopic Rhythm", [3415])},
    "AtrialEscapeBeat": {"10:8400": ("Atrial escape beat", [3335])},
    "AtrialFibrillation": {"3:3128": ("Atrial fibrillation", [3038])},
    "AtrialFibrillationRhythm": {"10:9472": ("Atrial fibrillation", [3415])},
    "AtrialFlutter": {"3:3276": ("Atrial flutter", [3038])},
    "AtrialFlutterRhythm": {"10:9456": ("Atrial flutter", [3415])},
    "AtrialFlutterWave": {"10:1792": ("Atrial flutter wave", [3335])},
    "AtrialPrematureContraction": {
        "10:8272": ("Atrial premature contraction (beat)", [3335])
    },
    "AtrialTachycardia": {"10:9392": ("Atrial Tachycardia", [3415])},
    "AtriumAntibradycardiaPaceSpike": {
        "10:4352": ("atrium Antibradycardia pace spike", [3335])
    },
    "AtriumAntitachycardiaPaceSpike": {
        "10:6400": ("atrium Antitachycardia pace spike", [3335])
    },
    "AtriumCardioversionSpike": {"10:7424": ("atrium Cardioversion spike", [3335])},
    "AtriumDefibrillationSpike": {"10:8448": ("atrium Defibrillation spike", [3335])},
    "AtypicalSpikeAndWaveComplex": {
        "2:23928": ("Atypical spike and wave complex", [3035])
    },
    "AutomaticAtrialTachycardia": {"10:9440": ("Automatic Atrial Tachycardia", [3415])},
    "AuxiliaryBipolarLead1": {"2:127": ("Auxiliary bipolar lead 1", [3001])},
    "AuxiliaryBipolarLead2": {"2:128": ("Auxiliary bipolar lead 2", [3001])},
    "AuxiliaryBipolarLead3": {"2:129": ("Auxiliary bipolar lead 3", [3001])},
    "AuxiliaryBipolarLead4": {"2:130": ("Auxiliary bipolar lead 4", [3001])},
    "AuxiliaryUnipolarLead1": {"2:75": ("Auxiliary unipolar lead 1", [3001])},
    "AuxiliaryUnipolarLead2": {"2:76": ("Auxiliary unipolar lead 2", [3001])},
    "AuxiliaryUnipolarLead3": {"2:77": ("Auxiliary unipolar lead 3", [3001])},
    "AuxiliaryUnipolarLead4": {"2:78": ("Auxiliary unipolar lead 4", [3001])},
    "AvfAugmentedVoltageFoot": {"2:64": ("aVF, augmented voltage, foot", [3001])},
    "AvlAugmentedVoltageLeft": {"2:63": ("aVL, augmented voltage, left", [3001])},
    "Avr": {"2:65": ("−aVR", [3001])},
    "AvrAugmentedVoltageRight": {"2:62": ("aVR, augmented voltage, right", [3001])},
    "BackgroundActivity": {"2:23560": ("Background activity", [3035])},
    "BackgroundActivityAlpha": {"2:23592": ("Background activity alpha", [3035])},
    "BackgroundActivityArrhythmicDelta": {
        "2:23640": ("Background activity arrhythmic delta", [3035])
    },
    "BackgroundActivityBeta": {"2:23568": ("Background activity beta", [3035])},
    "BackgroundActivityBisynchronousDelta": {
        "2:23632": ("Background activity bisynchronous delta", [3035])
    },
    "BackgroundActivityBisynchronousTheta": {
        "2:23616": ("Background activity bisynchronous theta", [3035])
    },
    "BackgroundActivityDelta": {"2:23624": ("Background activity delta", [3035])},
    "BackgroundActivityGamma": {"2:23584": ("Background activity gamma", [3035])},
    "BackgroundActivitySigma": {"2:23576": ("Background activity sigma", [3035])},
    "BackgroundActivitySlowFusedTransients": {
        "2:23648": ("Background activity slow fused transients", [3035])
    },
    "BackgroundActivityTheta": {"2:23608": ("Background activity theta", [3035])},
    "BackgroundMuActivity": {"2:23600": ("Background Mu activity", [3035])},
    "BaselineDescription": {"10:11412": ("Baseline description", [3691])},
    "BifascicularBlockBeat": {"10:8560": ("bifascicular block beat", [3335])},
    "BilateralBundleBranchBlockBeat": {
        "10:8592": ("bilateral bundle-branch block beat", [3335])
    },
    "BipolarUncorrectedXYZLeadSystem": {
        "10:11277": ("Bipolar uncorrected XYZ lead system", [3263])
    },
    "BrachycardiaAny": {"10:10432": ("Bradycardia", [3415])},
    "Bradycardia": {"3:3084": ("Bradycardia", [3038])},
    "BurstSuppression": {"2:23952": ("Burst suppression", [3035])},
    "C1": {"7:1129": ("C1", [3030])},
    "C2": {"7:1134": ("C2", [3030])},
    "C3": {"7:1137": ("C3", [3030])},
    "C4": {"7:1142": ("C4", [3030])},
    "C5": {"7:1145": ("C5", [3030])},
    "C6": {"7:1150": ("C6", [3030])},
    "CP1": {"7:1153": ("CP1", [3030])},
    "CP2": {"7:1158": ("CP2", [3030])},
    "CP3": {"7:1161": ("CP3", [3030])},
    "CP4": {"7:1166": ("CP4", [3030])},
    "CP5": {"7:1169": ("CP5", [3030])},
    "CP6": {"7:1174": ("CP6", [3030])},
    "CalibrationPulse": {"10:2240": ("Calibration pulse (individual pulse)", [3335])},
    "CalibrationSignal": {"10:10448": ("Calibration signal (sustained)", [3415])},
    "CanineFifthRightIntercostalSpaceNearEdgeOfSternum": {
        "2:148": ("Canine, fifth right intercostal space near edge of sternum", [3001])
    },
    "CanineOverDorsalSpinousProcessOf7thThoracicVertebra": {
        "2:151": (
            "Canine, over dorsal spinous process of 7th thoracic vertebra",
            [3001],
        )
    },
    "CanineSixthLeftIntercostalSpaceAtCostochondralJunction": {
        "2:150": (
            "Canine, sixth left intercostal space at costochondral junction",
            [3001],
        )
    },
    "CanineSixthLeftIntercostalSpaceNearEdgeOfSternum": {
        "2:149": ("Canine, sixth left intercostal space near edge of sternum", [3001])
    },
    "CardioversionSpike": {"10:7168": ("Cardioversion spike", [3335])},
    "ChestLead": {"2:86": ("Chest lead", [3001])},
    "ChestLeadPerV1AndV1RPlacement": {
        "2:99": ("Chest lead per V1 and V1R placement", [3001])
    },
    "ChestLeadPerV2AndV2RPlacement": {
        "2:100": ("Chest lead per V2 and V2R placement", [3001])
    },
    "ChestLeadPerV3AndV3RPlacement": {
        "2:101": ("Chest lead per V3 and V3R placement", [3001])
    },
    "ChestLeadPerV4AndV4RPlacement": {
        "2:102": ("Chest lead per V4 and V4R placement", [3001])
    },
    "ChestLeadPerV5AndV5RPlacement": {
        "2:19": ("Chest lead per V5 and V5R placement", [3001])
    },
    "ChestLeadPerV6AndV6RPlacement": {
        "2:103": ("Chest lead per V6 and V6R placement", [3001])
    },
    "ChestLeadPerV7AndV8RPlacement": {
        "2:104": ("Chest lead per V7 and V8R placement", [3001])
    },
    "ChestLeadSymmetricPlacement": {
        "2:98": ("Chest lead (symmetric placement)", [3001])
    },
    "ChestManubriumLead": {"2:105": ("Chest-manubrium lead", [3001])},
    "ChestManubriumLeadPerV1Placement": {
        "2:106": ("Chest-manubrium lead per V1 placement", [3001])
    },
    "ChestManubriumLeadPerV2Placement": {
        "2:107": ("Chest-manubrium lead per V2 placement", [3001])
    },
    "ChestManubriumLeadPerV3Placement": {
        "2:108": ("Chest-manubrium lead per V3 placement", [3001])
    },
    "ChestManubriumLeadPerV4Placement": {
        "2:109": ("Chest-manubrium lead per V4 placement", [3001])
    },
    "ChestManubriumLeadPerV5Placement": {
        "2:20": ("Chest-manubrium lead per V5 placement", [3001])
    },
    "ChestManubriumLeadPerV6Placement": {
        "2:110": ("Chest-manubrium lead per V6 placement", [3001])
    },
    "ChestManubriumLeadPerV7Placement": {
        "2:121": ("Chest-manubrium lead per V7 placement", [3001])
    },
    "CircumHead": {"2:22784": ("Circum head", [3040])},
    "ClinicalSeizureDischarge": {"3:3264": ("Clinical seizure discharge", [3038])},
    "CompleteAVDissociation": {"10:9760": ("Complete AV dissociation", [3415])},
    "Cpz": {"7:1020": ("CPz", [3030])},
    "CubeXYZLeadSystem": {"10:11276": ("Cube XYZ lead system", [3263])},
    "Cz": {"7:1016": ("Cz", [3030])},
    "DefibrillationSpike": {"10:8192": ("Defibrillation spike", [3335])},
    "DefibrillatorLeadAnteriorLateral": {
        "2:73": ("Defibrillator lead: anterior-lateral", [3001])
    },
    "DeltaWave": {"10:1216": ("Delta wave", [3335])},
    "DerivedLeadAvf": {"2:114": ("Derived Lead aVF", [3001])},
    "DerivedLeadAvl": {"2:113": ("Derived Lead aVL", [3001])},
    "DerivedLeadAvr": {"2:112": ("Derived Lead aVR", [3001])},
    "DerivedLeadI": {"2:31": ("Derived Lead I", [3001])},
    "DerivedLeadII": {"2:32": ("Derived Lead II", [3001])},
    "DerivedLeadIII": {"2:111": ("Derived Lead III", [3001])},
    "DerivedLeadV1": {"2:33": ("Derived Lead V1", [3001])},
    "DerivedLeadV2": {"2:34": ("Derived Lead V2", [3001])},
    "DerivedLeadV3": {"2:35": ("Derived Lead V3", [3001])},
    "DerivedLeadV4": {"2:36": ("Derived Lead V4", [3001])},
    "DerivedLeadV5": {"2:37": ("Derived Lead V5", [3001])},
    "DerivedLeadV6": {"2:38": ("Derived Lead V6", [3001])},
    "Desaturation": {"3:3246": ("Desaturation", [3038])},
    "Diaphragma": {"7:552": ("Diaphragma", [3031])},
    "DiaphragmaLeft": {"7:553": ("Diaphragma, left", [3031])},
    "DiaphragmaRight": {"7:554": ("Diaphragma, right", [3031])},
    "DominantBeat": {"10:8240": ("Dominant beat", [3335])},
    "E0": {"7:1320": ("E0", [3033])},
    "EASILeadAI": {"2:133": ("EASI Lead AI", [3001])},
    "EASILeadAS": {"2:132": ("EASI Lead AS", [3001])},
    "EASILeadES": {"2:131": ("EASI Lead ES", [3001])},
    "EASIUpperSternumLead": {"2:134": ("EASI upper sternum lead", [3001])},
    "ECGArtifactualActivity": {"2:24240": ("ECG artifactual activity", [3035])},
    "EMGComplexRepetitiveDischarge": {
        "2:24448": ("EMG complex repetitive discharge", [3036])
    },
    "EMGCrampDischarge": {"2:24464": ("EMG cramp discharge", [3036])},
    "EMGDoubletWaveform": {"2:24360": ("EMG doublet waveform", [3036])},
    "EMGEndplateNoise": {"2:24392": ("EMG endplate noise", [3036])},
    "EMGEndplateSpike": {"2:24400": ("EMG endplate spike", [3036])},
    "EMGFasciculationPotential": {"2:24432": ("EMG fasciculation potential", [3036])},
    "EMGFibrillationPotential": {"2:24416": ("EMG fibrillation potential", [3036])},
    "EMGInsertionalActivity": {"2:24384": ("EMG insertional activity", [3036])},
    "EMGMotorUnitPotential": {"2:24352": ("EMG motor unit potential", [3036])},
    "EMGMultipletWaveform": {"2:24376": ("EMG multiplet waveform", [3036])},
    "EMGMyokymicDischarge": {"2:24456": ("EMG myokymic discharge", [3036])},
    "EMGMyotonicDischarge": {"2:24440": ("EMG myotonic discharge", [3036])},
    "EMGPositiveSharpWave": {"2:24424": ("EMG positive sharp wave", [3036])},
    "EMGTripletWaveform": {"2:24368": ("EMG triplet waveform", [3036])},
    "EMGUnspecifiedIterativeDischarge": {
        "2:24408": ("EMG unspecified iterative discharge", [3036])
    },
    "EMGUnspecifiedWaveform": {"2:24336": ("EMG unspecified waveform", [3036])},
    "EMGWaveformAfterDischarge": {"2:24472": ("EMG waveform after discharge", [3036])},
    "EMGWaveformUnderVoluntaryControl": {
        "2:24344": ("EMG waveform under voluntary control", [3036])
    },
    "El1": {"7:1325": ("El1", [3033])},
    "El2": {"7:1329": ("El2", [3033])},
    "El3": {"7:1333": ("El3", [3033])},
    "El4": {"7:1337": ("El4", [3033])},
    "El5": {"7:1341": ("El5", [3033])},
    "El6": {"7:1345": ("El6", [3033])},
    "El7": {"7:1349": ("El7", [3033])},
    "Ela": {"7:1389": ("Ela", [3033])},
    "Elb": {"7:1393": ("Elb", [3033])},
    "ElectrodeInstrumentalArtifactualActivity": {
        "2:24208": ("Electrode instrumental artifactual activity", [3035])
    },
    "Ell": {"7:1381": ("ElL", [3033])},
    "EntireBeatPonToToffExcludingU": {
        "10:1472": ("Entire Beat (Pon to Toff, excluding U)", [3335])
    },
    "EntireBeatQonToToffExcludingPAndU": {
        "10:1536": ("Entire Beat (Qon to Toff, excluding P and U)", [3335])
    },
    "EntireQRSExcludingPTAndU": {
        "10:1600": ("Entire QRS (excluding P, T and U)", [3335])
    },
    "EpilepticOrPotentiallyEpileptogenicActivity": {
        "2:23888": ("Epileptic or potentially epileptogenic activity", [3035])
    },
    "EpilepticOrPotentiallyEpileptogenicSharpWave": {
        "2:23896": ("Epileptic or potentially epileptogenic sharp wave", [3035])
    },
    "EpilepticOrPotentiallyEpileptogenicSpike": {
        "2:23904": ("Epileptic or potentially epileptogenic spike", [3035])
    },
    "Er1": {"7:1354": ("Er1", [3033])},
    "Er2": {"7:1358": ("Er2", [3033])},
    "Er3": {"7:1362": ("Er3", [3033])},
    "Er4": {"7:1366": ("Er4", [3033])},
    "Er5": {"7:1370": ("Er5", [3033])},
    "Er6": {"7:1374": ("Er6", [3033])},
    "Er7": {"7:1378": ("Er7", [3033])},
    "Era": {"7:1398": ("Era", [3033])},
    "Erb": {"7:1402": ("Erb", [3033])},
    "Erl": {"7:1386": ("ErL", [3033])},
    "ExternalInterferenceArtifactualActivity": {
        "2:24272": ("External interference Artifactual activity", [3035])
    },
    "ExternalPacingLeadAnteriorPosterior": {
        "2:74": ("External pacing lead: anterior-posterior", [3001])
    },
    "ExtraocularMuscleActivity": {"2:24160": ("Extraocular muscle activity", [3035])},
    "EyeBlink": {"2:24280": ("Eye blink", [3037])},
    "EyeBlinks": {"2:24048": ("Eye blinks", [3035])},
    "EyeRelatedActivity": {"2:24040": ("Eye-related activity", [3035])},
    "EyeRelatedActivityElectroretinogram": {
        "2:24112": ("Eye-related activity electroretinogram", [3035])
    },
    "EyeRelatedPhotodrivingActivity": {
        "2:24088": ("Eye-related photodriving activity", [3035])
    },
    "EyeRelatedPhotomyogenicActivity": {
        "2:24096": ("Eye-related photomyogenic activity", [3035])
    },
    "EyeRelatedPhotoparadoxysmalActivity": {
        "2:24104": ("Eye-related photoparadoxysmal activity", [3035])
    },
    "EyesClosed": {"2:24320": ("Eyes closed", [3037])},
    "EyesOpen": {"2:24328": ("Eyes open", [3037])},
    "F1": {"7:1049": ("F1", [3030])},
    "F10": {"7:1086": ("F10", [3030])},
    "F2": {"7:1054": ("F2", [3030])},
    "F3": {"7:1057": ("F3", [3030])},
    "F4": {"7:1062": ("F4", [3030])},
    "F5": {"7:1065": ("F5", [3030])},
    "F6": {"7:1070": ("F6", [3030])},
    "F7": {"7:1073": ("F7", [3030])},
    "F8": {"7:1078": ("F8", [3030])},
    "F9": {"7:1081": ("F9", [3030])},
    "FC1": {"7:1089": ("FC1", [3030])},
    "FC2": {"7:1094": ("FC2", [3030])},
    "FC3": {"7:1097": ("FC3", [3030])},
    "FC4": {"7:1102": ("FC4", [3030])},
    "FC5": {"7:1105": ("FC5", [3030])},
    "FC6": {"7:1110": ("FC6", [3030])},
    "FT10": {"7:1126": ("FT10", [3030])},
    "FT7": {"7:1113": ("FT7", [3030])},
    "FT8": {"7:1118": ("FT8", [3030])},
    "FT9": {"7:1121": ("FT9", [3030])},
    "FastIrregularEyeMovements": {"2:24072": ("Fast irregular eye movements", [3035])},
    "Fcz": {"7:1012": ("FCz", [3030])},
    "FirstDegreeAVBlock": {"10:9632": ("First Degree AV Block", [3415])},
    "FirstDegreeSABlock": {"10:9776": ("First Degree SA Block", [3415])},
    "FlashStimulus": {"2:53539": ("Flash stimulus", [3041])},
    "Fp1": {"7:1041": ("Fp1", [3030])},
    "Fp2": {"7:1042": ("Fp2", [3030])},
    "Fpz": {"7:1000": ("Fpz", [3030])},
    "FrankLeadA": {"2:27": ("Frank Lead A", [3001])},
    "FrankLeadC": {"2:26": ("Frank Lead C", [3001])},
    "FrankLeadE": {"2:25": ("Frank Lead E", [3001])},
    "FrankLeadF": {"2:29": ("Frank Lead F", [3001])},
    "FrankLeadH": {"2:30": ("Frank Lead H", [3001])},
    "FrankLeadI": {"2:24": ("Frank Lead I", [3001])},
    "FrankLeadM": {"2:28": ("Frank Lead M", [3001])},
    "FrankXYZLeadSystem": {"10:11274": ("Frank XYZ lead system", [3263])},
    "FusionOfVentricularAndNormalBeat": {
        "10:8352": ("Fusion of ventricular and normal beat", [3335])
    },
    "Fz": {"7:1008": ("Fz", [3030])},
    "GlossokineticArtifactualActivity": {
        "2:24256": ("Glossokinetic artifactual activity", [3035])
    },
    "Grade1AVBlock": {"3:3146": ("First-degree AV block", [3038])},
    "Grade2AVBlock": {"3:3148": ("Second-degree AV block", [3038])},
    "HighPassFilter": {"10:11404": ("High pass filter", [3691])},
    "HighPassFilterDescription": {"10:11406": ("High pass filter description", [3691])},
    "IdioventricularVentricularEscapeRhythm": {
        "10:9856": ("Idioventricular (ventricular escape) rhythm", [3415])
    },
    "IncompleteLeftBundleBranchBlockBeat": {
        "10:8480": ("Incomplete left bundle branch block beat", [3335])
    },
    "IncompleteRightBundleBranchBlockBeat": {
        "10:8512": ("Incomplete right bundle branch block beat", [3335])
    },
    "Interpolator": {"10:11414": ("Interpolator", [3691])},
    "InterpolatorDescription": {"10:11416": ("Interpolator description", [3691])},
    "InterpolatorSNR": {"10:11418": ("Interpolator SNR", [3690])},
    "IntraventricularConductionDisturbance": {
        "10:8608": ("intraventricular conduction disturbance", [3335])
    },
    "IrregularHeartRate": {"3:3158": ("Irregular heart rate", [3038])},
    "IrregularRhythm": {"3:3118": ("Irregular rhythm", [3038])},
    "IrregularRhythmLowHeartRate": {"10:10352": ("Irregular rhythm", [3415])},
    "IsoelectricPointOrSegment": {"10:1856": ("Isoelectric point or segment", [3335])},
    "IsoelectricRegionFromGlobalQRSOnsetToSpecificLeadOnset": {
        "10:1280": (
            "Isoelectric region from global QRS onset to specific lead onset",
            [3335],
        )
    },
    "IsoelectricRegionFromSpecificLeadQRSOffsetToGlobalOffset": {
        "10:1344": (
            "Isoelectric region from specific lead QRS Offset to global offset",
            [3335],
        )
    },
    "IsolatedQRSLikeArtifact": {"10:2176": ("Isolated QRS-like artifact", [3335])},
    "IsorhythmicAVDissociation": {"10:9744": ("Isorhythmic AV dissociation", [3415])},
    "Iz": {"7:1036": ("Iz", [3030])},
    "JPoint": {"10:2048": ("J-point", [3335])},
    "JPointAmplitudePerLead": {"2:1024": ("J point Amplitude, per lead", [3688])},
    "JunctionalNodalEscapeBeat": {
        "10:8416": ("Junctional (nodal) escape beat", [3335])
    },
    "JunctionalNodalPrematureContraction": {
        "10:8288": ("Junctional (nodal) premature contraction", [3335])
    },
    "JunctionalTachycardia": {"10:9584": ("Junctional Tachycardia", [3415])},
    "LambdaWave": {"2:23880": ("Lambda wave", [3035])},
    "LeadANehbAnterior": {"2:71": ("Lead A (Nehb - Anterior)", [3001])},
    "LeadCH5": {"2:122": ("Lead CH5", [3001])},
    "LeadCR5": {"2:125": ("Lead CR5", [3001])},
    "LeadDNehbDorsal": {"2:70": ("Lead D (Nehb - Dorsal)", [3001])},
    "LeadDisconnected": {"3:268": ("Lead disconnected", [3039])},
    "LeadI": {"2:1": ("Lead I", [3001])},
    "LeadII": {"2:2": ("Lead II", [3001])},
    "LeadIII": {"2:61": ("Lead III", [3001])},
    "LeadJNehbInferior": {"2:72": ("Lead J (Nehb - Inferior)", [3001])},
    "LeadV1": {"2:3": ("Lead V1", [3001])},
    "LeadV2": {"2:4": ("Lead V2", [3001])},
    "LeadV2R": {"2:10": ("Lead V2R", [3001])},
    "LeadV3": {"2:5": ("Lead V3", [3001])},
    "LeadV3R": {"2:11": ("Lead V3R", [3001])},
    "LeadV4": {"2:6": ("Lead V4", [3001])},
    "LeadV4R": {"2:12": ("Lead V4R", [3001])},
    "LeadV5": {"2:7": ("Lead V5", [3001])},
    "LeadV5R": {"2:13": ("Lead V5R", [3001])},
    "LeadV6": {"2:8": ("Lead V6", [3001])},
    "LeadV6R": {"2:14": ("Lead V6R", [3001])},
    "LeadV7": {"2:9": ("Lead V7", [3001])},
    "LeadV7R": {"2:15": ("Lead V7R", [3001])},
    "LeadV8": {"2:66": ("Lead V8", [3001])},
    "LeadV8R": {"2:68": ("Lead V8R", [3001])},
    "LeadV9": {"2:67": ("Lead V9", [3001])},
    "LeadV9R": {"2:69": ("Lead V9R", [3001])},
    "LeadVFNonaugmentedVoltageVectorOfLL": {
        "2:90": ("Lead VF, nonaugmented voltage, vector of LL", [3001])
    },
    "LeadVLNonaugmentedVoltageVectorOfLA": {
        "2:89": ("Lead VL, nonaugmented voltage, vector of LA", [3001])
    },
    "LeadVRNonaugmentedVoltageVectorOfRA": {
        "2:88": ("Lead VR, nonaugmented voltage, vector of RA", [3001])
    },
    "LeadX": {"2:16": ("Lead X", [3001])},
    "LeadY": {"2:17": ("Lead Y", [3001])},
    "LeadZ": {"2:18": ("Lead Z", [3001])},
    "LeftAnteriorFascicularBlockBeat": {
        "10:8528": ("Left anterior fascicular block beat", [3335])
    },
    "LeftArmLead": {"2:21": ("Left Arm Lead", [3001])},
    "LeftAtriumAntibradycardiaPaceSpike": {
        "10:4864": ("left atrium Antibradycardia pace spike", [3335])
    },
    "LeftBundleBranchBlockBeat": {"10:8464": ("Left bundle branch block beat", [3335])},
    "LeftLegLead": {"2:23": ("Left Leg Lead", [3001])},
    "LeftPosteriorFascicularBlockBeat": {
        "10:8544": ("Left posterior fascicular block beat", [3335])
    },
    "LeftVentricleAntibradycardiaPaceSpike": {
        "10:5632": ("left ventricle Antibradycardia pace spike", [3335])
    },
    "LowHeartRateVariability": {"10:10368": ("Low Heart Rate Variability", [3415])},
    "LowPassFilter": {"10:11402": ("Low pass filter", [3691])},
    "LowPassFilterCutoffFreq": {"10:11403": ("Low pass filter cutoff freq", [3690])},
    "LownGanongLevineSyndrome": {"10:10160": ("Lown-Ganong-Levine syndrome", [3415])},
    "LownGanongLevineSyndromeNormalQRS": {
        "10:8688": ("Lown-Ganong-Levine syndrome", [3335])
    },
    "MasonLikarLeadPositionsElectrodesPlacedIndividually": {
        "10:11266": (
            "Mason-Likar lead positions, electrodes placed individually",
            [3263],
        )
    },
    "MasonLikarLeadPositionsV1V6InElectrodePad": {
        "10:11267": ("Mason-Likar lead positions, V1-V6 in electrode pad", [3263])
    },
    "McfeeParungaoXYZLeadSystem": {
        "10:11275": ("McFee-Parungao XYZ lead system", [3263])
    },
    "ModerateNoise": {"10:11216": ("Moderate noise", [3680])},
    "ModerateNoiseBeatsCanBeDetectedButCannotBeClassified": {
        "10:11216": (
            "Moderate Noise, beats can be detected but cannot be classified",
            [3335],
        )
    },
    "ModifiedChestLead": {
        "2:91": ("Modified chest lead (left arm indifferent)", [3001])
    },
    "ModifiedChestLeadPerV1Placement": {
        "2:92": ("Modified chest lead per V1 placement", [3001])
    },
    "ModifiedChestLeadPerV2Placement": {
        "2:93": ("Modified chest lead per V2 placement", [3001])
    },
    "ModifiedChestLeadPerV3Placement": {
        "2:94": ("Modified chest lead per V3 placement", [3001])
    },
    "ModifiedChestLeadPerV4Placement": {
        "2:95": ("Modified chest lead per V4 placement", [3001])
    },
    "ModifiedChestLeadPerV5Placement": {
        "2:96": ("Modified chest lead per V5 placement", [3001])
    },
    "ModifiedChestLeadPerV6Placement": {
        "2:97": ("Modified chest lead per V6 placement", [3001])
    },
    "ModifiedLimbLead": {"2:126": ("Modified limb lead", [3001])},
    "MovementArtifactualActivity": {
        "2:24216": ("Movement artifactual activity", [3035])
    },
    "MultifocalAtrialTachycardia": {
        "10:9424": ("Multifocal Atrial Tachycardia", [3415])
    },
    "MultipleIndependentSpikesAndAsynchronousSlowWaves": {
        "2:23960": ("Multiple independent spikes and asynchronous slow waves", [3035])
    },
    "MultipleSpike": {"2:23912": ("Multiple spike", [3035])},
    "MusculiAbdominis": {"7:556": ("Musculi abdominis", [3031])},
    "MusculiAbdominisLeft": {"7:557": ("Musculi abdominis, left", [3031])},
    "MusculiAbdominisRight": {"7:558": ("Musculi abdominis, right", [3031])},
    "MusculiBulbi": {"7:256": ("Musculi bulbi", [3031])},
    "MusculiBulbiLeft": {"7:257": ("Musculi bulbi, left", [3031])},
    "MusculiBulbiRight": {"7:258": ("Musculi bulbi, right", [3031])},
    "MusculiCapitis": {"7:252": ("Musculi capitis", [3031])},
    "MusculiCapitisLeft": {"7:253": ("Musculi capitis, left", [3031])},
    "MusculiCapitisRight": {"7:254": ("Musculi capitis, right", [3031])},
    "MusculiColli": {"7:388": ("Musculi colli", [3031])},
    "MusculiColliLeft": {"7:389": ("Musculi colli, left", [3031])},
    "MusculiColliRight": {"7:390": ("Musculi colli, right", [3031])},
    "MusculiDiaphragmatisPelvis": {"7:580": ("Musculi diaphragmatis pelvis", [3031])},
    "MusculiDiaphragmatisPelvisLeft": {
        "7:581": ("Musculi diaphragmatis pelvis, left", [3031])
    },
    "MusculiDiaphragmatisPelvisRight": {
        "7:582": ("Musculi diaphragmatis pelvis, right", [3031])
    },
    "MusculiDorsi": {"7:424": ("Musculi dorsi", [3031])},
    "MusculiDorsiLeft": {"7:425": ("Musculi dorsi, left", [3031])},
    "MusculiDorsiRight": {"7:426": ("Musculi dorsi, right", [3031])},
    "MusculiFacialesEtMasticatores": {
        "7:284": ("Musculi faciales et masticatores", [3031])
    },
    "MusculiFacialesEtMasticatoresLeft": {
        "7:285": ("Musculi faciales et masticatores, left", [3031])
    },
    "MusculiFacialesEtMasticatoresRight": {
        "7:286": ("Musculi faciales et masticatores, right", [3031])
    },
    "MusculiIntercostales": {"7:548": ("Musculi intercostales", [3031])},
    "MusculiIntercostalesLeft": {"7:549": ("Musculi intercostales, left", [3031])},
    "MusculiIntercostalesRight": {"7:550": ("Musculi intercostales, right", [3031])},
    "MusculiInterosseiDorsales": {"7:772": ("Musculi interossei dorsales", [3031])},
    "MusculiInterosseiDorsalesLeft": {
        "7:773": ("Musculi interossei dorsales, left", [3031])
    },
    "MusculiInterosseiDorsalesRight": {
        "7:774": ("Musculi interossei dorsales, right", [3031])
    },
    "MusculiInterosseiPalmares": {"7:776": ("Musculi interossei palmares", [3031])},
    "MusculiInterosseiPalmaresLeft": {
        "7:777": ("Musculi interossei palmares, left", [3031])
    },
    "MusculiInterosseiPalmaresRight": {
        "7:778": ("Musculi interossei palmares, right", [3031])
    },
    "MusculiInterpsinalesRight": {"7:514": ("Musculi interpsinales, right", [3031])},
    "MusculiInterspinales": {"7:512": ("Musculi interspinales", [3031])},
    "MusculiInterspinalesCervicis": {
        "7:516": ("Musculi interspinales cervicis", [3031])
    },
    "MusculiInterspinalesCervicisLeft": {
        "7:517": ("Musculi interspinales cervicis, left", [3031])
    },
    "MusculiInterspinalesCervicisRight": {
        "7:518": ("Musculi interspinales cervicis, right", [3031])
    },
    "MusculiInterspinalesLeft": {"7:513": ("Musculi interspinales, left", [3031])},
    "MusculiInterspinalesLumborum": {
        "7:524": ("Musculi interspinales lumborum", [3031])
    },
    "MusculiInterspinalesLumborumLeft": {
        "7:525": ("Musculi interspinales lumborum, left", [3031])
    },
    "MusculiInterspinalesLumborumRight": {
        "7:526": ("Musculi interspinales lumborum, right", [3031])
    },
    "MusculiInterspinalesThoracis": {
        "7:520": ("Musculi interspinales thoracis", [3031])
    },
    "MusculiInterspinalesThoracisLeft": {
        "7:521": ("Musculi interspinales thoracis, left", [3031])
    },
    "MusculiInterspinalesThoracisRight": {
        "7:522": ("Musculi interspinales thoracis, right", [3031])
    },
    "MusculiLaringis": {"7:376": ("Musculi laringis", [3031])},
    "MusculiLaringisLeft": {"7:377": ("Musculi laringis, left", [3031])},
    "MusculiLaringisRight": {"7:378": ("Musculi laringis, right", [3031])},
    "MusculiLinguae": {"7:368": ("Musculi linguae", [3031])},
    "MusculiLinguaeLeft": {"7:369": ("Musculi linguae, left", [3031])},
    "MusculiLinguaeRight": {"7:370": ("Musculi linguae, right", [3031])},
    "MusculiLumbricales": {"7:768": ("Musculi lumbricales", [3031])},
    "MusculiLumbricalesLeft": {"7:769": ("Musculi lumbricales, left", [3031])},
    "MusculiLumbricalesRight": {"7:770": ("Musculi lumbricales, right", [3031])},
    "MusculiLunbricales": {"7:984": ("Musculi lunbricales", [3031])},
    "MusculiLunbricalesLeft": {"7:985": ("Musculi lunbricales, left", [3031])},
    "MusculiLunbricalesRight": {"7:986": ("Musculi lunbricales, right", [3031])},
    "MusculiMembriSuperioriRight": {
        "7:602": ("Musculi membri superiori, right", [3031])
    },
    "MusculiMembriSuperioris": {"7:600": ("Musculi membri superioris", [3031])},
    "MusculiMembriSuperiorisLeft": {
        "7:601": ("Musculi membri superioris, left", [3031])
    },
    "MusculiMultifidii": {"7:508": ("Musculi multifidii", [3031])},
    "MusculiMultifidiiLeft": {"7:509": ("Musculi multifidii, left", [3031])},
    "MusculiMultifidiiRight": {"7:510": ("Musculi multifidii, right", [3031])},
    "MusculiThoracis": {"7:528": ("Musculi thoracis", [3031])},
    "MusculiThoracisLeft": {"7:529": ("Musculi thoracis, left", [3031])},
    "MusculiThoracisRight": {"7:530": ("Musculi thoracis, right", [3031])},
    "MusculusAbductorDigitiMinimiFoot": {
        "7:972": ("Musculus abductor digiti minimi", [3031])
    },
    "MusculusAbductorDigitiMinimiFootLeft": {
        "7:973": ("Musculus abductor digiti minimi, left", [3031])
    },
    "MusculusAbductorDigitiMinimiFootRight": {
        "7:974": ("Musculus abductor digiti minimi", [3031])
    },
    "MusculusAbductorDigitiMinimiHand": {
        "7:756": ("Musculus abductor digiti minimi", [3031])
    },
    "MusculusAbductorDigitiMinimiHandLeft": {
        "7:757": ("Musculus abductor digiti minimi, left", [3031])
    },
    "MusculusAbductorDigitiMinimiHandRight": {
        "7:758": ("Musculus abductor digiti minimi, right", [3031])
    },
    "MusculusAbductorHallucis": {"7:960": ("Musculus abductor hallucis", [3031])},
    "MusculusAbductorHallucisLeft": {
        "7:961": ("Musculus abductor hallucis, left", [3031])
    },
    "MusculusAbductorHallucisRight": {
        "7:962": ("Musculus abductor hallucis, right", [3031])
    },
    "MusculusAbductorPollicisBrevis": {
        "7:740": ("Musculus abductor pollicis brevis", [3031])
    },
    "MusculusAbductorPollicisBrevisLeft": {
        "7:741": ("Musculus abductor pollicis brevis, left", [3031])
    },
    "MusculusAbductorPollicisBrevisRight": {
        "7:742": ("Musculus abductor pollicis brevis, right", [3031])
    },
    "MusculusAbductorPollicisLonguss": {
        "7:720": ("Musculus abductor pollicis longuss", [3031])
    },
    "MusculusAbductorPollicisLongussLeft": {
        "7:721": ("Musculus abductor pollicis longuss, left", [3031])
    },
    "MusculusAbductorPollicisLongussRight": {
        "7:722": ("Musculus abductor pollicis longuss, right", [3031])
    },
    "MusculusAdductorBrevis": {"7:860": ("Musculus adductor brevis", [3031])},
    "MusculusAdductorBrevisLeft": {"7:861": ("Musculus adductor brevis, left", [3031])},
    "MusculusAdductorBrevisRight": {
        "7:862": ("Musculus adductor brevis, right", [3031])
    },
    "MusculusAdductorHallucis": {"7:968": ("Musculus adductor hallucis", [3031])},
    "MusculusAdductorHallucisLeft": {
        "7:969": ("Musculus adductor hallucis, left", [3031])
    },
    "MusculusAdductorHallucisRight": {"7:970": ("Musculus adductor hallucis", [3031])},
    "MusculusAdductorLongus": {"7:856": ("Musculus adductor longus", [3031])},
    "MusculusAdductorLongusLeft": {"7:857": ("Musculus adductor longus, left", [3031])},
    "MusculusAdductorLongusRight": {
        "7:858": ("Musculus adductor longus, right", [3031])
    },
    "MusculusAdductorMagnus": {"7:864": ("Musculus adductor magnus", [3031])},
    "MusculusAdductorMagnusLeft": {"7:865": ("Musculus adductor magnus, left", [3031])},
    "MusculusAdductorMagnusRight": {
        "7:866": ("Musculus adductor magnus, right", [3031])
    },
    "MusculusAdductorPollicis": {"7:752": ("Musculus adductor pollicis", [3031])},
    "MusculusAdductorPollicisLeft": {
        "7:753": ("Musculus adductor pollicis, left", [3031])
    },
    "MusculusAdductorPollicisRight": {
        "7:754": ("Musculus adductor pollicis, right", [3031])
    },
    "MusculusAnconeus": {"7:656": ("Musculus anconeus", [3031])},
    "MusculusAnconeusLeft": {"7:657": ("Musculus anconeus, left", [3031])},
    "MusculusAnconeusRight": {"7:658": ("Musculus anconeus, right", [3031])},
    "MusculusAuricularisPosterior": {
        "7:300": ("Musculus auricularis posterior", [3031])
    },
    "MusculusAuricularisPosteriorLeft": {
        "7:301": ("Musculus auricularis posterior, left", [3031])
    },
    "MusculusAuricularisPosteriorRight": {
        "7:302": ("Musculus auricularis posterior, right", [3031])
    },
    "MusculusBicepsBrachii": {"7:628": ("Musculus biceps brachii", [3031])},
    "MusculusBicepsBrachiiLeft": {"7:629": ("Musculus biceps brachii, left", [3031])},
    "MusculusBicepsBrachiiRight": {"7:630": ("Musculus biceps brachii, right", [3031])},
    "MusculusBicepsFemoris": {"7:872": ("Musculus biceps femoris", [3031])},
    "MusculusBicepsFemorisCaputBreve": {
        "7:880": ("Musculus biceps femoris Caput breve", [3031])
    },
    "MusculusBicepsFemorisCaputBreveLeft": {
        "7:881": ("Musculus biceps femoris Caput breve, left", [3031])
    },
    "MusculusBicepsFemorisCaputBreveRight": {
        "7:882": ("Musculus biceps femoris Caput breve, right", [3031])
    },
    "MusculusBicepsFemorisCaputLongum": {
        "7:876": ("Musculus biceps femoris Caput longum", [3031])
    },
    "MusculusBicepsFemorisCaputLongumLeft": {
        "7:877": ("Musculus biceps femoris Caput longum, left", [3031])
    },
    "MusculusBicepsFemorisCaputLongumRight": {
        "7:878": ("Musculus biceps femoris Caput longum, right", [3031])
    },
    "MusculusBicepsFemorisLeft": {"7:873": ("Musculus biceps femoris, left", [3031])},
    "MusculusBicepsFemorisRight": {"7:874": ("Musculus biceps femoris, right", [3031])},
    "MusculusBrachialis": {"7:632": ("Musculus brachialis", [3031])},
    "MusculusBrachialisLeft": {"7:633": ("Musculus brachialis, left", [3031])},
    "MusculusBrachialisRight": {"7:634": ("Musculus brachialis, right", [3031])},
    "MusculusBrachioradialis": {"7:692": ("Musculus brachioradialis", [3031])},
    "MusculusBrachioradialisLeft": {
        "7:693": ("Musculus brachioradialis, left", [3031])
    },
    "MusculusBrachioradialisRight": {
        "7:694": ("Musculus brachioradialis, right", [3031])
    },
    "MusculusBuccinator": {"7:340": ("Musculus buccinator", [3031])},
    "MusculusBuccinatorLeft": {"7:341": ("Musculus buccinator, left", [3031])},
    "MusculusBuccinatorRight": {"7:342": ("Musculus buccinator, right", [3031])},
    "MusculusCapitisLongus": {"7:396": ("Musculus capitis longus", [3031])},
    "MusculusCapitisLongusLeft": {"7:397": ("Musculus capitis longus, left", [3031])},
    "MusculusCapitisLongusRight": {"7:398": ("Musculus capitis longus, right", [3031])},
    "MusculusCoccygeus": {"7:588": ("Musculus coccygeus", [3031])},
    "MusculusCoccygeusLeft": {"7:589": ("Musculus coccygeus, left", [3031])},
    "MusculusCoccygeusRight": {"7:590": ("Musculus coccygeus, right", [3031])},
    "MusculusCoracobrachialis": {"7:636": ("Musculus coracobrachialis", [3031])},
    "MusculusCoracobrachialisLeft": {
        "7:637": ("Musculus coracobrachialis, left", [3031])
    },
    "MusculusCoracobrachialisRight": {
        "7:638": ("Musculus coracobrachialis, right", [3031])
    },
    "MusculusCricothyroideus": {"7:380": ("Musculus cricothyroideus", [3031])},
    "MusculusCricothyroideusLeft": {
        "7:381": ("Musculus cricothyroideus, left", [3031])
    },
    "MusculusCricothyroideusRight": {
        "7:382": ("Musculus cricothyroideus, right", [3031])
    },
    "MusculusDeltoideus": {"7:604": ("Musculus deltoideus", [3031])},
    "MusculusDeltoideusLeft": {"7:605": ("Musculus deltoideus, left", [3031])},
    "MusculusDeltoideusRight": {"7:606": ("Musculus deltoideus, right", [3031])},
    "MusculusDepressorAnguliOris": {
        "7:308": ("Musculus depressor anguli oris", [3031])
    },
    "MusculusDepressorAnguliOrisLeft": {
        "7:309": ("Musculus depressor anguli oris, left", [3031])
    },
    "MusculusDepressorAnguliOrisRight": {
        "7:310": ("Musculus depressor anguli oris, right", [3031])
    },
    "MusculusDepressorLabiiInferioris": {
        "7:332": ("Musculus depressor labii inferioris", [3031])
    },
    "MusculusDepressorLabiiInferiorisLeft": {
        "7:333": ("Musculus depressor labii inferioris, left", [3031])
    },
    "MusculusDepressorLabiiInferiorisRight": {
        "7:334": ("Musculus depressor labii inferioris, right", [3031])
    },
    "MusculusDigastricus": {"7:404": ("Musculus digastricus", [3031])},
    "MusculusDigastricusLeft": {"7:405": ("Musculus digastricus, left", [3031])},
    "MusculusDigastricusRight": {"7:406": ("Musculus digastricus, right", [3031])},
    "MusculusDigastricusVenterAnterior": {
        "7:408": ("Musculus digastricus, Venter anterior", [3031])
    },
    "MusculusDigastricusVenterAnteriorLeft": {
        "7:409": ("Musculus digastricus, Venter anterior, left", [3031])
    },
    "MusculusDigastricusVenterAnteriorRight": {
        "7:410": ("Musculus digastricus, Venter anterior, right", [3031])
    },
    "MusculusDigastricusVenterPosterior": {
        "7:412": ("Musculus digastricus, Venter posterior", [3031])
    },
    "MusculusDigastricusVenterPosteriorLeft": {
        "7:413": ("Musculus digastricus, Venter posterior, left", [3031])
    },
    "MusculusDigastricusVenterPosteriorRight": {
        "7:414": ("Musculus digastricus, Venter posterior, right", [3031])
    },
    "MusculusErectorSpinae": {"7:472": ("Musculus erector spinae", [3031])},
    "MusculusErectorSpinaeLeft": {"7:473": ("Musculus erector spinae, left", [3031])},
    "MusculusErectorSpinaeRight": {"7:474": ("Musculus erector spinae, right", [3031])},
    "MusculusExtensorCarpiRadialisBrevis": {
        "7:700": ("Musculus extensor carpi radialis brevis", [3031])
    },
    "MusculusExtensorCarpiRadialisBrevisLeft": {
        "7:701": ("Musculus extensor carpi radialis brevis, left", [3031])
    },
    "MusculusExtensorCarpiRadialisBrevisRight": {
        "7:702": ("Musculus extensor carpi radialis brevis, right", [3031])
    },
    "MusculusExtensorCarpiRadialisLongus": {
        "7:696": ("Musculus extensor carpi radialis longus", [3031])
    },
    "MusculusExtensorCarpiRadialisLongusLeft": {
        "7:697": ("Musculus extensor carpi radialis longus, left", [3031])
    },
    "MusculusExtensorCarpiRadialisLongusRight": {
        "7:698": ("Musculus extensor carpi radialis longus, right", [3031])
    },
    "MusculusExtensorCarpiUlnaris": {
        "7:712": ("Musculus extensor carpi ulnaris", [3031])
    },
    "MusculusExtensorCarpiUlnarisLeft": {
        "7:713": ("Musculus extensor carpi ulnaris, left", [3031])
    },
    "MusculusExtensorCarpiUlnarisRight": {
        "7:714": ("Musculus extensor carpi ulnaris, right", [3031])
    },
    "MusculusExtensorDigitiMinimi": {
        "7:708": ("Musculus extensor digiti minimi", [3031])
    },
    "MusculusExtensorDigitiMinimiLeft": {
        "7:709": ("Musculus extensor digiti minimi, left", [3031])
    },
    "MusculusExtensorDigitiMinimiRight": {
        "7:710": ("Musculus extensor digiti minimi, right", [3031])
    },
    "MusculusExtensorDigitorum": {"7:704": ("Musculus extensor digitorum", [3031])},
    "MusculusExtensorDigitorumBrevis": {
        "7:956": ("Musculus extensor digitorum brevis", [3031])
    },
    "MusculusExtensorDigitorumBrevisLeft": {
        "7:957": ("Musculus extensor digitorum brevis, left", [3031])
    },
    "MusculusExtensorDigitorumBrevisRight": {
        "7:958": ("Musculus extensor digitorum brevis, right", [3031])
    },
    "MusculusExtensorDigitorumLeft": {
        "7:705": ("Musculus extensor digitorum, left", [3031])
    },
    "MusculusExtensorDigitorumLongus": {
        "7:896": ("Musculus extensor digitorum longus", [3031])
    },
    "MusculusExtensorDigitorumLongusLeft": {
        "7:897": ("Musculus extensor digitorum longus, left", [3031])
    },
    "MusculusExtensorDigitorumLongusRight": {
        "7:898": ("Musculus extensor digitorum longus, right", [3031])
    },
    "MusculusExtensorDigitorumRight": {
        "7:706": ("Musculus extensor digitorum, right", [3031])
    },
    "MusculusExtensorHallucisBrevis": {
        "7:952": ("Musculus extensor hallucis brevis", [3031])
    },
    "MusculusExtensorHallucisBrevisLeft": {
        "7:953": ("Musculus extensor hallucis brevis, left", [3031])
    },
    "MusculusExtensorHallucisBrevisRight": {
        "7:954": ("Musculus extensor hallucis brevis, right", [3031])
    },
    "MusculusExtensorHallucisLongus": {
        "7:900": ("Musculus extensor hallucis longus", [3031])
    },
    "MusculusExtensorHallucisLongusLeft": {
        "7:901": ("Musculus extensor hallucis longus, left", [3031])
    },
    "MusculusExtensorHallucisLongusRight": {
        "7:902": ("Musculus extensor hallucis longus, right", [3031])
    },
    "MusculusExtensorIndicis": {"7:732": ("Musculus extensor indicis", [3031])},
    "MusculusExtensorIndicisLeft": {
        "7:733": ("Musculus extensor indicis, left", [3031])
    },
    "MusculusExtensorIndicisRight": {
        "7:734": ("Musculus extensor indicis, right", [3031])
    },
    "MusculusExtensorPollicisBrevis": {
        "7:724": ("Musculus extensor pollicis brevis", [3031])
    },
    "MusculusExtensorPollicisBrevisLeft": {
        "7:725": ("Musculus extensor pollicis brevis, left", [3031])
    },
    "MusculusExtensorPollicisBrevisRight": {
        "7:726": ("Musculus extensor pollicis brevis, right", [3031])
    },
    "MusculusExtensorPollicisLongus": {
        "7:728": ("Musculus extensor pollicis longus", [3031])
    },
    "MusculusExtensorPollicisLongusLeft": {
        "7:729": ("Musculus extensor pollicis longus, left", [3031])
    },
    "MusculusExtensorPollicisLongusRight": {
        "7:730": ("Musculus extensor pollicis longus, right", [3031])
    },
    "MusculusFlexorCarpiRadialis": {
        "7:664": ("Musculus flexor carpi radialis", [3031])
    },
    "MusculusFlexorCarpiRadialisLeft": {
        "7:665": ("Musculus flexor carpi radialis, left", [3031])
    },
    "MusculusFlexorCarpiRadialisRight": {
        "7:666": ("Musculus flexor carpi radialis, right", [3031])
    },
    "MusculusFlexorCarpiUlnaris": {"7:672": ("Musculus flexor carpi ulnaris", [3031])},
    "MusculusFlexorCarpiUlnarisLeft": {
        "7:673": ("Musculus flexor carpi ulnaris, left", [3031])
    },
    "MusculusFlexorCarpiUlnarisRight": {
        "7:674": ("Musculus flexor carpi ulnaris, right", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisFoot": {
        "7:976": ("Musculus flexor digiti minimi brevis", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisFootLeft": {
        "7:977": ("Musculus flexor digiti minimi brevis, left", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisFootRight": {
        "7:978": ("Musculus flexor digiti minimi brevis, right", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisHand": {
        "7:760": ("Musculus flexor digiti minimi brevis", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisHandLeft": {
        "7:761": ("Musculus flexor digiti minimi brevis, left", [3031])
    },
    "MusculusFlexorDigitiMinimiBrevisHandRight": {
        "7:762": ("Musculus flexor digiti minimi brevis, right", [3031])
    },
    "MusculusFlexorDigitorumLongus": {
        "7:948": ("Musculus flexor digitorum longus", [3031])
    },
    "MusculusFlexorDigitorumLongusLeft": {
        "7:949": ("Musculus flexor digitorum longus, left", [3031])
    },
    "MusculusFlexorDigitorumLongusRight": {
        "7:950": ("Musculus flexor digitorum longus, right", [3031])
    },
    "MusculusFlexorDigitorumProfundus": {
        "7:680": ("Musculus flexor digitorum profundus", [3031])
    },
    "MusculusFlexorDigitorumProfundusLeft": {
        "7:681": ("Musculus flexor digitorum profundus, left", [3031])
    },
    "MusculusFlexorDigitorumProfundusRight": {
        "7:682": ("Musculus flexor digitorum profundus, right", [3031])
    },
    "MusculusFlexorDigitorumSuperficialis": {
        "7:676": ("Musculus flexor digitorum superficialis", [3031])
    },
    "MusculusFlexorDigitorumSuperficialisLeft": {
        "7:677": ("Musculus flexor digitorum superficialis, left", [3031])
    },
    "MusculusFlexorDigitorumSuperficialisRight": {
        "7:678": ("Musculus flexor digitorum superficialis, right", [3031])
    },
    "MusculusFlexorHallucisBrevis": {
        "7:964": ("Musculus flexor hallucis brevis", [3031])
    },
    "MusculusFlexorHallucisBrevisLeft": {
        "7:965": ("Musculus flexor hallucis brevis, left", [3031])
    },
    "MusculusFlexorHallucisBrevisRight": {
        "7:966": ("Musculus flexor hallucis brevis, right", [3031])
    },
    "MusculusFlexorPollicisBrevis": {
        "7:744": ("Musculus flexor pollicis brevis", [3031])
    },
    "MusculusFlexorPollicisBrevisLeft": {
        "7:745": ("Musculus flexor pollicis brevis, left", [3031])
    },
    "MusculusFlexorPollicisBrevisRight": {
        "7:746": ("Musculus flexor pollicis brevis, right", [3031])
    },
    "MusculusFlexorPollicisLongus": {
        "7:684": ("Musculus flexor pollicis longus", [3031])
    },
    "MusculusFlexorPollicisLongusLeft": {
        "7:685": ("Musculus flexor pollicis longus, left", [3031])
    },
    "MusculusFlexorPollicisLongusRight": {
        "7:686": ("Musculus flexor pollicis longus, right", [3031])
    },
    "MusculusGastrocnemius": {"7:920": ("Musculus gastrocnemius", [3031])},
    "MusculusGastrocnemiusCaputLaterale": {
        "7:924": ("Musculus gastrocnemius Caput laterale", [3031])
    },
    "MusculusGastrocnemiusCaputLateraleLeft": {
        "7:925": ("Musculus gastrocnemius Caput laterale, left", [3031])
    },
    "MusculusGastrocnemiusCaputLateraleRight": {
        "7:926": ("Musculus gastrocnemius Caput laterale, right", [3031])
    },
    "MusculusGastrocnemiusCaputMediale": {
        "7:928": ("Musculus gastrocnemius Caput mediale", [3031])
    },
    "MusculusGastrocnemiusCaputMedialeLeft": {
        "7:929": ("Musculus gastrocnemius Caput mediale, left", [3031])
    },
    "MusculusGastrocnemiusCaputMedialeRight": {
        "7:930": ("Musculus gastrocnemius Caput mediale, right", [3031])
    },
    "MusculusGastrocnemiusLeft": {"7:921": ("Musculus gastrocnemius, left", [3031])},
    "MusculusGastrocnemiusRight": {"7:922": ("Musculus gastrocnemius, right", [3031])},
    "MusculusGenioglossus": {"7:372": ("Musculus genioglossus", [3031])},
    "MusculusGenioglossusLeft": {"7:373": ("Musculus genioglossus, left", [3031])},
    "MusculusGenioglossusRight": {"7:374": ("Musculus genioglossus, right", [3031])},
    "MusculusGluteusMaximus": {"7:796": ("Musculus gluteus maximus", [3031])},
    "MusculusGluteusMaximusLeft": {"7:797": ("Musculus gluteus maximus, left", [3031])},
    "MusculusGluteusMaximusRight": {
        "7:798": ("Musculus gluteus maximus, right", [3031])
    },
    "MusculusGluteusMedius": {"7:800": ("Musculus gluteus medius", [3031])},
    "MusculusGluteusMediusLeft": {"7:801": ("Musculus gluteus medius, left", [3031])},
    "MusculusGluteusMediusRight": {"7:802": ("Musculus gluteus medius, right", [3031])},
    "MusculusGluteusMinimus": {"7:804": ("Musculus gluteus minimus", [3031])},
    "MusculusGluteusMinimusLeft": {"7:805": ("Musculus gluteus minimus, left", [3031])},
    "MusculusGluteusMinimusRight": {
        "7:806": ("Musculus gluteus minimus, right", [3031])
    },
    "MusculusGmellus": {"7:820": ("Musculus gmellus", [3031])},
    "MusculusGmellusLeft": {"7:821": ("Musculus gmellus, left", [3031])},
    "MusculusGmellusRight": {"7:822": ("Musculus gmellus, right", [3031])},
    "MusculusGracilis": {"7:868": ("Musculus gracilis", [3031])},
    "MusculusGracilisLeft": {"7:869": ("Musculus gracilis, left", [3031])},
    "MusculusGracilisRight": {"7:870": ("Musculus gracilis, right", [3031])},
    "MusculusIliopsoas": {"7:792": ("Musculus iliopsoas", [3031])},
    "MusculusIliopsoasLeft": {"7:793": ("Musculus iliopsoas, left", [3031])},
    "MusculusIliopsoasRight": {"7:794": ("Musculus iliopsoas, right", [3031])},
    "MusculusInfraspinatus": {"7:612": ("Musculus infraspinatus", [3031])},
    "MusculusInfraspinatusLeft": {"7:613": ("Musculus infraspinatus, left", [3031])},
    "MusculusInfraspinatusRight": {"7:614": ("Musculus infraspinatus, right", [3031])},
    "MusculusInterosseiDorsales": {"7:988": ("Musculus interossei dorsales", [3031])},
    "MusculusInterosseiDorsalesLeft": {
        "7:989": ("Musculus interossei dorsales, left", [3031])
    },
    "MusculusInterosseiDorsalesRight": {
        "7:990": ("Musculus interossei dorsales, right", [3031])
    },
    "MusculusInterosseiPlantares": {"7:992": ("Musculus interossei plantares", [3031])},
    "MusculusInterosseiPlantaresLeft": {
        "7:993": ("Musculus interossei plantares, left", [3031])
    },
    "MusculusInterosseiPlantaresRight": {
        "7:994": ("Musculus interossei plantares, right", [3031])
    },
    "MusculusLatissimusDorsi": {"7:440": ("Musculus latissimus dorsi", [3031])},
    "MusculusLatissimusDorsiLeft": {
        "7:441": ("Musculus latissimus dorsi, left", [3031])
    },
    "MusculusLatissimusDorsiRight": {
        "7:442": ("Musculus latissimus dorsi, right", [3031])
    },
    "MusculusLevatorAnguliOris": {"7:336": ("Musculus levator anguli oris", [3031])},
    "MusculusLevatorAnguliOrisLeft": {
        "7:337": ("Musculus levator anguli oris, left", [3031])
    },
    "MusculusLevatorAnguliOrisRight": {
        "7:338": ("Musculus levator anguli oris, right", [3031])
    },
    "MusculusLevatorLabiiSuperioris": {
        "7:324": ("Musculus levator labii superioris", [3031])
    },
    "MusculusLevatorLabiiSuperiorisAlaequeNasi": {
        "7:328": ("Musculus levator labii superioris alaeque nasi", [3031])
    },
    "MusculusLevatorLabiiSuperiorisAlaequeNasiLeft": {
        "7:329": ("Musculus levator labii superioris alaeque nasi, left", [3031])
    },
    "MusculusLevatorLabiiSuperiorisAlaequeNasiRight": {
        "7:330": ("Musculus levator labii superioris alaeque nasi, right", [3031])
    },
    "MusculusLevatorLabiiSuperiorisLeft": {
        "7:325": ("Musculus levator labii superioris, left", [3031])
    },
    "MusculusLevatorLabiiSuperiorisRight": {
        "7:326": ("Musculus levator labii superioris, right", [3031])
    },
    "MusculusLevatorScapulae": {"7:452": ("Musculus levator scapulae", [3031])},
    "MusculusLevatorScapulaeLeft": {
        "7:453": ("Musculus levator scapulae, left", [3031])
    },
    "MusculusLevatorScapulaeRight": {
        "7:454": ("Musculus levator scapulae, right", [3031])
    },
    "MusculusMasseter": {"7:348": ("Musculus masseter", [3031])},
    "MusculusMasseterLeft": {"7:349": ("Musculus masseter, left", [3031])},
    "MusculusMasseterRight": {"7:350": ("Musculus masseter, right", [3031])},
    "MusculusMentalis": {"7:344": ("Musculus mentalis", [3031])},
    "MusculusMentalisLeft": {"7:345": ("Musculus mentalis, left", [3031])},
    "MusculusMentalisRight": {"7:346": ("Musculus mentalis, right", [3031])},
    "MusculusMylohyoideus": {"7:416": ("Musculus mylohyoideus", [3031])},
    "MusculusMylohyoideusLeft": {"7:417": ("Musculus mylohyoideus, left", [3031])},
    "MusculusMylohyoideusRight": {"7:418": ("Musculus mylohyoideus, right", [3031])},
    "MusculusObliquusExternusAbdominis": {
        "7:564": ("Musculus obliquus externus abdominis", [3031])
    },
    "MusculusObliquusExternusAbdominisLeft": {
        "7:565": ("Musculus obliquus externus abdominis, left", [3031])
    },
    "MusculusObliquusExternusAbdominisRight": {
        "7:566": ("Musculus obliquus externus abdominis, right", [3031])
    },
    "MusculusObliquusInferior": {"7:280": ("Musculus obliquus inferior", [3031])},
    "MusculusObliquusInferiorLeft": {
        "7:281": ("Musculus obliquus inferior, left", [3031])
    },
    "MusculusObliquusInferiorRight": {
        "7:282": ("Musculus obliquus inferior, right", [3031])
    },
    "MusculusObliquusInternusAbdominis": {
        "7:568": ("Musculus obliquus internus abdominis", [3031])
    },
    "MusculusObliquusInternusAbdominisLeft": {
        "7:569": ("Musculus obliquus internus abdominis, left", [3031])
    },
    "MusculusObliquusInternusAbdominisRight": {
        "7:570": ("Musculus obliquus internus abdominis, right", [3031])
    },
    "MusculusObliquusSuperior": {"7:276": ("Musculus obliquus superior", [3031])},
    "MusculusObliquusSuperiorLeft": {
        "7:277": ("Musculus obliquus superior, left", [3031])
    },
    "MusculusObliquusSuperiorRight": {
        "7:278": ("Musculus obliquus superior, right", [3031])
    },
    "MusculusObturator": {"7:816": ("Musculus obturator", [3031])},
    "MusculusObturatorLeft": {"7:817": ("Musculus obturator, left", [3031])},
    "MusculusObturatorRight": {"7:818": ("Musculus obturator, right", [3031])},
    "MusculusOccipitofrontalisVenterFrontalis": {
        "7:288": ("Musculus occipitofrontalis, Venter frontalis", [3031])
    },
    "MusculusOccipitofrontalisVenterFrontalisLeft": {
        "7:289": ("Musculus occipitofrontalis, Venter frontalis, left", [3031])
    },
    "MusculusOccipitofrontalisVenterFrontalisRight": {
        "7:290": ("Musculus occipitofrontalis, Venter frontalis, right", [3031])
    },
    "MusculusOpponensDigitiMinimi": {
        "7:764": ("Musculus opponens digiti minimi", [3031])
    },
    "MusculusOpponensDigitiMinimiLeft": {
        "7:765": ("Musculus opponens digiti minimi, left", [3031])
    },
    "MusculusOpponensDigitiMinimiRight": {
        "7:766": ("Musculus opponens digiti minimi, right", [3031])
    },
    "MusculusOpponensPollicis": {"7:748": ("Musculus opponens pollicis", [3031])},
    "MusculusOpponensPollicisLeft": {
        "7:749": ("Musculus opponens pollicis, left", [3031])
    },
    "MusculusOpponensPollicisRight": {
        "7:750": ("Musculus opponens pollicis, right", [3031])
    },
    "MusculusOrbicularisOculi": {"7:292": ("Musculus orbicularis oculi", [3031])},
    "MusculusOrbicularisOculiLeft": {
        "7:293": ("Musculus orbicularis oculi, left", [3031])
    },
    "MusculusOrbicularisOculiParsOrbitalis": {
        "7:296": ("Musculus orbicularis oculi, Pars orbitalis", [3031])
    },
    "MusculusOrbicularisOculiParsOrbitalisLeft": {
        "7:297": ("Musculus orbicularis oculi, Pars orbitalis, left", [3031])
    },
    "MusculusOrbicularisOculiParsOrbitalisRight": {
        "7:298": ("Musculus orbicularis oculi, Pars orbitalis, right", [3031])
    },
    "MusculusOrbicularisOculiRight": {
        "7:294": ("Musculus orbicularis oculi, right", [3031])
    },
    "MusculusOrbicularisOris": {"7:304": ("Musculus orbicularis oris", [3031])},
    "MusculusOrbicularisOrisLeft": {
        "7:305": ("Musculus orbicularis oris, left", [3031])
    },
    "MusculusOrbicularisOrisRight": {
        "7:306": ("Musculus orbicularis oris, right", [3031])
    },
    "MusculusPalmarisBrevis": {"7:736": ("Musculus palmaris brevis", [3031])},
    "MusculusPalmarisBrevisLeft": {"7:737": ("Musculus palmaris brevis, left", [3031])},
    "MusculusPalmarisBrevisRight": {
        "7:738": ("Musculus palmaris brevis, right", [3031])
    },
    "MusculusPalmarisLongus": {"7:668": ("Musculus palmaris longus", [3031])},
    "MusculusPalmarisLongusLeft": {"7:669": ("Musculus palmaris longus, left", [3031])},
    "MusculusPalmarisLongusRight": {
        "7:670": ("Musculus palmaris longus, right", [3031])
    },
    "MusculusPectineus": {"7:852": ("Musculus pectineus", [3031])},
    "MusculusPectineusLeft": {"7:853": ("Musculus pectineus, left", [3031])},
    "MusculusPectineusRight": {"7:854": ("Musculus pectineus, right", [3031])},
    "MusculusPectoralisMajor": {"7:532": ("Musculus pectoralis major", [3031])},
    "MusculusPectoralisMajorLeft": {
        "7:533": ("Musculus pectoralis major, left", [3031])
    },
    "MusculusPectoralisMajorRight": {
        "7:534": ("Musculus pectoralis major, right", [3031])
    },
    "MusculusPectoralisMinor": {"7:536": ("Musculus pectoralis minor", [3031])},
    "MusculusPectoralisMinorLeft": {
        "7:537": ("Musculus pectoralis minor, left", [3031])
    },
    "MusculusPectoralisMinorRight": {
        "7:538": ("Musculus pectoralis minor, right", [3031])
    },
    "MusculusPeroneusBrevis": {"7:912": ("Musculus peroneus brevis", [3031])},
    "MusculusPeroneusBrevisLeft": {"7:913": ("Musculus peroneus brevis, left", [3031])},
    "MusculusPeroneusBrevisRight": {
        "7:914": ("Musculus peroneus brevis, right", [3031])
    },
    "MusculusPeroneusLongus": {"7:908": ("Musculus peroneus longus", [3031])},
    "MusculusPeroneusLongusLeft": {"7:909": ("Musculus peroneus longus, left", [3031])},
    "MusculusPeroneusLongusRight": {
        "7:910": ("Musculus peroneus longus, right", [3031])
    },
    "MusculusPiriformis": {"7:812": ("Musculus piriformis", [3031])},
    "MusculusPiriformisLeft": {"7:813": ("Musculus piriformis, left", [3031])},
    "MusculusPiriformisRight": {"7:814": ("Musculus piriformis, right", [3031])},
    "MusculusPlantaris": {"7:936": ("Musculus plantaris", [3031])},
    "MusculusPlantarisLeft": {"7:937": ("Musculus plantaris, left", [3031])},
    "MusculusPlantarisRight": {"7:938": ("Musculus plantaris, right", [3031])},
    "MusculusPopliteus": {"7:940": ("Musculus popliteus", [3031])},
    "MusculusPopliteusLeft": {"7:941": ("Musculus popliteus, left", [3031])},
    "MusculusPopliteusRight": {"7:942": ("Musculus popliteus, right", [3031])},
    "MusculusPronatorQuadratus": {"7:688": ("Musculus pronator quadratus", [3031])},
    "MusculusPronatorQuadratusLeft": {
        "7:689": ("Musculus pronator quadratus, left", [3031])
    },
    "MusculusPronatorQuadratusRight": {
        "7:690": ("Musculus pronator quadratus, right", [3031])
    },
    "MusculusPronatorTeres": {"7:660": ("Musculus pronator teres", [3031])},
    "MusculusPronatorTeresLeft": {"7:661": ("Musculus pronator teres, left", [3031])},
    "MusculusPronatorTeresRight": {"7:662": ("Musculus pronator teres, right", [3031])},
    "MusculusPterygoideus": {"7:356": ("Musculus Pterygoideus", [3031])},
    "MusculusPterygoideusLateralis": {
        "7:360": ("Musculus Pterygoideus lateralis", [3031])
    },
    "MusculusPterygoideusLateralisLeft": {
        "7:361": ("Musculus Pterygoideus lateralis, left", [3031])
    },
    "MusculusPterygoideusLateralisRight": {
        "7:362": ("Musculus Pterygoideus lateralis, right", [3031])
    },
    "MusculusPterygoideusLeft": {"7:357": ("Musculus Pterygoideus, left", [3031])},
    "MusculusPterygoideusMedialis": {
        "7:364": ("Musculus Pterygoideus, medialis", [3031])
    },
    "MusculusPterygoideusMedialisLeft": {
        "7:365": ("Musculus Pterygoideus, medialis, left", [3031])
    },
    "MusculusPterygoideusMedialisRight": {
        "7:366": ("Musculus Pterygoideus, medialis, right", [3031])
    },
    "MusculusPterygoideusRight": {"7:358": ("Musculus Pterygoideus, right", [3031])},
    "MusculusPuborectalis": {"7:584": ("Musculus puborectalis", [3031])},
    "MusculusPuborectalisLeft": {"7:585": ("Musculus puborectalis, left", [3031])},
    "MusculusPuborectalisRight": {"7:586": ("Musculus puborectalis, right", [3031])},
    "MusculusQuadratusFemoris": {"7:824": ("Musculus quadratus femoris", [3031])},
    "MusculusQuadratusFemorisLeft": {
        "7:825": ("Musculus quadratus femoris, left", [3031])
    },
    "MusculusQuadratusFemorisRight": {
        "7:826": ("Musculus quadratus femoris, right", [3031])
    },
    "MusculusQuadratusLumborum": {"7:576": ("Musculus quadratus lumborum", [3031])},
    "MusculusQuadratusLumborumLeft": {
        "7:577": ("Musculus quadratus lumborum, left", [3031])
    },
    "MusculusQuadratusLumborumRight": {
        "7:578": ("Musculus quadratus lumborum, right", [3031])
    },
    "MusculusQuadratusPlantae": {"7:980": ("Musculus quadratus plantae", [3031])},
    "MusculusQuadratusPlantaeLeft": {
        "7:981": ("Musculus quadratus plantae, left", [3031])
    },
    "MusculusQuadratusPlantaeRight": {
        "7:982": ("Musculus quadratus plantae, right", [3031])
    },
    "MusculusQuadricepsFemoris": {"7:832": ("Musculus quadriceps femoris", [3031])},
    "MusculusQuadricepsFemorisLeft": {
        "7:833": ("Musculus quadriceps femoris, left", [3031])
    },
    "MusculusQuadricepsFemorisRight": {
        "7:834": ("Musculus quadriceps femoris, right", [3031])
    },
    "MusculusRectusAbdominis": {"7:560": ("Musculus rectus abdominis", [3031])},
    "MusculusRectusAbdominisLeft": {
        "7:561": ("Musculus rectus abdominis, left", [3031])
    },
    "MusculusRectusAbdominisRight": {
        "7:562": ("Musculus rectus abdominis, right", [3031])
    },
    "MusculusRectusFemoris": {"7:836": ("Musculus rectus femoris", [3031])},
    "MusculusRectusFemorisLeft": {"7:837": ("Musculus rectus femoris, left", [3031])},
    "MusculusRectusFemorisRight": {"7:838": ("Musculus rectus femoris, right", [3031])},
    "MusculusRectusInferior": {"7:264": ("Musculus rectus inferior", [3031])},
    "MusculusRectusInferiorLeft": {"7:265": ("Musculus rectus inferior, left", [3031])},
    "MusculusRectusInferiorRight": {
        "7:266": ("Musculus rectus inferior, right", [3031])
    },
    "MusculusRectusLateralis": {"7:272": ("Musculus rectus lateralis", [3031])},
    "MusculusRectusLateralisLeft": {
        "7:273": ("Musculus rectus lateralis, left", [3031])
    },
    "MusculusRectusLateralisRight": {
        "7:274": ("Musculus rectus lateralis, right", [3031])
    },
    "MusculusRectusMedialis": {"7:268": ("Musculus rectus medialis", [3031])},
    "MusculusRectusMedialisLeft": {"7:269": ("Musculus rectus medialis, left", [3031])},
    "MusculusRectusMedialisRight": {
        "7:270": ("Musculus rectus medialis, right", [3031])
    },
    "MusculusRectusSuperior": {"7:260": ("Musculus rectus superior", [3031])},
    "MusculusRectusSuperiorLeft": {"7:261": ("Musculus rectus superior, left", [3031])},
    "MusculusRectusSuperiorRight": {
        "7:262": ("Musculus rectus superior, right", [3031])
    },
    "MusculusRhomboideusMajor": {"7:444": ("Musculus rhomboideus major", [3031])},
    "MusculusRhomboideusMajorLeft": {
        "7:445": ("Musculus rhomboideus major, left", [3031])
    },
    "MusculusRhomboideusMajorRight": {
        "7:446": ("Musculus rhomboideus major, right", [3031])
    },
    "MusculusRhomboideusMinor": {"7:448": ("Musculus rhomboideus minor", [3031])},
    "MusculusRhomboideusMinorLeft": {
        "7:449": ("Musculus rhomboideus minor, left", [3031])
    },
    "MusculusRhomboideusMinorRight": {
        "7:450": ("Musculus rhomboideus minor, right", [3031])
    },
    "MusculusRisorius": {"7:312": ("Musculus risorius", [3031])},
    "MusculusRisoriusLeft": {"7:313": ("Musculus risorius, left", [3031])},
    "MusculusRisoriusRight": {"7:314": ("Musculus risorius, right", [3031])},
    "MusculusSartorius": {"7:828": ("Musculus sartorius", [3031])},
    "MusculusSartoriusLeft": {"7:829": ("Musculus sartorius, left", [3031])},
    "MusculusSartoriusRight": {"7:830": ("Musculus sartorius, right", [3031])},
    "MusculusSemimembranosus": {"7:888": ("Musculus semimembranosus", [3031])},
    "MusculusSemimembranosusLeft": {
        "7:889": ("Musculus semimembranosus, left", [3031])
    },
    "MusculusSemimembranosusRight": {
        "7:890": ("Musculus semimembranosus, right", [3031])
    },
    "MusculusSemispinalis": {"7:492": ("Musculus semispinalis", [3031])},
    "MusculusSemispinalisCapitis": {"7:504": ("Musculus semispinalis capitis", [3031])},
    "MusculusSemispinalisCapitisLeft": {
        "7:505": ("Musculus semispinalis capitis, left", [3031])
    },
    "MusculusSemispinalisCapitisRight": {
        "7:506": ("Musculus semispinalis capitis, right", [3031])
    },
    "MusculusSemispinalisCervicis": {
        "7:500": ("Musculus semispinalis cervicis", [3031])
    },
    "MusculusSemispinalisCervicisLeft": {
        "7:501": ("Musculus semispinalis cervicis, left", [3031])
    },
    "MusculusSemispinalisCervicisRight": {
        "7:502": ("Musculus semispinalis cervicis, right", [3031])
    },
    "MusculusSemispinalisLeft": {"7:493": ("Musculus semispinalis, left", [3031])},
    "MusculusSemispinalisRight": {"7:494": ("Musculus semispinalis, right", [3031])},
    "MusculusSemispinalisThoracis": {
        "7:496": ("Musculus semispinalis thoracis", [3031])
    },
    "MusculusSemispinalisThoracisLeft": {
        "7:497": ("Musculus semispinalis thoracis, left", [3031])
    },
    "MusculusSemispinalisThoracisRight": {
        "7:498": ("Musculus semispinalis thoracis, right", [3031])
    },
    "MusculusSemitendinosus": {"7:884": ("Musculus semitendinosus", [3031])},
    "MusculusSemitendinosusLeft": {"7:885": ("Musculus semitendinosus, left", [3031])},
    "MusculusSemitendinosusRight": {
        "7:886": ("Musculus semitendinosus, right", [3031])
    },
    "MusculusSerratusAnterior": {"7:544": ("Musculus serratus anterior", [3031])},
    "MusculusSerratusAnteriorLeft": {
        "7:545": ("Musculus serratus anterior, left", [3031])
    },
    "MusculusSerratusAnteriorRight": {
        "7:546": ("Musculus serratus anterior, right", [3031])
    },
    "MusculusSerratusPosterior": {"7:456": ("Musculus serratus posterior", [3031])},
    "MusculusSerratusPosteriorLeft": {
        "7:457": ("Musculus serratus posterior, left", [3031])
    },
    "MusculusSerratusPosteriorRight": {
        "7:458": ("Musculus serratus posterior, right", [3031])
    },
    "MusculusSoleus": {"7:932": ("Musculus soleus", [3031])},
    "MusculusSoleusLeft": {"7:933": ("Musculus soleus, left", [3031])},
    "MusculusSoleusRight": {"7:934": ("Musculus soleus, right", [3031])},
    "MusculusSphincterAni": {"7:592": ("Musculus sphincter ani", [3031])},
    "MusculusSphincterAniExternus": {
        "7:596": ("Musculus sphincter ani externus", [3031])
    },
    "MusculusSpinalis": {"7:476": ("Musculus spinalis", [3031])},
    "MusculusSpinalisCapitis": {"7:488": ("Musculus spinalis capitis", [3031])},
    "MusculusSpinalisCapitisLeft": {
        "7:489": ("Musculus spinalis capitis, left", [3031])
    },
    "MusculusSpinalisCapitisRight": {
        "7:490": ("Musculus spinalis capitis, right", [3031])
    },
    "MusculusSpinalisCervicis": {"7:484": ("Musculus spinalis cervicis", [3031])},
    "MusculusSpinalisCervicisLeft": {
        "7:485": ("Musculus spinalis cervicis, left", [3031])
    },
    "MusculusSpinalisCervicisRight": {
        "7:486": ("Musculus spinalis cervicis, right", [3031])
    },
    "MusculusSpinalisLeft": {"7:477": ("Musculus spinalis, left", [3031])},
    "MusculusSpinalisRight": {"7:478": ("Musculus spinalis, right", [3031])},
    "MusculusSpinalisThoracis": {"7:480": ("Musculus spinalis thoracis", [3031])},
    "MusculusSpinalisThoracisLeft": {
        "7:481": ("Musculus spinalis thoracis, left", [3031])
    },
    "MusculusSpinalisThoracisRight": {
        "7:482": ("Musculus spinalis thoracis, right", [3031])
    },
    "MusculusSplenius": {"7:468": ("Musculus splenius", [3031])},
    "MusculusSpleniusCapitis": {"7:460": ("Musculus splenius capitis", [3031])},
    "MusculusSpleniusCapitisLeft": {
        "7:461": ("Musculus splenius capitis, left", [3031])
    },
    "MusculusSpleniusCapitisRight": {
        "7:462": ("Musculus splenius capitis, right", [3031])
    },
    "MusculusSpleniusCervicis": {"7:464": ("Musculus splenius cervicis", [3031])},
    "MusculusSpleniusCervicisLeft": {
        "7:465": ("Musculus splenius cervicis, left", [3031])
    },
    "MusculusSpleniusCervicisRight": {
        "7:466": ("Musculus splenius cervicis, right", [3031])
    },
    "MusculusSpleniusLeft": {"7:469": ("Musculus splenius, left", [3031])},
    "MusculusSpleniusRight": {"7:470": ("Musculus splenius, right", [3031])},
    "MusculusSternocleidomastoideus": {
        "7:400": ("Musculus Sternocleidomastoideus", [3031])
    },
    "MusculusSternocleidomastoideusLeft": {
        "7:401": ("Musculus Sternocleidomastoideus, left", [3031])
    },
    "MusculusSternocleidomastoideusRight": {
        "7:402": ("Musculus Sternocleidomastoideus, right", [3031])
    },
    "MusculusSubclavius": {"7:540": ("Musculus subclavius", [3031])},
    "MusculusSubclaviusLeft": {"7:541": ("Musculus subclavius, left", [3031])},
    "MusculusSubclaviusRight": {"7:542": ("Musculus subclavius, right", [3031])},
    "MusculusSubscapularis": {"7:624": ("Musculus subscapularis", [3031])},
    "MusculusSubscapularisLeft": {"7:625": ("Musculus subscapularis, left", [3031])},
    "MusculusSubscapularisRight": {"7:626": ("Musculus subscapularis, right", [3031])},
    "MusculusSupinator": {"7:716": ("Musculus supinator", [3031])},
    "MusculusSupinatorLeft": {"7:717": ("Musculus supinator, left", [3031])},
    "MusculusSupinatorRight": {"7:718": ("Musculus supinator, right", [3031])},
    "MusculusSupraspinatus": {"7:608": ("Musculus supraspinatus", [3031])},
    "MusculusSupraspinatusLeft": {"7:609": ("Musculus supraspinatus, left", [3031])},
    "MusculusSupraspinatusRight": {"7:610": ("Musculus supraspinatus, right", [3031])},
    "MusculusTemporalis": {"7:352": ("Musculus temporalis", [3031])},
    "MusculusTemporalisLeft": {"7:353": ("Musculus temporalis, left", [3031])},
    "MusculusTemporalisRight": {"7:354": ("Musculus temporalis, right", [3031])},
    "MusculusTensorFasciaeLatae": {"7:808": ("Musculus tensor fasciae latae", [3031])},
    "MusculusTensorFasciaeLataeLeft": {
        "7:809": ("Musculus tensor fasciae latae, left", [3031])
    },
    "MusculusTensorFasciaeLataeRight": {
        "7:810": ("Musculus tensor fasciae latae, right", [3031])
    },
    "MusculusTeresMajor": {"7:620": ("Musculus teres major", [3031])},
    "MusculusTeresMajorLeft": {"7:621": ("Musculus teres major, left", [3031])},
    "MusculusTeresMajorRight": {"7:622": ("Musculus teres major, right", [3031])},
    "MusculusTeresMinor": {"7:616": ("Musculus teres minor", [3031])},
    "MusculusTeresMinorLeft": {"7:617": ("Musculus teres minor, left", [3031])},
    "MusculusTeresMinorRight": {"7:618": ("Musculus teres minor, right", [3031])},
    "MusculusTibialisAnterior": {"7:892": ("Musculus tibialis anterior", [3031])},
    "MusculusTibialisAnteriorLeft": {
        "7:893": ("Musculus tibialis anterior, left", [3031])
    },
    "MusculusTibialisAnteriorRight": {
        "7:894": ("Musculus tibialis anterior, right", [3031])
    },
    "MusculusTibialisPosterior": {"7:944": ("Musculus tibialis posterior", [3031])},
    "MusculusTibialisPosteriorLeft": {
        "7:945": ("Musculus tibialis posterior, left", [3031])
    },
    "MusculusTibialisPosteriorRight": {
        "7:946": ("Musculus tibialis posterior, right", [3031])
    },
    "MusculusTransversusAbdominis": {
        "7:572": ("Musculus transversus abdominis", [3031])
    },
    "MusculusTransversusAbdominisLeft": {
        "7:573": ("Musculus transversus abdominis, left", [3031])
    },
    "MusculusTransversusAbdominisRight": {
        "7:574": ("Musculus transversus abdominis, right", [3031])
    },
    "MusculusTrapezius": {"7:436": ("Musculus trapezius", [3031])},
    "MusculusTrapeziusLeft": {"7:437": ("Musculus trapezius, left", [3031])},
    "MusculusTrapeziusRight": {"7:438": ("Musculus trapezius, right", [3031])},
    "MusculusTricepsBrachii": {"7:640": ("Musculus triceps brachii", [3031])},
    "MusculusTricepsBrachiiCaputLaterale": {
        "7:648": ("Musculus triceps brachii, Caput laterale", [3031])
    },
    "MusculusTricepsBrachiiCaputLateraleLeft": {
        "7:649": ("Musculus triceps brachii, Caput laterale, left", [3031])
    },
    "MusculusTricepsBrachiiCaputLateraleRight": {
        "7:650": ("Musculus triceps brachii, Caput laterale, right", [3031])
    },
    "MusculusTricepsBrachiiCaputLongum": {
        "7:644": ("Musculus triceps brachii, Caput longum", [3031])
    },
    "MusculusTricepsBrachiiCaputLongumLeft": {
        "7:645": ("Musculus triceps brachii, Caput longum, left", [3031])
    },
    "MusculusTricepsBrachiiCaputLongumRight": {
        "7:646": ("Musculus triceps brachii, Caput longum, right", [3031])
    },
    "MusculusTricepsBrachiiCaputMediale": {
        "7:652": ("Musculus triceps brachii, Caput mediale", [3031])
    },
    "MusculusTricepsBrachiiCaputMedialeLeft": {
        "7:653": ("Musculus triceps brachii, Caput mediale, left", [3031])
    },
    "MusculusTricepsBrachiiCaputMedialeRight": {
        "7:654": ("Musculus triceps brachii, Caput mediale, right", [3031])
    },
    "MusculusTricepsBrachiiLeft": {"7:641": ("Musculus triceps brachii, left", [3031])},
    "MusculusTricepsBrachiiRight": {
        "7:642": ("Musculus triceps brachii, right", [3031])
    },
    "MusculusTyroartenoideus": {"7:384": ("Musculus tyroartenoideus", [3031])},
    "MusculusTyroartenoideusLeft": {
        "7:385": ("Musculus tyroartenoideus, left", [3031])
    },
    "MusculusTyroartenoideusRight": {
        "7:386": ("Musculus tyroartenoideus, right", [3031])
    },
    "MusculusVastusIntermedius": {"7:844": ("Musculus vastus intermedius", [3031])},
    "MusculusVastusIntermediusLeft": {
        "7:845": ("Musculus vastus intermedius, left", [3031])
    },
    "MusculusVastusIntermediusRight": {
        "7:846": ("Musculus vastus intermedius, right", [3031])
    },
    "MusculusVastusLateralis": {"7:840": ("Musculus vastus lateralis", [3031])},
    "MusculusVastusLateralisLeft": {
        "7:841": ("Musculus vastus lateralis, left", [3031])
    },
    "MusculusVastusLateralisRight": {
        "7:842": ("Musculus vastus lateralis, right", [3031])
    },
    "MusculusVastusMedialis": {"7:848": ("Musculus vastus medialis", [3031])},
    "MusculusVastusMedialisLeft": {"7:849": ("Musculus vastus medialis, left", [3031])},
    "MusculusVastusMedialisRight": {
        "7:850": ("Musculus vastus medialis, right", [3031])
    },
    "MusculusZygomaticusMajor": {"7:316": ("Musculus zygomaticus major", [3031])},
    "MusculusZygomaticusMajorLeft": {
        "7:317": ("Musculus zygomaticus major, left", [3031])
    },
    "MusculusZygomaticusMajorRight": {
        "7:318": ("Musculus zygomaticus major, right", [3031])
    },
    "MusculusZygomaticusMinor": {"7:320": ("Musculus zygomaticus minor", [3031])},
    "MusculusZygomaticusMinorLeft": {
        "7:321": ("Musculus zygomaticus minor, left", [3031])
    },
    "MusculusZygomaticusMinorRight": {
        "7:322": ("Musculus zygomaticus minor, right", [3031])
    },
    "MusculustricepsSurae": {"7:916": ("Musculustriceps surae", [3031])},
    "MusculustricepsSuraeLeft": {"7:917": ("Musculustriceps surae, left", [3031])},
    "MusculustricepsSuraeRight": {"7:918": ("Musculustriceps surae, right", [3031])},
    "MyoclonicActivity": {"2:24176": ("Myoclonic activity", [3035])},
    "MyogenicHemifacialSpasms": {"2:24152": ("Myogenic hemifacial spasms", [3035])},
    "MyogenicNoncerebralActivity": {
        "2:24120": ("Myogenic noncerebral activity", [3035])
    },
    "MyogenicNoncerebralFacialSynkinesis": {
        "2:24144": ("Myogenic noncerebral facial synkinesis", [3035])
    },
    "MyogenicNoncerebralMyokymia": {
        "2:24136": ("Myogenic noncerebral myokymia", [3035])
    },
    "MyogenicPalatalMyoclonus": {"2:24128": ("Myogenic palatal myoclonus", [3035])},
    "MyogenicTremorActivity": {"2:24168": ("Myogenic tremor activity", [3035])},
    "NEHBLeadSystem": {"10:11280": ("NEHB lead system", [3263])},
    "NegativeLowRightScapulaLead": {
        "2:124": ("negative: low right scapula Lead", [3001])
    },
    "NegativeRightInfraclavicularFossa": {
        "2:123": ("negative: right infraclavicular fossa", [3001])
    },
    "NerviCervicales": {"7:92": ("Nervi cervicales", [3032])},
    "NerviCervicalesLeft": {"7:93": ("Nervi cervicales, left", [3032])},
    "NerviCervicalesRight": {"7:94": ("Nervi cervicales, right", [3032])},
    "NerviCraniales": {"7:8": ("Nervi craniales", [3032])},
    "NerviCranialesLeft": {"7:9": ("Nervi craniales, left", [3032])},
    "NerviCranialesRight": {"7:10": ("Nervi craniales, right", [3032])},
    "NerviLumbales": {"7:168": ("Nervi lumbales", [3032])},
    "NerviLumbalesLeft": {"7:169": ("Nervi lumbales, left", [3032])},
    "NerviLumbalesRight": {"7:170": ("Nervi lumbales, right", [3032])},
    "NerviSacralesAtNervusCoccygeus": {
        "7:204": ("Nervi sacrales at Nervus coccygeus", [3032])
    },
    "NerviSacralesAtNervusCoccygeusLeft": {
        "7:205": ("Nervi sacrales at Nervus coccygeus, left", [3032])
    },
    "NerviSacralesAtNervusCoccygeusRight": {
        "7:206": ("Nervi sacrales at Nervus coccygeus, right", [3032])
    },
    "NerviSpinales": {"7:88": ("Nervi spinales", [3032])},
    "NerviSpinalesLeft": {"7:89": ("Nervi spinales, left", [3032])},
    "NerviSpinalesRight": {"7:90": ("Nervi spinales, right", [3032])},
    "NerviSubscapulares": {"7:156": ("Nervi subscapulares", [3032])},
    "NerviSubscapularesLeft": {"7:157": ("Nervi subscapulares, left", [3032])},
    "NerviSubscapularesRight": {"7:158": ("Nervi subscapulares, right", [3032])},
    "NerviThoracici": {"7:164": ("Nervi thoracici", [3032])},
    "NerviThoraciciLeft": {"7:165": ("Nervi thoracici, left", [3032])},
    "NerviThoraciciRight": {"7:166": ("Nervi thoracici, right", [3032])},
    "NervusAbducensVI": {"7:48": ("Nervus abducens (VI)", [3032])},
    "NervusAbducensVILeft": {"7:49": ("Nervus abducens (VI), left", [3032])},
    "NervusAbducensVIRight": {"7:50": ("Nervus abducens (VI), right", [3032])},
    "NervusAcessoriusXIRadicesCraniales": {
        "7:76": ("Nervus acessorius (XI), Radices craniales", [3032])
    },
    "NervusAcessoriusXIRadicesCranialesLeft": {
        "7:77": ("Nervus acessorius (XI), Radices craniales, left", [3032])
    },
    "NervusAcessoriusXIRadicesCranialesRight": {
        "7:78": ("Nervus acessorius (XI), Radices craniales, right", [3032])
    },
    "NervusAcessoriusXIRadicesSpinales": {
        "7:80": ("Nervus acessorius (XI), Radices spinales", [3032])
    },
    "NervusAcessoriusXIRadicesSpinalesLeft": {
        "7:81": ("Nervus acessorius (XI), Radices spinales, left", [3032])
    },
    "NervusAcessoriusXIRadicesSpinalesRight": {
        "7:82": ("Nervus acessorius (XI), Radices spinales, right", [3032])
    },
    "NervusAxillaris": {"7:160": ("Nervus axillaris", [3032])},
    "NervusAxillarisLeft": {"7:161": ("Nervus axillaris, left", [3032])},
    "NervusAxillarisRight": {"7:162": ("Nervus axillaris, right", [3032])},
    "NervusCochlearis": {"7:64": ("Nervus cochlearis", [3032])},
    "NervusCochlearisLeft": {"7:65": ("Nervus cochlearis, left", [3032])},
    "NervusCochlearisRight": {"7:66": ("Nervus cochlearis, right", [3032])},
    "NervusCutaneusAntebrachiiLateralis": {
        "7:112": ("Nervus cutaneus antebrachii lateralis", [3032])
    },
    "NervusCutaneusAntebrachiiLateralisLeft": {
        "7:113": ("Nervus cutaneus antebrachii lateralis, left", [3032])
    },
    "NervusCutaneusAntebrachiiLateralisRight": {
        "7:114": ("Nervus cutaneus antebrachii lateralis, right", [3032])
    },
    "NervusCutaneusAntebrachiiMedialis": {
        "7:116": ("Nervus cutaneus antebrachii medialis", [3032])
    },
    "NervusCutaneusAntebrachiiMedialisLeft": {
        "7:117": ("Nervus cutaneus antebrachii medialis, left", [3032])
    },
    "NervusCutaneusAntebrachiiMedialisRight": {
        "7:118": ("Nervus cutaneus antebrachii medialis, right", [3032])
    },
    "NervusCutaneusFemorisLateralis": {
        "7:188": ("Nervus cutaneus femoris lateralis", [3032])
    },
    "NervusCutaneusFemorisLateralisLeft": {
        "7:189": ("Nervus cutaneus femoris lateralis, left", [3032])
    },
    "NervusCutaneusFemorisLateralisRight": {
        "7:190": ("Nervus cutaneus femoris lateralis, right", [3032])
    },
    "NervusFacialisVII": {"7:52": ("Nervus facialis (VII)", [3032])},
    "NervusFacialisVIILeft": {"7:53": ("Nervus facialis (VII), left", [3032])},
    "NervusFacialisVIIRight": {"7:54": ("Nervus facialis (VII), right", [3032])},
    "NervusFemoralis": {"7:196": ("Nervus femoralis", [3032])},
    "NervusFemoralisLeft": {"7:197": ("Nervus femoralis, left", [3032])},
    "NervusFemoralisRight": {"7:198": ("Nervus femoralis, right", [3032])},
    "NervusFibularisCommunis": {"7:216": ("Nervus fibularis communis", [3032])},
    "NervusFibularisCommunisLeft": {
        "7:217": ("Nervus fibularis communis, left", [3032])
    },
    "NervusFibularisCommunisRight": {
        "7:218": ("Nervus fibularis communis, right", [3032])
    },
    "NervusFibularisProfundus": {"7:220": ("Nervus fibularis profundus", [3032])},
    "NervusFibularisProfundusLeft": {
        "7:221": ("Nervus fibularis profundus, left", [3032])
    },
    "NervusFibularisProfundusRight": {
        "7:222": ("Nervus fibularis profundus, right", [3032])
    },
    "NervusFibularisSuperficialis": {
        "7:224": ("Nervus fibularis superficialis", [3032])
    },
    "NervusFibularisSuperficialisLeft": {
        "7:225": ("Nervus fibularis superficialis, left", [3032])
    },
    "NervusFibularisSuperficialisRight": {
        "7:226": ("Nervus fibularis superficialis, right", [3032])
    },
    "NervusGlossopharingeusIX": {"7:68": ("Nervus glossopharingeus (IX)", [3032])},
    "NervusGlossopharingeusIXLeft": {
        "7:69": ("Nervus glossopharingeus (IX), left", [3032])
    },
    "NervusGlossopharingeusIXRight": {
        "7:70": ("Nervus glossopharingeus (IX), right", [3032])
    },
    "NervusHypoglossusXII": {"7:84": ("Nervus hypoglossus (XII)", [3032])},
    "NervusHypoglossusXIILeft": {"7:85": ("Nervus hypoglossus (XII), left", [3032])},
    "NervusHypoglossusXIIRight": {"7:86": ("Nervus hypoglossus (XII), right", [3032])},
    "NervusIlioInguinalis": {"7:184": ("Nervus ilio-inguinalis", [3032])},
    "NervusIlioInguinalisLeft": {"7:185": ("Nervus ilio-inguinalis, left", [3032])},
    "NervusIlioInguinalisRight": {"7:186": ("Nervus ilio-inguinalis, right", [3032])},
    "NervusIliohypogastricus": {"7:180": ("Nervus iliohypogastricus", [3032])},
    "NervusIliohypogastricusLeft": {
        "7:181": ("Nervus iliohypogastricus, left", [3032])
    },
    "NervusIliohypogastricusRight": {
        "7:182": ("Nervus iliohypogastricus, right", [3032])
    },
    "NervusInfraorbitalis": {"7:40": ("Nervus infraorbitalis", [3032])},
    "NervusInfraorbitalisLeft": {"7:41": ("Nervus infraorbitalis, left", [3032])},
    "NervusInfraorbitalisRight": {"7:42": ("Nervus infraorbitalis, right", [3032])},
    "NervusIschiadicus": {"7:212": ("Nervus ischiadicus", [3032])},
    "NervusIschiadicusLeft": {"7:213": ("Nervus ischiadicus, left", [3032])},
    "NervusIschiadicusRight": {"7:214": ("Nervus ischiadicus, right", [3032])},
    "NervusMandibularis": {"7:44": ("Nervus mandibularis", [3032])},
    "NervusMandibularisLeft": {"7:45": ("Nervus mandibularis, left", [3032])},
    "NervusMandibularisRight": {"7:46": ("Nervus mandibularis, right", [3032])},
    "NervusMaxillaris": {"7:36": ("Nervus maxillaris", [3032])},
    "NervusMaxillarisLeft": {"7:37": ("Nervus maxillaris, left", [3032])},
    "NervusMaxillarisRight": {"7:38": ("Nervus maxillaris, right", [3032])},
    "NervusMedianus": {"7:120": ("Nervus medianus", [3032])},
    "NervusMedianusLeft": {"7:121": ("Nervus medianus, left", [3032])},
    "NervusMedianusNerviDigitalesPalmaresProprii": {
        "7:128": ("Nervus medianus, Nervi digitales palmares proprii", [3032])
    },
    "NervusMedianusNerviDigitalesPalmaresPropriiLeft": {
        "7:129": ("Nervus medianus, Nervi digitales palmares proprii, left", [3032])
    },
    "NervusMedianusNerviDigitalesPalmaresPropriiRight": {
        "7:130": ("Nervus medianus, Nervi digitales palmares proprii, right", [3032])
    },
    "NervusMedianusRight": {"7:122": ("Nervus medianus, right", [3032])},
    "NervusMusculocutaneus": {"7:108": ("Nervus musculocutaneus", [3032])},
    "NervusMusculocutaneusLeft": {"7:109": ("Nervus musculocutaneus, left", [3032])},
    "NervusMusculocutaneusRight": {"7:110": ("Nervus musculocutaneus, right", [3032])},
    "NervusObturatorius": {"7:192": ("Nervus obturatorius", [3032])},
    "NervusObturatoriusLeft": {"7:193": ("Nervus obturatorius, left", [3032])},
    "NervusObturatoriusRight": {"7:194": ("Nervus obturatorius, right", [3032])},
    "NervusOculomotoriusIII": {"7:16": ("Nervus oculomotorius (III)", [3032])},
    "NervusOculomotoriusIIILeft": {
        "7:17": ("Nervus oculomotorius (III), left", [3032])
    },
    "NervusOculomotoriusIIIRight": {
        "7:18": ("Nervus oculomotorius (III), right", [3032])
    },
    "NervusOphtalmicus": {"7:28": ("Nervus ophtalmicus", [3032])},
    "NervusOphtalmicusLeft": {"7:29": ("Nervus ophtalmicus, left", [3032])},
    "NervusOphtalmicusRight": {"7:30": ("Nervus ophtalmicus, right", [3032])},
    "NervusOpticusII": {"7:12": ("Nervus opticus (II)", [3032])},
    "NervusOpticusIILeft": {"7:13": ("Nervus opticus (II), left", [3032])},
    "NervusOpticusIIRight": {"7:14": ("Nervus opticus (II), right", [3032])},
    "NervusPhrenicus": {"7:96": ("Nervus phrenicus", [3032])},
    "NervusPhrenicusLeft": {"7:97": ("Nervus phrenicus, left", [3032])},
    "NervusPhrenicusRight": {"7:98": ("Nervus phrenicus, right", [3032])},
    "NervusPlantarisLateralis": {"7:240": ("Nervus plantaris lateralis", [3032])},
    "NervusPlantarisLateralisLeft": {
        "7:241": ("Nervus plantaris lateralis, left", [3032])
    },
    "NervusPlantarisLateralisRight": {
        "7:242": ("Nervus plantaris lateralis, right", [3032])
    },
    "NervusPlantarisMedialis": {"7:236": ("Nervus plantaris medialis", [3032])},
    "NervusPlantarisMedialisLeft": {
        "7:237": ("Nervus plantaris medialis, left", [3032])
    },
    "NervusPlantarisMedialisRight": {
        "7:238": ("Nervus plantaris medialis, right", [3032])
    },
    "NervusPudendus": {"7:244": ("Nervus pudendus", [3032])},
    "NervusPudendusLeft": {"7:245": ("Nervus pudendus, left", [3032])},
    "NervusPudendusRight": {"7:246": ("Nervus pudendus, right", [3032])},
    "NervusRadialis": {"7:148": ("Nervus radialis", [3032])},
    "NervusRadialisLeft": {"7:149": ("Nervus radialis, left", [3032])},
    "NervusRadialisRamusSuperficialis": {
        "7:152": ("Nervus radialis Ramus superficialis", [3032])
    },
    "NervusRadialisRamusSuperficialisLeft": {
        "7:153": ("Nervus radialis Ramus superficialis, left", [3032])
    },
    "NervusRadialisRamusSuperficialisRight": {
        "7:154": ("Nervus radialis Ramus superficialis, right", [3032])
    },
    "NervusRadialisRight": {"7:150": ("Nervus radialis, right", [3032])},
    "NervusSaphenus": {"7:200": ("Nervus saphenus", [3032])},
    "NervusSaphenusLeft": {"7:201": ("Nervus saphenus, left", [3032])},
    "NervusSaphenusRight": {"7:202": ("Nervus saphenus, right", [3032])},
    "NervusSupraorbitalis": {"7:32": ("Nervus supraorbitalis", [3032])},
    "NervusSupraorbitalisLeft": {"7:33": ("Nervus supraorbitalis, left", [3032])},
    "NervusSupraorbitalisRight": {"7:34": ("Nervus supraorbitalis, right", [3032])},
    "NervusSurealis": {"7:232": ("Nervus surealis", [3032])},
    "NervusSurealisLeft": {"7:233": ("Nervus surealis, left", [3032])},
    "NervusSurealisRight": {"7:234": ("Nervus surealis, right", [3032])},
    "NervusThoracicusLongus": {"7:104": ("Nervus thoracicus longus", [3032])},
    "NervusThoracicusLongusLeft": {"7:105": ("Nervus thoracicus longus, left", [3032])},
    "NervusThoracicusLongusRight": {
        "7:106": ("Nervus thoracicus longus, right", [3032])
    },
    "NervusTibialis": {"7:228": ("Nervus tibialis", [3032])},
    "NervusTibialisLeft": {"7:229": ("Nervus tibialis, left", [3032])},
    "NervusTibialisRight": {"7:230": ("Nervus tibialis, right", [3032])},
    "NervusTrigeminusV": {"7:24": ("Nervus trigeminus (V)", [3032])},
    "NervusTrigeminusVLeft": {"7:25": ("Nervus trigeminus (V), left", [3032])},
    "NervusTrigeminusVRight": {"7:26": ("Nervus trigeminus (V), right", [3032])},
    "NervusTrochlearisIV": {"7:20": ("Nervus trochlearis (IV)", [3032])},
    "NervusTrochlearisIVLeft": {"7:21": ("Nervus trochlearis (IV), left", [3032])},
    "NervusTrochlearisIVRight": {"7:22": ("Nervus trochlearis (IV), right", [3032])},
    "NervusUlnaris": {"7:132": ("Nervus ulnaris", [3032])},
    "NervusUlnarisLeft": {"7:133": ("Nervus ulnaris, left", [3032])},
    "NervusUlnarisNerviDigitalesPalmaresProprii": {
        "7:144": ("Nervus ulnaris, Nervi digitales palmares proprii", [3032])
    },
    "NervusUlnarisNerviDigitalesPalmaresPropriiLeft": {
        "7:145": ("Nervus ulnaris, Nervi digitales palmares proprii, left", [3032])
    },
    "NervusUlnarisNerviDigitalesPalmaresPropriiRight": {
        "7:146": ("Nervus ulnaris, Nervi digitales palmares proprii, right", [3032])
    },
    "NervusUlnarisRight": {"7:134": ("Nervus ulnaris, right", [3032])},
    "NervusVagusX": {"7:72": ("Nervus vagus (X)", [3032])},
    "NervusVagusXLeft": {"7:73": ("Nervus vagus (X), left", [3032])},
    "NervusVagusXRight": {"7:74": ("Nervus vagus (X), right", [3032])},
    "NervusVestibularis": {"7:60": ("Nervus vestibularis", [3032])},
    "NervusVestibularisLeft": {"7:61": ("Nervus vestibularis, left", [3032])},
    "NervusVestibularisRight": {"7:62": ("Nervus vestibularis, right", [3032])},
    "NervusVestibulocochlearisVIII": {
        "7:56": ("Nervus vestibulocochlearis (VIII)", [3032])
    },
    "NervusVestibulocochlearisVIIILeft": {
        "7:57": ("Nervus vestibulocochlearis (VIII), left", [3032])
    },
    "NervusVestibulocochlearisVIIIRight": {
        "7:58": ("Nervus vestibulocochlearis (VIII), right", [3032])
    },
    "NoECGSignalIsAvailable": {"10:11248": ("No ECG signal is available", [3335])},
    "NoNoise": {"10:11200": ("No Noise", [3335, 3680])},
    "NoSignal": {"10:11248": ("No signal", [3680])},
    "NonConductedPWave": {"10:8320": ("Non-conducted p-wave (blocked)", [3335])},
    "NonRespiratorySinusArrhythmia": {
        "10:9312": ("Non-Respiratory Sinus Arrhythmia", [3415])
    },
    "NonsustainedVentricularTachycardia": {
        "10:10032": ("Nonsustained Ventricular Tachycardia (paroxysmal)", [3415])
    },
    "NormalBeatSinusBeatNormalConduction": {
        "10:8208": ("Normal beat (sinus beat, normal conduction)", [3335])
    },
    "NormalSinusRhythm": {"10:9232": ("Normal Sinus Rhythm", [3415])},
    "Notch": {"10:768": ("Notch", [3335])},
    "NotchFilter": {"10:11407": ("Notch filter", [3691])},
    "NotchFilterBandwidth": {"10:11409": ("Notch filter bandwidth", [3690])},
    "NotchFilterDescription": {"10:11410": ("Notch filter description", [3691])},
    "NotchFilterFrequency": {"10:11408": ("Notch filter frequency", [3690])},
    "NystagmoidEyeMovements": {"2:24056": ("Nystagmoid eye movements", [3035])},
    "Nz": {"7:996": ("Nz", [3030])},
    "O1": {"7:1209": ("O1", [3030])},
    "O2": {"7:1214": ("O2", [3030])},
    "OsborneWave": {"10:1408": ("Osborne wave", [3335])},
    "OtherEyeMovement": {"2:24312": ("Other eye movement", [3037])},
    "Oz": {"7:1032": ("Oz", [3030])},
    "P1": {"7:1177": ("P1", [3030])},
    "P10": {"7:1206": ("P10", [3030])},
    "P2": {"7:1182": ("P2", [3030])},
    "P3": {"7:1185": ("P3", [3030])},
    "P3AmplitudePerLead": {"2:3072": ("P3 amplitude, per lead", [3688])},
    "P4": {"7:1190": ("P4", [3030])},
    "P5": {"7:1193": ("P5", [3030])},
    "P6": {"7:1198": ("P6", [3030])},
    "P9": {"7:1201": ("P9", [3030])},
    "PAxis": {"2:16128": ("P Axis", [3229])},
    "PDurationGlobal": {"2:16184": ("P duration global", [3228, 3689])},
    "PDurationPerLead": {"2:6656": ("P duration per lead", [3228, 3687])},
    "PMaximumAmplitudePerLead": {"2:1280": ("P maximum amplitude, per lead", [3688])},
    "PMinimumAmplitudePerLead": {"2:1536": ("P minimum amplitude, per lead", [3688])},
    "PO3": {"7:1233": ("PO3", [3030])},
    "PO4": {"7:1238": ("PO4", [3030])},
    "PO7": {"7:1241": ("PO7", [3030])},
    "PO8": {"7:1246": ("PO8", [3030])},
    "POffsetToQRSOnsetDurationPerLead": {
        "2:7168": ("P offset to QRS onset duration, per lead", [3687])
    },
    "POnsetToP1DurationPerLead": {
        "2:4608": ("P onset to P1 duration, per lead", [3687])
    },
    "POnsetToP2DurationPerLead": {
        "2:4864": ("P onset to P2 duration, per lead", [3687])
    },
    "POnsetToP3DurationPerLead": {
        "2:5120": ("P onset to P3 duration, per lead", [3687])
    },
    "PPIntervalGlobal": {"2:16140": ("PP interval global", [3228])},
    "PPIntervalPerLead": {"2:32768": ("PP interval per lead", [3228])},
    "PPTimePeriodGlobal": {"2:16140": ("PP time period, global", [3689])},
    "PPTimePeriodPerLead": {"2:32768": ("PP time period, per lead", [3687])},
    "PQSegmentTimePeriodGlobal": {
        "2:16148": ("PQ segment time period, global", [3689])
    },
    "PQSegmentTimePeriodPerLead": {
        "2:33536": ("PQ segment time period, per lead", [3687])
    },
    "PQTimePeriodGlobal": {"2:16144": ("PQ time period, global", [3689])},
    "PQTimePeriodPerLead": {"2:33280": ("PQ time period, per lead", [3687])},
    "PRIntervalGlobal": {"2:15872": ("PR interval global", [3228])},
    "PRIntervalPerLead": {"2:7168": ("PR interval per lead", [3228])},
    "PRSegment": {"10:1920": ("PR Segment", [3335])},
    "PRTimePeriodGlobal": {"2:15872": ("PR time period, global", [3689])},
    "PWave": {"10:256": ("P wave", [3335])},
    "PWaveSecondDeflectionInPWave": {
        "10:320": ("P' wave (second deflection in P wave)", [3335])
    },
    "PWaveThirdDeflectionInPWave": {
        "10:384": ("P'' wave (third deflection in P wave)", [3335])
    },
    "PacemakerFusionBeat": {"10:8720": ("Pacemaker Fusion beat", [3335])},
    "PacemakerLearningBeat": {"10:8752": ("Pacemaker Learning beat", [3335])},
    "PacerArtifact": {"3:3294": ("Pacer artifact", [3038])},
    "PadValue": {"10:11397": ("Pad value", [3690])},
    "ParoxsysmalAtrialTachycardia": {
        "10:9408": ("Paroxsysmal Atrial Tachycardia", [3415])
    },
    "PeriodicAndQuasiperiodicCerebralActivity": {
        "2:23968": ("Periodic and quasiperiodic cerebral activity", [3035])
    },
    "PeriodicBurstsWithSuppressions": {
        "2:24032": ("Periodic bursts with suppressions", [3035])
    },
    "PeriodicCerebralComplexes": {"2:24000": ("Periodic cerebral complexes", [3035])},
    "PeriodicEpileptiformDischarges": {
        "2:23992": ("Periodic epileptiform discharges", [3035])
    },
    "PeriodicMovementsOfSleep": {"2:24184": ("Periodic movements of sleep", [3035])},
    "PeriodicMovementsOfSleepWithArousals": {
        "2:24192": ("Periodic movements of sleep with arousals", [3035])
    },
    "PeriodicSharpWaves": {"2:24016": ("Periodic sharp waves", [3035])},
    "PeriodicSuppressions": {"2:24024": ("Periodic suppressions", [3035])},
    "PeriodicTriphasicWaves": {"2:23984": ("Periodic triphasic waves", [3035])},
    "Pg1": {"7:1305": ("Pg1", [3030])},
    "Pg2": {"7:1306": ("Pg2", [3030])},
    "PhantomSpikeAndWaveActivity": {
        "2:23864": ("Phantom spike and wave activity", [3035])
    },
    "Platysma": {"7:392": ("Platysma", [3031])},
    "PlatysmaLeft": {"7:393": ("Platysma, left", [3031])},
    "PlatysmaRight": {"7:394": ("Platysma, right", [3031])},
    "PlexusBrachialis": {"7:100": ("Plexus brachialis", [3032])},
    "PlexusBrachialisLeft": {"7:101": ("Plexus brachialis, left", [3032])},
    "PlexusBrachialisRight": {"7:102": ("Plexus brachialis, right", [3032])},
    "PlexusLumbalis": {"7:176": ("Plexus lumbalis", [3032])},
    "PlexusLumbalisLeft": {"7:177": ("Plexus lumbalis, left", [3032])},
    "PlexusLumbalisRight": {"7:178": ("Plexus lumbalis, right", [3032])},
    "PlexusLumbosacralis": {"7:172": ("Plexus lumbosacralis", [3032])},
    "PlexusLumbosacralisLeft": {"7:173": ("Plexus lumbosacralis, left", [3032])},
    "PlexusLumbosacralisRight": {"7:174": ("Plexus lumbosacralis, right", [3032])},
    "PlexusSacralis": {"7:208": ("Plexus sacralis", [3032])},
    "PlexusSacralisLeft": {"7:209": ("Plexus sacralis, left", [3032])},
    "PlexusSacralisRight": {"7:210": ("Plexus sacralis, right", [3032])},
    "PolymorphicVentricularTachycardia": {
        "10:10064": ("Polymorphic Ventricular Tachycardia", [3415])
    },
    "PositiveOccipitalSharpTransientOfSleep": {
        "2:23776": ("Positive occipital sharp transient of sleep", [3035, 3046])
    },
    "PowerLineProblem": {"3:236": ("Power line problem", [3039])},
    "PowerSupplyProblem": {"3:458": ("Power supply problem", [3039])},
    "Poz": {"7:1028": ("POz", [3030])},
    "PreExcitation": {"10:10096": ("pre-excitation", [3415])},
    "PreExcitationBeat": {"10:8624": ("pre-excitation", [3335])},
    "PrecordialLead": {"2:87": ("Precordial lead", [3001])},
    "PseudoOrthogonalXYZLeadSystem": {
        "10:11278": ("Pseudo-orthogonal XYZ lead system", [3263])
    },
    "PulseArtifactualActivity": {"2:24232": ("Pulse artifactual activity", [3035])},
    "Pz": {"7:1024": ("Pz", [3030])},
    "QAmplitudePerLead": {"2:1792": ("Q amplitude, per lead", [3688])},
    "QDurationPerLead": {"2:7680": ("Q duration, per lead", [3687])},
    "QRSAxis": {"2:16132": ("QRS axis", [3229])},
    "QRSDurationGlobal": {"2:16156": ("QRS duration global", [3228, 3689])},
    "QRSDurationPerLead": {"2:7936": ("QRS duration per lead", [3228, 3687])},
    "QSWave": {"10:512": ("QS wave", [3335])},
    "QTDurationGlobal": {"2:16160": ("QT duration, global", [3689])},
    "QTDurationPerLead": {"2:8192": ("QT duration, per lead", [3687])},
    "QTIntervalGlobal": {"2:16160": ("QT interval global", [3228])},
    "QTIntervalPerLead": {"2:8192": ("QT interval per lead", [3228])},
    "QTUTimePeriodGlobal": {"2:16004": ("QTU time period, global", [3689])},
    "QTUTimePeriodPerLead": {"2:34560": ("QTU time period, per lead", [3687])},
    "QWave": {"10:448": ("Q wave", [3335])},
    "QtcGlobalUsingBazettFormula": {
        "2:15880": ("QTc global using Bazett formula", [3227])
    },
    "QtcGlobalUsingFredericiaFormula": {
        "2:15892": ("QTc global using Fredericia formula", [3227])
    },
    "QtcIntervalGlobal": {"2:15876": ("QTc interval global", [3227])},
    "QtcIntervalPerLead": {"2: 33792": ("QTc interval per lead", [3227])},
    "QtcPerLeadUsingBazettFormula": {
        "2:34048": ("QTc per lead using Bazett formula", [3227])
    },
    "QtcPerLeadUsingFredericiaFormula": {
        "2:34304": ("QTc per lead using Fredericia formula", [3227])
    },
    "QuasiperiodicCerebralSharpWaves": {
        "2:24008": ("Quasiperiodic cerebral sharp waves", [3035])
    },
    "QuasiperiodicTriphasicWaves": {
        "2:23976": ("Quasiperiodic triphasic waves", [3035])
    },
    "R1AmplitudePerLead": {"2:12800": ("R1 amplitude, per lead", [3688])},
    "R1DurationPerLead": {"2:11264": ("R1 duration, per lead", [3687])},
    "R2AmplitudePerLead": {"2:13056": ("R2 amplitude, per lead", [3688])},
    "R2DurationPerLead": {"2:11520": ("R2 duration, per lead", [3687])},
    "R3AmplitudePerLead": {"2:13312": ("R3 amplitude, per lead", [3688])},
    "R3DurationPerLead": {"2:11776": ("R3 duration, per lead", [3687])},
    "RAmplitudePerLead": {"2:2048": ("R amplitude, per lead", [3688])},
    "REM": {"2:24296": ("REM", [3037])},
    "ROnTPrematureVentricularBeat": {
        "10:8368": ("R-on-T premature ventricular beat", [3335])
    },
    "RRIntervalGlobal": {"2:16168": ("RR interval global", [3228])},
    "RRIntervalPerLead": {"2:33024": ("RR interval per lead", [3228])},
    "RRTimePeriodGlobal": {"2:16000": ("RR time period, global", [3689])},
    "RRTimePeriodPerLead": {"2:33024": ("RR time period, per lead", [3687])},
    "RWave": {"10:576": ("R wave", [3335])},
    "RWaveSecondDeflectionInRWave": {
        "10:640": ("R' wave (second deflection in R Wave)", [3335])
    },
    "RWaveThirdDeflectionInRWave": {
        "10:704": ("R'' wave (third deflection in R Wave)", [3335])
    },
    "RamusDorsalisNerviUlnaris": {"7:136": ("Ramus dorsalis nervi ulnaris", [3032])},
    "RamusDorsalisNerviUlnarisLeft": {
        "7:137": ("Ramus dorsalis nervi ulnaris, left", [3032])
    },
    "RamusDorsalisNerviUlnarisRight": {
        "7:138": ("Ramus dorsalis nervi ulnaris, right", [3032])
    },
    "RamusPalmarisNerviMediani": {"7:124": ("Ramus palmaris nervi mediani", [3032])},
    "RamusPalmarisNerviMedianiLeft": {
        "7:125": ("Ramus palmaris nervi mediani, left", [3032])
    },
    "RamusPalmarisNerviMedianiRight": {
        "7:126": ("Ramus palmaris nervi mediani, right", [3032])
    },
    "RamusPalmarisNerviUlnaris": {"7:140": ("Ramus palmaris nervi ulnaris", [3032])},
    "RamusPalmarisNerviUlnarisLeft": {
        "7:141": ("Ramus palmaris nervi ulnaris, left", [3032])
    },
    "RamusPalmarisNerviUlnarisRight": {
        "7:142": ("Ramus palmaris nervi ulnaris, right", [3032])
    },
    "RapidEyeMovements": {"2:24080": ("Rapid eye movements", [3035])},
    "ReentrantAVNodalTachycardia": {
        "10:9616": ("Reentrant AV nodal tachycardia", [3415])
    },
    "RespiratoryArtifactualActivity": {
        "2:24248": ("Respiratory artifactual activity", [3035])
    },
    "RespiratorySinusArrhythmia": {"10:9296": ("Respiratory Sinus Arrhythmia", [3415])},
    "RhythmicSharpWaves": {"2:23944": ("Rhythmic sharp waves", [3035])},
    "RightArmLead": {"2:22": ("Right Arm Lead", [3001])},
    "RightAtriumAntibradycardiaPaceSpike": {
        "10:4608": ("right atrium Antibradycardia pace spike", [3335])
    },
    "RightBundleBranchBlockBeat": {
        "10:8496": ("Right bundle branch block beat", [3335])
    },
    "RightLegLead": {"2:147": ("Right Leg Lead", [3001])},
    "RightVentricleAntibradycardiaPaceSpike": {
        "10:5376": ("right ventricle Antibradycardia pace spike", [3335])
    },
    "S1AmplitudePerLead": {"2:13568": ("S1 amplitude, per lead", [3688])},
    "S1DurationPerLead": {"2:12032": ("S1 duration, per lead", [3687])},
    "S2AmplitudePerLead": {"2:13824": ("S2 amplitude, per lead", [3688])},
    "S2DurationPerLead": {"2:12288": ("S2 duration, per lead", [3687])},
    "S3AmplitudePerLead": {"2:14080": ("S3 amplitude, per lead", [3688])},
    "S3DurationPerLead": {"2:12544": ("S3 duration, per lead", [3687])},
    "SAmplitudePerLead": {"2:2304": ("S amplitude, per lead", [3688])},
    "STAmplitudePerLead": {"2:768": ("ST amplitude, per lead", [3688])},
    "STChange": {"10:2304": ("ST change", [3335])},
    "STMeasurementPoint": {"10:2112": ("ST measurement point", [3335])},
    "STSegment": {"10:1984": ("ST Segment", [3335])},
    "SWave": {"10:832": ("S wave", [3335])},
    "SWaveSecondDeflectionInSWave": {
        "10:896": ("S' wave (second deflection in S Wave)", [3335])
    },
    "SWaveThirdDeflectionInSWave": {
        "10:960": ("S'' wave (third deflection in S Wave)", [3335])
    },
    "Saccade": {"2:24288": ("Saccade", [3037])},
    "SampleRate": {"10:11393": ("Sample rate", [3690])},
    "SampleSkew": {"10:11399": ("Sample skew", [3690])},
    "SecondDegreeAVBlock": {"10:9648": ("Second Degree AV Block", [3415])},
    "SecondDegreeAVBlockTypeIIMobitzTypeII": {
        "10:9680": ("Second Degree AV Block Type II (Mobitz Type II)", [3415])
    },
    "SecondDegreeAVBlockTypeIWenckebachOrMobitzTypeI": {
        "10:9664": (
            "Second Degree AV Block Type I (Wenckebach, or Mobitz Type I)",
            [3415],
        )
    },
    "SecondDegreeSABlockTypeII": {
        "10:9808": ("Second Degree SA Block Type II", [3415])
    },
    "SecondDegreeSABlockTypeIWenckebach": {
        "10:9792": ("Second Degree SA Block Type I (Wenckebach)", [3415])
    },
    "Sensitivity": {"10:11394": ("Sensitivity", [3690])},
    "SevereNoise": {"10:11232": ("Severe noise", [3680])},
    "SevereNoiseBeatsCannotBeDetectedOrClassified": {
        "10:11232": ("Severe Noise, beats cannot be detected or classified", [3335])
    },
    "SharpAndSlowWaveComplex": {"2:23936": ("Sharp and slow wave complex", [3035])},
    "SharpAppearingOrEpileptiformActivity": {
        "2:23816": ("Sharp appearing or epileptiform activity", [3035])
    },
    "SharpSpikes": {"3:3270": ("Sharp spikes", [3038])},
    "SharpTransient": {"2:23824": ("Sharp transient", [3035])},
    "SinusArrhythmia": {"10:9280": ("Sinus Arrhythmia", [3415])},
    "SinusBradycardia": {"10:9248": ("Sinus Bradycardia", [3415])},
    "SinusRhythm": {"10:9216": ("Sinus Rhythm", [3415])},
    "SinusTachycardia": {"3:3262": ("Sinus tachycardia", [3038])},
    "SinusTachycardiaRhythm": {"10:9264": ("Sinus Tachycardia", [3415])},
    "SleepActivityAndEvent": {"2:23736": ("Sleep activity and event", [3035, 3046])},
    "SleepArousal": {"2:23800": ("Sleep arousal", [3035, 3046])},
    "SleepAwakening": {"2:23808": ("Sleep awakening", [3035, 3046])},
    "SleepFWave": {"2:23760": ("Sleep F wave", [3035, 3046])},
    "SleepKComplex": {"2:23768": ("Sleep K complex", [3035, 3046])},
    "SleepSawtoothWave": {"2:23784": ("Sleep sawtooth wave", [3035, 3046])},
    "SleepSpindle": {"2:23744": ("Sleep spindle", [3035, 3046])},
    "SleepStageREM": {"2:23680": ("Sleep stage REM", [3035, 3046])},
    "SleepStageREMWithSleepSpindle": {
        "2:23688": ("Sleep stage REM with sleep spindle", [3035, 3046])
    },
    "SleepStageShift": {"2:23792": ("Sleep stage shift", [3035, 3046])},
    "SleepStageUnspecified": {"2:23656": ("Sleep stage unspecified", [3035, 3046])},
    "SleepStageUnstageable": {"2:23664": ("Sleep stage unstageable", [3035, 3046])},
    "SleepStageWake": {"2:23672": ("Sleep stage wake", [3035, 3046])},
    "SleepVWave": {"2:23752": ("Sleep V wave", [3035, 3046])},
    "SlowEyeMovement": {"2:24304": ("Slow eye movement", [3037])},
    "SlowEyeMovements": {"2:24064": ("Slow eye movements", [3035])},
    "SlowVentricularTachycardiaIdioventricularTachycardia": {
        "10:9904": (
            "Slow Ventricular Tachycardia (Idioventricular Tachycardia)",
            [3415],
        )
    },
    "SmallSharpSpike": {"2:23840": ("Small sharp spike", [3035])},
    "Sp1": {"7:1313": ("Sp1", [3030])},
    "Sp2": {"7:1314": ("Sp2", [3030])},
    "SpikeAndWaveComplex": {"2:23920": ("Spike and wave complex", [3035])},
    "SpikesAndWaves": {"3:3254": ("Spikes and waves", [3038])},
    "Standard12LeadAndCC5CM5ML": {
        "10:11288": ("Standard 12-lead and CC5-CM5-ML", [3263])
    },
    "Standard12LeadAndCM5CC5CH5": {
        "10:11289": ("Standard 12-lead and CM5-CC5-CH5", [3263])
    },
    "Standard12LeadAndNEHB": {"10:11287": ("Standard 12-lead and NEHB", [3263])},
    "Standard12LeadAndXYZ": {"10:11286": ("Standard 12-lead and XYZ", [3263])},
    "Standard12LeadExtendedToTheLeftByV7V8V9": {
        "10:11292": ("Standard 12-lead extended to the left by V7, V8, V9", [3263])
    },
    "Standard12LeadExtendedToTheRightByV5RV4RV3R": {
        "10:11291": ("Standard 12-lead extended to the right by V5R, V4R, V3R", [3263])
    },
    "Standard12LeadPositionsElectrodesPlacedIndividually": {
        "10:11265": (
            "Standard 12-lead positions, electrodes placed individually",
            [3263],
        )
    },
    "Standard12LeadPositionsOneIntercostalSpaceHigher": {
        "10:11272": ("Standard 12-lead positions one intercostal space higher", [3263])
    },
    "Standard12LeadWithExtraLeadsToTheRightAndOrLeftSides": {
        "10:11290": (
            "Standard 12-lead with extra leads to the right and/or left sides",
            [3263],
        )
    },
    "SupraventricularAtrialOrJunctionalEctopicRhythm": {
        "10:9488": ("Supraventricular (atrial or junctional) Ectopic Rhythm", [3415])
    },
    "SupraventricularEscapeBeat": {"10:8384": ("Supraventricular escape beat", [3335])},
    "SupraventricularExtrasystole": {
        "3:3190": ("Supraventricular extrasystole", [3038])
    },
    "SupraventricularParoxsysmalTachycardia": {
        "10:9520": ("Supraventricular Paroxsysmal Tachycardia", [3415])
    },
    "SupraventricularPrematureContraction": {
        "10:8256": ("Supraventricular premature contraction", [3335])
    },
    "SupraventricularTachycardia": {
        "10:9504": ("Supraventricular Tachycardia (atrial or junctional)", [3415])
    },
    "SustainedMonomorphicVentricularTachycardia": {
        "10:10048": ("Sustained Monomorphic Ventricular Tachycardia", [3415])
    },
    "SwallowingAndChewingArtifactualActivity": {
        "2:24264": ("Swallowing and chewing artifactual activity", [3035])
    },
    "SweatOfGalvanicArtifactualActivity": {
        "2:24224": ("Sweat of galvanic artifactual activity", [3035])
    },
    "T1": {"7:1297": ("T1", [3030])},
    "T10": {"7:1270": ("T10", [3030])},
    "T2": {"7:1298": ("T2", [3030])},
    "T3": {"7:1249": ("T3", [3030])},
    "T4": {"7:1254": ("T4", [3030])},
    "T5": {"7:1257": ("T5", [3030])},
    "T6": {"7:1262": ("T6", [3030])},
    "T9": {"7:1265": ("T9", [3030])},
    "TAxis": {"2:16136": ("T axis", [3229])},
    "TMaximumAmplitudePerLead": {"2:2560": ("T maximum amplitude, per lead", [3688])},
    "TMinimumAmplitudePerLead": {"2:2816": ("T minimum amplitude, per lead", [3688])},
    "TP10": {"7:1286": ("TP10", [3030])},
    "TP7": {"7:1273": ("TP7", [3030])},
    "TP8": {"7:1278": ("TP8", [3030])},
    "TP9": {"7:1281": ("TP9", [3030])},
    "TUFusedWave": {"10:1664": ("TU fused wave", [3335])},
    "TWave": {"10:1024": ("T wave", [3335])},
    "TWaveAlternans": {"10:10416": ("T-wave alternans", [3415])},
    "TWaveChange": {"10:2368": ("T-wave change", [3335])},
    "TWaveSecondDeflectionInTWave": {
        "10:1088": ("T' wave (second deflection in T Wave)", [3335])
    },
    "ThirdDegreeAVBlock": {"3:3258": ("Third-degree AV block", [3038])},
    "ThirdDegreeAVBlockCompleteAVBlock": {
        "10:9696": ("Third Degree AV Block (complete AV block)", [3415])
    },
    "ThirdDegreeSABlockCompleteSABlock": {
        "10:9824": ("Third Degree SA Block (complete SA block)", [3415])
    },
    "TimeOffset": {"10:11400": ("Time offset", [3690])},
    "TimeSkew": {"10:11398": ("Time skew", [3690])},
    "TorsadesDePointesVentricularTachycardia": {
        "10:10080": ("Torsades de Pointes Ventricular Tachycardia", [3415])
    },
    "TransthoracicAntibradycardiaPaceSpike": {
        "10:5888": ("transthoracic Antibradycardia pace spike", [3335])
    },
    "TransthoracicAntitachycardiaPaceSpike": {
        "10:6912": ("transthoracic Antitachycardia pace spike", [3335])
    },
    "TransthoracicCardioversionSpike": {
        "10:7936": ("transthoracic Cardioversion spike", [3335])
    },
    "TransthoracicDefibrillationSpike": {
        "10:8960": ("transthoracic Defibrillation spike", [3335])
    },
    "TrifascicularBlockBeat": {"10:8576": ("trifascicular block beat", [3335])},
    "TriphasicWave": {"2:23856": ("Triphasic wave", [3035])},
    "UWave": {"10:1152": ("U wave", [3335])},
    "UnclassifiableBeat": {"10:8736": ("Unclassifiable beat", [3335])},
    "Unspecified12LeadSystem": {"10:11264": ("Unspecified 12-lead system", [3263])},
    "UnspecifiedLead": {"2:0": ("Unspecified lead", [3001])},
    "UnspecifiedXYZLeadSystem": {"10:11273": ("Unspecified XYZ lead system", [3263])},
    "VentricleAntitachycardiaPaceSpike": {
        "10:6656": ("ventricle Antitachycardia pace spike", [3335])
    },
    "VentricleCardioversionSpike": {
        "10:7680": ("ventricle Cardioversion spike", [3335])
    },
    "VentricleDefibrillationSpike": {
        "10:8704": ("ventricle Defibrillation spike", [3335])
    },
    "VentricularActivationTime": {"10:2432": ("Ventricular Activation Time", [3335])},
    "VentricularActivationTimePerLead": {
        "2:11008": ("Ventricular activation time, per lead", [3687])
    },
    "VentricularAntibradycardiaPaceSpike": {
        "10:5120": ("ventricular Antibradycardia pace spike", [3335])
    },
    "VentricularBigeminy": {"10:9920": ("Ventricular Bigeminy", [3415])},
    "VentricularCouplet": {"10:9952": ("Ventricular Couplet", [3415])},
    "VentricularDemandModePacing": {
        "10:10240": ("Ventricular Demand Mode Pacing", [3415])
    },
    "VentricularEscapeBeat": {"10:8432": ("Ventricular escape beat", [3335])},
    "VentricularFibrillation": {"10:10016": ("Ventricular Fibrillation", [3415])},
    "VentricularFlutter": {"10:10000": ("Ventricular Flutter", [3415])},
    "VentricularFlutterWave": {"10:1728": ("Ventricular flutter wave", [3335])},
    "VentricularParasystole": {"10:9872": ("Ventricular Parasystole", [3415])},
    "VentricularPrematureContractionBeat": {
        "10:8336": ("Ventricular premature contraction beat", [3335])
    },
    "VentricularRhythm": {"10:9840": ("Ventricular rhythm", [3415])},
    "VentricularRun": {"10:9968": ("Ventricular Run", [3415])},
    "VentricularTachycardia": {
        "10:9984": ("Ventricular Tachycardia (nonparoxysmal)", [3415])
    },
    "VentricularTrigeminy": {"10:9936": ("Ventricular Trigeminy", [3415])},
    "WanderingPacemakerBetweenTheSinusNodeAndTheAVNode": {
        "10:9344": (
            "Wandering Pacemaker between the sinus node and the A-V node",
            [3415],
        )
    },
    "WanderingSinusPacemakerWithinTheSinusNode": {
        "10:9328": ("Wandering Sinus Pacemaker within the sinus node", [3415])
    },
    "Wicket": {"2:23832": ("Wicket", [3035])},
    "WolfParkinsonTypeA": {"10:10128": ("Wolf-Parkinson type A", [3415])},
    "WolfParkinsonTypeAQRSPositiveInV1V2": {
        "10:8656": ("Wolf-Parkinson type A", [3335])
    },
    "WolfParkinsonTypeB": {"10:10144": ("Wolf-Parkinson type B", [3415])},
    "WolfParkinsonTypeBQRSNegativeInV1V2": {
        "10:8672": ("Wolf-Parkinson type B", [3335])
    },
    "WolfParkinsonWhiteSyndrome": {
        "10:10112": ("Wolf-Parkinson-White syndrome", [3415])
    },
    "WolfParkinsonWhiteSyndromeLessSpecific": {
        "10:8640": ("Wolf-Parkinson-White syndrome", [3335])
    },
    "XYZLeadsDerivedFromStandard12Lead": {
        "10:11279": ("XYZ leads derived from standard 12-lead", [3263])
    },
    "ZeroOffset": {"10:11395": ("Zero offset", [3690])},
    "ZetaWave": {"2:23848": ("Zeta wave", [3035])},
    "_12LeadDerivedFromFrankXYZLeads": {
        "10:11269": ("12-lead derived from Frank XYZ leads", [3263])
    },
    "_12LeadDerivedFromNonStandardLeads": {
        "10:11270": ("12-lead derived from non-standard leads", [3263])
    },
    "_12LeadElectrodePad": {"10:11268": ("12-lead electrode pad", [3263])},
    "_12LeadForBicycleExerciseTestingLimbLeadsOnBackOfPatient": {
        "10:11271": (
            "12-lead for bicycle exercise testing, limb leads on back of patient",
            [3263],
        )
    },
    "_12LeadFromEASILeadsESASAIByDowerEASITransformation": {
        "10:11284": (
            "12-lead from EASI leads (ES, AS, AI) by Dower/EASI transformation",
            [3263],
        )
    },
    "_12LeadFromFrankLeadsXYZLeadsByDowerTransformation": {
        "10:11283": (
            "12-lead from Frank leads XYZ leads by Dower transformation",
            [3263],
        )
    },
    "_12LeadFromLimbLeadsIIIAndOneOrMoreVLeads": {
        "10:11285": ("12-lead from Limb Leads (I, II) and one or more V leads", [3263])
    },
    "_14And6HzPositiveBursts": {"2:23872": ("14 and 6 Hz positive bursts", [3035])},
    "_3LeadSystemCC5CM5CH5": {"10:11282": ("3-lead system, CC5-CM5-CH5", [3263])},
    "_3LeadSystemCC5CM5ML": {"10:11281": ("3-lead system, CC5-CM5-ML", [3263])},
}

concepts["BARI"] = {
    "PosteriorDescendingSeptalPerforators": {
        "9": (
            "Posterior descending septal perforators",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "RamusLaterals": {
        "28A": (
            "Ramus Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stDiagonalCoronaryArteryLaterals": {
        "15A": (
            "1st Diagonal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_1stMarginalCoronaryArteryLaterals": {
        "20A": (
            "1st Marginal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_2ndDiagonalCoronaryArteryLaterals": {
        "16A": (
            "2nd Diagonal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_2ndMarginalCoronaryArteryLaterals": {
        "21A": (
            "2nd Marginal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_3rdDiagonalCoronaryArteryLaterals": {
        "29A": (
            "3rd Diagonal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
    "_3rdMarginalCoronaryArteryLaterals": {
        "22A": (
            "3rd Marginal Coronary Artery Laterals",
            [3014, 3015, 3604, 3827, 6102, 6109, 6117, 7151, 7192, 8134, 9514],
        )
    },
}

concepts["I10"] = {
    "ElevatedProstateSpecificAntigen": {
        "R97.20": ("Elevated Prostate Specific Antigen", [6327])
    }
}

concepts["ITIS_TSN"] = {
    "PeromyscusLeucopus": {"180278": ("Peromyscus leucopus", [7454])},
    "PeromyscusManiculatus": {"180276": ("Peromyscus maniculatus", [7454])},
    "SigmodonGenus": {"180346": ("Sigmodon genus", [7454])},
}

concepts["NEU"] = {
    "AnteriorThalamicRadiation": {
        "1726": ("anterior thalamic radiation", [7702, 7710, 8134])
    },
    "ArcuateFasciculus": {"2063": ("arcuate fasciculus", [7703, 7710, 8134])},
    "CorticobulbarTract": {"1319": ("corticobulbar tract", [7702, 7710, 8134])},
    "CorticospinalTract": {"1320": ("corticospinal tract", [7702, 7710, 8134])},
    "InferiorOpticRadiationMeyerLoop": {
        "1466": ("inferior optic radiation (Meyer's loop)", [7702, 7710, 8134])
    },
    "InferiorThalamicRadiation": {
        "2082": ("inferior thalamic radiation", [7702, 7710, 8134])
    },
    "PosteriorThalamicRadiation": {
        "2083": ("posterior thalamic radiation", [7702, 7710, 8134])
    },
    "StriaTerminalis": {"286": ("stria terminalis", [7704, 7710, 8134])},
    "SuperiorLongitudinalFasciculus": {
        "2080": ("superior longitudinal fasciculus", [7703, 7710, 8134])
    },
    "SuperiorOpticRadiationBaumLoop": {
        "3473": ("superior optic radiation (Baum's loop)", [7702, 7710, 8134])
    },
    "SuperiorThalamicRadiation": {
        "2081": ("superior thalamic radiation", [7702, 7710, 8134])
    },
}

concepts["NDC"] = {
    "Definity": {"11994-011-04": ("Definity", [12030])},
    "Optison": {"0407-2707-03": ("Optison", [12030])},
}
